<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace Ron\Controls;

use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\GuiUtils;
use Ron\Http\Csrf;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\Utils\Attributes;


/**
 * Reprezentuje jeden sloupec v \Ron\Controls\Browser.
 */
class BrowserField
{
    const FIELD_TYPE_DEFAULT = 'default';
    const FIELD_TYPE_HIDDEN = 'hidden';
    const FIELD_TYPE_LINE = 'line';
    const FIELD_TYPE_TEXT = 'text';
    const FIELD_TYPE_SELECT = 'select';
    const FIELD_TYPE_URL = 'url';
    const FIELD_TYPE_EMAIL = 'email';       // emailova adresa - zobrazi se jako link mailto://
    const FIELD_TYPE_COMMENT = 'comment';   // poznamka zobrazena na dalsim radku

    /** @var Database */
    public $database;

    public $type;

    /** @var string     Kod sloupce */
    public $code;
    /** @var bool       Umoznuje skryt sloupec */
    public $visible = true;

    public $sqlFieldName;
    /** @var string     SQL vyraz, ktery se pouzije pro razeni - kdyz neni nastaven, pouzije se sqlFieldName */
    public $sortSQL;
    /** @var string     SQL vyraz, ktery se pouzije pro razeni - kdyz neni nastaven, pouzije se sqlFieldName [DESC] */
    public $sortSQLDesc;
    // TODO: napad - pridat priznak, ze se maji v tabulce zvyraznit radky, na kterych se dane pole zmenilo (pokud se podle nej radi) - pridanim  border-top border-dark (pripadne i uzivatelsky nastavenou css tridu)
    // napr. pri serazeni podle datumu se oddeli radky, na kterych zacinaji zaznamy pro dalsi den - resil jsem napr. v seznamu RadekSluzebniCestyBrowser

    /** @var int        index pole v radku nactenem z databaze */
    public $sqlFieldIndex;
    /** @var int|string index pole, ktere se pouzije pro titulek */
    public $titleFieldIndex = null;
    /** @var string     Text, ktery se zobrazi na zacatku titulku */
    public $titlePrefix = '';
    /** @var int|string Index pole, ktere se zobrazi v zavorce */
    public $sqlFieldIndex2 = null;
    /** @var string     Oddelovac druheho zobrazeneho pole. Pokud konci "(" tak se na konec prida i prava zavorka ")" */
    public $sqlFieldIndex2Separator = ' (';
    /** @var bool       priznak, jestli se ma pro nacteni pouzit Database::castWideMemo */
    public $castWideMemo = false;

    /** @var bool       Pokud je true, tak se pole vynecha pri automatickem sestavovani GROUP BY casti SQL dotazu */
    public $excludeFromGroupBy = false;

    /** @var string Prefix pro preklad retezcu pro SELECT */
    public $translationPrefix;
    /** @var string Prefix pro preklad titulku SELECT */
    public $titleTranslationPrefix;
    /** @var int|string    Index nebo kod pole, ktere se pouzije jako kod pro preklad */
    public $translationValueIndex;
    /** @var bool       Priznak, ze se ma prelozit text nacteny z databaze */
    public $translateText = false;


    /** @var int[]      Asociativni pole atributu (nazev atributu => index pole) */
    public $attributeIndexes = [];

    /** @ var string     Nepovinne ID pole - pro zjisteni pouzijte metodu getId */
    //protected $id;

    /** @var string     Text titulku */
    public $header;
    public $headerColSpan;
    /** @var string     CSS trida titulku. Pokud neni nastavena, pouzije se stejna jako pro datovou bunku */
    public $headerClass = null;
    /** @var string[]   Asociativni pole atributu hlavicky */
    public $headerAttributes = [];

    public $footer;
    /** @var int        Pokud je > 1, tak se nasledujici footer(y) nezobrazi*/
    public $footerColSpan;
    /** @var bool       Urcuje, jestli se ma v paticce zobrazit soucet */
    public $showSum;
    /** @var mixed      Soucet - pocita se behem renderovani hodnot */
    protected $sum;

    /** @var string     Formatovaci retezec pro zobrazeni */
    public $format;
    /** @var string     Text, ktery se zobrazi misto hodnoty pole (pouze pro nektere typy - link,text) */
    public $text;

    /** @var string     Odkaz - pokud nekonci znakem "=", tak k nemu prida jeste "&record=..." */
    public $link;
    /** @var string[]   Atributy pro odkaz */
    public $linkAttributes = null;
    /** @var ?string    Prefix pro vytvoreni csrfkey */
    public $csrfPrefix = null;
    /** @var string|int Index pole pro vytvoreni csrfkey. Klic se vytvori z prefixu a hodnoty pole se zadanym indexem a v linku nahradi retezec %CSRF% */
    public $csrfKeyIndex = null;

    /** @var string     Zarovnani textu - left, right, center */
    public $align;
    /** @var bool       Jestli se muze podle tohoto pole radit nebo ne */
    public $ordering;
    /** @var bool       Jestli muze byt text zalomeny */
    public $wrap;
    /** @var array      Seznam hodnot, ktere se zobrazi pro pole select (podle hodnoty v databazi) */
    public $options;

    public $style;
    public $className;
    public $rowSpan;

    /** @var mixed      Pokud je nastavena filterValue, pole se zobrazi jen v pripade, ze ma tuto hodnotu.
     *                  Pokud je to funkce, tak se pole zobrazi v pripade, ze funkce vraci true.
     *                  Lze pouzit napriklad pro zobrazeni nejakeho priznaku, ze radek splnuje nejakou podminku. */
    public $filterValue;

    /** @var bool       Priznak, ze se ma konvertovat text (default = true) */
    public $htmlConvert;


    /**
     * Inicializuje sloupec browseru.
     * @param Database $database
     * @param array $options
     */
    function __construct($database = null, $options = null)
    {
        $this ->database = $database;

        $this ->type = self::FIELD_TYPE_DEFAULT;
        $this ->align = '';

        $this ->sqlFieldIndex = null;
        $this ->footerColSpan = 0;

        $this ->htmlConvert = true;

        $this ->sum = 0;

        if ($options) {
            $this ->init($options);
        }
    }


    /**
     * Inicializuje pole podle asociativniho pole $options
     * @param mixed $options
     */
    private function init($options) {
        foreach ($options as $key => $value) {
            switch ($key) {
                case 'type' :
                    $this ->type = $value;
                    break;

                case 'sqlFieldName' :
                    $this ->sqlFieldName = $value;
                    break;

                case 'sqlIndex' :
                    $this ->sqlFieldIndex = $value;
                    break;

                case 'visible' :
                    $this ->visible = $value;
                    break;

                case 'header' :
                    $this ->header = $value;
                    break;

                case 'headerTextId' :
                    $this ->header = TextManager::_($value);
                    break;

                case 'format' :
                    $this ->format = $value;
                    break;

                case 'align' :
                    $this ->align = $value;
                    break;

                case 'link' :
                    $this ->link = $value;
                    break;

//                case 'wrap' :
//                    $this ->wrap = $value;
//                    break;

                case 'ordering' :
                    $this ->ordering = $value;
                    break;

                case 'text' :
                    $this ->text = $value;
                    break;

                case 'options' :
                    $this ->options = $value;
                    break;

//                case 'headerColSpan' :
//                    $this ->headerColSpan = $value;
//                    break;

//                case 'footerColSpan' :
//                    $this ->footerColSpan = $value;
//                    break;

                default :
                    $this ->{$key} = $value;
                    break;
            }
        }
    }


    /**
     * @param string $type
     * @return BrowserField
     */
    public function setType($type)
    {
        $this->type = $type;
        return $this;
    }


    /**
     * @param string $code
     * @return BrowserField
     */
    public function setCode($code)
    {
        $this->code = $code;
        return $this;
    }

    /**
     * @param boolean $visible
     * @return BrowserField
     */
    public function setVisible($visible)
    {
        $this->visible = $visible;
        return $this;
    }

    /**
     * @param mixed $sqlFieldName
     * @param null $sqlFieldIndex   Pokud je zadano, nastavi se i index pole
     * @return BrowserField
     */
    public function setSqlFieldName($sqlFieldName, $sqlFieldIndex = null)
    {
        $this->sqlFieldName = $sqlFieldName;
        if (isset($sqlFieldIndex)) {
            $this ->sqlFieldIndex = $sqlFieldIndex;
        }
        return $this;
    }

    /**
     * @param int $sqlFieldIndex
     * @return BrowserField
     */
    public function setSqlFieldIndex($sqlFieldIndex)
    {
        $this->sqlFieldIndex = $sqlFieldIndex;
        return $this;
    }

    /**
     * @param int $sqlFieldIndex2
     * @param null|string $sqlFieldIndex2Separator
     * @return BrowserField
     */
    public function setSqlFieldIndex2($sqlFieldIndex2, $sqlFieldIndex2Separator = null)
    {
        $this->sqlFieldIndex2 = $sqlFieldIndex2;
        if (isset($sqlFieldIndex2Separator)) {
            $this ->sqlFieldIndex2Separator = $sqlFieldIndex2Separator;
        }

        return $this;
    }

    /**
     * @param string $sqlFieldIndex2Separator
     * @return BrowserField
     */
    public function setSqlFieldIndex2Separator($sqlFieldIndex2Separator)
    {
        $this->sqlFieldIndex2Separator = $sqlFieldIndex2Separator;
        return $this;
    }



    /**
     * @param boolean $castWideMemo
     * @return $this
     */
    public function setCastWideMemo($castWideMemo)
    {
        $this->castWideMemo = $castWideMemo;
        return $this;
    }


    /**
     * @param boolean $excludeFromGroupBy
     * @return $this
     */
    public function setExcludeFromGroupBy($excludeFromGroupBy)
    {
        $this->excludeFromGroupBy = $excludeFromGroupBy;
        return $this;
    }




    /**
     * @param int $titleFieldIndex
     * @param string $titlePrefix
     * @return BrowserField
     */
    public function setTitleFieldIndex($titleFieldIndex, $titlePrefix = null)
    {
        $this->titleFieldIndex = $titleFieldIndex;
        if (isset($titlePrefix)) {
            $this ->titlePrefix = $titlePrefix;
        }

        return $this;
    }

    /**
     * @param string $titlePrefix
     * @return BrowserField
     */
    public function setTitlePrefix($titlePrefix)
    {
        $this->titlePrefix = $titlePrefix;
        return $this;
    }


    /**
     * @param string $translationPrefix
     * @param int|string $translationValueIndex
     * @return $this
     */
    public function setTranslationPrefix($translationPrefix, $translationValueIndex = null)
    {
        $this->translationPrefix = $translationPrefix;
        if (isset($translationValueIndex)) {
            $this->translationValueIndex = $translationValueIndex;
        }
        return $this;
    }

    /**
     * @param string $titleTranslationPrefix
     * @return $this
     */
    public function setTitleTranslationPrefix($titleTranslationPrefix)
    {
        $this->titleTranslationPrefix = $titleTranslationPrefix;
        return $this;
    }

    /**
     * @param int|string $translationValueIndex
     * @return $this
     */
    public function setTranslationValueIndex($translationValueIndex)
    {
        $this->translationValueIndex = $translationValueIndex;
        return $this;
    }

    /**
     * @param boolean $translateText
     * @return $this
     */
    public function setTranslateText($translateText)
    {
        $this->translateText = $translateText;
        return $this;
    }


    /**
     * @param \int[] $attributeIndexes
     * @return BrowserField
     */
    public function setAttributeIndexes($attributeIndexes)
    {
        $this->attributeIndexes = $attributeIndexes;
        return $this;
    }

    /**
     * @param \int[] $attributeIndexes
     * @return BrowserField
     */
    public function addAttributeIndexes($attributeIndexes)
    {
        if ($attributeIndexes) {
            foreach ($attributeIndexes as $key => $value) {
                $this ->attributeIndexes[$key] = $value;
            }
        }

        return $this;
    }



    /**
     * @param string $sortSQL
     * @param string $sortSQLDesc   pokud je zadano, nastavi se i SQL pro sestupne razeni
     * @return BrowserField
     */
    public function setSortSQL($sortSQL, $sortSQLDesc = null)
    {
        $this->sortSQL = $sortSQL;
        if (isset($sortSQLDesc)) {
            $this ->sortSQLDesc = $sortSQLDesc;
        }

        $this ->ordering = true;

        return $this;
    }

    /**
     * @param string $sortSQLDesc
     * @return BrowserField
     */
    public function setSortSQLDesc($sortSQLDesc)
    {
        $this->sortSQLDesc = $sortSQLDesc;
        return $this;
    }

    /**
     * @param string $id
     * @return BrowserField
     * @deprecated
     */
    public function setId($id)
    {
        return $this;
    }

    /**
     * @param string $header
     * @param int $headerColSpan    Nepovinne lze zadat i hodnotu pro colspan
     * @return BrowserField
     */
    public function setHeader($header, $headerColSpan = null)
    {
        $this->header = $header;
        if (isset($headerColSpan)) {
            $this ->headerColSpan = $headerColSpan;
        }
        return $this;
    }

    /**
     * Nastavi text hlavicky - nejdrive jej prelozi
     * @param string $headerText
     * @param int $headerColSpan    Nepovinne lze zadat i hodnotu pro colspan
     * @return BrowserField
     */
    public function setHeaderText($headerText, $headerColSpan = null)
    {
        $this->header = TextManager::_($headerText);
        if (isset($headerColSpan)) {
            $this ->headerColSpan = $headerColSpan;
        }
        return $this;
    }

    /**
     * @param mixed $headerColSpan
     * @return BrowserField
     */
    public function setHeaderColSpan($headerColSpan)
    {
        $this->headerColSpan = $headerColSpan;
        return $this;
    }

    /**
     * @param string $headerClass
     * @return BrowserField
     */
    public function setHeaderClass($headerClass)
    {
        $this->headerClass = $headerClass;
        return $this;
    }


    /**
     * @param \string[] $headerAttributes
     * @return $this
     */
    public function setHeaderAttributes($headerAttributes)
    {
        $this->headerAttributes = $headerAttributes;
        return $this;
    }


    /**
     * @param string $name
     * @param string $value
     * @return $this
     */
    public function addHeaderAttribute($name, $value)
    {
        $this->headerAttributes[$name] = $value;
        return $this;
    }




    /**
     * @param mixed $footer
     * @return BrowserField
     */
    public function setFooter($footer)
    {
        $this->footer = $footer;
        return $this;
    }

    /**
     * Nastavi text paticky - nejdrive jej prelozi
     * @param string $footerText
     * @return BrowserField
     */
    public function setFooterText($footerText)
    {
        $this->footer = TextManager::_($footerText);
        return $this;
    }


    /**
     * @param int $footerColSpan
     * @return BrowserField
     */
    public function setFooterColSpan($footerColSpan)
    {
        $this->footerColSpan = $footerColSpan;
        return $this;
    }


    /**
     * @param boolean $showSum
     * @return BrowserField
     */
    public function setShowSum($showSum)
    {
        $this->showSum = $showSum;
        return $this;
    }


    /**
     * @param string $format
     * @return BrowserField
     */
    public function setFormat($format)
    {
        $this->format = $format;
        return $this;
    }

    /**
     * @param string $text
     * @return BrowserField
     */
    public function setText($text)
    {
        $this->text = $text;
        return $this;
    }

    /**
     * @param string $link
     * @param string[] $linkAttributes
     * @return BrowserField
     */
    public function setLink($link, $linkAttributes = null)
    {
        $this->link = $link;
        if (isset($linkAttributes)) {
            $this ->linkAttributes = $linkAttributes;
        }
        return $this;
    }


    /**
     * @param string[] $linkAttributes
     * @return BrowserField
     */
    public function setLinkAttributes($linkAttributes)
    {
        $this->linkAttributes = $linkAttributes;
        return $this;
    }


    /**
     * @param string|null $csrfPrefix
     * @return BrowserField
     */
    public function setCsrfPrefix(?string $csrfPrefix): BrowserField
    {
        $this->csrfPrefix = $csrfPrefix;
        return $this;
    }

    /**
     * Nastavi index pole pro generovani CSRF klice a pripadne prefix
     * @param int|string|null $csrfKeyIndex
     * @param ?string $csrfPrefix
     * @return BrowserField
     */
    public function setCsrfKeyIndex($csrfKeyIndex, $csrfPrefix = null)
    {
        $this->csrfKeyIndex = $csrfKeyIndex;
        if (isset($csrfPrefix)) {
            $this ->csrfPrefix = $csrfPrefix;
        }
        return $this;
    }

    /**
     * @param string $align
     * @return BrowserField
     */
    public function setAlign($align)
    {
        $this->align = $align;
        return $this;
    }

    /**
     * @param boolean $ordering
     * @return BrowserField
     */
    public function setOrdering($ordering)
    {
        $this->ordering = $ordering;
        return $this;
    }

    /**
     * @param boolean $wrap
     * @return BrowserField
     */
    public function setWrap($wrap)
    {
        $this->wrap = $wrap;
        return $this;
    }

    /**
     * Nastavi texty, ktere se zobrazi pro jednotlive hodnoty. Pokud je zadan retezec "yesno", nastavi se retezce Ne/Ano
     * @param array|string $options
     * @return BrowserField
     */
    public function setOptions($options)
    {
        if ($options === "yesno") {
            $this ->options = [0 => TextManager::_('Str_No'), 1 => TextManager::_('Str_Yes')];
        } else {
            $this->options = $options;
        }

        return $this;
    }

    /**
     * @param mixed $style
     * @return BrowserField
     */
    public function setStyle($style)
    {
        $this->style = $style;
        return $this;
    }

    /**
     * @param mixed $className
     * @return BrowserField
     */
    public function setClassName($className)
    {
        $this->className = $className;
        return $this;
    }

    /**
     * @param mixed $rowSpan
     * @return BrowserField
     */
    public function setRowSpan($rowSpan)
    {
        $this->rowSpan = $rowSpan;
        return $this;
    }

    /**
     * @param boolean $htmlConvert
     * @return BrowserField
     */
    public function setHtmlConvert($htmlConvert)
    {
        $this->htmlConvert = $htmlConvert;
        return $this;
    }


    /**
     * @param mixed $filterValue
     * @return BrowserField
     */
    public function setFilterValue($filterValue)
    {
        $this->filterValue = $filterValue;
        return $this;
    }



    /**
     * Nacte nastaveni jedne vlastnosti (atributu) z konfigurace
     * @param $key
     * @param $value
     * @return BrowserField
     */
    function loadConfigurationItem($key, $value)
    {
        switch ($key) {
            case 'align' :
            case 'sqlFieldName' :
            case 'sortSQL' :
            case 'sortSQLDesc' :
            case 'format' :
            case 'link' :
            case 'linkAttributes' :
            case 'style' :
            case 'className' :
            case 'filterValue' :
            case 'translationPrefix' :
            case 'titleTranslationPrefix' :
            case 'sqlFieldIndex2Separator' :
            case 'type' :
            case 'code' :  // aby nova pole mohla mid kod - napr. kvuli moznosti razeni
                $this ->{$key} = $value;
                break;

            case 'class' :
                $this ->className = $value;
                break;

            case 'addClass' :
                $this ->className .= ($this ->className ? ' ' : '') . $value;
                break;

            case 'header' :
            case 'footer' :
            case 'text' :
            case 'titlePrefix' :
                $this ->{$key} = TextManager::_($value);
                break;

            case 'visible' :
            case 'wrap' :
            case 'ordering' :
            case 'sqlFieldIndex' :
            case 'headerColSpan' :
            case 'headerClass' :
            case 'footerColSpan' :
            case 'htmlConvert' :
            case 'translateText' :
            case 'excludeFromGroupBy' :
            case 'showSum' :
                $this ->{$key} = (int)$value;
                break;

            case 'sqlFieldIndex2' :
            case 'titleFieldIndex' :
            case 'translationValueIndex' :
                $this ->{$key} = ($value !== '')
                    ? ( ($value[0] >= '0'  &&  $value[0] <= '9') ? (int)$value : $value ) // muze byt cislo i nazev pole
                    : null;
                break;
        }

        return $this;
    }


    /**
     * Zpracuje konfiguraci sloupce nactenou z XML pomoci @XmlLoader
     * @param \ArrayAccess $confArray
     * @return BrowserField
     */
    function loadConfigurationArray($confArray)
    {
        if (!$confArray instanceof \ArrayAccess) {
            return $this;
        }

        foreach ($confArray as $key => $value) {
            if ($value instanceof \ArrayAccess) {
                // hodnoty definovane pomoci vnorenych tagu spoji do jednoho textu
                $newVal = '';
                foreach ($value as $valA) {
                    if (isset($valA['#text'])) {
                        $newVal .= ($newVal ? "\n" : '') . $valA['#text'];
                    }
                }

                $value = $newVal;
            }

            $this ->loadConfigurationItem($key, $value);
        }

        return $this;
    }

    /**
     * @return string
     * @deprecated
     */
    public function getId()
    {
        return $this ->sqlFieldName;
    }


    /**
     * Vraci true, pokud se jedna o ciselna data
     * @return bool
     */
    public function isNumberFormat()
    {
        switch ($this ->format) {
            case 'int' :
            case 'kc' :
            case 'curr' :
            case 'float' :
            case 'number0' :
            case 'number1' :
            case 'number_1' :
            case 'number2' :
            case 'number_2' :
            case 'number3' :
            case 'number_3' :
            case 'hodiny' :
            case 'minuty' :
            case 'sekundy' :
                return true;
        }

        return false;
    }


    function getFieldClass()
    {
        $class = $this ->className ?: '';

        if ($this ->align) {
            $class .= ($class ? ' ' : '') . 'align-' . $this ->align;
        }

        if (!$this ->wrap) {
            $class .= ($class ? ' ' : '') . 'no-wrap';
        }

        return $class;
    }


    /**
     * Vraci class pro bunku hlavicky
     * @param int $sort     Priznak, ze je seznam podle tohoto pole serazen
     * @return string
     */
    function getHeaderClass($sort = 0)
    {
        $class = $this->headerClass ?? $this->getFieldClass();

        if ($sort > 0) {
            $class .= ($class ? ' ' : '') . 'sorted';
        } else if ($sort < 0) {
            $class .= ($class ? ' ' : '') . 'sorteddesc';
        }

        return $class;
    }


    /**
     * @param $value
     * @param $rowNumber
     * @param $recordId
     * @param $rowData
     * @return string
     */
    function getDataCellClass($value, $rowNumber, $recordId, $rowData)
    {
        $result = $this ->getFieldClass();

        if ($this ->isNumberFormat()  &&  $value < 0) {
            $result .= ($result ? ' ' : '') . 'minus';
        }

        return $result;
    }


    /**
     * @param int $fieldIndex
     * @param int $sortField
     * @param string $url
     * @return string
     */
    function getHeaderText($fieldIndex, $sortField, $url = null)
    {
        return $this ->header;
    }


    /**
     * @return string[]
     */
    function getHeaderAttributes()
    {
        return $this ->headerAttributes;
    }

    /**
     * @param int $fieldIndex
     * @param int $sortField
     * @param string $url
     * @return string
     */
    function getHeader($fieldIndex, $sortField, $url = null)
    {
        $sort = 0;
        if ($sortField == $fieldIndex + 1) {
            $sort = 1;
        } else if ($sortField == -$fieldIndex - 1) {
            $sort = -1;
        }

        $class = $this ->getHeaderClass($sort);
        $classStr = $class ? " class='{$class}' " : '';

        switch ($this ->type) {
            case self::FIELD_TYPE_HIDDEN :
            case self::FIELD_TYPE_COMMENT :
                return '';

            case self::FIELD_TYPE_LINE :
                return "<th $classStr" . ($this ->code ? " data-code='{$this ->code}'" : '') . "></th>";
        }

        $attrs = array_merge(
            $this ->code ? ['data-code' => $this ->code] : [],
            ((int)$this->headerColSpan > 1) ? ['colspan' => $this ->headerColSpan] : [],
            $class ? ['class' => $class] : [],
            $this ->translationPrefix ? ['data-translation-prefix' => $this ->translationPrefix] : [],
            $this ->getHeaderAttributes() ?: []);

//        if ($this ->code) {
//            $result .= " data-code='{$this ->code}'";
//        }
//
//        if ((int)$this->headerColSpan > 1) {
//            $result .= ' colspan="' . $this ->headerColSpan . '"';
//        }
        $result =
            '<th ' .
            GuiUtils::getAttributesHtml($attrs) .
            '>';

        if ($this ->ordering  &&  $url  &&  $this ->code) {
            if ($this ->sqlFieldName || $this ->sortSQL) {
                $result .= '<a href="' . StringUtils::concatenateHtmlParams($url,
                        'sort=' . ($sort === 1 ? '-' : '') . $this ->code) .
                        //'sort=' . ($fieldIndex + 1 == abs($sortField) ? -$sortField : ($fieldIndex + 1))) .
                    '">';
            }

            $result .= $this ->getHeaderText($fieldIndex, $sortField, $url);

            if ($this ->sqlFieldName || $this ->sortSQL) {
                $result .= '</a>';
            }
        } else {
            $result .= $this ->getHeaderText($fieldIndex, $sortField, $url);
        }

        if ($sort > 0) {
            $result .= '<i class="inline-glyph fa glyph-sort-up"></i>';
        } else if ($sort < 0) {
            $result .= '<i class="inline-glyph fa glyph-sort-down"></i>';
        }

        $result .= '</th>';

        return $result;
    }


    /**
     * @param int $fieldIndex
     * @param string $url
     * @return string
     */
    function getFooterText($fieldIndex, $url = null)
    {
        if (isset($this ->footer)) {
            return $this ->footer;
        }

        if ($this ->showSum) {
            return $this ->format
                ? DBUtils::getFormatedValue($this ->database, $this ->format, $this ->sum)
                : $this ->sum;
        }

        return '';
    }


    function getFooterClass()
    {
        return
            'browserfooter ' .
            $this ->getFieldClass() .
            ( ($this ->showSum  &&  $this ->sum < 0) ? ' minus' : '');
    }


    /**
     * @param int $fieldIndex
     * @param null $url
     * @return string
     */
    function getFooter($fieldIndex, $url = null)
    {
        if ($this ->type === self::FIELD_TYPE_HIDDEN  ||  $this ->type === self::FIELD_TYPE_COMMENT) {
            return '';
        }

        $result =
            '<td class="' . $this ->getFooterClass() . '"' .
            ( ((int)$this ->footerColSpan > 1)
                ? ' colspan="' . (int)$this ->footerColSpan . '"'
                : ''
            ) .
            '>';

        return $result . $this ->getFooterText($fieldIndex, $url) . '</td>';
    }


    /**
     * Vraci URL odkaz pro zadany zaznam.
     * @param mixed $recordId           Id zaznamu - hodnota primarniho pole z browseru (pokud je v browseru nastaveno)
     * @param BrowserRowArray $rowData  Hodnoty pro aktualni radek
     * @return string
     */
    function getLinkUrl($recordId, $rowData)
    {
        if ($this ->link) {
            $recordStr = substr($this ->link, -1) == '='
                ? ''
                : '&record=';

            $link = $this ->link;

            if ($this ->csrfKeyIndex !== null) {
                $csrfKey = Csrf::getCsrfKey(($this ->csrfPrefix ?? '') . $rowData[$this ->csrfKeyIndex]);
                $link = str_replace('%CSRF%', $csrfKey, $link);
            }

            return
                $link .
                ( (($recordId ?? '') != '') ? $recordStr . $recordId : '');
        }

        return '';
    }

    /**
     * @param string $text
     * @param mixed $recordId
     * @param BrowserRowArray $rowData
     * @return string
     */
    function getLink($text, $recordId, $rowData)
    {
        $result = '';
        $url = $this ->getLinkUrl($recordId, $rowData);

        if ($url) {
            $attributesStr = GuiUtils::getAttributesHtml(array_merge(['href' => $url], $this ->linkAttributes ?: []));

            $result .=
                '<a ' . $attributesStr . '>';
        }

        $result .= $text;

        if ($url) {
            $result .= '</a>';
        }

        return $result;
    }


    /**
     * Funkce urcuje, jestli ma byt pole pro zadanou hodnotu zobrazeno
     * @param $value
     * @param $rowData
     * @return bool
     */
    function isValueVisible($value, $rowData)
    {
        if (!isset($this ->filterValue)) {
            return true;
        }

        if (is_callable($this ->filterValue)) {
            $func = $this ->filterValue;
            return $func($value, $rowData);
        }

        if ($this ->filterValue === true) {
            return (bool)$value;
        }

        if ($this ->filterValue === false) {
            return !(bool)$value;
        }

        return $value == $this ->filterValue;
    }

    /**
     * @param mixed $value
     * @param int|null $rowNumber
     * @param string $recordId
     * @param BrowserRowArray $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$this ->isValueVisible($value, $rowData)) {
            return '';
        }

        $text2 = '';

        if (isset($this ->sqlFieldIndex2)  &&  $this ->sqlFieldIndex2 !== ''  ) {
            // sqlFieldIndex2 muze byt primo index (integer nebo string), nebo nazev pole, nebo @<nazev pole> (pak vola getViewText prislusneho sloupce)
            $value2 = $rowData[$this ->sqlFieldIndex2];
            $text2 = (isset($value2) && $value2 !== '')
                ? $this ->sqlFieldIndex2Separator . StringUtils::htmlTextConvert(
                    $this->translateText ? TextManager::_($rowData[$this->sqlFieldIndex2]) : $rowData[$this->sqlFieldIndex2]) .
                    ($this ->sqlFieldIndex2Separator && $this ->sqlFieldIndex2Separator[strlen($this ->sqlFieldIndex2Separator) - 1] === '(' ? ')' : '')
                : '';
        }

        if ($this ->showSum  &&  isset($value)) {
            $this ->sum += $value;
        }

        $translationIndex = $this->translationValueIndex ?? $this->sqlFieldIndex;
        if ($translationIndex < 0  &&  $rowData) {
            $translationIndex = count($rowData) + $translationIndex;
        }
        if ($this ->translationPrefix && isset($rowData[$translationIndex])  &&  $rowData[$translationIndex] !== '') {
            $value = TextManager::_($this ->translationPrefix . $rowData[$translationIndex], $value);
//            if (isset($translation)) {
//                $value = $translation;
//            }
        } else if ($this ->translateText  &&  $value) {
            $value = TextManager::_($value);
        }

        switch ($this ->type) {
            case self::FIELD_TYPE_LINE :
                return ($rowNumber > 0)
                    ? $rowNumber
                    : '';

            case self::FIELD_TYPE_DEFAULT :
            case self::FIELD_TYPE_HIDDEN :
            case self::FIELD_TYPE_COMMENT :
                if (!isset($value)  &&  !$text2) {
                    return '';
                }
                return $this ->getLink(
                    (($this ->htmlConvert ||  $this ->format) ? DBUtils::getFormatedValue($this ->database, $this ->format, $value) : $value) . $text2,
                    $recordId,
                    $rowData);

            case self::FIELD_TYPE_EMAIL :
                return $value
                    ? "<a href='mailto:$value'>$value</a>"
                    : '';

            case self::FIELD_TYPE_TEXT :
                return $this ->getLink($this ->text . $text2, $recordId, $rowData);

            case self::FIELD_TYPE_URL :
                if (!$value) {
                    return '';
                }
                return
                    '<a href="' . $value . '" ' . GuiUtils::getAttributesHtml($this ->linkAttributes) . '>' .
                    ($this ->text
                        ? ($this ->text === 'IMG'
                            ? '<i class="inline-glyph fa glyph-link"></i>'
                            : $this ->text
                        )
                        : $value
                    ) .
                    '</a>' .
                    $text2;

            case self::FIELD_TYPE_SELECT :
                return isset($value)
                    ? $this ->getLink($this ->options[$value] . $text2, $recordId, $rowData)
                    : '';                
        }

        return $text2;
    }


    function getHtmlTitle($rowNumber, $recordId, $rowData)
    {
        if (isset($this ->titleFieldIndex)) {
            $title = $rowData[$this ->titleFieldIndex];

            if ($title  &&  $this ->titlePrefix) {
                $title = $this ->titlePrefix . $title;
            }
        } else {
            $title = null;
        }

        if ($this ->titleTranslationPrefix) {
            $translationIndex = isset($this->translationValueIndex) ? $this->translationValueIndex : $this->titleFieldIndex;

            if (isset($rowData[$translationIndex])  &&  $rowData[$translationIndex] !== '') {
                return TextManager::_($this ->translationPrefix . $rowData[$translationIndex], $title);
            }
        }

        return $title;
    }


    function getAttribute($name, $rowData, $index)
    {
        return $rowData && isset($rowData[$index])
            ? $rowData[$index]
            : null;
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param mixed $recordId
     * @param BrowserRowArray $rowData
     * @return Attributes
     */
    function getAttributes($value, $rowNumber, $recordId, $rowData)
    {
        $attrs = new Attributes();

        $class = $this ->getDataCellClass($value, $rowNumber, $recordId, $rowData);
        if ($class) {
            $attrs['class'] = $class;
        }

        $title = $this ->getHtmlTitle($rowNumber, $recordId, $rowData);
        if ($title) {
            $attrs['title'] = $title;
        }

        if ($this ->style) {
            $attrs['style'] = $this ->style;
        }

        if ($this ->rowSpan  &&  (int)$this ->rowSpan > 1) {
            $attrs['rowspan'] = (int)$this ->rowSpan;
        }

        if ($this ->attributeIndexes) {
            foreach ($this ->attributeIndexes as $attrName => $index) {
                $value = $this ->getAttribute($attrName, $rowData, $index);

                if (isset($value)) {
                    $attrs[$attrName] = $value;
                }
            }
        }

        return $attrs;
    }


//    /**
//     * Vraci text pro zobrazeni
//     * @param mixed $value
//     * @param int $rowNumber
//     * @param string $recordId
//     * @param BrowserRowArray $rowData
//     * @return string
//     */
//    function getFieldCellText($value, $rowNumber, $recordId, $rowData)
//    {
//        $class = $this ->getDataCellClass($value, $rowNumber, $recordId, $rowData);
//        $classStr = $class ? " class='{$class}' " : '';
//
//        $attrsStr = $this ->getAttributes($rowNumber, $recordId, $rowData);
//
//        switch ($this ->type) {
//            case self::FIELD_TYPE_HIDDEN :
//            case self::FIELD_TYPE_COMMENT :
//                return '';
//
//            case self::FIELD_TYPE_LINE :
//                if ($rowNumber > 0) {
//                    return "<td $classStr $attrsStr>$rowNumber</td>";
//                }
//                return "<td $classStr $attrsStr></td>";
//
//
//            case self::FIELD_TYPE_DEFAULT :
//            case self::FIELD_TYPE_TEXT :
//            case self::FIELD_TYPE_SELECT :
//            default :
//                return
//                    '<td' . $classStr . $attrsStr .
//                    ($this ->style ? ' style="' . $this ->style . '"' : '') .
//                    (($this ->rowSpan  &&  (int)$this ->rowSpan > 1) ? ' rowspan="' . (int)$this ->rowSpan . '"' : '') .
//                    '>' .
//                    $this ->getViewText($value, $rowNumber, $recordId, $rowData) .
//                    '</td>';
//        }
//    }

}
