<?php
/**
 * Trida Dialog pro zobrazeni modalniho dialogu.
 * stanislav.elbl, 08.01.2018
 */

namespace Ron\Controls;


use Ron\GuiUtils;
use Ron\MVC\ViewObject;
use Ron\TextManager;

class DialogElement
{
    /** @var string     HTML tag */
    public $tag;

    /** @var string     Id elementu v HTML */
    public $id;
    /** @var string */
    public $class;

    /** @var string[]   Pole dalsich html parametru  */
    public $htmlParams = [];


    /**
     * DialogElement constructor.
     */
    public function __construct()
    {
    }


    /**
     * @return string
     */
    public function getTag()
    {
        return $this->tag;
    }

    /**
     * @param string $tag
     * @return $this
     */
    public function setTag($tag)
    {
        $this->tag = $tag;
        return $this;
    }


    /**
     * @return string
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param string $id
     * @return $this
     */
    public function setId($id)
    {
        $this->id = $id;
        return $this;
    }


    /**
     * @return string
     */
    public function getClass()
    {
        return $this->class;
    }

    /**
     * @param string $class
     * @return $this
     */
    public function setClass($class)
    {
        $this->class = $class;
        return $this;
    }


    /**
     * @param string $class
     * @return $this
     */
    public function addClass($addClass)
    {
        $this->class .=
            ($this ->class ? ' ' : '') .
            $addClass;
        return $this;
    }


    public function setHtmlParam($name, $value)
    {
        $this ->htmlParams[$name] = $value;
        return $this;
    }


    /**
     * Vraci zakladni HTML parametry (podle vlastnosti id, class a pripadne dalsich)
     * @return string[]
     */
    protected function getBasicRenderParams()
    {
        $renderParams = [];

        if ($this ->class) {
            $renderParams['class'] = $this ->class;
        }
        if ($this ->id) {
            $renderParams['id'] = $this ->id;
        }

        return $renderParams;
    }

    public function getRenderParams()
    {
        return array_merge(
            $this ->getBasicRenderParams(),
            $this ->htmlParams ?: []
        );
//        $renderParams = $this ->getBasicRenderParams();
//
//        foreach ($this ->htmlParams as $paramName => $paramValue) {
//            $renderParams[$paramName] = $paramValue;
//        }
//
//        return $renderParams;
    }


    /**
     * Vraci retezec parametru pro vygenerovani do HTML
     * @param $renderParams
     * @return string
     */
    public function getParamsHtml($renderParams)
    {
        return GuiUtils::getAttributesHtml($renderParams);
//        $result = '';
//        foreach ($renderParams as $name => $value) {
//            $result .= ' ' . $name . '="' . StringUtils::htmlTextConvert($value) . '"';
//        }
//
//        return $result;
    }
}


class DialogButton extends DialogElement
{
    /** @var string     Nazev elementu (parametr name) v HTML */
    public $name;
    /** @var string     Parametr value v HTML (text v buttonu) */
    public $value;
    /** @var string     typ inputu v HTML - obvykle submit nebo button */
    public $type = 'submit';


    /**
     * DialogButton constructor.
     */
    public function __construct()
    {
        parent::__construct();
        $this->tag = 'input';
        $this->class = 'submitbutton';
    }

    /**
     * Vraci button "Storno"
     * @param string $id
     * @param string $value
     * @return $this
     */
    public static function cancelButton($id = 'btn_cancel', $value = null)
    {
        return
            (new DialogButton())
                ->setId($id)
                ->setType('button')
                ->setClass('button cancel close-dialog')
                ->setValue($value ?: 'Cancel');
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $name
     * @return $this
     */
    public function setName($name)
    {
        $this->name = $name;
        return $this;
    }

    /**
     * @return string
     */
    public function getValue()
    {
        return $this->value;
    }

    /**
     * @param string $value
     * @return $this
     */
    public function setValue($value)
    {
        $this->value = TextManager::_($value);
        return $this;
    }


    /**
     * @return string
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @param string $type
     * @return $this
     */
    public function setType($type)
    {
        $this->type = $type;
        return $this;
    }

    protected function getBasicRenderParams()
    {
        $renderParams = parent::getBasicRenderParams();
        if ($this ->name) $renderParams['name'] = $this ->name;
        if ($this ->type) $renderParams['type'] = $this ->type;
        if (isset($this ->value)) $renderParams['value'] = $this ->value;

        return $renderParams;
    }


    public function getHtml()
    {
        $renderParams = $this ->getRenderParams();

        return "<{$this ->tag} " . $this ->getParamsHtml($renderParams) . "/>\n";
    }
}


class Dialog extends DialogElement implements ViewObject
{
    /** @var string */
    protected $title;
    /** @var string|ViewObject  View pro vygenerovani obsahu formulare */
    protected $view;
    /** @var string     Text hlasky o nacitani dat - pokud je nastaven, vytvori se */
    protected $waitMessage;
    /** @var string     HTML obsah dialogu */
    protected $htmlContent = null;

    /** @var bool|string|DialogButton[] */
    protected $buttons = [];

    public $showCloseButton;

    /** @var array      Asociativni pole parametru - slouzi pro predani jakychkoliv parametru */
    public $parameters = [];

    public $dialogClass = 'modal-dialog';

    /**
     * @var string|bool|string[]   Urcuje, jestli se zobrazi formular. Muze obsahovat:
     *  - true/false (zobrazi se defaultne nebo se vubec nezobrazi)
     *  - string - v tom pripade je to HTML oteviraciho tagu formulare a na konci se vlozi </form>
     *  - string[] - v tom pripade jsou to atributy HTML tagu form
     */
    public $formTag = false;


    /**
     * Dialog constructor.
     */
    public function __construct()
    {
        parent::__construct();
        $this ->tag = ($GLOBALS['TEST_NEW_DIALOG'] ?? false) ? 'dialog' : 'div';
        $this ->class = ($GLOBALS['test-bootstrap'] ?? false)
            ? 'modal'  // fade
            : (($GLOBALS['TEST_NEW_DIALOG'] ?? false) ? '' : 'rondialog');
    }

    /**
     * @return mixed
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @param mixed $title
     * @return $this
     */
    public function setTitle($title)
    {
        $this->title = TextManager::_($title);
        return $this;
    }

    /**
     * @return string
     */
    public function getView()
    {
        return $this->view;
    }

    /**
     * @param string|ViewObject $view
     * @return $this
     */
    public function setView($view)
    {
        $this->view = $view;
        return $this;
    }


    /**
     * @return string|null
     */
    public function getHtmlContent()
    {
        return $this->htmlContent;
    }

    /**
     * @param string|null $htmlContent
     * @return Dialog
     */
    public function setHtmlContent($htmlContent): Dialog
    {
        $this->htmlContent = $htmlContent;
        return $this;
    }




    /**
     * @return string
     */
    public function getDialogClass(): string
    {
        return $this->dialogClass;
    }

    /**
     * @param string $dialogClass
     * @return Dialog
     */
    public function setDialogClass(string $dialogClass): Dialog
    {
        $this->dialogClass = $dialogClass;
        return $this;
    }

    /**
     * @param string $dialogClass
     * @return Dialog
     */
    public function addDialogClass(string $dialogClass): Dialog
    {
        $this->dialogClass .=
            ($this ->dialogClass  &&  $dialogClass ? ' ' : '') .
            $dialogClass;
        return $this;
    }





    /**
     * @param bool|string|string[] $formTag
     * @return Dialog
     */
    public function setFormTag($formTag)
    {
        $this->formTag = $formTag;
        return $this;
    }


    /**
     * @param string $name
     * @param mixed $value
     * @return $this
     */
    public function addParameter($name, $value)
    {
        $this ->parameters[$name] = $value;
        return $this;
    }


    public function getParameter($name)
    {
        return $this ->parameters[$name] ?? null;
    }


    /**
     * @return string
     */
    public function getWaitMessage()
    {
        return $this->waitMessage;
    }

    /**
     * Nastavi text hlasky informujici napriklad o nacitani dat
     * @param string|bool $waitMessage
     * @return $this
     */
    public function setWaitMessage($waitMessage)
    {
        if ($waitMessage === true) {
            $this ->waitMessage = TextManager::_('Probiha_nacitani');
        } else {
            $this ->waitMessage = $waitMessage;
        }

        return $this;
    }


    /**
     * @param DialogButton $button
     * @return $this
     */
    public function addButton(DialogButton $button)
    {
        $this ->buttons[] = $button;
        return $this;
    }


    protected function getBasicRenderParams()
    {
        $renderParams = parent::getBasicRenderParams();

        if ($GLOBALS['TEST_NEW_DIALOG'] ?? false) {
            $renderParams['role'] = 'dialog';
        } else if ($GLOBALS['test-bootstrap'] ?? false) {
            $renderParams['tabindex'] = '-1';
            $renderParams['aria-hidden'] = 'true';
        } else if ($this ->title) {
            $renderParams['title'] = $this ->title;

            $renderParams['class'] = ($renderParams['class'] ?? '') . ' ' . $this ->dialogClass;
        }

        return $renderParams;
    }


    public function renderContent()
    {
        if ($this ->htmlContent) {
            echo $this ->htmlContent;
            return;
        }

        if (!$this ->view) {
            return;
        }

        if ($this ->view instanceof ViewObject) {
            $this ->view ->render();
        } else {
            require($this ->view);
        }
    }


    public function renderButtons()
    {
        if (is_array($this ->buttons)) {
            foreach ($this->buttons as $button) {
                echo $button->getHtml();
            }
        } else if (is_string($this ->buttons)) {
            echo $this ->buttons;
        }
    }


    public function renderLoadingMessage()
    {
        if (!isset($this ->waitMessage)) {
            return;
        }
        echo '<div class="dialogwaitmessage" style="display: none">' . $this ->waitMessage . '</div>';
    }


    protected function getFormTagHtml()
    {
        if ($this ->formTag === true) {
            return '<form method="POST">';
        }

        if (is_string($this ->formTag)) {
            return $this ->formTag;
        }

        if ($this ->formTag) {
            return
                '<form ' .
                GuiUtils::getAttributesHtml($this->formTag) .
                '>';
        }

        return '';
    }

    public function renderHead()
    {
        $renderParams = $this->getRenderParams();

        if ($GLOBALS['TEST_NEW_DIALOG'] ?? false) {
            echo
                "<{$this ->tag} " . $this->getParamsHtml($renderParams) . ">\n" .  // <dialog>
                $this ->getFormTagHtml() .
                '   <div class="dialog-header">' .
                ($this ->title ?? '') .
                "</div>\n";

            echo
                '   <div class="dialog-body">' .
                "\n";
        } else if ($GLOBALS['test-bootstrap'] ?? false) {
            echo
                "<{$this ->tag} " . $this->getParamsHtml($renderParams) . ">\n" .  // div modal
                $this ->getFormTagHtml() .
                ' <div class="' . $this ->dialogClass . '">' . "\n" .
                '  <div class="modal-content">' . "\n" .
                '   <div class="modal-header">';

            echo '<h5 class="modal-title">' . ($this ->title ?? '') . '</h5>';    // id="exampleModalLabel"

            if ($this ->showCloseButton) {
                echo '<button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button>';
            }

            echo "</div>\n"; // modal-header

            echo
                '   <div class="modal-body">' .
                "\n";
        } else {
            echo
                "<{$this ->tag} " . $this->getParamsHtml($renderParams) . ">\n" .
                $this ->getFormTagHtml() .
                '<div class="content">';

            $this->renderLoadingMessage();
        }
    }

    public function renderTail()
    {
        if ($GLOBALS['TEST_NEW_DIALOG'] ?? false) {
            echo
                "   </div>\n"; // dialog-body

            if ($this ->buttons) {
                echo '   <div class="dialog-footer">';
                $this ->renderButtons();
                echo "</div>\n";
            }

            echo
                ($this ->formTag ? "</form>\n" : '') .
                "</{$this ->tag}>\n"; // modal

        } else if ($GLOBALS['test-bootstrap'] ?? false) {
            echo
                "   </div>\n"; // modal-body

            if ($this ->buttons) {
                echo '   <div class="modal-footer">';
                $this ->renderButtons();
                echo "</div>\n";
            }

            echo
                "  </div>\n" . // modal-content
                " </div>\n" . // modal-dialog
                ($this ->formTag ? "</form>\n" : '') .
                "</{$this ->tag}>\n"; // modal
        } else {
            echo '</div>';
            if ($this ->buttons) {
                echo '<div class="buttons">';
                $this ->renderButtons();
                echo '</div>';
            }
            echo
                ($this ->formTag ? "</form>\n" : '') .
                "</{$this ->tag}>\n";
        }
    }

    public function render()
    {
        $this ->renderHead();
        $this ->renderContent();
        $this ->renderTail();
    }
}

