<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\DB;

/**
 * Kurzor obsahuje data nactena z databaze. Umoznuje sekvencni prochazeni dat
 * a u nekterych databazi muze implementovat i "preskoceni" na libovolny nacteny zaznam.
 */
abstract class Cursor
{
    /**
     * @var Database
     */
    protected $database;

    /**
     * Cursor constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;
    }


    /**
     * @return Database
     */
    public function getDatabase() {
        return $this ->database;
    }

    /**
     * @param Database $database
     */
    public function setDatabase($database)
    {
        $this->database = $database;
    }


    /**
     * Metoda close by se mela volat vzdy po ukonceni prace s kurzorem.
     * Zajistuje napriklad uvolneni pameti a podobne.
     */
    public function close() {

    }

    /**
     * Nacte prvni/dalsi zaznam. Vraci jej jako pole hodnot. Pokud uz zadny dalsi zaznam neni, vraci false
     * @return array|bool
     */
    abstract public function fetch();


    /**
     * Prevede data nactena z databaze (napriklad string) na timestamp (unix timestamp)
     * @param mixed $str
     * @return int|null
     */
    public function stringToTimestamp($str)
    {
        return $this ->database
            ? $this ->database ->stringToTimestamp($str)
            : $str;
    }


    /**
     * Prevede data nactena z databaze na denni cas (pocet hodin)
     * @param $str
     * @return float
     */
    public function stringToTimeOfDay($str)
    {
        return $this ->database
            ? $this ->database ->stringToTimeOfDay($str)
            : $str;
    }


    /**
     * Zpracuje hodnotu nactenou z BLOB pole
     * @param $value
     * @return mixed
     */
    public function stringToBinary($value)
    {
        return $this ->database
            ? $this ->database ->stringToBinary($value)
            : $value;
    }


    /**
     * Vraci pocet nactenych zaznamu (Neni implementovano na vsech typech DB!)
     * @return int|false
     * @throws \Exception
     */
    public function numRows()
    {
        return false;
    }


    /**
     * Nastavi kurzor na zadany radek (tj. na zaznam se zadanym indexem). Cislovani radku je od 0.
     * @param int $rowNumber
     * @return bool
     * @throws \Exception
     */
    public function seek(/** @noinspection PhpUnusedParameterInspection */ $rowNumber)
    {
        throw new \Exception('Chyba! Metoda seek neni implementovana!');
    }

    /**
     * Nastavi kurzor na posledni nacteny zaznam, takze dalsi volani fetch() vrati tento posledni zaznam.
     * @return bool
     * @throws \Exception
     */
    public function last()
    {
        throw new \Exception('Chyba! Metoda last neni implementovana!');
    }
}
