<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\DB;

use Ron\RonException;

/**
 * Databazova vyjimka. Obsahuje navic chybove hlaseni z databaze a SQL dotaz, ve kterem doslo k chybe.
 * POZOR! tyto texty by se nemely zobrazovat beznym uzivatelum!
 * Class DBException
 */
class DBException extends RonException
{
    /** @var Database */
    public $database;

    /** @var null|string    SQL vyraz, ve kterem doslo k chybe - z bezpecnostnich duvodu nezobrazovat beznemu uzivateli! */
    public $sql;
    /** @var null|string    Databazova chyba - z bezpecnostnich duvodu nezobrazovat beznemu uzivateli! */
    public $dbMessage;

    /**
     * @param Database|DBException  $database
     * @param string    $sql
     * @param string    $message
     * @param string    $dbMessage
     * @param \Exception $previous
     */
    public function __construct($database, $sql = null, $message = null, $previous = null, $dbMessage = null)
    {
        $this ->database = ($database instanceof Database)
            ? $database
            : null;

        if ($database instanceof DBException) {
            parent::__construct(
                $message ?: $database ->getMessage(),
                $database ->fieldName,
                $database ->errorDescription,  // $this ->sql - odstraneno kvuli bezpecnosti!
                $database ->code, $previous ?: $database);

            $this ->database = $database ->database;

            $this ->dbMessage = $dbMessage ?: $database ->dbMessage;
            $this ->sql = $sql ?: $database ->sql;
        } else {
            parent::__construct(
                ($message ?: 'V aplikaci doslo k chybe!'),  // . $this ->dbMessage - odstraneno kvuli bezpecnosti!
                null,
                null,  // $this ->sql - odstraneno kvuli bezpecnosti!
                0, $previous);

            $this->sql = $sql;
            $this->dbMessage = $dbMessage ?: ($database ? $database->lastErrorText() : null);
        }
    }

    public function isPrimaryKeyViolation()
    {
        return
            strpos(strtolower($this ->dbMessage ?? ''), 'violation of primary') !== false
            || strpos($this ->dbMessage ?? '', 'ORA-00001') !== false;
    }


    public function isForeignKeyViolation()
    {
        return
            strpos(strtolower($this ->dbMessage ?? ''), 'reference constraint') !== false;
    }
}
