<?php
/**
 * stanislav.elbl, 23.09.2021 13:42
 */

namespace Ron\DB;

interface SqlDialect
{
    /**
     * Vraci retezec identifikujici databazi
     * @return string
     */
    public function getDbType();


    /**
     * Vraci SQL operator, ktery se pouziva pro spojovani retezcu
     * @return string
     */
    public function getConcatenateOperator();


    /**
     * Vraci seznam funkci, ktere se nahrazuji v SQL vyrazech
     * @return string
     */
    public function getReplaceFunctions();


    public function getPdoParameterLobOptions();


    /**
     * Vraci SQL vyraz pro zjisteni roku ze zadaneho podvyrazu
     * @param $text
     * @return mixed
     */
    public function extractYearSQL($text);


    /**
     * Prevede hodnotu datoveho pole nacteneho z databaze na timestamp
     * @param mixed $data
     * @return int|null
     */
    public function dataToTimestamp($data);


    /**
     * Vraci SQL konstantu pro zadany cas (unix timestamp)
     * @param int $timestamp
     * @return string
     */
    public function timestampToSQL($timestamp);


    /**
     * Vraci SQL konstantu pro zadany datum (unix timestamp) - cas ignoruje
     * @param int $timestamp
     * @return string
     */
    public function dateToSQL($timestamp);


    /**
     * Vraci SQL konstantu pro zadany cas (unix timestamp) bez datumu
     * @param int $timestamp
     * @return string
     */
    public function timeToSQL($timestamp);

    /**
     * Vraci SQL vyraz pro zjisteni nove hodnoty pro zadanou sekvenci
     * @param string $sequenceName
     * @param bool $standalone  Urcuje, jestli se jedna o samostatny vyraz pro zjisteni nove hodnoty, nebo je to soucast jineho vyrazu (napr. INSERT...)
     * @return string
     */
    public function nextSequenceValueSQL(string $sequenceName, $standalone = true): string;

    /**
     * Vraci SQL dotaz pro zjisteni, jestli existuje sekvence zadaneho jmena.
     * @param string $sequenceName
     * @return string
     */
    public function sequenceExistsSQL(string $sequenceName): string;

//    /**
//     * Vraci SQL konstantu pro vlozeni binarnich dat
//     * @param $text
//     * @return mixed
//     */
//    public function stringToBinarySQL($text);
}
