<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\Http;

use Ron\KeyValuesArray;
use Ron\StringUtils;

class HttpHeaders extends KeyValuesArray
{
    const CONTENT_TYPE = 'Content-Type';
    const CONTENT_DISPOSITION = 'Content-Disposition';
    const CACHE_CONTROL = 'Cache-Control';
    const PRAGMA = 'Pragma';
    const CONTENT_TRANSFER_ENCODING = 'Content-Transfer-Encoding';
    const CONTENT_ENCODING = 'Content-Encoding';
    const CONTENT_EXPIRES = 'Expires';
    const CONTENT_LENGTH = 'Content-Length';
    const LOCATION = 'Location';
    const LAST_MODIFIED = 'Last-Modified';
    const EXPIRES = 'Expires';

    private static $nonce = null;

    public function __construct()
    {
        parent::__construct();
    }


    /**
     * @return null
     */
    public static function getNonce()
    {
        if (!isset(self::$nonce)) {
            self::$nonce = bin2hex(random_bytes(8));
        }
        return self::$nonce;
    }


    /**
     * Nastavi header zadaneho jmena na zadanou hodnotu
     * @param string $name
     * @param string $value   Pokud je zadano null, tak prislusny header odstrani, jinak jej nastavi na tuto hodnotu
     * @return $this
     */
    public function setHeader($name, $value) {
        $this ->setValue($name, $value);
        return $this;
    }


    /**
     * Prida header zadaneho jmena.
     * @param $name
     * @param $value
     * @return $this
     */
    public function addHeader($name, $value) {
        $this ->addValue($name, $value);
        return $this;
    }


    /**
     * @param $filename
     * @param int $contentLength
     * @param string $charset  Nepovinny parametr se znakovou sadou
     * @return $this
     */
    public function addHeadersForContent($filename, $contentLength = null, $charset = null)
    {
        // V exploreru je nejaka chyba, kvuli ktere se spravne nezobrazuji
        // soubory, pokud se na ne primo klikne a zvoli se "otevrit".
        // Nenasel jsem reseni, ktere by nejak spravne fungovalo. Tohle reseni
        // funguje pro obrazky a text, jine soubory se musi ulozit.

        // TODO: patri to sem?
        ini_set('zlib.output_compression', 'Off');

        // header("HTTP/1.1 200 OK");
        // header("Status: 200 OK");

        $disposition = 'inline';

        $charsetStr = $charset ? '; charset=' . $charset : '';

        $fileext = substr(strrchr($filename, "."), 1);
        switch (strtolower($fileext)) {
            case 'pdf' :
                $this ->setHeader(HttpHeaders::CONTENT_TYPE, 'application/pdf');
                break;

            case 'jpg' :
            case 'jpeg' :
                $this ->setHeader(HttpHeaders::CONTENT_TYPE, 'image/jpeg');
                break;

            case 'mpg' :
                $this ->setHeader(HttpHeaders::CONTENT_TYPE, 'video/mpg');
                break;

            case 'mp4' :
                $this ->setHeader(HttpHeaders::CONTENT_TYPE, 'video/mp4');
                break;

            case 'gif' :
                $this ->setHeader(HttpHeaders::CONTENT_TYPE, 'image/gif');
                break;

            case 'png' :
                $this ->setHeader(HttpHeaders::CONTENT_TYPE, 'image/png');
                break;

            case 'bmp' :
                $this ->setHeader(HttpHeaders::CONTENT_TYPE, 'image/bmp');
                break;

            case 'txt' :
            case 'php' :
                $this ->setHeader(HttpHeaders::CONTENT_TYPE, 'text/plain' . $charsetStr);
                break;

            case 'css' :
                $this ->setHeader(HttpHeaders::CONTENT_TYPE, 'text/css');
                break;

            case 'ical' :
            case 'ics' :
                $disposition = 'attachment';
                $this ->setHeader(HttpHeaders::CONTENT_TYPE, 'text/calendar; charset=utf-8');
                //header('Content-Type: text/plain');
                break;

            case 'js' :
                $this ->setHeader(HttpHeaders::CONTENT_TYPE, 'application/javascript');
                break;

            case 'json' :
                $this ->setHeader(HttpHeaders::CONTENT_TYPE, 'application/json; charset=utf-8');
                break;

            default :
                $disposition = 'attachment';
                $this ->setHeader(HttpHeaders::CONTENT_TYPE, 'application/octet-stream');
        }

        $this ->setHeader(HttpHeaders::CONTENT_DISPOSITION, $disposition . '; filename="' . $filename . '"');

        //header("Cache-Control: private, must-revalidate");
        //header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
        //header("Cache-Control: cache, must-revalidate");
        $this ->setHeader(HttpHeaders::CACHE_CONTROL, 'private');
        $this ->setHeader(HttpHeaders::PRAGMA, 'private');
        //header('Pragma: no-cache'); //HTTP/1.0
        $this ->setHeader(HttpHeaders::CONTENT_TRANSFER_ENCODING, 'binary');
        //header('Content-Encoding: FU MS');
        //header('Expires: Sun, 01 Jul 2005 00:00:00 GMT');

        if ($contentLength) {
            $this ->setHeader(HttpHeaders::CONTENT_LENGTH, $contentLength);
        }

        return $this;
    }


    /**
     * Vypise hlavicky na vystup
     */
    public function render()
    {
        foreach ($this ->getData() as $name => $value) {
            if (is_array($value)) {
                foreach($value as $arrayValue) {
                    header(($name ? "$name: " : '') . $arrayValue, false);
                }
            } else {
                header(($name ? "$name: " : '') . $value);
            }
        }
    }


    public function renderDebugScript()
    {
        $nonce = self::getNonce();
        echo "<script nonce='{$nonce}'>\n";

        foreach ($this ->getData() as $name => $value) {
            if (is_array($value)) {
                foreach($value as $arrayValue) {
                    echo 'console.log("DEBUG> header ' . $name . ': ", "' . StringUtils::htmlTextConvert($arrayValue) . '"); ';
                }
            } else {
                echo 'console.log("DEBUG> header ' . $name . '", "' . StringUtils::htmlTextConvert($value) . '"); ';
            }
        }

        echo "</script>\n";
    }
}
