<?php
/**
 * UTF8
 * stanislav.elbl, 16.06.2016
 */

namespace Ron;

class HttpUtils
{
    /**
     * Vraci true, pokud je aktualni pripojeni zabezpecene (https) - pozor nemusi fungovat 100%!
     * @return bool
     */
    public static function isSSL()
    {
        if ( isset($_SERVER['HTTPS']) ) {
            if ( 'on' == strtolower($_SERVER['HTTPS']) )
                return true;
            if ( '1' == $_SERVER['HTTPS'] )
                return true;
        } elseif ( isset($_SERVER['SERVER_PORT']) && ( '443' == $_SERVER['SERVER_PORT'] ) ) {
            return true;
        }
        return false;
    }


    /**
     * Vraci absolutni url pro zadanou relativni url, nebo aktualni skript
     * @param string $url
     * @param string $params
     * @return string
     */
    public static function getAbsoluteURL($url = null, $params = null)
    {
        return
            (self::isSSL() ? 'https://' : 'http://') .
            $_SERVER['HTTP_HOST'] .
            (isset($url) ? $url : $_SERVER['SCRIPT_NAME']) .
            ($params ?: '');
    }


    public static function urlEncode($text) {
        $result = urlencode($text ?? '');
        //$result = str_replace("+", "%2B", $fileName);
        $result = str_replace(' ', '%20', $result);

        return $result;
    }


    /**
     * Sestavi relativni URL ze zadaneho asociativniho pole parametru. Pokud je zadana neprazdna $baseUrl, tak k ni parametry pripoji, jinak da na zacatek znak '?'
     * @param array $params
     * @param string $baseUrl
     * @return string
     */
    public static function getQueryStringFromParams($params, $baseUrl = null)
    {
        $result = $baseUrl ? StringUtils::appendSuffix($baseUrl, '&') : '?';

        if ($params) {
            $first = true;
            foreach ($params as $name => $value) {
                if ($name) {
                    $urlValue =
                        $value === true
                            ? '1'
                            : ($value === false
                                ? '0'
                                : $value);
                    $result .= ($first ? '' : '&') . $name . '=' . self::urlEncode($urlValue);
                    $first = false;
                }
            }
        }

        return $result;
    }
}
