<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace Ron\MVC;

use Ron\Controls\Browser;
use Ron\Controls\InputForm;
use Ron\DB\Database;


class BrowserController implements Controller
{
    /** @var Database */
    var $database;

    /** @var string  menuId*/
    var $action;

    /** @var callable */
    var $createBrowserFunction;
    /** @var callable */
    var $createFormFunction;


    /**
     * @param Database $database
     * @param string   $action
     * @param callable $createBrowserFunction
     * @param callable $createFormFunction
     */
    function __construct($database, $action = null, $createBrowserFunction = null, $createFormFunction = null)
    {
        $this ->database = $database;
        $this ->action = $action;

        $this ->createBrowserFunction = $createBrowserFunction;
        $this ->createFormFunction = $createFormFunction;
    }


    /**
     * @return ViewObject|string
     */
    function getBrowser()
    {
        if ($this ->createBrowserFunction) {
            $createBrowserFunction = $this ->createBrowserFunction;
            /** @var Browser $browser */
            $browser = $createBrowserFunction();

            if ($browser) $browser ->load();
            return $browser;
        }

        return null;
    }


    /**
     * @param int $recordId
     * @return ViewObject|string
     */
    function getRecordViewObject($recordId)
    {
        return null;
    }


    /**
     * @param string $formAction
     * @return InputForm
     */
    function getForm($formAction = 'new')
    {
        if ($this ->createFormFunction) {
            $createFormFunction = $this ->createFormFunction;
            return $createFormFunction($formAction);
        }

        return null;
    }


    /**
     * @return ViewObject|string
     */
    function formAction()
    {
        /** @var \Ron\Controls\InputForm $f */
        $f = $this ->getForm();

        if (!$f) return null;

        if (!$f ->handleAction()) {
            return $f;
        }

        if ($this ->action) {
            return new Redirect(null, '?menuid=' . $this ->action);
        }

        return null;
    }


    function detailAction($recordKey)
    {
        $view = $this ->getRecordViewObject($recordKey);
        if ($view) return $view;

        // no detail view => create form
        $f = $this ->getForm('edit');
        if (!$f) return null;

        return $f;
    }


    /**
     * @param string $action
     * @return string
     */
    public function handleAction($action)
    {
        if (isset($_GET['formaction'])  ||  isset ($_POST['formaction'])) {
            // formular pro editaci
            $form = $this ->formAction();
            if ($form) return $form;
        }

        if (isset($_GET['record'])  &&  $_GET['record']) {
            // detail
            $detail = $this ->detailAction($_GET['record']);
            if ($detail) return $detail;
        }

        // browser
        return $this ->getBrowser();
    }

}
