<?php

/**
 * UTF8
 */

namespace Ron;

use Ron\DB\Cursor;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\MVC\ViewObject;

class OlapField
{
    var $index;   // index pole v Query
    var $format;  // format pro zobrazeni hodnoty
    var $align;   // zarovnani

    var $caption; // nadpis - pouzije se pouze v hlavicce sloupce


    function __construct($index = 0, $format = '', $align = '', $caption = '')
    {
        $this->index = $index;
        $this->format = $format;
        $this->align = $align;
        $this->caption = $caption;

        //$this->values = [];
    }


    function getValue($database, $record)
    {
        return $record[$this->index];
    }


    function getHTMLValue($database, $value)
    {
        return DBUtils::getFormatedValue($database, $this->format, $value);
    }

}



class OlapData
{
    var $values;
    var $span;   // slouceni bunek - spocita se pred zobrazenim

    function __construct()
    {
        $this->values = [];
        $this->span = [];
    }


    /**
     * @param Database $database
     * @param $record
     * @param OlapField[] $fields
     */
    function loadValues($database, $record, $fields)
    {
        $this->values = [];
        $this->span = [];

        foreach ($fields as $field) {
            $this->values[] = $field->getValue($database, $record);
        }
    }


    /**
     * porovnani s jinym objektem
     * vraci -1, pokud je hodnota mensi nez hodnota v $olapdata, 0 pokud jsou stejne, 1 pokud je vetsi
     * @param OlapData $olapdata
     * @return int
     */
    function compareTo($olapdata)
    {
        foreach ($this->values as $i => $value) {
            if ($olapdata->values[$i] > $value) return -1;
            if ($olapdata->values[$i] < $value) return 1;
        }

        return 0;
    }


    /**
     * pricteni hodnot (pouzije se pro datove bunky)
     * @param Database $database
     * @param array $record
     * @param OlapField[] $fields
     */
    function addValues($database, $record, $fields)
    {
        foreach ($fields as $i => $field) {
            if (isset($this->values[$i])) $this->values[$i] += $record[$field->index];
            else $this->values[$i] = $record[$field->index];
        }
    }
}



class OlapCube implements  ViewObject
{
    /** @var Database */
    var $database;

    var $title;         // nadpis browseru
    var $tableId;       // id tabulky v html strance

    var $sql;
    /** @var OlapField[]   indexy poli, ktere se pouziji pro radky */
    var $rowFields;
    /** @var OlapField[]   indexy poli, ktere se pouziji pro sloupce */
    var $colFields;
    /** @var OlapField[]   indexy datovych poli */
    var $dataFields;

    /** @var OlapData[]  vodorovne vrstvy - jiz nactene hodnoty (pole objektu OlapData) */
    var $rowFieldValues;
    /** @var OlapData[]  svisle vrstvy - jiz nactene hodnoty (pole objektu OlapData) */
    var $colFieldValues;

    /** @var OlapData[][] */
    var $data; // nactena zpracovana data - dvojrozmerne pole, prvni index je radek, druhy sloupec


    function __construct($database)
    {
        $this->database = $database;

        // definice dat
        $this->rowFields = [];
        $this->colFields = [];
        $this->dataFields = [];

        // nactena data
        $this->rowFieldValues = [];  // vodorovne vrstvy - jiz nactene hodnoty (pole objektu OLAPField)
        $this->colFieldValues = [];  // svisle vrstvy - jiz nactene hodnoty (pole objektu OLAPField)
        $this->data = [];
    }


    /**
     * @param OlapField $field
     */
    function addRowField($field)
    {
        $this ->rowFields[] = $field;
    }

    /**
     * @param OlapField $field
     */
    function addColField($field)
    {
        $this ->colFields[] = $field;
    }

    /**
     * @param OlapField $field
     */
    function addDataField($field)
    {
        $this ->dataFields[] = $field;
    }


    /**
     * vlozi novy radek na zadany index tj. vlozi novy objekt $olapdata jako hlavicku radku a vytvori take objekty pro datove bunky
     * @param int $index
     * @param OlapData $olapData
     */
    function insertRow($index, $olapData)
    {
        // vlozi hlavicku radku
        array_splice($this->rowFieldValues, $index, 0, [$olapData]);

        // vytvori datove bunky pro vsechny sloupce ...
        $rowdata = [];

        foreach ($this->colFieldValues as $col) {
            $rowdata[] = new OlapData();
        }

        // ... a vlozi datovy radek
        array_splice($this->data, $index, 0, [$rowdata]);
    }


    /**
     * vlozi novy sloupec na zadany index tj. vlozi novy objekt $olapdata jako hlavicku sloupce
     * a ve vsech radcich vytvori a vlozi nove objekty pro datove bunky
     * @param int $index
     * @param OlapData $olapData
     */
    function insertColumn($index, $olapData)
    {
        // vlozi hlavicku sloupce
        array_splice($this->colFieldValues, $index, 0, [$olapData]);

        // vytvori a vlozi datove bunky pro vsechny radky
        foreach ($this->data as $i => $datarow) {
            $newdata = new OlapData();

            //array_splice($datarow, $index, 0, array($newdata)); - pozor, puvodni radek v poli zustane nezmeneny!
            array_splice($this->data[$i], $index, 0, [$newdata]);
        }
    }


    /**
     * Vraci index radku pro vlozeni zadaneho zaznamu
     * Pokud jeste pro zadany zaznam radek neexistuje, tak ho prida
     * @param $record
     * @return int
     */
    function findRowForRecord($record)
    {
        $recorddata = new OlapData();
        $recorddata->loadValues($this ->database, $record, $this->rowFields);

        // vyhledani hodnoty pulenim intervalu
        $min = 0;
        $max = count($this->rowFieldValues) - 1;

        while ($min <= $max) {
            $index = (int)(($min + $max) / 2);

            switch ($this->rowFieldValues[$index]->compareTo($recorddata)) {
                case -1 :
                    $min = $index + 1;
                    break;

                case 1 :
                    $max = $index - 1;
                    break;

                case 0 :
                    return $index;
            }
        }

        // nenasel zadanou hodnotu - zaznam je potreba vlozit mezi $max a $min
        //array_splice($this -> f_rowfieldvalues, $min, 0, array($recorddata));
        $this->insertRow($min, $recorddata);

        return $min;
    }


    /**
     * Vraci index radku pro vlozeni zadaneho zaznamu
     * Pokud jeste pro zadany zaznam radek neexistuje, tak ho prida
     * @param $record
     * @return int
     */
    function findColForRecord($record)
    {
        $recorddata = new OlapData();
        $recorddata->loadValues($this ->database, $record, $this->colFields);

        // vyhledani hodnoty pulenim intervalu
        $min = 0;
        $max = count($this->colFieldValues) - 1;

        while ($min <= $max) {
            $index = (int)(($min + $max) / 2);

            switch ($this->colFieldValues[$index]->compareTo($recorddata)) {
                case -1 :
                    $min = $index + 1;
                    break;

                case 1 :
                    $max = $index - 1;
                    break;

                case 0 :
                    return $index;
            }
        }


        // nenasel zadanou hodnotu - zaznam je potreba vlozit mezi $max a $min
        //array_splice($this -> f_colfieldvalues, $min, 0, array($recorddata));
        $this->insertColumn($min, $recorddata);

        return $min;
    }


    /**
     * Zpracuje zadany radek - tj. najde nebo prida v tabulce odpovidajici bunku a pricte k ni hodnotu
     * @param $record
     */
    function processRecord($record)
    {
        // najde radek a sloupec pro zaznam (pokud jeste neexistuje, tak je automaticky vlozen)
        $row = $this->findRowForRecord($record); // radek prvni!
        $col = $this->findColForRecord($record);

        // pricte data
        $this->data[$row][$col]->addValues($this ->database, $record, $this->dataFields);
    }


    /**
     * Nacte a zpracuje data ze zadane query.
     * @param Cursor $cursor
     */
    function loadDBData($cursor)
    {
        while ($row = $cursor ->fetch()) {
            $this->processRecord($row);
        }
    }


    /**
     * nastavi "span" jednotlivych bunek - tj. pres kolik bunek tabulky se ma zobrazit (colspan a rowspan)
     */
    function setCellSpan()
    {
        // radky
        foreach ($this->rowFields as $rowfieldindex => $rowfield) {
            // pro kazdy sloupec "radkove" hlavicky - tj. podle poctu "rowfield"
            $lastvalue = null;
            $lastindex = -1; // index pos
            $cnt = 0;


            foreach ($this->rowFieldValues as $rowindex => $rowdata) // projede vsechny radky
            {
                $newvalue = false;

                if ($lastindex > 0) {
                    if ($rowdata->values[$rowfieldindex] != $lastvalue) $newvalue = true;
                    else {
                        // jeste zkontrolovat predchozi sloupec
                        if ($rowfieldindex > 0)
                            if ($this->rowFieldValues[$rowindex]->span[$rowfieldindex - 1]) $newvalue = true;
                    }
                } else {
                    // prvni zaznam
                    $newvalue = true;
                }

                if ($newvalue) {
                    // hodnota v aktualni bunce je jina nez predchozi
                    if ($lastindex >= 0) $this->rowFieldValues[$lastindex]->span[$rowfieldindex] = $cnt;

                    $cnt = 1;
                    $lastindex = $rowindex;
                    $lastvalue = $rowdata->values[$rowfieldindex];
                } else {
                    $cnt++;
                }
            }

            if ($lastindex >= 0) $this->rowFieldValues[$lastindex]->span[$rowfieldindex] = $cnt;
        }
    }


    /**
     * Vraci HTML text pro zobrazeni. Pouzije se v ShowData, ale take v XML formularich
     * ! musi byt platne XML !
     * @return string
     */
    function getHTML()
    {
        // ZAHLAVI RADKU | NACTENE         SLOUPCE
        //               | Data1   Data2   Data1  Data2
        // --------------|-----------------------------
        // NACTENE  ...  | NACTENE  ...    DATOVE  ...
        //          ...  | BUNKY    ...    ...     ...
        // RADKY    ...  | ...      ...    ...     ...

        $result = '<TABLE class="browser rowOver">';

        $result .= '<TR>';

        // zahlavi radku
        foreach ($this->rowFields as $field) {
            $alignstr = $field->align ? "align='{$field -> align}'" : '';
            $result .= "<TH $alignstr rowspan='2'>" . $field->caption . '</TH>';
        }

        // hlavicka nactenych sloupcu - datove hodnoty
        $colspanstr = '';
        if (count($this->dataFields) > 1) $colspanstr = 'colspan="' . count($this->dataFields) . '"';

        foreach ($this->colFieldValues as $colvalue) {
            $result .= "<TH $colspanstr>";
            foreach ($colvalue->values as $i => $value) $result .= $this->colFields[$i]->getHTMLValue($this ->database, $value);
            $result .= '</TH>';
        }

        $result .= "</TR>\n";

        $result .= '<TR>';
        // druha hlavicka sloupcu - nadpisy datovych hodnot
        foreach ($this->colFieldValues as $colfield) {
            foreach ($this->dataFields as $datafield) $result .= '<TH>' . $datafield->caption . '</TH>';
        }
        $result .= "</TR>\n";


        // jednotlive radky ----------------------------------------------------
        foreach ($this->rowFieldValues as $rowindex => $rowdata) {
            $result .= "<TR>";

            // hlavicka radku - prida sloupec tabulky pro kazde datove pole radku
            foreach ($this->rowFields as $i => $field) {
                if ($rowdata->span[$i]) {
                    $alignstr = $field->align ? "align='{$field -> align}'" : '';

                    if ($rowdata->span[$i] > 1) $rowspanstr = 'rowspan="' . $rowdata->span[$i] . '"';
                    else $rowspanstr = '';

                    $result .= "<TH $alignstr $rowspanstr>" . $field->getHTMLValue($this ->database, $rowdata->values[$i]) . '</TH>';
                }
            }

            // datove bunky aktualniho radku
            foreach ($this->data[$rowindex] as $data) {
                foreach ($this->dataFields as $i => $datafield) {
                    $alignstr = $datafield->align ? "align='{$datafield -> align}'" : '';

                    $result .= "<TD $alignstr>";
                    $result .= $datafield->getHTMLValue($this ->database, $data->values[$i]);
                    $result .= '</TD>';
                }
            }

            $result .= "</TR>\n";
        }

        $result .= "</TABLE>\n";

        return $result;
    }


    function loadData()
    {
        $this->rowFieldValues = [];
        $this->colFieldValues = [];
        $this->data = [];

        $cursor = $this ->database ->getSqlResult($this->sql);
        $this->loadDBData($cursor);
    }


    function render()
    {
        $this->loadData();
        $this->setCellSpan();
        echo $this->getHTML();
    }


    function getXML()
    {
        // POZOR! musi vracet jeden korenovy tag!
        $this->loadData();
        $this->setCellSpan();
        return $this->getHTML();
    }
}


//function TestOLAPCube($connection)
//{
//  $cube = new OlapCube();
//  $cube -> sql =
//     'SELECT Mesic, TypCasoveSlozkyMzdy, Hodiny, Dny, OsobniCislo '.
//     '  FROM MesicniVysledekVypoctu '.
//     " WHERE RC = '123456' AND TypCasoveSlozkyMzdy < '2' AND Hodiny <> 0 ".
//     ' ORDER BY Mesic';
//
//  $cube -> rowFields = array( new OlapField(0, 'date', 'left', 'Obdobi'), new OlapField(4, '', 'right', 'Os.cislo') );  // pole, ktera se pouziji pro radky
//  $cube -> colFields = array( new OlapField(1) );  // pole, ktera se pouziji pro sloupce
//  $cube -> dataFields = array( new OlapField(2, 'hodiny', 'right', 'Hodiny'), new OlapField(3, 'number_2', 'center', 'Dny') ); // pole, ktera se pouziji pro datove bunky
//
////function OLAPField($index = 0, $format = '', $align = '', $caption = '')
//
//  $cube -> Show($connection);
//}
//
