<?php

namespace Ron\Utils;

use Ron\StringUtils;

class Attributes implements \ArrayAccess, \Countable
{
    /** @var array */
    protected $data = [];

    /**
     * Attributes constructor.
     * @param ?array $initialData
     */
    public function __construct($initialData = null)
    {
        if ($initialData) {
            $this ->data = $initialData;
        }
    }


    /**
     * @param array $data
     */
    public function setData($data)
    {
        $this->data = $data;
    }


    public function offsetExists($offset): bool
    {
        return isset($this ->data[$offset]);
    }


    #[\ReturnTypeWillChange]    // kvuli podpore PHP 7 nelze nastavit typy
    public function offsetGet($offset)
    {
        return $this ->data[$offset] ?? null;
    }

    public function offsetSet($offset, $value): void
    {
        $this ->data[$offset] = $value;
    }

    public function offsetUnset($offset): void
    {
        unset($this ->data[$offset]);
    }

    public function count(): int
    {
        return $this ->data
            ? count($this ->data)
            : 0;
    }


    /**
     * Prida zadana data
     * @param array|Attributes $data
     * @return Attributes
     */
    public function append($data): Attributes
    {
        if ($data) {
            foreach ($data as $name => $value) {
                $this->data[$name] = $value;
            }
        }

        return $this;
    }


    public function __toString()
    {
        $result = '';

        foreach ($this ->data as $name => $value) {
            $result .=
                ($result ? ' ' : '') .
                $name . '="' . StringUtils::htmlTextConvert($value) . '"';
        }

        return $result;
    }
}
