<?php
/**
 * stanislav.elbl, 24.02.2021
 */

namespace RonPortal\Admin;

use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\DataOutputObject;
use Ron\MVC\DownloadFileOutputObject;
use Ron\MVC\HttpController;
use Ron\MVC\Model;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\System\AgentManager;
use RonPortal\System\ReportServer;
use RonPortal\User;
use Tracy\Debugger;


class SessionViewObject implements ViewObject
{
    public function render()
    {
        echo '<H1>SESSION</H1>';
        Debugger::dump($_SESSION);
    }
}


class InfoPhpViewObject implements ViewObject
{
    public function render()
    {
        phpinfo();
    }
}


class AdminController extends HttpController
{
    /** @var Database */
    var $database;

    /**
     * AdminController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
        $this->enableAccessWithoutLogin = true;
    }


    /**
     * Vraci info printServeru.
     * @return string
     * @throws \Exception
     */
    function loadPrintServerInfo()
    {
        //$server = ReportServer::createInstance();

        try {
            //Model::setData('reportServerInfo', $server ->getInfo());
            Model::setData('reportServerInfo', ReportServer::getServerInfo());
        } catch (\Exception $e) {
            //$server->close();
            throw $e;
        }
    }


    function downloadGlobalSettings()
    {
        $sql = 'SELECT ' .
            $this ->database ->castWideMemo('nastaveni') . ', ' .
            $this ->database ->castWideMemo('nastaveniJidelny') . ', ' .
            $this ->database ->castWideMemo('nastaveniPersonalistiky') . ', ' .
            $this ->database ->castWideMemo('nastaveniSpolecne') .
            ' FROM NastaveniProIntraweb';

        $cursor = $this ->database ->getSqlResult($sql);
        $row = $cursor ->fetch();

        if ($row) {

            return new DataOutputObject(
                "nastaveniSpolecne:\n" . $row[3] . "\n\n" .
                "nastaveni (dochazka):\n" . $row[0] . "\n\n" .
                "nastaveniJidelny:\n" . $row[1] . "\n\n" .
                "nastaveniPersonalistiky:\n" . $row[2] . "\n\n",
                'nastaveni_ron_portalu.txt', TextManager::getInstance() ->getCharSet());
        }

        return new DataOutputObject('NIC TU ZATIM NENI...', 'nastaveni_uzivatele.txt', TextManager::getInstance() ->getCharSet());
    }


    public function adminView()
    {
        if (isset($_GET['testagentads'])  &&  (int)$_GET['testagentads']) {
            try {
                MessageManager::addInfoMessage( StringUtils::fromUTF8('Poznámka - test spuštění agenta by měl skončit chybovou hláškou "Chybně zadáno číslo úlohy!" V logu aplikace by měla být informace o spuštění agenta.') );
                AgentManager::executeADS('ADS_Agent_Processing.exe', '-alwaysLog -sendMessagesToStdOut -user:' . User::getLoginName(), false);
            } catch (\Exception $e) {
                MessageManager::addErrorMessage($e ->getMessage());
            }
        }

        if (isset($_GET['testagentperson'])  &&  (int)$_GET['testagentperson']) {
            try {
                MessageManager::addInfoMessage( StringUtils::fromUTF8('Poznámka - test spuštění agenta by měl skončit chybovou hláškou "Chybně zadáno číslo úlohy!" V logu aplikace by měla být informace o spuštění agenta.') );
                AgentManager::executePersonAgent('-alwaysLog -sendMessagesToStdOut -user:' . User::getLoginName(), false);
            } catch (\Exception $e) {
                MessageManager::addErrorMessage($e ->getMessage());
            }
        }

        if (isset($_GET['errorlog'])  &&  (int)$_GET['errorlog']) {
            return new DownloadFileOutputObject(ini_get('error_log'), null, 'php_errors.log');
        }

        if (isset($_GET['globalsettings'])) {
            return $this->downloadGlobalSettings();
        }

        if (isset($_GET['session'])) {
            return new SessionViewObject();
        }

        try {
            $this->loadPrintServerInfo();
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
        }

        return new BaseViewObject('view/admin.php');
    }


    public function passwordForm()
    {
        return 'view/admin_login.php';
    }

    public function adminLogin()
    {
        if (!isset($_POST['adminpassword'])) return $this ->passwordForm();

        global $admin_page_password_hash;

        if ( password_verify($_POST['adminpassword'], $admin_page_password_hash ?: '$2y$10$BFW3YmedexM7dnIYJW6uxuBIKD1RrasVfXti/3H3r/dpBDoVXqBWu') ) {

            return $this ->adminView();
        }

        MessageManager::addErrorMessage('Špatné heslo. Hash: ' . password_hash($_POST['adminpassword'], PASSWORD_DEFAULT));

        return 'view/admin_login.php';
    }


    /**
     * @param string $action
     * @return \Ron\MVC\InternalRedirect|\Ron\MVC\OutputObject|\Ron\MVC\Redirect|\Ron\MVC\ViewObject|string
     * @throws \Exception
     */
    public function handleAction($action)
    {
        if (!User::isAuthenticated()) {
            global $disable_admin_page; // moznost zakazat admin bez prihlaseni
            if ($disable_admin_page) {
                throw new NeniPravoException();
            }
            return $this ->adminLogin();
        }

        Prava::nastavObdobiProPrava(time(), time());

        if (Prava::vratHodnotuPrava('admin') != 3) {
            throw new \Exception( StringUtils::fromUTF8('Pro zvolenou akci nemáte právo! Právo pro administraci musí být nastaveno na hodnotu "Vše".') );
        }

        if ($action == 'infophp') {
            return new InfoPhpViewObject();
        }

        HtmlHeaders::getInstance()->setTitle(TextManager::_('RON_Portal') . ' - ' . TextManager::_('Admin'));

        return $this ->adminView();
    }

}
