<?php

namespace RonPortal\Ankety;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\System\RecordNotFoundException;
use RonPortal\User;

class Anketa
{
    /** @var int */
    public $id;

    /** @var string */
    public $nazev;

    /** @var string */
    public $popis;

    /** @var int Druh pripojeneho dotazniku (melo by byt 12) */
    public $druhHodnoceni;

    /** @var int    Cislo pripojeneho dotazniku */
    public $temaCislo;

    /** @var int    Datum pocatku platnosti ankety (muze byt null!) */
    public $platnostOd;

    /** @var int    Datum konce platnosti ankety (muze byt null!) */
    public $platnostDo;


    private function loadRow(Database $database, $row)
    {
        $this ->id = (int)$row[0];
        $this ->nazev = $row[1];
        $this ->popis = $row[2];
        $this ->druhHodnoceni = (int)$row[3];
        $this ->temaCislo = (int)$row[4];
        $this ->platnostOd = $database ->stringToTimestamp($row[5]);
        $this ->platnostDo = $database ->stringToTimestamp($row[6]);
    }


    /**
     * @param Database $database
     * @param int $id
     * @return static
     * @throws RecordNotFoundException
     * @throws DBException
     */
    public static function loadAnketa(Database $database, int $id)
    {
        $params = $database ->createParameters();
        $cursor = $database ->getSqlResult(
            'SELECT Id, Nazev, Popis, DruhHodnoceni, TemaCislo,' .
            '  PlatnostOd, PlatnostDo' . // 4,5
            ' FROM Anketa WHERE Id = ' . $params ->addParameter($id),
            $params
        );
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new RecordNotFoundException();
        }

        $result = new static();
        $result ->loadRow($database, $row);

        return $result;
    }


    /**
     * @param Database $database
     * @param int $id
     * @return void
     * @throws NeniPravoException|DBException
     */
    public function zkontrolujPravoNaAnketu($database)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT AnketaProRozdeleni.IdAnkety FROM AnketaProRozdeleni, RozdeleniOsob, PracovniPomer' .
            ' WHERE AnketaProRozdeleni.IdAnkety = ' . $params ->addParameter($this ->id) .
            ' AND AnketaProRozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni' .
            ' AND AnketaProRozdeleni.Rozdeleni = RozdeleniOsob.Rozdeleni' .
            ' AND RozdeleniOsob.RC = PracovniPomer.RC' .
            ' AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
            ' AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
            ' AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter(time()) .
            ' AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter(time()) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
            ' AND PracovniPomer.DatumNastupu <= ' . $params ->addDateParameter(time()) .
            ' AND (PracovniPomer.DatumUkonceni >= ' . $params ->addDateParameter(time()) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
            ' AND PracovniPomer.RC = ' . $params ->addParameter(User::getRC());

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new NeniPravoException();
        }

        if ($this ->platnostOd  &&  $this ->platnostOd > time()) {
            throw new NeniPravoException(TextManager::_('Anketa_jeste_neni_platna'));
        }

        if ($this ->platnostDo  &&  $this ->platnostDo < DateUtils::today()) {
            throw new NeniPravoException(TextManager::_('Anketa_uz_neni_platna'));
        }
    }
}
