<?php
/**
 * stanislav.elbl, 10.04.2022 0:08
 */

namespace RonPortal\Benefity;

use Ron\Controls\ComboBox;
use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;


class BenefitFormField extends FormField
{
    /** @var string     RC osoby */
    protected $rc;

    /**
     * @param Database $database
     * @param string $rc
     */
    public function __construct($database, $rc)
    {
        parent::__construct(FormField::FIELD_TYPE_SELECT);
        $this ->rc = $rc;

        $params = $database->createParameters();

        $this ->setSelectSql(
            new SqlStatement(
                'SELECT Benefit.ID, Benefit.Popis' .
                ' FROM Benefit' .
                " INNER JOIN TypBenefitu ON (TypBenefitu.Kod = Benefit.TypBenefitu)\n" .
                ' WHERE Benefit.PlatnostOd <= ' . $params->addDateParameter(time()) .
                '  AND (Benefit.PlatnostDo >= ' . $params->addDateParameter(time()) . " OR  Benefit.PlatnostDo IS NULL)\n" .
                '  AND' .
                '  ( (Benefit.NeomezenyPocet = 1) ' .
                '     OR' .
                '    ((SELECT COALESCE(SUM(PB.PocetBenefitu), 0) AS CerpanoBenefitu' .
                '      FROM PohybBenefitu PB' .
                '      WHERE Benefit.ID = PB.IdBenefitu AND PB.DatumSchvaleni < ' . $params->addDateParameter(DateUtils::incDay(time()) - 1) .
                "       AND (PB.DruhPohybu IN (22,21,41,42,32))) < Benefit.PocetCelkem))\n" .
                '  AND' .
                "  ( ((SELECT COUNT(*) FROM BenefitProRozdeleni WHERE BenefitProRozdeleni.IdBenefitu = Benefit.ID) = 0)\n" .
                '      OR (EXISTS (SELECT * FROM BenefitProRozdeleni' .
                '          INNER JOIN RozdeleniOsob RO' .
                '          ON RO.KodTypuRozdeleni = BenefitProRozdeleni.KodTypuRozdeleni AND RO.Rozdeleni = BenefitProRozdeleni.Rozdeleni' .
                '          WHERE BenefitProRozdeleni.IdBenefitu = Benefit.ID' .
                '          AND RO.RC = ' . $params->addParameter($this ->rc) .
                '          AND RO.PlatnostOd <= ' . $params->addDateParameter(time()) .
                '          AND (RO.PlatnostDo >= ' . $params->addDateParameter(time()) . ' OR RO.PlatnostDo IS NULL)' .
                '      )' .
                " ))\n" .
                ' AND (%FILTER%)' .
                ' ORDER BY %ORDER%',
                $params),
            '1 = 1', 'Benefit.Popis, Benefit.PlatnostOd'
        );
    }


//    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
//    {
//        if ($canEdit) {
//            $params = $database->createParameters();
//
//            $sql =
//                'SELECT Benefit.ID, Benefit.Popis' .
//                ' FROM Benefit' .
//                " INNER JOIN TypBenefitu ON (TypBenefitu.Kod = Benefit.TypBenefitu)\n" .
//                ' WHERE Benefit.PlatnostOd <= ' . $params->addDateParameter(time()) .
//                '  AND (Benefit.PlatnostDo >= ' . $params->addDateParameter(time()) . " OR  Benefit.PlatnostDo IS NULL)\n" .
//                '  AND' .
//                '  ( (Benefit.NeomezenyPocet = 1) ' .
//                '     OR' .
//                '    ((SELECT COALESCE(SUM(PB.PocetBenefitu), 0) AS CerpanoBenefitu' .
//                '      FROM PohybBenefitu PB' .
//                '      WHERE Benefit.ID = PB.IdBenefitu AND PB.DatumSchvaleni < ' . $params->addDateParameter(DateUtils::incDay(time()) - 1) .
//                "       AND (PB.DruhPohybu IN (22,21,41,42,32))) < Benefit.PocetCelkem))\n" .
//                '  AND' .
//                "  ( ((SELECT COUNT(*) FROM BenefitProRozdeleni WHERE BenefitProRozdeleni.IdBenefitu = Benefit.ID) = 0)\n" .
//                '      OR (EXISTS (SELECT * FROM BenefitProRozdeleni' .
//                '          INNER JOIN RozdeleniOsob RO' .
//                '          ON RO.KodTypuRozdeleni = BenefitProRozdeleni.KodTypuRozdeleni AND RO.Rozdeleni = BenefitProRozdeleni.Rozdeleni' .
//                '          WHERE BenefitProRozdeleni.IdBenefitu = Benefit.ID' .
//                '          AND RO.RC = ' . $params->addParameter($this ->rc) .
//                '          AND RO.PlatnostOd <= ' . $params->addDateParameter(time()) .
//                '          AND (RO.PlatnostDo >= ' . $params->addDateParameter(time()) . ' OR RO.PlatnostDo IS NULL)' .
//                '      )' .
//                " ))\n" .
//                ' ORDER BY Benefit.Popis, Benefit.PlatnostOd';
//
//            $combo = new ComboBox($this ->fieldName);
//            $combo->emptyRow = ' ';
//            $combo->dataTypeComparison = false;
//            $combo->className .= 'form-input';
//            //$combo->htmlParams = "title='" . ronText("vyber_benefitu") . "'" . 'onChange="window.location.href=&quot;' . $url . '&amp;benefit=&quot; + encodeURIComponent(this.value);"';
//            $combo->showCode = false;
//            $combo->loadData($database, $sql, $value, $params);
//
//            $combo ->render();
//        } else {
//            parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
//        }
//    }
}
