<?php
/**
 * pavel.rudol, 6.5.2022 11:37
 */

namespace RonPortal\Benefity;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\Http\Csrf;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\StringUtils;
use RonPortal\Controls\DokumentyView;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\Prava;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\Security;
use RonPortal\System\AgentManager;
use RonPortal\System\RecordNotFoundException;
use RonPortal\User;
use function Ron\ronText;

class BenefityController extends JsonController
{
    /** @var  Database $database */
    private $database;

    /** @var  PravaNaOsoby $pravo_rezervovat */
    public $pravo_rezervovat;

    /** @var  PravaNaOsoby $pravo_schvalit */
    public $pravo_schvalit;

    /** @var  PravaNaOsoby $pravo_vydat */
    public $pravo_vydat;

    /** @var  PravaNaOsoby $pravo_vznik */
    public $pravo_vznik;

    /** @var PravaNaOsoby $pravo_sablony */
    public $pravo_sablony;


    /**
     * Benefity constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;

        Prava::nastavObdobiProPrava(time(), time());

        $this->pravo_rezervovat = Prava::vratPrava('benefit_rezervace');
        $this->pravo_schvalit = Prava::vratPrava('benefit_schvaleni');
        $this->pravo_vydat = Prava::vratPrava('benefit_vydani');
        $this->pravo_vznik = Prava::vratPrava('benefit_vznik_bodu');
        $this->pravo_sablony = Prava::vratPrava('benefit_sablona');
    }

    /**
     * @param Database $database
     * @return bool
     * @throws Exception
     */
    public static function zapocitatNeschvaleneRezervace($database)
    {
        if (!isset($_SESSION['benefity_zapocitatrezervace'])) {
            $params = $database->createParameters();
            $_SESSION['benefity_zapocitatrezervace'] = (bool)DBUtils::getSqlValue($database, 'SELECT Val FROM SYS_VAR_GLOBAL WHERE Name = ' . $params->addParameter('Benefit_ZapocitatNeschvRezervaceBodu'), $params);
        }

        return $_SESSION['benefity_zapocitatrezervace'];
    }

    /**
     * Menu benefity
     */
    private function hlavickaVyberAkce()
    {
        $sekce = [];
        if (Prava::vratHodnotuPrava('skoleni_prohlizeni') != 0) {
            $sekce['pohyby'] = ronText('prehled_pohybu_benefitu');
        }

        if ($this->pravo_sablony->jsouNejakePrava()) {
            $sekce['sablony'] = ronText('sablony_benefitu');
        }

        $aktivni_sekce = ((isset($_GET['action']) && $_GET['action'])
            ? $_GET['action']
//            : (isset($_GET['formaction']) ? 'zapisabsolvovani' : ''));
            : '');

        Model::addData('header',  ronText('MI_Benefity'));
        Model::addData('aktivni_sekce', $aktivni_sekce);
        Model::addData('sekce', $sekce);
        Model::addData('url', '?menuid=benefity&action=');
        Model::addData('wrap_class', 'benefity');
        Model::addViewObject(new BaseViewObject('view/appsections.php'));
    }

    /**
     * @param $idBenefitu
     * @param $idDokumentu
     * @return DokumentOutputObject
     * @throws Exception
     */
    private function downloadDokument($idBenefitu, $idDokumentu)
    {
        Security::validateCSRFGetParam();
        //pravo se kontroluje v handle action, staci aby mel jedno z prav na benefity

        //Kontrola, zdali existuje u dotycneho benefitu prohlizeny dokument
        $params = $this->database->createParameters();
        $sql =
            'SELECT DokumentBenefitu.ID ' .
            'FROM DokumentBenefitu LEFT JOIN Benefit ON (DokumentBenefitu.id = Benefit.idDokumentu) ' .
            "WHERE DokumentBenefitu.idBenefitu = " . $params->addParameter($idBenefitu) . " AND DokumentBenefitu.ID = " . $params->addParameter($idDokumentu);

        $existujeDokument = DBUtils::getSqlValue($this->database, $sql, $params);

        if ($existujeDokument == '') {
            throw new NeniPravoException();
        }

        return new DokumentOutputObject($this->database, $idDokumentu);
    }

    /**
     * Pokud je datum generovani zaniku starsi nez dnes, spusti agenta pro generovani zaniku bodu.
     * @return void
     * @throws DBException
     */
    private function generovaniZaniku()
    {
        $params = $this->database->createParameters();
        $sql = "SELECT Val FROM SYS_VAR_GLOBAL Where Name = " . $params->addParameter("Benefit_GenerovaniZanikuKeDni");

        $datumGenerovaniZaniku = DBUtils::getSqlValue($this->database, $sql, $params);

        $datumZaniku = "";
        if ($datumGenerovaniZaniku != "") {
            $datumZaniku = DateUtils::dateStringToTimestamp(explode("_", $datumGenerovaniZaniku)[0]);
        }

        $dnes = DateUtils::today();

        //Pokud je datum zániku včerejší, pak provede generování zániků
        if ($datumZaniku < $dnes) {
            try {
                AgentManager::executePersonAgent(51);
            } catch (Exception $e) {
                MessageManager::addInfoMessage(ronText('chyba_generovani_zaniku') . ": " . $e->getMessage());
            }
        }
    }

    /**
     * Nastavi stav pohybu. Cerpani a/nebo schvaleni (viz argument $cerpani)
     * @param $idPohybuBenefitu
     * @param bool $cerpani nastavi cerpani a/nebo schvaleni (pri cerpani doplni i schvaleni, pokud nebylo nastaveno)
     * @return Redirect
     * @throws Exception
     */
    public function nastavStavPohybu($idPohybuBenefitu, $cerpani = false)
    {
        Csrf::validateCSRFGetParam(($cerpani ? 'cerpanibenefitu_' : 'schvalenibenefitu_') . $idPohybuBenefitu);

//        $pohyb = PohybBenefitu::nactiPohybBenefitu($this->database, $idPohybuBenefitu);
//        $rec = PohybBenefituDBRecord::loadFromObject($this->database, $pohyb);

        $rec = new PohybBenefituDBRecord($this ->database, $this);
        $rec ->addPohybBenefituFields();
        $params = $this ->database ->createParameters();
        $rec ->setWhere('PohybBenefitu.Id = ' . $params ->addParameter($idPohybuBenefitu), $params);

        if (!$rec ->loadRecord()) {
            throw new RecordNotFoundException();
        }

        if ($cerpani) {
            if (!$this->pravo_vydat->lzeZobrazitRC($rec['RC'])) {
                throw new NeniPravoException();
            }

            if ($rec['DatumCerpani']) {
                //Pokud omylem uzivatel uklada znova (z jine zalozky..) pak jiz pouze Redirect
                return new Redirect(null, "?menuid=benefity");
            }

            $rec->setVydani();
        } else {
            if (!($this->pravo_schvalit->lzeZobrazitRC($rec['RC']))) {
                throw new NeniPravoException();
            }

            if ($rec['DatumSchvaleni']) {
                //Pokud omylem uzivatel uklada znova (z jine zalozky..) pak jiz pouze Redirect
                return new Redirect(null, "?menuid=benefity");
            }
        }

        //schvaleni se doplni vzdy, pokud nebylo
        $rec->setSchvaleni();

        $rec->runAgent();

        return new Redirect(null, '?menuid=benefity');
    }

    /**
     * Zobrazi formular pro zadani rezervace bodu.
     * @return Redirect|PohybBenefituForm
     * @throws DBException
     * @throws NeniPravoException
     */
    public function rezervaceBodu()
    {
        if (!$this->pravo_rezervovat->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        $form = new PohybBenefituForm($this->database, true);

        if ($form->handleAction()) {
            return new  Redirect(null, "?menuid=benefity");
        }

        return $form;
    }

    /**
     * Zobrazi formular pro zadani vzniku bodu.
     * @return Redirect|PohybBenefituForm
     * @throws DBException
     * @throws NeniPravoException
     */
    public function vznikBodu()
    {
        if (!$this->pravo_vznik->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        $form = new PohybBenefituForm($this->database);

        if ($form->handleAction()) {
            return new  Redirect(null, "?menuid=benefity");
        }

        return $form;
    }

    /**
     * Zobrazi formular s detailem pohybu a umozni jeho editaci.
     * @return Redirect|PohybBenefituForm
     * @throws DBException
     * @throws NeniPravoException
     */
    public function pohybBenefitu()
    {
        $pohyb = null;
        if (isset($_GET['record'])) {
            $pohyb = PohybBenefitu::nactiPohybBenefitu($this->database, $_GET['record']);
        }

        Prava::nastavObdobiProPrava($pohyb->datumVlozeni);

        switch ($pohyb->druhPohybu) {
            case PohybBenefitu::DRUH_POHYBU_REZERVACE_BODU :
                if (!$this->pravo_rezervovat->lzeZobrazitRC($pohyb->rc)) {
                    throw new NeniPravoException();
                }
                break;
            case PohybBenefitu::DRUH_POHYBU_VZNIK_BODU :
                if (!$this->pravo_vznik->lzeZobrazitRC($pohyb->rc)) {
                    throw new NeniPravoException();
                }
                break;
        }

        $form = new PohybBenefituForm($this->database, $pohyb->druhPohybu == PohybBenefitu::DRUH_POHYBU_REZERVACE_BODU || $pohyb->druhPohybu == PohybBenefitu::DRUH_POHYBU_CERPANI_BODU, $pohyb);

        if ($form->handleAction()) {
            return new Redirect(null, "?menuid=benefity");
        }

        return $form;
    }

    /**
     * Zobrazi formular pro editaci sablony benefitu
     * @return Redirect|SablonaBenefituForm
     * @throws Exception
     */
    public function upravitSablonuRezervace()
    {
        $form = new SablonaBenefituForm($this->database);

        // TODO: overit moznost editace - viz metoda vyhodnoceniPodminekProEditaceZaznamu
        if ($form->handleAction()) {
            return new  Redirect(null, "?menuid=benefity&action=sablony");
        }

        return $form;
    }

    /**
     * Zobrazi seznam sablon benefitu.
     * @return SablonyBenefituBrowser
     * @throws NeniPravoException
     */
    public function browserSablonyBenefitu()
    {
        if (!$this->pravo_sablony->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        $this->hlavickaVyberAkce();

        $b = new SablonyBenefituBrowser($this->database);
        $b->load();

        return $b;
    }

    /**
     * Zobrazi seznam sablon benefitu nebo formular pro zadani/editaci sablony.
     * @return Redirect|SablonaBenefituForm|SablonyBenefituBrowser
     * @throws NeniPravoException
     */
    public function sablonyBenefitu()
    {
        if (isset($_POST['formaction'])) {
            // vlozeni/ulozeni sablony
            return $this ->upravitSablonuRezervace();
        }

        if (isset($_GET['record'])  &&  (int)$_GET['record']) {
            // editace
            return $this ->upravitSablonuRezervace();
        }

        if (isset($_GET['formaction'])  &&  $_GET['formaction'] == 'new') {
            // vlozeni nove sablony
            return $this ->upravitSablonuRezervace();
        }

        return $this ->browserSablonyBenefitu();
    }

    /**
     * Smaze zvolenou sablonu prihlaseneho uzivatele
     * @param int $id ID sablony
     * @return Redirect
     * @throws DBException
     * @throws Exception
     */
    public function odstranitSablonu($id)
    {
        Csrf::validateCSRFGetParam("benefit_del_sablona_$id");

        $params = $this->database->createParameters();

        $rec = new RonPortalDBRecord($this->database, 'SablonaRezervaceBenefitu', $this);

        $rec->addField('Id', 'int');
        $rec->addField('RC', '');
        $rec->addField('DruhPohybu', 'int');
        $rec->addField('IdBenefitu', 'int');
        $rec->addField('Poradi', 'int');
        $rec->addField('PocetBenefitu', 'int');
        $rec->addField('PocetBodu', 'int');
        $rec->addField('TypDoplatku', 'int');
        $rec->addField('DoplatekCastka', 'float');
        $rec->addField('TypKontrolySablony', 'int');
        $rec->addField('CasovaJednotka', 'int');
        $rec->addField('PocetCasJednotek', 'int');
        $rec->addInsertLogFields();
        $rec->addUpdateLogFields();

        //Odstranit sablonu jine osoby lze pouze pres aplikaci
        $rec->setWhere(
            "ID = " . $params->addParameter($id) .
            " AND RC = " . $params->addParameter(User::getRC()) ,
            $params);

        $rec->deleteRecord();

        return new  Redirect(null, "?menuid=benefity&action=sablony");
    }

    /**
     * Nacte detaily benefitu do JSON objektu.
     * @param JsonOutputObject $output
     * @throws Exception
     */
    function nactiPodrobnostiBenefitu($output)
    {
        if (!isset($_GET['id'])) {
            throw new Exception('Neni zadano ID benefitu!');
        }

        $benefit = Benefit::nactiBenefit($this ->database, (int)$_GET['id']);

        $output ->addText('typBenefitu', $benefit ->typBenefitu);
        $output ->addText('platnostOd', StringUtils::format('date', $benefit ->platnostOd));
        $output ->addText('platnostDo', StringUtils::format('date', $benefit ->platnostDo));
        $output ->addText('doplatekCastka', StringUtils::format('number', $benefit ->doplatekCastka));
        $output ->addText('hodnotaVBodech', $benefit ->hodnotaVBodech);
        $output ->addText('maxPocetJedenZam', $benefit ->neomezenyPocetJedenZam ? TextManager::_('neomezeno') : $benefit ->maxPocetJedenZam);
        $output ->addText('automatickeSchvaleni', TextManager::_($benefit ->automatickeSchvaleni ? 'Str_Yes' : 'Str_No'));
        $output ->addText('popisDetail', $benefit ->popisDetail);
        $output ->addData('variabilniHodnotaVBodech', $benefit ->variabilniHodnotaVBodech);

        //throw new Exception("jeste NENI IMPLEMENTOVANO!");
    }


    /**
     * Nacte dokumenty pro view.
     * @param JsonOutputObject $output
     * @throws Exception
     */
    function nactiDokumentyBenefitu($output)
    {
        if (!isset($_GET['id'])) {
            throw new Exception('Neni zadano ID benefitu!');
        }

        $idBenefitu = (int)$_GET['id'];

        $params = $this ->database ->createParameters();

        $view = new DokumentyView($this ->database);
        $view
            ->setDownloadUrl("?menuid=benefity&action=dokumentdownload" . Security::csrfUrlParameter() . "&id=$idBenefitu&record=")
            ->loadData(new SqlStatement(
                'SELECT ' . DokumentyView::getSqlSelectPart() .
                ' FROM DokumentBenefitu, Dokument, TypDokumentu' .
                ' WHERE DokumentBenefitu.IdBenefitu = ' . $params ->addParameter($idBenefitu) .
                ' AND Dokument.Id = DokumentBenefitu.Id' .
                ' AND TypDokumentu.Kod = Dokument.Typ' .
                ' ORDER BY Dokument.Datum',
                $params
            ));

        $output ->addData('html', $view ->getHtmlContent(true));
    }

    /**
     * @inheritDoc
     */
    public function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'podrobnostibenefitu' :
                $this ->nactiPodrobnostiBenefitu($output);
                break;

            case 'dokumentybenefitu' :
                $this ->nactiDokumentyBenefitu($output);
                break;
        }
    }

    public function handleAction($action)
    {
        if (!($this->pravo_vznik->jsouNejakePrava() || $this->pravo_schvalit->jsouNejakePrava() ||
            $this->pravo_vydat->jsouNejakePrava() || $this->pravo_rezervovat->jsouNejakePrava() || $this->pravo_sablony->jsouNejakePrava())){
            throw new NeniPravoException();
        }

        $this ->addTranslationsScript('benefity');

        TextManager::getInstance()->loadStrings('benefity.ini');
        HtmlHeaders::getInstance()->addScript('script/benefity.js');
        HtmlHeaders::getInstance()->setTitle(TextManager::_('RON_Portal') . ' - ' . TextManager::_('Benefity'));
        HtmlHeaders::getInstance()->setIcon(RonPortalSettings::getValue('shortcut_icon') ?: 'images/icon_person.png');

        $action = isset($_GET['action']) ? $_GET['action'] : null;

        switch ($action) {
            case 'vznikbodu':
                return $this->vznikBodu();
            case 'rezervacebodu':
                return $this->rezervaceBodu();
            case 'pohyb' :
                return $this->pohybBenefitu();

            case 'schvalenirezervace':
                //Přes odkaz
                return $this->nastavStavPohybu($_GET['record']);

            case 'cerpanirezervace':
                //Přes odkaz
                return $this->nastavStavPohybu($_GET['record'], true);

            case 'sablony' :
                return $this->sablonyBenefitu();

            case 'odstranitsablonu' :
                return $this->odstranitSablonu($_GET['record']);

            case 'translations' :
                return $this->translations();

            case 'dokumentdownload' :
                return $this->downloadDokument((int)$_GET['id'], (int)$_GET['record']);

            case 'podrobnostibenefitu' :
            case 'dokumentybenefitu' :
                // json
                return parent::handleAction($action);
            case 'pohyby' :
            default :
                //Generovani zaniku (nutne pro rezervovani z bodu), standardne by mel spustit agent automatizovane
                //(zde jen pro pripad, ze se tak nestane, dohodnuto s DaVa)
                $this->generovaniZaniku();

                $this->hlavickaVyberAkce();
                $b = new BenefityBrowser($this->database);
                $b->load();

                return $b;
        }
    }
}