<?php
/**
 * stanislav.elbl, 13.05.2022 13:14
 */

namespace RonPortal\Benefity;
use Ron\DB\Database;
use Ron\MessageManager;

/**
 * Class PohybBenefitu
 * @package RonPortal\Benefity
 */
class PohybBenefitu
{
    // druhy pohybu
    const DRUH_POHYBU_VZNIK_BODU = 11;
    const DRUH_POHYBU_VZNIK_BENEFITU = 12;
    const DRUH_POHYBU_CERPANI_BODU = 21;
    const DRUH_POHYBU_CERPANI_BENEFITU = 22;
    const DRUH_POHYBU_ZANIK_BODU = 31;
    const DRUH_POHYBU_ZANIK_BENEFITU = 32;
    const DRUH_POHYBU_REZERVACE_BODU = 41;
    const DRUH_POHYBU_REZERVACE_BENEFITU = 42;
    const DRUH_POHYBU_ZANIK_REZERVACE_BODU = 51;
    const DRUH_POHYBU_ZANIK_REZERVACE_BENEFITU = 52;
    // zdroje
    const DRUH_ZDROJE_RUCNI = 0;
    const DRUH_ZDROJE_DOCHAZKA = 1;
    const DRUH_ZDROJE_APLIKACE = 2;
    const DRUH_ZDROJE_PORTAL = 3;
    const DRUH_ZDROJE_IMPORT = 4;
    const DRUH_ZDROJE_SABLONA = 5;

    /** @var int    Id pohybu */
    public $id;

    /** @var string */
    public $rc;

    /** @var int    Pozor! Pro vznik/zanik bodu muze byt null! */
    public $idBenefitu;

    /** @var int    Doplneno z tabulky Prac. pomer (POZOR! neni jednoznacne!) */
    public $idPP;

    /** @var int    jedna z konstant DRUH_... */
    public $druhPohybu;
    /** @var int    jedna z konstant DRUH_ZDROJE */
    public $druhZdroje;
    /** @var string */
    public $schvalil;
    /** @var string */
    public $cerpaniUserName;
    /** @var int    0=bez, 1=hotovost, 2=srazka ze mzdy */
    public $typDoplatku;
    /** @var float */
    public $doplatekCastka;

    /** @var int */
    public $datumCerpani;
    /** @var int */
    public $datumOmezeniPlatnosti;
    /** @var int */
    public $datumRezervaceDo;
    /** @var int */
    public $datumVlozeni;
    /** @var int */
    public $datumSchvaleni;

    /** @var int */
    public $pocetBodu;
    /** @var int */
    public $pocetBenefitu;

    /** @var string */
    public $poznamka;

    /** @var string     Doplneno z tabulky S_DruhPohybuBenefitu */
    public $popisPohybuBenefitu;
    /** @var string     Doplneno z tabulky Benefit */
    public $typBenefitu;

    /**
     * Nacte 1 zaznam pohybu benefitu, ktery predstavuje rezervovani/vydani benefitu nebo pohyb s body (vznik/zanik)
     * @param Database $database
     * @param $id
     * @return PohybBenefitu
     * @throws \Exception
     */
    public static function nactiPohybBenefitu($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT ' .
            ' PB.ID, PB.RC, PB.DruhPohybu, PB.DruhZdroje, PB.IdBenefitu, PB.DatumVlozeni,' . // 0..5
            ' PB.DatumSchvaleni, PB.Schvalil, PB.DatumRezervaceDo,' .  // 6..8
            ' PB.DatumCerpani, PB.Cerpani_Username, PB.PocetBodu, PB.PocetBenefitu, PB.TypDoplatku, PB.DoplatekCastka,' .  // 9..14
            ' PB.DatumOmezeniPlatnosti, ' . $database->castWideMemo('PB.Poznamka') . ',' .// 15,16
            ' PB.Insert_Username, PB.Insert_Timestamp, PB.Update_Timestamp, PB.Update_Username,' .  // 17..20
            ' (SELECT MAX(PracovniPomer.ID) FROM PracovniPomer WHERE PracovniPomer.RC = PB.RC),' .  // 21
            ' S_DruhPohybuBenefitu.Popis, Benefit.TypBenefitu' .  // 22..23
            ' FROM PohybBenefitu PB' .
            ' LEFT JOIN PracovniPomer ON (PB.RC = PracovniPomer.RC)' .
            ' LEFT JOIN S_DruhPohybuBenefitu ON (S_DruhPohybuBenefitu.Kod = PB.DruhPohybu)' .
            ' LEFT JOIN Benefit ON (Benefit.ID = PB.IdBenefitu)' .
            ' WHERE PB.ID = ' . $params->addParameter((int)$id);

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor->close();

        if (!$row){
            throw new \Exception(\Ron\ronText('pohyb_benefitu_neexistuje'));
        }

        $pohybBenefitu = new PohybBenefitu();
        $pohybBenefitu->nactiPohybBenefituZRadku($database, $row);

        return $pohybBenefitu;
    }


    /**
     * @param Database $database
     * @param $row
     */
    protected function nactiPohybBenefituZRadku($database, $row)
    {
        $this->id = (int)$row[0];
        $this->rc = $row[1];
        $this->druhPohybu = (int)$row[2];
        $this->druhZdroje = (int)$row[3];
        // Pro body je hodnota "id_benefitu" NULL, jelikoz body nejsou napojeny na konkretni benefit
        $this->idBenefitu = $row[4];
        $this->datumVlozeni = $database->stringToTimestamp($row[5]);
        $this->datumSchvaleni = $database->stringToTimestamp($row[6]);
        $this->schvalil = $row[7];
        $this->datumRezervaceDo = $database->stringToTimestamp($row[8]);
        $this->datumCerpani = $database->stringToTimestamp($row[9]);
        $this->cerpaniUserName = $row[10];
        $this->pocetBodu = $row[11];
        $this->pocetBenefitu = $row[12];
        $this->typDoplatku = (int)$row[13];  // null = 0
        $this->doplatekCastka = isset($row[14]) ? (float)$row[14] : null;
        $this->datumOmezeniPlatnosti = $database->stringToTimestamp($row[15]);
        $this->poznamka = $row[16];

        $this->idPP = $row[21];
        $this->popisPohybuBenefitu = $row[22];
        $this->typBenefitu = $row[23];
    }
}
