<?php
/**
 * Roman.Szablatura RON Software 28.3.2017
 */

namespace RonPortal\Common;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use function Ron\ronText;

/**
 * Class Navsteva
 */
class Navsteva
{
    public $id;
    public $typNavstevy;
    public $kod;
    public $titul;
    public $prijmeni;
    public $jmeno;
    public $zamestnanecRC;
    public $zamestnanecOsobniCislo;
    public $zamestnanecDatumNastupu;
    public $zamestnanecTypRozdeleni;
    public $zamestnanecRozdeleni;
    public $firma;
    public $kodTypuRozdeleni_Firma;
    public $rozdeleni_Firma;
    public $prichod;
    public $odchod;

    /** @var string     RC prirazene externi osoby (karty) */
    public $osobaRC;
    /** @var string     OsobniCislo prirazene externi osoby (karty) */
    public $osobniCislo;
    /** @var int        DatumNastupu prirazene externi osoby (karty) */
    public $datumNastupu;
    /** @var string     Oznaceni prirazene karty (pres osobu) */
    public $oznaceniMedia;
    //public $osobaPrijmeni;

    public $pristeStejneMedium;

    public $vozidlo;
    public $navstivenaOsobaRC;
    public $jmenoNavstiveneOsoby;
    public $doklad;
    public $duvodNavstevy;
    public $duvodNavstevy2;
    public $kodTypuRozdeleni;
    public $rozdeleni;
    public $predemOcekavanaNavsteva;
    public $ocekavanyPrichod;
    public $kodTypuRozdeleni_Vratnice;
    public $rozdeleni_Vratnice;
    public $uzivatelskaPole;
    public $rozdeleni_VratnicePopis;
    public $typNavstevyPopis;
    public $zamestnanecPrijmeniJmenoTitul;
    //public $dobaNavstevy;
    public $insert_UserName;
    public $insert_Timestamp;
    public $update_UserName;
    public $update_Timestamp;
    public $prichodZapsalRC;
    public $navstevuSchvalil;
    public $navstevuSchvalilDatum;


    /**
     * @param Database $database
     * @param $id
     * @return Navsteva
     * @throws Exception
     */
    public static function nactiNavstevu($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT Navsteva.id, TypNavstevy, Navsteva.kod, Navsteva.titul, Navsteva.prijmeni, Navsteva.jmeno,' .   // 0..5
            ' zamestnanecRC, zamestnanecOsobniCislo, zamestnanecDatumNastupu, ' .                                   // 6..8
            ' zamestnanecTypRozdeleni, zamestnanecRozdeleni, firma,' .                                              // 9..11
            ' KodTypuRozdeleni_Firma, Rozdeleni_Firma, prichod, odchod,' .                                          // 12..15
            ' Navsteva.OsobaRC, Navsteva.OsobniCislo, Navsteva.DatumNastupu, pristeStejneMedium, ' .                // 16..19 - Prirazene ID medium
            ' vozidlo, navstivenaOsobaRC, jmenoNavstiveneOsoby, doklad, {fn_CASTWIDEMEMO(Navsteva.duvodNavstevy)},' .   // 20..24
            ' Navsteva.KodTypuRozdeleni, Navsteva.Rozdeleni, PredemOcekavanaNavsteva, Navsteva.OcekavanyPrichod,' . // 25..28
            ' KodTypuRozdeleni_Vratnice, Rozdeleni_Vratnice, ' .                                                    // 29..30
            ' Navsteva.Uzivatelska1, Navsteva.Uzivatelska2, Navsteva.Uzivatelska3, Navsteva.Uzivatelska4, Navsteva.Uzivatelska5,' .     // 31 - 35
            ' Navsteva.Uzivatelska6, Navsteva.Uzivatelska7, Navsteva.Uzivatelska8, Navsteva.Uzivatelska9, Navsteva.Uzivatelska10,' .    // 36 - 40
            ' Navsteva.Insert_UserName, Navsteva.Insert_Timestamp, Navsteva.Update_UserName, Navsteva.Update_Timestamp,' . // 41..44
            ' PrichodZapsalRC, Navsteva.Schvalil, Navsteva.DT_Schvaleni,' .         // 45..47
            ' DuvodNavstevy.popis, Karta.OznaceniMedia, Rozdeleni.Popis' .          // 48..50

            ' FROM Navsteva' .
            ' LEFT OUTER JOIN DuvodNavstevy' .
            '  ON (Navsteva.duvodNavstevy2 = DuvodNavstevy.kod)' .
            ' LEFT OUTER JOIN PrirazeniKarty' .
            '  ON PrirazeniKarty.RC = Navsteva.OsobaRC' .
            '  AND PrirazeniKarty.OsobniCislo = Navsteva.OsobniCislo' .
            '  AND PrirazeniKarty.DatumNastupu = Navsteva.DatumNastupu' .
            ' LEFT OUTER JOIN Karta' .
            '  ON Karta.Kod = PrirazeniKarty.KodKarty' .
            ' LEFT OUTER JOIN Rozdeleni' .
            '  ON  Rozdeleni.KodTypuRozdeleni = KodTypuRozdeleni_Vratnice' .
            '  AND Rozdeleni.Kod = Rozdeleni_Vratnice' .
            ' WHERE Navsteva.Id = ' . $params->addParameter($id);

        $cursor = $database->getSqlResult($sql, $params);
        if (!$row = $cursor->fetch()){
            throw new Exception(ronText('navsteva_neexistuje'));
        }
        $cursor ->close();

        $navsteva = new Navsteva();
        $navsteva->id = $id;
        $navsteva->nactiNavstevuZRadku($database, $row);

        return $navsteva;
    }

    /**
     * @param Database $database
     * @param $rc
     * @return string
     * @throws DBException
     */
    private function jmenoZamestnance($database, $rc)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul' .
            ' FROM Osoba' .
            ' WHERE RC = ' . $params->addParameter($rc);

        $cursor = $database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        return $row
            ? $row[0] . ' ' . $row[1] . ($row[2] ? ', ' . $row[2] : '')
            : '';
    }

    /**
     * @param Database $database
     * @param $row
     * @throws DBException
     */
    private function nactiNavstevuZRadku($database, $row)
    {
        $this->id = $row[0];
        $this->typNavstevy = $row[1];
        $this->kod = $row[2];
        $this->titul = $row[3];
        $this->prijmeni = $row[4];
        $this->jmeno = $row[5];
        $this->zamestnanecRC = $row[6];
        $this->zamestnanecOsobniCislo = $row[7];
        $this->zamestnanecDatumNastupu = $database->stringToTimestamp($row[8]);
        $this->zamestnanecTypRozdeleni = $row[9];
        $this->zamestnanecRozdeleni = $row[10];
        $this->firma = $row[11];
        $this->kodTypuRozdeleni_Firma = $row[12];
        $this->rozdeleni_Firma = $row[13];
        $this->prichod = $database->stringToTimestamp($row[14]);
        $this->odchod = $database->stringToTimestamp($row[15]);

        // prirazena karta
        $this->osobaRC = $row[16];
        $this->osobniCislo = $row[17];
        $this->datumNastupu = $database->stringToTimestamp($row[18]);
        $this->pristeStejneMedium = $row[19];

        $this->vozidlo = $row[20];
        $this->navstivenaOsobaRC = $row[21];
        $this->jmenoNavstiveneOsoby = $row[22];
        $this->doklad = $row[23];
        $this->duvodNavstevy = $row[24];
        $this->duvodNavstevy2 = $row[48];
        $this->kodTypuRozdeleni = $row[25];
        $this->rozdeleni = $row[26];
        $this->predemOcekavanaNavsteva = $row[27];
        $this->ocekavanyPrichod = $database->stringToTimestamp($row[28]);
        $this->kodTypuRozdeleni_Vratnice = $row[29];
        $this->rozdeleni_Vratnice = $row[30];
        $this->rozdeleni_VratnicePopis = $row[50];

        $this->uzivatelskaPole = [];
        for ($i=1; $i <=10; $i++) {
            $this->uzivatelskaPole[$i] = $row[30+$i];
        }

        $this->typNavstevyPopis = $row[1]
            ? ronText('vlastni')
            : ronText('cizi');

        $this->zamestnanecPrijmeniJmenoTitul = $this->jmenoZamestnance($database, $row[6]);
        //$this->dobaNavstevy = $this->dobaNavstevy();
        $this->insert_UserName = $row[41];
        $this->insert_Timestamp = $database->stringToTimestamp($row[42]);
        $this->update_UserName = $row[43];
        $this->update_Timestamp = $database->stringToTimestamp($row[44]);
        $this->prichodZapsalRC = $this->jmenoZamestnance($database, $row[45]);

        $this->oznaceniMedia = $row[49];
        //$this->osobaPrijmeni = $this->nacteniOsobaPrijmeni($database);

        $this->navstevuSchvalil = $row[46];
        $this->navstevuSchvalilDatum = $database->stringToTimestamp($row[47]);
    }
}
