<?php
/**
 * roman.Szablatura 27.3.2017
 */

namespace RonPortal\Common;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\Http\Csrf;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\StringUtils;
use RonPortal\RonPortalSettings;
use RonPortal\System\Report;
use RonPortal\System\ReportOutputObject;
use function Ron\ronText;
use Ron\TextManager;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\User;

/**
 * Class NavstevyController
 */
class NavstevyController extends JsonController
{
    const XML_EVENT_SCHVALENI = 'NAVSTEVA_SCHVALENI';
    const XML_EVENT_AFTER_SCHVALENI = 'NAVSTEVA_AFTER_SCHVALENI';

    /** @var  Database $database */
    private $database;

    public function __construct(Database $database)
    {
        $this->database = $database;
    }

    /**
     * @param string $formAction
     * @return Redirect|NavstevaForm
     * @throws Exception
     */
    private function navstevyForm($formAction)
    {
        if (isset($_POST['record'])) {
            $idNavstevy = $_POST['record'];
        } else if (isset($_GET['record'])) {
            $idNavstevy = $_GET['record'];
        } else {
            $idNavstevy = null;
        }

        if (isset($_POST['ocekavana'])) {
            $ocekavana = $_POST['ocekavana'];
        } else if (isset($_GET['ocekavana'])) {
            $ocekavana = $_GET['ocekavana'];
        } else {
            $ocekavana = '';
        }

        $navsteva = ($formAction == 'edit')
            ? Navsteva::nactiNavstevu($this->database, $idNavstevy)
            : null;

        if ($navsteva) {
            $prava = Prava::vratPrava('navstevy');
            if (!$prava ->lzeZobrazitRC($navsteva ->navstivenaOsobaRC)) {
                throw new NeniPravoException();
            }
        }

        $form = new NavstevaForm($this->database, $formAction, $ocekavana, $navsteva);

        if($form->handleAction()){
            $rozdelenivratnice = (isset($_SESSION['rozdelenivratnice']) && $_SESSION['rozdelenivratnice'] != "") ? ("&rozdelenivratnice=" . $_SESSION['rozdelenivratnice']) : "";
            return new Redirect(null, "?menuid=navstevy$rozdelenivratnice");
        }
        return $form;
    }


    private function zkontrolujPravoNaNavstevu($id, $typPrava = 'navstevy', $zkontrolovatPriznak = false)
    {
        $params = $this ->database ->createParameters();
        $osobaRC = DBUtils::getSqlValue($this->database,
            'SELECT navstivenaOsobaRC FROM Navsteva' .
            ' WHERE ID = '. $params ->addParameter((int)$id),
            $params);

        // Pokud ma pravo jen na ocekavane navstevy (priznak) nebo nema pravo na RC navstivene osoby
        $prava = Prava::vratPrava($typPrava);

        if ($zkontrolovatPriznak  &&  $prava ->priznak) { // priznak nastaven => uzivatel muze zapisovat pouze ocekavane navstevy
            throw new NeniPravoException();
        }

        if (!$prava->lzeZobrazitRC($osobaRC)) {
            throw new NeniPravoException();
        }
    }

    /**
     * @param int $id
     * @return Redirect
     * @throws DBException
     * @throws Exception
     */
    private function potvrzeniOdchodu($id)
    {
        $this ->zkontrolujPravoNaNavstevu($id, 'navstevy', true); // zkontrolovat i priznak, tj. ze muze editovat i prichod a odchod

        $recVlozitOdchod = new RonPortalDBRecord($this->database, 'Navsteva', $this);

        $recVlozitOdchod->addField('Id', 'int', true);
        $recVlozitOdchod->addField('TypNavstevy', 'int', true);
        $recVlozitOdchod->addField('Prijmeni', 'string', true);
        $recVlozitOdchod->addField('PredemOcekavanaNavsteva', 'int', true);

        $recVlozitOdchod->addField('Odchod', 'timestamp', false, 0,time());
        $recVlozitOdchod->addField('Update_UserName', 'string', false, 25, User::getLoginName());
        $recVlozitOdchod->addField('Update_Timestamp', 'timestamp', false, 0,time());

        $params = $this->database->createParameters();
        $recVlozitOdchod->setWhere('Id = ' . $params->addParameter($id), $params);

        $recVlozitOdchod->loadAndUpdateRecord();

        return new Redirect(null,'?menuid=navstevy');
    }


    /**
     * @param $idNavstevy
     * @return BaseViewObject
     * @throws Exception
     */
    private function zobrazDetailNavstevy($idNavstevy)
    {
        $navsteva = Navsteva::nactiNavstevu($this->database, $idNavstevy);

        $prava = Prava::vratPrava('navstevy');
        if (!$prava ->lzeZobrazitRC($navsteva ->navstivenaOsobaRC)) {
            throw new NeniPravoException();
        }

        Model::addData('navsteva', $navsteva);
        Model::addData('reportlist', Report::loadFormReports($this ->database, 'Navstevy'));

        return new BaseViewObject('ronportal/common/view_navsteva.php');
    }


//    /**
//     * @param $idpp
//     * @param JsonOutputObject $output
//     * @throws DBException
//     */
//    private function nacteniZamestnance($idpp, $output)
//    {
//        $params = $this->database->createParameters();
//        $sql =
//            'SELECT Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul' . // , RO.Rozdeleni, Rozdeleni.Popis' .
//            ' FROM PracovniPomer PP INNER JOIN Osoba ON (Osoba.rc = PP.rc)' .
//            //' LEFT OUTER JOIN RozdeleniOsob RO ON ((RO.rc = PP.rc) AND (RO.OsobniCislo = PP.OsobniCislo) ' .
//            //' AND (RO.DatumNastupu = PP.DatumNastupu) AND (RO.KodTypuRozdeleni = 6) AND (RO.PlatnostOd <= ' . $params->addParameter($this->database->timestampToParam(time())) .') '.
//            //' AND ((RO.PlatnostDo >=  ' . $params->addParameter($this->database->timestampToParam(time())) . ' ) OR (RO.PlatnostDo IS NULL)))) ' .
//            //' LEFT JOIN Rozdeleni ON ((Rozdeleni.KodTypuRozdeleni = RO.KodTypuRozdeleni) AND (Rozdeleni.Kod = RO.Rozdeleni))) '.
//            ' WHERE ( (PP.datumNastupu <=  ' . $params->addParameter($this->database->timestampToParam(time())) . ' ) ' .
//            ' AND ((PP.datumUkonceni >=  ' . $params->addParameter($this->database->timestampToParam(time())) . ' ) OR (PP.datumUkonceni IS NULL)) AND (PP.Zamestnanec > 0) ) ' .
//            ' AND PP.ID = ' . $params->addParameter($idpp);
//
//        $cursor = $this->database->getSqlResult($sql, $params);
//        $row = $cursor->fetch();
//        $cursor ->close();
//
//        if ($row) {
//            $output ->addData('prijmeni', StringUtils::jsonText($row[0]));
//            $output ->addData('jmeno', StringUtils::jsonText($row[1]));
//            $output ->addData('titul', StringUtils::jsonText($row[2]));
//            //$output ->addData('rozdeleni', StringUtils::jsonText($row[3]));
//            //$output ->addData('popis', StringUtils::jsonText($row[4]));
//        }
//    }


    /**
     * @param int $id
     * @param bool $returnToBrowser
     * @return Redirect
     * @throws DBException
     * @throws Exception
     */
    private function schvaleniNavstevy($id, $returnToBrowser)
    {
        try {
            Csrf::validateCSRFGetParam();

            $this ->zkontrolujPravoNaNavstevu($id, 'navstevy_schvaleni');

            $rec = new RonPortalDBRecord($this->database, 'Navsteva', $this);

            $rec->addField('Id', 'int', true);
            $rec->addField('TypNavstevy', 'int', true);
            $rec->addField('PredemOcekavanaNavsteva', 'int', true);

            $rec->addField('navstivenaOsobaRC', '', true);
            $rec->addField('jmenoNavstiveneOsoby', '', true);
            $rec->addField('jmeno', '', true);
            $rec->addField('prijmeni', 'string', true);
            $rec->addField('prichod', 'timestamp', true);
            $rec->addField('OcekavanyPrichod', 'timestamp', true);
            $rec->addField('duvodNavstevy', 'memo', true);

            $rec->addField('Schvalil', 'string', false, 25, User::getLoginName());
            $rec->addField('DT_Schvaleni', 'timestamp', false, 0, time());
            $rec->addField('Update_UserName', 'string', false, 25, User::getLoginName());
            $rec->addField('Update_Timestamp', 'timestamp', false, 0, time());

            $params = $this->database->createParameters();

            $rec->setWhere('Id = ' . $params->addParameter($id), $params);

            $rec->loadRecord();

            $rec->executeXmlEvents(self::XML_EVENT_SCHVALENI);
            $rec->updateRecord();
            $rec->executeXmlEvents(self::XML_EVENT_AFTER_SCHVALENI);
        } catch (DBException $e) {
            throw $e;
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        if ($returnToBrowser) {
            return new Redirect(null, '?menuid=navstevy#schvaleno');
        }

        return new Redirect(null,'?menuid=navstevy&action=zaznam&record=' . $id);
    }



    function report()
    {
        if (!isset($_GET['id'])) {
            throw new Exception('Neni zadana navsteva pro tisk!');
        }

        $this ->zkontrolujPravoNaNavstevu((int)$_GET['id']);

//        $prava = Prava::vratPrava('navstevy');

//        $navsteva = Navsteva::nactiNavstevu($this ->database, (int)$_GET['id']);
//        if (!$prava ->lzeZobrazitRC($navsteva ->navstivenaOsobaRC)) {
//            throw new NeniPravoException();
//        }

        $result = new ReportOutputObject();
        $result->loadReportFromGet($this->database, 'Navstevy');

        $result->integerParams['ID_Navstevy'] = $_GET['id'];

        return $result;
    }


//    function handleJsonAction($action, $output)
//    {
//        switch ($_GET['action']){
//            case 'nactenizamestnance':
//                $this->nacteniZamestnance($_GET['idpp'], $output);
//                break;
//        }
//    }

    /**
     * @param string $cisloDokladu
     * @param JsonOutputObject $output
     * @return void|null
     * @throws DBException
     */
    private function predchoziNavsteva(string $cisloDokladu, JsonOutputObject $output)
    {
        $platnost = RonPortalSettings::getIntValue('platnost_skoleni_navstevy');
        if (!$platnost) {
            return;
        }

        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT DatumAbsolvovaniSkoleni, Prichod' .
            ' FROM Navsteva WHERE Doklad = ' . $params ->addParameter($cisloDokladu) .
            ' AND Prichod >= ' . $params ->addDateParameter( DateUtils::incDay(time(), -2*$platnost)) .
            ' AND DatumAbsolvovaniSkoleni IS NOT NULL' .
            ' ORDER BY Prichod DESC',
            $params
        );

        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            $datumAbsolvovani = $this ->database ->stringToTimestamp($row[0]);
            $output ->addData('DatumAbsolvovaniSkoleni', StringUtils::format('date', $datumAbsolvovani));

            $platnostDo = DateUtils::incDay($datumAbsolvovani, $platnost);
            $output ->addData('PlatnostSkoleniDo', StringUtils::format('date', $platnostDo));

            $output ->addData('SkoleniPlatne', $platnostDo >= DateUtils::today() ? 1 : 0);
        }
    }


    function handleJsonAction($action, $output)
    {
        if (!Prava::jsouNejakePrava('navstevy')) {
            throw new NeniPravoException();
        }

        switch ($action) {
            case 'predchozinavsteva' :
                $this ->predchoziNavsteva($_GET['doklad'] ?? '', $output);
                break;
        }
    }


    /**
     * @param string $action
     * @return mixed
     * @throws DBException
     * @throws Exception
     */
    public function handleAction($action)
    {
        Prava::nastavObdobiProPrava(time(), time());

        TextManager::getInstance()->loadStrings('navstevy.ini');

        HtmlHeaders::getInstance()->addScript('script/navstevy.js');
        HtmlHeaders::getInstance()->setTitle(\Ron\ronText('RON_Portal') . ' - ' . \Ron\ronText('Navstevy'));

        $this ->addTranslationsScript('navstevy');

        if (!Prava::jsouNejakePrava('navstevy')){
            throw new NeniPravoException();
        }

        // Pri spusteni se kontroluje v URL parametr "rozdelenivratnice" pokud je, nacte se jeho hodnota do session.
        // Pote se jiz pracuje pouze s danou session
        if (isset($_GET['rozdelenivratnice'])){
            $_SESSION['rozdelenivratnice'] = $_GET['rozdelenivratnice'];
        }

        if (isset($_POST['formaction'])) {
            $formAction = $_POST['formaction'];
        } else if (isset($_GET['formaction'])) {
            $formAction = $_GET['formaction'];
        } else {
            $formAction = null;
        }

        if ($formAction){
            return $this->navstevyForm($formAction);
        }


        if ($_GET['action'] ?? false) {
            switch ($_GET['action']) {
//                case 'nactenizamestnance' :
//                    return parent::handleAction($action);

                case 'translations':
                    return $this->translations();

                case 'zaznam':
                    return $this->zobrazDetailNavstevy($_GET['record']);

                case 'potvrzeniodchodu':
                    return $this->potvrzeniOdchodu((int)$_GET['id']);

                case 'schvaleninavstevy':
                    return $this->schvaleniNavstevy((int)$_GET['id'], $_GET['browser'] ?? false);

                case 'report' :
                    return $this->report();

                default :   // json
                    return parent::handleAction($_GET['action']);
            }
        }

        $b = new NavstevyBrowser($this->database);
        $b->load();
        return $b;
    }
}
