<?php
/**
 * stanislav.elbl, 25.01.2021
 * Seznam poznamek ke dni (z tabulky PoznamkaKeDni)
 */

namespace RonPortal\Common;

use Ron\Controls\BrowserField;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RozdeleniUtils;


/**
 * Class PoznamkaKeDniDetailField
 * Proklik na dialog
 */
class PoznamkaKeDniDetailField extends BrowserField
{

    /**
     * PoznamkaKeDniDetailField constructor.
     */
    public function __construct()
    {
        parent::__construct();
        $this ->setType(BrowserField::FIELD_TYPE_TEXT);
        //$this ->link = '#';
    }

    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return
            "<span class='open-dialog js' data-dialog='#poznamky_ke_dni_dialog'" .
            //" data-ronform-url='?menuid=poznamkykedni&action=loaddata&record={$rowData[0]}'>" .
            " data-ronform-record='{$rowData['col-id']}'>" .
            parent::getViewText($value, $rowNumber, $recordId, $rowData) .
            '</span>';
    }
}


/**
 * Class PoznamkyKeDniBrowser
 * Seznam poznamek ke dni (z tabulky PoznamkaKeDni)
 */
class PoznamkyKeDniBrowser extends RonPortalBrowser
{

    /**
     * PoznamkyKeDniBrowser constructor.
     * @param Database $database
     * @param string $action
     */
    public function __construct($database, $action)
    {
        parent::__construct($database, $action);

        $this ->title = TextManager::_('Poznamky_ke_dni');
        $this ->code = 'browserpoznamkykedni';

        $this ->sqlFromPart =
            'PoznamkaKeDni INNER JOIN TypPoznamkyKeDni' .
            '  ON PoznamkaKeDni.Typ = TypPoznamkyKeDni.Kod' .
            ' LEFT OUTER JOIN Rozdeleni' .
            '  ON Rozdeleni.KodTypuRozdeleni = PoznamkaKeDni.KodTypuRozdeleni' .
            '  AND Rozdeleni.Kod = PoznamkaKeDni.Rozdeleni';

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PoznamkaKeDni.Id')
            ->setCode('col-id');

        $this ->addField()
            ->setSqlFieldName('PoznamkaKeDni.DatumOd') ->setFormat('date') ->setHeaderText('Datum_od')
            ->setCode('col-datum-od')
            ->setOrdering(true);

        $this ->defaultSort = '-col-datum-od';

        $this ->addField()
            ->setSqlFieldName('PoznamkaKeDni.DatumDo') ->setFormat('date') ->setHeaderText('Datum_do')
            ->setCode('col-datum-do')
            ->setOrdering(true);

        $this ->addField()
            ->setSqlFieldName('TypPoznamkyKeDni.Popis') ->setHeaderText('Typ')
            ->setCode('col-typ')
            ->setOrdering(true);

        $this ->addField()
            ->setSqlFieldName('PoznamkaKeDni.Popis') ->setHeaderText('Popis')
            ->setCode('col-popis')
            ->setOrdering(true);

        $this ->addField()
            ->setSqlFieldName('Rozdeleni.Popis') ->setHeaderText('Rozdeleni')
            ->setCode('col-rozdeleni')
            ->setOrdering(true);

        $this ->addField()
            ->setSqlFieldName('PoznamkaKeDni.Rozdeleni') ->setHeaderText('Kod')
            ->setCode('col-kod-rozdeleni')
            ->setOrdering(true);

        $this ->addField(new PoznamkaKeDniDetailField())
            ->setCode('col-detail') ->setText('Detail')
            ->setClassName('dialogopen');
    }


    public function setFilterForm()
    {
        $this ->filterForm = new RonPortalFilterForm($this ->database, 'poznamkykedni', true);

        $this ->filterForm ->addFilterRozdeleni1(null, Prava::vratPrava('poznamkykedni'));
//        $this ->filterForm ->nastavRozdeleni(true);
//        $this ->filterForm ->f_pravarozdeleni = Prava::vratPrava('poznamkykedni');

        $this ->filterForm ->addFilter(
            (new FilterFormFilterDatumOdDo($this ->filterForm))
                ->setPovolitPrazdneDatumDo(true)
                ->addLink([
                        'datumOd' => DateUtils::today(),
                        'datumDo' => null,
                        'text' => TextManager::_('Poznamky_ke_dni_aktualni')
                    ]
                )
                ->addLink([
                        'datumOd' => DateUtils::today(),
                        'datumDo' => DateUtils::today(),
                        'text' => TextManager::_('Dnes')
                    ]
                )
                ->setRequired(false),
            true
        );

//        $this ->filterForm ->nastavDatumOdDo(true, false);
//        $this ->filterForm ->povolitPrazdneDatumDo = true;

//        $this ->filterForm ->datumOdDoLinks = [
//            [
//                'datumOd' => DateUtils::today(),
//                'datumDo' => null,
//                'text' => TextManager::_('Poznamky_ke_dni_aktualni')
//            ],
//            [
//                'datumOd' => DateUtils::today(),
//                'datumDo' => DateUtils::today(),
//                'text' => TextManager::_('Dnes')
//            ],
//        ];

        $this ->filterForm ->addFilter(
            (new FilterFormFilter('vlozitNovouPoznamku', FilterFormFilter::TYPE_TEXT))
                ->setPosition(FilterFormFilter::POSITION_BELOW_FORM)
                ->setDefaultValue(
                    '<a href="#" class="open-dialog" data-dialog="#poznamky_ke_dni_dialog">' .
                    TextManager::_('Vlozit_novou_poznamku') .
                    '</a>')
                ->setRowClass('margin-top-2')
                ->setCaption(false)
        );

//        echo
//            '<div class="margin-top-2">' .
//            '<span class="open-dialog" data-dialog="#poznamky_ke_dni_dialog">' .
//            '<a href="#">' . TextManager::_('Vlozit_novou_poznamku') . '</a>' .
//            '</span>' .
//            '</div>';
    }


    public function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        if ($this ->filterForm ->getFilterValue('datumod')) {
            $datumDo = $this ->filterForm ->getFilterValue('datumod', 'datumDo');

            $result .=
                ($result ? ' AND ' : '') .
                'DatumDo >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod')) .
                ($datumDo
                    ? ' AND DatumOd <= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod', 'datumDo'))
                    : ''
                );
        }

        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            if ($rozdeleniFilter ->getValue()) {
                $filterRozdeleni = RozdeleniUtils::sqlFilterRozdeleni($this ->database, 'PoznamkaKeDni',
                    $rozdeleniFilter ->getTypRozdeleni(), $rozdeleniFilter ->getValue(), $rozdeleniFilter ->getValue('vcetnePodrizenych'),
                    $params);

                if ($filterRozdeleni) {
                    $result .=
                        ($result ? ' AND ' : '') .
                        $filterRozdeleni;
                }
            }
        }

//        if (isset($this ->filterForm ->kodRozdeleni)) {
//            $filterRozdeleni = RozdeleniUtils::sqlFilterRozdeleni($this ->database, 'PoznamkaKeDni',
//                $this ->filterForm ->kodTypuRozdeleni, $this ->filterForm ->kodRozdeleni, $this ->filterForm ->vcetnePodrizenychRozdeleni, $params);
//
//            if ($filterRozdeleni) {
//                $result .=
//                    ($result ? ' AND ' : '') .
//                    $filterRozdeleni;
//            }
//        }

        $prava = Prava::vratPrava('poznamkykedni');
        $filtrPrava = $prava ->sqlfiltrRozdeleni(
            'PoznamkaKeDni.DatumOd','PoznamkaKeDni.DatumDo',
            // $this ->filterForm ->getFilterValue('datumod'),  $this ->filterForm ->getFilterValue('datumod', 'datumDo'),
            $params,
            1); // v pripade, ze nejde filtrovat podle prav, tak nezobrazi nic

        if ($filtrPrava) {
            $result .= ($result ? ' AND ' : '') . ' (' . $filtrPrava . ')';
        }

        return $result
            ? new SqlStatement($result, $params)
            : '';
    }


    public function render()
    {
        parent::render();

        // zobrazi dialog pro vkladani a editaci
        $form = new PoznamkyKeDniFormular($this ->database);
        $form ->render();
    }
}
