<?php

namespace RonPortal\Common;

use Ron\Controls\BrowserField;
use Ron\Controls\BrowserRowArray;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\FilterFormFilterRozdeleni;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Kapacity\KapacitySettings;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\TypRozdeleni;
use RonPortal\User;

class EditaceUdalostiField extends BrowserField
{
//    var $f_MuzeEditovat;
//    var $f_ZpetnaEditaceSekund;
    /** @var PravaNaOsoby */
    var $prava;

    /** @var Database */
    var $database;

    var $rozdeleniPK;

    /**
     * @param PravaNaOsoby $prava
     * @param Database $database
     * @param string $rozdeleniPK
     */
    function __construct($prava, $database, $rozdeleniPK)
    {
        parent::__construct($database);

        $this->prava = $prava;
        $this->rozdeleniPK = $rozdeleniPK;

        $this
            ->setType(BrowserField::FIELD_TYPE_TEXT)
            ->setLink('?menuid=provoznidenik&action=denikform&formaction=edit&record=')
            ->setText(TextManager::_('editovat'));
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param BrowserRowArray $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($this->prava->priznak
            && (($this->prava->priznak2 == 0) || ($this->database->stringToTimestamp($rowData[1]) > DateUtils::incMinute(time(),- $this->prava->priznak2)))
            && (($this->prava->lzeZobrazitRC($rowData['col-rc'], time(), time()))
                && ((($this->prava->jePravoNaRozdeleni($this->prava->typRozdeleni,$rowData['col-kodrozdeleni']))) ||
                    ($this->rozdeleniPK == $rowData['col-kodrozdeleni'] &&  User::hasRC($rowData['col-rc'])))
            )
        ) {
            return
                parent::getViewText($value, $rowNumber, $recordId, $rowData);
        }

        return '';
    }
}

class ProvozniDenikBrowser extends RonPortalBrowser
{

    var $pravaZobrazit;
    var $pravaZapsat;

    /** @var int    Typ rozdeleni, podle ktereho se zaznamy filtruji */
    private $typRozdeleni;
    /** @var string|null    Kod rozdeleni - muze se urcit podle planovani kapacit */
    private $rozdeleni;

    /** @var string[]   Seznam XML formularu k aktualne nactenemu typu udalosti */
    private $seznamXmlFormularu = [];

    /**
     * ProvozniDenikBrowser constructor.
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database, 'provoznidenik');

        $this->pravaZobrazit = Prava::vratPrava('pr_denik_read');
        $this->pravaZapsat = Prava::vratPrava('pr_denik_write');
        Prava::nastavObdobiProPrava(time(),time());
        $this->setTitle('provozni_denik');
        $this->code = 'browserprovoznidenik';
        $this->tableId = 'browserprovoznidenik';

        $this->sqlSelectPart =
            'ProvozniDenik.ID, ProvozniDenik.DatumCas, ' . // 0, 1
            '(SELECT MAX(OsobniCislo) FROM PracovniPomer WHERE PracovniPomer.RC = ProvozniDenik.RC ' .
            '   AND PracovniPomer.DatumNastupu <= {fn_GETDATE()}' .
            '   AND (PracovniPomer.DatumUkonceni >= {fn_GETDATE()} OR PracovniPomer.DatumUkonceni IS NULL) ' .
            '   AND NOT EXISTS(SELECT RC FROM PracovniPomer PP WHERE PP.RC = PracovniPomer.RC AND PP.DatumNastupu > PracovniPomer.DatumNastupu '.
            '     AND PP.DatumNastupu <= {fn_GETDATE()}' .
            '     AND (PP.DatumUkonceni >= {fn_GETDATE()} OR PP.DatumUkonceni IS NULL) )) AS OsobniCislo'; // 2

        $this->sqlFromPart =
            'ProvozniDenik JOIN TypUdalostiPD ON (TypUdalostiPD.Kod = ProvozniDenik.TypUdalosti)'.
            'LEFT OUTER JOIN Rozdeleni ON ((Rozdeleni.KodTypuRozdeleni = ProvozniDenik.KodTypuRozdeleni) AND (Rozdeleni.Kod = ProvozniDenik.KodRozdeleni))'.
            'LEFT OUTER JOIN Osoba ON (Osoba.RC = ProvozniDenik.RC)';


        $this->primaryField = 0;

        //$this->browserUrl = '?menuid=provoznidenik&';

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.RC')
            ->setCode('col-rc');

        $this ->addField()
            ->setSqlFieldName('ProvozniDenik.DatumCas', 1)
            ->setCode('col-datum')
            ->setHeaderText('Date')
            ->setFormat('date')
            ->setOrdering(true);

        $this ->defaultSort = 'col-datum';

        $this ->addField()
            ->setSqlFieldName('ProvozniDenik.DatumCas', 1)
            ->setCode('col-cas')
            ->setHeaderText('Cas')
            ->setFormat('time')
            ->setOrdering(true);

        $this ->addField()
            ->setSqlFieldName('ProvozniDenik.KodRozdeleni')
            ->setCode('col-kodrozdeleni')
            ->setHeaderText('Kod')
            ->setOrdering(true);

        $this ->addField()
            ->setSqlFieldName('Rozdeleni.Popis')
            ->setCode('col-rozdeleni-popis')
            ->setHeaderText('rozdeleni')
            ->setOrdering(true);

        $this ->addField()
            ->setSqlFieldName('TypUdalostiPD.Popis')
            ->setCode('col-typudalosti')
            ->setHeaderText('popis_typu_udalosti')
            ->setOrdering(true);

        $this ->addField()
            ->setSqlFieldName('Osoba.Prijmeni')
            ->setCode('col-prijmeni')
            ->setHeaderText('Prijmeni')
            ->setOrdering(true);

        $this ->addField()
            ->setSqlFieldName('Osoba.Jmeno')
            ->setCode('col-jmeno')
            ->setHeaderText('Jmeno')
            ->setOrdering(true);

        $this ->addField()
            ->setSqlFieldName('OsobniCislo', 2)
            ->setCode('col-osobnicislo')
            ->setHeaderText('Osobni_cislo')
            ->setOrdering(true);

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('ProvozniDenik.Popis')
            ->setCode('col-popis')
            ->setHeaderColSpan(-1)
            ->setClassName('font-weight-bold')
            ->setHeaderText(TextManager::_('popis') . ': ');

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('ProvozniDenik.Poznamka') ->setCastWideMemo(true)
            ->setCode('col-poznamka')
            ->setHeaderColSpan(-1)
            ->setClassName('poznamka')
            ->setHeaderText(TextManager::_('text_cinnosti') . ': ');


        $this->rozdeleni = null;

        if (RonPortalSettings::getIntValue('pd_pouzit_rozdeleni_kapacity')) {
            try {
                // nacte rozdeleni podle planovani kapacit - pokud uzivatel nejake pro aktualni den ma
                $this->rozdeleni = ProvozniDenikController::vratRozdeleniPlanovaniKapacit($database, User::getRC(), time());
            } catch (\Exception $e) {
                // v pripade, ze jich je vice (muze nastat, pokud ma vice PP), tak chybu jen zobrazi
                MessageManager::addWarning($e->getMessage());
            }
        }

        if (RonPortalSettings::getIntValue('pd_pouzit_rozdeleni_kapacity') && (isset($this->rozdeleni))) {
            $this->typRozdeleni = RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI);
        }
        else {
            $this->typRozdeleni = $this->pravaZobrazit->typRozdeleni;
        }

        $this->addField(new EditaceUdalostiField($this->pravaZapsat, $database, $this->rozdeleni));
    }


    function setFilterForm()
    {
        //$this ->filterForm = new RonPortalFilterForm($this ->database, 'provoznidenik', true);
        $this ->filterForm = new RonPortalFilterForm($this ->database, 'provoznidenik', true);

        $this ->filterForm ->addFilter(
            (new FilterFormFilterDatumOdDo($this ->filterForm))
                ->setCaption(TextManager::_('datum_udalosti') . ' ' . TextManager::_('Od')),
            true
        );

        // rozdeleni
        $filtrRozdeleni = (new FilterFormFilterRozdeleni($this ->filterForm, 'rozdeleni'))
            ->setTypRozdeleni($this ->typRozdeleni)
            ->setShowVcetnePodrizenych(0)
            ->setPrava($this ->pravaZobrazit);

        if (isset($this ->rozdeleni)) {
            $params = $this ->database ->createParameters();
            $filtrRozdeleni
                ->setOptionsSqlFilter(new SqlStatement('Kod = ' . $params ->addParameter($this ->rozdeleni), $params));
        }
        else {
            $filtrRozdeleni
                ->setJenVlastniRozdeleni(2)
                ->setPouzitPopisTypuRozdeleni(true);
        }

        $this ->filterForm ->addFilterRozdeleni($filtrRozdeleni);


        $this ->filterForm ->addFilter(
            (new FilterFormFilter('udalostpd', FilterFormFilter::TYPE_OPTIONS))
                ->addItem('', TextManager::_('_vse_'))
                ->setOptionsSql(
                    'SELECT Kod, Popis FROM TypUdalostiPD WHERE %FILTER% ORDER BY %ORDER%', 'Aktivni = 1', 'Popis')
                ->setCaption(TextManager::_('jen_udalost'))
                ->setAddClass('ronselect')
                ->setSqlFilter('TypUdalostiPD.Kod = ')
        );

        $this ->filterForm ->addFilter(
            (new FilterFormFilter('jenaktivniudalostpd', FilterFormFilter::TYPE_CHECKBOX))
                ->setCaption(TextManager::_('jen_aktivni_udalost'))
                ->setSqlFilter('TypUdalostiPD.Aktivni = 1')
        );

        // zobrazeni rozdeleni podle planovani kapacit
        if (RonPortalSettings::getIntValue('pd_pouzit_rozdeleni_kapacity') && isset($this->rozdeleni)) {
            $rozdeleni = Rozdeleni::loadRozdeleni($this ->database, $this ->typRozdeleni, $this ->rozdeleni);
            $this ->filterForm ->addFilter(
                (new FilterFormFilter('pouzito_rozdeleni', FilterFormFilter::TYPE_TEXT))
                    ->setCaption(false)
                    ->setDefaultValue(
                        sprintf(TextManager::_('Pouzito_rozdeleni_z_PK'),
                            StringUtils::htmlTextConvert(TypRozdeleni::getPopisTypuRozdeleni($this ->database, $this ->typRozdeleni)),
                            StringUtils::htmlTextConvert($rozdeleni ->popis)
                        )
                    )
                    ->setAddClass('font-weight-bold')
                    ->setRowClass('margin-top-3')
            );
        }


        // odkaz na vlozeni noveho zaznamu
        if ($this->pravaZapsat->jsouNejakePrava()) {
            $this ->filterForm ->addFilter(
                (new FilterFormFilter('nova_udalost', FilterFormFilter::TYPE_TEXT))
                    ->setDefaultValue('<a class="text_link" href="?menuid=provoznidenik&action=denikform&formaction=new">' . TextManager::_('nova_udalost') . '</a>')
                    ->setPosition(FilterFormFilter::POSITION_BELOW_FORM)
                    ->setRowClass('margin-top-2 margin-bottom-2')
                    ->setCaption(false)
            );
        }

        //         $combo = new ComboBox('udalostPD');
        //        $combo->className = 'app_combo adsfilterforminput_full';
        //        $combo ->emptyRow = ' ' ;
        //        $combo->loadData($this ->database, $sql, $_SESSION['udalostPD']);
        //        $combo ->htmlParams = 'onChange="if (jenUdalostPD.checked) submit();"';
        //        $combo->render();
    }


//    /**
//     * @param string $sqlFilter
//     * @throws Exception
//     */
//    function load($sqlFilter = '')
//    {
//        if (isset($_POST['adsfilterform'])) {
//            $_SESSION['jenAktivniUdalostPD'] = isset($_POST['jenAktivniUdalostPD']);

//            $_SESSION['jenUdalostPD'] = isset($_POST['jenUdalostPD']);

//            $_SESSION['udalostPD'] = $_POST['udalostPD'];
//        }

//        $this ->filterForm = new ProvozniDenikFilterForm($this ->database, $this->pravaZobrazit,$this->pravaZapsat->jsouNejakePrava(),$this->typRozdeleni, $this->rozdeleni);
//        $this ->filterForm ->loadData();
//
//        parent::load($sqlFilter);
//    }

    function renderFilter()
    {
        $this ->filterForm ->render();
    }

    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();

        $result = $this ->filterForm ->buildSqlFilter($params);

        if (RonPortalSettings::getIntValue('pd_pouzit_rozdeleni_kapacity') && isset($this->rozdeleni)) {
            //rozdeleni bude zobrazeno podle prirazeneho rozdeleni v planovani kapacit
            $result .=
                ($result ? ' AND ' : '') .
                'EXISTS(SELECT Rozdeleni FROM PlanovaniKapacit WHERE PlanovaniKapacit.RC = ' . $params ->addParameter(User::getRC()) .
                ' AND PlanovaniKapacit.KodTypuRozdeleni = ' . (int)RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI) .
                ' AND PlanovaniKapacit.Rozdeleni = ProvozniDenik.KodRozdeleni' .
                ' AND PlanovaniKapacit.Datum = ' . $params ->addDateParameter(time()) .
                ')';
        }
        else {
            if ($this->pravaZobrazit->jsouNejakePrava()) {
                $result .=
                    ($result ? ' AND ' : '') .
                    "((" . $this->pravaZobrazit->sqlfiltrRozdeleni($this ->filterForm ->getOmezitObdobiOd(), $this ->filterForm ->getOmezitObdobiDo(), $params) . ')' .
                    ' OR (ProvozniDenik.RC = ' . $params ->addParameter(User::getRC()) . ')' .
                    ')';
            } else {
                if ($this->pravaZapsat->priznak) {
                    $result .=
                        ($result ? ' AND ' : '') .
                        '(ProvozniDenik.RC = ' . $params->addParameter(User::getRC());

                    $result .=
                        ' AND (' . $this->pravaZapsat->sqlfiltrRozdeleni($this ->filterForm ->getOmezitObdobiOd(), $this ->filterForm ->getOmezitObdobiDo()) . ')';

                    if ($this->pravaZapsat->priznak2 > 0) {
                        $result .=
                            ' AND (ProvozniDenik.DatumCas >= ' . $params->addTimestampParameter(DateUtils::incMinute(time(), $this->pravaZapsat->priznak2)) . ')';
                    }
                } else {
                    $result .=
                        ($result ? ' AND ' : '') .
                        '1 = 0';
                }
            }
        }

//        if ($_SESSION['jenAktivniUdalostPD']) {
//            if ($result) {
//                $result .= ' AND';
//            }
//            $result .= ' TypUdalostiPD.Aktivni = 1';
//        }

//        if ($_SESSION['jenUdalostPD']) {
//            if ($result) {
//                $result .= ' AND';
//            }
//            $result .= " TypUdalostiPD.Kod = '" . $this ->database ->addSlashes($_SESSION['udalostPD']) . "'";
//        }

//        if (RonPortalSession::rozdeleni($this->filterForm->f_typrozdeleni)) {
//            if ($result) {
//                $result .= ' AND';
//            }
//            $result .= ' ProvozniDenik.KodTypuRozdeleni = '. (int)$this->pravaZobrazit->typRozdeleni .
//                " AND ProvozniDenik.KodRozdeleni = '" . $this ->database ->addSlashes(RonPortalSession::rozdeleni($this->filterForm->f_typrozdeleni)) . "'";
//        }

        // filtry rozdeleni
        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            if ($rozdeleniFilter ->getValue()) {
                $result .=
                    ($result ? ' AND ' : '') .
                    ' ProvozniDenik.KodTypuRozdeleni = '. (int)$rozdeleniFilter ->getValue('typRozdeleni') .
                    ' AND ProvozniDenik.KodRozdeleni = ' . $params ->addParameter($rozdeleniFilter ->getValue());

            }
        }


        // datum od-do
        if ($this ->filterForm ->getFilterValue('datumod')) {
            $result .=
                ($result ? ' AND ' : '') .
                'ProvozniDenik.DatumCas >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod'));
        }
        if ($this ->filterForm ->getFilterValue('datumod', FilterFormFilterDatumOdDo::SPEC_DATUM_DO)) {
            $result .=
                ($result ? ' AND ' : '') .
                'ProvozniDenik.DatumCas < ' . $params ->addDateParameter(
                    DateUtils::incDayStart($this ->filterForm ->getFilterValue('datumod', FilterFormFilterDatumOdDo::SPEC_DATUM_DO))
                );
        }

        return new SqlStatement($result, $params);
    }

    function load($sqlFilter = '')
    {
        parent::load($sqlFilter);

        //pokud je vybrany konkretni typ udalosti (nebo po vlozeni noveho zaznamu], vyhledaji se XML sablony zacinajici kodem typu udalosti,
        // pro kazdou takovou sablonu se zobrazi odkaz na vyplneni
        $typUdalosti = $_SESSION['typUdalostiPD'] ?? $this ->filterForm ->getFilterValue('udalostpd'); // prave vyplnena udalost ve formulari nebo vybrana ve filtru

        if ($typUdalosti) {
            $params = $this->database->createParameters();
            $popisTypu = DBUtils::getSqlValue($this->database,
                'SELECT Popis FROM TypUdalostiPD WHERE Kod = ' . $params->addParameter($typUdalosti),
                $params);

            unset($_SESSION['typUdalostiPD']);

            if ($popisTypu) {
                $params = $this->database->createParameters();
                $cursor = $this->database->getSqlResult(
                    'SELECT ID, Popis FROM SablonaXMLFormulare' .
                    ' WHERE PredavaciProces IS NULL AND Popis LIKE ' . $params->addParameter($popisTypu . '%'),
                    $params
                );

                while ($row = $cursor->fetch()) {
                    $this ->seznamXmlFormularu[$row[0]] = $row[1];
                }
            }
        }
    }

    public function render()
    {
        parent::render();

        if ($this ->seznamXmlFormularu) {
            foreach ($this ->seznamXmlFormularu as $id => $nazev) {
                echo
                    '<div class="margin-top-2">' .
                    "<a href='?menuid=vyplnitdotaznik&record=$id'>" . TextManager::_('PD_vyplnit_formular') . ': ' . $nazev . "</a>" .
                    '</div>';
            }
        }
    }


}
