<?php

namespace RonPortal\Controls;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\GuiUtils;
use Ron\StringUtils;
use Ron\TextManager;

/**
 * Filtr pro zadani datumu a pripadne i casu
 */
class FilterFormFilterDatum extends FilterFormFilter
{
    /**
     * Jmeno filtru pro zadani casu. Pokud neni zadano, tak se zadava jen datum
     * @var string|null
     */
    protected $casInputName = null;

    /**
     * Seznam odkazu pro rychle nastaveni datumu - asociativni pole ve formatu "nadpis" => timestamp
     * @var array
     */
    protected $datumLinks;

    public function __construct(string $name, ?string $casInputName = null)
    {
        parent::__construct($name, FilterFormFilter::TYPE_EDIT);
        $this ->casInputName = $casInputName;

        $this ->defaultCssClass = 'form-control inline-form-control inputDate';
        $this ->htmlAttributes = ['size' => 10, 'maxlength' => 10, 'data-roncalendar-submit' => '1'];

        $this ->defaultValue = time();

        $this ->definesPeriod = true;

        $this ->setCaption(TextManager::_('Datum'));

        if (!$casInputName) {
            $this ->datumLinks = [TextManager::_('Today') => DateUtils::today()];
        }
    }

    /**
     * @param array $datumLinks
     * @return FilterFormFilterDatum
     */
    public function setDatumLinks(array $datumLinks): FilterFormFilterDatum
    {
        $this->datumLinks = $datumLinks;
        return $this;
    }


    public function getValue(?string $specification = null)
    {
        switch ($specification) {
            case 'date' :
                return isset($this ->value) ? StringUtils::format('date', (int)$this ->value) : null;

            case 'time' :
                return isset($this ->value) ? StringUtils::format('time', (int)$this ->value) : null;

            case 'datumOd' :
            case 'datumDo' :
                return $this ->value;
        }

        return parent::getValue($specification);
    }


    public function load(Database $database)
    {
        if (!$this->enabled) {
            return;
        }

        $valueStr = trim($_GET[$this->name] ?? '');

        if ($valueStr !== '') {
            try {
                $datum = DateUtils::dateStringToTimestamp($valueStr);

                if ($this ->casInputName  &&  trim($_GET[$this ->casInputName] ?? '') !== '') {
                    $datum = DateUtils::timeStringToTimestamp(trim($_GET[$this ->casInputName]), $datum);
                }

                $this ->value = $datum;
            } catch (\Exception $e) {
                $this->loadDefaultValue($database);
            }

            $this->storeValue($database);
        } else {
            $this->loadDefaultValue($database);
        }
    }


    /**
     * Zobrazi odkazy pro rychle nastaveni datumu
     * @return void
     */
    public function renderLinks()
    {
        if (!$this ->datumLinks) {
            return;
        }

        echo "\n<ul class='linkslist margin-left-2'>";

        foreach ($this ->datumLinks as $nadpis => $datum) {
            $rowId = $this ->rowId === true
                ? 'filterrow_' . $this ->name
                : $this ->rowId;

            if ($datum !== $this->value) {

                $params = [
                    'class' => 'js setValue',
                    'data-trigger-change' => '1',
                    'data-submit' => '1',
                    'data-selectors-count' => '1',
                    // datum
                    'data-selector1' => ($rowId ? "#{$rowId} " : '') .  "input[name={$this ->name}]",
                    'data-value1' => $datum ? StringUtils::format('date', $datum) : ''
                ];

                if ($this ->casInputName) {
                    $params['data-selectors-count'] = 2;
                    $params['data-selector2'] = ($rowId ? "#{$rowId} " : '') .  "input[name={$this ->casInputName}]";
                    $params['data-value1'] = $datum ? StringUtils::format('time', $datum) : '';
                }

                echo
                    '<li ' . GuiUtils::getAttributesHtml($params) . '>' .
                    $nadpis .
                    '</li>';
            }
        }

        echo "</ul>\n";
    }

    public function render()
    {
//        $limitsStr =
//            (isset($this ->maxDatum) ? 'data-roncalendar-maxdate="' . date('j.n.Y', $this ->maxDatum) . '"' : '') .
//            (isset($this ->minDatum) ? ' data-roncalendar-mindate="' . date('j.n.Y', $this ->minDatum) . '"' : '');
        echo "<input " . GuiUtils::getAttributesHtml(array_merge(
                [
                    'class' => $this ->getCssClass(),
                    'name' => $this ->name,
                    'value' => $this ->getValue('date') ?? ''
                ],
                $this ->htmlAttributes ?? [])) .
            '/>';

        echo "<i class='showcalendar inline-glyph glyph-button fa glyph-calendar' data-roncalendar-inputname='{$this ->name}'></i>";

        if ($this->casInputName) {
            echo '<input ' . GuiUtils::getAttributesHtml(
                    [
                        'name' => $this ->casInputName,
                        'class' => 'margin-left form-control inline-form-control inputTime',
                        'value' => $this ->getValue('time') ?? '',
                        'maxlength' => 8,
                        'size' => 5
                    ]) .
                '/>';
        }

        $this ->renderLinks();
    }

}
