<?php

namespace RonPortal\Controls;

use Ron\TextManager;

/**
 * Filtrovaci element pro zadani roku - krome hodnoty zobrazuje taky buttonky pro rychly prechod na predchozi/dalsi rok
 */
class FilterFormFilterRok extends FilterFormFilter
{
    public function __construct(string $name)
    {
        parent::__construct($name, FilterFormFilter::TYPE_INTEGER);

        $this
            ->setDefaultValue((int)date('Y', time()))
            ->setMinValue('1800')
            ->setMaxValue('2500')
            ->setDefinesPeriod(true)
            ->setCaption(TextManager::_('Rok'))
            ->setHtmlAttributes(['size' => 10, 'maxlength' => 4, 'id' => "filterformfilter_$name"]);
    }


    public function getValue(?string $specification = null)
    {
        switch ($specification) {
            case 'datumOd' :
                return mktime(0, 0, 0, 1, 1, $this ->value);

            case 'datumDo' :
                return mktime(0, 0, 0, 12, 31, $this ->value);
        }

        return parent::getValue($specification);
    }


    /**
     * Za inputem pro zadani roku zobrazi jeste tlacitka pro prechod na predchozi a nasledujici rok
     * @return void
     */
    public function render()
    {
        echo
            '<span class="no-wrap">';

        parent::render();

        if ($this ->value  &&  ($this ->htmlAttributes['id'] ?? null)) {
            $selector = '#' . $this ->htmlAttributes['id'];

            echo
                //"<input id='ronportalfilter_rok' type='text' size='10' maxlength='4' name='rok' value='{$this ->rok}'>" .
                '<i class="inline-glyph glyph-button fa glyph-left setValue" data-value="' . ($this->value - 1) . '" data-submit="1" data-selector="' . $selector . '" data-trigger-change="1"></i>' .
                '<i class="inline-glyph glyph-button fa glyph-right setValue" data-value="' . ($this->value + 1) . '" data-submit="1" data-selector="' . $selector . '" data-trigger-change="1"></i>';
        }

        echo
            '</span>';
    }

}
