<?php

namespace RonPortal\Controls;

use Ron\DB\Database;
use Ron\GuiUtils;

/**
 * Filtr pro prepinani vice voleb (tj. muze byt zapnuto vice voleb najednou)
 */
class FilterFormFilterSwitchLinks extends FilterFormFilter
{
    public function __construct(string $name, string $type = self::TYPE_LINKS)
    {
        parent::__construct($name, $type);
        $this ->defaultCssClass = 'linkslist multiple-links-list';
    }


    protected function loadSubmittedValue()
    {
        $itemAll = null;
        $found = false;
        $itemAllChecked = false;
        $valueLoaded = false;

        foreach ($this ->items as $item) {
            if ($item ->enabled) {
                if ($item ->key === '') {
                    // volba "vse"
                    $itemAll = $item; // zapamatuje si volbu "vse"
                    if (isset($_GET[$this ->name])) {
                        $item ->active = (int)$_GET[$this ->name];
                        $valueLoaded = true;
                    }

                    $itemAllChecked = $item ->active;
                } else {
                    // ostatni volby
                    if (isset($_GET[$this ->name . '_' . $item ->key])) {
                        $item ->active = (int)$_GET[$this ->name . '_' . $item ->key];
                        $valueLoaded = true;
                    }

                    if ($item ->active) {
                        // existuje minimalne jedna zapnuta volba
                        $found = true;
                    }
                }
            }
        }

        if ($valueLoaded  &&  !$found  &&  $itemAll) {
            // nenasel zadnou zaskrtnutou volbu - automaticky zaskrtne "vse"
            $itemAll ->active = true;
        } else if ($itemAllChecked) {
            // v URL je zapnuta volba "vse" - vsechny ostatni automaticky vypne
            foreach ($this ->items as $item) {
                if ($item !== $itemAll) {
                    $item ->active = false;
                }
            }
        } else if ($itemAll) {
            // nektera z voleb je zapnuta - automaticky vypne volbu "vse"
            $itemAll ->active = false;
        }

        $this ->value = $this ->getValue("#options_str");
        return $valueLoaded;
//        // value nastavi na true, pokud nasel alespon jednu zaskrtnutou volbu (jinou nez "vse")
//        $this ->value = $found && !$itemAllChecked;
    }


    public function loadDefaultValue(Database $database)
    {
        parent::loadDefaultValue($database);

        $selectedOptions = explode(';', $this ->value ?? '');

        foreach ($this ->items as $item) {
            if (!$selectedOptions) {
                $item ->active = $item ->key === '';
            } else {
                $item ->active = in_array($item ->key, $selectedOptions);
            }
        }
    }




//    public function load(Database $database)
//    {
//        $itemAll = null;
//        $found = false;
//        $itemAllChecked = false;
//
//        if (!$this ->enabled) {
//            return;
//        }
//
//        $this->loadOptions($database);
//
//        foreach ($this ->items as $item) {
//            if ($item ->enabled) {
//                if ($item ->key === '') {
//                    // volba "vse"
//                    $itemAll = $item; // zapamatuje si volbu "vse"
//                    $item ->active = (int)($_GET[$this ->name] ?? 0);
//                    $itemAllChecked = $item ->active;
//                } else {
//                    // ostatni volby
//                    $item ->active =
//                        (int)($_GET[$this ->name . '_' . $item ->key] ??
//                        $this ->getDefaultValueByName($database, 0, '_' . $item ->key));
//
//                    if ($item ->active) {
//                        // existuje minimalne jedna zapnuta volba
//                        $found = true;
//                    }
//                }
//            }
//        }
//
//        if (!$found  &&  $itemAll) {
//            // nenasel zadnou zaskrtnutou volbu - automaticky zaskrtne "vse"
//            $itemAll ->active = true;
//        } else if ($itemAllChecked) {
//            // v URL je zapnuta volba "vse" - vsechny ostatni automaticky vypne
//            foreach ($this ->items as $item) {
//                if ($item !== $itemAll) {
//                    $item ->active = false;
//                }
//            }
//        } else if ($itemAll) {
//            // nektera z voleb je zapnuta - automaticky vypne volbu "vse"
//            $itemAll ->active = false;
//        }
//
//        // value nastavi na true, pokud nasel alespon jednu zaskrtnutou volbu (jinou nez "vse")
//        $this ->value = $found && !$itemAllChecked;
//    }


    public function render()
    {
        foreach ($this ->items as $item) {
            if ($item ->enabled) {
                if ($item ->key !== ''  ||  !$item ->active) {
                    $suffix = ($item ->key !== '' ? '_' . $item ->key : '');

                    $id =
                        ($this ->htmlAttributes['id'] ?? $this ->name);

                    $value = $item ->active ? 1 : 0;

                    echo "<input type='hidden' name='{$this ->name}$suffix' id='$id$suffix' value='$value'/>";
                }
            }
        }

        echo
            '<ul ' . GuiUtils::getAttributesHtml(array_merge(
                [
                    'class' => $this ->getCssClass()
                ],
                $this ->getDefaultHtmlAttrs(),
                $this ->htmlAttributes ?? [])) .
            '>';

        foreach ($this ->items as $item) {
            if ($item ->enabled) {
                $suffix = ($item ->key !== '' ? '_' . $item ->key : '');

                $linkParams = [
                    'href' => '#'
                ];

                if ($item ->key !== ''  ||  !$item ->active) {  // pokud je volba "vse" zapnuta, tak negeneruje input
                    $linkParams['class'] = 'setValue' . ($item ->cssClass ? ' ' . $item ->cssClass : '');
                    $linkParams['data-value'] = $item ->active ? '0' : '1';
                    $linkParams['data-submit'] = '1';
                    $linkParams['data-selector'] = '#' . ($this ->htmlAttributes['id'] ?? $this ->name) . $suffix;
                }

                $cssClass = $item ->cssClass . ($item ->active ? ' selectedlink' : '');
                echo
                    '<li' . ($cssClass ? " class='$cssClass'" : '') . '>' .
                    '<a ' . GuiUtils::getAttributesHtml($linkParams) . '>' .
                    $item ->caption .
                    '</a>' .
                    '</li>';
            }
        }

        echo "</ul>\n";
    }


    public function getValue(?string $specification = null)
    {
        if ($specification === null) {
            return $this ->value;
        }

        if ($specification == '#options'  ||  $specification == '#options_str') {
            $result = [];
            foreach ($this ->items as $item) {
                if ($item ->active  &&  $item ->key !== '') {
                    $result[] = $item ->key;
                }
            }

            return ($specification == '#options_str')
                ? implode(';', $result)
                : $result;
        }

        foreach ($this ->items as $item) {
            if ($item ->key === $specification) {
                return $item ->active;
            }
        }

        return false;
    }


    public function storeValue(Database $database)
    {
        //echo "JSME TU - LINKS ";
        parent::storeValue($database); // TODO: Change the autogenerated stub
    }


    public function getUrlParameters()
    {
        $result = [];

        foreach ($this ->items as $item) {
            if ($item ->enabled) {
                $name = ($item->key === '')
                    ? $this->name
                    : $this->name . '_' . $item->key;

                $result[$name] = $item ->active ? 1 : 0;
            }
        }

        return $result;
    }


}
