<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Controls;

use Exception;
use Ron\Controls\ComboBox;
use Ron\Controls\ComboBoxes;
use Ron\Controls\ConfigurableObject;
use Ron\Controls\DateInput;
use Ron\Controls\ObjectConfiguration;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\Profiler;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;
use RonPortal\Kalendar;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\RozdeleniUtils;
use RonPortal\TypRozdeleni;
use RonPortal\User;
use RonPortal\NastaveniUzivatele;
use Tracy\Debugger;

/**
 * Formular pro zobrazeni a nacitani filtru nad seznamy.
 */
class RonPortalFilterForm implements ViewObject, ConfigurableObject
{
    /** @var Database */
    var $database;

    protected $useGet;
    protected $getMenuId;   // pokud je nastavena a je $useGet=true, prida se hidden promenna menuid
    public $getAction = ''; // action formulare pouzita pri $useGet=true (neni povinna)

    var $f_formaction;      // akce formulare - nemusi byt nastavena

    protected $hiddenFields = []; // asociativni pole jmeno=>hodnota

    /** @var string         Nadpis stranky - pokud je zadan, vygeneruje <h1> */
    public $header = null;

    /** @var string         Prefix nazvu promennych ulozenych v session  */
    //public $sessionPrefix = '';

    var $autoSubmit;

    /** @var string[]   atributy HTML formulare */
    public $formAttributes = ['class' => 'submitWorkingInfo'];

    /** @var bool   Priznak urcuje, jestli jsou podporovany uzivatelske filtry. Nastavi se automaticky pri volani buildSqlFilter */
    protected $customFilterSupported = false;

    public $obdobiNeboDatumPovinne = false;

    /** @var bool   Priznak, ze uz byla nactena XML konfigurace formulare */
    protected $configurationLoaded = false;

//    /**
//     * @var bool       priznak, jestli ma zobrazit vyber obdobi
//     * @deprecated
//     */
//    var $showObdobi;
//    /** @var string     aplikace, ze ktere ma zobrazit obdobi */
//    var $obdobiAplikace;
    //var $jenOtevreneObdobi;
//    /** @var bool       pokud neni obdobi povinne, zobrazi se i zaskrtavatko "jen obdobi" */
//    var $obdobiPovinne;
//    /** @var ComboBoxObdobi */
//    var $comboObdobi;

//    /** @var int        Pocatek vybraneho obdobi (nastavi se v load) */
//    var $vybraneObdobiOd;
//    /** @var int        Konec vybraneho obdobi (nastavi se v load) */
//    var $vybraneObdobiDo;

//    /** @var bool       priznak, jestli ma zobrazit vyber variantniho obdobi */
//    var $showVariantni;
//    /** @var ComboBoxObdobi */
//    var $comboVariantni;

    /** @var bool           zapina zadavani datumu od-do */
    var $showDatumOdDo;
    /** @var bool           priznak, jestli je zadani obdobi povinne (pokud ano nezobrazuje checkbox) */
    var $datumOdDoPovinne;
    /** @var string         Titulek pro datum */
    var $nadpisDatumOdDo;
    /** @var string|string[]|array[]    Umoznuje nastavit text, ktery se zobrazi za filtrem pro datum od-do. Pokud se zada pole, lze pouzit "zkratky", napr. ['today', 'currentMonth'] */
    var $datumOdDoLinks = null;
//    /** @var bool           Urcuje, jestli se zobrazi checkbox pred nadpisem */
//    var $datumOdDo_CheckboxFirst = false;
    /** @var int            Pokud je zadano, pouzije se jako vychozi hodnota pro datumOd a datumDo  */
    public $defaultDatumOdDo;
    public $datumOdDoSessionPrefix = 'filterform_';    // TODO: predelat na FilterFormFilter  // SE 23.1.2024 - zmeneno na filterform_

    /** @var bool       priznak, ze se ma filtrovat datum */
    var $filtrovatDatum;
    /** @var int        datum od zadane uzivatelem */
    var $datumOd;
    /** @var int        datum do zadane uzivatelem */
    var $datumDo;

//    /** @var string     Umoznuje nastavit text, ktery se zobrazi za filtrem pro datum */
//    var $datumLinks = null;

//    /** @var bool           zadani konkretniho datumu */
//    var $showDatum;
//    /** @var int            maximalni datum, ktere je mozne zadat */
//    var $maxDatum = null;
//    /** @var int            minimalni datum, ktere je mozne zadat */
//    var $minDatum = null;
    /** @var int            aktualni hodnota zadaneho datumu */
    var $datum;

//    /** @var bool           Urcuje, jestli ma pod formularem zobrazit seznam vsech dnu obdobi */
//    public $zobrazitDnyObdobi = false;

//    /** @var string         SQL dotaz, ktery nacte podrobnosti ke dnum obdobi - ty se pak zobrazi v radcich pod dny. Prvni sloupec musi vracet datum. */
//    public $sqlPodrobnostiDnu = null;
//    /** @var string         Nadpis, ktery se zobrazi pred radkem pro podrobnosti dnu. */
//    public $nadpisPodrobnostiDnu = null;

//    /** @var bool       Jestli se ma zobrazit vyber roku */
//    public $showRok = false;
//    /** @var int        Vybrany rok */
//    public $rok;

    /**
     * @var bool           priznak, jestli ma zobrazit vyber rozdeleni
     * @deprecated
     */
    var $showRozdeleni;
    /** @var int            Kod typu rozdeleni pro filtrovani */
    var $kodTypuRozdeleni;
    /** @deprecated     - jen pro stary kod - pouzijte $kodTypuRozdeleni */
    var $f_typrozdeleni;
    var $vlastniTypRozdeleni;
    /** @var integer        0=vse, 1=jen vlastni, 2=podle prav (prava mohou byt podle jineho typu rozdeleni!), 3=podle prav, nezobrazovat nadrazene  */
    var $jenVlastniRozdeleni;
    /** @var string */
    var $filtrRozdeleni;
    /**
     * @var PravaNaOsoby   Typ prava, podle ktereho se muze odfiltrovat rozdeleni
     * @deprecated
     */
    var $f_pravarozdeleni;

    var $pouzitPopisTypuRozdeleni;
    var $popisTypuRozdeleni;
    /** @var bool           Urcuje, jestli se ma zobrazit volba "vcetne podrizenych" */
    var $showVcetnePodrizenychRozdeleni;
    /** @var int            Hodnota pro select "vcetne podrizenych" 0=jen vybrane, 1=vcetne podrizenych, 2=jen podrizene, 3=jen primo podrizene. Po load() se nastavi na skutecnou hodnotu. */
    var $vcetnePodrizenychRozdeleni;
    /** @var  ComboBoxes    Vyber rozdeleni 1 (podle popisu a kodu) */
    var $comboBoxesRozdeleni;
    /** @var  string        Kod vybraneho rozdeleni - nastavi se v load */
    var $kodRozdeleni;

    /** @var FilterFormFilterRozdeleni[]    Seznam filtru na rozdeleni */
    protected $rozdeleniFilters = [];

    /**
     * @var bool
     * @deprecated
     */
//    var $showRozdeleni2;  // priznak, jestli ma zobrazit vyber rozdeleni
    /** @var int            2. kod typu rozdeleni pro filtrovani */
    var $kodTypuRozdeleni2;
    /** @deprecated     Pouzijte $kodTypuRozdeleni2! */
    var $f_typrozdeleni2;
    var $vlastniTypRozdeleni2;
    var $jenVlastniRozdeleni2;
    var $f_filtrrozdeleni2;
    var $pouzitPopisTypuRozdeleni2;
    var $popisTypuRozdeleni2;
    var $showVcetnePodrizenychRozdeleni2;
    /** @var int            Hodnota pro select "vcetne podrizenych" 0=jen vybrane, 1=vcetne podrizenych, 2=jen podrizene, 3=jen primo podrizene. Po load() se nastavi na skutecnou hodnotu. */
    var $vcetnePodrizenychRozdeleni2;
    /** @var  ComboBoxes    Vyber rozdeleni 2 (podle popisu a kodu) */
    var $comboBoxesRozdeleni2;
    /** @var  string        Kod vybraneho rozdeleni 2 - nastavi se v load */
    var $kodRozdeleni2;


    //var $f_showosoba;
    /** @var bool       priznak, jestli se maji zobrazit osoby - bud podle zadanych prav nebo podle predavaciho procesu */
    var $showOsoby;
    /** @var string     kod predavaciho procesu, podle ktereho se vybira */
    var $kodProcesu;
    var $typRozdeleniProces;
    /** @var  */
    var $f_prava;          // prava na osoby - pokud jsou zadana, nepouzije se predavaci proces!
    /** @var int        obdobi pro urceni prav (zarazeni do pristupove skupiny v zadanem obdobi) */
    var $osobyDatumOd;
    /** @var int        pokud je 0 a zaroven je zobrazen vyber obdobi, pouzije se datum dle obdobi */
    var $osobyDatumDo;
    // vyber osoby probiha bud podle prav nebo podle predavaciho procesu
    /** @var string     SQL pro filtrovani seznamu osob */
    var $osobyFiltr;
    var $vcetneExternich;
    var $povolitVsechyOsoby;
    var $downloadFotoOsobyUrl;
    /** @var  ComboBoxZamestnanec */
    var $comboOsoba;

    /** @var int        Id PP vybrane osoby - nastavi se v load */
    var $vybranaOsoba;
    /** @var string     RC vybrane osoby - nastavi se v load */
    var $vybranaOsobaRC;
    /** @var string     Osobni cislo vybraneho pp - nastavi se v load */
    var $vybranaOsobaOsobniCislo;
    /** @var int        Datum nastupu vybraneho pp - nastavi se v load */
    var $vybranaOsobaDatumNastupu;
    /** @var bool       Priznak, ze se maji zobrazit vsechny osoby */
    var $vsechnyOsoby = false;

    /** @var FilterFormFilterObdobi       Filtr na obdobi */
    protected $filterObdobi;

    /** @var FilterFormFilter[]     Seznam filtru */
    protected $filters = [];

    var $name;

    var $showMessages;

    var $skrytTlacitkoNastavit = false;

    var $f_classname;

    var $formClass = 'form ronportalfilterform position-relative';
    var $htmlId;

    /** @var int    Urcuje pocatek obdobi, podle ktereho se omezi nektere filtry nebo prava - nastavi se automaticky podle filtru, ktere maji $definesPeriod=true */
    protected $omezitObdobiOd;
    /** @var int    Urcuje konec obdobi, podle ktereho se omezi nektere filtry nebo prava */
    protected $omezitObdobiDo;


    /**
     * @param Database $database
     * @param string|string[] $action    Pro POST se pouzije jako action formulare. Pro GET se prida promenna menuid=$action nebo vsechny promenne za asociativniho pole
     * @param bool $useGet      Priznak, jestli ma formular pouzivat GET
     */
    public function __construct($database, $action, $useGet = false)
    {
        $this ->database = $database;

//        $this->showObdobi = false;
//        $this->showVariantni = false;
        $this->showRozdeleni = false;
//        $this->showRozdeleni2 = false;
        $this->showOsoby = false;
        $this->autoSubmit = true;

        $this->name = 'adsappfilter';
        //$this -> f_formaction = '';  - zmeneno 13.3. - po vlozeni dochazky filterform nefungoval

        if ($useGet) {
            $this ->setUseGet(true, $action);
        } else  {
            $this->f_formaction = '?menuid=' . $action;
        }

        $this->f_classname = 'filtercontainer d-flex';

        $this->showMessages = true; // na konci zobrazi chybove hlasky errormanagera
    }


    /**
     * @param bool $value
     * @param string|string[] $action bud akce (menuid, napr. "dochazka") nebo asociativni pole parametru
     * @return RonPortalFilterForm
     * @throws Exception
     */
    public function setUseGet($value, $action)
    {
        if (!$value  &&  $this ->useGet) {
            // Momentalne to funguje spravne jen primo v konstruktoru! Navic asi blbne vyber rozdeleni!
            throw new Exception("Chyba v RonPortalFilterForm! USE GET nelze vypnout!");
        }

        $this ->useGet = $value;
        if (is_array($action)) {
            foreach ($action as $key => $fieldValue) {
                $this ->addHiddenField($key, $fieldValue);
            }
        } else {
            if ($action) {
                //$this->addHiddenField('menuid', $action);
                $this ->getMenuId = $action;
            }
        }

        return $this;
    }


    /**
     * @param string $menuId
     * @return RonPortalFilterForm
     */
    public function nastavAkciGet($menuId)
    {
        $this ->useGet = true;
        $this ->getMenuId = $menuId;
        return $this;
    }

    /**
     * Prida hidden pole, nebo nastavi jeho hodnotu
     * @param string $name
     * @param string $value
     * @return RonPortalFilterForm
     */
    public function addHiddenField($name, $value)
    {
        $this ->hiddenFields[$name] = $value;
        return $this;
    }


    //Metody pro nastaveni formulare (zapnuti/vgypnuti jednotlivych filtru)
    public function addFilterObdobi($aplikace = 'ADS', $required = true, $jenOtevrene = false)
    {
        $f = (new FilterFormFilterObdobi($this, $aplikace))
                ->setRequired($required)
                ->setJenOtevreneObdobi($jenOtevrene);

        $this ->addFilter($f,true);

        return $f;
    }


    /**
     * Zapne zobrazeni vyberu pro datum od, datum do
     * @param bool $show
     * @param bool $povinne
     * @param string $nadpis
     * @return RonPortalFilterForm
     * @deprecated - pouzijte addFilter a FilterFormFilterDatumOdDo
     */
    public function nastavDatumOdDo($show, $povinne, $nadpis = '')
    {
        if ($GLOBALS['development']) {
            MessageManager::addWarning('DEV> Funkce RonPortalFilterForm::nastavDatumOdDo je zastarala!');
        }

        $this->showDatumOdDo = $show;
        $this->datumOdDoPovinne = $povinne;
        $this->nadpisDatumOdDo = $nadpis;
//        $this->datumOdDo_CheckboxFirst = $checkboxFirst;
        return $this;
    }


    /**
     * Zobrazi vyber rozdeleni
     * @param bool $show
     * @param int $typRozdeleni
     * @param int $jenVlastni 0=vse, 1=jen vlastni, 2=podle prav (prava muzou byt podle jineho typu rozdeleni!), 3=podle prav, nezobrazovat nadrazene
     * @param string $filtrRozdeleni podminka, ktera se doplni do SQL podminky
     * @param bool $pouzitPopisTypuRozdeleni jestli se zobrazi pred vyberem rozdeleni popis typu rozdeleni, nebo popis zadany v nastaveni
     * @return RonPortalFilterForm
     * @deprecated  Pouzijte addFilterRozdeleni1
     */
    public function nastavRozdeleni($show, $typRozdeleni = 0, $jenVlastni = 0, $filtrRozdeleni = null, $pouzitPopisTypuRozdeleni = false)
    {
        if ($GLOBALS['development']) {
            MessageManager::addWarning('DEV> Funkce RonPortalFilterForm::nastavRozdeleni je zastarala!');
        }
        $this->showRozdeleni = $show;

        if (is_null($filtrRozdeleni)) {
            $filtrRozdeleni = "(Rozdeleni.cssClass IS NULL  OR  Rozdeleni.cssClass <> 'hidden')";
        }

        // pokud neni zadan typ rozdeleni, tak pouzije vychozi
        if ($typRozdeleni) {
            $this ->kodTypuRozdeleni = $this->f_typrozdeleni = $typRozdeleni;
            $this->jenVlastniRozdeleni = $jenVlastni;
            $this ->vlastniTypRozdeleni = true;
        } else {
            $this ->kodTypuRozdeleni = $this->f_typrozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP);
            $this->jenVlastniRozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_JEN_VLASTNI);
            $this ->vlastniTypRozdeleni = false;
        }

        $this->filtrRozdeleni = $filtrRozdeleni;
        $this ->pouzitPopisTypuRozdeleni = $pouzitPopisTypuRozdeleni;
        $this->showVcetnePodrizenychRozdeleni = true;
        $this->vcetnePodrizenychRozdeleni = null;

        return $this;
    }


    /**
     * @param bool $deprecated_show
     * @param int|null $typRozdeleni
     * @param bool $jenVlastni
     * @param string $filtrRozdeleni
     * @param bool $pouzitPopisTypuRozdeleni jestli se zobrazi pred vyberem rozdeleni popis typu rozdeleni, nebo popis zadany v nastaveni
     * @return RonPortalFilterForm
     * @deprecated  Pouzijte addFilterRozdeleni2
     */
    public function nastavRozdeleni2($deprecated_show, $typRozdeleni = null, $jenVlastni = false, $filtrRozdeleni = '', $pouzitPopisTypuRozdeleni = false)
    {
        if ($GLOBALS['development']) {
            MessageManager::addWarning('DEV> Funkce RonPortalFilterForm::nastavRozdeleni2 je zastarala!');
        }

        return $this;
    }


    /**
     * Prida filtr pro rozdeleni 1 podle nastaveni Portalu
     * @param ?int $jenVlastniRozdeleni Urcuje, jestli se zobrazi jen vlastni a podrizena rozdeleni uzivatele (viz FilterFormFilterRozdeleni::$jenVlastniRozdeleni). Pokud je null, tak se nacte hodnota z nastaveni Portalu
     * @param ?PravaNaOsoby $prava      Pokud je zadan typ prava, tak se podle prav odfiltruji rozdeleni
     * @return FilterFormFilterRozdeleni|null
     */
    public function addFilterRozdeleni1(?int $jenVlastniRozdeleni = null, ?PravaNaOsoby $prava = null)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) {
            return null;
        }

        if ($prava && !$prava ->zobrazitVyberRozdeleni()) {
            return null;
        }

        $filter = $this ->addFilterRozdeleni('rozdeleni', RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP));
        $caption = RonPortalSettings::getValue(RonPortalSettings::ROZDELENI_NADPIS);
        if ($caption) {
            $filter->setCaption(TextManager::_($caption));
        }

        $jenVlastniRozdeleni = $jenVlastniRozdeleni ?? RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_JEN_VLASTNI);

        $filter
            ->setUserSettingsName('ads_rozdeleni')
            ->setSessionName('ads_rozdeleni')
            ->setJenVlastniRozdeleni($jenVlastniRozdeleni)
            ->setPrava($prava);

        return $filter;
    }


    /**
     * Prida filtr pro rozdeleni 2 podle nastaveni Portalu
     * @param ?int $jenVlastniRozdeleni Urcuje, jestli se zobrazi jen vlastni a podrizena rozdeleni uzivatele (viz FilterFormFilterRozdeleni::$jenVlastniRozdeleni). Pokud je null, tak se nacte hodnota z nastaveni Portalu
     * @param ?PravaNaOsoby $prava      Pokud je zadan typ prava, tak se podle prav odfiltruji rozdeleni
     * @return FilterFormFilterRozdeleni|null
     */
    public function addFilterRozdeleni2(?int $jenVlastniRozdeleni = null, ?PravaNaOsoby $prava = null)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
            return null;
        }

        if ($prava && !$prava ->zobrazitVyberRozdeleni()) {
            return null;
        }

        $filter = $this ->addFilterRozdeleni('rozdeleni2', RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP));
        $caption = RonPortalSettings::getValue(RonPortalSettings::ROZDELENI2_NADPIS);
        if ($caption) {
            $filter->setCaption(TextManager::_($caption));
        }

        $jenVlastniRozdeleni = $jenVlastniRozdeleni ?? RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_JEN_VLASTNI);

        $filter
            ->setUserSettingsName('ads_rozdeleni2')
            ->setSessionName('ads_rozdeleni2')
            ->setJenVlastniRozdeleni($jenVlastniRozdeleni)
            ->setPrava($prava);

        return $filter;
    }


    /**
     * Vraci seznam vsech definovanych filtru rozdeleni
     * @return FilterFormFilterRozdeleni[]
     */
    public function getRozdeleniFilters(): array
    {
        return $this->rozdeleniFilters;
    }


    /**
     * Vraci seznam URL parametru z jednotlivych filtru (tak jak by je vygeneroval formular pri odeslani aktualnich hodnot)
     * @return array
     */
    public function getUrlParameters($includeMenuId = false)
    {
        $result = ($includeMenuId && $this ->getMenuId)
            ? ['menuid' => $this ->getMenuId]
            : [];

        foreach ($this ->filters as $filter) {
            if ($filter ->includeInUrl) {
                $result = array_merge($result, $filter ->getUrlParameters());
            }
        }

        foreach ($this ->rozdeleniFilters as $filter) {
            if ($filter ->includeInUrl) {
                $result = array_merge($result, $filter ->getUrlParameters());
            }
        }

        return $result;
    }

    /**
     * Nahradi v parametrech promenne %<nazev_filtru>% aktualni hodnotou jednotlivych filtru
     * @param string $url
     * @return array|string|string[]|null
     */
    public function replaceUrlParameters($url)
    {
        if (!$url  ||  strpos($url, '%') === false) {
            // url je prazdna nebo neobsahuje zadne parametry
            return $url;
        }

        $paramNames = [];
        $paramValues = [];
        foreach ($this ->filters as $filter) {
            $filterParams = $filter ->getUrlParameters();
            foreach ($filterParams as $paramName => $paramValue) {
                $paramNames[] = "%$paramName%";
                $paramValues[] = StringUtils::htmlTextConvert($paramValue);
            }
        }

        foreach ($this ->rozdeleniFilters as $filter) {
            $filterParams = $filter ->getUrlParameters();
            foreach ($filterParams as $paramName => $paramValue) {
                $paramNames[] = "%$paramName%";
                $paramValues[] = StringUtils::htmlTextConvert($paramValue);
            }
        }

        return str_replace($paramNames, $paramValues, $url);
    }

    /**
     * Prida novy filtr pro rozdeleni
     * @param FilterFormFilterRozdeleni|string $filter    Filtr, ktery se ma pridat, nebo jmeno filtru, ktery se ma vytvorit a pridat
     * @param ?int $typRozdeleni    Kod typu rozdeleni - pouzije se v pripade, ze $name
     * @return FilterFormFilterRozdeleni
     */
    public function addFilterRozdeleni($filter, $typRozdeleni = null)
    {
        if ($filter instanceof FilterFormFilterRozdeleni) {
            $name = $filter ->name;
        } else {
            $name = $filter;
            $filter = new FilterFormFilterRozdeleni($this, $name);
            $filter ->setTypRozdeleni($typRozdeleni);
        }
        $this ->rozdeleniFilters[$name] = $filter;

        return $filter;
    }


    /**
     * @param bool $povolitVsechnyOsoby
     * @param string|PravaNaOsoby|PravaNaOsoby[] $prava
     * @return FilterFormFilterPP
     */
    public function addFilterPP($povolitVsechnyOsoby = false, $prava = null): FilterFormFilterPP
    {
        $filter = new FilterFormFilterPP($this);
        if (is_string($prava)) {
            $filter ->setPrava(Prava::vratPrava($prava));
        } else if ($prava) {
            $filter ->setPrava($prava);
        }

        $filter ->setPovolitVsechnyOsoby($povolitVsechnyOsoby);

        $this ->addFilter($filter);

        return $filter;
    }

    /**
     * jestli se maji zobrazit osoby a podle jakych prav a v jakem obdobi
     * pokud neni obdobi zadano, tak se pouzije aktualne vybrane obdobi
     * @param bool $show zda se ma zobrazit vyber osoby
     * @param PravaNaOsoby|PravaNaOsoby[] $prava prava na zobrazeni osoby
     * @param int $datumOd      Datum pro kontrolu prav - pokud je 0, urci se automaticky podle obdobi
     * @param int $datumDo      -//-
     * @param bool $vcetneExternich vcetne externich osob
     * @return RonPortalFilterForm
     * @deprecated - pouzijte addFilterPP
     */
    public function nastavOsoby($show, $prava = null, $datumOd = 0, $datumDo = 0, $vcetneExternich = false)
    {
        if ($GLOBALS['development']) {
            MessageManager::addWarning('DEV> Funkce RonPortalFilterForm::nastavOsoby je zastarala!');
        }
        $this->showOsoby = $show;
        //$this->f_showosoba = $show; // pro kompatibilitu stareho kodu
        $this->f_prava = $prava;
        $this->osobyDatumOd = $datumOd;
        $this->osobyDatumDo = $datumDo;

        $this->vcetneExternich = $vcetneExternich;

        $this->kodProcesu = null;
        $this->typRozdeleniProces = null;

        return $this;
    }



    /**
     * Nastavi filtry pro personalistiku (zatim jen vyber osob) dle nastaveni (pokud je nastaveno zobrazeni pracovnich pomeru do budoucna)
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     */
    public function nastavPersonalistiku($prava)
    {
        $datumOd = 0;
        $datumDo = 0;

        $pocetMesicu = RonPortalSettings::getIntValue('person_pocet_mesicu_dopredu');

        if ($pocetMesicu) {
            $datumOd = DateUtils::today();
            $datumDo = DateUtils::incMonth($datumOd, $pocetMesicu);

            $this->osobyDatumOd = $datumOd; // aby fungovalo nastavVybraneObdobiProPrava
            $this->osobyDatumDo = $datumDo;

            Prava::nastavObdobiProPrava($datumOd, $datumDo);
        }

        //$this->nastavOsoby(true, $prava, $datumOd, $datumDo);

        $this ->addFilter(
            (new FilterFormFilterPP($this))
                ->setPrava($prava)
                ->setDatumOd($datumOd)
                ->setDatumDo($datumDo)
        );
    }



    /**
     * Podle zapnutych a uzivatelem nastavenych filtru vraci pocatek zvoleneho obdobi
     * @return int
     */
    public function getOmezitObdobiOd(): int
    {
        return $this->omezitObdobiOd ?? time();
    }

    /**
     * Podle zapnutych a uzivatelem nastavenych filtru vraci konec zvoleneho obdobi
     * @return int
     */
    public function getOmezitObdobiDo(): int
    {
        return $this->omezitObdobiDo ?? $this ->getOmezitObdobiOd();
    }


    /**
     * Nastavi filtrovani datumu podle zadanych hodnot (z POST nebo GET - viz loadDatumOdDo)
     * @throws Exception
     */
    protected function nastavHodnotyDatumOdDo($filtrovat, $datumOdStr, $datumDoStr)
    {
        $this ->filtrovatDatum = $this ->datumOdDoPovinne ||
            ($this ->obdobiNeboDatumPovinne ?
                !$this ->getFilterValue('obdobi', 'jenObdobi')
                : $filtrovat);
        try {
            $this->datumOd = $datumOdStr
                ? DateUtils::dateStringToTimestamp($datumOdStr)
                : null;

            $this->datumDo = ($datumDoStr)
                ? DateUtils::dateStringToTimestamp($datumDoStr)
                : null;

            if (!$this->datumDo) {
                $this->datumDo = $this->datumOd;
            }

        } catch (Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        $_SESSION[$this ->datumOdDoSessionPrefix . 'filtrjendatumoddo'] = $filtrovat;
        $_SESSION[$this ->datumOdDoSessionPrefix . 'datumod'] = $this ->datumOd;
        $_SESSION[$this ->datumOdDoSessionPrefix . 'datumdo'] = $this ->datumDo;
    }


    /**
     * @throws Exception
     */
    public function loadDatumOdDo()
    {
        if (isset($_POST['adsfilterform'])) {
            $this ->nastavHodnotyDatumOdDo(
                $this ->datumOdDoPovinne ||
                ($this ->obdobiNeboDatumPovinne
                    ? (int)($_POST['jenobdobi'] ?? 1) == 2
                    : ($_POST['jendatumoddo'] ?? null)),
                $_POST['datumod'] ?? null, $_POST['datumdo'] ?? null);
        } else if (isset($_GET['datumod'])) {
            $this ->nastavHodnotyDatumOdDo(
                $this ->datumOdDoPovinne ||
                ($this ->obdobiNeboDatumPovinne
                    ? (int)($_GET['jenobdobi'] ?? 1) == 2
                    : $_GET['jendatumoddo'] ?? null),
                $_GET['datumod'] ?? null, $_GET['datumdo'] ?? null);
        } else {
            $this ->filtrovatDatum =
                $this ->datumOdDoPovinne ||
                ($this ->obdobiNeboDatumPovinne
                    ? !$this ->getFilterValue('obdobi', 'jenObdobi')
                    : $_SESSION['filtrjendatumoddo'] ?? false);

            $this->datumOd = $_SESSION[$this ->datumOdDoSessionPrefix . 'datumod'] ?? null;
            $this->datumDo = $_SESSION[$this ->datumOdDoSessionPrefix .'datumdo'] ?? null;
        }

        if (!$this ->datumOd) {
            $this ->datumOd = $this ->defaultDatumOdDo ?? DateUtils::firstDayOfMonth(time());
            $this ->datumDo = $this ->defaultDatumOdDo ?? DateUtils::lastDayOfMonth(time());

            $_SESSION[$this ->datumOdDoSessionPrefix . 'filtrjendatumoddo'] = $this ->filtrovatDatum;
            $_SESSION[$this ->datumOdDoSessionPrefix . 'datumod'] = $this ->datumOd;
            $_SESSION[$this ->datumOdDoSessionPrefix . 'datumdo'] = $this ->datumDo;
        }

        if (!$this->omezitObdobiOd  &&  $this ->filtrovatDatum) {
            $this ->omezitObdobiOd = $this ->datumOd;
            $this ->omezitObdobiDo = $this ->datumDo;
        }
    }


    public function nastavVybraneObdobiProPrava()
    {
        if ($this ->omezitObdobiOd) {
            $datumOd = $this ->omezitObdobiOd;
            $datumDo = $this ->omezitObdobiDo ?? $datumOd;
        } else if ($this->osobyDatumOd) {
            $datumOd = $this->osobyDatumOd;
            $datumDo = $this->osobyDatumDo;
        } else {
            $datumOd = time();
            $datumDo = time();
        }

        Prava::nastavObdobiProPrava($datumOd, $datumDo);
    }


    /**
     * @param int $podrizena    0=bez podrizenych, 1=vcetne podrizenych, 2=vcetne primo podrizenych
     * @return string
     */
    public function sestavFiltrRozdeleni($podrizena)
    {
        $datumOd = $this ->omezitObdobiOd ?? time();
        $datumDo = $this ->omezitObdobiDo ?? $datumOd;

        // TODO: predelat na parametry!
        if ($podrizena == 1) {
            // vcetne podrizenych
            return
                'EXISTS (SELECT * FROM RozdeleniOsob, HierarchieRozdeleni ' .
                '  WHERE HierarchieRozdeleni.TypPodrizene = Rozdeleni.KodTypuRozdeleni' .
                '    AND HierarchieRozdeleni.KodPodrizene = Rozdeleni.Kod' .
                '    AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.TypNadrazene' .
                '    AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.KodNadrazene' .
                "    AND RozdeleniOsob.RC = '" . $this->database->addSlashes(User::getRC()) . "'" .
                '    AND RozdeleniOsob.PlatnostOd <= ' . $this->database->dateToString($datumDo) .
                '    AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->dateToString($datumOd) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ')';
        } else if ($podrizena == 2) {
            // vcetne primo podrizenych
            return
                'EXISTS (SELECT * FROM RozdeleniOsob, Rozdeleni R2' .
                ' WHERE R2.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
                '    AND R2.Kod = RozdeleniOsob.Rozdeleni' .
                '    AND R2.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                '    AND (R2.Kod = Rozdeleni.Kod OR R2.Kod = Rozdeleni.Rodic)' .
                "    AND RozdeleniOsob.RC = '" . $this->database->addSlashes(User::getRC()) . "'" .
                '    AND RozdeleniOsob.PlatnostOd <= ' . $this->database->dateToString($datumDo) .
                '    AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->dateToString($datumOd) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ')';
        } else {
            // jen vlastni`
            return
                'EXISTS (SELECT * FROM RozdeleniOsob ' .
                '  WHERE Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni' .
                '    AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni' .
                "    AND RozdeleniOsob.RC = '" . $this ->database ->addSlashes(User::getRC()) . "'" .
                '    AND RozdeleniOsob.PlatnostOd <= ' . $this ->database ->dateToString($datumDo) .
                '    AND (RozdeleniOsob.PlatnostDo >= ' . $this ->database ->dateToString($datumOd) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ')';
        }
    }


    public function filtrRozdeleniPodlePrav($typRozdeleni)
    {
        if (!$this->f_pravarozdeleni) {
            return '';
        }

        if (!$this->f_pravarozdeleni->typRozdeleni) {
            // neni nastaven typ rozdeleni
            return '';
        }

        if ($this->f_pravarozdeleni->typRozdeleni != $typRozdeleni) {
            // filtruje se podle jineho typu rozdeleni nez prava
            return '';
        }

        switch ($this->f_pravarozdeleni->prava) {
            case 10 : // vlastni rozdeleni
            case 11 : //   -//- krome sebe
                return $this->sestavFiltrRozdeleni(0); // podrizena = 0 (bez podrizenych)

            case 12 : // vlastni a podrizena
            case 13 : //   -//- krome sebe
            case 14 : // podrizena
            case 15 : // podrizena a sebe
                return $this->sestavFiltrRozdeleni(1); // (vcetne podrizenych)

            case 22 : // vlastni a primo podrizena
            case 23 : //   -//- krome sebe
            case 24 : // primo podrizena
            case 25 : //   -//- a sebe
            case 26 : //   -//- krome sebe
                return $this->sestavFiltrRozdeleni(2); // vcetne primo podrizenych
        }

        return '';
    }



    public function setUserSettingsItem($name, $htmlName, $post, $get)
    {
        if ($post  &&  isset($_POST[$htmlName])) {
            if (User::isAuthenticated()) {
                NastaveniUzivatele::setUserSettingsItem($this->database, $name, $_POST[$htmlName]);
            }
            $_SESSION[$name] = $_POST[$htmlName];
            return $_POST[$htmlName];
        }

        if ($get  &&  isset($_GET[$htmlName])) {
            if (User::isAuthenticated()) {
                NastaveniUzivatele::setUserSettingsItem($this->database, $name, $_GET[$htmlName]);
            }
            $_SESSION[$name] = $_GET[$htmlName];
            return $_GET[$htmlName];
        }

        if (!User::isAuthenticated()) {
            return null;
        }

        if (!isset($_SESSION[$name])) {
            $_SESSION[$name] = NastaveniUzivatele::getUserSettingsItem($this->database, $name);
        }

        return $_SESSION[$name];
    }


//    public function setUserSettingsItemCheckBox($name, $htmlName, $htmlNameSubmit, $post, $get, $defaultValue = null)
//    {
//        $newValue = null;
//
//        if ($post  &&  isset($_POST[$htmlNameSubmit])) {
//            NastaveniUzivatele::setUserSettingsItem($this ->database, $name, isset($_POST[$htmlName]));
//            $newValue = isset($_POST[$htmlName]);
//        } else if ($get  &&  isset($_GET[$htmlNameSubmit])) {
//            NastaveniUzivatele::setUserSettingsItem($this->database, $name, isset($_GET[$htmlName]));
//            $newValue = isset($_GET[$htmlName]);
//        } else if (isset($_SESSION[$name])) {
//            return $_SESSION[$name];
//        } else {
//            $newValue = NastaveniUzivatele::getUserSettingsItem($this ->database, $name, $defaultValue);
//            if (!isset($newValue)) {
//                $newValue = $defaultValue;
//            }
//        }
//
//        if (isset($newValue)) {
//            $_SESSION[$name] = $newValue;
//        }
//
//        return $newValue;
//    }


    /**
     * @param $typ
     * @return bool
     * @throws Exception
     */
    public function loadPopisTypuRozdeleni($typ)
    {
        $typ = TypRozdeleni::getTypRozdeleni($this ->database, $typ);
        return $typ ->popis;
    }


    /**
     * OK
     * @param Option $option1
     * @param Option $option2
     * @return int
     */
    protected static function compareOptionsRozdeleni($option1, $option2)
    {
        if (!$option1) {
            return $option2 ? -1 : 0;
        }

        if (!$option2) return 1;

        if ($option1 ->key < $option2 ->key) return -1;
        if ($option1 ->key > $option2 ->key) return 1;

        return 0;
    }

    /**
     * @param string $htmlKod
     * @param int $typRozdeleni
     * @param bool $jenVlastni
     * @param $filtr
     * @param $value
     * @return ComboBoxes
     * @throws Exception
     */
    protected function loadDataRozdeleni($htmlKod, $typRozdeleni, $jenVlastni, $filtr, $value)
    {
//        if ($this->showObdobi || $this->showVariantni) {
//            $datumOd = $this ->vybraneObdobiOd;
//            $datumDo = $this->vybraneObdobiDo;
//        } else if ($this->showDatumOdDo && ($this->datumOdDoPovinne || $this ->filtrovatDatum)) {
//            $datumOd = $this ->datumOd;
//            $datumDo = $this ->datumDo;
//        } else if ($this->showDatum) {
//            $datumOd = $this ->datum;
//            $datumDo = $datumOd;
//        } else {
//            $datumOd = time();
//            $datumDo = null;
//        }
        $datumOd = $this ->omezitObdobiOd ?? time();
        $datumDo = $this ->omezitObdobiDo ?? $datumOd;

        if (($jenVlastni == 2 || $jenVlastni == 3)  &&  $this ->f_pravarozdeleni) {
            $sql = $this ->f_pravarozdeleni ->sqlRozdeleni($this ->database,
                'Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass',
                (int)$typRozdeleni, $datumOd, $datumDo, $jenVlastni == 2);
        } else {
            $sql =
                'SELECT Kod, Popis, cssClass FROM Rozdeleni ' .
                ' WHERE KodTypuRozdeleni = ' . (int)$typRozdeleni .
                ' AND (DatumZruseni >= ' . $this ->database ->dateToString($datumOd) . ' OR DatumZruseni IS NULL) ';

            if ($datumDo)
                $sql .=
                    ' AND DatumVytvoreni <= ' . $this ->database ->dateToString($datumDo);

            $filtrRozdeleni = ($jenVlastni == 1)
                ? $this->sestavFiltrRozdeleni(1)  // vcetne podrizenych
                : $this->filtrRozdeleniPodlePrav($typRozdeleni);

            if ($filtrRozdeleni) {
                $sql .= ' AND (' . $filtrRozdeleni . ')';
            }
        }

        if ($filtr) {
            if ($sql instanceof SqlStatement) {
                $sql ->append(' AND (' . $filtr . ')');
            } else {
                $sql .= ' AND (' . $filtr . ')';
            }
        }

        if ($this ->useGet  &&  isset($_GET[$htmlKod])) {
            // pokud pouziva GET, tak povoli v URL vybrat i rozdeleni, do ktereho vybrana osoba nepatri
            // (jinak by neslo prepnout rozdeleni)
            // POZOR! V URL pak muze byt jina osoba nez skutecne nactena
        } else if (isset($_GET['osoba'])) {
            if (isset($value) && $value !== '' ) {
                // pokud je nastavena osoba v URL adrese, tak zkontroluje, jestli tato osoba patri do aktualne vybraneho rozdeleni
                // a pokud ne, tak rozdeleni vypne - je to nutne, aby fungovaly napr. odkazy v emailech
                if (!Rozdeleni::jeVRozdeleni($this->database, null, $_GET['osoba'], $typRozdeleni, $value, 3, $datumOd, $datumDo)) { // 3 = vcetne podrizenych
                    $value = '';
                }
            }
        }

        MessageManager::addDebugLog('Vyber rozdeleni ' . $typRozdeleni . ':');
        MessageManager::addDebugLog($sql);

        // vyber rozdeleni podle popisu
        $combo1 = new ComboBox($htmlKod);
        $combo1
            ->setClassName('form-control adsfilterforminput_popis ronselect')
            ->setCssClassIndex(2)
            ->setShowCode(true)
            ->setEmptyRow(TextManager::_('_vse_'))
            ->setTranslationPrefix("db_rozdeleni_{$typRozdeleni}_popis_")
            ->setHtmlParams([
                'data-kod-typu-rozdeleni' => $typRozdeleni,
                'data-ron-select2' => "#ronportalfilter_{$htmlKod}_kod",
                'data-ron-autosubmit' => ($this ->autoSubmit ? '1' : '0'),
                'data-insert-position' => 'before',
                'data-max-visible-rows' => 500,
                'readonly' => 'readonly'
            ]);

        $sqlPopis = $sql instanceof SqlStatement
            ? new SqlStatement($sql ->sqlText . ' ORDER BY 2', $sql ->parameters)
            : $sql . ' ORDER BY Rozdeleni.Popis';
        $combo1->loadData($this ->database, $sqlPopis, $value);
        //$combo1->render(); // 0, "data-ron-select2='{$htmlkod}_kod'" . ($this ->f_autosubmit ? ' data-ron-autosubmit="1"' : '') ); // .' style="width:100%;margin:0px" '

//        if (isset($combo1->f_selectedvalue)) {
//            $result = $combo1->f_selectedvalue;
//        }
        $options2 = [];
        foreach ($combo1 ->data as $option) {
            $option->params['data-popis'] = $option ->caption;
            $options2[] = $option;
        }

        usort($options2, 'RonPortal\Controls\RonPortalFilterForm::compareOptionsRozdeleni');


        // vyber kodu rozdeleni
        $combo2 = new ComboBox(null); // $htmlKod . '_kod'
        $combo2->className = 'form-control adsfilterforminput_kod desktop-only ronselect';
        $combo2->cssClassIndex = 2;
        $combo2->captionIndex = 0; // kod - kod
        $combo2->emptyRow = TextManager::_('_vse_');
//        $sqlKod = $sql instanceof SqlStatement
//            ? new SqlStatement($sql ->sqlText . ' ORDER BY Rozdeleni.Kod', $sql ->parameters)
//            : $sql . ' ORDER BY Rozdeleni.Kod';
//        $combo2->loadData($this ->database, $sqlKod, $value);

        $combo2 ->valueFieldName = 'key';
        $combo2 ->keyFieldName = 'key';
        $combo2 ->titleFieldName = 'caption';
        $combo2 ->cssClassFieldName = 'cssClass';

        $combo2 ->addOptions($options2);

        $combo2->setSelectedOption($value);

        $combo2->htmlParams =
            "data-ron-select2='$htmlKod'" . ($this ->autoSubmit ? ' data-ron-autosubmit="1"' : '') .
            ' data-columns="value;popis"' .
            ' data-drop-down-align="right"' .
            " id='ronportalfilter_{$htmlKod}_kod'";
        //$combo2->render(); // Show(0, "data-ron-select2='$htmlkod'" . ($this ->f_autosubmit ? ' data-ron-autosubmit="1"' : '') ); // .' style="width:100%;margin:0px" '

        return new ComboBoxes($combo1, $combo2);
    }


    /**
     * @throws Exception
     */
    public function loadRozdeleni()
    {
        $typRozdeleniSuffix = $this ->vlastniTypRozdeleni ? $this ->kodTypuRozdeleni : null;

        // nacte rozdeleni zadane uzivatelem
        $this ->setUserSettingsItem('ads_rozdeleni' . ($typRozdeleniSuffix ? '_' . $typRozdeleniSuffix : ''), 'rozdeleni', true, true);
        $this ->vcetnePodrizenychRozdeleni = (int)$this ->setUserSettingsItem('ads_rozdeleni_vcetne_podrizenych' . ($typRozdeleniSuffix ? '_' . $typRozdeleniSuffix : ''),
            'rozdeleni_vcetne_podrizenych', true, true);
//        $this ->vcetnePodrizenychRozdeleni = $this ->setUserSettingsItemCheckBox(
//            'ads_rozdeleni_vcetne_podrizenych' . ($typRozdeleniSuffix ? '_' . $typRozdeleniSuffix : ''),
//            'rozdeleni_vcetne_podrizenych', 'rozdeleni',
//            true, true, $this->vcetnePodrizenychRozdeleni);

        if ($this->pouzitPopisTypuRozdeleni) {
            $this->popisTypuRozdeleni = $this->loadPopisTypuRozdeleni($this->kodTypuRozdeleni);
        }

        $this ->comboBoxesRozdeleni = $this ->loadDataRozdeleni('rozdeleni',
            $this->kodTypuRozdeleni, $this->jenVlastniRozdeleni, $this->filtrRozdeleni,
            RonPortalSession::rozdeleni($typRozdeleniSuffix));

        if (isset($this ->comboBoxesRozdeleni ->combo1 ->selectedOption)) {
            $this ->kodRozdeleni = $this ->comboBoxesRozdeleni ->combo1 ->selectedOption ->key;
            RonPortalSession::setRozdeleni($this ->comboBoxesRozdeleni ->combo1 ->selectedOption ->key, $typRozdeleniSuffix);
        } else {
            $this ->kodRozdeleni = null;
        }

        //RonPortalSession::setRozdeleniVcetnePodrizenych(isset($_POST['rozdeleni_vcetne_podrizenych']), $typRozdeleniSuffix);

        //if (isset($noverozdeleni)) S_SetRozdeleni($noverozdeleni);
    }


//    /**
//     * @throws Exception
//     */
//    public function loadRozdeleni2()
//    {
//        $typRozdeleniSuffix = $this ->vlastniTypRozdeleni2 ? $this ->kodTypuRozdeleni2 : null;
//
//        $this->setUserSettingsItem('ads_rozdeleni2' . ($typRozdeleniSuffix ? '_' . $typRozdeleniSuffix : ''), 'rozdeleni2', true, true);
//        $this ->vcetnePodrizenychRozdeleni2 = (int)$this ->setUserSettingsItem('ads_rozdeleni2_vcetne_podrizenych' . ($typRozdeleniSuffix ? '_' . $typRozdeleniSuffix : ''),
//            'rozdeleni2_vcetne_podrizenych', true, true);
//
//        if ($this->pouzitPopisTypuRozdeleni2) {
//            $this->popisTypuRozdeleni2 = $this->loadPopisTypuRozdeleni($this->kodTypuRozdeleni2);
//        }
//        $this ->comboBoxesRozdeleni2 = $this ->loadDataRozdeleni('rozdeleni2',
//            $this->kodTypuRozdeleni2, $this->jenVlastniRozdeleni2,
//            $this->f_filtrrozdeleni2, RonPortalSession::rozdeleni2($typRozdeleniSuffix));
//
//        if (isset($this ->comboBoxesRozdeleni2 ->combo1 ->selectedOption)) {
//            $this ->kodRozdeleni2 = $this ->comboBoxesRozdeleni2 ->combo1 ->selectedOption ->key;
//            RonPortalSession::setRozdeleni2($this ->comboBoxesRozdeleni2 ->combo1 ->selectedOption ->key, $typRozdeleniSuffix);
//        } else {
//            $this ->kodRozdeleni2 = null;
//        }
//    }


    private function createComboOsoba()
    {
        $this ->comboOsoba = new ComboBoxZamestnanec('osoba');
        $this ->comboOsoba ->htmlParams['class'] = 'ronselect form-control adsfilterforminput_full';
        $this ->comboOsoba ->htmlParams['data-ron-autosubmit'] = $this ->autoSubmit ? '1' : '0';
        $this ->comboOsoba ->htmlParams['id'] = 'adsfilterform_osoba';
        $this ->comboOsoba ->htmlParams['data-max-visible-rows'] = '200';
        //    'data-columns' => 'text'
    }

//    public function loadOsobyPodleProcesu()
//    {
//        $this ->createComboOsoba();
//        $this ->comboOsoba ->loadDataPredavaciProces($this ->database,
//            $this->kodProcesu, $this->typRozdeleniProces,
//            false, RonPortalSession::vybranaOsoba()); // emptyrow = false
//
//        $this ->comboOsoba ->nastavOsobuDoSession();
//        $this ->vybranaOsoba = $this ->comboOsoba ->idPP;
//        $this ->vybranaOsobaRC = $this ->comboOsoba ->rc;
//        $this ->vybranaOsobaOsobniCislo = $this ->comboOsoba ->osobniCislo;
//        $this ->vybranaOsobaDatumNastupu = $this ->comboOsoba ->datumNastupu;
//
//        if (!$this ->vybranaOsobaRC || !$this ->vybranaOsobaOsobniCislo) {
//            MessageManager::addErrorMessage( TextManager::_('Neni_pracovni_pomer') );
//        }
//    }


    public function loadOsobyPodlePrav()
    {
        if ($this ->povolitVsechyOsoby) {
            $this->vsechnyOsoby = (bool)($_SESSION['vsechnyOsoby'] ?? false); // RonPortalSession::vsechnyOsoby();
        }

        if (!Prava::jsouNejakePrava($this->f_prava)) {
            RonPortalSession::smazudajezamestnance();

            MessageManager::addErrorMessage( TextManager::_('Nemate_pravo') );
        }

        $datumOd = $this ->osobyDatumOd ?: $this ->omezitObdobiOd ?? time();
        $datumDo = $this ->osobyDatumDo ?: $this ->omezitObdobiDo ?? $datumOd;

        // pokud je zaroven zobrazeno rozdeleni, tak osoby profiltruje
        $this ->createComboOsoba();

        if ($this->showRozdeleni) {
            //$typRozdeleniSuffix = $this ->vlastniTypRozdeleni ? $this ->f_typrozdeleni : null;
            //$this ->comboOsoba->nastavFiltrRozdeleni($this->f_typrozdeleni, RonPortalSession::rozdeleni($typRozdeleniSuffix), RonPortalSession::rozdeleniVcetnePodrizenych($typRozdeleniSuffix));
            $this ->comboOsoba->nastavFiltrRozdeleni($this->kodTypuRozdeleni, $this ->kodRozdeleni, $this ->vcetnePodrizenychRozdeleni);
        }

//        if ($this->showRozdeleni2) {
//            //$typRozdeleniSuffix = $this ->vlastniTypRozdeleni2 ? $this ->f_typrozdeleni2 : null;
//            //$this ->comboOsoba->nastavFiltrRozdeleni2($this->f_typrozdeleni2, RonPortalSession::rozdeleni2($typRozdeleniSuffix), RonPortalSession::rozdeleni2VcetnePodrizenych($typRozdeleniSuffix));
//            $this ->comboOsoba->nastavFiltrRozdeleni2($this->kodTypuRozdeleni2, $this ->kodRozdeleni2, $this ->vcetnePodrizenychRozdeleni2);
//        }
        $this ->comboOsoba->filtr = $this->osobyFiltr; // obecny filtr, napr. jen osoby, kterym se zpracovava dochazka

        foreach ($this ->rozdeleniFilters as $rozdeleniFilter) {
            if ($rozdeleniFilter ->getValue()) {
                $this->comboOsoba->addFiltrRozdeleni(
                    (int)$rozdeleniFilter->getTypRozdeleni(),
                    $rozdeleniFilter->getValue(),
                    (int)$rozdeleniFilter->getValue('vcetnePodrizenych')
                );
            }
        }

        if ($this ->povolitVsechyOsoby) {
            if (isset($_POST['osoba'])) {
                $this ->vsechnyOsoby = $_POST['osoba'] == '';
            } else if (isset($_GET['osoba'])) {
                $this ->vsechnyOsoby = $_GET['osoba'] == '';
            } else if ( !RonPortalSession::vybranaOsoba() ) {
                $this ->vsechnyOsoby = true;
            }

            $_SESSION['vsechnyOsoby'] = $this ->vsechnyOsoby; // RonPortalSession::setVsechnyOsoby($this ->vsechnyOsoby);
        }

        $selectedValue = ($this ->povolitVsechyOsoby  &&  $this ->vsechnyOsoby)
            ? ''
            : RonPortalSession::vybranaOsoba();

        if ($_GET['idpp'] ?? null) {
            $selectedValue = (int)$_GET['idpp']; // pro kompatibilitu se starymi odkazy, ktere muzou obsahovat idpp=... misto osoba=...
        }

        $this ->comboOsoba->loadDataPP($this ->database,
            $datumOd, $datumDo, $this->f_prava,
            $this ->povolitVsechyOsoby ? TextManager::_('_vsichni_'): false,  // $emptyRow
            $selectedValue,
            $this->vcetneExternich); // vcetneExternich

        if ($this ->povolitVsechyOsoby) {
            if (!$this ->comboOsoba ->selectedOption  ||  !$this ->comboOsoba ->selectedOption ->key) {
                // Pokud jsou povoleny vsechny osoby a v selectu je vybrana volba "vsichni", tak nastavi setVsechnyOsoby
                // Je to potreba v situaci, kdy byla puvodne v session osoba, na kterou nema uzivatel pravo
                $this->vsechnyOsoby = true;
                $_SESSION['vsechnyOsoby'] = true; // RonPortalSession::setVsechnyOsoby(true);
            }
        }

        if (!$this ->povolitVsechyOsoby  ||  !$this ->vsechnyOsoby) {
            $this ->comboOsoba->nastavOsobuDoSession();
            $this ->vybranaOsoba = $this ->comboOsoba ->idPP;
            $this ->vybranaOsobaRC = $this ->comboOsoba ->rc;
            $this ->vybranaOsobaOsobniCislo = $this ->comboOsoba ->osobniCislo;
            $this ->vybranaOsobaDatumNastupu = $this ->comboOsoba ->datumNastupu;

            if (!$this ->vybranaOsobaRC || !$this ->vybranaOsobaOsobniCislo) {
                MessageManager::addErrorMessage( TextManager::_('Neni_pracovni_pomer') );
            }
        }
    }


    protected function loadFiltersConfiguration($filtersConfiguration)
    {
        foreach ($filtersConfiguration as $filterConfiguration) {
            $name = $filterConfiguration['name'] ?? null;
            if ($name) {
                if (RonPortalObjectConfiguration::ronPortalConfigurationItemEnabled($filterConfiguration)) {
                    $filter = $this->getFilter($name);
                    if (!$filter  &&  ($filterConfiguration['type'] ?? null)) { // pro pridani noveho filtru musi byt zadan atribut type
                        $filter = new FilterFormFilter($name);
                        $this->filters[$name] = $filter;
                    }

                    if ($filter) {
                        $objectConfiguration = new RonPortalObjectConfiguration($this->database, $filter);
                        $objectConfiguration->loadConfigurationArray($filterConfiguration);
                    }
                }
            }
        }
    }

    protected function loadRozdeleniFiltersConfiguration($filtersConfiguration)
    {
        foreach ($filtersConfiguration as $filterConfiguration) {
            $name = $filterConfiguration['name'] ?? null;
            if ($name) {
                $filter = $this->rozdeleniFilters[$name] ?? null;
                if (!$filter  &&  ($filterConfiguration['typRozdeleni'] ?? null)) {
                    $filter = new FilterFormFilterRozdeleni($this, $name);
                    $this ->addFilterRozdeleni($filter);
                }
                if ($filter) {
                    $objectConfiguration = new ObjectConfiguration($filter);
                    $objectConfiguration->loadConfigurationArray($filterConfiguration);
                }
            }
        }
    }

    /**
     * Nacte polozku z konfigurace formulare
     * @param string $key
     * @param mixed $value
     * @param string $valueText
     * @return void
     */
    public function loadConfigurationItem($key, $value, $valueText)
    {
        switch ($key) {
            case 'obdobiNeboDatumPovinne' :
            case 'showDatumOdDo' :
            case 'datumOdDoPovinne' :
//            case 'datumOdDo_CheckboxFirst' :
//            case 'zobrazitDnyObdobi' :
//            case 'showRozdeleni' :
//            case 'kodTypuRozdeleni' :
//            case 'jenVlastniRozdeleni' :
//            case 'showVcetnePodrizenychRozdeleni' :
//            case 'showRozdeleni2' :
//            case 'kodTypuRozdeleni2' :
//            case 'jenVlastniRozdeleni2' :
//            case 'showVcetnePodrizenychRozdeleni2' :
//            case 'vcetneExternich' :
            case 'showMessages' :
            case 'skrytTlacitkoNastavit' :
                $this ->{$key} = (int)$value;
                break;

            case 'jenOtevreneObdobi' : // jen pro kompatibilitu
                $this ->filterObdobi ->jenOtevreneObdobi = (int)$value;
                break;

            case 'obdobiPovinne' : // jen pro kompatibilitu
                $this ->getFilter('obdobi') ->setRequired((int)$value);
                break;

//            case 'showObdobi' :  // TODO: odstranit?
//                $this ->nastavObdobi((int)$value, $this ->obdobiAplikace, false);
//                break;

//            case 'povolitVsechnyOsoby' :    // kvuli preklepu samostatne!
//                $this ->povolitVsechyOsoby = (int)$value;
//                break;

//            case 'nadpisDatumOdDo' :
            case 'datumOdDoLinks' :
//            case 'filtrRozdeleni' :
//            case 'filtrRozdeleni2' :
//            case 'osobyFiltr' :
            case 'formClass' :
//            case 'sqlPodrobnostiDnu' :
//            case 'nadpisPodrobnostiDnu' :
                $this ->{$key} = $value;
                break;

            case 'pravaRozdeleni' :
                if ($value) {
                    $this ->f_pravarozdeleni = Prava::vratPrava($value);
                }
                break;

            case 'filter' :
                $this ->loadFiltersConfiguration($value);
                break;

            case 'rozdelenifilter' :
                $this ->loadRozdeleniFiltersConfiguration($value);
                break;
        }
    }


    /**
     * Nacte konfiguraci z pole nacteneho pomoci XMLLoader (bud z konfigurace browseru nebo ze samostatneho XML config_filterform)
     * @param $a
     * @return void
     */
    public function loadConfigurationArray($a)
    {
        $objectConfiguration = new RonPortalObjectConfiguration($this ->database, $this);
        $objectConfiguration ->loadConfigurationArray($a);
    }


    public function loadConfiguration()
    {
        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->select(
            'SELECT ObsahXML FROM SablonaXMLFormulare WHERE Popis = ' . $params ->addParameter('config_filterform'),
            $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row  &&  $row[0]) {
            $a = XmlLoader::loadXmlToArray($row[0], [
                    XmlLoader::PARAM_CONVERT_FROM_UTF8 => true,
                    XmlLoader::PARAM_INCLUDE_ATTRIBUTES => true,
                    XmlLoader::PARAM_RAW_DATA_ELEMENT => 'html'
                ]
            );

            $this ->loadConfigurationArray($a);
        }

        $this ->configurationLoaded = true;
    }


    /**
     * Nacte hodnoty filtru ze zadaneho seznamu
     * @param string $position
     * @return void
     * @throws DBException
     */
    protected function loadFiltersList(string $position)
    {
        foreach ($this ->filters as $filter) {
            if ($filter ->position === $position) {
                $filter->load($this->database);
                // pokud filtr definuje obdobi, tak podle nej nastavi datumOd a datumDo
                if ($filter->definesPeriod  &&  $filter ->enabled) {   //  && !$this->omezitObdobiOd -
                    $datumOd = $filter->getValue('datumOd');
                    if ($datumOd) {
                        $this->omezitObdobiOd = $datumOd;
                        $this->omezitObdobiDo = $filter->getValue('datumDo') ?? $datumOd;
                    }
                }

                $this ->afterLoadFilter($filter);
            }
        }
    }

    protected function loadTopFilters()
    {
        $this ->loadFiltersList(FilterFormFilter::POSITION_TOP);
    }

    protected function loadFilters()
    {
        $this ->loadFiltersList(FilterFormFilter::POSITION_DEFAULT);
    }

    protected function loadBellowFormFilters()
    {
        $this ->loadFiltersList(FilterFormFilter::POSITION_BELOW_FORM);
    }

    protected function loadAboveFormFilters()
    {
        $this ->loadFiltersList(FilterFormFilter::POSITION_ABOVE_FORM);
    }


    /**
     * Volano po nacteni jednotlivych filtru
     * @param FilterFormFilter|string $filter   POZOR! Pro nektere stare filtry predava jen jmeno (string)
     * @return void
     */
    protected function afterLoadFilter($filter)
    {

    }


    /**
     * @throws Exception
     */
    public function loadData()
    {
        Profiler::addItem("");
        $item = Profiler::addItem('RonPortalFilterForm::loadData ----------', 'APP');

        if (!$this ->configurationLoaded) {
            $this->loadConfiguration();
        }

        $this ->loadAboveFormFilters();

        $this ->loadTopFilters();

//        if ($this ->showObdobi) {
//            $this ->loadObdobi();
//        }
        if ($this ->showDatumOdDo) {    // musi byt az za obdobi - kvuli zpracovani volby $obdobiNeboDatumPovinne
            $this ->loadDatumOdDo();
        }

//        if ($this ->showVariantni) {
//            $this ->loadVariantni();
//        }

//        if ($this ->showDatum  ||  $this ->zobrazitDnyObdobi) {
//        if ($this ->zobrazitDnyObdobi) {
//            $this ->loadDatum();
//        }
//        if ($this ->showRok) {
//            $this ->loadRok();
//        }

        $this ->nastavVybraneObdobiProPrava();

        if ($this ->showRozdeleni) {
            $this ->loadRozdeleni();
        }
//        if ($this ->showRozdeleni2) {
//            $this ->loadRozdeleni2();
//        }

        foreach ($this ->rozdeleniFilters as $rozdeleniFilter) {
            $rozdeleniFilter ->load($this ->database);
            $this ->afterLoadFilter($rozdeleniFilter);
        }

        if ($this->showOsoby) {
            $this ->loadOsobyPodlePrav();
//            if ($this ->f_prava) {
//                $this ->loadOsobyPodlePrav();
//            } else {
//                $this ->loadOsobyPodleProcesu();
//            }
            $this ->afterLoadFilter('osoba');
        }

        $this ->loadFilters();

        $this ->loadBellowFormFilters();

        Profiler::endItem($item);
        Profiler::addItem("");
    }


    /**
     * Prida filtr do seznamu filtru.
     * @param FilterFormFilter $filter
     * @param bool $top     Urcuje, jestli se ma pridat na zacatek seznamu filtru
     * @return RonPortalFilterForm
     */
    public function addFilter(FilterFormFilter $filter, bool $top = false): RonPortalFilterForm
    {
        if ($top) {
            $filter ->setPosition(FilterFormFilter::POSITION_TOP);
        }

        $this->filters[$filter->name] = $filter;
        $filter ->filterForm = $this;

        return $this;
    }


    /**
     * Vraci filtr se zadanym jmenem. Pokud neexistuje, vraci null.
     * @param string $filterName
     * @return FilterFormFilter|null
     */
    public function getFilter(string $filterName): ?FilterFormFilter
    {
        $result = $this ->filters[$filterName] ?? $this ->rozdeleniFilters[$filterName] ?? null;
        if ($result) {
            return $result;
        }

        switch ($filterName) {
            case 'obdobi' :
                return $this ->filterObdobi;
        }

        return null;
    }


    /**
     * Vraci aktualni hodnotu filtru se zadanym jmenem
     * @param string $filterName
     * @return mixed|null
     */
    public function getFilterValue(string $filterName, ?string $specification = null)
    {
        $filter = $this ->getFilter($filterName);
        return $filter
            ? $filter ->getValue($specification)
            : null;
    }

    // --------------------------------------------------------------------------
    // Metody pro zobrazeni formulare (RENDER)
    public function renderFilter(FilterFormFilter $filter)
    {
        if (!$filter ->enabled) {
            return;
        }

        if ($filter ->rowClass !== false) {
            $id = $filter ->rowId === true
                ? 'filterrow_' . $filter ->name
                : $filter ->rowId;

            $otherParamsHtml = '';
            foreach ($filter ->otherSettings as $settingName => $settingValue) {
                if (is_string($settingValue)  ||  is_numeric($settingValue)) {
                    $otherParamsHtml .=
                        " data-$settingName='" . StringUtils::htmlTextConvert($settingValue) . "'";
                }
            }


            echo
                "<div class='{$filter ->rowClass}'" . ($id ? " id='$id'" : '') .
                " data-name='{$filter ->name}'" .
                $otherParamsHtml .
                '>';
        }

        if ($filter ->caption !== false) {
            echo
                '<span ' . GuiUtils::getAttributesHtml(array_merge(['class' => 'filtercaption'], $filter ->captionHtmlAttributes ?? [])) . '>';

            $filter->renderCaption();

            echo
                '</span>';
        }

        $filter ->render();

        if ($filter ->rowClass !== false) {
            echo '</div>';
        }

        echo "\n";
    }


    /**
     * @param int $datumOd
     * @param int $datumDo
     * @param string $text
     * @param string[]|null $otherHtmlParams
     * @return void
     */
    public function renderDateLink($datumOd, $datumDo, $text, $otherHtmlParams = null)
    {
        $selected = $this ->datumOd === $datumOd  &&  $this ->datumDo === $datumDo;

        $params = [
            'class' => 'js setValue' . ($selected ? ' selectedlink' : ''),
            'data-trigger-change' => '1',
            'data-submit' => '1',
            'data-selectors-count' => '4',
            // datum od
            'data-selector1' => '#filterrow_datum_od_do input[name=datumod]',
            'data-value1' => $datumOd ? StringUtils::format('date', $datumOd) : '',
            // datum do
            'data-selector2' => '#filterrow_datum_od_do input[name=datumdo]',
            'data-value2' => $datumDo ? StringUtils::format('date', $datumDo) : '',
            // checkbox jen datum od
            'data-selector3' => '#filterrow_datum_od_do input[name=' . ($this ->obdobiNeboDatumPovinne ? 'jenobdobi' : 'jendatumoddo') . ']',
            'data-value3' => '1',
            // checkbox jen obdobi
            'data-selector4' => '#filterrow_obdobi input[name=jenobdobi]',
            'data-value4' => '0'
        ];

        if ($otherHtmlParams) {
            $params = array_merge($params, $otherHtmlParams);
        }

        echo
            '<li ' . GuiUtils::getAttributesHtml($params) . '>' .
            $text .
            '</li>';
    }

    /**
     * Umoznuje zobrazit dalsi obsah za vyber datumu od-do
     */
    public function renderDatumOdDoLinks()
    {
        if (is_string($this ->datumOdDoLinks)) {
            echo $this ->datumOdDoLinks;
        } else if (is_array($this ->datumOdDoLinks)) {
            echo "\n<ul class='linkslist margin-left-2'>";

            foreach ($this ->datumOdDoLinks as $item) {
                if (is_string($item)) {
                    switch ($item) {
                        case 'today' :
                        case 'currentDay' :
                            $this ->renderDateLink(DateUtils::today(), DateUtils::today(), TextManager::_('Today'), ['id' => 'datum_od_do_today']);
                            break;

                        case 'currentMonth' :
                            $this ->renderDateLink(DateUtils::firstDayOfMonth(time()), DateUtils::lastDayOfMonth(time()), TextManager::_('Aktualni_mesic'));
                            break;

                        case 'prevMonth' :
                            $this ->renderDateLink(DateUtils::incMonth(DateUtils::firstDayOfMonth(time()), -1), DateUtils::incDay(DateUtils::firstDayOfMonth(time()), -1), TextManager::_('Predchozi_mesic'));
                            break;

                        case 'currentQuarter' :
                            $this ->renderDateLink(
                                DateUtils::firstDayOfQuarter(time()),
                                DateUtils::incDay(DateUtils::incMonth(DateUtils::firstDayOfQuarter(time()), 3), -1),
                                TextManager::_('Aktualni_ctvrtleti'));
                            break;

                        default :
                            echo  $item;
                    }
                } else if (is_array($item)) {
                    // polozka je pole ve tvaru "datumOd" => ..., "datumDo" => ..., "text" => ..., "htmlParams" => []
                    $this ->renderDateLink($item['datumOd'], $item['datumDo'], $item['text'], $item['htmlParams'] ?? null);
                }
            }

            echo "</ul>\n";
        }
    }


    /**
     * Zobrazeni filtru datum od - do
     */
    public function renderDatumOdDo()
    {
        $nadpis = $this->nadpisDatumOdDo ?: TextManager::_('Datum_od');

        echo '<div class="filterrow" id="filterrow_datum_od_do">';

        echo '<span class="filtercaption">';
        if ($this->datumOdDoPovinne) {
            echo $nadpis; // .' &nbsp;&nbsp;';
        } else {
            $params = ['type' => $this ->obdobiNeboDatumPovinne ? 'radio' : 'checkbox'];
            if (!$this ->obdobiNeboDatumPovinne  &&  $this ->getFilter('obdobi')) {
                $params['data-excluded-checkboxes'] = 'input[name=jenobdobi]';
            }
            GuiUtils::renderCheckbox(
                $this ->obdobiNeboDatumPovinne ? 'jenobdobi' : 'jendatumoddo',
                $nadpis, $this ->filtrovatDatum, $params,
                'clickAutoSubmit',
                $this ->obdobiNeboDatumPovinne ? 2 : 1, true);
        }
        echo '</span>';
echo '<div>'; // kvuli bootstrap
        //echo '<span Xstyle="white-space:nowrap;">';
        DateInput::renderDateSelect('datumod', date('j.n.Y', $this ->datumOd), $this ->autoSubmit); // submitonenter

        echo '&nbsp;&nbsp;&nbsp; ' . TextManager::_('Do') . ' &nbsp;&nbsp;';
        DateInput::renderDateSelect('datumdo', $this ->datumDo ? date('j.n.Y', $this ->datumDo) : null, $this ->autoSubmit); // submitonenter
        //echo '</span>';

        $this ->renderDatumOdDoLinks();
echo '</div>';
        echo "</div>\n";
    }


    public function renderRozdeleni()
    {
        if (!$this->kodTypuRozdeleni) {
            return false;
        }

        echo '<div class="filterrow" id="filterrow_rozdeleni">';
        echo
            '<span class="filtercaption">' .
            ($this->popisTypuRozdeleni
                ? StringUtils::htmlTextConvert($this->popisTypuRozdeleni)
                : TextManager::_(RonPortalSettings::getValue(RonPortalSettings::ROZDELENI_NADPIS))) .
            '</span>';

        echo '<div class="adsfilterforminput_full input-group">';

        $this ->comboBoxesRozdeleni ->combo1 ->render();
        $this ->comboBoxesRozdeleni ->combo2 ->render();

        echo '</div> ';

        if ($this->showVcetnePodrizenychRozdeleni) {
            //$typRozdeleniSuffix = $this ->vlastniTypRozdeleni ? $this ->kodTypuRozdeleni : null;
            $value = $this ->vcetnePodrizenychRozdeleni; // RonPortalSession::rozdeleniVcetnePodrizenych($typRozdeleniSuffix);

            echo
                '<select name="rozdeleni_vcetne_podrizenych"' . ($this ->autoSubmit ? ' data-ron-autosubmit="1"' : '') . '>' .
                '<option value="0"' . (!$value ? " selected='selected'" : '') . '>' . TextManager::_('Rozdeleni_jen_vybrane') . '</option>' .
                '<option value="1"' . ($value == 1 ? " selected='selected'" : '') . '>' . TextManager::_('Rozdeleni_vcetne_podrizenych') . '</option>' .
                '<option value="2"' . ($value == 2 ? " selected='selected'" : '') . '>' . TextManager::_('Rozdeleni_jen_podrizena') . '</option>' .
                '<option value="3"' . ($value == 3 ? " selected='selected'" : '') . '>' . TextManager::_('Rozdeleni_jen_primo_podrizena') . '</option>' .
                '</select>';
        }

        echo "</div>\n";

        return true;
    }


//    public function renderRozdeleni2()
//    {
//        if (!$this->kodTypuRozdeleni2) return false;
//
//        echo '<DIV class="filterrow" id="filterrow_rozdeleni2">';
//        echo '<SPAN class="filtercaption">' . ($this->popisTypuRozdeleni2 ? StringUtils::htmlTextConvert($this->popisTypuRozdeleni2) : TextManager::_(RonPortalSettings::getValue(RonPortalSettings::ROZDELENI2_NADPIS))) . '</SPAN>';
//
//        $this ->comboBoxesRozdeleni2 ->combo1 ->render();
//        $this ->comboBoxesRozdeleni2 ->combo2 ->render();
//
//        if ($this->showVcetnePodrizenychRozdeleni2) {
//            //$typRozdeleniSuffix = $this ->vlastniTypRozdeleni2 ? $this ->kodTypuRozdeleni2 : null;
//            $value = $this ->vcetnePodrizenychRozdeleni2; // RonPortalSession::rozdeleni2VcetnePodrizenych($typRozdeleniSuffix);
//
//            echo
//                '<select name="rozdeleni2_vcetne_podrizenych"' . ($this ->autoSubmit ? ' data-ron-autosubmit="1"' : '') . '>' .
//                '<option value="0"' . (!$value ? " selected='selected'" : '') . '>' . TextManager::_('Rozdeleni_jen_vybrane') . '</option>' .
//                '<option value="1"' . ($value == 1 ? " selected='selected'" : '') . '>' . TextManager::_('Rozdeleni_vcetne_podrizenych') . '</option>' .
//                '<option value="2"' . ($value == 2 ? " selected='selected'" : '') . '>' . TextManager::_('Rozdeleni_jen_podrizena') . '</option>' .
//                '<option value="3"' . ($value == 3 ? " selected='selected'" : '') . '>' . TextManager::_('Rozdeleni_jen_primo_podrizena') . '</option>' .
//                '</select>';
//        }
//
//        echo "</DIV>\n";
//
//        return true;
//    }


    public function renderOsoby()
    {
        echo '<DIV class="filterrow" id="filterrow_osoba">';
        echo '<SPAN class="filtercaption">' . TextManager::_('Osoba') . '</SPAN>';

        $this ->comboOsoba ->render();

        if ($this ->f_prava) {
            if ($this->comboOsoba->data && count($this->comboOsoba->data) > 1) {
                // tlacitka pro posun na dalsi a predchozi PP
                echo '<span id="adsfilterform_pp_fast" class="desktop-only">';
                echo '<i class="inline-glyph glyph-button fa glyph-left comboBoxMove" data-step="-1" data-submit="1" data-selector="#adsfilterform_osoba" data-trigger-change="1"></i>';
                echo '<i class="inline-glyph glyph-button fa glyph-right comboBoxMove" data-submit="1" data-selector="#adsfilterform_osoba" data-trigger-change="1"></i>';
                echo '</span>';
            }
        }

        if ($this ->comboOsoba ->obsahujeVlastniPP  &&  $this ->comboOsoba ->rc != User::getRC()) {
            // zobrazi ikonu pro prechod na vlastni PP
            echo
                '<i id="vybratVlastniBtn" class="inline-glyph glyph-button fa glyph-own-data comboBoxMove"' .
                " data-value='{$this ->comboOsoba ->idVlastnihoPP}'" .
                ' title="' . TextManager::_('Vybrat_vlastni_zaznamy') .'"' .
                ' data-submit="1" data-selector="#adsfilterform_osoba" data-trigger-change="1"></i>';
        }

        if ($this ->povolitVsechyOsoby  &&  !$this ->vsechnyOsoby) {
            // zobrazi ikonu pro prechod na vsechny osoby
            echo
                '<i id="vybratVsechnyOsobyBtn" class="inline-glyph glyph-button fa glyph-all-persons comboBoxMove"' .
                " data-value=''" .
                ' title="' . TextManager::_('Vse') .'"' .
                ' data-submit="1" data-selector="#adsfilterform_osoba" data-trigger-change="1"></i>';
        }

        echo "</DIV>\n";
    }


    public function renderFilters()
    {
        foreach ($this ->filters as $filter) {
            if ($filter ->position === FilterFormFilter::POSITION_TOP) {
                $this->renderFilter($filter);
            }
        }

        if ($this->filterObdobi  &&  $this ->filterObdobi ->enabled) {
            $this ->renderFilter($this ->filterObdobi);
        }

        if ($this->showDatumOdDo) {
            $this->renderDatumOdDo();
        }
        if ($this->showRozdeleni) {
            $this->renderRozdeleni();
        }
//        if ($this->showRozdeleni2) {
//            $this->renderRozdeleni2();
//        }

        foreach ($this ->rozdeleniFilters as $rozdeleniFilter) {
            $this ->renderFilter($rozdeleniFilter);
        }

        if ($this->showOsoby) {
            $this->renderOsoby();
        }

        foreach ($this ->filters as $filter) {
            if ($filter ->position === FilterFormFilter::POSITION_DEFAULT) {
                $this->renderFilter($filter);
            }
        }
    }


//    public function buildSqlPodrobnostiDnu()
//    {
//        return $this ->sqlPodrobnostiDnu;
//    }


//    public function getHtmlPodrobnostiKeDni($row)
//    {
//        return $row[1] ?? '';
//    }


//    public function renderDnyObdobi()
//    {
//        $podrobnosti = [];
//
//        $sqlPodrobnosti = $this ->buildSqlPodrobnostiDnu();
//        if ($sqlPodrobnosti) {
//            $cursor = $this ->database ->getSqlResult($sqlPodrobnosti);
//            while ($row = $cursor ->fetch()) {
//                $date = $this ->database ->stringToTimestamp($row[0]);
//                $den = (int)date('j', $date);
//                $podrobnosti[$den] = $row;
//            }
//        }
//
//        echo
//            '<div id="filter_dnyobdobi" class="margin-top-2 margin-bottom">' .
//            '<input type="hidden" name="datum" id="filter_dny_obdobi_datum" value="' . DateUtils::dateToUrlString($this ->datum) . '"/>';
//
//        $htmlElement = 'span';
//        if ($podrobnosti) {
//            echo
//                '<table><tr>' . ($this ->nadpisPodrobnostiDnu ? '<td></td>' : '');
//            $htmlElement = 'td';
//        }
//
//        $rowPodrobnosti = $this ->nadpisPodrobnostiDnu
//            ? '<td>' . $this ->nadpisPodrobnostiDnu . '</td>'
//            : '';
//
//        $datum = $this ->getOmezitObdobiOd();
//        while ($datum <= $this ->getOmezitObdobiDo()) {
//            $className =
//                'js margin-right padding-all setValue ' .
//                Kalendar::getStyleForDate($this ->database, $datum, false) .
//                ($datum === $this ->datum ? ' font-weight-bold' : '');
//
//            $valueStr = DateUtils::dateToUrlString($datum);
//
//            echo
//                "<$htmlElement class='$className' data-selector='#filter_dny_obdobi_datum' data-value='$valueStr' data-submit='1'>" .
//                date('j.n.', $datum) .
//                "</$htmlElement>";
//
//            $rowPodrobnosti .= '<td class="align-center">' . $this ->getHtmlPodrobnostiKeDni($podrobnosti[(int)date('j', $datum)] ?? null) . '</td>';
//
//            $datum = DateUtils::incDay($datum);
//        }
//
//        if ($podrobnosti) {
//            echo
//                '</tr>' .
//                "<tr>$rowPodrobnosti</tr>" .
//                '</table>';
//        }
//
//        echo '</div>';
//    }

    public function renderFoto()
    {
        if ($this ->downloadFotoOsobyUrl) {
            $idPP = null;

            if ($this ->showOsoby  &&  !($this ->povolitVsechyOsoby && $this ->vsechnyOsoby)  &&  $this ->comboOsoba  &&  $this ->comboOsoba ->idFotografie) {
                $idPP = $this ->comboOsoba ->idPP;
            } else if ($this ->getFilterValue('osoba', 'idFotografie')) {
                $idPP = $this ->getFilterValue('osoba');
            }

            if ($idPP) {
                echo '<div class="fotoDochazka ronFoto desktop-only"><img alt="Foto" src="' . $this->downloadFotoOsobyUrl . $idPP . '"></div>';
            }
        }
    }

    /**
     * vraci SQL filtr ( EXISTS ... ) pro vybrane rozdeleni
     * @param $tabulka
     * @param int $vcetnePodrizenych 0=jen vybrane, 1=vcetne podrizenych, 2=jen podrizne, 3=jen primo podrizene
     * @param int $datumOd
     * @param int $datumDo
     * @param Parameters $params
     * @return string
     */
    public function vratSQLFiltrRozdeleni($tabulka, $vcetnePodrizenych, $datumOd = 0, $datumDo = 0, $params = null)
    {
        if (!$this->showRozdeleni) {
            return '';
        }
        if (!$this->kodTypuRozdeleni) {
            return '';
        }
        if ($this ->kodRozdeleni === null) {
            return '';
        }

        $datumOd = $datumOd ?: ($this ->omezitObdobiOd ?? time());
        $datumDo = $datumDo ?: ($this ->omezitObdobiDo ?? $datumOd);

        return RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, $tabulka,
            $this->kodTypuRozdeleni, $this ->kodRozdeleni,
            $datumOd, $datumDo, $vcetnePodrizenych, $params);
    }


    /**
     */
    public function render()
    {
        if ($this ->header) {
            echo "<h1>{$this ->header}</h1>\n";
        }

        $actionStr = '';
        if ($this ->getAction) {
            $actionStr = ' action="' . $this->getAction . '"';
        } else if ($this->f_formaction) {
            $actionStr = ' action="' . $this->f_formaction . '"';
        }

        $attrs = GuiUtils::getAttributesHtml($this ->formAttributes);
        echo
            '<form method="' . ($this ->useGet ? 'GET' : 'POST') . '" name="' . $this->name . '"' .
            ' data-ron-autosubmit="' . ($this ->autoSubmit ? '1' : '0' ) . '" ' .
            ' data-custom-filter-supported="' . ($this ->customFilterSupported ? '1' : '0') . '" ' .
            $attrs .
            $actionStr . '>';

        if ($this ->useGet) {
            if ($this ->getMenuId) {
                echo "<input type='hidden' name='menuid' value='{$this ->getMenuId}' />\n";
            }
        } else {
            echo '<input type="hidden" name="adsfilterform" value="1">';
        }

        foreach ($this ->hiddenFields as $fieldName => $fieldValue) {
            $val = StringUtils::htmlTextConvert($fieldValue);
            echo "<input type='hidden' name='$fieldName' value='$val' />\n";
        }

        foreach ($this ->filters as $filter) {
            if ($filter ->position === FilterFormFilter::POSITION_ABOVE_FORM) {
                $this->renderFilter($filter);
            }
        }

        $idStr = $this ->htmlId ? " id='{$this ->htmlId}'" : '';
        echo '<div class="' . $this ->formClass . '"><div class="form-content">';
        echo "<div class='{$this->f_classname}' {$idStr}>\n";

        echo '<div class="filtercontainercol col">';
        // vnitrni tabulka - jednotlive filtry
        $this->renderFilters();
        echo '</div>';

        // zobrazi tlacitko nastavit - kdyby nefungoval javascript
        if (!$this ->skrytTlacitkoNastavit) {
            echo '<div class="filtercontainercol col">';
            echo '<input type="submit" class="submitbutton" value="' . TextManager::_('Nastavit') . '">';
            echo '</div>';
        }

        $this->renderFoto();

        echo
            "</div>\n" .
            "</div>" .  // form-content
            "<i class='ronportalfilterform-submit-indicator d-none fa glyph-spinner fa-spin submit-indicator position-absolute position-top position-right margin-all-2'></i>" .
            "</div>\n"; // ronportalfilterform

        foreach ($this ->filters as $filter) {
            if ($filter ->position === FilterFormFilter::POSITION_BELOW_FORM) {
                $this->renderFilter($filter);
            }
        }

//        if ($this ->zobrazitDnyObdobi) {
//            $this ->renderDnyObdobi();
//        }

        echo "</form>\n";

        if ($this ->showMessages) {
            MessageManager::getInstance() ->showMessages();
        }
    }


    public function buildSqlFilter($params = null)
    {
        $resultParams = $params;
        $result = '';

        foreach ($this ->filters as $filter) {
            if ($filter ->enabled) {
                if (!$resultParams) {
                    $resultParams = $this->database->createParameters();
                }

                $filterSql = $filter ->buildSQLFilter($this ->database, $resultParams);

                if ($filterSql) {
                    $result .=
                        ($result ? ' AND ' : '') .
                        $filterSql .
                        "\n";
                }
            }
        }

        $this ->customFilterSupported = true;

        return ($result && !$params)
            ? new SqlStatement($result, $resultParams)
            : $result;
    }


    public function getVariables()
    {
        $result = ($this ->filterObdobi  &&  $this ->filterObdobi ->enabled)
            ? $this ->filterObdobi ->getVariables()
            : [];

        foreach ($this ->filters as $filter) {
            if ($filter ->enabled) {
                $result = array_merge($result, $filter ->getVariables());
            }
        }

        foreach ($this ->rozdeleniFilters as $filter) {
            if ($filter ->enabled) {
                $result = array_merge($result, $filter ->getVariables());
            }
        }

        return $result;
    }
}
