<?php
/**
 * stanislav.elbl, 28.03.2022 15:43
 */

namespace RonPortal\Controls;

use Ron\Controls\ConfigurableObject;
use Ron\Controls\ObjectConfiguration;
use Ron\DB\Database;
use Ron\DB\DBException;
use RonPortal\User;
use RonPortal\XML\XmlFormsList;
use Tracy\Debugger;

/**
 * Obecna trida pro nacitani konfigurace z XML textu.
 * Class RonPortalObjectConfiguration
 */
class RonPortalObjectConfiguration extends ObjectConfiguration
{
    /** @var Database */
    protected $database;

    /** @var XmlFormsList */
    public $xmlFormsList;

    /**
     * RonportalObjectConfiguration constructor.
     * @param Database $database
     * @param ConfigurableObject $configurableObject
     */
    public function __construct(Database $database, $configurableObject)
    {
        parent::__construct($configurableObject);
        $this->database = $database;
        $this->xmlFormsList = new XmlFormsList();
    }


    public static function ronPortalConfigurationItemEnabled($item)
    {
        if ($item === null) {
            return false;
        }

        if (!$item instanceof \ArrayAccess) {
            return true;
        }

        if ($item['profil']  &&  !User::jeVProfilu($item['profil'])) {
            return false;
        }

        if ($item['menuid']  &&  (($_GET['menuid'] ?: '') != $item['menuid'])) {
            return false;
        }

        return true;
    }

    /**
     * Vraci false v pripade, ze je pouziti polozky nejak omezeno a aktualni uzivatel toto omezeni nesplnuje (napriklad nepatri do nastaveneho profilu)
     * @param $item
     * @return bool
     */
    public function configurationItemEnabled($item)
    {
        return self::ronPortalConfigurationItemEnabled($item);
    }


    protected function loadConfigurationItem($key, $value)
    {
        if (!$this ->configurationItemEnabled($value)) {
            return;
        }

        if ($key === 'xml') {
            // xml formular, ktery se ma zobrazit
            $this ->xmlFormsList ->loadSettings($value);
        } else {
            parent::loadConfigurationItem($key, $value);
        }
    }


    /**
     * Nacte konfiguraci objektu (napr. formulare, browseru) se zadanym kodem z databaze
     * @param Database $database
     * @param string $code
     * @return string Vraci text konfiguracniho XML
     * @throws DBException
     */
    public static function loadConfigurationFromDatabase($database, $code)
    {
        if (!$code) {
            return null;
        }

        $params = $database ->createParameters();
        $cursor = $database ->select(
            'SELECT ObsahXML FROM SablonaXMLFormulare' .
            ' WHERE Popis = ' . $params ->addParameter('config_' . $code),
            $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? $row[0] : null;
    }


    /**
     * Nacte a zpracuje konfiguraci objektu se zadanym kodem
     * @param $code
     * @throws DBException
     */
    public function loadConfiguration($code)
    {
        $xmlText = self::loadConfigurationFromDatabase($this ->database, $code);

        if ($xmlText) {
            $this ->loadConfigurationXml($xmlText);
        }
    }


    /**
     * Renderuje obsah XML formularu, ktere maji zobrazeny byt na zadane pozici
     * @param $position
     */
    public function renderXmlForms($position)
    {
        if ($this ->xmlFormsList) {
            $this ->xmlFormsList ->renderXmlForms($position);
        }
    }

    /**
     * Zpracuje XML formulare
     * @param $variables
     */
    public function processXmlForms($variables)
    {
        if ($this ->xmlFormsList) {
            $this ->xmlFormsList ->processXMLForms($this ->database, $variables);
        }
    }
}
