<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Controls;

use Ron\Controls\BrowserField;
use Ron\DB\Database;
use Ron\StringUtils;
use Ron\TextManager;

/**
 * Pole pro nacteni a zobrazeni rozdeleni osoby - vyhledava jen podle RC nebo podle prac. pomeru
 * Class RozdeleniOsobyBrowserField
 * @package RonPortal\Controls
 */
class RozdeleniOsobyBrowserField extends BrowserField
{
    public $typRozdeleni;

    public $zobrazitKod = true;
    public $zobrazitPopis = false;

    /** @var int    Index pole s rodnym cislem osoby (povinne) */
    public $indexRC = null;
    /** @var int    Index pole s OsobnimCislem */
    public $indexOsobniCislo = null;
    /** @var int    Index pole s datem nastupu (nepovinne) */
    public $indexDatumNastupu = null;

    /** @var int    Index pole s datumem od (nepovinne) */
    public $indexDatumOd = null;
    /** @var int    Index pole s datumem do (nepovinne) */
    public $indexDatumDo = null;

    /** @var int    Obdobi prirazeni na rozdeleni */
    public $datumOd;
    public $datumDo;


    /**
     * @param Database $database
     * @param int $typRozdeleni
     */
    function __construct($database, $typRozdeleni)
    {
        parent::__construct($database);

        $this->database = $database;
        $this->typRozdeleni = $typRozdeleni;

        $this->datumOd = time();
        $this->datumDo = time();

        $this->header = TextManager::_('Rozdeleni');
    }


    function loadConfigurationItem($key, $value)
    {
        switch ($key) {
            case 'zobrazitKod' :
            case 'zobrazitPopis' :
            case 'typRozdeleni' :
                $this ->{$key} = (int)$value;
                return $this;

            case 'indexDatumOd' :
            case 'indexDatumDo' :
            case 'indexOsobniCislo' :
            case 'indexDatumNastupu' :
                $this ->{$key} = $value;
                return $this;
        }

        return parent::loadConfigurationItem($key, $value);
    }


    function getHeaderAttributes()
    {
        return array_merge([
            'data-zobrazitKod' => $this ->zobrazitKod ? '1' : '0',
            'data-zobrazitPopis' => $this ->zobrazitPopis ? '1' : '0',
            'data-typRozdeleni' => $this ->typRozdeleni
            ],
            $this ->headerAttributes ?: []
        );
    }


    /**
     * nastavni indexu poli v master tabulce (tj. v $rowdata predanem do GetViewText)
     * @param $indexRC
     * @param int|null $indexOsC
     * @param int|null $indexNastup
     */
    function nastavIndexPole($indexRC, $indexOsC = null, $indexNastup = null)
    {
        $this->indexRC = $indexRC;
        $this->indexOsobniCislo = $indexOsC;
        $this->indexDatumNastupu = $indexNastup;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$this->database) {
            return '';
        }

        if (!isset($this->indexRC)  ||  !$rowData[$this->indexRC]) { // zadna osoba
            return '';
        }

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Rozdeleni.Kod, Rozdeleni.Popis ' .
            ' FROM RozdeleniOsob, Rozdeleni' .
            ' WHERE RozdeleniOsob.KodTypuRozdeleni = ' . $params ->addParameter((int)$this->typRozdeleni) .
            '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
            '   AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni ' .
            '   AND RozdeleniOsob.RC = ' . $params->addParameter($rowData[$this->indexRC]);

        if (isset($this->indexOsobniCislo)  &&  isset($this->indexDatumNastupu)) {
            // vyber podle pracovniho pomeru (ne jen podle RC)
            $sql .=
                '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($rowData[$this->indexOsobniCislo]) .
                '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addDateParameter($this ->database ->stringToTimestamp($rowData[$this->indexDatumNastupu]));
        }

        // urceni datumu, ke kteremu se rozdeleni nacita
        if (isset($this->indexDatumOd)) {
            $datumOd = $this ->database ->stringToTimestamp($rowData[$this->indexDatumOd]);
            $datumDo = 0;

            if (isset($this->indexDatumDo)) {
                $datumDo = $this ->database ->stringToTimestamp($rowData[$this->indexDatumDo]);
            }
        } else {
            $datumOd = $this->datumOd;
            $datumDo = $this->datumDo;
        }

        if (!$datumDo) {
            $datumDo = $datumOd;
        }

        $sql .=
            '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter($datumDo) .
            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter($datumOd) . ' OR RozdeleniOsob.PlatnostDo IS NULL)';

        $sql .=
            ' ORDER BY RozdeleniOsob.PlatnostOd ';

        $result = '';

        $cursor = $this ->database ->select($sql, $params);

        while ($row = $cursor ->fetch()) {
            // nacetl zaznam
            $result .=
                ($result ? ', ' : '') .
                ($this->zobrazitKod ? StringUtils::htmlTextConvert($row[0]) : '') .
                ($this->zobrazitKod && $this->zobrazitPopis ? ' - ' : '') .
                ($this->zobrazitPopis ? StringUtils::htmlTextConvert($row[1]) : '');
        }
        $cursor ->close();

        return $result;
    }
}
