<?php
/**
 * stanislav.elbl, 02.11.2015
 */

namespace RonPortal\Dochazka;

use Ron\CalendarItem;
use Ron\DB\Database;
use Ron\DB\Parameters;


class AbsenceCalendarItem extends CalendarItem
{
    /** @var string */
    public $kodOperace;

    function computeUid($idPP, $poradi)
    {
        if (!$this ->dateTimeFrom) {
            throw new \Exception('Chyba v AbsenceCalendarItem::computeUid! Nejdrive musi byt nastaven datum!');
        }

        $this ->uid = "ronportal.a.$idPP." . date("Y-m-d", $this ->dateTimeFrom) . "." . $poradi;
    }


    function setTimes($delkaAbsence, $datumOd, $datumDo, $casOd, $casDo)
    {
        $dateFrom = new \DateTime();
        $dateFrom ->setTimestamp($datumOd);

        $dateTo = new \DateTime();
        $dateTo ->setTimestamp($datumDo);

        $this ->wholeDay = false;

        switch ($delkaAbsence) {
            case 1 : // prvni pulka
                $dateFrom ->setTime(8, 0);  // TODO: udelat to nejak nastavitelne?
                $dateTo ->setTime(12, 0);
                break;

            case 2 : // druha pulka
                $dateFrom ->setTime(12, 0);  // TODO: udelat to nejak nastavitelne?
                $dateTo ->setTime(16, 0);
                break;

            case 3 : // od-do
            case 4 : // kontinualne
//                $dateFrom ->setTime( date('G', $casOd), date('i', $casOd) ); // prevod pres date kvuli letni/zimni
//                $dateTo ->setTime( date('G', $casDo), date('i', $casDo) );   // prevod pres date kvuli letni/zimni
                $dateFrom ->setTime( floor($casOd / 3600), floor(($casOd % 3600) / 60), $casOd %60 );
                $dateTo ->setTime( floor($casDo / 3600), floor(($casDo % 3600) / 60), $casDo %60 );
                break;

            default :
                $this ->wholeDay = true;
                break;
        }

        $this ->dateTimeFrom = $dateFrom ->getTimestamp();
        $this ->dateTimeTo = $dateTo ->getTimestamp();
    }


    /**
     * @param Database $database
     * @param string $kodOperace
     * @return string|null
     */
    public static function loadBusyStatus($database, $kodOperace)
    {
        $params = $database ->createParameters();
        $cursor = $database ->getSqlResult(
            'SELECT Ical_BusyStatus FROM OperaceDochazky WHERE Kod = ' . $params ->addParameter($kodOperace),
            $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? $row[0] : null;
    }

    /**
     * @param Database $database
     * @param Absence $absence
     * @return AbsenceCalendarItem[]
     * @throws \Exception
     */
    public static function loadAbsence($database, $absence)
    {
        $cal = new AbsenceCalendarItem();

        $cal ->setTimes($absence ->delka, $absence ->datumOd, $absence ->datumDo, $absence ->casOd, $absence ->casDo);

        $cal ->computeUid($absence ->f_PP ->idPP, $absence ->poradi);
        $cal ->summary = $absence ->popisAbsence . ': ' . $absence ->f_PP ->jmenoPrijmeniKratke;
        $cal ->description = $absence ->poznamka;
        $cal ->cancelled = $absence ->schvaleni == 4  ||  $absence ->schvaleni == 2;
        $cal ->busyStatus = self::loadBusyStatus($database, $absence ->operaceDochazky);

        $cal ->kodOperace = $absence ->operaceDochazky;

        // Pokud je na vice dnu tak vracet vice zaznamu?
        return [$cal];
    }


    /**
     * @param AbsenceDBRecord $absenceRec
     * @return AbsenceCalendarItem[]
     * @throws \Exception
     */
    public static function loadAbsenceRec($absenceRec)
    {
        $cal = new AbsenceCalendarItem();

        $cal ->setTimes($absenceRec ->fDelka ->value, $absenceRec ->fDatumOd ->value, $absenceRec ->fDatumDo ->value, $absenceRec ->fCasOd ->value, $absenceRec ->fCasDo ->value);

        $cal ->computeUid($absenceRec ->pp->idPP, $absenceRec ->fPoradi ->value);
        $cal ->summary = $absenceRec ->popisOperace . ': ' . $absenceRec ->pp->jmenoPrijmeniKratke;
        $cal ->description = $absenceRec ->fPoznamka ->value;
        $cal ->cancelled = $absenceRec ->fSchvaleni ->value == 4  ||  $absenceRec ->fSchvaleni ->value == 2;
        $cal ->busyStatus = self::loadBusyStatus($absenceRec ->database, $absenceRec ->fOperace ->value);

        $cal ->kodOperace = $absenceRec ->fOperace ->value;

        // Pokud je na vice dnu tak vracet vice zaznamu?
        return [$cal];
    }


    /**
     * @param Database $database
     * @param string $filter
     * @param Parameters $params
     * @return AbsenceCalendarItem[]
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    public static function loadEvents($database, $filter, $params)
    {
        $sql =
            'SELECT PracovniPomer.Id, Absence.DatumOd, Absence.DatumDo, ' . //0..2
            '       Absence.Delka, Absence.CasOd, Absence.CasDo, Absence.Poradi, ' . //3..6
            '       Absence.OperaceDochazky, OperaceDochazky.Popis, ' . //7..8
            '       Absence.Schvaleni, ' . $database ->castWideMemo('Absence.Poznamka') . ', ' . //9,10
            '       Osoba.Jmeno, Osoba.Prijmeni, PracovniPomer.OsobniCislo, ' . // 11..13
            '       OperaceDochazky.PritomnostProMonitorovani, OperaceDochazky.Ical_BusyStatus ' . // 14..15
            '  FROM Absence, Osoba, PracovniPomer, OperaceDochazky ' .
            ' WHERE PracovniPomer.RC = Osoba.RC' .
            '   AND Absence.RC = PracovniPomer.RC' .
            '   AND Absence.OsobniCislo = PracovniPomer.OsobniCislo' .
            '   AND Absence.DatumNastupu = PracovniPomer.DatumNastupu' .
            '   AND OperaceDochazky.Kod = Absence.OperaceDochazky' .
            '   AND ' . $filter .
            ' ORDER BY Absence.DatumOd';

        $cursor = $database ->getSqlResult($sql, $params);

        $result = [];

        while ($row = $cursor ->fetch()) {
            $cal = new AbsenceCalendarItem();

            $cal ->kodOperace = $row[7];

            $cal ->setTimes($row[3],
                $database ->stringToTimestamp($row[1]), $database ->stringToTimestamp($row[2]),
                $database ->stringToSecondsOfDay($row[4]), $database ->stringToSecondsOfDay($row[5]));
//
//            $cal->dateTimeFrom = $database ->stringToTimestamp($row[1]); // datumod
//            $cal->dateTimeTo = $database ->stringToTimestamp($row[2]);   // datumdo
//            $cal->wholeDay = true; // prozatim

            $cal ->computeUid($row[0], $row[6]);

            $cal ->summary = $row[8] . ': ' . $row[11] . ' ' . $row[12]; // operace
            $cal ->description = $row[10]; // poznamka
            $cal ->cancelled = $row[9] == 4  ||  $row[9] == 2; // zrusena nebo zamitnuta absence
            if (isset($row[15]) && trim($row[15]) != '') {
                // operace ma primo zadan busy status
                $cal ->busyStatus = $row[15];
            } else if (!(int)$row[14]) {
                // busy status podle pritomnosti pro monitorovani
                $cal ->busyStatus = 'OOF'; // Out Of Office
            }

            $result[] = $cal;
        }

        $cursor ->close();

        return $result;
    }


    /**
     * @param Database $database
     * @param string $login
     * @param bool   $jenSchvalene
     * @return AbsenceCalendarItem[]
     * @throws \Ron\DB\DBException
     */
    public static function loadUserEvents($database, $login, $jenSchvalene)
    {
        $params = $database ->createParameters();

        $filter =
            'Osoba.IntrawebLogin = ' . $params->addParameter($login) .
            '   AND Absence.DatumDo >= ' . $params ->addDateParameter(time()) .  // jen budouci
            ($jenSchvalene
                ? ' AND (Absence.Schvaleni = 1  OR  Absence.Schvaleni = 3  OR  Absence.Schvaleni = 5) ' // schvalene pripadne "zadost o zruseni" a "zamitnuta zadost o zruseni"
                : ' AND Absence.Schvaleni <> 4  AND  Absence.Schvaleni <> 2'); // ignoruje jen zrusene a zamitnute

        return self::loadEvents($database, $filter, $params);
    }


    /**
     * @param Database $database
     * @param bool   $jenSchvalene
     * @param int $kodTypuRozdeleni Nepovinnny filtr na rozdeleni
     * @param string $rozdeleni
     * @return AbsenceCalendarItem[]
     * @throws \Ron\DB\DBException
     */
    public static function loadAllEvents($database, $jenSchvalene, $kodTypuRozdeleni = null, $rozdeleni = null)
    {
        $params = $database ->createParameters();
        $filter =
            'Absence.DatumDo >= ' . $params ->addDateParameter(time()) .  // jen budouci
            ($jenSchvalene
                ? ' AND (Absence.Schvaleni = 1  OR  Absence.Schvaleni = 3  OR  Absence.Schvaleni = 5) ' // schvalene pripadne "zadost o zruseni" a "zamitnuta zadost o zruseni"
                : ' AND Absence.Schvaleni <> 4  AND  Absence.Schvaleni <> 2'); // ignoruje jen zrusene a zamitnute

        if ($kodTypuRozdeleni  &&  $rozdeleni) {
            $filter .=
                ' AND EXISTS(' .
                '  SELECT * FROM RozdeleniOsob, HierarchieRozdeleni ' .
                '  WHERE Absence.RC = RozdeleniOsob.RC' .
                '   AND Absence.OsobniCislo = PracovniPomer.OsobniCislo' .
                '   AND Absence.DatumNastupu = PracovniPomer.DatumNastupu' .
                '   AND RozdeleniOsob.PlatnostOd <= Absence.DatumDo' .
                '   AND (RozdeleniOsob.PlatnostDo >= Absence.DatumOd  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                '   AND HierarchieRozdeleni.typPodrizene = RozdeleniOsob.KodTypuRozdeleni' .
                '   AND HierarchieRozdeleni.kodPodrizene = RozdeleniOsob.Rozdeleni' .
                '   AND HierarchieRozdeleni.typNadrazene = ' . (int)$kodTypuRozdeleni .
                "   AND HierarchieRozdeleni.kodNadrazene = " . $params ->addParameter($rozdeleni) .
                ')';
        }

        return self::loadEvents($database, $filter, $params);
    }


    /**
     * @param Database $database
     * @param int  $idPP
     * @param int  $date
     * @param int  $poradi
     * @return AbsenceCalendarItem[]
     */
    public static function loadEvent($database, $idPP, $date, $poradi)
    {
        $params = $database ->createParameters();

        $filter =
            'PracovniPomer.Id = ' . $params ->addParameter((int)$idPP) .
            ' AND Absence.DatumOd = ' . $params ->addDateParameter($date) .  // jen budouci
            ' AND Absence.Poradi = ' . $params ->addParameter((int)$poradi);

        return self::loadEvents($database, $filter, $params);
    }
}

