<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\StringUtils;
use Ron\TextManager;

class HistoriePruchoduBrowser extends DochazkaBrowser
{

    function __construct($database)
    {
        parent::__construct($database, 'historiepruchodu');

        $this->title = TextManager::_('Historie_pruchodu');
        $this->nacistSvatky = false;

        $this->cssClass = 'browser grid browserdenni historiepruchodu mobile-table-rows';
        $this->code = 'browserhistoriepruchodu';

        $this ->zobrazitFiltrPoznamky = true;
    }


    function nactiData()
    {
        $params = $this ->database ->createParameters();
        $filter = $this ->filterForm ->buildSqlFilter($params);

        $sql =
            'SELECT HistoriePruchodu.DatumCasPruchodu, OperaceDochazky.PopisProDochazku, HistoriePruchodu.Poznamka, HistoriePruchodu.Terminal, Terminal.Popis' .
            ' FROM HistoriePruchodu, OperaceDochazky, Terminal' .
            ' WHERE HistoriePruchodu.KodOperace = OperaceDochazky.Kod' .
            ' AND HistoriePruchodu.Terminal = Terminal.Kod' .
            ($filter ? ' AND ' . $filter : '') .
            ' AND HistoriePruchodu.RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            ' AND HistoriePruchodu.OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            ' AND HistoriePruchodu.DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')) .
            ' AND HistoriePruchodu.DatumCasPruchodu >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('obdobi')) .
            ' AND HistoriePruchodu.DatumCasPruchodu <= ' . $params ->addTimestampParameter(DateUtils::endOfTheDay($this ->filterForm ->getFilterValue('obdobi', 'datumDo'))) .
            ' ORDER BY HistoriePruchodu.DatumCasPruchodu';

        return $this ->database ->getSqlResult($sql, $params);
    }


    function vratTextKZaznamu($zaznam, $posledni)
    {
        $result =
            date('H:i', $this ->database ->stringToTimestamp($zaznam[0])) . ' &nbsp ' . StringUtils::htmlTextConvert($zaznam[1]);

        $result .=
            ' <DIV class="terminaldochazky" title="' .
            TextManager::_('Terminal') . ' ' . StringUtils::htmlTextConvert($zaznam[3]) . ' (' . StringUtils::htmlTextConvert($zaznam[4]) . ')">(' .
            StringUtils::htmlTextConvert($zaznam[3]) .
            ')</DIV>';

        if ($zaznam[2]  &&  $this ->filterForm ->getFilterValue('zobrazitpoznamky')) {
            $result .= '<DIV class="poznamka">' . StringUtils::htmlTextConvert($zaznam[2]) . '</DIV>';
        }

        return $result;
    }

}
