<?php
/**
 * stanislav.elbl, 27.02.2021
 */

namespace RonPortal\Dochazka;

use Exception;
use Ron\Controls\BrowserField;
use Ron\Controls\BrowserRowArray;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatum;
use RonPortal\Controls\FotoOsobyBrowserField;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Controls\RonPortalObjectConfiguration;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniOsob;
use RonPortal\RozdeleniUtils;
use RonPortal\User;


class FilterAktualniStav extends FilterFormFilter
{
    public function render()
    {
        parent::render();

        if (!$this ->value) {
            echo
                '<span class="margin-left-2 setValue" data-value="1" data-selector="#filtr_checkbox_aktualnistav" data-submit="1">' .
                "<a href='#'>" . TextManager::_('Aktualni_stav') . "</a>" .
                '</span>';
        }
    }
}


class MonitorovaniFilterForm extends RonPortalFilterForm
{
    const SETTINGS_JEN_HLAVNI_ROZDELENI = 'monitorovani_jennejvyssirozdeleni';

    /** @var FilterFormFilter */
    private $filtrDatum;

//    public $aktualniStav;
//    public $datumCas;

    public function __construct($database, $action, $kontrolovatprava)
    {
        parent::__construct($database, $action, true);

//        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) {
//            $jenVlastni = RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_JEN_VLASTNI);
//            if (!User::getRC()) {
//                $jenVlastni = 0; // pro monitorovani bez prihlaseni
//            }
//
//            $filtr = RonPortalSettings::getIntValue(self::SETTINGS_JEN_HLAVNI_ROZDELENI) ? 'Rodic IS NULL' : '';
//            $this ->nastavRozdeleni(true, 0, $jenVlastni, $filtr); // typRozdeleni=0, jinak by se musel zadavat i v RonPortalSession::rozdeleni()!
//
//            if ($kontrolovatprava) {
//                $this->f_pravarozdeleni = Prava::vratPrava('monitorovani');
//            }
//        }

        // Informace o poslednim nacteni dat z terminalu
        $this ->addFilter(
            ($filterNacteniInfo = new FilterFormFilter('posledninacteni', FilterFormFilter::TYPE_TEXT))
                ->setPosition(FilterFormFilter::POSITION_ABOVE_FORM)
                ->setRowId('monitorovani_posledninacteni')
                ->setCaption(false)
        );

        $sql = 'SELECT MAX(CasPoslednihoNacteni) FROM StavTerminalu';
        try {
            $cursor = $this->database->select($sql);

            $row = isset($cursor) ? $cursor ->fetch() : null;

            if ($row) {
                $filterNacteniInfo
                    ->setCaption(TextManager::_('Cas_posledniho_nacteni'))
                    ->setAddClass('font-weight-bold')
                    ->setDefaultValue(DBUtils::getFormatedValue($this ->database, 'datetime', $row[0]));
            } else {
                $filterNacteniInfo ->enabled = false;
            }
        } catch (Exception $exception) {
            $filterNacteniInfo
                ->setRowClass('errortext')
                ->setDefaultValue('Chyba pri nacitani casu posledniho nacteni. ' . $exception->getMessage());
        }

        // Co monitorovat
        $this ->addFilter(
            (new FilterFormFilter('monitorovatco', FilterFormFilter::TYPE_LINKS))
                ->addItem(0, TextManager::_('Pritomni'))
                ->addItem(1, TextManager::_('Nepritomni'))
                ->addItem(2, TextManager::_('Vsichni'))
                ->setDefaultValue(0)
                ->setSessionName('comonitorovat')
                ->setPosition(FilterFormFilter::POSITION_ABOVE_FORM)
                ->setCaption(false)
        );



        $rozdeleniFilter = $this->addFilterRozdeleni1(User::getRC() ? null : 0, $kontrolovatprava ? Prava::vratPrava('monitorovani') : null);
        if ($rozdeleniFilter  &&  RonPortalSettings::getIntValue(self::SETTINGS_JEN_HLAVNI_ROZDELENI)) {
            $rozdeleniFilter->setOptionsSqlFilter('Rodic IS NULL');
        }

        $this ->addFilter(
            (new FilterAktualniStav('aktualnistav', FilterFormFilter::TYPE_CHECKBOX))
                ->setCaption(TextManager::_('Aktualni_stav'))
                ->setSessionName(null)  // pokud neni zadano v URL, tak se vzdy zobrazi aktualni stav
                ->setDefaultValue(1),
            true);


        $this ->filtrDatum = (new FilterFormFilterDatum('datum', 'cas'))
                ->setSessionName(null)  // pokud neni zadano v URL, tak se zobrazi aktualni stav
                ->setCaption(TextManager::_('Datum_a_cas'));
        $this ->addFilter($this ->filtrDatum,true);

        $this ->addFilter(
            (new FilterFormFilter('terminal', FilterFormFilter::TYPE_COMBOBOXES))
                ->setSessionName('monitorovat_terminal')
                ->setCaption(TextManager::_('Terminal'))
                ->setOptionsSql('SELECT Kod, Popis FROM Terminal WHERE %FILTER% ORDER BY %ORDER%', 'Aktivni = 1', 'Popis')
                ->setOptionsSqlOrder2('Kod')
                ->addItem('', TextManager::_('_vse_'))
                ->addItem2('', TextManager::_('_vse_'))
                ->setHtmlAttributes(['id' => 'monitorovani_filtr_terminal', 'data-ron-select2' => '#monitorovani_filtr_kod_terminalu', 'data-ron-autosubmit' => '1'])
                ->setHtmlAttributes2(['id' => 'monitorovani_filtr_kod_terminalu', 'data-ron-select2' => '#monitorovani_filtr_terminal', 'data-ron-autosubmit' => '1'])
                ->setSqlFilter('Terminal.Kod = ')
        );

        $this ->addFilter(
            (new FilterFormFilter('operace', FilterFormFilter::TYPE_COMBOBOXES))
                ->setSessionName('monitorovat_operaci')
                ->setCaption(TextManager::_('Operace'))
                ->setOptionsSql('SELECT Kod, Popis FROM OperaceDochazky WHERE %FILTER% ORDER BY %ORDER%', 'Systemova <> 1', 'Popis')
                ->setOptionsSqlOrder2('Kod')
                ->addItem('', TextManager::_('_vse_'))
                ->addItem2('', TextManager::_('_vse_'))
                ->setHtmlAttributes(['id' => 'monitorovani_filtr_operace', 'data-ron-select2' => '#monitorovani_filtr_kod_operace', 'data-ron-autosubmit' => '1'])
                ->setHtmlAttributes2(['id' => 'monitorovani_filtr_kod_operace', 'data-ron-select2' => '#monitorovani_filtr_operace', 'data-ron-autosubmit' => '1'])
                ->setSqlFilter('OperaceDochazky.Kod = ')
        );
    }

    function loadData()
    {
        parent::loadData();

        if ($this ->getFilterValue('aktualnistav')) {
            $this ->filtrDatum ->setValue(time());
        }
    }
}


/**
 * Umoznuje nastavit samostatne pravo pro zobrazeni hodnoty ve sloupci (tj. nezobrazi hodnotu pro zamestnance, na ktereho uzivatel nema pravo)
 */
class MonitorovaniBrowserField extends BrowserField
{
    /** @var PravaNaOsoby */
    protected $pravo = null;

    function loadConfigurationItem($key, $value)
    {
        if ($key === 'pravo') {
            $this ->pravo = $value
                ? Prava::vratPrava($value)
                : null;

            return $this;
        }

        return parent::loadConfigurationItem($key, $value);
    }

    function isValueVisible($value, $rowData)
    {
        if ($this ->pravo  &&  !$this ->pravo ->lzeZobrazitPP((int)$rowData['col-pp-id'])) {
            return false;
        }

        return parent::isValueVisible($value, $rowData);
    }
}


/**
 * Pokud se jedna o absenci, tak zobrazi i DatumDo
 */
class DatumCasMonitorovaniField extends MonitorovaniBrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-absence-datumdo']) {
            // jedna se o absenci
            return $rowData['@col-absence-datumod'] . $this ->sqlFieldIndex2Separator . $rowData['@col-absence-datumdo'];
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}


class MonitorovaniButton_BrowserField extends BrowserField
{
    private $osobaIDnumber;
    private $nastaveniSloupce;

    /**
     * @param int $osobaIDnumber
     * @param array|null $nastaveniSloupce
     */
    public function __construct($osobaIDnumber, $nastaveniSloupce)
    {
        parent::__construct();
        $this->osobaIDnumber = $osobaIDnumber;
        $this->nastaveniSloupce = $nastaveniSloupce;

        if (!$nastaveniSloupce['buttonid']) {
            MessageManager::addErrorMessage("Sloupec typu {$nastaveniSloupce['typ']} musi mit nastaven atribut buttonid!");
        }

        Prava::nastavObdobiProPrava(DateUtils::today());
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $osoba = $rowData[$this->osobaIDnumber];
        $buttonID = $this->nastaveniSloupce['buttonid'];
        $action = $this->nastaveniSloupce['typ'];
        $nadpis = $this->nastaveniSloupce['nadpis'];

        $prava = null;

        if ($this->nastaveniSloupce['typ'] === 'vlozeniabsence') {
            $prava = Prava::vratPrava('absence', 'absence_' . $this->nastaveniSloupce['operace']);
        }
        if ($this->nastaveniSloupce['typ'] === 'vlozenidochazky') {
            $prava = Prava::vratPrava('vkladani');
        }
        if ($this->nastaveniSloupce['typprava']) {
            $prava = Prava::vratPrava($this->nastaveniSloupce['typprava']);
        }

        if ($prava && !$prava ->lzeZobrazitPP($osoba)) {
            return '';
        }

        return
            "<form class='submitWorkingInfo sendPosition' method='POST' action='?menuid=monitorovani'>
                <input type='hidden' name='action' value='$action' />
                <input type='hidden' name='buttonid' value='$buttonID' />
                <input type='hidden' name='osoba' value='$osoba' />
                <input type='hidden' name='x' />
                <input type='hidden' name='y' />
                <input type='submit' class='submitbutton' value='$nadpis' />
            </form>";
    }
}

/**
 * Pole zobrazuje ikony rozdeleni prirazenych osobe
 */
class RozdeleniIMG_BrowserField extends BrowserField
{
    public $typRozdeleni;

    public function __construct($database, $kodTypuRozdeleni)
    {
        parent::__construct($database);

        $this->typRozdeleni = $kodTypuRozdeleni;
    }

    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $idPP = $rowData[3];

        $ikony = RozdeleniOsob::nactiIkonyRozdeleni($this->database, $idPP, $this->typRozdeleni, time(), time());

        return RozdeleniOsob::getIkonyHtml($ikony);
    }
}


class MonitorovaniPocetDleOperace
{
    /** @var string kod operace */
    public $kod;
    /** @var string popis operace */
    public $popis;
    /** @var int */
    public $pocetPritomno;
    /** @var int */
    public $pocetNepritomno;
    /** @var int */
    public $pocetCelkem;

    /**
     * @param MonitorovaniPocetDleOperace[] $pole
     */
    public static function seradOperace(&$pole)
    {
        uasort($pole, 'RonPortal\Dochazka\MonitorovaniPocetDleOperace::porovnejOperace');
    }

    /**
     * @param MonitorovaniPocetDleOperace $operace1
     * @param MonitorovaniPocetDleOperace $operace2
     * @return int
     */
    public static function porovnejOperace($operace1, $operace2)
    {
        return strnatcasecmp($operace1->popis ?? '', $operace2->popis ?? '');
    }
}


/**
 * Class MonitorovaniOsobBrowser
 */
class MonitorovaniOsobBrowser extends RonPortalBrowser
{
    const SETTINGS_SLOUPCE = 'sloupcemonitorovani';
    const SETTINGS_TRIDIT = 'triditmonitorovani';
    const SETTINGS_POUZIT_RYCHLE = 'pouzitrychlemonitorovani';

    var $zobrazovaneSloupce;
    var $editovat;
    //var $aktualniStav;

    var $kontrolovatPrava;
    /** @var  boolean nastavi se na true, pokud se ma zobrazovat sloupec s rozdelenim osoby */
    var $zobrazitRozdeleni;

    /** @var int[]  Seznam kodu typu rozdeleni, ktere se maji pripojit k SQL dotazu - tj. pripoji zaznam Rozdeleni prirazene osobe na danem radku */
    private $pripojitTypyRozdeleni = [];

    var $indexPoleProTrideni;

    private $refreshTimeout = 180;

    /** @var \ArrayAccess   Konfigurace se nacita jeste pred vytvorenim filtrovaciho formulare, takze si zapamatuje konfiguraci filterformu a pouzije ji dodatecne */
    private $filterFormConfiguration = null;

    //    /** @var MonitorovaniSloupecRozdeleni[] seznam pouzitych kodu typu rozdeleni */
//    private $kodyTypuRozdeleni = [];

    public $nastaveniSloupcuXML;

    public function __construct($database, $action)
    {
        parent::__construct($database, $action);

        $this->title = TextManager::_('Monitorovani');

        $this->kontrolovatPrava = $action !== 'monitorovaninolog';
        $this ->cssClass .= ' monitorovani oddEvenRows';

        // Nastavi pole $this->f_sql_fields, $this->f_fieldsformat a $this->f_header
        // podle nastaveni ulozeneho v databazi
        $this->indexPoleProTrideni = -1;

        $this ->sqlSelectPart =
            'PracovniPomer.RC AS RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, PracovniPomer.Id,' . // 0..3
            ' PracovniPomer.TypPracovnihoPomeru, PracovniPomer.DatumUkonceni,' . // 4,5
            ' Osoba.Jmeno, Osoba.Prijmeni, Osoba.Titul,' . //6..8
            " PracovniPomer.Zamestnanec,\n" . // 9
            ' Monitorovani.Pritomnost, Monitorovani.DatumCas, Monitorovani.DatumDo, ' . // 10..12
            ' Monitorovani.KodOperace, Monitorovani.Terminal, Monitorovani.Poznamka, Monitorovani.KodTypuRozdeleni, Monitorovani.Rozdeleni,' . // 13..17
            ' OperaceDochazky.Popis' . // 18
            "\n";

        $this->nactiNastaveni();

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Monitorovani.DatumCas', 11)
            ->setCode('col-absence-datumod') ->setFormat('date');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Monitorovani.DatumDo', 12)
            ->setCode('col-absence-datumdo') ->setFormat('date');

        $this->sestavHlavicku();
    }


    public function addField($field = null)
    {
        if (!$field instanceof BrowserField) {
            $newField = new MonitorovaniBrowserField($this ->database);
            $newField ->setType($field ?: BrowserField::FIELD_TYPE_DEFAULT);
            $field = $newField;
        }

        return parent::addField($field);
    }


    public function loadConfigurationItem($key, $value, $valueText)
    {
        switch ($key) {
            case 'refreshTimeout' :
                $this ->refreshTimeout = (int)$value;
                break;

            case 'column' :
                // uz je nacteno v metode nactiNastaveni()
                break;

            case 'filterform' :
                $this ->filterFormConfiguration = $value;
                break;

            default :
                parent::loadConfigurationItem($key, $value, $valueText);
        }
    }


    /**
     * @throws DBException
     * @throws Exception
     */
    protected function nactiNastaveni()
    {
        $text = RonPortalSettings::getBvarSettings($this->database, 'RonPortal_monitorovani_nastaveni_xml');

        if ($text === '') {
            $str = RonPortalSettings::getValue(self::SETTINGS_SLOUPCE);
            $header = [];
            StringUtils::parseStringToArrays($str, $header, $sloupce, true);

            foreach ($sloupce as $i => $sloupec) {
                $sloupecNastaveni = [];
                $sloupecNastaveni['typ'] = $sloupec;
                $sloupecNastaveni['nadpis'] = $header[$i];

                $rozdeleni = explode('_', $sloupec);

                if (($rozdeleni[0] === 'r') || ($rozdeleni[0] === 'rkod') || ($rozdeleni[0] === 'rvedoucipopis') || ($rozdeleni[0] === 'rvedouci') || ($rozdeleni[0] === 'rzastupce')) {
                    $sloupecNastaveni['typrozdeleni'] = $rozdeleni[1];

                    switch ($rozdeleni[0]) {
                        case 'r':
                            $sloupecNastaveni['typ'] = 'rozdp';
                            break;
                        case 'rkod':
                            $sloupecNastaveni['typ'] = 'rozd';
                            break;
                        case 'rvedoucipopis':
                            $sloupecNastaveni['typ'] = 'rozdvedoucipopis';
                            break;
                        case 'rvedouci':
                            $sloupecNastaveni['typ'] = 'rozdvedouci';
                            break;
                        case 'rzastupce':
                            $sloupecNastaveni['typ'] = 'rozdzastupce';
                            break;
                    }
                }

                if (strpos($sloupec, 'uzivatelska') !== false) {
                    $sloupecNastaveni['typ'] = 'uzivatelska';
                    $sloupecNastaveni['index'] = substr($sloupec, -1);
                }

                $this->zobrazovaneSloupce[] = $sloupecNastaveni;
            }

            return;
        }

        $loader = new XmlLoader();
        $loader->includeAttributes = true;
        $loader->prefixToSubarray = true;
        $loader->ignorePrefixes = true;
        $loader->convertFromUtf8 = true;
        $this->nastaveniSloupcuXML = $loader->readXml($text);

        $this->zobrazovaneSloupce =
            $this->nastaveniSloupcuXML['sloupecmonitorovani'] ??
            $this ->nastaveniSloupcuXML['column'] ??
            [];

        $this ->objectConfiguration = new RonPortalObjectConfiguration($this ->database, $this);
        $this ->objectConfiguration ->loadConfigurationArray($this ->nastaveniSloupcuXML);
    }


    public function setFilterForm()
    {
        $this->filterForm = new MonitorovaniFilterForm($this->database, $this ->action, $this->kontrolovatPrava);

        if ($this ->filterFormConfiguration  instanceof \ArrayAccess) {
            foreach ($this ->filterFormConfiguration as $item) {
                $this ->filterForm ->loadConfigurationArray($item);
            }
        }

    }


    /**
     * Sestavi hlavicku a seznam poli podle nastaveni
     */
    protected function sestavHlavicku()
    {
        $this ->addField(BrowserField::FIELD_TYPE_LINE);

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PracovniPomer.Id', 3)
            ->setCode('col-pp-id');

        $this ->zobrazitRozdeleni = false;
        $jePoleProTrideni = false;
        $poleProTrideni = strtolower(RonPortalSettings::getValue(self::SETTINGS_TRIDIT) ?? '');

        foreach ($this->zobrazovaneSloupce as $i => $sloupec) {
            $newField = null;

            $zobrazitSloupec = true;
            if (Prava::existujePravo('monitorovani_' . $sloupec['typ']) ) {
                $zobrazitSloupec = Prava::jsouNejakePrava('monitorovani_' . $sloupec['typ']);
            }
            if (isset($sloupec['typprava']) && $sloupec['typprava']) {
                $zobrazitSloupec = $zobrazitSloupec && Prava::jsouNejakePrava($sloupec['typprava']);
            }

            if ($zobrazitSloupec) {
                switch (strtolower($sloupec['typ'] ?? '')) {
                    case 'rc' :
                        $newField = $this->addField()
                            ->setSqlFieldName('PracovniPomer.RC', 0)
                            ->setCode('col-rc');
                        break;

                    case 'osobni' :
                        $newField = $this->addField()
                            ->setSqlFieldName('PracovniPomer.OsobniCislo', 1)
                            ->setCode('col-osobnicislo')
                            ->setHeaderText('Osobni_cislo') ->setAlign('right');
                        break;
                    case 'datumnastupu' :
                        $newField = $this->addField()
                            ->setSqlFieldName('PracovniPomer.DatumNastupu', 2)
                            ->setCode('col-datumnastupu')
                            ->setFormat('date') ->setAlign('center');
                        break;

                    case 'cislosmlouvy' :
                        $newField = $this->addField()
                            ->setSqlFieldName('PracovniPomer.CisloPracovniSmlouvy')
                            ->setCode('col-cislopracovnismlouvy') ->setAlign('right');
                        break;

                    case 'kodtypupomeru' :
                        $newField = $this->addField()
                            ->setSqlFieldName('PracovniPomer.TypPracovnihoPomeru')
                            ->setCode('col-typpracovnihopomeru');
                        break;

                    case 'datumukonceni' :
                        $newField = $this->addField()
                            ->setSqlFieldName('PracovniPomer.DatumUkonceni')
                            ->setCode('col-datumukonceni')
                            ->setFormat('date') ->setAlign('center');
                        break;

                    case 'jmeno' :
                        $newField = $this->addField()
                            ->setSqlFieldName('Osoba.Jmeno', 6)
                            ->setCode('col-jmeno') ->setHeaderText('Jmeno');
                        break;
                    case 'prijmeni' :
                        $newField = $this->addField()
                            ->setSqlFieldName('Osoba.Prijmeni', 7)
                            ->setCode('col-prijmeni') ->setHeaderText('Prijmeni');
                        break;
                    case 'celejmeno' :
                        $newField = $this->addField(new JmenoPrijmeniBrowserField($this ->database, 6, 7, 8))
                            //->setSqlFieldName('Osoba.Prijmeni, Osoba.Jmeno')
                            ->setCode('col-jmenoprijmeni')
                            ->setHeaderText('Jmeno_a_prijmeni');
                        break;
                    case 'titul' :
                        $newField = $this->addField()
                            ->setSqlFieldName('Osoba.Titul', 8)
                            ->setCode('col-titul');
                        break;
                    case 'telefon' :
                        $newField = $this->addField()
                            ->setSqlFieldName('Osoba.Telefon')
                            ->setCode('col-telefon');
                        break;
                    case 'mobil' :
                        $newField = $this->addField()
                            ->setSqlFieldName('Osoba.Mobil')
                            ->setCode('col-mobil');
                        break;
                    case 'zamestnanec' :
                        $newField = $this->addField()
                            ->setSqlFieldName('PracovniPomer.Zamestnanec', 9)
                            ->setCode('col-zamestnanec');
                        break;
                    case 'casoperace' :
                        $newField = $this->addField(new DatumCasMonitorovaniField($this ->database))
                            ->setSqlFieldName('Monitorovani.DatumCas', 11) ->setFormat('datetime')
                            ->setCode('col-casoperace')
                            ->setHeaderText('Cas_operace') ->setAlign('right')
                            ->setSqlFieldIndex2Separator(' - '); // pro zobrazeni absence
                        break;
                    case 'terminal' :
                        $newField = $this->addField()
                            ->setSqlFieldName('Monitorovani.Terminal', 14) ->setHeaderText('Terminal')
                            ->setCode('col-terminal')
                            ->setAlign('right');
                        break;
                    case 'operace' :
                        $newField = $this->addField()
                            ->setSqlFieldName('OperaceDochazky.Popis', 18) ->setHeaderText('Operace')
                            ->setCode('col-operace-popis');
                        break;
                    case 'pritomnost' :
                        $newField = $this->addField()
                            ->setSqlFieldName('Monitorovani.Pritomnost', 10) ->setHeaderText('Pritomnost')
                            ->setCode('col-pritomnost')
                            ->setFormat('anone') ->setAlign('center');
                        break;
                    case 'umisteni' :
                        $newField = $this->addField()
                            ->setSqlFieldName('Umisteni.Popis') ->setHeaderText('Umisteni')
                            ->setCode('col-umisteni-popis');
                        break;
                    case 'poznamka' :
                        $newField = $this->addField()
                            ->setSqlFieldName('Monitorovani.Poznamka', 15) ->setHeaderText('Poznamka')
                            ->setCode('col-poznamka');
                        break;
                    case 'rozdeleni' : // popis rozdeleni prirazeneho zaznamu dochazky/absence
                        $newField = $this->addField()
                            ->setSqlFieldName('Rozdeleni.Popis')
                            ->setCode('col-rozdeleni-popis');
                        break;
                    case 'kodrozdeleni' :
                        $newField = $this->addField() // kod rozdeleni prirazeneho zaznamu dochazky/absence
                            ->setSqlFieldName('Rozdeleni.Kod')
                            ->setCode('col-rozdeleni');
                        break;

                    case 'uzivatelska' :
                        $newField = $this->addField()
                            ->setSqlFieldName('Osoba.Uzivatelska' . (int)$sloupec['index'])
                            ->setCode("col-uzivatelska{$sloupec['index']}");
                        if ($poleProTrideni === 'uzivatelska' . (int)$sloupec['index']) {
                            $jePoleProTrideni = true;
                        }
                        break;

                    case 'uzivatelskapp' :
                        $newField = $this->addField()
                            ->setSqlFieldName('PracovniPomer.Uzivatelska' . (int)$sloupec['index'])
                            ->setCode("col-pp-uzivatelska{$sloupec['index']}");
                        break;

                    case 'kodrozdeleniosoby' :
                        $newField = $this->addField()
                            ->setSqlFieldName('RozdeleniOsoby.Kod')
                            ->setCode('col-rozdeleni-kod');
                        $this->zobrazitRozdeleni = true;
                        break;
                    case 'rozdeleniosoby' :
                        $newField = $this->addField()
                            ->setSqlFieldName('RozdeleniOsoby.Popis')
                            ->setCode('col-rozdeleni-popis');
                        $this->zobrazitRozdeleni = true;
                        break;

                    case 'foto' :
                        //$newField = $this->addField(new Monitorovani_FotoField($this->action))
                        $newField = $this->addField(new FotoOsobyBrowserField('col-pp-id', '?menuid=' . $this->action . '&fotopp=', false))
                            ->setSqlFieldName('Osoba.IdFotografie')
                            ->setCode('col-foto');
                        break;

                    case 'fotoimg' :
                        //$newField = $this->addField(new Monitorovani_FotoField($this->action, true))
                        $newField = $this->addField(new FotoOsobyBrowserField('col-pp-id', '?menuid=' . $this->action . '&fotopp=', true))
                            ->setSqlFieldName('Osoba.IdFotografie')
                            ->setCode('col-inlinefoto');
                        break;

                    case 'email' :
                        $newField = $this->addField(BrowserField::FIELD_TYPE_EMAIL)
                            ->setSqlFieldName('Osoba.Email')
                            ->setCode('col-email');
                        break;

                    case 'rozdimg':
                        $newField = $this->addField(new RozdeleniIMG_BrowserField($this->database, $sloupec['typrozdeleni']))
                            ->setCode('col-rozdeleni-ikony');
                        break;

                    case 'rozdp':
                    case 'rozd':
                    case 'rozdvedoucipopis':
                    case 'rozdvedouci':
                    case 'rozdzastupce':
                        $newField =  $this ->addFieldRozdeleni($sloupec);
//                        $newField = $this->addField() ->setSqlFieldName('18');
//                        $this->kodyTypuRozdeleni[$i] = new MonitorovaniSloupecRozdeleni($sloupec['typrozdeleni'], $sloupec['typ']);
                        break;

                    case 'vlozeniabsence':
                        if (empty($sloupec['typprava'])) {
                            $prava = Prava::vratPrava('absence', 'absence_' . $sloupec['operace']);
                            $zobrazitSloupec = Prava::jsouNejakePrava($prava);
                        }
                        if ($zobrazitSloupec) {
                            $newField = $this->addField(new MonitorovaniButton_BrowserField(3, $sloupec));
                        }
                        break;
                    case 'vlozenidochazky':
                        if (empty($sloupec['typprava'])) {
                            $zobrazitSloupec = Prava::jsouNejakePrava('vkladani');
                        }
                        if ($zobrazitSloupec) {
                            $newField = $this->addField(new MonitorovaniButton_BrowserField(3, $sloupec));
                        }
                        break;

                    case '' :
                        $newField = $this ->addField(BrowserField::FIELD_TYPE_DEFAULT);
                        break;

                    default :
                        $newField = $this->addField(BrowserField::FIELD_TYPE_TEXT) ->setText($sloupec['typ']);
                }
            }

            if ($newField) {
                if (isset($sloupec['nadpis'])) {
                    $newField->header = TextManager::_($sloupec['nadpis']);
                }

                if (isset($sloupec['class'])) {
                    $newField->className = $sloupec['class'];
                }

                $newField ->loadConfigurationArray($sloupec);
            }
        }

        if (!$jePoleProTrideni  &&  $poleProTrideni  &&  substr($poleProTrideni, 0, 11) === 'uzivatelska') {
            // prida pole pro trideni, aby bylo v sql dotazu
            $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName("Osoba.$poleProTrideni" )
                ->setCode('col-' . $poleProTrideni);
        }
    }


    private function addFieldRozdeleni($sloupec)
    {
        $kodTypuRozdeleni = (int)$sloupec['typrozdeleni'];

        // Zajisti pripojeni tabulky Rozdeleni k SQL dotazu - s aliasem Rozdeleni<kodTypu> (napr. Rozdeleni1 pro typ rozdeleni 1)
        $this ->pripojitTypyRozdeleni[$kodTypuRozdeleni] = $kodTypuRozdeleni;

        $kodStr = (($kodTypuRozdeleni < 0) ? '_' : '') . abs($kodTypuRozdeleni);
        switch ($sloupec['typ']) {
            case 'rozdp':
                return ($this ->addField(new MonitorovaniBrowserField($this ->database)))
                    ->setSqlFieldName("Rozdeleni{$kodStr}.Popis");

            case 'rozd':
                return ($this ->addField(new MonitorovaniBrowserField($this ->database)))
                    ->setSqlFieldName("Rozdeleni{$kodStr}.Kod");

            case 'rozdvedoucipopis':
                return ($this ->addField(new MonitorovaniBrowserField($this ->database)))
                    ->setSqlFieldName("Rozdeleni{$kodStr}.Vedouci");

            case 'rozdvedouci':
                return $this ->addField(new MonitorovaniBrowserField($this ->database))
                    ->setSqlFieldName("(SELECT Osoba.Prijmeni {fn_concat()} ' ' {fn_concat()} COALESCE(Osoba.Jmeno,'') FROM Osoba WHERE Osoba.RC = Rozdeleni{$kodStr}.VedouciRC)");
            case 'rozdzastupce':
                return $this ->addField(new MonitorovaniBrowserField($this ->database))
                    ->setSqlFieldName("(SELECT Osoba.Prijmeni {fn_concat()} ' ' {fn_concat()} COALESCE(Osoba.Jmeno,'') FROM Osoba WHERE Osoba.RC = Rozdeleni{$kodStr}.ZastupceRC)");
        }

        return null;
    }


    protected function vratPoleProTrideni()
    {
        $this->indexPoleProTrideni = -1;

        switch (RonPortalSettings::getValue(self::SETTINGS_TRIDIT)) {
            case "kodtypupomeru" :
                $this->indexPoleProTrideni = 4;
                return "PracovniPomer.TypPracovnihoPomeru";

            case "titul"        :
                $this->indexPoleProTrideni = 8;
                return "osoba.titul";

            case "zamestnanec"  :
                $this->indexPoleProTrideni = 9;
                return "PracovniPomer.Zamestnanec";

            case "uzivatelska1" :
            case "uzivatelska2" :
            case "uzivatelska3" :
            case "uzivatelska4" :
            case "uzivatelska5" :
            case "uzivatelska6" :
            case "uzivatelska7" :
            case "uzivatelska8" :
            case "uzivatelska9" :
                $this ->indexPoleProTrideni = 'col-' . RonPortalSettings::getValue(self::SETTINGS_TRIDIT);
                return 'Osoba.' . RonPortalSettings::getValue(self::SETTINGS_TRIDIT);
        }

        return '';
    }


    private function castChar($sqlText, $size, $memo) {
        if ($this ->database ->getDbType() === 'oracle') {
            return "CAST($sqlText AS NVARCHAR2($size))";
        }

        if ($this ->database ->getDbType() === 'interbase') {
            return $memo
                ? "CAST(SUBSTRING($sqlText FROM 1 FOR $size) as varchar($size))"
                : $sqlText;
        }

        return $memo
            ? "CAST($sqlText AS nvarchar($size))"
            : $sqlText;
    }

    function getSQL($sqlFilter = '')
    {
        $datumCas = $this ->filterForm ->getFilterValue('datum');

        $sql = 'SELECT ' . $this ->getSelectSQLPart() . "\n";

        $params = $this ->database ->createParameters();

        if (RonPortalSettings::getIntValue(self::SETTINGS_POUZIT_RYCHLE) && $this->filterForm ->getFilterValue('aktualnistav')) {
            $unionMonitorovani =
                '(SELECT RC, OsobniCislo, DatumNastupu, DatumCasOperace DatumCas, KodOperace, Terminal, ' .
                $this ->castChar('Poznamka', 250, false) . ' Poznamka,' .
                ' Pritomnost, KodTypuRozdeleni, Rozdeleni, NULL DatumDo' .
                '   FROM PuvDochProMonitorovani' .
                '  UNION ALL' .
                '  SELECT RC, OsobniCislo, DatumNastupu, DatumOd DatumCas, OperaceDochazky KodOperace, NULL,' .
                $this ->castChar('Absence.Poznamka', 250, true) . ' Poznamka,' .
                //($this->database->getDbType() === 'interbase' ? ' CAST(SUBSTRING(Absence.Poznamka FROM 1 FOR 250) as varchar(250))' : ' CAST(Absence.Poznamka AS nvarchar(250)) Poznamka') . ',' .
                '    OperaceDochazky.PritomnostProMonitorovani Pritomnost, NULL, NULL, DatumDo' .
                '   FROM Absence, OperaceDochazky' .
                '   WHERE OperaceDochazky.Kod = Absence.OperaceDochazky' .
                '    AND (Absence.DatumOd <= ' . $params->addDateParameter($datumCas) . ')' .
                '    AND (Absence.DatumDo >= ' . $params->addDateParameter($datumCas) . ')' .
                '    AND Absence.Schvaleni <> 2  AND  Absence.Schvaleni <> 4 ' . // nezobrazuje zamitnute a zrusene
                '    AND OperaceDochazky.PritomnostProMonitorovani <> 3' .  // nezobrazuje operace s nastavenim "nemonitorovat"
                ') Monitorovani';
        } else {
            $unionMonitorovani =
                '(SELECT RC, OsobniCislo, DatumNastupu, DatumCasOperace DatumCas, KodOperace, Terminal, Poznamka, Pritomnost, KodTypuRozdeleni, Rozdeleni, NULL DatumDo' .
                '   FROM PuvDochProMonitorovani) Monitorovani';
        }

        $sql .=
            ' FROM' .
            ' PracovniPomer' .
            "\nLEFT OUTER JOIN " . $unionMonitorovani .
            '   ON Monitorovani.RC = PracovniPomer.RC AND Monitorovani.OsobniCislo = PracovniPomer.OsobniCislo AND Monitorovani.DatumNastupu = PracovniPomer.DatumNastupu' .
            "\nINNER JOIN Osoba ON Osoba.RC = PracovniPomer.RC" .
            "\nLEFT OUTER JOIN OperaceDochazky" . // SE 11.10.2022 - prepsano na LEFT OUTER - aby se zobrazovali i zamestnanci bez posledni operace
            '   ON OperaceDochazky.Kod = Monitorovani.KodOperace' .
            "\nLEFT OUTER JOIN Terminal" .
            '   ON Terminal.Kod = Monitorovani.Terminal' .
            "\nLEFT OUTER JOIN Umisteni".
            '   ON Umisteni.Kod = Terminal.KodUmisteni' .
            "\nLEFT OUTER JOIN Rozdeleni" .
            '   ON Rozdeleni.KodTypuRozdeleni = Monitorovani.KodTypuRozdeleni' .
            '   AND Rozdeleni.Kod = Monitorovani.Rozdeleni';

        if ($this ->zobrazitRozdeleni) {
            $sql .=
                "\nLEFT OUTER JOIN RozdeleniOsob" .
                '   ON RozdeleniOsob.RC = PracovniPomer.RC AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter($datumCas) .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter($datumCas) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                '   AND RozdeleniOsob.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                ' LEFT OUTER JOIN Rozdeleni RozdeleniOsoby ' .
                '  ON RozdeleniOsoby.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni AND RozdeleniOsoby.Kod = RozdeleniOsob.Rozdeleni';
        }

        foreach ($this ->pripojitTypyRozdeleni as $kodTypuRozdeleni) {
            $kodStr = (($kodTypuRozdeleni < 0) ? '_' : '') . abs($kodTypuRozdeleni);

            $sql .=
                "\nLEFT OUTER JOIN Rozdeleni Rozdeleni{$kodStr}" .
                "   ON Rozdeleni{$kodStr}.KodTypuRozdeleni = $kodTypuRozdeleni" .
                "   AND Rozdeleni{$kodStr}.Kod = (SELECT MIN(RozdeleniOsob.Rozdeleni) FROM RozdeleniOsob" .
                "     WHERE RozdeleniOsob.KodTypuRozdeleni = $kodTypuRozdeleni" .
                '     AND RozdeleniOsob.RC = PracovniPomer.RC' .
                '     AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
                '     AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
                '     AND RozdeleniOsob.PlatnostOd <= ' . $params->addDateParameter($datumCas) .
                '     AND (RozdeleniOsob.PlatnostDo >= ' . $params->addDateParameter($datumCas) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                ' )';

        }

        $sql .=
            "\nWHERE PracovniPomer.DatumNastupu <= " . $params ->addDateParameter($datumCas) .
            ' AND (PracovniPomer.DatumUkonceni >= ' . $params ->addDateParameter($datumCas) . ' OR PracovniPomer.DatumUkonceni IS NULL) ';

        if (RonPortalSettings::getIntValue('monitorovani_skryt_ext')) {
            $sql .= ' AND PracovniPomer.Zamestnanec <> 0 ';
        }

        if ($this ->kontrolovatPrava) {
            $filtrPrava = Prava::vratPrava('monitorovani')->sqlFiltr('PracovniPomer', time(), time(), $params);
            if ($filtrPrava) {
                $sql .= ' AND ' . $filtrPrava;
            }
        }

        // filtrovaci formular - rozdeleni
        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            if ($rozdeleniFilter ->getValue()) {
                $sqlFilterRozdeleni =
                    RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'PracovniPomer',
                        $rozdeleniFilter ->getTypRozdeleni(), $rozdeleniFilter ->getValue(),
                        DateUtils::startOfTheDay($datumCas), $datumCas, $rozdeleniFilter ->getValue('vcetnePodrizenych'),
                        $params);
                if ($sqlFilterRozdeleni) {
                    $sql .= "\n AND (" . $sqlFilterRozdeleni . ')';
                }
            }
        }

        // filtrovaci formular - ostatni (operace, terminal, uzivatelsky definovane)
        $filterFormSQL = $this ->filterForm ->buildSqlFilter($params);
        if ($filterFormSQL) {
            $sql .= ' AND ' . $filterFormSQL;
        }

        if ($this ->sqlWherePart) {
            // sql filtr - napriklad nakonfigurovany v XML
            $sql .= ' AND ' . $this ->sqlWherePart;
        }

        // Razeni
        $trideni = $this->vratPoleProTrideni();
        $sql .=
            "\nORDER BY " .
            ($trideni ? $trideni . ', ' : '') .  // pokud se ma podle nejakeho pole tridit, tak musi byt na prvnim miste razeni
            'Osoba.Prijmeni, Osoba.Jmeno, RC, DatumCas DESC';

        return new SqlStatement($sql, $params);
    }


    function zobrazPoctyOsob($pritomno, $nepritomno, $poctyDleOperace)
    {
        echo '<div id="monitorovani_pocty">';

        echo '<table class="browser">';

        MonitorovaniPocetDleOperace::seradOperace($poctyDleOperace);

        echo
            '<thead>' .
            '<tr><th>' . TextManager::_('Operace') . '</th><th>' . TextManager::_('Pritomni') . '</th><th>' . TextManager::_('Nepritomni') . '</th><th>' . TextManager::_('Celkem') . '</th></tr>' .
            '</thead>';

        echo '<tbody class="border-bottom">';
        /** @var MonitorovaniPocetDleOperace $prehledOperace */
        foreach ($poctyDleOperace as $index => $prehledOperace) {
            echo '<tr><td>' . StringUtils::htmlTextConvert($prehledOperace->popis) . " </td><td class='align-right'>" . $prehledOperace->pocetPritomno . "</td>
                        <td class='align-right'>" . $prehledOperace->pocetNepritomno . "</td><td class='align-right'>" . $prehledOperace->pocetCelkem . "</td></tr>";
        }
        echo '</tbody>';

        echo
            '<tbody>' .
            '<tr class="browserfooter"><td>' . TextManager::_('Celkem') . "</td><td class='align-right'>$pritomno</td><td class='align-right'>$nepritomno</td><td class='align-right'>" . ($pritomno+$nepritomno) . "</td></tr>" .
            '</tbody>';

        echo '</table>';

        echo "</div>\n";
    }


    function zobrazOddelovac($row)
    {
        $pocet = $this ->visibleFieldsCount(); // count($this->fields) - 1; // presah do dalsich bunek
        echo
            "<tr class='oddelovacmonitorovani border-bottom'><td></td><td colspan='$pocet'>" .
            StringUtils::htmlTextConvert($row[$this->indexPoleProTrideni]) .
            "</td></tr>\n";
    }


//    public function enableLine($values) - netreba - prava se kontroluji uz na urovni SQL dotazu
//    {
//        // Zkontroluje, jestli ma uzivatel pravo monitorovat osobu
//        if (!$this->kontrolovatPrava) {
//            return true;
//        }
//
//        // ? ? ? nastavit nejdrive obdobi pro prava ? A co prava v pripade divani na jiny cas nez aktualni ?
//        return Prava::vratPrava('monitorovani')->lzeZobrazitPP($values['col-pp-id'], time(), time());
//    }


    /**
     * @throws Exception
     */
    function renderContent()
    {
        //$datum = $_SESSION['monitorovatcas'];
        $datum = $this ->filterForm ->getFilterValue('datum');

        $pritomno = 0;
        $nepritomno = 0;

        $poctyDleOperaci = [];

        //$cursor = getSqlResult($this ->database, $sql);

        $this->renderTableStart();
        echo '<tbody>';

        $i = 1;

        // Promenne pro zapamatovani posledni osoby, aby se nezobrazovala vicekrat
        // (v pripade, ze ma nejakou operaci i planovanou absenci)
        $old_rc = '';
        $old_osobni = '';
        $old_datumnastupu = 0;

        $old_trideni = NULL;

//        $prava = Prava::vratPrava('monitorovani');

        $browserRowArray = new BrowserRowArray($this ->fields);

        while ($row = $this ->cursor ->fetch()) {
            $browserRowArray ->setData($row);
            if (!RonPortalSettings::getIntValue(self::SETTINGS_POUZIT_RYCHLE)  ||  !$this->filterForm ->getFilterValue('aktualnistav')) {
                // nepouziva se rychle monitorovani (bud neni povoleno, nebo se nejedna o aktualni stav)
                $pruchod = DochazkaUtils::vratPosledniPruchodSQL($this->database, $datum, $row[0], $row[1], $this ->database ->stringToTimestamp($row[2]));

                $browserRowArray[10] = $pruchod['Pritomnost'] ?? null;
                $browserRowArray[11] = $pruchod['DatumCas'] ?? null;
                $browserRowArray[12] = $pruchod['DatumDo'] ?? null;
                $browserRowArray[13] = $pruchod['Operace'] ?? null; // kod operace
                $browserRowArray[14] = $pruchod['Terminal'] ?? null; // terminal
                $browserRowArray[15] = $pruchod['Poznamka'] ?? null; // poznamka
                $browserRowArray[17] = $pruchod['Rozdeleni'] ?? null; // kod rozdeleni
                $browserRowArray[18] = $pruchod['PopisOperace'] ?? null; // popis operace
                $browserRowArray['col-rozdeleni-popis'] = $pruchod['PopisRozdeleni'] ?? null; // popis rozdeleni
            } else {
                if ($browserRowArray[10]  &&  !$browserRowArray[12]  &&  $this ->database ->stringToTimestamp($browserRowArray[11]) < DateUtils::incDay($datum, -1)) {
                    // zaznam v PuvDochProMonitorovani je starsi nez 1 den => predpoklada se, ze uz neplati
                    $browserRowArray[10] = 0; // zrusim pritomnost
                }
            }

            if ($this ->enableLine($browserRowArray)) {
                if ($old_rc != $row[0] || $old_osobni != $row[1] || $old_datumnastupu != $row[2]) {
                    // Osoba jeste nebyla zobrazena
                    $old_rc = $row[0];
                    $old_osobni = $row[1];
                    $old_datumnastupu = $row[2];

//                    if ( (($this ->filterForm ->getFilterValue('operace') === $browserRowArray[13])  ||  null == $this ->filterForm ->getFilterValue('operace')) && - netreba - filtruje se v SQL
//                         (($this ->filterForm ->getFilterValue('terminal') === $browserRowArray[14])  ||  null === $this ->filterForm ->getFilterValue('terminal'))
//                    ) {
                        $pocetOperace = $poctyDleOperaci[$browserRowArray[13]] ?? new MonitorovaniPocetDleOperace();

                        $pocetOperace->kod = $browserRowArray[13];
                        $pocetOperace->popis = $browserRowArray[18];
                        $pocetOperace->pocetCelkem++;

                        if ($browserRowArray[10]) {
                            $pocetOperace->pocetPritomno++;
                            $pritomno++;
                        } else {
                            $pocetOperace->pocetNepritomno++;
                            $nepritomno++;
                        }

                        $poctyDleOperaci[$browserRowArray[13]] = $pocetOperace;

                        if (($browserRowArray[10] && $this ->filterForm ->getFilterValue('monitorovatco') != 1) ||
                            (!$browserRowArray[10] && $this ->filterForm ->getFilterValue('monitorovatco'))
                        ) {

                            // Trideni podle nektereho pole - nastaveno v aplikaci v nastaveni intrawebu
                            if ($this->indexPoleProTrideni >= 0) {
                                if ($old_trideni != $browserRowArray[$this->indexPoleProTrideni]) {
                                    $old_trideni = $browserRowArray[$this->indexPoleProTrideni];
                                    $this->zobrazOddelovac($browserRowArray);
                                }
                            }

                            $this ->renderLine($browserRowArray, $browserRowArray[10] ? ' monitorovanipritomen' : ' monitorovaninepritomen', $i);
                            $i++;
                        }
//                    }
                }
            }
        }

        echo "</tbody>\n";

        $this->renderTableEnd();

        $this->zobrazPoctyOsob($pritomno, $nepritomno, $poctyDleOperaci);
    }



    function render()
    {
        echo
            '<div id="monitorovani"' .
            (($this ->refreshTimeout  &&  $this->filterForm ->getFilterValue('aktualnistav'))
                ? " class='auto-refresh' data-timeout='{$this ->refreshTimeout}' data-url='?menuid={$this ->action}'"
                : ''
            ) .
            '>';

        parent::render();

        echo "</div>\n";
    }


}
