<?php
/**
 * stanislav.elbl, 02.03.2022 15:10
 */

namespace RonPortal\Dochazka;

use Ron\Controls\ConfigurableObject;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DebugException;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\ViewObject;
use Ron\RonException;
use Ron\TextManager;
use Ron\XmlLoader;
use RonPortal\Controls\FilterFormFilterRok;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Controls\RonPortalObjectConfiguration;
use RonPortal\Kalendar;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use Tracy\Debugger;

class RocniPrehledView extends BaseViewObject implements ConfigurableObject
{
    /** @var Database */
    protected $database;

    /** @var int        Rok, ktery se ma zobrazit */
    protected $rok;

    /** @var PracovniPomer    Pracovniho pomeru, pro ktery se nactou data */
    protected $pp;

    /** @var string     Prefix pro ID elementu v HTML (pro moznost vice prehledu na jedne strance */
    public $prefix = 'rocniprehled_';

    /** @var RonPortalObjectConfiguration */
    protected $configuration;

    /** @var RonPortalFilterForm */
    protected $filterForm;

    /** @var AbsenceFormular */
    protected $absenceDialog;

    public $title;

    protected $htmlParams = [];

    /** @var PravaNaOsoby */
    protected $prava;


    /**
     * RocniPrehledView constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        parent::__construct('ronportal/dochazka/view_rocni_prehled.php');

        $this->database = $database;

        $this ->prava = Prava::vratPrava('absence_prehled');
        //Nastavi pravo na rozdeleni, kde muze nekoho videt. POZOR: nejde nastavit soucet prav, pokud tedy u schvalovani ma pouze jednu osobu, uvidi pouze jeji rozdeleni.
        if (!$this ->prava->jsouNejakePrava()) {
            $this ->prava = Prava::vratPrava('prohlizeni');

            if (!$this ->prava->jsouNejakePrava()) {
                $this ->prava = Prava::vratPrava('absence');
            }
        }

        $this ->title = TextManager::_('Rocni_prehled');

        $this ->filterForm = new RonPortalFilterForm($this ->database, 'rocniprehled', true);
        $this ->filterForm ->addFilter(
            (new FilterFormFilterRok('rok')),
            true
        );

        $this ->filterForm ->addFilterRozdeleni1(null, $this ->prava);
        $this ->filterForm ->addFilterRozdeleni2(null, $this ->prava);

        $this ->filterForm ->addFilterPP(false, $this ->prava);
    }


    public function loadConfigurationItem($key, $value, $valueText)
    {
        if (strpos($key, 'html:') === 0) {
            if (!$this->htmlParams) {
                $this->htmlParams = [];
            }

            $this->htmlParams[substr($key, 5)] = $valueText;
        } else {
            switch ($key) {
                case 'title' :
                    $this ->{$key} = TextManager::_($valueText);
                    break;
            }
        }
    }


    protected function loadConfiguration()
    {
        $this ->configuration = new RonPortalObjectConfiguration($this ->database, $this);
        $this ->configuration ->loadConfiguration('rocniprehled');
    }


    /**
     * Nacte data potrebna pro renderovani
     * - svatky podle kalendare osoby
     * @throws DBException
     * @throws DebugException
     * @throws RonException
     */
    public function loadData()
    {
        $this ->loadConfiguration();

        $this ->filterForm ->loadData();

        $this ->pp = PracovniPomer::nactiPP($this ->database, $this ->filterForm ->getFilterValue('osoba'));
        $this ->rok = $this ->filterForm ->getFilterValue('rok');

        Kalendar::nactiSvatkyOsoby($this ->database, $this ->pp,
            mktime(0, 0, 0, 1, 1, $this ->rok),
            mktime(0, 0, 0, 12, 31, $this ->rok)
        );

        if (!$this ->prava ->lzeZobrazitPP($this ->pp ->idPP)) {
            MessageManager::addErrorMessage(TextManager::_('Nemate_pravo'));
        }

        if (Prava::lzeZobrazitPP('absence', $this ->pp ->idPP, mktime(0, 0, 0, 1, 1, $this ->rok), mktime(0, 0, 0, 12, 31, $this ->rok))) {
            $this ->absenceDialog = new AbsenceFormular($this ->database, true);
            if (!$this ->absenceDialog ->htmlParams) {
                $this ->absenceDialog ->htmlParams = [];
            }
            $this ->absenceDialog ->htmlParams['id'] = 'absence_dialog';
            $this ->absenceDialog ->loadConfiguration();
        }

        $variables = [
            'idPP' => $this ->pp ->idPP,
            'RC' => $this ->pp ->rc,
            'osobniCislo' => $this ->pp ->osobniCislo,
            'datumNastupu' => $this ->pp ->datumNastupu,
            'jmeno' => $this ->pp ->jmeno,
            'prijmeni' => $this ->pp ->prijmeni,
            'jmenoPrijmeni' => $this ->pp ->jmenoPrijmeni,
            'rok' => $this ->rok
        ];

        $this ->configuration ->processXmlForms($variables);
    }


    /**
     * Vraci pocet sloupcu, ktere se maji zobrazit
     * @return int
     */
    protected function getColumnsCount()
    {
        $result = 0;

        for ($month = 1;  $month <= 12;  $month++) {
            // sestavi datum prvniho dne v mesici
            $date = mktime(0, 0, 0, $month, 1, $this ->rok);
            // zjisti, kterym dnem v tydnu mesic zacina (1-7)
            $firstWeekDay = (int)date('w', $date);
            if ($firstWeekDay <= 0) {
                $firstWeekDay += 7; // nedele - nechci 0, ale 7
            }

            // a pricte pocet dnu v mesici
            $cnt = date('t', $date) + $firstWeekDay - 1;

            if ($cnt > $result) {
                $result = $cnt;
            }
        }

        return $result;
    }


//    public function render()
//    {
//        // nadpis
//        echo '<h1>' . TextManager::_('Rocni_prehled') . ' ' . $this ->rok . '</h1>';
//
//        // zjisti pocet sloupcu, ktere je potreba vyrenderovat
//        $columnsCount = $this ->getColumnsCount();
//
//        echo "<div id='rocniPrehled' class='rocniprehled defaultfocus' tabindex='1' data-rok='{$this ->rok}' data-url='?menuid=rocniprehled&action=loaddata&idpp={$this ->pp ->idPP}&rok={$this ->rok}'>";
//
//        // hlavicka s nadpisy dnu
//        echo
//            '<div class="rocniprehled-row rocniprehled-header bordersx background-table-header color-table-header">' .
//            // prazdna bunka (nad nazvy mesicu)
//            '<div class="rocniprehled-mesic padding-all-2">' .
//            '</div>';
//
//        echo
//            '<div class="rocniprehled-data">';
//
//        for ($i = 1;  $i <= $columnsCount;  $i++) {
//            echo
//                '<div class="rocniprehled-dayheader' . ($i > 7 ? ' other-week' : '') . '">' .
//                TextManager::_('Day_short_' . ($i % 7)) .
//                '</div>';
//        }
//        echo
//            '</div>' .
//            "</div>\n";
//
//        $indexDne = 0;
//
//        // radky pro jednotlive mesice
//        for ($month = 1;  $month <= 12;  $month++) {
//            $aktualniMesic = date('Y') == $this ->rok  &&  date('n') == $month;
//            $rowClass = "rocniprehled-row rocniprehled-$month border-bottom border-left" . ($aktualniMesic ? ' aktualni-mesic' : '');
//
//            echo "<div class='$rowClass'>\n";
//
//            // nadpis mesice
//            echo "  <div class='rocniprehled-mesic padding-all-2'>" . TextManager::_("Mesic_$month") . '</div>';
//
//            // nadpisy dnu pro mobilni zobrazeni
//            echo
//                "<div class='rocniprehled-weekheader background-table-header color-table-header'>";
//
//            for ($weekDay = 1;  $weekDay <= 7;  $weekDay++) {
//                echo
//                    '<div class="rocniprehled-dayheader">' .
//                    TextManager::_('Day_short_' . ($weekDay % 7)) .
//                    '</div>';
//            }
//
//            echo
//                "</div>\n";
//
//            // data za mesic
//            echo "  <div class='rocniprehled-data'>";
//
//            // datum prvniho dne v mesici
//            $firstDate = mktime(0, 0, 0, $month, 1, $this ->rok);
//
//            // zjisti, kterym dnem v tydnu mesic zacina (1-7)
//            $firstWeekDay = (int)date('w', $firstDate);
//            if ($firstWeekDay <= 0) {
//                $firstWeekDay += 7; // nedele - nechci 0, ale 7
//            }
//
//            $indexInRow = 0;
//
//            // vyrenderuje prazdne bunky od pondeli do prvniho dne v mesici
//            for ($i = 1;  $i < $firstWeekDay;  $i++) {
//                echo '<div class="rocniprehled-prazdnabunka"></div>';
//                $indexInRow++;
//            }
//
//            $daysCount = (int)date('t', $firstDate);
//
//            // vyrenderuje bunky pro jednotlive dny
//            for ($i = 1;  $i <= $daysCount;  $i++) {
//                $date = mktime(0, 0, 0, $month, $i, $this ->rok);
//
//                //$className = 'rocniprehled-den' . (($i + $firstWeekDay - 2) % 7 < 5 ? '' : ' background-vikend');
//                $className = 'rocniprehled-den ' . Kalendar::getBackgroundStyleForDate($this ->database, $date);
//                echo "<div class='$className' id='den_{$month}_{$i}' data-index='$indexDne' data-index-in-row='$indexInRow' data-day-in-month='$i' data-month='$month'><div class='rocniprehled-datum'>$i</div></div>";
//                $indexDne++;
//                $indexInRow++;
//            }
//
//            // prazdne bunky za koncem mesice
//            for ($i = $daysCount + $firstWeekDay;  $i <= $columnsCount;  $i++) {
//                echo '<div class="rocniprehled-prazdnabunka"></div>';
//            }
//
//            echo "</div>\n"; // konec rocniprehled-data
//
//            echo "</div>\n"; // konec radku
//        }
//
//        echo '</div>';
//    }
}
