<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MessageManager;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;


class SchvaleniDochazky
{
    const SETTINGS_EDITACE_SCHVALENE = 'editaceschvalenedochazky';

    var $schvaleni1;
    var $schvaleni2;
    var $schvaleni3;

    var $zpracovanoVeMzdach;

    // dalsi pole se defaultne nenacitaji (viz load)
    var $poznamka;

    var $schvalil1;
    var $datumSchvaleni1;

    var $schvalil2;
    var $datumSchvaleni2;

    var $schvalil3;
    var $datumSchvaleni3;


    /**
     * @param Database $database
     * @param array $row
     * @param bool $nacistVse
     */
    private function loadRow($database, $row, $nacistVse = false)
    {
        $this->schvaleni1 = $row[0] ? $database ->stringToTimestamp($row[0]) : 0;
        $this->schvaleni2 = $row[1] ? $database ->stringToTimestamp($row[1]) : 0;
        $this->schvaleni3 = $row[2] ? $database ->stringToTimestamp($row[2]) : 0;
        $this->zpracovanoVeMzdach = $row[3];

        if ($nacistVse) {
            $this->schvalil1 = $row[4];
            $this->datumSchvaleni1 = $database ->stringToTimestamp($row[5]);

            $this->schvalil2 = $row[6];
            $this->datumSchvaleni2 = $database ->stringToTimestamp($row[7]);

            $this->schvalil3 = $row[8];
            $this->datumSchvaleni3 = $database ->stringToTimestamp($row[9]);

            $this->poznamka = $row[10];
        }
    }


    /**
     * Nacte posledni datumy schvaleni dochazky osoby v zadanem rozsahu. Pokud se nacita jen jedno obdobi a nacistvse je true, nacita i podrobnosti.
     * @param Database $database
     * @param string   $RC
     * @param string   $osobniCislo
     * @param int      $datumNastupu
     * @param int      $datumOd
     * @param int|null $datumDo
     * @param bool     $nacistVse
     */
    function load($database, $RC, $osobniCislo, $datumNastupu, $datumOd, $datumDo = null, $nacistVse = false)
    {
        $this->schvaleni1 = 0;
        $this->schvaleni2 = 0;
        $this->schvaleni3 = 0;

        $this->zpracovanoVeMzdach = 0;

        $this->schvalil1 = null;
        $this->datumSchvaleni1 = null;

        $this->schvalil2 = null;
        $this->datumSchvaleni2 = null;

        $this->schvalil3 = null;
        $this->datumSchvaleni3 = null;

        $this->poznamka = null;

        $datumOd = $datumOd ?: time();
        $datumDo = $datumDo ?: $datumOd;

        $params = $database ->createParameters();

        if ($datumOd == $datumDo) {
            $sql =
                'SELECT ZpracovanoDo, ZkontrolovanoDo, SchvalenoDo, ZpracovanoVeMzdach ' .
                ($nacistVse
                    ? ', Zpracoval, DT_Zpracovani, Zkontroloval, DT_Zkontrolovani, Schvalil, DT_Schvaleni, ' . $database ->castWideMemo('Poznamka')
                    : '') .
                ' FROM SchvaleniDochazky' .
                ' WHERE RC = ' . $params ->addParameter($RC) .
                ' AND OsobniCislo = ' . $params ->addParameter($osobniCislo) .
                ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($datumNastupu)) .
                " AND Aplikace = 'ADS'" .
                ' AND PocatekObdobi = ' . $params ->addParameter($database ->dateToParam(DateUtils::firstDayOfMonth($datumOd)));
        } else {
            $sql =
                'SELECT MAX(ZpracovanoDo), MAX(ZkontrolovanoDo), MAX(SchvalenoDo), null ' .
                ' FROM SchvaleniDochazky' .
                ' WHERE RC = ' . $params ->addParameter($RC) .
                ' AND OsobniCislo = ' . $params ->addParameter($osobniCislo) .
                ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($datumNastupu)) .
                " AND Aplikace = 'ADS'" .
                ' AND PocatekObdobi >= ' . $params ->addParameter($database ->dateToParam(DateUtils::firstDayOfMonth($datumOd))) .
                ' AND PocatekObdobi <= ' . $params ->addParameter($database ->dateToParam($datumDo));
        }

        MessageManager::addDebugLog('Schvaleni: ' . $sql);

        $cursor = $database ->getSqlResult($sql, $params);

        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            $this ->loadRow($database, $row, $nacistVse);
        }
    }

    function nejvyssiUrovenSchvaleni($datum)
    {
        if ($datum <= $this ->schvaleni3) {
            return 3;
        }
        if ($datum <= $this ->schvaleni2) {
            return 2;
        }
        if ($datum <= $this ->schvaleni1) {
            return 1;
        }
        return 0;
    }


    function classNejvyssiUroven($datum)
    {
        $uroven = $this ->nejvyssiUrovenSchvaleni($datum);
        if ($uroven) {
            return "schvaleni$uroven";
        }

        return '';
    }


    /**
     * Vraci true, pokud je zadane datum schvaleno zadanou urovni. Pokud neni uroven zadana, tak kontroluje vsechny urovne
     * @param int $datum
     * @param int $uroven
     * @return bool
     */
    function jeSchvaleno($datum, $uroven = null)
    {
        switch ($uroven) {
            case 1 : // zelena
                return  $datum <= $this ->schvaleni1;

            case 2 : // zluta
                return $datum <= $this ->schvaleni2;

            case 3 : // cervena
                return $datum <= $this ->schvaleni3;
        }

        $datum = DateUtils::startOfTheDay($datum);

        if ($this ->zpracovanoVeMzdach) {
            return true;
        }

        if ($datum <= $this->schvaleni1) {
            return true;
        }
        if ($datum <= $this->schvaleni2) {
            return true;
        }
        if ($datum <= $this->schvaleni3) {
            return true;
        }

        return false;
    }


    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datum
     * @return SchvaleniDochazky
     */
    public static function nactiSchvaleni($database, $pp, $datum)
    {
        $schvaleni = new static();
        $schvaleni ->load($database, $pp ->rc, $pp ->osobniCislo, $pp ->datumNastupu, $datum);
        return $schvaleni;
    }



    /**
     * Nacte seznam schvaleni pro zadane obdobi (muze to byt i vice mesicu)
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @return SchvaleniDochazky[]
     */
    public static function nactiSchvaleniVObdobi($database, $pp, $datumOd, $datumDo)
    {
        $params = $database ->createParameters();

        $datumOd = DateUtils::firstDayOfMonth($datumOd);
        $datumDo = DateUtils::firstDayOfMonth($datumDo);

        $sql =
            'SELECT ZpracovanoDo, ZkontrolovanoDo, SchvalenoDo, ZpracovanoVeMzdach ' .
            ' FROM SchvaleniDochazky' .
            ' WHERE RC = ' . $params ->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
            " AND Aplikace = 'ADS'" .
            ' AND PocatekObdobi >= ' . $params ->addParameter($database ->dateToParam(DateUtils::firstDayOfMonth($datumOd))) .
            ' AND PocatekObdobi <= ' . $params ->addParameter($database ->dateToParam(DateUtils::firstDayOfMonth($datumDo)));

        $result = [];
        $cursor = $database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $schvaleni = new SchvaleniDochazky();
            $schvaleni ->loadRow($database, $row);
            $result[] = $schvaleni;
        }
        $cursor ->close();

        return $result;
    }


    /**
     * Vraci true, pokud je v zadanem obdobi schvalena dochazka (kontroluje $datumOd a $datumDo)
     * @param $database
     * @param $pp
     * @param $datumOd
     * @param $datumDo
     * @return bool
     */
    public static function jeSchvalenoObdobi($database, $pp, $datumOd, $datumDo)
    {
        $seznamSchvaleni = self::nactiSchvaleniVObdobi($database, $pp, $datumOd, $datumDo);
        foreach ($seznamSchvaleni as $schvaleni) {
            if ($schvaleni ->jeSchvaleno($datumOd)) {
                return true;
            }
            if ($schvaleni ->jeSchvaleno($datumDo)) {
                return true;
            }
        }

        return false;
    }


    /**
     * Vraci true, pokud je mozne editovat zadanou osobu pro dane datum.
     * @param int $idPP
     * @param int $datum
     * @param int $datumPravOd
     * @param int $datumPravDo
     * @return bool
     */
    function lzeEditovatPP($idPP, $datum, $datumPravOd = null, $datumPravDo = null)
    {
        if ($this ->zpracovanoVeMzdach) {
            return false;
        }

        if (!RonPortalSettings::getIntValue(self::SETTINGS_EDITACE_SCHVALENE)) {
            // Editace schvalenych dnu neni povolena -> lze editovat, pokud neni schvaleno
            return !$this->jeSchvaleno($datum);
        }

        if ($datum <= $this->schvaleni3) {
            // schvaleno urovni 3
            return false;
        }

        if ($datum > $this->schvaleni2 && $datum > $this->schvaleni1) {
            // neni vubec schvaleno
            return true;
        }

        if (!$datumPravOd) {
            $datumPravOd = $datum;
        }
        if (!$datumPravDo) {
            $datumPravDo = $datum;
        }

        // schvaleno urovni 1 nebo 2
        $prava = Prava::vratPrava('schvaleni');
        if ($prava->lzeZobrazitPP($idPP, $datumPravOd, $datumPravDo)) {
            // mam prava na uroven 3 => muzu editovat
            return true;
        }

        if ($datum <= $this->schvaleni2) {
            // je schvaleno urovni 2 a nemam prava na uroven 3 => nemuzu editovat
            return false;
        }

        // je schvaleno urovni 1 a nemam pravo na uroven 3 - muzu editovat, pokud mam pravo na schvaleni urovne 2
        $prava = Prava::vratPrava('zkontrolovani');

        return $prava->lzeZobrazitPP($idPP, $datumPravOd, $datumPravDo);
    }


    /**
     * Vraci prvni datum zadaneho obdobi, ktere lze editovat. Pokud nemuze editovat zadne, vraci null
     * @param int $idPP
     * @param int $datumOd
     * @param int $datumDo
     * @return int
     * @throws \Exception
     */
    function prvniEditovatelnyDen($idPP, $datumOd, $datumDo)
    {
        if ($this ->zpracovanoVeMzdach) {
            return null;
        }

        $datum = $datumOd;

        if (!RonPortalSettings::getIntValue(self::SETTINGS_EDITACE_SCHVALENE)) {
            // nelze editovat schvalenou dochazku
            if ($this ->schvaleni1 >= $datumDo  ||  $this ->schvaleni2 >= $datumOd  ||  $this ->schvaleni3 >= $datumDo) {
                // jiz je schvaleno do konce obdobi
                return null;
            }

            if ($this ->schvaleni1  &&  $datum <= $this ->schvaleni1) {
                $datum = DateUtils::incDayStart($this ->schvaleni1);
            }

            if ($this ->schvaleni2  &&  $datum <= $this ->schvaleni2) {
                $datum = DateUtils::incDayStart($this ->schvaleni2);
            }

            if ($this ->schvaleni3  &&  $datum <= $this ->schvaleni3) {
                $datum = DateUtils::incDayStart($this ->schvaleni3);
            }

            return $datum;
        }


        if ($this ->schvaleni3) {
            if ($this->schvaleni3 >= $datumDo) {
                // uz je schvalene cele obdobi
                return null;
            }

            if ($this ->schvaleni3 >= $datum) {
                $datum = DateUtils::incDayStart($this->schvaleni3);
            }
        }

        $prava = Prava::vratPrava('schvaleni');
        if ($prava->lzeZobrazitPP($idPP, $datum, $datum)) return $datum; // mam prava na uroven 3 => muzu editovat od $datum

        if ($this ->schvaleni2) {
            if ($this->schvaleni2 >= $datumDo) {
                // uz je schvalene cele obdobi
                return null;
            }

            if ($this ->schvaleni2 >= $datum) {
                $datum = DateUtils::incDayStart($this->schvaleni2);
            }
        }

        $prava = Prava::vratPrava('zkontrolovani');
        if ($prava->lzeZobrazitPP($idPP, $datum, $datum)) return $datum; // mam prava na uroven 2 => muzu editovat od $datum

        if ($this ->schvaleni1) {
            if ($this->schvaleni1 >= $datumDo) {
                // uz je schvalene cele obdobi
                return null;
            }

            if ($this ->schvaleni1 >= $datum) {
                $datum = DateUtils::incDayStart($this->schvaleni1);
            }
        }

        return $datum;
    }


    function renderDiv($datum)
    {
        $datum = DateUtils::startOfTheDay($datum);

        global $STR_ZpracovanoDo;   // kompat
        global $STR_ZkontrolovanoDo;// kompat
        global $STR_SchvalenoDo;    // kompat

        // zelena
        if ($this->schvaleni1) $titlestr = 'title="' . ($STR_ZpracovanoDo ?: \Ron\ronText('Dochazka_schvaleni_1_do')) . ' ' . date('j.n.Y', $this->schvaleni1) . '"';
        else $titlestr = '';
        if ($datum <= $this->schvaleni1) echo "<DIV class='schvaleni1 schvalenibox' $titlestr></DIV>";
        else echo "<DIV class='schvalenibox' $titlestr></DIV>";

        // zluta
        if ($this->schvaleni2) $titlestr = 'title="' . ($STR_ZkontrolovanoDo ?: \Ron\ronText('Dochazka_schvaleni_2_do')) . ' ' . date('j.n.Y', $this->schvaleni2) . '"';
        else $titlestr = '';
        if ($datum <= $this->schvaleni2) echo "<DIV class='schvaleni2 schvalenibox' $titlestr></DIV>";
        else echo "<DIV class='schvalenibox' $titlestr></DIV>";

        // cervena
        if ($this->schvaleni3) $titlestr = 'title="' . ($STR_SchvalenoDo ?: \Ron\ronText('Dochazka_schvaleni_3_do')) . ' ' . date('j.n.Y', $this->schvaleni3) . '"';
        else $titlestr = '';
        if ($datum <= $this->schvaleni3) echo "<DIV class='schvaleni3 schvalenibox' $titlestr></DIV>";
        else echo "<DIV class='schvalenibox' $titlestr></DIV>";
    }


    /**
     * Vraci true, pokud ma uzivatel nejake prava na schvalovani zadaneho PP
     * @param int $idPP
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws \Exception
     */
    public static function lzeSchvalovatOsobu($idPP, $obdobiOd = null, $obdobiDo = null)
    {
        $prava = Prava::vratPrava('zpracovani');
        if ($prava -> lzeZobrazitPP($idPP, $obdobiOd, $obdobiDo)) return true;

        $prava = Prava::vratPrava('zkontrolovani');
        if ($prava -> lzeZobrazitPP($idPP, $obdobiOd, $obdobiDo)) return true;

        $prava = Prava::vratPrava('schvaleni');
        if ($prava -> lzeZobrazitPP($idPP, $obdobiOd, $obdobiDo)) return true;

        return false; // nema prava pro zadnou uroven
    }


    /**
     * vraci true, pokud aktualni uzivatel muze schvalit zadany PP svym nejvyssim levelem
     * (tj. jeste neni schvaleno do zadaneho datumu)
     * @param $idPP
     * @param int $datum
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws \Exception
     */
    function lzeSchvalitNejvyssiLevel($idPP, $datum, $obdobiOd, $obdobiDo)
    {
        $prava = Prava::vratPrava('schvaleni');
        if ($prava->lzeZobrazitPP($idPP, $obdobiOd, $obdobiDo)) {
            return $this->schvaleni3 < $datum;
        }

        $prava = Prava::vratPrava('zkontrolovani');
        if ($prava->lzeZobrazitPP($idPP, $obdobiOd, $obdobiDo)) {
            return $this->schvaleni2 < $datum;
        }

        $prava = Prava::vratPrava('zpracovani');
        if ($prava->lzeZobrazitPP($idPP, $obdobiOd, $obdobiDo)) {
            return $this->schvaleni1 < $datum;
        }

        return false; // nema prava
    }


    /**
     * vraci true, pokud aktualni uzivatel muze odschvalit zadany PP svym nejvyssim levelem (tj. je uz schvaleno)
     * @param int $idPP
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws \Exception
     */
    function lzeOdschvalitNejvyssiLevel($idPP, $obdobiOd, $obdobiDo)
    {
        $prava = Prava::vratPrava('schvaleni');
        if ($prava->lzeZobrazitPP($idPP, $obdobiOd, $obdobiDo)) {
            if ($prava->priznak) return false; // odschvaleni je zakazano v pravech
            return $this->schvaleni3 >= $obdobiOd;
        }

        $prava = Prava::vratPrava('zkontrolovani');
        if ($prava->lzeZobrazitPP($idPP, $obdobiOd, $obdobiDo)) {
            if ($prava->priznak) return false; // odschvaleni je zakazano v pravech
            return $this->schvaleni2 >= $obdobiOd;
        }

        $prava = Prava::vratPrava('zpracovani');
        if ($prava->lzeZobrazitPP($idPP, $obdobiOd, $obdobiDo)) {
            if ($prava->priznak) return false; // odschvaleni je zakazano v pravech
            return $this->schvaleni1 >= $obdobiOd;
        }

        return false; // nema prava
    }


    /**
     * @param int $idPP
     * @param int $uroven Uroven schvaleni (1/2/3)
     * @param int $datum
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool Vraci true, pokud uzivatel muze schvalit pracovni pomer $idpp zadanou urovni
     * @throws \Exception
     */
    function lzeSchvalitUroven($idPP, $uroven, $datum, $obdobiOd, $obdobiDo)
    {
        $prava = null;

        switch ($uroven) {
            case 1 :
                if ($this->schvaleni1 >= $datum) return false; // uz je schvaleno
                $prava = Prava::vratPrava('zpracovani');
                break;

            case 2 :
                if ($this->schvaleni2 >= $datum) return false; // uz je schvaleno
                $prava = Prava::vratPrava('zkontrolovani');
                break;

            case 3 :
                if ($this->schvaleni3 >= $datum) return false; // uz je schvaleno
                $prava = Prava::vratPrava('schvaleni');
                break;
        }

        if (!$prava) return false;

        return $prava->lzeZobrazitPP($idPP, $obdobiOd, $obdobiDo);
    }


    /**
     * @param int $idPP
     * @param int $uroven
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws \Exception
     */
    function lzeOdschvalitUroven($idPP, $uroven, $obdobiOd, $obdobiDo)
    {
        $prava = null;

        switch ($uroven) {
            case 1 :
                if (!$this->schvaleni1) return false; // jeste neni schvaleno
                $prava = Prava::vratPrava('zpracovani');
                break;

            case 2 :
                if (!$this->schvaleni2) return false;
                $prava = Prava::vratPrava('zkontrolovani');
                break;

            case 3 :
                if (!$this->schvaleni3) return false;
                $prava = Prava::vratPrava('schvaleni');
                break;
        }

        if (!$prava) return false;

        if ( $prava ->priznak ) return false; // odschvaleni je zakazano v pravech

        return $prava->lzeZobrazitPP($idPP, $obdobiOd, $obdobiDo);
    }
}
