<?php
/**
 * stanislav.elbl, 16.03.2021
 */

namespace RonPortal\Dochazka;

use Ron\Controls\Dialog;
use Ron\Controls\DialogButton;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\Http\Csrf;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalScriptEvents;
use RonPortal\RonPortalSession;
use RonPortal\User;

class SchvalovaniController extends JsonController
{

    /** @var Database */
    var $database;

    /**
     * SchvalovaniController constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;
    }


//    public static function lzeSchvalovatOsobu($idpp)
//    {
//        $prava = Prava::vratPrava('zpracovani');
//        if ($prava -> lzeZobrazitPP($idpp, RonPortalSession::obdobi(), RonPortalSession::konecObdobi())) return true;
//
//        $prava = Prava::vratPrava('zkontrolovani');
//        if ($prava -> lzeZobrazitPP($idpp, RonPortalSession::obdobi(), RonPortalSession::konecObdobi())) return true;
//
//        $prava = Prava::vratPrava('schvaleni');
//        if ($prava -> lzeZobrazitPP($idpp, RonPortalSession::obdobi(), RonPortalSession::konecObdobi())) return true;
//
//        return false; // nema prava pro zadnou uroven
//    }


    /**
     * Vraci ID XML formulare s nastavenim seznamu pro schvalovani dochazky
     * @return bool|int
     */
    function getXMLSchvalovani()
    {
        $xmlid = DBUtils::getSqlValue($this ->database, "SELECT Id FROM SablonaXMLFormulare WHERE Popis = 'SCHVALOVANI_DOCHAZKY'");
        return $xmlid ? $xmlid : false;
    }


    function schvalovaniBrowser()
    {
        $xmlId = ($_GET['force_no_xml'] ?? false)
            ? null
            : $this->getXMLSchvalovani();

        $obdobi = null;

        if ($xmlId) {
            // zobrazeni uzivatelsky nastaveneho seznamu
            $form = new SchvalovaniXML($this ->database);
            $form->load($xmlId);

            $obdobi = $form ->getPocatekObdobi();

            Model::addViewObject($form);

            $dialog = (new Dialog())
                ->setView('ronportal/dochazka/view_schvaleni_progress.php')
                ->setTitle('Schvalovani')
                ->setId('schvaleni_progress')
                ->addButton(
                    (new DialogButton())
                        ->setValue('Presto_schvalit')
                        ->setId('schvaleno_ok')
                        ->addClass('d-none')
                )
                ->addButton(
                    (new DialogButton())
                        ->setValue('Preskocit')
                        ->setId('schvaleno_preskocit')
                        ->addClass('d-none')
                )
                ->addButton(DialogButton::cancelButton('schvaleno_storno'));

            Model::addViewObject($dialog);
            //Model::addViewObject( new BaseViewObject('ronportal/dochazka/view_schvaleni_progress.php'));

        } else {
            // standardni zobrazeni seznamu osob
            $b = new SchvalovaniDochazkyBrowser($this->database);
            $b->load();

            $obdobi = $b ->filterForm ->getFilterValue('obdobi');

            Model::addViewObject($b);
        }


        Model::addViewObject(new SchvaleniView($this->database, $obdobi));

        return true;
    }


    function hromadneSchvalovani()
    {
        $obdobi = isset($_GET['obdobi']) ? DateUtils::urlStringToDate($_GET['obdobi']) : RonPortalSession::obdobi();

        $form = new HromadneSchvaleniFormular($this ->database);
        $form->nastavObdobi($obdobi);
        $form->nactiData();

        return $form;
    }


    function provedHromadneSchvaleni()
    {
        if (!$_POST['obdobi']) throw new \Exception('Chyba - neni zadano obdobi!');

        $form = new HromadneSchvaleniFormular($this ->database);
        $form->nastavObdobi( DateUtils::urlStringToDate($_POST['obdobi']) );

        try {
            if ($form ->zapisData()) {
                return new Redirect(null, "?menuid=schvalovani&schvaleniOK"); // parametr nepodstatny
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        $form->nactiData();
        return $form;
    }


    /**
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function loadData($output)
    {
        $pp = PracovniPomer::nactiPP($this ->database, (int)$_GET['idpp']);

        $pocatekobdobi = DateUtils::dateStringToTimestamp($_GET['obdobi']);
        if (!$pocatekobdobi) {
            throw new \Exception('Chyba - spatne zadane obdobi!');
        }

        $obdobi = Obdobi::vratObdobi($this ->database, 'ADS', $pocatekobdobi);

        //nactiPravaUzivatele($this ->database, S_GetUserRC());
        Prava::nastavObdobiProPrava($obdobi ->zacatek, $obdobi ->konec);

        if ( !SchvaleniDochazky::lzeSchvalovatOsobu($pp ->idPP, $obdobi ->zacatek, $obdobi ->konec) ) {
            // zkontroluje pravo na zobrazeni
            $prava = Prava::vratPrava('prohlizeni');
            if ( !$prava ->lzeZobrazitPP($pp ->idPP) ) {
                // jeste zkontroluje pravo na editaci
                $prava = Prava::vratPrava('editace');
                if ( !$prava ->lzeZobrazitPP($pp ->idPP)) {
                    if (!User::getRC()) {
                        // doslo k odhlaseni uzivatele
                        throw new \Exception('Pravdepodobne doslo k automatickemu odhlaseni! Prihlaste se prosim znovu.');
                    } else {
                        throw new NeniPravoException();
                    }
                }
            }
        }

        $schvaleni = new SchvaleniDochazky();

        $schvaleni->load($this->database, $pp->rc, $pp->osobniCislo, $pp->datumNastupu, $obdobi->zacatek, $obdobi->zacatek, true);

        $output ->addData('idpp', $pp->idPP);
        $output ->addData('pocatekObdobi', date('j.n.Y', $obdobi->zacatek));
        $output ->addText('jmeno', $pp->jmenoPrijmeni, false); // htmlconvert = false
        $output ->addData('zpracovanoDo', StringUtils::jsonDate($schvaleni->schvaleni1));
        $output ->addText('zpracoval', $schvaleni->schvalil1 . ($schvaleni->datumSchvaleni1 ? ' (' . DateUtils::fineDateString($schvaleni->datumSchvaleni1, true) . ')' : ''));
        $output ->addData('pravoZpracovat', Prava::vratPrava('zpracovani') ->lzeZobrazitPP($pp->idPP) ? 1 : 0);
        $output ->addData('zkontrolovanoDo', StringUtils::jsonDate($schvaleni->schvaleni2));
        $output ->addText('zkontroloval', $schvaleni->schvalil2 . ($schvaleni->datumSchvaleni2 ? ' (' . DateUtils::fineDateString($schvaleni->datumSchvaleni2, true) . ')' : ''));
        $output ->addData('pravoZkontrolovat', Prava::vratPrava('zkontrolovani') ->lzeZobrazitPP($pp->idPP) ? 1 : 0);
        $output ->addData('schvalenoDo', StringUtils::jsonDate($schvaleni->schvaleni3));
        $output ->addText('schvalil', $schvaleni->schvalil3 . ($schvaleni->datumSchvaleni3 ? ' (' . DateUtils::fineDateString($schvaleni->datumSchvaleni3, true) . ')' : ''));
        $output ->addData('pravoSchvalit', Prava::vratPrava('schvaleni') ->lzeZobrazitPP($pp->idPP) ? 1 : 0);
        $output ->addData('zpracovanoVeMzdach', (int)$schvaleni->zpracovanoVeMzdach);
        $output ->addText('poznamka', $schvaleni->poznamka);
    }


    /**
     * Spusti skript SCHVALENIDOCHAZKY_IW_BEFORE, Schvali a pak jeste provede IW_AFTER
     */
    function provedSchvaleniOsoby()
    {
        Csrf::validateCSRFPostParam('schvalenidochazky');

        $rec = new SchvaleniDochazkyRecord($this->database, $this);

        // nacte puvodni data
        $exists = $rec->loadDataJS();

        $rec->loadPost(); // nacte data odeslana uzivatelem

        $rec->zkontrolujAZapisSchvaleni(!$exists);
    }


    /**
     * Spusti skript SCHVALENIDOCHAZKY_IW_CHECK
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function checkSchvaleniOsoby($output)
    {
        $rec = new SchvaleniDochazkyRecord($this->database, $this);

        $rec->loadDataJS();

        // nacte data odeslana uzivatelem
        $rec->loadPost();

        if ($rec->isChanged()) {
            $ok = true;

            try {
                $scriptNames = [RonPortalScriptEvents::SCHVALENI_DOCHAZKY_CHECK];
                if ($rec ->fZpracovano ->isChanged()) {
                    $scriptNames[] = RonPortalScriptEvents::SCHVALENI_DOCHAZKY_1_CHECK;
                }
                if ($rec ->fZkontrolovano ->isChanged()) {
                    $scriptNames[] = RonPortalScriptEvents::SCHVALENI_DOCHAZKY_2_CHECK;
                }
                if ($rec ->fSchvaleno ->isChanged()) {
                    $scriptNames[] = RonPortalScriptEvents::SCHVALENI_DOCHAZKY_3_CHECK;
                }

                $rec->executeScripts($scriptNames);
            } catch (\Exception $e) {
                $output ->addError($e ->getMessage());
                $output->setResult('warning');
                $ok = false;
            }

            if ($ok) {
                // check probehl v poradku, provede rovnou i schvaleni
                $this->provedSchvaleniOsoby();
            }
        }
    }


    /**
     * @param string $action
     * @param JsonOutputObject $output
     * @throws NeniPravoException
     * @throws \Exception
     */
    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'loaddata' :
                $this ->loadData($output);
                return;

            case 'schvalit' :
                // spusteni skriptu pro overeni - v pripade chyby se zobrazi dotaz, jestli chce pokracovat
                // a pokud neni chyba, nebo skript neexistuje, provede schvaleni
                $this ->checkSchvaleniOsoby($output);
                return;

            case 'schvalit_no_check' :
                // spusteni skriptu pro kontrolu a zapis schvaleni (bez skriptu pro overeni - uzivatel uz potvrdil)
                $this ->provedSchvaleniOsoby();
                return;

            default :
                throw new \Exception("chyba ve SchvalovaniController::handleJsonAction - neznama akce!");
        }
    }


    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('dochazka.ini');

        HtmlHeaders::getInstance()
            ->addScript('script/dochazka.js')
            ->addCss('css/dochazka.css')
            ->setAppTitle('Schvalovani');

        if (isset($_POST['schvaleniaction'])) {
            switch ($_POST['schvaleniaction']) {
//                case 'provedschvaleni' :
//                    throw new \Exception('NENI IMPLEMENTOVANO!');
//                    return provedSchvaleniDochazky($connection);

                case 'hromadneschvaleni' :
                    return $this ->provedHromadneSchvaleni();
            }
        }

        if (isset($_GET['hromadneschvaleni'])  &&  $_GET['hromadneschvaleni']) {
            return $this ->hromadneSchvalovani();
        }

        if (isset($_GET['action'])) {
            switch ($_GET['action']) {
                case 'loaddata' :
                case 'schvalit' :
                case 'schvalit_no_check' :
                    return parent::handleAction($action);
            }
        }

        return $this ->schvalovaniBrowser();
    }

}
