<?php
/**
 * stanislav.elbl, 16.03.2021
 */

namespace RonPortal\Dochazka;

use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\SqlStatement;
use Ron\HttpUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RozdeleniBrowserField;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;


/**
 * Seznam zamestnancu pro schvalovani dochazky - zobrazuje se, pokud neni vytvoren XML formular SCHVALOVANI_DOCHAZKY
 */
class SchvalovaniDochazkyBrowser extends RonPortalBrowser
{
    /** @var BrowserField   Sloupec pro proklik na editaci */
    private $fEditace;


    function __construct($database)
    {
        parent::__construct($database, 'schvalovani');

        $this ->title = TextManager::_('Schvalovani');
        $this ->code = 'browserschvaldoch';

        $this ->tableId = 'seznam_schvalovani_dochazky';

        $this ->sqlSelectPart = 'PracovniPomer.ID'; // 0

//            (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1) ? RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1) : 'null') . ', ' . // 15
//            (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2) ? RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2) : 'null'); // 16
//
//        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1)) {
//            $this->addField()
//                ->setSqlFieldName(RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1), 15)
//                ->setCode('col-pole1') ->setOrdering(1)
//                ->setHtmlConvert(false);
//        }

        $this->addField()
            ->setSqlFieldName('Osoba.Titul') ->setHeaderText('Titul') ->setOrdering(true)
            ->setCode('col-titul');

        // zapamatuje si pole, aby mohl spravne nastavit link podle zvoleneho obdobi (nutno nastavit az po zobrazeni filtru)
        $this->addField()
            ->setSqlFieldName('Osoba.Prijmeni') ->setHeaderText('Prijmeni') ->setOrdering(true)
            ->setCode('col-prijmeni')
            ->setSortSQL('Prijmeni, Jmeno');

        $this->defaultSort = 'col-prijmeni';

        $this->addField()
            ->setSqlFieldName('Osoba.Jmeno') ->setHeaderText('Jmeno') ->setOrdering(true)
            ->setCode('col-jmeno')
            ->setSortSQL('Osoba.Jmeno, Osoba.Prijmeni');

        $this->addField()
            ->setSqlFieldName('PracovniPomer.OsobniCislo') ->setHeaderText('Osobni_cislo') ->setOrdering(true)
            ->setCode('col-osobnicislo');

//        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2)) {
//            $this->addField()
//                ->setSqlFieldName(RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2), 16) ->setOrdering(1)
//                ->setCode('col-pole2')
//                ->setHtmlConvert(false);
//        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP)) {
            $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('Rozdeleni.Kod')
                ->setCode('col-rozdeleni-kod')
                ->setOrdering(true);

            $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('Rozdeleni.Popis')
                ->setCode('col-rozdeleni-popis')
                ->setOrdering(true);

            $this ->addField(new RozdeleniBrowserField($this ->database, 'col-rozdeleni-kod', 'col-rozdeleni-popis'));
        }

        // schvaleni 1
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SchvaleniDochazky.DT_Zpracovani')
            ->setCode('col-dt_zpracovani')
            ->setFormat('finedatetime');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SchvaleniDochazky.Zpracoval')
            ->setCode('col-zpracoval')
            ->setSqlFieldIndex2('@col-dt_zpracovani');

        $this ->addField()
            ->setSqlFieldName('SchvaleniDochazky.ZpracovanoDo') ->setHeaderText('Dochazka_schvaleni_1_do')
            ->setCode('col-schvaleni1')
            ->setTitleFieldIndex('@col-zpracoval')
            ->setFormat('date') ->setAlign('center') ->setOrdering(true)
            ->setClassName('textschvaleni1');

        // schvaleni 2
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SchvaleniDochazky.DT_Zkontrolovani')
            ->setCode('col-dt_zkontrolovani')
            ->setFormat('finedatetime');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SchvaleniDochazky.Zkontroloval')
            ->setCode('col-zkontroloval')
            ->setSqlFieldIndex2('@col-dt_zkontrolovani');

        $this ->addField()
            ->setSqlFieldName('SchvaleniDochazky.ZkontrolovanoDo') ->setHeaderText('Dochazka_schvaleni_2_do')
            ->setCode('col-schvaleni2')
            ->setTitleFieldIndex('@col-zkontroloval')
            ->setFormat('date') ->setAlign('center') ->setOrdering(true)
            ->setClassName('textschvaleni2');

        // schvaleni 3
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SchvaleniDochazky.DT_Schvaleni')
            ->setCode('col-dt_schvaleni')
            ->setFormat('finedatetime');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SchvaleniDochazky.Schvalil')
            ->setCode('col-schvalil')
            ->setSqlFieldIndex2('@col-dt_schvaleni');

        $this ->addField()
            ->setSqlFieldName('SchvaleniDochazky.SchvalenoDo') ->setHeaderText('Dochazka_schvaleni_3_do')
            ->setCode('col-schvaleni3')
            ->setTitleFieldIndex('@col-schvalil')
            ->setFormat('date') ->setAlign('center') ->setOrdering(true)
            ->setClassName('textschvaleni3');


        $this ->fEditace = $this ->addField(BrowserField::FIELD_TYPE_TEXT)
            ->setText(TextManager::_('dochazka'))
            ->setLink('?menuid=zpracovanadochazka&obdobi=%OBDOBI%&osoba=', ['target' => 'zpracovanadochazkawin'])
            ->setCode('col-edit');

        $this->sqlWherePart =
            '   PracovniPomer.Zamestnanec > 0 ' .   // nezobrazuje externi osoby
            '   AND PracovniPomer.ZpracovatDochazku = 1 ';

        $this->primaryField = 0;
    }


    function setFilterForm()
    {
        $this -> filterForm = new SchvalovaniFilterForm($this ->database);
    }

    function loadFilterData()
    {
        parent::loadFilterData();
        $this ->fEditace ->link = str_replace('%OBDOBI%', DateUtils::dateToUrlString($this ->filterForm ->getFilterValue('obdobi')), $this ->fEditace ->link);
    }

    public function getSqlFromPart()
    {
        // sql dotaz se nastavi az po vyberu obdobi
        $params = $this ->database ->createParameters();

        $result =
            '(( Osoba INNER JOIN PracovniPomer' .
            '  ON Osoba.RC = PracovniPomer.RC' .
            ")\n" .
            ' INNER JOIN Obdobi' .
            "  ON Obdobi.Aplikace = 'ADS'" .
            ' AND Obdobi.PocatekObdobi = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('obdobi')) .
            ")\n" .
            ' LEFT OUTER JOIN SchvaleniDochazky' .
            '  ON SchvaleniDochazky.RC = PracovniPomer.RC ' .
            ' AND SchvaleniDochazky.OsobniCislo = PracovniPomer.OsobniCislo ' .
            ' AND SchvaleniDochazky.DatumNastupu = PracovniPomer.DatumNastupu ' .
            " AND SchvaleniDochazky.Aplikace = 'ADS' " .
            ' AND SchvaleniDochazky.PocatekObdobi = Obdobi.PocatekObdobi' .
            "\n";

        if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP) ) {
            $result =
                '( ( ' .
                $result .
                ')' .
                ' LEFT OUTER JOIN RozdeleniOsob' .
                ' ON RozdeleniOsob.RC = PracovniPomer.RC' .
                ' AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
                ' AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
                ' AND RozdeleniOsob.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP) .
                ' AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('obdobi', 'datumDo')) .
                ' AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('obdobi', 'datumDo')) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ') ' .
                ' LEFT OUTER JOIN Rozdeleni' .
                ' ON Rozdeleni.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP) .
                ' AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni' .
                "\n";
        }

        return new SqlStatement($result, $params);
    }


    function renderFilter()
    {
        $this ->filterForm ->render();

        $params = '&obdobi=' . DateUtils::dateToUrlString($this ->filterForm ->getFilterValue('obdobi'));

        $filtrR1 = $this ->filterForm ->getFilter('rozdeleni');
        if ($filtrR1  &&  $filtrR1 ->enabled) {
            $params .=
                '&rozdeleni=' . HttpUtils::urlEncode($filtrR1 ->getValue()) .
                '&rozdeleni_vcetne_podrizenych=' . (int)$filtrR1 ->getValue('vcetnePodrizenych');
        }
//        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) {
//            $params .=
//                '&rozdeleni=' . HttpUtils::urlEncode($this ->filterForm ->kodRozdeleni) .
//                '&rozdeleni_vcetne_podrizenych=' . (int)$this ->filterForm ->vcetnePodrizenychRozdeleni;
//        }

        $filtrR2 = $this ->filterForm ->getFilter('rozdeleni2');
        if ($filtrR2  &&  $filtrR2 ->enabled) {
            $params .=
                '&rozdeleni2=' . HttpUtils::urlEncode($filtrR2 ->getValue()) .
                '&rozdeleni2_vcetne_podrizenych=' . (int)$filtrR2 ->getValue('vcetnePodrizenych');
        }
//        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
//            $params .=
//                '&rozdeleni2=' . HttpUtils::urlEncode($this ->filterForm ->kodRozdeleni2) .
//                '&rozdeleni2_vcetne_podrizenych=' . (int)$this ->filterForm ->vcetnePodrizenychRozdeleni2;
//        }

        echo
            '<a href="?menuid=schvalovani&hromadneschvaleni=1' . $params . '">' .
            TextManager::_('Schvalit_hromadne') .
            '</a><br>';
    }



    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();

        $result = $this ->filterForm ->buildSqlFilter($params);

        $result .=
            ($result ? ' AND ' : '') .
            'PracovniPomer.DatumNastupu <= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('obdobi', 'datumDo')) . // jen platne PP
            ' AND ( PracovniPomer.DatumUkonceni >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('obdobi')) . ' OR PracovniPomer.DatumUkonceni IS NULL )';

        $filtrPrava = Prava::sqlFiltrPodlePrav(
            [Prava::vratPrava('zpracovani'), Prava::vratPrava('zkontrolovani'), Prava::vratPrava('schvaleni')],
            'PracovniPomer',
            $this ->filterForm ->getFilterValue('obdobi'), $this ->filterForm ->getFilterValue('obdobi', 'datumDo'),
            false,
            $params);

        if ($filtrPrava) {
            $result .= ' AND (' . $filtrPrava . ')';
        }


        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            $rozdeleniSQL = $rozdeleniFilter ->sestavSQLFiltr($this ->database, 'PracovniPomer', $params);
            if ($rozdeleniSQL) {
                $result .= ' AND (' . $rozdeleniSQL . ')';
            }
        }

        return new SqlStatement($result, $params);
    }

    public function getTableAttributes()
    {
        // doplni k defaultnim atributum data-obdobi
        $result = parent::getTableAttributes() ?? [];
        $result['data-obdobi'] = DateUtils::dateToUrlString($this ->filterForm ->getFilterValue('obdobi'));
        return $result;
    }


    function getRowAttributes($values, $className, $rowNumber)
    {
        return
            parent::getRowAttributes($values, 'schvalovani_row cursor-pointer ' . $className, $rowNumber) .
            ' data-idpp="' . (int)$values[0] . '"';
    }


    function enableLine($values)
    {
        return SchvaleniDochazky::lzeSchvalovatOsobu($values[0], $this ->filterForm ->getFilterValue('obdobi'), $this ->filterForm ->getFilterValue('obdobi', 'datumDo'));
    }
}
