<?php
/**
 * stanislav.elbl, 26.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\Parameters;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\PracovniPomer;



class SmenaProDen
{
    /** @var int    Datum zaznamu smeny pro den */
    public $datum;

    /** @var string Kod smeny */
    public $kod;
    /** @var string Nazev smeny*/
    public $popis;
    /** @var int    Priznak, jestli se jedna o variabilni smenu */
    public $variabilni;

    /** @var int    Cas pocatku smeny v sekundach od pulnoci */
    public $pocatekSmeny;
    /** @var int    Cas konce smeny v sekundach od pulnoci */
    public $konecSmeny;
    /** @var int    Delka smeny v sekundach */
    public $delkaSmeny;

    /** @var int    Cas pocatku pevneho jadra smeny v sekundach od pulnoci */
    public $pocatekPevneCasti;
    /** @var int    Cas konce pevneho jadra smeny v sekundach od pulnoci */
    public $konecPevneCasti;
    /** @var int    Delka pevneho jadra smeny v sekundach */
    public $delkaPevneCasti;

    public $delena;

    /** @var int    Cas pocatku preruseni smeny v sekundach od pulnoci */
    public $preruseniOd;
    /** @var int    Cas konce preruseni smeny v sekundach od pulnoci */
    public $preruseniDo;

    /** @var string     Kod puvodni smeny se naplni pouze pri nacitani smen podle kapacit. */
    public $kodPuvodniSmeny;

    /** @var int    Priznak, ze se ma urcit den prirazeni podle konce smeny */
    public $denDochPodleKonceSmeny;


    /**
     * Nacte seznam prirazenych smen ze zadaneho SQL dotazu
     * @param Database $database
     * @param string $sql
     * @param Parameters $params
     * @return SmenaProDen[]
     */
    private static function nactiSmenySQL($database, $sql, $params, $nacistPuvodniKod = false)
    {
        $result = [];

        $cursor = $database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $smena = new SmenaProDen();

            $smena ->datum = $database ->stringToTimestamp($row[0]);

            $smena ->kod = $row[1];
            $smena ->popis = TextManager::_('db_smena_popis_' . $row[1], $row[2]);
            $smena ->variabilni = (int)$row[3];

            if ($smena ->variabilni) {
                $smena ->pocatekSmeny = $database ->stringToSecondsOfDay($row[10]);
                $smena ->konecSmeny = $database ->stringToSecondsOfDay($row[11]);
                $smena ->delkaSmeny = $database ->stringToSecondsOfDay($row[12]);
            } else {
                $smena ->pocatekSmeny = $database ->stringToSecondsOfDay($row[4]);
                $smena ->konecSmeny = $database ->stringToSecondsOfDay($row[5]);
                $smena ->delkaSmeny = $database ->stringToSecondsOfDay($row[6]);
            }

            $smena ->pocatekPevneCasti = $row[7] ? $database ->stringToSecondsOfDay($row[7]) : null;
            $smena ->konecPevneCasti = $row[8] ? $database ->stringToSecondsOfDay($row[8]) : null;
            $smena ->delkaPevneCasti = $row[9] ? $database ->stringToSecondsOfDay($row[9]) : null;

            $smena ->delena = (int)$row[13];
            if ($smena ->delena) {
                $smena ->preruseniOd = $row[14] ? $database ->stringToSecondsOfDay($row[14]) : null;
                $smena ->preruseniDo = $row[15] ? $database ->stringToSecondsOfDay($row[15]) : null;
            }

            $smena ->denDochPodleKonceSmeny = (int)$row[16];

            if ($nacistPuvodniKod) {
                $smena ->kodPuvodniSmeny = $row[17];
            }

            $result[] = $smena;
        }

        $cursor ->close();

        return $result;
    }


    /**
     * Nacte smeny zadane osoby z planovani kapacit
     * @param Database $database
     * @param PracovniPomer|string $pp_rc
     * @param int $datumOd
     * @param int $datumDo
     * @return SmenaProDen[]
     */
    static function nactiSmenyPodleKapacit(Database $database, $pp_rc, $datumOd, $datumDo)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT PlanovaniKapacit.Datum, Smena.Kod, Smena.Popis, Smena.Variabilni, ' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 4..6
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevneCasti, ' . // 7..9
            ' PlanovaniKapacit.Smena_Pocatek, PlanovaniKapacit.Smena_Konec, PlanovaniKapacit.Smena_Delka, ' . // 10..12
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo, Smena.DenDochPodleKonceSmeny, ' . // 13..16
            ' PlanovaniKapacit.PuvodniPlan_Smena' . // 17
            '  FROM PlanovaniKapacit LEFT OUTER JOIN Smena' .   // musi byt outer kvuli moznosti zobrazeni puvodniho planu (sloupec KAP_PUV)
            '  ON PlanovaniKapacit.Smena = Smena.Kod' .
            ' WHERE PlanovaniKapacit.Datum >= ' . $params ->addDateParameter($datumOd) .
            '   AND PlanovaniKapacit.Datum <= ' . $params ->addDateParameter($datumDo);

        if ($pp_rc instanceof PracovniPomer) {
            $sql .=
                '   AND PlanovaniKapacit.RC = ' . $params->addParameter($pp_rc->rc) .
                '   AND PlanovaniKapacit.OsobniCislo = ' . $params->addParameter($pp_rc->osobniCislo) .
                '   AND PlanovaniKapacit.DatumNastupu = ' . $params->addDateParameter($pp_rc->datumNastupu);
        } else {
            $sql .=
                '   AND PlanovaniKapacit.RC = ' . $params->addParameter($pp_rc);
        }

        $sql .=
            ' ORDER BY PlanovaniKapacit.Datum ';

        MessageManager::addDebugLog('nactiSmenyPodleKapacit: ' . $sql);
        return self::nactiSmenySQL($database, $sql, $params, true);
    }


    /**
     * Nacte smeny osoby z prirazenych kalendaru
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @param string $filter
     * @return SmenaProDen[]
     */
    static function nactiSmenyPodleKalendare($database, $pp, $datumOd, $datumDo, $filter = '')
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT SmenaProDenKalendare.Den, Smena.Kod, Smena.Popis, Smena.Variabilni,' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny,' . // 4..6
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevneCasti,' . // 7..9
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny,' . // 10..12 - pro variabilni smenu
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo, Smena.DenDochPodleKonceSmeny' . // 13..16
            '  FROM PrirazeniKalendare, SmenaProDenKalendare, Smena' .
            ' WHERE SmenaProDenKalendare.Kalendar = PrirazeniKalendare.Kalendar' .
            '   AND PrirazeniKalendare.RC = ' . $params ->addParameter($pp ->rc) .
            '   AND PrirazeniKalendare.OsobniCislo = ' . $params ->addParameter($pp->osobniCislo) .
            '   AND PrirazeniKalendare.DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp->datumNastupu)).
            '   AND PrirazeniKalendare.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '   AND (PrirazeniKalendare.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR PrirazeniKalendare.PlatnostDo IS NULL) ' .
            '   AND SmenaProDenKalendare.Den >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '   AND SmenaProDenKalendare.Den <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '   AND PrirazeniKalendare.PlatnostOd <= SmenaProDenKalendare.Den' .
            '   AND (PrirazeniKalendare.PlatnostDo >= SmenaProDenKalendare.Den OR PrirazeniKalendare.PlatnostDo IS NULL)' .
            '   AND Smena.Kod = SmenaProDenKalendare.Smena' .
            ($filter ? ' AND (' . $filter . ')' : '') .
            ' ORDER BY SmenaProDenKalendare.Den';

        MessageManager::addDebugLog('nactiSmenyPodleKalendare: ' . $sql);
        return self::nactiSmenySQL($database, $sql, $params);
    }


    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @return SmenaProDen[]
     */
    static function nactiSmenyOsoby($database, $pp, $datumOd, $datumDo)
    {
        $params = $database ->createParameters();

        $sql =
            // smeny pro kalendare, ktere se nepocitaji podle planovani kapacit
            'SELECT SmenaProDenKalendare.Den, Smena.Kod, Smena.Popis, Smena.Variabilni,' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny,' . // 4..6
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevneCasti,' . // 7..9
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny,' . // 10..12 - pro variabilni smenu
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo, Smena.DenDochPodleKonceSmeny' . // 13..16
            "\nFROM PrirazeniKalendare, SmenaProDenKalendare, Smena, Kalendar\n" .
            ' WHERE SmenaProDenKalendare.Kalendar = PrirazeniKalendare.Kalendar ' .
            '   AND PrirazeniKalendare.RC = ' . $params ->addParameter($pp ->rc) .
            '   AND PrirazeniKalendare.OsobniCislo = ' . $params ->addParameter($pp->osobniCislo) .
            '   AND PrirazeniKalendare.DatumNastupu = ' . $params ->addDateParameter($pp->datumNastupu) .
            '   AND SmenaProDenKalendare.Den >= ' . $params ->addDateParameter($datumOd) .
            '   AND SmenaProDenKalendare.Den <= ' . $params ->addDateParameter($datumDo) .
            '   AND PrirazeniKalendare.PlatnostOd <= SmenaProDenKalendare.Den' .
            '   AND (PrirazeniKalendare.PlatnostDo >= SmenaProDenKalendare.Den OR PrirazeniKalendare.PlatnostDo IS NULL)' .
            '   AND Smena.Kod = SmenaProDenKalendare.Smena' .
            '   AND Kalendar.Kod = PrirazeniKalendare.Kalendar' .
            '   AND Kalendar.DochazkaPodlePlanKapacit = 0' .

            "\nUNION\n" .

            'SELECT PlanovaniKapacit.Datum, Smena.Kod, Smena.Popis, Smena.Variabilni,' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny,' . // 4..6
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevneCasti,' . // 7..9
            ' PlanovaniKapacit.Smena_Pocatek, PlanovaniKapacit.Smena_Konec, PlanovaniKapacit.Smena_Delka,' . // 10..12
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo, Smena.DenDochPodleKonceSmeny' . // 13..16
            "\nFROM PrirazeniKalendare, PlanovaniKapacit, Smena, Kalendar\n" .
            ' WHERE PlanovaniKapacit.Smena = Smena.Kod ' .
            '   AND PlanovaniKapacit.Datum >= ' . $params ->addDateParameter($datumOd) .
            '   AND PlanovaniKapacit.Datum <= ' . $params ->addDateParameter($datumDo) .
            '   AND PlanovaniKapacit.RC = ' . $params->addParameter($pp->rc) .
            '   AND PlanovaniKapacit.OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
            '   AND PlanovaniKapacit.DatumNastupu = ' . $params->addDateParameter($pp->datumNastupu) .
            '   AND PrirazeniKalendare.RC = ' . $params ->addParameter($pp ->rc) .
            '   AND PrirazeniKalendare.OsobniCislo = ' . $params ->addParameter($pp->osobniCislo) .
            '   AND PrirazeniKalendare.DatumNastupu = ' . $params ->addDateParameter($pp->datumNastupu) .
            '   AND PrirazeniKalendare.PlatnostOd <= PlanovaniKapacit.Datum' .
            '   AND (PrirazeniKalendare.PlatnostDo >= PlanovaniKapacit.Datum OR PrirazeniKalendare.PlatnostDo IS NULL)' .
            '   AND Kalendar.Kod = PrirazeniKalendare.Kalendar' .
            '   AND Kalendar.DochazkaPodlePlanKapacit = 1' .

            "\nORDER BY 1";

        MessageManager::addDebugLog('nactiSmenyOsoby: ' . $sql);
        return self::nactiSmenySQL($database, $sql, $params);
    }


    /**
     * Nacte smeny podle kalendare prirazeneho k rozdeleni zadane osoby
     * @param Database $database
     * @param int $kodTypuRozdeleni
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @return SmenaProDen[]
     */
    static function nactiSmenyPodleKalendareRozdeleni($database, $kodTypuRozdeleni, $pp, $datumOd, $datumDo)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT SmenaProDenKalendare.Den, Smena.Kod, Smena.Popis, Smena.Variabilni,' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny,' . // 4..6
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevneCasti,' . // 7..9
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny,' . // 10..12 - pro variabilni smenu
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo, Smena.DenDochPodleKonceSmeny' . // 13..16
            '  FROM RozdeleniOsob, Rozdeleni, SmenaProDenKalendare, Smena ' .
            ' WHERE SmenaProDenKalendare.Kalendar = Rozdeleni.Kalendar ' .
            '   AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni' .
            '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni' .
            '   AND Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter((int)$kodTypuRozdeleni) .
            '   AND RozdeleniOsob.RC = ' . $params ->addParameter($pp ->rc) .
            '   AND RozdeleniOsob.OsobniCislo = ' . $params ->addParameter($pp->osobniCislo) .
            '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp->datumNastupu)).
            '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
            '   AND SmenaProDenKalendare.Den >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '   AND SmenaProDenKalendare.Den <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '   AND RozdeleniOsob.PlatnostOd <= SmenaProDenKalendare.Den' .
            '   AND (RozdeleniOsob.PlatnostDo >= SmenaProDenKalendare.Den OR RozdeleniOsob.PlatnostDo IS NULL)' .
            '   AND Smena.Kod = SmenaProDenKalendare.Smena' .
            ' ORDER BY SmenaProDenKalendare.Den ';

        return self::nactiSmenySQL($database, $sql, $params);
    }


    /**
     * Nacte smeny podle kalendare prirazeneho ke zmenenemu rozdeleni osoby (prerazeni v planovani kapacit).
     * POZOR! Muze vracet i zaznamy, ktere maji vse krome datumu NULL (zamestnanec muze byt prerazen na jine
     * rozdeleni, ale jeste nemusi mit prirazenu smenu)
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $puvodniPlan
     * @return SmenaProDen[]
     */
    static function nactiSmenyPodleZmenenehoRozdeleni($database, $pp, $datumOd, $datumDo, $puvodniPlan)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT PlanovaniKapacit.Datum, Smena.Kod, Smena.Popis, Smena.Variabilni,' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny,' . // 4..6
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevneCasti,' . // 7..9
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny,' . // 10..12 - pro variabilni smenu
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo, Smena.DenDochPodleKonceSmeny' . // 13..16
            ' FROM PlanovaniKapacit INNER JOIN Rozdeleni ' .
            ($puvodniPlan
                ? ' ON Rozdeleni.KodTypuRozdeleni = PlanovaniKapacit.PuvodniPlan_KodTypu' .
                '   AND Rozdeleni.Kod = PlanovaniKapacit.PuvodniPlan_Rozdeleni'
                : ' ON Rozdeleni.KodTypuRozdeleni = PlanovaniKapacit.KodTypuRozdeleni' .
                '   AND Rozdeleni.Kod = PlanovaniKapacit.Rozdeleni'
            ) .
            ' LEFT OUTER JOIN SmenaProDenKalendare ' .
            '  ON SmenaProDenKalendare.Kalendar = Rozdeleni.Kalendar ' .
            '  AND SmenaProDenKalendare.Den = PlanovaniKapacit.Datum' .
            ' LEFT OUTER JOIN Smena ' .
            '  ON Smena.Kod = SmenaProDenKalendare.Smena' .
            ' WHERE PlanovaniKapacit.Datum <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '   AND PlanovaniKapacit.Datum >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '   AND PlanovaniKapacit.RC = ' . $params ->addParameter($pp ->rc) .
            '   AND PlanovaniKapacit.OsobniCislo = ' . $params ->addParameter($pp->osobniCislo) .
            '   AND PlanovaniKapacit.DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp->datumNastupu)).
            ' ORDER BY PlanovaniKapacit.Datum ';

        // POZOR! Vse krome datumu muze byt NULL!
        return self::nactiSmenySQL($database, $sql, $params);
    }

    /**
     * Vraci pocet smen zadaneho pracovniho pomeru v zadanem rozsahu datumu
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @return int
     * @throws DBException
     */
    public static function pocetSmenOsoby(Database $database, PracovniPomer $pp, $datumOd, $datumDo)
    {
        // Pocet podle kalendare
        $params = $database ->createParameters();
        $sql =
            'SELECT COUNT(SDK.Smena)' .
            ' FROM ((PrirazeniKalendare PK' .
            '   INNER JOIN Kalendar ON (Kalendar.Kod = PK.Kalendar))' .
            '   INNER JOIN SmenaProDenKalendare SDK ON (SDK.Kalendar = PK.Kalendar))' .
            ' WHERE SDK.Den >= ' . $params ->addDateParameter($datumOd) .
            '   AND SDK.Den <= ' . $params ->addDateParameter($datumDo) .
            '   AND PK.PlatnostOd <= SDK.Den' .
            '   AND (PK.PlatnostDo >= SDK.den  OR  PK.PlatnostDo IS NULL)' .
            '   AND Kalendar.DochazkaPodlePlanKapacit = 0' .
            '   AND PK.RC = ' . $params ->addParameter($pp ->rc) .
            '   AND PK.OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            '   AND PK.DatumNastupu = ' . $params ->addDateParameter($pp ->datumNastupu);
        $pocetDleKalendare = DBUtils::getSqlValue($database, $sql, $params);

        // Pocet smen z planovani kapacit
        $params2 = $database ->createParameters();
        $sql2 =
            ' SELECT COUNT(PLK.Smena)' .
            ' FROM (PrirazeniKalendare PK' .
            '   INNER JOIN PlanovaniKapacit PLK ON PLK.RC = PK.RC AND PLK.OsobniCislo = PK.OsobniCislo AND PLK.DatumNastupu = PK.DatumNastupu)' .
            '   INNER JOIN Kalendar ON Kalendar.Kod = PK.Kalendar' .
            ' WHERE PLK.Datum >= ' . $params2 ->addDateParameter($datumOd) .
            '   AND PLK.Datum <= ' . $params2 ->addDateParameter($datumDo) .
            '   AND PK.PlatnostOd <= PLK.Datum' .
            '   AND (PK.PlatnostDo >= PLK.Datum  OR  PK.PlatnostDo IS NULL)' .
            '   AND Kalendar.DochazkaPodlePlanKapacit = 1' .
            '   AND PK.RC = ' . $params2 ->addParameter($pp ->rc) .
            '   AND PK.OsobniCislo = ' . $params2 ->addParameter($pp ->osobniCislo) .
            '   AND PK.DatumNastupu = ' . $params2 ->addDateParameter($pp ->datumNastupu);
        $pocetDleKapacit = DBUtils::getSqlValue($database, $sql2, $params2);

        return $pocetDleKalendare + $pocetDleKapacit;
    }


    /**
     * Nacte smeny povolene v zadanem kalendari z tabulky PouzivaneSmeny - POZOR - nenacita $datum!
     * @param Database $database
     * @param string $kodKalendare
     * @return SmenaProDen[]
     */
    static function nactiPovoleneSmenyKalendare($database, $kodKalendare)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT NULL, Smena.Kod, Smena.Popis, Smena.Variabilni,' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny,' . // 4..6
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevneCasti,' . // 7..9
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny,' . // 10..12 - pro variabilni smenu
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo, Smena.DenDochPodleKonceSmeny' . // 13..16
            '  FROM PouzivaneSmeny, Smena ' .
            ' WHERE Smena.Kod = PouzivaneSmeny.KodSmeny' .
            '   AND PouzivaneSmeny.KodKalendare = ' . $params ->addParameter($kodKalendare) .
            ' ORDER BY PouzivaneSmeny.Poradi '; // kdyz jich je vic

        return self::nactiSmenySQL($database, $sql, $params);
    }


    /**
     * Sestavi retezec s popisem zaznamu podle zadaneho formatu
     * @param $format
     * @return array|mixed|string|string[]
     */
    public function format($format = null)
    {
        if (!isset($format)) {
            return
                '<span title="' . StringUtils::htmlTextConvert($this ->popis) . '">' . StringUtils::htmlTextConvert($this ->kod) . '</span>';
        }

        return str_replace(
            ['%POPIS%', '%KOD%', '%DELKA%', '%CASOD%', '%CASDO%'],
            [
                StringUtils::htmlTextConvert($this ->popis),
                StringUtils::htmlTextConvert($this ->kod),
                StringUtils::format('secondsofday', $this ->delkaSmeny),
                StringUtils::format('secondsofday', $this ->pocatekSmeny),
                StringUtils::format('secondsofday', $this ->konecSmeny),
            ],
            $format
        );
    }


    /**
     * Vraci true, pokud zadany cas spada do smeny (tj. smena zacina nejpozdeji v zadany cas a konci nejdrive v zadany cas)
     * @param int $cas  Muze to byt jen cas nebo i datum+cas
     * @return bool
     */
    public function smenaObsahujeCas(int $cas): bool
    {
        if ($cas < 0  ||  $cas >= 86400) {
            $cas = DateUtils::timeOf($cas);
        }

        return ($this ->pocatekSmeny <= $this ->konecSmeny)
            ? $cas >= $this->pocatekSmeny  &&  $cas <= $this ->konecSmeny
            : $cas >= $this->pocatekSmeny  ||  $cas <= $this ->konecSmeny;
    }


    /**
     * Vraci true, pokud zadany cas spada do nabehove doby od pocatku smeny
     * @param int $casPocatku   Muze byt zadan pouze cas nebo i datum+cas
     * @param int|null $nabehovaDoba
     * @return bool
     */
    public function pocatekJeVNabehoveDobe(int $casPocatku, ?int $nabehovaDoba)
    {
        $cas = ($casPocatku < 0  ||  $casPocatku >= 86400)
            ? DateUtils::timeOf($casPocatku)
            : $casPocatku;

        if (!$nabehovaDoba) {
            return $cas === $this ->pocatekSmeny;
        }

        if ($casPocatku >= $this ->pocatekSmeny - $nabehovaDoba  &&  $casPocatku <= $this ->pocatekSmeny + $nabehovaDoba) {
            // zadany cas pocatku spada do nabehove smeny (a neni to pres pulnoc)
            return true;
        }

        if ($this ->pocatekSmeny < $nabehovaDoba  &&  $casPocatku >= $this ->pocatekSmeny - $nabehovaDoba + 86400) {
            // nabehova doba zasahuje pred pulnoc a zadany $casPocatku je tesne pred pulnoci
            return true;
        }

        if ($this ->pocatekSmeny + $nabehovaDoba >= 86400  &&  $casPocatku + 86400 <= $this ->pocatekSmeny + $nabehovaDoba) {
            // nabehova doba zasahuje za pulnoc a zadany $casPocatku je tesne po pulnoci
            return true;
        }

        return false;

//        $pocitatCasOd = $this ->pocatekSmeny < $nabehovaDoba
//            ? $this ->pocatekSmeny - $nabehovaDoba + 24 * 3600 // pres pulnoc
//            : $this ->pocatekSmeny - $nabehovaDoba;
//        $pocitatCasDo = $this ->pocatekSmeny + $nabehovaDoba;
//        if ($pocitatCasDo > 86400) {
//            $pocitatCasDo = $pocitatCasDo - 86400; // pres pulnoc
//        }
//
//        return $pocitatCasDo < $pocitatCasOd
//            ? $cas <= $pocitatCasOd  &&  $cas >= $pocitatCasDo  // pres pulnoc
//            : $cas >= $pocitatCasOd  &&  $cas <= $pocitatCasDo;
    }



}
