<?php
/**
 * stanislav.elbl, 01.03.2021
 */

namespace RonPortal\Dochazka;

use Ron\Controls\ComboBox;
use Ron\Controls\Option;
use Ron\Controls\Options;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\GuiUtils;
use Ron\Http\Csrf;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\Controller;
use Ron\MVC\InternalRedirect;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\ComboBoxZamestnanec;
use RonPortal\NastaveniUzivatele;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalApp;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalXMLEvents;
use RonPortal\Security;
use RonPortal\TypRozdeleni;
use RonPortal\User;
use RonPortal\XML\RonPortalXMLForm;

class VkladaniDochazkyController extends JsonController
{
    const TERMINAL_VYBRANY = 1;
    const TERMINAL_URL = 2;
    const TERMINAL_USER_UTILS = 3;
    const TERMINAL_RUCNE = 4;

    const SETTINGS_POCET_DNU_ZPETNE = 'pocetdnuzpetnevkladani';
    /** @var string     0 = nezapisovat terminal, 1 = vybrany terminal, 2 = podle URL, 3 = podle user_utils.php */
    const SETTINGS_ZPUSOB_TERMINAL = 'vkladani_zpusob_terminal';
    /** @var string     urcuje, jestli musi byt povinne zadan kod terminalu pri vkladani (pro zpusob 2) */
    const SETTINGS_POVINNY_TERMINAL = 'vkladani_povinnyterminal';
    const SETTINGS_TERMINAL = 'vkladani_terminal';
    /** @var string     co se ma pri vkladani zapsat do pole karta: 0 = nic, 1 = nazev pocitace (vychozi hodnota), 2 = IP adresa pocitace, 3 = x (java) */
    const SETTINGS_KARTA = 'vkladani_karta';
    /** @var string  jestli je povoleno vkladani pomoci "terminalu" (default=1) */
    const SETTINGS_POVOLIT_TERMINAL = 'vkladani_povolit_terminal';
    const SETTINGS_ROZSIRENA_HISTORIE = 'rozsirenahistoriepruchodu';


    /** @var  Database */
    var $database;

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        $this ->database = $database;
    }


    public static $povoleneOperace = null;

    /**
     * Vraci true, pokud uzivatel muze vkladat zadanou operaci
     * @param string $kod
     * @param bool   $historie  priznak, jestli se jedna o vkladani do historie
     * @return bool
     */
    public static function lzeVlozitOperaci($kod, $historie)
    {
        if (!isset(self::$povoleneOperace)) {
            self::$povoleneOperace = []; // prazdne pole => muze vse

            $prava = Prava::vratPrava($historie ? 'historie' : 'vkladani');
            if (trim($prava ->priznak3 ?? '') !== '') {
                $pomA = explode(';', $prava ->priznak3);
                foreach($pomA as $operace) {
                    self::$povoleneOperace[] = strtolower($operace);
                }
            }
        }

        if (!self::$povoleneOperace) {
            // uzivatel nema omezene operace
            return true;
        }

        return in_array(strtolower($kod), self::$povoleneOperace);
    }


    /**
     * Pouziva se v ajaxu i primo pri generovani html stranky - nacte seznam pruchodu, ktere se nabidnou ve vkladani dochazky
     * @param Database $database
     * @param int $idPP
     * @return Option[]
     * @throws DBException
     */
    public static function vyberPruchoduProVkladani($database, $idPP)
    {
        $zpracovani = DochazkaUtils::datumCasZpracovaniDochazky($database);
        if ($zpracovani < 0) {
            $zpracovani = DateUtils::startOfTheDay(time());
        }

        $minDatum = DateUtils::incDay($zpracovani, -RonPortalSettings::getIntValue(self::SETTINGS_POCET_DNU_ZPETNE));

        $idPP = (int)$idPP;

        $params = $database ->createParameters();
        $sql =
            'SELECT DatumCasPruchodu, Terminal, Terminal.Popis ' .
            ' FROM HistoriePruchodu, PracovniPomer, Terminal ' .
            ' WHERE HistoriePruchodu.RC = PracovniPomer.RC ' .
            '   AND HistoriePruchodu.OsobniCislo = PracovniPomer.OsobniCislo ' .
            '   AND HistoriePruchodu.DatumNastupu = PracovniPomer.DatumNastupu ' .
            '   AND PracovniPomer.Id = ' . $params ->addParameter((int)$idPP) .
            '   AND Terminal.Kod = HistoriePruchodu.Terminal ' .
            '   AND DatumCasPruchodu >= ' . $params ->addDateParameter($minDatum) .
            ' ORDER BY DatumCasPruchodu DESC ';

        $cursor = $database ->getSqlResult($sql, $params);

        $result = [];

        $newItem = new Option(0, TextManager::_('Aktualni_cas'));
        $result[] = $newItem;

        $class2 = false;
        $lastDay = 0;

        while ($row = $cursor ->fetch()) {
            $cas = $database ->stringToTimestamp($row[0]);
            if (DateUtils::startOfTheDay($cas) != $lastDay) {
                $class2 = !$class2;
                $lastDay = DateUtils::startOfTheDay($cas);
            }

            $newItem = new Option($cas, date('j.n.Y G:i', $cas) . " ({$row[2]})");
            $newItem ->cssClass = $class2 ? 'browsercolor1' : null;
            $result[] = $newItem;
        }
        $cursor ->close();

        return $result;
    }


    public static function GetTerminalVkladaniURL()
    {
        return $_SESSION['terminalprovkladani'];
    }


//    /**
//     * @param bool $lzeMenitCas
//     * @param bool $historie
//     * @throws DBException
//     */
//    private function nactiOperace($lzeMenitCas, $historie)
//    {
//        $seznamOperaci = OperaceDochazky::nactiOperace($this ->database,
//            'ManualneVkladat_Intraweb = 1', null, 'PopisProDochazku');
//
//        $vybranaOperace = isset($_POST['operace']) ? $_POST['operace'] : null;
//        $operace = []; // data pro javaScript
//
//        $operaceOptions = new Options();  // data pro comboBox
//        if (!isset($vybranaOperace)  ||  $vybranaOperace == '') {
//            $operaceOptions->data[] = new Option('', ' ');
//            $operace[] = ['kod' => '', 'popis' => '' ];
//        }
//
//        $jeEditovatelnaOperace = false; // priznak, ktery se nastavi na true, pokud je mozne u nektere operace editovat cas
//
//        foreach ($seznamOperaci as $operaceDochazky) {
//            if (self::lzeVlozitOperaci($operaceDochazky ->kod, $historie)) {
//                $operaceOptions->data[] = new Option($operaceDochazky ->kod, $operaceDochazky ->popisProDochazku);
//                $operace[] = [
//                    'kod' => StringUtils::jsonText($operaceDochazky ->kod),
//                    'popis' => StringUtils::jsonText($operaceDochazky ->popisProDochazku),
//                    'lzeNastavitCas' => (int)$operaceDochazky ->lzeNastavitCas
//                ];
//
//                if ((int)$operaceDochazky ->lzeNastavitCas) {
//                    $jeEditovatelnaOperace = true;
//                }
//            }
//        }
//
//        Model::addData('operaceOptions', $operaceOptions);
//        Model::addData('operace', $operace);
//
//        Model::addData('lzeMenitCas', $lzeMenitCas || $jeEditovatelnaOperace);
//        Model::addData('vybranaOperace', $vybranaOperace);
//    }


//    private function nactiRozdeleni($kodTypuRozdeleni)
//    {
//        // posledni zadanou hodnotu nacte z cookie - aby si na kazde pobocce pamatoval posledni zadanou hodnotu
//        $vybraneRozdeleni = $_POST['rozdeleni'] ?? $_COOKIE['vkladani_rozdeleni'] ?? null;
//
//        $params = $this ->database ->createParameters();
//
//        $datum =  time();
//
//        $sql =
//            'SELECT Kod, Popis FROM Rozdeleni' .
//            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter( (int)$kodTypuRozdeleni ) .
//            '   AND (DatumZruseni IS NULL  OR  DatumZruseni >= ' . $params ->addDateParameter($datum) . ')' .
//            ' ORDER BY Popis';
//
//        $cursor = $this ->database ->getSqlResult($sql, $params);
//        $rozdeleniOptions = new Options();
//        $rozdeleni = []; // data pro javaScript
//
//        if (!RonPortalSettings::getIntValue('vkladani_povinnerozdeleni')  ||  !isset($vybraneRozdeleni)) {
//            $rozdeleniOptions ->addOption(new Option('', ' '));
//            $rozdeleni[] = [
//                'kod' => '',
//                'popis' => ''
//            ];
//        }
//
//        while ($row = $cursor ->fetch()) {
//            $rozdeleniOptions ->addOption(new Option($row[0], $row[1]));
//            $rozdeleni[] = [
//                'kod' => StringUtils::jsonText($row[0]),
//                'popis' => StringUtils::jsonText($row[1])
//            ];
//        }
//
//        $cursor ->close();
//
//        Model::addData('rozdeleniOptions', $rozdeleniOptions);
//
//        $typRozdeleni = TypRozdeleni::getTypRozdeleni($this ->database, $kodTypuRozdeleni);
//        Model::addData('typRozdeleni', $typRozdeleni);
//        Model::addData('rozdeleni', $rozdeleni); // data pro javascript
//
//        Model::addData('vybraneRozdeleni', $vybraneRozdeleni);
//    }


    public static function nactiZvolenyTerminal($database)
    {
        if (RonPortalSettings::getIntValue(self::SETTINGS_ZPUSOB_TERMINAL) != self::TERMINAL_URL) {
            return null;
        }

        $kod = self::GetTerminalVkladaniURL();

        if (RonPortalSettings::getIntValue(self::SETTINGS_POVINNY_TERMINAL) && !$kod) {
            // terminal se zadava v URL a je povinny, ale neni zadan
            MessageManager::addErrorMessage( TextManager::_('Neni_terminal_vkladani') );
        }

        if ($kod) {
            $params = $database ->createParameters();
            $popis = DBUtils::getSqlValue($database,
                'SELECT Popis FROM Terminal WHERE Kod = ' . $params ->addParameter($kod),
                $params);

            if (!$popis) {
                MessageManager::addErrorMessage( sprintf(TextManager::_('Nenalezen_terminal_vkladani'), $kod) );
                return null;
            }

            return [
                'kod' => $kod,
                'popis' => $popis
            ];
        }

        return null;
    }

//    /**
//     * Zobrazi formular pro vlozeni operace
//     * @param bool $historie    Urcuje, jestli se jedna o vkladani do historie pruchodu
//     * @return ViewObject
//     * @throws \Exception
//     */
//    function formularProVlozeni($historie)
//    {
//        $prava = Prava::vratPrava($historie ? 'historie' : 'vkladani');
//
//        if (!Prava::jsouNejakePrava($prava)) {
//            throw new NeniPravoException();
//        }
//
//        if (RonPortalSettings::getIntValue(self::SETTINGS_ZPUSOB_TERMINAL) == self::TERMINAL_URL && RonPortalSettings::getIntValue(self::SETTINGS_POVINNY_TERMINAL) && !self::GetTerminalVkladaniURL()) {
//            // terminal se zadava v URL a je povinny, ale neni zadan
//            throw new \Exception( TextManager::_('Neni_terminal_vkladani') );
//        }
//
//        $terminal = self::nactiZvolenyTerminal($this ->database); // nacte kod a popis terminalu zvoleneho v URL
//        Model::addData('vybranyTerminal', $terminal);
//
//        Model::addData('historie', $historie);
//        Model::addData('pravaVkladani', $prava);
//        Model::addData('povolitViceDnu', !$historie  &&  $prava ->priznak);
//
//        $this ->nactiOperace($prava ->priznak, $historie);
//
//        $datumPrava = time();
//        if (isset($_POST['datumoperace'])  &&  $_POST['datumoperace']) {
//            try {
//                $datumPrava = DateUtils::dateStringToTimestamp($_POST['datumoperace']);
//            } catch (\Exception $e) {
//                // ignoruje = pouzije time()
//            }
//        }
//
//        Prava::nastavObdobiProPrava($datumPrava, $datumPrava);
//
//        $comboOsoba = new ComboBoxZamestnanec('osoba'); // jmeno promenne
//        $comboOsoba ->filtr = 'PracovniPomer.ZpracovatDochazku = 1';
//        $comboOsoba ->htmlParams['id'] = 'vyberOsobyProVkladani';
//        //$comboOsoba ->className = 'form-input';
//        $comboOsoba ->loadDataPP($this ->database,
//            $datumPrava, $datumPrava,
//            $prava, ' ', RonPortalSession::vybranaOsoba(), false); // $emptyrow = true, $selectedvalue, $vcetneexternich = false
//        $comboOsoba ->nastavOsobuDoSession();
//
//        Model::addData('comboOsoba', $comboOsoba);
//
//        if (!$historie  &&  RonPortalSettings::getIntValue(self::SETTINGS_PODLE_HISTORIE)) {
//            Model::addData('vyberPruchodu', self::vyberPruchoduProVkladani($this->database, RonPortalSession::vybranaOsoba()));
//        }
//
//        if ($typRozdeleni = RonPortalSettings::getIntValue('vkladani_rozdeleni')) {
//            $this ->nactiRozdeleni($typRozdeleni);
//        }
//
//        if (!$historie  &&  RonPortalSettings::getIntValue('vkladani_zpusob_terminal') == self::TERMINAL_RUCNE) {
//            // terminal vybere uzivatel
//            $vybranyTerminal = NastaveniUzivatele::getUserSettingsItem($this ->database, 'vkladani_vybrany_terminal');
//
//            $comboTerminal = new ComboBox('terminal');
//            $comboTerminal ->className = 'form-input'; // "app_combo ronform-field";
//            $comboTerminal ->htmlParams = 'id="vyberTerminaluProVkladani"';
//            $comboTerminal ->loadData($this ->database,
//                "SELECT Kod, Popis FROM Terminal WHERE Aktivni = 1 AND KodUrceni IN (1, 6, 7)",
//                $vybranyTerminal);
//            Model::addData('comboTerminal', $comboTerminal);
//        }
//
//        return new BaseViewObject('ronportal/dochazka/view_vkladani_dochazky.php');
//    }


    /**
     * @throws DBException
     */
    function terminalProVkladani()
    {
        if (isset($_GET['backaction'])  &&  ($_GET['backaction'] != '')) {
            Model::addData('backaction', StringUtils::secureString($_GET['backaction']));
        }

        $operaceList = OperaceDochazky::nactiOperace($this ->database, 'ManualneVkladat_Intraweb = 1', null, RonPortalSettings::getValue('vkladani_razeni_tlacitek') ?: 'Kod');

        $seznamPP = PracovniPomer::nactiPPOsoby($this ->database, User::getRC());
        if (!$seznamPP) { // nemelo by nastat
            throw new \Exception('Nemate platny pracovni pomer!');
        }

        $prava = Prava::vratPrava('vkladani');
        if (trim($prava ->priznak3 ?? '') !== '') {
            // uzivatel ma omezene operace, ktere muze vkladat - profiltruje seznam
            $newOperaceList = [];

            foreach($operaceList as $operace) {
                if ($this ->lzeVlozitOperaci($operace ->kod, false)) {
                    $newOperaceList[] = $operace;
                }
            }

            $operaceList = $newOperaceList;
        }

        Model::addData('operace', $operaceList);
        Model::addData('pracovniPomery', $seznamPP);

        $terminal = self::nactiZvolenyTerminal($this ->database); // nacte kod a popis terminalu zvoleneho v URL
        Model::addData('vybranyTerminal', $terminal);

        $vybranyPP = NastaveniUzivatele::getUserSettingsItem($this ->database, 'vkladani_vybrany_pp');
        if ($vybranyPP) {
            $nasel = false;
            // zkontroluje, jestli je platny
            for ($i = 0;  !$nasel && $i < count($seznamPP); $i++ ) {
                if ($seznamPP[$i] ->idPP == $vybranyPP) {
                    $nasel = true;
                }
            }

            if (!$nasel) {
                $vybranyPP = null;
            }
        }

        if (!$vybranyPP) {
            $vybranyPP = $seznamPP[0] ->idPP;
        }

        Model::addData('vybranyPP', (int)$vybranyPP);

        if (RonPortalSettings::getIntValue('vkladani_obarvit_tlacitka')) {
            // do hlavicky prida CSS pro obarveni tlacitek podle barvy operace
            $css = '';
            foreach ($operaceList as $operace) {
                if ($operace->barvaProVykresleni) {
                    $css .= '#tlacitko_operace_' . $operace->kod . ' { background-color: ' . GuiUtils::getHtmlColor($operace->barvaProVykresleni) . "}\n";
                }
            }

            if ($css) {
                HtmlHeaders::getInstance()->addHeader("<style>$css</style>");
            }
        }

        return new BaseViewObject('ronportal/dochazka/view_vkladani_terminal.php');
    }


    function zobrazVkladani()
    {
        if (!RonPortalSettings::getIntValue(self::SETTINGS_POVOLIT_TERMINAL, 1)) {
            // terminal neni vubec povolen
            $terminalMode = false;
        } else if (isset($_GET['terminalmode'])) {
            $terminalMode = (int)$_GET['terminalmode'];
            // zapamatuje si posledni volbu uzivatele
            NastaveniUzivatele::setUserSettingsItem($this ->database,
                'vkladani_pouzit_rozsirene',
                $terminalMode ? null : '1');
        } else {
            $terminalMode = !NastaveniUzivatele::getUserSettingsItem($this ->database, 'vkladani_pouzit_rozsirene');
        }

        if ($terminalMode) {
            return $this ->terminalProVkladani();
        }

        return $this ->formularVkladani(false);
    }


    /**
     * @return null|string
     */
    function vratKodTerminaluProVkladani()
    {
        switch ( RonPortalSettings::getIntValue(self::SETTINGS_ZPUSOB_TERMINAL) ) {
            case 0 :
                // nevkladat
                return null;

            case self::TERMINAL_VYBRANY :
                // vybrany terminal
                return RonPortalSettings::getValue(self::SETTINGS_TERMINAL) == '' ? null : RonPortalSettings::getValue(self::SETTINGS_TERMINAL);

            case self::TERMINAL_URL :
                // podle url
                return self::GetTerminalVkladaniURL();

            case self::TERMINAL_USER_UTILS :
                // podle user_utils.php
                require_once('user_utils.php');
                return \userGetTerminalCode($this ->database);

            case self::TERMINAL_RUCNE :
                if (isset($_POST['terminal'])) {
                    NastaveniUzivatele::setUserSettingsItem($this->database, 'vkladani_vybrany_terminal', $_POST['terminal']);
                    return $_POST['terminal'];
                } else {
                    return null;
                }

            default :
                return null;
        }
    }


    /**
     * Upravi zaznam v dochazce pro monitorovani
     * @param Database $database
     * @param PracovniPomer $pracovniPomer
     * @param int $datum
     * @param string $kodOperace
     * @param string $poznamka
     * @param string $terminal
     * @param string|object $logDescription
     * @throws DBException
     */
    public static function aktualizujMonitorovani($database, $pracovniPomer, $datum, $kodOperace, $poznamka, $terminal, $logDescription)
    {
        $operace = OperaceDochazky::nactiOperaci($database, $kodOperace);
        $pritomnost = (int)$operace ->pritomnostProMonitorovani;

        if ($pritomnost == 3) {
            // nemonitorovat
            return;
        }

        $rec = new RonPortalDBRecord($database, 'PuvDochProMonitorovani', $logDescription);
        $rec ->addStringField('RC', true, 15, $pracovniPomer ->rc);
        $rec ->addStringField('OsobniCislo', true, 15, $pracovniPomer ->osobniCislo);
        $rec ->addDateField('DatumNastupu', true, $pracovniPomer ->datumNastupu);

        $fieldCas = $rec ->addTimestampField('DatumCasOperace', false, $datum);
        $rec ->addStringField('KodOperace', false, 10, $kodOperace);
        $rec ->addStringField('Terminal', false, 10, $terminal);
        $rec ->addIntField('Pritomnost', $pritomnost >= 2, $pritomnost); // 2=ignorovat, 3=nemonitorovat
        $rec ->addStringField('Poznamka', false, 250, $poznamka ?: NULL);

        $params = $database ->createParameters();
        $rec ->setWhere(
            'RC = ' . $params->addParameter($pracovniPomer ->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pracovniPomer ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addDateParameter($pracovniPomer ->datumNastupu),
            $params
        );

        if ($rec ->loadRecord()) {
            if (!$fieldCas ->originalValue  ||  $fieldCas ->originalValue < $datum) {
                $rec ->updateRecord();
            }
        } else {
            $rec ->insertRecord();
        }
    }


    /**
     * @return string|null
     */
    public static function vratCisloKarty()
    {
        switch ( RonPortalSettings::getIntValue(VkladaniDochazkyController::SETTINGS_KARTA, 1) ) {
            case 1 : // nazev pocitace
                $karta = $_SERVER['REMOTE_ADDR'];
                // pokusi se prelozit ip adresu pocitace
                return $karta
                    ? trim(gethostbyaddr($karta))
                    : null;

            case 2 : // ip adresa
                return trim($_SERVER['REMOTE_ADDR']);

            case 3 :
                return trim($_POST['IPAdresaJava'] ?? '');

            case 0 :
            default :
                return null;
        }
    }


    /**
     * Vlozeni operace - pouziva se pri vlozeni pomoci "terminalu"
     * @param int $datum
     * @param string $operace
     * @param string $poznamka
     * @param PracovniPomer $pracovniPomer
     * @param string $terminal
     * @param bool $povolitUpdate
     * @throws DBException
     * @throws \Exception
     */
    function vlozJednuOperaci($datum, $operace, $poznamka, $pracovniPomer, $terminal)
    {
        $karta = self::vratCisloKarty();

        $rec = new RonPortalDBRecord($this ->database, 'PuvodniDochazka', $this);

        $rec ->addField('DatumCasOperace', 'timestamp', true, 0, $datum);

        $rec ->addField('RC', '', true, 15, $pracovniPomer ->rc);
        $rec ->addField('OsobniCislo', '', true, 15, $pracovniPomer ->osobniCislo);
        $rec ->addField('DatumNastupu', 'timestamp', true, 0, $pracovniPomer ->datumNastupu);
        $rec ->addField('', '', true, 0, '   ' . $pracovniPomer ->jmenoPrijmeniKratke);

        $rec ->addField('KodOperace', '', false, 10, $operace);
        $rec ->addField('VlozenoManualne', 'int', false, 0, 1);
        $rec ->addField('Uzivatel', '', false, 25, User::getLoginName());
        $rec ->addField('Karta', '', false, 30, trim($karta));

        $rec ->addField('Poznamka', '', false, 250, (trim($poznamka ?? '') === '') ? null : $poznamka);

        $rec ->addField('Terminal', '', false, 10, $terminal);

        if ( ($typRozdeleni = RonPortalSettings::getIntValue('vkladani_rozdeleni')) ) {
            $rec ->addField('KodTypuRozdeleni', 'int', false, 0, null);
            $rec ->addField('Rozdeleni', '', false, 30, null);
        }

        $params = $this ->database ->createParameters();
        $filtr =
            'RC = ' . $params ->addParameter($pracovniPomer ->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($pracovniPomer ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addDateParameter($pracovniPomer ->datumNastupu) .
            ' AND DatumCasOperace = ' . $params ->addTimestampParameter($datum);

        $xmlParams = ['idpp' => $pracovniPomer ->idPP, 'jmenoPrijmeni' => $pracovniPomer ->jmenoPrijmeni];
        if ($rec ->loadRecord($filtr, $params)) {
            throw new \Exception(TextManager::_('Chyba_vkladani_puvodni_existuje'));
        }

        $rec ->executeXmlEvents(RonPortalXMLEvents::VLOZENI_BEFORE, 'insert', $xmlParams);
        $rec->insertRecord();
        $rec ->executeXmlEvents(RonPortalXMLEvents::VLOZENI_AFTER, 'insert', $xmlParams);

        $this ->aktualizujMonitorovani($this ->database, $pracovniPomer, $datum, $operace, $poznamka, $terminal, $this);
    }


//    /**
//     * @param int $datumOd
//     * @param int $datumDo
//     * @param string $casStr Pozor! Cas je zadan jako retezec, protoze pri prevodu na timestamp zalezi na datumu (kvuli letnimu a zimnimu casu)
//     * @param string $operace
//     * @param string $poznamka
//     * @param PracovniPomer $pracovniPomer
//     * @param string $rozdeleni
//     * @param string $terminal
//     * @param bool $povolitUpdate
//     * @throws NeniPravoException
//     * @throws \Exception
//     */
//    function provedVlozeni($datumOd, $datumDo, $casStr, $operace, $poznamka, $pracovniPomer, $rozdeleni, $terminal, $povolitUpdate = false)
//    {
//        if (!self::lzeVlozitOperaci($operace, false)) { // historie=false
//            throw new NeniPravoException();
//        }
//
//        for ($datum = $datumOd;  $datum <= $datumDo;  $datum = DateUtils::incDay($datum, 1)) {
//            $cas = DateUtils::timeStringToTimestamp($casStr, $datum);
//
//            $this ->vlozJednuOperaci($cas, $operace, $poznamka, $pracovniPomer, $rozdeleni, $terminal, $povolitUpdate);
//
//            MessageManager::addInfoMessage( sprintf(TextManager::_('Vlozeni_probehlo'), date('j.n.Y G:i', $cas)) );
//        }
//    }


//    /**
//     * @param int $datum
//     * @param string $operace
//     * @param string $poznamka
//     * @param PracovniPomer $pracovniPomer
//     * @throws \Exception
//     */
//    function vlozDataDoHistorie($datum, $operace, $poznamka, $pracovniPomer)
//    {
//        $prirazeniKarty = PrirazeniKarty::vratCisloKarty(
//            $this ->database, $datum,
//            $pracovniPomer ->rc, $pracovniPomer ->osobniCislo, $pracovniPomer ->datumNastupu);
//
//        if (!$prirazeniKarty) {
//            throw new \Exception( TextManager::_('Neni_prirazena_karta') );
//        }
//
//        $terminal = RonPortalSettings::getValue(RonPortalSettings::HISTORIE_TERMINAL);
//
//        $rec = new RonPortalDBRecord($this ->database, 'HistoriePruchodu', $this);
//        $rec ->addField('Terminal', '', false, 10, $terminal);
//        $rec ->addField('DatumCasPruchodu', 'timestamp', false, 0, $datum);
//        $rec ->addField('Karta', '', false, 30, $prirazeniKarty ->kod);
//        $rec ->addField('KodOperace', '', false, 10, $operace);
//        $rec ->addField('Uspesny', 'int', false, 0, 1);
//        $rec ->addField('RC', '', false, 15, $pracovniPomer ->rc);
//        $rec ->addField('OsobniCislo', '', false, 15, $pracovniPomer ->osobniCislo);
//        $rec ->addField('DatumNastupu', 'timestamp', false, 0, $pracovniPomer ->datumNastupu);
//        $rec ->addField('', '', false, 0, '   ' . $pracovniPomer ->jmenoPrijmeni);
//        $rec ->addField('VlozenoManualne', 'int', false, 0, 1);
//        $rec ->addField('Uzivatel', '', false, 20, User::getLoginName());
//
//        if ($poznamka) {
//            $rec ->addField('Poznamka', '', false, 250, $poznamka);
//        }
//
//        $rec ->insertRecord();
//
//        $this ->aktualizujMonitorovani($this ->database, $pracovniPomer, $datum, $operace, $poznamka, $terminal, $this);
//    }


//    /**
//     * Overi data zadana uzivatelem a provede vlozeni
//     * @param $historie
//     * @return bool|int|mixed|resource|void
//     * @throws \Exception
//     */
//    function vlozDataDoDochazky($historie)
//    {
//        if ( isset($_POST['zapnoutvicednu'])  &&  $_POST['zapnoutvicednu'] ) {
//            return $this ->formularProVlozeni($historie);
//        }
//
//        $vicednu = !$historie && $_POST['vicednu'];
//        $prava = Prava::vratPrava($historie ? 'historie' : 'vkladani');
//
//        Prava::nastavObdobiProPrava(time(), time());
//
//        // zkontroluje, jestli muze zamestnanec vkladat dochazku
//        if (!$prava ->jsouNejakePrava()) {
//            throw new NeniPravoException();
//        }

//        try {
            // zkontroluje, jestli jsou vyplnene vsechny udaje
//            if (!isset($_POST['operace']) || $_POST['operace'] === '') {
//                throw new RonException(TextManager::_('Nutno_zadat_operaci'), 'operace');
//            }

//            if (RonPortalSettings::getIntValue('vkladani_rozdeleni')) {
//                if (RonPortalSettings::getIntValue('vkladani_povinnerozdeleni')) {
//                    if (!isset($_POST['rozdeleni']) || $_POST['rozdeleni'] === '') {
//                        $typRozdeleni = TypRozdeleni::getTypRozdeleni($this->database, RonPortalSettings::getIntValue('vkladani_rozdeleni'));
//                        throw new \Exception(sprintf(TextManager::_('Nutno_zadat_rozdeleni_vkladani'), $typRozdeleni->popis));
//                    }
//                }
//
//                if (isset($_POST['rozdeleni'])) {
//                    setcookie('vkladani_rozdeleni', $_POST['rozdeleni'], time() + 30000000);
//                }
//            }

//            $lzemenitcas = $prava->priznak;
//
//            if (!$lzemenitcas) {
//                $lzemenitcas = OperaceDochazky::lzeNastavitCasOperace($this ->database, $_POST['operace']);
//            }
//            $provest_update = false;

//            if ($lzemenitcas) {
//                if (!$_POST['datumoperace']) {
//                    throw new RonException(TextManager::_('Musite_zadat_datum_operace'), 'datumoperace');
//                }
//                if (!$_POST['casoperace']) {
//                    throw new RonException(TextManager::_('Nutno_zadat_cas_operace'), 'casoperace');
//                }
//                if ($vicednu && !$_POST['datumoperacedo']) {
//                    throw new RonException(TextManager::_('Nutno_zadat_datum_operace_do'), 'datumoperacedo');
//                }
//            } else {
//                $vicednu = false;
//
//                if (RonPortalSettings::getIntValue(self::SETTINGS_PODLE_HISTORIE) && ($_POST['pruchody'] ?? false)) {
//                    // je vybran zaznam z historie pruchodu
//                    $_POST['datumoperace'] = date('j.n.Y', $_POST['pruchody']);
//                    $_POST['casoperace'] = date('G:i:s', $_POST['pruchody']);
//
//                    $provest_update = true; // pokusi se provest update
//                } else {
//                    // pouzije se aktualni cas
//                    $_POST['datumoperace'] = date('j.n.Y');
//                    $_POST['casoperace'] = date('G:i:s');
//                }
//            }

//            $ppomer = PracovniPomer::nactiPP($this ->database, (int)$_POST['osoba']);
//
//            if (!$ppomer->vyberNactenyPP($this->database, $prava)) {
//                throw new NeniPravoException();
//            }

//            // zkontroluje spravnost zadanych dat a casu
//            try {
//                $datum = DateUtils::dateStringToTimestamp($_POST['datumoperace']);
//            } catch (\Exception $e) {
//                throw new RonException($e ->getMessage(), 'datumoperace');
//            }
//
//            try {
//                $cas = DateUtils::timeStringToTimestamp($_POST['casoperace'], $datum);
//                // Pozor! $cas uz obsahuje i datum
//            } catch (\Exception $e) {
//                throw new RonException($e ->getMessage(), 'casoperace');
//            }
//
//            if ($vicednu) {
//                try {
//                    $datumdo = DateUtils::dateStringToTimestamp($_POST['datumoperacedo']);
//                } catch (\Exception $e) {
//                    throw new \Exception('Nespravne_koncove_datum');
//                }
//
//                if ($datumdo < $datum) {
//                    throw new \Exception(TextManager::_('Pocatecni_datum_vetsi_nez_koncove'));
//                }
//            } else {
//                $datumdo = $datum;
//            }

//            // zkontroluje jestli nebylo zadano datum mensi nez posledni zpracovani
//            $zpracovani = DochazkaUtils::datumCasZpracovaniDochazky($this->database);
//
//            if (DateUtils::incDay($datum, RonPortalSettings::getIntValue(self::SETTINGS_POCET_DNU_ZPETNE)) <= $zpracovani) {
//                throw new \Exception(TextManager::_('Datum_mensi_nez_posledni_zpracovani'));
//            }

//            if (RonPortalSettings::getIntValue(self::SETTINGS_KONTROLA_SCHVALENI) > 0) {
//                // zkontroluje, jestli se nevklada do schvalene dochazky
//                // ma smysl predevsim kdyz je povoleno zpetne vkladani
//                $schvaleni = new SchvaleniDochazky();
//                $schvaleni->load($this->database, $ppomer->rc, $ppomer->osobniCislo, $ppomer->datumNastupu, $datum, $datumdo);
//
//                switch (RonPortalSettings::getIntValue(self::SETTINGS_KONTROLA_SCHVALENI)) {
//                    case 1 :
//                        if ($schvaleni && $datum <= $schvaleni->schvaleni3) {
//                            throw new \Exception(TextManager::_('Datum_mensi_nez_schvaleni'));
//                        }
//
//                    case 2 :
//                        if ($schvaleni && $datum <= $schvaleni->schvaleni2) {
//                            throw new \Exception(TextManager::_('Datum_mensi_nez_zkontrolovani'));
//                        }
//
//                    case 3 :
//                        if ($schvaleni && $datum <= $schvaleni->schvaleni1) {
//                            throw new \Exception(TextManager::_('Datum_mensi_nez_zpracovano_do'));
//                        }
//                }
//            }
//
//            Security::validateCSRFPostParam();

            // provede vlozeni
//            if ($historie) {
//                $this->vlozDataDoHistorie($cas, $_POST['operace'], $_POST['poznamka'], $ppomer);
//                return new Redirect(null, '?menuid=historiepruchodu');
//            } else {
//                if (RonPortalSettings::getIntValue(self::SETTINGS_ZPUSOB_TERMINAL) == self::TERMINAL_URL
//                    && RonPortalSettings::getIntValue(self::SETTINGS_POVINNY_TERMINAL)
//                    && !self::GetTerminalVkladaniURL()
//                ) {
//                    throw new \Exception(TextManager::_('Neni_terminal_vkladani'));
//                }

//                $rozdeleni = null;
//                if (RonPortalSettings::getIntValue('vkladani_rozdeleni')) {
//                    $rozdeleni = $_POST['rozdeleni'];
//                }

//                $this->provedVlozeni(
//                    $datum, $datumdo,
//                    $_POST['casoperace'], $_POST['operace'], $_POST['poznamka'], $ppomer, $rozdeleni,
//                    $this->vratKodTerminaluProVkladani(),
//                    $provest_update);

//                if (isset($_POST['bezuzavreni']) && $_POST['bezuzavreni']) {
//                    $_POST['operace'] = '';
//                    return $this->formularProVlozeni(false);
//                } else {
//                    if (RonPortalXMLForm::$eventRedirect) {
//                        // Nektery z volanych XML formularu nastavil redirect
//                        return RonPortalXMLForm::$eventRedirect;
//                    }
//
//                    if (!Prava::vratPrava('prohlizeni') ->jsouNejakePrava()) {
//                        return $this->formularProVlozeni(false);
//                    }
//
//                    if (isset($_POST['backaction'])  &&  ($_POST['backaction'] != '')) {
//                        return new Redirect(null, '?menuid=' . StringUtils::secureString($_POST['backaction']));
//                    }
//
//                    return new Redirect(null, '?menuid=dochazkazamestnance&osoba=' . $ppomer ->idPP);
//                }
//            }
//        } catch (DBException $e) {
//            MessageManager::addErrorMessage(RonPortalApp::getDebugMode()
//                ? $e->getMessage()
//                : TextManager::_('Chyba_pri_vkladani_puvodni')
//            );
//            return $this->formularProVlozeni($historie);
//        } catch (\Exception $e) {
//            MessageManager::addErrorMessage($e);
//            return $this ->formularProVlozeni($historie);
//        }
//    }


    function vlozPodleTerminalu()
    {
        try {
            Prava::nastavObdobiProPrava(time(), time());
            $operace = $_POST['kodOperace'];


            if (!self::lzeVlozitOperaci($operace, false)) { // historie=false
                throw new NeniPravoException();
            }

            $pPomer = PracovniPomer::nactiPP($this->database, (int)$_POST['idpp']);

            $prava = Prava::vratPrava('vkladani');
            if (!$prava ->lzeZobrazitPP($pPomer ->idPP)) {
                throw new NeniPravoException();
            }

            Csrf::validateCSRFPostParam('vkladani_terminal');

            $cas = time();
            $this->vlozJednuOperaci($cas, $operace, $_POST['poznamka'], $pPomer, $this->vratKodTerminaluProVkladani());
            MessageManager::addInfoMessage(sprintf(TextManager::_('Vlozeni_probehlo'), date('j.n.Y G:i', $cas)));

            if (RonPortalXMLForm::$eventRedirect) {
                // Nektery z volanych XML formularu nastavil redirect
                return RonPortalXMLForm::$eventRedirect;
            }

            if (isset($_POST['backaction'])  &&  ($_POST['backaction'] != '')) {
                return new Redirect(null, '?menuid=' . StringUtils::secureString($_POST['backaction']));
            }

            if (Prava::jsouNejakePrava('prohlizeni')) {
                return new Redirect(null, '?menuid=dochazkazamestnance&osoba=' . $pPomer ->idPP);
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        if (isset($_POST['backaction'])  &&  ($_POST['backaction'] != '')) {
            return new Redirect(null, '?menuid=' . StringUtils::secureString($_POST['backaction']));
        }

        return new Redirect(null, '?menuid=vlozeni&terminalmode=1');
    }


    /**
     * Zobrazi formular pro vkladani nebo zpracuje data odeslana timto formularem a vraci redirect
     * @param bool $historie
     * @return Redirect|VkladaniDochazkyForm|null
     * @throws \Exception
     */
    private function formularVkladani($historie)
    {
        $form = new VkladaniDochazkyForm($this ->database, $historie);
        if (!$form ->handleAction()) {
            return $form;
        }

        if (RonPortalXMLForm::$eventRedirect) {
            // Nektery z volanych XML formularu nastavil redirect
            return RonPortalXMLForm::$eventRedirect;
        }

        if ($historie) {
            return new Redirect(null, '?menuid=historiepruchodu&osoba=' . ($_POST['osoba'] ?? ''));
        }

        $backAction = StringUtils::secureString($_POST['backaction'] ?? '');

        if ($_POST['bezuzavreni'] ?? false) {
            // uzivatel kliknul na tlacitko "vlozit bez uzavreni"
            return new Redirect(null, 'menuid=' . ($historie ? 'historie' : 'vlozeni') . ($backAction ? "&backaction=$backAction" : ''));
        }

        if (isset($_POST['backaction'])  &&  ($_POST['backaction'] != '')) {
            return new Redirect(null, '?menuid=' . StringUtils::secureString($_POST['backaction']));
        }

        if (!Prava::vratPrava('prohlizeni') ->jsouNejakePrava()) {
            // uzivatel kliknul na tlacitko "vlozit bez uzavreni", nebo nema pravo na prohlizeni => redirect na stejny formular
            return new Redirect(null, 'menuid=' . ($historie ? 'historie' : 'vlozeni'));
        }

        return new Redirect(null, '?menuid=dochazkazamestnance&osoba=' . (int)($_POST['osoba'] ?? 0) );
    }


    /**
     * Nacte seznam pruchodu pro zadanou osobu
     * @param JsonOutputObject $output
     * @throws DBException
     */
    private function pruchodyProDen($output)
    {
        $idPP = $_GET['idpp'];
        Prava::nastavObdobiProPrava(time());
        if (!Prava::lzeZobrazitPP('vkladani', $idPP)) {
            throw new NeniPravoException();
        }

        $options = self::vyberPruchoduProVkladani($this ->database, $idPP); // hidden = false

        $jsonData = [];
        foreach ($options as $option) {
            $jsonData[] = $option ->getJson();
        }

        $output ->addData('options', $jsonData);
    }


    function handleJsonAction($action, $output)
    {
        switch ($_GET['action'] ?? '') {
            case 'pruchodyproden' :
                $this ->pruchodyProDen($output);
                break;
        }
    }


    /**
     * @param string $action
     * @return InternalRedirect|JsonOutputObject|Redirect|ViewObject
     * @throws \Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('dochazka.ini');
        HtmlHeaders::getInstance() ->addScript('script/dochazka.js');
        HtmlHeaders::getInstance() ->addCss('css/dochazka.css');
        HtmlHeaders::getInstance() ->setAppTitle( TextManager::_('Dochazka_title'));

        Prava::nastavObdobiProPrava(time(), time());

        if ($_POST['formaction'] ?? false) {
            return $this ->formularVkladani($action === 'historie');
        }

        if (($_POST['vkladaniaction'] ?? '') === 'terminal') {
            return $this ->vlozPodleTerminalu();
        }
//        if (isset($_POST['vkladaniaction'])) {
//            switch ($_POST['vkladaniaction']) {
//                case 'terminal' :
//                    return $this ->vlozPodleTerminalu();
//
//                case 'provedvlozeni' :
//                    return $this ->vlozDataDoDochazky(false);
//
//                case 'vlozdohistorie' :
//                    if (RonPortalSettings::getIntValue(self::SETTINGS_ROZSIRENA_HISTORIE)) {
//                        return new InternalRedirect('historierozsirena');
//                    } else {
//                        // tady by se nemel dostat
//                        throw new \Exception("Chyba v controlleru VkladaniDochazkyController!");
//                        //return $this->vlozDataDoDochazky(true);
//                    }
//            }
//        }

        if ( isset($_GET['action']) ) {
            return parent::handleAction($action);
        }

        switch ($action) {
            case 'vlozeni' :
                return $this ->zobrazVkladani();

            case 'historie' :
                if (RonPortalSettings::getIntValue(self::SETTINGS_ROZSIRENA_HISTORIE)) {
                    return new InternalRedirect('historierozsirena');
                }

                return $this ->formularVkladani(true);
        }

        throw new \Exception('VkladaniDochazkyController - neznama akce!');
    }
}
