<?php
/**
 * stanislav.elbl, 27.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DB\Database;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\PracovniPomer;
use RonPortal\Prava;

class ZaznamAbsence
{
    /** @var int */
    public $idPP;

    /** @var int */
    public $datumOd;
    /** @var int */
    public $datumDo;

    public $poradi;

    public $operaceDochazky;
    public $popisOperace;
    public $zkratkaProPlanovani;
    public $barvaProVykresleni;

    /** @var int  0=cela smena, 1/2=prvni/druha pulka, 3=cas od-do, 4=kontinualne */
    public $delka;
    public $casOd;
    public $casDo;

    public $schvaleni;

    /** @var string[]   Uzivatelska1 az Uzivatelska5*/
    public $uzivatelska;


    /**
     * @param Database $database
     * @param array $row
     */
    protected function nactiRadek($database, $row)
    {
        $this ->operaceDochazky = $row[0];
        $this ->popisOperace = TextManager::_('db_operacedochazky_popis_' . $row[0], $row[1]);
        $this ->zkratkaProPlanovani = $row[2];
        $this ->barvaProVykresleni = $row[3];

        $this ->poradi = $row[4];
        $this ->datumOd = $database->stringToTimestamp($row[5]);
        $this ->datumDo = $database->stringToTimestamp($row[6]);

        $this ->delka = $row[7];
        $this ->casOd = $database ->stringToTimestamp($row[8]);
        $this ->casDo = $database ->stringToTimestamp($row[9]);

        $this ->schvaleni = $row[10];

        $this ->uzivatelska = [1 => $row[11], 2 => $row[12], 3 => $row[13], 4 => $row[14], 5 => $row[15]];
    }

    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @return ZaznamAbsence[]
     */
    public static function nactiAbsenceOsoby($database, $pp, $datumOd, $datumDo)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT OperaceDochazky.Kod, OperaceDochazky.Popis, OperaceDochazky.ZkratkaProPlanovani, OperaceDochazky.BarvaProVykresleni, ' . // 0..3
            ' Absence.Poradi, Absence.DatumOd, Absence.DatumDo, Absence.Delka, Absence.CasOd, Absence.CasDo, Absence.Schvaleni,' . // 4..10
            ' Absence.Uzivatelska1, Absence.Uzivatelska2, Absence.Uzivatelska3, Absence.Uzivatelska4, Absence.Uzivatelska5' . // 11..15
            ' FROM Absence, OperaceDochazky ' .
            ' WHERE Absence.OperaceDochazky = OperaceDochazky.Kod ' .
            ' AND RC = ' . $params ->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
            ' AND DatumDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            ' AND DatumOd <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            ' AND Schvaleni <> 4  AND  Schvaleni <> 2 ' . // zrusene a zamitnute ignoruje
            ' ORDER BY DatumOd, Poradi';

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani absenci!');

        $result = [];
        while ($row = $cursor ->fetch()) {
            $absence = new static();
            $absence ->idPP = $pp ->idPP;
            $absence ->nactiRadek($database, $row);

            $result[] = $absence;
        }

        $cursor ->close();

        return $result;
    }


    public function format($format)
    {
        if (!$format) {
            return $this ->popisOperace;
        }

        switch ($this ->delka) {
            case 0 :
                $delkaStr = \Ron\ronText('Absence_cela_smena');
                break;

            case 1 :
                $delkaStr = \Ron\ronText('Absence_pulka_smeny1');
                break;

            case 2 :
                $delkaStr = \Ron\ronText('Absence_pulka_smeny2');
                break;

            case 3 :
                $delkaStr = StringUtils::format('time', $this ->casOd) . ' - ' . StringUtils::format('time', $this ->casDo);
                break;

            case 4 :
                $delkaStr =
                    StringUtils::format('date', $this ->datumOd) . ' ' . StringUtils::format('time', $this ->casOd) .
                    ' - ' .
                    StringUtils::format('date', $this ->datumDo) . ' ' . StringUtils::format('time', $this ->casDo);
                break;

            default :
                $delkaStr = '';
        }


        return str_replace(
            ['%KOD%', '%POPIS%', '%ZKRATKA%', '%DELKA%', '%UZIVATELSKA1%', '%UZIVATELSKA2%', '%UZIVATELSKA3%', '%UZIVATELSKA4%', '%UZIVATELSKA5%'],
            [
                StringUtils::htmlTextConvert($this ->operaceDochazky),
                StringUtils::htmlTextConvert($this ->popisOperace),
                StringUtils::htmlTextConvert($this ->zkratkaProPlanovani),
                $delkaStr,
                StringUtils::htmlTextConvert($this ->uzivatelska[1] ?: ''),
                StringUtils::htmlTextConvert($this ->uzivatelska[2] ?: ''),
                StringUtils::htmlTextConvert($this ->uzivatelska[3] ?: ''),
                StringUtils::htmlTextConvert($this ->uzivatelska[4] ?: ''),
                StringUtils::htmlTextConvert($this ->uzivatelska[5] ?: ''),
            ],
            $format);
    }


    public function html($format, $titleFormat, $element = 'span', $backAction='')
    {
        if (!$backAction) {
            $backAction = Prava::vratPrava('absence_schval') ->jsouNejakePrava() ? 'schvalovaniabsence' : 'planovani';
        }

        $title = $this ->format($titleFormat);
        $class = $this ->schvaleni ? 'absence_schvalena' : 'absence_neschvalena';
        return
            "<$element class='$class' title='$title'>" .
            "<a href='?menuid=absence&action=zobrazabsenci&idpp={$this ->idPP}&datumod={$this ->datumOd}&poradi={$this->poradi}&backaction=$backAction'>" .
            $this ->format($format) .
            "</a>" .
            "</$element>";
    }
}
