<?php
/**
 * pavel.rudol, 31.10.2022 11:27
 */

namespace RonPortal\Dovednosti;

use Exception;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\DataOutputObject;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\TextViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controller\PritomnostController;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\Dochazka\DochazkaUtils;
use RonPortal\System\FotoOutputObject;
use RonPortal\User;


class DovednostiController extends PritomnostController
{
    /** @var PravaNaOsoby */
    private $pravaZobrazeni;
    /** @var PravaNaOsoby */
    private $pravaEditace;
    /** @var PravaNaOsoby */
    private $pravaSmazani;

    /**
     * DovednostiController constructor.
     * @param Database $database
     * @throws Exception
     */
    public function __construct(Database $database)
    {
        TextManager::getInstance()->loadStrings('person.ini');

        Prava::nastavObdobiProPrava(time(), time());

        $this->pravaZobrazeni = Prava::vratPrava('doved_ziskane');
        $this->pravaEditace = Prava::vratPrava('doved_edit');
        $this->pravaSmazani = Prava::vratPrava('doved_smazani');

        parent::__construct($database, $this->pravaZobrazeni);

        HtmlHeaders::getInstance()->setIcon(RonPortalSettings::getValue('shortcut_icon') ?: 'images/icon_person.png');
        HtmlHeaders::getInstance()->addScript('script/personscripts.js');
        HtmlHeaders::getInstance()->addCss('css/person.css');
    }

    /**
     * Zjisti jestli je k ziskane dovednosti pripojen dokument.
     * @param Database $database
     * @param string $osobaRC
     * @param string $dovednost kod dovednosti
     * @param int $uroven kod urovne
     * @param int $datumZiskani datum ziskani dovednosti
     * @return bool true, pokud je k ziskane dovednosti pripojen dokument
     * @throws DBException
     * @throws Exception
     */
    public static function jePripojenDokumentDovednosti($database, $osobaRC, $dovednost, $uroven, $datumZiskani)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT COUNT(RC) FROM DokumentyOsoby
                 WHERE DokumentyOsoby.Dovednost_RC = " . $params->addParameter($osobaRC) . " 
                   AND DokumentyOsoby.Dovednost_Kod = " . $params->addParameter($dovednost) . "
                   AND DokumentyOsoby.Dovednost_Uroven = " . $params->addParameter($uroven) . "
                   AND DokumentyOsoby.Dovednost_DatumZiskani = " . $params->addParameter($database->timestampToParam($datumZiskani));

        $cursor = $database->getSqlResult($sql, $params);

        return ($cursor->fetch()[0] > 0);
    }

    /**
     * Provede smazani ziskane dovednosti osoby.
     * Pred smazaznim kontroluje pravo na zvolenou osobu a zda neni k dovednosti pripojen dokument.
     * @return Redirect
     * @throws Exception
     */
    private function zrusDovednost()
    {
        $params = $this->database->createParameters();
        $osobaRC = DBUtils::getSqlValue($this->database, 'SELECT RC FROM PracovniPomer WHERE ID = ' . $params->addParameter($_GET['osoba']), $params);
        $kodDovednosti = $_GET['dovednost'];
        $urovenDovednosti = (int)$_GET['uroven'];
        $datumZiskani = $_GET['datumziskani'];

        if (!$this->pravaSmazani->lzeZobrazitRC($osobaRC)) {
            throw new NeniPravoException();
        }

        if (static::jePripojenDokumentDovednosti($this->database, $osobaRC, $kodDovednosti, $urovenDovednosti, $datumZiskani)) {
            throw new Exception(TextManager::_('Dovednosti_nelze_smazat_dokument'));
        }

        $rec = new RonPortalDBRecord($this->database, 'DovednostOsoby');
        $rec->addField('RC', '', true);
        $rec->addField('KodDovednosti', '', true);
        $rec->addField('KodUrovne', 'int', true);
        $rec->addField('DatumZiskani', 'timestamp', true);

        $params = $this->database->createParameters();

        $sql =
            " RC = " . $params->addParameter($osobaRC) .
            " AND KodDovednosti = " . $params->addParameter($kodDovednosti) .
            " AND KodUrovne = " . $params->addParameter($urovenDovednosti) .
            " AND DatumZiskani = " . $params->addParameter($this->database->timestampToParam($datumZiskani));

        $rec->setWhere($sql, $params);

        try {
            if ($rec->deleteRecord()) {
                MessageManager::addInfoMessage(TextManager::_('Dovednosti_smazano'));
                return new Redirect('', '?menuid=maticedovednosti&action=dovednostiosoby&idosoby=' . (int)$_GET['osoba']);
            }
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        return $this->redirectToView();
    }

    /** Zobrazi fotografii osoby z databaze.
     * @param int $idOsoby id pracovniho pomeru osoby
     * @return FotoOutputObject
     * @throws NeniPravoException
     * @throws Exception
     * @throws DBException
     */
    private function downloadFoto($idOsoby)
    {
        $params = $this->database->createParameters();
        $osobaRC = DBUtils::getSqlValue($this->database, 'SELECT RC FROM PracovniPomer WHERE ID = ' . $params->addParameter($idOsoby), $params);

        if (!(($this->pravaEditace->lzeZobrazitRC($osobaRC, time(), time())) || ($this->pravaZobrazeni->lzeZobrazitRC($osobaRC, time(), time())))) {
            throw new NeniPravoException();
        }

        $foto = new FotoOutputObject($this->database);
        $foto->nactiIdFotkyOsoby($osobaRC);

        return $foto;
    }

    /** Umozni stahnout/zobrazit dokument pripojeny k dosazene dovednosti. Pred stazenim zkontroluje zda ma uzivatel pravo na osobu a zda je dokument skutecne pripojen k dosazene dovednosti.
     * @param int $idDokumentu
     * @param int $idOsoby
     * @return OutputObject
     * @throws Exception
     */
    private function downloadDokument($idDokumentu, $idOsoby)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_DOVEDNOSTI)) {
            throw new Exception('Tato funkce neni povolena!');
        }

        $sql =
            "SELECT Dokument.ID " .
            "FROM Dokument JOIN DokumentyOsoby ON (DokumentyOsoby.ID = Dokument.ID) " .
            "  JOIN DovednostOsoby ON (DovednostOsoby.RC = DokumentyOsoby.RC AND DovednostOsoby.KodDovednosti = DokumentyOsoby.Dovednost_Kod " .
            "       AND DovednostOsoby.KodUrovne = DokumentyOsoby.Dovednost_Uroven AND DovednostOsoby.DatumZiskani = DokumentyOsoby.Dovednost_DatumZiskani) " .
            "  JOIN TypDokumentu ON (TypDokumentu.Kod = Dokument.Typ) " .
            "WHERE Dokument.ID = " . (int)$idDokumentu .
            "  AND (Dokument.ZobrazitNaIntrawebu = 1  OR  (Dokument.ZobrazitNaIntrawebu = 2  AND  TypDokumentu.ZobrazitNaIntrawebu = 1)) " .
            "  AND DokumentyOsoby.RC = DovednostOsoby.RC " .
            "  AND EXISTS(SELECT RC FROM PracovniPomer WHERE PracovniPomer.RC = DokumentyOsoby.RC AND PracovniPomer.ID = " . (int)$idOsoby .
            "           AND ((" . $this->pravaEditace->sqlFiltr('PracovniPomer', time(), time()) . ") OR " .
            "                (" . $this->pravaZobrazeni->sqlFiltr('PracovniPomer', time(), time()) . ")) " .
            ")";

        $cursor = $this->database->getSqlResult($sql);
        $row = $cursor->fetch();
        if (!$row) {
            throw new Exception('Zadany dokument nebyl nalezen nebo na nej nemate pravo!');
        }

        return new DokumentOutputObject($this->database, $idDokumentu);
    }


    /** Nahraje dokument z dialogu do databaze a pripoji pripadne k dosazene dovednosti.
     * @param JsonOutputObject $output
     */
    function uploadDokument($output)
    {
        try {
            // zkontroluje prava
            $idOsoby = (int)$_POST['osoba'];

            if (!$this->pravaEditace->lzeZobrazitPP($idOsoby)) {
                throw new NeniPravoException();
            }

            // nacte RC osoby
            $rc = DBUtils::getSqlValue($this->database, 'SELECT RC FROM PracovniPomer WHERE ID = ' . $idOsoby);
            if ($rc === false) {
                throw new Exception(StringUtils::fromUTF8("Chyba při vkládání dokumentu osoby - nenalezen pracovní poměr, ID: $idOsoby!"));
            }

            // vlozi dokument do databaze
            $id = DokumentyUtils::vlozDokumentJson($this->database, 'soubor', $_POST['typ'], $_POST['popis'], $output, 'Dovednost osoby');

            //Dovednost osoby ve tvaru: kod dovednosti~kod urovne~datum ziskani
            $dovednost = explode('~', $_POST['Dovednost']);

            // Vlozi zaznam do vazebni tabulky
            $rec = new RonPortalDBRecord($this->database, 'DokumentyOsoby');
            $rec->addField('RC', '', false, 15, $rc);
            $rec->addField('Dovednost_RC', '', false, 15, $rc);
            $rec->addField('Dovednost_Kod', '', false, 40, $dovednost[0]);
            $rec->addField('Dovednost_Uroven', 'int', false, 0, (int)$dovednost[1]);
            $rec->addField('Dovednost_DatumZiskani', 'timestamp', false, 0, DateUtils::dateStringToTimestamp($dovednost[2]));
            $rec->addField('Id', 'int', false, 0, $id);
            $rec->insertRecord();

            $output ->addText('den', StringUtils::format('date', time()));

            $params = $this->database->createParameters();
            $output ->addText('dovednost',
                DBUtils::getSqlValue($this->database, 'SELECT Popis FROM Dovednost WHERE Kod = ' . $params->addParameter($dovednost[0]), $params));
            $params = $this->database->createParameters();
            $output ->addText('dovednosturoven', DBUtils::getSqlValue($this->database,
                'SELECT Popis FROM UrovenDovednosti WHERE KodDovednosti = ' . $params->addParameter($dovednost[0]) . ' AND KodUrovne = ' . $params->addParameter($dovednost[1]),
                $params));
        } catch (Exception $e) {
            $output->addError($e);
        }
    }

    /** Umožní stažení CSV souboru se seznamem dovedností z browseru dovedností.
     * @param int $idOsoby
     * @return DovednostiBrowser
     * @throws Exception
     * @throws DBException
     */
    private function downloadDovednostiCSV($idOsoby)
    {
        $b = new DovednostiBrowser($this->database, '?' . $_SERVER['QUERY_STRING'] . '&', $idOsoby);
        $b->load();

        return $b;
    }

    /** Provede vložení/ukončení typového pracovního místa (RozdeleniOsob - konkretni typ z nastaveni)
     * @return Redirect
     * @throws Exception
     */
    private function vlozeniAktualizaceTPM()
    {
        $idOsoby = $_POST['idosoby'];

        $tpm = (isset($_POST['tpm_kod']) && ($_POST['tpm_kod'] != '')) ? $_POST['tpm_kod'] : $_POST['typovemisto'];
        if ((isset($_POST['tpm_platnost_od'])) && ($_POST['tpm_platnost_od'] != '')) {
            $platnostOd = DateUtils::dateStringToTimestamp($_POST['tpm_platnost_od']);
            $platnostDo = DateUtils::dateStringToTimestamp($_POST['platnost_do']);
        } else {
            $platnostOd = DateUtils::dateStringToTimestamp($_POST['platnost_od']);
            $platnostDo = null;
        }

        $poznamka = isset($_POST['poznamka']) ? $_POST['poznamka'] : null;

        $pPomer = new PracovniPomer();
        $pPomer->nactiPPPodleId($this->database, $idOsoby);
        if (!$pPomer->lzeZobrazitPP($this->pravaEditace, $platnostOd, $platnostOd)) {
            throw new NeniPravoException();
        }

        $rec = new RonPortalDBRecord($this->database, 'RozdeleniOsob');
        $rec->addField('KodTypuRozdeleni', 'int', true, 0, RonPortalSettings::getIntValue('dovednosti_typ_rozdeleni_tpm'));
        $rec->addField('Rozdeleni', '', true, 30, $tpm);
        $rec->addField('RC', '', true, 15, $pPomer->rc);
        $rec->addField('OsobniCislo', '', true, 15, $pPomer->osobniCislo);
        $rec->addField('DatumNastupu', 'timestamp', true, 0, $pPomer->datumNastupu);
        $rec->addField('PlatnostOd', 'timestamp', true, 0, $platnostOd);
        $rec->addField('PlatnostDo', 'timestamp', false, 0, $platnostDo);
        $rec->addField('Poznamka', '', false, 80, $poznamka);

        $params = $this->database->createParameters();

        $sqlWhere =
            '     KodTypuRozdeleni = ' . $params->addParameter(RonPortalSettings::getIntValue('dovednosti_typ_rozdeleni_tpm')) .
            " AND Rozdeleni = " . $params->addParameter($tpm) .
            " AND RC = " . $params->addParameter($pPomer->rc) .
            " AND OsobniCislo = " . $params->addParameter($pPomer->osobniCislo) .
            " AND DatumNastupu = " . $params->addParameter($this->database->timestampToParam($pPomer->datumNastupu)) .
            " AND PlatnostOd = " . $params->addParameter($this->database->timestampToParam($platnostOd));

        $rec->setWhere($sqlWhere, $params);

        try {
            $rec->updateOrInsert();
        } catch (Exception $exception) {
            throw new Exception(StringUtils::fromUTF8('Chyba při vložení TPM: ' . $exception->getMessage()));
        }

        return new Redirect('', '?menuid=maticedovednosti&action=' . $_POST['dovednostaction'] . '&osoba=' . $idOsoby);
    }

    /**
     * Zajistí přesměrování po vložení podle práv.
     * @return Redirect
     */
    private function redirectToView()
    {
        if ($this->pravaZobrazeni->jsouNejakePrava()) {
            $action = '&action=' . $_SESSION['dovednostibrowser'] ?: 'pozadovane';
        } else {
            $action = '&action=dovednostiosoby';
        }

        return new Redirect(null, '?menuid=maticedovednosti' . $action);
    }

    /** Zobrazí přehled typových pracovních míst osoby s možností přidání a ukončení.
     * @param int $typRozdeleniTypovePracovniMisto
     * @param int $idOsoby
     * @throws DBException
     */
    private function typovaPracovniMistaOsoby($typRozdeleniTypovePracovniMisto, $idOsoby)
    {
        $b = new TypovePracovniMistoOsobyBrowser($this->database, '?' . $_SERVER['QUERY_STRING'] . '&', $idOsoby, $typRozdeleniTypovePracovniMisto);
        $b->load();

        Model::addViewObject($b);

        $params = $this->database->createParameters();

        $sqlTypoveMista =
            'SELECT Rozdeleni.Kod, Rozdeleni.Popis 
            FROM Rozdeleni 
                JOIN VazbaRozdeleni ON (VazbaRozdeleni.typPodrizene = Rozdeleni.KodTypuRozdeleni AND VazbaRozdeleni.kodPodrizene = Rozdeleni.Kod)
                JOIN Rozdeleni PM ON (VazbaRozdeleni.typNadrazene = PM.KodTypuRozdeleni AND VazbaRozdeleni.kodNadrazene = PM.Kod)
                JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = PM.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PM.Kod)
                JOIN PracovniPomer ON (PracovniPomer.RC = RozdeleniOsob.RC AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu)
            WHERE PM.KodTypuRozdeleni = -1 
              AND Rozdeleni.KodTypuRozdeleni = ' . $params->addParameter($typRozdeleniTypovePracovniMisto) . ' 
              AND PracovniPomer.DatumNastupu <=  ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) . ' 
              AND (PracovniPomer.DatumUkonceni >= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) . ' OR PracovniPomer.DatumUkonceni IS NULL)
              AND PracovniPomer.ID = ' . $params->addParameter($idOsoby) . '
              AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) . '
              AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) . ' OR RozdeleniOsob.PlatnostDo IS NULL)';

        $comboTypoveMista = new ComboBox('typovemisto');
        $comboTypoveMista->loadSQL($this->database, $sqlTypoveMista, $params);
        $comboTypoveMista->htmlParams = 'data-ron-select2="typovemisto_kod"';

        $comboTypoveMistaKod = new ComboBox('typovemisto_kod');
        $comboTypoveMistaKod->loadSQL($this->database, $sqlTypoveMista, $params, 0, 0);
        $comboTypoveMistaKod->htmlParams = 'data-ron-select2="typovemisto"';

        $pracPomer = new PracovniPomer();
        $pracPomer->nactiPPPodleId($this->database, $idOsoby);

        Model::addData('pracPomerOsoby', $pracPomer);
        Model::addData('combotypovemista', $comboTypoveMista);
        Model::addData('combotypovemistakod', $comboTypoveMistaKod);
        Model::addViewObject(new BaseViewObject('ronportal/dovednosti/view_novetpm.php'));
    }

    /** Zobrazí browser dovednosti osoby + seznam dokumentu osoby s možností vložit nový.
     * @param int $idOsoby
     * @param bool $zobrazitJmeno
     * @return string
     * @throws Exception
     */
    private function prehledDovednostiOsoby($idOsoby, $zobrazitJmeno = false)
    {
        if ($zobrazitJmeno) {
            $pp = PracovniPomer::nactiPP($this->database, $idOsoby);

            Model::addViewObject(new TextViewObject('<h1>' . TextManager::_('Dovednosti_detail_osoby') . '</h1>'));
            $table =
                "<table class='infotable'>" .
                "<tr><td><strong>" . StringUtils::htmlTextConvert($pp->jmenoPrijmeni) . "</strong></td></tr>" .
                "</table>" .

                "<br/>" .
                "<a href='?menuid=maticedovednosti&action=" . $_SESSION['dovednostibrowser'] . "'>" . TextManager::_('Zpet') . "</a>";

            Model::addViewObject(new TextViewObject($table));
        }

        $typRozdeleniTypovePracovniMisto = (int)RonPortalSettings::getIntValue('dovednosti_typ_rozdeleni_tpm');
        if ($typRozdeleniTypovePracovniMisto) {
            $this->typovaPracovniMistaOsoby($typRozdeleniTypovePracovniMisto, $idOsoby);
        }

        $b = new DovednostiBrowser($this->database, '?' . $_SERVER['QUERY_STRING'] . '&', $idOsoby);
        $b->load();

        Model::addViewObject($b);

        Model::addViewObject(new TextViewObject("<A id='dokumenty'></A>\n"));

        Model::addViewObject(new TextViewObject("<form id='b_dokumentydovednosti'>"));

        $b = new DokumentyDovednosti($this->database, $idOsoby); /*::zobrazDokumentyDovednosti($database, $idPP, $lzeEditovat);*/
        $b->load();

        Model::addViewObject($b);

        Model::addViewObject(new TextViewObject("</form>"));

        // Pokud lze editovat, zobrazi odkaz "vlozit novy"
        if ($this->pravaEditace->lzeZobrazitPP($idOsoby)) {
            Model::addViewObject(new TextViewObject("<br/>
                <div class='normaltext'><a href='#' class='show-dokument-dialog' >" . TextManager::_('Vlozit_novy_dokument') . "</a></div>"));

//            onclick='showDokumentDovednostiDialog(" . "$idOsoby);'

            $form = new NovyDokumentDovednostiForm($this->database, $idOsoby);
            Model::addViewObject($form);
        }

        return 'view/appview.php';
    }

    /** Zobrazi/zpracuje formular nove dovednosti.
     * @return Redirect|string
     * @throws Exception
     */
    private function dovednostiFormular()
    {
        if (!$this->pravaEditace->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        $idOsoby = (int)($_GET['osoba'] ?? ($_POST['osoba'] ?? 0));

        //pokud neni vybrana osoba, nastavi jako id pracovniho pomeru, prvni id platneho pracovniho pomeru prihlasene osoby
        //7.12.2023 uzivatel nemusi mit pravo na sebe, pote se nenastavi nic
        if ($idOsoby == 0 && $this->pravaEditace->lzeZobrazitRC(User::getRC())) {
            $idOsoby = PracovniPomer::nactiPPOsoby($this->database, User::getRC())[0]->idPP;
        }

        //pokud neni zadane id PP osoby v URL, pouzije se vybrana osoba ze session (pokud je nastavena)
        //22.8.2019 zruseno nastaveni osoby ze session, muze zpusobovat problem s pravy, kdy v jinem browseru vybere osobu na kterou zde nema pravo
        /*if (($idOsoby == 0) && (RonPortalSession::vybranaOsoba())) {
            $idOsoby = RonPortalSession::setVybranaOsoba()vybranaOsoba();
        }*/

        //uzivatel muze editovat dovednosti, pokud je zadana osoba, tak musi mit pravo primo na ni
        if (!(($idOsoby == 0) || $this->pravaEditace->lzeZobrazitPP($idOsoby))) {
            throw new NeniPravoException();
        }

        $form = new DovednostOsobyForm($this->database, $_GET['backurl'] ?? 'ziskane', $idOsoby);

        if ($form->handleAction()) {
            MessageManager::addInfoMessage(TextManager::_('Dovednosti_vlozeni_probehlo'));
            return $this->redirectToView();
        }

        $this->hlavickaVyberAkce();

        Model::addViewObject($form);

        return $this->prehledDovednostiOsoby($idOsoby);
    }

    /**
     * @return string
     * @throws Exception
     */
    private function dovednostiOsoby()
    {
        $idOsoby = 0;
        if (isset($_GET['osoba'])) {
            $idOsoby = $_GET['osoba'];
        }

        if (!$this->pravaZobrazeni->lzeZobrazitPP($idOsoby)) {
            throw new NeniPravoException();
        }

        return $this->prehledDovednostiOsoby($idOsoby, true);
    }

    /** Zobrazí přehled - matici získaných dovedností.
     * @return DataOutputObject|SeznamDovednostiOsobBrowser
     * @throws NeniPravoException
     * @throws Exception
     */
    private function ziskaneDovednosti()
    {
        if (!$this->pravaZobrazeni->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        $this->hlavickaVyberAkce();

        $matice = new SeznamDovednostiOsobBrowser($this->database, 'ziskane');

        $matice->loadZiskaneDovednosti();

        if (isset($_GET['format']) && $_GET['format'] == 'csv') {
            return $matice->outputCSV();
        }

        return $matice;
    }

    /** Zobrazí přehled - matici získaných dovedností.
     * @return DataOutputObject|SeznamDovednostiOsobBrowser
     * @throws NeniPravoException
     * @throws Exception
     */
    private function vyhledaniOsobDovednosti()
    {
        if (!$this->pravaZobrazeni->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        $this->hlavickaVyberAkce();

        $matice = new SeznamDovednostiOsobVyhledani($this->database, 'ziskane');

        $matice->loadZiskaneDovednosti();

        $matice->seradOsoby();

        if (isset($_GET['format']) && $_GET['format'] == 'csv') {
            return $matice->outputCSV();
        }

        return $matice;
    }

    /** Zobrazí přehled - matici požadovaných dovedností.
     * @return DataOutputObject|SeznamDovednostiOsobBrowser
     * @throws NeniPravoException
     * @throws Exception
     */
    private function pozadovaneDovednosti()
    {
        if (!$this->pravaZobrazeni->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        $this->hlavickaVyberAkce();

        $matice = new SeznamDovednostiOsobBrowser($this->database, 'pozadovane');

        $matice->loadPozadovaneDovednosti();

        if (isset($_GET['format']) && $_GET['format'] == 'csv') {
            return $matice->outputCSV();
        }

        if (isset($_GET['format']) && $_GET['format'] == 'csv') {
            return $matice->outputCSV();
        }

        return $matice;
    }

    /** Nacte data osoby pro formular, seznam urovni dovednosti, seznam dovednosti pro osobu (muze byt filtrovan dle typoveho pracovniho mista osoby)
     * @param $output
     * @return void
     */
    private function loadJSONData($output)
    {
        if (isset($_GET['dovednost'])) {
            $this->nactiUrovneDovednosti($output, $_GET['dovednost']);
        }

        if (isset($_GET['osoba'])) {
            $this->nactiDovednostiOsoby($output, $_GET['osoba']);
        }
    }

    /** Nacte seznam dovednosti pro osobu - muze byt odfiltrovan dle prirazeneho typoveho pracovniho mista osoby, pokud se pouziva.
     * @param JsonOutputObject $output
     * @param int $osoba ID PP osoby
     * @return void
     */
    private function nactiDovednostiOsoby($output, $osoba)
    {
        try {
            if (!$osoba) {
                throw new Exception(StringUtils::fromUTF8('Není vybrána osoba.'));
            }

            $filtrTPM = '';

            $typRozdeleniTPM = RonPortalSettings::getIntValue('dovednosti_typ_rozdeleni_tpm');

            $params = $this->database->createParameters();

            //pokud je v nastaveni zvolen typ rozdeleni pro typova pracovni mista, bude mozne zamestnanci zadat jen dovednosti
            // pozadovane pro rozdeleni (typove pracovni misto) zvoleneho zamestnance
            if (($typRozdeleniTPM != -1) && ($typRozdeleniTPM != 0)) {
                $filtrTPM =
                    ' AND EXISTS(SELECT Rozdeleni FROM RozdeleniOsob
                                   JOIN PracovniPomer ON (PracovniPomer.RC = RozdeleniOsob.RC AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo
                                        AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu)
                                   JOIN PozadavekNaDovednost ON (PozadavekNaDovednost.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni
                                        AND PozadavekNaDovednost.KodRozdeleni = RozdeleniOsob.Rozdeleni)
                                 WHERE PracovniPomer.ID = ' . $params->addParameter($osoba) .
                    '              AND RozdeleniOsob.KodTypuRozdeleni = ' . $params->addParameter($typRozdeleniTPM) .
                    '              AND RozdeleniOsob.PlatnostOd <= ' . $params->addDateParameter(DateUtils::today()) .
                    '              AND (RozdeleniOsob.PlatnostDo >= ' . $params->addDateParameter(DateUtils::today()) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
                    '              AND PozadavekNaDovednost.PlatnostOd <= ' . $params->addDateParameter(DateUtils::today()) .
                    '              AND (PozadavekNaDovednost.PlatnostDo >= ' . $params->addDateParameter(DateUtils::today()) . ' OR PozadavekNaDovednost.PlatnostDo IS NULL) ' .
                    '              AND PozadavekNaDovednost.KodDovednosti = Dovednost.Kod ' .
                    ')';
            }

            $sql = "SELECT Kod, Popis FROM Dovednost " .
                "WHERE Aktivni = 1" .
                "  AND EXISTS(SELECT KodUrovne FROM UrovenDovednosti WHERE KodDovednosti = Dovednost.Kod)" .
                //17.3.2017 RON-PR dovednosti, ktere jsou slozene nebo definovane hodnotou nelze rucne zadat
                "  AND Slozena = 0 " .
                "  AND DefinovanaHodnotou = 0
                    $filtrTPM
                    ORDER BY Popis";

            try {
                $cursor = $this->database->getSqlResult($sql, $params);
            } catch (Exception $exception) {
                throw new Exception(StringUtils::fromUTF8('Nelze načíst dovednosti.'));
            }

            $dovednosti = [];
            $urovneProDovednost = null;

            while ($row = $cursor->fetch()) {
                if (is_null($urovneProDovednost)) {
                    $urovneProDovednost = $row[0];
                }

                $dovednost =
                    ['kod' => StringUtils::jsonText($row[0]),
                    'popis' => StringUtils::jsonText($row[1])];
                $dovednosti[] = $dovednost;
            }

            $output->addData('dovednosti', $dovednosti);
            if (!is_null($urovneProDovednost)) {
                $this->nactiUrovneDovednosti($output, $urovneProDovednost);
            }
        } catch (Exception $e) {
            $output->addError($e);
        }
    }

    /** Nacte urovne dovednosti pro formular vlozeni dosazene dovednosti osoby.
     * @param JsonOutputObject $output
     * @param string $kodDovednosti kód dovednosti
     * @return void
     */
    private function nactiUrovneDovednosti($output, $kodDovednosti)
    {
        try {
            if (!$kodDovednosti) {
                throw new Exception(StringUtils::fromUTF8('Není vybrána dovednost.'));
            }

            $dovednost = new DovednostOsoby($kodDovednosti);

            try {
                $dovednost->nactiUrovne($this->database);
            } catch (Exception $exception) {
                throw new Exception(StringUtils::fromUTF8('Nelze načíst úrovně dovednosti.'));
            }

            $urovne = [];

            if (is_array($dovednost->urovneDovednosti)) {
                foreach ($dovednost->urovneDovednosti as $kodUrovne => $popisUrovne) {
                    $uroven = array(
                        'kod' => (int)$kodUrovne,
                        'popis' => StringUtils::jsonText($popisUrovne),
                    );
                    $urovne[] = $uroven;
                }
            }

            $output->addData('urovne', $urovne);
        } catch (Exception $e) {
            $output->addError($e);
        }
    }

    /** Zobrazi zalozky pro prepinani dosazenych/pozadovanych dovednosti.
     * @return void
     */
    private function hlavickaVyberAkce()
    {
        $sekce = [];
        if ($this->pravaZobrazeni->jsouNejakePrava())
            $sekce = ['ziskane' => TextManager::_('MI_Dovednosti_ziskane'), 'pozadovane' => TextManager::_('MI_Dovednosti_pozadovane'),
                'vyhledani' => TextManager::_('MI_Dovednosti_hledani')];
        if ($this->pravaEditace->jsouNejakePrava())
            $sekce = array_merge($sekce, ['novadovednost' => TextManager::_('MI_Dovednosti_nove')]);

        Model::addData('header', TextManager::_('MI_Dovednosti'));
        Model::addData('aktivni_sekce', (isset($_GET['action']) ? $_GET['action'] : (isset($_GET['formaction']) ? 'novadovednost' : '')));
        Model::addData('sekce', $sekce);
        Model::addData('url', '?menuid=maticedovednosti&action=');
        Model::addViewObject(new BaseViewObject('view/appsections.php'));
    }

    /** Zpracovani JSON pozadavku, nacteni dat (dovednosti, urovne), nahrani dokumentu, pritomnost osob (v nadrazenem controlleru).
     * @param $action
     * @param $output
     * @return void
     * @throws Exception
     */
    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'loaddata' :
                $this ->loadJSONData($output);
                break;
            case 'dokumentupload' :
                $this->uploadDokument($output);
                break;

            default :
                parent::handleJsonAction($action, $output); // pritomnost
        }
    }

    /** Zobrazí matici (přehled) dovedností/formulář pro vložení.
     * @return Redirect|SeznamDovednostiOsobBrowser|string
     * @throws NeniPravoException
     * @throws Exception
     */
    private function handleMaticeDovednosti()
    {
        if (isset($_POST['formaction'])) $formaction = $_POST['formaction'];
        else if (isset($_GET['formaction'])) $formaction = $_GET['formaction'];
        else $formaction = '';

        if ($formaction) {
            // formular (editace, insert)
            return $this->dovednostiFormular();
        }

        $dovednostiaction = $_GET['action'] ?? '';

        switch ($dovednostiaction) {
            case 'novadovednost':
                return $this->dovednostiFormular();
            case 'dovednostiosoby':
                return $this->dovednostiOsoby();
            case 'pozadovane' :
                return $this->pozadovaneDovednosti();
            case 'zrusitdovednost' :
                return $this->zrusDovednost();
            case 'vyhledani' :
                return $this->vyhledaniOsobDovednosti();
            case 'ziskane' :
            default :
                return $this->ziskaneDovednosti();
        }
    }


    /** Práce s dokumenty (nahrání/zobrazení) + stažení seznamu úrovní pro dovednosti.
     * @return DokumentOutputObject|DovednostiBrowser|FotoOutputObject|JsonOutputObject|OutputObject|Redirect
     * @throws Exception
     */
    private function handleDovednosti()
    {
        switch ($_GET['action']) {
            case 'pritomnost' :
            case 'loaddata' :
            case 'dokumentupload' :
                return parent::handleAction($_GET['action']);

            case 'foto' :
                return $this->downloadFoto((int)$_GET['osoba']);

            case 'download' :
                return $this->downloadDokument((int)$_GET['record'], (int)$_GET['dovednostosoby']);

            case 'downloadcsv' :
                return $this->downloadDovednostiCSV((int)$_GET['osoba']);

            case 'editujtpm' :
                return $this->vlozeniAktualizaceTPM();

        }

        throw new Exception('Chyba v DovednostiController::handleDovednosti! Neznama akce!');
    }

    /**
     * @param string $action
     * @return DataOutputObject|DokumentOutputObject|DovednostiBrowser|FotoOutputObject|JsonOutputObject|OutputObject|Redirect|SeznamDovednostiOsobBrowser|SeznamDovednostiOsobVyhledani|string
     * @throws Exception
     */
    public function handleAction($action)
    {
        HtmlHeaders::getInstance()
            ->setAppTitle('Dovednosti');

        //pokud je povoleno v aplikaci a uzivatel ma pravo na zobrazeni, nebo editaci dovednosti
        if (!(RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_DOVEDNOSTI) && ($this->pravaZobrazeni->jsouNejakePrava() || $this->pravaEditace->jsouNejakePrava()))) {
            throw new NeniPravoException();
        }

        switch ($action) {
            case 'dovednosti':
                return $this->handleDovednosti();
            case 'maticedovednosti':
                return $this->handleMaticeDovednosti();
        }

        throw new Exception('Chyba v DovednostiController::handleAction! Neznama akce!');
    }

}
