<?php
/**
 * pavel.rudol, 01.11.2022 17:50
 */

namespace RonPortal\Dovednosti;

use Exception;
use Ron\Controls\Option;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterItem;
use RonPortal\Controls\FilterFormFilterRozdeleni;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;

class DovednostiFilterForm extends RonPortalFilterForm
{
    public $vedouciJmenoPrijmeni;
    public $zastupceJmenoPrijmeni;

    /** @var string urcuje zda se jedna o 'pozadovane', nebo 'ziskane' dovednosti */
    private $typZobrazeni;

    private $vedouciIDosoby;
    private $zastupceIDosoby;

    private $typRozdeleniFiltr1;
    private $typRozdeleniPracMisto;

    private $zobrazitFiltrKalendar;

    /** @var FilterFormFilterRozdeleni */
    private $filterRozdeleni_PM;

    /** @var bool Urcuje, jestli se bude filtr pro rozdeleni 2 omezeny podle tabulky VazbaRozdeleni  */
    private $filtrovatRozdeleni2DleVazby;

    /**
     * DovednostiFilterForm constructor.
     * @param Database $database
     * @param string $typZobrazeni urcuje zda se jedna o 'pozadovane', nebo 'ziskane' dovednosti
     * @param int $typRozdeleniFiltr1
     * @param int $typRozdeleniPracMisto
     * @param bool $zobrazitFiltrKalendar
     */
    public function __construct($database, $typZobrazeni, $typRozdeleniFiltr1, $typRozdeleniPracMisto, $zobrazitFiltrKalendar)
    {
        parent::__construct($database, 'maticedovednosti');

        $this->typZobrazeni = $typZobrazeni;

        $this->setUseGet(true, ['menuid' => 'maticedovednosti', 'action' => $this->typZobrazeni]);

        $this->f_classname = 'filtercontainer dovednostifilter';

        $this->typRozdeleniFiltr1 = $typRozdeleniFiltr1;
        $this->typRozdeleniPracMisto = $typRozdeleniPracMisto;

        $this->filtrovatRozdeleni2DleVazby = $typRozdeleniPracMisto !== -1;

        $this ->addFilterRozdeleni(
            (new FilterFormFilterRozdeleni($this, 'rozdeleni'))
                ->setTypRozdeleni($this->typRozdeleniFiltr1)
                ->setJenVlastniRozdeleni(2)
                ->setPouzitPopisTypuRozdeleni(true)
                ->setPrava(Prava::vratPrava('doved_ziskane'))
        );

        $this ->filterRozdeleni_PM = $this ->addFilterRozdeleni(
            (new FilterFormFilterRozdeleni($this, 'rozdeleni2'))
                ->setTypRozdeleni($this->typRozdeleniPracMisto)
                ->setJenVlastniRozdeleni(0)
                ->setPouzitPopisTypuRozdeleni(true)
                ->setPrava(Prava::vratPrava('doved_ziskane'))
        );

//        $this->nastavRozdeleni(true, $this->typRozdeleniFiltr1, 2, '', true);
//        $this->nastavRozdeleni2(true, $this->typRozdeleniPracMisto, 0, '', true);
//        $this->f_pravarozdeleni = Prava::vratPrava('doved_ziskane');

        $this->zobrazitFiltrKalendar = $zobrazitFiltrKalendar;

        $this->addFilters();
    }

    /** Prida filtry pro prehled ziskanych/pozadovanych dovednosti. U vyhledani se nepouzije, filtry se pridaji jinym zpusobem.
     * @return void
     */
    protected function addFilters()
    {
        if ($this->zobrazitFiltrKalendar) {
            $filtrKalendar = new FilterFormFilter('kalendar', FilterFormFilter::TYPE_OPTIONS);
            $filtrKalendar
                ->setCaption(TextManager::_('Kalendar'))
                ->setItems(['' => new FilterFormFilterItem('', TextManager::_('_vse_'))])
                ->setOptionsSql('SELECT Kod, Popis FROM Kalendar WHERE Aktivni = 1 AND %FILTER% ORDER BY %ORDER%', '1 = 1', 'Popis')
                ->setAddClass('ronselect')
                ->setOptionsTranslationPrefix("db_kalendar_popis_")
                ->setSessionName('kalendar')
                ->setRowId('filterrow_kalendar');
            $this->addFilter($filtrKalendar);
        }

        $filtrTypDovednosti = new FilterFormFilter('typdovednosti', FilterFormFilter::TYPE_OPTIONS);
        $filtrTypDovednosti
            ->setCaption(TextManager::_('Dovednosti_typ_dovednosti'))
            ->setItems(['' => new FilterFormFilterItem('', TextManager::_('_vse_'))])
            ->setOptionsSql('SELECT Kod, Popis FROM TypDovednosti WHERE EXISTS(SELECT Kod FROM Dovednost WHERE Dovednost.TypDovednosti = TypDovednosti.Kod AND Aktivni = 1) AND %FILTER% ORDER BY %ORDER%',
                '1 = 1', 'Popis')
            ->setAddClass('ronselect')
            ->setOptionsTranslationPrefix("db_typdovednosti_popis_")
            ->setSessionName('typdovednosti')
            ->setRowId('filterrow_typdovednosti');
        $this->addFilter($filtrTypDovednosti);

        $filtrDovednost = new FilterFormFilter('dovednost', FilterFormFilter::TYPE_OPTIONS);
        $filtrDovednost
            ->setCaption(TextManager::_('Dovednosti'))
            ->setItems(['' => new FilterFormFilterItem('', TextManager::_('_vse_'))])
            ->setOptionsSql('SELECT Kod, Popis FROM Dovednost WHERE Aktivni = 1 AND %FILTER% ORDER BY %ORDER%',
                '1 = 1', 'Popis')
            ->setAddClass('ronselect')
            ->setOptionsTranslationPrefix("db_dovednost_popis_")
            ->setSessionName('dovednost')
            ->setRowId('filterrow_dovednost');
        $this->addFilter($filtrDovednost);

        $filtrUroven = new FilterFormFilter('urovendovednosti', FilterFormFilter::TYPE_OPTIONS);
        $filtrUroven
            ->setCaption(TextManager::_('Dovednosti_uroven_dovednosti'))
            ->setItems(['' => new FilterFormFilterItem('', TextManager::_('_vse_'))])
            ->setOptionsSql(
                'SELECT DISTINCT KodUrovne, KodUrovne KUP FROM UrovenDovednosti WHERE EXISTS(SELECT Kod FROM Dovednost WHERE Dovednost.Kod = UrovenDovednosti.KodDovednosti AND Aktivni = 1) AND %FILTER% ORDER BY %ORDER%',
                '1 = 1', 'KodUrovne')
            ->setAddClass('ronselect')
            ->setSessionName('urovendovednosti')
            ->setRowId('urovendovednosti');
        $this->addFilter($filtrUroven);

        if ($this->typZobrazeni == 'ziskane') {
            $filtrExpirace = new FilterFormFilter('expirace', FilterFormFilter::TYPE_CHECKBOX);
            $filtrExpirace
                ->setSessionName('expiracedovednosti')
                ->setCaption2(TextManager::_('Dovednosti_zobrazit_expiraci'));
            $this ->addFilter($filtrExpirace);
        }
    }


    public function loadConfigurationItem($key, $value, $valueText)
    {
        if ($key === 'filtrovatRozdeleni2') {
            $this ->filtrovatRozdeleni2DleVazby = (int)$value;
        } else {
            parent::loadConfigurationItem($key, $value, $valueText);
        }
    }


    /** Pro zvolene rozdeleni zobrazi fotografie a jmena vedouciho a zastupce.
     * @return void
     */
    function renderFoto()
    {
        if ($this->vedouciIDosoby) {
            echo
                "<td><div class='fotoDochazka ronFoto'><img src='?menuid=dovednosti&action=foto&osoba=$this->vedouciIDosoby' alt='vedouci'>" .
                "<strong>" . TextManager::_('Dovednosti_vedouci') . ": </strong>" . StringUtils::htmlTextConvert($this->vedouciJmenoPrijmeni) . "</div></td>";
        }
        if ($this->zastupceIDosoby) {
            echo
                "<td><div class='fotoDochazka ronFoto'><img src='?menuid=dovednosti&action=foto&osoba=$this->zastupceIDosoby' alt='zastupce'>" .
                "<strong>" . TextManager::_('Dovednosti_zastupce') . ": </strong>" . StringUtils::htmlTextConvert($this->zastupceJmenoPrijmeni) . "</div></td>";
        }
    }

    /** Nacte ID PP vedouciho a zastupce zvoleneho rozdeleni.
     * @throws DBException
     */
    private function loadVedouciRozdeleni()
    {
        if ($this->getFilterValue('rozdeleni')) {
            $params = $this->database->createParameters();
            $sql =
                'SELECT MAX(PP_Vedouci.ID), MAX(PP_Zastupce.ID), O_Vedouci.Jmeno, O_Vedouci.Prijmeni, O_Vedouci.Titul, O_Zastupce.Jmeno, O_Zastupce.Prijmeni, O_Zastupce.Titul
                 FROM Rozdeleni
                   LEFT OUTER JOIN PracovniPomer PP_Vedouci ON (PP_Vedouci.RC = Rozdeleni.VedouciRC)
                   LEFT OUTER JOIN Osoba O_Vedouci ON (O_Vedouci.RC = PP_Vedouci.RC)
                   LEFT OUTER JOIN PracovniPomer PP_Zastupce ON (PP_Zastupce.RC = Rozdeleni.ZastupceRC)
                   LEFT OUTER JOIN Osoba O_Zastupce ON (O_Zastupce.RC = PP_Zastupce.RC)
                 WHERE KodTypuRozdeleni = ' . $params->addParameter($this ->getFilterValue('rozdeleni', 'typRozdeleni')) .
                ' AND Kod = ' . $params->addParameter($this->getFilterValue('rozdeleni')) . ' ' .
                'GROUP BY O_Vedouci.Jmeno, O_Vedouci.Prijmeni, O_Vedouci.Titul, O_Zastupce.Jmeno, O_Zastupce.Prijmeni, O_Zastupce.Titul';

            $cursor = $this->database->getSqlResult($sql, $params);

            $row = $cursor->fetch();

            $this->vedouciIDosoby = $row[0];
            $this->vedouciJmenoPrijmeni = ($row[4] ? $row[4] . ' ' : '') . $row[3] . ' ' . $row[2];
            $this->zastupceIDosoby = $row[1];
            $this->zastupceJmenoPrijmeni = ($row[7] ? $row[7] . ' ' : '') . $row[6] . ' ' . $row[5];
        }
    }


    protected function afterLoadFilter($filter)
    {
        parent::afterLoadFilter($filter);

        if ($this ->filtrovatRozdeleni2DleVazby  &&   $this ->filterRozdeleni_PM ->enabled  &&  $filter instanceof FilterFormFilterRozdeleni  &&  $filter ->name === 'rozdeleni'  &&  $filter ->getValue() !== null) {
            // po nacteni hodnoty filtru "rozdeleni" profiltruje seznam pracovnich mist
            $params = $this->database->createParameters();

            //3.10.2017 odfiltrovat seznam TPM podle vybraneho rozdeleni ve filtru 1,
            // jen TPM, ktera maji zamestnanci (vybrani ve filtru 1) prirazene ve vazbe na jim prirazenem pracovnim miste
            $this ->filterRozdeleni_PM ->setOptionsSqlFilter(
                new SqlStatement(
                    'EXISTS(SELECT *' .
                    ' FROM RozdeleniOsob R_ZAM JOIN RozdeleniOsob R_PM ON (R_PM.RC = R_ZAM.RC AND R_PM.OsobniCislo = R_ZAM.OsobniCislo AND R_PM.DatumNastupu = R_ZAM.DatumNastupu)' .
                    ' JOIN VazbaRozdeleni ON (VazbaRozdeleni.typNadrazene = R_PM.KodTypuRozdeleni AND VazbaRozdeleni.kodNadrazene = R_PM.Rozdeleni)' .
                    ' WHERE R_ZAM.KodTypuRozdeleni = ' . $params->addParameter($filter->getTypRozdeleni()) .
                    '  AND R_ZAM.Rozdeleni = ' . $params->addParameter($filter->getValue()) .
                    '  AND R_ZAM.PlatnostOd <= ' . $params->addDateParameter(time()) .
                    '  AND (R_ZAM.PlatnostDo >= ' . $params->addDateParameter(time()) . ' OR R_ZAM.PlatnostDo IS NULL)' .
                    '  AND R_PM.KodTypuRozdeleni = ' . $params->addParameter(-1) .  //pracovni misto - vazany typ by mel byt vzdy na pracovni pozici -1
                    '  AND R_PM.PlatnostOd <= ' . $params->addDateParameter(time()) .
                    '  AND (R_PM.PlatnostDo >= ' . $params->addDateParameter(time()) . ' OR R_PM.PlatnostDo IS NULL)' .
                    '  AND VazbaRozdeleni.typPodrizene = Rozdeleni.KodTypuRozdeleni' .
                    '  AND VazbaRozdeleni.kodPodrizene = Rozdeleni.Kod' .
                    '  )',
                    $params
                )
            );
        }
    }


    /** Nacte filtry, na zaklade zvoleneho rozdeleni, muze nastavit filtr na typove pracovni misto (TPM), pokud se pouzivaji.
     * @throws Exception
     */
    function loadData()
    {
        parent::loadData();
/*
        $filtrTPM = '';

        //3.10.2017 odfiltrovat seznam TPM podle vybraneho rozdeleni ve filtru 1,
        // jen TPM, ktera maji zamestnanci (vybrani ve filtru 1) prirazene ve vazbe na jim prirazenem pracovnim miste
        if (($this->typRozdeleniPracMisto != -1) && ($this->getFilterValue('rozdeleni'))) {
            $params = $this->database->createParameters();

            $filtrTPM =
                'EXISTS(SELECT 1 ' .
                'FROM RozdeleniOsob R_ZAM JOIN RozdeleniOsob R_PM ON (R_PM.RC = R_ZAM.RC AND R_PM.OsobniCislo = R_ZAM.OsobniCislo AND R_PM.DatumNastupu = R_ZAM.DatumNastupu) ' .
                '   JOIN VazbaRozdeleni ON (VazbaRozdeleni.typNadrazene = R_PM.KodTypuRozdeleni AND VazbaRozdeleni.kodNadrazene = R_PM.Rozdeleni) ' .
                'WHERE R_ZAM.KodTypuRozdeleni = ' . $params->addParameter($this->typRozdeleniFiltr1) .
                "  AND R_ZAM.Rozdeleni = " . $params->addParameter($this->getFilterValue('rozdeleni')) .
                '  AND R_ZAM.PlatnostOd <= ' . $params->addTimestampParameter(time()) .
                '  AND (R_ZAM.PlatnostDo >= ' . $params->addTimestampParameter(time()) . ' OR R_ZAM.PlatnostDo IS NULL) ' .
                '  AND R_PM.KodTypuRozdeleni = -1' .  //pracovni misto
                '  AND R_PM.PlatnostOd <= ' . $params->addTimestampParameter(time()) .
                '  AND (R_PM.PlatnostDo >= ' . $params->addTimestampParameter(time()) . ' OR R_PM.PlatnostDo IS NULL) ' .
                '  AND VazbaRozdeleni.typPodrizene = Rozdeleni.KodTypuRozdeleni ' .
                '  AND VazbaRozdeleni.kodPodrizene = Rozdeleni.Kod ' .
                '  )';

            $this->f_filtrrozdeleni2 = new SqlStatement($filtrTPM, $params);
        }

        if ($this ->showRozdeleni2) $this ->loadRozdeleni2();
*/
        $this->loadVedouciRozdeleni();
    }
}