<?php
/**
 * stanislav.elbl, 24.03.2021
 */

namespace RonPortal\Dovednosti;

use Exception;
use Ron\Controls\ComboBox;
use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecordField;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\OsobaFormField;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Dokument;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\User;


class OsobaDovednostField extends OsobaFormField
{
    public $idPP;

    public function __construct($prava, $datumPlatnostiPP, $idPP = null)
    {
        parent::__construct($prava, $datumPlatnostiPP, $idPP);
        $this->idPP = $idPP;
    }


    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($this->idPP !== 0)
            $value = $this->idPP;

        //$canEdit = $this->editable;
        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
    }

}

/**
 * Zobrazi pole pro vyber dovednosti (pokud je editovatelne)
 */
class DovednostField extends FormField
{
    private $idPP;

    /**
     *
     * @param int $idPP id vybraneho pracovniho pomeru
     */
    function __construct($idPP)
    {
        parent::__construct();

        $this->fieldType = 'dovednost';

        $this->idPP = $idPP;
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     * @throws Exception
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        $edit = (isset($_POST[$this->fieldName . "_edit"]) ? $_POST[$this->fieldName . "_edit"] : $canEdit);

        if ($edit) {
            //if ( !isset($value) ) $value = RonPortalSession::vybranaOsoba();

            $combo = new ComboBox($this->fieldName); // jmeno promenne

            $filtrTPM = '';

            $typRozdeleniTPM = RonPortalSettings::getIntValue('dovednosti_typ_rozdeleni_tpm');

            $params = $database->createParameters();

            if ($this->idPP) {
                $filtrPracPomer = 'PracovniPomer.ID = ' . $params->addParameter($this->idPP);
            } else {
                $filtrPracPomer =
                    'PracovniPomer.RC = ' . $params->addParameter(User::getRC()) .
                    ' AND PracovniPomer.DatumNastupu <= ' . $params->addDateParameter(time()) .
                    ' AND (PracovniPomer.DatumUkonceni >= ' . $params->addDateParameter(time()) . ' OR PracovniPomer.DatumUkonceni IS NULL)';
            }

            //pokud je v nastaveni zvolen typ rozdeleni pro typova pracovni mista, bude mozne zamestnanci zadat jen dovednosti
            // pozadovane pro rozdeleni (typove pracovni misto) zvoleneho zamestnance
            if (($typRozdeleniTPM != -1) && ($typRozdeleniTPM != 0)) {
                $filtrTPM =
                    ' AND EXISTS(SELECT Rozdeleni FROM RozdeleniOsob
                                   JOIN PracovniPomer ON (PracovniPomer.RC = RozdeleniOsob.RC AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo
                                        AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu)
                                   JOIN PozadavekNaDovednost ON (PozadavekNaDovednost.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni
                                        AND PozadavekNaDovednost.KodRozdeleni = RozdeleniOsob.Rozdeleni)
                                 WHERE (' . $filtrPracPomer . ')' .
                    '              AND RozdeleniOsob.KodTypuRozdeleni = ' . $params->addParameter($typRozdeleniTPM) .
                    '              AND RozdeleniOsob.PlatnostOd <= ' . $params->addDateParameter(DateUtils::today()) .
                    '              AND (RozdeleniOsob.PlatnostDo >= ' . $params->addDateParameter(DateUtils::today()) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
                    '              AND PozadavekNaDovednost.PlatnostOd <= ' . $params->addDateParameter(DateUtils::today()) .
                    '              AND (PozadavekNaDovednost.PlatnostDo >= ' . $params->addDateParameter(DateUtils::today()) . ' OR PozadavekNaDovednost.PlatnostDo IS NULL) ' .
                    '              AND PozadavekNaDovednost.KodDovednosti = Dovednost.Kod ' .
                    ')';
            }

            $sql = "SELECT Kod, Popis FROM Dovednost " .
                "WHERE Aktivni = 1" .
                "  AND EXISTS(SELECT KodUrovne FROM UrovenDovednosti WHERE KodDovednosti = Dovednost.Kod)" .
                //17.3.2017 RON-PR dovednosti, ktere jsou slozene nebo definovane hodnotou nelze rucne zadat
                "  AND Slozena = 0 " .
                "  AND DefinovanaHodnotou = 0
                    $filtrTPM
                    ORDER BY Popis";

            $combo->emptyRow = ' ';
            $combo->loadData($database, $sql, $this->defaultValue, $params);
            $combo->htmlParams = $this->otherHtmlParams;
            $combo->htmlParams['id'] = $this->fieldId;
            $combo->setClassName($this->getFieldCssClass());
            $combo->render();
        } else {
            echo '<input type="hidden" name="' . $this->fieldName . '" value="' . $this->defaultValue . '" />';
            echo '<input type="hidden" name="' . $this->fieldName . "_edit" . '" value="' . ($this->readOnly ? '0' : '1') . '" />';
            echo $this->loadName($database, $this->defaultValue) . "\n";
        }
    }

    function createDBRecordField($database, $tablename, $insert, $utf8 = false)
    {
        $field = new DBRecordField($this->sqlFieldName, '', $this->readOnly, $this->maxLength);
        $field->value = trim($this ->getPostValueText($utf8));

        return $field;
    }

    /**
     *
     * @param Database $database
     * @param $value
     * @return string
     * @throws Exception
     */
    function loadName($database, $value)
    {
        $params = $database->createParameters();
        $sql = "SELECT Popis FROM Dovednost WHERE Kod = " . $params->addParameter($value);

        $cursor = $database ->getSqlResult($sql, $params);

        if (!$row = $cursor->fetch()) {
            throw new Exception('Chyba - dovednost: ' . $this->defaultValue . ' nebyla nalezena!');
        }

        return $row[0];
    }

}

class DovednostOsobyForm extends RonPortalForm
{
    const XML_EVENT_BEFORE = 'DOVEDNOST_BEFORE';
    const XML_EVENT_AFTER = 'DOVEDNOST_AFTER';
    const XML_CONTEXT_INS = 'insert';
    const XML_CONTEXT_UPD = 'update';

    private $prava;
    private $pracPomer = null;
    private $dovednost;
    private $editDovednost = true;

    /**
     * DovednostForm constructor.
     * @param Database $database
     * @param int $idPP
     * @throws Exception
     */
    function __construct($database, $backURL, $idPP = 0)
    {
        parent::__construct($database);

        Prava::nastavObdobiProPrava(time(),time());
        $this ->prava = Prava::vratPrava('doved_edit');

        $this->setXmlEventsName('DOVEDNOST');

        if ($idPP) {
            $this ->pracPomer = PracovniPomer::nactiPP($this ->database, $idPP);
        }
        $this->nactiParametr();

        if (isset($_POST['dovednost'])) { $this->dovednost = filter_input(INPUT_POST, 'dovednost'); }

        $this ->htmlAction = '?menuid=maticedovednosti';
        if (Prava::vratHodnotuPrava('doved_ziskane') != 0) {
            $stornoURL = '?menuid=maticedovednosti&action='.$backURL;
        } else {
            $stornoURL = '?menuid=maticedovednosti&action=novadovednost';
        }

        $this -> stornoAction = $stornoURL;

        $this -> sqlTableName = 'DovednostOsoby';

        //$this -> dbLogSrcDescription = 'Formular nove dovednosti osoby';
        $this ->code = 'formdovednostosoby';

        $this->sqlSelectPart =
            'DovednostOsoby.RC, DovednostOsoby.KodDovednosti, DovednostOsoby.KodUrovne, DovednostOsoby.DatumZiskani, DovednostOsoby.PlatnostDo, '. //0 - 4
            $this->database->castWideMemo('DovednostOsoby.Poznamka') . ', ' . // 5
            'DovednostOsoby.Insert_UserName, DovednostOsoby.Insert_Timestamp, DovednostOsoby.Update_UserName, DovednostOsoby.Update_Timestamp, '. //6 - 9
            'DovednostOsoby.SpecifikaceUrovne'; //10

        $this->addField(new OsobaDovednostField('doved_edit', time(), $this ->pracPomer ? $this ->pracPomer ->idPP : null))
            ->setCaption('Osoba') ->setReadOnly(true)
            ->setCode('field-rc')
            ->setFieldId('osoba_vyber')
            ->setOtherHtmlParams([
                'class' => 'form-control ronselect',
                'data-columns' => 'text;osobni-cislo',
                'data-captions' => TextManager::_('Jmeno') . ';' . TextManager::_('Osobni_cislo'),
            ]);

        $this->addField(new DovednostField($this->pracPomer ? $this->pracPomer->idPP : null))
            ->setFieldName('dovednost') ->setSqlFieldName('DovednostOsoby.KodDovednosti') ->setRequired(true)
            ->setCode('field-dovednost')
            ->setFieldId('dovednost_vyber')
            ->addFieldCssClass('form-control')
            ->setCaption('Dovednost') ->setDefaultValue($this->dovednost)
            ->setReadOnly(!$this ->editDovednost);

        $params = $this ->database ->createParameters();
        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('urovenDovednosti') ->setSqlFieldName('DovednostOsoby.KodUrovne') ->setSqlFieldIndex(2) ->setRequired(true)
            ->setCode('field-kodurovne')
            ->setRowId('urovneDovednosti') ->setPrimary(true) ->setCaption('Dovednosti_uroven_dovednosti')
            ->setDataType('int')
            ->setSelectSql(new SqlStatement(
                    "SELECT KodUrovne, Popis FROM UrovenDovednosti" .
                    " WHERE KodDovednosti = " . $params ->addParameter($this->dovednost) .
                    " ORDER BY KodUrovne, Popis",
                    $params)
            );

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('specifikaceUrovneDovednosti') ->setSqlFieldName('DovednostOsoby.SpecifikaceUrovne') ->setSqlFieldIndex(10)
            ->setCode('field-specifikaceurovne')
            ->setCaption('Dovednosti_specifikace_urovne')->setRowId('specifikaceUrovneDovednosti') ->setRequired(true) ->setDefaultValue(0)
            ->setDataType('int') ->setSelectDataTypeTest(false)
            ->setSelectSql(
                "SELECT Kod, Popis FROM S_SpecUrovneDovednosti " .
                //"WHERE KodDovednosti = '" . $database->addSlashes($this->f_dovednost) . "'" .
                "ORDER BY Kod, Popis");

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumZiskani') ->setSqlFieldName('DovednostOsoby.DatumZiskani')
            ->setCode('field-datumziskani')
            ->setSqlFieldIndex(3) ->setCaption('Dovednosti_datum_ziskani') ->setPrimary(true)
            ->setRequired(true) ->setMaxLength(10)
            ->setDefaultValue(StringUtils::format('date', DateUtils::today()));

        //[FORM_FIELD_TYPE => 'date', FORM_FIELD_NAME => 'PlatnostDo', FORM_FIELD_SQL_NAME => 'DovednostOsoby.PlatnostDo', FORM_FIELD_SQL_INDEX => 4, FORM_FIELD_CAPTION => $STR_PlatnostDo, FORM_FIELD_MAXLENGTH => 10], nastavuje agent

        $this->addField(FormField::FIELD_TYPE_FILE)
            ->setFieldName('dokument')->setCaption('Pripojit_dokument')
            ->setCode('field-dokument');

        $params = $this ->database ->createParameters();
        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('typdokumentu')->setSqlFieldName(null)
            ->setCode('field-typdokumentu')
            ->setCaption('typ_dokumentu')
            ->setDataType('int')
            ->setSelectSql(new SqlStatement(
                    'SELECT Kod, Popis FROM TypDokumentu' .
                    ' WHERE ZobrazitNaIntrawebu = 1 AND Kod >= 0 AND KategorieDokumentu = ' . $params->addParameter(4) . //dokumenty schopnosti
                    ' ORDER BY Kod',
                    $params)
            );

        $this ->addField()
            ->setFieldName('popisdokumentu')->setSqlFieldName(null)
            ->setCaption('popis_dokumentu')
            ->setCode('field-popisdokumentu');

        $this ->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka') ->setSqlFieldName('DovednostOsoby.Poznamka') ->setSqlFieldIndex(5)
            ->setCode('field-poznamka')
            ->setCaption('Poznamka');

//        $this ->addField(FormField::FIELD_TYPE_FILE)
//            ->setCode('field-dokument')
//            ->setFieldName('Dokument');

        $this ->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setSqlFieldName('DovednostOsoby.Insert_Timestamp') ->setSqlFieldIndex(7) ->setReadOnly(true)
            ->setCode('field-insert_timestamp');

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setSqlFieldName('Insert_UserName') ->setSqlFieldIndex(6) ->setReadOnly(true)
            ->setCode('field-insert_username')
            ->setDefaultValue(RonPortalUtils::getLogUserName());

        $this ->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setSqlFieldName('DovednostOsoby.Update_Timestamp') ->setSqlFieldIndex(9)
            ->setCode('field-update_timestamp');

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setSqlFieldName('Update_UserName') ->setSqlFieldIndex(8)
            ->setCode('field-update_username')
            ->setDefaultValue(RonPortalUtils::getLogUserName());
    }

    /**
     *
     * Nacte z URL adresy osobu, (dovednost)
     * @return void Nacte z URL adresy osobu, (dovednost)
     */
    function nactiParametr()
    {
        if (isset($_GET['dovednost'])) {
            $this ->dovednost = $_GET['dovednost'];

            //pokud lze zobrazit matici ziskanych dovednosti, nebude pro prokliku z matice (pri zvolene dovednosti) aktivni vyber dovednosti
            if (Prava::vratHodnotuPrava('doved_ziskane') != 0)
                $this->editDovednost = false;
        }
    }

    function getWhereCondition()
    {
        $params = $this->database->createParameters();
        return new SqlStatement(
            "RC = " . $params->addParameter($this->pracPomer->rc) .
            " AND KodDovednosti = " . $params->addParameter($this->dovednost), $params
        );
    }


    function createDBRecordFields($rec, $insert)
    {
        $rec->addField('RC', '', true, 15, $this->pracPomer->rc);
        parent::createDBRecordFields($rec, $insert);
    }

    function vlozDokument($rec)
    {
        //pokud nebyl dokument prilozen, neprovadi se vkladani
        if ($_FILES['dokument']['error'] === UPLOAD_ERR_NO_FILE) {
            return;
        }

        try {
            if ($_POST['typdokumentu'] == '') {
                throw new Exception('Není vybrán typ dokumentu.');
            }
            if ($_POST['popisdokumentu'] == '') {
                throw new Exception('Není zadán popis dokumentu.');
            }

            $idDokumentu = DokumentyUtils::vlozDokument($this->database, 'dokument', $_POST['typdokumentu'], $_POST['popisdokumentu']);

            $dokumentRec = new RonPortalDBRecord($this->database, 'DokumentyOsoby');
            $dokumentRec->addStringField('RC', true)->setValue($rec['RC']);
            $dokumentRec->addIntField('id', true)->setValue($idDokumentu);

            $dokumentRec->addStringField('Dovednost_RC')->setNewValue($rec['RC']);
            $dokumentRec->addStringField('Dovednost_Kod')->setNewValue($rec['KodDovednosti']);
            $dokumentRec->addIntField('Dovednost_Uroven')->setNewValue($rec['KodUrovne']);
            $dokumentRec->addDateField('Dovednost_DatumZiskani')->setNewValue($rec['DatumZiskani']);

            $params = $this->database->createParameters();
            $dokumentRec->setWhere("RC = " . $params->addParameter($rec['RC']) . ' AND id = ' . $params->addParameter($idDokumentu), $params);

            try {
                $dokumentRec->updateOrInsert();
            } catch (Exception $exception) {
                throw new Exception('Chyba zapisu dokumenty osoby');
            }
        } catch (Exception $exception) {
            throw new Exception('Chyba pri vkladani dokumentu dovednosti: ' . $exception->getMessage());
        }
    }


    /**
     *
     * @param RonPortalDBRecord $rec zaznam, ktery se bude vkladat
     * @return bool vraci true, pokud muze byt zaznam vlozen
     * @throws Exception
     */
    function beforeInsertRecord($rec)
    {
        if (!$this->prava->lzeZobrazitRC($rec['RC'])) {
            throw new NeniPravoException();
        }

        if ($this->checkUrovenDovednosti($rec['RC'], $rec['KodDovednosti'], $rec['KodUrovne'], $rec['DatumZiskani'])) {
            throw new Exception(TextManager::_('Dovednosti_uroven_existuje'));
        }

        if ($rec instanceof RonPortalDBRecord  &&  $this ->xmlEventBeforePost) {
            $rec ->executeXmlEvents($this ->xmlEventBeforePost, 'insert',
                $this ->getXmlEventsParams($rec, 'insert', $this ->xmlEventBeforePost)
            );
        }

        return true;
    }

    public function insertRecord()
    {
        if (!$this->validate(false)) {
            return false;
        }

        $rec = $this->createDBRecord();
        $this ->createDBRecordFields($rec, true);

        try {
            $this->database->beginTransaction();

            try {
                if (!$this->beforeInsertRecord($rec)) {
                    return false;
                }

                $rec->insertRecord();

                $result = $this->afterInsertRecord($rec);

                $this->vlozDokument($rec);
            } catch (Exception $exception) {
                $this->database->rollbackTransaction();
                throw $exception;
            }

            $this->database->commitTransaction();
        } catch (Exception $exception) {
            throw new Exception('Chyba pri zapisu dovednosti: ' . $exception->getMessage());
        }

        return $result;
    }

    /**
     *
     * @param string $rc
     * @param string $dovednost
     * @param string $uroven
     * @param int $datumZiskani
     * @return bool true pokud uz uroven ma
     * @throws DBException
     */
    function checkUrovenDovednosti($rc, $dovednost, $uroven, $datumZiskani)
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT RC FROM DovednostOsoby" .
            " WHERE RC = " . $params->addParameter($rc) . " AND KodDovednosti = " . $params->addParameter($dovednost) .
            " AND KodUrovne = " . $params->addParameter($uroven) .
            " AND DatumZiskani = " . $params->addParameter($this->database->dateToParam($datumZiskani));

        $cursor = $this->database ->getSqlResult($sql, $params);

        if (!$row = $cursor->fetch()) {
            return false;
        }

        if ($row[0]) {
            return true;
        }

        return false;
    }
}
