<?php
/**
 * stanislav.elbl, 28.02.2021
 */

namespace RonPortal\Editace;

use Exception;
use Ron\Controls\EditField;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MessageManager;
use RonPortal\Dochazka\MesicniVysledek;
use RonPortal\Dochazka\SchvaleniDochazky;
use RonPortal\Obdobi;
use RonPortal\RonPortalXMLEvents;
use RonPortal\User;

/**
 * Editace mesicnich vysledku pro jednu osobu.
 * Class EditaceMesicnich
 */
class EditaceMesicnich extends MesicniData
{
    /** @var OsobaProEditaci */
    public $osoba;
    public $tableId;


    /**
     * @param Database $database
     */
    public function __construct($database)
    {
        parent::__construct($database);
        $this ->prefixHTML = 'editaceMV_';
    }


    /**
     * @param string $name
     * @param string $format
     * @param PolozkaEditace $polozka
     * @param bool $readonly
     * @return EditField
     */
    protected function createEditField($name, $format, $polozka, $readonly)
    {
        $result = new EditField();

        $result ->format = $format ?: 'number2';
        $result ->skrytNuly = $polozka ->skrytNuly;
        //$result -> f_ReadOnly = $polozka -> f_ReadOnly = $readonly;
        $result ->readOnly = $this ->f_readonly || $readonly;
        $result ->name = $this ->prefixHTML . $name;

        $result ->sirkaPole = $this ->sirkaPole;

        return $result;
    }


    /**
     * @param OsobaProEditaci $osoba
     * @param MesicniVysledek $vysledek
     * @return bool
     */
    protected function nastavMesicniVysledek($osoba, $vysledek)
    {
        $result = false;

        foreach ($this ->polozky as $polozka) {
            if ($polozka ->typ === 'MV'  &&  $polozka ->kod === $vysledek ->typCasoveSlozkyMzdy) {
                if ($polozka ->zobrazitHodiny) { // hodiny se maji zobrazit
                    $polozka ->editHodiny ->setValue($vysledek ->hodiny);
                }

                if ($polozka ->zobrazitDny) {
                    $polozka ->editDny ->setValue($vysledek ->dny);
                }

                if ($polozka ->zobrazitKalendarniDny) {
                    $polozka ->editKalendarniDny ->setValue($vysledek ->kalendarniDny);
                }

                $result = true; // nasel sloupec se zadanou slozkou
                // pokracuje dale - stejna casova slozka tam muze byt vicekrat (napr. hodiny i dny)
            }
        }

        return $result;
    }


    /**
     * @param $osoba OsobaProEditaci
     * @throws DBException
     * @throws Exception
     */
    protected function nactiMesicniVysledkyOsoby($osoba)
    {
        foreach ($this ->polozky as $polozka) {
            if ($polozka ->typ === 'MV') {
                $uzamknuto = $this ->uzamknoutPolozku($polozka, $osoba);
                $lzeEditovat = !$uzamknuto && $this->jePravoNaPolozku($polozka, $osoba);

                $polozka ->editHodiny = $this ->createEditField('_H_' . $polozka ->kod, 'hodiny', $polozka, $polozka ->zobrazitHodiny !== 2 || !$lzeEditovat);
                $polozka ->editDny = $this ->createEditField('_D_' . $polozka ->kod, $polozka ->format, $polozka, $polozka ->zobrazitDny !== 2 || !$lzeEditovat);
                $polozka ->editKalendarniDny = $this ->createEditField('_K_' . $polozka ->kod, $polozka ->format, $polozka, $polozka ->zobrazitKalendarniDny !== 2 || !$lzeEditovat);
            }
        }

        parent::nactiMesicniVysledkyOsoby($osoba);
    }



    protected function nastavMzdovouPolozku($osoba, $mzdovaPolozka)
    {
        foreach ($this->polozky as $polozka) {
            if ($polozka ->typ === 'MP'  &&  $polozka->kod == $mzdovaPolozka ->mzdovaPolozka) {
                if (!$polozka->rozlisitRozdeleni
                    || ($this->f_TypRozdeleni == $mzdovaPolozka ->kodTypuRozdeleni && $this->f_Rozdeleni == $mzdovaPolozka ->rozdeleni)
                ) {
                    if ($polozka->zobrazitPocetJednotek) {
                        // pocet jednotek se ma zobrazit
                        $polozka->editPocetJednotek->setValue($mzdovaPolozka ->pocetJednotek);
                    }

                    if ($polozka->zobrazitCenuZaJednotku) {
                        $polozka->editCenaZaJednotku->setValue($mzdovaPolozka ->cenaZaJednotku);
                    }

                    if ($polozka->zobrazitCastku) {
                        $polozka->editCastka->setValue($mzdovaPolozka ->castka);
                    }
                }
            }
        }
    }



    /**
     * @param OsobaProEditaci $osoba
     */
    function nactiMzdovePolozkyOsoby($osoba)
    {
        // vytvori pole pro jednotlive polozky
        foreach ($this ->polozky as $polozka) {
            if ($polozka ->typ = 'MP') {
                $uzamknuto = $this ->uzamknoutPolozku($polozka, $osoba);

                $lzeEditovat = $uzamknuto ? false : $this ->jePravoNaPolozku($polozka, $osoba);

                $polozka ->editPocetJednotek = $this ->createEditField('_P_' . $polozka ->kod, $polozka ->format, $polozka, $polozka ->zobrazitPocetJednotek != 2 || !$lzeEditovat);
                $polozka ->editCenaZaJednotku = $this ->createEditField('_J_' . $polozka ->kod, $polozka ->format, $polozka, $polozka ->zobrazitCenuZaJednotku != 2 || !$lzeEditovat);
                $polozka ->editCastka = $this ->createEditField('_C_' . $polozka ->kod, $polozka ->format, $polozka, $polozka ->zobrazitCastku != 2 || !$lzeEditovat);
            }
        }

        parent::nactiMzdovePolozkyOsoby($osoba);
    }


    /**
     * @param OsobaProEditaci $osoba
     */
    function nactiSchvaleni($osoba)
    {
        $osoba ->schvaleni = new SchvaleniDochazky();
        $osoba ->schvaleni ->load($this ->database, $osoba ->rc, $osoba ->osobniCislo, $osoba ->datumNastupu, $this ->obdobi ->zacatek);
    }


    /**
     * @param $osoba OsobaProEditaci
     * @returns void   Nacte data pro rezim jedne osoby
     */
    function nactiDataOsoby($osoba)
    {
        $this ->nactiSchvaleni($osoba);
        // if ($osoba ->Uzamknout($this ->f_konecobdobi, 1)) $this ->f_readonly = true; // kontrola schvaleni dochazky do konce obdobi (prozatim natvrdo jakakoliv uroven!)

        // nacte data z databaze
        if ($this ->obsahujeMV()) {
            $this ->nactiMesicniVysledkyOsoby($osoba);
        }

        if ($this ->obsahujeMP()) {
            if ($osoba->f_PravoEditovatMP || $osoba->f_PravoZobrazitMP) {
                $this->nactiMzdovePolozkyOsoby($osoba);
            }
        }
    }



    function nastavPravaNaOsobu($osoba)
    {
        $osoba ->f_PravoEditovatMV = $this ->f_pravaMVEdit ->lzeZobrazitPP($osoba ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec);
        $osoba ->f_PravoEditovatMP = $this ->f_pravaMPEdit ->lzeZobrazitPP($osoba ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec);

        $osoba ->f_PravoZobrazitMV = $osoba ->f_PravoEditovatMV || $this ->f_pravaMV ->lzeZobrazitPP($osoba ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec);
        $osoba ->f_PravoZobrazitMP = $osoba ->f_PravoEditovatMP || $this ->f_pravaMP ->lzeZobrazitPP($osoba ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec);

        // upravi viditelnost polozek podle nastaveni a profilu uzivatele
        foreach ($this ->polozky as $polozka) {
            if ($polozka ->zobrazit && $polozka->profil) {
                // vypne sloupec, pokud je pro jiny profil uzivatele
                $polozka->zobrazit = User::jeVProfilu($polozka->profil);
            }

            if ($polozka ->zobrazit) {
                // vypne sloupec, pokud uzivatel nema prislusna prava
                if ($polozka ->prava  &&  !$polozka ->prava ->lzeZobrazitPP($osoba ->idPP)) {
                    // uzivatel nema zadna prava na tento sloupec => vypne ho
                    $polozka ->zobrazit = false;
                }
            }
        }
    }


    /**
     * Nacte data a pripadne zapise zmenene hodnoty. Vraci false v pripade chyby, true pokud nedoslo k zadne zmene a jinak cislo >= 1
     * @param OsobaProEditaci $osoba
     * @param Obdobi $obdobi
     */
    function nactiData($osoba, $obdobi)
    {
        $this ->nastavPrava(false); // $povolitvse = false

//        $this ->nastavObdobi(RonPortalSession::obdobi());
//
//        $this ->osoba = new OsobaProEditaci();
//        $this ->osoba ->nactiAktualniOsobu();

        $this ->nastavObdobi($obdobi);
        $this ->osoba = $osoba;

        $this ->nastavPravaNaOsobu($this ->osoba);

        $this ->nactiDataOsoby($this ->osoba);

//        if (isset($_POST[$this ->prefixHTML . 'POST'])) {
//            if ($this ->nactiPost()) {
//                return $this ->zapisDataOsoby();
//            } else {
//                return MessageManager::addErrorMessage( \Ron\ronText('Spatna_hodnota') );
//            }
//        }
//
//        return true;
    }


    /**
     * Nacte hodnoty zadane uzivatelem. V pripade chyby (spatny format) vraci false
     * @return bool
     * @throws Exception
     */
    function nactiPost()
    {
        if (!isset($_POST[$this ->prefixHTML . 'POST'])) {
            return true;
        }

        $result = true;

        foreach ($this ->polozky as $radek) {
            //if (!$radek ->prava  ||  $radek ->prava ->lzeZobrazitPP($this ->osoba ->idPP)) {
            if ($radek ->zobrazit) {
                $ok = true;

                if ($this ->osoba->f_PravoEditovatMV) {
                    if ($radek->zobrazitHodiny == 2) {
                        $ok &= $radek->editHodiny->nactiPost();
                    }
                    if ($radek->zobrazitDny == 2) {
                        $ok &= $radek->editDny->nactiPost();
                    }
                    if ($radek->zobrazitKalendarniDny == 2) {
                        $ok &= $radek->editKalendarniDny->nactiPost();
                    }
                }

                if ($this ->osoba->f_PravoEditovatMP) {
                    if ($radek->zobrazitPocetJednotek == 2) {
                        $ok &= $radek->editPocetJednotek->nactiPost();
                    }
                    if ($radek->zobrazitCenuZaJednotku == 2) {
                        $ok &= $radek->editCenaZaJednotku->nactiPost();
                    }
                    if ($radek->zobrazitCastku == 2) {
                        $ok &= $radek->editCastka->nactiPost();
                    }
                }

                if (!$ok) {
                    MessageManager::addErrorMessage( \Ron\ronText('Spatna_hodnota_pro') . $radek ->nadpis);
                }

                $result &= $ok;
            }
        }

        return $result;
    }



    function refreshData()
    {
        $this ->nactiDataOsoby($this ->osoba);
    }



    /**
     * @param EditField $editfield
     * @param bool $zobrazit
     * @return string
     */
    protected function vratTextSloupce($editfield, $zobrazit)
    {
        if (!$zobrazit  ||  !$editfield) {
            return '';
        }

        if ($this ->f_readonly) {
            $editfield ->readOnly = true;
        }

        //if ($zobrazit == 2 && !$editfield ->f_ReadOnly) $this ->f_poceteditu++;

        return $editfield ->getFieldHTML();
    }


    /**
     * Vraci obsah HTML formulare
     * @return string
     */
    protected function vratObsahFormulare()
    {
        $idStr = $this ->tableId ? " id='{$this ->tableId}'" : '';

        $result =
            "<input type='hidden' name='{$this ->prefixHTML}POST' value='1'>" .
            "<table class='browser' style='{$this ->cssStyl}'$idStr>";

        //$result .= "<CAPTION> nadpisek ... </CAPTION>";

        // zjisti, ktere sloupce se maji zobrazit
        $zobrazitHodiny = false;
        $zobrazitDny = false;
        $zobrazitKalDny = false;

        $zobrazitJednotku = false;
        $zobrazitCenu = false;
        $zobrazitCastku = false;

        foreach ($this ->polozky as $radek) {
            if ($radek ->zobrazit) {
                if ($radek->zobrazitHodiny) {
                    $zobrazitHodiny = true;
                }
                if ($radek->zobrazitDny) {
                    $zobrazitDny = true;
                }
                if ($radek->zobrazitKalendarniDny) {
                    $zobrazitKalDny = true;
                }

                if ($radek->zobrazitPocetJednotek) {
                    $zobrazitJednotku = true;
                }
                if ($radek->zobrazitCenuZaJednotku) {
                    $zobrazitCenu = true;
                }
                if ($radek->zobrazitCastku) {
                    $zobrazitCastku = true;
                }
            }
        }

        // hlavicka
        $result .=
            '<thead><tr>' .
            '<th></th>' .
            ($zobrazitHodiny ? '<th>' . \Ron\ronText('Vysledek_Hodiny') . '</th>' : '') .
            ($zobrazitDny ? '<th>' . \Ron\ronText('Vysledek_Dny') . '</th>' : '') .
            ($zobrazitKalDny ? '<th>' . \Ron\ronText('Vysledek_Kalendarni_dny') . '</th>' : '') .
            ($zobrazitJednotku ? '<th>' . \Ron\ronText('Pocet_jednotek') . '</th>' : '') .
            ($zobrazitCenu ? '<th>' . \Ron\ronText('Cena_za_jednotku') . '</th>' : '') .
            ($zobrazitCastku ? '<th>' . \Ron\ronText('Castka') . '</th>' : '') .
            '</tr></thead>';

        $color2 = false;

        //$this ->f_poceteditu = 0;

        // jednotlive radky
        foreach ($this ->polozky as $radek) {
            if ($radek ->zobrazit) {
                $classname = $color2 ? 'browsercolor2' : 'browsercolor1';
                $color2 = !$color2;

                $result .=
                    '<tr class="' . $classname . '" ' .
                    ($radek->styl ? " style='{$radek ->styl}'" : '') .
                    '><td>' . $radek->nadpis . '</td>';


                if ($zobrazitHodiny) {
                    $result .=
                        '<td class="align-right' . (($radek->editHodiny && $radek->editHodiny->hodnota < 0) ? ' minus' : '') . '">' .
                        $this->vratTextSloupce($radek->editHodiny, $radek->zobrazitHodiny) .
                        '</td>';
                }

                if ($zobrazitDny) {
                    $result .=
                        '<td class="align-right' . (($radek->editDny && $radek->editDny->hodnota < 0) ? ' minus' : '') . '">' .
                        $this->vratTextSloupce($radek->editDny, $radek->zobrazitDny) .
                        '</td>';
                }

                if ($zobrazitKalDny) {
                    $result .=
                        '<td class="align-right' . (($radek->editKalendarniDny && $radek->editKalendarniDny->hodnota < 0) ? ' minus' : '') . '">' .
                        $this->vratTextSloupce($radek->editKalendarniDny, $radek->zobrazitKalendarniDny) .
                        '</td>';
                }

                if ($zobrazitJednotku) {
                    $result .=
                        '<td class="align-right">' .
                        $this->vratTextSloupce($radek->editPocetJednotek, $radek->zobrazitPocetJednotek) .
                        '</td>';
                }

                if ($zobrazitCenu) {
                    $result .=
                        '<td class="align-right">' .
                        $this->vratTextSloupce($radek->editCenaZaJednotku, $radek->zobrazitCenuZaJednotku) .
                        '</td>';
                }

                if ($zobrazitCastku) {
                    $result .=
                        '<td class="align-right' . (($radek->editCastka  &&  $radek->editCastka->hodnota < 0) ? ' minus' : '') . '">' .
                        $this->vratTextSloupce($radek->editCastka, $radek->zobrazitCastku) .
                        '</td>';
                }

                $result .= '</tr>';
            }
        }

        $result .= "</table>\n";

        return $result;
    }


    function zobrazData()
    {
        echo $this ->vratObsahFormulare();
    }



    /**
     * V pripade chyby vraci false (napr. spatne zadana hodnota)
     * @param $osoba
     * @param PolozkaEditace $polozka
     * @return bool|int
     */
    function zapisMzdovouPolozkuOsoby($osoba, $polozka)
    {
        $pocetJednotek = null;
        $cenaZaJednotku = null;
        $castka = null;

        $OK = true;

        if ($polozka ->zobrazitCastku == 2  &&  $polozka ->editCastka) { // editovatelna castka
            // pokud je editovatelna castka tak (prozatim?) nastavi pocet=1, cenaZaJednotku=castka
            if ($polozka ->editCastka ->chyba) {
                $OK = false;
            } else if ($polozka ->editCastka ->puvodniText != $polozka ->editCastka ->text) {
                $castka = $polozka ->editCastka ->hodnota;
            }
        }
        //else {
            // editace poctu a jednotkove ceny zatim neni implementovana!
        //}

        if ($OK && (isset($pocetJednotek) || isset($cenaZaJednotku) || isset($castka))) {
            // doslo ke zmene nektere hodnoty mesicniho vysledku
            try {
                $this->zapisMzdovouPolozkuDoDB($osoba, $polozka, $castka, $cenaZaJednotku, $pocetJednotek);
                return 1;
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e);
                return false;
            }
        }

        return $OK;
    }


    /**
     * V pripade chyby vraci false (napr. spatne zadana hodnota), pokud nic nebylo zmeneno vraci true, jinak vraci pocet zmenenych zaznamu (1)
     * @param OsobaProEditaci $osoba
     * @param PolozkaEditace $radek
     * @return bool|int
     * @throws Exception
     * @throws DBException
     */
    function zapisMesicniVysledekOsoby($osoba, $radek)
    {

        $hodiny = null;
        $dny = null;
        $kalendarniDny = null;

        $smazat = false;

        if ($radek ->zobrazitHodiny == 2) {
            if ($radek ->editHodiny ->chyba) {
                // nemelo by nastat - jiz bylo zkontrolovano
                throw new Exception('Chyba pri zapisu mesicnich vysledku - nelze zapsat chybnou hodnotu!');
            }

            if ($radek ->editHodiny ->puvodniText != $radek ->editHodiny ->text) {
                $hodiny = $radek ->editHodiny ->hodnota;
            }
            if ($radek ->editHodiny ->text === ''  &&  $radek ->smazatPrazdny) {
                $smazat = true;
            }
        }

        if ($radek ->zobrazitDny == 2) {
            if ($radek ->editDny ->chyba) {
                throw new Exception('Chyba pri zapisu mesicnich vysledku - nelze zapsat chybnou hodnotu!');
            }

            if ($radek ->editDny ->puvodniText != $radek ->editDny ->text) {
                $dny = $radek ->editDny ->hodnota;
            }
            if ($radek ->editDny ->text === ''  &&  $radek ->smazatPrazdny) {
                $smazat = true;
            }
        }

        if ($radek ->zobrazitKalendarniDny == 2) {
            if ($radek ->editKalendarniDny ->chyba) {
                throw new Exception('Chyba pri zapisu mesicnich vysledku - nelze zapsat chybnou hodnotu!');
            }

            if ($radek ->editKalendarniDny ->puvodniText != $radek ->editKalendarniDny ->text) {
                $kalendarniDny = $radek ->editKalendarniDny ->hodnota;
            }
            if ($radek ->editKalendarniDny ->text === ''  &&  $radek ->smazatPrazdny) {
                $smazat = true;
            }
        }

        if (isset($hodiny) || isset($dny) || isset($kalendarniDny)  ||  $smazat  ||  $radek ->smazatNulovy) {
            // doslo ke zmene nektere hodnoty mesicniho vysledku

            if ($radek ->rozlisitRozdeleni == 3) {
                // nemelo by nastat - pokud je rozlisitRozdeleni=3, tak je editace zakazana
                return false;
            } else if ($radek ->rozlisitRozdeleni == 2) {
                // rozlisuje se rozdeleni - podle rozdeleni zamestnance
                if (!isset($osoba ->kodRozdeleni)) {
                    throw new Exception('Chyba - ' . $osoba ->jmenoPrijmeni . ', ' . ' os. c. ' . $osoba ->osobniCislo . ' nema prirazeno rozdeleni!');
                }
                $rec = new MesicniVysledek_R_ERecord($this ->database, $osoba, $this ->obdobi ->zacatek, $radek ->kod, $this ->f_TypRozdeleni, $osoba ->kodRozdeleni, $this);
            } else if ($radek ->rozlisitRozdeleni && $this ->f_TypRozdeleni && $this ->f_Rozdeleni) {
                // rozlisuje se rozdeleni - podle uzivatelem vybraneho rozdeleni ve formulari
                if (!isset($this ->f_Rozdeleni)) {
                    throw new Exception('Chyba - neni vybrano rozdeleni - nelze editovat sloupec s casovou slozkou ' . $radek ->kod);
                }
                $rec = new MesicniVysledek_R_ERecord($this ->database, $osoba, $this ->obdobi ->zacatek, $radek ->kod, $this->f_TypRozdeleni, $this->f_Rozdeleni, $this);
            } else {
                $rec = new MesicniVysledekVypoctuRecord($this ->database, $osoba, $this ->obdobi ->zacatek, $radek ->kod, $this);
            }

            if (isset($hodiny)) {
                $rec ->fHodiny ->setNewValue($hodiny);
            }
            if (isset($dny)) {
                $rec ->fDny ->setNewValue($dny);
            }
            if (isset($kalendarniDny)) {
                $rec ->fKalendarniDny ->setNewValue($kalendarniDny);
            }

            if ($rec ->loadRecord()) {
                // zaznam uz existuje
                if ($smazat) {
                    $rec ->deleteRecord();
                    return true; // doslo ke zmene
                } else {
                    if ($radek ->smazatNulovy  &&
                        $rec ->fHodiny ->value === 0.0  &&
                        $rec ->fDny ->value === 0.0  &&
                        $rec ->fKalendarniDny ->value === 0.0  &&
                        ($rec ->fPoznamka ->value === ''  ||  !isset($rec ->fPoznamka ->value)) )
                    {
                        $rec ->deleteRecord();
                        return true;
                    } else {
                        $pom = $rec ->updateRecord(); // v pripade uspesne zmeny vraci cislo >= 1
                        if ($pom === true) {
                            // vse OK, ale nedoslo ke zmene
                            return false;
                        }
                        return $pom > 0;
                    }
                }
            } else {
                // zaznam jeste neexistuje
                if ($smazat) {
                    // nedoslo ke zmene
                    return false;
                }

                $rec ->insertRecord();
                return true;
            }
        }

        return false;
    }


    /**
     * V pripade chyby vraci false (napr. spatne zadana hodnota), pokud se nic nezmenilo vraci true, jinak cislo >= 1
     * @return bool
     */
    function zapisDataOsoby()
    {
        $jeZmena = false;

        foreach ($this ->polozky as $radek) {
            //if (!$radek ->prava  ||  $radek ->prava ->lzeZobrazitPP($this ->osoba ->f_IdPP)) {
            if ($radek ->zobrazit) {
                if ($this->zapisMesicniVysledekOsoby($this ->osoba, $radek)) {
                    $jeZmena = true;
                }

                // TODO Predelat! Vyhodit vyjimku, kdyz je chyba, vracet true pri zmene, jinak false
                $pom = $this->zapisMzdovouPolozkuOsoby($this ->osoba, $radek);
                if ($pom === false) {
                    // false = chyba
                    $result = false;
                } else if ($pom !== true) {
                    // true = nic nebylo zmeneno, 1 = zaznam byl zmenen
                    $jeZmena = true;
                }
            }
        }

        $this ->executeXmlEvent($this ->osoba, RonPortalXMLEvents::EDITACE_MESICNICH_AFTER, $jeZmena);

        return $jeZmena;
    }


}
