<?php
/**
 * stanislav.elbl, 26.02.2021
 */

namespace RonPortal\Editace;

use Ron\DB\Database;
use Ron\DB\DBException;
use RonPortal\Dochazka\OperaceDochazky;
use RonPortal\Dochazka\PrirazeniKalendare;
use RonPortal\Dochazka\SmenaProDen;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalXMLEvents;
use RonPortal\XML\RonPortalXMLForm;

class EditaceUtils
{
    const SETTINGS_ROZDELENI_POVINNE = 'rozdelenieditacepovinne';

    /**
     * @param Database $database
     * @param string   $kodOperace
     * @return bool
     */
    public static function jeRozdeleniDochazkyPovinne($database, $kodOperace)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)) {
            return false;
        }

        if ( RonPortalSettings::getIntValue(self::SETTINGS_ROZDELENI_POVINNE)) {
            return true;
        }

        return OperaceDochazky::rozdeleniDochazkyPovinneProOperaci($database, $kodOperace);
    }


    /**
     * Pro zadany PP a den vraci vychozi smenu
     * - smenu, ktera je v kalendari pro dany den
     * - kdyz zadna neni, tak prvni ze seznamu pouzivanych smen daneho kalendare
     *
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $den
     * @param int|null $datumCasPocatku
     * @return SmenaProDen|null
     * @throws DBException
     */
    public static function vychoziSmenaProOsobu(Database $database, $pp, $den, $casOd = null, $casDo = null): ?SmenaProDen
    {
        $kal = PrirazeniKalendare::nactiKalendarProDen($database, $pp ->idPP, $den);
        if (!$kal) {
            return null;
        }

        /** @var SmenaProDen $smenaKal Smena podle kalendare (podle kapacit nebo ze Smeny II - dle nastaveni) */
        $smenaKal = null;
        /** @var SmenaProDen $smenaKal Smena obsahujici zadany cas pocatku */
        $smenaObsahujiciCas = null;

        if ($kal ->dochazkaPodlePlanKapacit) {
            // nacte naplanovanou smenu
            $smeny = SmenaProDen::nactiSmenyPodleKapacit($database, $pp, $den, $den);
            $smenaKal = $smeny[0] ?? null;

            if ( $smenaKal  &&  (!isset($casOd)  ||  ($smenaKal ->pocatekJeVNabehoveDobe($casOd, $kal ->nabehovaDobaProUrceniSmen))) ) {
                // pokud neni zadan cas pocatku nebo je zadan cas pocatku a spada do nabehove doby, tak vraci tuto smenu
                return $smenaKal;
            }
        }

        $smeny = SmenaProDen::nactiSmenyPodleKalendare($database, $pp, $den, $den);
        if ($smeny) {
            if (!isset($casOd)  ||  ($smeny[0] ->pocatekJeVNabehoveDobe($casOd, $kal ->nabehovaDobaProUrceniSmen))) {
                // pokud neni zadan cas pocatku nebo je zadan cas pocatku a spada do nabehove doby, tak vraci tuto smenu
                return $smeny[0];
            }

            if (!$smenaKal) {
                $smenaKal = $smeny[0];
            }
        }

        if ($smenaKal  &&  isset($casOd)  &&  $smenaKal ->smenaObsahujeCas($casOd)) {
            $smenaObsahujiciCas = $smenaKal;
        }

        $smeny = SmenaProDen::nactiPovoleneSmenyKalendare($database, $kal ->kod);
        foreach ($smeny as $smena) {
            if (!isset($casOd)  ||  ($smena ->pocatekJeVNabehoveDobe($casOd, $kal ->nabehovaDobaProUrceniSmen))) {
                // pokud neni zadan cas pocatku nebo je zadan cas pocatku a spada do nabehove doby, tak vraci tuto smenu
                return $smena;
            }

            if (!$smenaObsahujiciCas  &&  $smena ->smenaObsahujeCas($casOd)) {
                $smenaObsahujiciCas = $smena;
            }
        }

        return $smenaObsahujiciCas ?? $smenaKal ?? $smeny[0] ?? null;
    }


    public static function jePravoEditovatOperaci($operace)
    {
        $prava = Prava::vratPrava('editace');
        if ($prava ->priznak3 == '') {
            return true;
        }

        return in_array($operace, explode(';', $prava ->priznak3));
    }


    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $den
     * @param string $context
     * @param bool $exceptionOnError
     */
    public static function editaceDochazkyEvent($database, $pp, $den, $context, $exceptionOnError = true)
    {
        $xmlParams = [
            'idPP' => $pp ->idPP,
            'RC' => $pp ->rc,
            'osobniCislo' => $pp ->osobniCislo,
            'datumNastupu' => $pp ->datumNastupu,
            'den' => $den
        ];

        RonPortalXMLForm::xmlEvents($database, RonPortalXMLEvents::EDITACE_DOCHAZKY_EVENT,
            $xmlParams, $context, false, $exceptionOnError);

    }
}
