<?php
/**
 * stanislav.elbl, 27.02.2021
 * Formular pro hromadne rozdeleni dennich vysledku osoby - stary zpusob zobrazeni (dny na radcich)
 */

namespace RonPortal\Editace;

use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use RonPortal\Dochazka\DenniVysledek;
use RonPortal\Dochazka\SchvaleniDochazky;
use RonPortal\Kalendar;
use RonPortal\NastaveniUzivatele;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\System\AgentManager;
use RonPortal\TypRozdeleni;

class DenRozdeleniDV
{
    /** @var int */
    var $datum;

    /** @var ZaznamRozdeleniVysledku[]   Nactene zaznamy */
    var $zaznamy;

    /** @var float      hodnota casove slozky pro porovnani */
    var $odpracovano;

    /** @var string     hodnota zadana uzivatelem */
    var $zadanaHodnota;
    /** @var string     kolik zbyva rozdelit */
    var $zbyvaStr;
    /** @var string     textova hodnota - pri nacteni totez co zbyvastr, ale muze se zmenit podle POST */
    var $hodnotaStr;

    /** @var float */
    var $soucet;

    /** @var float      kolik je celkem rozdeleno na druhou casovou slozku */
    var $celkemCasovaSlozka2 = 0;
    /** @var string     pouzije se pouze pri POST, jinak se nacita dynamicky po vyberu rozdeleni */
    var $hodnotaRozdeleni2Str = '';


    /* @param int $datum */
    function __construct($datum)
    {
        $this->datum = $datum;
        $this->zaznamy = [];
    }


    /**
     * @return array
     */
    function getJsonObject()
    {
        $result = [];
        $result['datum'] = date('j.n.Y', $this->datum);
        $result['odpracovano'] = StringUtils::format('minuty', round($this->odpracovano * 60));
        $result['zaznamy'] = [];
        foreach ($this->zaznamy as $zaznam) {
            $result['zaznamy'][] = ['rozdeleni' => StringUtils::jsonText($zaznam->rozdeleni, false), 'casovaSlozka' => StringUtils::jsonText($zaznam->casovaSlozka), 'id' => $zaznam->js_id];
        }

        return $result;
    }

}


/**
 * Class HromadneRozdeleniDVFormular
 */
class HromadneRozdeleniDVFormular implements ViewObject
{
    const SETTINGS_FORMAT = 'dv_formatzobrazeni';

    /** @var Database */
    private $database;

    /* @var PracovniPomer */
    var $pp;
    /* @var Obdobi */
    var $obdobi;
    /* @var SchvaleniDochazky */
    var $schvaleni;

    /* @var int */
    var $kodTypuRozdeleni;
    /* @var TypRozdeleni */
    var $typRozdeleni;

    /* @var int         uzivatelem zadane datum od */
    var $editovatDatumOd;
    /* @var int         uzivatelem zadane datum do */
    var $editovatDatumDo;

    /** @var string */
    var $kalendar;

    /* @var DenRozdeleniDV[] */
    var $dataList;

    var $readOnly;

    var $vybraneRozdeleni;

    /** @var string     Kod casove slozky pro druhe rozdeleni */
    var $casovaSlozka2;
    /** @var string     Nadpis druhe casove slozky k rozdeleni */
    var $nadpisCS2;
    /** @var string     Nadpis druhe casove slozky - celkovy soucet */
    var $nadpisCS2Celkem;



    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param Obdobi $obdobi
     */
    function __construct($database, $pp, $obdobi)
    {
        $this ->database = $database;
        $this ->pp = $pp;
        $this ->obdobi = $obdobi;

        $this ->kodTypuRozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::DV_TYP_ROZDELENI);
    }



    /**
     * @param int $datum
     * @return DenRozdeleniDV
     */
    private function vratZaznamProDen($datum)
    {
        foreach ($this ->dataList as $den)
            if ($den ->datum == $datum) return $den;

        return null;
    }


    private function inicializujData()
    {
        $this ->dataList = [];

        for ($datum = $this ->obdobi ->zacatek; $datum <= $this ->obdobi ->konec; $datum = DateUtils::incDay($datum, 1)) {
            $denrozdeleni = new DenRozdeleniDV($datum);
            $this ->dataList[] = $denrozdeleni;
        }
    }



    function nactiDataOdpracovano()
    {
        if (!RonPortalSettings::getValue(RonPortalSettings::DV_CASOVA_SLOZKA_SOUCET)) {
            return;
        }

        $dv = DenniVysledek::nactiDenniVysledky($this ->database, $this ->pp, $this ->obdobi ->zacatek, $this ->obdobi ->konec, RonPortalSettings::getValue(RonPortalSettings::DV_CASOVA_SLOZKA_SOUCET));
        foreach ($dv as $denniVysledek) {
            $zaznam = $this ->vratZaznamProDen( $denniVysledek ->den );
            if ($zaznam) $zaznam ->odpracovano = $denniVysledek ->hodiny;
        }
    }



    private function nactiDataDB()
    {
        $this ->inicializujData();

        $this ->schvaleni = new SchvaleniDochazky();
        $this ->schvaleni ->load($this ->database,
            $this ->pp ->rc, $this ->pp ->osobniCislo, $this ->pp ->datumNastupu,
            $this ->obdobi ->zacatek);

        $this ->typRozdeleni = TypRozdeleni::getTypRozdeleni($this ->database, $this ->kodTypuRozdeleni);

        $this ->kalendar = Kalendar::vratKalendarOsoby($this ->database,
            $this ->pp ->rc, $this ->pp ->osobniCislo, $this ->pp ->datumNastupu,
            $this ->obdobi ->zacatek, $this ->obdobi ->konec);

        $this ->nactiDataOdpracovano();

        // Sloupec pro editaci dalsi casove slozky (slouzi napr. pro dodatecne rozdeleni prescasu na zakazky)
        $this ->casovaSlozka2 = RonPortalSettings::getValue('dv_casovaslozka2');
        if ($this ->casovaSlozka2) {
            $this ->nadpisCS2 = \Ron\ronText('Rozdeleni_DV_CS_2');
            $this ->nadpisCS2Celkem = \Ron\ronText('Rozdeleni_DV_CS_2');
            if (!$this ->nadpisCS2  ||  !$this ->nadpisCS2Celkem) {
                $params = $this ->database->createParameters();
                $popis = DBUtils::getSqlValue($this->database,
                    'SELECT Popis FROM TypCasoveSlozkyMzdy WHERE Kod = ' . $params->addParameter($this->casovaSlozka2),
                    $params);
                if (!$this ->nadpisCS2) {
                    $this->nadpisCS2 = $popis;
                }
                if (!$this ->nadpisCS2Celkem) {
                    $this->nadpisCS2Celkem = \Ron\ronText('Celkem') . ' ' . $popis;
                }
            }
        }

        // cache pro prava na schvalovani - aby pro kazdy zaznam nezjistoval, jestli je uzivatel v rozdeleni
        $pravaSchvalovat = [];

        $slozky = explode(';', RonPortalSettings::getValue(RonPortalSettings::DV_CASOVE_SLOZKY));

        $itemid = 1;
        $items = ZaznamRozdeleniVysledku::nactiZaznamyOsoby($this ->database, $this ->kodTypuRozdeleni, $this ->pp, null, $this ->obdobi ->zacatek, $this ->obdobi ->konec);
        foreach ($items as $item) {
            if (in_array($item ->casovaSlozka, $slozky, true)) {
                $denRozdeleni = $this->vratZaznamProDen($item->den);
                if ($denRozdeleni) {
                    $item->js_id = 'zaznamrozdeleniDV_' . $itemid;
                    $itemid++;

                    if (!isset($pravaSchvalovat[$item->rozdeleni])) {
                        // doplni cache
                        $pravaSchvalovat[$item->rozdeleni] = Prava::vratPrava('schval_rozdeleni_dv')->jePravoNaRozdeleni($this->kodTypuRozdeleni, $item->rozdeleni, $item->den);
                    }
                    $item->pravoSchvalovat = $pravaSchvalovat[$item->rozdeleni];

                    $denRozdeleni->zaznamy[] = $item;
                    $denRozdeleni->soucet += $item->hodiny;
                }
            } else if (!strcasecmp($item->casovaSlozka, $this ->casovaSlozka2)) {
                $denRozdeleni = $this->vratZaznamProDen($item->den);
                if ($denRozdeleni) {
                    $denRozdeleni ->celkemCasovaSlozka2 += $item ->hodiny;
                }
            }
        }

        // prednastavit hodnotu (=zbytek)
        foreach ($this ->dataList as $den) {
            $zbyva = RonPortalSettings::getValue(RonPortalSettings::DV_CASOVA_SLOZKA_SOUCET) ? round($den ->odpracovano * 60 - $den ->soucet * 60) : 0;
            $den ->zbyvaStr = $zbyva ? StringUtils::format('minuty', $zbyva) : '';
            $den ->hodnotaStr = $den ->zbyvaStr;
        }

        return true;
    }


    /**
     * @return boolean
     */
    function nactiData()
    {
        Prava::nastavObdobiProPrava( $this ->obdobi ->zacatek, $this ->obdobi ->konec );

        // nastavi editovany datum od-do (podle volby uzivatele - muze to byt jen cast obdobi)
        $this ->editovatDatumOd = $this ->obdobi ->zacatek;
        $this ->editovatDatumDo = $this ->obdobi ->konec;

        if (isset($_GET['datumod'])) {
            try {
                $this ->editovatDatumOd = DateUtils::urlStringToDate($_GET['datumod']);

                if ($this ->editovatDatumOd < $this ->obdobi ->zacatek) {
                    $this ->editovatDatumOd = $this ->obdobi ->zacatek;
                }
                if ($this ->editovatDatumOd > $this ->obdobi ->konec) {
                    $this ->editovatDatumOd = $this ->obdobi ->konec;
                }
            } catch (\Exception $e) {
                // vyjimku ignoruje
            }
        }

        if (isset($_GET['datumdo'])) {
            try {
                $this ->editovatDatumDo = DateUtils::urlStringToDate($_GET['datumdo']);

                if ($this ->editovatDatumDo < $this ->obdobi ->zacatek) {
                    $this ->editovatDatumDo = $this ->obdobi ->zacatek;
                }
                if ($this ->editovatDatumDo > $this ->obdobi ->konec) {
                    $this ->editovatDatumDo = $this ->obdobi ->konec;
                }
            } catch (\Exception $e) {
                // vyjimku ignoruje
            }
        }

        $this ->readOnly = $this ->obdobi ->uzavreno;

        $prava = Prava::vratPrava('prohlizeni');
        $lzezobrazit = $prava ->lzeZobrazitPP($this ->pp ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec);

        if (!$this ->readOnly || !$lzezobrazit) {
            // jeste zkontroluje pravo na editaci
            $prava = Prava::vratPrava('editace');

            $jePravoEditovat = $prava ->lzeZobrazitPP($this ->pp ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec);
            if (!$jePravoEditovat) {
                // nemuze editovat, jeste zkontroluje, jestli ma pravo editovat rozdeleni vysledku
                $prava = Prava::vratPrava('editace_r');
                $jePravoEditovat = $prava ->lzeZobrazitPP($this ->pp ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec);
            }

            if ($jePravoEditovat) {
                // ma pravo na editaci => muze zobrazit
                $lzezobrazit = true;
            } else {
                // nema pravo na editaci => readonly
                $this ->readOnly = true;
            }
        }

        if (!$lzezobrazit) {
            // nema pravo zobrazit ani editovat
            throw new NeniPravoException();
        }

        $this ->nactiDataDB();
    }


    function lzeEditovatRozdeleni2($den)
    {
        if ($this ->schvaleni ->schvaleni3 >= $den) return false;
        if ($this ->readOnly) return false;

        return true;
    }

    /**
     * @param DenRozdeleniDV $den
     * @param string $rozdeleni
     * @param string $casovaSlozka
     * @param float  $hodnota
     * @throws \Exception
     */
    function zapisRozdeleni($den, $rozdeleni, $casovaSlozka, $hodnota)
    {
        RozdeleniVysledkuUtils::overSchvaleni($this ->database, $rozdeleni, $den ->datum);

        $rec = new RozdeleniDennichVysledkuRecord($this ->database,
            (int)$this ->kodTypuRozdeleni, $this ->pp ->rc, $this ->pp ->osobniCislo, $this ->pp ->datumNastupu,
            $den ->datum, $this);

        $params = $this ->database ->createParameters();
        $filtr =
            ' Rozdeleni = ' . $params->addParameter($rozdeleni) .
            ' AND TypCasoveSlozkyMzdy = ' . $params->addParameter($casovaSlozka);

        if ($rec ->loadRecord($filtr, $params)) {
            // zaznam uz existuje - pricte zadanou hodnotu
            $rec ->fHodnota ->setNewValue($rec ->fHodnota ->originalValue + $hodnota);

            if ( round($rec ->fHodnota ->value, 5) > 0 ) {   // nesmi byt zaporne
                if ($rec ->fHodnota ->value  &&  RonPortalSettings::getIntValue(RonPortalSettings::DV_MINIMALNI_CAS) > 0  &&  60 * $rec ->fHodnota ->value < RonPortalSettings::getIntValue(RonPortalSettings::DV_MINIMALNI_CAS)) {
                    throw new \Exception( sprintf(\Ron\ronText('Nelze_vykazat_mene'), RonPortalSettings::getIntValue(RonPortalSettings::DV_MINIMALNI_CAS)));
                }

                $rec ->updateRecord($filtr, $params);
            } else {
                $rec ->deleteRecord($filtr, $params);
            }
        } else {
            $rec ->fRozdeleni ->value = $rozdeleni;
            $rec ->fSlozka ->value = $casovaSlozka;

            $rec ->fHodnota ->value = $hodnota;

            if (RonPortalSettings::getIntValue(RonPortalSettings::DV_MINIMALNI_CAS) > 0  &&  60 * $rec ->fHodnota ->value < RonPortalSettings::getIntValue(RonPortalSettings::DV_MINIMALNI_CAS)) {
                throw new \Exception( sprintf(\Ron\ronText('Nelze_vykazat_mene'), RonPortalSettings::getIntValue(RonPortalSettings::DV_MINIMALNI_CAS)));
            }

            $rec ->insertRecord();
        }
    }


    /**
     * @param DenRozdeleniDV $den
     * @param string $rozdeleni
     * @param float $hodnota
     * @return bool
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function zapisRozdeleni2($den, $rozdeleni, $hodnota)
    {
        //RozdeleniVysledkuUtils::overSchvaleni($this ->database, $rozdeleni, $den ->datum);
        if (!$this ->casovaSlozka2) return false;

        $rec = new RozdeleniDennichVysledkuRecord($this ->database,
            (int)$this ->kodTypuRozdeleni, $this ->pp ->rc, $this ->pp ->osobniCislo, $this ->pp ->datumNastupu,
            $den ->datum, $this);

        $params = $this ->database ->createParameters();
        $filtr =
            ' Rozdeleni = ' . $params->addParameter($rozdeleni) .
            ' AND TypCasoveSlozkyMzdy = ' . $params->addParameter($this ->casovaSlozka2);

        if ($rec ->loadRecord($filtr, $params)) {
            // zaznam uz existuje - provede update nebo delete
            if ($hodnota) {
                $rec ->fHodnota ->setNewValue($hodnota);
                if (!$rec ->isChanged()) return false;

                $rec ->updateRecord($filtr, $params);
                return true;
            } else {
                $rec ->deleteRecord($filtr, $params);
                return true; // doslo ke zmene
            }
        } else {
            if ($hodnota) {
                $rec->fRozdeleni->value = $rozdeleni;
                $rec->fSlozka->value = $this->casovaSlozka2;
                $rec->fHodnota->value = $hodnota;

                $rec->insertRecord();
                return true;
            }
        }

        return false;
    }


    /**
     * Vymaze rozdeleni za cele odfiltrovane obdobi
     * @param int $datumOd
     * @param int $datumDo
     * @return bool
     */
    function vymazData($datumOd, $datumDo)
    {
        $datum = $this ->schvaleni ->prvniEditovatelnyDen($this ->pp ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec);
        if (!$datum) {
            // nemuzu editovat
            return;
        }

        if ($datum > $datumOd) {
            $datumOd = $datum;
        }

        $rec = new RozdeleniDennichVysledkuRecord($this ->database,
            (int)$this ->kodTypuRozdeleni, $this ->pp ->rc, $this ->pp ->osobniCislo, $this ->pp ->datumNastupu,
            null, $this);
        $params = $this ->database ->createParameters();
        $rec ->setWhere(
            'DenniVysledek_R_E.RC = ' . $params->addParameter($this ->pp ->rc) .
            ' AND DenniVysledek_R_E.OsobniCislo = ' . $params->addParameter($this ->pp ->osobniCislo) .
            ' AND DenniVysledek_R_E.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->pp ->datumNastupu)) .
            ' AND DenniVysledek_R_E.Den >= ' . $params ->addParameter($this ->database ->dateToParam($datumOd)) .
            ' AND DenniVysledek_R_E.Den <= ' . $params ->addParameter($this ->database ->dateToParam($datumDo)) .
            ' AND DenniVysledek_R_E.KodTypuRozdeleni = ' . $params ->addParameter((int)$this ->kodTypuRozdeleni)  .
            // odfiltruje jen zaznamy, ktere nejsou schvalene
            ' AND NOT EXISTS (' .
            ' SELECT * FROM SchvaleniRozdeleniDoch ' .
            ' WHERE SchvaleniRozdeleniDoch.Den = DenniVysledek_R_E.Den' .
            ' AND SchvaleniRozdeleniDoch.KodTypuRozdeleni = DenniVysledek_R_E.KodTypuRozdeleni' .
            ' AND SchvaleniRozdeleniDoch.Rozdeleni = DenniVysledek_R_E.Rozdeleni' .
            ' AND SchvaleniRozdeleniDoch.Schvaleno = 1' .
            ')',
            $params);

        $rec ->deleteRecords();

        AgentManager::prepocetDochazky($this ->pp ->idPP, $datumOd, $datumDo);
    }


    /**
     * Vraci Redirect na stejny formular nebo na zpracovanou dochazku
     * @param $showForm
     * @return Redirect
     */
    function redirect($showForm)
    {
        if ($showForm) {
            $params = '&idpp=' . $this ->pp ->idPP . '&obdobi=' . DateUtils::dateToUrlString($this ->obdobi ->zacatek);

            if ($this ->editovatDatumOd != $this ->obdobi ->zacatek) {
                $params .= '&datumod=' . DateUtils::dateToUrlString($this ->editovatDatumOd);
            }

            if ($this ->editovatDatumDo != $this ->obdobi ->konec) {
                $params .= '&datumdo=' . DateUtils::dateToUrlString($this ->editovatDatumDo);
            }

            return new Redirect(null, '?menuid=rozdelenidv' . $params);
        }

        return new Redirect(null, '?menuid=zpracovanadochazka&osoba=' . $this ->pp ->idPP . '&obdobi=' . DateUtils::dateToUrlString($this ->obdobi ->zacatek));
    }

    /**
     * Zpracuje POST a vraci Redirect nebo sam sebe
     * @return Redirect|ViewObject
     * @throws \Exception
     */
    function zpracujPost()
    {
        if (isset($_POST['stornoDV'])) return $this ->redirect(false); // redirect na zpracovanou dochazku

        // nacte zadane datum od - do
        $result = true;

        try {
            $datumOd = DateUtils::dateStringToTimestamp($_POST['rozdelenidv_datumod']);
            $this ->editovatDatumOd = $datumOd;
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            $result = false;
            $datumOd = 0;
        }

        try {
            $datumDo = DateUtils::dateStringToTimestamp($_POST['rozdelenidv_datumdo']);
            $this ->editovatDatumDo = $datumDo;
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            $result = false;
            $datumDo = 0;
        }

        if ($result) {
            if ($datumOd > $datumDo) {
                MessageManager::addErrorMessage(\Ron\ronText('Pocatecni_datum_vetsi_nez_koncove'));
                $result = false;
            }
        }

        if (isset($_POST['vymazatDV'])) {
            $this ->vymazData($datumOd, $datumDo);

            return $this ->redirect(true); // znovu zobrazi stejny formular
        }

        // vybrane rozdeleni a casova slozka
        $rozdeleni = trim($_POST['kod_rozdeleni']);
        $casovaSlozka = trim($_POST['casova_slozka']);

        if ($rozdeleni != '') {
            $this ->vybraneRozdeleni = $rozdeleni;
        }

        if ($rozdeleni == '') {
            $result = MessageManager::addErrorMessage( \Ron\ronText('Chybi_povinna_hodnota') );
        }
        if ($casovaSlozka == '') {
            $result = MessageManager::addErrorMessage( \Ron\ronText('Chybi_povinna_hodnota') );
        }

        // zkontroluje zadane hodnoty DV
        foreach ($this ->dataList as $index => $den) {
            // pole DenRozdeleniDV
            if ($den ->datum >= $datumOd && $den ->datum <= $datumDo) {
                $den ->hodnotaStr = $_POST["rozdelenidv_$index"];

                if (trim($den ->hodnotaStr) != '') {
                    try {
                        $hodnota = DateUtils::timeStringToFloat(trim($den ->hodnotaStr), true); // $povolitZkraceny = true

                        if (RonPortalSettings::getValue(RonPortalSettings::DV_CASOVA_SLOZKA_SOUCET) && round($den ->odpracovano - $den ->soucet - $hodnota, 5) < 0) {
                            // prekrocen limit
                            MessageManager::addErrorMessage(date('j.n.Y', $den ->datum) . ': ' . \Ron\ronText('Nelze_rozdelit_vice_dv') );
                            $result = false;
                        }

                    } catch (\Exception $e) {
                        $result = MessageManager::addErrorMessage( \Ron\ronText('Spatna_hodnota_pro') . date('j.n.Y', $den ->datum), "rozdelenidv_$index" );
                    }
                }

                if ($this ->casovaSlozka2) {
                    $den ->hodnotaRozdeleni2Str = $_POST["rozdelenidv2_$index"];
                    if (trim($den ->hodnotaRozdeleni2Str) != '') {
                        try {
                            $hodnota = DateUtils::timeStringToFloat(trim($den ->hodnotaRozdeleni2Str), true); // $povolitZkraceny = true
                        } catch (\Exception $e) {
                            $result = MessageManager::addErrorMessage( \Ron\ronText('Spatna_hodnota_pro') . date('j.n.Y', $den ->datum), "rozdelenidv2_$index" );
                        }
                    }
                }
            }
        }

        if (!$result) {
            return $this;
        }

        $this ->vybraneRozdeleni = null; // v dalsim kroku musi uzivatel zase vybrat

        $prvniZmena = 0;

        // zapise DV
        foreach ($this ->dataList as $index => $den) { // pole DenRozdeleniDV
            if ($den ->datum >= $datumOd && $den ->datum <= $datumDo) {
                if (trim($den ->hodnotaStr) != '') {
                    // melo by byt OK, format jiz byl zkontrolovan
                    $hodnota = DateUtils::timeStringToFloat(trim($den ->hodnotaStr), true); // $povolitZkraceny = true

                    // zadava se formou "rozdilu" - tj. pridat nebo odebrat
                    if ($hodnota) {
                        try {
                            $this->zapisRozdeleni($den, $rozdeleni, $casovaSlozka, $hodnota);
                            if (!$prvniZmena) $prvniZmena = $den->datum;
                        } catch (\Exception $e) {
                            MessageManager::addErrorMessage(date('j.n.Y', $den->datum) . ': ' . $e->getMessage());
                            $result = false;
                        }
                    }

                }

                if ($this ->casovaSlozka2) {
                    // rozdeleni druhe casove slozky se zadava v absolutni hodnote
                    try {
                        $hodnota = $den->hodnotaRozdeleni2Str != ''
                            ? DateUtils::timeStringToFloat(trim($den->hodnotaRozdeleni2Str), true) // $povolitZkraceny = true
                            : 0;

                        if ($this->zapisRozdeleni2($den, $rozdeleni, $hodnota)) {
                            if (!$prvniZmena) $prvniZmena = $den->datum;
                        }
                    } catch (\Exception $e) {
                        MessageManager::addErrorMessage(date('j.n.Y', $den->datum) . ': ' . $e->getMessage());
                        $result = false;
                    }
                }
            }
        }

        if ($prvniZmena) {
            AgentManager::prepocetDochazky($this ->pp ->idPP, $prvniZmena, $this ->obdobi ->konec);
        }

        NastaveniUzivatele::setUserSettingsItem($this ->database, NastaveniUzivatele::CASOVA_SLOZKA_ROZDELENI_DV, $casovaSlozka);

        $this ->nactiDataDB();

        if ($result) {
            // vysledky zapsany - pokracuje bud stejnym formularem nebo na zpracovanou dochazku (viz metoda redirect)
            return $this ->redirect(isset($_POST['zapsatDV']));
        }

        return $this;
    }


    /**
     * @param $hodnota
     * @param bool $kod
     * @param bool $chyba
     */
    function zobrazVyberRozdeleniVysledku($hodnota, $kod, $chyba = false)
    {
        $sql = RozdeleniVysledkuUtils::getSQLVyberRozdeleniDV($this ->database, $this ->pp, $this ->kodTypuRozdeleni, $hodnota, $this->obdobi->zacatek, $this->obdobi->konec, $kod);

        $combo = new ComboBox($kod ? 'kod_rozdeleni1' : 'kod_rozdeleni');
        $combo ->alwaysShowSelected = true;
        $combo ->emptyRow = (isset($hodnota) && $hodnota != '') ? null : ' ';
        $combo ->className = 'form-control' . ($chyba ? 'error' : '');
        $combo ->cssClassIndex = 2;

        if ($kod) {
            $combo->captionIndex = 0;
            $combo->titleIndex = 1;
            $combo->htmlParams = "id='vyberRozdeleni'" . ($this->readOnly ? ' disabled="disabled"' : '');
        } else {
            $combo ->htmlParams = "id='vyberRozdeleniPopis'" . ($this ->readOnly ? ' disabled="disabled"' : '');
        }

        $combo ->loadData($this ->database, $sql, $hodnota);
        $combo ->render();
    }


    function zobrazCasoveSlozkyVysledku()
    {
        $sql = RozdeleniVysledkuUtils::getSQLVyberCasSlozkyDV($this ->database);

        $value = '';

        if (isset($_POST['casova_slozka'])) {
            $value = $_POST['casova_slozka'];
        } else {
            $slozky = explode(';', RonPortalSettings::getValue(RonPortalSettings::DV_CASOVE_SLOZKY));
            if (trim($slozky[0]) != '') $value = trim($slozky[0]); // natvrdo zadano v nastaveni intrawebu
            else {
                // nacte z nastaveni uzivatele
                NastaveniUzivatele::getUserSettingsItem($this ->database, NastaveniUzivatele::CASOVA_SLOZKA_ROZDELENI_DV);
            }
        }

        $combo = new ComboBox('casova_slozka');
        //if ($chyba) $combo -> f_classname = 'app_input_error';
        $combo ->emptyRow = isset($value) ? false : ' ';
        $combo ->loadData($this ->database, $sql, $value);
        $combo ->render();
    }


    /**
     * Exportuje datovy objekt pro javascript
     */
    function showJSONData()
    {
        $dataRozdeleniDV = [
            'datumOd' => StringUtils::jsonDate($this ->obdobi ->zacatek),
            'datumDo' => StringUtils::jsonDate($this ->obdobi ->konec),
            'idPP' => $this ->pp ->idPP
        ];

        $dny = [];
        foreach ($this ->dataList as $den) {
            $dny[] = $den ->getJsonObject();
        }

        $dataRozdeleniDV['dny'] = $dny;

        echo '<script type="text/javascript">';
        echo 'ronPortal.rozdeleniDV(' . json_encode($dataRozdeleniDV) . ');';
        echo '</script>';
    }


    /**
     * @param ZaznamRozdeleniVysledku $item
     * @return string
     */
    function getItemText(ZaznamRozdeleniVysledku $item): string
    {
        $className = 'js zaznamrozdelenidv' . ($item->rozdeleniSchvaleno ? ' schvaleno' : '');
        $hint = StringUtils::htmlTextConvert($item->popisRozdeleni . ' - ' . $item->popisCasoveSlozky);

        $result =
            "<span id='{$item ->js_id}' class='$className' title='$hint' data-ron-rozdeleni='" . StringUtils::htmlTextConvert($item->rozdeleni) . "'>";

//        if ($item->pravoSchvalovat) {
//            $result .= '<a href="?menuid=rozdelenidv&rozdeleni=' . StringUtils::htmlTextConvert($item->rozdeleni) . '&obdobi=' . StringUtils::jsonDate($item->den) . '">';
//        }

        if (RonPortalSettings::getValue(self::SETTINGS_FORMAT)) {
            $result .= str_replace(
                [
                    '%HODINY%', '%KOD%', '%POPIS%', '%KOD_CS%', '%POPIS_CS%',
                    '%UZIVATELSKA1%', '%UZIVATELSKA2%', '%UZIVATELSKA3%', '%UZIVATELSKA4%', '%UZIVATELSKA5%',
                    '%UZIVATELSKA6%', '%UZIVATELSKA7%', '%UZIVATELSKA8%', '%UZIVATELSKA9%', '%UZIVATELSKA10%'
                ],
                [
                    StringUtils::format('minuty', round($item->hodiny * 60)),
                    StringUtils::htmlTextConvert($item ->rozdeleni), // %KOD%
                    StringUtils::htmlTextConvert($item->popisRozdeleni),
                    // casova slozka
                    StringUtils::htmlTextConvert($item->casovaSlozka),
                    StringUtils::htmlTextConvert($item->popisCasoveSlozky),
                    // uzivatelske
                    StringUtils::htmlTextConvert($item ->uzivatelska1),
                    StringUtils::htmlTextConvert($item ->uzivatelska2),
                    StringUtils::htmlTextConvert($item ->uzivatelska3),
                    StringUtils::htmlTextConvert($item ->uzivatelska4),
                    StringUtils::htmlTextConvert($item ->uzivatelska5),
                    StringUtils::htmlTextConvert($item ->uzivatelska6),
                    StringUtils::htmlTextConvert($item ->uzivatelska7),
                    StringUtils::htmlTextConvert($item ->uzivatelska8),
                    StringUtils::htmlTextConvert($item ->uzivatelska9),
                    StringUtils::htmlTextConvert($item ->uzivatelska10)
                ],

                RonPortalSettings::getValue(self::SETTINGS_FORMAT)
            );
        } else {
            $result .= StringUtils::htmlTextConvert($item->rozdeleni) . ' (' . StringUtils::format('minuty', round($item->hodiny * 60)) . ') ' . StringUtils::htmlTextConvert($item->casovaSlozka);
        }

        if ($item->rozdeleniSchvaleno) {
            $result .= '<span class="inline-glyph fa glyph-lock"></span>';
        }
        $result .= '</span>';

        return $result;
    }


    function render()
    {
        if (!isset($this ->dataList)) {
            throw new \Exception('Chyba ve funkci HromadneRozdeleniDVFormular::render - nejsou nactena data!');
        }

        $_SESSION['editace_backaction'] = 'rozdelenidv'; // aby se po editaci dne vracel do seznamu rozdeleni DV

        require 'ronportal/editace/view_rozdeleni_dv_v1.php';

        $this ->showJSONData();
    }
}
