<?php
/**
 * stanislav.elbl, 08.07.2015
 */

namespace RonPortal\ELearning;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\StringUtils;
use RonPortal\RonPortalDBRecord;

class ObsahKurzu
{
    /** @var int */
    private $id;
    /** @var int */
    private $idRevizeKurzu;

    /** @var int */
    private $poradi;

    /** @var int */
    private $stav;  // 1 = aktivni, 2 = zrusena

    /** @var string */
    private $nazev;

    /** @var string */
    private $text;

    /** @var string   kod pro html odkazy */
    private $kodStrany;

    private $zobrazitHlavicku;
    private $zobrazitPaticku;

    private $povolitPokracovatBezTestu;

    /** @var int  Pripojeny test */
    var $druhHodnoceni;
    var $temaCislo;

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return int
     */
    public function getIdRevizeKurzu()
    {
        return $this->idRevizeKurzu;
    }

    /**
     * @param int $idRevizeKurzu
     */
    public function setIdRevizeKurzu($idRevizeKurzu)
    {
        $this->idRevizeKurzu = $idRevizeKurzu;
    }


    /**
     * @return int
     */
    public function getPoradi()
    {
        return $this->poradi;
    }

    /**
     * @return int
     */
    public function getStav()
    {
        return $this->stav;
    }


    /**
     * @return string
     */
    public function getNazev()
    {
        return $this->nazev;
    }

    /**
     * @param string $nazev
     */
    public function setNazev($nazev)
    {
        $this->nazev = $nazev;
    }



    /**
     * @return string
     */
    public function getText()
    {
        return $this->text;
    }

    /**
     * @return string
     */
    public function getKodStrany()
    {
        return $this->kodStrany;
    }

    /**
     * @return mixed
     */
    public function getZobrazitHlavicku()
    {
        return $this->zobrazitHlavicku;
    }

    /**
     * @return mixed
     */
    public function getZobrazitPaticku()
    {
        return $this->zobrazitPaticku;
    }

    /**
     * @return mixed
     */
    public function getPovolitPokracovatBezTestu()
    {
        return $this->povolitPokracovatBezTestu;
    }




    /**
     * @param Database $database
     * @param int $idRevizeKurzu
     * @return ObsahKurzu[]|bool
     */
    public static function loadObsah($database, $idRevizeKurzu)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT Id, Poradi, Nazev, KodStrany, ZobrazitHlavicku, ZobrazitPaticku, ' . // 0..5
            $database->castWideMemo('Obsah') . ', ' . // 6
            ' DruhHodnoceni, TemaCislo, PovolitPokracovatBezTestu' . // 7..9
            ' FROM ObsahEKurzu' .
            ' WHERE IdRevizeKurzu = ' . $params ->addParameter((int)$idRevizeKurzu) .
            '   AND Stav <> 2' .
            ' ORDER BY Poradi, Id';

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani obsahu kurzu!');

        $result = [];
        while ($rowData = $cursor->fetch()) {
            $obsah = new ObsahKurzu();

            $obsah->id = (int)$rowData[0];
            $obsah->idRevizeKurzu = (int)$idRevizeKurzu;
            $obsah->poradi = (int)$rowData[1];
            $obsah->nazev = $rowData[2];
            $obsah->kodStrany = $rowData[3];
            $obsah->zobrazitHlavicku = (int)$rowData[4];
            $obsah->zobrazitPaticku = (int)$rowData[5];
            $obsah->text = $rowData[6];

            $obsah ->druhHodnoceni = $rowData[7];
            $obsah ->temaCislo = $rowData[8];
            $obsah ->povolitPokracovatBezTestu = (int)$rowData[9];

            $result[] = $obsah;
        }

        return $result;
    }


    public function getJsonObsah()
    {
        if (!isset($this->text)) return null;

        $text = str_replace('%DOWNLOAD%', '%DOWNLOAD%&amp;revize=' . $this ->idRevizeKurzu, $this->text);

        return StringUtils::jsonText($text);
    }


    public function getJson()
    {
        return [
            'id' => $this->id,
            'poradi' => $this->poradi,
            'stav' => $this ->stav,
            'nazev' => StringUtils::jsonText($this->nazev),
            'kodStrany' => isset($this ->kodStrany) ? StringUtils::jsonText($this ->kodStrany) : null,
            'zobrazitHlavicku' => (int)$this ->zobrazitHlavicku,
            'zobrazitPaticku' => (int)$this ->zobrazitPaticku,
            'obsah' => $this ->getJsonObsah(), //  isset($this->text) ? StringUtils::jsonText($this->text) : null,
            'druhHodnoceni' => isset($this ->druhHodnoceni) ? (int)$this ->druhHodnoceni : null,
            'temaCislo' => isset($this ->temaCislo) ? (int)$this ->temaCislo : null,
            'povolitPokracovatBezTestu' => $this ->povolitPokracovatBezTestu
        ];
    }


    /**
     * @param Database $database
     * @param int $idRevizeKurzu
     * @param object|string $logDescription
     * @param string $obsah
     */
    public function insertRecord($database, $idRevizeKurzu, $logDescription, $obsah = null)
    {
        $rec = new RonPortalDBRecord($database, 'ObsahEKurzu', $logDescription);

        $this ->idRevizeKurzu = (int)$idRevizeKurzu;

        $this ->poradi = SQLUtils::newSerialNumber($database, 'ObsahEKurzu', 'Poradi', 'IdRevizeKurzu = ' . $this ->idRevizeKurzu . ' AND Stav <> 2');
        $this ->id = SQLUtils::newSerialNumber($database, 'ObsahEKurzu', 'Id');
        $this ->stav = 1;
        $this ->zobrazitHlavicku = 1;
        $this ->zobrazitPaticku = 1;

        if (!$this ->nazev) {
            $this ->nazev = \Ron\ronText('Nazev_nove_strany') . ' ' . $this ->poradi;
        }

        $rec ->addField('Id', 'int', false, 0, $this ->id);
        $rec ->addField('IdRevizeKurzu', 'int', false, 0, $this ->idRevizeKurzu);
        $rec ->addField('Poradi', 'int', false, 0, $this ->poradi);
        $rec ->addField('Stav', 'int', false, 0, $this ->stav);

        $rec ->addField('Nazev', '', false, 160, $this ->nazev);

        $rec ->addField('ZobrazitHlavicku', 'int', false, 0, 1);
        $rec ->addField('ZobrazitPaticku', 'int', false, 0, 1);

        if ($obsah) {
            $rec ->addField('Obsah', '', false, 0, $obsah);
            $this ->text = $obsah;
        }

        $rec ->insertRecord();
    }
}
