<?php
/**
 * stanislav.elbl, 15.09.2015
 */

namespace RonPortal\ELearning;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SQLUtils;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\Controller;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\TextViewObject;
use Ron\MVC\ViewObject;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\Log;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\System\Report;
use RonPortal\System\ReportOutputObject;
use RonPortal\User;


class PrezentaceController implements Controller
{
    /** @var Database */
    private $database;

    /**
     * PrezentaceController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    /**
     * @param RevizeEKurzu $revize
     * @param int $index
     * @param KurzOsoby $kurzOsoby
     * @return string
     */
    public function replaceLinks($revize, $index, $kurzOsoby = null)
    {
        $text = ($revize ->getObsah()[$index] ->getText()) ?? '';

        $baseLink = $kurzOsoby
            ? '?menuid=prezentace&kurzosoby=' . $kurzOsoby ->id
            : '?menuid=prezentace&revize=' . $revize ->getId();

        Model::addData('baseLink', $baseLink);

        $text = str_replace('%NEXT_PAGE%', $baseLink . '&pageindex=' . ($index + 1), $text);
        $text = str_replace('%PREV_PAGE%', $baseLink . '&pageindex=' . ($index ? $index - 1 : 0), $text);
        $text = str_replace('%FIRST_PAGE%', $baseLink . '&pageindex=0', $text);
        $text = str_replace('%LAST_PAGE%', $baseLink . '&pageindex=' . ( sizeof($revize ->getObsah()) - 1 ), $text);

        $text = str_replace('%CLOSE%', '#close', $text);
        $text = str_replace('%CERTIFIKAT%', '?menuid=prezentace&action=certifikat&id=' . ($kurzOsoby ? $kurzOsoby ->id : ''), $text);

        // pokud je to bezny uzivatel, tak nahradi odkazy pro download (spravce muze stahnout jakoukoliv prilohu)
        if ($kurzOsoby) {
            // link v textu je ve formatu menuid=kurzy&%DOWNLOAD%&priloha=id
            $text = str_replace('%DOWNLOAD%', 'action=download&kurzosoby=' . $kurzOsoby ->id, $text);
        } else {
            $text = str_replace('%DOWNLOAD%', '%DOWNLOAD%&revize=' . $revize ->getId(), $text);
        }

        // nahradi odkazy na konkretni stranky (se zadanym kodem)
//        list[list.length] = { text: dataEditace.obsahKurzu[i].nazev, value: "%PAGE#" + dataEditace.obsahKurzu[i].kodStrany + "%" }
        $lastPos = 0;
        $newText = '';

        $pos = StringUtils::strpos($text, '%PAGE#', 0);
        while ( $pos !== false ) {
            // najde konec kodu stranky z odkazu %PAGE#<kod_odkazu>%
            $pos2 = StringUtils::strpos($text, '%', $pos + 1);

            if ($pos2) {

                $newText .=
                    StringUtils::substr($text, $lastPos, $pos - $lastPos) . // vse pred %PAGE#...
                    $baseLink .
                    '&code=' . StringUtils::substr($text, $pos + 6, $pos2 - $pos - 6);

                // najde dalsi odkaz
                $lastPos = $pos2 + 1;
                $pos = StringUtils::strpos($text, '%PAGE#', $lastPos);
            } else {
                // nenasel - vse az do konce textu zkopiruje
                $lastPos = $pos;
                $pos = false;
            }
        }

        // doplni konec textu
        $newText .= StringUtils::substr($text, $lastPos);

        return $newText;
    }


    /**
     * Zkontroluje, jestli uz uzivatel dokoncil uspesne cele skoleni (muze obsahovat vice kurzu)
     * @param KurzOsoby $kurzOsoby
     */
    function zkontrolujAbsolvovaniSkoleni($kurzOsoby)
    {
        $params = $this ->database ->createParameters();

        $sql =
            'SELECT EKurzDruhuSkoleni.IdKurzu, RevizeEKurzu.Id, EKurzOsoby.Id, EKurzDruhuSkoleni.Povinne' .
            ' FROM (GenerovaneSkoleni INNER JOIN EKurzDruhuSkoleni ON EKurzDruhuSkoleni.KodDruhuSkoleni = GenerovaneSkoleni.DruhSkoleni)' .
            ' LEFT OUTER JOIN ' .
            '    (RevizeEKurzu INNER JOIN EKurzOsoby ON EKurzOsoby.IdRevize = RevizeEKurzu.Id  AND  EKurzOsoby.Stav = 3)' .
            '   ON EKurzDruhuSkoleni.IdKurzu = RevizeEKurzu.IdKurzu' .
            '  AND EKurzOsoby.RC_Skoleni = GenerovaneSkoleni.RC' .
            '  AND EKurzOsoby.DruhSkoleni = GenerovaneSkoleni.DruhSkoleni' .
            '  AND EKurzOsoby.DatumVznikuPovinnosti = GenerovaneSkoleni.DatumVznikuPovinnosti' .
            ' WHERE GenerovaneSkoleni.RC = ' . $params ->addParameter($kurzOsoby ->rcSkoleni) .
            '   AND GenerovaneSkoleni.DruhSkoleni = ' . $params ->addParameter($kurzOsoby ->druhSkoleni) .
            '   AND GenerovaneSkoleni.DatumVznikuPovinnosti = ' . $params ->addTimestampParameter($kurzOsoby ->datumVznikuPovinnosti);

        $cursor = $this ->database ->getSqlResult($sql, $params);

        $absolvovanoVse = true;

        $logText = "EKurzy pripojene ke skoleni:\n";
        while ($row = $cursor ->fetch()) {
            $logText .=
                'IdKurzu: ' . $row[0] .
                (!$row[3] ? ' (nepovinny)' : '') .
                ($row[1] ? ', ID absolvovane revize: ' . $row[1] . ' (EKurzuOsoby.Id = ' . $row[2] . ")" : '') .
                "\n";

            if ($row[3]  &&  !$row[2]) {
                // kurz je povinny, ale nebyl jeste uspesne absolvovan
                $absolvovanoVse = false;
            }
        }

        if ($absolvovanoVse) {
            $rec = new GenerovaneSkoleniDBRecord($this ->database, $kurzOsoby ->rcSkoleni, $kurzOsoby ->druhSkoleni, $kurzOsoby ->datumVznikuPovinnosti, $this);
            $rec ->logText = "Vsechny povinne kurzy byly uspesne absolvovany\n" . $logText;

            $rec ->nastavStav();
        } else {
            Log::addRecordToDBLog($this ->database, 'GenerovaneSkoleni', 'UPDATE', $this,
                "Nejsou absolvovany vsechny kurzy pripojene ke skoleni\n" .
                'RC = ' . User::getRC() . ', uzivatel ' . User::getUserName() . ' (' . User::getLoginName() . ")\n" .
                $logText
                );
        }
    }


    /**
     * Pokud je nastavena sestava pro certifikat a ma se zapsat do dokumentu, tak jej zapise
     * @param KurzOsoby $kurzOsoby
     */
    function zapisCertifikat($kurzOsoby)
    {
        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT RevizeEKurzu.SestavaProCertifikat, RevizeEKurzu.ulozitCertifikatDoDok, SestavyProIntraweb.Nazev' . // 0..2
            ' FROM RevizeEKurzu, SestavyProIntraweb' .
            ' WHERE SestavyProIntraweb.Kod = RevizeEKurzu.SestavaProCertifikat' .
            '  AND RevizeEKurzu.Id = ' . $params ->addParameter((int)$kurzOsoby ->idRevize),
            $params
        );
        $sestavaRow = $cursor ->fetch();
        $cursor ->close();

        if (!$sestavaRow  ||  !$sestavaRow[0]  ||  !$sestavaRow[1]) {
            return;
        }

        $kodSestavy = $sestavaRow[0];

        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT DokumentyOsoby.Id' .
            ' FROM DokumentyOsoby' .
            ' WHERE EKurzOsoby = ' . $params ->addParameter($kurzOsoby ->id),
            $params
        );
        $row = $cursor ->fetch();
        $cursor ->close();
        if ($row  &&  $row[0]) {
            // dokument uz je vytvoren
            return;
        }

        $report = Report::loadReport($this ->database, $kodSestavy);

        $certificate = ReportOutputObject::downloadReport($report,
            null, null, null, null, ['EKurzOsoby' => $kurzOsoby ->id]);

        if (!$certificate) {
            MessageManager::addErrorMessage('Pri generovani certifikatu doslo k chybe!');
        }

        $idDokumentu = DokumentyUtils::vytvorDokument($this ->database, $certificate, $kodSestavy . '.pdf', 0, $sestavaRow[2], $this);

        if ($idDokumentu) {
            $rec = new RonPortalDBRecord($this ->database, 'DokumentyOsoby', $this);
            $rec ->addField('Id', 'int', true, 0, $idDokumentu);
            $rec ->addField('RC', '', true, 15, $kurzOsoby ->rc);
            $rec ->addField('EKurzOsoby', 'int', true, 0, $kurzOsoby ->id);
            $rec ->addLogText('Vygenerovani certifikatu ke kurzu');
            $rec ->insertRecord();
        }
    }


    /**
     * Zkontroluje, jestli uzivatel uspesne vyplnil testy na vsech strankach, ke kterym je test pripojen.
     * @param $kurzOsoby
     * @param int|null $poradiStranky   Pokud je zadano, tak kontroluje jen stranky pred zadanou stranou
     * @param bool $jenPovinne          Pokud je true, kontroluji se jen stranky, ktere maji PovolitPokracovatBezTestu = 0
     * @return bool|int                 Pokud je vse OK vraci true. Pokud je zadano $poradiStranky, vraci id strany s prvnim nevyplnenym testem, jinak vraci false.
     */
    protected function zkontrolujAbsolvovaniTestu($kurzOsoby, $poradiStranky = null, $jenPovinne = false)
    {
        $params = $this ->database ->createParameters();

        $sql =
            'SELECT ObsahEKurzu.Id, PripraveneTemaHodnoceni.Test_Stav, ObsahEKurzu.Poradi ' .
            ' FROM EKurzOsoby INNER JOIN ObsahEKurzu ' .
            '   ON EKurzOsoby.IDRevize = ObsahEKurzu.IdRevizeKurzu ' .
            ' INNER JOIN TestEKurzu ' .
            '   ON TestEKurzu.IdRevize = ObsahEKurzu.IdRevizeKurzu ' .
            '  AND TestEKurzu.DruhHodnoceni = ObsahEKurzu.DruhHodnoceni ' .
            '  AND TestEKurzu.TemaCislo = ObsahEKurzu.TemaCislo ' .
            ' LEFT OUTER JOIN PripraveneTemaHodnoceni ' .
            '   ON PripraveneTemaHodnoceni.DruhHodnoceni = TestEKurzu.DruhHodnoceni ' .
            '  AND PripraveneTemaHodnoceni.TemaCislo = TestEKurzu.TemaCislo ' .
            '  AND PripraveneTemaHodnoceni.EKurzOsoby = EKurzOsoby.Id ' .
            '  AND PripraveneTemaHodnoceni.Test_Stav = 3 ' . // jen uspesne vyplnene
            ' WHERE EKurzOsoby.Id = ' . $params ->addParameter( $kurzOsoby ->id ) .
            '  AND ObsahEKurzu.Stav <> 2' . // smazane stranky ignoruje
            (isset($poradiStranky)
                ? '  AND ObsahEKurzu.Poradi < ' . $params ->addParameter($poradiStranky)
                : '') .
            ($jenPovinne
                ? '  AND ObsahEKurzu.PovolitPokracovatBezTestu = 0'
                : '') .
            ' ORDER BY ObsahEKurzu.Poradi';

        $cursor = $this ->database ->getSqlResult($sql, $params);

        $result = true;

        while ($result === true  &&  $row = $cursor ->fetch()) {
            // u kazdeho testu zkontroluje, jestli je uspesne vyplnen
            if ($row[1] != 3) {
                $result = $poradiStranky
                    ? (int)$row[0] // pokud je zadan
                    : false;
            }
        }

        $cursor ->close();

        return $result;
    }

    /**
     * Zkontroluje, jestli uz uzivatel dokoncil uspesne cely kurz
     * @param KurzOsoby $kurzOsoby
     * @return string
     */
    function zkontrolujAbsolvovani($kurzOsoby)
    {
        if (!$this ->zkontrolujAbsolvovaniTestu($kurzOsoby)) {
            return false;
        }

        $downloadCertificate = false;

        // OK - vse uspesne vyplneno
        $rec = new EKurzOsobyDBRecord($this ->database, $this, $kurzOsoby ->id);
        if ($rec ->loadRecord()) {
            $rec ->fStav ->setNewValue(3); // absolvovano uspesne
            $rec ->logInfoMessage = 'Uspesne absolvovani kurzu';
            $rec ->updateRecord();

            $downloadCertificate = true;
        }

        $this ->zkontrolujAbsolvovaniSkoleni($kurzOsoby);

        if ($downloadCertificate) {
            $this ->zapisCertifikat($kurzOsoby);
        }

        return true;
    }




    /**
     * HTML prezentace - zobrazi jednu stranu
     * @param RevizeEKurzu $revize
     * @param KurzOsoby $kurzOsoby
     * @param int $index
     * @return Redirect
     */
    function testOsoby($revize, $kurzOsoby, $index)
    {
        $testEKurzu = TestEKurzu::nactiTestEKurzu($this ->database,
            $revize ->getId(), $revize->getObsah()[$index]->druhHodnoceni, $revize->getObsah()[$index]->temaCislo);

        $testEKurzu ->test = Test::nactiTestOsoby($this ->database,
            $revize->getObsah()[$index]->druhHodnoceni, $revize->getObsah()[$index]->temaCislo,
            $kurzOsoby); // , 'Test_stav <= 1'  - neabsolvovano, nebo probiha

        $zobrazitInfo = false;
        $kliknutoSpustitTest = isset($_POST['spustitTest']) ? $_POST['spustitTest'] : false;
        $spustitTest = false;

        if (!$testEKurzu ->test) {
            // osoba jeste nezacala test vyplnovat nebo ma jen neuspesne pokusy
            $zobrazitInfo = $testEKurzu ->casovyLimit > 0  &&  !$kliknutoSpustitTest; // pokud je casovy limit, zobrazi jen info
            $spustitTest = !$zobrazitInfo;
        } else if ($testEKurzu ->test ->test_stav <= Test::TEST_STAV_PROBIHA) { // probiha, nebo jeste nezacal
            // probiha => bude normalne zobrazen
        } else if ($testEKurzu ->test ->test_stav == Test::TEST_STAV_ABSOLVOVANO_USPESNE) {
            // vyplneno uspesne
            $zobrazitInfo = true;
        } else {
            // vyplneno neuspesne
            $testEKurzu ->loadPocetAbsolvovanych($this ->database, $kurzOsoby);
            $spustitTest = $testEKurzu ->lzeSpustitTest()  &&  $kliknutoSpustitTest;
            $zobrazitInfo = !$spustitTest;
        }

        if ($spustitTest) {
            // uzivatel kliknul na "spustit test" nebo se ma spustit automaticky
            $testEKurzu->test = Test::vygenerujTest($this->database,
                $revize->getObsah()[$index]->druhHodnoceni, $revize->getObsah()[$index]->temaCislo,
                $kurzOsoby, $this);
        }

        Model::addViewObject( new TextViewObject('<h1>' . StringUtils::htmlTextConvert($testEKurzu ->test ? $testEKurzu ->test ->popisTematu : '') . '</h1>') );

        if ( $zobrazitInfo ) {
            // zadny test neni rozpracovan - zobrazi info s tlacitkem "spustit test"
            $zobrazitTest = true;

            if (!$testEKurzu->test) {
                $zobrazitTest = false;
                $testEKurzu->test = new Test($this->database);
                $testEKurzu->test->nactiSablonu($revize->getObsah()[$index]->druhHodnoceni, $revize->getObsah()[$index]->temaCislo);
            }

            Model::addData('testEKurzu', $testEKurzu);
            Model::addViewObject(new BaseViewObject('ronportal/elearning/view_test_info.php'));

            if (!$zobrazitTest) {
                return null;
            }
        }

        // TODO: sem dat podminku   if (!$testEKurzu ->test) return;   ? ? ?

        $testEKurzu ->test ->idKurzuOsoby = $kurzOsoby ->id;
        $testEKurzu ->test ->pageIndex = $index;

        $poLimitu = false;

        // kontrola casoveho limitu
        if ($testEKurzu ->casovyLimit > 0  &&  $testEKurzu ->test ->test_datumCasPocatku) {
            if ($testEKurzu ->test ->test_datumCasPocatku + $testEKurzu ->casovyLimit < time()) {
                $poLimitu = true;
            }
        }

        if (isset($_POST['testaction'])  &&  $_POST['testaction'] == 'uloztest') {
            if ( $testEKurzu ->test ->ulozVyplnenyTest($poLimitu ? 2 : 1) ) {
                if ($poLimitu) {
                    $testEKurzu ->zkontrolujMaximalniPocetPokusu($this ->database, $kurzOsoby);
                    // vyprsel casovy limit
                    return new Redirect(null, "?menuid=prezentace&kurzosoby=" . $kurzOsoby ->id);
                } if (isset($_POST['odeslat'])) {
                    // vyhodnoceni
                    if ($testEKurzu ->test ->prepocitejDotaznik()) {
                        $testEKurzu ->zkontrolujMaximalniPocetPokusu($this ->database, $kurzOsoby);
                        MessageManager::addInfoMessage( \Ron\ronText('Test_byl_uspesne_ulozen') );
                        return new Redirect(null, "?menuid=prezentace&kurzosoby=" . $kurzOsoby ->id);
                    } else {
                        // chyba pri prepoctu!
                        MessageManager::addErrorMessage('Pri prepoctu bodu doslo k chybe!');
                    }
                } else {
                    // jen ulozit
                    MessageManager::addInfoMessage( \Ron\ronText('Test_byl_uspesne_ulozen') );
                    return new Redirect(null, "?menuid=prezentace&kurzosoby=" . $kurzOsoby ->id);
                }
            }
        }

        $editace = $testEKurzu ->test ->test_stav < 2; // editace = u nedokonceneho true
// - nedava uplne smysl a chova se to spatne (zobrazuje tlacitko "spustit test" a navic nezvyraznuje spatne a spravne odpovedi)
//        if (!$testEKurzu ->pocetPokusu  &&  $testEKurzu ->test ->test_stav == 4) {
//            // absolvovano neuspesne, ale pocet pokusu neni omezen => edituje porad ten samy test
//            $editace = true;
//        }

        Model::addData('testViewObject', new TestViewObject($testEKurzu, $editace));
        return null;
    }


    /**
     * HTML prezentace - zobrazi jednu stranu
     * @param Kurz $kurz
     * @param RevizeEKurzu $revize
     * @param KurzOsoby $kurzOsoby
     * @return string
     */
    function prezentaceHTML($kurz, $revize, $kurzOsoby = null)
    {
        $index = $revize ->findPage(
            isset($_GET['code']) ? $_GET['code'] : null,
            (isset($_GET['pageid'])  &&  (int)$_GET['pageid']) ? (int)$_GET['pageid'] : null,
            isset($_GET['pageindex']) ? $_GET['pageindex'] : ($kurzOsoby ? (int)$kurzOsoby ->stranka : 0)     // muze byt first, last, nebo cislo
        );

        if ($kurzOsoby) {
            $absolvovanoVse = $this->zkontrolujAbsolvovaniTestu($kurzOsoby, $revize ->getObsah()[$index] ->getPoradi(), true);

            if ($absolvovanoVse !== true) {
                // nejsou absolvovany povinne testy => prepne na prvni stranky s nevyplnenym povinnym testem
                $index = $revize ->findPage(null, $absolvovanoVse);
            }
        }

        if ($revize ->getObsah()[$index] ->temaCislo) {
            // aktualni strana, obsahuje test
            TextManager::getInstance() ->loadStrings('dotaznik.ini');

            if ($kurzOsoby) {
                $result = $this ->testOsoby($revize, $kurzOsoby, $index);
                if ($result) {
                    return $result;
                }
            } else {
                // testovaci zobrazeni
                $testEKurzu = new TestEKurzu();
                $testEKurzu ->test = new Test($this->database);
                $testEKurzu ->test->nactiSablonu($revize->getObsah()[$index]->druhHodnoceni, $revize->getObsah()[$index]->temaCislo);
                Model::addData('testViewObject', new TestViewObject($testEKurzu, true)); // editace=true
            }
        }

        if ($kurzOsoby) {
            $kurzOsoby ->ulozStranku($this ->database, $index);

            if ($index == count($revize ->getObsah()) - 1 ) {
                // je na posledni strance
                $this ->zkontrolujAbsolvovani($kurzOsoby);
            }
        }


        // generovani obsahu stranky
        HtmlHeaders::getInstance() ->setTitle($kurz ->getNazev());

        $text = $this ->replaceLinks($revize, $index, $kurzOsoby);

        Model::addData('index', $index);
        Model::addData('text', $text);

        HtmlHeaders::getInstance()
            ->addBodyAttribute('class', 'prezentace' . ($revize ->getObsah()[$index]->getZobrazitHlavicku() ? '' : ' noheader'));

        return 'ronportal/elearning/view_prezentace_html.php';
    }


    /**
     * @param int $idRevize
     * @param KurzOsoby $kurzOsoby
     * @return string
     * @throws \Exception
     */
    function prezentace($idRevize, $kurzOsoby = null)
    {
        $revizeA = RevizeEKurzu::loadRevize($this ->database, null, $idRevize);
        if (!$revizeA  ||  !count($revizeA)) {
            throw new \Exception('Chyba - pozadovana revize kurzu nebyla nalezena!');
        }

        $revize = $revizeA[0];

        $kurz = new Kurz();
        $kurz->load($this ->database, $revize ->getIdKurzu());

        $revize ->loadContent($this ->database);

        Model::addData('kurz', $kurz);
        Model::addData('revize', $revize);

//        if ( $revize ->getZpusobPrezentace() ) {
//            return 'elearning/view_prezentace.php';
//        } else {
            return $this ->prezentaceHTML($kurz, $revize, $kurzOsoby);
//        }
    }


    private function getAssets() {
        return [
            'ron' => 'RON Software spol. s r.o.',
        ];
    }


    function loadAssets() {
        $result = new JsonOutputObject();

        try {
            $assets = $this ->getAssets();

            $jsonItems = [];
            foreach ($assets as $value => $tooltip) {
                $jsonItems[] = ['value' => StringUtils::jsonText($value), 'text' => StringUtils::jsonText($value)];
            }

            $result ->addData('items', $jsonItems);
        } catch (RonException $e) {
            $result ->addError($e ->getMessage(), $e ->getFieldName(), $e ->getErrorDescription());
        } catch (\Exception $e) {
            $result ->addError($e ->getMessage());
        }

        return $result;
    }


    function loadAsset($code) {
        $result = new JsonOutputObject();

        try {
            $assets = $this ->getAssets();
            if ($code) $result ->addText('text', $assets[$code]);
            // TODO: udelat databazi assetu

        } catch (RonException $e) {
            $result ->addError($e ->getMessage(), $e ->getFieldName(), $e ->getErrorDescription());
        } catch (\Exception $e) {
            $result ->addError($e ->getMessage());
        }

        return $result;
    }


    /**
     * Overi, jestli je zamestnanec skutecne prihlasen na zadane skoleni
     * @param $druhSkoleni
     * @param $datumVznikuPovinnosti
     * @throws \Exception
     */
    function overPrihlaseniNaSkoleni($druhSkoleni, $datumVznikuPovinnosti)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT RC FROM GenerovaneSkoleni ' .
            ' WHERE RC = ' . $params ->addParameter(User::getRC()) .
            '   AND DruhSkoleni = ' . $params ->addParameter($druhSkoleni) .
            '   AND DatumVznikuPovinnosti = ' . $params ->addTimestampParameter($datumVznikuPovinnosti);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        if (!$cursor ->fetch()) {
            throw new \Exception('Chyba! Na zadane skoleni nejste prihlasen!'); // nemelo by nastat (blby link)
        }
        $cursor ->close();
    }


    /**
     * @param string $druhSkoleni
     * @param int $idKurzu
     * @return int
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function overEKurzDruhuSkoleni($druhSkoleni, $idKurzu)
    {
        // overi, jestli je kurz pripojen ke skoleni, je aktivni, ma platnou revizi
        // zaroven nacte posledni platnou revizi
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT EKurz.Stav, EKurz.PlatnostOd, EKurz.PlatnostDo, RevizeEKurzu.Id' .
            '  FROM EKurz INNER JOIN EKurzDruhuSkoleni ' .
            '    ON EKurzDruhuSkoleni.IdKurzu = EKurz.Id ' .
            '  LEFT OUTER JOIN RevizeEKurzu ' .
            '    ON RevizeEKurzu.IdKurzu = EKurz.Id ' .
            '   AND RevizeEKurzu.Stav = 1 ' . // jen aktivni
//            '   AND RevizeEKurzu.PlatnostOd <= ' . $params ->addParameter($this ->database ->dateToParam(time())) .
//            '   AND (RevizeEKurzu.PlatnostDo >= ' . $params ->addParameter($this ->database ->dateToParam(time())) . '  OR  RevizeEKurzu.PlatnostDo IS NULL) ' .
            '   AND RevizeEKurzu.PlatnostOd <= ' . $params ->addTimestampParameter(time()) .
            '   AND (RevizeEKurzu.PlatnostDo >= ' . $params ->addTimestampParameter(time()) . '  OR  RevizeEKurzu.PlatnostDo IS NULL) ' .
            ' WHERE EKurzDruhuSkoleni.KodDruhuSkoleni = ' . $params ->addParameter($druhSkoleni) .
            '   AND EKurz.Id = ' . $params ->addParameter($idKurzu) .
            ' ORDER BY RevizeEKurzu.PlatnostOd DESC'; // prvni zaznam bude nejnovejsi revize

        $cursor = $this ->database ->getSqlResult($sql, $params);
        if (!$row = $cursor ->fetch()) {
            throw new \Exception('Chyba! Kurz neexistuje nebo neni pripojen ke skoleni!');
        }
        $cursor ->close();

        // zkontroluje EKurz.Stav (musi byt 1)
        if ($row[0] == 0) {
            throw new \Exception( \Ron\ronText('Kurz_se_teprve_pripravuje'));
        } else if ($row[0] != 1) {
            throw new \Exception( \Ron\ronText('Kurz_uz_neni_aktivni') );
        }

        // zkontroluje platnost do
        if ($row[2]  &&  $this ->database ->stringToTimestamp($row[2]) < DateUtils::startOfTheDay(time()) ) {
            throw new \Exception( \Ron\ronText('Kurz_uz_neni_platny') );
        }

        // zkontroluje platnost od
        if ($row[1]  &&  $this ->database ->stringToTimestamp($row[1]) > time() ) {
            throw new \Exception( \Ron\ronText('Kurz_jeste_neni_platny') );
        }

        // overi platnost revize
        if (!$row[3]) {
            throw new \Exception( \Ron\ronText('Kurz_nema_platnou_revizi') );
        }

        return $row[3];
    }


    /**
     * Zjisti, jestli jeste nezahajil tento kurz
     * @return Redirect
     * @throws \Exception
     */
    function zahajitKurz()
    {
        $druhSkoleni = $_GET['druhskoleni'];
        $datumVznikuPovinnosti = (int)$_GET['datumvznikupovinnosti']; // DateUtils::dateStringToTimestamp($_GET['datumvznikupovinnosti']); - timestamp, protoze je tam i cas
        $idKurzu = (int)$_GET['zahajitkurz'];

        $this ->overPrihlaseniNaSkoleni($druhSkoleni, $datumVznikuPovinnosti);

        $idPosledniRevize = $this ->overEKurzDruhuSkoleni($druhSkoleni, $idKurzu);

        // zjisti, jestli jeste nezacal (= neexistuje prislusny zaznam EKurzOsoby)
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT EKurzOsoby.Id, EKurzOsoby.Stav ' .
            '  FROM EkurzOsoby, RevizeEKurzu ' .
            ' WHERE EKurzOsoby.RC = ' . $params ->addParameter(User::getRC()) .
            '   AND EKurzOsoby.RC_Skoleni = EKurzOsoby.RC ' .
            '   AND EKurzOsoby.DruhSkoleni = ' . $params ->addParameter($druhSkoleni) .
            '   AND EKurzOsoby.DatumVznikuPovinnosti = ' . $params ->addTimestampParameter($datumVznikuPovinnosti) . // vcetne casu!
            '   AND (EKurzOsoby.Stav = 0  OR  EKurzOsoby.Stav = 1)' . // neabsolvovano nebo probiha  TODO: co s ukoncenyma?
            '   AND EKurzOsoby.IdRevize = RevizeEKurzu.Id' .
            '   AND RevizeEKurzu.IdKurzu = ' . $params ->addParameter($idKurzu);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            // uzivatel uz kurz zacal => presmeruje na nej
            return new Redirect(null, "?menuid=prezentace&kurzosoby=" . $row[0]);
        }

        $rec = new EKurzOsobyDBRecord($this ->database, $this);
        $rec ->logInfoMessage = 'Zahajeni kurzu';

        $rec ->fIdRevize ->setNewValue($idPosledniRevize);
        $rec ->fDruhSkoleni ->setNewValue($druhSkoleni);
        $rec ->fDatumVznikuPovinnosti ->setNewValue($datumVznikuPovinnosti);

        $kurzOsoby = SQLUtils::newSerialNumber($this ->database, 'EKurzOsoby', 'Id');
        $rec ->fId ->setNewValue($kurzOsoby);

        $rec ->insertRecord();

        return new Redirect(null, "?menuid=prezentace&kurzosoby=" . $kurzOsoby);
    }


    function kurzOsoby()
    {
        $kurzOsoby = KurzOsoby::load($this ->database, $_GET['kurzosoby']);

        if ($kurzOsoby ->rc != User::getRC()) {
            // patri jinemu uzivateli!
            throw new NeniPravoException();
        }

        // existuje EKurzOsoby => ma pravo
        return $this ->prezentace($kurzOsoby ->idRevize, $kurzOsoby);
    }


    public function certifikat()
    {
        Prava::nastavObdobiProPrava(time());

        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT EKurzOsoby.RC, RevizeEKurzu.SestavaProCertifikat, DokumentyOsoby.Id, EKurzOsoby.Id' .
            ' FROM EKurzOsoby INNER JOIN RevizeEKurzu' .
            '   ON RevizeEKurzu.Id = EKurzOsoby.IdRevize' .
            '  LEFT OUTER JOIN DokumentyOsoby' .
            '   ON DokumentyOsoby.EKurzOsoby = EKurzOsoby.Id' .
            ' WHERE EKurzOsoby.Id = ' . $params ->addParameter((int)$_GET['id']),
            $params
        );
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new \Exception('Chyba - pozadovany zaznam nebyl nalezen!');
        }

        if (!Prava::vratPrava('prohl_osobni') ->lzeZobrazitRC($row[0])) {
            if (!Prava::vratPrava('skoleni_prohlizeni') ->lzeZobrazitRC($row[0])) {
                throw new NeniPravoException();
            }
        }

        if ($row[2]) {
            // certifikat je uz ulozen v dokumentech osoby
            return new DokumentOutputObject($this ->database, $row[2]);
        }

        $result = new ReportOutputObject();
        $result ->loadReport($this ->database, $row[1]);
        $result ->integerParams = ['EKurzOsoby' => $row[3]];

        return $result;
    }

    /**
     * @param string $action
     * @return string|ViewObject|OutputObject
     * @throws \Exception
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::ELEARNING_POVOLEN)) {
            throw new \Exception('ELearning neni povolen!');
        }

        TextManager::getInstance() ->loadStrings('elearning.ini');

        if ( isset($_GET['asset']) ) {
            return $this ->loadAsset($_GET['asset']);
        }

        if ( isset($_GET['zahajitkurz']) ) {
            return $this ->zahajitKurz();
        }

        if ( isset($_GET['kurzosoby']) ) {
            return $this ->kurzOsoby();
        }

        if (isset($_GET['action'])) {
            switch ($_GET['action']) {
                case 'certifikat' :
                    return $this->certifikat();
            }
        }

        if (!Prava::vratHodnotuPrava('editacekurzu')) {
            throw new NeniPravoException();
        }

        if (isset($_GET['action'])) {
            switch ($_GET['action']) {
                case 'assets' :
                    return $this->loadAssets();
            }
        }

        TextManager::getInstance() ->loadStrings('elearning.ini');

        return $this ->prezentace((int)$_GET['revize']);
    }

}
