<?php
/**
 * stanislav.elbl, 08.07.2015
 */

namespace RonPortal\ELearning;

use Ron\DB\Database;


class RevizeEKurzu
{
    /** @var int */
    private $id;
    /** @var int */
    private $idKurzu;

    /** @var int */
    private $stav;

    /** @var string */
    private $popis;

    /** @var string */
    private $autor;

    /** @var int */
    private $platnostOd;
    /** @var int */
    private $platnostDo;

    private $zpusobPrezentace;
    private $hlavicka;
    private $paticka;


    /** @var PrilohaKurzu[] */
    private $prilohy;

    /** @var ObsahKurzu[] */
    private $obsah;

    /** @var  TestEKurzu[] */
    private $testy;


    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return int
     */
    public function getIdKurzu()
    {
        return $this->idKurzu;
    }

    /**
     * @return int
     */
    public function getStav()
    {
        return $this->stav;
    }

    /**
     * @return string
     */
    public function getPopis()
    {
        return $this->popis;
    }

    /**
     * @return string
     */
    public function getAutor()
    {
        return $this->autor;
    }

    /**
     * @return int
     */
    public function getPlatnostOd()
    {
        return $this->platnostOd;
    }

    /**
     * @return int
     */
    public function getPlatnostDo()
    {
        return $this->platnostDo;
    }

    /**
     * @return mixed
     */
    public function getZpusobPrezentace()
    {
        return $this->zpusobPrezentace;
    }

    /**
     * @return mixed
     */
    public function getHlavicka()
    {
        return $this->hlavicka;
    }

    /**
     * @return mixed
     */
    public function getPaticka()
    {
        return $this->paticka;
    }


    /**
     * @return ObsahKurzu[]
     */
    public function getObsah()
    {
        return $this->obsah;
    }

    /**
     * @return PrilohaKurzu[]
     */
    public function getPrilohy()
    {
        return $this->prilohy;
    }

    /**
     * @return TestEKurzu[]
     */
    public function getTesty()
    {
        return $this->testy;
    }



    /**
     * @param Database $database
     * @param int $idKurzu
     * @param int $idRevize   Pokud je null, nacte vsechny revize, pokud je 0, nacte jen aktivni, jinak zadanou
     * @return RevizeEKurzu[]|bool
     */
    public static function loadRevize($database, $idKurzu = null, $idRevize = null)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT Id, Popis, Stav, PlatnostOd, PlatnostDo, Autor, ' . // 0..5
            $database ->castWideMemo('Hlavicka') . ', ' . $database ->castWideMemo('Paticka') . ', ZpusobPrezentace, IdKurzu ' . // 6..9
            ' FROM RevizeEKurzu';

        if ( isset($idRevize) ) {
            if ($idRevize) {
                $sql .= ' WHERE Id = ' . $params ->addParameter((int)$idRevize);
            } else {
                // posledni aktivni
                $sql .=
                    ' WHERE IdKurzu = ' . $params ->addParameter((int)$idKurzu) .
                    ' AND Stav = 1';
            }
        } else {
            $sql .= ' WHERE IdKurzu = ' . $params ->addParameter((int)$idKurzu);
        }

        $sql .=
            ' ORDER BY PlatnostOd DESC';

        $cursor = $database ->getSqlResult($sql, $params);

        $result = [];
        while ($rowData = $cursor ->fetch()) {
            $revize = new static();

            $revize ->id = $rowData[0];
            $revize ->idKurzu = $rowData[9];
            $revize ->stav = $rowData[2];
            $revize ->platnostOd = $database->stringToTimestamp($rowData[3]);
            $revize ->platnostDo = $rowData[4] ? $database->stringToTimestamp($rowData[4]) : null;
            $revize ->popis = $rowData[1];
            $revize ->autor = $rowData[5];

            $revize ->hlavicka = $rowData[6];
            $revize ->paticka = $rowData[7];
            $revize ->zpusobPrezentace = (int)$rowData[8];

            $result[] = $revize;
        }

        $cursor ->close();

        return $result;
    }


    function loadContent($database)
    {
        $this->obsah = ObsahKurzu::loadObsah($database, $this->getId());

        $this->prilohy = PrilohaKurzu::loadPrilohy($database, $this->getId());
        $this->testy = TestEKurzu::loadTesty($database, $this->getId());
    }


    /**
     * Vraci stranku se zadanym kodem / id / indexem
     * @param string $kod
     * @param int $id
     * @param int $index
     * @return int
     * @throws \Exception
     */
    function findPage($kod, $id = null, $index = null)
    {
        if ($kod) {
            foreach ($this->getObsah() as $index => $strana) {
                if ($strana->getKodStrany() == $kod) return $index;
            }

            throw new \Exception('Chyba - stranka se zadanym kodem neexistuje!');
        } else if ($id) {
            foreach ($this->getObsah() as $index => $strana) {
                if ($strana->getId() == $id) return $index;
            }

            throw new \Exception('Chyba - stranka se zadanym Id neexistuje!');
        } else {
            if ($index === 'first') {
                $index = 0;
            } else if ($index === 'last') {
                $index = sizeof($this->getObsah()) - 1;
            }

            if ($index >= 0 && $index < sizeof($this->getObsah())) {
                return $index;
            }

            throw new \Exception('Chyba - stranka se zadanym indexem neexistuje!');
        }
    }


    function getJson()
    {
        $obj = [
            'id' => $this->id,  // id kurzu
            'revize' => $this->getId(),
            'obsahKurzu' => [],
            //'prilohy' => [],
            'testy' => []
        ];

        foreach ($this->getObsah() as $strana) {
            $obj['obsahKurzu'][] = $strana->getJson();
        }

//        foreach ($this->getPrilohy() as $priloha) {
//            $obj['prilohy'][] = $priloha->getJson();
//        }

        foreach ($this->getTesty() as $test) {
            $obj['testy'][] = $test->getJson();
        }


        return $obj;
    }
}
