<?php

namespace RonPortal\Hodnoceni;

use Exception;
use Ron\Controls\Browser;
use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\Controls\RozdeleniOsobyBrowserField;
use Ron\StringUtils;
use RonPortal\RozdeleniUtils;
use function Ron\ronText;


class GenerovaniField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return "<span class='generovani clickLink js' data-typdialogu='generovani' data-id='{$rowData['col-idpp']}'>" .  ronText('Generovat') . "</span>";
    }
}


class ExpandField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        /**
         * Zmeni class u elementu this, nebo u elementu se selektorem zadanym v data-selector (pokud je zadan)
         * - odstrani tridu data-remove-class
         * - pridat tridu data-add-class
         * - prepne tridu data-toggle-class
         */
        if ($rowData['col-pocetvysledku']) {
            return "<div class='hodnoceniToggle expand' data-expand-selector='.vysledkyhodnoceni_{$rowNumber}' data-hide-selector='.vysledkyhodnoceni_{$rowNumber}, .hodnoceni_radek_{$rowNumber}' data-toggle-class='d-none'>&nbsp;</div>";
        }

        return '';
    }
}

class HodnoceniOsobBrowser extends RonPortalBrowser
{
    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database, 'hodnoceniosob');

        $this->title = ronText('Hodnoceni_osob');

        $this->code = 'browserhodnoceniosob';

        $this->sqlFromPart = "Osoba";

        $this->defaultSort = 'col-prijmeni';
        //$this->primaryField = 0;

        // rozdeleni
        $fRozdeleni = new RozdeleniOsobyBrowserField($database, RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI));
        $fRozdeleni ->nastavIndexPole('col-rc'); // index pole HodnocenyRC
        $this ->addField($fRozdeleni)
            ->setCode('col-rozdeleni');

        // osoba
        $this->addField()
            ->setSqlFieldName('Osoba.Jmeno') ->setHeaderText('Jmeno') ->setOrdering(true)
            ->setCode('col-jmeno')
            ->setSortSQL('Osoba.Jmeno, Osoba.Prijmeni') ->setSortSQLDesc('Osoba.Jmeno DESC, Osoba.Prijmeni DESC');

        $this->addField()
            ->setSqlFieldName('Osoba.Prijmeni') ->setHeaderText('Prijmeni') ->setOrdering(true)
            ->setCode('col-prijmeni')
            ->setSortSQL('Osoba.Prijmeni, Osoba.Jmeno') ->setSortSQLDesc('Osoba.Prijmeni DESC, Osoba.Jmeno DESC');

        $this->addField(new GenerovaniField($this->database))->setCode('col-generovani');

        $this->addField(new ExpandField())->setCode('col-zobrazenidetailu');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setCode('col-rc')
            ->setSqlFieldName('Osoba.RC');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setCode('col-pocetvysledku')
            ->setSqlFieldName('(SELECT COUNT(ID) FROM VysledekHodnoceni WHERE HodnocenyRC = Osoba.RC)');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setCode('col-idpp')
            ->setSqlFieldName('(SELECT MAX(ID) FROM PracovniPomer WHERE PracovniPomer.RC = Osoba.RC)');
    }


    function setFilterForm()
    {
        $this ->filterForm = new HodnoceniFilterForm($this ->database, 'hodnoceniosob', false);
        $this ->filterForm ->ucelHodnoceniFilter
            ->setSqlFilter(null);   // sestavi se az v getSQLFilter
        //$this ->filterForm ->loadData();
    }


    function renderFilter()
    {
        $this ->filterForm ->render();

        echo "<div class='padding-top-2'><a class='hodnoceniToggle' data-selector='.hodnoceni, .vysledekhodnoceni' data-expanded-selector='.hodnoceniToggle' data-toggle-class='d-none' href='#' >" . ronText('Zobrazit_vse') . "</a></div>";
    }


    function getSQLFilter()
    {
        $datumOd = $this ->filterForm ->getFilterValue('datumod') ?? DateUtils::today();
        $datumDo = $this ->filterForm ->getFilterValue('datumod', 'datumDo') ?? DateUtils::today();

        $prava = Prava::vratPrava('hodnoceni');

        $params = $this->database->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        $result .=
            ($result ? ' AND ' : '') .
            'EXISTS(SELECT ID FROM PracovniPomer WHERE PracovniPomer.RC = Osoba.RC AND PracovniPomer.ZpracovatPersonalistiku = 1 ' .
            '        AND EXISTS(SELECT RC FROM EvidencniStav JOIN TypEvidencnihoStavu ON (TypEvidencnihoStavu.Kod = EvidencniStav.KodEvidencnihoStavu) ' .
            '                   WHERE (PracovniPomer.RC = EvidencniStav.RC) AND (PracovniPomer.OsobniCislo = EvidencniStav.OsobniCislo) AND (PracovniPomer.DatumNastupu = EvidencniStav.DatumNastupu)' .
            '                     AND (PracovniPomer.DatumNastupu <= ' . $params->addDateParameter($datumDo) . ')' .
            '                     AND ((PracovniPomer.DatumUkonceni >= ' . $params->addDateParameter($datumDo) . ') OR (PracovniPomer.DatumUkonceni IS NULL))' .
            '                     AND (EvidencniStav.PlatnostOd <= ' . $params->addDateParameter($datumDo) . ')' .
            '                     AND ((EvidencniStav.PlatnostDo >= ' . $params->addDateParameter($datumDo) . ') OR (EvidencniStav.PlatnostDo IS NULL))' .
            '                     AND (TypEvidencnihoStavu.ZahrnoutDoAgend = 1)  ' .
            '                   )' .
            '        AND ' . $prava->sqlFiltr('PracovniPomer', $datumOd, $datumDo, $params) .
            ')';


        if ($this->filterForm->getFilterValue('osoba')) {
            $result .=
                ' AND Osoba.RC = ' . $params->addParameter($this->filterForm->getFilterValue('osoba', 'rc'));
        }

        // "Jen rozdeleni"
        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            if ($rozdeleniFilter ->getValue() !== null) {
                $result .=
                    ($result ? ' AND ' : '') .
                    $rozdeleniFilter ->sestavSQLFiltr($this ->database, 'Osoba', $params, true);
//                    RozdeleniUtils::sqlFilterRozdeleniPP($this ->database,
//                        'Osoba',
//                        $this ->filterForm ->kodTypuRozdeleni, $this ->filterForm ->kodRozdeleni,
//                        $datumOd, $datumDo,
//                        $this ->filterForm ->vcetnePodrizenychRozdeleni, $params, true);
            }
        }
//        if (isset($this ->filterForm ->kodRozdeleni)) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                RozdeleniUtils::sqlFilterRozdeleniPP($this ->database,
//                    'Osoba',
//                    $this ->filterForm ->kodTypuRozdeleni, $this ->filterForm ->kodRozdeleni,
//                    $datumOd, $datumDo,
//                    $this ->filterForm ->vcetnePodrizenychRozdeleni, $params, true);
//        }

        // "Jen ucel hodnoceni"
        if ($this ->filterForm ->getFilterValue('ucelhodnoceni') !== null) {
            $result .=
                ' AND (EXISTS(SELECT ID FROM VysledekHodnoceni' .
                ' WHERE VysledekHodnoceni.HodnocenyRC = Osoba.RC' .
                ' AND VysledekHodnoceni.KodUcelu = ' . $params->addParameter($this ->filterForm ->getFilterValue('ucelhodnoceni')) .
                ')) ';
        }

        return new SqlStatement($result, $params);
    }


    /**
     * @param string $rc
     * @param $rowNumber
     * @param $className
     * @throws Exception
     */
    private function zobrazVysledky($rc, $rowNumber, $className)
    {
        $datumPocatku = isset($_GET['datumpocatku'])
            ? DateUtils::dateStringToTimestamp($_GET['datumpocatku'])
            : null;

        $ucelHodnoceni = isset($_GET['ucel']) ? $_GET['ucel'] : null;

        $params = $this->database->createParameters();

        $sql =
            'SELECT ID, PocatekObdobi, KonecObdobi, UcelHodnoceni.Popis, UcelHodnoceni.Kod' .
            ' FROM VysledekHodnoceni JOIN UcelHodnoceni ON (VysledekHodnoceni.KodUcelu = UcelHodnoceni.Kod)' .
            ' WHERE HodnocenyRC = ' . $params->addParameter($rc);

        if ($this ->filterForm ->getFilterValue('ucelhodnoceni') !== null) {
            $sql .=
                ' AND VysledekHodnoceni.KodUcelu = ' . $params->addParameter($this ->filterForm ->getFilterValue('ucelhodnoceni'));
        }

        if ($this ->filterForm ->getFilterValue('datumod')) {
            $sql .=
                ' AND VysledekHodnoceni.KonecObdobi >= ' . $params->addDateParameter($this ->filterForm ->getFilterValue('datumod'));
        }

        if ($this ->filterForm ->getFilterValue('datumod', 'datumDo')) {
            $sql .=
                ' AND VysledekHodnoceni.PocatekObdobi <= ' . $params->addDateParameter($this ->filterForm ->getFilterValue('datumod', 'datumDo'));
        }

        //if (HodnoceniFilterForm::getJenTemaHodnoceni()) {
        if ($this ->filterForm ->getFilterValue('temahodnoceni') !== null) {
            $sql .=
                ' AND EXISTS(SELECT PripraveneTemaHodnoceni.ID ' .
                ' FROM HodnoceniOsoby INNER JOIN PripraveneTemaHodnoceni ON HodnoceniOsoby.ID = PripraveneTemaHodnoceni.ID ' .
                '      INNER JOIN TemaHodnoceni ON TemaHodnoceni.DruhHodnoceni = PripraveneTemaHodnoceni.DruhHodnoceni AND TemaHodnoceni.TemaCislo = PripraveneTemaHodnoceni.TemaCislo ' .
                ' WHERE HodnoceniOsoby.IdVysledku = VysledekHodnoceni.ID ' .
                '   AND TemaHodnoceni.TemaCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('temahodnoceni')) .
                ')';
        }

        $sql .=
            " ORDER BY ID";

        $cursor = $this->database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()) {
            $hiddenStr = 'd-none';
            $expandedStr = '';
            if ((!is_null($datumPocatku) && ($datumPocatku == $this->database->stringToTimestamp($row[1]))) && (!is_null($ucelHodnoceni) && ($ucelHodnoceni == $row[4]))) {
                $hiddenStr = '';
                $expandedStr = 'expanded';
            }

            $colspan = $this ->columnCount - 4;
            echo
                "<tr class='$className vysledekhodnoceni vysledkyhodnoceni_{$rowNumber} {$hiddenStr}'>" .
                "<td colspan='2' class='padding-left-3'>" . StringUtils::htmlTextConvert($row[3]) . "</td>" .
                "<td class=''>" . DBUtils::getFormatedValue($this->database,'date', $row[1]) . "</td>" .
                "<td class=''>" . DBUtils::getFormatedValue($this->database,'date', $row[2]) . "</td>" .
                "<td colspan='$colspan' class='hodnoceniToggle expand {$expandedStr}' data-selector='.hodnoceni_{$row[0]}' data-toggle-class='d-none'>&nbsp;</td>" .
                "</tr>";

            $this->zobrazSlozky($row[0], $rowNumber, $className, $hiddenStr);
        }

    }

    /**
     * zobrazi (pod)radky se slozkama hodnoceni
     * @param $idVysledku
     * @param $rowNumber
     * @param $className
     * @param $hiddenStr
     * @throws DBException
     */
    function zobrazSlozky($idVysledku, $rowNumber, $className, $hiddenStr)
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT PripraveneTemaHodnoceni.ID, PripraveneTemaHodnoceni.Stav, " . // 0,1
            "   PripraveneTemaHodnoceni.DatumHodnoceni, PripraveneTemaHodnoceni.Hodnotici, " . //2,3
            "   TemaHodnoceni.PopisTematu, HodnoceniOsoby.Znamka, HodnoceniOsoby.Koeficient, " . //4 - 6
            "   PripraveneTemaHodnoceni.HodnotilRC, Osoba.Prijmeni, Osoba.Jmeno, PoradiPredavani.Vlastni" . //7 - 10

            " FROM HodnoceniOsoby " .
            " INNER JOIN PripraveneTemaHodnoceni ON (HodnoceniOsoby.ID = PripraveneTemaHodnoceni.ID) " .
            " INNER JOIN TemaHodnoceni ON (TemaHodnoceni.DruhHodnoceni = PripraveneTemaHodnoceni.DruhHodnoceni AND TemaHodnoceni.TemaCislo = PripraveneTemaHodnoceni.TemaCislo) " .
            " INNER JOIN PoradiPredavani ON ((PoradiPredavani.KodProcesu = HodnoceniOsoby.KodProcesu) AND (PoradiPredavani.PocatecniUroven = HodnoceniOsoby.PocatecniUroven) AND (PoradiPredavani.Poradi = HodnoceniOsoby.PoradiPredavani))" .
            " LEFT OUTER JOIN Osoba ON (Osoba.RC = PripraveneTemaHodnoceni.HodnotilRC)" .
            " WHERE HodnoceniOsoby.IdVysledku = " . $params->addParameter($idVysledku);

        if ($this ->filterForm ->getFilterValue('temahodnoceni') !== null) {
            $sql .=
                ' AND TemaHodnoceni.TemaCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('temahodnoceni'));
        }

        $cursor = $this ->database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $colspan = $this ->columnCount - 4;

            echo
                "<tr class='$className hodnoceni hodnoceni_radek_{$rowNumber} hodnoceni_{$idVysledku} {$hiddenStr}'>" .

                // popis tematu
                "<td colspan='2' class='poznamka hodnocenipodrobnosti padding-left-4'>" .
                "<a href='?menuid=hodnoceni&backaction=hodnoceniosob&record={$row[0]}'>" . StringUtils::htmlTextConvert($row[4]) . "</a>" .
                "</td>" .

                // hodnotitel nebo "Nastav hodnotitele"
                "<td class='poznamka hodnocenipodrobnosti'>" .
                ($row[7]    //pokud je prirazeny hodnotitel, zobrazi jmeno a prijmeni pokud zatim neni a v aktualni urovni neni predavani nastaveno jako vlastni zobrazi odkaz na prirazeni hodnotitele
                    ? StringUtils::htmlTextConvert("$row[9] $row[8]")
                    : ($row[10] ? "" : "<span class='nastavhodnotitele js clickLink' data-typdialogu='nastavhodnotitele' data-id='{$row[0]}'>" . ronText('Nastav_hodnotitele') . "</span>")) . // hodnotici
                "</td>" .

                // datum hodnoceni
                "<td class='poznamka hodnocenipodrobnosti'>" . DBUtils::getFormatedValue($this ->database, "finedatetime", $row[2]) . "</td>" .

                // stav
                "<td colspan='$colspan' class='poznamka hodnocenipodrobnosti'>" .
                ronText('Stav_hodnoceni_' . $row[1]) .
                "</td>" .

                "</tr>\n";
        }
    }


    /**
     * @param array $values
     * @param string $className
     * @param int $rowNumber
     * @param bool $edit
     * @throws Exception
     */
    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        $this->zobrazVysledky($values['col-rc'], $rowNumber, $className);

        $this->renderSeparator();
    }



}
