<?php

namespace RonPortal\Hodnoceni;

use Exception;
use Ron\Controls\Browser;
use Ron\Controls\BrowserField;
use Ron\Controls\LinksList;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\PredavaciProcesyUtils;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\Controls\RozdeleniOsobyBrowserField;
use RonPortal\RozdeleniUtils;
use RonPortal\User;
use Ron\StringUtils;
use function Ron\ronText;


/**
 * Sloupec s vysledkem hodnoceni, ktery je videt jen u ukoncenych hodnoceni
 * Class VysledekField
 * @package Ron\Personalistika
 */
class VysledekField extends BrowserField
{
    private $zobrazitVzdy;

    /**
     * @param bool $zobrazitVzdy
     */
    function __construct($zobrazitVzdy)
    {
        parent::__construct();

        // nastaveni, jestli se zobrazi vysledek i u nedokoncenych hodnoceni
        $this->zobrazitVzdy = $zobrazitVzdy;

        $this ->format = 'number_2';
        $this ->align = 'right';
        $this ->ordering = true;

        $this ->header = ronText('Vysledek_hodnoceni');

        $this ->sqlFieldIndex2 = 'col-upresnenivysledku';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$this->zobrazitVzdy  &&  !$rowData['col-stavhodnoceni']) {
            // uzivatel nesmi videt neukoncene a toto neni ukoncene
            return '-';
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}


/**
 * Seznam s vysledky hodnoceni osob (tabulka VysledekHodnoceni)
 */
class VysledekHodnoceniBrowser extends RonPortalBrowser
{
    var $zobrazitPrubezneVysledky;

    /** @var bool   Nastavi se na true, pokud ma prava na podrizene (=je v predavacim procesu) */
    var $pravaNaPodrizene;

//    /** @var bool|int|mixed     Urcuje, jestli se ma zobrazit seznam vlastnich hodnoceni, nebo hodnoceni podrizenych */
//    public $zobrazitPodrizene = false;

//    /** @var  HodnoceniFilterForm */
//    var $filterForm;

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database, 'vysledkyhodnoceni');

        $this->title = ronText('Vysledky_hodnoceni');
        $this->code = 'browservysledekhodnoceni';

        $this->sqlFromPart =
            '( (VysledekHodnoceni INNER JOIN Osoba ' .
            '   ON VysledekHodnoceni.HodnocenyRC = Osoba.RC ' .
            '  )' .
            '  INNER JOIN UcelHodnoceni ' .
            '  ON VysledekHodnoceni.KodUcelu = UcelHodnoceni.Kod ' .
            ') ';

        $this ->sqlSelectPart =
            'VysledekHodnoceni.ID'; // 0

        $this->defaultSort = 'col-prijmeni';
        $this->primaryField = 0;

//        if (isset($_GET['filtrhodnoceni'])) {
//            $this ->zobrazitPodrizene = (int)($_GET['filtrhodnoceni']);
//            $_SESSION['filtrvysledekhodnoceni'] = $this ->zobrazitPodrizene;
//        } else {
//            $this ->zobrazitPodrizene = $_SESSION['filtrvysledekhodnoceni'] ?? false;
//        }

        //$this ->browserUrl =

        // RC, OsobniCislo, DatumNastupu
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('VysledekHodnoceni.HodnocenyRC')
            ->setCode('col-rc');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('VysledekHodnoceni.HodnocenyOsobniCislo')
            ->setCode('col-osobnicislo');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('VysledekHodnoceni.HodnocenyDatumNastupu')
            ->setCode('col-datumnastupu');

        // rozdeleni
        $fRozdeleni = new RozdeleniOsobyBrowserField($database, RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI));
        $fRozdeleni ->nastavIndexPole('col-rc', 'col-osobnicislo', 'col-datumnastupu'); // index pole HodnocenyRC
        $fRozdeleni ->indexDatumOd = 'col-pocatekobdobi';
        $fRozdeleni ->indexDatumDo = 'col-konecobdobi';
        $fRozdeleni->setCode('col-rozdeleni');
        $this ->addField($fRozdeleni);


        // osoba
        $this->addField()
            ->setSqlFieldName('Osoba.Jmeno')
            ->setCode('col-jmeno')
            ->setHeaderText('Jmeno') ->setOrdering(true);
        $this->addField()
            ->setSqlFieldName('Osoba.Prijmeni')
            ->setCode('col-prijmeni')
            ->setHeaderText('Prijmeni')->setOrdering(true);

        // popis
        $this->addField()
            ->setSqlFieldName('UcelHodnoceni.Popis')
            ->setCode('col-popis')
            ->setHeaderText('Popis') ->setOrdering(true);

        // Obdobi hodnoceni (od-do)
        $this->addField()
            ->setSqlFieldName('VysledekHodnoceni.PocatekObdobi')->setHeaderText('Datum_od')->setFormat('date')
            ->setCode('col-pocatekobdobi')
            ->setOrdering(true);
        $this->addField()
            ->setSqlFieldName('VysledekHodnoceni.KonecObdobi') ->setHeaderText('Datum_do') ->setFormat('date')
            ->setCode('col-konecobdobi')
            ->setOrdering(true);

        // stav
        $this->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('VysledekHodnoceni.Stav') ->setHeaderText('Stav_hodnoceni') ->setOrdering(true)
            ->setCode('col-stavhodnoceni')
            ->setOptions([
                0 => ronText('Stav_vysledku_Neukonceno'),
                1 => ronText('Stav_vysledku_Ukonceno'),
                -1 => ronText('Stav_vysledku_Vyrazeno')
            ]);

        // vysledek
        // nejdrive urci, jestli se zobrazi vysledek i u nedokoncenych hodnoceni
        // - bere se (alespon prozatim) stejne nastaveni jako pro povoleni hromadneho schvalovani
        //   tj. kdyz muze hromadne schvalovat, tak zaroven vidi prubezne vysledky hodnoceni
        $seznamUrovni = PredavaciProcesyUtils::seznamUrovniPredavaniUzivatele($this ->database);
        $this->zobrazitPrubezneVysledky = PredavaciProcesyUtils::muzeHromadneSchvalovatHodnoceni($seznamUrovni);

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('VysledekHodnoceni.UpresneniVysledku')
            ->setCode('col-upresnenivysledku');

        $this->addField(new VysledekField($this->zobrazitPrubezneVysledky))
            ->setSqlFieldName('VysledekHodnoceni.CelkovyVysledek')
            ->setCode('col-vysledek')
            ->setVisible(RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_VYSLEDKY_HODNOCENI) & 1);   // znamka nebo oboji

        $this->addField(new VysledekField($this->zobrazitPrubezneVysledky))
            ->setSqlFieldName('VysledekHodnoceni.CelkovyKoeficient')
            ->setCode('col-vysledek')
            ->setVisible(RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_VYSLEDKY_HODNOCENI) & 2);   // koeficient nebo oboji

        $this ->pravaNaPodrizene = PredavaciProcesyUtils::jeVNejakemProcesu($this ->database);
    }


    function setFilterForm()
    {
        $this ->filterForm = new HodnoceniFilterForm($this ->database, 'vysledkyhodnoceni', true);
        $this ->filterForm ->pridejFiltrZnamky();
        //$this ->filterForm ->zobrazitZnamky = true;

        $this ->filterForm ->addFilter(
            (new FilterFormFilter('filtrhodnoceni', FilterFormFilter::TYPE_HIDDEN))
            ->setDefaultValue(0)
        );
    }


    /**
     * @param string $sqlFilter
     * @throws Exception
     */
    function load($sqlFilter = '')
    {
//        if ($this ->pravaNaPodrizene) {
//            if (isset($_GET['filtrhodnoceni'])) {
//                $_SESSION['filtrvysledekhodnoceni'] = $_GET['filtrhodnoceni'];
//            }
//        } else {
//            // neni v predavacim procesu - uvidi jen "moje hodnoceni"
//            $_SESSION['filtrvysledekhodnoceni'] = 0;
//        }
//
        parent::load($sqlFilter);

        $this ->browserUrl =
            '?menuid=vysledkyhodnoceni&filtrhodnoceni=' . ($this ->filterForm ->getFilterValue('filtrhodnoceni') ? 1 : 0);
    }


    function renderFilter()
    {
        if ($this ->pravaNaPodrizene) {
            LinksList::renderLinks(
                [0 => ronText('Moje_hodnoceni'), 1 => ronText('Hodnoceni_podrizenych')],
                $this ->filterForm ->getFilterValue('filtrhodnoceni') ? 1 : 0, '?menuid=vysledkyhodnoceni&filtrhodnoceni=');
        }

        parent::renderFilter();
    }




    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();

        $result = $this ->filterForm ->buildSqlFilter($params);

        $result .=
            ($result ? ' AND ' : '') .
            '(VysledekHodnoceni.Stav <> -1)'; // nezobrazuje vyrazene - bylo to zapoznamkovano (viz nahore) - nevim proc

        $kodTypuRozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI);

        if ($this ->filterForm ->getFilterValue('filtrhodnoceni')) {
            // hodnoceni podrizenych
            if ($this->filterForm->getFilterValue('osoba')) {
                $result .= ' AND VysledekHodnoceni.HodnocenyRC = ' . $params->addParameter($this->filterForm->getFilterValue('osoba', 'rc'));
            } else {
                $result .=
                    ' AND EXISTS ' .
                    '( SELECT PracovniPomer.Id ' .
                    '  FROM PracovniPomer, RozdeleniOsob R_PracovniMisto, PMProUrovne, PoradiPredavani, ' .
                    '       RozdeleniOsob R_JA, HierarchieRozdeleni H, RozdeleniOsob R_Hodnoceny, PredavaciProces ' .
                    ' WHERE PracovniPomer.RC = ' . $params ->addParameter(User::getRC()) .
                    // jen "moje" urovne predavani podle pracovniho mista
                    '   AND R_PracovniMisto.RC = PracovniPomer.RC ' .
                    '   AND R_PracovniMisto.OsobniCislo = PracovniPomer.OsobniCislo ' .
                    '   AND R_PracovniMisto.DatumNastupu = PracovniPomer.DatumNastupu ' .
                    '   AND R_PracovniMisto.PlatnostOd <= ' . $params ->addDateParameter(time()) .
                    '   AND (R_PracovniMisto.PlatnostDo >= ' . $params ->addDateParameter(time()) . '  OR  R_PracovniMisto.PlatnostDo IS NULL) ' .
                    '   AND R_PracovniMisto.KodTypuRozdeleni = PMProUrovne.KodTypuRozdeleni ' .
                    '   AND R_PracovniMisto.Rozdeleni = PMProUrovne.KodRozdeleni ' .
                    '   AND PoradiPredavani.KodProcesu = UcelHodnoceni.KodProcesu ' .
                    '   AND PoradiPredavani.PocatecniUroven = VysledekHodnoceni.KodVstupniUrovne ' .
                    '   AND PoradiPredavani.KodUrovne = PMProUrovne.KodUrovne ' .

                    '   AND (PoradiPredavani.Vlastni = 0  OR  (PoradiPredavani.Vlastni = 1  AND  VysledekHodnoceni.HodnocenyRC = PracovniPomer.RC) )' .
                    // doplneno 11/2017 - pokud je uroven "pouze vlastni", tak to musi byt vlastni hodnoceni

                    '   AND PredavaciProces.Kod = PoradiPredavani.KodProcesu' .

                    // jen podrizena rozdeleni (vzhledem k PP!)
                    '   AND R_Ja.RC = PracovniPomer.RC ' .
                    '   AND R_Ja.OsobniCislo = PracovniPomer.OsobniCislo ' .
                    '   AND R_Ja.DatumNastupu = PracovniPomer.DatumNastupu ' .
                    '   AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
                    '   AND H.KodNadrazene = R_Ja.Rozdeleni ' .
                    '   AND H.TypPodrizene = R_Hodnoceny.KodTypuRozdeleni ' .
                    '   AND H.KodPodrizene = R_Hodnoceny.Rozdeleni ' .
                    "   AND (R_JA.KodTypuRozdeleni = PredavaciProces.KodTypuRozdeleni OR (PredavaciProces.KodTypuRozdeleni IS NULL AND R_JA.KodTypuRozdeleni = $kodTypuRozdeleni))" .
                    "   AND R_Hodnoceny.KodTypuRozdeleni = R_JA.KodTypuRozdeleni" .
                    '    AND R_Hodnoceny.RC = VysledekHodnoceni.HodnocenyRC' .
                    '    AND R_Ja.PlatnostOd <= VysledekHodnoceni.KonecObdobi' .
                    '    AND (R_Ja.PlatnostDo >= VysledekHodnoceni.PocatekObdobi  OR  R_Ja.PlatnostDo IS NULL)' .
                    '    AND R_Hodnoceny.PlatnostOd <= VysledekHodnoceni.KonecObdobi' .
                    '    AND (R_Hodnoceny.PlatnostDo >= VysledekHodnoceni.PocatekObdobi  OR  R_Hodnoceny.PlatnostDo IS NULL)' .

                    ')';
            }
        } else {
            // moje hodnoceni
            $result .= ' AND VysledekHodnoceni.HodnocenyRC = ' . $params ->addParameter(User::getRC());
        }

        // "Jen rozdeleni"
        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            if ($rozdeleniFilter ->getValue() !== null) {
                $result .=
                    ($result ? ' AND ' : '') .
                    RozdeleniUtils::sqlFilterRozdeleniPP($this ->database,
                        'Osoba',
                        $rozdeleniFilter ->getTypRozdeleni(), $rozdeleniFilter ->getValue(),
                        'VysledekHodnoceni.PocatekObdobi', 'VysledekHodnoceni.KonecObdobi',
                        $rozdeleniFilter ->getValue('vcetnePodrizenych'),
                        $params,
                        true);
            }
        }
//        if ($this ->filterForm ->getFilterValue('filtrhodnoceni') && $this ->filterForm ->kodRozdeleni) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                RozdeleniUtils::sqlFilterRozdeleniPP($this ->database,
//                    'Osoba',
//                    $this ->filterForm ->kodTypuRozdeleni, $this ->filterForm ->kodRozdeleni,
//                    'VysledekHodnoceni.PocatekObdobi', 'VysledekHodnoceni.KonecObdobi',
//                    $this ->filterForm ->vcetnePodrizenychRozdeleni, $params, true);
//        }

//        // "Jen ucel hodnoceni"
//        if (HodnoceniFilterForm::getJenUcelHodnoceni() && HodnoceniFilterForm::getUcelHodnoceni()) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                '(VysledekHodnoceni.KodUcelu = ' . $params ->addParameter((int)HodnoceniFilterForm::getUcelHodnoceni()) . ') ';
//        }

        // Obdobi
        if ($this ->filterForm ->getFilterValue('datumod')) {
            $result .=
                ($result ? ' AND ' : '') .
                'VysledekHodnoceni.KonecObdobi >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod'));
        }

        if ($this ->filterForm ->getFilterValue('datumod', 'datumDo')) {
            $result .=
                ($result ? ' AND ' : '') .
                'VysledekHodnoceni.PocatekObdobi <= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod', 'datumDo'));
        }

        $znamkaOd = $this ->filterForm ->getFilterValue('jenznamkaoddo', 'znamkaOd');
        if ($znamkaOd !== null) {
            $result .=
                ($result ? ' AND ' : '') .
                'VysledekHodnoceni.CelkovyVysledek >= ' . $params ->addParameter((float)$znamkaOd);
        }

        $znamkaDo = $this ->filterForm ->getFilterValue('jenznamkaoddo', 'znamkaDo');
        if ($znamkaDo !== null) {
            $result .=
                ($result ? ' AND ' : '') .
                'VysledekHodnoceni.CelkovyVysledek <= ' . $params ->addParameter((float)$znamkaDo);
        }

        return new SqlStatement($result, $params);
    }


    /**
     * zobrazi (pod)radky se slozkama hodnoceni
     * @param $idVysledku
     * @param $className
     * @param $jeUzavreno
     * @throws Exception
     */
    function zobrazSlozky($idVysledku, $className, $jeUzavreno)
    {
        $params = $this ->database ->createParameters();

        $sql =
            "SELECT PripraveneTemaHodnoceni.ID, PripraveneTemaHodnoceni.Stav, " .
            "   PripraveneTemaHodnoceni.DatumHodnoceni, PripraveneTemaHodnoceni.Hodnotici, " .
            "   TemaHodnoceni.PopisTematu, HodnoceniOsoby.Znamka, HodnoceniOsoby.Koeficient " .

            " FROM ( HodnoceniOsoby INNER JOIN PripraveneTemaHodnoceni " .
            "        ON HodnoceniOsoby.ID = PripraveneTemaHodnoceni.ID " .
            "      ) " .
            "      INNER JOIN TemaHodnoceni " .
            "      ON TemaHodnoceni.DruhHodnoceni = PripraveneTemaHodnoceni.DruhHodnoceni " .
            "      AND TemaHodnoceni.TemaCislo = PripraveneTemaHodnoceni.TemaCislo " .
            " WHERE HodnoceniOsoby.IdVysledku = " . $params ->addParameter((int)$idVysledku) .
            "   AND ( HodnoceniOsoby.IdNadrazenehoHodnoceni IS NULL  OR  HodnoceniOsoby.IdNadrazenehoHodnoceni = 0)";

        $cursor = $this ->database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            echo "<tr class='$className'>";
            echo
                "<td></td><td></td><td class='poznamka hodnocenipodrobnosti'>" . $row[3] . "</td>" . // hodnotici
                "<td class='poznamka hodnocenipodrobnosti'><a href='?menuid=hodnoceni&backaction=vysledkyhodnoceni&record={$row[0]}'>" . StringUtils::htmlTextConvert($row[4]) . "</a></td>" . // popis tematu
                "<td colspan='2' class='poznamka hodnocenipodrobnosti'>" . DBUtils::getFormatedValue($this ->database, "finedatetime", $row[2]) . "</td>"; // datum hodnoceni

            // stav
            echo "<td class='poznamka hodnocenipodrobnosti'>";
            switch ($row[1]) {
                case 0 :
                case 1 :
                case 2 :
                case 3 :
                case 4 :
                case 5 :
                    echo ronText('Stav_hodnoceni_' . $row[1]);
                    break;

                default :
                    echo $row[1];
            }
            echo "</td>";

            if ( RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_VYSLEDKY_HODNOCENI) & 1 ) { // znamka nebo oboji
                echo "<td class='poznamka hodnocenipodrobnosti align-right'>";
                if ($this->zobrazitPrubezneVysledky || $jeUzavreno) {
                    if ($row[5]) echo number_format($row[5], 2);
                }
                echo "</td>";
            }

            if ( RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_VYSLEDKY_HODNOCENI) & 2 ) { // koeficient nebo oboji
                echo "<td class='poznamka hodnocenipodrobnosti align-right'>";
                if ($this->zobrazitPrubezneVysledky || $jeUzavreno) {
                    if ($row[6]) echo number_format($row[6], 2);
                }
                echo "</td>";
            }

            echo "</tr>\n";
        }

        $cursor ->close();
    }


    /**
     * @param array $values
     * @param string $className
     * @param int $rowNumber
     * @param bool $edit
     * @throws Exception
     */
    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        $this->zobrazSlozky($values[0], $className, $values['col-stavhodnoceni']); // $values['col-stavhodnoceni'] - priznak, jestli je uzavreno

        $this->renderSeparator();
    }

}
