<?php
/**
 * Roman.Szablatura RON Software 13.09.2019
 */

namespace RonPortal\Jidelna;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\Csrf;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Log;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\System\AgentManager;
use RonPortal\System\FotoOutputObject;

/**
 * Class JidelnaObjednavkyController
 * @package Ron\RonPortal\Jidelna
 */
class ObjednavkyController extends JsonController
{

    /** @var Database $database */
    public $database;

    /** @var PravaNaOsoby $pravo_prevedeni_objednavky */
    public $pravo_prevedeni_objednavky;

    /** @var PravaNaOsoby $pravo_zruseni_objednavky */
    public $pravo_zruseni_objednavky;



    /**
     * JidelnaObjednavkyController constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;
//        $this->pravo_zobrazeni_objednavek = Prava::vratPrava('prohl_jidelna');
        //TODO Doplnit prava na zruseni a prevedeni objednavky
        $this->pravo_prevedeni_objednavky = Prava::vratPrava('jidelna_prevedeni_objednavky');
        $this->pravo_zruseni_objednavky = Prava::vratPrava('jidelna_zruseni_objednavky');

        $this ->addTranslationsScript('jidelnaobjednavky');
        HtmlHeaders::getInstance()->addScript('script/jidelnascripts.js');
        TextManager::getInstance()->loadStrings('jidelna.ini');
        TextManager::getInstance()->loadStrings('person.ini');
    }


    /**
     * @param $id
     * @return BaseViewObject
     * @throws \Exception
     */
    public function detailObjednavky($id)
    {
        $objednavka = DetailObjednavky::nactiDetail($this->database, $id);
        if (!Prava::lzeZobrazitPP('prohl_jidelna', $objednavka ->idPP, $objednavka ->den)) {
            throw new NeniPravoException();
        }

        // Zpristupni dle prava tlacitka pro prevod / odstraneni objednavky
        Model::addData('pravo_prevedeni_objednavky', $this->pravo_prevedeni_objednavky);
        Model::addData('pravo_zruseni_objednavky', $this->pravo_zruseni_objednavky);
        Model::addData('objednavka', $objednavka);

        return new BaseViewObject('ronportal/jidelna/view_detail_objednavky.php');
    }


    /**
     * @param $idpolozky
     * @return FotoOutputObject|null
     * @throws DBException
     */
    private function fotoPolozkyJidelnicku($idpolozky)
    {
        $params = $this->database->createParameters();
        $sql =
            "SELECT KodSerialFoto FROM PolozkaJidelnicku WHERE ID = " . $params->addParameter($idpolozky);

        $cursor = $this->database->getSqlResult($sql, $params);

        if ($row = $cursor->fetch()) {
            return new FotoOutputObject($this->database, $row[0]);
        }
        $cursor->close();

        return null;
    }

//    /**
//     * @param int $osoba
//     * @param $id
//     * @throws DBException
//     */
//    public function zrusitObjednavkuPresDetail($osoba, $id)
//    {
//        $this->zrusitObjednavku($osoba, $id);
//    }


    /**
     * Vytvori DBRecord pro zadanou objednavku
     * @param int $idObjednavky
     * @param string $logText
     * @return RonPortalDBRecord
     */
    private function createObjednaneJidloDBRecord($idObjednavky = null, $logText = null)
    {
        $rec = new RonPortalDBRecord($this ->database, 'ObjednaneJidlo', $this);
        if ($logText) {
            $rec ->addLogText($logText);
        }
        $rec ->addField('Id', 'serial', true);
        $rec ->addIntField('IdPolozkyJidelnicku', true);
        $rec ->addStringField('RC',true, 15);
        $rec ->addStringField('OsobniCislo',true, 15);
        $rec ->addDateField('DatumNastupu', true);

        $rec ->addIntField('PoradiVydani', true);

        $rec ->addTimestampField('DatumCasObjednani', true, time());
        $rec ->addStringField('Uzivatel',true,25, RonPortalUtils::getLogUserName());

        $rec ->addTimestampField('DatumCasZruseni', true);

        $rec ->addIntField('StavVlozeniNaBurzu', true);
        $rec ->addTimestampField('DatumCasVlozeniNaBurzu', true);

        if ($idObjednavky) {
            $params = $this->database->createParameters();
            $rec->setWhere('Id = ' . $params->addParameter($idObjednavky), $params);
        }

        return $rec;
    }

    /**
     * Zrusi zadanou objednavku. Kontroluje, jestli ma uzivatel pravo a jestli neni uzavrene obdobi.
     * @param int $idObjednavky
     * @return void
     * @throws DBException
     * @throws Exception
     */
    private function zrusitObjednavku($idObjednavky)
    {
        Csrf::validateCSRFPostParam('zruseni_objednavky_' . $idObjednavky);

        // Nacte informace o objednavce
        $params = $this->database->createParameters();
        $sql =
            'SELECT ObjednaneJidlo.Id, ObjednaneJidlo.PoradiVydani,' . // 0,1
            ' PolozkaJidelnicku.Jidelna, PolozkaJidelnicku.Den, PolozkaJidelnicku.TypDennihoJidla,' . // 2..4
            ' ObjednaneJidlo.DatumCasVydani, ObjednaneJidlo.DatumCasZruseni,' . // 5..6
            ' PracovniPomer.Id, PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, ' . // 7..10
            ' Osoba.Prijmeni, Osoba.Jmeno, PolozkaJidelnicku.TypDennihoJidla, PolozkaJidelnicku.CisloNabidky, PolozkaJidelnicku.PopisNabidky' . // 11..15

            ' FROM ObjednaneJidlo JOIN PolozkaJidelnicku ON PolozkaJidelnicku.Id = ObjednaneJidlo.IDPolozkyJidelnicku' .
            '  JOIN PracovniPomer ON PracovniPomer.RC = ObjednaneJidlo.RC AND PracovniPomer.OsobniCislo = ObjednaneJidlo.OsobniCislo' .
            '   AND PracovniPomer.DatumNastupu = ObjednaneJidlo.DatumNastupu' .
            '  JOIN Osoba ON Osoba.RC = ObjednaneJidlo.RC' .
            ' WHERE ObjednaneJidlo.ID = ' . $params->addParameter($idObjednavky);

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor->close();

        if (!$row) {
            throw new Exception(TextManager::_('Objednavka_nebyla_nalezena'));
        }

        // overi prava uzivatele na zruseni objednavky
        $idPP = (int)$row[7];
        $den = $this ->database ->stringToTimestamp($row[3]);
        Prava::nastavObdobiProPrava($den);

        if (!$this->pravo_zruseni_objednavky ->lzeZobrazitPP($idPP, $den)) {
            throw new NeniPravoException();
        }

        // pokud uz je zrusena, tak nedela nic
        if ($row[6]) {
            return;
        }

        // zkontroluje, jestli uz neni uzavreno obdobi
        if (Obdobi::jeObdobiUzavreno($this ->database, 'JIDELNA', $den)) {
            throw new Exception(TextManager::_('Obdobi_uzavreno'));
        }

        // a jestli uz neni objednavka vydana
        if ($row[5]) {
            throw new Exception(TextManager::_('Jidelna_objednavka_vydana'));
        }

        // zapise datum zruseni
        $rec = $this ->createObjednaneJidloDBRecord(null, "Zruseni objednavky {$row[13]} {$row[14]} ({$row[15]}) - {$row[12]} {$row[11]}");
        $params = $this->database->createParameters();
        $rec->setWhere('Id = ' . $params->addParameter($idObjednavky) . ' AND DatumCasVydani IS NULL', $params); // nelze zrusit vydane!

        if ($rec ->loadRecord()) {
            $rec['DatumCasZruseni'] = time();
            $rec ->updateRecord();
        }

        // Musi opravit poradi vydeje ostatnich polozek pro stejny typ jidla,
        // jinak by to delalo problem u vydeje pri pridelovani priloh
        $params = $this->database->createParameters();
        $sql =
            'UPDATE ObjednaneJidlo ' .
            ' SET PoradiVydani = (PoradiVydani - 1) ' .
            " WHERE (RC = " . $params ->addParameter($row[8]) . ") " .
            "   AND (OsobniCislo = " . $params ->addParameter($row[9]) . ") " .
            '   AND (DatumNastupu = ' . $params ->addDateParameter($this->database->stringToTimestamp($row[10])) . ') ' .
            '   AND (DatumCasZruseni IS NULL) ' .
            '   AND (DatumCasVydani IS NULL) ' .
            '   AND (PoradiVydani >= ' . $params ->addParameter((int)$row[1]) . ') ' .
            '   AND (EXISTS(SELECT ID ' .
            '        FROM PolozkaJidelnicku ' .
            '        WHERE (PolozkaJidelnicku.ID = ObjednaneJidlo.IDPolozkyJidelnicku) ' .
            '          AND (PolozkaJidelnicku.KodTypuRozdeleni = 8) ' . // typ rozdeleni jidelna
            "          AND (PolozkaJidelnicku.Jidelna = " . $params ->addParameter($row[2]) . ") " .
            '          AND (PolozkaJidelnicku.Den = ' . $params ->addDateParameter($this->database->stringToTimestamp($row[3])) . ') ' .
            "          AND (PolozkaJidelnicku.TypDennihoJidla = " . $params ->addParameter($row[4]) . ") " .
            '       ))';

        try {
            $this->database->sql($sql, $params);
        } catch (Exception $e) {
            // chyba pri update
            Log::addRecordToDBLog($this->database,
                'ObjednaneJidlo', 'ERROR', $this,
                'Chyba pri oprave poradi vydeje objednanych jidel!' . "\n" . $e ->getMessage() . "\n\n$sql");
        }

        AgentManager::vyuctovaniJidelny($idPP);
    }



//    public function zrusitObjednavku_OLD($osoba, $id)
//    {
//        $ppomer = PracovniPomer::nactiPP($this ->database, $osoba);
//
//        $polozkaJidelnicku = new PolozkaJidelnicku();
//        $polozkaJidelnicku->nactiPolozkuACelkovyPocetObjednavek($this->database, $id, $ppomer);
//
//        //Na pravo 'jidelna_prevedeni'
//        Prava::nastavObdobiProPrava($polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);
//
//        if (!$this->pravo_zruseni_objednavky ->lzeZobrazitPP($ppomer ->idPP, $polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den)) {
//            throw new NeniPravoException();
//        }
//
//        // Nacte seznam objednavek k zadane polozce
//        $params = $this ->database ->createParameters();
//        $sql =
//            'SELECT ObjednaneJidlo.Id, ObjednaneJidlo.PoradiVydani, ' .
//            ' PolozkaJidelnicku.Jidelna, PolozkaJidelnicku.Den, PolozkaJidelnicku.TypDennihoJidla, ObjednaneJidlo.DatumCasVydani ' .
//
//            ' FROM ObjednaneJidlo LEFT JOIN PolozkaJidelnicku ON (PolozkaJidelnicku.Id = ObjednaneJidlo.IDPolozkyJidelnicku) ' .
//            ' LEFT JOIN PracovniPomer ON (PracovniPomer.RC = ObjednaneJidlo.RC AND PracovniPomer.OsobniCislo = ObjednaneJidlo.OsobniCislo
//                AND PracovniPomer.DatumNastupu = ObjednaneJidlo.DatumNastupu) ' .
//
//            ' WHERE IDPolozkyJidelnicku = ' . $params ->addParameter($polozkaJidelnicku ->f_id) .
//            '  AND PracovniPomer.ID = ' . $params ->addParameter($ppomer->idPP) .
//            '  AND DatumCasZruseni IS NULL ' .
//            ' ORDER BY PoradiVydani DESC';
//
//        $cursor = $this->database->getSqlResult($sql, $params);
//        $radek_objednavky = $cursor->fetch();
//        $cursor ->close();
//
//
//        //Objednavka je zrusena
//        if (!$radek_objednavky) {
//            throw new Exception(TextManager::_('Objednavka_nebyla_nalezena'));
//        }
//
//        //Objednavka je vydana nebo zrusena
//        if ($radek_objednavky[5]) {
//            throw new Exception(TextManager::_('Jidelna_objednavka_vydana'));
//        }
//
//        $params = $this ->database ->createParameters();
//        $sql =
//            'UPDATE ObjednaneJidlo SET DatumCasZruseni = ' . $params ->addParameter($this->database->timestampToParam(time())) .
//            ' WHERE Id = ' . $params ->addParameter((int)$radek_objednavky[0]);
//
//        $this->database->sql($sql, $params);
//
//        // zaloguje zruseni objednavky
//        Log::addRecordToDBLog($this->database, 'ObjednaneJidlo', 'UPDATE', $this,
//            'Zruseni objednavky: RC=' . $ppomer ->rc . ', PolozkaJidelnicku=' . $polozkaJidelnicku ->f_id . "\n   " .
//            $ppomer->jmenoPrijmeniKratke . "\n   " .
//            date('j.n.Y', $polozkaJidelnicku->f_den) . ', ' .
//            $polozkaJidelnicku->f_typdennihojidla . ' ' .
//            $polozkaJidelnicku->f_cislonabidky . ' - ' .
//            $polozkaJidelnicku->f_popisnabidky .
//            "\n\n" .
//            $sql);
//
//        // Musi opravit poradi vydeje ostatnich polozek pro stejny typ jidla,
//        // jinak by to delalo problem u vydeje pri pridelovani priloh
//        $params = $this->database->createParameters();
//        $sql =
//            'UPDATE ObjednaneJidlo ' .
//            ' SET PoradiVydani = (PoradiVydani - 1) ' .
//            " WHERE (RC = " . $params ->addParameter($ppomer ->rc) . ") " .
//            "   AND (OsobniCislo = " . $params ->addParameter($ppomer ->osobniCislo) . ") " .
//            '   AND (DatumNastupu = ' . $params ->addParameter($this->database->timestampToParam($ppomer ->datumNastupu)) . ') ' .
//            '   AND (DatumCasZruseni IS NULL) ' .
//            '   AND (DatumCasVydani IS NULL) ' .
//            '   AND (PoradiVydani >= ' . (int)$radek_objednavky[1] . ') ' .
//            '   AND (EXISTS(SELECT ID ' .
//            '        FROM PolozkaJidelnicku ' .
//            '        WHERE (PolozkaJidelnicku.ID = ObjednaneJidlo.IDPolozkyJidelnicku) ' .
//            '          AND (PolozkaJidelnicku.KodTypuRozdeleni = 8) ' . // typ rozdeleni jidelna
//            "          AND (PolozkaJidelnicku.Jidelna = " . $params ->addParameter($radek_objednavky[2]) . ") " .
//            '          AND (PolozkaJidelnicku.Den = ' . $params ->addParameter($this->database->timestampToParam($this->database->stringToTimestamp($radek_objednavky[3]))) . ') ' .
//            "          AND (PolozkaJidelnicku.TypDennihoJidla = " . $params ->addParameter($radek_objednavky[4]) . ") " .
//            '       ))';
//
//        try {
//            $this->database->sql($sql, $params);
//            MessageManager::addInfoMessage(TextManager::_("Jidelna_zruseni_objednavky"));
//        } catch (Exception $e) {
//            // chyba pri update
//            MessageManager::addErrorMessage($e);
//            Log::addRecordToDBLog($this->database,
//                'ObjednaneJidlo', 'ERROR', $this,
//                'Chyba pri oprave poradi vydeje objednanych jidel!' . "\n" . $e ->getMessage() . "\n\n$sql");
//        }
//
//        AgentManager::vyuctovaniJidelny($ppomer ->idPP);
//
//        //redirect na browser
//        return new Redirect(null, "?menuid=jidelnaobjednavky");
//    }


    /**
     * @param JsonOutputObject $json
     * @param int   $idObjednavky
     * @param int   $prijemce
     * @throws DBException
     * @throws Exception
     */
    public function prevestObjednavku($json, $idObjednavky, $prijemce)
    {
        Csrf::validateCSRFPostParam('prevod_objednavky_' . $idObjednavky);

        Prava::nastavObdobiProPrava(time(), time());

        // nacte detail objednavky
        $detail = DetailObjednavky::nactiDetail($this ->database, $idObjednavky);

        // kontrola prav
        if (!$this->pravo_prevedeni_objednavky ->lzeZobrazitPP($detail ->idPP)) {
            throw new NeniPravoException();
        }

        if (!$this->pravo_prevedeni_objednavky ->lzeZobrazitPP($prijemce)) {
            throw new NeniPravoException();
        }

        if (Obdobi::jeObdobiUzavreno($this ->database, 'JIDELNA', $detail ->den)) {
            throw new Exception(TextManager::_('Obdobi_uzavreno'));
        }

        if ($detail ->datumCasVydani) {
            throw new Exception(TextManager::_('Objednavka_jiz_byla_vydana'));
        }

        if ($detail ->datumCasZruseni) {
            throw new Exception(TextManager::_('Objednavka_jiz_byla_zrusena'));
        }

        $pPomerPrijemce = PracovniPomer::nactiPP($this ->database, $prijemce);

        if ($pPomerPrijemce ->idPP === $detail ->idPP) {
            // neni co prevadet - objednavka jiz patri zadanemu prijemci
            return;
        }

        $polozkaJidelnicku = PolozkaJidelnicku::nactiPolozuJidelnicku($this->database, $detail ->idPolozkyJidelnicku);

        $this ->database ->beginTransaction();
        try {
            // zrusi puvodni objednavku
            $rec = $this ->createObjednaneJidloDBRecord(
                $detail ->idObjednavky,
                'Prevedeni objednavky na jineho stravnika - zruseni puvodni objednavky'
            );
            $rec ->findFieldByName('DatumCasZruseni') ->setNewValue(time());

//            $rec = new RonPortalDBRecord($this ->database, 'ObjednaneJidlo', $this);
//            $rec ->addLogText('Prevedeni objednavky na jineho stravnika - zruseni puvodni objednavky');
//            $rec ->addField('Id', 'int', true);
//            $rec ->addField('IdPolozkyJidelnicku', 'int', true);
//            $rec ->addField('RC', '', true);
//            $rec ->addField('OsobniCislo', '', true);
//            $rec ->addField('DatumNastupu', 'timestamp', true);
//            $rec ->addField('DatumCasZruseni', 'timestamp', false, 0, time()); // zrusi objednavku
//
//            $rec ->addField('StavVlozeniNaBurzu', 'int', true);
//            $rec ->addField('DatumCasVlozeniNaBurzu', 'timestamp', true);
//
//            $params = $this ->database ->createParameters();
//            $rec ->setWhere('Id = ' . $params ->addParameter($detail ->idObjednavky), $params);

            if ($rec ->loadRecord()) {
                $rec ->updateRecord();
            } else {
                throw new Exception('Chyba pri zruseni objednavky puvodniho stravnika: Objednavka nenalezena!');
            }

            // ... a vytvori novou pro prijemce
            //$newId = SQLUtils::newSerialNumber($this->database, 'ObjednaneJidlo', 'ID');
            $newPoradi = $polozkaJidelnicku->novePoradiVydani($this->database,
                $pPomerPrijemce->rc, $pPomerPrijemce->osobniCislo, $pPomerPrijemce->datumNastupu);

            $newRec = $this ->createObjednaneJidloDBRecord(null, 'Prevedeni objednavky na jineho stravnika - vlozeni nove objednavky');
            $newRec['IdPolozkyJidelnicku'] = $detail ->idPolozkyJidelnicku;
            $newRec['RC'] = $pPomerPrijemce->rc;
            $newRec['OsobniCislo'] = $pPomerPrijemce->osobniCislo;
            $newRec['DatumNastupu'] = $pPomerPrijemce->datumNastupu;
            $newRec['PoradiVydani'] = $newPoradi;
            $newRec ->insertRecord();

//            $rec = new RonPortalDBRecord($this->database, 'ObjednaneJidlo', $this);
//            $rec ->addLogText('Prevedeni objednavky na jineho stravnika - vlozeni nove objednavky)');
//            $rec ->addField('Id', 'int', true, 0, $newId);
//            $rec ->addField('IdPolozkyJidelnicku', 'int', true, 0, $detail ->idPolozkyJidelnicku);
//            $rec ->addField('RC', '', true, 15, $pPomerPrijemce->rc);
//            $rec ->addField('OsobniCislo', '', true, 15, $pPomerPrijemce->osobniCislo);
//            $rec ->addField('DatumNastupu', 'timestamp', true, 0, $pPomerPrijemce->datumNastupu);
//            $rec ->addField('PoradiVydani', 'int', true, 0, $newPoradi);
//            $rec ->addField('DatumCasObjednani', 'timestamp', true, 0, time());
//            $rec ->addField('Uzivatel', '', true, 25, RonPortalUtils::getLogUserName());
//            $rec->insertRecord();

            $this ->database ->commitTransaction();
        } catch (Exception $e) {
            $this ->database ->rollbackTransaction();
            throw $e;
        }

        AgentManager::vyuctovaniJidelny($detail ->idPP);
        AgentManager::vyuctovaniJidelny($prijemce);
    }


    /**
     * Vlozi zadanou objednavku do burzy
     * @param int $idObjednavky
     * @param JsonOutputObject $output
     * @return void
     * @throws Exception
     */
    function vlozObjednavkuDoBurzy($idObjednavky, $output)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::BURZA_POVOLENA)) {
            throw new Exception('Burza neni povolena!');
        }

        Csrf::validateCSRFPostParam('objednavka_do_burzy_' . $idObjednavky);

        $detail = DetailObjednavky::nactiDetail($this ->database, $idObjednavky);
        $ppomer = PracovniPomer::nactiPP($this ->database, $detail ->idPP);

        $polozkaJidelnicku = new PolozkaJidelnicku();
        $polozkaJidelnicku->nactiPolozkuACelkovyPocetObjednavek($this->database, $detail ->idPolozkyJidelnicku, $ppomer);

        Prava::nastavObdobiProPrava($polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);

        $prava = Prava::vratPrava('jidelna_do_burzy');
        if (!$prava ->lzeZobrazitPP($ppomer ->idPP, $polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den)) {
            throw new NeniPravoException();
        }

        $manager = new ObjednavkyManager($this ->database);
        $manager ->nactiData($ppomer,
            $polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);

        $manager ->nactiStavObjednavani($polozkaJidelnicku);
        $polozkaJidelnicku ->f_pocetobjednavekosoby = $manager ->pocetObjednavekOsobyProPolozku($polozkaJidelnicku ->f_id);

        if (!$manager->lzePredatObjednavkuDoBurzy($polozkaJidelnicku)) {
            throw new Exception(TextManager::_('Objednavku_nelze_predat_do_burzy'));
        }

        Burza::vlozPolozkuDoBurzy($this->database, $ppomer, $polozkaJidelnicku, $detail ->idObjednavky);

        $detail = DetailObjednavky::nactiDetail($this ->database, $idObjednavky);
        $output ->addText('datumCasVlozeniNaBurzu', StringUtils::jsonText(StringUtils::format('finedatetime', $detail ->datumCasVlozeniNaBurzu)));
    }

    /**
     * @param JsonOutputObject $output
     * @param $jidelna
     * @throws DBException
     * @throws Exception
     */
    function seznamOsob($output, $jidelna)
    {
        $osoby = [
            ['idpp' => '']
        ];

        $params = $this->database->createParameters();
        $sql =
            'SELECT DISTINCT PP.ID, Osoba.Prijmeni, Osoba.Jmeno' .
            ' FROM PracovniPomer PP' .
            ' INNER JOIN RozdeleniOsob RO ON (PP.RC = RO.RC AND PP.OsobniCislo = RO.OsobniCislo AND PP.DatumNastupu = RO.DatumNastupu)' .
            ' INNER JOIN Osoba ON (PP.RC = Osoba.RC)' .

            ' WHERE PP.DatumNastupu <= ' . $params->addDateParameter(time()) .
            ' AND (PP.DatumUkonceni >= ' . $params->addDateParameter(time()) . ' OR PP.DatumUkonceni IS NULL)' .
            ' AND RO.PlatnostOd <= ' . $params->addDateParameter(time()) .
            ' AND (RO.PlatnostDo >= ' .$params->addDateParameter(time()) . ' OR RO.PlatnostDo IS NULL)' .
            ' AND RO.KodTypuRozdeleni = 8' .
            ($jidelna ? ' AND RO.Rozdeleni = ' . $params->addParameter($jidelna)  : '') .
            ' ORDER BY Osoba.Prijmeni';

        $cursor = $this->database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()){
            if ($this->pravo_prevedeni_objednavky->lzeZobrazitPP($row[0])) {
                $osoby[] = [
                    'idpp' => StringUtils::jsonText($row[0]),
                    'prijmeni' => StringUtils::jsonText($row[1]),
                    'jmeno' => StringUtils::jsonText($row[2]),
                    'jmenoPrijmeni' => StringUtils::jsonText( ($row[1] ?: '') . ' ' . ($row[2] ?: ''))
                ];
            }
        }

        $cursor ->close();

        $output ->addData('items', $osoby);
        $output->compress = 'gzip';
    }


    /**
     * @param $action
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'seznamosob' :
                //Nacte osoby dle prava osoby pro danou jidelnu
                $this ->seznamOsob($output, $_GET['jidelna'] ?? null);
                break;

            case 'prevodobjednavkynaosobu' :
                $this->prevestObjednavku($output, $_POST['cisloObjednavky'], $_POST['novaOsobaIDPP']);
                break;

            case 'delete' :
                $this ->zrusitObjednavku((int)$_POST['idObjednavky']);
                break;

            case 'doburzy' :
                $this ->vlozObjednavkuDoBurzy((int)$_POST['idObjednavky'], $output);
                break;

            default :
                throw new \Exception('Neznama akce!');
        }

    }



    /**
     * @param string $action
     * @return OutputObject|Redirect|ViewObject|string
     * @throws \Exception
     */
    public function handleAction($action)
    {
        if (!Prava::vratPrava('prohl_jidelna')) {
            throw new NeniPravoException();
        }
//        if (!($this->pravo_prevedeni_objednavky->jsouNejakePrava() || $this->pravo_zruseni_objednavky->jsouNejakePrava())){
//            throw new NeniPravoException();
//        }

        if (isset($_GET['action']) && $_GET['action'] == 'zobrazfoto') {
            return $this->fotoPolozkyJidelnicku((int)$_GET['idpolozky']);
        }

        HtmlHeaders::getInstance()->setIcon(RonPortalSettings::getValue('shortcut_icon') ?: 'images/icon_jidelna.png');
        HtmlHeaders::getInstance() ->setTitle( TextManager::_('RON_Portal') . ' - ' . TextManager::_('Jidelna'));
        HtmlHeaders::getInstance() ->addCss('css/jidelna.css');

        Prava::nastavObdobiProPrava(time(), time());

        if ($_POST['jidelnazrusitobjednavku'] ?? false) {
            try {
                $this->zrusitObjednavku((int)$_POST['id']);
            } catch (DBException $e) {
                throw $e;
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e);
                return $this->detailObjednavky($_POST['id']);
            }
            MessageManager::addSuccessMessage(TextManager::_('Jidelna_zruseni_objednavky'));

            return new Redirect(null, '?menuid=jidelnaobjednavky');
        }


        if (isset($_GET['action'])) {
            switch ($_GET['action']) {
                case 'seznamosob':
                case 'prevodobjednavkynaosobu':
                case 'delete' :
                case 'doburzy' :
                    return parent::handleAction($action);

                case 'detailobjednavkyjidla':
                    return $this->detailObjednavky($_GET['id']);

//                case 'objednavkuodstranitodkazem':
//                    return $this->zrusitObjednavku_OLD($_GET['osoba'], $_GET['id']);

//            case 'detailobjednavkyprevodunaosobu':
//                return $this->detailObjednavkyPrevoduNaOsobu($_GET['idpp'], $_GET['cisloObjednavky']);

                case 'translations':
                    return $this->translations();
            }
        }

        Model::addViewObject(new BaseViewObject('ronportal/jidelna/view_prevedeni_objednavky.php'));

        $b = new SeznamObjednavekBrowser($this->database);
        $b->load();

        return $b;
    }
}
