<?php

/**
 * Prehled vyuctovani stravy po mesicich
 */

namespace RonPortal\Jidelna;

use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilterRok;
use RonPortal\Controls\RonPortalBrowser;

class UcetStravnikaBrowser extends RonPortalBrowser
{
    /**
     * @param \Ron\DB\Database $database
     */
    function __construct($database)
    {
        parent::__construct($database, 'ucetstravnika');

        $this->title = TextManager::_('Ucet_stravnika');
        $this->code = 'browserucetstravnika';

        $this->sqlFromPart = 'UcetStravnikaZaObdobi, Obdobi';
        $this->sqlWherePart =
            'UcetStravnikaZaObdobi.Obdobi = Obdobi.PocatekObdobi' .
            " AND Obdobi.Aplikace = 'JIDELNA' ";

        //$this->primaryField = 0;

        $this->addField()
            ->setSqlFieldName('Obdobi.Popis')
            ->setCode('col-obdobi') ->setHeaderText('Obdobi') ->setOrdering(true)
            ->setSortSQL('Obdobi.PocatekObdobi');

//        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
//            ->setSqlFieldName('Obdobi.PocatekObdobi')
//            ->setCode('col-pocatekobdobi');

        $this->defaultSort = '-col-obdobi';

        $this->addField()
            ->setSqlFieldName('UcetStravnikaZaObdobi.Castka_Celkem') ->setHeaderText('Celkem_za_stravu')
            ->setCode('col-castkacelkem') ->setFormat('curr') ->setAlign('right') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('UcetStravnikaZaObdobi.Castka_Dotace')
            ->setCode('col-castkadotace') ->setHeaderText('Castka_dotace') ->setFormat('curr') ->setAlign('right') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('UcetStravnikaZaObdobi.Castka_SumaOperaciHotove')
            ->setCode('col-sumahotove') ->setHeaderText('Hotovost') ->setFormat('curr') ->setAlign('right') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('UcetStravnikaZaObdobi.Castka_AktualniStav')
            ->setCode('col-aktualnistav') ->setHeaderText('Aktualni_stav_uctu') ->setFormat('curr') ->setAlign('right') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('UcetStravnikaZaObdobi.Pocet_NarokNaDotace')
            ->setCode('col-naroknadotace') ->setHeaderText('Narok_na_dotace') ->setFormat('number_2') ->setAlign('right') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('UcetStravnikaZaObdobi.Pocet_Odebrano')
            ->setCode('col-pocetodebrano') ->setHeaderText('Pocet_odebrano') ->setVisible(false)
            ->setFormat('number_2') ->setAlign('right') ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('UcetStravnikaZaObdobi.Pocet_Dotovano')
            ->setCode('col-pocetdotovano') ->setHeaderText('Pocet_dotovano') ->setVisible(false)
            ->setFormat('number_2') ->setAlign('right') ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('UcetStravnikaZaObdobi.Pocet_Nedotovano')
            ->setCode('col-pocetnedotovano') ->setHeaderText('Pocet_nedotovano') ->setVisible(false)
            ->setFormat('number_2') ->setAlign('right') ->setOrdering(true);
    }


    function setFilterForm()
    {
        $this ->filterForm = $this->createFilterForm(null, 'prohl_jidelna', false, true);
        $this ->filterForm ->addFilter(
            (new FilterFormFilterRok('rok'))
                ->setMinValue(1970)
        );
        $this ->filterForm ->addFilter(new StavUctuFilter($this ->filterForm));
//        parent::setFilterForm();
//        $this ->filterForm ->nastavAkciGet('ucetstravnika');
//        $this ->filterForm ->addFilter(new StavUctuFilter($this ->filterForm));
    }


//    function load($sqlFilter = '')
//    {
//        if (isset($_GET['rok'])) {
//            $_SESSION['rok'] = (int)$_GET['rok'];
//        }
//
//        if (!isset($_SESSION['rok'])) {
//            $_SESSION['rok'] = date('Y');
//        }
//
//        parent::load($sqlFilter);
//    }


//    function renderFilter()
//    {
//        parent::renderFilter();
//
//        $rok = $_SESSION['rok'];
//
//        echo
//            '<div class="margin-top-2 margin-bottom-2">' .
//            '<a href="?menuid=ucetstravnika&rok=' . ($rok - 1) . '"> << </a>' .
//            '<span class="margin-all-2 padding-all-2">' . $rok . '</span>' .
//            '<a href="?menuid=ucetstravnika&rok=' . ($rok + 1) . '"> >> </a>' .
//            '</div>';
//    }


    function getSQLFilter()
    {
        //$rok = $_SESSION['rok'];
        $rok = $this ->filterForm ->getFilterValue('rok');

        $params = $this ->database ->createParameters();

        return new SqlStatement(
            'UcetStravnikaZaObdobi.Obdobi >= ' . $params ->addDateParameter(mktime(0, 0, 0, 1, 1, $rok)) .
            ' AND UcetStravnikaZaObdobi.Obdobi <= ' . $params ->addDateParameter(mktime(0, 0, 0, 12, 31, $rok)) .
            ' AND UcetStravnikaZaObdobi.RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            ' AND UcetStravnikaZaObdobi.OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            ' AND UcetStravnikaZaObdobi.DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')),
            $params);
    }
}

