<?php
/**
 * stanislav.elbl, 21.08.2015
 */

namespace RonPortal\Kapacity;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\Csrf;
use Ron\MessageManager;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\VarDumpViewObject;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Dochazka\AbsenceDBRecord;
use RonPortal\Dochazka\DenniPoznamkaDochazky;
use RonPortal\Dochazka\DokumentDochazky;
use RonPortal\Dochazka\OperaceDochazky;
use RonPortal\Dochazka\Smena;
use RonPortal\Dochazka\SmenaProDen;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\Editace\DenniPoznamkaDochazkyRecord;
use RonPortal\Log;
use RonPortal\NastaveniUzivatele;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\PoznamkaKeDni;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalScriptEvents;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\System\AgentManager;
use RonPortal\User;
use RonPortal\XML\RonPortalXMLForm;


class KapacityJSController extends JsonController
{
    /** @var  Database */
    protected $database;

    /**
     * @param Database $database
     */
    public function __construct($database)
    {
        $this ->database = $database;
    }


    /**
     * @param $idPP
     * @return PracovniPomer
     * @throws Exception
     */
    protected function nactiPP($idPP)
    {
        return PracovniPomer::nactiPP($this ->database, $idPP);
    }


    /**
     * Hromadne nacte dokumenty ke dni dochazky. Nacita dokumenty ze zadaneho obdobi.
     * @param int    $datumOd
     * @param int    $datumDo
     * @param int|int[] $idPP  Id pracovniho pomeru nebo seznam id PP, pro ktere se maji dokumenty nacist
     * @param JsonOutputObject $outputObject
     * @throws Exception
     */
    public function nactiDokumentyDochazkyZDB($datumOd, $datumDo, $idPP, $outputObject)
    {
        if (is_array($idPP)) {
            $prava = Prava::vratPrava('kapacity');
            $pravaEdit = Prava::vratPrava('kapacity_edit');

            //$dokumenty = DokumentDochazky::nactiDokumentyDochazky($this ->database, $datumOd, $datumDo, [$prava, $pravaEdit], $filter);
            $dokumenty = DokumentDochazky::nactiDokumentyOsob($this->database, $datumOd, $datumDo, $idPP, [$prava, $pravaEdit]);
        } else {
            $dokumenty = DokumentDochazky::nactiDokumentyOsoby($this ->database, $datumOd, $datumDo, $idPP);
        }

        $jsonDokumenty = [];
        foreach ($dokumenty as $dokument) {
            $jsonDokumenty[] = $dokument ->getJson();
        }

        $outputObject->addData('dokumenty', $jsonDokumenty);
    }


    /**
     * @param int $datumOd
     * @param int $datumDo
     * @param int[] $seznamPP
     * @param JsonOutputObject $outputObject
     */
    public function nactiSluzebniCestyZDB($datumOd, $datumDo, $seznamPP, $outputObject)
    {
        $seznamSC = PlanovaniKapacit_SluzebniCesta::nactiSluzebniCestyZDB($this ->database, $datumOd, $datumDo, $seznamPP);

        $result = [];
        foreach ($seznamSC as $sc) {
            $result[] = $sc ->getJson();
        }

        $outputObject->addData('sluzebniCesty', $result);
    }


    /**
     * @param JsonOutputObject $outputObject
     * @throws Exception
     */
    public function nactiDataOsobyProRefresh($outputObject)
    {
        $idpp = (int)$_GET['idpp'];
        $obdobiOd = DateUtils::urlStringToDate($_GET['datumod']);
        $obdobiDo = DateUtils::urlStringToDate($_GET['datumdo']);

        if (!$idpp || !$obdobiOd || !$obdobiDo) {
            throw new Exception('Chyba pri nacitani dat osoby! Nejsou zadany pozadovane parametry!');
        }

        Prava::nastavObdobiProPrava($obdobiOd, $obdobiDo);

        $rezimUprav = RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV)  &&  isset($_GET['rezimuprav'])  &&  (int)$_GET['rezimuprav'];

        // prava nekontroluji, protoze osoba muze byt z jineho rozdeleni
        $osoba = new PlanovaniKapacit_Osoba();
        $osoba->nactiDataProRefresh($this ->database, $idpp, $obdobiOd, $obdobiDo, $rezimUprav);

        $outputObject ->addData('osoby', [$osoba ->getJson()]);

        $this ->nactiDokumentyDochazkyZDB($obdobiOd, $obdobiDo, $idpp, $outputObject);

        //$prava = [Prava::vratPrava('kapacity'), Prava::vratPrava('kapacity_edit')];
        $jsonKontroly = VysledekKontrolyPlanovani::nactiKontrolyJSON($this ->database, $obdobiOd, $obdobiDo, $idpp, null, null);
        $outputObject ->addData('kontroly', $jsonKontroly);

        $jsonPoznamky = DenniPoznamkaDochazky::nactiPoznamkyDochazkyJSON($this ->database, $obdobiOd, $obdobiDo, $idpp);
        $outputObject ->addData('poznamky', $jsonPoznamky);
    }


    /**
     * Prepocita delku nebo konec smeny podle nastaveni vkladani prestavek
     * @param JsonOutputObject $outputObject
     * @throws Exception
     */
    public function prepocetDelkySmeny($outputObject)
    {
        if (trim($_GET['pocatek']) == '') {
            throw new Exception(\Ron\ronText('Chyba_neni_zadan_pocatek'));
        }
        if (trim($_GET['konec']) == '') {
            throw new Exception(\Ron\ronText('Chyba_neni_zadan_konec'));
        }

        $pocatek = DateUtils::timeStringToTimestamp($_GET['pocatek'], 0);   // $datum);
        $konec = DateUtils::timeStringToTimestamp($_GET['konec'], 0, true); // $datum);

        $preruseniOd = (isset($_GET['preruseniOd'])  &&  trim($_GET['preruseniOd']) != '')
            ? DateUtils::timeStringToTimestamp($_GET['preruseniOd'], 0)
            : null;

        $preruseniDo = (isset($_GET['preruseniDo'])  &&  trim($_GET['preruseniDo']) != '')
            ? DateUtils::timeStringToTimestamp($_GET['preruseniDo'], 0, true)
            : null;

//        $smena = Smena::nactiSmenu($this ->database, $_GET['kod']);
//        $delka = $smena
//            ? (int)($smena ->vypocitejDelkuSmeny($this ->database, $pocatek, $konec, $preruseniOd, $preruseniDo) / 60)
//            : 0;

        $delkaStr = DateUtils::minutesToString((int)( Smena::vratDelkuSmeny($this ->database, $pocatek, $konec, $_GET['kod'], $preruseniOd, $preruseniDo)  / 60));
        $outputObject->addText('delka', $delkaStr); // DateUtils::minutesToString($delka)
    }



    /**
     * Prepocita delku nebo konec smeny podle nastaveni vkladani prestavek
     * @param JsonOutputObject $outputObject
     * @throws Exception
     */
    public function prepocetKonceSmeny($outputObject)
    {
        if (trim($_GET['pocatek']) == '') {
            throw new Exception(\Ron\ronText('Chyba_neni_zadan_pocatek'));
        }
        if (trim($_GET['delka']) == '') {
            throw new Exception(\Ron\ronText('Chyba_neni_zadana_delka'));
        }

        $pocatek = DateUtils::timeStringToTimestamp($_GET['pocatek'], 0);
        $delka = DateUtils::timeStringToTimestamp($_GET['delka'], 0, true);

        $konecStr = DateUtils::minutesToString((int)( Smena::vratKonecSmeny($this->database, $pocatek, $delka, $_GET['kod']) / 60));

        if ($konecStr !== false) {
            $outputObject->addText('konec', $konecStr);
        }
    }


    /**
     * @param string $obdobiStr
     * @return Obdobi
     * @throws Exception
     */
    private function vratObdobi($obdobiStr)
    {
        $datumObdobi = DateUtils::urlStringToDate($obdobiStr);
        if (!$datumObdobi) {
            throw new Exception( 'Chyba - spatne zadane obdobi (' . $obdobiStr . ')!');
        }

        $obdobi = new Obdobi();
        if (!$obdobi->nactiObdobiProDen($this ->database, 'ADS', $datumObdobi)) {
            throw new Exception('Chyba - nelze nacist zadane obdobi!');
        };

        return $obdobi;
    }


    /**
     * @param int $idPP
     * @return PracovniPomer
     * @throws Exception
     */
    private function vratPP($idPP)
    {
        return PracovniPomer::nactiPP($this ->database, (int)$idPP);
    }

    /**
     * Hromadne nacte poznamky k dochazce
     * @param JsonOutputObject $outputObject
     * @throws Exception
     */
    private function nactiPoznamkyDochazky($outputObject)
    {
        $ppArray = (isset($_POST['idpp']) && $_POST['idpp'] != '')
            ? explode(';', $_POST['idpp'])
            : null;

        if (!$ppArray || !count($ppArray)) {
            // neni co nacitat, nejsou zadane PP
            return;
        }

        $obdobi = $this ->vratObdobi($_POST['obdobi']);

        Prava::nastavObdobiProPrava($obdobi->zacatek, $obdobi->konec);

        $prava = [Prava::vratPrava('kapacity'), Prava::vratPrava('kapacity_edit')];
        $jsonPoznamky = DenniPoznamkaDochazky::nactiPoznamkyDochazkyJSON($this ->database, $obdobi->zacatek, $obdobi->konec, $ppArray, $prava);
        $outputObject ->addData('poznamky', $jsonPoznamky);
    }


    /**
     * Hromadne nacte vysledek kontrol planovani
     * @param JsonOutputObject $outputObject
     * @throws Exception
     */
    private function nactiKontroly($outputObject)
    {
        $ppArray = (isset($_POST['idpp']) && $_POST['idpp'] != '')
            ? explode(';', $_POST['idpp'])
            : null;

        if ($ppArray && !count($ppArray)) {
            $ppArray = null;
        }

        $datum = $_GET['datum'] ?? ($_POST['datum'] ?? null);

        if ($datum) {
            $obdobi = $this ->vratObdobi($datum);
            $datum = DateUtils::urlStringToDate($datum);
        } else {
            $obdobi = $this ->vratObdobi($_GET['obdobi'] ?? $_POST['obdobi']);
        }

        Prava::nastavObdobiProPrava($obdobi->zacatek, $obdobi->konec);

        $rozdeleni = StringUtils::fromUTF8($_POST['rozdeleni']);
        $typRozdeleni = $rozdeleni ? KapacitySettings::typRozdeleniProVyber() : null;

        //$prava = [Prava::vratPrava('kapacity'), Prava::vratPrava('kapacity_edit')];
        $jsonKontroly = VysledekKontrolyPlanovani::nactiKontrolyJSON($this ->database,
            $datum ?: $obdobi->zacatek, $datum ?: $obdobi->konec, $ppArray, $typRozdeleni, $rozdeleni);
        $outputObject ->addData('kontroly', $jsonKontroly);
    }

    /**
     * Hromadne nacte dokumenty k dochazce
     * @param JsonOutputObject $outputObject
     * @throws Exception
     */
    private function nactiDokumentyDochazky($outputObject)
    {
        $ppArray = (isset($_POST['idpp']) && $_POST['idpp'] != '')
            ? explode(';', $_POST['idpp'])
            : null;

        if (!$ppArray || !count($ppArray)) {
            // neni co nacitat, nejsou zadane PP
            return;
        }

        $obdobi = $this->vratObdobi($_POST['obdobi']);

        Prava::nastavObdobiProPrava($obdobi->zacatek, $obdobi->konec);

        $this ->nactiDokumentyDochazkyZDB($obdobi->zacatek, $obdobi->konec, $ppArray, $outputObject);
    }


    /**
     * Hromadne nacte sluzebni cesty
     * @param JsonOutputObject $outputObject
     * @throws Exception
     */
    private function nactiSluzebniCesty($outputObject)
    {
        $ppArray = (isset($_POST['idpp']) && $_POST['idpp'] != '')
            ? explode(';', $_POST['idpp'])
            : null;

        if (!$ppArray || !count($ppArray)) {
            // neni co nacitat, nejsou zadane PP
            return;
        }

        $obdobi = $this ->vratObdobi($_POST['obdobi']);

        Prava::nastavObdobiProPrava($obdobi->zacatek, $obdobi->konec);

        $this ->nactiSluzebniCestyZDB($obdobi->zacatek, $obdobi->konec, $ppArray, $outputObject);
    }


    /**
     * Vlozi nahrany dokument do databaze
     * @throws Exception
     */
    private function uploadFile()
    {
        $idPP = (int)$_POST['idpp'];
        $pp = $this ->vratPP($idPP);

        Csrf::validateCSRFPostParam('pk_' . $pp ->idPP);

        $den = DateUtils::dateStringToTimestamp($_POST['den']);
        if (!$den) {
            throw new Exception( \Ron\ronText('Bad_date_format') );
        }
        $obdobi = DateUtils::firstDayOfMonth($den);

        Prava::nastavObdobiProPrava($den, $den);

        $prava = Prava::vratPrava('editacedokument');
        if (!$prava ->lzeZobrazitPP($idPP)) {
            throw new NeniPravoException();
        }

        $typ = (int)$_POST['typDokumentu'];

        NastaveniUzivatele::setUserSettingsItem($this ->database, 'pk_typdokumentu', $typ, 0, true);

        $idDokumentu = DokumentyUtils::vlozDokument($this ->database, 'upload_file', $typ, $_POST['popis'], $this, true); // z javascriptu => utf8=true

        // vlozi zaznam do vazebni tabulky
        $rec = new RonPortalDBRecord($this ->database, 'DokumentDochazky', $this);

        $rec ->addField('Id', 'int', false, 0, $idDokumentu);
        $rec ->addField('PocatekObdobi', 'date', false, 0, $obdobi);
        $rec ->addField('Aplikace', '', false, 10, 'ADS');
        $rec ->addField('RC', '', false, 30, $pp ->rc);
        $rec ->addField('OsobniCislo', '', false, 30, $pp ->osobniCislo);
        $rec ->addField('DatumNastupu', 'date', false, 0, $pp ->datumNastupu);
        $rec ->addField('Den', 'date', false, 0, $den);

        $rec ->insertRecord();
    }


    /**
     * @throws Exception
     */
    private function deleteDokument()
    {
        $idPP = (int)$_POST['idpp'];
        $pp = $this ->vratPP($idPP);

        Csrf::validateCSRFPostParam('pk_' . $pp ->idPP);

        $den = DateUtils::dateStringToTimestamp($_POST['den']);
        $obdobi = DateUtils::firstDayOfMonth($den);

        Prava::nastavObdobiProPrava($den, $den);

        $prava = Prava::vratPrava('editacedokument');
        if (!$prava ->lzeZobrazitPP($idPP)) {
            throw new NeniPravoException();
        }

        if (Obdobi::jeObdobiUzavreno($this ->database, 'ADS', $den, $den)) {
            throw new Exception( \Ron\ronText('Obdobi_uzavreno') );
        }

        $params = $this ->database ->createParameters();
        DokumentyUtils::smazDokument($this ->database, (int)$_POST['id'], $this, 'DokumentDochazky',
            'RC = ' . $params->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addDateParameter($pp ->datumNastupu) .
            ' AND PocatekObdobi = ' . $params ->addDateParameter($obdobi),
            $params
        );

    }


    /**
     * Nacte seznam dokumentu pro detail (po uploadu)
     * @param $outputObject
     * @throws Exception
     */
    function refreshDokumenty($outputObject)
    {
        // zkontrolovat prava
        $pp = $this ->vratPP((int)$_GET['idpp']);

        $den = DateUtils::dateStringToTimestamp($_GET['datum']);

        Prava::nastavObdobiProPrava($den, $den);

        $this ->nactiDokumentyDochazkyZDB($den, $den, $pp ->idPP, $outputObject);
    }


    /**
     * Nacte poznamky ke dnum
     * @param JsonOutputObject $outputObject
     * @throws Exception
     */
    private function nactiPoznamkyKeDni($outputObject)
    {
        $datumObdobi = DateUtils::urlStringToDate($_GET['obdobi']);
        if (!$datumObdobi) {
            throw new Exception('Chyba - spatne zadane obdobi (' . $_GET['obdobi'] . ')!');
        }

        $typRozdeleni = isset($_GET['rozdeleni']) ? KapacitySettings::typRozdeleniProVyber() : null;
        $rozdeleni = isset($_GET['rozdeleni']) ? StringUtils::fromUTF8($_GET['rozdeleni']) : null;

        $obdobi = new Obdobi();
        if (!$obdobi->nactiObdobiProDen($this ->database, 'ADS', $datumObdobi)) {
            throw new Exception('Chyba - nelze nacist zadane obdobi!');
        };
        $poznamky = PoznamkaKeDni::nactiPoznamkyKeDni($this ->database, $obdobi ->zacatek, $obdobi ->konec, $typRozdeleni, $rozdeleni);

        $jsonPoznamky = [];

        foreach ($poznamky as $poznamka) {
            $jsonPoznamky[] = $poznamka ->getJson();
        }

        $outputObject->addData('poznamkyKeDni', $jsonPoznamky);
    }


    /**
     * Nacteni seznamu smen pouzivanych uzivatelem
     * @param JsonOutputObject $outputObject
     * @throws Exception
     */
    private function vyberSmenUzivatele($outputObject)
    {
        $seznamSmen = Smena::nactiSmenyUzivatele($this ->database, 'Smena.ZkratkaProPlanovani IS NOT NULL AND Smena.Aktivni = 1');

        $smeny = [];
        foreach ($seznamSmen as $smena) {
            $smeny[] = $smena ->getJson();
        }

        $outputObject->addData('smeny', $smeny);

//        $outputObject->addData('sirkaBunky',
//            (int)NastaveniUzivatele::getUserSettingsItem($this->database,
//                'pk_sirkabunky', RonPortalSettings::getIntValue(KapacitySettings::SIRKA_BUNKY)));
//
//        $outputObject->addData('vyskaBunky',
//            (int)NastaveniUzivatele::getUserSettingsItem($this->database,
//                'pk_vyskabunky', RonPortalSettings::getIntValue(KapacitySettings::VYSKA_BUNKY)));
    }


    /**
     * Zapise vybrane (oblibene) smeny uzivatele
     * @throws Exception
     */
    private function zapisSmenyUzivatele()
    {
        $smeny = [];
        for ($i = 0;  $i < $_POST['cnt'];  $i++) {
            $smeny[] = StringUtils::fromUTF8($_POST["smena$i"]);
        }

        // TODO: presunout do Smena::zapisSmenyUzivatele ?

        $params = $this ->database ->createParameters();
        // odstranit nevybrane smeny
        $sql =
            'DELETE FROM SmenyUzivatele ' .
            ' WHERE RC = ' . $params->addParameter(User::getRC());

        $filtrsmeny = '';
        foreach ($smeny as $kodsmeny) {
            $filtrsmeny .=
                ($filtrsmeny ? ' OR ' : '') .
                ' Smena = ' . $params->addParameter($kodsmeny);
        }

        if ($filtrsmeny) {
            $sql .= ' AND NOT (' . $filtrsmeny . ')';
        }

        try {
            $this->database->sql($sql, $params);
        } catch (DBException $e) {
            Log::addRecordToDBLog($this ->database, 'SmenyUzivatele', 'ERROR', $this, $sql . "\n\n" . $e ->dbMessage );
            throw new DBException($e, null, 'Chyba pri mazani nevybranych smen!');
        }

        Log::addRecordToDBLog($this ->database, 'SmenyUzivatele', 'DELETE', $this, $sql);

        // zapsat vybrane smeny
        if ( count($smeny) ) {
            $params = $this ->database ->createParameters();

            $sql =
                'INSERT INTO SmenyUzivatele (RC, Smena) ' .
                ' SELECT Osoba.RC, Smena.Kod ' .
                ' FROM Osoba, Smena ' .
                ' WHERE Osoba.RC = ' . $params ->addParameter(User::getRC()) .
                ' AND (';

            $prvni = true;
            foreach ($smeny as $kodsmeny) {
                if (!$prvni) $sql .= ' OR ';
                $prvni = false;
                $sql .=
                    'Kod = ' . $params->addParameter($kodsmeny);
            }
            // Pozor! Exist musi byt az za parametry, jinak to na ODBC nefunguje!
            $sql .=
                ')' .
                ' AND NOT EXISTS (' .
                '  SELECT * FROM SmenyUzivatele S2 ' .
                '  WHERE S2.RC = Osoba.RC' .
                '  AND S2.Smena = Smena.Kod ' .
                ' )';


            try {
                $this->database->sql($sql, $params);
            } catch (DBException $e) {
                Log::addRecordToDBLog($this ->database, 'SmenyUzivatele', 'ERROR', $this, $sql . "\n\n" . $e ->dbMessage );
                throw new DBException($e, null, 'Chyba pri zapisu vybranych smen!');
            }

            Log::addRecordToDBLog($this ->database, 'SmenyUzivatele', 'INSERT', $this, $sql);
        }
    }


    /**
     * Zpracuje data z dialogu s nastavenim uzivatele (vybrane smeny, velikost bunky)
     * @param JsonOutputObject $output
     * @throws Exception
     */
    private function zapisNastaveniUzivatele(JsonOutputObject $output)
    {
        $detailni = (int)($_POST['detailniZobrazeni'] ?? 0);

        if (isset($_POST['sirkaBunky'])) {
            if (trim($_POST['sirkaBunky']) === '') {
                $newValue = RonPortalSettings::getIntValue(
                    $detailni ? KapacitySettings::SIRKA_BUNKY_DETAILNI : KapacitySettings::SIRKA_BUNKY,
                    $detailni ? 75 : 35);
            } else if ((int)$_POST['sirkaBunky'] >= 16) {
                $newValue = (int)$_POST['sirkaBunky'];
            } else {
                $newValue = 16;
            }

            NastaveniUzivatele::setUserSettingsItem($this->database,
                $detailni
                    ? 'pk_sirkabunky_detailni'
                    : 'pk_sirkabunky',
                $newValue,
                RonPortalSettings::getIntValue(
                    $detailni ? KapacitySettings::SIRKA_BUNKY_DETAILNI : KapacitySettings::SIRKA_BUNKY,
                    $detailni ? 75 : 35)
            );

            $output ->addData('sirkaBunky', $newValue);
        }

        if (isset($_POST['vyskaBunky'])) {
            if (trim($_POST['vyskaBunky']) === '') {
                $newValue = RonPortalSettings::getIntValue(
                    $detailni ? KapacitySettings::VYSKA_BUNKY_DETAILNI : KapacitySettings::VYSKA_BUNKY,
                    $detailni ? 75 : 35);
            } else if ((int)$_POST['vyskaBunky'] >= 16) {
                $newValue = (int)$_POST['vyskaBunky'];
            } else {
                $newValue = 16;
            }
            NastaveniUzivatele::setUserSettingsItem($this->database,
                $detailni
                    ? 'pk_vyskabunky_detailni'
                    : 'pk_vyskabunky',
                $newValue,
                RonPortalSettings::getIntValue(
                    $detailni ? KapacitySettings::VYSKA_BUNKY_DETAILNI : KapacitySettings::VYSKA_BUNKY,
                    $detailni ? 75 : 35)
            );

            $output ->addData('vyskaBunky', $newValue);
        }

        $this ->zapisSmenyUzivatele();
    }


    /**
     * @param PracovniPomer $pp
     * @param int  $datumOd
     * @param int  $datumDo
     * @param bool $rezimUprav
     * @return array
     * @throws Exception
     */
    private function zapisSmenyProRefresh($pp, $datumOd, $datumDo, $rezimUprav)
    {
        $smeny = PlanovaniKapacit_PlanovanaSmena::nactiSmeny($this ->database, $pp ->idPP, $datumOd, $datumDo, $rezimUprav);

        $result = [];
        foreach ($smeny as $smena) {
            $result[] = $smena ->getJSON(false);
        }

        return $result;
    }


    /**
     * Zapise naplanovane smeny
     * @param JsonOutputObject $outputObject
     * @throws Exception
     */
    private function zapisPlanovanouSmenu($outputObject)
    {
        $pp = PracovniPomer::nactiPP($this ->database, $_POST['idpp']);

        Csrf::validateCSRFPostParam('pk_' . $pp ->idPP);

        $datumOd = DateUtils::dateStringToTimestamp($_POST['datumod']);
        $datumDo = isset($_POST['datumdo'])
            ? DateUtils::dateStringToTimestamp($_POST['datumdo'])
            : $datumOd;

        if (!$datumOd || !$datumDo) {
            throw new Exception('Chyba - spatne zadane datum!');
        }

        $kodSmeny = StringUtils::fromUTF8($_POST['smena']);

        $rezimUprav =
            RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV)
            &&  isset($_POST['rezimuprav'])
            &&  $_POST['rezimuprav'];

        $rozdeleni = isset($_POST['rozdeleni'])
            ? StringUtils::fromUTF8($_POST['rozdeleni'])
            : null;

        KapacityUtils::zapisPlanovanouSmenu($this ->database, $pp, $datumOd, $datumDo, $kodSmeny, $rezimUprav, $rozdeleni, $this);

        $outputObject ->addData('planovaneSmeny', $this ->zapisSmenyProRefresh($pp, $datumOd, $datumDo, $rezimUprav));
    }


    /**
     * @throws Exception
     */
    private function naplanujPodleKalendare()
    {
        $denOd = DateUtils::dateStringToTimestamp($_POST['datumod']);
        $denDo = DateUtils::dateStringToTimestamp($_POST['datumdo']);

        $rezimUprav = RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV)  &&  isset($_POST['rezimuprav'])  &&  $_POST['rezimuprav'];

        $rozdeleni = isset($_POST['rozdeleni'])
            ? StringUtils::fromUTF8($_POST['rozdeleni'])
            : null;

        $idPP = (int)$_POST['idpp'];
        Csrf::validateCSRFPostParam('pk_' . $idPP);

        if (RonPortalSettings::getIntValue('kapacity_smenypodlerozdeleni')) {
            KapacityUtils::naplanujPodleKalendareRozdeleni($this ->database, $idPP, $denOd, $denDo, $rezimUprav, $rozdeleni, $this);
        } else {
            KapacityUtils::naplanujPodleKalendare($this ->database, $idPP, $denOd, $denDo, $rezimUprav, $rozdeleni, $this);
        }
    }


    /**
     * @throws NeniPravoException
     * @throws DBException
     */
    private function zapisHarmonogram()
    {
        $pp = $this ->nactiPP($_POST['idpp']);

        Csrf::validateCSRFPostParam('pk_' . $pp ->idPP);

        $denOd = DateUtils::dateStringToTimestamp($_POST['datumod']);
        $denDo = DateUtils::dateStringToTimestamp($_POST['datumdo']);

        Prava::nastavObdobiProPrava($denOd, $denDo);

        $rezimUprav = RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV)  &&  isset($_POST['rezimuprav'])  &&  $_POST['rezimuprav'];
        $rozdeleni = isset($_POST['rozdeleni'])
            ? StringUtils::fromUTF8($_POST['rozdeleni'])
            : null;

        $kod = StringUtils::fromUTF8($_POST['harmonogram']);
        $harmonogram = Harmonogram::nactiHarmonogram($this ->database, $kod);
        if (!$harmonogram) {
            throw new Exception('Harmonogram s kodem ' . $kod . ' nebyl nalezen!');
        }
        $harmonogram ->nactiDny($this ->database);

        if ($harmonogram ->podleDneVTydnu) {
            // harmonogram zacina vzdy v pondeli - $index dne nastavi podle prvniho datumu
            $index = date('w', $denOd);
            if (!$index) {
                $index = 7;  // nedele nema byt 0 jako v PHP, ale 7
            }
        } else {
            $index = 1;
        }

        $datum = $denOd;
        $mesic = (int)date('n', $datum);

        try {
            while ($datum <= $denDo) {
                if (isset($harmonogram->dny[$index])) {
                    $rec = new PlanovaniKapacitRecord($this->database, $pp, $this);
                    $rec->logInfoMessage = StringUtils::fromUTF8('Naplanovani smen podle harmonogramu');
                    $rec->fHarmonogram->setNewValue($kod);
                    $rec->zapisSmenu($datum, $harmonogram->dny[$index]->smena, $rezimUprav, $rozdeleni);
                } else {
                    // smazat smenu? - neni treba, v kalendari zaznam je s prazdnou smenou
                }

                $datum = DateUtils::incDayStart($datum);
                $index++;
                if ($index > $harmonogram->pocetDnuCyklu) $index = 1;
            }
        } catch (NeniPravoException $e) {
            // pokud nastane vyjimka v nasledujicim mesici, tak ji spolkne (uzivatel data nevidi, takze by ho to jen zmatlo)
            if ((int)date('n', $datum) == $mesic) {
                throw $e;
            }
        }
    }


    /**
     * Nastavi rozdeleni (prerazeni na jine stredisko nebo rozdeleni 2)
     * @param bool $rozdeleni2  Priznak, jestli se ma nastavit rozdeleni2
     * @throws Exception
     */
    private function nastavRozdeleni($rozdeleni2 = false)
    {
        $denOd = DateUtils::dateStringToTimestamp($_POST['datumod']);
        $denDo = DateUtils::dateStringToTimestamp($_POST['datumdo']);

        if (!$denOd || !$denDo) {
            throw new Exception('Chyba - spatne zadane datum!');
        }

        if (Obdobi::jeObdobiUzavreno($this ->database, 'ADS', $denOd, $denDo)) {
            throw new Exception( \Ron\ronText('Obdobi_uzavreno') );
        }

        $pp = $this ->nactiPP($_POST['idpp']);
        $rozdeleni = StringUtils::fromUTF8($_POST['rozdeleni']);

        Csrf::validateCSRFPostParam('pk_' . $pp ->idPP);

        Prava::nastavObdobiProPrava($denOd, $denDo);

        $rezimUprav = RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV) && isset($_POST['rezimuprav']) && $_POST['rezimuprav'];

        $den = $denOd;
        while ($den <= $denDo) {
            $rec = new PlanovaniKapacitRecord($this->database, $pp, $this);
            if ($rozdeleni2) {
                $rec->zapisRozdeleni2($den, $rozdeleni, $rezimUprav);
            } else {
                $rec->zapisRozdeleni1($den, $rozdeleni, $rezimUprav);
            }
            $den = DateUtils::incDayStart($den, 1);
        }

        // Pokud uzivatel zaskrtnul "preplanovat smeny ...", tak je preplanuje
        if (!$rozdeleni2  &&  RonPortalSettings::getIntValue('kapacity_smenypodlerozdeleni')  &&  (int)$_POST['preplanovatSmeny']) {
            $prava = Prava::vratPrava('kapacity_edit');
            if ($prava ->jePravoNaRozdeleni(RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI), $rozdeleni, $denOd, $denDo)) {
                KapacityUtils::naplanujPodleKalendareRozdeleni($this ->database, (int)$_POST['idpp'], $denOd, $denDo, $rezimUprav, $rozdeleni, $this);
            }
        }
    }

    /**
     * Zapise nebo zrusi potvrzeni naplanovanych smen
     * @throws Exception
     */
    private function potvrditSmeny()
    {
        $pp = $this ->nactiPP($_POST['idpp']);

        Csrf::validateCSRFPostParam('pk_' . $pp ->idPP);

        $denOd = DateUtils::dateStringToTimestamp($_POST['datumod']);
        $denDo = DateUtils::dateStringToTimestamp($_POST['datumdo']);

        Prava::nastavObdobiProPrava($denOd, $denDo);

        $rezimUprav = RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV) && isset($_POST['rezimuprav']) && $_POST['rezimuprav'];

        $rec = new PlanovaniKapacitRecord($this->database, $pp, $this);
        $rec->nastavPotvrzeni($denOd, $denDo, $rezimUprav, (int)$_POST['hodnota']);
    }


    /**
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @param string $kodOperace
     * @param bool $rezimUprav
     * @throws Exception
     */
    private function zapisAbsenciDoDB($pp, $datumOd, $datumDo, $kodOperace, $rezimUprav)
    {
        // udaje zadane uzivatelem (mohou byt zadane napr. XML formularem)
        $poznamka = (isset($_POST['poznamka'])  &&  $_POST['poznamka'] != '') ? StringUtils::fromUTF8($_POST['poznamka'])  :  '';
        $delka = isset($_POST['delka'])  ?  (int)$_POST['delka']  :  0;
        $casOd = isset($_POST['casod']) ? DateUtils::timeStringToTimestamp($_POST['casod']) : null;
        $casDo = isset($_POST['casdo']) ? DateUtils::timeStringToTimestamp($_POST['casdo']) : null;

        if ($delka == 3  ||  $delka == 4) {

        }

        // TODO: presunout kontrolu do AbsenceDBRecord? (aby nesla naplanovat ani z Planovani absenci)
        // TODO: nacist zaznamy rozdeleni
        KapacityUtils::zkontrolujSchvaleni($this->database, $pp, $datumOd, $rezimUprav);

        // Operace ktere nejsou pocatek intervalu se mohou prekryvat s jinou operaci
        $operace = OperaceDochazky::nactiOperaci($this ->database, $kodOperace);

        $newRec = new AbsenceDBRecord($this ->database, $pp);

        if ( $poznamka ) {
            $newRec ->fPoznamka ->value = $poznamka;
        }

        $newRec ->fDelka ->value = $delka;
        $newRec ->fCasOd ->value = $casOd;
        $newRec ->fCasDo ->value = $casDo;

        if ( isset($_POST['kodtypurozdeleni'])  &&  (int)$_POST['kodtypurozdeleni'] &&  isset($_POST['rozdeleni'])) {
            $typRozdeleni = (int)$_POST['kodtypurozdeleni'];
            $rozdeleni = StringUtils::fromUTF8($_POST['rozdeleni']);

            $newRec ->fKodTypuRozdeleni ->value = $typRozdeleni;
            $newRec ->fRozdeleni ->value = $rozdeleni;
        }

        $newRec ->nastavSchvaleniPodleOperace($operace);

        $filtr = null;
        $filtrParams = null;
        if (!$operace ->pocatekIntervalu) {
            $filtrParams = $this ->database ->createParameters();
            // neni pocatek intervalu - prekryvani kontroluje jen se stejnou operaci - od 10/2014
            $filtr = 'OperaceDochazky = ' . $filtrParams->addParameter($kodOperace);
        }

        $rec = new AbsenceDBRecord($this ->database, $pp);
        $rec ->loadAbsenceVObdobi($datumOd, $datumDo, $filtr, $filtrParams);

        //$zapsano = false; // priznak, jestli uz byla absence zapsana do databaze

        while ($rec ->next()) {
            $zkratit = true;

            $operace2 = null;

            if (isset($rec->fOperace->value)) { // &&  $rec->fOperace->value != $operace->kod) { - druha podminka vypnuta 12.3.2018 -
                // zkontroluje jestli se nalezena absence muze prekryvat (= neni pocatek intervalu) - od 10/2014
                // v tom pripade ji neni potreba mazat
                // Pozor - muze byt i null (napr. v rezimu uprav, pokud byla smazana) - proto je nutny test isset

                $operace2 = OperaceDochazky::nactiOperaci($this ->database, $rec->fOperace->originalValue);

                if (!$operace2->pocatekIntervalu) {
                    $zkratit = false;
                }
            }

            if ($zkratit  &&  $operace2) {
                if ( !$rec ->jePravoEditovatAbsenci($pp, $operace2) ) {
                    if (!$newRec ->fSchvaleni ->value  &&  RonPortalSettings::getIntValue('prekryvaniabsenci')) {
                        // nova absence neni schvalena - povoli ji naplanovat, protoze se muze prekryvat s jinou
                        // kontrola prekryvani se bude resit az pri schvaleni
                        $zkratit = false;
                    } else {
                        // uzivatel nema pravo planovat (takze ani mazat) nalezenou absenci
                        throw new Exception(\Ron\ronText('Prekryvani_absenci'));
                    }
                }

                if ($zkratit) {
                    $rec->zkratAbsenci($datumOd, $datumDo, $rezimUprav, $delka, $casOd, $casDo);
                }
            }

        }

        // vlozi novou absenci
        $newRec ->vlozNovouAbsenci($datumOd, $datumDo, $operace, $rezimUprav);
    }


    /**
     * Zapise absenci dle parametru v POST a vraci seznam absenci v danem obdobi
     * @param JsonOutputObject $outputObject
     * @throws NeniPravoException
     * @throws Exception
     */
    private function zapisPlanovanouAbsenci($outputObject)
    {
        $pp = $this ->nactiPP($_POST['idpp']);

        Csrf::validateCSRFPostParam('pk_' . $pp ->idPP);

        $datumOd = DateUtils::dateStringToTimestamp($_POST['datumod']);
        $datumDo = DateUtils::dateStringToTimestamp($_POST['datumdo']);

        $kodOperace = StringUtils::fromUTF8($_POST['absence']);

        if (!AbsenceDBRecord::zkontrolujPravoNaAbsenci($this->database, $pp->idPP, $datumOd, $datumDo, $kodOperace)) {
            throw new NeniPravoException();
        }

        $rezimUprav = isset($_POST['rezimuprav']) ? (int)$_POST['rezimuprav'] : 0;

        // kontrola schvaleni dochazky a uzavreneho obdobi
        AbsenceDBRecord::zkontrolujObdobiAbsence($this ->database, $datumOd, $datumDo, $pp);

        // kontrola schvaleni planu kapacit - kvuli pujcovani nestaci zkontrolovat prvni den!
        KapacityUtils::zkontrolujSchvaleniProObdobi($this ->database, $pp, $datumOd, $datumDo, $rezimUprav);

        $this->zapisAbsenciDoDB($pp, $datumOd, $datumDo, $kodOperace, $rezimUprav);

        $seznamAbsenci = PlanovaniKapacit_PlanovanaAbsence::nactiAbsence($this ->database,
            $pp ->idPP, $datumOd, $datumDo, false, $rezimUprav);
        $absenceJson = [];
        foreach ($seznamAbsenci as $absence) {
            $absenceJson[] = $absence ->getJSON();
        }
        $outputObject ->addData('idpp', $pp ->idPP);
        $outputObject ->addData('absence', $absenceJson);
    }


    /**
     * Zkrati, prerusi nebo smaze absence v zadanem obdobi. Vraci pocet smazanych nebo editovanych zaznamu, nebo false v pripade chyby.
     * @param PracovniPomer $pp
     * @param int  $datumOd
     * @param int  $datumDo
     * @param bool $rezimUprav
     * @return int|false
     * @throws Exception
     */
    private function smazPlanovaneAbsence($pp, $datumOd, $datumDo, $rezimUprav)
    {
        $rec = new AbsenceDBRecord($this ->database, $pp);

        $rec->loadAbsenceVObdobi($datumOd, $datumDo);

        $result = true;
        $cnt = 0;

        $errormsg_operace = '';
        $errors_cnt = 0;

        while ($rec->next()) {
            $kodOperace = $rec ->fOperace ->originalValue;
            if (!$rezimUprav  &&  !isset($kodOperace)) {
                // pokud je operace null, tak v rezimu planovani zkontroluje i puvodni operaci - jinak by se nesmazala z databaze
                $kodOperace = $rec ->fOperacePuvodni ->originalValue;
            }

            if ( $kodOperace ) { // muze byt null napr. v rezimu uprav pokud uz byla smazana
                $operace = OperaceDochazky::nactiOperaci($this->database, $kodOperace);

                if ($rec->jePravoEditovatAbsenci($pp, $operace, true)) { // delete = true
                    // TODO: kontrolovat jen mazanou cast
                    AbsenceDBRecord::zkontrolujObdobiAbsence($this ->database, $datumOd, $datumDo, $pp);

                    if (!$rec->zkratAbsenci($datumOd, $datumDo, $rezimUprav)) {
                        $result = false;
                    }
                    else {
                        $cnt++;
                    }

                } else {
                    // nema pravo
                    $errormsg_operace = sprintf(\Ron\ronText('Neni_pravo_smazat_udalost'), $operace->popis);
                    $errors_cnt++;
                    $result = false;
                }
            }
        }

        if ($errors_cnt === 1) {
            throw new Exception($errormsg_operace);
        }
        else if ($errors_cnt) {
            throw new Exception(\Ron\ronText('Nelze_smazat_vsechny_bunky'));
        }

        return $result ? $cnt : false;
    }


    /**
     * @throws Exception
     */
    private function smazDataPlanovani()
    {
        $pp = $this ->nactiPP($_POST['idpp']);

        Csrf::validateCSRFPostParam('pk_' . $pp ->idPP);

        $datumOd = DateUtils::dateStringToTimestamp($_POST['datumod']);
        $datumDo = DateUtils::dateStringToTimestamp($_POST['datumdo']);
        if (!$datumOd || !$datumDo) {
            throw new Exception('Chyba pri mazani dat planovani - spatne zadane datum!');
        }

        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        $rezimUprav = RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV) && isset($_POST['rezimuprav']) && $_POST['rezimuprav'];

        KapacityUtils::zkontrolujSchvaleni($this ->database, $pp, $datumOd, $rezimUprav);

        if (isset($_POST['smazatRozdeleni']) && $_POST['smazatRozdeleni']) {
            if (!RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI)) {
                throw new Exception(StringUtils::fromUTF8('Chyba - nelze smazat rozdeleni - prerazovani zamestnancu neni povoleno!'));
            }

            $rec = new PlanovaniKapacitRecord($this->database, $pp, $this);
            $rec->smazRozdeleni($datumOd, $datumDo, $rezimUprav);

        } else {
            // smazani naplanovanych smen a absenci
            $rec = new PlanovaniKapacitRecord($this->database, $pp, $this);
            $rec->smazNaplanovaneSmeny($datumOd, $datumDo, $rezimUprav, $_POST['rozdeleni'] ?? null);

            $this->smazPlanovaneAbsence($pp, $datumOd, $datumDo, $rezimUprav);
        }
    }


    /**
     * Smaze jen jednu konkretni absenci (v detailu planovani)
     * @param PracovniPomer $pp
     * @param int  $datum
     * @param int  $poradi
     * @param bool $rezimUprav
     * @throws Exception
     */
    private function smazPlanovanouAbsenci($pp, $datum, $poradi, $rezimUprav)
    {
        $rec = new AbsenceDBRecord($this ->database, $pp);

        $params = $this ->database ->createParameters();
        $filtr =
            'DatumOd = ' . $params ->addParameter($this ->database ->dateToParam($datum)) .
            ' AND Poradi = ' . $params ->addParameter((int)$poradi);

        if ( !$rec ->loadRecord($filtr, $params) ) {
            throw new Exception(StringUtils::fromUTF8('Chyba zaznam absence nebyl nalezen!'));
        }

        if (!AbsenceDBRecord::jePravoNaSmazani($this ->database, $pp ->idPP, $datum, $rec->fOperace->value, $rec ->fSchvaleni ->value)) {
            // uzivatel nema pravo mazat nalezenou absenci
            $operace = OperaceDochazky::nactiOperaci($this ->database, $rec ->fOperace ->value);

            throw new Exception( sprintf( \Ron\ronText('Neni_pravo_smazat_udalost'), $operace ->popis) );
        }

        // kontrola obdobi a schvaleni
        AbsenceDBRecord::zkontrolujObdobiAbsence($this->database, $rec ->fDatumOd->value, $rec ->fDatumDo->value, $pp);

        if ($rezimUprav) {
            if (isset($rec->fOperacePuvodni->value)) {
                // rezim uprav - jen vynuluje absenci, puvodni absence zustava
                $rec->fOperace->setNewValue(null);
                $rec->updateRecord($filtr, $params);
                return;
            }
        }

        $rec ->deleteRecord($filtr, $params);
    }


    /**
     * Smazani jedne konkretni absence
     * @throws Exception
     */
    private function smazAbsenci()
    {
        $pp = $this ->nactiPP($_POST['idpp']);

        Csrf::validateCSRFPostParam('pk_' . $pp ->idPP);

        $datumOd = DateUtils::dateStringToTimestamp($_POST['datumod']);
        $poradi = (int)$_POST['poradi'];

        $rezimUprav = RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV) && isset($_POST['rezimuprav']) && $_POST['rezimuprav'];

        $this->smazPlanovanouAbsenci($pp, $datumOd, $poradi, $rezimUprav);
    }


    /**
     * Nacte data pro zobrazeni detailu (smena, absence, poznamka)
     * @param JsonOutputObject $outputObject
     * @param int  $idpp
     * @param int  $datum
     * @param bool $rezimUprav
     * @throws Exception
     */
    private function nactiDenPlanovani($outputObject, $idpp, $datum, $rezimUprav)
    {
        Prava::nastavObdobiProPrava($datum, $datum);

        $pp = $this ->nactiPP($idpp);

        $outputObject->addData('den', date('j.n.Y', $datum));
        $outputObject->addData('idPP', $idpp);

        $poznamka = DenniPoznamkaDochazky::vratPoznamkuDochazky($this ->database, $datum, $pp ->idPP);

        if (isset($poznamka)) {
            $outputObject->addText('poznamkaDochazky', $poznamka);
        }

        $smena = PlanovaniKapacit_PlanovanaSmena::nactiDataProDetail($this->database, $datum, $pp->rc, $pp->osobniCislo, $pp->datumNastupu, $rezimUprav);
        $obj = $smena->getJSON(true); // detail = true
        $outputObject->addData('planovanaSmena', $obj);

        $jsonAbsence = [];

        $seznamAbsenci = PlanovaniKapacit_PlanovanaAbsence::nactiAbsence($this->database,
            $pp->idPP, $datum, $datum, true, $rezimUprav); // detail = true

        foreach ($seznamAbsenci as $absence) {
            $absence->lzeEditovat = AbsenceDBRecord::zkontrolujPravoNaAbsenci($this->database, $pp->idPP, $absence->datumOd, $absence->datumDo, $absence->kod);
            $absence->lzeSmazat = $absence->lzeEditovat;

            if (RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI) && !$absence->autoSchvalovat && $absence->lzeEditovat) {
                // pokud je schvalena, tak zakaze editaci (pokud nema uzivatel prava na schvalovani)
                if ($absence->schvaleni) {
                    $absence->lzeEditovat = AbsenceDBRecord::zkontrolujPravoNaAbsenci($this->database, $pp->idPP, $absence->datumOd, $absence->datumDo, $absence->kod, 'absence_schval');

                    if (!$absence->lzeEditovat) {
                        // kontrola, jestli ma uzivatel povoleno smazat schvalene absence
                        $absence->lzeSmazat = AbsenceDBRecord::zkontrolujPravoNaAbsenci($this->database, $pp->idPP, $absence->datumOd, $absence->datumDo, $absence->kod, 'absence_delete');
                    } else {
                        $absence->lzeSmazat = true;
                    }
                }

            }

            $jsonAbsence[] = $absence->getJSON(true);
        }

        $outputObject->addData('planovaneAbsence', $jsonAbsence);

        $this ->nactiDokumentyDochazkyZDB($datum, $datum, $pp ->idPP, $outputObject);

        $pravoDokumenty = Prava::vratPrava('editacedokument');
        $outputObject ->addData('pravoVkladatDokumenty', $pravoDokumenty ->lzeZobrazitPP($idpp, $datum, $datum) ? 1 : 0);
    }


    /**
     * Nacte data pro zobrazeni detailu (smena, absence, poznamka)
     * @param JsonOutputObject $outputObject
     * @throws Exception
     */
    private function denPlanovani($outputObject)
    {
        $idpp = (int)$_GET['idpp'];
        $datum = DateUtils::dateStringToTimestamp($_GET['datum']);
        $rezimUprav = (int)$_GET['rezimuprav'];

        $this ->nactiDenPlanovani($outputObject, $idpp, $datum, $rezimUprav);
    }


    /**
     * Nacte seznam rozdeleni 2
     * @param JsonOutputObject $outputObject
     * @throws Exception
     */
    private function seznamRozdeleni2($outputObject)
    {
        $params = $this ->database ->createParameters();

        $datum =
            isset($_GET['datum'])
                ? DateUtils::urlStringToDate($_GET['datum'])
                : (isset($_GET['obdobi']) ? DateUtils::urlStringToDate($_GET['obdobi']) : time());

        $data = [
            ['kod' => '', 'popis' => '']
        ];

        if (RonPortalSettings::getIntValue('kapacity_vyberrozdeleni2')) {
            // 1=pouze prirazene, 2=vcetne podrizenych, 3=jen podrizene
            if (!isset($_GET['idpp']) || !(int)$_GET['idpp']) {
                throw new Exception('Chyba pri nacitani rozdeleni 2 - neni zadan Id pracovniho pomeru!');
            }

            $seznamRozdeleni = Rozdeleni::loadRozdeleniOsoby($this ->database,
                (int)$_GET['idpp'], (int)RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2),
                $datum, $datum, RonPortalSettings::getIntValue('kapacity_vyberrozdeleni2') - 1);

            foreach ($seznamRozdeleni as $rozdeleni) {
                $data[] = ['kod' => StringUtils::jsonText($rozdeleni ->kod), 'popis' => StringUtils::jsonText($rozdeleni ->popis)];
            }

        } else {
            // vsechna rozdeleni
            $sql =
                'SELECT Kod, Popis FROM Rozdeleni' .
                ' WHERE KodTypuRozdeleni = ' . $params->addParameter((int)RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2)) .
                '   AND (DatumZruseni IS NULL  OR  DatumZruseni >= ' . $params->addParameter($this->database->dateToParam($datum)) . ')' .
                ' ORDER BY Popis';

            $cursor = $this ->database ->getSqlResult($sql, $params);
            while ($row = $cursor ->fetch()) {
                $data[] = ['kod' => StringUtils::jsonText($row[0]), 'popis' => StringUtils::jsonText($row[1])];
            }
            $cursor ->close();
        }


        $outputObject ->addData('rozdeleni2', $data);
    }


    /**
     * @param PracovniPomer $pp
     * @param integer $indexhtml    index absence v POST
     * @param bool    $rezimUprav
     * @return bool
     * @throws Exception
     */
    private function zapisDetailAbsence($pp, $indexhtml, $rezimUprav)
    {
        // TODO: vyresit rezim uprav? Napr. po editaci delky nebo casu se zmena projevi i v puvodnim planu
        // Reseni: absenci zrusit nastavenim OperaceDochazky = null a vlozit novou absenci s jinym nastavenim
        if (!isset($_POST["absence{$indexhtml}_poradi"])) {
            return false; // neni zadana absence pro editaci
        }

        $poradi = (int)$_POST["absence{$indexhtml}_poradi"];

        $rec = new AbsenceDBRecord($this->database, $pp);

        // datum od
        if (isset($_POST["absence{$indexhtml}_datumod"])) {
            $newDatum = DateUtils::dateStringToTimestamp($_POST["absence{$indexhtml}_datumod"]);
            if (!$newDatum) {
                throw new RonException(\Ron\ronText('Bad_date_format'), 'datumod');
            }
            $rec ->fDatumOd ->setNewValue($newDatum);
        }

        // datum do
        if (isset($_POST["absence{$indexhtml}_datumdo"])) {
            $newDatum = DateUtils::dateStringToTimestamp($_POST["absence{$indexhtml}_datumdo"]);
            if (!$newDatum) {
                throw new RonException(\Ron\ronText('Bad_date_format'), 'datumdo');
            }
            $rec ->fDatumDo ->setNewValue($newDatum);
        }

        // delka absence a pripadne cas od - do
        if (isset($_POST["absence{$indexhtml}_delka"])) {
            // zmena delky absence (celodenni/1.pulka smeny a podobne) - nutno prekontrolovat prekryvani (probehne dale)!
            $rec ->fDelka ->setNewValue($_POST["absence{$indexhtml}_delka"]);
        }

        if (isset($_POST["absence{$indexhtml}_casod"])) {
            $casod = DateUtils::timeStringToTimestamp($_POST["absence{$indexhtml}_casod"], 0);
            $rec ->fCasOd ->setNewValue($casod);
        }


        if (isset($_POST["absence{$indexhtml}_casdo"])) {
            $casdo = DateUtils::timeStringToTimestamp($_POST["absence{$indexhtml}_casdo"], 0);
            $rec ->fCasDo ->setNewValue($casdo);
        }


        if (isset($_POST["absence{$indexhtml}_poznamka"])) {
            $rec ->fPoznamka ->setNewValue(StringUtils::fromUTF8($_POST["absence{$indexhtml}_poznamka"]));
        }

        $datumOdPuvodni = DateUtils::dateStringToTimestamp($_POST["absence{$indexhtml}_datumod_puvodni"]);

        $params = $this ->database ->createParameters();
        $filtr =
            'DatumOd = ' . $params ->addParameter($this ->database ->dateToParam($datumOdPuvodni)) .  // datum a operace podle puvodnich hodnot !
            ' AND Poradi = ' . $params ->addParameter($poradi);

        if (!$rec->loadRecord($filtr, $params)) {
            throw new Exception('Vlozeni noveho zaznamu absence neni implementovano!');
        }

        if (!$rec->isChanged()) {
            return true;
        }

        $kodOperace = (RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV) && !$rezimUprav)
            ? $rec ->fOperacePuvodni ->value
            : $rec ->fOperace ->value;

        // kontrola prav
        if (!AbsenceDBRecord::zkontrolujPravoNaAbsenci($this->database, $pp->idPP, $rec ->fDatumOd ->value, $rec ->fDatumDo ->value, $kodOperace)) {
            throw new NeniPravoException();
        }

        // pokud uz je schvalena, tak take kontrola prava na schvaleni
        if ($rec ->fSchvaleni ->value  &&  RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI)) {
            $operace = OperaceDochazky::nactiOperaci($this->database, $kodOperace);

            if (!$operace->autoSchvalovat &&
                !AbsenceDBRecord::zkontrolujPravoNaAbsenci($this->database, $pp->idPP, $rec ->fDatumOd ->value, $rec ->fDatumDo->value, $kodOperace, $typPrava = 'absence_schval')
            ) {
                throw new Exception( \Ron\ronText('Nemate_pravo') . sprintf(StringUtils::fromUTF8(' (absence %s je jiz schvalena)'), $kodOperace));
            }
        }

        if (!$rec->editujAbsenci($filtr, $params)) {
            throw new Exception('Pri editaci zaznamu absence doslo k chybe!');
        }

        return true;
    }


    /**
     * @param JsonOutputObject $outputObject
     * @throws Exception
     */
    private function zapisDenPlanovaniKapacit($outputObject)
    {
        $den = DateUtils::dateStringToTimestamp($_POST['datum']);
        $rezimUprav = RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV) && isset($_POST['rezimuprav']) && $_POST['rezimuprav'];

        $editovaneRozdeleni = isset($_POST['rozdeleni'])
            ? StringUtils::fromUTF8($_POST['rozdeleni'])
            : null;

        if (Obdobi::jeObdobiUzavreno($this ->database, 'ADS', $den, $den)) {
            throw new Exception( \Ron\ronText('Obdobi_uzavreno') );
        }

        $pp = $this ->nactiPP($_POST['idpp']);

        Csrf::validateCSRFPostParam('pk_' . $pp ->idPP);

        Prava::nastavObdobiProPrava($den, $den);

        if (isset($_POST['detail_kodsmeny'])) {
            $kodSmeny = StringUtils::fromUTF8($_POST['detail_kodsmeny']);

            if (isset($_POST['smena_pocatek']) && isset($_POST['smena_konec']) && isset($_POST['smena_delka'])) {
                $pocatek = DateUtils::timeStringToTimestamp($_POST['smena_pocatek']);
                $konec = DateUtils::timeStringToTimestamp($_POST['smena_konec'], 0, true);
                if ($konec === 86400) {
                    $konec = 86399;  // 24:00 prevede na 23:59:59
                }
                $delka = DateUtils::timeStringToTimestamp($_POST['smena_delka'], 0, true);
            } else {
                $pocatek = null;
                $konec = null;
                $delka = null;
            }

            if (isset($_POST['smena_preruseniOd'])) {
                $preruseniOd = DateUtils::timeStringToTimestamp($_POST['smena_preruseniOd'], 0);
                $preruseniDo = DateUtils::timeStringToTimestamp($_POST['smena_preruseniDo'], 0, true);
            } else {
                $preruseniOd = null;
                $preruseniDo = null;
            }



            $rec = new PlanovaniKapacitRecord($this->database, $pp, $this);
            if ($kodSmeny === '') {
                // TODO: kdyz je nastaveno rozdeleni 2, tak se smaze taky - je to OK?
                $rec->smazNaplanovaneSmeny($den, $den, $rezimUprav, $editovaneRozdeleni);
            } else {
                $smena = Smena::nactiSmenu($this ->database, $kodSmeny);

                if (RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2)) {
                    $rozdeleni2 = (isset($_POST['rozdeleni2'])  &&  $_POST['rozdeleni2'] !== '')
                        ? StringUtils::fromUTF8($_POST['rozdeleni2'])
                        : null;

                    if ($rozdeleni2) {
                        $rec ->lzePriraditRozdeleni2($den, $rozdeleni2);
                    }

                    $rec ->nastavRozdeleni2($rozdeleni2, $rezimUprav);
                }

                $delkaSmeny = $smena ->variabilni ? $delka : $smena ->delkaSmeny;

                KapacityUtils::zkontrolujMaximalniPocetHodin($this ->database, $pp, $den, $den, $delkaSmeny, $rezimUprav);

                $rec->zapisSmenu(
                    $den, $kodSmeny, $rezimUprav, // rezim uprav = false
                    $editovaneRozdeleni,
                    $pocatek, $konec, $delkaSmeny,
                    $preruseniOd, $preruseniDo);
            }
        }

        if (isset($_POST['poznamkadochazky'])) {
            $poznamka = StringUtils::fromUTF8($_POST['poznamkadochazky']);

            $rec = new DenniPoznamkaDochazkyRecord($this ->database, $pp, $den, $this);

            $rec->zapisPoznamku($poznamka);
        }

        for ($i = 1; $i <= (int)$_POST['pocetabsenci']; $i++) {
            $this->zapisDetailAbsence($pp, $i, $rezimUprav);
        }

        $this->nactiDenPlanovani($outputObject, $pp->idPP, $den, $rezimUprav); // refresh dat
    }



    /**
     * Schvaleni planu osoby v aktivnim rezimu
     * @throws Exception
     */
    private function schvalitKapacity()
    {
        if (RonPortalSettings::getIntValue(KapacitySettings::ZPUSOB_SCHVALOVANI)) {
            throw new Exception('Schvalovani po jednotlivych osobach neni povoleno!');
        }

        $pp = $this ->nactiPP($_POST['idpp']);

        Csrf::validateCSRFPostParam('pk_' . $pp ->idPP);

        $obdobi = $this ->vratObdobi($_POST['obdobi']);

        if ($_POST['datumdo']) {
            $denDo = DateUtils::urlStringToDate($_POST['datumdo']);
            if (!$denDo) {
                throw new Exception('Chyba - spatne zadane datum!');
            }

            if ($denDo < $obdobi->zacatek || $denDo > $obdobi->konec) {
                throw new Exception('Chyba - zadane datum je mimo povoleny rozsah!');
            }
        } else {
            $denDo = null;
        }

        Prava::nastavObdobiProPrava($denDo ?: $obdobi ->zacatek);

        $rezimUprav = RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV) && isset($_POST['rezimuprav']) && $_POST['rezimuprav'];
        $uroven = isset($_POST['uroven']) ? (int)$_POST['uroven'] : 0;

        if (!$uroven) {
            throw new Exception('Chyba - neni zadana uroven schvaleni!');
        }

        SchvaleniKapacitDBRecord::schvalKapacity($this ->database, $pp, $uroven, $obdobi ->zacatek, $rezimUprav, $denDo);
    }



    /**
     * Schvalovani v pasivnim rezimu - lze schvalit vsechny urovne najednou
     * @param JsonOutputObject $output
     * @throws Exception
     */
    private function schvaleniOsoby($output)
    {
        if (RonPortalSettings::getIntValue(KapacitySettings::ZPUSOB_SCHVALOVANI)) {
            throw new Exception('Schvalovani po jednotlivych osobach neni povoleno!');
        }

        $obdobi = Obdobi::vratObdobi($this ->database, 'ADS', DateUtils::urlStringToDate($_GET['obdobi']));
        if ($obdobi ->uzavreno) {
            throw new Exception( \Ron\ronText('Obdobi_uzavreno') );
        }

        Prava::nastavObdobiProPrava($obdobi ->zacatek, $obdobi ->konec);

        $pp = PracovniPomer::nactiPP($this ->database, (int)$_GET['idpp']);
        $output ->addData('jmeno', StringUtils::jsonText($pp ->jmenoPrijmeni));

        $rezimUprav = isset($_GET['rezimUprav']) && (int)$_GET['rezimUprav'];

        try {
            if (isset($_GET['schvaleni1'])) {
                $datumDo = $_GET['schvaleni1'] != ''
                    ? DateUtils::dateStringToTimestamp($_GET['schvaleni1'])
                    : null;

                SchvaleniKapacitDBRecord::schvalKapacity($this->database, $pp, 1, $obdobi->zacatek, $rezimUprav, $datumDo);
            }

            if (isset($_GET['schvaleni2'])) {
                $datumDo = $_GET['schvaleni2'] != ''
                    ? DateUtils::dateStringToTimestamp($_GET['schvaleni2'])
                    : null;

                SchvaleniKapacitDBRecord::schvalKapacity($this->database, $pp, 2, $obdobi->zacatek, $rezimUprav, $datumDo);
            }

            if (isset($_GET['schvaleni3'])) {
                $datumDo = $_GET['schvaleni3'] != ''
                    ? DateUtils::dateStringToTimestamp($_GET['schvaleni3'])
                    : null;

                SchvaleniKapacitDBRecord::schvalKapacity($this->database, $pp, 3, $obdobi->zacatek, $rezimUprav, $datumDo);
            }
        } catch (Exception $e) {
            throw new Exception($pp ->jmenoPrijmeniKratke . ' (' . $pp ->osobniCislo . '): ' . $e ->getMessage(), $e ->getCode(), $e);
        }
    }


    /**
     * Schvaleni rozdeleni v aktivnim rezimu
     * @param JsonOutputObject $output
     * @throws Exception
     */
    private function schvalitRozdeleni($output)
    {
        // Schvaleni planu celeho rozdeleni
        if (!RonPortalSettings::getIntValue(KapacitySettings::ZPUSOB_SCHVALOVANI)) {
            throw new Exception('Schvalovani celeho rozdeleni neni povoleno!');
        }

        $rozdeleni = Rozdeleni::loadRozdeleni($this ->database, KapacitySettings::typRozdeleniProVyber(), StringUtils::fromUTF8($_POST['rozdeleni']));

        Csrf::validateCSRFPostParam('pk_r_' . $rozdeleni ->kod);

        $obdobi = $this ->vratObdobi($_POST['obdobi']);

        if ($_POST['datumdo']) {
            $denDo = DateUtils::urlStringToDate($_POST['datumdo']);
            if (!$denDo) {
                throw new Exception('Chyba - spatne zadane datum!');
            }

            if ($denDo < $obdobi->zacatek || $denDo > $obdobi->konec) {
                throw new Exception('Chyba - zadane datum je mimo povoleny rozsah!');
            }
        } else {
            $denDo = null;
        }

        Prava::nastavObdobiProPrava($obdobi ->zacatek, $obdobi ->konec);

        $rezimUprav = RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV) && isset($_POST['rezimuprav']) && $_POST['rezimuprav'];
        $uroven = isset($_POST['uroven']) ? (int)$_POST['uroven'] : 0;

        if (!$uroven) {
            throw new Exception('Chyba - neni zadana uroven schvaleni!');
        }

        $error = null;

        try {
            SchvaleniPlanuRozdeleniDBRecord::schvalKapacity($this->database, $rozdeleni, $uroven, $obdobi->zacatek, $rezimUprav, $denDo);
        } catch (Exception $e) {
            $error = $e;
        }

        // nacte aktualni hodnotu schvaleni (chyba mohla nastat az po schvaleni, napriklad v naslednem XML)
        $schvaleni = SchvaleniKapacitRozdeleni::nactiSchvaleni($this ->database, $obdobi ->zacatek, $rozdeleni ->typ, $rozdeleni ->kod);
        $output ->addData('schvaleni', $schvaleni ->getJsonData($rezimUprav));

        if ($error) {
            throw $error;
        }
    }


    /**
     * Provede prepocet dochazky
     * @throws Exception
     */
    private function prepocet()
    {
        if (!RonPortalSettings::getSpousteniADS()) {
            // neni nastavena cesta - prepocet se neprovadi - nemelo by nastat, protoze neni zobrazeno tlacitko
            throw new Exception( 'Neni nastaveno spousteni agentu. Nelze provest prepocet!');
        }

        $tlacitkoPrepocet = RonPortalSettings::getIntValue(KapacitySettings::TLACITKO_PREPOCET);

        if (!$tlacitkoPrepocet) {
            throw new Exception('Prepocet neni povolen!');
        }

        $pp = $this ->nactiPP($_POST['idpp']);

        $datumOd = DateUtils::dateStringToTimestamp($_POST['datumod']);
        $datumDo = DateUtils::dateStringToTimestamp($_POST['datumdo']);

        $parametrSkriptu = "IdPP={$_POST['idpp']};obdobi=" . date('j.n.Y', $datumOd);

        AgentManager::executeScriptEvents($this ->database,
            RonPortalScriptEvents::CATEGORY_KAPACITY, RonPortalScriptEvents::KAPACITY_PREPOCET_BEFORE,
            '"' . $parametrSkriptu . '"');

        $puvodniOd = 0;
        if ($tlacitkoPrepocet === 3) {
            // neschvalene podle puvodni
            $schvaleni = SchvaleniKapacitPP::nactiSchvaleni($this ->database, $datumOd, $pp->idPP);

            $schvalenoDo = $schvaleni->posledniSchvaleni(false); // rezimUprav = false
            $puvodniOd = $schvalenoDo ? DateUtils::incDay($schvalenoDo, 1) : $datumOd;

        } else if ($tlacitkoPrepocet === 2) {
            // cele podle puvodni
            $puvodniOd = $datumOd;
        }

        if ($puvodniOd && $puvodniOd <= $datumDo) {
            // prepocet podle puvodni (cele obdobi nebo jen neschvalena cast)
            $params = '8 ' .
                ' -interval:' . date('Y-n-j', $puvodniOd) . ';' . date('Y-n-j', $datumDo) .
                ' -IDPP:' . $_POST['idpp'];

            AgentManager::executeADS('ADS_Agent_Processing.exe', $params);
        }

        if ($tlacitkoPrepocet === 1 ||
            ($tlacitkoPrepocet === 3 && $puvodniOd > $datumOd)
        ) {
            // prepocet vysledku
            $params = '3 ' .
                ' -interval:' . date('Y-n-j', $datumOd) . ';' . date('Y-n-j', $datumDo) .
                ' -IDPP:' . $_POST['idpp'];

            AgentManager::executeADS('ADS_Agent_Processing.exe', $params);
        }

        if ($tlacitkoPrepocet === 4  ||  $tlacitkoPrepocet === 5) {
            // doplnit puvodni [s presahem do dalsiho mesice]
            if ($tlacitkoPrepocet === 5) {
                // s presahem do dalsiho mesice (napr. kvuli prepoctu nocnich smen)
                if (date('n', $datumDo) != date('n', DateUtils::incDay($datumDo))) {
                    // datumDo je posledni den v mesici - zpracuje i prvni den nasledujiciho mesice
                    $datumDo = DateUtils::incDay($datumDo);
                }
            }

            $params = '10 ' .
                ' -interval:' . date('Y-n-j', $datumOd) . ';' . date('Y-n-j', $datumDo) .
                ' -IDPP:' . $_POST['idpp'];

            AgentManager::executeADS('ADS_Agent_Processing.exe', $params);
        }

        AgentManager::executeScriptEvents($this ->database,
            RonPortalScriptEvents::CATEGORY_KAPACITY, RonPortalScriptEvents::KAPACITY_PREPOCET_AFTER,
            '"' . $parametrSkriptu . '"');
    }


    /**
     * Prepocet kontrol planovani
     * @param JsonOutputObject $output
     * @throws Exception
     */
    private function provedKontrolu($output)
    {
        if (!RonPortalSettings::getInstance() ->getIntValue('kapacity_kontrolovat_podminky')) {
            throw new Exception('Kontrola podminek planovani neni zapnuta!');
        }

        if (!RonPortalSettings::getSpousteniADS()) {
            // neni nastavena cesta - prepocet se neprovadi - nemelo by nastat, protoze neni zobrazeno tlacitko
            throw new Exception('Neni nastaveno spousteni agentu. Nelze provest kontrolu!');
        }

        $kodRozdeleni = $_GET['rozdeleni']; // StringUtils::fromUTF8($_GET['rozdeleni']);
        $typRozdeleni = KapacitySettings::typRozdeleniProVyber();

        //throw new \Exception($typRozdeleni . '/' . $kodRozdeleni);

        // nacte rozdeleni
        $rozdeleni = Rozdeleni::loadRozdeleni($this ->database, $typRozdeleni, $kodRozdeleni);
        if ($_GET['obdobi']) {
            $obdobi = $this->vratObdobi($_GET['obdobi']);
            $result = AgentManager::executeADS('ADS_Agent_CapacityPlanning.exe',
                '0 -period:' . date('Y-n-j', $obdobi->zacatek) . " -subDiv:{$rozdeleni ->typ}/{$rozdeleni ->kod}",
                true, true);
        } else if ($_GET['datum']) {
            $datum = DateUtils::urlStringToDate($_GET['datum']);
            $datumOd = DateUtils::firstDayOfMonth($datum);
            $result = AgentManager::executeADS('ADS_Agent_CapacityPlanning.exe',
                '0 -interval:' . date('Y-n-j', $datumOd) . ';' . date('Y-n-j', $datum) . " -subDiv:{$rozdeleni ->typ}/{$rozdeleni ->kod}",
                true, true);
        }

        $output ->addData('taskId', $result ->taskId);
    }


    /**
     * @param JsonOutputObject $output
     * @throws Exception
     */
    private function mojeSmeny($output)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_KAPACITY)) {
            return;
        }
        if (!Prava::vratPrava('prohlizeni') ->lzeZobrazitRC(User::getRC())  &&  !Prava::vratPrava('kapacity') ->lzeZobrazitRC(User::getRC())) {
            // nema pravo
            return;
        }

        $smeny = SmenaProDen::nactiSmenyPodleKapacit($this ->database, User::getRC(), DateUtils::today(), DateUtils::incDay(DateUtils::today(), 14));

        $html =
            '<table class="rowOver clickHref clickLink dashboardTable" data-ron-href="?menuid=prehledsmen">' .
            '<tr><th>' . \Ron\ronText('Date') . '</th><th>' . \Ron\ronText('Smena') . '</th><th>' . \Ron\ronText('Od') . ' - ' . \Ron\ronText('Do') . '</th></tr>';
        foreach ($smeny as $smena) {
            $html .=
                '<tr>' .
                '<td>' . StringUtils::format('date', $smena ->datum) . '</td>' .
                '<td>' . StringUtils::htmlTextConvert($smena ->popis) .  '</td>' .
                '<td class="align-center">' . StringUtils::format('secondsofday', $smena ->pocatekSmeny) . ' - ' . StringUtils::format('secondsofday', $smena ->konecSmeny) . '</td>' .
                '</tr>';
        }
        $html .= '</table>';

        $output ->addText('html', $html);
        $output ->addText('url', '?menuid=prehledsmen');
    }


    /**
     * @param JsonOutputObject $output
     * @throws Exception
     */
    private function podminkyPlanovani($output)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_KAPACITY)) {
            return;
        }

        $data = SadaPodminekPlanovani::nactiPodminky($this ->database,
            KapacitySettings::typRozdeleniProVyber(), StringUtils::fromUTF8($_GET['rozdeleni']),
            DateUtils::urlStringToDate($_GET['datumod']), DateUtils::urlStringToDate($_GET['datumdo']));

        $json = [];
        foreach ($data as $sadaPodminek) {
            $json[] = $sadaPodminek ->getJson();
        }


        $output ->addData('podminky', $json);
    }


    /**
     * Zapise konkretni polozku do uzivatelskeho nastaveni. Data jsou nactena z POST.
     * @param JsonOutputObject $output
     * @return void
     * @throws Exception
     */
    private function saveSettings($output)
    {
        $itemName = $_POST['itemName'] ?? null;
        $value = $_POST['value'] ?? null;

        switch ($itemName) {
            case 'detailnizobrazeni' :
            case 'velikostsouctu' :
            case 'zobrazit_filtr_osoby' :
                $value = (int)$value;
                $defaultValue = 0;
                break;

            case 'velikostlegendy' :
                $value = (int)$value;
                $defaultValue = 1;
                break;

            default :
                throw new Exception('Chyba pri ukladani uzivatelskeho nastaveni - neznama polozka nastaveni!');
        }

        if (!NastaveniUzivatele::setUserSettingsItem($this ->database,'pk_' . $itemName, $value, $defaultValue)) {
            throw new Exception('Pri ukladani uzivatelskeho nastaveni doslo k chybe!');
        };
    }

    /**
     * @param JsonOutputObject $output
     * @return void
     * @throws Exception
     */
    private function provedXmlAkci($output)
    {
        $xmlId = (int)$_POST['xmlid'];
        Csrf::validateCSRFPostParam('pk_xmlakce_' . $xmlId);

        RonPortalXMLForm::xmlEventWithId($this ->database, 'PK_Akce_', $xmlId,
            [
                'rezimuprav' => (int)$_POST['rezimuprav'],
                'rozdeleni' => $_POST['rozdeleni'],
                'obdobi' => DateUtils::urlStringToDate($_POST['obdobi'])
            ],
            '');

        $output ->addMessages(MessageManager::getInstance());
    }


    /**
     * @throws NeniPravoException
     * @throws DBException
     * @throws \Exception
     */
    public function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'nactidataosoby' :
                $this->nactiDataOsobyProRefresh($output);
                break;

            case 'delkasmeny' :
                $this->prepocetDelkySmeny($output);
                break;

            case 'konecsmeny' :
                $this->prepocetKonceSmeny($output);
                break;

            case 'poznamky' :
                $this->nactiPoznamkyDochazky($output);
                break;

            case 'kontroly' :
                $this ->nactiKontroly($output);
                break;

            case 'poznamkykedni' :
                $this->nactiPoznamkyKeDni($output);
                break;

            // editace dat
            case 'zapisplanovanousmenu' :
                $this->zapisPlanovanouSmenu($output);
                break;

            case 'podlekalendare' :
                $this->naplanujPodleKalendare();
                break;

            case 'zapisharmonogram' :
                $this->zapisHarmonogram();
                break;

            case 'zapisplanovanouabsenci' :
                $this->zapisPlanovanouAbsenci($output);
                break;

            case 'zmenarozdeleni' :
                $this->nastavRozdeleni();
                break;

            case 'zmenarozdeleni2' :
                $this->nastavRozdeleni(true);
                break;

            case 'smazdataplanovani' :
                $this->smazDataPlanovani();
                break;

            case 'smazabsenci' :
                $this->smazAbsenci();
                break;

            case 'nactidenplanovanikapacit' :
                $this->denPlanovani($output);
                break;

            case 'seznamrozdeleni2' :
                $this->seznamRozdeleni2($output);
                break;

            case 'zapisdenplanovanikapacit' :
                $this->zapisDenPlanovaniKapacit($output);
                break;

            case 'potvrdit' :
                $this->potvrditSmeny();
                break;

            case 'schvalit' :
                $this->schvalitKapacity();
                break;

            case 'schvalitrozdeleni' :
                $this->schvalitRozdeleni($output);
                break;

            case 'prepocet' :
                $this->prepocet();
                break;

            case 'nactidataplanovanikapacit' :
                PlanovaniKapacit::nactiDataPlanovani($this->database, $output);
                break;

            case 'nactihlavickuplanovanikapacit' :
                PlanovaniKapacit::nactiHlavickuPlanovani($this->database, $output);
                break;

            case 'dokumenty' :
                $this ->nactiDokumentyDochazky($output);
                break;

            case 'sc' :
                $this ->nactiSluzebniCesty($output);
                break;

            case 'upload' :
                $this->uploadFile();
                break;

            case 'deletedokument' :
                $this->deleteDokument();
                break;

            case 'refreshdokumenty' : // refresh dokumentu v detailu po uploadu
                $this->refreshDokumenty($output);
                break;

            case 'kontrola' :
                $this->provedKontrolu($output);
                break;

            case 'mojesmeny' : // data pro dashboard
                $this->mojeSmeny($output);
                break;

            case 'podminky' :
                $this ->podminkyPlanovani($output);
                break;

            case 'vybersmen' :
                $this->vyberSmenUzivatele($output);
                break;

            case 'zapisnastaveni' :
                $this->zapisNastaveniUzivatele($output);
                break;

            case 'schvaleni' :
                $this ->schvaleniOsoby($output);
                break;

            case 'savesettings' :
                $this ->saveSettings($output);
                break;

            case 'xmlakce' :
                $this ->provedXmlAkci($output);
                break;

            default :
                throw new Exception("Chyba v KapacityJSController::handleAction - neznama akce: {$_GET['action']}!");
        }
    }

    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('dochazka.ini');
        TextManager::getInstance() ->loadStrings('kapacity.ini');

        if ($_GET['action'] === 'smenypodlekalendare') {
            return new VarDumpViewObject(SmenaProDen::nactiSmenyPodleKalendareRozdeleni(
                $this->database, 11, PracovniPomer::nactiPP($this->database, $_GET['idpp']),
                DateUtils::dateStringToTimestamp($_GET['datumod']), DateUtils::dateStringToTimestamp($_GET['datumdo'])));
        }

        return parent::handleAction($action);
    }
}
