<?php
/*
   Rozhrani pro Javascript kapacity.js

   Format JSON pro HLAVICKU:
   { obdobi : [ { pocatek:..., konec:..., uzavreno:..., popis }, ... ],
     rozdeleni: [ { kod:..., popis:..., barva:... }, ... ],
     prepocet: 0/1/2,
     prepocetText: "Prepocitat",
     povolitRezimUprav: 0/1,
     result: "OK"
   }

*/

namespace RonPortal\Kapacity;

use Exception;
use Ron\DateUtils;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\GuiUtils;
use Ron\Http\Csrf;
use Ron\MessageManager;
use Ron\DB\Database;
use Ron\MVC\JsonOutputObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Dochazka\DokumentDochazky;
use RonPortal\Dochazka\MesicniVysledek;
use RonPortal\Dochazka\Smena;
use RonPortal\Kalendar;
use RonPortal\NastaveniUzivatele;
use RonPortal\Obdobi;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniOsob;
use RonPortal\RozdeleniUtils;
use RonPortal\User;


class PlanovaniKapacit
{
    const SETTINGS_JEN_V_EVIDENCNIM_STAVU = 'kapacity_jenvevidencnimstavu'; // vyzaduje MZDY!
    const SETTINGS_EVIDENCNI_STAVY = 'kapacity_evidencnistavy'; // vyzaduje MZDY!
    const SETTINGS_JEN_S_KALENDAREM = 'kapacity_jenskalendarem';
    const SETTINGS_PATICKA = 'kapacity_paticka';
    const SETTINGS_CIZI = 'kapacity_cizi';  // pujcovani zamestnancu
    //const SETTINGS_POCITAT_DNY_S_ABSENCI = 'kapacity_pocitatdnysabsenci';
    const SETTINGS_ABSENCE_NA_RADKU = 'kapacity_absencenaradku';
    const SETTINGS_ABSENCE_VE_SLOUPCI = 'kapacity_absencevesloupci';
    const SETTINGS_SMENY_PODLE_DNE = 'kapacity_povolitsmenypodledne';   // jestli se povolene smeny urcuji podle Smeny II v kalendari
    const SETTINGS_CASOVE_SLOZKY = 'kapacity_casoveslozky';         // zobrazeni casovych slozek na radku
    const SETTINGS_CASOVE_SLOZKY_D = 'kapacity_casoveslozkyd';      // zobrazeni dennich vysledku
    const SETTINGS_CASOVE_SLOZKY_CSS = 'kapacity_casoveslozky_css'; // nastaveni class radku podle mesicniho vysledku (nastavi se prislusna class, pokud jsou hodiny nenulove)

    /** @var Database */
    var $database;

    /** @var  PlanovaniKapacit_Osoba[] */
    var $seznamOsob;
    /** @var int[]  Slouzi ke kontrole, jestli uz je pracovni pomer v seznamu - aby nebyl zobrazeny 2x */
    var $nactenePP = [];
    /** @var  bool  Priznak se pouziva pro zobrazeni prehledu podle smen bez prihlaseni */
    var $nekontrolovatPrava;

    /** @var PlanovaniKapacit_Osoba[]   Pokud se ma zobrazit jen jedna osoba, tak do tohoto seznamu nacte seznam vsech osob (pouzije se pro zobrazeni seznamu osob ve filtru) */
    private $vsechnyOsoby;

    /** @var int[]  Asociativni pole barev podle rozdeleni (kod rozdeleni => hodnota barvy) */
    var $barvy = [];

    /** @var  PlanovaniKapacit_Den[] */
    var $dny;
    /** @var  PlanovaniKapacit_SloupecHlavicky[]    sloupce zobrazene v hlavicce (casove slozky) */
    var $sloupceHlavicky;
    /** @var  Smena[] */
    var $seznamSmen;
    /** @var  PlanovaniKapacit_Absence[] */
    var $seznamAbsenci;
    /** @var  PlanovaniKapacit_Kalendar[] */
    var $seznamKalendaru;
    /** @var  PlanovaniKapacit_Suma[] */
    var $soucty; // radky se soucty
    /** @var Harmonogram[] */
    var $harmonogramy;
    /** @var VysledekKontrolyPlanovani[]  kontroly nactene pro zobrazeni v HTML */
    var $kontroly;
    /** @var SadaPodminekPlanovani[] */
    var $sloty;

    /** @var int  pokud nebude nastaveno pred nacitanim dat, pouzije se obdobi aplikace */
    var $datumOd;
    var $datumDo;
    var $obdobiUzavreno;

    /** @var int */
    public $kodTypuRozdeleni;
    /** @var string */
    public $rozdeleni;
    /** @var int    0=jen vlastni, 1=vcetne podrizenych, 2=jen podrizene, 3=jen primo podrizene */
    public $rozdeleniVcetnePodrizenych = 1;
    /** @v8ar string */
    public $rozdeleni2;
    /** @var int    0=jen vlastni, 1=vcetne podrizenych, 2=jen podrizene, 3=jen primo podrizene */
    public $rozdeleni2VcetnePodrizenych = 1;
    /** @var ?string    Kod kalendare prirazeneho k editovanemu rozdeleni */
    public $kodKalendareRozdeleni = null;


    /** @var int?       Id pracovniho pomeru - pokud je zadan, tak se nactou data jen pro tento PP */
    public $idPP = null;

    /** @var SchvaleniKapacitRozdeleni */
    var $schvaleniRozdeleni = null;

    var $lzeEditovatCizi; // priznak, jestli je mozne editovat cizi osoby zarazene na aktualne zvolene rozdeleni

    var $pouziteSmeny; // asociativni pole [<kodsmeny>] => 1  - seznam vsech smen, ktere se vyskytuji v nactenych datech

    var $povolitRezimUprav;
    var $rezimUprav;

    public function __construct($database)
    {
        $this->database = $database;
    }


    public function nactiSmeny()
    {
        $this->seznamSmen = Smena::nactiSmenyUzivatele($this->database,
            'Smena.ZkratkaProPlanovani IS NOT NULL',    //"   AND Smena.ZkratkaProPlanovani <> '' ".  - nefunguje na Oracle!)
            'ZkratkaProPlanovani');
        if (!$this->seznamSmen) {
            $this->seznamSmen = [];
            return false;
        }

        return true;
    }


    /**
     * @param $kod
     * @return PlanovaniKapacit_Kalendar|null
     */
    public function najdiKalendar($kod)
    {
        // vraci kalendar se zadanym kodem, nebo null, pokud neexistuje
        foreach ($this->seznamKalendaru as $kalendar) {
            if ($kalendar->kod === $kod) {
                return $kalendar;
            }
        }

        return null;
    }


    public function najdiOsobu($idPP)
    {
        foreach ($this ->seznamOsob as $osoba) {
            if ($osoba ->idPP === $idPP) {
                return $osoba;
            }
        }

        return null;
    }


    private function pridejKalendar($kod)
    {
        $kalendar = $this->najdiKalendar($kod);
        if (!$kalendar) {
            $kalendar = new PlanovaniKapacit_Kalendar();

            $kalendar->nactiData($this->database, $kod,
                RonPortalSettings::getIntValue(self::SETTINGS_SMENY_PODLE_DNE) === 2,
                $this->datumOd, $this->datumDo);

            $this->seznamKalendaru[] = $kalendar;
        }
    }

    /**
     * @throws DBException
     * @throws Exception
     */
    public function nactiKalendare()
    {
        // Projde vsechny osoby a nacte podrobnosti k jejich kalendarum => nacita jen pouzite kalendare
        foreach ($this->seznamOsob as $osoba) {
            foreach ($osoba->seznamKalendaru as $prirazenyKalendar) {
                $this ->pridejKalendar($prirazenyKalendar ->kod);
            }
        }

        // nakonec jeste prida kalendar prirazeny k aktualnimu rozdeleni
        if (isset($this ->kodKalendareRozdeleni)) {
            $this ->pridejKalendar($this ->kodKalendareRozdeleni);
        }
    }



    // mesicni vysledky -------------------------------------------------------
    /**
     * @param PlanovaniKapacit_Osoba $osoba
     * @param string $kodCasoveSlozky
     * @param float $hodiny
     * @param float $dny
     * @param float $kalendarniDny
     * @return bool
     */
    public function nastavMesicniVysledek($osoba, $kodCasoveSlozky, $hodiny, $dny, $kalendarniDny)
    {
        $result = false;

        foreach ($this->sloupceHlavicky as $i => $sloupec) {
            if (!strcasecmp($kodCasoveSlozky, $sloupec->kodCasoveSlozky)) {
                switch ($sloupec->jednotka) {
                    case 'D' :
                        $hodnota = $dny;
                        $text = StringUtils::format('number_2', $dny);
                        break;

                    case 'K' :
                        $hodnota = $kalendarniDny;
                        $text = StringUtils::format('number_2', $kalendarniDny);
                        break;

                    default :
                        $hodnota = $hodiny;
                        $text = StringUtils::format('hodiny', $hodiny);
                        break;
                }

                if ($hodnota > 0) {
                    $class = "planovani_mv_plus planovani_mv{$kodCasoveSlozky}_plus";
                }
                else if ($hodnota < 0) {
                    $class = "planovani_mv_minus planovani_mv{$kodCasoveSlozky}_minus";
                }
                else {
                    $class = "planovani_mv_nula planovani_mv{$kodCasoveSlozky}_nula";
                }

                $osoba->classHlavicky[$i] = $class; // kvuli xml musi byt class zvlast
                $osoba->hodnotyHlavicky[$i] = $text;

                $result = true; // nasel prislusny sloupec
            }
        }

        return $result;
    }


    public function nactiSloupceHlavicky()
    {
        $this->sloupceHlavicky = [];

        // vytvori jednotlive sloupce
        $poma = explode(';', RonPortalSettings::getValue(self::SETTINGS_CASOVE_SLOZKY, ''));

        if ($poma) {
            foreach ($poma as $nastavenistr) {
                if ($nastavenistr) {
                    $sloupec = new PlanovaniKapacit_SloupecHlavicky();
                    $sloupec->nactiNastaveni($nastavenistr);

                    $this->sloupceHlavicky[] = $sloupec;
                }
            }
        }
    }


    public function doplnSloupecProSlot()
    {
        $sloupec = new PlanovaniKapacit_SloupecHlavicky();
        $sloupec ->kodCasoveSlozky = 'SLOT';
        $this ->sloupceHlavicky[] = $sloupec;
    }


    /**
     * Nacte mesicni vysledky (zadane $casoveSlozky) pro vsechny osoby z $seznamPP
     * @param $seznamPP
     * @param $casoveSlozky
     * @throws DBException
     */
    public function nactiMesicniVysledky($seznamPP, $casoveSlozky)
    {
        if ($casoveSlozky  &&  $seznamPP) {
            $seznamMV = MesicniVysledek::nactiMesicniVysledky($this ->database, $seznamPP, $this ->datumOd, $casoveSlozky);

            /** @var PlanovaniKapacit_Osoba $osoba */
            $osoba = null;

            $slozkyCSS = RonPortalSettings::getValue(self::SETTINGS_CASOVE_SLOZKY_CSS)
                ? explode(';', RonPortalSettings::getValue(self::SETTINGS_CASOVE_SLOZKY_CSS, ''))
                : [];

            foreach($seznamMV as $mesicniVysledek) {
                if (!$osoba || $osoba ->idPP !== $mesicniVysledek ->idPP) {
                    $osoba = $this ->najdiOsobu($mesicniVysledek ->idPP);
                }

                if ($osoba) {
                    $this->nastavMesicniVysledek($osoba, $mesicniVysledek ->typCasoveSlozkyMzdy, $mesicniVysledek ->hodiny, $mesicniVysledek ->dny, $mesicniVysledek ->kalendarniDny);

                    if ($mesicniVysledek ->typCasoveSlozkyMzdy === RonPortalSettings::getValue('kapacity_cs_soucet')) {
                        $osoba ->maximalniPocetHodin = $mesicniVysledek ->hodiny;
                    }

                    if ($mesicniVysledek ->hodiny  &&  in_array($mesicniVysledek ->typCasoveSlozkyMzdy, $slozkyCSS, true)) {
                        $osoba ->rowClasses =
                            ($osoba ->rowClasses ? $osoba ->rowClasses . ' ' : '') .
                            'mvrow_' . $mesicniVysledek->typCasoveSlozkyMzdy;
                    }
                }
            }
        }
    }


    public function nactiHodnotyHlavicky()
    {
        $seznamPP = [];
        $casoveSlozky = [];

        // nastavi hodnoty sloupcu pro vsechny osoby
        // a zaroven sestavi seznam osob a casovych slozek, ktere je potreba nacist
        foreach ($this->seznamOsob as $osoba) {
            $osoba->hodnotyHlavicky = [];

            $seznamPP[] = $osoba ->idPP;

            foreach ($this->sloupceHlavicky as $i => $sloupec) {
                switch ($sloupec->kodCasoveSlozky) {
                    case 'R' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->rozdeleni;
                        break;
                    case 'U1' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[0];
                        break;
                    case 'U2' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[1];
                        break;
                    case 'U3' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[2];
                        break;
                    case 'U4' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[3];
                        break;
                    case 'U5' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[4];
                        break;
                    case 'U6' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[5];
                        break;
                    case 'U7' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[6];
                        break;
                    case 'U8' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[7];
                        break;
                    case 'U9' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[8];
                        break;
                    case 'U10' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[9];
                        break;
                    case 'U11' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[10];
                        break;
                    case 'U12' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[11];
                        break;
                    case 'U13' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[12];
                        break;
                    case 'U14' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[13];
                        break;
                    case 'U15' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[14];
                        break;
                    case 'U16' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[15];
                        break;
                    case 'U17' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[16];
                        break;
                    case 'U18' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[17];
                        break;
                    case 'U19' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[18];
                        break;
                    case 'U20' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelske[19];
                        break;

                    case 'CAS_SCHVALENI1' :
                        $osoba ->hodnotyHlavicky[$i] = StringUtils::format('finedatetime', $osoba ->casSchvaleni1);
                        break;
                    case 'CAS_SCHVALENI2' :
                        $osoba ->hodnotyHlavicky[$i] = StringUtils::format('finedatetime', $osoba ->casSchvaleni2);
                        break;
                    case 'CAS_SCHVALENI3' :
                        $osoba ->hodnotyHlavicky[$i] = StringUtils::format('finedatetime', $osoba ->casSchvaleni3);
                        break;

                    case 'P1' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[0];
                        break;
                    case 'P2' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[1];
                        break;
                    case 'P3' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[2];
                        break;
                    case 'P4' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[3];
                        break;
                    case 'P5' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[4];
                        break;
                    case 'P6' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[5];
                        break;
                    case 'P7' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[6];
                        break;
                    case 'P8' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[7];
                        break;
                    case 'P9' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[8];
                        break;
                    case 'P10' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[9];
                        break;
                    case 'P11' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[10];
                        break;
                    case 'P12' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[11];
                        break;
                    case 'P13' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[12];
                        break;
                    case 'P14' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[13];
                        break;
                    case 'P15' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[14];
                        break;
                    case 'P16' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[15];
                        break;
                    case 'P17' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[16];
                        break;
                    case 'P18' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[17];
                        break;
                    case 'P19' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[18];
                        break;
                    case 'P20' :
                        $osoba->hodnotyHlavicky[$i] = $osoba->uzivatelskePP[19];
                        break;
                    //case 'DATSCHVAL1' :
                        //$osoba->hodnotyHlavicky[$i] = $osoba ->schva

                    default :
                        if ($sloupec->kodCasoveSlozky) {
                            $casoveSlozky[$sloupec->kodCasoveSlozky] = $sloupec->kodCasoveSlozky;
                        }
                }
            }
        }

        if (RonPortalSettings::getValue('kapacity_cs_soucet')) {
            $casoveSlozky[RonPortalSettings::getValue('kapacity_cs_soucet')] = RonPortalSettings::getValue('kapacity_cs_soucet');
        }

        if (RonPortalSettings::getValue(self::SETTINGS_CASOVE_SLOZKY_CSS)) {
            $slozky = explode(';', RonPortalSettings::getValue(self::SETTINGS_CASOVE_SLOZKY_CSS ?? ''));
            foreach ($slozky as $kodCs) {
                $casoveSlozky[$kodCs] = $kodCs;
            }

        }

        $this ->nactiMesicniVysledky($seznamPP, $casoveSlozky);
    }


    /**
     * Vraci true, pokud se ma zobrazit i rozdeleni osoby
     * @return bool
     */
    public function zobrazitRozdeleni()
    {
        foreach ($this->sloupceHlavicky as $sloupec) {
            if ($sloupec->kodCasoveSlozky === 'R') {
                return true; // ma se zobrazit rozdeleni
            }
        }

        return false;
    }


    public static function typRozdeleniProVyber()
    {
        return KapacitySettings::typRozdeleniProVyber();
    }


    /**
     * @param bool $cizi
     * @param bool $zobrazitrozdeleni
     * @param Parameters $params
     * @return string
     */
    private function getBasicSQL($cizi, $zobrazitrozdeleni, $params)
    {
        $sql =
            'SELECT DISTINCT PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu,' . // 0, 1, 2
            " Osoba.Jmeno, Osoba.Prijmeni, Osoba.Titul, PracovniPomer.Id, PracovniPomer.DatumUkonceni,\n" . // 3..7
            ($this ->rezimUprav
                ? ' SchvaleniKapacit.UpravyZpracovanyDo, SchvaleniKapacit.UpravyZkontrolovanyDo, SchvaleniKapacit.UpravySchvalenyDo, '  // 8..10
                : ' SchvaleniKapacit.ZpracovanoDo, SchvaleniKapacit.ZkontrolovanoDo, SchvaleniKapacit.SchvalenoDo, '  // 8..10
            ) .
            // uzivatelske - osoba
            "\n Osoba.uzivatelska1, Osoba.uzivatelska2, Osoba.uzivatelska3, Osoba.uzivatelska4, Osoba.uzivatelska5, " . // 11..15
            ' Osoba.uzivatelska6, Osoba.uzivatelska7, Osoba.uzivatelska8, Osoba.uzivatelska9, Osoba.uzivatelska10, ' . // 16..20
            ' Osoba.uzivatelska11, Osoba.uzivatelska12, Osoba.uzivatelska13, Osoba.uzivatelska14, Osoba.uzivatelska15, ' . // 21..25
            ' Osoba.uzivatelska16, Osoba.uzivatelska17, Osoba.uzivatelska18, Osoba.uzivatelska19, Osoba.uzivatelska20, ' . // 26..30
            // uzivatelske - pracovni pomer
            "\n PracovniPomer.uzivatelska1, PracovniPomer.uzivatelska2, PracovniPomer.uzivatelska3, PracovniPomer.uzivatelska4, PracovniPomer.uzivatelska5, " . // 31..35
            ' PracovniPomer.uzivatelska6, PracovniPomer.uzivatelska7, PracovniPomer.uzivatelska8, PracovniPomer.uzivatelska9, PracovniPomer.uzivatelska10, ' . // 36..40
            ' PracovniPomer.uzivatelska11, PracovniPomer.uzivatelska12, PracovniPomer.uzivatelska13, PracovniPomer.uzivatelska14, PracovniPomer.uzivatelska15, ' . // 41..45
            ' PracovniPomer.uzivatelska16, PracovniPomer.uzivatelska17, PracovniPomer.uzivatelska18, PracovniPomer.uzivatelska19, PracovniPomer.uzivatelska20, ' . // 46..50
            "\n SchvaleniKapacit.ZpracovanoDo, SchvaleniKapacit.ZkontrolovanoDo, SchvaleniKapacit.SchvalenoDo, " . // 51..53
            "\n" .
            ($this ->rezimUprav
                ? ' SchvaleniKapacit.Cas_UpravyZpracovany, SchvaleniKapacit.UpravyZpracoval, SchvaleniKapacit.Cas_UpravyZkontrolovany, SchvaleniKapacit.UpravyZkontroloval, SchvaleniKapacit.Cas_UpravySchvaleny, SchvaleniKapacit.UpravySchvalil'  // 54..59
                : ' SchvaleniKapacit.Cas_Zpracovano, SchvaleniKapacit.Zpracoval, SchvaleniKapacit.Cas_Zkontrolovano, SchvaleniKapacit.Zkontroloval, SchvaleniKapacit.Cas_Schvaleno, SchvaleniKapacit.Schvalil'  // 54..59
            );

        if ($zobrazitrozdeleni) {
            $sql .= ",\n RozdeleniOsob.Rozdeleni "; // 60
        }

        $sql .= "\n";

        if ($zobrazitrozdeleni || ($this ->kodTypuRozdeleni && isset($this ->rozdeleni))) {
            $sql .=
                ' FROM ( (PracovniPomer INNER JOIN Osoba ON Osoba.RC = PracovniPomer.RC )' .
                '  INNER JOIN RozdeleniOsob ON' .
                '    RozdeleniOsob.RC = PracovniPomer.RC' .
                '    AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
                '    AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
                '  INNER JOIN Rozdeleni ON' .
                '    Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni  AND  Rozdeleni.Kod = RozdeleniOsob.Rozdeleni' .
                ")\n" .
                ' LEFT OUTER JOIN SchvaleniKapacit' .
                '  ON SchvaleniKapacit.RC = PracovniPomer.RC' .
                '    AND SchvaleniKapacit.OsobniCislo = PracovniPomer.OsobniCislo' .
                '    AND SchvaleniKapacit.DatumNastupu = PracovniPomer.DatumNastupu' .
                "    AND SchvaleniKapacit.Aplikace = 'ADS' " .
                '    AND SchvaleniKapacit.PocatekObdobi = ' . $params->addDateParameter($this ->datumOd) .

                "\nWHERE (PracovniPomer.DatumNastupu <= " . $params->addDateParameter($this ->datumDo) . ')' .
                ' AND (PracovniPomer.DatumUkonceni IS NULL  OR  PracovniPomer.DatumUkonceni >= ' . $params ->addDateParameter($this ->datumOd) . ')' .

                // rozdeleni osoby platne ve zvolenem obdobi
                ' AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter($this ->datumDo) .
                ' AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter($this ->datumOd) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                ' AND (RozdeleniOsob.KodTypuRozdeleni = ' . (int)$this ->kodTypuRozdeleni . ') ';

            if ($cizi) {

                // vypujcene osoby z jinych rozdeleni
                $sql .=
                    "\nAND EXISTS (SELECT * FROM PlanovaniKapacit " .
                    ' WHERE PlanovaniKapacit.RC = PracovniPomer.RC ' .
                    '   AND PlanovaniKapacit.OsobniCislo = PracovniPomer.OsobniCislo ' .
                    '   AND PlanovaniKapacit.DatumNastupu = PracovniPomer.DatumNastupu ' .
                    '   AND PlanovaniKapacit.Datum >= ' . $params ->addDateParameter($this ->datumOd) .
                    '   AND PlanovaniKapacit.Datum <= ' . $params ->addDateParameter($this ->datumDo) .
                    '   AND PlanovaniKapacit.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI) .
                    '   AND PlanovaniKapacit.Rozdeleni = ' . $params->addParameter($this ->rozdeleni) .
                    ' ) ';
            } else if ($this ->kodTypuRozdeleni && isset($this ->rozdeleni)) {
                // zapnut filtr na rozdeleni => jen vybrane rozdeleni nebo podrizene
                // Poznamka: puvodne tu byl vyraz s OR, ale ten byl pomaly!
                if ($this ->rozdeleniVcetnePodrizenych === 1  ||  $this ->rozdeleniVcetnePodrizenych === 2) {
                    // vcetne podrizenych/jen podrizene
                    $sql .=
                        "\nAND EXISTS " .
                        '(SELECT * FROM HierarchieRozdeleni WHERE' .
                        ' HierarchieRozdeleni.TypNadrazene = ' . (int)$this->kodTypuRozdeleni .
                        ' AND HierarchieRozdeleni.KodNadrazene = ' . $params->addParameter($this->rozdeleni) .
                        " AND HierarchieRozdeleni.TypPodrizene = RozdeleniOsob.KodTypuRozdeleni " .
                        " AND HierarchieRozdeleni.KodPodrizene = RozdeleniOsob.Rozdeleni" .
                        (($this->rozdeleniVcetnePodrizenych === 2)
                            ? ' AND HierarchieRozdeleni.KodPodrizene <> HierarchieRozdeleni.KodNadrazene'   // jen podrizene
                            : ''
                        ) .
                        ')';
                } else if ($this ->rozdeleniVcetnePodrizenych === 3) {
                    // jen primo podrizene
                    $sql .=
                        "\nAND Rozdeleni.RozdeleniRodice = " .  (int)$this->kodTypuRozdeleni .
                        ' AND Rozdeleni.Rodic = ' . $params->addParameter($this->rozdeleni);
                } else {
                    $sql .=
                        "\nAND ( RozdeleniOsob.Rozdeleni = " . $params ->addParameter($this ->rozdeleni) . ')';
                }
            }
        } else {
            // neni treba nacitat rozdeleni - nepripojuje tabulku RozdeleniOsob
            $sql .=
                ' FROM (PracovniPomer INNER JOIN Osoba ON Osoba.RC = PracovniPomer.RC) ' .
                ' LEFT OUTER JOIN SchvaleniKapacit ' .
                ' ON SchvaleniKapacit.RC = PracovniPomer.RC ' .
                '    AND SchvaleniKapacit.OsobniCislo = PracovniPomer.OsobniCislo' .
                '    AND SchvaleniKapacit.DatumNastupu = PracovniPomer.DatumNastupu' .
                "    AND SchvaleniKapacit.Aplikace = 'ADS' " .
                '    AND SchvaleniKapacit.PocatekObdobi = ' . $params->addDateParameter($this ->datumOd) .
                "\nWHERE (PracovniPomer.DatumNastupu <= " . $params->addDateParameter($this ->datumDo) . ')' .
                ' AND (PracovniPomer.DatumUkonceni IS NULL  OR  PracovniPomer.DatumUkonceni >= ' . $params->addDateParameter($this ->datumOd) . ')';
        }

        $sql .=
            "\nAND PracovniPomer.Zamestnanec > 0 " . // nezobrazuje externi
            ' AND PracovniPomer.ZpracovatDochazku = 1 ';

        if (RonPortalSettings::getIntValue(self::SETTINGS_JEN_S_KALENDAREM)) {
            // zobrazit jen zamestnance s planovacim kalendarem
            $sql .=
                "\nAND EXISTS (" .
                '    SELECT * FROM PrirazeniKalendare, Kalendar ' .
                '    WHERE PrirazeniKalendare.RC = PracovniPomer.RC ' .
                '      AND PrirazeniKalendare.OsobniCislo = PracovniPomer.OsobniCislo ' .
                '      AND PrirazeniKalendare.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '      AND PrirazeniKalendare.PlatnostOd <= ' . $params->addDateParameter($this ->datumDo) .
                '      AND (PrirazeniKalendare.PlatnostDo >= ' . $params->addDateParameter($this ->datumOd) . '  OR  PrirazeniKalendare.PlatnostDo IS NULL)' .
                '      AND PrirazeniKalendare.Kalendar = Kalendar.Kod ' .
                '      AND Kalendar.DochazkaPodlePlanKapacit = 1' .
                ' )';
        }

        if (trim(RonPortalSettings::getValue(self::SETTINGS_EVIDENCNI_STAVY) ?? '')) {
            $stavy = explode(';', RonPortalSettings::getValue(self::SETTINGS_EVIDENCNI_STAVY, ''));
//            foreach ($stavy as $index => $stav) {
//                $stavy[$index] = "'" . $this ->database ->addSlashes($stav) . "'";
//            }

            if ($stavy) {
                $sql .=
                    "\nAND EXISTS (" .
                    '    SELECT * FROM EvidencniStav ' .
                    '    WHERE EvidencniStav.RC = PracovniPomer.RC ' .
                    '      AND EvidencniStav.OsobniCislo = PracovniPomer.OsobniCislo ' .
                    '      AND EvidencniStav.DatumNastupu = PracovniPomer.DatumNastupu ' .
                    '      AND EvidencniStav.PlatnostOd <= ' . $params->addDateParameter($this->datumDo) .
                    '      AND (EvidencniStav.PlatnostDo >= ' . $params->addDateParameter($this->datumOd) . '  OR  EvidencniStav.PlatnostDo IS NULL)' .
                    '      AND ' . SQLUtils::getINCondition($this ->database, 'EvidencniStav.KodEvidencnihoStavu', $stavy, $params) .
                    //'      AND EvidencniStav.KodEvidencnihoStavu IN (' . SQLUtils::getINCondition($this ->database, ) implode(', ', $stavy) . ')' .
                    ' )';
            }
        } else if (RonPortalSettings::getIntValue(self::SETTINGS_JEN_V_EVIDENCNIM_STAVU)) {
            $sql .=
                "\nAND EXISTS (" .
                '    SELECT * FROM EvidencniStav, TypEvidencnihoStavu ' .
                '    WHERE EvidencniStav.RC = PracovniPomer.RC ' .
                '      AND EvidencniStav.OsobniCislo = PracovniPomer.OsobniCislo ' .
                '      AND EvidencniStav.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '      AND EvidencniStav.PlatnostOd <= ' . $params ->addDateParameter($this ->datumDo) .
                '      AND (EvidencniStav.PlatnostDo >= ' . $params ->addDateParameter($this ->datumOd) . '  OR  EvidencniStav.PlatnostDo IS NULL)' .
                '      AND EvidencniStav.KodEvidencnihoStavu = TypEvidencnihoStavu.Kod ' .
                '      AND TypEvidencnihoStavu.ZarazenVeStavu = 1' .
                ' )';
        }

        if (isset($this ->rozdeleni2)  &&  RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
            $filtr = RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'PracovniPomer',
                RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), $this ->rozdeleni2,
                $this ->datumOd, $this ->datumDo, $this->rozdeleni2VcetnePodrizenych,
                $params);

            if ($filtr) {
                $sql .= "\nAND ($filtr)";
            }
        }

//        if ($this ->idPP) {       - filtrovani resi az pri nacitani seznamu, protoze v aktivnim rezimu potrebuje sestavit i seznam osob pro select
//            $sql .= "\nAND PracovniPomer.Id = " . $params ->addParameter($this ->idPP);
//        }

        return $sql;
    }


    /**
     * @return SqlStatement
     * @throws Exception
     */
    public function getSQLTextKapacity($cizi = false, $nekontrolovatPrava = false)
    {
        $zobrazitRozdeleni = $this ->zobrazitRozdeleni();

        $params = $this ->database ->createParameters();

        $sql = $this ->getBasicSQL($cizi, $zobrazitRozdeleni, $params) . "\n";

        // filtr podle prav
        if (!$cizi  &&  !$nekontrolovatPrava) {
            $prava = Prava::vratPrava('kapacity');

            $filtrPrava = Prava::sqlFiltrPodlePrav($prava, 'PracovniPomer', $this->datumOd, $this->datumDo, true, $params); // vynechatVcetneSebe=true (optimalizace)
            if ($filtrPrava) {
                $sql .= "\nAND ($filtrPrava)\n";
            }

            if (Prava::pravaVcetneSebe($prava)) {
                $sql .=
                    "\nUNION\n" .
                    $this ->getBasicSQL($cizi, $zobrazitRozdeleni, $params) .
                    "\nAND PracovniPomer.RC = " . $params ->addParameter(User::getRC());

            }
        }

        $sql .= "\n";

        // razeni
        $sort = isset($_SESSION['kapacity_sort']) ? (int)$_SESSION['kapacity_sort'] : 0;
        $desc = $sort < 0 ? ' DESC ' : '';
        if ($sort < 0) {
            $sort = -$sort;
        }

        // 0=default, 1=jmeno, 2=osobnicislo, 3=rozdeleni, 101..120=uzivatelska, 201..220=PP.uzivatelska
        if (!$sort) {
            $sort = $zobrazitRozdeleni ? 3 : 1; // pokud se ma zobrazit rozdeleni tak podle rozdeleni, jinak jen podle jmena
        }

        $sql .= ' ORDER BY ';

        if ($sort >= 101 && $sort <= 120) {
            // uzivatelska1..20
            $sql .= ' Osoba.uzivatelska' . ($sort - 100) . ' ' . $desc . ',  ';
        } else if ($sort >= 201 && $sort <= 220) {
            // PP.uzivatelska1..20
            $sql .= ' PracovniPomer.uzivatelska' . ($sort - 200) . ' ' . $desc . ',  ';
        } else if ($sort === 2) {
            $sql .= ' PracovniPomer.OsobniCislo ' . $desc . ',  ';
        }
        else if ($zobrazitRozdeleni && $sort === 3) {
            $sql .= ' RozdeleniOsob.Rozdeleni ' . $desc . ', ';
        }

        //    if (db_type() == "interbase") $sql = $sql." ORDER BY 8, 7, 1";
        //    else $sql = $sql." ORDER BY Osoba.prijmeni, Osoba.jmeno, PracovniPomer.RC";

        $sql .= ' Osoba.Prijmeni ' . $desc . ', Osoba.Jmeno ' . $desc . ', PracovniPomer.RC';

        return new SqlStatement($sql, $params);
    }


    /**
     * Nacte radky soucty (vybrane CS podle nastaveni)
     * @throws Exception
     */
    public function nactiSoucty()
    {
        $poma = explode(';', RonPortalSettings::getValue(self::SETTINGS_PATICKA, ''));
        foreach ($poma as $radekPaticky) {
            $arr = explode(':', $radekPaticky ?? '', 4);

            switch ($arr[0]) {
                case 'R' : // R:<kod_CS>:H/D/K:nadpis  - hodnota z tabulky DenniCasovaSlozka_R
                    $radek = new PlanovaniKapacit_Suma();
                    $radek ->kodCasoveSlozky = $arr[1];
                    $radek ->jednotka = $arr[2];
                    $radek ->nadpis = $arr[3];
                    $radek ->nactiDenniCasovaSlozkaR($this ->database,
                        self::typRozdeleniProVyber(), $this ->rozdeleni,
                        $this ->datumOd, $this ->datumDo);

                    $this ->soucty[] = $radek;
            }
        }
    }


    /**
     * Prida do seznamu osobu ze zadaneho DB radku
     * @throws DBException
     */
    public function pridejOsobu($row, $cizi)
    {
        $osoba = new PlanovaniKapacit_Osoba();
        $osoba ->cizi = $cizi;
        $osoba ->nactiData($this ->database, $row);
        $this ->seznamOsob[] = $osoba;

        if ($rozdeleniIkony = RonPortalSettings::getValue('kapacity_ikony_rozdeleni')) {
            // Nacte ikony podle rozdeleni osob
            $osoba ->ikony = RozdeleniOsob::nactiIkonyRozdeleni($this ->database, $osoba ->idPP, $rozdeleniIkony, $this ->datumOd, $this ->datumDo);
        }

        if ($rozdeleniBarvy = RonPortalSettings::getValue('kapacity_barvy_rozdeleni')) {
            // Nacte barvy podle rozdeleni osob
            $r = RozdeleniOsob::nactiRozdeleniOsoby($this ->database, $osoba ->idPP, $rozdeleniBarvy, $this ->datumOd, $this ->datumDo, 'Rozdeleni.BarvaProVykresleni IS NOT NULL');
            foreach ($r as $rozdeleniOsoby) {
                $this ->barvy[$rozdeleniOsoby ->kod] = $rozdeleniOsoby ->barvaProVykresleni;
                $osoba ->rowClasses .= ($osoba ->rowClasses ? ' ' : '') . 'pk_row_r_' . $rozdeleniOsoby ->kod;
            }
        }

        $osoba ->moznostEditaceSchvalenych = PlanovaniKapacit_Osoba::nactiPovoleniEditovatSchvalene($this ->database, $osoba ->idPP, $this ->datumOd, $this ->datumDo);

        return $osoba;
    }


    /**
     * @throws DBException
     * @throws Exception
     */
    public function nactiOsoby($jenEditovatelne = false)
    {
        $nekontrolovatPrava = !User::isAuthenticated(); // pro zobrazeni prehledu smen bez prihlaseni

        $sql = $this ->getSQLTextKapacity(false, $nekontrolovatPrava);

        MessageManager::addDebugLog($sql);

        $pravaZobrazeni = Prava::vratPrava('kapacity');
        $pravaEditace = Prava::vratPrava('kapacity_edit');
        $pravaSchvaleni1 = Prava::vratPrava('kapac_schval1');
        $pravaSchvaleni2 = Prava::vratPrava('kapac_schval2');
        $pravaSchvaleni3 = Prava::vratPrava('kapac_schval3');
        $pravaAbsence = Prava::vratPrava('absence');

        $cursor = $this ->database ->select($sql);

        while ($row = $cursor ->fetch()) {
            if (!isset($this ->nactenePP[$row[6]])) { // jeste nenacetl tohle idpp
                if ($nekontrolovatPrava) {
                    $lzeEditovat = false;
                    $lzeZobrazit = true;
                } else {
                    $lzeEditovat = $pravaEditace->lzeZobrazitPP($row[6], $this->datumOd, $this->datumDo);
                    $lzeZobrazit = $jenEditovatelne
                        ? $lzeEditovat
                        : $pravaZobrazeni->lzeZobrazitPP($row[6], $this->datumOd, $this->datumDo);

                    if (!$lzeEditovat  &&  $lzeZobrazit  &&  $pravaZobrazeni ->priznak2  &&  !RonPortalSettings::getIntValue('kapacity_zpusobschvalovani')) {
                        // uzivatel nemuze editovat a plan muze zobrazit az po schvaleni nejmene zadanou urovni
                        $lzeZobrazit = false;

                        if ($pravaZobrazeni ->priznak2 === 1) {
                            $lzeZobrazit = $this ->database->stringToTimestamp($row[8]) >= $this ->datumDo;
                        }

                        if (!$lzeZobrazit  &&  $pravaZobrazeni ->priznak2 <= 2) {
                            $lzeZobrazit = $this ->database->stringToTimestamp($row[9]) >= $this ->datumDo;
                        }

                        if (!$lzeZobrazit  &&  $pravaZobrazeni ->priznak2 <= 3) {
                            $lzeZobrazit = $this ->database->stringToTimestamp($row[10]) >= $this ->datumDo;
                        }
                    }
                }

                $this ->nactenePP[$row[6]] = true; // zapamatuje si, ze uz tohle PP nacetl

                if ($nekontrolovatPrava || $lzeZobrazit || $lzeEditovat) {
                    if (!$this ->idPP  ||  $this ->idPP == $row[6]) {
                        $osoba = $this->pridejOsobu($row, false); // cizi=false
                        $this ->vsechnyOsoby[] = $osoba;

                        $osoba->lzeEditovat = $lzeEditovat;

                        $osoba->lzeSchvalit1 = $pravaSchvaleni1->lzeZobrazitPP($osoba->idPP, $this->datumOd, $this->datumDo);
                        $osoba->lzeSchvalit2 = $pravaSchvaleni2->lzeZobrazitPP($osoba->idPP, $this->datumOd, $this->datumDo);
                        $osoba->lzeSchvalit3 = $pravaSchvaleni3->lzeZobrazitPP($osoba->idPP, $this->datumOd, $this->datumDo);

                        $osoba->lzePlanovatAbsence = $pravaAbsence->lzeZobrazitPP($osoba->idPP, $this->datumOd, $this->datumDo);
                    } else {
                        // prida osobu jen do seznamu vsech osob pro filtrovani
                        $osoba = new PlanovaniKapacit_Osoba();
                        $osoba ->nactiData($this ->database, $row);
                        $this ->vsechnyOsoby[] = $osoba;
                    }
                }
            }
        }
    }


    /**
     * @throws DBException
     * @throws Exception
     */
    public function nactiCiziOsoby($jenEditovatelne = false)
    {
        if (!RonPortalSettings::getIntValue(self::SETTINGS_CIZI)) {
            return; // funkce neni zapnuta
        }

        if (!RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI)) {
            return; // neni nastaven typ rozdeleni
        }
        if (!isset($this ->rozdeleni)) {
            return; // neni vybrano zadne rozdeleni
        }

        // uzivatel musi mit prava na vse nebo podle rozdeleni
        // a zaroven ve filtru vybrane rozdeleni, na ktere ma pravo
        $pravaZobrazeni = Prava::vratPrava('kapacity');
        if (!$pravaZobrazeni ->jePravoNaRozdeleni($this ->kodTypuRozdeleni, $this ->rozdeleni)) {
            return;
        }

        $pravaEditace = Prava::vratPrava('kapacity_edit');
        $this ->lzeEditovatCizi =
            $pravaEditace ->prava === Prava::PRAVA_POUZE_VLASTNI ||     // aby mohl editovat sve vlastni udaje
            $pravaEditace ->jePravoNaRozdeleni($this ->kodTypuRozdeleni, $this ->rozdeleni);

        $pravaSchvaleni1 = Prava::vratPrava('kapac_schval1');
        $pravaSchvaleni2 = Prava::vratPrava('kapac_schval2');
        $pravaSchvaleni3 = Prava::vratPrava('kapac_schval3');

        $sql = $this ->getSQLTextKapacity(true); // cizi = true
        MessageManager::addDebugLog('Osoby z jinych rozdeleni: ' . $sql);

        $cursor = $this ->database ->select($sql);

        while ($row = $cursor ->fetch()) {
            if (!isset($this ->nactenePP[$row[6]])) {
                if (!$jenEditovatelne  ||  $pravaEditace ->lzeZobrazitPP($row[6], $this ->datumOd, $this ->datumDo)) {
                    $this->nactenePP[$row[6]] = true;
                    $osoba = $this->pridejOsobu($row, true); // cizi=true
                    // Doplneno 9.1.2025 - aby mohl editovat osoby na zmenenem rozdeleni i po schvaleni (pokud je muze schvalovat)
                    $osoba->lzeSchvalit1 = $pravaSchvaleni1->lzeZobrazitPP($osoba->idPP, $this->datumOd, $this->datumDo);
                    $osoba->lzeSchvalit2 = $pravaSchvaleni2->lzeZobrazitPP($osoba->idPP, $this->datumOd, $this->datumDo);
                    $osoba->lzeSchvalit3 = $pravaSchvaleni3->lzeZobrazitPP($osoba->idPP, $this->datumOd, $this->datumDo);

                }
            }
        }
    }


    public function getSeznamIdPP()
    {
        $seznamIdPP = [];
        foreach ($this ->seznamOsob as $osoba) {
            $seznamIdPP[] = $osoba ->idPP;
        }
        return $seznamIdPP;
    }


    /**
     * @throws DBException
     */
    public function nactiPrirazeneKalendare()
    {
        $seznamIdPP = $this ->getSeznamIdPP();
        $seznamKalendaru = PlanovaniKapacit_PrirazeniKalendare::nactiKalendare($this ->database,
            $seznamIdPP, $this ->datumOd, $this ->datumDo);

        /** @var PlanovaniKapacit_Osoba $osoba */
        $osoba = null;
        foreach ($seznamKalendaru as $kalendar) {
            if (!$osoba  ||  $osoba ->idPP !== $kalendar ->idPP) {
                $osoba = $this ->najdiOsobu($kalendar ->idPP);
            }

            if ($osoba) {
                $osoba ->seznamKalendaru[] = $kalendar;
            }
        }
    }

    public function nactiAbsence($rezimUprav)
    {
        $seznamIdPP = $this ->getSeznamIdPP();

        $seznamAbsenci = PlanovaniKapacit_PlanovanaAbsence::nactiAbsence($this ->database,
            $seznamIdPP, $this ->datumOd, $this ->datumDo, false, $rezimUprav);

        /** @var PlanovaniKapacit_Osoba $osoba */
        $osoba = null;
        foreach ($seznamAbsenci as $absence) {
            if (!$osoba  ||  $osoba ->idPP !== $absence ->idPP) {
                $osoba = $this ->najdiOsobu($absence ->idPP);
            }

            if ($osoba) {
                $osoba ->seznamAbsenci[] = $absence;
            }
        }
    }


    public function nactiPlanovaneSmeny($rezimUprav, $nacistRozdeleni = false)
    {
        $seznamIdPP = $this ->getSeznamIdPP();
        $seznamSmen = PlanovaniKapacit_PlanovanaSmena::nactiSmeny($this ->database,
            $seznamIdPP, $this ->datumOd, $this ->datumDo, $rezimUprav, $nacistRozdeleni);

        /** @var PlanovaniKapacit_Osoba $osoba */
        $osoba = null;
        foreach ($seznamSmen as $smena) {
            if (!$osoba  ||  $osoba ->idPP !== $smena ->idPP) {
                $osoba = $this ->najdiOsobu($smena ->idPP);
            }

            if ($osoba) {
                $osoba ->seznamSmen[] = $smena;
            }
        }
    }

    /**
     * Poznamky ke dni (zobrazene v hlavicce) - pouziva se jen v pasivnim rezimu
     * @throws Exception
     */
    public function nactiPoznamkyKeDni()
    {
        PlanovaniKapacit_Den::nactiPoznamky($this ->database, $this ->dny, $this ->kodTypuRozdeleni, $this ->rozdeleni);
    }



    public function nactiSluzebniCesty()
    {
        $seznamPP = $this ->getSeznamIdPP();
        $seznamSC = PlanovaniKapacit_SluzebniCesta::nactiSluzebniCestyZDB($this ->database, $this ->datumOd, $this ->datumDo, $seznamPP);
        foreach ($seznamSC as $sc) {
            $osoba = $this ->najdiOsobu($sc ->idPP);
            if ($osoba) {
                if (!$osoba ->sluzebniCesty)  $osoba ->sluzebniCesty= [];

                $osoba ->sluzebniCesty[] = $sc;
            }
        }
    }


    public function nactiDokumenty()
    {
        $seznamPP = $this ->getSeznamIdPP();

        $seznamDokumentu = DokumentDochazky::nactiDokumentyOsob($this ->database, $this ->datumOd, $this ->datumDo, $seznamPP, null);
        foreach ($seznamDokumentu as $dokument) {
            $osoba = $this ->najdiOsobu($dokument ->idPP);
            if ($osoba) {
                if (!$osoba ->dokumentyDochazky) $osoba ->dokumentyDochazky = [];

                $osoba ->dokumentyDochazky[] = $dokument;
            }
        }
    }


    public function nactiKontroly()
    {
        $this ->kontroly = [];

        $seznamPP = $this ->getSeznamIdPP();
        $kontroly = VysledekKontrolyPlanovani::nactiKontroly($this ->database, $this ->datumOd, $this ->datumDo, $seznamPP, $this ->kodTypuRozdeleni, $this ->rozdeleni);

        foreach ($kontroly as $kontrola) {
            if ($kontrola ->idPP) {
                $osoba = $this->najdiOsobu($kontrola->idPP);
                if ($osoba) {
                    if (!$osoba->kontroly) $osoba->kontroly = [];
                    $osoba->kontroly[] = $kontrola;
                }
            } else {
                $this ->kontroly[] = $kontrola;
            }
        }

        // priradi kontroly ke dnum
        foreach ($this ->kontroly as $kontrola) {
            $indexDne = date('j', $kontrola ->datum) - date('j', $this ->datumOd);
            if (!$this ->dny[$indexDne] ->kontroly) $this ->dny[$indexDne] ->kontroly = [];
            $this ->dny[$indexDne] ->kontroly[] = $kontrola;
        }

        foreach ($this ->seznamOsob as $osoba) {
            if ($osoba ->kontroly) {
                $classes = [];
                foreach ($osoba ->kontroly as $kontrola) {
                    $className = $kontrola ->cssClass ?: 'kontrola_' . $kontrola ->kategorie;
                    if (!in_array($className, $classes)) {
                        $classes[] = $className;
                    }
                }
                if ($classes) {
                    $osoba ->rowClasses .= ($osoba ->rowClasses ? ' ' : '') . implode(' ', $classes);
                }
            }
        }
    }


    /**
     * Nacte zakladni data planovani.
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @param string $rozdeleni
     * @param int $rozdeleniVcetnePodrizenych
     * @param string $rozdeleni2
     * @param int $rozdeleni2VcetnePodrizenych
     * @param bool $rezimUprav
     * @param bool $nacistRozdeleni
     * @param bool $jenEditovatelne
     * @throws DBException
     */
    public function nactiData($obdobiOd, $obdobiDo, $rozdeleni, $rozdeleniVcetnePodrizenych, $rozdeleni2, $rozdeleni2VcetnePodrizenych, $rezimUprav, $nacistRozdeleni = false, $jenEditovatelne = false)
    {
        $this ->dny = [];
        $this ->seznamOsob = [];
        $this ->seznamSmen = [];
        $this ->seznamAbsenci = [];
        $this ->seznamKalendaru = [];
        $this ->pouziteSmeny = [];
        $this ->soucty = [];
        $this ->harmonogramy = [];
        $this ->sloty = [];

        $this ->nactiSloupceHlavicky(); // vytvori seznam sloupcu podle retezce nastaveni

        $this ->datumOd = $obdobiOd;
        $this ->datumDo = $obdobiDo;

        $this ->kodTypuRozdeleni = self::typRozdeleniProVyber();
        $this ->rozdeleni = $rozdeleni;
        $this ->rozdeleniVcetnePodrizenych = (int)$rozdeleniVcetnePodrizenych;
        $this ->rozdeleni2 = $rozdeleni2 !== '' ? $rozdeleni2 : null;
        $this ->rozdeleni2VcetnePodrizenych = (int)$rozdeleni2VcetnePodrizenych;

        $this ->povolitRezimUprav = RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV);
        $this ->rezimUprav = $rezimUprav;

        Prava::nastavObdobiProPrava($this ->datumOd, $this ->datumDo);

        // nacte kalendar - prednostne podle zvoleneho rozdeleni ...
        $kalendarProSvatky = isset($this ->rozdeleni)
            ? Kalendar::vratKalendarRozdeleni($this ->database, $this ->kodTypuRozdeleni, $this ->rozdeleni)
            : null;

        $this ->kodKalendareRozdeleni = $kalendarProSvatky;

        if (!isset($kalendarProSvatky)) {
            // a kdyz neni prirazen, tak podle aktualniho uzivatele
            $kalendarProSvatky = Kalendar::vratKalendarUzivatele($this->database, $this->datumOd, $this->datumDo);
        }

        // zjisti, jestli je obdobi uzavreno
        $this ->obdobiUzavreno = Obdobi::jeObdobiUzavreno($this ->database, 'ADS', $this ->datumOd, $this ->datumOd);

        if (RonPortalSettings::getIntValue('kapacity_zpusobschvalovani')) {
            // schvaluje se cele rozdeleni najednou
            $this ->schvaleniRozdeleni = SchvaleniKapacitRozdeleni::nactiSchvaleni($this ->database, $this ->datumOd, $this ->kodTypuRozdeleni, $this ->rozdeleni);

            $pravaZobrazeni = Prava::vratPrava('kapacity');
            $pravaEditace = Prava::vratPrava('kapacity_edit');

            if ($pravaZobrazeni ->priznak2  // muze zobrazit az po schvaleni planu nekterou urovni
                && !$pravaEditace ->jePravoNaRozdeleni($this ->kodTypuRozdeleni, $this ->rozdeleni, $this ->datumOd, $this ->datumDo)
            ) {
                $lzeZobrazit = false;
                if ($pravaZobrazeni ->priznak2 == 1) {
                    $lzeZobrazit = $this ->schvaleniRozdeleni->schvaleni1 >= $this ->datumDo;
                }

                if (!$lzeZobrazit  &&  $pravaZobrazeni ->priznak2 <= 2) {
                    $lzeZobrazit = $this ->schvaleniRozdeleni->schvaleni2 >= $this ->datumDo;
                }

                if (!$lzeZobrazit  &&  $pravaZobrazeni ->priznak2 <= 3) {
                    $lzeZobrazit = $this ->schvaleniRozdeleni->schvaleni3 >= $this ->datumDo;
                }

                if (!$lzeZobrazit) {
                    throw new Exception( \Ron\ronText('Nelze_zobrazit_neschvaleny_plan') );
                }
            }


        }

        // naplni seznam dnu
        $this ->dny = PlanovaniKapacit_Den::nactiDnyProObdobi($this ->database, $this ->datumOd, $this ->datumDo, $kalendarProSvatky);

        // nacte seznam osob
        $this ->nactiOsoby($jenEditovatelne);
        $this ->nactiCiziOsoby($jenEditovatelne);

        $this ->nactiPlanovaneSmeny($rezimUprav, $nacistRozdeleni);
        $this ->nactiAbsence($rezimUprav);

        $this ->nactiHodnotyHlavicky();
        $this ->nactiSoucty();
    }


    /**
     * Nacte data planovani pro HTML
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @param string $rozdeleni
     * @param int $rozdeleniVcetnePodrizenych
     * @param string $rozdeleni2
     * @param int $rozdeleni2VcetnePodrizenych
     * @param bool $rezimUprav
     * @throws Exception
     */
    public function nactiDataProHtml($obdobiOd, $obdobiDo, $rozdeleni, $rozdeleniVcetnePodrizenych, $rozdeleni2, $rozdeleni2VcetnePodrizenych, $rezimUprav)
    {
        $this->nactiData($obdobiOd, $obdobiDo, $rozdeleni, $rozdeleniVcetnePodrizenych, $rozdeleni2, $rozdeleni2VcetnePodrizenych, $rezimUprav, true);

        // naplni seznam pouzitych smen
        foreach ($this->seznamOsob as $osoba) {
            foreach ($osoba->seznamSmen as $smena) {
                $this->pouziteSmeny[$smena->kod] = 1;
            }
        }

        $this->nactiPoznamkyKeDni();
        $this->nactiSluzebniCesty();
        $this->nactiDokumenty();
        $this->nactiKontroly();
    }


    /**
     * Nacte data planovani pro Json
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @param string $rozdeleni
     * @param int $rozdeleniVcetnePodrizenych
     * @param string $rozdeleni2
     * @param bool $rezimUprav
     * @throws DBException
     */
    public function nactiDataProJson($obdobiOd, $obdobiDo, $rozdeleni, $rozdeleniVcetnePodrizenych, $rozdeleni2, $rezimUprav)
    {
        $this ->nactiData($obdobiOd, $obdobiDo,
            $rozdeleni, $rozdeleniVcetnePodrizenych, $rozdeleni2, true,
            $rezimUprav, false, RonPortalSettings::getIntValue('kapacity_aktivni_jen_editovatelne'));

        // v aktivnim rezimu nacte absence, kalendare a prirazeni kalendaru
        $this ->nactiPrirazeneKalendare();

        $this->seznamAbsenci = PlanovaniKapacit_Absence::nactiSeznamOperaci($this->database, true);
        $this ->nactiKalendare();
        $this ->nactiSmeny();

        $this ->harmonogramy = Harmonogram::nactiHarmonogramy($this ->database, 'ZkratkaProPlanovani IS NOT NULL');
    }


    /**
     * @param PodminkaPlanovaniKapacit[] $podminky
     */
    public function nactiRozdeleniOsob($podminky)
    {
        if (!$podminky) {
            return;
        }

        $seznamPP = [];
        foreach ($this ->seznamOsob as $osoba) {
            $seznamPP[] = $osoba ->idPP;
        }

        if (!$seznamPP) {
            return;
        }

        $params = $this ->database ->createParameters();
        $sql = '';
        foreach ($podminky as $podminka) {
            if ($sql) {
                $sql .= ' OR ';
            }
            $sql .=
                '(RozdeleniOsob.KodTypuRozdeleni = ' . $params ->addParameter((int)$podminka ->kodTypuRozdeleni) .
                ' AND RozdeleniOsob.Rozdeleni = ' . $params ->addParameter($podminka ->rozdeleni) . ')';
        }

        $seznam = RozdeleniOsob::nactiRozdeleniOsoby($this ->database, $seznamPP, '*', $this ->datumOd, $this ->datumDo, new SqlStatement($sql, $params));

        foreach ($seznam as $rozdeleniOsoby) {
            $osoba = $this ->najdiOsobu($rozdeleniOsoby ->idPP);
            if ($osoba) {
                $osoba ->rozdeleniOsoby[] = $rozdeleniOsoby;
            }
        }
    }


    public function nactiSloty()
    {
        $this ->sloty = SadaPodminekPlanovani::nactiPodminky($this ->database,
            $this ->kodTypuRozdeleni, $this ->rozdeleni,
            $this ->datumOd, $this ->datumDo,
            'PodminkaPlanovaniKapacit.SlotProPlanovani = 1');

        // vytvori seznam podminek, pro ktere je potreba nacist rozdeleni osob
        $podminkySRozdelenim = [];
        foreach ($this ->sloty as $sada) {
            foreach ($sada ->podminky as $podminka) {
                if ($podminka ->kodTypuRozdeleni  &&  $podminka ->rozdeleni) {
                    $podminkySRozdelenim[$podminka ->kodTypuRozdeleni . '/' . $podminka ->rozdeleni] = $podminka;
                }
            }
        }
        $this ->nactiRozdeleniOsob($podminkySRozdelenim);
    }


    /**
     * vraci seznam absenci, ktere se maji zapocitavat jako asociativni pole "kod" => 1
     * @param string $settingsStr
     * @return array
     */
    private function getSettingsSoucty($settingsStr)
    {
        $settingsArr = explode(';', $settingsStr ?? '');
        $result = [];
        foreach ($settingsArr as $kod) {
            if ($kod != '') {
                $result[StringUtils::jsonText($kod)] = 1;
            }
        }
        return $result;
    }

    /**
     * @param JsonOutputObject $outputObject
     * @throws DBException
     */
    public function zapisData($outputObject)
    {

        $outputObject ->addData('uzavreno', $this ->obdobiUzavreno ? 1 : 0);

        $outputObject ->addData('pocitatAbsenceNaRadku', $this ->getSettingsSoucty(RonPortalSettings::getValue(self::SETTINGS_ABSENCE_NA_RADKU, '*')));
        $outputObject ->addData('pocitatAbsenceVeSloupci', $this ->getSettingsSoucty(RonPortalSettings::getValue(self::SETTINGS_ABSENCE_VE_SLOUPCI, '')));
        $outputObject ->addData('absencePlanovatDoSlotu', RonPortalSettings::getIntValue('kapacity_absence_do_slotu'));
        //if (RonPortalSettings::getIntValue(self::SETTINGS_POCITAT_DNY_S_ABSENCI)) $outputObject ->addData('pocitatDnySAbsenci', 1);

        if (RonPortalSettings::getIntValue(KapacitySettings::EDITACE_SCHVALENYCH)) {
            $outputObject ->addData('lzeEditovatSchvalene', 1);
        }
        if (RonPortalSettings::getIntValue(KapacitySettings::ZAKAZAT_SNIZIT_SCHVALENI)) {
            $outputObject ->addData('zakazatSnizitSchvaleni', 1);
        }

        $outputObject ->addData('zpusobSchvalovani', RonPortalSettings::getIntValue(KapacitySettings::ZPUSOB_SCHVALOVANI));

        if ($level = Prava::vratPrava('kapacity_edit') ->priznak2) {
            // editovat se muze az po schvaleni poctu zamestnancu
            $outputObject ->addData('editovatAzPoSchvaleniPoctu', $level);
           $schvaleniPodminek = SchvaleniPodminekPlanovani::nactiSchvaleni($this ->database, $this ->datumOd, $this ->kodTypuRozdeleni, $this ->rozdeleni);

            switch ($level) {
                case 3 :
                    $outputObject ->addData('schvaleniPozadavkuDo', StringUtils::jsonDate($schvaleniPodminek ->schvaleni3));
                    break;
                case 2 :
                    $outputObject ->addData('schvaleniPozadavkuDo', StringUtils::jsonDate($schvaleniPodminek ->schvaleni2));
                    break;
                case 1 :
                default :
                    $outputObject ->addData('schvaleniPozadavkuDo', StringUtils::jsonDate($schvaleniPodminek ->schvaleni1));
                    break;
            }
        }

        $outputObject ->addData('vyberSmen', RonPortalSettings::getIntValue(self::SETTINGS_SMENY_PODLE_DNE));

        $outputObject ->addData('sirkaBunky', KapacitySettings::sirkaBunky($this ->database));
        $outputObject ->addData('vyskaBunky', KapacitySettings::vyskaBunky($this ->database));
        $outputObject ->addData('sirkaBunkyDetailni', KapacitySettings::sirkaBunkyDetailni($this ->database));
        $outputObject ->addData('vyskaBunkyDetailni', KapacitySettings::vyskaBunkyDetailni($this ->database));

        $outputObject ->addData('rozdeleni', StringUtils::jsonText($this ->rozdeleni) );
        $outputObject ->addData('lzeEditovatCizi', $this ->lzeEditovatCizi ? 1 : 0);
        $outputObject ->addData('schvalovaniAbsenci', RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI) ? 1 : 0);

        $outputObject ->addData('povolitRezimUprav', $this ->povolitRezimUprav ? 1 : 0);
        $outputObject ->addData('minSchvaleniProUpravy', RonPortalSettings::getIntValue(KapacitySettings::MIN_SCHVALENI_PRO_UPRAVY));
        $outputObject ->addData('rezimUprav', $this ->rezimUprav ? 1 : 0);
        //$outputObject ->addData('povolitZmenuRozdeleni', RonPortalSettings::getIntValue(RonPortalSettings::KAPACITY_TYP_ROZDELENI) ? 1 : 0);
        //$outputObject ->addData('povolitZmenuRozdeleni2', RonPortalSettings::getIntValue(RonPortalSettings::KAPACITY_TYP_ROZDELENI2) ? 1 : 0);

        if ($this ->datumOd <= time()  &&  $this ->datumDo >= DateUtils::today()) {
            $outputObject ->addData('vychoziDen', (int)date('j', time()));
        }

        if ($this ->schvaleniRozdeleni) {
            $outputObject ->addData('schvaleni', $this ->schvaleniRozdeleni ->getJsonData($this ->rezimUprav));
        }

        // seznam dnu (datum, typ dne)
        $dny = [];
        foreach ($this ->dny as $den) {
            $dny[] = $den ->getJSON();
        }
        $outputObject ->addData('dny', $dny);

        // seznam dodatecnych sloupcu (mesicni vysledky)
        $sloupce = [];
        foreach ($this ->sloupceHlavicky as $sloupec) {
            $sloupce[] = $sloupec ->getJson();
        }
        $outputObject ->addData('sloupceHlavicky', $sloupce);

        // seznam smen
        $smeny = [];
        foreach ($this ->seznamSmen as $smena) {
            $smeny[] = $smena ->getJson();
        }
        $outputObject ->addData('smeny', $smeny);

        // seznam absenci (operace)
        $seznamAbsenci = [];
        foreach ($this ->seznamAbsenci as $absence) {
            $seznamAbsenci[] = $absence ->getJson();
        }
        $outputObject ->addData('absence', $seznamAbsenci);

        $seznamKalendaru = [];
        foreach ($this ->seznamKalendaru as $kalendar) {
            $seznamKalendaru[] = $kalendar ->getJson();
        }
        $outputObject ->addData('kalendare', $seznamKalendaru);

        $seznamHarmonogramu = [];
        foreach ($this ->harmonogramy as $harmonogram) {
            $seznamHarmonogramu[] = $harmonogram ->getJson();
        }
        $outputObject ->addData('harmonogramy', $seznamHarmonogramu);

        if ($this ->sloty) {
            $sloty = [];
            foreach ($this->sloty as $sadaPodminek) {
                $sloty[] = $sadaPodminek ->getJson();
            }
            $outputObject ->addData('podminkyProSloty', $sloty);
        }


        // paticka
        $radkyPaticky = [];

        foreach ($this ->soucty as $radek) {
            $soucty = [];

            //$pomdatum = $this ->f_zacatekobdobi;
            foreach ($radek ->data as $sloupec_suma) {
                //echo
                //    "      <polozkapaticky datum='" . date('d.m.Y', $pomdatum) . "'>$sloupec_suma</polozkapaticky>\n";
                //$pomdatum = incday_start($pomdatum, 1);
                $soucty[] = StringUtils::jsonText($sloupec_suma);
            }

            $obj = [
                'nadpis' => StringUtils::jsonText($radek ->nadpis),
                'data' => $soucty
            ];

            $radkyPaticky[] = $obj;
        }

        $outputObject ->addData('radkyPaticky', $radkyPaticky);

        $xml_form_absence = DBUtils::getSqlValue($this ->database, "SELECT Id FROM SablonaXMLFormulare WHERE Popis = 'PK_Absence_INSERT'");
        if ($xml_form_absence) {
            $outputObject ->addData('XML_Absence', (int)$xml_form_absence);
        }

        // barvy podle rozdeleni
        $style = '';
        foreach ($this ->barvy as $kodRozdeleni => $barva) {
            if ($barva) {
                $style .=
                    ".pk_row_r_{$kodRozdeleni} th, .pk_row_r_{$kodRozdeleni} td { background-color: " . GuiUtils::getHtmlColor($barva) . "; }\n" .
                    ".pk_row_r_{$kodRozdeleni} td.planovani_vikend { background-color: " . GuiUtils::getHtmlColor($barva, 0.95) . "; }\n" // vikend tmavsi
                ;
            }
        }
        $outputObject ->addData('style', StringUtils::jsonText($style));


        // osoby
        $osoby = [];
        foreach ($this ->seznamOsob as $osoba) {
            $osoby[] = $osoba ->getJson();
        }
        $outputObject ->addData('osoby', $osoby);

        // osoby pro filtr
        if ($this ->idPP  &&  $this ->vsechnyOsoby) {
            $vsechnyOsoby = [];
            foreach ($this ->vsechnyOsoby as $osoba) {
                $vsechnyOsoby[] = $osoba ->getJson(true); // jen hlavicka
            }
            $outputObject ->addData('vsechnyOsoby', $vsechnyOsoby);
        }
    }



    // -----     PlanovaniKapacit     -----------------------------------------
//    function zapisSouctyHTML()
//    {
//        foreach ($this ->soucty as $radek) {
//            echo '<tr>';
//
//            echo '<TH></TH>'; // sloupec "osobni cislo"
//            echo '<TH class="footercaptions lastheadercol" colspan="' . (1 + count($this ->sloupceHlavicky)) . '">' . $radek ->nadpis . '</TH>';
//
//            // data
//            foreach ($radek ->data as $index => $sloupec_suma) {
//                $classStr = $index ? '' : 'class="firstdatacol"';
//                echo "<TH $classStr>$sloupec_suma</TH>";
//            }
//
//            echo "</tr>\n";
//        }
//    }


    // Vraci SQL filtr podle seznamu pouzitych smen - pro pouziti v legende
    public function vratFiltrProPouziteSmeny($jmenoPole)
    {
        $result = '';
        foreach ($this ->pouziteSmeny as $kod => $pouzita) {
            $result .= ($result ? '  OR  ' : '') . "$jmenoPole = '$kod'";
        }

        if ($result) {
            return '(' . $result . ')';
        }

        return '';
    }


    /**
     * Nacte data pro aktivni rezim
     * @param Database $database
     * @param JsonOutputObject $outputObject
     * @throws Exception
     */
    public static function nactiDataPlanovani($database, $outputObject)
    {
        if (!isset($_GET['obdobi'])) {
            throw new Exception('Chyba: neni zadano obdobi!');
        }

        // nastavi nove obdobi - promenna by mela obsahovat pocatek obdobi
        $pocatekObdobi = DateUtils::urlStringToDate($_GET['obdobi']);

        $obdobi = Obdobi::vratObdobi($database, 'ADS', $pocatekObdobi);

        if ($obdobi) {
            $obdobiOd = $obdobi->zacatek;
            $obdobiDo = $obdobi->konec;

            RonPortalSession::setObdobi($obdobi ->zacatek, $obdobi ->konec);
        } else {
            $obdobiOd = DateUtils::firstDayOfMonth($pocatekObdobi);
            $obdobiDo = DateUtils::lastDayOfMonth($pocatekObdobi);
        }

        Prava::nastavObdobiProPrava($obdobiOd, $obdobiDo);

        if (isset($_GET['rozdeleni'])) {
            $rozdeleni = StringUtils::fromUTF8($_GET['rozdeleni']);
            RonPortalSession::setRozdeleni( $rozdeleni );
        } else {
            $rozdeleni = RonPortalSession::rozdeleni();
        }

        $vcetnePodrizenych = (int)($_GET['vcetnepodrizenych'] ?? 1);

        $dataPlanovani = new PlanovaniKapacit($database);

        if ($_GET['osoba'] ?? null) {
            $dataPlanovani ->idPP = (int)$_GET['osoba'];
        }

        $rezimUprav = RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV)  &&  isset($_GET['rezimuprav'])  &&  $_GET['rezimuprav'];

        $dataPlanovani->nactiDataProJson($obdobiOd, $obdobiDo, $rozdeleni, $vcetnePodrizenych, null, $rezimUprav);

        if (isset($_GET['planovaniPoSlotech'])  && (int)$_GET['planovaniPoSlotech']) {
            $dataPlanovani ->nactiSloty();
        }

        $dataPlanovani ->zapisData($outputObject);
    }


    /**
     * @param Database $database
     * @param int $obdobi
     * @param int $konecObdobi
     * @return string
     */
    public static function sqlSeznamRozdeleniProPlanovani($database, $obdobi, $konecObdobi)
    {
        $typRozdeleni = self::typRozdeleniProVyber();
        $jenVlastni = ($typRozdeleni == RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) // oprava 9.1.2019 - pokud je stejny typ rozdeleni jako ve filtru, tak bere nastaveni "jen vlastni" z filtru
            ? RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_JEN_VLASTNI)
            : 0;
        //$jenVlastni = RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_JEN_VLASTNI); - toto asi nefunguje

        if ($jenVlastni === 2  ||  $jenVlastni === 3) {
            $prava = Prava::vratPrava('kapacity');
            $sql = $prava ->sqlRozdeleni($database, 'Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.BarvaProVykresleni, Rozdeleni.Kalendar', (int)$typRozdeleni, $obdobi, $konecObdobi, $jenVlastni == 2);
            if ($sql instanceof SqlStatement) {
                $sql ->sqlText .= ' ORDER BY Popis';
            } else {
                $sql .= ' ORDER BY Popis';
            }
        } else {
            // TODO: upravit na parametry
            $sql =
                'SELECT Kod, Popis, BarvaProVykresleni, Kalendar FROM Rozdeleni ' .
                ' WHERE KodTypuRozdeleni = ' . $typRozdeleni .
                '   AND (DatumZruseni >= ' . $database ->dateToString($obdobi) . ' OR DatumZruseni IS NULL) ';

            $filtr = KapacityUtils::filtrRozdeleniProPlanovani($database, $typRozdeleni, $obdobi, $konecObdobi);
            if ($filtr) $sql .= " AND ($filtr) ";

            $sql .= ' ORDER BY Popis';
        }

        return $sql;
    }


    /**
     * Vraci json pole se seznamem rozdeleni podle zadaneho SQL dotazu
     * @param Database $database
     * @param string $sql
     * @param Parameters $params
     * @param string $selectedValue
     * @param bool $setRozdeleni
     * @return array
     * @throws DBException
     */
    public static function seznamRozdeleniJson($database, $sql, $params = null, $selectedValue = null, $setRozdeleni = false)
    {
        $seznamRozdeleni = [];

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani seznamu rozdeleni!');

        $foundSelected = false; // priznak, jestli je nektere rozdeleni vybrano
        $first = null;

        while ($row = $cursor ->fetch()) {
            if (!$first) {
                $first = $row[0]; // zapamatuje si prvni
            }

            $itemRozdeleni = [
                'kod' => StringUtils::jsonText($row[0], false),
                'popis' =>StringUtils::jsonText($row[1], false),
                'barva' => GuiUtils::getHtmlColor($row[2])
            ];

            if ($row[3]) {
                $itemRozdeleni['kalendar'] = StringUtils::jsonText($row[3]);
            }

            if ( isset($selectedValue)  &&  !strcasecmp($row[0], $selectedValue) ) { // doplneno isset kvuli warningu v PHP 8.1
                RonPortalSession::setRozdeleni($row[0]);
                $foundSelected = true;
            }

            $seznamRozdeleni[] = $itemRozdeleni;
        }
        $cursor ->close();

        if ($setRozdeleni  &&  !$foundSelected  &&  isset($first)) {
            // pokud nebylo vybrano zadne rozdeleni, tak nastavi prvni ze seznamu
            RonPortalSession::setRozdeleni($first);
        }

        return $seznamRozdeleni;
    }


    /**
     * @param Database $database
     * @param JsonOutputObject $outputObject
     * @throws DBException
     */
    public static function nactiHlavickuPlanovani($database, $outputObject)
    {
        $seznamObdobi = [];

        // Seznam obdobi
        $sql =
            'SELECT PocatekObdobi, KonecObdobi, Uzavreno, Popis FROM Obdobi ' .
            " WHERE Aplikace = 'ADS' ORDER BY PocatekObdobi DESC";

        $cursor = $database ->getSqlResult($sql);
        $obdobi = isset($_GET['obdobi']) ? DateUtils::urlStringToDate($_GET['obdobi']) : RonPortalSession::obdobi();
        $konecObdobi = null;

        while ($row = $cursor ->fetch()) {
            $pom_pocatek = (int)$database ->stringToTimestamp($row[0]);
            $pom_konec = (int)$database ->stringToTimestamp($row[1]);

            $popisT = TextManager::_('db_obdobi_popis_' . date('n', $pom_pocatek), '');
            $popis = $popisT
                ? sprintf($popisT, date('Y', $pom_pocatek))
                : $row[3];


            $itemObdobi = [
                'pocatek' => DateUtils::dateToUrlString($pom_pocatek),
                'konec' => $pom_konec,
                'uzavreno' => (int)$row[2],
                'popis' => StringUtils::jsonText($popis, false)
            ];

            if ($pom_pocatek === $obdobi) {
                $itemObdobi['selected'] = 1;
                $konecObdobi = $pom_konec;
            }

            $seznamObdobi[] = $itemObdobi;
        }
        $cursor ->close();

        $outputObject ->addData('obdobi', $seznamObdobi);
        $outputObject ->addData('vybraneObdobi', DateUtils::dateToUrlString($obdobi));

        // Seznam rozdeleni pro filtr
        $vybraneRozdeleni = isset($_GET['rozdeleni']) ? rawurldecode($_GET['rozdeleni']) : RonPortalSession::rozdeleni();
        //$vybraneRozdeleni = isset($_GET['rozdeleni']) ? StringUtils::fromUTF8($_GET['rozdeleni']) : RonPortalSession::rozdeleni();
        $sql = self::sqlSeznamRozdeleniProPlanovani($database, $obdobi, $konecObdobi);
        $outputObject ->addData('rozdeleni', self::seznamRozdeleniJson($database, $sql, null, $vybraneRozdeleni, true) );
        // pozor! vybrane rozdeleni se mohlo pri nacitani zmenit (pokud jej nenasel) - proto se bere ze session
        $outputObject ->addData('vybraneRozdeleni', StringUtils::jsonText(RonPortalSession::rozdeleni()));
        $outputObject ->addData('vcetnePodrizenych', (int)($_GET['vcetnepodrizenych'] ?? 1));

        $outputObject ->addData('csrfkey', Csrf::getCsrfKey('pk_r_' . $vybraneRozdeleni));

        // seznam rozdeleni pro prerazovani
        if (RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI)) {
            if (RonPortalSettings::getIntValue('kapacity_rozd_dle_prav')) {
                // uzivatel muze prerazovat jen na rozdeleni, na ktera ma sam pravo
                $sql = self::sqlSeznamRozdeleniProPlanovani($database, $obdobi, $konecObdobi);
                $params = null;
            } else {
                $params = $database->createParameters();
                $sql =
                    'SELECT Kod, Popis, BarvaProVykresleni, Kalendar FROM Rozdeleni ' .
                    ' WHERE KodTypuRozdeleni = ' . $params->addParameter(RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI)) .
                    '   AND (DatumZruseni >= ' . $params->addParameter($database->timestampToParam($obdobi)) . ' OR DatumZruseni IS NULL) ' .
                    ' ORDER BY Kod';
            }

            $outputObject ->addData('seznamRozdeleni', self::seznamRozdeleniJson($database, $sql, $params) );
        }

        if (RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2)) {
            $params = $database ->createParameters();
            $sql =
                'SELECT Kod, Popis, BarvaProVykresleni, Kalendar FROM Rozdeleni ' .
                ' WHERE KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2)) .
                '   AND (DatumZruseni >= ' . $params ->addParameter($database ->timestampToParam($obdobi)) . ' OR DatumZruseni IS NULL) ' .
                ' ORDER BY Kod';

            $items = self::seznamRozdeleniJson($database, $sql, $params);
            $seznamRozdeleni2 = [];
            foreach ($items as $item) {
                $seznamRozdeleni2[$item['kod']] = $item;
            }

            $outputObject ->addData('seznamRozdeleni2', $seznamRozdeleni2 );
        }

        if (RonPortalSettings::getIntValue(KapacitySettings::TLACITKO_PREPOCET)) {
            $outputObject ->addData('prepocet', RonPortalSettings::getIntValue(KapacitySettings::TLACITKO_PREPOCET));
        }

        if (RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV)) {
            $outputObject ->addData('povolitRezimUprav', 1);
        }

        if (RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI)) {
            $outputObject ->addData('povolitZmenuRozdeleni', 1);
        }

        if (RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2)) {
            $outputObject ->addData('povolitZmenuRozdeleni2', 1);

            $outputObject->addData('vyberRozdeleni2', RonPortalSettings::getIntValue('kapacity_vyberrozdeleni2'));
        }

        $outputObject ->addData('pocetZnakuPoznamky', RonPortalSettings::getIntValue('kapacity_znaku_poznamka')); // kolik znaku poznamky se ma zobrazit

        $velikost = (int)NastaveniUzivatele::getUserSettingsItem($database, 'pk_velikostlegendy', '1');
        if ($velikost !== 1) {
            $outputObject ->addData('velikostLegendy', $velikost);
        }

        $outputObject ->addData('velikostSouctu', (int)NastaveniUzivatele::getUserSettingsItem($database, 'pk_velikostsouctu', '0'));
    }

}
