<?php

namespace RonPortal\Kapacity;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\StringUtils;
use RonPortal\RonPortalSettings;

class PlanovaniKapacit_PrirazeniKalendare
{
    /** @var int */
    public $idPP;

    /** @var int */
    public $datumOd;
    /** @var int|null */
    public $datumDo;
    /** @var string */
    public $kod;


    public function getJson() {
        $result = [
            'kod' => StringUtils::jsonText($this ->kod),
            'datumOd' => date('j.n.Y', $this ->datumOd)
        ];

        if ($this ->datumDo) {
            $result['datumDo'] = date('j.n.Y', $this ->datumDo);
        }

        return $result;
    }

    /**
     * Nacte prirazene kalendare zadanym osobam pro jednotlive dny v zadanem obdobi
     * - podle prirazenych kalendaru
     * - podle prirazeneho rozdeleni (pokud je zadan typ rozdeleni pro planovani)
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param int|int[] $idPP
     * @return PlanovaniKapacit_PrirazeniKalendare[]
     * @throws DBException
     */
    public static function nactiKalendare($database, $idPP, $datumOd, $datumDo)
    {
        $result = [];

        $params = $database ->createParameters();

        if (RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI) > 0) {
            // je zadan typ rozdeleni pro planovani - kalendar se urci podle rozdeleni osoby
            $sql =
                'SELECT PracovniPomer.Id, RozdeleniOsob.PlatnostOd, RozdeleniOsob.PlatnostDo, Rozdeleni.Kalendar' .
                ' FROM RozdeleniOsob, Rozdeleni, PracovniPomer' .
                ' WHERE Rozdeleni.Kod = RozdeleniOsob.Rozdeleni' .
                '   AND Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI)) .
                '   AND RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                '   AND Rozdeleni.Kalendar IS NOT NULL' .
                '   AND RozdeleniOsob.RC = PracovniPomer.RC' .
                '   AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
                '   AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)';

            if (is_array($idPP)) {
                $sql .= ' AND (0 = 1';
                foreach ($idPP as $id) {
                    $sql .= ' OR PracovniPomer.Id = ' . $params ->addParameter($id);
                }
                $sql .= ')';
            } else {
                $sql .= ' AND PracovniPomer.Id = ' . $params ->addParameter($idPP);
            }

            $sql .=
                ' ORDER BY RozdeleniOsob.PlatnostOd';

        } else {
            // neni zadan typ rozdeleni pro planovani - kalendar se urci z karty zamestnance
            $sql =
                'SELECT PracovniPomer.Id, PlatnostOd, PlatnostDo, Kalendar' .
                ' FROM PrirazeniKalendare, PracovniPomer' .
                ' WHERE PrirazeniKalendare.RC = PracovniPomer.RC' .
                '   AND PrirazeniKalendare.OsobniCislo = PracovniPomer.OsobniCislo' .
                '   AND PrirazeniKalendare.DatumNastupu = PracovniPomer.DatumNastupu' .
                '   AND PrirazeniKalendare.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
                '   AND (PrirazeniKalendare.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR PrirazeniKalendare.PlatnostDo IS NULL)';

            if (is_array($idPP)) {
                $sql .= ' AND (0 = 1';
                foreach ($idPP as $id) {
                    $sql .= ' OR PracovniPomer.Id = ' . $params ->addParameter($id);
                }
                $sql .= ')';
            } else {
                $sql .= ' AND PracovniPomer.Id = ' . $params ->addParameter($idPP);
            }

            $sql .=
                ' ORDER BY PlatnostOd';
        }

        $cursor = $database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $kalendar = new self();

            $kalendar->idPP = (int)$row[0];
            $kalendar->datumOd = $database ->stringToTimestamp($row[1]);
            $kalendar->datumDo = $database ->stringToTimestamp($row[2]);
            $kalendar->kod = $row[3];

            if ($kalendar ->datumOd < $datumOd) {
                $kalendar ->datumOd = $datumOd;
            }
            if (!$kalendar ->datumDo || $kalendar ->datumDo > $datumDo) {
                $kalendar ->datumDo = $datumDo;
            }

            $result[] = $kalendar;
        }

        $cursor ->close();

        return $result;
    }
}
