<?php
/**
 * stanislav.elbl, 10.03.2021
 */

namespace RonPortal\Kapacity;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecordField;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalScriptEvents;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\RonPortalXMLEvents;
use RonPortal\Rozdeleni;
use RonPortal\System\AgentManager;


class PlanovaniKapacitRecord extends RonPortalDBRecord
{
    /** @var PracovniPomer */
    var $pp;

    /**#@+
     * @var DBRecordField
     */
    var $fDatum;
    var $fSmena;
    var $fSmena_Pocatek;
    var $fSmena_Konec;
    var $fSmena_Delka;
    var $fSmena_PreruseniOd;
    var $fSmena_PreruseniDo;

    var $fRozdeleni;
    var $fKodTypuRozdeleni;
    var $fRozdeleni2;
    var $fKodTypuRozdeleni2;

    var $fHarmonogram;

    var $fPuvodni_Rozdeleni;
    var $fPuvodni_KodTypu;
    var $fPuvodni_Rozdeleni2;
    var $fPuvodni_KodTypu2;

    var $fPuvodniSmena;
    var $fPuvodni_Pocatek;
    var $fPuvodni_Konec;
    var $fPuvodni_Delka;
    var $fPuvodni_PreruseniOd;
    var $fPuvodni_PreruseniDo;


    var $fPotvrzeno;
    var $fPuvodni_Potvrzeno;

    /** @var ?array     Parametry pro */
    protected $xmlParameters = null;

    /**#@-*/


    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param string|object $logDescription
     */
    function __construct($database, $pp, $logDescription = null)
    {
        parent::__construct($database, 'PlanovaniKapacit', $logDescription ?: 'Planovani kapacit');

        $this->pp = $pp;

        $this->addField('RC', '', true, 15, $pp->rc); // readonly = true
        $this->addField('OsobniCislo', '', true, 15, $pp->osobniCislo);
        $this->addField('DatumNastupu', 'timestamp', true, 0, $pp->datumNastupu);
        $this->addField('', '', true, 0, '   ' . $pp->jmenoPrijmeni . ', id ' . $pp->idPP);

        $this->fDatum = $this->addField('Datum', 'timestamp', true);
        $this->fSmena = $this->addField('Smena', '', true, 10);

        $this->fSmena_Pocatek = $this->addField('Smena_Pocatek', 'time', true);
        $this->fSmena_Konec = $this->addField('Smena_Konec', 'time', true);
        $this->fSmena_Delka = $this->addField('Smena_Delka', 'time', true);
        $this->fSmena_PreruseniOd = $this->addField('Smena_PreruseniOd', 'time', true);
        $this->fSmena_PreruseniDo = $this->addField('Smena_PreruseniDo', 'time', true);

        $this->fRozdeleni = $this->addField('Rozdeleni', '', true, 30);
        $this->fKodTypuRozdeleni = $this->addField('KodTypuRozdeleni', 'int', true);

        $this->fRozdeleni2 = $this->addField('Rozdeleni2', '', true, 30);
        $this->fKodTypuRozdeleni2 = $this->addField('KodTypuRozdeleni2', 'int', true);

        $this->fHarmonogram = $this ->addField('Harmonogram', '', true);

        $this->fPuvodniSmena = $this->addField('PuvodniPlan_Smena', '', true, 10);
        $this->fPuvodni_Pocatek = $this->addField('PuvodniPlan_Smena_Pocatek', 'time', true);
        $this->fPuvodni_Konec = $this->addField('PuvodniPlan_Smena_Konec', 'time', true);
        $this->fPuvodni_Delka = $this->addField('PuvodniPlan_Smena_Delka', 'time', true);
        $this->fPuvodni_PreruseniOd = $this->addField('PuvodniPlan_PreruseniOd', 'time', true);
        $this->fPuvodni_PreruseniDo = $this->addField('PuvodniPlan_PreruseniDo', 'time', true);

        $this->fPuvodni_Rozdeleni = $this->addField('PuvodniPlan_Rozdeleni', '', true, 30);
        $this->fPuvodni_KodTypu = $this->addField('PuvodniPlan_KodTypu', 'int', true);
        $this->fPuvodni_Rozdeleni2 = $this->addField('PuvodniPlan_Rozdeleni2', '', true, 30);
        $this->fPuvodni_KodTypu2 = $this->addField('PuvodniPlan_KodTypu2', 'int', true);

        $this ->fPotvrzeno = $this->addField('Potvrzeno', 'int', true, 0, 0); // povinne pole!
        $this ->fPuvodni_Potvrzeno = $this->addField('PuvodniPlan_Potvrzeno', 'int', true, 0, 0); // povinne pole!

        $field_u = $this->addField('Update_UserName', '', false, 25, RonPortalUtils::getLogUserName() );
        $field_u->updateLog = true; // jedna se o logovaci pole
        //$field_u->noInsert = true; - zapoznamkovano 19.4.2016 na pozadavek JS

        $field_t = $this->addField('Update_TimeStamp', 'timestamp', false, 25, time());
        $field_t->updateLog = true; // jedna se o logovaci pole
        //$field_t->noInsert = true;

        $params = $this ->database ->createParameters();
        $this->setWhere(
            'RC = ' . $params->addParameter($this->pp->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($this->pp->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($this->pp->datumNastupu)),
            $params);
    }


    function nastavRozdeleni2($kod, $rezimUprav = false)
    {
        if (isset($kod)  &&  $kod != '') {
            $this ->fKodTypuRozdeleni2 ->setNewValue( RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2) );
            $this ->fRozdeleni2 ->setNewValue($kod);

            if (!$rezimUprav) {
                $this ->fPuvodni_KodTypu2 ->setNewValue( RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2) );
                $this ->fPuvodni_Rozdeleni2 ->setNewValue($kod);
            }
        } else {
            $this ->fKodTypuRozdeleni2 ->setNewValue( null );
            $this ->fRozdeleni2 ->setNewValue( null );

            if (!$rezimUprav) {
                $this ->fPuvodni_KodTypu2 ->setNewValue( null );
                $this ->fPuvodni_Rozdeleni2 ->setNewValue( null );
            }
        }
    }


    function updateRecord($filtr = '', $filtrParams = null)
    {
        $this->executeXmlEvents(RonPortalXMLEvents::KAPACITY_BEFORE_POST, 'update', $this ->xmlParameters);

        $parametrSkriptu =
            'context=update' .
            ';idpp=' . $this ->pp ->idPP .
            ';datum=' . date('j.n.Y', $this ->fDatum ->value) .
            ';smena=' . ($this->fSmena->value ?? '');
        AgentManager::executeScriptEvents($this ->database, RonPortalScriptEvents::CATEGORY_KAPACITY, RonPortalScriptEvents::KAPACITY_BEFORE_POST, '"' . $parametrSkriptu . '"');

        $result = parent::updateRecord($filtr, $filtrParams);

        AgentManager::executeScriptEvents($this ->database, RonPortalScriptEvents::CATEGORY_KAPACITY, RonPortalScriptEvents::KAPACITY_AFTER_POST, '"' . $parametrSkriptu . '"');

        $this->executeXmlEvents(RonPortalXMLEvents::KAPACITY_AFTER_POST, 'update', $this ->xmlParameters);

        return $result;
    }


    function insertRecord()
    {
        $this->executeXmlEvents(RonPortalXMLEvents::KAPACITY_BEFORE_POST, 'insert', $this ->xmlParameters);

        $parametrSkriptu =
            'context=insert' .
            ';idpp=' . $this ->pp ->idPP .
            ';datum=' . date('j.n.Y', $this ->fDatum ->value) .
            ';smena=' . ($this->fSmena->value ?? '');
        AgentManager::executeScriptEvents($this ->database, RonPortalScriptEvents::CATEGORY_KAPACITY, RonPortalScriptEvents::KAPACITY_BEFORE_POST, '"' . $parametrSkriptu . '"');

        parent::insertRecord();

        AgentManager::executeScriptEvents($this ->database, RonPortalScriptEvents::CATEGORY_KAPACITY, RonPortalScriptEvents::KAPACITY_AFTER_POST, '"' . $parametrSkriptu . '"');
        $this->executeXmlEvents(RonPortalXMLEvents::KAPACITY_AFTER_POST, 'insert', $this ->xmlParameters);
    }


    function doDelete($filtr, $filtrParams = null)
    {
        $this->executeXmlEvents(RonPortalXMLEvents::KAPACITY_BEFORE_POST, 'delete', $this ->xmlParameters);

        $parametrSkriptu =
            'context=delete' .
            ';idpp=' . $this ->pp ->idPP .
            ';datum=' . date('j.n.Y', $this ->fDatum ->value) .
            ';smena=' . ($this->fSmena->value ?? '');
        AgentManager::executeScriptEvents($this ->database, RonPortalScriptEvents::CATEGORY_KAPACITY, RonPortalScriptEvents::KAPACITY_BEFORE_POST, '"' . $parametrSkriptu . '"');

        parent::doDelete($filtr, $filtrParams);

        AgentManager::executeScriptEvents($this ->database, RonPortalScriptEvents::CATEGORY_KAPACITY, RonPortalScriptEvents::KAPACITY_AFTER_POST, '"' . $parametrSkriptu . '"');
        $this->executeXmlEvents(RonPortalXMLEvents::KAPACITY_AFTER_POST, 'delete', $this ->xmlParameters);
    }


    /**
     * Vraci kod rozdeleni zadaneho typu prirazeneho osobe.
     * @param int $typRozdleni
     * @param int $den
     * @param int $nastaveniPrirazeni   0=nic nehladat-vraci null, 1=primarni, 2=jakekoliv prirazene
     * @return string
     * @throws \Ron\DB\DBException
     */
    private function vratRozdeleniOsoby($typRozdleni, $den, $nastaveniPrirazeni)
    {
        if (!$nastaveniPrirazeni) {
            return null;
        }

        // 1=primarni, 2=jakekoliv prirazene
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Rozdeleni' .
            ' FROM RozdeleniOsob ' .
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter($typRozdleni) .
            ' AND RC = ' . $params ->addParameter($this ->pp ->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($this ->pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->pp ->datumNastupu)) .
            ' AND PlatnostOd <= ' . $params ->addParameter($this ->database ->dateToParam($den)) .
            ' AND (PlatnostDo >= ' . $params ->addParameter($this ->database ->dateToParam($den)) . ' OR PlatnostDo IS NULL)';

        if ($nastaveniPrirazeni == 1) {
            $sql .= ' AND Hlavni = 1';
        }

        $sql .= 'ORDER BY PlatnostOd DESC, Rozdeleni';

        $result = null;

        $cursor = $this ->database ->getSqlResult($sql, $params);
        if ($row = $cursor ->fetch()) {
            $result = $row[0];
        }
        $cursor ->close();

        return $result;
    }


    /**
     * @param int $den
     * @param string $smena
     * @param bool $rezimUprav
     * @param string $editovaneRozdeleni
     * @param int $pocatek
     * @param int $konec
     * @param int $delka
     * @param int $preruseniOd
     * @param int $preruseniDo
     * @throws NeniPravoException
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function zapisSmenu($den, $smena, $rezimUprav = false, $editovaneRozdeleni = null, $pocatek = null, $konec = null, $delka = null, $preruseniOd = null, $preruseniDo = null)
    {
        $pravaEditace = Prava::vratPrava('kapacity_edit');

        $this ->xmlParameters = ['rozdeleni' => $editovaneRozdeleni];

        $this->fDatum->value = $den;
        $this->fSmena->setNewValue($smena);

        if (!$rezimUprav) {
            $this->fPuvodniSmena->setNewValue($smena);
        }

        if (isset($pocatek) || isset($konec) || isset($delka)) {
            // nastaveni variabilni smeny
            $this->fSmena_Pocatek->setNewValue($pocatek);
            $this->fSmena_Konec->setNewValue($konec);
            $this->fSmena_Delka->setNewValue($delka);

            if (!$rezimUprav) {
                $this->fPuvodni_Pocatek->setNewValue($pocatek);
                $this->fPuvodni_Konec->setNewValue($konec);
                $this->fPuvodni_Delka->setNewValue($delka);
            }
        }

        if (isset($preruseniOd)) {
            // nastaveni pevne prestavky
            $this ->fSmena_PreruseniOd ->setNewValue($preruseniOd);
            $this ->fSmena_PreruseniDo ->setNewValue($preruseniDo);

            if (!$rezimUprav) {
                $this ->fPuvodni_PreruseniOd ->setNewValue($preruseniOd);
                $this ->fPuvodni_PreruseniDo ->setNewValue($preruseniDo);
            }
        }

        $params = $this ->database ->createParameters();
        $filter =
            'Datum = ' . $params ->addParameter($this ->database ->dateToParam($den));

        if ($this->loadRecord($filter, $params)) {
            if ($pravaEditace ->prava != Prava::PRAVA_POUZE_VLASTNI  &&  isset($this->fRozdeleni->value)) {  // pro nastaveni "pouze vlastni" se rozdeleni nekontroluje
                if (!$pravaEditace->jePravoNaRozdeleni(RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI), $this->fRozdeleni->value)) {
                    throw new NeniPravoException();
                }
            } else if (!$pravaEditace->lzeZobrazitPP($this->pp->idPP, $den, $den)) {
                throw new NeniPravoException();
            }

            if ($this ->isChanged()) {
                KapacityUtils::zkontrolujSchvaleni($this ->database, $this ->pp, $den, $rezimUprav, $this->fRozdeleni->value);
                if (isset($this->fRozdeleni->originalValue)  &&  $this->fRozdeleni->value != $this->fRozdeleni->originalValue  &&  RonPortalSettings::getIntValue(KapacitySettings::ZPUSOB_SCHVALOVANI)) {
                    // pri zmene rozdeleni zkontroluje, jestli neni schvalene puvodni rozdeleni
                    KapacityUtils::zkontrolujSchvaleni($this ->database, $this ->pp, $den, $rezimUprav, $this->fRozdeleni->originalValue);
                }

                $this->updateRecord($filter, $params);
            }
        } else {
            if (!$pravaEditace->lzeZobrazitPP($this->pp->idPP, $den, $den)) {
                throw new NeniPravoException();
            }

            KapacityUtils::zkontrolujSchvaleni($this->database, $this->pp, $den, $rezimUprav, $this->fRozdeleni->value);

            if (!$this ->fRozdeleni ->value) {
                // pri vkladani noveho zaznamu automaticky priradi rozdeleni (primarni rozdeleni zamestnance)
                $rozdeleniAutomaticky = RonPortalSettings::getIntValue('kapacity_automatickyrozdeleni1');
                if ($rozdeleniAutomaticky === 3) {
                    $rozdeleni = $editovaneRozdeleni;
                } else if ($rozdeleniAutomaticky) {
                    $rozdeleni = $this->vratRozdeleniOsoby(
                        RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI),
                        $den,
                        $rozdeleniAutomaticky);
                } else {
                    $rozdeleni = null;
                }

                if (isset($rozdeleni)) {
                    $this ->fKodTypuRozdeleni ->setNewValue(RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI));
                    $this ->fRozdeleni ->setNewValue($rozdeleni);

                    if (!$rezimUprav) {
                        $this ->fPuvodni_KodTypu ->setNewValue(RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI));
                        $this ->fPuvodni_Rozdeleni ->setNewValue($rozdeleni);
                    }
                }
            }

            if (RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2)  &&  !$this ->fRozdeleni2 ->value) {
                // pri vkladani noveho zaznamu automaticky priradi rozdeleni2 (primarni rozdeleni zamestnance)
                $rozdeleni = $this ->vratRozdeleniOsoby(
                    RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2),
                    $den,
                    RonPortalSettings::getIntValue('kapacity_automatickyrozdeleni2'));

                if (isset($rozdeleni)) {
                    $this ->nastavRozdeleni2($rozdeleni, $rezimUprav);
                }
            }

            $this->insertRecord();
        }
    }


    /**
     * @param int $den
     * @param int $typRozdeleni
     * @param string $rozdeleni
     * @param bool  $nastavitRozdeleni2
     * @param bool  $rezimUprav
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function zapisRozdeleni($den, $typRozdeleni, $rozdeleni, $nastavitRozdeleni2, $rezimUprav)
    {
        $pravaeditace = Prava::vratPrava('kapacity_edit');

        $this->fDatum->value = $den;

        if ($rozdeleni == '') $rozdeleni = null;
        $typRozdeleni = isset($rozdeleni) ? $typRozdeleni : null;

        // priznak, jestli se ma zkontrolovat, jestli uz neni naplanovana smena
        // - kontrola se provadi pouze u rozdeleni1, pokud neni uzivatelem vypnuta
        $kontrolovatSmenu = false;

        if ($nastavitRozdeleni2) {
            $this->fRozdeleni2->setNewValue($rozdeleni);
            $this->fKodTypuRozdeleni2->setNewValue($typRozdeleni);

            if (!$rezimUprav) {
                $this->fPuvodni_Rozdeleni2->setNewValue($rozdeleni);
                $this->fPuvodni_KodTypu2->setNewValue($typRozdeleni);
            }
        } else {
            $kontrolovatSmenu = !RonPortalSettings::getIntValue('kapacity_cizi_zmena');

            $this->fRozdeleni->setNewValue($rozdeleni);
            $this->fKodTypuRozdeleni->setNewValue($typRozdeleni);

            if (!$rezimUprav) {
                $this->fPuvodni_Rozdeleni->setNewValue($rozdeleni);
                $this->fPuvodni_KodTypu->setNewValue($typRozdeleni);
            }
        }

        $params = $this ->database ->createParameters();
        $filter =
            'Datum = ' . $params ->addParameter($this ->database ->dateToParam($den));


        if ($this->loadRecord($filter, $params)) {
            // zaznam uz existuje
            if ($this ->isChanged()) {
                if ($kontrolovatSmenu && $this->fSmena->value) {
                    // uz je naplanovana smena => nelze zmenit rozdeleni
                    throw new \Exception(\Ron\ronText('Chyba_Nelze_zmenit_rozdeleni_je_smena'));
                }

                if (!$pravaeditace->lzeZobrazitPP($this->pp->idPP, $den, $den)) {
                    throw new NeniPravoException();
                }

                if (!$nastavitRozdeleni2) {
                    // meni se rozdeleni 1 => musi zkontrolovat schvaleni
                    // rozdeleni, na ktere presouvam nesmi byt schvaleno (a pokud se schvaluje po osobach, tak nesmi byt schvalena osoba)
                    KapacityUtils::zkontrolujSchvaleni($this->database, $this->pp, $den, $rezimUprav, $rozdeleni);

                    if (RonPortalSettings::getIntValue(KapacitySettings::ZPUSOB_SCHVALOVANI)) {
                        // a v pripade, ze se schvaluje po rozdeleni, tak nesmi byt schvaleno ani puvodni rozdeleni
                        // (pokud se schvaluje jen po jednotlivych osobach, tak se tato kontrola neprovadi)
                        KapacityUtils::zkontrolujSchvaleni($this->database, $this->pp, $den, $rezimUprav, $this ->fRozdeleni ->originalValue);
                    }
                }

                $this->updateRecord($filter, $params);
            }
        } else {
            if (!$pravaeditace->lzeZobrazitPP($this->pp->idPP, $den, $den)) {
                throw new NeniPravoException();
            }

            if ($this->isChanged()) {
                // rozdeleni, na ktere presouvam nesmi byt schvaleno
                KapacityUtils::zkontrolujSchvaleni($this->database, $this->pp, $den, $rezimUprav, $rozdeleni);

                $this->insertRecord();
            }
        }
    }


    /**
     * @param int $den
     * @param string $rozdeleni
     * @param bool $rezimUprav
     * @throws \Exception
     */
    function zapisRozdeleni1($den, $rozdeleni, $rezimUprav)
    {
        if (!RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI)) {
            throw new \Exception('Chyba! Prerazeni na jine rozdeleni neni povoleno v nastaveni aplikace!');
        }

        $this ->zapisRozdeleni($den, RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI), $rozdeleni, false, $rezimUprav);
    }


    /**
     * Zkontroluje, jestli je mozne zamestnanci priradit zadane rozdeleni2
     * @param int $den
     * @param string $rozdeleni
     * @throws \Exception
     */
    public function lzePriraditRozdeleni2($den, $rozdeleni)
    {
        switch (RonPortalSettings::getIntValue('kapacity_vyberrozdeleni2')) {
            case 1 : // jen vlastni
            case 2 : // vlastni a podrizena - musi byt v nadrazenem nebo v zadanem
            case 3 : // podrizena - musi byt v nadrazenem
                if (!Rozdeleni::jeVRozdeleni($this ->database, null, $this ->pp ->idPP,
                    RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2), $rozdeleni,
                    RonPortalSettings::getIntValue('kapacity_vyberrozdeleni2') - 1,
                    $den, $den)) {
                    throw new \Exception( \Ron\ronText('Zamestnanec_nema_prirazeno_rozdeleni_2_pk') );
                };
                break;

            case 0 :
                // vsechna
                break;
        }
    }


    /**
     * @param int $den
     * @param string $rozdeleni
     * @param bool $rezimUprav
     * @throws \Exception
     */
    function zapisRozdeleni2($den, $rozdeleni, $rezimUprav)
    {
        if (!RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2)) {
            throw new \Exception('Chyba! Prerazeni na jine rozdeleni 2 neni povoleno v nastaveni aplikace!');
        }

        $this ->lzePriraditRozdeleni2($den, $rozdeleni);

        $this ->zapisRozdeleni($den, RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2), $rozdeleni, true, $rezimUprav);
    }


    /**
     * Smaze smeny naplanovane na zadane obdobi. Vraci pocet smazanych/updatovanych zaznamu.
     * POZOR! Pokud se nezmeni zadny zaznam, vraci 0!
     * @param int $denOd
     * @param int $denDo
     * @param bool $rezimUprav
     * @param ?string $editovaneRozdeleni   editovane rozdeleni (nemusi byt zadano, protoze funkce se vola i z jinych agend nez planovani kapacit)
     * @return int
     * @throws NeniPravoException
     * @throws \Exception
     * @throws DBException
     */
    function smazNaplanovaneSmeny($denOd, $denDo, $rezimUprav, $editovaneRozdeleni = null)
    {
        // pokud je zmemeno rozdeleni, tak zaznam nemaze, ale pole nastavi na null!
        $this->fSmena->setNewValue(null);
        $this->fSmena_Pocatek->setNewValue(null);
        $this->fSmena_Konec->setNewValue(null);
        $this->fSmena_Delka->setNewValue(null);

        if (isset($editovaneRozdeleni)) {
            $this ->xmlParameters = ['rozdeleni' => $editovaneRozdeleni];
        }

        if (!$rezimUprav) {
            $this->fPuvodniSmena->setNewValue(null);

            $this->fPuvodni_Pocatek->setNewValue(null);
            $this->fPuvodni_Konec->setNewValue(null);
            $this->fPuvodni_Delka->setNewValue(null);
        }

        $params = $this ->database ->createParameters();
        $this->loadRecords(
            'Datum >= ' . $params ->addParameter($this ->database ->dateToParam($denOd)) .
            ' AND Datum <= ' . $params ->addParameter($this ->database ->dateToParam($denDo)),
            $params);

        $pravaEditace = Prava::vratPrava('kapacity_edit');

        $result = 0;

        while ($this->next()) {
            $doUpdate = false;

            if (isset($this->fRozdeleni->value)) {
                // je nastaveno rozdeleni (=prerazeni na jine stredisko) => smaze jen hodnoty poli
                if ($pravaEditace ->prava === Prava::PRAVA_POUZE_VLASTNI) {
                    // uzivatel s pravy "pouze vlastni" muze mazat vlastni zaznamy, i kdyz jsou prerazene na jine rozdeleni
                    if (!$pravaEditace ->lzeZobrazitPP($this->pp->idPP)) {
                        throw new NeniPravoException();
                    }
                } else if (!$pravaEditace->jePravoNaRozdeleni(RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI), $this->fRozdeleni->value)) {
                    throw new NeniPravoException();
                }

                $doUpdate = true; // provede update
            } else if ($rezimUprav && isset($this->fPuvodniSmena->value)) {
                // je nastavena smena v puvodnim planu => smaze jen hodnoty poli
                if (!$pravaEditace->lzeZobrazitPP($this->pp->idPP, $denOd, $denDo)) {
                    throw new NeniPravoException();
                }

                $doUpdate = true;
            } else {
                if (!$pravaEditace->lzeZobrazitPP($this->pp->idPP, $denOd, $denDo)) {
                    throw new NeniPravoException();
                }

                $deleteParams = $this ->database ->createParameters();
                $this->doDelete(
                    'Datum = ' . $deleteParams ->addParameter($this ->database ->dateToParam($this->fDatum->value)),
                    $deleteParams);

                $result++; // doslo ke smazani => zapocita zaznam
            }

            if ($doUpdate) {
                $updateParams = $this ->database ->createParameters();
                $updateStatus = $this->updateRecord(
                    'Datum = ' . $updateParams ->addParameter($this ->database ->dateToParam($this->fDatum->value)),
                    $updateParams);

                if ($updateStatus !== true) {
                    // doslo ke zmene zaznamu
                    $result++;
                }
            }
        }

        return $result;
    }


    /**
     * Smaze prerazeni na jine rozdeleni v zadanem obdobi. Pro tyto dny nesmi byt naplanovana smena!
     * POZOR! Pokud se nezmeni zadny zaznam, vraci 0!
     * @param int $denOd
     * @param int $denDo
     * @param bool $rezimUprav
     * @return bool
     * @throws \Exception
     */
    function smazRozdeleni($denOd, $denDo, $rezimUprav)
    {
        $this->fRozdeleni->setNewValue(null); // v rezimu uprav zaznam nemaze, ale nastavuje rozdeleni na null

        $params = $this ->database ->createParameters();
        $this->loadRecords(
            'Datum >= ' . $params ->addParameter($this ->database ->dateToParam($denOd)) .
            ' AND Datum <= ' . $params ->addParameter($this ->database ->dateToParam($denDo)),
            $params);

        $pravaEditace = Prava::vratPrava('kapacity_edit');

        while ($this->next()) {
            if (!$pravaEditace->lzeZobrazitPP($this ->pp ->idPP, $this->fDatum ->value, $this->fDatum ->value)) {
                throw new NeniPravoException();
            }

            if ($this ->fSmena ->value) {
                // standardne by nemelo nastat - kontrola se provadi predem
                throw new \Exception(\Ron\ronText('Chyba_Nelze_smazat_zaznam_je_smena'));
            }

            if ($this->fRozdeleni->originalValue) {
                KapacityUtils::zkontrolujSchvaleni($this->database, $this->pp, $this->fDatum->value, $rezimUprav, $this->fRozdeleni->originalValue);
            }

            $doDelete = true;
            $deleteParams = $this ->database ->createParameters();
            $filter = 'Datum = ' . $deleteParams ->addParameter($this ->database ->dateToParam($this->fDatum->value));

            if ($rezimUprav) {
                if ($this ->fPuvodni_Rozdeleni ->value  ||  $this ->fPuvodniSmena ->value) {
                    // v puvodnim planu je nastaveno rozdeleni nebo smena => provede jen update
                    $doDelete = false;
                    $this ->updateRecord($filter, $deleteParams);
                }
            }

            if ($doDelete) {
                $this ->doDelete($filter, $deleteParams);
            }
        }

        return true;
    }


    function nastavPotvrzeni($denOd, $denDo, $rezimUprav, $value)
    {
        $this ->fPotvrzeno ->setNewValue($value);
        if (!RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV)  ||  !$rezimUprav) {
            $this ->fPuvodni_Potvrzeno ->setNewValue($value);
        }

        $params = $this ->database ->createParameters();
        $this->loadRecords(
            'Datum >= ' . $params ->addParameter($this ->database ->dateToParam($denOd)) .
            ' AND Datum <= ' . $params ->addParameter($this ->database ->dateToParam($denDo)),
            $params);

        $pravaEditace = Prava::vratPrava('kapacity_edit');

        while ($this->next()) {
            if (isset($this->fRozdeleni->value)) {
                // je nastaveno rozdeleni (=prerazeni na jine stredisko) => zkontroluje prava na rozdeleni
                if (!$pravaEditace->jePravoNaRozdeleni(RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI), $this->fRozdeleni->value)) {
                    throw new NeniPravoException();
                }

                //$doUpdate = true; // provede update
            } else if (!$pravaEditace->lzeZobrazitPP($this->pp->idPP, $this ->fDatum ->value, $this ->fDatum ->value)) {
                throw new NeniPravoException();
            }

            $updateParams = $this ->database ->createParameters();
            $this->updateRecord('Datum = ' . $updateParams ->addParameter($this ->database ->dateToParam($this->fDatum->value)), $updateParams);
        }

        return true;
    }

}
