<?php
/**
 * stanislav.elbl, 02.09.2019
 * Planovani kapacit podle nadefinovanych podminek - planuje se pro jeden den a vybiraji se zamestnanci
 * pro pozadovane pozice.
 */

namespace RonPortal\Kapacity;

use Ron\DateUtils;
use Ron\DB\Cursor;
use Ron\DB\Database;
use Ron\GuiUtils;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\TextViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDny;
use RonPortal\Controls\FilterFormFilterObdobi;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\RozdeleniOsob;

class PlanovaniPodminekController extends JsonController
{
    /** @var Database */
    var $database;


    /**
     * PlanovaniPodminekController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    /**
     * @param string $kodRozdeleni
     * @param int $datum
     * @param int $typRozdeleni
     * @param bool $kontrolovatPrava
     * @return Rozdeleni
     * @throws NeniPravoException
     */
    private function vratRozdeleni($kodRozdeleni, $datum, $typRozdeleni = null, $kontrolovatPrava = true)
    {
        if (!isset($typRozdeleni)) {
            $typRozdeleni = KapacitySettings::typRozdeleniProVyber();
        }

        $rozdeleni = Rozdeleni::loadRozdeleni($this ->database, $typRozdeleni, $kodRozdeleni);

        if ($kontrolovatPrava) {
            Prava::nastavObdobiProPrava($datum);
            $prava = Prava::vratPrava('kapacity');

            if (!$prava->jePravoNaRozdeleni($typRozdeleni, $kodRozdeleni, $datum, $datum)) {
                throw new NeniPravoException();
            }
        }

        return $rozdeleni;
    }


    /**
     * Nacte lidi prerazene na zadane rozdeleni v zaznamu Planovani
     * @param integer $typRozdeleni
     * @param string $kodRozdeleni
     * @param int $datum
     * @param string $smena     Pokud je zadana, tak nacita jen lidi z teto smeny
     * @return Cursor
     * @throws \Ron\DB\DBException
     */
    private function nactiCiziOsoby($typRozdeleni, $kodRozdeleni, $datum, $smena = null)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT PracovniPomer.Id, PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu,'. // 0..3
            '  Osoba.Jmeno, Osoba.Prijmeni, Osoba.Titul,' . // 4..6
            '  PlanovaniKapacit.Datum, PlanovaniKapacit.Smena, PlanovaniKapacit.KodTypuRozdeleni, PlanovaniKapacit.Rozdeleni,' . // 7..10
            '  PlanovaniKapacit.KodTypuRozdeleni2, PlanovaniKapacit.Rozdeleni2,' . // 11,12
            '  Absence.DatumOd, Absence.DatumDo, Absence.Delka, Absence.CasOd, Absence.CasDo, Absence.OperaceDochazky,' . // 13..18
            '  OperaceDochazky.Popis, OperaceDochazky.BarvaProVykresleni' . // 19..20
            "\nFROM PlanovaniKapacit INNER JOIN PracovniPomer" .
            '   ON PracovniPomer.RC = PlanovaniKapacit.RC' .
            '   AND PracovniPomer.OsobniCislo = PlanovaniKapacit.OsobniCislo' .
            '   AND PracovniPomer.DatumNastupu = PlanovaniKapacit.DatumNastupu' .
            "\nINNER JOIN Osoba" .
            '   ON Osoba.RC = PlanovaniKapacit.RC' .
            "\nLEFT OUTER JOIN Absence" .
            '   ON Absence.RC = PracovniPomer.RC' .
            '   AND Absence.OsobniCislo = PracovniPomer.OsobniCislo' .
            '   AND Absence.DatumNastupu = PracovniPomer.DatumNastupu' .
            '   AND Absence.Schvaleni <> 2  AND  Absence.Schvaleni <> 4 ' . // nenacita zamitnute a zrusene
            '   AND Absence.DatumDo >= ' . $params ->addParameter($this ->database ->dateToParam($datum)) .
            '   AND Absence.DatumOd <= ' . $params ->addParameter($this ->database ->dateToParam($datum)) .
            "\nLEFT OUTER JOIN OperaceDochazky" .
            '   ON OperaceDochazky.Kod = Absence.OperaceDochazky' .
            // osoby ze zadaneho rozdeleni
            "\nWHERE PlanovaniKapacit.KodTypuRozdeleni = " . $params ->addParameter($typRozdeleni) .
            '  AND PlanovaniKapacit.Rozdeleni = ' . $params ->addParameter($kodRozdeleni) .
            '  AND PlanovaniKapacit.Datum = ' . $params ->addParameter($this ->database ->dateToParam($datum)) .
            (isset($smena) ? "\nAND PlanovaniKapacit.Smena = " . $params ->addParameter($smena) : '' ) .
            "\nAND PracovniPomer.Zamestnanec > 0 " . // nezobrazuje externi
            '  AND PracovniPomer.ZpracovatDochazku = 1 ' .
            ' ORDER BY PracovniPomer.Id'
        ;

        return $this ->database ->getSqlResult($sql, $params);
    }

    /**
     * @param integer $typRozdeleni
     * @param string $kodRozdeleni
     * @param int $datum
     * @param string $smena
     * @return Cursor
     */
    private function nactiVlastniOsoby($typRozdeleni, $kodRozdeleni, $datum, $smena = null)
    {
        // nacte lidi za zadaneho rozdeleni, kteri nejsou prerazeni
        // Pozor! Pro jeden PP muze nacist vice zaznamu (muze mit vice absenci)
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT PracovniPomer.Id, PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu,'. // 0..3
            '  Osoba.Jmeno, Osoba.Prijmeni, Osoba.Titul,' . // 4..6
            '  PlanovaniKapacit.Datum, PlanovaniKapacit.Smena, PlanovaniKapacit.KodTypuRozdeleni, PlanovaniKapacit.Rozdeleni,' . // 7..10
            '  PlanovaniKapacit.KodTypuRozdeleni2, PlanovaniKapacit.Rozdeleni2,' . // 11,12
            '  Absence.DatumOd, Absence.DatumDo, Absence.Delka, Absence.CasOd, Absence.CasDo, Absence.OperaceDochazky,' . // 13..18
            '  OperaceDochazky.Popis, OperaceDochazky.BarvaProVykresleni' . // 19..20
            "\nFROM RozdeleniOsob INNER JOIN PracovniPomer" .
            '   ON PracovniPomer.RC = RozdeleniOsob.RC' .
            '   AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo' .
            '   AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu' .
            "\nINNER JOIN Osoba" .
            '   ON Osoba.RC = PracovniPomer.RC' .
            "\nLEFT OUTER JOIN PlanovaniKapacit" .
            '   ON RozdeleniOsob.RC = PlanovaniKapacit.RC' .
            '   AND RozdeleniOsob.OsobniCislo = PlanovaniKapacit.OsobniCislo' .
            '   AND RozdeleniOsob.DatumNastupu = PlanovaniKapacit.DatumNastupu' .
            '   AND PlanovaniKapacit.Datum = ' . $params ->addParameter($this ->database ->dateToParam($datum)) .
            "\nLEFT OUTER JOIN Absence" .
            '   ON Absence.RC = PracovniPomer.RC' .
            '   AND Absence.OsobniCislo = PracovniPomer.OsobniCislo' .
            '   AND Absence.DatumNastupu = PracovniPomer.DatumNastupu' .
            '   AND Absence.Schvaleni <> 2  AND  Absence.Schvaleni <> 4 ' . // nenacita zamitnute a zrusene
            '   AND Absence.DatumDo >= ' . $params ->addParameter($this ->database ->dateToParam($datum)) .
            '   AND Absence.DatumOd <= ' . $params ->addParameter($this ->database ->dateToParam($datum)) .
            "\nLEFT OUTER JOIN OperaceDochazky" .
            '   ON OperaceDochazky.Kod = Absence.OperaceDochazky' .
            // osoby ze zadaneho rozdeleni
            "\nWHERE RozdeleniOsob.PlatnostOd <= " . $params ->addParameter($this ->database ->dateToParam($datum)) .
            '  AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($this ->database ->dateToParam($datum)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
            '  AND RozdeleniOsob.KodTypuRozdeleni = ' . $params ->addParameter($typRozdeleni) .
            '  AND RozdeleniOsob.Rozdeleni = ' . $params ->addParameter($kodRozdeleni) .
            // ma platny pracovni pomer
            '  AND PracovniPomer.DatumNastupu <= ' . $params ->addParameter($this ->database ->dateToParam($datum)) .
            '  AND (PracovniPomer.DatumUkonceni >= ' . $params ->addParameter($this ->database ->dateToParam($datum)) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
            // neni v planovani prerazen na jine rozdeleni
            '  AND PlanovaniKapacit.Rozdeleni IS NULL ' .
            (isset($smena) ? "\nAND PlanovaniKapacit.Smena = " . $params ->addParameter($smena) : '' ) .
            "\nAND PracovniPomer.Zamestnanec > 0 " . // nezobrazuje externi
            '  AND PracovniPomer.ZpracovatDochazku = 1 ' .
            ' ORDER BY PracovniPomer.Id'
        ;

        return $this ->database ->getSqlResult($sql, $params);
    }


    private function vratOsobaJson($row)
    {
        return [
            'id' => $row[0],
            'jmenoPrijmeni' => StringUtils::jsonText($row[5] . ' ' . $row[4] .($row[6] ? ', ' . $row[6] : '')),
            'osobniCislo' => StringUtils::jsonText($row[2]),
            'smena' => StringUtils::jsonText($row[8]),
            'rozdeleni2' => StringUtils::jsonText($row[12]),
            'absence' => []
        ];
    }

    /**
     * @param array $data
     * @param Cursor $cursor
     */
    private function nactiOsoby(&$data, $cursor)
    {
        $lastIdPP = null;
        while ($row = $cursor ->fetch()) {
            if ($lastIdPP !== $row[0]) {
                $data[] = $this ->vratOsobaJson($row);
                $lastIdPP = $row[0];
            }

            // Doplni k zaznamu absenci
            if ($row[13]) {
                $data[count($data) - 1]['absence'][] = [
                    'delka' => $row[15],
                    'datumOd' => date('j.n.Y', $this->database->stringToTimestamp($row[13])),
                    'datumDo' => date('j.n.Y', $this->database->stringToTimestamp($row[14])),
                    'casOd' => $row[16] ? date('G:i', $this->database->stringToTimestamp($row[16])) : '',
                    'casDo' => $row[17] ? date('G:i', $this->database->stringToTimestamp($row[17])) : '',
                    'kod' => StringUtils::jsonText($row[18]),
                    'popis' => StringUtils::jsonText($row[19]),
                    'barva' => GuiUtils::getHtmlColor($row[20])
                ];
            }
        }
    }

    /**
     * @param JsonOutputObject $output
     * @throws \Ron\DB\DBException
     */
    function nactiDataRozdeleni($output)
    {
        $datum = DateUtils::urlStringToDate($_GET['datum']);
        $rozdeleni = $this ->vratRozdeleni(StringUtils::fromUTF8($_GET['rozdeleni']), $datum);

        $data = [];

        $cursor = $this ->nactiVlastniOsoby($rozdeleni ->typ, $rozdeleni ->kod, $datum);
        $this ->nactiOsoby($data, $cursor);
        $cursor ->close();

        $cursor = $this ->nactiCiziOsoby($rozdeleni ->typ, $rozdeleni ->kod, $datum);
        $this ->nactiOsoby($data, $cursor);
        $cursor ->close();

        $output ->addData('osoby', $data);
    }


    /**
     * @param JsonOutputObject $output
     * @throws NeniPravoException
     * @throws \Exception
     */
    function nactiDataOsoby($output)
    {
        // "?menuid=planovanipodminek&action=nactiosobu&id={idPP}&datum={datum}&rozdeleni={kodRozdeleni}
        $idPP = (int)$_GET['id'];
        $rezimUprav = false;

        $pp = PracovniPomer::nactiPP($this ->database, $idPP);

        $datum = DateUtils::urlStringToDate($_GET['datum']);
        /** @var Rozdeleni $rozdeleni */
        //$rozdeleni = $this ->vratRozdeleni(StringUtils::fromUTF8($_GET['rozdeleni']), $datum); POZOR! Nemusi byt zadane!

        $typRozdeleni2 = RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2);

        $output ->addData('jmeno', StringUtils::jsonText($pp ->jmeno . ' ' . $pp ->prijmeni . ', ' . \Ron\ronText('OsCislo') . ' ' . $pp ->osobniCislo));

        // nacte pozice, ktere ma zamestnanec prirazene
        $rozdeleniOsoby = RozdeleniOsob::nactiRozdeleniOsoby($this ->database, $pp, $typRozdeleni2, $datum, $datum);
        $rozdeleniJson = [];
        foreach ($rozdeleniOsoby as $r) {
            $jsonRec = [
                'kod' => StringUtils::jsonText($r ->kod),
                'popis' => StringUtils::jsonText($r ->popis),
                'platnostOd' => StringUtils::jsonDate($r ->platnostOd),
                'platnostDo' => StringUtils::jsonDate($r ->platnostDo),
                'poznamka' => StringUtils::jsonText($r ->poznamka)
            ];

            $rozdeleniJson[] = $jsonRec;
        }


        $output ->addData('rozdeleni2', $rozdeleniJson);

        $datumOd = DateUtils::incDayStart($datum, -4);
        $datumDo = DateUtils::incDayStart($datum, 4);

        $output ->addData('datumOd', StringUtils::jsonDate($datumOd));
        $output ->addData('datumDo', StringUtils::jsonDate($datumDo));

        $smeny = PlanovaniKapacit_PlanovanaSmena::nactiSmeny($this ->database, $idPP, $datumOd, $datumDo, $rezimUprav, true);
        $smenyJson = [];
        foreach ($smeny as $smena) {
            $jsonS = $smena ->getJSON(true);
            $jsonS['barva'] = GuiUtils::getHtmlColor($smena ->barva);
            $jsonS['zkratka'] = StringUtils::jsonText($smena ->zkratka);
            $smenyJson[] = $jsonS;
        }

        $output ->addData('smeny', $smenyJson);

        $seznamAbsenci = PlanovaniKapacit_PlanovanaAbsence::nactiAbsence($this ->database,
            $idPP, $datumOd, $datumDo, true, $rezimUprav);
        $absenceJson = [];
        foreach ($seznamAbsenci as $absence) {
            $jsonA = $absence ->getJSON();
            $jsonA['barva'] = GuiUtils::getHtmlColor($absence ->barva);
            $jsonA['zkratka'] = StringUtils::jsonText($absence ->zkratka);
            $absenceJson[] = $jsonA;
        }

        $output ->addData('absence', $absenceJson);
    }


    /**
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function nactiPodrobnostiPodminky($output)
    {
        $kodRozdeleni2 = StringUtils::fromUTF8($_GET['rozdeleni2']);

        $datum = DateUtils::urlStringToDate($_GET['datum']);

        $typRozdeleni2 = RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2);
        /** @var Rozdeleni $rozdeleni */
        $rozdeleni2 = $this ->vratRozdeleni($kodRozdeleni2, $datum, $typRozdeleni2, false);

        $output ->addData('kod', StringUtils::jsonText($rozdeleni2 ->kod));
        $output ->addData('popis', StringUtils::jsonText($rozdeleni2 ->popis));

        // nacte seznam osob prirazenych na rozdeleni2
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Osoba.Jmeno, Osoba.Prijmeni, Osoba.Titul,' . // 0..2
            ' PracovniPomer.Id, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, PracovniPomer.DatumUkonceni,' . // 3..6
            ' RozdeleniOsob.PlatnostOd, RozdeleniOsob.PlatnostDo, RozdeleniOsob.Poznamka,' . // 7..9
            ' PlanovaniKapacit.Smena, PlanovaniKapacit.Rozdeleni, PlanovaniKapacit.Rozdeleni2,' . // 10..12
            ' Smena.ZkratkaProPlanovani, Smena.BarvaProVykresleni' . // 13,14
            ' FROM RozdeleniOsob INNER JOIN PracovniPomer' .
            '  ON PracovniPomer.RC = RozdeleniOsob.RC' .
            '  AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo' .
            '  AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu' .
            ' INNER JOIN Osoba ON Osoba.RC = PracovniPomer.RC' .
            // pripoji planovanou smenu v dany den ...
            ' LEFT OUTER JOIN PlanovaniKapacit' .
            '  ON PlanovaniKapacit.RC = PracovniPomer.RC' .
            '  AND PlanovaniKapacit.OsobniCislo = PracovniPomer.OsobniCislo' .
            '  AND PlanovaniKapacit.DatumNastupu = PracovniPomer.DatumNastupu' .
            '  AND PlanovaniKapacit.Datum = ' . $params ->addParameter($this ->database ->dateToParam($datum)) .
            // ... a podrobnosti smeny
            ' LEFT OUTER JOIN Smena' .
            '  ON Smena.Kod = PlanovaniKapacit.Smena' .

            ' WHERE RozdeleniOsob.KodTypuRozdeleni = ' . $params->addParameter($rozdeleni2 ->typ) .
            ' AND RozdeleniOsob.Rozdeleni = ' . $params ->addParameter($rozdeleni2 ->kod) .
            ' AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($this ->database ->dateToParam($datum)) .
            ' AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($this ->database ->dateToParam($datum)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
            ' AND PracovniPomer.DatumNastupu <= ' . $params ->addParameter($this ->database ->dateToParam($datum)) .
            ' AND (PracovniPomer.DatumUkonceni >= ' . $params ->addParameter($this ->database ->dateToParam($datum)) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .

            ' ORDER BY PlanovaniKapacit.Smena, RozdeleniOsob.PlatnostDo' // nejdrive ti co nejsou na zadne smene
            ;

        $cursor = $this ->database ->getSqlResult($sql, $params);
        //$osoby = [];    // json seznam

        /** @var int[] $seznamIdPP      Seznam nactenych idPP */
        $seznamIdPP = [];
        /** @var mixed[] $seznamOsob    Seznam nactenych osob indexovany podle idPP - pro snadnejsi doplneni absenci */
        $seznamOsob = [];
        while ($row = $cursor ->fetch()) {
            if ($row[10]) {
                $smena = [
                    'kod' => StringUtils::jsonText($row[10]),
                    'rozdeleni' => StringUtils::jsonText($row[11]),
                    'rozdeleni2' => StringUtils::jsonText($row[12]),
                    'zkratka' => StringUtils::jsonText($row[13]),
                    'barva' => GuiUtils::getHtmlColor($row[14])
                ];
            } else {
                $smena = null;
            }

            $osoba = [
                'idPP' => $row[3],
                'jmeno' => StringUtils::jsonText($row[1] . ' ' . $row[0]),
                'osobniCislo' => $row[4],
                'platnostOd' => StringUtils::jsonDate($this ->database ->stringToTimestamp($row[7])),
                'platnostDo' => StringUtils::jsonDate($this ->database ->stringToTimestamp($row[8])),
                'poznamka' => StringUtils::jsonText($row[9]),
                'smena' => $smena
            ];
            //$osoby[] = $osoba;

            $seznamIdPP[] = $row[3];
            $seznamOsob[$row[3]] = $osoba;
        }
        $cursor ->close();

        // nacte absence nalezenych osob
        $seznamAbsenci = PlanovaniKapacit_PlanovanaAbsence::nactiAbsence($this ->database,
            $seznamIdPP, $datum, $datum, false, false);
        foreach ($seznamAbsenci as $absence) {
            if (isset($seznamOsob[$absence ->idPP])) {
                $absenceJson = $absence ->getJSON(false);
                $absenceJson['barva'] = GuiUtils::getHtmlColor($absence ->barva);
                $absenceJson['zkratka'] = StringUtils::jsonText($absence ->zkratka);

                $seznamOsob[$absence ->idPP]['absence'] = $absenceJson;
            }
        }

        $osoby = array_values($seznamOsob);
        $output ->addData('osoby', $osoby);
    }


    /**
     * Zkontroluje, jestli je zadana pozice ($rozdeleni2) na zadanem stredisku/lince ($rozdeleni) pro dany den a smenu povolena
     * - to znamena, ze je naplanovana v podminkach na zadany den
     * @param Rozdeleni $rozdeleni
     * @param int $datum
     * @param string $kodSmeny
     * @param string $rozdeleni2
     * @throws \Exception
     */
    public function jePozicePovolena($rozdeleni, $datum, $kodSmeny, $rozdeleni2)
    {
        if ($kodSmeny === '') {
            $kodSmeny = null;
        };

        if ($rozdeleni2 === '') {
            $rozdeleni2 = null;
        }


        if (!isset($kodSmeny) && !isset($rozdeleni2)) {
            // neni co kontrolovat
            return;
        }

        // nacte vsechny platne sady
        $sadyPodminek = SadaPodminekPlanovani::nactiSadyPodminekProDen($this ->database, $rozdeleni ->typ, $rozdeleni ->kod, $datum);

        // a pokusi se v nich vyhledat podminku platnou pro zadanou smenu a rozdeleni2
        foreach ($sadyPodminek as $sada) {
            foreach ($sada->podminky as $podminka) {
                if ( (!isset($kodSmeny)  ||  ($podminka ->typPodminky == PodminkaPlanovaniKapacit::TYP_SMENA && $podminka ->smena === $kodSmeny) )
                    && (!isset($rozdeleni2)  ||  ($podminka ->typPodminky == PodminkaPlanovaniKapacit::TYP_SMENA  &&  $podminka ->rozdeleni === $rozdeleni2))) {
                    // nasel spravnou podminku => tj. na dane smene je pozice povolena
                    // pocet nekontroluje, ale mozna by se tu mohlo jeste zkontrolovat, ze maximalni pocet je null nebo >0
                    return;
                }
            }
        }

        throw new \Exception(\Ron\ronText('Pozice_neni_povolena'));
    }


    function zapisZaznamOsoby($idPP, $datum, $kodSmeny, $kodRozdeleni, $kodRozdeleni2, $rezimUprav)
    {
        $rozdeleni = $kodRozdeleni != '' ? $this ->vratRozdeleni($kodRozdeleni, $datum) : null;

        if (Obdobi::jeObdobiUzavreno($this ->database, 'ADS', $datum, $datum)) {
            throw new \Exception( \Ron\ronText('Obdobi_uzavreno') );
        }

        $pp = PracovniPomer::nactiPP($this ->database, $idPP);

        Prava::nastavObdobiProPrava($datum, $datum);

        if (!RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV)) {
            $rezimUprav = false;
        }

        // overi, jestli je pozice pro dany den povolena
        if ($rozdeleni  &&  $kodRozdeleni2 != '') {
            $this->jePozicePovolena($rozdeleni, $datum, $kodSmeny, $kodRozdeleni2);
        }

        $rec = new PlanovaniKapacitRecord($this->database, $pp, $this);

        // rozdeleni (stredisko)
        $rec ->fKodTypuRozdeleni ->setNewValue($rozdeleni ? $rozdeleni ->typ : null);
        $rec ->fRozdeleni ->setNewValue($rozdeleni ? $rozdeleni ->kod : null);
        if (!$rezimUprav) {
            $rec ->fPuvodni_KodTypu ->setNewValue($rozdeleni ? $rozdeleni ->typ : null);
            $rec ->fPuvodni_Rozdeleni ->setNewValue($rozdeleni ? $rozdeleni ->kod : null);
        }

        // rozdeleni 2 (linka)
        if ($kodRozdeleni2 != '') {
            // kontrola, jestli lze zamestnanci priradit zadane rozdeleni 2
            $rec ->lzePriraditRozdeleni2($datum, $kodRozdeleni2);
            // nastavi hodnoty
            $rec->fKodTypuRozdeleni2->setNewValue(RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2));
            $rec->fRozdeleni2->setNewValue($kodRozdeleni2);
            if (!$rezimUprav) {
                $rec->fPuvodni_KodTypu2->setNewValue(RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2));
                $rec->fPuvodni_Rozdeleni2->setNewValue($kodRozdeleni2);
            }
        } else {
            $rec->fKodTypuRozdeleni2 ->setNewValue(null);
            $rec->fRozdeleni2->setNewValue(null);
            if (!$rezimUprav) {
                $rec->fPuvodni_KodTypu2 ->setNewValue(null);
                $rec->fPuvodni_Rozdeleni2->setNewValue(null);
            }
        }

        $rec ->zapisSmenu($datum, $kodSmeny != '' ? $kodSmeny : null, $rezimUprav);
    }

    /**
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function zapisOsobu($output)
    {
        $idPP = (int)$_POST['id'];
        $datum = DateUtils::urlStringToDate($_POST['datum']);
        /** @var Rozdeleni $rozdeleni */
        $kodRozdeleni = StringUtils::fromUTF8($_POST['rozdeleni']);
        $kodRozdeleni2 = StringUtils::fromUTF8($_POST['rozdeleni2']);
        $kodSmeny = StringUtils::fromUTF8($_POST['smena']);

        $this ->zapisZaznamOsoby($idPP, $datum, $kodSmeny, $kodRozdeleni, $kodRozdeleni2, isset($_POST['rezimuprav']) && $_POST['rezimuprav']);
    }


    /**
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function nactiDataProKopii($output)
    {
        // get: datum1 (zdrojove), datum2 (cilove), rozdeleni
        $datum1 = DateUtils::urlStringToDate($_GET['datum1']);
        $datum2 = DateUtils::urlStringToDate($_GET['datum2']);
        $rozdeleni = $this ->vratRozdeleni(StringUtils::fromUTF8($_GET['rozdeleni']), $datum2);

        // nacte smeny
        $params = $this->database->createParameters();
        $sql =
            'SELECT SmenaProDenKalendare.Smena, SmenaProDenKalendare.Den, Smena.Popis, Smena.PocatekSmeny' .
            ' FROM Rozdeleni, SmenaProDenKalendare, Smena' .
            ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter($rozdeleni ->typ) .
            '  AND Rozdeleni.Kod = ' . $params ->addParameter($rozdeleni ->kod) .
            '  AND SmenaProDenKalendare.Kalendar = Rozdeleni.Kalendar' .
            '  AND (SmenaProDenKalendare.Den = ' . $params ->addParameter($this ->database ->dateToParam($datum1)) .
            '   OR SmenaProDenKalendare.Den = ' . $params ->addParameter($this ->database ->dateToParam($datum2)) . ')' .
            '  AND Smena.Kod = SmenaProDenKalendare.Smena' .
            ' ORDER BY SmenaProDenKalendare.Den, Smena.PocatekSmeny, Smena.Kod';

        $vsechny = ['kod' => '', 'popis' => \Ron\ronText('vsechny_smeny')];
        $smeny1 = [$vsechny];
        $smeny2 = [];

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $datum = $this ->database ->stringToTimestamp($row[1]);

            $smena = [
                'kod' => StringUtils::jsonText($row[0]),
                'popis' => StringUtils::jsonText($row[2])
            ];

            if ($datum === $datum1) {
                $smeny1[] = $smena;
            }
            // bez else, datum muze byt stejne pro zdroj i cil!
            if ($datum === $datum2) {
                $smeny2[] = $smena;
            }
        }
        $cursor ->close();

        $output ->addData('smeny1', $smeny1);
        $output ->addData('smeny2', $smeny2);
    }


    function zkopirujZaznamOsoby($row, $datum2, $smena2, $rezimUprav)
    {
//        'SELECT PracovniPomer.Id, PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu,'. // 0..3
//        '  Osoba.Jmeno, Osoba.Prijmeni, Osoba.Titul,' . // 4..6
//        '  PlanovaniKapacit.Datum, PlanovaniKapacit.Smena, PlanovaniKapacit.KodTypuRozdeleni, PlanovaniKapacit.Rozdeleni,' . // 7..10
//        '  PlanovaniKapacit.KodTypuRozdeleni2, PlanovaniKapacit.Rozdeleni2,' . // 11,12
//        '  Absence.DatumOd, Absence.DatumDo, Absence.Delka, Absence.CasOd, Absence.CasDo, Absence.OperaceDochazky,' . // 13..18
//        '  OperaceDochazky.Popis, OperaceDochazky.BarvaProVykresleni' . // 19..20
        $this ->zapisZaznamOsoby($row[0], $datum2, $smena2 ?: $row[8], $row[10], $row[12], $rezimUprav);
    }


    /**
     * zkopiruje plan zadaneho rozdeleni ze dne $datum1 do $datum2
     * @param int $typRozdeleni
     * @param string $kodRozdeleni
     * @param int $datum1
     * @param int $datum2
     * @param string $smena1 Pokud je zadana smena1, tak kopiruje jen tuto smenu
     * @param string $smena2 Musi byt zadana, pokud je zadana smena1
     * @param bool $rezimUprav
     */
    function zkopirujPlan($typRozdeleni, $kodRozdeleni, $datum1, $datum2, $smena1, $smena2, $rezimUprav)
    {
        $cursor = $this ->nactiVlastniOsoby($typRozdeleni, $kodRozdeleni, $datum1, $smena1);
        while ($row = $cursor ->fetch()) {
            $this->zkopirujZaznamOsoby($row, $datum2, $smena2, $rezimUprav);
        }

        $cursor = $this ->nactiCiziOsoby($typRozdeleni, $kodRozdeleni, $datum1, $smena1);
        while ($row = $cursor ->fetch()) {
            $this->zkopirujZaznamOsoby($row, $datum2, $smena2, $rezimUprav);
        }
    }

    /**
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function kopie($output)
    {
        // get: datum1 (zdrojove), datum2 (cilove), rozdeleni, smena1 (nepovinne), smena2 (nepovinne)
        $datum1 = DateUtils::urlStringToDate($_GET['datum1']);
        $datum2 = DateUtils::urlStringToDate($_GET['datum2']);
        $rozdeleni = $this ->vratRozdeleni(StringUtils::fromUTF8($_GET['rozdeleni']), $datum2);
        $rezimUprav = isset($_POST['rezimuprav']) && $_POST['rezimuprav'];

        $smena1 = StringUtils::fromUTF8($_GET['smena1']);
        if (trim($smena1) == '') $smena1 = null;

        if (isset($smena1)) {
            $smena2 = StringUtils::fromUTF8($_GET['smena2']);
            if (trim($smena2) == '') $smena2 = null;
        } else {
            $smena2 = null;
        }

        Prava::nastavObdobiProPrava($datum2);
        $prava = Prava::vratPrava('kapacity');

//        if (!$prava->jePravoNaRozdeleni($rozdeleni ->typ, $rozdeleni ->kod, $datum2, $datum2)) {
//            throw new NeniPravoException();
//        }

        // nacte seznam podrizenych rozdeleni
        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT TypPodrizene, KodPodrizene FROM HierarchieRozdeleni' .
            ' WHERE TypNadrazene = ' . $params ->addParameter($rozdeleni ->typ) .
            '  AND KodNadrazene = ' . $params ->addParameter($rozdeleni ->kod),
            $params
        );

        while ($row = $cursor ->fetch()) {
            if ($prava->jePravoNaRozdeleni($row[0], $row[1], $datum2, $datum2)) {
                $this->zkopirujPlan($row[0], $row[1], $datum1, $datum2, $smena1, $smena2, $rezimUprav);
            }
        }
    }


    /**
     * @param string $action
     * @param JsonOutputObject $output
     */
    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'datarozdeleni' :
                $this ->nactiDataRozdeleni($output);
                break;

            case 'nactiosobu' :
                $this ->nactiDataOsoby($output);
                break;

            case 'nactipodminku' :
                $this ->nactiPodrobnostiPodminky($output);
                break;

            case 'zapisosobu' :
                $this ->zapisOsobu($output);
                break;

            case 'nactidataprokopii' :
                $this ->nactiDataProKopii($output);
                break;

            case 'kopie' :
                $this ->kopie($output);
                break;
        }
    }


    public function showForm()
    {
        HtmlHeaders::getInstance() ->addCss('css/s_kapacity.css');
        HtmlHeaders::getInstance() ->addScript('script/planovani_podminek.js');
        $this ->addTranslationsScript('planovanipodminek');

        HtmlHeaders::getInstance() ->setTitle(\Ron\ronText('RON_Portal') . ' - ' . \Ron\ronText('Planovani_podminek'));

        //$typRozdeleni = KapacitySettings::typRozdeleniProVyber();
        $prava = Prava::vratPrava('kapacity');

        $filterForm = new KapacityFilterForm($this ->database, 'planovanipodminek');
        //$filterForm = new RonPortalFilterForm($this ->database, 'planovanipodminek', true);
        //$filterForm ->nastavObdobi(true, 'ADS', false); // jenOtevrene = false

        $filterForm ->header = \Ron\ronText('Planovani_podminek');
//        $filterForm ->f_pravarozdeleni = $prava;
//        $filterForm ->nastavRozdeleni(true, $typRozdeleni, 0, '', true);
        //$filterForm ->nastavDatum(true);
        $filterForm ->addFilter((new FilterFormFilterObdobi($filterForm)) ->setRequired(true), true);
//        $filterForm ->nastavObdobi(true, 'ADS', false, true);
//        $filterForm ->zobrazitDnyObdobi = true;
        $filterForm ->addFilter(new FilterFormFilterDny($filterForm, 'datum'));

        $filterForm ->loadData();

        Model::addViewObject($filterForm);

        $rozdeleni = Rozdeleni::loadRozdeleni($this ->database,
            $filterForm ->getFilterValue('rozdeleni', 'typRozdeleni'),
            $filterForm ->getFilterValue('rozdeleni')
        );

        //$obdobi = Obdobi::vratObdobi($this ->database, 'ADS', $filterForm ->vybraneObdobiOd);
        $datum = $filterForm ->getFilterValue('datum');

        Prava::nastavObdobiProPrava($datum);

        if (!$prava ->jePravoNaRozdeleni($rozdeleni ->typ, $rozdeleni ->kod, $filterForm ->getOmezitObdobiOd(), $filterForm ->getOmezitObdobiDo())) {
            MessageManager::addErrorMessage(\Ron\ronText('Nemate_pravo'));

            return new TextViewObject('');
        }

        $form = new PlanovaniPodminekFormular($this ->database);
        $form ->loadData($rozdeleni, $datum, $filterForm->vcetnePodrizenychRozdeleni);

        return $form;
    }


    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('kapacity.ini');

        if (isset($_GET['action'])) {
            switch ($_GET['action']) {
                case 'translations' :
                    return $this->translations();
            }

            return parent::handleAction($action);
        }

        return $this ->showForm();
    }
}

