<?php
/**
 * stanislav.elbl, 10.03.2021
 */

namespace RonPortal\Kapacity;

use Ron\StringUtils;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;
use RonPortal\Schvaleni;

abstract class SchvaleniKapacit extends Schvaleni
{
    /** @var int */
    public $schvaleniUprav1;
    /** @var int */
    public $schvaleniUprav2;
    /** @var int */
    public $schvaleniUprav3;


    /**
     * @param int $uroven
     * @return PravaNaOsoby
     */
    public function vratPravoProUroven($uroven)
    {
        return Prava::vratPrava('kapac_schval' . $uroven);
    }


    public function jeSchvalenoPlanovani($datum, $rezimUprav)
    {
        if (!$rezimUprav) return $this->jeSchvalenoDatum($datum);

        return $this ->schvaleniUprav1 >= $datum  ||  $this ->schvaleniUprav2 >= $datum  ||  $this ->schvaleniUprav3 >= $datum;
    }


    public function lzeEditovatKapacity($datum, $rezimUprav)
    {
        $lzeEditovatSchvalene = RonPortalSettings::getIntValue(KapacitySettings::EDITACE_SCHVALENYCH);
        if (!$lzeEditovatSchvalene) {
            return !$this->jeSchvalenoPlanovani($datum, $rezimUprav);
        }

        $schvalenaUroven = $this ->vratUrovenSchvaleniKapacit($datum, $rezimUprav);

        if (!$schvalenaUroven) return true;
        if ($schvalenaUroven > 2) return false; // schvaleno nejvyssi urovni => nelze

        if ($this ->lzeSchvalovatDen(3, $datum)) {
            return true; // uzivatel ma pravo schvalovat uroven 3 => muze editovat
        }

        if ($schvalenaUroven > 1) return false;

        if ($this ->lzeSchvalovatDen(2, $datum)) {
            return true; // uzivatel ma pravo schvalovat uroven 2 => muze editovat
        }

        return false; // schvaleno urovni 1 a uzivatel nema pravo schvalovat vyssi uroven
    }

    /**
     * @param int $datum
     * @param bool $rezimUprav
     * @return int
     */
    function vratUrovenSchvaleniKapacit($datum, $rezimUprav)
    {
        if (!$rezimUprav) {
            return $this->vratUrovenSchvaleni($datum);
        }

        if ($this ->schvaleniUprav3 >= $datum) {
            return 3;
        }
        if ($this ->schvaleniUprav2 >= $datum) {
            return 2;
        }
        if ($this ->schvaleniUprav1 >= $datum) {
            return 1;
        }

        return 0;
    }


    /**
     * Vraci schvaleni zadane urovne pro zadany rezim ($rezimUprav)
     * @param int  $level
     * @param bool $rezimUprav
     * @return int
     */
    function vratSchvaleni($level, $rezimUprav)
    {
        switch ($level) {
            case 3 :
                return $rezimUprav ? $this ->schvaleniUprav3 : $this ->schvaleni3;
            case 2 :
                return $rezimUprav ? $this ->schvaleniUprav2 : $this ->schvaleni2;
            default :
                return $rezimUprav ? $this ->schvaleniUprav1 : $this ->schvaleni1;
        }
    }


    function posledniSchvaleni($rezimUprav)
    {
        if ($rezimUprav) {
            return max($this->schvaleniUprav1, $this->schvaleniUprav2, $this->schvaleniUprav3);
        } else {
            return max($this->schvaleni1, $this->schvaleni2, $this->schvaleni3);
        }
    }


    /**
     * Vraci datum, do ktereho je schvalen puvodni plan. Musi byt schvalen minimalne nastavenou urovni
     * @return int
     */
    public function getSchvaleniPuvodnihoPlanu()
    {
        /** @var int $result */
        $result = null;

        $minSchvaleni = RonPortalSettings::getIntValue(KapacitySettings::MIN_SCHVALENI_PRO_UPRAVY);

        if ($minSchvaleni) {
            if ($this ->schvaleni1  &&  $minSchvaleni == 1) {
                $result = $this ->schvaleni1;
            }

            if ($this ->schvaleni2  &&  $minSchvaleni <= 2) {
                if (!$result  ||  $result < $this ->schvaleni2) {
                    $result = $this ->schvaleni2;
                }
            }

            if ($this ->schvaleni3) {
                if (!$result  ||  $result < $this ->schvaleni3) {
                    $result = $this ->schvaleni3;
                }
            }
        }

        return $result;
    }


    /**
     * Vraci HTML class pro zobrazeni priznaku schvaleni (podle nejvyssiho schvaleni v danem dni)
     * @param int  $datum
     * @param bool $rezimUprav
     * @return string
     */
    public function getHtmlClassKapacity($datum, $rezimUprav)
    {
        $uroven = $this ->vratUrovenSchvaleniKapacit($datum, $rezimUprav);
        if (!$uroven) return '';

        return 'schvaleni' . $uroven;
    }


    public function getJsonData($rezimUprav) {
        if ($rezimUprav) {
            return [
                'schvaleni1Str' => StringUtils::jsonDate($this ->schvaleniUprav1),
                'schvaleni2Str' => StringUtils::jsonDate($this ->schvaleniUprav2),
                'schvaleni3Str' => StringUtils::jsonDate($this ->schvaleniUprav3),
                'schvaleniPuvodnihoPlanuStr' =>StringUtils::jsonDate($this ->getSchvaleniPuvodnihoPlanu())
            ];
        } else {
            return [
                'schvaleni1Str' => StringUtils::jsonDate($this ->schvaleni1),
                'schvaleni2Str' => StringUtils::jsonDate($this ->schvaleni2),
                'schvaleni3Str' => StringUtils::jsonDate($this ->schvaleni3)
            ];
        }
    }
}
