<?php
/**
 * stanislav.elbl, 02.03.2018
 * Formular schvaleni v pasivnim rezimu
 */

namespace RonPortal\Kapacity;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\TextManager;
use RonPortal\Obdobi;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;

class SchvaleniKapacitForm extends BaseViewObject
{
    /** @var Database */
    public $database;

    /** @var bool */
    public $rezimUprav;

    /** @var PlanovaniKapacit */
    public $data;

    /** @var bool[]     Priznak, jestli muze schvalovat jednotlive urovne */
    public $pravaSchvaleni;
    /** @var bool[]     Priznak, jestli muze odschvalit jednotlive urovne */
    public $pravaOdschvaleni;

    /** @var int[]      Minimalni datum schvaleni pro kazdou uroven */
    public $minSchvaleni;
    /** @var int[]      Maximalni ... */
    public $maxSchvaleni;


    public $lzeSchvalovat;

    /**
     * SchvaleniKapacitForm constructor.
     * @param Database $database
     * @param bool $rezimUprav
     */
    public function __construct($database, $rezimUprav)
    {
        parent::__construct('ronportal/kapacity/view_schvaleni.php');
        $this ->database = $database;
        $this ->rezimUprav = $rezimUprav;
    }


    private function nactiDataPodleOsob()
    {
        foreach ($this ->data ->seznamOsob as $osoba) {
            // pro kazdou uroven zkontroluje, jestli muze uzivatel schvalovat alespon jednu osobu
            if ($osoba ->lzeSchvalit1) {
                $this ->pravaSchvaleni[1] = true;
                if (!$osoba ->schvaleni1 || $osoba ->schvaleni1 < $this ->minSchvaleni[1]) {
                    $this ->minSchvaleni[1] = $osoba ->schvaleni1;
                }
                if ($osoba ->schvaleni1 > $this ->maxSchvaleni[1]) {
                    $this ->maxSchvaleni[1] = $osoba ->schvaleni1;
                }
            }

            if ($osoba ->lzeSchvalit2) {
                $this ->pravaSchvaleni[2] = true;
                if (!$osoba ->schvaleni2 || $osoba ->schvaleni2 < $this ->minSchvaleni[2]) {
                    $this ->minSchvaleni[2] = $osoba ->schvaleni2;
                }
                if ($osoba ->schvaleni2 > $this ->maxSchvaleni[2]) {
                    $this ->maxSchvaleni[2] = $osoba ->schvaleni2;
                }
            }

            if ($osoba ->lzeSchvalit3) {
                $this ->pravaSchvaleni[3] = true;
                if (!$osoba ->schvaleni3 || $osoba ->schvaleni3 < $this ->minSchvaleni[3]) {
                    $this ->minSchvaleni[3] = $osoba ->schvaleni3;
                }
                if ($osoba ->schvaleni3 > $this ->maxSchvaleni[3]) {
                    $this ->maxSchvaleni[3] = $osoba ->schvaleni3;
                }
            }

            // kontrola prava pro "odschvaleni"
            $pom = !RonPortalSettings::getIntValue(KapacitySettings::ZAKAZAT_SNIZIT_SCHVALENI); // jestli je/neni zapnuto omezeni snizeni data schvaleni

            if ($osoba ->lzeSchvalit3) {
                if ($osoba ->schvaleni3 > 0) {
                    $this ->pravaOdschvaleni[3] = true;
                }
                $pom = true;
            }

            if ($osoba ->lzeSchvalit2) {
                if ($pom && $osoba ->schvaleni2 > 0) {
                    // jen pokud ma pravo na vyssi uroven nebo je tato kontrola vypnuta
                    $this ->pravaOdschvaleni[2] = true;
                }
                $pom = true;
            }

            if ($osoba ->lzeSchvalit1 && $pom && $osoba ->schvaleni1 > 0) {
                $this ->pravaOdschvaleni[1] = true;
            }
        }
    }


    private function nactiDataPodleRozdeleni()
    {
        $this ->minSchvaleni[1] = $this ->maxSchvaleni[1] = $this ->data ->schvaleniRozdeleni
            ? ($this ->rezimUprav ? $this ->data ->schvaleniRozdeleni ->schvaleniUprav1 : $this ->data ->schvaleniRozdeleni ->schvaleni1)
            : null;
        $this ->minSchvaleni[2] = $this ->maxSchvaleni[2] = $this ->data ->schvaleniRozdeleni
            ? ($this ->rezimUprav ? $this ->data ->schvaleniRozdeleni ->schvaleniUprav2 : $this ->data ->schvaleniRozdeleni ->schvaleni2)
            : null;
        $this ->minSchvaleni[3] = $this ->maxSchvaleni[3] = $this ->data ->schvaleniRozdeleni
            ? ($this ->rezimUprav ? $this ->data ->schvaleniRozdeleni ->schvaleniUprav3 : $this ->data ->schvaleniRozdeleni ->schvaleni3)
            : null;

        $prava = Prava::vratPrava('kapac_schval1');
        $this ->pravaSchvaleni[1] = $prava ->jePravoNaRozdeleni($this ->data ->kodTypuRozdeleni, $this ->data ->rozdeleni);

        $prava = Prava::vratPrava('kapac_schval2');
        $this ->pravaSchvaleni[2] = $prava ->jePravoNaRozdeleni($this ->data ->kodTypuRozdeleni, $this ->data ->rozdeleni);

        $prava = Prava::vratPrava('kapac_schval3');
        $this ->pravaSchvaleni[3] = $prava ->jePravoNaRozdeleni($this ->data ->kodTypuRozdeleni, $this ->data ->rozdeleni);

        // kontrola prava pro "odschvaleni"
        $pom = !RonPortalSettings::getIntValue(KapacitySettings::ZAKAZAT_SNIZIT_SCHVALENI); // jestli je/neni zapnuto omezeni snizeni data schvaleni

        if ($this ->pravaSchvaleni[3]) {
            if ($this ->minSchvaleni[3] > 0) {
                $this ->pravaOdschvaleni[3] = true;
            }
            $pom = true;
        }

        if ($this ->pravaSchvaleni[2]) {
            if ($pom && $this ->minSchvaleni[2] > 0) {
                // jen pokud ma pravo na vyssi uroven nebo je tato kontrola vypnuta
                $this ->pravaOdschvaleni[2] = true;
            }
            $pom = true;
        }

        if ($this ->pravaSchvaleni[1] && $pom && $this ->minSchvaleni[1] > 0) {
            $this ->pravaOdschvaleni[1] = true;
        }
    }

    /**
     * Nacte aktualni hodnoty ze seznamu osob (prava, datumy)
     * @param PlanovaniKapacit $data
     */
    public function load($data)
    {
        $this ->data = $data;

        $this ->pravaSchvaleni = [ 1 => false, 2 => false, 3 => false];
        $this ->pravaOdschvaleni = [ 1 => false, 2 => false, 3 => false];
        $this ->minSchvaleni = [ 1 => $data ->datumDo, 2 => $data ->datumDo, 3 => $data ->datumDo];
        $this ->maxSchvaleni = [ 1 => null, 2 => null, 3 => null];

        if (RonPortalSettings::getIntValue(KapacitySettings::ZPUSOB_SCHVALOVANI)) {
            $this->nactiDataPodleRozdeleni();
        } else {
            $this->nactiDataPodleOsob();
        }

        $this ->lzeSchvalovat =
            $this ->pravaSchvaleni[1] || $this ->pravaSchvaleni[2] || $this ->pravaSchvaleni[3]
            || $this ->pravaOdschvaleni[1] || $this ->pravaOdschvaleni[2] || $this ->pravaOdschvaleni[3];
    }


    public function getSeznamPP()
    {
        $result = '';
        foreach ($this ->data ->seznamOsob as $osoba) {
            $result .= $osoba ->idPP . ';';
        }
        return $result;
    }


    public static function prevedDatumSchvaleni($htmlName, $minDate, $maxDate)
    {
        if (trim($_POST[$htmlName]) === '') {
            return null;
        }

        try {
            $result = DateUtils::dateStringToTimestamp($_POST[$htmlName]);
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e, $htmlName);
            return false;
        }

        if ($result < $minDate  ||  $result > $maxDate) {
            MessageManager::addErrorMessage(
                sprintf(TextManager::_('Datum_musi_byt_v_rozsahu'), date('j.n.Y', $minDate), date('j.n.Y', $maxDate)),
                $htmlName);
            return false;
        }

        return $result;
    }



    /**
     * Zpracuje post - nacte zadane hodnoty a zapise je pro vsechny osoby
     * @param Database $database
     * @param PlanovaniKapacit $data
     * @return bool
     */
    /*
    private static function schvaleniOsob($database, $data)
    {
        $rezimUprav = isset($_POST['rezimUprav'])  &&  (int)$_POST['rezimUprav'];

        foreach ($data->seznamOsob as $osoba) {
            if ($osoba ->lzeSchvalit1 && $_POST['zmenitschvaleni1']) {
                // bylo odeslano schvaleni 1 a mam pravo na tuto osobu
                if ($_POST['schvaleni1']) {
                    $schvaleni1 = self::prevedDatumSchvaleni('schvaleni1', $data ->datumOd, $data ->datumDo);
                    if (!$schvaleni1) return false;
                } else {
                    $schvaleni1 = null;
                }

                if (RonPortalSettings::getIntValue(KapacityUtils::SETTINGS_ZAKAZAT_SNIZIT_SCHVALENI) && $schvaleni1 < $osoba ->schvaleni1 && !($osoba ->lzeSchvalit2 || $osoba ->lzeSchvalit3)) {
                    MessageManager::addErrorMessage(\Ron\ronText('Nelze_snizit_schvaleni_PK'));
                } else {
                    $osoba ->nastavSchvaleni($database, $data->datumOd, 1, $schvaleni1, $rezimUprav);
                }
            }

            if ($osoba ->lzeSchvalit2 && $_POST['zmenitschvaleni2']) {
                // bylo odeslano schvaleni 2 a mam pravo na tuto osobu
                if ($_POST['schvaleni2']) {
                    $schvaleni2 = self::prevedDatumSchvaleni('schvaleni2', $data ->datumOd, $data ->datumDo);
                    if (!$schvaleni2) return false;
                } else {
                    $schvaleni2 = null;
                }

                if (RonPortalSettings::getIntValue(KapacityUtils::SETTINGS_ZAKAZAT_SNIZIT_SCHVALENI) && $schvaleni2 < $osoba ->schvaleni2 && !$osoba ->lzeSchvalit3) {
                    MessageManager::addErrorMessage(\Ron\ronText('Nelze_snizit_schvaleni_PK'));
                } else {
                    $osoba ->nastavSchvaleni($database, $data->datumOd, 2, $schvaleni2, $rezimUprav);
                }
            }

            if ($osoba ->lzeSchvalit3 && $_POST['zmenitschvaleni3']) {
                // bylo odeslano schvaleni 3 a mam pravo na tuto osobu
                if ($_POST['schvaleni3']) {
                    $schvaleni3 = self::prevedDatumSchvaleni('schvaleni3', $data ->datumOd, $data ->datumDo);
                    if (!$schvaleni3) return false;
                } else {
                    $schvaleni3 = null;
                }

                $osoba ->nastavSchvaleni($database, $data->datumOd, 3, $schvaleni3, $rezimUprav);
            }

        }

        return true;
    }
    */


    /**
     * Zpracuje post - nacte zadane hodnoty a zapise je
     * @param Database $database
     * @throws Exception
     */
    private static function schvaleniRozdeleni($database)
    {
        $rezimUprav = isset($_POST['rezimUprav'])  &&  (int)$_POST['rezimUprav'];
        $kodTypuRozdeleni = KapacitySettings::typRozdeleniProVyber();
        $kodRozdeleni = $_POST['rozdeleni'];

        $rozdeleni = Rozdeleni::loadRozdeleni($database, $kodTypuRozdeleni, $kodRozdeleni);

        if (!$kodRozdeleni) {
            throw new Exception(TextManager::_('Nejdrive_vyberte_rozdeleni_pro_schvaleni'));
        }

        $obdobi = Obdobi::vratObdobi($database, 'ADS', (int)$_POST['obdobi']);
        if ($obdobi ->uzavreno) {
            throw new Exception( TextManager::_('Obdobi_uzavreno') );
        }

        Prava::nastavObdobiProPrava($obdobi ->zacatek, $obdobi ->konec);

        // kvuli volani xml udalosti vytvori dbrecord, ale samotne schvaleni probiha jen pres agenta
        if ($_POST['zmenitschvaleni1']) {
            $datumSchvaleni1 = self::prevedDatumSchvaleni('schvaleni1', $obdobi ->zacatek, $obdobi ->konec);
            if ($datumSchvaleni1 !== false) {
                $pravaSchvaleni1 = Prava::vratPrava('kapac_schval1');
                if ($pravaSchvaleni1->jePravoNaRozdeleni($rozdeleni->typ, $rozdeleni->kod, $datumSchvaleni1 ?: $obdobi ->zacatek, $datumSchvaleni1 ?: $obdobi->konec)) {
                    SchvaleniPlanuRozdeleniDBRecord::schvalKapacity($database, $rozdeleni, 1, $obdobi ->zacatek, $rezimUprav, $datumSchvaleni1);
                }
            }
        }

        if ($_POST['zmenitschvaleni2']) {
            $datumSchvaleni2 = self::prevedDatumSchvaleni('schvaleni2', $obdobi ->zacatek, $obdobi ->konec);
            if ($datumSchvaleni2 !== false) {
                $pravaSchvaleni2 = Prava::vratPrava('kapac_schval2');
                if ($pravaSchvaleni2->jePravoNaRozdeleni($rozdeleni->typ, $rozdeleni->kod, $datumSchvaleni2 ?: $obdobi ->zacatek, $datumSchvaleni2 ?: $obdobi->konec)) {
                    SchvaleniPlanuRozdeleniDBRecord::schvalKapacity($database, $rozdeleni, 2, $obdobi ->zacatek, $rezimUprav, $datumSchvaleni2);
                }
            }
        }

        if ($_POST['zmenitschvaleni3']) {
            $datumSchvaleni3 = self::prevedDatumSchvaleni('schvaleni3', $obdobi ->zacatek, $obdobi ->konec);
            if ($datumSchvaleni3 !== false) {
                $pravaSchvaleni3 = Prava::vratPrava('kapac_schval3');
                if ($pravaSchvaleni3->jePravoNaRozdeleni($rozdeleni->typ, $rozdeleni->kod, $datumSchvaleni3 ?: $obdobi ->zacatek, $datumSchvaleni3 ?: $obdobi->konec)) {
                    SchvaleniPlanuRozdeleniDBRecord::schvalKapacity($database, $rozdeleni, 3, $obdobi ->zacatek, $rezimUprav, $datumSchvaleni3);
                }
            }
        }
    }


    /**
     * Zpracuje post - nacte zadane hodnoty a zapise je - pouziva se pouze pro schvaleni celeho rozdeleni.
     * Pokud se schvaluji jednotlive osoby, tak se pouziva AJAX.
     * @param Database $database
     * @throws Exception
     */
    public static function provedSchvaleni($database)
    {
        if (!$_POST['ulozitschvaleni']) {
            return;
        }

        if (RonPortalSettings::getIntValue('kapacity_zpusobschvalovani')) {
            // hromadne schvaleni celeho rozdeleni
            self::schvaleniRozdeleni($database);
        } else {
            // schvalovani po jednotlivych osobach
            //self::schvaleniOsob($database, $data);
            // Uz se nepouziva, probiha pres AJAX po jednotlivych osobach
            // Zde jen pro jistotu chybova hlaska
            throw new Exception('Chyba ve funkci provedSchvaleni - nepodporovana metoda schvalovani po osobach!');
        }
    }
}
