<?php

namespace RonPortal\Kapacity;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use RonPortal\PravaNaOsoby;
use RonPortal\Rozdeleni;

class SchvaleniPlanuRozdeleniDBRecord extends SchvaleniKapacitAbstractRecord
{
    /** @var Rozdeleni */
    public $rozdeleni;

    /**
     * @param Database $database
     * @param Rozdeleni $rozdeleni
     * @param int $obdobi
     * @param string|object $logDescription
     */
    public function __construct($database, $rozdeleni, $obdobi, $logDescription = 'Planovani kapacit')
    {
        parent::__construct($database, 'SchvaleniPlanuRozdeleni', $obdobi, $logDescription);

        $this ->rozdeleni = $rozdeleni;

        $this ->addField('KodTypuRozdeleni', 'int', true, 0, $rozdeleni ->typ); // readonly = true
        $this ->addField('Rozdeleni', '', true, 30, $rozdeleni ->kod); // readonly = true

        $this ->fSchvaleni1 = $this ->addField('Schvaleni1', 'date', true);
        $this ->fSchvalil1 = $this ->addField('Schvalil1', '', true, 25);
        $this ->fSchvalil1 ->updateLog = true;
        $this ->fCas_Schvaleni1 = $this ->addField('Cas_Schvaleni1', 'timestamp', true);
        $this ->fCas_Schvaleni1 ->updateLog = true;

        $this ->fSchvaleni2 = $this ->addField('Schvaleni2', 'date', true);
        $this ->fSchvalil2 = $this ->addField('Schvalil2', '', true, 25);
        $this ->fSchvalil2 ->updateLog = true;
        $this ->fCas_Schvaleni2 = $this ->addField('Cas_Schvaleni2', 'timestamp', true);
        $this ->fCas_Schvaleni2 ->updateLog = true;

        $this ->fSchvaleni3 = $this ->addField('Schvaleni3', 'date', true);
        $this ->fSchvalil3 = $this ->addField('Schvalil3', '', true, 25);
        $this ->fSchvalil3 ->updateLog = true;
        $this ->fCas_Schvaleni3 = $this ->addField('Cas_Schvaleni3', 'timestamp', true);
        $this ->fCas_Schvaleni3 ->updateLog = true;

        $this ->fSchvaleniUprav1 = $this ->addField('SchvaleniUprav1', 'date', true);
        $this ->UpravySchvalil1 = $this ->addField('UpravySchvalil1', '', true, 25);
        $this ->UpravySchvalil1 ->updateLog = true;
        $this ->Cas_SchvaleniUprav1 = $this ->addField('Cas_SchvaleniUprav1', 'timestamp', true);
        $this ->Cas_SchvaleniUprav1 ->updateLog = true;

        $this ->fSchvaleniUprav2 = $this ->addField('SchvaleniUprav2', 'date', true);
        $this ->UpravySchvalil2 = $this ->addField('UpravySchvalil2', '', true, 25);
        $this ->UpravySchvalil2 ->updateLog = true;
        $this ->Cas_SchvaleniUprav2 = $this ->addField('Cas_SchvaleniUprav2', 'timestamp', true);
        $this ->Cas_SchvaleniUprav2 ->updateLog = true;

        $this ->fSchvaleniUprav3 = $this ->addField('SchvaleniUprav3', 'date', true);
        $this ->UpravySchvalil3 = $this ->addField('UpravySchvalil3', '', true, 25);
        $this ->UpravySchvalil3 ->updateLog = true;
        $this ->Cas_SchvaleniUprav3 = $this ->addField('Cas_SchvaleniUprav3', 'timestamp', true);
        $this ->Cas_SchvaleniUprav3 ->updateLog = true;

        $params = $database ->createParameters();
        $this ->setWhere(
            ' KodTypuRozdeleni = ' . $params ->addParameter($rozdeleni ->typ) .
            ' AND Rozdeleni = ' . $params->addParameter($rozdeleni ->kod) .
            " AND Aplikace = 'ADS'" .
            ' AND PocatekObdobi = ' . $params ->addParameter($database ->dateToParam($obdobi)),
            $params);
    }


    /**
     * @param PravaNaOsoby $pravo
     * @return bool
     */
    public function zkontrolujPravo($pravo)
    {
        return $pravo ->jePravoNaRozdeleni($this ->rozdeleni ->typ, $this ->rozdeleni ->kod);
    }


    /**
     * @param int $uroven
     * @param int $datumDo
     * @throws Exception
     * @throws DBException
     */
    public function overKontroly($uroven, $datumDo)
    {
        throw new Exception('Schvaleni rozdeleni se musi provest agentem!');
    }


    /**
     * @param int $uroven
     * @param bool $rezimUprav
     * @param int $datumDo
     * @return string
     */
    public function getParametrSkriptu($uroven, $rezimUprav, $datumDo)
    {
        return
            "KodTypuRozdeleni={$this ->rozdeleni ->typ};Rozdeleni={$this ->rozdeleni ->kod};" .
            parent::getParametrSkriptu($uroven, $rezimUprav, $datumDo);
    }


    public function getAgentParameters($uroven, $rezimUprav, $hodnota)
    {
        return '2 -subDiv:' . $this ->rozdeleni ->typ . '/' . $this ->rozdeleni ->kod  . parent::getAgentParameters($uroven, $rezimUprav, $hodnota);
    }


    /**
     * @param Database $database
     * @param Rozdeleni $rozdeleni
     * @param int $uroven
     * @param int $obdobi
     * @param bool $rezimUprav
     * @param int $hodnota  Datum, do kdy se ma schvalit
     * @throws Exception
     */
    public static function schvalKapacity($database, $rozdeleni, $uroven, $obdobi, $rezimUprav, $hodnota)
    {
        $rec = new static($database, $rozdeleni, $obdobi);
        $rec->zapisSchvaleni($uroven, $rezimUprav, $hodnota);
    }
}
