<?php
/**
 * pavel.rudol, 29.4.2016 11:10
 * Seznam zadosti o zastup zamestnancu
 */

namespace RonPortal\Kapacity;


use Exception;
use Ron\Controls\Browser;
use Ron\Controls\BrowserField;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\FilterFormFilterItem;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;


class ZastupujiciZamestnanecField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //pokud je osoba prirazena, zobrazi prijmeni
        if ($rowData['col-osoba-prijmeni']) {
            return $rowData['col-osoba-prijmeni'] . ' ' . $rowData['col-osoba-jmeno'];
        }

        //pokud neni prirazena osoba a zaznam je schvalen, zobrazi odkaz na prirazeni osoby
        if ($rowData['col-typschvaleni'] == 1) {
            $prava = Prava::vratPrava('zastupy_prideleni');
            $datum = $this ->database ->stringToTimestamp($rowData['col-datum']);

            if ($prava->jePravoNaRozdeleni($rowData['col-kodtypurozdeleni'], $rowData['col-kodrozdeleni'], $datum, $datum)) {
                return "<a href='?menuid=zastup&action=priradit&formaction=edit&record=" . $rowData[0] . "'>" . TextManager::_('Zastup_priradit') . "</a>";
            }
        }

        return '';
    }

}

/**
 * Odkaz na schvaleni - podle prav
 * Class SchvaleniZastupuField
 * @package RonPortal\Kapacity
 */
class SchvaleniZastupuField extends BrowserField
{
    function getHtmlTitle($rowNumber, $recordId, $rowData)
    {
        switch ($rowData['col-typschvaleni']) {
            case 1 :
                return TextManager::_('Zastup_schvaleni');
            case 2 :
                return TextManager::_('Zastup_zamitnuti');
            case 3 :
                return TextManager::_('Zastup_zruseni');
        }

        return parent::getHtmlTitle($rowNumber, $recordId, $rowData);
    }

    function getDataCellClass($value, $rowNumber, $recordId, $rowData)
    {
        switch ($rowData['col-typschvaleni']) {
            case 1 :
                return 'zastup_textschvaleni';
            case 2 :
                return 'zastup_textzamitnuti';
            case 3 :
                return 'zastup_textzruseni';
        }

        return parent::getDataCellClass($value, $rowNumber, $recordId, $rowData);
    }
}


class SchvaleniZastupuEditField extends BrowserField
{
    /** @var  PravaNaOsoby */
    var $prava;

    /** @var ZastupyBrowser */
    var $browser;

    public $inputsCount = 0;


    /**
     * @param Browser $browser
     * @param PravaNaOsoby $prava
     * @param string $caption
     */
    function __construct($browser, $prava, $caption)
    {
        parent::__construct($browser ->database);

        $this ->header =
            "<span id='schvaleniZastupuMenu_Header'>$caption</span>";

        $this ->browser = $browser;

        $this ->prava = $prava;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $schvaleno = ($rowData['col-typschvaleni'] == 1) ? 1 : 0;

        $hint = '';
        if ($schvaleno) {
            $hint = $rowData['col-schvalil'];
            if ($rowData['col-schvaleni-datum']) {
                $hint .= ' (' . DBUtils::getFormatedValue($this ->database, 'finedatetime', $rowData['col-schvaleni-datum']) . ')';
            }
        }

        if ($rowData['col-typschvaleni'] != 3  &&  $this ->prava ->jePravoNaRozdeleni($rowData['col-kodtypurozdeleni'], $rowData['col-kodrozdeleni'], $this ->database ->stringToTimestamp($rowData['col-datum']))) {
            // neni zruseno a muze schvalovat
            $index = $this ->inputsCount;
            $this ->inputsCount++;

            return
                "<input type='hidden' name='NastavitSchvaleni_{$rowData[0]}' value='1' />" .
                "<input type='hidden' name='idZastup_{$index}' value='{$rowData[0]}'/>" .
                "<input type='checkbox' class='schvaleni_zastupu_checkbox' name='Schvaleni_{$rowData[0]}' " .
                "id='Schvaleni_{$rowData[0]}' data-original-value='$schvaleno' data-id-zastupu='{$rowData[0]}'" .
                ($schvaleno ? " checked='checked'" : '') .
                ( $hint ? " title='$hint'" : '' ) .
                "/>\n";
        }

        return '<span' . ( $hint ? " title='$hint'" : '' ) . '>' . TextManager::_($schvaleno ? 'Str_Yes' : 'Str_No') . '</span>';
    }


    /**
     * @param int $fieldpos
     * @param string $url
     * @return string
     */
    function getFooterText($fieldpos, $url = null)
    {
        if ($this ->inputsCount) {
            return "<input type='checkbox' id='zastupy_schvaleni_ALL' title='' />\n";
        }

        return '';
    }

}


class ZruseniZastupuField extends BrowserField
{
    /** @var  PravaNaOsoby */
    var $prava;

    /**
     * @param Database $database
     * @param PravaNaOsoby $prava
     */
    function __construct($database, $prava)
    {
        parent::__construct($database);
        $this->prava = $prava;
        $this->code = 'col-delete';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-idpp']) {
            // je prirazena osoba, nejde smazat
            return '';
        }

        if ($rowData['col-typschvaleni']) {
            // jiz je schvaleno/zamitnuto, nejde smazat
            return '';
        }

        // zkontroluje prava na editaci zastupu
        if ($this->prava->jePravoNaRozdeleni($rowData['col-kodtypurozdeleni'], $rowData['col-kodrozdeleni'], $this ->database ->stringToTimestamp($rowData['col-datum']), $this ->database ->stringToTimestamp($rowData['col-datum']))) {
            return GuiUtils::changeLocationQuest('<i class="inline-glyph glyph-button fa glyph-delete"></i>', TextManager::_('Zastup_opravdu_zrusit'),
                '?menuid=zastup&action=zrusitzastup&record=' . $rowData[0]);
        }

        return '';
    }
}

class ZastupyBrowser extends RonPortalBrowser
{
    private $schvalovani;

    private $pravaVlozit;
    private $pravaSchvalit;
    private $pravaPriradit;
    private $pravaZobrazit;
    private $pravaZrusit;

    private $schvaleniZastupuEditField;

    /**
     * ZastupyBrowser constructor.
     * @param Database $database
     * @param bool $schvalovani
     * @throws Exception
     */
    public function __construct($database, $schvalovani = false)

    {
        parent::__construct($database, 'zastup');

        $this->title = TextManager::_('Zastup_zadosti');

        //$this->browserUrl = '?menuid=zastup';
        $this->code = 'browserzastupy';
        $this->tableId = 'seznamzastupu';

        $this->pravaVlozit = Prava::vratPrava('zastupy_vlozeni');
        $this->pravaSchvalit = Prava::vratPrava('zastupy_schvaleni');
        $this->pravaPriradit = Prava::vratPrava('zastupy_prideleni');
        $this->pravaZobrazit = Prava::vratPrava('zastupy_prohl');
        $this->pravaZrusit = Prava::vratPrava('zastupy_zruseni');

        $this->schvalovani = $schvalovani;
        $this->showFooter = $schvalovani;

        $this->sqlSelectPart =
            'ZadostOZastup.ID'; // 0


        $this->sqlFromPart =
            'ZadostOZastup LEFT JOIN Osoba ON (ZadostOZastup.RC = Osoba.RC) ' .
            ' LEFT JOIN PracovniPomer ON (PracovniPomer.RC = ZadostOZastup.RC AND PracovniPomer.OsobniCislo = ZadostOZastup.OsobniCislo AND PracovniPomer.DatumNastupu = ZadostOZastup.DatumNastupu) ' .
            ' LEFT JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = ZadostOZastup.KodTypuRozdeleni AND Rozdeleni.Kod = ZadostOZastup.Rozdeleni) ' .
            ' LEFT JOIN Rozdeleni Pozice ON (Pozice.KodTypuRozdeleni = ZadostOZastup.KodTypuRozdeleni2 AND Pozice.Kod = ZadostOZastup.Rozdeleni2) ' .
            ' LEFT JOIN DuvodZastupu ON (DuvodZastupu.Kod = ZadostOZastup.DuvodZadosti)' .
            ' LEFT JOIN Kategorie PoziceKategorie ON (PoziceKategorie.Kod = ZadostOZastup.KodKategorie AND PoziceKategorie.TypKategorie = ZadostOZastup.TypKategorie)';

        $this->primaryField = 0;

        $this->addField()
            ->setSqlFieldName('ZadostOZastup.Datum') ->setHeaderText('Zastup_datum')
            ->setCode('col-datum')
            ->setAlign('right') ->setOrdering(true) ->setFormat('date') ->setLink('?menuid=zastup&action=detail');

        $this->addField()
            ->setSqlFieldName('ZadostOZastup.CasOd') ->setHeaderText('Zastup_cas_od')
            ->setCode('col-casod')
            ->setAlign('right') ->setOrdering(true) ->setFormat('time');

        $this->addField()
            ->setSqlFieldName('ZadostOZastup.CasDo') ->setHeaderText('Zastup_cas_do')
            ->setCode('col-casdo')
            ->setAlign('right') ->setOrdering(true) ->setFormat('time');

        $this->addField()
            ->setSqlFieldName('ZadostOZastup.Rozdeleni') ->setHeaderText('Kod')
            ->setCode('col-kodrozdeleni')
            ->setAlign('right') ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('Rozdeleni.Popis') ->setHeaderText('Zastup_rozdeleni')
            ->setCode('col-popisrozdeleni')
            ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('COALESCE(Pozice.Popis, PoziceKategorie.Popis) AS PopisPozice') ->setHeaderText('Zastup_pozice')
            ->setCode('col-pozice')
            ->setOrdering(true)
            ->setSortSQL('PopisPozice');

        if ($this->schvalovani) {
            $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
                // datum schvaleni
                ->setSqlFieldName('ZadostOZastup.Schvaleni') ->setFormat('date')
                ->setCode('col-schvaleni-datum');

            $this ->schvaleniZastupuEditField = new SchvaleniZastupuEditField($this, $this->pravaSchvalit, TextManager::_('Zastup_schvaleni'));
            $this->addField($this ->schvaleniZastupuEditField)
                ->setCode('col-schvaleniedit')
                ->setAlign('center');
        } else {
            $this->addField(new SchvaleniZastupuField($this ->database))
                ->setSqlFieldName('ZadostOZastup.Schvaleni') ->setFormat('date')
                ->setCode('col-schvaleni')
                ->setHeaderText('Zastup_schvaleni')
                ->setSqlFieldIndex2('col-schvalil');
        }

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Prijmeni')
            ->setCode('col-osoba-prijmeni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Jmeno')
            ->setCode('col-osoba-jmeno');

        $this->addField(new ZastupujiciZamestnanecField($this->database))
            ->setCode('col-prijmeni')
            ->setHeaderText('Zastup_osoba') ->setOrdering(true)
            ->setSortSQL('Osoba.Prijmeni, Osoba.Jmeno') ->setSortSQLDesc('Osoba.Prijmeni DESC, Osoba.Jmeno DESC');


//        $this->addField()
//            ->setSqlFieldName('Osoba.Jmeno', 10) ->setHeaderText('Jmeno')
//            ->setCode('col-jmeno')
//            ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('DuvodZastupu.Popis') ->setHeaderText('Zastup_duvod')
            ->setCode('col-duvod')
            ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('ZadostOZastup.Poznamka') ->setHeaderText('Zastup_poznamka')
            ->setCode('col-poznamka')
            ->setOrdering(true);

        $this->addField(new ZruseniZastupuField($database, $this->pravaVlozit));

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('ZadostOZastup.Specifikace')
            ->setCode('col-specifikace') ->setClassName('poznamka')
            ->setHeaderColSpan(1);

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('ZadostOZastup.TypSchvaleni')
            ->setCode('col-typschvaleni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('ZadostOZastup.KodTypuRozdeleni')
            ->setCode('col-kodtypurozdeleni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('ZadostOZastup.Schvalil')
            ->setCode('col-schvalil');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PracovniPomer.Id')
            ->setCode('col-idpp');

        $this->defaultSort = 'col-datum';
    }

    function setFilterForm()
    {
        $this->filterForm = new RonPortalFilterForm($this->database,'zastup', true);

        //$this->filterForm->nastavObdobi(true, 'ADS', false, false);
        //$this->filterForm->nastavDatumOdDo(true, false);
        $this ->filterForm ->addFilterObdobi('ADS', false);
        $this ->filterForm ->addFilter(
            (new FilterFormFilterDatumOdDo($this ->filterForm)),
            true
        );

        $this ->filterForm ->obdobiNeboDatumPovinne = true;

//        $this->filterForm->nastavRozdeleni(true, RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI), 0, '', true);
//        $this->filterForm->nastavRozdeleni2(true, RonPortalSettings::getIntValue(KapacitySettings::TYP_ROZDELENI_POZICE, -1), false, '', true);

        $this ->filterForm ->addFilterRozdeleni('rozdeleni', RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI))
            ->setPouzitPopisTypuRozdeleni(true);

        $this ->filterForm ->addFilterRozdeleni('rozdeleni2', RonPortalSettings::getIntValue(KapacitySettings::TYP_ROZDELENI_POZICE, -1))
            ->setPouzitPopisTypuRozdeleni(true);

        $this->filterForm->addFilter(
            (new FilterFormFilter('duvodzadosti', FilterFormFilter::TYPE_OPTIONS))
                ->setCaption(TextManager::_('Zastup_jen_duvod'))
                ->addItem('', TextManager::_('_vse_'))
                ->setOptionsSql(
                    'SELECT Kod, Popis FROM DuvodZastupu WHERE %FILTER% ORDER BY %ORDER%',
                    '1 = 1',
                    'Popis')
                ->setSqlFilter('DuvodZadosti =')

        );

        $this ->filterForm ->addFilter(
            (new FilterFormFilter('urovenschvaleni'))
                ->setCaption(TextManager::_('Zastup_uroven_schvaleni'))
                ->setSessionName('zastupyUrovenSchvaleni')
                ->addItem(
                    (new FilterFormFilterItem('', TextManager::_('Vse')))
                        ->setSqlFilter('1 = 1')
                )
                ->addItem(-1, TextManager::_('Zastup_ke_schvaleni'))
                ->addItem(
                    (new FilterFormFilterItem(-2, TextManager::_('Zastup_bez_schvaleni')))
                        ->setSqlFilter('ZadostOZastup.TypSchvaleni = 0')
                )
                ->addItem(1, TextManager::_('Zastup_schvaleni'))
                ->addItem(2, TextManager::_('Zastup_zamitnuti'))
                ->addItem(3, TextManager::_('Zastup_zruseni'))
                ->addItem(
                    (new FilterFormFilterItem(100, TextManager::_('Zastup_k_prirazeni')))
                        ->setSqlFilter('ZadostOZastup.RC IS NULL')
                )
                ->setSqlFilter('ZadostOZastup.TypSchvaleni =')
                ->setDefaultValue('')
        );


        if (!$this->schvalovani) {
            if ($this->pravaVlozit->jsouNejakePrava()) {
                $this ->filterForm ->addFilter(
                    (new FilterFormFilter('novy_zaznam', FilterFormFilter::TYPE_TEXT))
                        ->setUrl('?menuid=zastup&formaction=new')
                        ->setDefaultValue(TextManager::_('New_record'))
                        ->setPosition(FilterFormFilter::POSITION_BELOW_FORM)
                        ->setCaption(false)
                        ->setRowClass(false)
                );
                //echo "<div class='normaltext' id='zastup_links'><a href='?menuid=zastup&formaction=new'>" . TextManager::_('New_record') . "</a></div>\n";
            }

            if ($this->pravaSchvalit->jsouNejakePrava()) {
                $this ->filterForm ->addFilter(
                    (new FilterFormFilter('schvalit_hromadne', FilterFormFilter::TYPE_TEXT))
                        ->setDefaultValue(
                            "<input type='button' class='button clickHref' name='zobrazithromadne' value='" . TextManager::_('Zastup_schvalit_hromadne') . "'" .
                            "  data-ron-href='?menuid=zastup&schvalovani=1'/>"
                        )
                        ->setPosition(FilterFormFilter::POSITION_BELOW_FORM)
                        ->setRowClass(false)
                        ->setCaption(false)
                        ->setAddClass('margin-left-4')
                );
            }
        }
    }


//    public function renderFilter()
//    {
//        $this->filterForm->render();
//
//        if (!$this->schvalovani) {
//            if ($this->pravaVlozit->jsouNejakePrava()) {
//                echo "<div class='normaltext' id='zastup_links'><a href='?menuid=zastup&formaction=new'>" . TextManager::_('New_record') . "</a></div>\n";
//            }
//
//            if ($this->pravaSchvalit->jsouNejakePrava()) {
//                echo
//                    "<span class='margin-left-4'>" .
//                    "<input type='button' class='button clickHref' name='zobrazithromadne' value='" . TextManager::_('Zastup_schvalit_hromadne') . "'" .
//                    "  data-ron-href='?menuid=zastup&schvalovani=1'/>" .
//                    "</span>";
//            }
//        }
//    }

    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

//        // uroven schvaleni (vse, ke schvaleni, ...)
//        $result .= $result ? ' AND ' : '';
//        switch (RonPortalSession::getValue('zastupyUrovenSchvaleni')) {
//            case -2 : // bez schvaleni
//                $result .= 'ZadostOZastup.TypSchvaleni = 0 ';
//                break;
//
//            case 1 : // schvalene
//                $result .= 'ZadostOZastup.TypSchvaleni = 1 ';
//                break;
//
//            case 2 : // zamitnute
//                $result .= 'ZadostOZastup.TypSchvaleni = 2 ';
//                break;
//
//            case 3 : // zrusene
//                $result .= 'ZadostOZastup.TypSchvaleni = 3 ';
//                break;
//
//            case -1 : // ke schvaleni (dale se filtruje v enable line podle konkretnich prav)
//                $result .= 'ZadostOZastup.TypSchvaleni = 0 ';
//                break;
//
//            case 100: //zaznamy, kde neni osoba, dale se filtruje v enable line podle prav
//                $result .= 'ZadostOZastup.RC IS NULL ';
//                break;
//
//            default : // vse
//                $result .= ' 1 = 1 ';
//        }

        // obdobi nebo datum od-do
        if ($this ->filterForm ->getOmezitObdobiOd()) {
            $result .=
                ($result ? ' AND ' : '') .
                'ZadostOZastup.Datum >= ' . $params ->addDateParameter($this ->filterForm ->getOmezitObdobiOd());
        }

        if ($this ->filterForm ->getOmezitObdobiDo()) {
            $result .=
                ($result ? ' AND ' : '') .
                'ZadostOZastup.Datum <= ' . $params ->addDateParameter($this ->filterForm ->getOmezitObdobiDo());
        }

        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            if ($rozdeleniFilter ->getValue() !== null) {
                // Filtr rozdeleni2 filtruje pozici
                if ($rozdeleniFilter ->name === 'rozdeleni2') {
                    $result .=
                        ($result ? ' AND ' : '') .
                        RozdeleniUtils::sqlFilterRozdeleni($this ->database, 'ZadostOZastup',
                            (int)$rozdeleniFilter->getTypRozdeleni(), $rozdeleniFilter->getValue(),
                            $rozdeleniFilter->getValue('vcetnePodrizenych'),
                            $params,
                            ['KodTypuRozdeleni2', 'Rozdeleni2']);
                } else {
                    $result .=
                        ($result ? ' AND ' : '') .
                        RozdeleniUtils::sqlFilterRozdeleni($this ->database, 'ZadostOZastup',
                            (int)$rozdeleniFilter->getTypRozdeleni(), $rozdeleniFilter ->getValue(),
                            $rozdeleniFilter ->getValue('vcetnePodrizenych'),
                            $params);
                }
            }
        }

//        if ($this ->filterForm ->kodRozdeleni) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                RozdeleniUtils::sqlFilterRozdeleni($this ->database, 'ZadostOZastup',
//                    (int)$this->filterForm->kodTypuRozdeleni, $this ->filterForm ->kodRozdeleni,
//                    $this ->filterForm ->vcetnePodrizenychRozdeleni);
//        }
//
//        if ($this ->filterForm ->kodRozdeleni2) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                RozdeleniUtils::sqlFilterRozdeleni($this ->database, 'ZadostOZastup',
//                    (int)$this->filterForm->kodTypuRozdeleni2, $this ->filterForm ->kodRozdeleni2,
//                    $this ->filterForm ->vcetnePodrizenychRozdeleni2,
//                    null,
//                    ['KodTypuRozdeleni2', 'Rozdeleni2']);
//        }

//        if ($this ->filterForm instanceof ZastupyFilterForm  &&  $this ->filterForm ->jenDuvodZadosti) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                " DuvodZadosti = '" . $this->database->addSlashes($this ->filterForm ->duvodZadosti) . "'";
//        }

        return new SqlStatement($result, $params);
    }


    /**
     * @param array $values
     * @param int $datumod
     * @param int $datumdo
     * @return bool
     * @throws Exception
     */
    function zastupEnableLine($values, $datumod, $datumdo)
    {
        if (RonPortalSession::getValue('zastupyUrovenSchvaleni') == -1) {
            // filtr Ke schvaleni - zkontroluje, jestli to ceka na me

            if ($values['col-typschvaleni'] == 1) {
                // jiz je schvaleno nejvyssi urovni = neceka na nikoho
                return false;
            }

            return $this->pravaSchvalit->jePravoNaRozdeleni($values['col-kodtypurozdeleni'], $values['col-kodrozdeleni'], $datumod, $datumdo);
            // neni schvaleno zadnou urovni => pokud mam pravo, ceka na me
        }

        if (RonPortalSession::getValue('zastupyUrovenSchvaleni') == 100) {
            // filtr K prirazeni - zkontroluje, jestli to ceka na me

            if ($values['col-typschvaleni'] != 1) {
                return false;
            }

            return $this->pravaPriradit->jePravoNaRozdeleni($values['col-kodtypurozdeleni'], $values['col-kodrozdeleni'], $datumod, $datumdo);
            // neni schvaleno zadnou urovni => pokud mam pravo, ceka na me
        }

        // Zobrazi zaznam, pokud mam kterekoliv pravo
        if (!$this->schvalovani) {
            if ($this->pravaVlozit->jePravoNaRozdeleni($values['col-kodtypurozdeleni'], $values['col-kodrozdeleni'], $datumod, $datumdo)) {
                return true;
            }

            if ($this->pravaPriradit->jePravoNaRozdeleni($values['col-kodtypurozdeleni'], $values['col-kodrozdeleni'], $datumod, $datumdo)) {
                return true;
            }

            //pokud ma uzivatel pravo zobrazit prirazenou osobu, nebo rozdeleni zobrazi se mu zaznam
            if (($this->pravaZobrazit->jePravoNaRozdeleni($values['col-kodtypurozdeleni'], $values['col-kodrozdeleni'], $datumod, $datumdo)) ||
                ($this->pravaZobrazit->lzeZobrazitPP($values['col-idpp'], $datumod, $datumdo))) {
                return true;
            }
        }

        if ($this->pravaSchvalit->jePravoNaRozdeleni($values['col-kodtypurozdeleni'], $values['col-kodrozdeleni'], $datumod, $datumdo)) {
            return true;
        }

        //pravo zrusit musi mit zaroven na rozdeleni, kam je podana zadost, i na prirazeneho zamestnance
        if (($this->pravaZrusit->jePravoNaRozdeleni($values['col-kodtypurozdeleni'], $values['col-kodrozdeleni'], $datumod, $datumdo)) &&
            ($this->pravaZrusit->lzeZobrazitPP($values['col-idpp'], $datumod, $datumdo))) {
            return true;
        }

        return false;
    }


    /**
     * @param array $values
     * @return bool
     * @throws Exception
     */
    function enableLine($values)
    {
        $datumOd = $this ->database ->stringToTimestamp($values['col-datum']);
        $datumDo = $datumOd;

        if (!$this ->zastupEnableLine($values, $datumOd, $datumDo)) {
            return false;
        }

        return true;
    }

    /**
     * @param array  $values
     * @param string $className
     * @param int    $rowNumber
     * @return string   Vraci atributy pro <tr>
     */
    function getRowAttributes($values, $className, $rowNumber)
    {
        $datum = $this ->database ->stringToTimestamp($values['col-datum']);
        $schvalovat = $this ->schvalovani  &&  $this ->pravaSchvalit->jePravoNaRozdeleni($values['col-kodtypurozdeleni'], $values['col-kodrozdeleni'], $datum, $datum);

        if ($values['col-typschvaleni'] == 1) {
            $className .= ' zastup_schvalen';
        } else if ($values['col-typschvaleni'] == 2) {
            $className .= ' zastup_zamitnut';
        }

        return
            parent::getRowAttributes($values, $className, $rowNumber) .
            " data-id='{$values[0]}'" . // predevsim pro hromadne schvalovani
            " id='zastup_row_{$values[0]}'";
    }


    function renderContent()
    {
        if ($this ->schvalovani) {
            echo
                '<form method="post" class="submitWorkingInfo" name="hromadneSchvalovaniZastupu" action="?menuid=zastup">' .
                '<input type="hidden" name="hromadneSchvaleni" value="1"/>';

            echo
                '<ul class="popupmenu"  id="schvaleniZastupuMenu">' .
                "<li data-hodnota-schvaleni='1'>" . TextManager::_('Oznacit_vse') . "</li>" .
                "<li data-hodnota-schvaleni='0'>" . TextManager::_('Zrusit_oznaceni') . "</li>" .
                "<li data-hodnota-schvaleni='-1'>" . TextManager::_('form.Original') . "</li>" .
                '</ul>';
        }

        parent::renderContent();

        if ($this->schvalovani) {
            echo
                "<input type='hidden' name='pocet' value='{$this ->schvaleniZastupuEditField ->inputsCount}'>" .
                "<div class='buttons'>" .
                "<input type='submit' class='submitbutton' value='" . TextManager::_('Provest') . "' id='provestHromadneSchvaleniZastupu' />" .
                '<input type="button" class="button clickHref" data-ron-href="?menuid=zastup" value="' . TextManager::_('Cancel') . '" id="stornoHromadneSchvaleniZastupu" />' .
                '</div>'.
                "</form>\n";
        } else if ($this->pravaVlozit->jsouNejakePrava()) {
            echo
                "<div class='normaltext'><a href='?menuid=zastup&formaction=new'>" .
                TextManager::_('New_record') .
                "</a></div>\n";
        }
    }
}
