<?php
/**
 * pavel.rudol, 20.12.2024 11:00
 */

namespace RonPortal\Mzdy;

use Ron\DB\Database;
use RonPortal\PracovniPomer;

class OznameniDavekNZ
{
    /** @var int */
    public $idPP;

    /** @var PracovniPomer */
    public $pracovniPomer;

    /** @var int */
    public $id;
    public $cisloDokladu;
    /**
     * @var int
     *  0 - neověřeno (po zadání dávky implicitní. Po odeslání na OSSZ další stavy) - již nelze editovat číslo dokladu a druh dávky <br>
     *  1 - odesláno k ověření <br>
     *  2 - ověřeno - po zadání přišla odpověď OK <br>
     *  3 - ověřeno chyba - po zadání přišla odpověď CHYBA - dále se nesmí zpracovávat, dokud není opraveno číslo rozhodnutí nebo kontakt zaměstnance) <br>
     *  4 - evidováno pro další změny (jen pro ověřené dávky, pokud není ukončená) <br>
     *  5 - ukončeno evidování změn (jen pro ověřené dávky) - po ukončení dávky příznakem Ukončená dávka <br>
     *
     */
    public $status;
    public $druhDavky;
    /** @var bool */
    public $ePodani;
    public $duvodPece;
    public $idRodinnehoPrislusnika;

    /**
     * @param Database $database
     * @param int $id
     * @return OznameniDavekNZ|null
     */
    public static function load($database, $id)
    {
        $params = $database->createParameters();

        /*
         * ePodani, DruhDavky, CisloRozhodnutiStatus, ZadostOdDne, ZadostDoDne, OznacenoUzivatelem, DatumCasVlozeni, TypRozdeleni, Rozdeleni, RC_RodinnyPrislusnik,
            DuvodOtcovske, DuvodPece, nazevZarizeniSkoly, ICZarizeniSkoly, spolecnaDomacnost, jeOsamely, vPeciDiteDo16Let, narokNaPPMjinouOsobou, narokNaRPjinaOsobaNecerpa,
            jinaFOParagraf57, pecovalPoCelouDobu, KodRodVztah, KodVztah, JeStridani, predpokladanyDenPorodu
         * */

        $sql =
            'SELECT PracovniPomer.ID, OznameniKDavkamNZ.ID, OznameniKDavkamNZ.RC, OznameniKDavkamNZ.OsobniCislo, OznameniKDavkamNZ.DatumNastupu, OznameniKDavkamNZ.CisloDokladu, OznameniKDavkamNZ.CisloRozhodnutiStatus,
            OznameniKDavkamNZ.DruhDavky, OznameniKDavkamNZ.ePodani, OznameniKDavkamNZ.DuvodPece, RC_RodinnyPrislusnik
            FROM OznameniKDavkamNZ JOIN PracovniPomer ON (PracovniPomer.RC = OznameniKDavkamNZ.RC AND PracovniPomer.OsobniCislo = OznameniKDavkamNZ.OsobniCislo AND PracovniPomer.DatumNastupu = OznameniKDavkamNZ.DatumNastupu)
            WHERE (OznameniKDavkamNZ.ID = ' . $params->addParameter($id) . ')';

        $cursor = $database->select($sql, $params);

        $row = $cursor->fetch();

        if ($row) {
            $oznameniDavekNZ = new OznameniDavekNZ();
            $oznameniDavekNZ->idPP = (int)$row[0];
            $oznameniDavekNZ->pracovniPomer = PracovniPomer::nactiPP($database, $row[0]);
            $oznameniDavekNZ->id = (int)$row[1];
            $oznameniDavekNZ->cisloDokladu = $row[5];
            $oznameniDavekNZ->status = (int)$row[6];
            $oznameniDavekNZ->druhDavky = $row[7];
            $oznameniDavekNZ->ePodani = $row[8] == 1;
            $oznameniDavekNZ->duvodPece = (int)$row[9];
            $oznameniDavekNZ->idRodinnehoPrislusnika = $row[10];

            return $oznameniDavekNZ;
        }

        return null;
    }
}