<?php
/**
 * pavel.rudol, 01.09.2023 14:43
 */

namespace RonPortal\Mzdy;

use DateTime;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Personalistika\ZdravotniStavOsoby;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalSettings;
use RonPortal\Zamestnanci\AdresaOsoby;

class DitePoplatnika
{
    public $id;
    public $jmeno;
    public $prijmeni;
    public $datumNarozeni;
    public $vekNaZacatkuRoku;
    public $vekNaKonciRoku;
    public $ztpp;
    public $poradDitete = 0;
    public $poradDiteteNavrh = null;
    public $rodneCislo;

    public $datumVlozeniNavrhu;
    public $zmenaOd;
    public $elektronickyPodpis;
    public $elektronickyPodpisUcetni;

    public $skolkovne;
}

class PartnerPoplatnika
{
    public $id;
    public $jmeno;
    public $prijmeni;
    public $datumNarozeni;
    public $ztpp;
    public $rodneCislo;
}

class ZmenaOdpoctu
{
    public $popis;
    public $zmenaOd;
    public $elektronickyPodpis;
    public $elektronickyPodpisUcetni;
}

class VyuctovaniDaneForm extends RocniVyuctovaniDaneRecord
{
    /** @var PracovniPomer */
    public $pracovniPomer;

    public $typDokladuNerezidenta;
    public $cisloDokladuNerezidenta;
    public $statDokladuNerezidenta;
    public $statDokladuNerezidenta_popis;
    public $typDanovehoIdentifikatoru_popis;

    //TODO: nechat?
    public $schvaleneDatumOdNavrhuOdpoctu;

    /** @var AdresaOsoby */
    public $bydliste;

    /** @var ZdravotniStavOsoby */
    public $zdravotniStav;

    public $odpocetNavrhDatumVlozeni;
    public $odpocetNavrhDatumOd;
    public $odpocetNavrhZakladni;
    public $odpocetNavrhZdravotniStav;
    public $odpocetNavrhZTPP;
    public $odpocetNavrhStudent;
    
    public $odpocetDatumOd;
    public $odpocetZakladni;
    public $odpocetZdravotniStav;
    public $odpocetZTPP;
    public $odpocetStudent;

    /** @var DitePoplatnika[]|null */
    public $deti;

    /** @var DitePoplatnika[]|null */
    public $zmenyPoradiDeti;

    /** @var ZmenaOdpoctu[]|null */
    public $zmenyOdpoctu;

    /** @var PartnerPoplatnika */
    public $partnerPoplatnika;

    /** @var bool urceni, zda se jedna o formular prohlaseni, nebo vyuctovani */
    private $prohlaseni = true;

    public function __construct(Database $database)
    {
        parent::__construct($database, $this);

        $this->addStringField('CisloDanoveIdentifikace', true);
        $this->addStringField('StatDanoveRezidence', true);
        $this->addIntField('TypDanovehoIdentifikatoru', true);

        $this->addDateField('OsobniUdaj.DatumNarozeni', true);
        $this->addStringField('OsobniUdaj.RodneCislo', true);
        $this->addStringField('Stat.Popis', true);
    }

    public function setPracovniPomer(PracovniPomer $pracovniPomer)
    {
        $this->pracovniPomer = $pracovniPomer;
    }

    private function overPracovniPomer()
    {
        $platnePP = PracovniPomer::nactiPPOsoby($this->database, $this['RC']);

        $prirazenPlatnyPP = false;
        $vybranPlatnyPP = false;
        foreach ($platnePP as $pracovniPomer) {
            if ($pracovniPomer->rc == $this['ProhlaseniPortalRC'] and $pracovniPomer->osobniCislo == $this['ProhlaseniPortalOsc'] and $pracovniPomer->datumNastupu == $this['ProhlaseniPortalDatNastup']) {
                $prirazenPlatnyPP = true;
            }
            if ($pracovniPomer->rc == $this->pracovniPomer->rc and $pracovniPomer->osobniCislo == $this->pracovniPomer->osobniCislo and $pracovniPomer->datumNastupu == $this->pracovniPomer->datumNastupu) {
                $vybranPlatnyPP = true;
            }
        }

        if (!$prirazenPlatnyPP) {
            if (!$vybranPlatnyPP) {
                MessageManager::addWarning(sprintf(TextManager::_('Prohlaseni_prirazen_neplatny_pp'), $this['ProhlaseniPortalOsc'], StringUtils::format('date', $this['ProhlaseniPortalDatNastup'])));
            } else if ($this['ProhlaseniPortalOsc'] != '') {
                MessageManager::addWarning(sprintf(TextManager::_('Prohlaseni_neplatny_pp_provest_zmenu'), $this['ProhlaseniPortalOsc'], StringUtils::format('date', $this['ProhlaseniPortalDatNastup']),
                    $this->pracovniPomer->osobniCislo, StringUtils::format('date', $this->pracovniPomer->datumNastupu)));
            }
        }
    }

    function getSelectSQL($filtr = '', $targetParams = null, $filtrParams = null)
    {
        $sql =
            'SELECT ' . $this ->getSelectPartSQL() .
            '  FROM RocniVyuctovaniDane ' .
            '   JOIN OsobniUdaj ON (OsobniUdaj.RC = RocniVyuctovaniDane.RC) ' .
            '   LEFT OUTER JOIN Stat ON (Stat.Kod = RocniVyuctovaniDane.StatDanoveRezidence) ' .
            ' WHERE (' . $this ->where . ')';

        $targetParams ->appendParameters($this ->whereParams);

        if ($filtr) {
            $sql .= ' AND (' . $filtr . ')';
            if ($filtrParams) {
                $targetParams ->appendParameters($filtrParams);
            }
        }

        return $sql;
    }

    public function loadRecord($filtr = '', $filtrParams = null, $zmeny = false, $prohlaseni = true)
    {
        $result = parent::loadRecord();

        //zkontroluje zda je k prohlaseni prirazen platny PP, pokud neni, tak vyzve ke zmene
        if ($prohlaseni && $this->lzeEditovatProhlaseni()) {
            $this->overPracovniPomer();
        }

        $datumOd = $this['rok'];
        $datumDo = DateUtils::lastDayOfYear($datumOd);

        if ($this['DanovyNerezident'] == 1) {
            $this->nactiPrukazNerezidenta($this['DatumProhlaseni'] ?: $this['RocniVyuctovaniDane.Rok']);
        }
        switch ($this['TypDanovehoIdentifikatoru']) {
            case 0:
                $this->typDanovehoIdentifikatoru_popis = 'R';
                break;
            case 1:
                $this->typDanovehoIdentifikatoru_popis = 'D';
                break;
            case 2:
                $this->typDanovehoIdentifikatoru_popis = 'S';
                break;
            case 3:
                $this->typDanovehoIdentifikatoru_popis = 'J';
                break;
        }

        $this->nactiBydlistePoplatnika($this["DatumProhlaseniPortal"] ?: DateUtils::today());
        $this->nactiOdpocet($datumOd, $datumDo);
        $this->nactiDetiPoplatnika($datumOd, $datumDo);
        $this->nactiOdpocetNaDite(DateUtils::today(), $datumDo);

        if ($zmeny) {
            $this->nactiOdpocetNavrh(2, $datumOd, $datumDo);
            $this->nactiOdpocetNavrh(3, $datumOd, $datumDo);
            $this->nactiOdpocetNaDiteNavrh(2, $datumOd, $datumDo);
            $this->nactiOdpocetNaDiteNavrh(3, $datumOd, $datumDo);
            $this->nactiZdravotniStav($this['ProhlaseniDatumZmeny'] ?: DateUtils::today());
        } else {
            if ($prohlaseni) {
                $stav = 0; //navrh
                if ($this['ProhlaseniPortalStav'] == RocniVyuctovaniDaneRecord::PROHLASENI_SCHVALENO) {
                    $stav = 1; //schvalena zmena
                }
            } else {
                $stav = 4; // dodatecny navrh
                if ($this['ZadostPortalStav'] == RocniVyuctovaniDaneRecord::ZADOST_SCHVALENO_UCETNI) {
                    $stav = 5; // schvaleny dotatecny navrh
                }
            }

            $this->nactiOdpocetNavrh($stav, $datumOd, $datumDo);
            $this->nactiOdpocetNaDiteNavrh($stav, $datumOd, $datumDo);
            //partner se na zmenovem formulari nezobrazuje
            $this->nactiPartneraPoplatnika(!$prohlaseni);
            $this->nactiZdravotniStav($this['DatumProhlaseniPortal'] ?: DateUtils::lastDayOfYear($this->rok->value));
        }

        $this->prohlaseni = $prohlaseni;

        if (!$this->prohlaseni) {
            $this->nactiSkolkovneNaDite();
        }

        return $result;
    }

    private function nactiPrukazNerezidenta($datum)
    {
        $params = $this->database->createParameters();

        //doklad potvrzující totožnost - má být z průkazů - OP, Pas, ŘP v tomto pořadí
        $sql =
            "SELECT CisloPrukazu, TypPrukazu.Popis, Stat.Popis, Stat.Kod
            FROM Prukaz 
                JOIN TypPrukazu ON Prukaz.TypPrukazu = TypPrukazu.Kod 
                LEFT OUTER JOIN Stat ON Prukaz.VydalStat = Stat.Kod
            WHERE RC = " . $params->addParameter($this['rc']) .
            " AND PlatnostOd < " . $params->addDateParameter($datum) .
            " AND (PlatnostDo >= " . $params->addDateParameter($datum) . " OR PlatnostDo IS NULL) " .
            "ORDER BY TypPrukazu";

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();
        if ($row) {
            $this->cisloDokladuNerezidenta = $row[0];
            $this->typDokladuNerezidenta = $row[1];
            $this->statDokladuNerezidenta_popis = $row[2];
            $this->statDokladuNerezidenta = $row[3];
        }
    }

    private function nactiDatumOdSchvalenehoOdpoctu()
    {
        $params = $this->database->createParameters();
        $sql =
            "SELECT DatumOd 
            FROM OdpocetNavrh 
            WHERE RC = " . $params->addParameter($this->pracovniPomer->rc) .
            " AND OsobniCislo = " . $params->addParameter($this->pracovniPomer->osobniCislo) .
            " AND DatumNastupu = " . $params->addDateParameter($this->pracovniPomer->datumNastupu) .
            " AND Stav = 1 " .
            " AND DatumOd >= " . $params->addDateParameter($this['rok']) .
            " AND DatumOd <= " . $params->addDateParameter(DateUtils::lastDayOfYear($this['rok']));

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();
        if (($row) && $row[0] != '') {
            $this->schvaleneDatumOdNavrhuOdpoctu = $this->database->stringToTimestamp($row[0]);
        }
    }

    public function nactiOdpocet($datumOd, $datumDo)
    {
        $params = $this->database->createParameters();
        $sql =
            "SELECT DatumOd, DatumDo, ZakladniOdpocet, NaZdravotniStav, ZTPP, OdpocetNaStudenta
            FROM Odpocet
            WHERE RC = " . $params->addParameter($this->pracovniPomer->rc) .
            " AND OsobniCislo = " . $params->addParameter($this->pracovniPomer->osobniCislo) .
            " AND DatumNastupu = " . $params->addDateParameter($this->pracovniPomer->datumNastupu) .
            " AND DatumOd <= " . $params->addDateParameter($datumDo) .
            " AND (DatumDo >= " . $params->addDateParameter($datumOd) . " OR DatumDo IS NULL)" .
            " ORDER BY DatumOd DESC";

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();
        if ($row) {
            $this->odpocetDatumOd = $this->database->stringToTimestamp($row[0]);
            $this->odpocetZakladni = $row[2];
            $this->odpocetZdravotniStav = $row[3];
            $this->odpocetZTPP = $row[4];
            $this->odpocetStudent = $row[5];
        }
    }

    /**
     * @param int $stav stav zmeny: 0 = navrh, 1 = schvaleno, 2 = zmena, 3 = schvalena zmena
     * @return void
     * @throws DBException
     */
    public function nactiOdpocetNavrh($stav, $datumOd, $datumDo)
    {
        //pro schvalene zmeny nacist zmeny ze vsech PP

        $params = $this->database->createParameters();
        $sql =
            "SELECT DatumOd, ZakladniOdpocet, NaZdravotniStav, ZTPP, OdpocetNaStudenta, Stav, {fn_CASTWIDEMEMO(ElektronickyPodpis)}, {fn_CASTWIDEMEMO(ElektronickyPodpisUcetni)}, DatumVlozeniNavrhu
            FROM OdpocetNavrh
            WHERE RC = " . $params->addParameter($this->pracovniPomer->rc);

        if ($stav != 3) {
            $sql .=
            " AND OsobniCislo = " . $params->addParameter($this->pracovniPomer->osobniCislo) .
            " AND DatumNastupu = " . $params->addDateParameter($this->pracovniPomer->datumNastupu);
        }
        $sql .=
            " AND DatumOd >= " . $params->addDateParameter($datumOd) .
            " AND DatumOd <= " . $params->addDateParameter($datumDo) .
            " AND Stav = " . $params->addParameter($stav) .
            " ORDER BY DatumOd";

        $cursor = $this->database->getSqlResult($sql, $params);

        if ($stav == 3) {
            $zmeny = [];

            $prvni = true;

            $zakladni = null;
            $zdravotniStav = null;
            $student = null;

            //TODO: porovnani z predchozim platnym pred nahlasenim zmeny nefunguje, protoze stav v odpocetZakladni ... je po schvaleni jiz zmenen, musel by se nacist stav pred schvalenim
//            if (isset($this->odpocetZakladni)) {
//                $zakladni = $this->odpocetZakladni;
//                $zdravotniStav = $this->odpocetZdravotniStav;
//                $student = $this->odpocetStudent;
//
//                $prvni = false;
//            }

            while ($row = $cursor->fetch()) {
                $zmena = new ZmenaOdpoctu();

                $zmena->zmenaOd = $this->database->stringToTimestamp($row[0]);
                $zmena->elektronickyPodpis = $row[6];
                $zmena->elektronickyPodpisUcetni = $row[7];
                $zmena->popis = $this->popisZmenyOdpoctu($prvni, (int)$row[1], (int)$row[4], (int)$row[2], (int)$zakladni, (int)$student, (int)$zdravotniStav);

                $zakladni = $row[1];
                $zdravotniStav = $row[2];
                $student = $row[4];

                $prvni = false;
                $zmeny[] = $zmena;
            }

            if (empty($zmeny)) {
                $this->zmenyOdpoctu = null;
            } else {
                $this->zmenyOdpoctu = $zmeny;
            }
        } else {
            $row = $cursor->fetch();
            if ($row) {
                $this->odpocetNavrhDatumVlozeni = $this->database->stringToTimestamp($row[8]);
                $this->odpocetNavrhDatumOd = $this->database->stringToTimestamp($row[0]);
                $this->odpocetNavrhZakladni = $row[1];
                $this->odpocetNavrhZdravotniStav = $row[2];
                $this->odpocetNavrhZTPP = $row[3];
                $this->odpocetNavrhStudent = $row[4];
            }
        }
    }

    public function nactiOdpocetNaDite($datumOd, $datumDo)
    {
        $params = $this->database->createParameters();
        $sql =
            "SELECT DatumOd, DatumDo, RC_Ditete, PoradiDitete, RodinnyPrislusnik.DatumNarozeni, RodinnyPrislusnik.ZdravotniStav, RodinnyPrislusnik.DrzitelPrukazuZTTP
            FROM OdpocetNaDite 
                JOIN RodinnyPrislusnik ON RodinnyPrislusnik.RC_RodinnehoPrislusnika = OdpocetNaDite.RC_Ditete
            WHERE OdpocetNaDite.RC = " . $params->addParameter($this->pracovniPomer->rc) .
            " AND OdpocetNaDite.OsobniCislo = " . $params->addParameter($this->pracovniPomer->osobniCislo) .
            " AND OdpocetNaDite.DatumNastupu = " . $params->addDateParameter($this->pracovniPomer->datumNastupu) .
            " AND DatumOd <= " . $params->addDateParameter($datumDo) .
            " AND (DatumDo >= " . $params->addDateParameter($datumOd) . " OR DatumDo IS NULL)" .
            " ORDER BY DatumOd";

        $cursor = $this->database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()) {
            if (isset($this->deti[$row[2]])) {
                $dite = $this->deti[$row[2]];
                $dite->poradDitete = $row[3];
//                $dite->zmenaOd = $this->database->stringToTimestamp($row[0]);
            }
        }
    }

    /**
     * @param int $stav stav zmeny: 0 = navrh, 1 = schvaleno, 2 = zmena, 3 = schvalena zmena
     * @return void
     * @throws DBException
     */
    public function nactiOdpocetNaDiteNavrh(int $stav, $datumOd, $datumDo)
    {
        $params = $this->database->createParameters();
        $sql =
            "SELECT DatumOd, RC_Ditete, PoradiDitete, RodinnyPrislusnik.DatumNarozeni, RodinnyPrislusnik.ZdravotniStav, RodinnyPrislusnik.DrzitelPrukazuZTTP,
                RodinnyPrislusnik.Jmeno, RodinnyPrislusnik.Prijmeni, RodinnyPrislusnik.RodneCisloRP, {fn_CASTWIDEMEMO(ElektronickyPodpis)}, {fn_CASTWIDEMEMO(ElektronickyPodpisUcetni)}, DatumVlozeniNavrhu
            FROM OdpocetNaDiteNavrh 
                JOIN RodinnyPrislusnik ON RodinnyPrislusnik.RC_RodinnehoPrislusnika = OdpocetNaDiteNavrh.RC_Ditete
            WHERE OdpocetNaDiteNavrh.RC = " . $params->addParameter($this->pracovniPomer->rc);

        if ($stav != 3) {
            $sql .=
            " AND OdpocetNaDiteNavrh.OsobniCislo = " . $params->addParameter($this->pracovniPomer->osobniCislo) .
            " AND OdpocetNaDiteNavrh.DatumNastupu = " . $params->addDateParameter($this->pracovniPomer->datumNastupu);
        }
        $sql .=
            " AND OdpocetNaDiteNavrh.OsobniCislo = " . $params->addParameter($this->pracovniPomer->osobniCislo) .
            " AND OdpocetNaDiteNavrh.DatumNastupu = " . $params->addDateParameter($this->pracovniPomer->datumNastupu) .
            " AND Stav = " . $params->addParameter($stav) .
            " AND DatumOd >= " . $params->addDateParameter($datumOd) .
            " AND DatumOd <= " . $params->addDateParameter($datumDo) .
            " ORDER BY DatumOd";

        $cursor = $this->database->getSqlResult($sql, $params);

        if ($stav == 3) {
            $deti = [];
        }

        while ($row = $cursor->fetch()) {
            if ($stav == 3) {
                $dite = new DitePoplatnika();
                $dite->id = $row[1];
                $dite->jmeno = $row[6];
                $dite->prijmeni = $row[7];
                $dite->datumNarozeni = $this->database->stringToTimestamp($row[3]);
                $dite->ztpp = $row[5];
                $dite->rodneCislo = $row[8];
                $dite->poradDiteteNavrh = $row[2];

                $pocatekRoku = new DateTime();
                $pocatekRoku->setTimestamp($this->rok->value);
                $datumNarozeni = new DateTime();
                $datumNarozeni->setTimestamp($dite->datumNarozeni);

                $dite->vekNaZacatkuRoku = $pocatekRoku->diff($datumNarozeni);

                $dite->zmenaOd = $this->database->stringToTimestamp($row[0]);
                $dite->elektronickyPodpis = $row[9];
                $dite->elektronickyPodpisUcetni = $row[10];
                $deti[] = $dite;
            } else {
                if (isset($this->deti[$row[1]])) {
                    $dite = $this->deti[$row[1]];
                    $dite->zmenaOd = $this->database->stringToTimestamp($row[0]);
                    $dite->datumVlozeniNavrhu = $this->database->stringToTimestamp($row[11]);
                    $dite->poradDiteteNavrh = $row[2];
                }
            }
        }

        if (empty($deti)) {
            $this->zmenyPoradiDeti = null;
        } else {
            $this->zmenyPoradiDeti = $deti;
        }
    }

    public function nactiDetiPoplatnika($datumOd, $datumDo)
    {
        $params = $this->database->createParameters();
        $sql =
            "SELECT RodinnyPrislusnik.RC_RodinnehoPrislusnika, RodinnyPrislusnik.Jmeno, RodinnyPrislusnik.Prijmeni, RodinnyPrislusnik.DatumNarozeni, RodinnyPrislusnik.DrzitelPrukazuZTTP, RodinnyPrislusnik.RodneCisloRP
            FROM ClenoveRodiny 
                JOIN RodinnyPrislusnik ON (RodinnyPrislusnik.RC_RodinnehoPrislusnika = ClenoveRodiny.RC_RodinnehoPrislusnika)
            WHERE ClenoveRodiny.RC  = " . $params->addParameter($this->pracovniPomer->rc) .
            " AND ClenoveRodiny.RodinnyVztah IN (4,5,6,7) " . //syn, dcera, vnuk, vnucka
            " AND RodinnyPrislusnik.DatumNarozeni <= " . $params->addDateParameter($datumDo) .
            " AND ClenoveRodiny.StejnaAdresa = 1 " .
            " AND ClenoveRodiny.VyzivovanaOsoba = 1 " .
            " AND (ClenoveRodiny.VyzivovanaOsobaOd IS NULL OR ClenoveRodiny.VyzivovanaOsobaOd <= " . $params->addDateParameter($datumDo) . ")" .
            " AND (ClenoveRodiny.VyzivovanaOsobaDo IS NULL OR ClenoveRodiny.VyzivovanaOsobaDo >= " . $params->addDateParameter($datumOd) . ")";

        $cursor = $this->database->getSqlResult($sql, $params);

        $deti = [];

        while ($row = $cursor->fetch()) {
            $dite = new DitePoplatnika();
            $dite->id = $row[0];
            $dite->jmeno = $row[1];
            $dite->prijmeni = $row[2];
            $dite->datumNarozeni = $this->database->stringToTimestamp($row[3]);
            $dite->ztpp = $row[4];
            $dite->rodneCislo = $row[5];

            $pocatekRoku = new DateTime();
            $pocatekRoku->setTimestamp($this->rok->value);
            $datumNarozeni = new DateTime();
            $datumNarozeni->setTimestamp($dite->datumNarozeni);
            $konecRoku = new DateTime();
            $konecRoku->setTimestamp(DateUtils::lastDayOfYear($this->rok->value));

            $dite->vekNaZacatkuRoku = $pocatekRoku->diff($datumNarozeni);
            $dite->vekNaKonciRoku = $konecRoku->diff($datumNarozeni);

            //na formulari se zobrazi jen deti, ktere na zacatku roku nemeli 26 let
            if ($dite->vekNaZacatkuRoku->y < 26) {
                $deti[$row[0]] = $dite;
            }
        }

        if (empty($deti)) {
            $this->deti = null;
        } else {
            $this->deti = $deti;
        }
    }

    private function nactiSkolkovneNaDite()
    {
        $params = $this->database->createParameters();
        $sql =
            "SELECT RC_RodinnehoPrislusnika, Skolkovne 
            FROM SkolkovneNaDite
            WHERE Rok = " . $params->addDateParameter($this['rok']) .
            " AND RC = " . $params->addParameter($this->pracovniPomer->rc);

        $cursor = $this->database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()) {
            if (isset($this->deti[$row[0]])) {
                $this->deti[$row[0]]->skolkovne = (float)$row[1];
            }
        }
    }

    private function nactiPartneraPoplatnika($vyuctovaniDaneOdecet = false)
    {
        if ($vyuctovaniDaneOdecet) {
            $rodinnyVztah = '0,1,8,9'; //manzel, manzelka, reg. partner, reg. partnerka
        } else {
            $rodinnyVztah = '0,1,2,3,8,9'; //2 = druh, 3 = druzka
        }
        $params = $this->database->createParameters();

        //nalézt posledního partnera pokud jich je v historii více, a pokud je pouze jeden pak nemusí mít datum platnosti vůbec vyplněn
        $sql =
            "SELECT RodinnyPrislusnik.RC_RodinnehoPrislusnika, RodinnyPrislusnik.Jmeno, RodinnyPrislusnik.Prijmeni, RodinnyPrislusnik.DatumNarozeni, RodinnyPrislusnik.DrzitelPrukazuZTTP, RodinnyPrislusnik.RodneCisloRP,
                (SELECT MAX(DrzitelPrukazuZTTP) FROM RodinnyPrislusnik RP_ZTP WHERE RP_ZTP.RC_RodinnehoPrislusnika = RodinnyPrislusnik.RC_RodinnehoPrislusnika 
                    AND (RP_ZTP.DatumPriznaniZTTP IS NULL OR RP_ZTP.DatumPriznaniZTTP <= ". $params->addDateParameter($this->rok->value) . ") 
                    AND (RP_ZTP.DatumPlatnostiZTPP IS NULL OR RP_ZTP.DatumPlatnostiZTPP >= ". $params->addDateParameter(DateUtils::lastDayOfYear($this->rok->value)) . "))
            FROM ClenoveRodiny 
                JOIN RodinnyPrislusnik ON (RodinnyPrislusnik.RC_RodinnehoPrislusnika = ClenoveRodiny.RC_RodinnehoPrislusnika)
            WHERE ClenoveRodiny.RC  = " . $params->addParameter($this->pracovniPomer->rc) .
            " AND ClenoveRodiny.RodinnyVztah IN (" . $rodinnyVztah . ") " .
            " AND ClenoveRodiny.StejnaAdresa = 1 " .
            " AND (ClenoveRodiny.PartnerDatumOd IS NULL OR ClenoveRodiny.PartnerDatumOd = (SELECT MAX (PartnerDatumOd) FROM ClenoveRodiny AS Partner " .
            "           WHERE Partner.RodinnyVztah IN (" . $rodinnyVztah . ") AND Partner.StejnaAdresa = 1 " .
            "            AND (Partner.PartnerDatumDo IS NULL OR Partner.PartnerDatumDo >= ". $params->addDateParameter($this->rok->value) . ") AND Partner.RC = " . $params ->addParameter($this->pracovniPomer->rc) . ") )" .
            " AND (ClenoveRodiny.PartnerDatumDo IS NULL OR ClenoveRodiny.PartnerDatumDo >= " . $params->addDateParameter($this->rok->value) . ")";

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();

        if ($row) {
            $partner = new PartnerPoplatnika();
            $partner->id = $row[0];
            $partner->jmeno = $row[1];
            $partner->prijmeni = $row[2];
            $partner->datumNarozeni = $this->database->stringToTimestamp($row[3]);
            $partner->ztpp = (int)$row[6] == 1;
            $partner->rodneCislo = $row[5];

            $this->partnerPoplatnika = $partner;
        }
    }

    /**
     * @return void
     * @throws DBException
     */
    private function nactiBydlistePoplatnika($datum)
    {
        $adresa = new AdresaOsoby();

        $adresy = AdresaOsoby::nactiAdresuOsoby($this->database, $this->rc->value, $datum, 1);

        foreach ($adresy as $adresaOsoby) {
            if ($adresaOsoby->platnostOd > $adresa->platnostOd)
                $adresa = $adresaOsoby;
        }

        $this->bydliste = $adresa;
    }

    private function nactiZdravotniStav($datum)
    {
        $this->zdravotniStav = ZdravotniStavOsoby::nactiStav($this->database, $this->pracovniPomer->rc, $datum);
        if (is_null($this->zdravotniStav)) {
            $this->zdravotniStav = new ZdravotniStavOsoby();
        }
    }

    private function popisZmenyOdpoctu($prvniZaznam, $zakladniOdpocet, $odpocetNaStudenta, $naZdravotniStav, $predchoziZakladniOdpocet, $predchoziOdpocetNaStudenta, $predchoziNaZdravotniStav)
    {
        $classZmena = "font-weight-bold";
        $classText = "";
        
        $result = "";
        //Substituce - Přidána/Zrušena
        $prohlaseniTextPridani = "<span class='$classZmena'>" . "&nbsp;" . TextManager::_('Prohlaseni_zmena_pridani'). "</span>" ;
        $prohlaseniTextZruseni = "<span class='$classZmena'>" . TextManager::_('Prohlaseni_zmena_zruseni'). "</span>" ;

        if ($prvniZaznam) {
            // pokud se jedna o prvni zaznam pak 1 považovat za přidání slevy

            if ($zakladniOdpocet == 1) {
                // Pridany slevy - jelikož se jedna o prvni zaznam neni vetev else
                $result .= $prohlaseniTextPridani .  "<span class='$classText'>" . TextManager::_('Prohlaseni_zmena_zakladni_slevy') . "</span>" . "<br>";
            }

            if ($odpocetNaStudenta == 1) {
                // Pridany slevy - jelikož se jedna o prvni zaznam neni vetev else
                $result .= $prohlaseniTextPridani . "</span>" . "<span class='$classText'>" . TextManager::_('Prohlaseni_zmena_sleva_studenta') . "</span>" . "<br>";
            }

            if ($naZdravotniStav > 0) {
                // Pridany slevy - jelikož se jedna o prvni zaznam neni vetev else
                $result .= $prohlaseniTextPridani . "</span>" . "<span class='$classText'>" . TextManager::_("Prohlaseni_zmena_slevy_zdravi_$naZdravotniStav") . "</span>" . "<br>";
            }

        } else {
            // zde jiz zjistovat zda doslo ke zmene - jaka byla predchozi hodnota Základní slevu na poplatníka ?

            $prohlaseniTextSlevaNaPoplatnika = "<span class='$classText'>" . TextManager::_('Prohlaseni_zmena_zakladni_slevy'). "</span>";
            if ($zakladniOdpocet > $predchoziZakladniOdpocet){
                // 0 -> 1 (Pridano)
                $result .= $prohlaseniTextPridani .  $prohlaseniTextSlevaNaPoplatnika . "<br>";
            } elseif ($zakladniOdpocet < $predchoziZakladniOdpocet){
                // 1 -> 0 (Zruseno)
                $result .= $prohlaseniTextZruseni .  $prohlaseniTextSlevaNaPoplatnika . "<br>";
            }


            if ($naZdravotniStav > $predchoziNaZdravotniStav && $predchoziNaZdravotniStav == 0) {
                // 0 -> 1 (Pridano)
                $result .= $prohlaseniTextPridani .  "<span class='$classText'>" . TextManager::_("Prohlaseni_zmena_slevy_zdravi_$naZdravotniStav") . "</span>" . "<br>";
            } elseif ($naZdravotniStav == 2 && $predchoziNaZdravotniStav == 0) {
                // 0 -> 2 (Pridano)
                $result .= $prohlaseniTextPridani .  "<span class='$classText'>" . TextManager::_("Prohlaseni_zmena_slevy_zdravi_$naZdravotniStav") . "</span>" . "<br>";
            } elseif ($naZdravotniStav > $predchoziNaZdravotniStav && $predchoziNaZdravotniStav == 1) {
                // 1 -> 2 (Pridano)
                $result .= $prohlaseniTextZruseni .  "<span class='$classText'>" . TextManager::_("Prohlaseni_zmena_slevy_zdravi_$predchoziNaZdravotniStav") . "</span>" . "<br>";
                $result .= $prohlaseniTextPridani .  "<span class='$classText'>" . TextManager::_("Prohlaseni_zmena_slevy_zdravi_$naZdravotniStav") . "</span>" . "<br>";
            } elseif ($naZdravotniStav == 1 && $naZdravotniStav < $predchoziNaZdravotniStav && $predchoziNaZdravotniStav == 2) {
                // 2 -> 1 (Zruseno)
                $result .= $prohlaseniTextZruseni .  "<span class='$classText'>" . TextManager::_("Prohlaseni_zmena_slevy_zdravi_$predchoziNaZdravotniStav") . "</span>" . "<br>";
                $result .= $prohlaseniTextPridani .  "<span class='$classText'>" . TextManager::_("Prohlaseni_zmena_slevy_zdravi_$naZdravotniStav") . "</span>" . "<br>";
            } elseif ($naZdravotniStav < $predchoziNaZdravotniStav && $predchoziNaZdravotniStav == 1) {
                // 1 -> 0 (Zruseno)
                $result .= $prohlaseniTextZruseni .  "<span class='$classText'>" . TextManager::_("Prohlaseni_zmena_slevy_zdravi_$predchoziNaZdravotniStav") . "</span>" . "<br>";
            } elseif ($naZdravotniStav == 0 && $predchoziNaZdravotniStav == 2) {
                // 2 -> 0 (Zruseno)
                $result .= $prohlaseniTextZruseni .  "<span class='$classText'>" . TextManager::_("Prohlaseni_zmena_slevy_zdravi_$predchoziNaZdravotniStav") . "</span>" . "<br>";
            }


            $prohlaseniTextSlevaStudent = "<span class='$classText'>" . TextManager::_('Prohlaseni_zmena_sleva_studenta'). "</span>" ;
            if ($odpocetNaStudenta > $predchoziOdpocetNaStudenta) {
                // 0 -> 1 (Pridano)
                $result .= $prohlaseniTextPridani .  $prohlaseniTextSlevaStudent . "<br>";
            } elseif ($odpocetNaStudenta < $predchoziOdpocetNaStudenta) {
                // 1 -> 0 (Zruseno)
                $result .= $prohlaseniTextZruseni .  $prohlaseniTextSlevaStudent . "<br>";
            }
        }

        return $result;
    }

    /** Prohlaseni lze editovat, pokud je povoleno elektronicke prohlaseni a prohlaseni neni ve schvalenem stavu.
     * @return bool
     */
    public function lzeEditovatProhlaseni()
    {
        if ($this->lzePodepsatProhlaseniElektronicky() && !$this->jeProhlaseniSchvaleno()) {
            //prohlaseni podepsane uzivatelem lze editovat, jen pokud je to povoleno v nastaveni portalu
            if ($this->findFieldByName('ProhlaseniPortalStav')->value == RocniVyuctovaniDaneRecord::PROHLASENI_PODEPSANO && !RonPortalSettings::getIntValue('dane_editovat_podepsane_prohlaseni')) {
                return false;
            }
            return true;
        }

        return false;
    }

    public function lzeEditovatVyuctovani()
    {
        if ($this->lzePodepsatZadostElektronicky() && !$this->jeZadostSchvalena() && DaneController::lzeEditovatVyuctovani($this['rok'], $this['ZadostPortalStav'])) {
            return true;
        }
        return false;
    }

    public function parseMemoToArray($memo)
    {
        $arr = [];

        $rows = explode("\n", $memo);
        foreach ($rows as $row) {
            $parsedRow = explode("$-$", $row);
            $arr[] = $parsedRow;
        }

        return $arr;
    }

    /** Vrati nadpis pro formular prohlaseni
     * @return string|null
     */
    public function nadpisProhlaseni()
    {
        switch ($this['ProhlaseniPortalStav']) {
            case 2:
                return TextManager::_("prohlaseni_vraceno");
            case 1:
                return TextManager::_("Mzdy_prohlaseni") . " - " . TextManager::_("Mzdy_schvaleno");
            case 0:
            default:
                return TextManager::_("Mzdy_prohlaseni");
        }
    }

    public function nadpisVyuctovani()
    {
         return TextManager::_("zadost") . " - " .  ($this['ProvestRocniVyuctovani'] == 0 ? TextManager::_("zadost_nezadam") : TextManager::_("zadost_zadam") );
    }

    private function renderCombo($optionsArray, $varName, $value, $editovatelne = false, $htmlParams = null)
    {
        $combo = new ComboBox($varName, $optionsArray);
        $combo->setDataTypeComparison(false);
        if (is_null($htmlParams) || !isset($htmlParams['class'])) {
            $combo->setClassName('');
        }

        if ($htmlParams) {
            $combo->htmlParams = $htmlParams;
        }

        if ((!$editovatelne) ||
            ($this->prohlaseni && $this->jeProhlaseniSchvaleno()) || // schvalene prohlaseni nebude editovatelne
            (!$this->prohlaseni && !$this->lzeEditovatVyuctovani())  //schvalene vyuctovani nelze editovat
        ) {
            $combo->htmlParams['disabled'] = 'true';
        }

        $combo->setSelectedOption($value);

        $combo->render();
    }

    public function renderComboProhlaseni()
    {
        $volby =
            [
                0 => '',
                1 => 'X',
//                2 => 'XD'
            ];

        $this->renderCombo($volby, 'prohlasenikdanicinim', $this['ProhlaseniPortal'] ?? 1, true);
    }

    public function renderComboOdpocet($varName, $value, $editovatelne = false, $htmlParams = null)
    {
        $volby =
            [
                0 => '',
                1 => 'X',
            ];

        $this->renderCombo($volby, $varName, $value, $editovatelne, $htmlParams);
    }

    public function renderComboPoradiDitete($varName, $value, $editovatelne = false, $htmlParams = null, $zmenaProhlaseni = false)
    {
        $volby =
            [
                0 => 'N',
                1 => '1',
                2 => '2',
                3 => '3',
            ];
        if ($zmenaProhlaseni) {
            $volby[''] = " ";
            if (is_null($value)) {
                $value = '';
            }
        }

        $this->renderCombo($volby, $varName, $value, $editovatelne, $htmlParams);
    }

    public function renderComboAnoNe($varName, $value, $editovatelne = false, $htmlParams = null)
    {
        $volby =
            [
                0 => TextManager::_('Str_No'),
                1 => TextManager::_('Str_Yes'),
            ];

        $this->renderCombo($volby, $varName, $value, $editovatelne, $htmlParams);
    }

    public function renderComboMesicZmeny($varName, $value, $editovatelne = false, $htmlParams = null)
    {
        $pocatecniMesic = 1;

        //Vyhleda se neuzavrene obdobi, ve kterem neni uzavrena mzda
        $params = $this->database->createParameters();
        $sql =
            "SELECT MIN(PocatekObdobi) 
            FROM Obdobi 
            WHERE Uzavreno = " . $params->addParameter(0) .
            " AND Aplikace = " . $params->addParameter('MZDY') .
            " AND PocatekObdobi >= " . $params->addDateParameter($this->rok->value) .
            " AND PocatekObdobi < " . $params->addDateParameter(DateUtils::lastDayOfYear($this->rok->value)) .
            " AND NOT EXISTS(SELECT RC FROM Mzda
                            WHERE RC = " . $params->addParameter($this->pracovniPomer->rc) .
                            " AND OsobniCislo = " . $params->addParameter($this->pracovniPomer->osobniCislo) .
                            " AND DatumNastupu = " . $params->addDateParameter($this->pracovniPomer->datumNastupu) .
                            " AND Mzda.Obdobi = Obdobi.PocatekObdobi " .
                            " AND Mzda.Schvaleno = 1" .
            ")";

        $otevreneObdobi = DBUtils::getSqlValue($this->database, $sql, $params);

        if ($otevreneObdobi) {
            $otevreneObdobi = $this->database->stringToTimestamp($otevreneObdobi);
            $pocatecniMesic = (int)date('n', $otevreneObdobi);
        }

        //Prednastavit aktualni mesic pokud se jedna o novou zmenu
        if (!isset($value)) {
            $value = date("n", time());
        }

        $this->renderComboMesice($varName, $value, $pocatecniMesic, false, $editovatelne, $htmlParams);
    }

    public function renderComboMesice($varName, $value, $pocatecniMesic = 1, $emptyValue = true, $editovatelne = true, $htmlParams = null)
    {
        $volby = [];

        if ($emptyValue) {
            $volby[0] = '';
        }

        for ($i = $pocatecniMesic; $i <=12; $i++) {
            $volby[$i] = TextManager::_("Month_$i");
        }

        $this->renderCombo($volby, $varName, $value, $editovatelne, $htmlParams);
    }

    public function renderComboBytovePotreby($varName, $value, $editovatelne = false, $htmlParams = null)
    {
        $volby =
            [
                0 => TextManager::_('zadost_prohlaseni_k_nezdanitelne_casti_neuplatnuje'),
                1 => TextManager::_('zadost_prohlaseni_k_nezdanitelne_casti_uplatnuje')
            ];

        $this->renderCombo($volby, $varName, $value, $editovatelne, $htmlParams);
    }

    public function renderMesiceCheckboxes($varName, $class, $mesice = '000000000000', $disabled = true)
    {
        $mesiceArr = str_split($mesice);
        GuiUtils::renderCheckbox("vse_$varName", TextManager::_('Vybrat_vse'), false, $disabled ? ['disabled' => 'true'] : null, "dane_vsechny_mesice $class");
        for ($i = 1; $i <= 12; $i++) {
            GuiUtils::renderCheckbox("{$varName}_{$i}", TextManager::_("Month_$i"), $_POST["{$varName}_{$i}"] ?? $mesiceArr[$i-1], $disabled ? ['disabled' => 'true'] : null, "mesic $class");
        }
    }

}