<?php
/** roman.Szablatura 22.11.2016 */

namespace RonPortal\Napady;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\TextManager;
use RonPortal\Prava;
use RonPortal\User;


/**
 * Class Napad
 * @package Ron\RonPortal\Napady
 */
class Napad
{
    /** @var int */
    public $id;
    /** @var int */
    public $datum;
    /** @var int        Kod stavu napadu */
    public $stav;
    /** @var string     Lookup StavNapadu.Popis */
    public $stavNapaduPopis;
    /** @var string */
    public $autorRC;
    /** @var string|null */
    public $oblast;
    /** @var string */
    public $nazev;
    /** @var float */
    public $rozpocet;
    /** @var string|null */
    public $pilotRC;
    /** @var string|null */
    public $priorita;
    /** @var string|null */
    public $vyjadreniPilota;
    /** @var string|null */
    public $podrobnyPopis;
    /** @var float|null */
    public $odmenaCastka;
    /** @var int|null */
    public $odmenaVyplacenaVObdobi;

    /** @var int|null   Lookup StavNapadu.Barva */
    public $barvaNapadu;

    /** @var string */
    public $autorPrijmeniJmeno;
    /** @var string */
    public $pilotPrijmeniJmeno;

    /** @var string|null */
    public $insert_UserName;
    /** @var int|null */
    public $insert_Timestamp;
    /** @var string|null */
    public $update_UserName;
    /** @var int|null */
    public $update_Timestamp;

    /** @var string|null    Lookup OblastNapadu.Popis */
    public $oblastNapaduPopis;

    /** @var int            COUNT(DokumentNapadu ...) */
    public $pocetDokumentu;


    /**
     * @param Database $database
     * @return Napad
     * @param $id
     * @throws Exception
     */
    public static function nactiNapad($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT ID, Napad.Datum, Napad.Stav, StavNapadu.Popis, ' . // 0 - 3
            ' Autor.RC, Oblast, Nazev, Rozpocet, Pilot.RC, Priorita, ' . $database->castWideMemo('VyjadreniPilota') .', ' . //4 - 10
            $database ->castWideMemo('PodrobnyPopis ') . ', OdmenaCastka, OdmenaVyplacenaVObdobi, StavNapadu.Barva, ' .     //11 - 14
            ' Autor.Prijmeni, Autor.Jmeno, Autor.Titul, '. // 15..17
            ' Pilot.Prijmeni, Pilot.Jmeno, Pilot.Titul, '. // 18..20

            //"Coalesce (Autor.Prijmeni, '') {fn_concat()} ' ' {fn_concat()} Coalesce (Autor.Jmeno, '') {fn_concat()} Coalesce (', ' {fn_concat()}  Autor.Titul,'') Autor, " .       //15
            //"Coalesce (Pilot.Prijmeni, '') {fn_concat()} ' ' {fn_concat()} Coalesce (Pilot.Jmeno, '') {fn_concat()} Coalesce (', ' {fn_concat()}  Pilot.Titul,'') Pilot, " .     //16
            'Napad.Insert_UserName, Napad.Insert_Timestamp, Napad.Update_UserName, Napad.Update_Timestamp, ' .             //17 - 20
            'OblastNapadu.Popis, (SELECT COUNT(*) FROM DokumentNapadu WHERE DokumentNapadu.IDNapadu = Napad.ID) ' .         //21 - 22

            "\nFROM Napad LEFT JOIN Osoba Autor ON (Napad.Autor = Autor.RC)".
            ' LEFT JOIN Osoba Pilot ON (Napad.Pilot = Pilot.RC)' .
            ' LEFT JOIN StavNapadu ON (Napad.Stav = StavNapadu.Kod)'.
            ' LEFT JOIN OblastNapadu ON  (OblastNapadu.Kod = Napad.Oblast)' .

            'WHERE ID = ' . $params->addParameter($id);

        $cursor = $database->getSqlResult($sql, $params);

        if (!$row = $cursor->fetch()){
            throw new Exception(TextManager::_('napad_neexistuje'));
        }
        $cursor ->close();

        $napad = new Napad();
        $napad->nactiNapadZRadku($database, $row);

        return $napad;
    }


    /**
     * @param Database $database
     * @param $row
     */
    private function nactiNapadZRadku($database, $row)
    {
        $this->id = (int)$row[0];
        $this->datum = $database->stringToTimestamp($row[1]);
        $this->stav = (int)$row[2];
        $this->stavNapaduPopis = $row[3];
        $this->autorRC = $row[4];
        $this->oblast = $row[5];
        $this->nazev = $row[6];
        $this->rozpocet = isset($row[7]) ? (float)$row[7] : null;
        $this->pilotRC = $row[8];
        $this->priorita = $row[9];

        $this->vyjadreniPilota = $row[10];
        $this->podrobnyPopis = $row[11];

        $this->odmenaCastka = isset($row[12]) ? (float)$row[12] : null;
        $this->odmenaVyplacenaVObdobi = $database->stringToTimestamp($row[13]);
        $this->barvaNapadu = $row[14];

        $this->autorPrijmeniJmeno = $row[15] . ' ' .$row[16] . ($row[17] ? ', ' . $row[17] : '');
        $this->pilotPrijmeniJmeno = $row[18] . ' ' .$row[19] . ($row[20] ? ', ' . $row[20] : '');;

        $this->insert_UserName = $row[21];
        $this->insert_Timestamp = $database->stringToTimestamp($row[22]);
        $this->update_UserName = $row[23];
        $this->update_Timestamp = $database->stringToTimestamp($row[24]);

        $this->oblastNapaduPopis = $row[25];
        $this->pocetDokumentu = (int)$row[26];
    }


    public function jePravoNaNapad()
    {
        //Kontrola podle data napadu
        Prava::nastavObdobiProPrava($this->datum, $this->datum);

        return
            Prava::vratPrava('napady') ->lzeZobrazitRC($this->autorRC) ||
            Prava::vratPrava('napady_admin') ->lzeZobrazitRC($this->autorRC) ||
            User::hasRC($this ->pilotRC);
    }
}

