<?php
/**
 * stanislav.elbl, 19.09.2022 12:50
 */

namespace RonPortal\Personalistika;

use Ron\Controls\BrowserField;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;

class DetailDistribuovanehoDokumentu extends RonPortalBrowser
{
    /** @var int */
    private $idDokumentu;
    /** @var int */
    private $platnostOd;
    /** @var int */
    private $idVzkazu;

    /** @var PravaNaOsoby  */
    var $pravaProhlizet;

    var $jenPrectene = 0; // vse

//    /** @var DistribuovaneDokumentyInfoField*/
//    var $filterForm;

    private $seznamOsob;

    /** @var array */
    private $dokumentInfoDBRow;

    /**
     * DistribuovaneDokumentyDetail constructor.
     * @param Database $database
     * @param $idDokumentu
     * @param $platnostOd
     * @param $idVzkazu
     * @throws \Exception
     */
    function __construct($database, $idDokumentu, $platnostOd, $idVzkazu)
    {
        parent::__construct($database, 'dokumenty');

        $this ->idDokumentu = $idDokumentu;
        $this ->platnostOd = $platnostOd;
        $this ->idVzkazu = $idVzkazu;

        // nacte informace o dokumentu pro zobrazeni ve filtru
        $params = $this->database->createParameters();

        $sql =
            'SELECT Dokument.Datum, VerzeDokumentu.Popis, DatumCas, Dokument.JmenoSouboru,' .    //0 - 3
            '  DokumentVzkazu.Precteno, TypDokumentu.Popis, VerzeDokumentu.PotvrditDo, TypDokumentu.Distribuovany' .    //4 - 7
            ' FROM Vzkaz, DokumentVzkazu, VerzeDokumentu, Osoba, Dokument, TypDokumentu' .
            ' WHERE DokumentVzkazu.Idvzkazu = Vzkaz.Id ' .
            '  AND Dokument.Typ = TypDokumentu.Kod ' .
            '  AND Vzkaz.RCPrijemce = Osoba.RC ' .
            '  AND DokumentVzkazu.IdDokumentu = ' . $params->addParameter($this->idDokumentu) .
            '  AND DokumentVzkazu.PlatnostOd = ' . $params->addTimestampParameter($this->platnostOd) .
            '  AND Dokument.Id = VerzeDokumentu.Id ' .
            '  AND DokumentVzkazu.IdDokumentu = VerzeDokumentu.Id and DokumentVzkazu.PlatnostOd = VerzeDokumentu.PlatnostOd ';


        $cursor = $this->database->getSqlResult($sql,$params);
        $this ->dokumentInfoDBRow = $cursor->fetch();
        $cursor ->close();

        $this->pravaProhlizet = Prava::vratPrava('dokumenty_prohl');

        $this->title = TextManager::_('detail_distribuovane_dokumenty');

        if (isset($_GET['prectene'])) {
            $this ->jenPrectene = (int)$_GET['prectene'];
            $_SESSION['jenPrecteneDokumenty'] = $this ->jenPrectene;
        } else {
            $this ->jenPrectene = $_SESSION['jenPrecteneDokumenty'] ?? 0;
        }

        $this->browserUrl =
            '?menuid=dokumenty&distribuovanedetail=1&dokumentid=' . $idDokumentu
            . '&vzkazid=' . $idVzkazu
            . '&platnostod=' . DateUtils::dateToUrlString($platnostOd)
            . '&prectene=' . $this ->jenPrectene
            . '&';

        $this->defaultSort = 1;
        $this->primaryField = 0;

        $this->sqlFromPart =
            'Vzkaz JOIN DokumentVzkazu ON (DokumentVzkazu.IdVzkazu = Vzkaz.Id) ' .
            'JOIN Osoba ON (Vzkaz.RCPrijemce = Osoba.RC)'.
            'JOIN PracovniPomer ON (Osoba.RC = PracovniPomer.RC)';

        $params = $database ->createParameters();
        $this->sqlWherePart = new SqlStatement(
            ' PracovniPomer.DatumNastupu <= ' . $params ->addDateParameter(time()) .
            ' and (PracovniPomer.DatumUkonceni >= ' . $params ->addDateParameter(time()) . ' OR PracovniPomer.DatumUkonceni IS NULL) ' .
            ' and DokumentVzkazu.idDokumentu = ' . $params ->addParameter((int)$idDokumentu) .
            ' and DokumentVzkazu.PlatnostOd = ' . $params ->addTimestampParameter($platnostOd),
            $params);

//        $this->filterForm = new DistribuovaneDokumentyInfoField($database, $idDokumentu, $idVzkazu, $platnostOd);
//        $this->filterForm ->loadData();

        // Prijmeni
        $this->addField()
            ->setSqlFieldName('Osoba.Prijmeni') ->setHeaderText('Prijmeni') ->setOrdering(true)
            ->setCode('col-prijmeni')
            ->setSortSQL('Osoba.Prijmeni, Osoba.Jmeno') ->setSortSQLDesc('Osoba.Prijmeni DESC, Osoba.Jmeno DESC');

        // Jmeno
        $this->addField()
            ->setSqlFieldName('Osoba.Jmeno') ->setHeaderText('Jmeno') ->setOrdering(true)
            ->setCode('col-jmeno')
            ->setSortSQL('Osoba.Jmeno, Osoba.Prijmeni') ->setSortSQLDesc('Osoba.Jmeno DESC, Osoba.Prijmeni DESC');

        //informace o precteni se zobrazi jen pokud je precteni dokumentu vyzadovano pro dany typ
        if ($this->dokumentInfoDBRow[7] != 2) {
            // Precteno
            $this->addField(BrowserField::FIELD_TYPE_SELECT)
                ->setSqlFieldName('DokumentVzkazu.Precteno')->setHeaderText('precteno')->setOrdering(true)
                ->setCode('col-precteno')
                ->setOptions([0 => TextManager::_('Str_No'), 1 => TextManager::_('Str_Yes')])
                ->setAlign('center');

            // Identifikace podpisu - memo
            $this->addField()
                ->setSqlFieldName('DokumentVzkazu.IdentifikacePodpisu')->setHeaderText('identifikace_podpisu')
                ->setCode('col-identifikacepodpisu');

            // Vyjadreni
            $this->addField(BrowserField::FIELD_TYPE_COMMENT)
                ->setSqlFieldName('DokumentVzkazu.Vyjadreni')->setHeader(TextManager::_('vyjadreni') . ': ')->setClassName('browserpoznamka')
                ->setCode('col-vyjadreni');
        }

        // hidden
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.RC') ->setCode('col-rc');

        $this->seznamOsob = [];

    }


    function setFilterForm()
    {
        // parent::setFilterForm(); - nedela nic
    }


    function renderFilter()
    {
        if ($this ->dokumentInfoDBRow) {
            echo
                '<div class="form middle-form">' .
                '<div class="form-content">';

            // podrobnosti o dokumentu
            echo
                '<div class="row"><div class="col-4 caption">' . TextManager::_('typ_dokumentu') . '</div><div class="col-8">' . $this->dokumentInfoDBRow[5] . '</div></div>' .
                '<div class="row"><div class="col-4 caption">' . TextManager::_('Popis') . '</div><div class="col-8 font-weight-bold">' . $this->dokumentInfoDBRow[1] . '</div></div>' .
                '<div class="row"><div class="col-4 caption">' . TextManager::_('platnost_dokumentu_od') . '</div><div class="col-8 font-weight-bold">' . DBUtils::getFormatedValue($this ->database, 'date', $this ->dokumentInfoDBRow[0]) . '</div></div>' .
                '<div class="row"><div class="col-4 caption">' . TextManager::_('datum_a_cas_distribuce') . '</div><div class="col-8">' . DBUtils::getFormatedValue($this ->database, 'datetime', $this ->dokumentInfoDBRow[2]) . '</div></div>' .
                ($this ->dokumentInfoDBRow[6]
                    ? '<div class="row"><div class="col-4 caption">' . TextManager::_('dokumenty_potvrdit_do') . '</div><div class="col-8 font-weight-bold">' . DBUtils::getFormatedValue($this ->database, 'date', $this ->dokumentInfoDBRow[6]) . '</div></div>'
                    : '');

            // odkaz na stahnuti
            echo
                '<div class="row margin-top-2">' .
                    '<div class="col-4 caption"></div>' .
                    '<div class="col-8 ">' .
                    '<a href="?menuid=dokumenty&distribuovanedokumenty=1&dokumentid=' . $this ->idDokumentu . '&vzkazid=' . $this ->idVzkazu . '&platnostod=' . DateUtils::dateToUrlString($this ->platnostOd) . '" target="_blank">' .
                    TextManager::_('nahled_dokumentu') .
                    '</a>' .
                    '</div>' .
                '</div>';

            //filtr na prectene jen pro distriuovane vyzadujici precteni
            if ($this->dokumentInfoDBRow[7] != 2) {
                // filtr na prectene dokumenty
                echo
                    '<div class="row margin-top-2">' .
                    '<div class="col-4 caption">' . TextManager::_('prectene_dokumenty') . '</div>' .
                    '<div>';

                LinksList::renderLinks(
                    [0 => TextManager::_('Vse'), 1 => TextManager::_('Str_Yes'), 2 => TextManager::_('Str_No')],
                    $this->jenPrectene,
                    '?menuid=dokumenty&distribuovanedetail=1&dokumentid=' . $this->idDokumentu
                    . '&vzkazid=' . $this->idVzkazu
                    . '&platnostod=' . DateUtils::dateToUrlString($this->platnostOd)
                    . '&prectene=',
                    '', false);

                echo
                    '</div>' .
                    '</div>';
            }

            // odkaz zpet
            echo
                '<div class="row margin-top-2">' .
                '<div class="col-12"><a href="?menuid=dokumenty">' . TextManager::_('Zpet') . '</a></div>' .
                '</div>';

            echo
                '</div>' .
                '</div>';
        }
    }

    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();

        $result = '(' . $this->pravaProhlizet->sqlFiltr('PracovniPomer', time(), time(), $params) .')';

        //filtr na prectene jen pro distriuovane vyzadujici precteni
        if ($this->dokumentInfoDBRow[7] != 2) {
            switch ($this->jenPrectene) {
                case 1:
                    $result .= ' AND DokumentVzkazu.Precteno = ' . $params->addParameter(1);
                    break;

                case 2:
                    $result .= ' AND DokumentVzkazu.Precteno = ' . $params->addParameter(0);
                    break;

                default:
                    break;
            }
        }

        return new SqlStatement($result, $params);
    }

    /**
     * @param array $values
     * @return bool
     */
    function enableLine($values)
    {
        //RC jiz v seznamu je, nebude se vypisovat
        if (array_key_exists($values['col-rc'],$this->seznamOsob)) {
            return false;
        }

        $this->seznamOsob[$values['col-rc']] = 1;
        return true;
    }

}