<?php
/**
 * roman.szablatura, 11.03.2016
 */

namespace RonPortal\Personalistika;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\Http\HtmlHeaders;
use Ron\HttpUtils;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;
use RonPortal\System\Report;
use RonPortal\System\ReportOutputObject;
use RonPortal\User;


/**
 * Class LekarskeProhlidkyController
 */
class LekarskeProhlidkyController extends JsonController
{
    const POVOLIT_LEKARSKE_PROHLIDKY = 'povolitprohlidky';

    /** @var Database  */
    private $database;

    /** @var  PravaNaOsoby */
    private $pravaProhlizet;

    /** @var  PravaNaOsoby */
    private $pravaAbsolvovat;

    /** @var  PravaNaOsoby */
    private $pravaZapis;

    /**
     * LekarskeProhlidkyController constructor.
     * @param Database $database
     * @throws \Exception
     */
    public function __construct(Database $database)
    {
        $this->database = $database;

        Prava::nastavObdobiProPrava(time(), time());

        $this->pravaProhlizet = Prava::vratPrava('lekarske_prohl');
        $this->pravaAbsolvovat = Prava::vratPrava('lekarske_absolv');
        $this->pravaZapis = Prava::vratPrava('lekarske_edit');
    }

    /**
     * @param $absolvovat
     * @return Redirect|LekarskaProhlidkaForm
     * @throws \Exception
     */
    private function lekarskaProhlidkaForm($absolvovat)
    {
        $pracPomer = null;

//        $action = (isset($_POST['formaction']) ? $_POST['formaction'] : (isset($_GET['formaction']) ? $_GET['formaction'] : ''));

//        if ($action != 'new') {
//            $pracPomer = PracovniPomer::nactiPP($this->database);
//
//            if (!($this->pravaProhlizet->lzeZobrazitRC($pracPomer->rc)
//                || $this->pravaAbsolvovat->lzeZobrazitRC($pracPomer->rc)
//                || $this->pravaZapis->lzeZobrazitRC($pracPomer->rc))
//            ) {
//                throw new NeniPravoException();
//            }
//        } else if (!$this ->pravaZapis ->jsouNejakePrava()) {
//            // nema pravo zapisovat => nemuze vytvorit novou prohlidku
//            throw new NeniPravoException();
//        }


        $f = new LekarskaProhlidkaForm($this->database, $this->pravaZapis, $this->pravaAbsolvovat, $absolvovat, $pracPomer);

        if ($f->handleAction()){
            return new Redirect(null, '?menuid=lekarskeprohlidky');
        }

        return $f;
    }

    /**
     * @return BaseViewObject
     * @throws \Exception
     */
    public function  zdravotniProhlidkaDetailView()
    {
        $pracPomer = new PracovniPomer();
        $pracPomer->nactiPPPodleId($this->database, (int)$_GET['osoba']);

        if (!$this->pravaProhlizet->lzeZobrazitRC($pracPomer->rc)) {
            throw new NeniPravoException();
        }

        $zdravotniprohlidka = LekarskaProhlidka::nactiProhlidku($this->database, $pracPomer, $_GET['kod'], $_GET['vznik']);

        $reportList = Report::loadFormReports($this ->database, 'LekarskeProhlidky', 'LekarskeProhlidky');

        Model::addData('zdravotniprohlidka', $zdravotniprohlidka);
        Model::addData('pravaProhlizet', $this->pravaProhlizet);
        Model::addData('reportList', $reportList);

        return new BaseViewObject('ronportal/personalistika/view_lekarska_prohlidka.php');
    }

    /**
     * @param int $idOsoby
     * @param int $id ID dokumentu
     * @return DokumentOutputObject
     * @throws \Exception
     */
    private function downloadDocument($idOsoby, $id)
    {
        $params = $this->database->createParameters();
        $osobaRC = DBUtils  ::getSqlValue($this->database,
            'SELECT RC FROM PracovniPomer WHERE ID = ' . $params->addParameter($idOsoby),
            $params);

        if (!$this->pravaProhlizet->lzeZobrazitRC($osobaRC, time(), time())) {
            throw new NeniPravoException();
        }

        $params = $this->database->createParameters();
        $sql =
            'SELECT Dokument.ID' .
            ' FROM GenerovaneLekProhlidky ' .
            '  JOIN DokumentyOsoby ON  (GenerovaneLekProhlidky.RC = DokumentyOsoby.GLP_RC' .
            '   AND  GenerovaneLekProhlidky.KodLekarskeProhlidky = DokumentyOsoby.GLP_KodLP' .
            '   AND  GenerovaneLekProhlidky.DatumVznikuPovinnosti = DokumentyOsoby.GLP_Datum)' .
            '  JOIN Dokument ON (DokumentyOsoby.id = Dokument.id)' .
            '  JOIN TypDokumentu ON (Dokument.typ = TypDokumentu.kod) ' .
            ' WHERE DokumentyOsoby.ID = ' . $params->addParameter((int)$id) .
            ' AND GenerovaneLekProhlidky.RC = ' . $params->addParameter($osobaRC) .
            ' AND ( Dokument.zobrazitNaIntrawebu = 1 OR (Dokument.zobrazitNaIntrawebu = 2 AND TypDokumentu.zobrazitNaIntrawebu = 1 ))';

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        if (!$row){
            throw new NeniPravoException();
        }

        return new DokumentOutputObject($this->database, $id);
    }

//    /**
//     * @param int $osoba
//     * @param string $kod kod lekarske prohlidky
//     * @param int $vznik datum vzniku povinnosti absolvovat lek. prohlidku
//     * @return DokumentUploadOutput
//     */
//    private function uploadDokument($osoba, $kod, $vznik)
//    {
//        $result = new DokumentUploadOutput();
//
//        try {
//            $params = $this->database->createParameters();
//            $osobaRC = DBUtils::getSqlValue($this->database,
//                "SELECT RC FROM PracovniPomer WHERE ID = " . $params->addParameter($osoba),
//                $params);
//
//            if (!$this->pravaProhlizet->lzeZobrazitRC($osobaRC)) {
//                throw new NeniPravoException();
//            }
//
//            $id = DokumentyUtils::vlozDokument($this ->database, 'soubor',
//                RonPortalSettings::getIntValue('lekprohl_typdokumentu'), $_POST['popis'],
//                $this);
//
//            // Vlozi zaznam do vazebni tabulky
//            $rec = new RonPortalDBRecord($this ->database, 'DokumentyOsoby', $this);
//
//            $rec ->addField('id', 'int', false, 0, $id);
//            $rec ->addField('rc', '', false, 15, $osobaRC);
//            $rec ->addField('GLP_RC', '', false, 15, $osobaRC);
//            $rec ->addField('GLP_KodLP', '', false, 10, $kod);
//            $rec ->addField('GLP_Datum', 'datetime', false, 0, $vznik);
//            $rec ->insertRecord();
//        } catch (\Exception $e) {
//            $result ->setError($e ->getMessage());
//        }
//
//        return $result;
//    }


    /**
     * @param int $idOsoby
     * @param string $kodLP kod lekarske prohlidky
     * @param int $datumPovinnosti datum vzniku povinnosti absolvovat lek. prohlidku
     * @return ReportOutputObject
     * @throws \Exception
     */
    private function report($idOsoby, $kodLP, $datumPovinnosti)
    {
        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $idOsoby);

        if ((!$this->pravaProhlizet->lzeZobrazitRC($pp->rc))) {
            throw new NeniPravoException();
        }

        $result = new ReportOutputObject();
        $result ->loadReportFromGet($this ->database, 'LekarskeProhlidky',false, 'LekarskeProhlidky');
        $result ->nastavPP($pp, false); // prava nekontrolovat - uz jsou zkontrolovana (a navic jsou jen pres RC)
        $result ->stringParams = ['kodLP' => $kodLP];
        $result ->dateParams = ['datumPovinnosti' => $datumPovinnosti];

        return $result;
    }

    /**
     * Dashboard "Moje prohlidky"
     * @param JsonOutputObject $output
     * @throws DBException
     */
    private function mojeProhlidky($output)
    {
        $html =
            '<table class="dashboardTable" data-ron-href="?menuid=lekarskeprohlidky">' .
            '<tr><th>' . TextManager::_('datum_vzniku_povinnosti') . '</th><th>' . TextManager::_('popis_zdravotni_prohlidky') . '</th><th></th></tr>';

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT (SELECT MAX(ID) FROM PracovniPomer WHERE PracovniPomer.RC = GenerovaneLekProhlidky.RC ), ' .  // 0
            '  GenerovaneLekProhlidky.KodLekarskeProhlidky, GenerovaneLekProhlidky.DatumVznikuPovinnosti, LekarskaProhlidka.Popis ' . // 1..3
            ' FROM GenerovaneLekProhlidky JOIN LekarskaProhlidka ON (LekarskaProhlidka.Kod = GenerovaneLekProhlidky.KodLekarskeProhlidky)' .
            ' WHERE GenerovaneLekProhlidky.RC = ' . $params ->addParameter(User::getRC()) .
            '  AND GenerovaneLekProhlidky.DatumAbsolvovani IS NULL' .
            ' ORDER BY DatumVznikuPovinnosti';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $html .=
                '<tr>' .
                '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[2])) . '</td>' .
                '<td>' . StringUtils::htmlTextConvert($row[3]) . '</td>' .
                "<td><a href='?menuid=lekarskeprohlidky&action=detail&osoba={$row[0]}&kod=" . HttpUtils::urlEncode($row[1]) .
                    "&vznik=" . $this->database->stringToTimestamp($row[2]) . "'>" . TextManager::_('Podrobnosti').'</a></td>' .
                '</tr>';
        }

        $html .= '</table>';

        $output ->addText('html', $html);
    }

    /**
     * Dashboard "Prohlidky podrizenych"
     * @param JsonOutputObject $output
     * @throws DBException
     */
    private function prohlidkyPodrizenych($output)
    {
        $html =
            '<table class="dashboardTable" data-ron-href="?menuid=lekarskeprohlidky">' .
            '<tr>' .
                '<th>' . TextManager::_('Zamestnanec') . '</th>' .
                '<th>' . TextManager::_('datum_vzniku_povinnosti') . '</th>' .
                '<th>' . TextManager::_('popis_zdravotni_prohlidky') . '</th>' .
                '<th></th>' .
            '</tr>';

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT (SELECT MAX(ID) FROM PracovniPomer WHERE PracovniPomer.RC = GenerovaneLekProhlidky.RC ), ' .  // 0
            '  GenerovaneLekProhlidky.KodLekarskeProhlidky, GenerovaneLekProhlidky.DatumVznikuPovinnosti, LekarskaProhlidka.Popis, ' . // 1..3
            ' Osoba.Prijmeni, Osoba.Jmeno ' . // 4..5
            ' FROM GenerovaneLekProhlidky JOIN LekarskaProhlidka ON (LekarskaProhlidka.Kod = GenerovaneLekProhlidky.KodLekarskeProhlidky)' .
            '   JOIN Osoba ON (Osoba.RC = GenerovaneLekProhlidky.RC)' .
            ' WHERE GenerovaneLekProhlidky.DatumAbsolvovani IS NULL' .


            '  AND EXISTS(SELECT PracovniPomer.RC FROM PracovniPomer ' .
            '             WHERE PracovniPomer.RC = GenerovaneLekProhlidky.RC ' .
            '               AND PracovniPomer.DatumNastupu <= ' . $params->addParameter($this->database->dateToParam(time())) .
            '               AND (PracovniPomer.DatumUkonceni >= ' . $params->addParameter($this->database->dateToParam(time())) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
            '               AND ' .  $this->pravaProhlizet->sqlFiltr('PracovniPomer', time(), time()) .
            '            )' .

            ' ORDER BY DatumVznikuPovinnosti, Osoba.Prijmeni, Osoba.Jmeno';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $html .=
                '<tr>' .
                '<td>' . StringUtils::htmlTextConvert($row[4]) . ' ' . StringUtils::htmlTextConvert($row[5]) . '</td>' .
                '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[2])) . '</td>' .
                '<td>' . StringUtils::htmlTextConvert($row[3]) . '</td>' .
                "<td><a href='?menuid=lekarskeprohlidky&action=detail&osoba={$row[0]}&kod=" . HttpUtils::urlEncode($row[1]) .
                    "&vznik=" . $this->database->stringToTimestamp($row[2]) . "'>" . TextManager::_('Podrobnosti').'</a></td>' .
                '</tr>';
        }

        $html .= '</table>';

        $output ->addText('html', $html);
    }

    /**
     * @param string $action
     * @param JsonOutputObject $output
     * @throws DBException
     */
    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'mojeprohlidky' :
                $this ->mojeProhlidky($output);
                break;
            case 'prohlidkypodrizenych' :
                $this ->prohlidkyPodrizenych($output);
                break;
        }
    }


    /**
     * @param string $action
     * @return BaseViewObject|ViewObject|OutputObject|string
     * @throws \Exception
     * @throws DBException
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue(self::POVOLIT_LEKARSKE_PROHLIDKY)){
            throw new NeniPravoException();
        }

        if(!($this->pravaProhlizet->jsouNejakePrava() || ($this->pravaAbsolvovat->jsouNejakePrava()) || ($this->pravaZapis->jsouNejakePrava()))){
            throw new NeniPravoException();
        }

        HtmlHeaders::getInstance()
            ->addCss('./css/person.css');

        TextManager::getInstance()->loadStrings('person.ini');

        $prohlidkyaction = $_GET['action'] ?? '';

        if (isset($_POST['formaction'])  ||  isset($_GET['formaction'])) {
            return $this->lekarskaProhlidkaForm(isset($_GET['absolvovat'])  &&  (int)$_GET['absolvovat']);
        }

        switch($prohlidkyaction) {
            // json akce
            case 'mojeprohlidky' :
            case 'prohlidkypodrizenych' :
                return parent::handleAction($action);

//            case 'lekarskaprohlidkaform':
//                return $this->lekarskaProhlidkaForm(false);
//            case 'absolvovat':
//                return $this->lekarskaProhlidkaForm(true);

            case 'detail':
                return $this->zdravotniProhlidkaDetailView();

            case 'download':
                return $this ->downloadDocument($_GET['osoba'],$_GET['dokument']);

//            case 'dokumentupload' :
//                return $this ->uploadDokument($_POST['idosoby'],$_POST['kod'],$_POST['vznik']);

            case 'report' :
                return $this ->report($_GET['osoba'], $_GET['kod'], $_GET['vznik']);

            default:
                $b = new LekarskeProhlidkyBrowser($this->database);
                $b->load();
                return $b;
        }
    }
}

///**
// * Class NovyDokumentLekarskaProhlidkaForm
// */
//class NovyDokumentLekarskaProhlidkaForm extends DokumentUploadForm
//{
//    function __construct($database, $osoba, $kod, $vznik )
//    {
//        parent::__construct($database, '?menuid=lekarskeprohlidky&action=dokumentupload');
//
//        $this->typDokumentu = RonPortalSettings::getIntValue('lekprohl_typdokumentu');
//
//        $this->hiddenFields = [
//            'dokumentlekarskeprohlidky' => '1',
//            'idosoby' => $osoba,
//            'kod' => $kod,
//            'vznik' => $vznik,
//        ];
//    }
//}
