<?php
/**
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 2.3.2022
 * Time: 13:16
 */

namespace RonPortal\Personalistika;

use Ron\DateUtils;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\User;

class VyplatniListkyBrowser extends RonPortalBrowser
{
    public function __construct($database)
    {
        parent::__construct($database, 'vyplatnilistky');

        $this->code = 'browservyplatnilistky';

        $this->title = TextManager::_('Vyplatni_listky');
        $this->tableId = 'Vyplatnilistky';
        $this->sqlSelectPart =
            'Dokument.Id';

        $this->sqlFromPart =
            'Dokument JOIN TypDokumentu ON (Dokument.Typ = TypDokumentu.Kod) ' .
            'JOIN DokumentyOsoby ON (Dokument.id = DokumentyOsoby.id)';
        $this->sqlWherePart =
            'TypDokumentu.Kod = -1' . //jen vyplatni listky
            ' AND TypDokumentu.zobrazitNaIntrawebu = 1';

        $this->primaryField = 0;
        $this->distinct = true;

        $this->cssClass = 'browser minwidth';

        $this ->addField()
            ->setSqlFieldName('Dokument.Datum') ->setHeaderText('Date') ->setFormat('date')
            ->setCode('col-datum')
            ->setOrdering(true) ->setClassName('dokument_datum');

        $this->defaultSort = 'col-datum'; // podle datumu

        $this->addField()
            ->setSqlFieldName('Dokument.Popis') ->setHeaderText('Popis')
            ->setCode('col-popis')
            ->setOrdering(true) ->setClassName('dokument_popis');

        $this->addField()
            ->setSqlFieldName('Dokument.JmenoSouboru') ->setLink('?menuid=vyplatnilistky') ->setLinkAttributes(['target' => '_blank'])
            ->setCode('col-jmenosouboru')
            ->setHeaderText('Dokument_nazev') ->setOrdering(true);

        $this ->browserUrl = '?menuid=vyplatnilistky&';
    }

    function setFilterForm()
    {
        $this ->filterForm = new RonPortalFilterForm($this ->database, $this ->action, true);

        $filtrDatum = new FilterFormFilterDatumOdDo($this->filterForm);
        $filtrDatum->setDatumOdDoLinks(['currentYear']);

        $this->filterForm->addFilter($filtrDatum);
    }

    function getSQLFilter()
    {
        $params = $this->database->createParameters();

        $sql =
            'DokumentyOsoby.RC = ' . $params->addParameter(User::getRC());

        $filter = $this->filterForm->buildSqlFilter($params);
        if (!empty($filter)) {
            $sql .= " AND " . $filter;
        }

        if ($this->filterForm->getFilterValue('datumod', 'filtrovatDatum')) {
            $sql .=
                " AND datum >= " . $params->addDateParameter($this->filterForm->getFilterValue('datumod', 'datumOd')) .
                " AND datum <= " . $params->addDateParameter($this->filterForm->getFilterValue('datumod', 'datumDo'));
        }

        return new SqlStatement($sql, $params);
    }


    function renderHeader()
    {
        parent::renderHeader();

        Prava::nastavObdobiProPrava(DateUtils::today());

        if (Prava::vratHodnotuPrava('osobni_heslo') != 0) {
            echo "<div id='vl_personal_paswd'><a href='?menuid=personalpassword'>" . TextManager::_('Nastaveni_osobniho_hesla') . "</a></div>";
        }
    }
}
