<?php
/**
 * Roman.Szablatura 15.3.2016
 */

namespace RonPortal\Pomucky;

use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use Ron\StringUtils;


class VelikostPracovniPomucky
{
    /** @var string     Char[10] */
    public $typVelikosti;

    /** @var string     Char[20] */
    public $velikost;

    /**
     * VelikostPracovniPomucky constructor.
     * @param string $typVelikosti
     * @param string $velikost
     */
    public function __construct($typVelikosti, $velikost)
    {
        $this->typVelikosti = $typVelikosti;
        $this->velikost = $velikost;
    }
}


/**
 * Class PracovniPomucka
 * @package Ron\Personalistika
 */
class PracovniPomucka
{
    public $kod;
    public $popis;

    public $typPomucky;
    public $typPomuckyPopis;

    public $inventarniCislo;
    public $vyrobniCislo;
    public $znacka;
    public $dodavatel;

    public $ucetniDoklad;
    public $porizovaciCena;
    /** @var int */
    public $konecZaruky;

    public $uzivatelskePoleNazev = [];
    public $uzivatelskePoleObsah = [];

    /** @var VelikostPracovniPomucky[] */
    public $velikosti;


    /**
     * @param Database $database
     * @param $record
     * @return PracovniPomucka
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    public static function nactiPomucku($database, $record)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT InventarniCislo, VyrobniCislo, Znacka, Dodavatel, UcetniDoklad, PorizovaciCena, KonecZaruky,' .  //0 - 6
            ' NazevUzivatelskehoPole1, NazevUzivatelskehoPole2, NazevUzivatelskehoPole3, NazevUzivatelskehoPole4, NazevUzivatelskehoPole5,' .    //7 - 11
            ' UzivatelskePole1, UzivatelskePole2, UzivatelskePole3, UzivatelskePole4, UzivatelskePole5,' .   //12 - 16
            ' PracovniPomucka.Kod, PracovniPomucka.TypPomucky, PracovniPomucka.Popis, TypPomucky.Popis' .   //17 - 20
            ' FROM PracovniPomucka LEFT OUTER JOIN  TypPomucky ON PracovniPomucka.TypPomucky = TypPomucky.Kod' .
            ' WHERE PracovniPomucka.Kod = ' . $params->addParameter($record);

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql,$params);

        if(!$row = $cursor->fetch()){
            throw new \Exception('pomucka_neexistuje');
        }

        $pomucka = new PracovniPomucka();
        $pomucka->nactiPomuckuZRadku($database, $row);
        return $pomucka;
    }

    /**
     * @param Database $database
     * @param $row
     */
    public function nactiPomuckuZRadku($database, $row)
    {
        $this->inventarniCislo = $row[0];
        $this->vyrobniCislo = $row[1];
        $this->znacka = $row[2];
        $this->dodavatel = $row[3];
        $this->ucetniDoklad = $row[4];
        $this->porizovaciCena = $row[5];
        $this->konecZaruky = $database->stringToTimestamp($row[6]);

        for($i = 1; $i <= 5; $i++){
            $this->uzivatelskePoleNazev[$i] = $row[$i + 6];
        }
        for($i = 1; $i <= 5; $i++){
            $this->uzivatelskePoleObsah[$i] = $row[$i + 11];
        }
        $this->kod = $row[17];
        $this->typPomucky = $row[18];
        $this->popis = $row[19];
        $this->typPomuckyPopis = $row[20];
    }


    /**
     * @param Database $database
     */
    public function nactiVelikosti($database)
    {
        $this ->velikosti = [];

        $params = $database ->createParameters();
        $cursor = $database ->getSqlResult(
            'SELECT VelikostPPomucky.TypVelikosti, VelikostPPomucky.Velikost' .
            ' FROM VelikostPPomucky, TypPomucky' .
            ' WHERE VelikostPPomucky.TypVelikosti = TypPomucky.TypVelikosti' .
            ' AND TypPomucky.Kod = ' . $params ->addParameter($this ->typPomucky),
            $params
        );

        while ($row = $cursor ->fetch()) {
            $this ->velikosti[] = new VelikostPracovniPomucky($row[0], $row[1]);
        }
    }


    public function getJson()
    {
        $result = [
            'kod' => StringUtils::jsonText($this ->kod),
            'popis' => StringUtils::jsonText($this ->popis),
            'typ' => StringUtils::jsonText($this ->typPomuckyPopis),
            'kodTypu' =>StringUtils::jsonText($this ->typPomucky),
            'inventarniCislo' => StringUtils::jsonText($this ->inventarniCislo),
            'vyrobniCislo' => StringUtils::jsonText($this ->vyrobniCislo),
            'znacka' => StringUtils::jsonText($this ->znacka),
            'konecZaruky' => StringUtils::format('date', $this ->konecZaruky)
        ];

        for($i = 1; $i <= 5; $i++) {
            if (isset($this->uzivatelskePoleNazev[$i])  &&  $this->uzivatelskePoleNazev[$i]) {
                $result["nadpisUzivatelske$i"] = StringUtils::jsonText($this->uzivatelskePoleNazev[$i]);
                $result["uzivatelske$i"] = StringUtils::jsonText($this->uzivatelskePoleObsah[$i]);
            }
        }

        if (isset($this ->velikosti)) {
            $typVelikosti = null;
            $velikostiJson = [];
            foreach ($this ->velikosti as $velikost) {
                $typVelikosti = $velikost ->typVelikosti;
                $velikostiJson[] = StringUtils::jsonText($velikost ->velikost);
            }

            $result['typVelikosti'] = StringUtils::jsonText($typVelikosti);
            $result['velikosti'] = $velikostiJson;
        }

        return $result;
    }


    /**
     * @param Database  $database
     * @param string    $kodPomucky
     * @param int       $idPP
     * @param string    $rc
     * @return string
     */
    public static function nactiDoporucenouVelikostPomucky($database, $kodPomucky, $idPP, $rc = null)
    {
        $params = $database ->createParameters();
        if ($idPP && $kodPomucky) {
            $sql =
                'SELECT VelikostPPOsoby.Velikost' .
                ' FROM VelikostPPOsoby, PracovniPomer, TypPomucky, PracovniPomucka' .
                ' WHERE PracovniPomer.Id = ' . $params ->addParameter($idPP) .
                ' AND PracovniPomer.RC = VelikostPPOsoby.RCOsoby' .
                ' AND TypPomucky.Kod = PracovniPomucka.TypPomucky' .
                ' AND TypPomucky.TypVelikosti = VelikostPPOsoby.TypVelikosti' .
                ' AND PracovniPomucka.Kod = ' . $params ->addParameter($kodPomucky);
        } else if ($rc && $kodPomucky) {
            $sql =
                'SELECT VelikostPPOsoby.Velikost' .
                ' FROM VelikostPPOsoby, TypPomucky, PracovniPomucka' .
                ' WHERE VelikostPPOsoby.RCOsoby = ' . $params ->addParameter($rc) .
                ' AND TypPomucky.Kod = PracovniPomucka.TypPomucky' .
                ' AND TypPomucky.TypVelikosti = VelikostPPOsoby.TypVelikosti' .
                ' AND PracovniPomucka.Kod = ' . $params ->addParameter($kodPomucky);
        } else {
            $sql = null;
        }

        if (!$sql) {
            return null;
        }

        $value = DBUtils::getSqlValue($database, $sql, $params);

        return ($value === false) ? null : $value;
    }
}


