<?php
/**
 * Roman.Szablatura, 18.2.2016
 */

namespace RonPortal\Pomucky;

use Exception;
use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\System\ReportOutputObject;
use RonPortal\User;


/**
 * Class PracovniPomuckyController
 * @package Ron\Personalistika
 */
class PracovniPomuckyController extends JsonController
{
    /** @var Database */
    private $database;

    /**
     * PracovniPomuckyController constructor.
     * @param Database $database
     * @throws Exception
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }

//    /**
//     * @param $record
//     * @return BaseViewObject BaseViewObject
//     * @throws Exception
//     */
//    private function pracovniPomuckyDetailView($record)
//    {
//        $pracovnipomucka = PracovniPomucka::nactiPomucku($this->database,$record);
//
//        Model::addData('pracovnipomucka', $pracovnipomucka);
//
//        return new BaseViewObject('person/view_pracovnipomuckydetail.php');
//    }

    /**
     * @return Redirect|VydejPracovniPomuckyForm
     * @throws Exception
     */
    private function pracovniPomuckyForm()
    {
        Prava::nastavObdobiProPrava(time());

        if (!Prava::jsouNejakePrava('pomucky_vydej')) {
            throw new NeniPravoException();
        }

        $pracPomer = null;

        $osoba = $_POST['osoba'] ?? ($_GET['osoba'] ?? '');

        if ($osoba != '') {
            $pracPomer = PracovniPomer::nactiPP($this->database, (int)$osoba);
        }

        if ($pracPomer && !Prava::vratPrava('pomucky_vydej') ->lzeZobrazitRC($pracPomer ->rc)) {
            throw new NeniPravoException();
        }

        $f = new VydejPracovniPomuckyForm($this->database, $pracPomer);

        if ($f->handleAction()){
            return new Redirect(null, '?menuid=pracovnipomucky');
        }

        return $f;
    }


    /**
     * Hromadny vydej pomucek podle naroku
     * @return Redirect|VydejNarokuForm
     */
    private function hromadnyVydej()
    {
        $f = new VydejNarokuForm($this ->database);

        if($f->handleAction()){
            return new Redirect(null, '?menuid=pracovnipomucky');
        }

        return $f;
    }

    /**
     * @return Redirect|EditaceExpiracePomuckyForm
     * @throws Exception
     */
    private function editaceExpirace()
    {
        /** @var int $osoba Id pracovniho pomeru*/
        $osoba = isset($_POST['osoba'])
            ? (int)$_POST['osoba']
            : (isset($_GET['osoba']) ? (int)$_GET['osoba'] : null);

        if (!$osoba) {
            throw new Exception('Neni zadana osoba!');
        }

        $pPomer = PracovniPomer::nactiPP($this->database, (int)$osoba);

        $f = new EditaceExpiracePomuckyForm($this->database, $pPomer);

        if($f->handleAction()){
            return new Redirect(null, '?menuid=pracovnipomucky');
        }

        return $f;
    }

    /**
     * @param $osoba
     * @param $pomucka
     * @param $datumnaroku
     * @param $poradi
     * @return Redirect
     * @throws Exception
     */
    private function potvrzeniPrevzeti($osoba, $pomucka, $datumnaroku, $poradi)
    {
        $pPomer = PracovniPomer::nactiPP($this ->database, (int)$osoba);

        if (!Prava::vratPrava('pomucky_potvrzeni')->lzeZobrazitRC($pPomer ->rc)) {
            throw new NeniPravoException();
        }

        $recPotvrzeniPrevzeti = new RonPortalDBRecord($this->database, 'PrirazenaPracPomucka', $this);

        $recPotvrzeniPrevzeti->addStringField('RC', true);
        $recPotvrzeniPrevzeti->addStringField('PracovniPomucka', true);
        $recPotvrzeniPrevzeti->addDateField('DatumNaroku', true);
        $recPotvrzeniPrevzeti->addIntField('Poradi', true);

        $recPotvrzeniPrevzeti->addField('PotvrzeniPrevzetiDatum', 'timestamp', false, 0,time());
        $recPotvrzeniPrevzeti->addField('PotvrzeniPrevzetiPopis', 'string', false, 100, User::getLoginName() . ' (' . User::getUserName() . ')');
        $recPotvrzeniPrevzeti->addField('Update_UserName', 'string', false, 25, User::getLoginName()) ->setUpdateLog(true);
        $recPotvrzeniPrevzeti->addField('Update_TimeStamp', 'timestamp', false, 0,time()) ->setUpdateLog(true);

        $params = $this->database->createParameters();
        $recPotvrzeniPrevzeti->setWhere(
            'RC = ' . $params->addParameter($pPomer ->rc) .
            ' AND PracovniPomucka = ' . $params->addParameter($pomucka) .
            ' AND DatumNaroku = ' . $params->addParameter($this->database->timestampToParam($datumnaroku))  .
            ' AND Poradi = ' . $params->addParameter($poradi), $params) ;

        $recPotvrzeniPrevzeti->loadAndUpdateRecord();

        return new Redirect(null,'?menuid=pracovnipomucky');
    }

    /**
     * @return ReportOutputObject
     * @throws Exception
     */
    private function report()
    {
        $vsechnyosoby = (int)($_GET['vsechnyosoby'] ?? 0) == 1;
        $datumod = $_GET['datumod'];
        $datumdo = $_GET['datumdo'];

        $rozdeleniPodrizene = ($_GET['rozdelenipodrizene'] ?? 0) == 1;
        $rozdeleni2Podrizene = ($_GET['rozdelenipodrizene2'] ?? 0) == 1;
        $jenTyp = ($_GET['jentyp'] ?? 0) == 1;
        $jenPomucka = ($_GET['jenpomucka'] ?? 0) == 1;
        $nevydane = ($_GET['nevydane'] ?? 0) == 1;
        $nevracene = ($_GET['nevracene'] ?? 0) == 1;
        $vracene = ($_GET['vracene'] ?? 0) == 1;
        $potvrzene = ($_GET['potvrzene'] ?? 0) == 1;
        $expirovane = ($_GET['expirovane'] ?? 0) == 1;
        $nepotvrzene = ($_GET['nepotvrzene'] ?? 0) == 1;

        $pp = null;
        if ($vsechnyosoby) {
            if (!Prava::jsouNejakePrava('pomucky_prohl')) {
                throw new NeniPravoException();
            }
        } else {
            $pp = PracovniPomer::nactiPP($this->database, (int)$_GET['osoba']);

            if ((!Prava::vratPrava('pomucky_prohl') ->lzeZobrazitRC($pp ->rc))) {
                throw new NeniPravoException();
            }
        }

        $result = new ReportOutputObject();
        $result ->loadReportFromGet($this ->database, 'PracovniPomucky');

        if (!$vsechnyosoby) {
            $result->nastavPP($pp);
        }

        $result ->integerParams = [
            'TypRozdeleni' => RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), // jen kvuli kompatibilite starych sestav
            'RozdeleniPodrizene' => ($rozdeleniPodrizene ? 1 : 0),      // jen kvuli kompatibilite starych sestav
            'TypRozdeleni2' => RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), // jen kvuli kompatibilite starych sestav
            'RozdeleniPodrizene2' => ($rozdeleni2Podrizene ? 1 : 0),    // jen kvuli kompatibilite starych sestav
            'JenTyp' => ($jenTyp ? 1 : 0),
            'JenPomucka' => ($jenPomucka ? 1 : 0),
            'Nevydane' => ($nevydane ? 1 : 0),
            'Nevracene' => ($nevracene ? 1 : 0),
            'Vracene' => ($vracene ? 1 : 0),
            'Nepotvrzene' => ($nepotvrzene ? 1 : 0),
            'Potvrzene' => ($potvrzene ? 1 : 0),
            'Expirovane' => ($expirovane ? 1 : 0),
            'VsechnyOsoby' => ($vsechnyosoby ? 1 : 0)  // jen kvuli kompatibilite starych sestav
        ];
        $result ->stringParams = [
            'Rozdeleni' => $_GET['rozdeleni'],   // jen kvuli kompatibilite starych sestav
            'Rozdeleni2' => $_GET['rozdeleni2'], // jen kvuli kompatibilite starych sestav
            'TypPomucky' => $_GET['typpomucky'],
            'Pomucka' => $_GET['pomucka']
        ];
        $result ->dateParams = [
            'pocatekZvolenehoObdobi' => $datumod,
            'konecZvolenehoObdobi' => $datumdo
        ];

        if (isset($_GET['rozdeleni'])  &&  $_GET['rozdeleni'] != '') {
            $rozdeleni = Rozdeleni::loadRozdeleni($this ->database,
                RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP),
                $_GET['rozdeleni']);
            $result ->nastavRozdeleni(true, $rozdeleni, $rozdeleniPodrizene);
        }

        if (isset($_GET['rozdeleni2'])  &&  $_GET['rozdeleni2'] != '') {
            $rozdeleni = Rozdeleni::loadRozdeleni($this ->database,
                RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP),
                $_GET['rozdeleni2']);
            $result ->nastavRozdeleni2(true, $rozdeleni, $rozdeleni2Podrizene);
        }

        return $result;
    }


    /**
     * Nacte data pro zobrazeni detailu pracovni pomucky se zadanym kodem.
     * @param JsonOutputObject $output
     * @param string $kod
     * @param int $idPP
     * @throws Exception
     */
    function loadDetailData($output, $kod, $idPP)
    {
        $pracovniPomucka = PracovniPomucka::nactiPomucku($this->database, $kod);
        $pracovniPomucka ->nactiVelikosti($this ->database);

        $output ->addData('detail', $pracovniPomucka ->getJson());

        if ($idPP) {
            $doporucenaVelikost = PracovniPomucka::nactiDoporucenouVelikostPomucky($this->database, $pracovniPomucka->kod, $idPP);
            $output ->addData('doporucenaVelikost', $doporucenaVelikost);
        }
    }


    function handleJsonAction($action, $output)
    {
        switch($_GET['action']) {
            case 'detaildata' :
                $this ->loadDetailData($output, StringUtils::fromUTF8($_GET['kod']), isset($_GET['idpp']) ? (int)$_GET['idpp'] : null);
                break;
        }
    }


    /**
     * @param string $action
     * @return BaseViewObject|OutputObject|Redirect|ViewObject|string
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_POMUCKY)) {
            throw new Exception('Agenda pomucek neni povolena!');
        }

        if (!Prava::jsouNejakePrava([
            Prava::vratPrava('pomucky_prohl'),
            Prava::vratPrava('pomucky_vydej'),
            Prava::vratPrava('pomucky_potvrzeni')])
        ) {
            throw new NeniPravoException();
        }

        TextManager::getInstance() ->loadStrings('person.ini');

        Prava::nastavObdobiProPrava(time(),time());

        if (isset($_GET['action'])) {
            switch ($_GET['action']) {
                case 'detaildata' :
                    return parent::handleAction($action);
            }
        }

        HtmlHeaders::getInstance()
            ->addCss('./css/person.css')
            ->addScript('script/personscripts.js')
            ->setAppTitle('pracovni_pomucky');

        switch(isset($_GET['action']) ? $_GET['action'] : ''){
            case 'pomuckyform':
                return $this->pracovniPomuckyForm();

            case 'hromadnyvydej':
                return $this->hromadnyVydej();

            case 'potvrzeniprevzeti':
                return $this->potvrzeniPrevzeti($_GET['osoba'],$_GET['pomucka'],$_GET['datumnaroku'],$_GET['poradi']);

            case 'editaceexpirace':
                return $this->editaceExpirace();

//            case 'detail':
//                return $this ->pracovniPomuckyDetailView($_GET['record']);

            case 'report':
                return $this->report();

            default:
                //return new TextViewObject('Pomucky');
                $b = new PracovniPomuckyBrowser($this ->database);
                $b ->load();
                return $b;
        }
    }
}
