<?php
/**
 * Roman.Szablatura 24.2.2016
 */

namespace RonPortal\Pomucky;

use Ron\Controls\ComboBox;
use Ron\Controls\FormField;
use Ron\Controls\FormRowArray;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecord;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\OsobaFormField;
use RonPortal\Controls\RonPortalForm;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalUtils;


/**
 * Class OsobaPracovnichPomucekField
 */
class OsobaPracovnichPomucekField extends OsobaFormField
{
    /**
     * @param string $prava
     * @param int $datumPlatnostiPP
     * @param PracovniPomer $pp
     */
    public function __construct($prava, $datumPlatnostiPP, $pp)
    {
        parent::__construct($prava, $datumPlatnostiPP, false);

        $this->pracovniPomer = $pp;
    }

    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($insert) {
            parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
        }
        elseif ($this->pracovniPomer) {
            echo '<input type="hidden" name="' . $this->fieldName . '" value="' . $this->pracovniPomer->idPP . '" />';
            //echo StringUtils::htmlTextConvert($this->pracovniPomer->jmenoPrijmeniKratke) . "\n";
            echo $this->pracovniPomer->jmenoPrijmeniKratke;
        }
    }
}



/**
 * Class VybratPracPomuckuField
 */
class VybratPracPomuckuField extends FormField
{
    /**
     * VybratPracPomuckuField constructor.
     */
    function __construct()
    {
        parent::__construct(FormField::FIELD_TYPE_SELECT);
        $this ->setFieldCssClass('form-control ronselect');
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($canEdit) {
            echo '<div class="row">';
            echo '<div class="col col-9">';
            $sql =
                //"SELECT PracovniPomucka.Kod,  +'" . \Ron\ronText('Typ') . ": ' + PracovniPomucka.TypPomucky + ' - " . \Ron\ronText('Popis') . ": ' + PracovniPomucka.Popis " .  //0 - 2
                "SELECT PracovniPomucka.Kod, PracovniPomucka.Popis {fn_concat()} ' " . \Ron\ronText('Typ') . ": ' {fn_concat()} PracovniPomucka.TypPomucky" .   //0 - 2
                ' FROM PracovniPomucka, TypPomucky ' .
                ' WHERE (PracovniPomucka.TypPomucky = TypPomucky.Kod) ' .
                ' AND ' .
                '  (PracovniPomucka.Stav=1 OR PracovniPomucka.Stav=2' .
                "   OR (PracovniPomucka.Stav=3 AND PracovniPomucka.VyrazenoDoData IS NOT NULL AND PracovniPomucka.VyrazenoDoData < " . $database->dateToString(time()) . ')' .
                ' )' .
                " AND ((TypPomucky.Sledovany=0) OR ((TypPomucky.Sledovany=1) " .
                " AND NOT exists(select * from PrirazenaPracPomucka where PracovniPomucka=PracovniPomucka.Kod and ((DatumVraceni IS NULL) OR (DatumVraceni > " . $database->dateToString(time()) .
                ')))))';

            $combo = new ComboBox($this ->fieldName);
            $combo
                ->setEmptyRow('&nbsp;')
                ->setShowCode(true)
                ->setHtmlParams(['data-ron-select2' => $this ->fieldName . '_kod', 'id' => 'kod_pracovni_pomucky', 'data-columns' => 'text;value'])
                ->setClassName($this ->fieldCssClass);

            //$combo->alwaysShowSelected = isset($value) && $value != '';
            $combo->loadData($database, $sql . ' ORDER BY PracovniPomucka.Popis', $value);
            $combo->render();
            echo '</div>';

            echo '<div class="col col-3">';
            $combo = new ComboBox($this ->fieldName . '_kod');

            //$combo->alwaysShowSelected = isset($value) && $value != '';
            $combo
                ->setEmptyRow('&nbsp;')
                ->setKeyIndex(0) ->setCaptionIndex(0)
                ->setAttrIndexes(['data-popis' => 1])
                ->setHtmlParams(['data-ron-select2' => $this ->fieldName, 'data-columns' => 'text;popis', 'data-drop-down-align' => 'right'])
                ->setClassName($this ->fieldCssClass);

            $combo->loadData($database, $sql . ' ORDER BY PracovniPomucka.Kod', $value);
            $combo->render();

            echo '</div>'; // col
            echo '</div>'; // row
        } else {
            $params = $database ->createParameters();
            $popisPomucky = DBUtils::getSqlValue($database,
                'SELECT PracovniPomucka.Popis' .
                ' FROM PracovniPomucka' .
                ' WHERE PracovniPomucka.Kod = ' . $params ->addParameter($value),
                $params);

            echo
                StringUtils::htmlTextConvert($popisPomucky) .
                '<input type="hidden" name="' . $this->fieldName . '" value="' . StringUtils::htmlTextConvert($value) . '"/>';
        }
    }
}


class DetailPracovniPomuckyField extends FormField
{
    /** @var Database */
    private $database;

    /** @var PracovniPomucka */
    public $pracovniPomucka;

    /**
     * DetailPracovniPomuckyField constructor.
     * @param $database
     */
    public function __construct($database)
    {
        parent::__construct(FormField::FIELD_TYPE_TEXT);
        $this->database = $database;
    }


    public function loadDatabaseValue(Database $database, ?FormRowArray $rowData)
    {
        parent::loadDatabaseValue($database, $rowData);

        $this ->pracovniPomucka = ($rowData[4] ?? null)
            ? PracovniPomucka::nactiPomucku($this ->database, $rowData[4])
            : null;
    }


    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        $value = $_POST['PracovniPomucka'] ?? ($rowData[4] ?? null);

        if ($value  &&  (!$this ->pracovniPomucka  ||  $this ->pracovniPomucka ->kod !== $value)) {
            $this ->pracovniPomucka = PracovniPomucka::nactiPomucku($this->database, $value);
        }

        // pouzito ve view
        $pracovniPomucka = $this ->pracovniPomucka;

        require 'ronportal/pomucky/view_detail_pomucky.php';
    }
}



/**
 * Class DoporucenaVelikostPomuckyField
 */
class DoporucenaVelikostPomuckyField extends FormField
{
    function __construct()
    {
        parent::__construct(FormField::FIELD_TYPE_TEXT);
        $this->caption = \Ron\ronText('doporucena_velikost');
    }


    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        //parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
        $idpp = null;
        $rc = null;

        if (isset($_POST['osoba'])) {
            $idpp = $_POST['osoba'] != '' ? (int)$_POST['osoba'] : null;
        } else if ($rowData) {
            $rc = $rowData[7];
        } else {
            $idpp = RonPortalSession::vybranaOsoba();
        }

        $pomucka = $_POST['PracovniPomucka'] ?? ($rowData ? $rowData[4] : null);

        $doporucenaVelikost = PracovniPomucka::nactiDoporucenouVelikostPomucky($database, $pomucka, $idpp, $rc);

        echo '<span id="doporucenaVelikost">' . StringUtils::htmlTextConvert($doporucenaVelikost) . '</span>';
    }
}

/**
 * Class VydanaVelikostPomuckyField
 */
class VydanaVelikostPomuckyField extends FormField
{
    function __construct()
    {
        parent::__construct(FormField::FIELD_TYPE_SELECT);
        $this->showEmptyValue = true;
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if (isset($_POST['PracovniPomucka'])) {
            $pomucka = $_POST['PracovniPomucka'];
        } else if (isset($rowData[4])) {
            $pomucka = $rowData[4];
        } else {
            $pomucka = '';
        }

        $params = $database ->createParameters();
        $this ->setSelectSql(new SqlStatement(
            'SELECT VelikostPPomucky.Velikost, VelikostPPomucky.Velikost' .
            ' FROM VelikostPPomucky, PracovniPomucka, TypPomucky' .
            ' WHERE VelikostPPomucky.TypVelikosti = TypPomucky.TypVelikosti' .
            ' AND PracovniPomucka.TypPomucky = TypPomucky.Kod' .
            ' AND PracovniPomucka.Kod = ' . $params ->addParameter($pomucka),
            $params
        ));


        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);

//        $sql =
//            "SELECT VelikostPPomucky.TypVelikosti+'~'+VelikostPPomucky.Velikost, VelikostPPomucky.Velikost " .
//            'FROM PracovniPomucka LEFT JOIN TypPomucky ON (PracovniPomucka.TypPomucky = TypPomucky.Kod) LEFT JOIN ' .
//            'TypVelikosti ON (TypPomucky.TypVelikosti = TypVelikosti.Kod) LEFT JOIN VelikostPPomucky ON (TypVelikosti.Kod = VelikostPPomucky.TypVelikosti) ' .
//            "WHERE PracovniPomucka.Kod = '" . (isset($_POST['pomucka']) ? $_POST['pomucka'] : $_GET['pomucka']) . "'";
//
//        $cursor = $database->getSqlResult($sql);
//        $row = $cursor->fetch();
//
//        if($row[0] != '') {
//            $combo = new ComboBox('velikost');
//            $combo->alwaysShowSelected = isset($value) && $value != '';
//            $combo->className .= 'app_combo adsfilterforminput_kod';
//            $combo->loadData($database, $sql, isset($_POST['doporucenaVelikostPomucky']) ? $_POST['doporucenaVelikostPomucky'] : 0);
////        $combo->showCode = 1;
//            $combo->render();
//        }
    }
}


/**
 * Class VydejPracovniPomuckyForm
 */
class VydejPracovniPomuckyForm extends RonPortalForm
{
    const TYP_POHYBU_VRACENI = 0;
    const TYP_POHYBU_VYDANI_PRIRAZENE = 1;
    const TYP_POHYBU_VYDANI_NOVE = 2;

    /** @var PracovniPomer */
    protected $pracovniPomer;

    /** @var string     Editovany zaznam - kod pomucky */
    protected $pomucka = null;
    /** @var int        Editovany zaznam - datum naroku */
    protected $datumNaroku = null;
    /** @var int        Editovany zaznam - poradi */
    protected $poradi = null;

    protected $typPohybu;

    /** @var DetailPracovniPomuckyField */
    protected $detailField;

    /**
     * EditaceDatumuVraceniPomuckyForm constructor.
     * @param Database $database
     * @param PracovniPomer $pp
     * @throws DBException
     * @throws \Exception
     */
    function __construct($database, $pp)
    {
        parent::__construct($database);

        $this ->pracovniPomer = $pp;

        $this->code = 'formvydejpomucky';

        $this->setXmlEventsName('PRACOVNI_POMUCKY');

        $this ->pomucka = $_POST['PracovniPomucka'] ?? ($_GET['pomucka'] ?? null);

        $this ->datumNaroku = $_POST['datumnaroku'] ?? ($_GET['datumnaroku'] ?? null);

        $this ->poradi = $_POST['poradi'] ?? ($_GET['poradi'] ?? null);

        $this->htmlAction = '?menuid=pracovnipomucky&action=pomuckyform';
        $this->stornoAction = '?menuid=pracovnipomucky';

        $this->cssClass .= ' pracovnipomuckyform';

        $this ->typPohybu = (int)($_POST['vydanipomucky'] ?? $_GET['vydanipomucky'] ?? 0);

        $individualni = (int)($_POST['individualni'] ?? ($_GET['individualni'] ?? 0));

        switch ($this ->typPohybu) {
            case self::TYP_POHYBU_VYDANI_NOVE :
                $this ->title = \Ron\ronText('vydej_pomucky_zamestnanci');
                break;

            case self::TYP_POHYBU_VRACENI :
                $this->title = \Ron\ronText('formular_vraceni_pomucky');
                break;

            default :
                $this->title = \Ron\ronText('formular_vydani_pomucky');
        }

        $this->sqlTableName = 'PrirazenaPracPomucka';
        $this->sqlSelectPart =
            'PrirazenaPracPomucka.DatumVydani, PrirazenaPracPomucka.DatumVraceni, ' .   // 0 - 1
             $database->castWideMemo('PrirazenaPracPomucka.Poznamka') . ' as Poznamka, PrirazenaPracPomucka.DatumExpirace, '. // 2 - 3
            'PrirazenaPracPomucka.PracovniPomucka, Individualni, OpakovanyNarok, ' .    //4 - 6
            'PrirazenaPracPomucka.RC, PrirazenaPracPomucka.NarokNerealizovat'; // 7 - 8     , PrirazenaPracPomucka.NarokNerealizovatDo - doplnit po vydani verze?

//        if (isset($_GET['osoba'])) {
//            $pracovniPomer = new PracovniPomer();
//            $pracovniPomer->nactiPPPodleId($database,$_GET['osoba']);
//            $pracovniPomer->vyberNactenyPP($database,Prava::vratPrava('pomucky_vydej'));
//        }

        $osobaField = new OsobaPracovnichPomucekField('pomucky_vydej', DateUtils::now(), $pp);
        $osobaField
            ->setCaption('Osoba')
            ->setOtherHtmlParams([
                'id' => 'osoba',
                'class' => 'form-input'
            ])
            ->setCode('field-osoba');
        $this->addField($osobaField);

        $this->addField(new VybratPracPomuckuField())
            ->setFieldName('PracovniPomucka') ->setSqlFieldIndex(4)
            ->setCaption('pracovni_pomucka')
            ->setReadOnly(true)
            ->setCode('field-pomucka');

        $this ->detailField = new DetailPracovniPomuckyField($this ->database);
        $this->addField($this ->detailField)
            ->setCode('field-detail');

        $this->addField(new DoporucenaVelikostPomuckyField())->setCode('field-doporucena-velikost');
        $this->addField(new VydanaVelikostPomuckyField())
            ->setFieldName('VelikostPP') ->setCaption('vydana_velikost')
            ->setFieldId('VelikostPP')->setCode('field-vydana-velikost');

        //Zobrazeni poli s daty a memo
        $this->addField(FormField::FIELD_TYPE_TEXT);

        if($individualni) {
            $this->addField(FormField::FIELD_TYPE_CHECK)
                ->setSqlFieldIndex(6) ->setFieldName('OpakovanyNarok')
                ->setCaption('opakovany')
                ->setCode('field-opakovany');
        }

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumVydani') ->setSqlFieldIndex(0) ->setCaption('datum_vydani')
            ->setCode('field-datum-vydani')
            ->setReadOnly(!$this ->typPohybu) ->setUseDefaultValueToEditNull((bool)$this ->typPohybu)
            ->setDefaultValue(StringUtils::format('date', DateUtils::today()));

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumVraceni') ->setSqlFieldIndex(1) ->setCaption('datum_vraceni')
            ->setCode('field-datum-vraceni')
            ->setReadOnly((bool)$this ->typPohybu) ->setUseDefaultValueToEditNull($this ->typPohybu == 0)
            ->setDefaultValue($this ->typPohybu == 0 ? StringUtils::format('date', DateUtils::today()) : null);

        $this->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka') ->setSqlFieldIndex(2) ->setCaption('Poznamka')
            ->setCode('field-poznamka');

        $this ->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('NarokNerealizovat') ->setSqlFieldIndex(8)
            ->setCode('field-naroknerealizovat')
            ->setReadOnly(true) ->setDefaultValue(0)
            ->setDataType("int");
    }


    /**
     * @return string
     * @throws DBException
     */
    function getWhereCondition()
    {
        if ($this ->pracovniPomer  &&  $this ->pomucka && $this ->datumNaroku && $this ->poradi) {
            $params = $this ->database ->createParameters();
            return new SqlStatement(
                'RC = ' . $params ->addParameter($this ->pracovniPomer ->rc) .
                ' AND PracovniPomucka = ' . $params ->addParameter($this ->pomucka) .
                ' AND DatumNaroku = ' .  $params ->addTimestampParameter($this ->datumNaroku) .
                ' AND Poradi = ' .  $params ->addParameter((int)$this ->poradi),
                $params);
        }

        return '';
    }

    public function getCsrfRequestId()
    {
        return
            $this ->code . '_' .
            ($this->isInsert()
                ? ''
                : $this ->pracovniPomer ->rc . '_' . $this ->pomucka
            );
    }


    public static function zkontrolujPravoNaPomucku($database, $typPomucky, $kodPomucky = null)
    {
        $pravo = Prava::vratPrava('pomucky_vydej');
        if (!$pravo ->priznak3) {
            // ok, pravo neni omezeno
            return;
        }

        if ($typPomucky) {
            if (!$pravo ->priznak3ObsahujeKod($typPomucky)) {
                throw new NeniPravoException();
            }
            return;
        }

        $pomucka = PracovniPomucka::nactiPomucku($database, $kodPomucky);
        if (!$pravo ->priznak3ObsahujeKod($pomucka ->typPomucky)) {
            throw new NeniPravoException();
        }
    }


    public function loadData($usePostValues)
    {
        parent::loadData($usePostValues);

        if ($this ->data['field-naroknerealizovat']) {
            $this ->readOnly = true;
        }

        if ($this ->detailField ->pracovniPomucka) {
            self::zkontrolujPravoNaPomucku($this ->database, $this ->detailField ->pracovniPomucka ->typPomucky);
        }
    }


    public function renderCommonFields()
    {
        parent::renderCommonFields();

        $individualni = (int)($_POST['individualni'] ?? $_GET['individualni'] ?? 0);

        echo
            '<input type="hidden" name="datumnaroku" value="' . ($this ->datumNaroku ?: '') . '" />' .
            '<input type="hidden" name="poradi" value="' . ($this ->poradi ?: '') . '" />' .
            '<input type="hidden" name="vydanipomucky" value="' . $this ->typPohybu . '" />' .
            '<input type="hidden" name="individualni" value="' . $individualni . '" />' .
            "\n";
    }


    function createDBRecordFields($rec, $insert)
    {
        if (isset($_POST['VelikostPP'])  &&  $_POST['VelikostPP'] != '') {
            // jeste nacte typ velikosti k zadane pomucce
            $params = $this ->database ->createParameters();
            $typVelikostiPomucky = DBUtils::getSqlValue($this ->database,
                'SELECT TypPomucky.TypVelikosti' .
                ' FROM PracovniPomucka, TypPomucky' .
                ' WHERE PracovniPomucka.Kod = ' . $params ->addParameter($_POST['PracovniPomucka']) .
                ' AND TypPomucky.Kod = PracovniPomucka.TypPomucky',
                $params);
        } else {
            $typVelikostiPomucky = null;
        }

        $rec->addStringField('RC', true, 15, $this->pracovniPomer->rc);      // $pracPomer->rc
        $rec->addStringField('OsobniCislo', true, 15, $this->pracovniPomer->osobniCislo);   // $pracPomer->osobniCislo
        $rec->addDateField('DatumNastupu', true, $this ->pracovniPomer->datumNastupu);  // $pracPomer->datumNastupu
        $rec->addLogText(' ' . $this ->pracovniPomer ->jmenoPrijmeni );

        //$result->addStringField('PracovniPomucka', true, 10,  $_POST['PracovniPomucka']);
        $rec->addField('DatumNaroku', 'date', true, 0, time());
        $rec->addField('Poradi', 'int', true, 0);
        $rec->addField('Individualni', 'int', true, 0, (int)$_POST['individualni']);
        $rec->addField('TypVelikostiPP', '', true, 10, $typVelikostiPomucky);
        //$result->addField('VelikostPP', '', true, 20, $velikostPomucky[1]);

        parent::createDBRecordFields($rec, $insert);

        $rec->addStringField('Insert_UserName', true, 25, RonPortalUtils::getLogUserName());
        $rec->addTimestampField('Insert_Timestamp', true, time());

        $rec->addStringField('Update_UserName', false, 25, RonPortalUtils::getLogUserName())
            ->setUpdateLog(true)
            ->setNoInsert(true);
        $rec->addTimestampField('Update_Timestamp', false, time())
            ->setUpdateLog(true)
            ->setNoInsert(true);

        self::zkontrolujPravoNaPomucku($this ->database, null, $rec['PracovniPomucka']);
    }


    /**
     * @param DBRecord $rec
     * @throws DBException
     */
    function setDBRecord($rec)
    {
        parent::setDBRecord($rec);
        $rec->setWhere($this->getWhereCondition());
    }


    /**
     * @param DBRecord $rec
     * @return bool
     * @throws \Exception
     * @throws DBException
     */
    function beforeInsertRecord($rec)
    {
//        if (isset($_POST['osoba'])) $osoba = $_POST['osoba'];
//        else $osoba = '';

        $pp = PracovniPomer::nactiPP($this ->database, (int)$_POST['osoba']);

        // Kontrola pohlavi
        $params = $this ->database ->createParameters();
        $pohlavi = DBUtils::getSqlValue($this->database,
            'SELECT Pohlavi FROM PracovniPomucka' .
            ' WHERE Kod = '. $params ->addParameter($rec['PracovniPomucka']),
            $params);

        if ($pohlavi) {
            // pomucka ma omezeni podle pohlavi => zkontroluje pohlavi osoby
            $params = $this ->database ->createParameters();
            $pohlaviOsoby = DBUtils::getSqlValue($this->database,
                'SELECT Pohlavi FROM OsobniUdaj' .
                ' WHERE RC = ' . $params ->addParameter($pp ->rc),
                $params);

            if ($pohlavi != $pohlaviOsoby) {
                throw new \Exception(TextManager::_('Vydej_pomucky_omezen_podle_pohlavi'));
            }
        }

        $pomucka = $_POST['PracovniPomucka'];

        $zadaneDatumVydani = $rec['DatumVydani'];
        $params = $this ->database ->createParameters();
        $jizVydaneVDatu = $this->database->stringToTimestamp(
            DBUtils::getSqlValue($this->database,
                'SELECT max (DatumVydani)' .
                ' FROM PrirazenaPracPomucka LEFT JOIN PracovniPomucka' .
                ' ON (PrirazenaPracPomucka.PracovniPomucka = PracovniPomucka.Kod)' .
                ' LEFT JOIN TypPomucky ON (PracovniPomucka.TypPomucky = TypPomucky.Kod)' .
                ' WHERE TypPomucky.Sledovany = 1 AND PrirazenaPracPomucka.PracovniPomucka = '. $params ->addParameter($pomucka),
            $params)
        );

        if($zadaneDatumVydani <  $jizVydaneVDatu){
            throw new \Exception(TextManager::_('datum_obsazeno'));
        }

        $datumNaroku = DateUtils::startOfTheDay(time());

        //umozni prirazeni stejne pomucky jedne osobe ve stejny aktualni den vlozenim noveho cisla poradi
        $params = $this ->database ->createParameters();

        $rec['Poradi'] = SQLUtils::newSerialNumber($this->database, 'PrirazenaPracPomucka', 'Poradi',
            'RC = ' . $params ->addParameter($pp ->rc) .
            ' AND PracovniPomucka = ' . $params ->addParameter($pomucka) .
            ' AND DatumNaroku = ' . $params ->addParameter($this->database->dateToParam($datumNaroku)),
            $params);

        return true;
    }

    /**
     * @param DBRecord $rec
     * @return bool
     * @throws \Exception
     */
    function beforeUpdateRecord($rec)
    {
        if($_POST['vydanipomucky'] == 0) {
            if ($rec['DatumVraceni'] == '') {
                throw new \Exception(TextManager::_('form.Missing_value') . ' ' . TextManager::_('datum_vraceni') . '!');
            }

            if($rec['DatumVraceni'] < $rec['DatumVydani']){
                throw new \Exception(TextManager::_('spatne_datum_vraceni'));
            }
        }

        return parent::beforeUpdateRecord($rec);
    }
}
