<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal;

use Ron\DB\Database;
use Ron\StringUtils;
use Ron\TextManager;

class RonPortalUtils
{
    /**
     * Zjisti, jestli je nainstalovana aplikace se zadanym kodem. Pokud ano, vrati timestamp nainstalovane verze SDF.
     * @param Database $database
     * @param string|string[] $kod
     * @return false|int timestamp verze SDF
     */
    public static function jeNainstalovanaAplikace($database, $kod)
    {
        $params = $database ->createParameters();
        $sql = 'SELECT applicationID, VersionTimeStamp FROM SYS_DDEF WHERE applicationID IN (';
        if (is_array($kod)) {
            $first = true;
            foreach ($kod as $kodAplikace) {
                $sql .= (!$first ? ', ' : '') . $params ->addParameter($kodAplikace);
                $first = false;
            }
        } else {
            $sql .= $params->addParameter($kod);
        }
        $sql .= ')';

        $cursor = $database->select($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            return $database->stringToTimestamp($row[1]);
        }

        return false;
    }


    /**
     * Vraci jmeno aktualniho uzivatele pro ulozeni do "logovaciho" pole (Schvalil, Zpracoval, ..., Hodnotici)
     * @param bool|int $orezat  Urcuje, jestli se ma jmeno orezat. Pokud je true, tak se oreze na 25 znaku, pokud je to cislo, tak se oreze na zadany pocet znaku
     * @return string
     */
    public static function getLogUserName($orezat = true)
    {
        if (User::isLoggedAs()) {
            $maxLength = is_int($orezat) ? $orezat - 2 : 23;
            $loginName = User::getRealLoginName();
            $name = User::getRealUserName();
        } else {
            $maxLength = is_int($orezat) ? $orezat : 25;
            $loginName = User::getLoginName();
            $name = User::getUserName();
        }

//        if (User::isLoggedAs()) {
//            $result = substr(User::getRealLoginName(), 0, 23) . '/z';
//        } else {
            switch (RonPortalSettings::getIntValue('logovane_jmeno')) {
                case 1 :
                    // loguje se jen uzivatelske jmeno
                    $result = $loginName;
                    break;

                case 2 :
                    // loguje se jmeno a prijmeni uzivatele
                    $result = $name;
                    break;

                case 0 :
                default :
                    $result = $loginName;

                    if ($orezat) {
                        if (StringUtils::strlen($result) < 12) {
                            // prida jeste cele jmeno v zavorce
                            //$result .= ' (' . substr(User::getUserName(), 0, 25 - strlen($result) - 3) . ')';
                            $result .= ' (' . StringUtils::substr($name, 0, $maxLength - StringUtils::strlen($result) - 3) . ')';
                        }
                    } else {
                        $result .= ' (' . $name . ')';
                    }
            }
//        }

        if ($orezat  &&  StringUtils::strlen($result) > $maxLength) {
            $result = StringUtils::substr($result, 0, $maxLength);
        }

        if (User::isLoggedAs()) {
            $result .= '/z';
        }

        return $result;
    }


    /**
     * @throws \Exception
     * @deprecated
     */
    public static function neniPravoException()
    {
        throw new NeniPravoException();
    }

}
