<?php
/**
 * stanislav.elbl, 30.11.2018
 */

namespace RonPortal;


use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\StringUtils;
use Ron\TextManager;

class RozdeleniOsob
{
    public $idPP;

    public $kodTypuRozdeleni;
    public $kod;

    public $popis;
    public $platnostOd;
    public $platnostDo;
    public $poznamka;

    public $cssClassObrazku;
    /** @var int */
    public $barvaProVykresleni;
    public $title;
    /** @var bool   Priznak, ze neni pokryte cele pozadovane obdobi - pouziva se pri nacitani ikon */
    public $nekompletni = false;


    public function getJson()
    {
        return [
            'idPP' => $this ->idPP,
            'kodTypuRozdeleni' => $this ->kodTypuRozdeleni,
            'kod' => StringUtils::jsonText($this ->kod),
            'popis' => StringUtils::jsonText($this ->popis),
            'barvaProVykresleni' => GuiUtils::getHtmlColor($this ->barvaProVykresleni),
            'platnostOdStr' => StringUtils::jsonDate($this ->platnostOd),
            'platnostDoStr' => $this ->platnostDo ? StringUtils::jsonDate($this ->platnostDo) : null
        ];
    }

    /**
     * @param Database $database
     * @param PracovniPomer|int|int[] $pp Pracovni pomer nebo id pracovniho pomeru
     * @param int|int[]|string $typRozdeleni Kod(y) typu rozdeleni nebo retezec kodu oddelenych strednikem
     * @param int $platnostOd
     * @param int $platnostDo
     * @param string|SqlStatement $filtr
     * @return RozdeleniOsob[]
     * @throws DBException
     */
    public static function nactiRozdeleniOsoby($database, $pp, $typRozdeleni, $platnostOd, $platnostDo = null, $filtr = null)
    {
        if (!$typRozdeleni) {
            return [];
        }

        if (!isset($platnostDo)) {
            $platnostDo = $platnostOd;
        }

        $params = $database ->createParameters();
        $sql =
            'SELECT Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.KodTypuRozdeleni, RozdeleniOsob.PlatnostOd, RozdeleniOsob.PlatnostDo,' .
            ' Rozdeleni.DatumVytvoreni, Rozdeleni.DatumZruseni, Rozdeleni.cssClassObrazku, Rozdeleni.BarvaProVykresleni,' . // 5..8
            ' RozdeleniOsob.Poznamka, PracovniPomer.Id' . // 9,10
            ' FROM RozdeleniOsob, Rozdeleni, PracovniPomer' .
            ' WHERE RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
            '  AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod' .
            '  AND RozdeleniOsob.RC = PracovniPomer.RC' .
            '  AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
            '  AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
            // jen platne v zadanem obdobi
            (isset($platnostOd)
                ? '  AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($platnostDo)) .
                  '  AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($platnostOd)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                  '  AND Rozdeleni.DatumVytvoreni <= ' . $params ->addParameter($database ->dateToParam($platnostDo)) .
                  '  AND (Rozdeleni.DatumZruseni >= ' . $params ->addParameter($database ->dateToParam($platnostOd)) . ' OR Rozdeleni.DatumZruseni IS NULL)'
                : ''
            );

        if ($pp instanceof PracovniPomer) {
            // jen pro zadanou osobu
            $sql .=
                '  AND RozdeleniOsob.RC = ' . $params->addParameter($pp->rc) .
                '  AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
                '  AND RozdeleniOsob.DatumNastupu = ' . $params->addParameter($database->dateToParam($pp->datumNastupu));
        } else if ($pp  &&  is_array($pp)  &&  isset($pp['rc'])) {
            // jen rc
            $sql .=
                '  AND RozdeleniOsob.RC = ' . $params->addParameter($pp['rc']);
        } else if ($pp  &&  is_array($pp)) {
            // neprazdny seznam idpp
            $sql .= ' AND PracovniPomer.Id IN (';
            $first = true;
            foreach ($pp as $idPP) {
                if ($first) {
                    $first = false;
                } else {
                    $sql .= ', ';
                }
                $sql .= $params ->addParameter($idPP);
            }
            $sql .= ')';
        } else {
            $sql .=
                '  AND PracovniPomer.Id = ' . $params ->addParameter((int)$pp);
        }

        if (is_string($typRozdeleni)) {
            if ($typRozdeleni !== '*') {
                // rozebere retezec kodu oddelenych strednikem
                $typRozdeleni = explode(';', $typRozdeleni);
            }
        }

        // jen zadane typy rozdeleni

        if (is_string($typRozdeleni)) {
            // nic - zpracovalo se v predchozim kroku - pokud dosel sem, tak se maji nacitat vsechny typy (zadano '*')
        } else if (is_array($typRozdeleni)) {
            $sql .= ' AND(';

            $first = true;
            foreach($typRozdeleni as $kodTypuRozdeleni) {
                if ($first) {
                    $first = false;
                } else {
                    $sql .= ' OR ';
                }
                $sql .= 'Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter((int)$kodTypuRozdeleni);
            }

            $sql .= ')';
        } else {
            $sql .= ' AND Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni);
        }

        if ($filtr) {
            if ($filtr instanceof SqlStatement) {
                $sql .= ' AND (' . $filtr ->sqlText . ')';
                $params ->appendParameters($filtr ->parameters);
            } else {
                $sql .= ' AND (' . $filtr . ')';
            }
        }

        $sql .= ' ORDER BY Rozdeleni.KodTypuRozdeleni, Rozdeleni.Kod, PlatnostOd';

        $result = [];

        $cursor = $database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $zaznam = new self();
            $zaznam ->kod = $row[0];
            $zaznam ->popis = $row[1];
            $zaznam ->title = $zaznam ->popis;
            $zaznam ->kodTypuRozdeleni = (int)$row[2];

            $zaznam ->platnostOd = $database ->stringToTimestamp($row[3]);
            $vytvoreni = $database ->stringToTimestamp($row[5]);
            if ($vytvoreni > $zaznam ->platnostOd) {
                $zaznam ->platnostOd = $vytvoreni;
            }

            $zaznam ->platnostDo = $database ->stringToTimestamp($row[4]);
            $zruseni = $database ->stringToTimestamp($row[6]);
            if ($vytvoreni > $zaznam ->platnostOd) {
                if (!isset($zaznam ->platnostDo)  ||  (isset($zruseni)  &&  $zruseni < $zaznam ->platnostDo)) {
                    $zaznam->platnostDo = $zruseni;
                }
            }

            $zaznam ->cssClassObrazku = $row[7];

            if ($row[8]) {
                $zaznam ->barvaProVykresleni = $row[8];
            }

            $zaznam ->poznamka = $row[9];

            $zaznam ->idPP = (int)$row[10];

            $result[] = $zaznam;
        }
        $cursor ->close();

        return $result;
    }


    /**
     * @param RozdeleniOsob[] $seznam
     * @param int    $typRozdeleni
     * @param string $kod
     * @return RozdeleniOsob
     */
    public static function najdiZaznamVSeznamu($seznam, $typRozdeleni, $kod)
    {
        if (!$seznam) {
            return null;
        }

        foreach ($seznam as $zaznam) {
            if ($zaznam ->kodTypuRozdeleni === $typRozdeleni  &&  $zaznam ->kod === $kod) {
                return $zaznam;
            }
        }

        return null;
    }

    /**
     * Nacte seznam rozdeleni osoby, ktere maji nastavenou ikonu (pole cssClassObrazku). Zaznamy se stejnym rozdelenim (napr. s ruznou platnosti) slouci do jednoho.
     * @param Database $database
     * @param PracovniPomer|int $pp Pracovni pomer nebo id pracovniho pomeru
     * @param int|int[]|string $typRozdeleni Kod(y) typu rozdeleni nebo retezec kodu oddelenych strednikem
     * @param $datumOd
     * @param $datumDo
     * @return RozdeleniOsob[]
     * @throws DBException
     */
    public static function nactiIkonyRozdeleni($database, $pp, $typRozdeleni, $datumOd, $datumDo)
    {
        $r = self::nactiRozdeleniOsoby($database, $pp, $typRozdeleni, $datumOd, $datumDo, 'Rozdeleni.cssClassObrazku IS NOT NULL');

        /** @var RozdeleniOsob[] $ikony */
        $ikony = [];
        foreach ($r as $rozdeleniOsoby) {
            $ikona = self::najdiZaznamVSeznamu($ikony, $rozdeleniOsoby ->kodTypuRozdeleni, $rozdeleniOsoby ->kod);
            if ($ikona) {
                // TODO: upravit platnost od - do a pripadne title
                if ($ikona ->platnostDo  &&  $ikona ->platnostDo < DateUtils::incDay($rozdeleniOsoby ->platnostOd, -1)) {
                    // mezi zaznamy je mezera
                    $ikona ->nekompletni = 1;
                }

                if (($ikona ->platnostDo  &&  $ikona ->platnostDo < $rozdeleniOsoby ->platnostDo)  ||  !$rozdeleniOsoby ->platnostDo) {
                    // prodlouzi platnost do
                    $ikona ->platnostDo = $rozdeleniOsoby ->platnostDo;
                }

                $ikona->title .= "\n" . TextManager::_('Platnost_od') . ' ' . StringUtils::format('date', $rozdeleniOsoby ->platnostOd);
                if ($rozdeleniOsoby ->platnostDo) {
                    $ikona->title .= ' ' . TextManager::_('Do') . ' ' . StringUtils::format('date', $rozdeleniOsoby ->platnostDo);
                }

            } else {
                $ikona = new self();
                $ikona->cssClassObrazku = $rozdeleniOsoby->cssClassObrazku;
                $ikona->kodTypuRozdeleni = $rozdeleniOsoby->kodTypuRozdeleni;
                $ikona->kod = $rozdeleniOsoby ->kod;
                $ikona->popis = $rozdeleniOsoby->popis;
                $ikona->platnostOd = $rozdeleniOsoby ->platnostOd;
                $ikona->platnostDo = $rozdeleniOsoby ->platnostDo;

                $platnostStr = '';
                if ($rozdeleniOsoby ->platnostOd > $datumOd) {
                    $ikona ->nekompletni = true; // neni pokryty zacatek obdobi
                    $platnostStr .= "\n" . TextManager::_('Platnost_od') . ' ' . StringUtils::format('date', $rozdeleniOsoby ->platnostOd);
                }
                if ($rozdeleniOsoby ->platnostDo) {
                    $platnostStr .= ' ' . ($platnostStr ? TextManager::_('Do') : "\n" . TextManager::_('Platnost_do')) . ' ' . StringUtils::format('date', $rozdeleniOsoby ->platnostDo);
                }

                $ikona->title = $rozdeleniOsoby->popis . $platnostStr;

                $ikony[] = $ikona;
            }
        }

        foreach ($ikony as $ikona) {
            if ($ikona ->platnostDo  &&  $ikona ->platnostDo < $datumDo) {
                $ikona ->nekompletni = true;
            }

            if ($ikona ->nekompletni) {
                $ikona ->cssClassObrazku .= ' partial';
            }
        }


        return $ikony;
    }


    public function getIkonaHtml()
    {
        return
            '<i class="' . $this->cssClassObrazku . '" title="' . StringUtils::htmlTextConvert($this->title) . '"></i>';
    }

        /**
     * @param RozdeleniOsob[] $ikony
     * @param bool $wrap
     * @return string
     */
    public static function getIkonyHtml($ikony, $wrap = true)
    {
        $result = $wrap ? '<span class="ikony_rozdeleni">' : '';

        if ($ikony) {
            foreach ($ikony as $ikona) {
                $result .= $ikona ->getIkonaHtml();
            }
        }

        $result .= $wrap ? '</span>' : '';

        return $result;
    }

}

