<?php
/**
 * Seznam skoleni z tabulky GenerovaneSkoleni
 */

namespace RonPortal\Skoleni;

use Exception;
use Ron\Controls\BrowserField;
use Ron\Controls\BrowserRowArray;
use Ron\Controls\ComboBox;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\GuiUtils;
use Ron\Http\App;
use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use function Ron\ronText;
use Ron\TextManager;
use RonPortal\Controls\OsobniCisloBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\NastaveniUzivatele;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;
use RonPortal\User;


/**
 * Class BrowserSkoleniFilterForm
 */
class BrowserSkoleniFilterForm extends SkoleniFilterForm
{
    protected $filterLinks;

    //public $filtrELearning;

    public $jenStavSkoleni;
    public $stavSkoleni;
    public $jenHodnoceneSkoleni;
    public $jenPropadleSkoleni;

    /**
     * BrowserSkoleniFilterForm constructor.
     * @param $database
     * @param $action
     */
    public function __construct($database, $action)
    {
        parent::__construct($database, $action, true);

        $this->f_classname = 'filtercontainer';
        $this->filterLinks = new LinksList();
        $this->filterLinks->itemsIdPrefix = 'skoleni_stav';
        $this->povolitVsechyOsoby = true;

        $prava = Prava::vratPrava('skoleni_prohlizeni');

//        $this->nastavDatumOdDo(true, false, TextManager::_('Skoleni_jen_datum_od_do'), true); // nepovinne
        $this->addFilter((new FilterFormFilterDatumOdDo($this))
            ->setCaption(TextManager::_('Skoleni_jen_datum_od_do'))
            ->setPosition(FilterFormFilter::POSITION_TOP)
            ->addLink([
                    'datumOd' => DateUtils::firstDayOfMonth(time()),
                    'datumDo' => DateUtils::lastDayOfMonth(time()),
                    'text' => TextManager::_('Aktualni_mesic')
                ]
            )
        );
        $this->addFilterPP(true, $prava);
//        $this ->nastavOsoby(true, $prava, time(), time()); // prava podle aktualniho casu

        $this ->addFilterRozdeleni1(null, $prava);
        $this ->addFilterRozdeleni2(null, $prava);
//        $this->nastavRozdeleni(true);
//        $this->f_pravarozdeleni = $prava;

        $data = self::getStavySkoleni($database);
        $this->filterLinks->addItemsFromArray($data, null, false);
        $this->filterLinks->url = '?menuid=skoleni&';

        $this ->addSpecifikaceFilter();
        $this ->addKategorieFilter();
        $this ->addDruhSkoleniFilter();

        $this ->addFilter((new FilterFormFilter('elearning', FilterFormFilter::TYPE_OPTIONS))
            ->setSessionName('filtrSkoleniElearning')
            ->setCaption(TextManager::_('Skoleni_filtr_elearning'))
            ->addItem(0, TextManager::_('_vse_'))
            ->addItem(1, TextManager::_('skoleni_pouze_elearning'))
            ->addItem(2, TextManager::_('skoleni_bez_elearning'))
        );
//        $combo->dataTypeComparison = false;
//        $combo->showCode = false;
//        $combo->selectedOption = $combo->findOption($this ->filtrELearning);
//        $combo->render();

        $this ->addFilter((new FilterFormFilter('jenPosledniZaznam', FilterFormFilter::TYPE_CHECKBOX))
            ->setSessionName('filtrSkoleniJenPosledniZaznam')
            ->setRowId('filterrow_jen_posledni_skoleni')
            ->setCaption2(TextManager::_('Skoleni_jen_posledni_zaznam'))
        );
    }


    public static $stavySkoleni = null;

    /**
     * @param Database $database
     * @return null
     */
    public static function getStavySkoleni($database)
    {
        if (self::$stavySkoleni) {
            return self::$stavySkoleni;
        }

        $sql =
            'SELECT Kod, Popis FROM S_StavPotvrzeni ORDER BY Kod';

        $data[-100] = ronText('Vse');

        try {
            $cursor = $database->getSqlResult($sql);

            while ($row = $cursor->fetch()) {
                $data[$row[0]] = ronText('Stav_potvrzeni_skoleni_' . $row[0], '') ?: $row[1];
            }
            $cursor ->close();
        } catch (Exception $exception) {
            MessageManager::addErrorMessage('Chyba pri nacitani stavu potvrzeni skoleni ' . $exception->getMessage());
        }

        $data[100] = ronText('Skoleni_jen_hodnoceni');
        $data[101] = ronText('Skoleni_propadle');

        self::$stavySkoleni = $data;
        return $data;
    }




    public function renderStavSkoleni()
    {
        echo '<div class="filterrow wrap">';
        echo '<span class="filtercaption">' . ronText('Stav_skoleni') . '</span>';

        $this->filterLinks->render();

        echo "</div>\n";
    }


//    public function renderJenPosledniZaznam()
//    {
//        echo '<div class="filterrow" id="filterrow_jen_posledni_skoleni">';
//
//        echo
//            '<span class="filtercaption">' .
//            '</span>';
//
//        GuiUtils::renderCheckbox('jenPosledniZaznam',
//            ronText('Skoleni_jen_posledni_zaznam'),
//            RonPortalSession::getValue('jenPosledniZaznam'), '', 'clickAutoSubmit', 1);
//
//        echo '</div>';
//    }


//    private function loadTypSkoleni()
//    {
//        if (isset($_POST['adsfilterform'])) {
//            $_SESSION['jenTypSkoleni'] = ($_POST['druhskoleni'] !== ''); // isset($_POST['jenTypSkoleni']);
//            $_SESSION['typSkoleni'] = $_POST['druhskoleni'];
//        }
//
//        if (isset($_GET['druhskoleni'])) {
//            $_SESSION['jenTypSkoleni'] = ($_GET['druhskoleni'] !== ''); // isset($_POST['jenTypSkoleni']);
//            $_SESSION['typSkoleni'] = $_GET['druhskoleni'];
//        }
//    }

//    private function loadPosledniZaznam()
//    {
//        if (isset($_POST['adsfilterform'])) {
//            $_SESSION['jenPosledniZaznam'] = isset($_POST['jenPosledniZaznam']);
//        }
//        if (isset($_GET['druhskoleni'])) {
//            $_SESSION['jenPosledniZaznam'] = isset($_GET['jenPosledniZaznam']);
//        }
//    }

//    private function loadElearning()
//    {
//        if (isset( $_GET['elearning'])) {
//            $this->filtrELearning = (int)$_GET['elearning'];
//            $_SESSION['filtrSkoleniElearning'] = $this->filtrELearning;
//        } else {
//            $this ->filtrELearning = $_SESSION['filtrSkoleniElearning'] ?? 0;
//        }
//    }


    private function loadStavSkoleni()
    {
        $vybraneStavy = NastaveniUzivatele::getUserSettingsItem($this->database, 'skoleni_filtr_stav');
        $this ->filterLinks ->loadSettingString($vybraneStavy);
        $this ->filterLinks ->loadGet();

        //je vybrano vse
        if (isset($_GET['-100']) && ($_GET['-100'] == 1)) {
            foreach ($this ->filterLinks->items as $item) {
                if ($item ->key != -100) {
                    $item->active = false;
                }
            }

            $this->jenStavSkoleni = false;
        } else {
            $this->filterLinks->findItem(-100)->active = false;
            $this->jenStavSkoleni = true;
        }

        $result = null;
        $this->jenHodnoceneSkoleni = false;
        $this->jenPropadleSkoleni = false;
        if ($this->jenStavSkoleni) {
            foreach ($this->filterLinks->items as $item) {
                if ((!(($item->key == -100) || ($item->key >= 100))) && $item->active) {
                    $result[] = $item->key;
                }
                //jen skoleni s hodnocenim
                if (($item->key == 100) && $item->active) {
                    $this->jenHodnoceneSkoleni = true;
                }
                //jen prosle skoleni
                if (($item->key == 101) && $item->active) {
                    $this->jenPropadleSkoleni = true;
                }
            }
            if ($result == null) {
                $this->jenStavSkoleni = false;
            }
            $this->stavSkoleni = $result;
        }

        if (($vybraneStavy == '') || (($result == null) && !($this->jenHodnoceneSkoleni || $this->jenPropadleSkoleni)))  {
            $this->filterLinks->activateItem(-100);
        }

        $vybraneNew = $this ->filterLinks ->getSettingsString();
        if ($vybraneNew != $vybraneStavy) {
            NastaveniUzivatele::setUserSettingsItem($this->database, 'skoleni_filtr_stav', $vybraneNew, '');
        }

        $_SESSION['jenStavSkoleni'] = $this->jenStavSkoleni;
        $_SESSION['stavSkoleni'] = $this->stavSkoleni;
        $_SESSION['skoleniHodnocene'] = $this->jenHodnoceneSkoleni;
    }

//    function renderDatumOdDo()
//    {
//        parent::renderDatumOdDo();
//        if ($this->nadpisDatumOdDo) $nadpis = $this->nadpisDatumOdDo;
//        else $nadpis = ronText('Skoleni_jen_datum_od_do');
//
//        echo '<DIV class="filterrow">';
//
//            echo '<SPAN class="filtercaption">';
//            if ($this->datumOdDoPovinne) {
//                echo $nadpis; // .' &nbsp;&nbsp;';
//            } else {
//                //Na prani MaKR checkbox jako prvni
//                GuiUtils::renderCheckbox('jendatumoddo', $nadpis, RonPortalSession::getFiltrovatDatum(), '', '', null, false);
//            }
//            echo '</SPAN>';
//
//            echo '<SPAN style="white-space:nowrap;">';
//            DateInput::renderDateSelect('datumod', date('j.n.Y', RonPortalSession::getDatumOd()), $this ->f_autosubmit); // submitonenter
//
//            echo '&nbsp;&nbsp;&nbsp; ' . ronText('Do') . ' &nbsp;&nbsp;';
//            DateInput::renderDateSelect('datumdo', date('j.n.Y', RonPortalSession::getDatumDo()), $this ->f_autosubmit); // submitonenter
//            echo '</SPAN>';
//
//        echo "</DIV>\n";
//    }

    public function loadData()
    {
//        $this->loadDruhSkoleni();

        $this->loadStavSkoleni();
        //$this->loadElearning();
        //$this->loadPosledniZaznam();
        //$this->loadSpecifikaceKurzu();
        //$this->loadKategorieSkoleni();

        if (!RonPortalSession::getValue('skoleniPrvniPristup')) {
            RonPortalSession::setValue('skoleniPrvniPristup', true);

            $params = $this->database->createParameters();
            $idpp = DBUtils::getSqlValue($this->database,
                'SELECT MAX(ID) FROM PracovniPomer' .
                ' WHERE RC = ' . $params->addParameter(User::getRC()) .
                ' AND DatumNastupu <= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) .
                ' AND (DatumUkonceni >= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) . ' OR DatumUkonceni IS NULL)',
                $params);
            RonPortalSession::setVybranaOsoba($idpp, null, null, null);
        }

        parent::loadData();
    }


    public function renderFilters()
    {
//        $this->comboOsoba->className .= ' ronselect';
//        $this->comboOsoba->htmlParams['data-ronselect-class'] = 'ronselect adsfilterforminput_full';
//        $this->comboOsoba->htmlParams['data-columns'] = 'text';

        parent::renderFilters(); // osoba, rozdeleni

        //$this->renderDruhSkoleni();
        //$this->renderSpecifikaceKurzu();
        //$this->renderKategorieSkoleni();
        //$this->renderElearning();

        //$this->renderJenPosledniZaznam();
        $this->renderStavSkoleni();
    }
}


class PrihlaseniField extends BrowserField
{
    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-elearning']) {
            return '';
        }

        if ($rowData['col-potvrzeni'] == 1) {
            if ($rowData['col-cisloskoleni']) {
                return
                    ronText('Skoleni_Prihlasen') .
                    ($rowData['col-popis-stavu-prihlaseni'] ? " ({$rowData['col-popis-stavu-prihlaseni']})" : '') .
                    ' ' .
                    $rowData['@col-datumpocatkuskoleni'];
            } else if ((int)$rowData['col-pocet-terminu']) {
                return
                    '<a href="?menuid=skoleni&action=zobrazprihlaseni&osoba=' . $rowData['col-idpp'] . '&typskoleni=' . $rowData['col-druhskoleni'] . '">' .
                    ronText('Skoleni_prihlasit') .
                    '</a>';
            } else {
                return
                    ronText('Skoleni_nejsouTerminy');
            }
        }

        return '';

    }
}


/**
 * Class AbsolvovaniField
 */
class AbsolvovaniField extends BrowserField
{
    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //pro druhy skoleni, ktere nejsou elearning
        if (!$rowData['col-elearning'] &&
            //u stavu potvrzeni 0,1,5,6 umoznit zapis absolvovani
            (($rowData['col-potvrzeni'] == 0) || ($rowData['col-potvrzeni'] == 1) || ($rowData['col-potvrzeni'] == 5) || ($rowData['col-potvrzeni'] == 6))
            && Prava::vratPrava('skoleni_absolvovani')->lzeZobrazitRC($rowData['col-rc'])
        ) {
            return '<a href="?menuid=skoleni&action=zapisabsolvovani&formaction=edit&osoba=' . $rowData['col-idpp'] . '&druhskoleni=' . $rowData['col-druhskoleni'] .
                '&datumpovinnosti=' . $this->database->stringToTimestamp($rowData['col-datumvznikupovinnosti']) . '">'
                . ronText('Skoleni_zapsat_absolvovani') . '</a>';
        }

        return '';
    }
}



/**
 * Class DatumSkoleniField
 */
class DatumSkoleniField extends BrowserField
{
    function getDataCellClass($value, $rowNumber, $recordId, $rowData)
    {
        return
            parent::getDataCellClass($value, $rowNumber, $recordId, $rowData) .
            ($this->database->stringToTimestamp($value) > DateUtils::today()
                ? ' skoleni_planovane'
                : ' skoleni_uplynule'
            );
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //absolvovano uspesne                               absolvovano neuspesne                       neucast, prominuto
        if (!(($rowData['col-potvrzeni'] == 3) || ($rowData['col-potvrzeni'] == 4) || ($rowData['col-potvrzeni'] >= 7))) {
            return '';
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}



class DetailSkoleni extends BrowserField
{
    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param BrowserRowArray $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        if ($rowData['col-druhskoleni'] && $rowData['col-datumvznikupovinnosti']) {

            $datumPovinnosti = $this->database->stringToTimestamp($rowData['col-datumvznikupovinnosti']);
            $druhSkoleni = StringUtils::htmlTextConvert(urlencode($rowData['col-druhskoleni']));

            $result .=
                "<span class='open-dialog js' data-dialog='#detail_skoleni_dialog'" .
                " data-ronform-url='?menuid=skolenijs&action=detailskoleni&idpp={$rowData["col-idpp"]}&druhskoleni=$druhSkoleni&datum=$datumPovinnosti'" .
                //" data-ronform-record='{$rowData['col-id']}'" .
                '>' .
                parent::getViewText($value, $rowNumber, $recordId, $rowData) .
                '</span>';
        }
        return $result;
    }
}


class DokumentySkoleniBrowserField extends BrowserField
{
    /** Vrati HTML pro vypsani seznamu dokumentu kurzu, kazdy dokument je vnoren do tagu span s class dokument_kurzu.
     * @param string $cisloSkoleni
     * @return string
     * @throws Exception
     */
    function vratDokumentyKurzu($cisloSkoleni, $osoba)
    {
        $result = '';

        $params = $this->database->createParameters();

        $sql =
            "SELECT Dokument.ID, Dokument.popis, Dokument.jmenoSouboru" . //0..2
            " FROM DokumentSkoleni JOIN Dokument ON (DokumentSkoleni.id = Dokument.id)" .
            "  JOIN TypDokumentu ON (TypDokumentu.kod = Dokument.typ)" .
            " WHERE DokumentSkoleni.cisloSkoleni = " . $params->addParameter($cisloSkoleni) .
            "  AND (Dokument.zobrazitNaIntrawebu = 1 OR (Dokument.zobrazitNaIntrawebu = 2 AND TypDokumentu.zobrazitNaIntrawebu = 1))";

        try {
            $cursor = $this->database->select($sql, $params);

            while ($row = $cursor->fetch()) {
                $osobaStr = '';
                if (isset($osoba)) {
                    $osobaStr = "&osoba=" . $osoba;
                }

                $linkStr = "?menuid=skoleni&action=download&cisloskoleni=" . urlencode($cisloSkoleni) . "&dokument={$row[0]}" . $osobaStr;

                $title = StringUtils::htmlTextConvert($row[1]) ; //. ' - ' . StringUtils::htmlTextConvert($row[2]);

                $result .=
                    "<span class='dokument_kurzu'><a target='_blank' href='$linkStr'>" .
                    "<i title='$title' class='inline-glyph fa glyph-attachment'></i>" .
                    "</a></span>";
            }
        } catch (Exception $exception) {
            global $development;

            if ($development || App::getDebugMode()) {
                throw $exception;
            }
        }

        return $result;
    }


    /**
     * Vrati HTML pro vypsani seznamu dokumentu absolvovaneho skoleni, kazdy dokument je vnoren do tagu span s class dokument_absolvovani.
     * @param BrowserRowArray $values
     * @return string
     * @throws Exception
     */
    function vratDokumentyAbsolvovanehoSkoleni($values)
    {
        $result = '';

        $params = $this->database->createParameters();

        $sql =
            "SELECT Dokument.ID, Dokument.popis, Dokument.jmenoSouboru " . //0..2
            "FROM DokumentyOsoby JOIN Dokument ON (DokumentyOsoby.id = Dokument.id) " .
            " JOIN TypDokumentu ON (TypDokumentu.kod = Dokument.typ) " .
            "WHERE DokumentyOsoby.RC = " . $params->addParameter($values['col-rc']) .  //RC osoby skoleni
            "  AND DokumentyOsoby.GSk_RC = DokumentyOsoby.RC" .
            "  AND DokumentyOsoby.GSk_DruhSkoleni = " . $params->addParameter($values['col-druhskoleni']) . //druh skoleni
            "  AND DokumentyOsoby.GSk_Datum = " . $params->addTimestampParameter($this ->database ->stringToTimestamp($values['col-datumvznikupovinnosti'])) .  //datum vzniku povinnosti
            "  AND (Dokument.zobrazitNaIntrawebu = 1 OR (Dokument.zobrazitNaIntrawebu = 2 AND TypDokumentu.zobrazitNaIntrawebu = 1))";

        try {
            $cursor = $this->database->select($sql, $params);

            while ($row = $cursor->fetch()) {
                $linkStr =
                    "?menuid=skoleni&action=download&osoba=" . $values['col-idpp'] .
                    "&druhskoleni=" . $values['col-druhskoleni'] .
                    "&datum=" . $this->database->stringToTimestamp($values['col-datumvznikupovinnosti']) .
                    "&dokument={$row[0]}";

                $title = StringUtils::htmlTextConvert($row[1]) ; //. ' - ' . StringUtils::htmlTextConvert($row[2]);

                $result .=
                    "<span class='dokument_absolvovani'><a target='_blank' href='$linkStr'>" .
                    "<i title='$title' class='inline-glyph fa glyph-attachment'></i>" .
                    "</a></span>";
            }
        } catch (Exception $exception) {
            global $development;

            if ($development || App::getDebugMode()) {
                throw $exception;
            }
        }

        return $result;
    }

    /**
     * Zobrazi dokumenty skoleni v jedne bunce tabulky. Dokumenty kurzu v divu s class skoleni_dokumenty_kurzu a dokumenty absolvovaneho skoleni
     * v divu s class skoleni_dokumenty_absolvovani.
     * @param BrowserRowArray $values
     * @return string
     */
    function zobrazDokumentySkoleni($values)
    {
        $result = '';

        $dokumentyKurzu = '';
        if ((int)$values['col-pocet-dokumentu-skoleni'] > 0) {
            $dokumentyKurzu = $this->vratDokumentyKurzu($values['col-cisloskoleni'], $values['col-rc'] !== User::getRC() ? $values['col-idpp'] : null);
        }

        $dokumentyAbsolvovane = '';
        if ((int)$values['col-pocet-dokumentu-absolvovaneho-skoleni'] > 0) {
            $dokumentyAbsolvovane = $this->vratDokumentyAbsolvovanehoSkoleni($values);
        }

        if (($dokumentyKurzu) || ($dokumentyAbsolvovane)) {
            if ($values['col-cisloskoleni']) {
                $result .= "<div class='skoleni_dokumenty_kurzu'>";
                $result .= $dokumentyKurzu;
                $result .= "</div>";
            }

            $result .= "<div class='skoleni_dokumenty_absolvovani'>";
            $result .= $dokumentyAbsolvovane;
            $result .= "</div>";
        }

        return $result;
    }

    /**
     * @param $value
     * @param $rowNumber
     * @param $recordId
     * @param BrowserRowArray $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //Zobrazeni dokumentu jen pokud nejake jsou
        if ((int)$rowData['col-pocet-dokumentu-skoleni'] > 0 || (int)$rowData['col-pocet-dokumentu-absolvovaneho-skoleni'] > 0) {
            return $this->zobrazDokumentySkoleni($rowData);
        }

        return '';
    }
}

class PlatnostSkoleniBrowserField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //absolvovano uspesne                   absolvovano neuspesne               neucast - omluven                   neucast - bez omluvy                    prominuto
        if (($rowData['col-potvrzeni'] == 3) || ($rowData['col-potvrzeni'] == 4) || ($rowData['col-potvrzeni'] == 7) || ($rowData['col-potvrzeni'] == 8) || ($rowData['col-potvrzeni'] == 9)) {
            //pro 32b PHP konci prevod na timestamp chybou a nema tedy smysl porovnavat, u 64b prevede spravne a zobrazi 31.12.9999
            if ($this->database->stringToTimestamp($rowData['col-platnostdo']) === false) {
                return '';
            }
            if (($rowData['col-platnostdo'] != '') && ($this->database->stringToTimestamp($rowData['col-platnostdo']) < DateUtils::today())) {
                return TextManager::_('Skoleni_propadle');
            }
        }

        return DBUtils::getFormatedValue($this->database, 'date', $rowData['col-platnostdo']);
    }
}

/**
 * Class SkoleniBrowser
 */
class SkoleniBrowser extends RonPortalBrowser
{
    /** @var PravaNaOsoby $pravoSkoleniProhlizeni */
    public $pravoSkoleniProhlizeni;

    /**
     * SkoleniBrowser constructor.
     * @param Database $database
     * @throws Exception
     */
    function __construct($database)
    {
        parent::__construct($database, 'skoleni');

        $this->pravoSkoleniProhlizeni = Prava::vratPrava('skoleni_prohlizeni');
        Prava::nastavObdobiProPrava(time(), time());

        $this->code = 'browserskoleni';
        $this ->maxPageRows = 100;

        $this ->tableHtmlParams['data-table-name'] = 'GenerovaneSkoleni';

        $this->sqlFromPart =
            'GenerovaneSkoleni SZ LEFT OUTER JOIN Skoleni S  ON S.CisloSkoleni = SZ.CisloSkoleni ' .
            '   LEFT OUTER JOIN DruhSkoleni D ON SZ.DruhSkoleni = D.Kod  ' .
            //'   LEFT OUTER JOIN S_StavPotvrzeni ON S_StavPotvrzeni.Kod = SZ.Potvrzeni' .
            '   LEFT OUTER JOIN Osoba ON SZ.RC = Osoba.RC' .
            '   LEFT OUTER JOIN S_StavSchvalPrihlNaKurz ON (S_StavSchvalPrihlNaKurz.Kod = SZ.SchvaleniPrihlasky)';


        $this->sqlWherePart = ' D.Aktivni = 1 AND D.ZobrazitNaIntrawebu = 1';

        $isFirebird = $database->getDbType() == 'interbase';

        if ($isFirebird) {
            $this->sqlSelectPart = 'Osoba.Prijmeni, Osoba.Jmeno, S.Popis, SZ.PozadovaneDatumSkoleni, SZ.DatumSkoleni, SZ.Potvrzeni';
        }

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SZ.RC')
            ->setCode('col-rc');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SZ.DruhSkoleni')
            ->setCode('col-druhskoleni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SZ.CisloSkoleni')
            ->setCode('col-cisloskoleni');

        $this->addField(new OsobniCisloBrowserField($database, 'col-rc'))
            ->setCode('col-osobnicislo');
            //->setVisible($this ->filterForm ->vsechnyOsoby);

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SZ.DatumVznikuPovinnosti') ->setFormat('date')
            ->setCode('col-datumvznikupovinnosti');

        $this ->addField()
            ->setSqlFieldName('Osoba.Prijmeni', $isFirebird ? 0 : null) ->setHeaderText('Prijmeni')
            ->setCode('col-prijmeni')
            ->setSortSQL($isFirebird ? '1' : 'Osoba.Prijmeni')
            //->setVisible($this ->filterForm ->vsechnyOsoby)
            ->setOrdering(true) ->setClassName('font-weight-bold');

        $this ->addField()
            ->setSqlFieldName('Osoba.Jmeno', $isFirebird ? 1 : null) ->setHeaderText('Jmeno')
            ->setCode('col-jmeno')
            ->setSortSQL($isFirebird ? '2' : 'Osoba.Jmeno')
            //->setVisible($this ->filterForm ->vsechnyOsoby)
            ->setOrdering(true) ->setClassName('font-weight-bold');

        $this->addField()
            ->setSqlFieldName('S.Popis', $isFirebird ? 2 : null) ->setHeaderText('Skoleni_popis_kurzu')
            ->setCode('col-popis-kurzu')
            ->setSortSQL($isFirebird ? '3' : 'S.Popis')
            ->setOrdering(true) ->setVisible(false) ->setClassName('font-weight-bold') ->setWrap(true);

        $this->addField()
            ->setSqlFieldName('D.Popis')
            ->setCode('col-popis-skoleni') ->setHeaderText('Druh_skoleni')
            ->setClassName('font-weight-bold')
            ->setSqlFieldIndex2('col-popisskoleni', ' - ');

        $this->addField()
            ->setSqlFieldName('SZ.PozadovaneDatumSkoleni', $isFirebird ? 3 : null) ->setHeaderText('Pozadovane_datum_skoleni')
            ->setCode('col-pozadovane-datum-skoleni')
            ->setSortSQL($isFirebird ? '4' : 'SZ.PozadovaneDatumSkoleni')
            ->setFormat('finedate') ->setOrdering(true) ->setAlign('center') ->setWrap(true); // wrap kvuli headeru

        $this->addField(new DatumSkoleniField($database))
            ->setSqlFieldName('SZ.DatumSkoleni', $isFirebird ? 4 : null) ->setHeaderText('Skoleni_datum')
            ->setCode('col-datumskoleni')
            ->setSortSQL($isFirebird ? '5' : 'SZ.DatumSkoleni')
            ->setFormat('finedate') ->setAlign('center') ->setOrdering(true) ->setWrap(true);

        $this->defaultSort = '-col-datumskoleni';

        $this ->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('SZ.Potvrzeni', $isFirebird ? 5 : null) ->setHeaderText('Stav_skoleni') ->setOptions(BrowserSkoleniFilterForm::getStavySkoleni($database))
            ->setCode('col-potvrzeni')
            ->setSortSQL($isFirebird ? '6' : 'SZ.Potvrzeni')
            ->setOrdering(true) ->setAlign('center') ->setWrap(true);

        $this->addField(new PlatnostSkoleniBrowserField($database))
            ->setCode('col-platnostskoleni')
            ->setHeaderText('Skoleni_platnost');

//        $this->addField()
//            ->setSqlFieldName('S_StavPotvrzeni.Popis') ->setHeaderText('Stav_skoleni')
//            ->setCode('col-popis-stavu-skoleni') ->setVisible(false)
//            ->setOrdering(true) ->setAlign('center') ->setWrap(true);

        $this->addField(new DetailSkoleni($database))
            ->setType(BrowserField::FIELD_TYPE_TEXT)
            ->setCode('col-detail')
            ->setAlign('center') ->setText(TextManager::_('Detail'));

        $this->addField(new DokumentySkoleniBrowserField($database))
            ->setCode('col-dokumenty') ->setHeaderText('Dokumenty')
            ->setAlign('center') ->setOrdering(false);

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName(
                '(SELECT MAX(ID) FROM PracovniPomer' .
                ' WHERE RC = SZ.RC  AND PracovniPomer.DatumNastupu <= {fn_GETDATE()}' .
                '  AND (PracovniPomer.DatumUkonceni >= {fn_GETDATE()} OR PracovniPomer.DatumUkonceni IS NULL))')
            ->setCode('col-idpp');

        $this->addField(new AbsolvovaniField($database))
            ->setHeaderText('Skoleni_zapsat_absolvovani')
            ->setClassName('prehled-skoleni-absolvovat')
            ->setCode('col-zapsat-absolvovani');

        $this->addField(new PrihlaseniField($this->database))
            ->setCode('col-prihlaseni-skoleni') ->setHeaderText('Skoleni_prihlaseni')
            ->setWrap(true) ->setClassName('prehled-skoleni-prihlaseni-na-kurz');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SZ.PopisSkoleni')
            ->setCode('col-popisskoleni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('S.DatumPocatkuSkoleni') ->setFormat('date')
            ->setCode('col-datumpocatkuskoleni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('S_StavSchvalPrihlNaKurz.Popis')
            ->setCode('col-popis-stavu-prihlaseni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('D.Elearning')
            ->setCode('col-elearning');

        // pocet vypsanych terminu
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName(
                "(SELECT COUNT(Skoleni.CisloSkoleni) " .
                "FROM Skoleni " .
                "WHERE Skoleni.DruhSkoleni = D.Kod" .
                "  AND Skoleni.StavSkoleni IN (0, 1) " .
                //'  AND (Skoleni.DatumUkonceniSkoleni >= ' . $this->database->timestampToString(time()) . ' OR Skoleni.DatumUkonceniSkoleni IS NULL)' .
                '  AND (Skoleni.DatumUkonceniSkoleni >= {fn_GETDATE()} OR Skoleni.DatumUkonceniSkoleni IS NULL)' .
                "  AND Skoleni.DatumPocatkuSkoleni >= {fn_GETDATE()}" .
                "  AND (Skoleni.DatumPotvrzeniUcasti >= {fn_GETDATE()} OR Skoleni.DatumPotvrzeniUcasti IS NULL)" .
                "  AND (Kapacita > (SELECT COUNT(RC) FROM GenerovaneSkoleni WHERE GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni))" .
                "  AND D.Elearning = 0" .
                ")")
            ->setCode('col-pocet-terminu');

        // pocet dokumentu skoleni
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName(
                "(SELECT COUNT(Dokument.ID)" .
                " FROM DokumentSkoleni JOIN Dokument ON (DokumentSkoleni.id = Dokument.id)" .
                " JOIN TypDokumentu ON (TypDokumentu.kod = Dokument.typ)" .
                " WHERE DokumentSkoleni.cisloSkoleni = SZ.cisloSkoleni" .
                "  AND (Dokument.zobrazitNaIntrawebu = 1 OR (Dokument.zobrazitNaIntrawebu = 2 AND TypDokumentu.zobrazitNaIntrawebu = 1))" .
                ")")
            ->setCode('col-pocet-dokumentu-skoleni');

        // pocet dokumentu absolvovaneho skoleni
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName(
                "(SELECT COUNT(Dokument.ID)" .
                " FROM DokumentyOsoby JOIN Dokument ON (DokumentyOsoby.id = Dokument.id)" .
                " JOIN TypDokumentu ON (TypDokumentu.kod = Dokument.typ)" .
                " WHERE DokumentyOsoby.RC = SZ.RC" .  //RC osoby skoleni
                "  AND DokumentyOsoby.GSk_RC = SZ.RC" .
                "  AND DokumentyOsoby.GSk_DruhSkoleni = SZ.DruhSkoleni" . //druh skoleni
                "  AND DokumentyOsoby.GSk_Datum = SZ.DatumVznikuPovinnosti" .  //datum vzniku povinnosti
                "  AND (Dokument.zobrazitNaIntrawebu = 1 OR (Dokument.zobrazitNaIntrawebu = 2 AND TypDokumentu.zobrazitNaIntrawebu = 1))" .
                ")")
            ->setCode('col-pocet-dokumentu-absolvovaneho-skoleni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName(
                '(SELECT COUNT(HodnoceniSkoleni.TemaCislo)' .
                ' FROM HodnoceniSkoleni' .
                ' WHERE S.CisloSkoleni = HodnoceniSkoleni.CisloSkoleni)')
            ->setCode('col-pocet-hodnoceni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SZ.PozadovanaPlatnostDo')
            ->setCode('col-platnostdo');

        $this->primaryField = 0;
    }



    function setFilterForm()
    {
        // parent::setFilterForm(); - parent nevolat
        $this ->filterForm = new BrowserSkoleniFilterForm($this ->database, $this ->action);
    }




//    /**
//     * @param string $sqlFilter
//     * @throws DBException
//     * @throws Exception
//     */
//    function load($sqlFilter = '')
//    {
//        $this ->loadConfiguration();
//
//        $this ->orderFieldIndex = $this ->getOrderFieldIndex();
//        $sqlText = $this ->getSQL($sqlFilter);
//
//        $this ->loadSQL($sqlText);
//    }


    function zobrazEKurzy($values, $className)
    {
        $params = $this ->database ->createParameters();

        $sql =
            'SELECT EKurz.ID, EKurz.Nazev, EKurz.PlatnostOd, EKurz.PlatnostDo, ' . // 0..3
            '  EKurzOsoby.Id, EKurzOsoby.Stav, EKurzOsoby.DatumCasZahajeni, EKurzOsoby.PosledniPristup, ' . // 4..7
            '  RevizeEKurzu.SestavaProCertifikat, RevizeEKurzu.UlozitCertifikatDoDok, SestavyProIntraweb.Nazev' . // 8..10
            ' FROM (EKurzDruhuSkoleni INNER JOIN EKurz  ' .
            '   ON EKurzDruhuSkoleni.IDKurzu = EKurz.ID) ' .
            ' LEFT OUTER JOIN (EKurzOsoby INNER JOIN RevizeEKurzu ON EKurzOsoby.IdRevize = RevizeEKurzu.Id)' .
            '   ON EKurzOsoby.RC_Skoleni = ' . $params ->addParameter($values['col-rc']) .
            '  AND EKurzOsoby.DruhSkoleni = ' . $params ->addParameter($values['col-druhskoleni']) .
            '  AND EKurzOsoby.DatumVznikuPovinnosti = ' . $params ->addTimestampParameter($this ->database ->stringToTimestamp($values['col-datumvznikupovinnosti'])) .
            '  AND RevizeEKurzu.IDKurzu = EKurz.ID ' .
            ' LEFT OUTER JOIN SestavyProIntraweb' .
            '  ON RevizeEKurzu.SestavaProCertifikat = SestavyProIntraweb.Kod' .
            ' WHERE EKurzDruhuSkoleni.KodDruhuSkoleni = ' . $params ->addParameter($values['col-druhskoleni']) .
            '   AND EKurz.Stav = 1' . // jen aktivni
            ' ORDER BY EKurzDruhuSkoleni.Poradi, EKurzOsoby.DatumCasZahajeni DESC';

        try {
            $cursor = $this->database->getSqlResult($sql, $params);

            while ($row = $cursor->fetch()) {
                echo "<tr class='$className'><td class='padding-left-4' colspan='" . ($this->columnCount) . "'>";

                if ($row[3] && $this->database->stringToTimestamp($row[3]) < DateUtils::today()) {
                    echo ronText('Platnost_kurzu_skoncila');
                } else {
                    if ($row[4]) {
                        // existuje zaznam EKurzOsoby
                        $linkStr = '?menuid=prezentace&kurzosoby=' . $row[4];
                    } else {
                        //MaKr: nezobrazovat odkaz pokud je kurz uspesne absolvovan
                        //    abs. uspesne           abs. neuspesne        prominuto            prominuto trvale
                        if (!(($values['col-potvrzeni'] == 3) || ($values['col-potvrzeni'] == 4) || ($values['col-potvrzeni'] == 9) || ($values['col-potvrzeni'] == 10))) {
                            $linkStr =
                                '?menuid=prezentace&zahajitkurz=' . $row[0] .
                                '&druhskoleni=' . StringUtils::htmlTextConvert($values['col-druhskoleni']) .
                                '&datumvznikupovinnosti=' . $this->database->stringToTimestamp($values['col-datumvznikupovinnosti']); // - je tam i cas!
                        }
                    }

                    if (isset($linkStr)) {
                        echo "<a target='_blank' href='$linkStr'>";
                        echo StringUtils::htmlTextConvert($row[1]);
                        echo "</a>";
                    }

                    switch ($row[5]) {
                        case 1 :
                            echo ' - <span class="ekurz_probiha">' . ronText('EKurz_probiha') . '</span>';
                            break;

                        case 3 :
                            echo ' - <span class="ekurz_absolvovan_uspesne">' . ronText('EKurz_absolvovan_uspesne') . '</span>';
                            if ($row[8]) {
                                // download certifikatu
                                // TODO: presunout do kontroleru skoleni?
                                echo ", <a target='_blank' href='?menuid=prezentace&action=certifikat&id={$row[4]}'>" . StringUtils::htmlTextConvert($row[10]) . '</a>';
                            }
                            break;

                        case 4 :
                            echo ' - <span class="ekurz_absolvovan_neuspesne">' . ronText('EKurz_absolvovan_neuspesne') . '</span>';
                            break;
                    }
                }

                echo "</td></tr>\n";
            }
            $cursor->close();
        } catch (Exception $exception) {
            global $development;
            if (App::getDebugMode()  ||  $development) {
                echo
                    "<tr class='browser-error-row'><td colspan={$this->columnCount}>" .
                    '<div class="errortext">' .
                    'Chyba pri nacitani ekurzu. ' .
                    ($exception instanceof DBException
                        ? StringUtils::htmlTextConvert($exception ->dbMessage)
                        : $exception->getMessage()
                    ) .
                    '</div>' .
                    '</td></tr>';
            }
            //MessageManager::addErrorMessage("Chyba pri nacitani ekurzu: " . $exception->getMessage());
        }
    }




    private function zobrazHodnoceniSkoleni($values, $className)
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT DISTINCT HodnoceniSkoleni.TypGenerovani, TemaHodnoceni.PopisTematu, PripraveneTemaHodnoceni.ID, HodnoceniSkoleni.DruhHodnoceni " . // 0 - 3
            "\nFROM PripraveneTemaHodnoceni JOIN HodnoceniSkoleni ON (PripraveneTemaHodnoceni.DruhHodnoceni = HodnoceniSkoleni.DruhHodnoceni " .
            "    AND PripraveneTemaHodnoceni.TemaCislo = HodnoceniSkoleni.TemaCislo) " .
            "   INNER JOIN TemaHodnoceni ON (TemaHodnoceni.DruhHodnoceni = HodnoceniSkoleni.DruhHodnoceni AND TemaHodnoceni.TemaCislo = HodnoceniSkoleni.TemaCislo) " .
            "   LEFT JOIN RozdeleniOsob RO_Hodnoceny ON (RO_Hodnoceny.RC = PripraveneTemaHodnoceni.HodnocenyRC AND RO_Hodnoceny.KodTypuRozdeleni = HodnoceniSkoleni.TypRozdeleni)" .
            "   LEFT JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = RO_Hodnoceny.KodTypuRozdeleni AND Rozdeleni.Kod = RO_Hodnoceny.Rozdeleni) " .
            "   LEFT JOIN RozdeleniOsob RO_Nadrizeny ON (RO_Nadrizeny.KodTypuRozdeleni = Rozdeleni.RozdeleniRodice AND RO_Nadrizeny.Rozdeleni = Rozdeleni.Rodic) ".
            "\nWHERE PripraveneTemaHodnoceni.HodnocenyRC = " . $params->addParameter($values['col-rc']) .
            "\n AND HodnoceniSkoleni.CisloSkoleni = " . $params->addParameter($values['col-cisloskoleni']) .
            "\n AND ((HodnoceniSkoleni.TypGenerovani = 3 AND PripraveneTemaHodnoceni.HodnocenyRC = " . $params->addParameter(User::getRC()) . ")" . // hodnoceni ucastnika (napr. skolitelem)
            "     OR (HodnoceniSkoleni.TypGenerovani = 2 AND HodnoceniSkoleni.RCHodnoticiOsoby = " . $params->addParameter(User::getRC()) . ")" . // hodnoceni skolitele ucastnikem
            "     OR (HodnoceniSkoleni.TypGenerovani = 1 " .
            "            AND RO_Hodnoceny.PlatnostOd <= " . $params->addDateParameter(time()) .
            "            AND (RO_Hodnoceny.PlatnostDo >= " . $params->addDateParameter(time()) .
            "                   OR RO_Hodnoceny.PlatnostDo IS NULL) " .
            "            AND RO_Nadrizeny.RC = " . $params->addParameter(User::getRC()) .
            "            AND RO_Nadrizeny.PlatnostOd <= " . $params->addDateParameter(time()) .
            "            AND (RO_Nadrizeny.PlatnostDo >= " . $params->addDateParameter(time()) .
            "                   OR RO_Nadrizeny.PlatnostDo IS NULL) " .
            "        ) " .
            "     )" .
            "\nORDER BY HodnoceniSkoleni.TypGenerovani DESC";

        $hodnoceni = null;

        try {
            $cursor = $this->database->select($sql, $params);
            while ($row = $cursor->fetch()) {

                $druhHodnoceni = '';
                switch ($row[0]) {
                    case 1 : //ucastnik dle typu rozdeleni
                    case 2 : //ucastnik zvolenou osobou
                        $druhHodnoceni = ronText('Skoleni_Hodnoceni_Ucastnika');
                        break;
                    case 3 : //skoleni ucastnikem
                        $druhHodnoceni = ronText('Skoleni_Hodnoceni');
                        break;
                }
                $hodnoceni .=
                    "<span class='skoleni_hodnoceni'><a href='?menuid=skoleni&action=hodnoceni&record=$row[2]'>" .
                    StringUtils::htmlTextConvert($row[1]) .
                    "</a> - " . $druhHodnoceni . "</span>";
            }
        } catch (Exception $exception) {
            echo
                "<tr class='browser-error-row'><td colspan='{$this->columnCount}'>" .
                '<div class="errortext">' .
                'Chyba pri nacitani hodnoceni skoleni. ' .
                ($exception instanceof DBException
                    ? StringUtils::htmlTextConvert($exception ->dbMessage)
                    : $exception->getMessage()
                ) .
                '</div>' .
                '</td></tr>';
        }

        if (!is_null($hodnoceni)) {
            echo
                "<tr class='$className'><td class='padding-left-4' colspan='{$this->columnCount}'>" .
                "<div class='skoleni_seznam_hodnoceni'>$hodnoceni</div>" .
                "</td></tr>\n";
        }
    }

    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        // Pokud je vybrano konkretni skoleni, zobrazi podrobnosti
//        if ($values[1]) {
        //Zobrazeni podrobnosti presunuto do detailu
//            $this->zobrazPodrobnostiSkoleni($values, $className);

        //Pokud ma prirazeny kurz nastavene hodnoceni
        if ($values['col-pocet-hodnoceni']) {
            $this->zobrazHodnoceniSkoleni($values, $className);
        }


        //e-learningove kurzy hleda jen pokud je druh skoleni e-learning
        if (RonPortalSettings::getIntValue(RonPortalSettings::ELEARNING_POVOLEN) && $values['col-elearning'] && User::hasRC($values['col-rc'])) {
            // pokud zobrazuje vlastni, tak zobrazi prokliky na elearningove kurzy
            $this ->zobrazEKurzy($values, $className);
        }

        $this ->renderSeparator();
    }

    public function getSQL($sqlFilter = '')
    {
        //v tabulce muze GenerovaneSkoleni muze vzniknout zaznam, ktery ma nizsi DatumVznikuPovinnosti, ale zadane pozdejsi datum absolvovani (jedna se tedy o novejsi zaznam)
        //proto se vyhledaji zvlast skoleni s nejvyssim datem skoleni (absolvovani), ktere nemaji vygenerovanou novou povinnost
        //a dalse se vyhledaji ty s nejvyssim datem vzniku povinnosti pro dany typ (generovane)
        if ($this ->filterForm ->getFilterValue('jenPosledniZaznam')) {
            //kvuli nastaveni poli pridanych v konfiguraci se select nacte jen jednou
            $selectPart = $this->getSelectSQLPart();

            $wherePart1 = SQLUtils::concatenateWherePartsOfSql(
                [$this->sqlWherePart, $sqlFilter, $this->getSQLFilter(), $this->getSQLFilterPosledniZaznamAbsolvovane()]);

            $wherePart2 = SQLUtils::concatenateWherePartsOfSql(
                [$this->sqlWherePart, $sqlFilter, $this->getSQLFilter(), $this->getSQLFilterPosledniZaznamGenerovane()]);

            if (!$wherePart1 instanceof SqlStatement) {
                $wherePart1 = new SqlStatement($wherePart1, $this->database->createParameters());
            }
            if (!$wherePart2 instanceof SqlStatement) {
                $wherePart2 = new SqlStatement($wherePart2, $this->database->createParameters());
            }

            $params = $this->database->createParameters();
            $params->appendParameters($wherePart1->parameters);
            $params->appendParameters($wherePart2->parameters);

            $orderBy = $this->getOrderSQLPart();

            // kvuli rychlosti je dotaz rozdelen na 2 poddotazy
            return new SqlStatement(
                'SELECT ' . $selectPart . "\n" .
                ' FROM ' . $this->sqlFromPart . "\n" .
                ($wherePart1->sqlText ? ' WHERE ' . $wherePart1->sqlText : '') .
                "\n\n" .
                "UNION\n\n" .
                'SELECT ' . $selectPart . "\n" .
                ' FROM ' . $this->sqlFromPart . "\n" .
                ($wherePart2->sqlText ? ' WHERE ' . $wherePart2->sqlText : '') .
                "\n\n" .
                ($orderBy ? 'ORDER BY ' . $orderBy : ''),
                $params
            );
        }

        return parent::getSQL($sqlFilter);
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $sql = $this ->filterForm ->buildSqlFilter($params);

        $datumOd = $this ->filterForm ->getFilterValue('datumod') ?? DateUtils::today();
        $datumDo = $this ->filterForm ->getFilterValue('datumod', 'datumDo') ?? $datumOd;
        
        //Overeni prav
        $sql .=
            ($sql ? "\n AND " : '') .
            "EXISTS(SELECT PP.RC FROM PracovniPomer PP" .
            " WHERE (PP.RC = Osoba.RC) " .
            ' AND (' . $this->pravoSkoleniProhlizeni->sqlFiltr('PP', time(), time(), $params) . ')' .
            ' AND (PP.DatumNastupu <= ' . $params ->addDateParameter($datumDo) . ')' .
            ' AND ((PP.DatumUkonceni >= ' . $params->addDateParameter($datumOd) . ') OR (PP.DatumUkonceni IS NULL))' .
            "\n";

        // jen vybrana osoba
        if ($this ->filterForm ->getFilterValue('osoba') !== null) {
            $sql .=
                ' AND PP.RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) . "\n";
        } else {
            foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
                if ($rozdeleniFilter ->getValue()) {
                    $sql .=
                        ' AND ' .
                        RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'PP',
                            $rozdeleniFilter ->getTypRozdeleni(), $rozdeleniFilter ->getValue(),
                            //(int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $this->filterForm->kodRozdeleni,
                            time(), time(),
                            $rozdeleniFilter->getValue('vcetnePodrizenych'),
                            $params);
                }
            }
//            if ($this->filterForm->kodRozdeleni) {
//                $sql .=
//                    ' AND ' .
//                    RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'PP',
//                        (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $this->filterForm->kodRozdeleni,
//                        time(), time(),
//                        $this->filterForm->vcetnePodrizenychRozdeleni,
//                        $params);
//            }
        }

        $sql .=
            ")\n";



        if ($this ->filterForm ->getFilterValue('datumod')) {
            $sql .=
                ($sql ? ' AND ' : '') .
                ' SZ.DatumSkoleni >= ' . $params->addDateParameter($datumOd) .
                ' AND SZ.DatumSkoleni <= ' . $params->addDateParameter($datumDo) .
                "\n";
        }


        if ($this->filterForm->jenStavSkoleni) {
            $sql .=
                ($sql ? ' AND ' : '') .
                SQLUtils::getINCondition($this->database, "SZ.Potvrzeni", $this->filterForm->stavSkoleni, $params);
        }


        if ($this->filterForm->jenHodnoceneSkoleni) {
            $sql .=
                ' AND ' .
                $this ->getSQLFilterHodnocene('SZ', $params);
        }

        if ($this->filterForm->jenPropadleSkoleni) {
            $sql .=
                ($sql ? ' AND ' : '') .
                ' SZ.PozadovanaPlatnostDo < ' . $params->addDateParameter(DateUtils::today()) .
                "\n";
        }


        //if (RonPortalSession::getValue('jenTypSkoleni')) {
        if ($this ->filterForm ->getFilterValue('druhskoleni')) {
            $sql .=
                ' AND SZ.DruhSkoleni = ' . $params->addParameter($this ->filterForm ->getFilterValue('druhskoleni')) .
                "\n";
        }

        if ($this ->filterForm ->getFilterValue('specifikace')) {
            $sql .=
                ' AND S.Specifikace = ' . $params->addParameter($this ->filterForm ->getFilterValue('specifikace')) .
                "\n";
        }


        if ($this ->filterForm ->getFilterValue('kategorie')) {
            $sql .=
                ' AND D.Kategorie = ' . $params->addParameter($this ->filterForm ->getFilterValue('kategorie')) .
                "\n";
        }


        if ($this ->filterForm ->getFilterValue('elearning')) {
            $sql .=
                ($sql ? ' AND ' : '') .
                ($this ->filterForm ->getFilterValue('elearning') == 1
                    ? 'D.Elearning = 1'
                    : 'D.Elearning = 0'
                );
        }

        return new SqlStatement($sql, $params);
    }


    /**
     * @param string $nazevTabulky
     * @param Parameters $params
     * @return string
     */
    public function getSQLFilterHodnocene($nazevTabulky, $params)
    {
        $datumOd = $this ->filterForm ->getFilterValue('datumod') ?? DateUtils::today();
        $datumDo = $this ->filterForm ->getFilterValue('datumod', 'datumDo') ?? $datumOd;

        return
            'EXISTS(SELECT DISTINCT HodnoceniSkoleni.TypGenerovani, TemaHodnoceni.PopisTematu, PripraveneTemaHodnoceni.ID, HodnoceniSkoleni.DruhHodnoceni' . // 0 - 3
            ' FROM PripraveneTemaHodnoceni JOIN HodnoceniSkoleni' .
            '  ON (PripraveneTemaHodnoceni.DruhHodnoceni = HodnoceniSkoleni.DruhHodnoceni' .
            '    AND PripraveneTemaHodnoceni.TemaCislo = HodnoceniSkoleni.TemaCislo)' .
            '  INNER JOIN TemaHodnoceni ON (TemaHodnoceni.DruhHodnoceni = HodnoceniSkoleni.DruhHodnoceni AND TemaHodnoceni.TemaCislo = HodnoceniSkoleni.TemaCislo)' .
            '  LEFT JOIN RozdeleniOsob RO_Hodnoceny ON (RO_Hodnoceny.RC = PripraveneTemaHodnoceni.HodnocenyRC AND RO_Hodnoceny.KodTypuRozdeleni = HodnoceniSkoleni.TypRozdeleni)' .
            '  LEFT JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = RO_Hodnoceny.KodTypuRozdeleni AND Rozdeleni.Kod = RO_Hodnoceny.Rozdeleni)' .
            '  LEFT JOIN RozdeleniOsob RO_Nadrizeny ON (RO_Nadrizeny.KodTypuRozdeleni = Rozdeleni.RozdeleniRodice AND RO_Nadrizeny.Rozdeleni = Rozdeleni.Rodic)' .
            "\n WHERE PripraveneTemaHodnoceni.HodnocenyRC = $nazevTabulky.RC" .
            "  AND HodnoceniSkoleni.CisloSkoleni = $nazevTabulky.CisloSkoleni" .
            '  AND PripraveneTemaHodnoceni.Stav <> 2' .
            '  AND ((HodnoceniSkoleni.TypGenerovani = 3 AND PripraveneTemaHodnoceni.HodnocenyRC = ' . $params->addParameter(User::getRC()) . ')' .
            '     OR (HodnoceniSkoleni.TypGenerovani = 2 AND HodnoceniSkoleni.RCHodnoticiOsoby = ' . $params->addParameter(User::getRC()) . ')' .
            '     OR (HodnoceniSkoleni.TypGenerovani = 1' .
            '       AND RO_Hodnoceny.PlatnostOd <= ' . $params->addDateParameter($datumDo) .
            '       AND (RO_Hodnoceny.PlatnostDo >= ' . $params->addDateParameter($datumDo) . ' OR RO_Hodnoceny.PlatnostDo IS NULL)' .
            '       AND RO_Nadrizeny.RC = ' . $params->addParameter(User::getRC()) .
            '       AND RO_Nadrizeny.PlatnostOd <= ' . $params->addDateParameter($datumDo) .
            '       AND (RO_Nadrizeny.PlatnostDo >= ' . $params->addDateParameter($datumDo) . ' OR RO_Nadrizeny.PlatnostDo IS NULL)' .
            '     )' .
            '  )' .
            ")\n";
    }

    private function getSQLFilterPosledniZaznamAbsolvovane()
    {
        $params = $this->database->createParameters();

        $sql =
            " (SZ.DatumSkoleni IS NOT NULL AND SZ.DatumSkoleni = (Select Max(DatumSkoleni) FROM GenerovaneSkoleni GES WHERE GES.DruhSkoleni = D.Kod AND GES.RC = SZ.RC" .
            ($this->filterForm->jenStavSkoleni
                ? ' AND GES.Potvrzeni = SZ.Potvrzeni'
                : ''
            ) .
            ($this->filterForm->jenHodnoceneSkoleni
                ? ' AND ' . $this ->getSQLFilterHodnocene('GES', $params)
                : ''
            ) .
            ($this->filterForm->jenPropadleSkoleni
                ? ' AND GES.PozadovanaPlatnostDo < ' . $params->addDateParameter(time())
                : ''
            ) .
            " AND NOT EXISTS(SELECT RC FROM GenerovaneSkoleni GES WHERE GES.DruhSkoleni = D.Kod AND GES.RC = SZ.RC AND GES.DatumSkoleni IS NULL)" .
            "     ))";

        return new SqlStatement($sql, $params);
    }

    private function getSQLFilterPosledniZaznamGenerovane()
    {
        $params = $this->database->createParameters();

        $sql =
            ' (SZ.DatumSkoleni IS NULL AND SZ.DatumVznikuPovinnosti = (Select Max(DatumVznikuPovinnosti) FROM GenerovaneSkoleni GES WHERE GES.DruhSkoleni = D.Kod AND GES.RC = SZ.RC' .
            ($this->filterForm->jenStavSkoleni
                ? ' AND GES.Potvrzeni = SZ.Potvrzeni'
                : ''
            ) .
            ($this->filterForm->jenHodnoceneSkoleni
                ? ' AND ' . $this ->getSQLFilterHodnocene('GES', $params)
                : ''
            ) .
            ($this->filterForm->jenPropadleSkoleni
                ? ' AND GES.PozadovanaPlatnostDo < ' . $params->addDateParameter(time())
                : ''
            ) .
            "))\n";

        return new SqlStatement($sql, $params);
    }


    public function render()
    {
        parent::render();

//        require 'person/view_detailskolenikurzu.php';
        $dialog = new DetailSkoleniDialog($this ->database);
        $dialog ->render();
    }
}
