<?php

/**
 * Pavel Rudol, Roman Szablatura RON Software
 *  29.11.2015
 */

namespace RonPortal\Skoleni;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\Http\Csrf;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\IcalOutputObject;
use Ron\MVC\InternalRedirect;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\MessageManagerViewObject;
use Ron\MVC\Model;
use Ron\DateUtils;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\RonException;
use RonPortal\Hodnoceni\HodnoceniViewObject;
use RonPortal\RonPortalUtils;
use RonPortal\Testy\DotaznikViewObject;
use function Ron\ronText;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\System\AgentManager;
use RonPortal\User;


class SkoleniController extends JsonController
{   
    const XML_EVENT_BEFORE = 'SKOLENI_PRIHLASENI_BEFORE';
    const XML_EVENT_AFTER = 'SKOLENI_PRIHLASENI_AFTER';
    const XML_CONTEXT_LOGIN = 'login';
    const XML_CONTEXT_LOGOUT = 'logout';
    const NASTAVENI_PONECHAT_NAVRH = 'skoleni_ponechat_navrh';
    
    /* @var $database Database */
    private $database;


    public function __construct($database)
    {
        $this->database = $database;

        TextManager::getInstance() ->loadStrings('skoleni.ini');
        TextManager::getInstance() ->loadStrings('dotaznik.ini');

        HtmlHeaders::getInstance() ->setIcon(RonPortalSettings::getValue('shortcut_icon') ?: 'images/icon_person.png');

        $this ->addTranslationsScript('skoleni');
    }


    /**
     *  Spusti personalniho agenta pro doplneni udaju pracovniho pomeru OsobniCislo, DatumNastupu, Kalendar, Smena do tabulky GenerovaneSkoleni.
     */
    private function doplnitPracovniPomer($seznamRC)
    {
        //pokud je predano vice RC, vytvori retezec oddeleny stredniky
        $seznamRCStr = is_array($seznamRC)
            ? implode(';', $seznamRC)
            : $seznamRC;

        if ($seznamRCStr != '') {
            try {
                AgentManager::executePersonAgent('4 -DoplnitUdajePP -RCZamestnance:"' . $seznamRCStr . '"');
            } catch (Exception $exception) {
                MessageManager::addErrorMessage($exception);
            }
        }
    }


    /**
     * Zjisti, jestli ma uzivatel pravo pro zobrazeni dokumentu skoleni
     * @param string $skoleni kod skoleni
     * @param int $dokument id dokumentu
     * @return bool
     * @throws DBException
     */
    private function jePravoNaDokumentSkoleni($skoleni, $dokument, $osobaRC)
    {
        if (isset($osobaRC)) {
            $pravaSkoleni = Prava::vratPrava('skoleni_prohlizeni');
            if (!$pravaSkoleni->lzeZobrazitRC($osobaRC, time(), time())) {
                throw new NeniPravoException();
            }
        }

        // bud je uzivatel prihlasen na skoleni, nebo musi byt dokument verejny
        if (!(SkoleniUtils::jeOsobaNaSkoleni($this ->database, $skoleni, $osobaRC ?? User::getRC()) || SkoleniUtils::jeDokumentSkoleniVerejny($this ->database, $dokument))) {
            return false;
        }

        // kontrola, jestli je dokument pripojen ke skoleni a ma se zobrazit na intrawebu
        $params = $this->database->createParameters();
        $sql =
            'SELECT Dokument.Id FROM DokumentSkoleni' .
            '  JOIN Dokument ON (Dokument.Id = DokumentSkoleni.Id)'.
            '  JOIN TypDokumentu ON (TypDokumentu.Kod = Dokument.Typ)' .
            ' WHERE CisloSkoleni = ' . $params->addParameter($skoleni) .
            '  AND Dokument.Id = ' . $params->addParameter($dokument) .
            '  AND (Dokument.ZobrazitNaIntrawebu = 1 OR (Dokument.ZobrazitNaIntrawebu = 2 AND TypDokumentu.ZobrazitNaIntrawebu = 1))';

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        return (bool)$row;
    }

    /**
     * @param string $osobaRC
     * @param string $druhSkoleni
     * @param int $datumPovinnosti
     * @param int $dokument
     * @throws Exception
     * @throws DBException
     */
    private function zkontrolujPravoNaDokumentSkoleniOsoby($osobaRC, $druhSkoleni, $datumPovinnosti, $dokument)
    {
        $pravaOsobniUdaje = Prava::vratPrava('prohl_osobni');
        $pravaSkoleni = Prava::vratPrava('skoleni_prohlizeni');

        if (!($pravaOsobniUdaje->lzeZobrazitRC($osobaRC, time(), time()) || $pravaSkoleni->lzeZobrazitRC($osobaRC, time(), time()))) {
            throw new NeniPravoException();
        }

        $params = $this->database->createParameters();

        $sql =
            "SELECT Dokument.id FROM DokumentyOsoby JOIN Dokument ON (DokumentyOsoby.Id = Dokument.Id) " .
            " JOIN TypDokumentu ON (TypDokumentu.Kod = Dokument.Typ) " .
            "WHERE DokumentyOsoby.RC = " . $params->addParameter($osobaRC) .  //RC osoby skoleni
            "  AND DokumentyOsoby.GSk_RC = " . $params->addParameter($osobaRC) .
            "  AND DokumentyOsoby.GSk_DruhSkoleni = " . $params->addParameter($druhSkoleni) .  //druh skoleni
            "  AND DokumentyOsoby.GSk_Datum = " . $params->addTimestampParameter($datumPovinnosti) .  //datum vzniku povinnosti
            "  AND (Dokument.ZobrazitNaIntrawebu = 1 OR (Dokument.ZobrazitNaIntrawebu = 2 AND TypDokumentu.ZobrazitNaIntrawebu = 1))" .
            "  AND Dokument.Id = " . $params->addParameter($dokument) ;

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        if (!$row) {
            throw new NeniPravoException();
        }
    }

    /**
     * 
     * @return DokumentOutputObject
     * @throws Exception
     */
    private function downloadDokumentSkoleni()
    {
        $dokument = (int)$_GET['dokument'];

        // cislo skoleni
        $skoleni = $_GET['cisloskoleni'] ?? '';

        //v url neni zadano konkretni skoleni, ani osoba ktera absolvovala skoleni
        if ($skoleni === '' && !(isset($_GET['osoba']) && isset($_GET['druhskoleni']) && isset($_GET['datum']))) {
            throw new NeniPravoException();
        }

        //v seznamu skoleni muze byt zobrazeny i dokument podrizene osoby na kurzu, ten je mozne stahnout
        $osobaRC = null;
        if (isset($_GET['osoba'])) {
            // muze se stahovat i dokument osoby, pak se overuje, ze je dokument pripojen ke skoleni osoby
            $params = $this ->database ->createParameters();
            $osobaRC = DBUtils::getSqlValue($this->database,
                'SELECT RC FROM PracovniPomer' .
                ' WHERE ID = ' . $params ->addParameter((int)$_GET['osoba']),
                $params);
        }

        if ($skoleni) {
            // stahuje se dokument skoleni => overi se pravo na skoleni
            // je mozne stahnout i dokument skoleni osoby, na kterou mam pravo
            if (!$this->jePravoNaDokumentSkoleni($skoleni, $dokument, $osobaRC)) {
                throw new NeniPravoException();
            }
        } else {
            if ( !(isset($_GET['osoba']) && isset($_GET['druhskoleni']) && isset($_GET['datum'])) ) {
                throw new NeniPravoException();
            }

            $this->zkontrolujPravoNaDokumentSkoleniOsoby($osobaRC, $_GET['druhskoleni'], (int)$_GET['datum'], $dokument);
        }

        return new DokumentOutputObject($this->database,$dokument);
    }


    /**
     * @return IcalOutputObject
     * @throws Exception
     */
    private function downloadIcal()
    {
        $skoleni = isset($_GET['cisloskoleni']) ? $_GET['cisloskoleni'] : '';

        $cal = SkoleniCalendarItem::loadKurz($this->database, $skoleni);
        if (!$cal) {
            throw new Exception('Kurz nebyl nalezen!');
        }

        return new IcalOutputObject($cal);
    }


    /**
     * zobrazi seznam skoleni, na ktere je mozne se prihlasit
     * @return ViewObject
     * @throws DBException
     * @throws Exception
     */
    private function zobrazPrihlaseniSkoleni()
    {
        if (Prava::vratHodnotuPrava('skoleni_prihlasit') == 0) {
            throw new NeniPravoException();
        }

        $this->hlavickaVyberAkce();

        HtmlHeaders::getInstance()->addScript('script/personscripts.js');   // dotazy pri kliknuti na prihlasit/odhlasit

        $b = new PrihlaseniNaSkoleniBrowser($this ->database);
        $b ->load();
        return $b;
    }

    /**
     * Prihlasi zadanou osobu ($osobaIDPP) na skoleni, pokud je dostupna kapacita, uzivatel ma na prihlaseni pravo
     * Vraci RC prihlasene osoby
     * @param string $cisloSkoleni kod skoleni
     * @param string $osobaIDPP osoba, ktera se prihlasuje na skoleni
     * @param int $datumPovinnosti
     * @param bool $spustitAgenta
     * @return string
     * @throws DBException
     * @throws Exception
     */
    private function prihlaseniSkoleni($cisloSkoleni, $osobaIDPP, $datumPovinnosti, $spustitAgenta = true)
    {
        $skoleni = Skoleni::nactiSkoleni($this ->database, $cisloSkoleni);

        $pp = PracovniPomer::nactiPP($this ->database, $osobaIDPP);

        $prava = Prava::vratPrava('skoleni_prihlasit');
        $pravaPotvrditPrihlaseni = Prava::vratPrava('skoleni_schvalit_prihlaseni');

        if (!$prava->lzeZobrazitRC($pp->rc)) {
            throw new NeniPravoException();
        }

        //skoleni musi byt ve stavu pripravuje se = 0 nebo probiha = 1
        if ($skoleni ->stavSkoleni != Skoleni::STAV_PRIPRAVUJE_SE  &&  $skoleni ->stavSkoleni != Skoleni::STAV_PROBIHA) {
            throw new Exception(sprintf(ronText('SkoleniChybnyStav'), $skoleni ->popis));
        }

        if ($skoleni ->cisloPlanovanehoSkoleni) {
            if (!SkoleniUtils::jeOsobaNaPlanovanemSkoleni($this ->database, $skoleni ->cisloPlanovanehoSkoleni, $pp->rc)) {
                throw new Exception(sprintf(ronText('SkoleniNeniPrihlasenPlan'), $pp->jmenoPrijmeniKratke, $skoleni ->cisloPlanovanehoSkoleni));
            }
        }

        //kontrola kapacity a jednoho prihlaseni osoby se provadi pro bezne kurzy, u e-learningovych se ignoruje
        if (!$skoleni ->elearning) {
            if (!$skoleni->jeVolnaKapacitaSkoleni($this ->database)) {
                throw new Exception(sprintf(ronText('SkoleniNaplnenaKapacita'), $skoleni ->popis, $pp->jmenoPrijmeniKratke));
            }

            if (!SkoleniUtils::jeVolnaKapacitaMistnosti($this ->database, $cisloSkoleni, $pp->rc)) {
                throw new Exception(sprintf(ronText('SkoleniNaplnenaKapacitaMistnosti'), $skoleni ->popis, $pp->jmenoPrijmeniKratke));
            }

            if (SkoleniUtils::jeOsobaNaSkoleni($this->database, $cisloSkoleni, $pp->rc)) {
                throw new Exception(sprintf(ronText('SkoleniJizPrihlasene'), $pp->jmenoPrijmeniKratke, $skoleni ->popis));
            }
        } else {
            if (SkoleniUtils::pocetNeabsolvovanychEKurzu($this ->database, $cisloSkoleni, $pp->rc) > 0) {
                throw new Exception(sprintf(ronText('SkoleniExistujiNeabsolvaneEkurzy'), $pp->jmenoPrijmeniKratke));
            }
        }

        //pro pouziti v XML k identifikaci vybraneho PP, napr. pro zapis do PK
        $xmlVariables = [
            'osobaRC' => $pp ->rc,
            'osobaOsobniCislo' => $pp ->osobniCislo,
            'osobaDatumNastupu' => $pp ->datumNastupu
        ];

        $uspesnaRegistrace = false;

        // TODO: predelat na GenerovaneSkoleniRec
        $record = new RonPortalDBRecord($this->database, 'GenerovaneSkoleni', $this);

        $record->addStringField('RC', true, 15, $pp->rc);
        $record->addLogText('  ' . $pp ->jmenoPrijmeni);
        $record->addStringField('DruhSkoleni', true, 10, $skoleni ->druhSkoleni);
        $record->addTimestampField('DatumVznikuPovinnosti', true, DateUtils::now());

        $record->addDateField('DatumSkoleni', false, $skoleni ->datumPocatkuSkoleni);
        $record->addStringField('CisloSkoleni', false, 40, $skoleni ->cisloSkoleni);
        $record->addDateField('DatumPrihlasky', false, time());
        $record->addIntField('PrihlasenoZAplikace', false, 1);
        $record->addStringField('Prihlasil', false, 25, User::getLoginName());

        $record->addIntField('Potvrzeni', true);

        $record->addIntField('SchvaleniPrihlasky', true);

        // dalsi readonly pole
        $record->addIntField('DuvodSkoleni', true, 1); //individualni duvod skoleni
        $record->addDateField('DatumZapisu', true, DateUtils::today());
        $record->addStringField('Vlozil', true, 25, User::getLoginName());
        $record->addDateField('PozadovaneDatumSkoleni', true, $skoleni ->datumPocatkuSkoleni);


        if ($skoleni ->pozadovatSchvaleniPrihl == 1) {
            //pokud ma uzivatel pravo na potvrzeni prihlaseni, potvzeni zapise, jinak se provadi samostatnou akci
            if ($pravaPotvrditPrihlaseni->lzeZobrazitRC($pp->rc)) {
                $record->findFieldByName('SchvaleniPrihlasky')->setNewValue(1);
                $record->addStringField('Vyhodnotil', false, 25, User::getLoginName());
                $record->addDateField('DatumVyhodnoceni', false, time());
                $record->addMemoField('PoznamkaSchvalovani', 'memo', false);
            } else {
                $record->findFieldByName('SchvaleniPrihlasky')->setNewValue(0);
                $uspesnaRegistrace = true;
            }
        } else {
            $record->findFieldByName('SchvaleniPrihlasky')->setNewValue(-1);
        }

        $record->addIntField('Generovane', true, 0);

        $params = $this->database->createParameters();

        //prihlasovani na jiz vygenerovany/rucne vlozeny zaznam
        if ($datumPovinnosti > 0) {
            $record->setWhere(
                "RC = " . $params->addParameter($pp->rc) .
                " AND DruhSkoleni = " . $params->addParameter($skoleni ->druhSkoleni) .
                " AND DatumVznikuPovinnosti = " . $params->addTimestampParameter($datumPovinnosti),
                $params);
        } else {
            $record->setWhere(
                "RC = " . $params->addParameter($pp->rc) .
                " AND DruhSkoleni = " . $params->addParameter($skoleni ->druhSkoleni) .
                " AND DatumSkoleni IS NULL " .
                " AND NOT EXISTS(SELECT * FROM GenerovaneSkoleni GS WHERE GS.RC = GenerovaneSkoleni.RC " .
                "  AND GS.DruhSkoleni = GenerovaneSkoleni.DruhSkoleni " .
                "  AND GS.DatumVznikuPovinnosti > GenerovaneSkoleni.DatumVznikuPovinnosti " .
                "  AND GS.CisloSkoleni IS NOT NULL)", $params);
        }

        if ($record->loadRecord()) {
            //osoba jiz ma vygenerovanou povinnost skoleni absolvovat, zapisou se informace o prihlasenem skoleni
            //pro povinnost = 0 (doporucene) se pri schvaleni nastavi potvrzeni na 1
            if ($record->findFieldByName('Potvrzeni')->value == 0)
                if (($record->findFieldByName('SchvaleniPrihlasky')->value == -1) || ($record->findFieldByName('SchvaleniPrihlasky')->value == 1)) {
                    $record->findFieldByName('Potvrzeni')->setNewValue(1);
                }

            if (!$record->executeXmlEvents(self::XML_EVENT_BEFORE, self::XML_CONTEXT_LOGIN, $xmlVariables)) {
                throw new Exception(ronText('SkoleniChybaXMLudalosti'));
            }
            
            $record->updateRecord();
        } else {
            //pro osobu neexistuje povinnost absolvovat skoleni (skoleni je doporucene) vlozi se novy zaznam
            $record->findFieldByName('Potvrzeni')->setNewValue(0);

            //pro schvalene nebo pokud se nevyzaduje schvaleni nastavi se potvrzeni na 1
            if (($record['SchvaleniPrihlasky'] == -1) || ($record['SchvaleniPrihlasky'] == 1)) {
                $record->findFieldByName('Potvrzeni')->setNewValue(1);
            }

            if (!$record->executeXmlEvents(self::XML_EVENT_BEFORE, self::XML_CONTEXT_LOGIN, $xmlVariables)) {
                throw new Exception(ronText('SkoleniChybaXMLudalosti'));
            }

            $record->insertRecord();
        }
        
        if (!$record->executeXmlEvents(self::XML_EVENT_AFTER, self::XML_CONTEXT_LOGIN, $xmlVariables)) {
            throw new Exception(ronText('SkoleniChybaXMLudalosti'));
        }

        if ($uspesnaRegistrace) {
            MessageManager::addInfoMessage(sprintf(ronText('Skoleni_uspesna_registrace'), $pp->jmenoPrijmeniKratke, $skoleni ->popis));
        } else {
            MessageManager::addInfoMessage(sprintf(ronText('SkoleniUspesnePrihlaseni'), $pp->jmenoPrijmeniKratke, $skoleni ->popis));
        }

        if ($spustitAgenta) {
            $this->doplnitPracovniPomer($pp->rc);
        }

        return $pp ->rc;
    }


    /**
     * Provede odhlaseni osoby ze skoleni (smaze zaznam, nebo zrusi prirazeni skoleni)
     * @param string $cisloSkoleni kod skoleni
     * @param string $osobaIDPP rc osoby
     * @param int $datumPovinnosti
     * @throws Exception
     */
    private function odhlaseniSkoleni($cisloSkoleni, $osobaIDPP, $datumPovinnosti)
    {
        $skoleni = Skoleni::nactiSkoleni($this ->database, $cisloSkoleni);

        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $osobaIDPP);

        //$json = isset($_GET['json']);

        $prava = Prava::vratPrava('skoleni_prihlasit');
        $pravaPotvrditPrihlaseni = Prava::vratPrava('skoleni_schvalit_prihlaseni');

        if (!$prava->lzeZobrazitRC($pp->rc)) {
            throw new NeniPravoException();
        }

        $smazat = isset($_GET['navrh']) && ($_GET['navrh'] == 'smazat');
        //pokud je v nastaveni zvoleno ponechat navrh, nikdy se nebude mazat
        if (RonPortalSettings::getIntValue(self::NASTAVENI_PONECHAT_NAVRH)) {
            $smazat = false;
        }

        //if (!$this->jeOsobaNaSkoleni($cisloSkoleni, $pp->rc)) {
        if (!SkoleniUtils::jeOsobaNaSkoleni($this ->database, $cisloSkoleni, $pp->rc)) {
            throw new Exception(sprintf(ronText('SkoleniNeniPrihlasene'), $pp->jmenoPrijmeniKratke, $skoleni ->popis));
        }

        $xmlVariables = [
            'osobaRC' => $pp->rc,
            'osobaOsobniCislo' => $pp->osobniCislo,
            'osobaDatumNastupu' => $pp->datumNastupu
        ];

//        if (!RonPortalSession::vsechnyOsoby()) {
//            //pro pouziti v XML k identifikace vybraneho PP, napr. pro zapis do PK
//            $xmlVariables = ['osobaRC' => RonPortalSession::osobaRC(),
//                'osobaOsobniCislo' => RonPortalSession::osobaOsobniCislo(),
//                'osobaDatumNastupu' => RonPortalSession::osobaNastup()];
//        }

        $record = new RonPortalDBRecord($this->database, 'GenerovaneSkoleni', $this);

        $record->addStringField('RC', true);
        $record->addLogText('  ' . $pp ->jmenoPrijmeni);
        $record->addStringField('DruhSkoleni', true, 10);
        $record->addTimestampField('DatumVznikuPovinnosti', true);

        $record->addStringField('CisloSkoleni', false, 40, null);
        $record->addDateField('DatumSkoleni', false, null);
        $record->addDateField('DatumPrihlasky', false, null);
        $record->addStringField('Prihlasil', false, 25, null);

        $record->addIntField('DuvodSkoleni', true);
        $record->addIntField('Potvrzeni', true);
        $record->addIntField('SchvaleniPrihlasky', true);

        $params = $this->database->createParameters();

        $record->setWhere(
            "RC = " . $params->addParameter($pp->rc) .
            " AND CisloSkoleni = " . $params->addParameter($cisloSkoleni) .
            ($datumPovinnosti > 0
                ? " AND DatumVznikuPovinnosti = " . $params->addTimestampParameter($datumPovinnosti)
                : ''),
            $params);
        
        if (!$record->loadRecord()) {
            throw new Exception(ronText('SkoleniChybaNacteniZaznamu'));
        }
        
        if (!$record->executeXmlEvents(self::XML_EVENT_BEFORE, $smazat ? 'delete' : self::XML_CONTEXT_LOGOUT, $xmlVariables)) {
            throw new Exception(ronText('SkoleniChybaXMLudalosti'));
        }

        if (!$smazat) {
            //skoleni se nemaze, jen se zrusi prirazene skoleni
            //prihlaska jiz byla schvalena, odhlaseni muze provest jen uzivatel s pravy pro schvaleni prihlasky, pri odhlaseni se zrusi zaroven schvaleni prihlasky
            if ($record->findFieldByName('SchvaleniPrihlasky')->value == 1) {
                if (!$pravaPotvrditPrihlaseni->lzeZobrazitRC($pp->rc))
                    throw new NeniPravoException();

                    $record->findFieldByName('SchvaleniPrihlasky')->setNewValue(0);

                //pro doporucene (individualni duvod skoleni - teoreticky i rucne vlozeny zaznam) se nastavi potvrzeni na 0
                //15.12.2023 RON-PaRu stav potvrzeni se nemeni, zustava potvrzeni = 1, stejne jako v aplikaci
//                if ($record->findFieldByName('DuvodSkoleni')->value == 1)
//                    $record->findFieldByName('Potvrzeni')->setNewValue(0);
            }

            $record->updateRecord();
        } else {
            //smaze se zaznam generovaneho skoleni
            $record->doDelete('');
        }
        
        if (!$record->executeXmlEvents(self::XML_EVENT_AFTER, $smazat ? 'delete' : self::XML_CONTEXT_LOGOUT, $xmlVariables)) {
            throw new Exception(ronText('SkoleniChybaXMLudalosti'));
        }

//        if ($json) {
//            return new JsonOutputObject(['text' => StringUtils::jsonText(sprintf(ronText('SkoleniUspesneOdhlaseni'), $pp->jmenoPrijmeniKratke, $skoleni ->popis))]);
//        }

        MessageManager::addInfoMessage(sprintf(ronText('SkoleniUspesneOdhlaseni'), $pp->jmenoPrijmeniKratke, $skoleni ->popis));
    }


    /**
     * provede potvrzeni(zamitnuti) prihlaseni osoby na skoleni (nastavi schvaleniprihlasky na hodnotu)
     * @param string $cisloSkoleni kod skoleni
     * @param string $osobaIDPP rc osoby
     * @param int $datumPovinnosti
     * @param int $hodnota - hodnota schvaleni: 1 = schvaleno, 2 = vraceno
     * @throws Exception
     */
    private function nastavSchvaleniPrihlaseni($cisloSkoleni, $osobaIDPP, $datumPovinnosti, $hodnota = 1)
    {
        Csrf::validateCSRFGetParam();

        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $osobaIDPP);

        $skoleni = Skoleni::nactiSkoleni($this ->database, $cisloSkoleni);

        if (!SkoleniUtils::jeOsobaNaSkoleni($this ->database, $cisloSkoleni, $pp->rc)) {
            throw new Exception(sprintf(ronText('SkoleniNeniPrihlasene'), $pp->jmenoPrijmeniKratke, $skoleni ->popis));
        }

        $prava = Prava::vratPrava('skoleni_schvalit_prihlaseni');

        if (!$prava->lzeZobrazitRC($pp->rc)) {
            throw new NeniPravoException();
        }

        $record = new GenerovaneSkoleniRecord($this ->database, $this);

        $params = $this->database->createParameters();
        $record->setWhere(
            'RC = ' . $params->addParameter($pp->rc) .
            ' AND CisloSkoleni = ' . $params->addParameter($cisloSkoleni) . // POZOR! TOTO NENI PRIMARNI KLIC! Predelat?
            ' AND DatumVznikuPovinnosti = ' . $params->addTimestampParameter($datumPovinnosti),
            $params);

        if (!$record->loadRecord()) {
            throw new Exception(ronText('SkoleniChybaNacteniZaznamu'));
        }

        $record->fVyhodnotil->setNewValue(User::getLoginName());
        $record->fDatumVyhodnoceni->setNewValue(DateUtils::today());
        $record->fSchvaleniPrihlasky->setNewValue($hodnota);

        //pro doporucene nastavi pri schvaleni potvrzeni na 1, pri zamitnuti necha 0
        if ((!$record->fPotvrzeni->value)) {
            if ($hodnota == 1) {
                $record->fPotvrzeni->setNewValue(1);
                MessageManager::addInfoMessage(sprintf(ronText('Skoleni_uspesne_schvaleni'), $pp->jmenoPrijmeniKratke, $skoleni ->popis));
            } else {
                $record->fPotvrzeni->setNewValue(0);
                MessageManager::addInfoMessage(sprintf(ronText('Skoleni_uspesne_zamitnuti'), $pp->jmenoPrijmeniKratke, $skoleni ->popis));
            }
        } elseif ($hodnota == 2) { //zamitnuti schvaleni zrusi u povinneho skoleni vazbu na kurz (cislo skoleni)
            $record->fCisloSkoleni->setNewValue(null);
            $record->fDatumSkoleni->setNewValue(null);
            $record->fDatumPrihlasky->setNewValue(null);
            $record->fPrihlasil->setNewValue(null);
        }

        $record->updateRecord();
    }

    /**
     * Zobrazi prehled skoleni s filtrem na osobu
     * @return ViewObject
     * @throws DBException
     * @throws Exception
     */
    private function prehledSkoleni()
    {
        $this->hlavickaVyberAkce();

        $browser = new SkoleniBrowser($this->database);
        $browser->load();

        return $browser;
    }

    /** Zobrazi/zpracuje formular absolvovani skoleni.
     * @return Redirect|AbsolvovaniSkoleniForm
     * @throws DBException
     * @throws Exception
     */
    private function absolvovaniSkoleniFormular()
    {
        $this->hlavickaVyberAkce();

        $osoba = isset($_POST['osoba'])
            ? (int)$_POST['osoba']
            : (isset($_GET['osoba']) ? (int)$_GET['osoba'] : null);

        $druhSkoleni = $_POST['DruhSkoleni'] ?? ($_GET['druhskoleni'] ?? null);

        $datumPovinnosti = isset($_POST['DatumPovinnosti'])
            ? (int)$_POST['DatumPovinnosti']
            : (isset($_GET['datumpovinnosti']) ? (int)$_GET['datumpovinnosti'] : null);

        $pp = $osoba
            ? PracovniPomer::nactiPP($this->database, $osoba)
            : null;

//        //Prava::nastavObdobiProPrava(isset($datumPovinnosti) ? $datumPovinnosti : DateUtils::now());
//        $prava = Prava::vratPrava('skoleni_absolvovani');
//
//        if ($formaction === 'new') {
//            //pri zobrazeni formulare pro vlozeni musi mit nejake prava
//            if (!$prava->jsouNejakePrava()) {
//                throw new NeniPravoException();
//            }
//        }
//        else {
//            //pri vkladani vybrane osoby, editace zaznamu s osobou musi mit pravo na konkretni osobu
//            if (!$prava->lzeZobrazitRC($pp->rc, DateUtils::now(), DateUtils::now())) {
//                throw new NeniPravoException();
//            }
//        }

        $form = new AbsolvovaniSkoleniForm($this->database, $pp, $druhSkoleni, $datumPovinnosti);

        if ($form->handleAction()) {
            return new Redirect(null, '?menuid=skoleni');
        }
        
        return $form;
    }

    /**
     * Menu skoleni
     */
    private function hlavickaVyberAkce()
    {
        $sekce = [];

        if (Prava::vratHodnotuPrava('skoleni_prohlizeni')) {
            $sekce['prehled'] = ronText('Skoleni_prehled');
        }

        if (Prava::vratHodnotuPrava('skoleni_vyslat')) {
            $sekce['vyslatnaskoleni'] = ronText('Skoleni_vyslat_na_skoleni');
        }

        if (Prava::vratHodnotuPrava('skoleni_absolvovani')) {
            $sekce['zapisabsolvovani'] = ronText('Skoleni_zapsat_absolvovani');
        }

        if (Prava::vratHodnotuPrava('skoleni_prihlasit')) {
            $sekce['zobrazprihlaseni'] = ronText('MI_Prihlaseni_na_skoleni');
        }

        if (Prava::vratHodnotuPrava('skoleni_prihlasit')) {
            $sekce['skolenikurzy'] = ronText('MI_Skoleni_a_kurzy');
            $sekce['prihlaseninaplanovaneskoleni'] = ronText('MI_prihlasovani_na_planovane_skoleni');
        }

        if (Prava::vratHodnotuPrava('skoleni_plan') != 0) {
            $sekce['planovaneskolenikurzy'] = ronText('MI_Planovane_Skoleni_Kurzy');
        }

//        if (Prava::vratHodnotuPrava('skoleni_plan') != 0) {
//            $sekce['vytvoreniskolenikurzu'] = ronText('MI_Zadat_novy_plan_kurzu');
//        }

        $aktivni_sekce = ((isset($_GET['action']) && $_GET['action'])
            ? $_GET['action']
            : (isset($_GET['formaction']) ? 'zapisabsolvovani' : ''));

        Model::addData('header',  ronText('Skoleni'));
        Model::addData('aktivni_sekce', $aktivni_sekce);
        Model::addData('sekce', $sekce);
        Model::addData('url', '?menuid=skoleni&action=');
        Model::addData('wrap_class', 'skoleni');
        Model::addViewObject(new BaseViewObject('view/appsections.php'));
    }

    /**
     * Zobrazi/zpracuje hodnoceni skoleni.
     * @return Redirect|ViewObject
     * @throws Exception
     */
    private function hodnoceniSkoleni()
    {
        HtmlHeaders::getInstance() ->addCss('css/dotaznik.css');
        HtmlHeaders::getInstance() ->addScript('script/hodnoceni.js');

        $id = (int)$_GET['record'];

        if (isset($_POST['hodnoceniaction'])  &&  $_POST['hodnoceniaction']) {
            // odeslani formulare - ulozeni hodnoceni nebo uzavreni
            switch ($_POST['hodnoceniaction']) {
                case 'ulozdotaznik' :
                    return $this ->ulozHodnoceni();

                case'uzavridotaznik' :
                    return $this ->uzavriHodnoceni($id);

                default :
                    throw new Exception('Chyba v metode SkoleniController::handleAction - neznamy typ akce!');
            }
        }

        $hodnoceniAction = $_GET['hodnoceniaction'] ?? '';

        return $this ->pripravenyDotaznik($id, $hodnoceniAction == 'editujhodnoceni', false);
    }

    /**
     * Zkontroluje, jestli ma uzivatel pravo na tento dotaznik
     * - jestli je pro zamestnance prihlaseneho na skoleni
     * - jestli je pro jeho vedouciho RCHodnoticiOsoby nebo je uzivatel na primo nadrizenem rozdeleni
     * - vraci 0 nebo false - nema pravo,  1 - muze prohlizet,  2 - muze editovat
     * @param Database $database
     * @param int  $id
     * @param bool $editace  Pokud je $editace = true a zjisti, ze nemuze editovat, tak rovnou vraci 0 a nekontroluje, jestli lze i cist
     * @return int
     * @throws Exception
     */
    public static function vratPravoNaHodnoceni($database, $id, $editace)
    {
        // zjisti stav, uroven predavani a rozdeleni osoby
        $sql =
            'SELECT PripraveneTemaHodnoceni.Stav, PripraveneTemaHodnoceni.HodnotilRC, ' . // 0, 1
            '    PripraveneTemaHodnoceni.HodnocenyRC, HodnoceniSkoleni.TypGenerovani, ' . // 2, 3
            '    HodnoceniSkoleni.RCHodnoticiOsoby, HodnoceniSkoleni.TypRozdeleni ' . // 4, 5
            ' FROM PripraveneTemaHodnoceni JOIN HodnoceniSkoleni ON ((PripraveneTemaHodnoceni.DruhHodnoceni = HodnoceniSkoleni.DruhHodnoceni) ' .
            '           AND (PripraveneTemaHodnoceni.TemaCislo = HodnoceniSkoleni.TemaCislo))' .
            ' WHERE PripraveneTemaHodnoceni.Id = ' . (int)$id ;

        MessageManager::addDebugLog($sql);

        $cursor = $database ->getSqlResult($sql);
        $row = $cursor ->fetch();
        $cursor ->close();
        if (!$row) {
            throw new Exception("Chyba - zadany dotaznik (cislo $id) nebyl nalezen nebo neni urcen pro hodnoceni skoleni!");
        }

        $result = 2; // lze editovat

        // pokud se ma editovat, zkontroluje, jestli neni uzavren
        if ($row[0] == 2 || $row[0] == 3) {
            // dotaznik je uzavren
            if ($editace) {
                return MessageManager::addErrorMessage('Chyba - dotaznik nelze editovat, protoze je jiz uzavren!');
            }

            $result = 1; // nelze editovat - jeste bude kontrolovat jesli lze cist
        }

        if ($row[1] == User::getRC()) return $result;
        // dotaznik vyplnil aktualni uzivatel, takze ma na jeho zobrazeni pravo
        // uz neni potreba kontrolovat stromovou strukturu

        switch ($row[3]) {
            case 1 : //dle typu rozdeleni
                $kodtypurozdeleni = $row[5];
                $obdobiod = DateUtils::today();
                $obdobido = DateUtils::today();

                $params = $database->createParameters();

                $sql =
                    'SELECT * ' . // kontrola, ze jsem (byl) v hodnocenem obdobi v primo nadrazenem rozdeleni
                    '  FROM RozdeleniOsob R_JA JOIN Rozdeleni R_Nad ON (R_Nad.RozdeleniRodice = R_Ja.KodTypuRozdeleni AND R_Nad.Rodic = R_Ja.Rozdeleni) ' .
                    '       JOIN RozdeleniOsob R_Hodnoceny ON (R_Hodnoceny.KodTypuRozdeleni = R_Nad.KodTypuRozdeleni AND R_Hodnoceny.Rozdeleni = R_Nad.Kod) ' .
                    "  WHERE R_Ja.RC = " . $params->addParameter(User::getRC()) .
                    '    AND R_JA.KodTypuRozdeleni = ' . $params->addParameter($kodtypurozdeleni) .
                    "    AND R_Hodnoceny.RC = " . $params->addParameter($row[2]) .
                    '    AND R_Ja.PlatnostOd <= ' . $params->addParameter($database->timestampToParam($obdobido)) .
                    '    AND (R_Ja.PlatnostDo >= ' . $params->addParameter($database->timestampToParam($obdobiod)) . '  OR  R_Ja.PlatnostDo IS NULL)' .
                    '    AND R_Hodnoceny.PlatnostOd <= ' . $params->addParameter($database->timestampToParam($obdobido)) .
                    '    AND (R_Hodnoceny.PlatnostDo >= ' . $params->addParameter($database->timestampToParam($obdobiod)) . '  OR  R_Hodnoceny.PlatnostDo IS NULL)';

                $cursor = $database->getSqlResult($sql, $params);

                $row = $cursor->fetch();
                if (!$row) {
                    return MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba - na zobrazeni zvoleneho dotazniku nemate pravo! Hodnocena osoba patri do jineho rozdeleni.'));
                }
                break;
            case 2 : //dle osoby
                if ($row[4] != User::getRC()) {
                    return MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba - na zvolene hodnoceni nemate pravo, nejste osobou urcenou k hodnoceni!'));
                }
                break;
            case 3 : //generovat - skoleni je vygenerovano pro ucastnika
                if ($row[2] != User::getRC()) {
                    return MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba - na zvolene hodnoceni nemate pravo!'));
                }
                break;
        }

        // vse je OK
        return $result; // nastaveno drive na 1 (jen cist) nebo 2 (i editace)
    }

    /**
     * @param int  $id
     * @param bool $editace
     * @return HodnoceniSkoleni
     * @throws Exception
     */
    function nactiHodnoceni($id, $editace)
    {
        $hodnoceni = new HodnoceniSkoleni($this ->database, $id);

        $pravo = self::vratPravoNaHodnoceni($this ->database, $id, $editace);
        $hodnoceni ->fPravo = $pravo;

        if (!$pravo  ||  ($editace  &&  $pravo < 2)) {
            throw new NeniPravoException();
        }

        return $hodnoceni;
    }

    /**
     * @param int  $id
     * @param bool $editace
     * @param bool $postValues
     * @return ViewObject
     * @throws Exception
     * @throws DBException
     */
    function pripravenyDotaznik($id, $editace, $postValues)
    {
        if (!$id) {
            throw new Exception('Chyba! Neni zadan Id dotazniku!');
        }

        $dotaznik = $this ->nactiHodnoceni($id, $editace);

        // kontrola, jestli se needituje uzavreny dotaznik
        if ($editace  &&  !$dotaznik ->jeEditovatelnyStav()) { // ukonceno, ukonceno-vraceno
            MessageManager::addErrorMessage( ronText('Nelze_Editovat_uzavrene_hodnoceni') );
            $editace = false;
        }

        // Nastavi hodnotitele dotazniku.
        if ($editace  &&  !$postValues) {
            if (!$dotaznik->nastavHodnotitele()) {
                return new MessageManagerViewObject(); // zobrazi chybove hlasky
            }
        }

        return new HodnoceniSkoleniViewObject($dotaznik, $editace, $postValues);
    }

    /** Provede ulozeni hodnoceni
     * @return Redirect|ViewObject
     * @throws Exception
     */
    function ulozHodnoceni()
    {
        $id = (int)$_GET['record'];
        if (!$id) {
            throw new Exception('Chyba! Neni zadan Id dotazniku!');
        }

        $dotaznik = $this ->nactiHodnoceni($id, true); // editace = true

        $dotaznik->fUlozenoVXML = null;
        $dotaznik->fPrepoctenoVXML = null;

        if ($dotaznik->fXMLSablona) {
            $dotaznik->vyplnDotaznikXML();
        }

        // pokud nebyl dotaznik ulozen uz v XML, nebo nema XML, tak jej ulozi nyni
        if (isset($dotaznik->fUlozenoVXML)) {
            MessageManager::addDebugMessage('Ulozeni odpovedi probehlo primo v XML formulari');
            $ulozeno_result = $dotaznik->fUlozenoVXML;
        } else {
            if ($dotaznik->fXMLSablona) {
                MessageManager::addDebugMessage('Ulozeni odpovedi neprobehlo v XML -> probehne nyni');
            }

            try {
                $ulozeno_result = $dotaznik->ulozVyplnenyDotaznik();
            } catch (DBException $e) {
                throw $e;
            } catch (RonException $e) {
                MessageManager::addErrorMessage($e ->getMessage(), $e ->fieldName, null, $e ->getErrorDescription());
                $ulozeno_result = false;
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e ->getMessage());
                $ulozeno_result = false;
            }
        }

        if ($ulozeno_result) {
            MessageManager::addInfoMessage(ronText('Hodnoceni_ulozeno'));
        } else {
            return new HodnoceniSkoleniViewObject($dotaznik, true, true);
        }

        // test, jestli se ma zaroven s ulozenim vratit nebo uzavrit (stisknuto tlacitko "uzavrit")
        if (isset($_POST['uzavrit']) && $_POST['uzavrit']) {
            if ($dotaznik->fVyplnenoVse) {
                return $this ->uzavriHodnoceni($id);
            }

            if (!$dotaznik->fVyplnenoVse) {
                MessageManager::addErrorMessage( ronText('Nelze_uzavrit_nevyplnene_hodnoceni') );
            }

            // pouzit redirect ? ? ?
            return $this ->pripravenyDotaznik($id, true, false); // editace = true, postvalues = false (uz je ulozeno)

        } else {
            // jen ulozeni - zobrazi v readonly
            return new Redirect(null, '?menuid=skoleni&action=hodnoceni&record=' . $id);
        }
    }

    /**
     * Uzavreni dotazniku (kliknuti na tlacitko Odeslat nebo Vratit k doplneni)
     * @param int $id
     * @return Redirect
     * @throws DBException
     * @throws Exception
     */
    public function uzavriHodnoceni($id)
    {
        if (!$id) {
            throw new Exception('Chyba! Neni zadan Id dotazniku!');
        }

        MessageManager::addDebugMessage("HodnoceniController::uzavriDotaznik id=$id ");

        $params = $this->database->createParameters();

        // zapise jmeno hodnoticiho a cas
        $sql =
            'UPDATE PripraveneTemaHodnoceni ' .
            "   SET HodnotilRC = " . $params ->addParameter(User::getRC()) . ", " .
            "       Hodnotici = " . $params ->addParameter(RonPortalUtils::getLogUserName()) . ", " .
            '       DatumHodnoceni = ' . $params->addParameter($this ->database ->timestampToParam(time())) . ', ' .
            '       Stav = 2 '  . //2 - hodnoceni ukonceno
            ' WHERE Id = ' . $params->addParameter($id);

        $this ->database ->sql($sql, $params);
        return new Redirect(null, "?menuid=skoleni");
    }


    /**
     * @return UcastNaSkoleniBrowser
     * @throws Exception
     */
    private function zobrazUcastSkoleni()
    {
        HtmlHeaders::getInstance()->addScript('script/skoleni.js');

        $b = new UcastNaSkoleniBrowser($this->database);
        $b->load();
        return $b;
    }


    /**
     * @param $kodKurzu
     * @return HodnoceniUcastnikuBrowser|Redirect
     * @throws Exception
     */
    private function zobrazHodnoceniUcastniku($kodKurzu)
    {
        if (!Prava::jsouNejakePrava('skoleni_absolvovani')) {
            throw new NeniPravoException();
        }

        $hodnoceniBrowser = new HodnoceniUcastnikuBrowser($this ->database, $kodKurzu);
        if (isset($_POST['HodnoceniUcastniku'])  &&  $_POST['HodnoceniUcastniku']) {
            try {
                $hodnoceniBrowser->ulozZmenyHodnoceni();
                return new Redirect(null, isset($_POST['BackButton'])
                    ?  '?menuid=skoleni&action=skolenikurzy'
                    : '?menuid=skoleni&action=skolenihodnoceni&record=' . $kodKurzu);
            } catch (DBException $e) {
                throw $e;
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e);
            }
        } else {
            $hodnoceniBrowser->load();
        }
        return $hodnoceniBrowser;
    }


    /**
     * @return SkoleniAKurzyBrowser
     * @throws Exception
     */
    private function zobrazSkoleniKurzy()
    {
        $this->hlavickaVyberAkce();

        // prava "skoleni_lektor" kontroluje browser
        $b = new SkoleniAKurzyBrowser($this->database);
        $b->load();
        return $b;
    }

    /**
     * @return ViewObject
     * @throws Exception
     */
    private function zobrazPlanovaneSkoleni()
    {
        $prava = Prava::vratPrava('skoleni_plan');

        if (!$prava->jsouNejakePrava()){
            throw new NeniPravoException();
        }

        $this->hlavickaVyberAkce();

        if (isset($_GET['formaction'])) {
            return $this ->detailPlanovaneSkoleni();
        }

        $b = new PlanovaneSkoleniBrowser($this->database);
        $b->load();
        return $b;
    }


    /**
     * @return ViewObject|Redirect
     * @throws Exception
     */
    public function detailPlanovaneSkoleni() // $cisloSkoleni, $nactenyPost = null)
    {
        $prava = Prava::vratPrava('skoleni_plan');
        if (!$prava ->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        $form = new PlanovaneSkoleniForm($this ->database);
        if ($form ->handleAction()) {
            return new Redirect(null, '?menuid=skoleni&action=planovaneskolenikurzy');
        }

        return $form;
    }


//    /**
//     * @param $hodnoty
//     * @throws DBException
//     * @throws Exception
//     */
//    public function pripojeniAutoraPlanovanehoKurzu($hodnoty)
//    {
//        //pripojit User na skoleni - jako autora a pokud ma pravo na prihlaseni dane osoby tak nastavit potvrzeni na 1
//        $pravoPrihlasit = Prava::vratPrava('skoleni_prihlasit');
//
//        $potvrzeni = 0;
//        if ($pravoPrihlasit->lzeZobrazitRC(User::getRC())){
//            $potvrzeni = 1;
//        }
//
//        $recZapis = new RonPortalDBRecord($this->database, 'PlanovaneSkoleniOsob', $this);
//
//        $recZapis->addField('CisloSkoleni', '', false, 10, $hodnoty->cisloSkoleni);
//        $recZapis->addField('RC', '', false, 15, User::getRC());
//        $recZapis->addField('Potvrzeni', 'int', false, 0, $potvrzeni);
//        $recZapis->addField('Vlozil', '', false, 25, User::getRC());
//        $recZapis->addField('DatumVlozeni', 'timestamp', false, 0, time());
//        $recZapis->addField('Vyhodnotil', '', false, 25, '');
//        $recZapis->addField('DatumVyhodnoceni', 'timestamp', false, 0, '');
//        $recZapis->addField('PrihlasenoZAplikace', 'int', false, 0, 1);
//
//        $params = $this->database->createParameters();
//        $recZapis->setWhere(
//            'RC  = ' . $params->addParameter(User::getRC()) .
//            ' AND CisloSkoleni = ' . $params->addParameter($hodnoty->cisloSkoleni), $params);
//
//        if (!$recZapis->loadRecord()) {
//            $recZapis->insertRecord();
//            MessageManager::addInfoMessage(StringUtils::fromUTF8('Byl/a jste na planovane skoleni prihlasen/a.'));
//        } else {
//            MessageManager::addInfoMessage(StringUtils::fromUTF8('Jiz jste na planovane skoleni prihlasen/a.'));
//        }
//    }

    /**
     * Prehled skoleni pro dashboard
     * @param JsonOutputObject $output
     * @throws DBException
     */
    private function mojeSkoleni($output)
    {
        $params = $this ->database ->createParameters();
        $sql =
            "SELECT DruhSkoleni.Kod, COALESCE(Skoleni.DatumPocatkuSkoleni, GS.PozadovaneDatumSkoleni), 
                    DruhSkoleni.Popis, Skoleni.DatumPocatkuSkoleni, GS.PozadovaneDatumSkoleni
			, CASE WHEN Skoleni.DatumPocatkuSkoleni IS NOT NULL THEN 1 ELSE 0 END
			, Skoleni.Popis
                FROM GenerovaneSkoleni GS 
                    LEFT JOIN DruhSkoleni ON (DruhSkoleni.Kod = GS.DruhSkoleni) 
                    LEFT JOIN Skoleni ON  (Skoleni.CisloSkoleni = GS.CisloSkoleni) 
                WHERE  (GS.RC= " . $params->addParameter(User::getRC()) .
            "  AND (GS.Potvrzeni=0 OR  (GS.Potvrzeni = 1  AND (Skoleni.StavSkoleni IS NULL  OR  Skoleni.StavSkoleni <> 2)) 
                OR GS.Potvrzeni=5 OR GS.Potvrzeni=6)) " .
            ' ORDER BY COALESCE(Skoleni.DatumPocatkuSkoleni, GS.PozadovaneDatumSkoleni)';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $html =
            '<table class="dashboardTable w-100 oddEvenRows" data-ron-href="?menuid=skoleni">';

        $html .=
            '<tr><th>' . ronText('Skoleni_termin_platnost') . '&nbsp;&nbsp;&nbsp;' .
            '</th><th>' . ronText('Skoleni_nazev_skoleni') .
            '</th><th>' . ronText('Skoleni_termin_povinnosti') . 
            '</th><th>' . ronText('Skoleni_prihlasen_na_kurz') . 
            '</th><th>' . ronText('Skoleni_popis_kurzu') . 
	    '</th><th></th></tr>';


        while ($row = $cursor ->fetch()) {
            MessageManager::addDebugMessage($row[3]);
            $html .=
                '<tr>' .
                '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[1])) . '</td>' .
                '<td>' . StringUtils::htmlTextConvert($row[2]) . ' - <strong>' . StringUtils::htmlTextConvert($row[0]) . '</strong></td>' .
                '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[4])) . '</td>' .
                '<td>' . StringUtils::format('anone', $row[5]) . '</td>' .
                '<td>' . StringUtils::htmlTextConvert($row[6]) . '</td>' .
                '</tr>';
        }

        $html .= '</table>';

        $cursor ->close();

        $output ->addText('html', $html);
        $output->addDebugMessage($sql);
    }


    /**
     * Prehled skoleni pro dashboard
     * @param JsonOutputObject $output
     * @throws Exception
     */
    private function skoleniPodrizenych($output)
    {
        $pravo = Prava::vratPrava('skoleni_prohlizeni');

        $params = $this->database->createParameters();
        $html =
            '<table class="dashboardTable w-100 oddEvenRows" data-ron-href="?menuid=skoleni">' .
            '<tr><th>' . ronText('Zamestnanec') . '</th>' .
            '<th>' . ronText('Skoleni_termin_platnost') . '&nbsp;&nbsp;' . '</th>' .
            '<th>' . ronText('Druh_skoleni') . '</th>' .
            '<th class="align-right">' . ronText('Skoleni_termin_povinnosti') . '</th>' .
            '</tr>';

        $sql =
            "SELECT DruhSkoleni.Kod, COALESCE(Skoleni.DatumPocatkuSkoleni, GenerovaneSkoleni.PozadovaneDatumSkoleni), 
                    DruhSkoleni.Popis, Osoba.Prijmeni, Osoba.Jmeno, Skoleni.DatumPocatkuSkoleni, GenerovaneSkoleni.PozadovaneDatumSkoleni 
                FROM (  (  ( GenerovaneSkoleni INNER JOIN Osoba ON (Osoba.RC = GenerovaneSkoleni.RC) INNER JOIN DruhSkoleni ON GenerovaneSkoleni.DruhSkoleni=DruhSkoleni.Kod) 
                INNER JOIN S_DuvodSkoleni ON GenerovaneSkoleni.DuvodSkoleni=S_DuvodSkoleni.Kod) 
                INNER JOIN S_StavPotvrzeni ON GenerovaneSkoleni.Potvrzeni=S_StavPotvrzeni.Kod) 
                LEFT OUTER JOIN Skoleni ON Skoleni.CisloSkoleni = GenerovaneSkoleni.CisloSkoleni 
                LEFT OUTER JOIN KategorieSkoleni ON DruhSkoleni.Kategorie=KategorieSkoleni.Kod 
                WHERE  (GenerovaneSkoleni.RC = Osoba.RC  AND (GenerovaneSkoleni.Potvrzeni=0 OR  (GenerovaneSkoleni.Potvrzeni = 1  AND (Skoleni.StavSkoleni IS NULL  OR  Skoleni.StavSkoleni <> 2)) )
                OR GenerovaneSkoleni.Potvrzeni=5 OR GenerovaneSkoleni.Potvrzeni=6) " .


        '  AND EXISTS(SELECT PracovniPomer.RC FROM PracovniPomer ' .
            '             WHERE PracovniPomer.RC = GenerovaneSkoleni.RC ' .
            '               AND PracovniPomer.DatumNastupu <= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) .
            '               AND (PracovniPomer.DatumUkonceni >= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
            '               AND ' .  $pravo->sqlFiltr('PracovniPomer', DateUtils::today(), DateUtils::today()) .
            '            )' .

            '   AND GenerovaneSkoleni.RC <> ' . $params->addParameter(User::getRC()) .

            ' ORDER BY COALESCE(Skoleni.DatumPocatkuSkoleni, GenerovaneSkoleni.PozadovaneDatumSkoleni)';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $html .=
                '<tr>' .
                '<td>' . StringUtils::htmlTextConvert($row[3]) . ' ' . StringUtils::htmlTextConvert($row[4]) . '</td>' .
                '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[1])) . '</td>' .
                '<td>' . StringUtils::htmlTextConvert($row[2]) . ' - <strong>' . StringUtils::htmlTextConvert($row[0]) . '</strong></td>' .
                '<td class="align-right">' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[6])) . '</td>' .
                '</tr>';
        }
        $cursor ->close();

        $html .= '</table>';

        $output ->addText('html', $html);
    }


    /**
     * @return ViewObject|Redirect
     * @throws DBException
     * @throws Exception
     */
    public function vyslatNaSkoleni()
    {
        // pripoji js pro nacteni volne kapacity kurzu
        HtmlHeaders::getInstance()->addScript('script/personscripts.js');

        $this->hlavickaVyberAkce();

        if (!Prava::vratHodnotuPrava('skoleni_vyslat')) {
            throw new NeniPravoException();
        }

        $form = new PozadavekNaSkoleniForm($this ->database);
        if ($form ->handleAction()) {
            return new Redirect(NULL, '?menuid=skoleni&action=vyslatnaskoleni');
        }
        return $form;
    }


    /**
     * @throws Exception
     */
    private function hromadnePrihlasitOsobyNaSkoleni()
    {
        $seznamRC = [];

        if(isset($_POST['prihlasitNaSkoleni'])) {
            foreach ($_POST['prihlasitNaSkoleni'] as $item => $radekKurzu) {
                $osoba = $radekKurzu['osoba'];
                $skoleni = $radekKurzu['cislo'];
                $datumPovinnosti = (int)$radekKurzu['datumpovinnosti'];

                if ((isset($radekKurzu['prihlaseno']) && $radekKurzu['prihlaseno']) && (!isset($radekKurzu['prihlasit']))) {
                    //Zrusit skoleni
                    try {
                        $this->odhlaseniSkoleni($skoleni, $osoba, $datumPovinnosti);
                    } catch (DBException $e) {
                        throw $e;
                    } catch (Exception $exception) {
                        MessageManager::addErrorMessage($exception);
                    }
                } elseif ((isset($radekKurzu['prihlaseno']) && $radekKurzu['prihlaseno'] == 0) && (isset($radekKurzu['prihlasit']) && $radekKurzu['prihlasit'] == 1)) {
                    //Prihlasit skoleni
                    try {
                        $rc = $this->prihlaseniSkoleni($skoleni, $osoba, $datumPovinnosti, false);
                        if (!in_array($rc, $seznamRC)) {
                            $seznamRC[] = $rc;
                        }
                    } catch (DBException $e) {
                        throw $e;
                    } catch (Exception $exception) {
                        MessageManager::addErrorMessage($exception);
                    }
                }
            }
        } else {
            MessageManager::addInfoMessage(ronText('Skoleni_hromadne_nevybran_uzivatel'));
        }

        $this->doplnitPracovniPomer($seznamRC);

        return new Redirect(null, '?menuid=skoleni&action=zobrazprihlaseni');
    }


    /**
     * @param string $action
     * @param JsonOutputObject $output
     * @throws Exception
     */
    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
//            case 'seznamfirem' :
//                $this ->downloadFirmy($output);
//                break;

            case 'mojeskoleni' :
                $this ->mojeSkoleni($output);
                break;

            case 'skolenipodrizenych' :
                $this ->skoleniPodrizenych($output);
                break;
        }
    }


    /**
     * @param string $action
     * @return BaseViewObject|InternalRedirect|JsonOutputObject|OutputObject|Redirect|ViewObject|string
     * @throws DBException
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_SKOLENI)) {
            throw new NeniPravoException();
        }

        HtmlHeaders::getInstance() ->setAppTitle('Skoleni');

        Prava::nastavObdobiProPrava(time(), time());

        $skoleniAction = $_GET['action'] ?? '';

        //Hromadne prihlaseni / odhlaseni osob na skoleni
        if (isset($_POST['hromadnePrihlasitSkoleni']) && $_POST['hromadnePrihlasitSkoleni'] == 1){
            $this->hromadnePrihlasitOsobyNaSkoleni();
        }

        $redirectAction = null;

        try {
            switch ($skoleniAction) {
                // json akce
                case 'mojeskoleni' :        // dashboard
                case 'skolenipodrizenych' : // dashboard
                    //case 'seznamfirem' :
                    return parent::handleAction($skoleniAction);

                case 'translations':
                    return $this->translations();

                case 'zobrazprihlaseni':
                    return $this->zobrazPrihlaseniSkoleni();

                case 'vyslatnaskoleni' :
                    return $this->vyslatNaSkoleni();

                case 'planovaneskolenikurzy':
                    //Browser a formular planovanych skoleni
                    return $this->zobrazPlanovaneSkoleni();

                case 'skolenikurzy':
                    return $this->zobrazSkoleniKurzy();

                case 'skoleniucast':    // ucast na dlouhodobem kurzu (proklik ze seznamu skolenikurzy)
                    return $this->zobrazUcastSkoleni();

                case 'skolenihodnoceni':    // odkaz "Hodnoceni ucastniku kurzu" z skolenikurzy ("Lektorem spravovane kurzy")
                    return $this->zobrazHodnoceniUcastniku($_GET['record']);

                case 'zapisabsolvovani':
                    return $this->absolvovaniSkoleniFormular();

                case 'prihlaseninaplanovaneskoleni':
                    $this->hlavickaVyberAkce();
                    $prihlaseniPlanController = new PrihlaseniNaPlanovaneSkoleniController($this->database);
                    return $prihlaseniPlanController->handleAction('');

                case 'prihlaseni':  // odkaz ze seznamu "Prihlaseni na kurz"
                    $redirectAction = 'zobrazprihlaseni';
                    Csrf::validateCSRFGetParam();
                    $this->prihlaseniSkoleni($_GET['cisloskoleni'], $_GET['osoba'], (int)$_GET['datumpovinnosti']);
                    return new Redirect(null, '?menuid=skoleni&action=zobrazprihlaseni');

                case 'odhlaseni':   // odkaz ze seznamu "Prihlaseni na kurz"
                    $redirectAction = 'zobrazprihlaseni';
                    Csrf::validateCSRFGetParam();
                    $this->odhlaseniSkoleni($_GET['cisloskoleni'], $_GET['osoba'], (int)$_GET['datumpovinnosti']);
                    return new Redirect(null, '?menuid=skoleni&action=zobrazprihlaseni');

                case 'schvaleniprihlaseni': // odkaz Schvalit ze seznamu "Prihlaseni na kurz"
                    $redirectAction = 'zobrazprihlaseni';
                    $this->nastavSchvaleniPrihlaseni($_GET['cisloskoleni'], $_GET['osoba'], (int)$_GET['datumpovinnosti']);
                    return new Redirect(null, '?menuid=skoleni&action=zobrazprihlaseni');

                case 'zamitnutiprihlaseni':
                    $redirectAction = 'zobrazprihlaseni';
                    $this->nastavSchvaleniPrihlaseni($_GET['cisloskoleni'], $_GET['osoba'], (int)$_GET['datumpovinnosti'], 2);
                    return new Redirect(null, '?menuid=skoleni&action=zobrazprihlaseni');

                case 'hodnoceni':
                    return $this->hodnoceniSkoleni();

                case 'download':
                    return $this->downloadDokumentSkoleni();

                case 'ical' :
                    return $this->downloadIcal();

                case 'prehled':
                default:
                    return $this->prehledSkoleni();
            }
        } catch (DBException $e) {
            throw $e;
        } catch (Exception $e) {
            if ($redirectAction) {
                MessageManager::addErrorMessage($e);
                return new Redirect(null, '?menuid=skoleni&action=' . $redirectAction);
            }
            throw $e;
        }
    }
}