<?php
/**
 * stanislav.elbl, 05.07.2022 15:14
 */

namespace RonPortal\Skoleni;

use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\RonPortalForm;

class TerminSkoleniFormular extends RonPortalForm
{
    /** @var string */
    private $cisloSkoleni;

    /** @var FormField */
    private $fCisloSkoleni;

    /** @var FormField */
    private $fNazevSkoleni;

    /**
     * TerminSkoleniFormular constructor.
     * @param \Ron\DB\Database $database
     * @param string $cisloSkoleni
     */
    public function __construct($database, $cisloSkoleni = null)
    {
        parent::__construct($database, 'new', true); // dialogmode = true

        $this ->cisloSkoleni = $cisloSkoleni;

        $this ->code = 'formterminskoleni';
        $this ->stornoAction = "#";

        $this ->setTitle('editace_terminu_skoleni');

        $this ->htmlParams = array_merge($this ->htmlParams, [
            'id' => 'termin_skoleni_dialog',
            'data-ronform-submit-url' => '?menuid=skolenijs&action=zapistermin',
            'data-ronform-after-submit' => 'reload'
        ]);

        $this ->sqlTableName = 'TerminSkoleni';

        $this ->setXmlEventsName('TERMIN_SKOLENI');

        $this ->fCisloSkoleni = $this ->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('CisloSkoleni') ->setTableName('TerminSkoleni') ->setPrimary(true) ->setReadOnly(true)
            ->addFieldCssClass('ronform-field') ->setDefaultValue($this ->cisloSkoleni)
            ->setCode('field-cisloskoleni')
            ->setOtherHtmlParams([
                'data-ronform-field-name' => 'CisloSkoleni'
            ]);

        if ($this ->cisloSkoleni) {
            $params = $this ->database ->createParameters();
            $nazev = DBUtils::getSqlValue($this ->database,
                'SELECT Popis FROM Skoleni WHERE CisloSkoleni = ' . $params ->addParameter($this ->cisloSkoleni),
                $params);
        } else {
            $nazev = null;
        }

        $this ->fNazevSkoleni = $this ->addField(FormField::FIELD_TYPE_TEXT)
            ->setIncludeToDBRecord(false) ->setCaption('Nazev') ->setReadOnly(true) ->setDefaultValue($nazev)
            ->setFieldName('NazevSkoleni') ->setSqlFieldName('Skoleni.Popis') ->addFieldCssClass('ronform-field font-weight-bold')
            ->setCode('field-nazev-skoleni')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'NazevSkoleni']);

        $this ->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('*DatumOriginal') ->setSqlFieldName('Datum') ->setPrimary(true) ->setDataType('date') ->setFormat('date') ->setIncludeToDBRecord(false)
            ->addFieldCssClass('ronform-field')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'DatumOriginal']);

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('Datum') ->setCaption('Datum')  // ->setReadOnly(true)
            ->addFieldCssClass('ronform-field')
            ->setCode('field-datum')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'Datum']);

        $this ->addField(FormField::FIELD_TYPE_TIME)
            ->setFieldName('Cas') ->setCaption('Cas')
            ->addFieldCssClass('ronform-field')
            ->setCode('field-cas')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'Cas']);

        $this ->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka') ->setCaption('Poznamka')
            ->addFieldCssClass('ronform-field')
            ->setCode('field-poznamka')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'Poznamka']);

        $this ->loadConfiguration();
    }


    public function getSqlFromPart()
    {
        return
            'TerminSkoleni INNER JOIN Skoleni' .
            ' ON Skoleni.CisloSkoleni = TerminSkoleni.CisloSkoleni';
    }


    protected function loadFieldValues($insert, $values, $usePostValues, $canEditRecord)
    {
        parent::loadFieldValues($insert, $values, $usePostValues, $canEditRecord);
    }


    private function existujeZaznam($cisloSkoleni, $datum)
    {
        $params = $this ->database ->createParameters();

        $zaznam = DBUtils::getSqlValue($this ->database,
            'SELECT CisloSkoleni FROM TerminSkoleni' .
            ' WHERE CisloSkoleni = ' . $params ->addParameter($cisloSkoleni) .
            ' AND Datum = ' . $params ->addDateParameter($datum),
            $params);

        return (bool)$zaznam;
    }


    function beforeInsertRecord($rec)
    {
        if ($this ->existujeZaznam($rec['CisloSkoleni'], $rec['Datum'])) {
            throw new \Exception(TextManager::_('termin_je_obsazen'));
        }
        return parent::beforeInsertRecord($rec);
    }


    public function getWhereCondition()
    {
        $params = $this ->database ->createParameters();

        $cisloSkoleni = isset($_POST['CisloSkoleni'])
            ? $_POST['CisloSkoleni']
            : $_GET['cisloskoleni'];

        $datum = isset($_POST['DatumOriginal'])
            ? DateUtils::dateStringToTimestamp($_POST['DatumOriginal'])
            : DateUtils::dateStringToTimestamp($_GET['datum']);

        return new SqlStatement(
            'TerminSkoleni.CisloSkoleni = ' . $params ->addParameter($cisloSkoleni) .
            ' AND Datum = ' . $params ->addDateParameter($datum),
            $params);
    }


    public function setDBRecordWhere($rec)
    {
        $rec ->setWhere($this ->getWhereCondition());
    }


//    // TOTO bylo v byvalem kodu provedeno po vlozeni noveho terminu - podle me je to zbytecnost
//    private function pripojUcastniky()
//    {
//        $params = $this->database->createParameters();
//        $sql = "SELECT GS.RC " .           //0
//            "FROM GenerovaneSkoleni GS " .
//            "JOIN Osoba ON (Osoba.RC = GS.RC) " .
//            "LEFT JOIN TerminSkoleni ON (TerminSkoleni.CisloSkoleni=GS.CisloSkoleni) LEFT JOIN PracovniPomer On (Osoba.RC = PracovniPomer.RC)" .
//            "LEFT JOIN UcastNaTerminuSkoleni UTS ON (UTS.CisloSkoleni=TerminSkoleni.CisloSkoleni AND UTS.RC=GS.RC AND UTS.Datum=TerminSkoleni.Datum) " .
//            "WHERE (GS.CisloSkoleni = " . $params->addParameter($kodKurzu) . ") AND (GS.Potvrzeni<5)  AND TerminSkoleni.Datum = " . $params ->addParameter($this ->database ->timestampToParam($datum)) . " " .
//            " AND PracovniPomer.ID = (SELECT Max(PP.ID) FROM PracovniPomer PP Where PP.RC = GS.RC) " .
//            "ORDER BY RC";
//
//        $cursor = $this->database->getSqlResult($sql, $params);
//
//        while ($row = $cursor->fetch()) {
//
//            $record = new RonPortalDBRecord($this->database, 'UcastNaTerminuSkoleni');
//            $record->logDescription = StringUtils::fromUTF8('Připojení účastníka k termínu');
//
//            $record->addField('CisloSkoleni', '', FALSE, 10, $kodKurzu);
//            $record->addField('Datum', 'timestamp', FALSE, 0, $datum);
//            $record->addField('RC', '', FALSE, 15, $row[0]);
//            $record->addField('Pritomen', 'int', FALSE, 0, 0);
//            $record->addField('DuvodNepritomnosti', 'int', FALSE, 0, 0);
//            $record->addField('Poznamka', 'memo', FALSE, 0, "");
//
//            $record->insertRecord(); //  throw new Exception(ronText('pripojeni_na_termin_selhalo'));
//        }
//    }
}
