<?php
/**
 * stanislav.elbl, 14.10.2014
 * Data pro kompaktni needitovatelne zobrazeni radku sluzebnich cest
 */

namespace RonPortal\SluzebniCesty;


use Ron\DateUtils;
use Ron\DB\Database;
use Ron\StringUtils;
use RonPortal\RonPortalSettings;


class KompaktniRadekSluzebniCesty
{
    /** @var int    Cas prvniho zapocitaneho radku*/
    public $datumCasOd;
    /** @var int    Cas posledniho zapocitaneho radku*/
    public $datumCasDo;

    public $popis = '';

    /** @var int    Celkovy pocet snidani */
    public $snidane = 0;
    /** @var int    Celkovy pocet obedu */
    public $obedy = 0;
    /** @var int    Celkovy pocet veceri */
    public $vecere = 0;

    
    function getDatumStr()
    {
        if ( DateUtils::startOfTheDay($this ->datumCasOd) == DateUtils::startOfTheDay($this ->datumCasDo) ) {
            return date('j.n.Y G:i', $this ->datumCasOd) . ' - ' . date('G:i', $this ->datumCasDo);
        } else {
            return date('j.n.Y G:i', $this ->datumCasOd) . ' - ' . date('j.n.Y G:i', $this ->datumCasDo);
        }
    }


    /**
     * @param Database $database
     * @param SluzebniCesta $sluzebniCesta
     * @returns KompaktniRadekSluzebniCesty[]
     */
    public static function nactiKompaktniRadkySC($database, $sluzebniCesta)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT RadekSluzebniCesty.Stav, RadekSluzebniCesty.DatumCasOd, RadekSluzebniCesty.Popis, '. // 0..2
            ' RadekSluzebniCesty.Stat, RadekSluzebniCesty.Misto, '. // 3,4
            ' RadekSluzebniCesty.PocetPJidel_S, RadekSluzebniCesty.PocetPJidel_O, RadekSluzebniCesty.PocetPJidel_V, '. // 5..7
            ' MistoSluzebniCesty.Popis '. // 8
            '  FROM RadekSluzebniCesty LEFT OUTER JOIN MistoSluzebniCesty '.
            '    ON MistoSluzebniCesty.Kod = RadekSluzebniCesty.Misto '.
            ' WHERE RadekSluzebniCesty.SluzebniCesta_ID = ' . $params ->addParameter((int)$sluzebniCesta ->id) .
            ' ORDER BY RadekSluzebniCesty.DatumCasOd';

        $cursor = $database ->getSqlResult($sql, $params);
        $result = [];

        /** @var $radek KompaktniRadekSluzebniCesty */
        $radek = null;
        $poslednistat = $sluzebniCesta ->f_stat;
        $poslednipopis = '';

        while ($row = $cursor ->fetch()) {
            if ( !$radek ) {
                $radek = new KompaktniRadekSluzebniCesty();
                $radek ->datumCasOd = $database ->stringToTimestamp($row[1]);
                $radek ->datumCasDo = $radek ->datumCasOd;
                $prvniradeksc = true;
                $poslednipopis = '';
            } else {
                $radek ->datumCasDo = $database ->stringToTimestamp($row[1]);
                $prvniradeksc = false;
            }

            // misto - jen pokud je ruzne od prednastaveneho mista
            if ( $row[4]  &&  $row[4] != RonPortalSettings::getValue(SluzebniCesta::SETTINGS_ZACATEK) ) {
                $radek ->popis .= ($radek ->popis ? ', ' : '') . StringUtils::htmlTextConvert($row[8]);
            }

            if ($row[2]  &&  $row[2] != $poslednipopis) {
                $radek ->popis .= ($radek ->popis ? ', ' : '') . StringUtils::htmlTextConvert($row[2]);
                $poslednipopis = $row[2];
            }

            // zmena statu => prida zkratku statu
            if ($row[3] != $poslednistat) {
                $radek ->popis .= ' (<strong>' . StringUtils::htmlTextConvert($row[3]) . '</strong>)';
                $poslednistat = $row[3];
            }

            $radek ->snidane += (int)$row[5];
            $radek ->obedy += (int)$row[6];
            $radek ->vecere += (int)$row[7];

            if ( $row[0] == 2  ||  (!$prvniradeksc  &&  $row[4]  &&  $row[4] == $sluzebniCesta ->mistoZacatku) ) {
                // konec
                $result [] = $radek;
                $radek = null;
            }
        }

        if ($radek) {
            $result[] = $radek;
        }

        $cursor ->close();

        return $result;
    }

}
