<?php


namespace RonPortal\SluzebniCesty;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\RonPortalDBRecord;

/**
 * Zaznamy z tabulky PredpokladaneNakladySC
 * Class PredpokladanyVydajSC
 */
class PredpokladanyVydajSC
{
    public $sluzebniCesta_ID;
    public $typ;
    public $pocetDnu;
    public $castka;
    public $castkaStr;
    public $kodMeny;

    public $popisTypu; // tabulka TypRUctovaniSlCesty

    /**
     * Nacte z databaze zadane predpokladane vydaje sluzebni cesty
     *
     * @param Database $database
     * @param int|null $idSluzebniCesty
     * @return PredpokladanyVydajSC[]
     * @throws DBException
     */
    static function nactiPredpokladaneVydaje(Database $database, ?int $idSluzebniCesty): array
    {
        $params = $database ->createParameters();
        if ($idSluzebniCesty) {
            $sql =
                'SELECT TypRUctovaniSlCesty.Kod, TypRUctovaniSlCesty.Popis, ' .
                '       PredpokladaneNakladySC.PocetDnu, PredpokladaneNakladySC.Castka, PredpokladaneNakladySC.KodMeny ' .
                ' FROM SluzebniCesta INNER JOIN TypRUctovaniSlCesty' .
                '  ON 1 = 1' .
                ' LEFT OUTER JOIN PredpokladaneNakladySC ' .
                '  ON TypRUctovaniSlCesty.Kod = PredpokladaneNakladySC.Typ ' .
                '  AND PredpokladaneNakladySC.SluzebniCesta_ID = SluzebniCesta.Id' . // $params ->addParameter((int)$idSluzebniCesty) .
                ' WHERE TypRUctovaniSlCesty.PredpokladaneVydaje = 1 ' .
                '  AND SluzebniCesta.Id = ' . $params->addParameter((int)$idSluzebniCesty) . // kvuli ODBC nemuze byt v ON
                ' ORDER BY TypRUctovaniSlCesty.Popis';
        } else {
            $sql =
                'SELECT TypRUctovaniSlCesty.Kod, TypRUctovaniSlCesty.Popis, NULL, NULL, NULL' .
                ' FROM TypRUctovaniSlCesty' .
                ' WHERE TypRUctovaniSlCesty.PredpokladaneVydaje = 1 ' .
                ' ORDER BY TypRUctovaniSlCesty.Popis';
        }

        $cursor = $database ->select($sql, $params);

        $result = [];

        while ($row = $cursor ->fetch()) {
            $rec = new PredpokladanyVydajSC();
            $rec ->sluzebniCesta_ID = $idSluzebniCesty;
            $rec ->typ = (int)$row[0];
            $rec ->popisTypu = TextManager::_('db_typructovanislcesty_popis_' . $row[0], $row[1]);
            $rec ->pocetDnu = isset($row[2]) ? (int)$row[2] : null;
            $rec ->castka = isset($row[3]) ? (float)$row[3] : null;
            $rec ->castkaStr = isset($rec ->castka)
                ? StringUtils::format('number2', $rec ->castka)
                : '';
            $rec ->kodMeny = $row[4];

            $result[] = $rec;
        }

        $cursor ->close();

        return $result;
    }


//    /**
//     * @param Database $database
//     * @param int $idSluzebniCesty
//     * @return bool  Ulozi/Smaze zaznam do/z databaze
//     * @throws \Exception
//     */
//    function store(Database $database, $idSluzebniCesty)
//    {
//        $rec = new RonPortalDBRecord($database, 'PredpokladaneNakladySC');
//
//        $rec ->addField('SluzebniCesta_ID', 'int', true, 0, $idSluzebniCesty);
//        $rec ->addField('Typ', 'int', true, 0, $this ->typ);
//        $rec ->addField('', '', true, 0, $this ->popisTypu); // jen pro zalogovani
//
//        $rec ->addField('PocetDnu', 'int', false, 0, $this ->pocetDnu);
//        $rec ->addField('Castka', 'float', false, 0, $this ->castka);
//        $rec ->addField('KodMeny', '', false, 10, $this ->kodMeny);
//
//        $params = $database ->createParameters();
//        $rec ->setWhere(
//            'SluzebniCesta_ID = ' . $params ->addParameter((int)$idSluzebniCesty) .
//            ' AND Typ = ' . $params ->addParameter($this ->typ),
//            $params);
//
//        $exists = $rec ->loadRecord();
//
//        $this ->sluzebniCesta_ID = $idSluzebniCesty; // kvuli insert
//
//        if ( $exists ) {
//            if ( isset($this ->pocetDnu)  ||  isset($this ->castka) ) {
//                return $rec ->updateRecord();
//            }
//            else {
//                return $rec ->deleteRecord();
//            }
//        } else {
//            if ( isset($this ->pocetDnu)  ||  isset($this ->castka) ) {
//                $rec ->insertRecord();
//            }
//
//            return true;
//        }
//
//    }
}

