<?php
/**
 * stanislav.elbl, 28.09.2021 21:39
 */

namespace RonPortal\SluzebniCesty;

use Ron\DB\Database;
use Ron\DB\DBRecordField;
use Ron\DB\SQLUtils;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalXMLEvents;

class RadekSCDBRecord extends RonPortalDBRecord
{
    /** @var int */
    public $idSC;

    /** @var  DBRecordField */
    public $fCisloRadku;
    /** @var  DBRecordField */
    public $fPopis;
    /** @var  DBRecordField */
    public $fKodOperace;
    /** @var  DBRecordField */
    public $fMisto;
    /** @var  DBRecordField */
    public $fStat;
    /** @var  DBRecordField */
    public $fDatumCasOd;
    /** @var  DBRecordField */
    public $fDopravniProstredek;
    /** @var  DBRecordField */
    public $PocetPJidel_S;
    /** @var  DBRecordField */
    public $PocetPJidel_O;
    /** @var  DBRecordField */
    public $PocetPJidel_V;

    /** @var  DBRecordField */
    public $fPoznamka;

    /**
     * @param Database $database
     * @param int $idSC
     * @param object|string $logDescription
     * @param int $cisloRadku
     */
    function __construct($database, $idSC, $logDescription, $cisloRadku = null)
    {
        parent::__construct($database, 'RadekSluzebniCesty', $logDescription);

        $this ->idSC = $idSC;

        $this ->beforePostEvent = RonPortalXMLEvents::SC_RADEK_BEFORE_POST;
        $this ->afterPostEvent = RonPortalXMLEvents::SC_RADEK_AFTER_POST;

        $this ->addField('SluzebniCesta_ID', 'int', true, 0, $idSC); // readonly = true
        $this ->fCisloRadku = $this ->addField('CisloRadku', 'int', true, 0, $cisloRadku);

        $this ->fPopis = $this ->addField('Popis', '', true, 160);
        $this ->fKodOperace = $this ->addField('KodOperace', '', true, 10);
        $this ->fMisto = $this ->addField('Misto', '', true, 30);
        $this ->fStat = $this ->addField('Stat', '', true, 10);
        $this ->fDatumCasOd = $this ->addField('DatumCasOd', 'timestamp', true);
        $this ->fDopravniProstredek = $this -> addField('DopravniProstredek', '', true, 10);
        $this ->fPocetPJidel_S = $this -> addField('PocetPJidel_S', 'int', true);
        $this ->fPocetPJidel_O = $this -> addField('PocetPJidel_O', 'int', true);
        $this ->fPocetPJidel_V = $this -> addField('PocetPJidel_V', 'int', true);
        $this ->fPoznamka = $this ->addField('Poznamka', '', true, 250);

        $params = $database ->createParameters();
        $where =
            'SluzebniCesta_ID = ' . $params ->addParameter((int)$idSC);

        if ($cisloRadku) {
            $where .= ' AND CisloRadku = ' . $params ->addParameter((int)$cisloRadku);
        }

        $this ->setWhere($where, $params);
    }


    function insertRadek() {
        $noveCislo = SQLUtils::newSerialNumber($this ->database,
            'RadekSluzebniCesty', 'CisloRadku', 'SluzebniCesta_ID = '.(int)$this ->idSC);

        $this ->fCisloRadku ->setNewValue( $noveCislo );

        $this ->insertRecord();

        return $noveCislo;
    }
}
