<?php
/**
 * stanislav.elbl, 24.10.2021 10:07
 */

namespace RonPortal\SluzebniCesty;

use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecord;
use Ron\DB\SqlStatement;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Controls\RozdeleniFormField;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;


/**
 * Pole pro editaci castky - u nekterych typu radku se zobrazuje vzdy kladne (i kdyz se jedna o vydej)
 * Class CastkaUctovaniSCFormField
 */
class CastkaUctovaniSCFormField extends FormField
{
    /** @var FormField */
    private $fTypRadkuUctovani;

    /**
     * CaskaUctovaniSCFormField constructor.
     * @param FormField $fTypRadkuUctovani
     */
    public function __construct(FormField $fTypRadkuUctovani)
    {
        parent::__construct(FormField::FIELD_TYPE_FLOAT);

        $this ->fTypRadkuUctovani = $fTypRadkuUctovani;
    }

    /**
     * @param $database
     * @param $rowData
     * @return mixed|string|\string[]
     */
    public function loadJsonValue($database, $rowData)
    {
        if ($rowData  &&  $this ->fTypRadkuUctovani ->sqlFieldIndex  &&  !RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZNAMENKO_UCTOVANI)) {
            $typUctovani = TypRadkuUctovaniSluzebniCesty::vratTypUctovani($database, $rowData[$this ->fTypRadkuUctovani ->sqlFieldIndex]);

            // pokud je u typu pohybu nastaven smer (prijem/vydej), tak jej zobrazi vzdy kladny
            if ($typUctovani ->druhPohybu == 1 || $typUctovani ->druhPohybu == 2) {
                return ($rowData && isset($this->sqlFieldIndex) && $this->sqlFieldIndex >= 0)
                    ? $this->getValueFromDBValue($database, abs($rowData[$this->sqlFieldIndex]), $rowData)
                    : $this->defaultValue;
            }
        }

        return parent::loadJsonValue($database, $rowData);
    }
}




class RadekUctovaniSCDialog extends RonPortalForm
{
    protected $sluzebniCesta;

    /**
     * @param Database $database
     * @param SluzebniCesta $sluzebniCesta
     */
    function __construct($database, $sluzebniCesta, $readOnly = false)
    {
        parent::__construct($database, 'new', true);

        $this->sluzebniCesta = $sluzebniCesta;

        $this->cssClass = 'large-form rondialog ronform'; // form-25-75
        $this->code = 'formradekuctovanisc';

        $this ->readOnly = $readOnly;

        //$this ->defaultRowClass = 'form-row';
        $this->defaultCaptionColClass = 'col-12 col-sm-3 caption';
        $this->defaultDataColClass = 'col-12 col-sm-9';

        $this->rowIdPrefix = 'row_uctovani_';

        $this->stornoAction = "#";

        $this ->title = TextManager::_('Radek_uctovani_sluzebni_cesty');

        $this->htmlParams = array_merge($this->htmlParams, [
            'id' => 'radek_uctovani_sc_dialog',
            //'data-ronform-url' => '?menuid=sluzebnicestyjs&action=radeksc&idsc=' . $this ->sluzebniCesta ->id,
            'data-ronform-submit-url' => '?menuid=sluzebnicestyjs&action=zapisradekuctovanisc',
            'data-ronform-after-submit' => 'reload',
            'data-typy-uctovani-nvv' => SluzebniCestyUtils::typyUctovaniNVV($database)
        ]);

        $this->sqlTableName = 'RadekUctovaniSCesty';

        $this->setXmlEventsName('SC_UCTOVANI');

        $vychoziHodnoty = $this->nactiHodnotyProNovyRadek();

        $this->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('SluzebniCesta_ID')->setDefaultValue($this->sluzebniCesta->id)->setPrimary(true)
            ->addFieldCssClass('ronform-field')
            ->setCode('field-sluzebnicesta_id')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'SluzebniCesta_ID']);

        $idParams = $this->database->createParameters();
        $this->addField(FormField::FIELD_TYPE_ID)
            ->setFieldName('CisloRadku')->setPrimary(true)
            ->setIdSqlFilter(new SqlStatement('SluzebniCesta_ID = ' . $idParams->addParameter($this->sluzebniCesta->id), $idParams))
            ->addFieldCssClass('ronform-field')
            ->setCode('field-cisloradku')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'CisloRadku']);

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('Den')->setCaption('Datum')
            ->addFieldCssClass('ronform-field') ->setCaptionColumnClass('col-6 col-sm-3 caption') ->setDataColumnClass('col-6 col-sm-9')
            ->setCode('field-den')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'Den'])
            ->setDefaultValue($vychoziHodnoty['den'] ? StringUtils::format('date', $vychoziHodnoty['den']) : '');

        $fieldTyp = $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Typ')->setCaption('Typ')->setRequired(true)
            ->addFieldCssClass('ronform-field')
            ->setCode('field-typ')->setFieldId('typ_radku_uctovani_sc')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'Typ'])
            ->setOptions(TypRadkuUctovaniSluzebniCesty::loadOptions($this->database, $this->sluzebniCesta->f_PP->idPP));

        // mena a kurz
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('KodMeny')->setCaption('Mena')->setCaptionColumnClass('col-6 col-sm-3 caption') ->setDataColumnClass('col-6 col-sm-3')
            ->setRequired(true)
            ->setCode('field-kodmeny')->setFieldId('uctovani_sc_kod_meny')
            ->addFieldCssClass('ronform-field')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'KodMeny'])
            ->setSelectSql('SELECT KodMeny_ISO, KodMeny_ISO AS Popis FROM Mena WHERE %FILTER% ORDER BY %ORDER%', 'Aktivni = 1', 'KodMeny_ISO');

        if (RonPortalSettings::getIntValue('vlastnikurzradkusc')) {
            $this->addField(FormField::FIELD_TYPE_CHECK)
                ->setFieldName('DM_VlastniKurz')->setCaption('Vlastni_kurz')
                ->setCode('field-dm_vlastnikurz')
                ->addFieldCssClass('ronform-field toggle-class-on-change')
                ->setDataColumnClass('col-2') ->setCaptionColumnClass('col-4 col-sm-3 caption')
                ->setOtherHtmlParams([
                    'data-ronform-field-name' => 'DM_VlastniKurz',
                    'data-selector' => '.vlastni-kurz-data',
                    'data-uncheck-class' => 'd-none'
                ]);

            $this->addField(FormField::FIELD_TYPE_FLOAT)
                ->setFieldName('DM_Kurz_Mnozstvi')->setNewRow(false)
                ->setCode('field-dm_kurz_mnozstvi')->setFormat('number_3')
                ->addFieldCssClass('ronform-field vlastni-kurz-data')->setCaptionColumnClass('d-none')->setDataColumnClass('col-2 col-sm-1')
                ->setOtherHtmlParams(['data-ronform-field-name' => 'DM_Kurz_Mnozstvi']);

            $this->addField(FormField::FIELD_TYPE_FLOAT)
                ->setFieldName('DM_Kurz')->setNewRow(false)
                ->setCode('field-dm_kurz')->setFormat('number_3')
                ->addFieldCssClass('ronform-field vlastni-kurz-data')->setCaptionColumnClass('col-2 col-sm-1 vlastni-kurz-data')->setDataColumnClass('col-2 col-sm-1')
                ->setOtherHtmlParams(['data-ronform-field-name' => 'DM_Kurz'])
                ->setCaption('<span id="vlastni_kurz_kod_meny"></span> = ');

            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setFieldName('DM_KodMeny')->setNewRow(false)
                ->setCode('field-dm_kodmeny')
                ->addFieldCssClass('ronform-field vlastni-kurz-data')->setCaptionColumnClass('d-none')->setDataColumnClass('col-1')
                ->setOtherHtmlParams(['data-ronform-field-name' => 'DM_KodMeny']);
        }

        // detaily nahrady za vlastni vozidlo
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-nvv')->setFieldId('group_podrobnosti_nvv')
            ->addFieldCssClass('margin-bottom-2 browsercolor2 padding-top padding-bottom d-none');

        $params = $this ->database ->createParameters();
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('NVV_Vozidlo') ->setCaption('Vozidlo')
            ->setCode('field-nvv_vozidlo') ->addFieldCssClass('ronform-field vozidlo-uctovani-sc')
            ->setSelectSql(new SqlStatement(
                'SELECT Vozidlo.RegistracniZnacka, Vozidlo.Popis, Vozidlo.Spotreba, Vozidlo.DruhPHM, Vozidlo.Jednostope' .
                ' FROM VozidloOsoby, Vozidlo' .
                ' WHERE VozidloOsoby.RegistracniZnacka = Vozidlo.RegistracniZnacka' .
                ' AND VozidloOsoby.RC = ' . $params->addParameter($this->sluzebniCesta->f_PP->rc) .
                ' AND (VozidloOsoby.PlatnostOd <= ' . $params ->addDateParameter(time()) . ' OR VozidloOsoby.PlatnostOd IS NULL)' . // kvuli neprirazenym firemnim vozidlum
                ' AND (VozidloOsoby.PlatnostDo >= ' . $params ->addDateParameter(time()) . ' OR VozidloOsoby.PlatnostDo IS NULL)' .
                ' AND (%FILTER%)' .
                ' ORDER BY (%ORDER%)',
                $params), '1 = 1', 'Vozidlo.Popis')
            ->setItemsAttributes(['data-spotreba:number_2' => 2, 'data-druhphm' => 3, 'data-jednostope' => 4])
            ->setSelectShowCode(true)
            ->setOtherHtmlParams([
                'data-ronform-field-name' => 'NVV_Vozidlo',
                'data-edit-spotreba' => '#NVV_SpotrebaVozidla',
                'data-edit-druhphm' => '#NVV_DruhPHM',
                'data-edit-jednostope' => '#NVV_Jednostope'
            ]);

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('NVV_Jednostope') ->setCaption('Jednostope')
            ->setCode('field-nvv_jednostope') ->setFieldId('NVV_Jednostope')
            ->addFieldCssClass('ronform-field')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'NVV_Jednostope']);

        $this->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('NVV_SpotrebaVozidla')->setCaption('Spotreba_TP')->setCaption2('Litru_100')
            ->setCode('field-nvv_spotrebavozidla') ->setFieldId('NVV_SpotrebaVozidla')
            ->setFieldCssClass('ronform-field inputFloat margin-right-2')//->setDataColumnClass('col-2')
            ->setDefaultValue($vychoziHodnoty['nvv_spotrebavozidla'])
            ->setOtherHtmlParams(['data-ronform-field-name' => 'NVV_SpotrebaVozidla']);

        $this->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('NVV_PocetUjetychKM')->setCaption('Pocet_km')
            ->setCode('field-nvv_spotrebavozidla')
            ->setFieldCssClass('ronform-field inputFloat')->setDataColumnClass('col-3')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'NVV_PocetUjetychKM']);

        $this ->addFieldTrasa();

        $this->addField(RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_NEPOCITAT_ZS_NEMENIT) ? FormField::FIELD_TYPE_CONSTANT : FormField::FIELD_TYPE_CHECK)
            ->setFieldName('NVV_NepocitatZaklSazbu')->setCaption('Nepocitat_zakladni_sazbu') ->setDataType('int')
            ->setCode('field-nvv_nepocitatzaklsazbu')
            ->setDefaultValue(RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_NEPOCITAT_ZAKLADNI_SAZBU))
            ->addFieldCssClass('ronform-field')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'NVV_NepocitatZaklSazbu']);

        $this->addField(FormField::FIELD_TYPE_RADIO)
            ->setFieldName('NVV_NeprokazanaCenaPHM')->setDataType('int') ->setDefaultValue(0)
            ->setCode('field-nvv_neprokazanacenaphm')
            ->addFieldCssClass('ronform-field nvv-neprokazana-cena-phm')
            ->setOptions([0 => TextManager::_('Prokaze_cenu_PHM'), 1 => TextManager::_('Neprokaze_cenu_PHM')]);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('NVV_DruhPHM')->setCaption('Druh_PHM')
            ->addFieldCssClass('ronform-field') ->setFieldId('NVV_DruhPHM')
            ->setCode('field-nvv_druhphm')->setRowId('row_uctovani_NVV_DruhPHM')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'NVV_DruhPHM'])
            ->setDefaultValue($vychoziHodnoty['nvv_druhphm'])
            ->setSelectSql('SELECT Kod, Popis FROM DruhPHM WHERE %FILTER% ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setTranslationPrefix('db_druhphm_popis_');

        $this->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('NVV_CenaPHM')->setCaption('Cena_PHM')->setCaption2('Kc_litr')->setFormat('number2')
            ->setCode('field-nvv_cenaphm')
            ->setFieldCssClass('ronform-field inputFloat margin-right-2')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'NVV_CenaPHM']);

        // konec - - - - detaily nahrady za vlastni vozidlo

        // Dalsi udaje
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-platba')->setFieldId('group_uctovanisc_platba')
            ->addFieldCssClass('margin-bottom-2 browsercolor2 padding-top padding-bottom');

        // Castka
        $this->addField(new CastkaUctovaniSCFormField($fieldTyp))
            ->setFieldName('Castka')->setCaption('Castka')  // ->setRequired(true) - pro nahradu se needituje!
            ->setCode('field-castka')
            //->setRequiredInDatabase(true) - ne, protoze pro nahradu se needituje a doplnuje se az dodatecne
            ->setRowId('row_uctovanisc_castka')->setFormat('number2')
            ->addFieldCssClass('ronform-field')//->setDataColumnClass('col-2')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'Castka']);


        // DPH - jen kdyz je povoleno v nastaveni - jinak je to konstanta
        $this->addField(RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZADAVAT_DPH) ? FormField::FIELD_TYPE_FLOAT : FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('CastkaDPH')->setCaption('SC_DPH') // ->setRequired(true) -> pro nahradu se needituje
            ->setFormat('number2')
            ->setDefaultValue(RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZADAVAT_DPH) ? null : 0)
            ->addFieldCssClass('ronform-field')->setDataColumnClass('col-2')
            ->setCode('field-castkadph')
            //->setRequiredInDatabase(true) - ne, protoze pro nahradu se needituje a doplnuje se az dodatecne
            ->setOtherHtmlParams(['data-ronform-field-name' => 'CastkaDPH']);
        //->setFormat('number2') ->setAlign('right');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('ZpusobPlatby')->setCaption('SC_Zpusob_platby') ->setDataType('int')
            ->setCode('field-zpusobplatby')
            ->addFieldCssClass('ronform-field')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'ZpusobPlatby'])
            ->setSelectSql('SELECT Kod, Popis FROM ZpusobPlatbySC WHERE %FILTER% ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setTranslationPrefix('db_zpusobplatbysc_popis_');

        // Dalsi udaje
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-podrobnosti');

        $this->addField()
            ->setFieldName('Popis')->setCaption('Popis_radku_uctovani_sc')->setMaxLength(160)
            ->addFieldCssClass('ronform-field')
            ->setCode('field-popis')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'Popis']);

        // rozdeleni
        if ($typR1 = RonPortalSettings::getIntValue('typrozdeleniusc')) {
            $f = new RozdeleniFormField($typR1,
                RonPortalSettings::getIntValue('rozdeleniusc_pouzevlastni'),
                $this ->sluzebniCesta ->f_PP);

            $f
                ->loadCaption($this ->database)
                ->setTypRozdeleniFieldName('KodTypuRozdeleni') ->setFieldName('Rozdeleni')
                ->setCode('field-rozdeleni')
                ->addFieldCssClass('ronform-field')
                ->setOtherHtmlParams(['data-ronform-field-name' => 'Rozdeleni']);

            $this->addField($f);
        }

        if ($typR2 = RonPortalSettings::getIntValue('typrozdeleniusc2')) {
            $f = new RozdeleniFormField($typR2,
                RonPortalSettings::getIntValue('rozdeleniusc2_pouzevlastni'),
                $this ->sluzebniCesta ->f_PP);

            $f
                ->loadCaption($this ->database)
                ->setTypRozdeleniFieldName('KodTypuRozdeleni2') ->setFieldName('Rozdeleni2')
                ->setCode('field-rozdeleni2')
                ->addFieldCssClass('ronform-field')
                ->setOtherHtmlParams(['data-ronform-field-name' => 'Rozdeleni2']);

            $this->addField($f);
        }

        // pole zkontrolovano
        if (Prava::vratPrava('slcesty_k_ucet') ->lzeZobrazitPP($this ->sluzebniCesta ->f_PP ->idPP)) {
            $this ->addField(FormField::FIELD_TYPE_CHECK)
                ->setFieldName('Zkontrolovano') ->setCaption('Uctovani_SC_zkontrolovano')
                ->addFieldCssClass('ronform-field')
                ->setCode('field-zkontrolovano')
                ->setCaptionColumnClass('col-6 col-sm-3 caption') ->setDataColumnClass('col-6 col-sm-2')
                ->setOtherHtmlParams(['data-ronform-field-name' => 'Zkontrolovano']);

            $this ->addField(FormField::FIELD_TYPE_TEXT)
                ->setFieldName('Zkontroloval')
                ->addFieldCssClass('ronform-field') ->setCaptionColumnClass('d-none') ->setDataColumnClass('col-12 col-sm-7')
                ->setNewRow(false)
                ->setOtherHtmlParams(['data-ronform-field-name' => 'Zkontroloval']);

        }

        $this->loadConfiguration();
    }


    private function addFieldTrasa()
    {
        $startStr = '';
        $cilStr = '';

        switch (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_VYPOCET_VZDALENOSTI)) {
            case 1 : // prednostne podle lookupu
                $startStr = $this->sluzebniCesta->mistoZacatkuPopis; // lookup
                if (!$startStr) {
                    $startStr = $this->sluzebniCesta->popisZacatku;
                }

                $cilStr = $this->sluzebniCesta->cilPopis; // lookup
                if (!$cilStr) {
                    $cilStr = $this->sluzebniCesta->popisCile;
                }
                break;

            case 2 : // prednostne podle popisu
                $startStr = $this->sluzebniCesta->popisZacatku;
                if (!$startStr) {
                    // lookup
                    $startStr = $this->sluzebniCesta->mistoZacatkuPopis;
                }

                $cilStr = $this->sluzebniCesta->popisCile;
                if (!$cilStr) {
                    // lookup
                    $cilStr = $this->sluzebniCesta->cilPopis;
                }
                break;

            default :
                return;
        }


        if ($startStr && $cilStr) {
            // proklik na zobrazeni trasy:
            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setCaptionColumnClass('d-none')->setDataColumnClass('col-6 col-sm-3') ->setNewRow(false)
                ->setText(
                    '<a target="_blank" href="https://www.google.com/maps/dir/' .
                    StringUtils::htmlTextConvert($startStr) . '/' .
                    ($this->sluzebniCesta->pres ? StringUtils::htmlTextConvert($this->sluzebniCesta->pres) . '/' : '') .
                    StringUtils::htmlTextConvert($cilStr) . '/' .
                    '">' . TextManager::_('Trasa') . '</a>');
        }
    }

    public function getWhereCondition()
    {
        $params = $this ->database ->createParameters();

        $cisloRadku = isset($_POST['CisloRadku'])
            ? (int)$_POST['CisloRadku']
            : (int)$_GET['record'];

        return new SqlStatement(
            'SluzebniCesta_ID = ' . $params ->addParameter($this ->sluzebniCesta ->id) .
            ' AND CisloRadku = ' . $params ->addParameter($cisloRadku),
            $params);
    }


    /**
     * Urci vychozi hodnoty podle hlavicky sluzebni cesty a jiz existujicich radku
     * @return array
     * @throws DBException
     */
    private function nactiHodnotyProNovyRadek()
    {
        $params = $this ->database ->createParameters();

        $sql =
            'SELECT SluzebniCesta.DatumCasOd, SluzebniCesta.DatumCasDo, MAX(RadekUctovaniSCesty.Den), ' .
            ' Vozidlo.DruhPHM, Vozidlo.Spotreba' . // 3,4
            '  FROM SluzebniCesta' .
            ' LEFT OUTER JOIN RadekUctovaniSCesty ON RadekUctovaniSCesty.SluzebniCesta_ID = SluzebniCesta.Id ' .
            ' LEFT OUTER JOIN Vozidlo ON SluzebniCesta.RegistracniZnacka = Vozidlo.RegistracniZnacka' .
            ' WHERE SluzebniCesta.Id = ' . $params ->addParameter((int)$this->sluzebniCesta ->id) .
            ' GROUP BY SluzebniCesta.DatumCasOd, DatumCasDo, SluzebniCesta.Stat, Vozidlo.DruhPHM, Vozidlo.Spotreba';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        $datum = null;

        if ($row) {
            if ($row[2]) {
                // nejedna se o prvni radek - datum podle posledniho zaznamu, cas nevyplneny
                $datum = $cursor ->stringToTimestamp($row[2]);
            } else {
                // jedna se o prvni radek - datum a cas podle zacatku sl. cesty
                $datum = $cursor ->stringToTimestamp($row[0]);
            }
        }

        return [
            'den' => $datum,
            'nvv_druhphm' => $row[3],
            'nvv_spotrebavozidla' => $row[4] ? StringUtils::format('number2', $row[4]) : null
        ];
    }


    public function getCsrfRequestId()
    {
        // csrf podle id sluzebni cesty
        return $this ->code . '_' . $this ->sluzebniCesta ->id;
    }


    /**
     * Zkontroluje zadana data. V pripade chyby zapise chybove hlaseni a vraci false
     * Pokud je vse spravne, tak vraci zadany cas (timestamp)
     * @param bool $jeNahrada
     * @param SluzebniCesta $sc
     * @throws \Exception
     */
    protected function zkontrolujUctovaniSluzebky($jeNahrada, $sc)
    {
        try {
            $datum = DateUtils::dateStringToTimestamp($_POST['Den'] ?? "");
        } catch (\Exception $e) {
            throw new RonException($e ->getMessage(), 'Den');
        }

        if (!RonPortalSettings::getIntValue('datumuctovani_mimosc')) {
            // Kontrola, jestli je zadane datum v rozsahu datumu sluzebni cesty
            if ($datum < DateUtils::startOfTheDay($sc->datumCasOd) || $datum > $sc->datumCasDo) {
                throw new RonException(
                    sprintf(TextManager::_('Datum_musi_byt_v_rozsahu'), StringUtils::format('date', $sc->datumCasOd), StringUtils::format('date', $sc->datumCasDo)),
                    'Den');
            }
        }

        if ($jeNahrada) {
            // nahrada za sluzebni vozidlo - musi byt vyplnene dalsi hodnoty
            if (!$_POST['NVV_SpotrebaVozidla']) {
                throw new RonException(TextManager::_('form.Missing_value') . TextManager::_('Spotreba_TP'), 'NVV_SpotrebaVozidla');
            }

            if (!$_POST['NVV_PocetUjetychKM']) {
                throw new RonException(TextManager::_('form.Missing_value') . TextManager::_('Pocet_km'), 'NVV_PocetUjetychKM');
            }

            if ($_POST['NVV_NeprokazanaCenaPHM']) {
                // neprokaze cenu PHM => musi vybrat druh PHM
                if (!$_POST['NVV_DruhPHM']) {
                    throw new RonException(TextManager::_('form.Missing_value') . TextManager::_('Druh_PHM') . ' x ' . $_POST['NVV_NeprokazanaCenaPHM'], 'NVV_DruhPHM');
                }
            } else {
                // prokaze cenu PHM => musi zadat cenu
                if (!$_POST['NVV_CenaPHM']) {
                    throw new RonException(TextManager::_('form.Missing_value') . TextManager::_('Cena_PHM'), 'NVV_CenaPHM');
                }
            }
        } else {
            // neni to nahrada za soukrome vozidlo - musi byt zadana castka
            if (!$_POST['Castka']) {
                throw new RonException(TextManager::_('Neni_zadana_castka'), 'Castka');
            }

            if (!StringUtils::isFloat($_POST['Castka'])) {
                throw new RonException(TextManager::_('Spatna_hodnota_pro') . TextManager::_('Castka'), 'Castka');
            }
        }

//        if (!$_POST['KodMeny']) {
//            throw new RonException(TextManager::_('Neni_zadana_mena'), 'KodMeny');
//        }

        if (isset($_POST['DM_VlastniKurz']) && $_POST['DM_VlastniKurz']) {
            // zkontroluje zadane kurzy
            if (!StringUtils::isFloat($_POST['DM_Kurz_Mnozstvi'])) {
                throw new RonException(TextManager::_('Neni_zadano_mnozstvi_pro_kurz'), 'DM_Kurz_Mnozstvi');
            }

            if (!StringUtils::isFloat($_POST['DM_Kurz'])) {
                throw new RonException(TextManager::_('Neni_zadan_kurz'), 'DM_Kurz');
            }
        }
    }


    public function checkRequiredFields($edit)
    {
        $kodTypu = (int)$_POST['Typ'];
        $typUctovani = TypRadkuUctovaniSluzebniCesty::vratTypUctovani($this ->database, $kodTypu);

        if (!TypRadkuUctovaniSluzebniCesty::lzeEditovatRadekUctovani($this ->sluzebniCesta ->f_PP ->idPP, $kodTypu)) {
            throw new NeniPravoException();
        }

        if (!parent::checkRequiredFields($edit)) {
            return false;
        }

        $jeNahrada = $typUctovani ? $typUctovani ->nahrada : false;
        $this ->zkontrolujUctovaniSluzebky($jeNahrada, $this ->sluzebniCesta);

        return true;
    }


    /**
     * Pomocna funkce - prepne zadana pole do readonly rezimu
     * @param DBRecord $rec
     * @param string[] $fields
     */
    private function setReadOnlyFields($rec, $fields)
    {
        foreach ($fields as $fieldName) {
            $f = $rec ->findFieldByName($fieldName);
            if ($f) {
                $f ->setReadOnly(true);
            }
        }
    }


    public function createDBRecordFields($rec, $insert)
    {
        parent::createDBRecordFields($rec, $insert);

        $fieldTyp = $rec ->findFieldByName('Typ');
        $fieldCastka = $rec->findFieldByName('Castka');

        if ($fieldTyp) {
            $typUctovani = TypRadkuUctovaniSluzebniCesty::vratTypUctovani($this->database, $_POST['Typ']);

            if (!$typUctovani) {
                // nemelo by nastat
                return;
            }

            if ($typUctovani ->nahrada) {
                $this ->setReadOnlyFields($rec, ['CastkaDPH']);
                // nahrada za vlastni vozidlo
                if ($fieldCastka) {
                    $fieldCastka ->setNewValue(0); // musi ze znovu preuctovat
                }

                $rec['CastkaDPH'] = 0;
                $rec['ZpusobPlatby'] = 0;
            } else {
                // Zpusob platby je v databazi povinny, ale pro "nahrady" se nezadava
                //throw new \Exception('ZpusobPlatby: -' . $rec['ZpusobPlatby'] . '-');
                if ($rec['ZpusobPlatby'] === null) {
                    $fieldIndex = $this ->findFieldIndexByCode('field-zpusobplatby');
                    $field = isset($fieldIndex) ? $this ->fields[$fieldIndex] : null;

                    throw new RonException(TextManager::_('form.Missing_value') . ($field ? $field->caption : 'ZpusobPlatby'), $field ? $field->fieldName : 'ZpusobPlatby');
                }

                // nejedna se o nahradu => pole ktera se tykaji nahrad nastavi readonly
                $this ->setReadOnlyFields($rec, [
                    'NVV_Vozidlo', 'NVV_SpotrebaVozidla', 'NVV_PocetUjetychKM',
                    'NVV_NepocitatZaklSazbu', 'NVV_NeprokazanaCenaPHM',
                    'NVV_CenaPHM', 'NVV_DruhPHM', 'NVV_Jednostope'
                ]);

                // nektere radky jsou vzdy vydej/prijem => je potreba spravne nastavit znamenko podle nastaveni typu radku
                if ($fieldCastka  &&  !$fieldCastka->readOnly) {
                    if ($typUctovani->druhPohybu === 1) {
                        // prijem
                        $fieldCastka->setNewValue(abs($fieldCastka->value));
                    } else if ($typUctovani->druhPohybu === 2) {
                        // vydej
                        $fieldCastka->setNewValue(-abs($fieldCastka->value));
                    }
                }
            }
        }

        if (Prava::vratPrava('slcesty_k_ucet') ->lzeZobrazitPP($this ->sluzebniCesta ->f_PP ->idPP)) {
            $fZkontrolovano = $rec ->findFieldByName('Zkontrolovano');
            if ($fZkontrolovano  &&  $fZkontrolovano ->value  &&  $fZkontrolovano ->isChanged()) {
                // uzivatel zaskrtnul "zkontrolovano" - doplni pole "Zkontroloval" a "DatumCasKontroly"
                $rec ->addTimestampField('DatumCasKontroly', false, time());
                $rec ->addStringField('Zkontroloval', false, 25, RonPortalUtils::getLogUserName());
            }
        }
    }


}
