<?php

namespace RonPortal\SluzebniCesty;

use Ron\Controls\FormField;
use Ron\Controls\Option;
use Ron\Controls\OptionLoader;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecord;
use Ron\DB\SqlStatement;
use Ron\MessageManager;
use Ron\RonException;
use Ron\TextManager;
use RonPortal\Controls\RonPortalForm;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\System\RecordNotFoundException;
use RonPortal\TypRozdeleni;
use Tracy\Dumper;

require_once 'ronportal/sluzebnicesty/sluzebnicestaformfields.php';



/**
 * Formular pro editaci sluzebni cesty
 * Class TextList
 */
class SluzebniCestaFormular extends RonPortalForm
{
    const SETTINGS_SAZBA_KAPESNEHO_PODLE_STATU = 'sazbakapesnehopodlestatu';
    const SETTINGS_MENIT_SAZBU_KAPESNEHO = 'sazbakapesnehoslcest';
    const SETTINGS_VYCHOZI_SAZBA_KAPESNEHO = 'vychozisazbakapesnehosc';     // nacita se podle nastaveni aplikace
    const SETTINGS_ROZDELENI_POVINNE = 'rozdeleniscpovinne';
    const SETTINGS_TYP_VYPOCTU = 'typvypoctucn';
    const SETTINGS_VKLADANI_POCET_DNU = 'editacesluzebnichpocetdnu';        // poznamka: funkcnost zmenena - kontroluje se pri vkladani, ne pri editaci

    public $idSC;

    /** @var SluzebniCesta  Pri editaci se nacita kvuli kontrolam prava atd. */
    private $sluzebniCesta;

    /** @var OsobaSCFormField */
    private $osobaField;

//    /** @var FormField      Pole pro vyber vozidla. */
//    private $registracniZnackaField;

    /** @var SluzebniCestaFormField      Pole pro zobrazeni maximalniho stravneho. */
    private $urceneDenniStravneField;

    /** @var SluzebniCestaFormField      Pole pro editaci sazby kapesneho - v zavislosti na nastaveni Portalu se muze urcit automaticky. */
    private $sazbaKapesnehoField;

    /** @var SluzebniCestaFormField      Pole pro vychozi stat. */
    private $statField;
    /** @var SluzebniCestaFormField      Pole pro cilovy stat. */
    private $cilovyStatField;

    /** @var FormField[] */
    private $fieldsPNPopis = [];
    /** @var FormField[] */
    private $fieldsPNPocetDnu = [];
    /** @var FormField[] */
    private $fieldsPNCastka = [];
    /** @var FormField[] */
    private $fieldsPNMena = [];


    /** @var FormField */
    private $kopirovatRadkyField;
    /** @var FormField */
    private $kopirovatUctovaniField;

    /** @var RozdeleniSCFormField */
    private $rozdeleniField;

    /**
     * @throws DBException
     */
    public function __construct(Database $database)
    {
        parent::__construct($database);

        $this->code = 'formsluzebnicesta';

        $this->defaultRowClass = 'row';
        $this->defaultCaptionColClass = 'col-12 col-sm-3 caption';
        $this->defaultDataColClass = 'col-12 col-sm-9';

        $this->sqlTableName = 'SluzebniCesta';

        $this->stornoAction = "?menuid=sluzebnicesty";

        $this->cssClass .= ' sluzebnicestaformular';

        $this->sqlSelectPart =
            'SluzebniCesta.KodTypuRozdeleni2, SluzebniCesta.Rozdeleni2,' .  // 0,1
            ' SluzebniCesta.SchvalenoNadrizenym, SluzebniCesta.Zauctovano, ' .  // 2,3
            ' SluzebniCesta.Schvaleni1, SluzebniCesta.Schvaleni2, SluzebniCesta.Schvaleni3, SluzebniCesta.Schvaleni4'; // 4..7


        $this->setXmlEventsName('SC'); // SC_BEFORE_POST a SC_AFTER_POST

        $this->htmlParams = [
            'id' => 'sluzebni_cesta_formular',
            'data-sazba-kapesneho-podle-statu' => RonPortalSettings::getIntValue(self::SETTINGS_SAZBA_KAPESNEHO_PODLE_STATU),
            'data-menit-sazbu-kapesneho' => RonPortalSettings::getIntValue(self::SETTINGS_MENIT_SAZBU_KAPESNEHO),
            'data-nacitat-adresu-rozdeleni' => RonPortalSettings::getIntValue('rozdelenisc_adresadocile'),
            //'data-id-pp' => isset($sluzebniCesta ->f_PP ->idPP) ? $sluzebniCesta ->f_PP ->idPP : ''
            'data-id-sc' => $this->getRecordPrimaryKeyValue()
        ];

        if ($this ->formAction === 'new') {
            $this ->htmlParams['data-nacist-data-pro-osobu'] = '1';
        }

        $this->title = TextManager::_('Sluzebni_cesta');

        // ID
        $this->addField(FormField::FIELD_TYPE_ID)
            ->setFieldName('Id') ->setPrimary(true) ->setSequence(true)
            ->setCode('field-id');

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('Zahranicni')
            ->setCode('field-zahranicni')->setDefaultValue(RonPortalSettings::getIntValue('sc_zahranicni'))
            ->setCaption(false)->setCaption2('Zahranicni', true, true)
            ->setRowId('hlavickasc_zahranicni')
            ->setFieldId('scZahranicni')->setDataColumnClass('col-12 align-right')
            ->setAppendFormClass('zahranicniSC');

        $this->osobaField = new OsobaSCFormField('sluzebnicesty');
        $this->addField($this->osobaField)
            ->setSqlFieldName('Id', 'PracovniPomer')->setCaption('Osoba')
            ->setCode('field-osoba')
            ->setRowId('hlavickasc_osoba')->setFieldId('Osoba')
            ->setRequired(true);

        $this->statField = new VychoziStatSCFormField();
        $this->addField($this->statField)
            ->setFieldName('Stat')->setCaption('Stat')
            ->setCode('field-stat')
            ->setSelectSql(
                'SELECT Kod, Popis FROM Stat WHERE %FILTER% ORDER BY %ORDER%',
                'PovolitVeSluzCeste = 1', 'Popis')
            ->setShowEmptyValue($this->isInsert())
            ->setTranslationPrefix('db_stat_popis_')
            ->setRowId('hlavickasc_stat')
            ->setOtherHtmlParams(['id' => 'vychoziStatSC']);

        // zacatek
        $this->addField(RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZACATEK_NEMENIT) ? FormField::FIELD_TYPE_CONSTANT : FormField::FIELD_TYPE_SELECT)
            ->setFieldName('MistoZacatku')->setCaption('Misto_zacatku')
            ->setReadOnly(RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZACATEK_NEMENIT))
            ->setCode('field-mistozacatku')
            ->setDefaultValue(RonPortalSettings::getValue(SluzebniCesta::SETTINGS_ZACATEK))
            ->setSelectSql(
                'SELECT Kod, Popis FROM MistoSluzebniCesty WHERE %FILTER% ORDER BY %ORDER%',
                'Aktivni = 1 AND Pocatek = 1', 'Popis')
            ->setShowEmptyValue(true)
            ->setTranslationPrefix('db_mistosluzebnicesty_popis_')
            ->setRowId('hlavickasc_zacatek');

        $this->addField()
            ->setFieldName('PopisMistaZacatku')
            ->setCode('field-popismistazacatku')
            ->setCaption(RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZACATEK_NEMENIT) ? 'Misto_zacatku' : 'Popis_mista_zacatku')
            ->setRowId('hlavickasc_popiszacatku')
            ->setMaxLength(160);

        // konec
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Cil')->setCaption('Cil_sc')
            ->setCode('field-cil')
            ->setSelectSql(
                'SELECT Kod, Popis, Stat FROM MistoSluzebniCesty WHERE %FILTER% ORDER BY %ORDER%',
                'Aktivni = 1 AND Cil = 1', 'Popis')
            ->setShowEmptyValue(true)
            ->setTranslationPrefix('db_mistosluzebnicesty_popis_')
            ->setItemsAttributes(['data-stat' => 2])
            ->setOtherHtmlParams(['data-copy-target' => '#CilovyStat', 'data-copy-source' => 'stat'])
            ->setFieldId('Cil_SC')
            ->setRowId('hlavickasc_cil');

        $this->addField()
            ->setFieldName('PopisCile')
            ->setCode('field-popiscile')
            ->setCaption('Popis_cile')
            ->setRowId('hlavickasc_popiscile')
            ->setMaxLength(160);

        $this->addField()
            ->setFieldName('Pres')->setCaption('Pres')
            ->setCode('field-pres')
            ->setRowId('hlavickasc_pres')
            ->setMaxLength(160);

        $this->cilovyStatField = new SluzebniCestaFormField(FormField::FIELD_TYPE_SELECT, SluzebniCestaFormField::JEN_ZAHRANICNI);
        $this->addField($this->cilovyStatField)
            ->setFieldName('CilovyStat')->setCaption('Cilovy_stat')
            ->setCode('field-cilovystat')
            ->setSelectSql(
                'SELECT Kod, Popis FROM Stat WHERE %FILTER% ORDER BY %ORDER%',
                'PovolitVeSluzCeste = 1', 'Popis')
            ->setShowEmptyValue(true)
            ->setTranslationPrefix('db_stat_popis_')
            ->setRowId('hlavickasc_cilovystat')
            ->setFieldCssClass('form-control ronselect') ->setFieldId('CilovyStat');

        // rozdeleni 1 - jen kdyz je vybran typ rozdeleni
        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI)) {
            $this->addField(FormField::FIELD_TYPE_GROUP)
                ->setCode('group-rozdeleni')
                ->setFieldId('hlavickasc_rozdeleni');

            $caption = TextManager::_('Rozdeleni_sc');
            if (!$caption) {
                try {
                    $caption = TypRozdeleni::getPopisTypuRozdeleni($this->database, RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI));
                } catch (RecordNotFoundException $e) {
                    $caption = 'Rozdeleni';
                }
            }

            $this ->rozdeleniField = new RozdeleniSCFormField(false);

            $this->addField($this ->rozdeleniField)
                ->setFieldName('Rozdeleni')->setCaption($caption)
                ->setCode('field-rozdeleni')
                ->setRequired(RonPortalSettings::getIntValue(self::SETTINGS_ROZDELENI_POVINNE));
            // melo by byt col-12 col-sm-6   a   col-12 col-sm-3

            if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ROZDELENI_POVOLIT_VYBER)) {
                $this->addField(FormField::FIELD_TYPE_TEXT)
                    ->setCaptionColumnClass('d-xs-none col-3 caption')
                    ->setText('<span id="vybratRozdeleniSC" class="js">' . TextManager::_('Vybrat_rozdeleni') . '</span>');
            }

            $this->addField(FormField::FIELD_TYPE_GROUP_END);  // ukonci group rozdeleni
        }

        // ucel
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Ucel')->setCaption('Ucel_sc')
            ->setCode('field-ucel')
            ->setSelectSql(
                'SELECT Kod, Popis FROM UcelSluzebniCesty WHERE %FILTER% ORDER BY %ORDER%',
                'Aktivni = 1', 'Popis')
            ->setShowEmptyValue($this->isInsert() || !RonPortalSettings::getIntValue(SluzebniCestyUtils::SETTINGS_POVINNY_UCEL))
            ->setTranslationPrefix('db_ucelsluzebnicesty_popis_')
            ->setRowId('hlavickasc_ucel')
            ->setRequired(RonPortalSettings::getIntValue(SluzebniCestyUtils::SETTINGS_POVINNY_UCEL));

        // cas od - do
        $casyAutomaticky = RonPortalSettings::getIntValue(SluzebniCestyUtils::SETTINGS_CASY_AUTOMATICKY);
        $this->addField($casyAutomaticky ? FormField::FIELD_TYPE_DATE : FormField::FIELD_TYPE_DATEANDTIME)
            ->setFieldName('DatumCasOd')
            ->setCode('field-datumcasod')
            ->setFieldId('DatumSCOd')
            ->setCaption('Zacatek')->setRequired(true)
            ->setDefaultValue($casyAutomaticky
                ? date('j.n.Y')
                : [date('j.n.Y'), ''])
            ->setRowId('hlavickasc_casod')
            ->setOtherHtmlParams(['data-roncalendar-next' => 'DatumCasDo']);

        $this->addField($casyAutomaticky ? FormField::FIELD_TYPE_DATE : FormField::FIELD_TYPE_DATEANDTIME)
            ->setFieldName('DatumCasDo')
            ->setCode('field-datumcasdo')
            ->setCaption('Konec')->setRequired(true)
            ->setDefaultValue($casyAutomaticky
                ? date('j.n.Y')
                : [date('j.n.Y'), ''])
            ->setRowId('hlavickasc_casdo')
            ->setOtherHtmlParams(['data-roncalendar-prev' => 'DatumCasOd']);

        // datum kurzu
        $this->addField(new SluzebniCestaFormField(FormField::FIELD_TYPE_DATE, SluzebniCestaFormField::JEN_ZAHRANICNI))
            ->setFieldName('DatumKurzu')->setCaption('Datum_kurzu')
            ->setCode('field-datumkurzu')
            ->setVisible(RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZADAVAT_DATUM_KURZU, 1))
            ->setRowId('hlavickasc_datumkurzu'); //->setRowClass('row jen-zahranicni-sc');

        $this->addField(new SluzebniCestaFormField(FormField::FIELD_TYPE_TEXT, SluzebniCestaFormField::JEN_ZAHRANICNI))
            ->setText(TextManager::_('Datum_kurzu_info'))
            ->setCode('field-datumkurzu-info')
            ->setFieldCssClass('forminfotext')
            ->setRowId('hlavickasc_datumkurzuinfo'); // ->setRowClass('row jen-zahranicni-sc');

        // sazba kapesneho
        $this->sazbaKapesnehoField = $this->addField(RonPortalSettings::getIntValue(self::SETTINGS_MENIT_SAZBU_KAPESNEHO)
            ? FormField::FIELD_TYPE_FLOAT
            : FormField::FIELD_TYPE_CONSTANT
        )
            ->setFieldName('SazbaKapesneho')->setFieldId('SazbaKapesneho')->setCaption('Sazba_kapesneho')
            ->setCode('field-sazbakapesneho')
            ->setDataType('float')
            ->setDefaultValue(RonPortalSettings::getFloatValue(self::SETTINGS_VYCHOZI_SAZBA_KAPESNEHO))
            ->setRequired(true) // v databazi je povinne, takze musi byt zadano
            ->setDataColumnClass('col-12 col-sm-3')
            ->setReadOnly(!RonPortalSettings::getIntValue(self::SETTINGS_MENIT_SAZBU_KAPESNEHO) && !RonPortalSettings::getIntValue(self::SETTINGS_SAZBA_KAPESNEHO_PODLE_STATU))
            ->setRowId('hlavickasc_sazbakapesneho');

        // dopravni prostredek a vozidlo
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('DopravniProstredek')->setCaption('Dopravni_prostredek')
            ->setCode('field-dopravniprostredek')
            ->setDataColumnClass('col-12 col-sm-3')
            ->setRowId('hlavickasc_dopravniprostredek')
            ->setShowEmptyValue(true)
            ->setTranslationPrefix('db_dopravniprostredek_popis_')
            ->setSelectSql(
                'SELECT Kod, Popis FROM DopravniProstredek WHERE %FILTER% ORDER BY %ORDER%',
                '1 = 1', 'Popis');

        //$this->registracniZnackaField =
        $this->addField(new RegistracniZnackaField(FormField::FIELD_TYPE_SELECT))
            ->setFieldName('RegistracniZnacka')->setCaption('Vozidlo')
            ->setCode('field-registracniznacka')
            ->setDataColumnClass('col-12 col-sm-3')
            ->setNewRow(false)
            ->setShowEmptyValue(true)
            ->setFieldId('RegistracniZnacka');

        // prenos do dochazky
        $this->addField(FormField::FIELD_TYPE_CONSTANT)    // slouzi jen pro nacteni dat
            ->setSqlFieldName('PrenasetDoDochazky')
            ->setCode('field-prenasetdodochazky')
            ->setIncludeToDBRecord(false);      // do dbrecordu se vlozi podle hodnoty pole field-typprenosudodochazky

        $this->addField(new PrenasetDoDochazkyFormField())
            ->setCode('field-typprenosudodochazky')
            ->setRowId('hlavickasc_prenosdodochazky')
            ->setDataColumnClass('col-12 col-sm-3');

        $casyPodleRadku = RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_CASY_PODLE_RADKU);
        $this->addField(($casyPodleRadku === -1) ? FormField::FIELD_TYPE_CHECK : FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('PocatekKonecPodleRadku')->setCaption('Casy_podle_radku')
            ->setCode('field-pocatekkonecpodleradku')
            ->setDataType('int')
            ->setRowId('hlavickasc_casypodleradku')
            ->setCaptionColumnClass('col-8 col-sm-3 caption')
            ->setDataColumnClass('col-4 col-sm-9 align-xs-right')
            ->setDefaultValue($casyPodleRadku === 1 ? 1 : 0)
            ->setReadOnly($casyPodleRadku !== -1);
        //->setVisible(RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_CASY_PODLE_RADKU) === -1);

        $this->addField(FormField::FIELD_TYPE_TEXT)
            ->setCode('field-pocatekkonecpodleradku-info')
            ->setText(TextManager::_('Casy_podle_radku_info'))->setFieldCssClass('forminfotext')
            ->setNewRow(false)
            ->setVisible(RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_CASY_PODLE_RADKU) === -1);

        $this->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('TypVypoctuCN')->setReadOnly(true)
            ->setCode('field-typvypoctucn')
            ->setDefaultValue(RonPortalSettings::getIntValue(self::SETTINGS_TYP_VYPOCTU))
            ->setDataType('int');

        // popis
        $this->addField()
            ->setFieldName('Popis')->setCaption('Popis_sc')
            ->setCode('field-popis')
            ->setRequired(RonPortalSettings::getIntValue('povinnypopisslcesty'))
            ->setRowId('hlavickasc_popis')
            ->setMaxLength(160);

        $rozlisitRozdeleni2Z = RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2) != RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2Z);

        // rozdeleni 2
        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2)) {
            $caption = TextManager::_('Rozdeleni_sc_2');
            if (!$caption) {
                try {
                    $caption = TypRozdeleni::getPopisTypuRozdeleni($this->database, RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2));
                } catch (RecordNotFoundException $e) {
                    $caption = 'Rozdeleni ' . RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2);
                }
            }

            $this->addField(
                new RozdeleniSCFormField(true, $rozlisitRozdeleni2Z ? SluzebniCestaFormField::JEN_TUZEMSKE : SluzebniCestaFormField::ZAHRANICNI_I_TUZEMSKE))
                ->setFieldName('Rozdeleni2')->setSqlFieldIndex(1)->setCaption($caption)
                ->setRowId('hlavickasc_rozdeleni2' . ($rozlisitRozdeleni2Z ? 'tuzemska' : ''))
                //->setRowClass('row' . ($rozlisitRozdeleni2Z ? ' jen-tuzemska-sc' : ''))
                ->setCode('field-rozdeleni2');

//            $this->addField(new RozdeleniFormField(RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2), RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ROZDELENI2_POUZE_VLASTNI)))
//                ->setFieldName('Rozdeleni2') ->setCaption($caption)
//                ->setRowId('hlavickasc_rozdeleni2' . ($rozlisitRozdeleni2Z ? 'tuzemska' : ''))
//                ->setRowClass('row' . ($rozlisitRozdeleni2Z ? ' jen-tuzemska-sc' : ''))
//                ->setCode('field-rozdeleni2');
            // melo by byt col-12 col-sm-6   a   col-12 col-sm-3
        }

        // rozdeleni 2 pro zahranicni SC (pokud je jine nez pro tuzemske)
        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2Z) && $rozlisitRozdeleni2Z) {
            $caption = TextManager::_('Rozdeleni_sc_2_zahranicni');
            if (!$caption) {
                try {
                    $caption = TypRozdeleni::getPopisTypuRozdeleni($this->database, RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2Z));
                } catch (RecordNotFoundException $e) {
                    $caption = 'Rozdeleni ' . RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2Z);
                }
            }

            $this->addField(new RozdeleniSCFormField(true, SluzebniCestaFormField::JEN_ZAHRANICNI))
                ->setFieldName('*Rozdeleni2Z')->setSqlFieldName('Rozdeleni2')->setSqlFieldIndex(1)->setCaption($caption)
                ->setRowId('hlavickasc_rozdeleni2zahranicni')
                //->setRowClass('row jen-zahranicni-sc')
                ->setCode('field-rozdeleni2Z');
            // melo by byt col-12 col-sm-6   a   col-12 col-sm-3
        }

        // uzivatelske polozky
        for ($i = 1; $i <= 5; $i++) {
            $this->addField()
                ->setFieldName('Uzivatelska' . $i)->setCaption(RonPortalSettings::getValue('slcesty_uzivatelska' . $i))
                ->setCode('field-uzivatelska' . $i)
                ->setRowId('hlavickasc_uzivatelska' . $i)
                ->setMaxLength(100)
                ->setVisible((bool)RonPortalSettings::getValue('slcesty_uzivatelska' . $i));
        }

        // kapesne
        $this->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('UrceneKapesne')->setCaption('Urcene_kapesne')
            //->setFormat('number_2')
            ->setCode('field-urcenekapesne')
            ->setCaptionColumnClass('col-6 col-sm-3 caption')->setDataColumnClass('col-3 col-sm-2')
            ->setRowId('hlavickasc_urcenestravne')
            ->setVisible(false);


        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('UrceneKapesneMena')->setCaption(false)
            ->setCode('field-urcenekapesnemena')
            ->setDataColumnClass('col-3 col-sm-1')
            ->setNewRow(false)
            ->setShowEmptyValue(true)
            ->setSelectSql(
                'SELECT KodMeny_ISO, KodMeny_ISO AS Popis FROM Mena WHERE %FILTER% ORDER BY %ORDER%',
                'Aktivni = 1', 'KodMeny_ISO')
            ->setVisible(false);

        // urcene stravne
        $this->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setSqlFieldName('UrceneDenniStravneMena')
            ->setCode('field-urcenedennistravnemena')
            ->setIncludeToDBRecord(false);

        $this->urceneDenniStravneField = $this->addField(new UrceneStravneFormField())
            ->setFieldName('UrceneDenniStravne')->setCaption('Maximalni_denni_stravne')
            ->setFormat('number_2')
            ->setCode('field-urcenedennistravne')
            ->setFieldId('maximalniDenniStravne')
            ->setNewRow(false)
            ->setCaptionColumnClass('col-6 col-sm-3 caption')->setDataColumnClass('col-6 col-sm-3')
            ->setVisible(false);

        // zpusob a mena vyuctovani
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('VyplatitHotove')->setCaption('Zpusob_vyuctovani_sc')
            ->setCode('field-vyplatithotove')
            ->setDataType('int')
            ->setCaptionColumnClass('col-6 col-sm-9 col-md-3 caption')->setDataColumnClass('col-6 col-sm-3')
            ->setRowId('hlavickasc_zpusobvyuctovani')
            ->setSelectSql(
                'SELECT Kod, Popis FROM S_ZpusobVyuctovaniSC WHERE %FILTER% ORDER BY %ORDER%',
                '1 = 1', 'Kod'
            )
            ->setShowEmptyValue(false)
            ->setDefaultValue(0)
            ->setVisible(false);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('ManaProVyplaceni')->setCaption('Mena_pro_vyplaceni_sc')
            ->setCode('field-manaprovyplaceni')
            ->setCaptionColumnClass('col-6 col-sm-9 col-md-3 caption')->setDataColumnClass('col-6 col-sm-3')
            ->setNewRow(false)
            ->setSelectSql(
                'SELECT KodMeny_ISO, KodMeny_ISO AS Popis FROM Mena WHERE %FILTER% ORDER BY %ORDER%',
                'Aktivni = 1', 'KodMeny_ISO')
            ->setShowEmptyValue(true)
            ->setVisible(false);

        // pozadovana zaloha
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-zaloha')
            ->setFieldId('hlavickasc_zaloha');

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('PozadovanaZaloha')->setCaption('Pozadovana_zaloha')
            ->setCode('field-pozadovanazaloha')
            ->setRowId('hlavickasc_pozadovanazaloha')->setRowClass('row form-section')
            ->setCaptionColumnClass('col-8 col-sm-3 caption')->setDataColumnClass('col-4 col-sm-9 align-xs-right')
            ->setFieldId('sc_pozadovanazaloha')
            ->setFieldCssClass('toggleClass')
            ->setAppendFormClass('pozadovanazalohaSC')
            ->setOtherHtmlParams([
                'data-selector' => '#sluzebni_cesta_formular',
                'data-check-class' => 'pozadovanazalohaSC'
            ]);

        $this->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('PozadovanaZalohaHotovost')->setCaption('Zaloha_v_hotovosti')
            ->setCode('field-pozadovanazalohahotovost')
            ->setRowId('hlavickasc_zalohahotovost')->setRowClass('row row-zaloha-sc')
            ->setCaptionColumnClass('col-12 col-sm-9 col-md-3 caption')->setDataColumnClass('col-3 col-sm-2');

        //<div class="col-6 mobile-only"></div>
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('PozadovanaZalohaHotMena')->setCaption(false)
            ->setCode('field-pozadovanazalohahotmena')
            ->setDataColumnClass('col-3 col-sm-1')
            ->setNewRow(false)
            ->setSelectSql(
                'SELECT KodMeny_ISO, KodMeny_ISO AS Popis FROM Mena WHERE %FILTER% ORDER BY %ORDER%',
                'Aktivni = 1', 'KodMeny_ISO')
            ->setShowEmptyValue(true);

        $this->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('PozadovanaZalohaUcet')->setCaption('Zaloha_na_ucet')
            ->setCode('field-pozadovanazalohaucet')
            ->setRowId('hlavickasc_zalohacastka')->setRowClass('row row-zaloha-sc')
            ->setCaptionColumnClass('col-12 col-sm-9 col-md-3 caption')->setDataColumnClass('col-3 col-sm-2');

        //<div class="col-6 mobile-only"></div>
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('PozadovanaZalohaUcetMena')->setCaption(false)
            ->setCode('field-pozadovanazalohaucetmena')
            ->setDataColumnClass('col-3 col-sm-1')
            ->setNewRow(false)
            ->setSelectSql(
                'SELECT KodMeny_ISO, KodMeny_ISO AS Popis FROM Mena WHERE %FILTER% ORDER BY %ORDER%',
                'Aktivni = 1', 'KodMeny_ISO')
            ->setShowEmptyValue(true);

        $this->addField()
            ->setFieldName('CisloUctuProZalohu')->setCaption('Cislo_uctu')
            ->setCode('field-cislouctuprozalohu')
            ->setCaptionColumnClass('col-6 col-sm-9 col-md-3 caption')->setDataColumnClass('col-6 col-sm-3')
            ->setRowId('hlavickasc_zalohaucet')->setRowClass('row row-zaloha-sc')
            ->setMaxLength(30);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('MenaUctuProZalohu')->setCaption('Vedeny_v_mene')
            ->setCode('field-menauctuprozalohu')
            ->setCaptionColumnClass('col-6 col-sm-9 col-md-3 caption')->setDataColumnClass('col-6 col-sm-3')
            ->setNewRow(false)
            ->setSelectSql(
                'SELECT KodMeny_ISO, KodMeny_ISO AS Popis FROM Mena WHERE %FILTER% ORDER BY %ORDER%',
                'Aktivni = 1', 'KodMeny_ISO')
            ->setShowEmptyValue(true);

        $this->addField()
            ->setFieldName('IBANUctuProZalohu')->setCaption('IBAN')
            ->setCode('field-pozadovanazalohaiban')
            ->setCaptionColumnClass('col-6 col-sm-9 col-md-3 caption')->setDataColumnClass('col-6 col-sm-3')
            ->setRowId('hlavickasc_zalohaiban')->setRowClass('row row-zaloha-sc')
            ->setMaxLength(40);

        $this->addField()
            ->setFieldName('SWIFTUctuProZalohu')->setCaption('SWIFT')
            ->setCode('field-pozadovanazalohaswift')
            ->setCaptionColumnClass('col-6 col-sm-9 col-md-3 caption')->setDataColumnClass('col-6 col-sm-3')
            ->setNewRow(false)
            ->setMaxLength(40);

        // predpokladane vydaje
        $vydaje = PredpokladanyVydajSC::nactiPredpokladaneVydaje($this->database, $this->getRecordPrimaryKeyValue());
        if ($vydaje) {
            // group, section header a nadpisy sloupecku
            $this->addField(FormField::FIELD_TYPE_GROUP)
                ->setCode('group-predpokladane-naklady')
                ->setFieldId('hlavickasc_predpokladanenaklady');

            $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                ->setCode('header-predpokladane-naklady')
                ->setRowId('hlavickasc_predpokladanenakladycaption')
                ->setCaption('Predpokladane_naklady');

            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setCode('nadpis-pn-pocet-dnu')
                ->setText(TextManager::_('NSC_Pocet_dnu'))
                ->setRowClass('row d-xs-none')
                ->setDataColumnClass('col-3');
            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setCode('nadpis-pn-castka')
                ->setText(TextManager::_('Castka'))->setCaption(false)->setNewRow(false)
                ->setDataColumnClass('col-3');
            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setCode('nadpis-pn-mena')
                ->setText(TextManager::_('Mena'))->setCaption(false)->setNewRow(false)
                ->setDataColumnClass('col-3');

            // predpokladane naklady
            foreach ($vydaje as $predpokladanyVydaj) {
                // nazev typu radku uctovani - napr "ubytovani"
                $this->fieldsPNPopis[$predpokladanyVydaj->typ] = $this->addField(FormField::FIELD_TYPE_TEXT)
                    ->setCode("pn-typ-{$predpokladanyVydaj ->typ}")->setCaption(false)
                    ->setText(TextManager::_("db_typructovanislcesty_popis_{$predpokladanyVydaj ->typ}", $predpokladanyVydaj->popisTypu))
                    ->setCaption(false)
                    ->setDataColumnClass('col-12 col-sm-3 caption');

                // nadpisy pro mobilni zobrazeni
                $this->addField(FormField::FIELD_TYPE_TEXT)
                    ->setCode("nadpis-pn-pocet-dnu-{$predpokladanyVydaj ->typ}")
                    ->setCaption(false)->setNewRow(false)
                    ->setText(TextManager::_('NSC_Pocet_dnu'))
                    ->setDataColumnClass('mobile-only col-4');
                $this->addField(FormField::FIELD_TYPE_TEXT)
                    ->setCode("nadpis-pn-castka-{$predpokladanyVydaj ->typ}")
                    ->setCaption(false)->setNewRow(false)
                    ->setText(TextManager::_('Castka'))
                    ->setDataColumnClass('mobile-only col-4');
                $this->addField(FormField::FIELD_TYPE_TEXT)
                    ->setCode("nadpis-pn-mena-{$predpokladanyVydaj ->typ}")
                    ->setCaption(false)->setNewRow(false)
                    ->setText(TextManager::_('Mena'))
                    ->setDataColumnClass('mobile-only col-4');

                // hodnoty
                $this->fieldsPNPocetDnu[$predpokladanyVydaj->typ] = $this->addField(FormField::FIELD_TYPE_INT)
                    ->setFieldName("*PN_PocetDnu_{$predpokladanyVydaj ->typ}")
                    ->setCaption(false)->setNewRow(false)
                    ->setCode("pn-pocet-dnu-{$predpokladanyVydaj ->typ}")
                    ->setDataColumnClass('col-4 col-sm-3')
                    ->setDefaultValue($predpokladanyVydaj->pocetDnu);

                $this->fieldsPNCastka[$predpokladanyVydaj->typ] = $this->addField(new PredpokladanyVydajCastkaFormField($predpokladanyVydaj))
                    ->setFieldName("*PN_Castka_{$predpokladanyVydaj ->typ}")
                    ->setCaption(false)->setNewRow(false)
                    ->setCode("pn-castka-{$predpokladanyVydaj ->typ}")
                    ->setDataColumnClass('col-4 col-sm-3')
                    ->setFormat('number2');
                //->setDefaultValue($predpokladanyVydaj ->castka);

                $this->fieldsPNMena[$predpokladanyVydaj->typ] = $this->addField(FormField::FIELD_TYPE_SELECT)
                    ->setFieldName("*PN_Mena_{$predpokladanyVydaj ->typ}")
                    ->setCaption(false)->setNewRow(false)
                    ->setCode("pn-mena-{$predpokladanyVydaj ->typ}")
                    ->setDataColumnClass('col-4 col-sm-3')
                    ->setShowEmptyValue(true)
                    ->setDefaultValue($predpokladanyVydaj->kodMeny)
                    ->setSelectSql(
                        'SELECT KodMeny_ISO, KodMeny_ISO AS Popis FROM Mena WHERE %FILTER% ORDER BY %ORDER%',
                        'Aktivni = 1', 'KodMeny_ISO');

            }

            $this->addField(FormField::FIELD_TYPE_GROUP_END);
        }

        // poznamky
        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('header-poznamka')->setCaption('Poznamka_sc')
            ->setRowId('hlavickasc_poznamkacaption');

        $this->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka')
            ->setCode('field-poznamka')
            ->setRowId('hlavickasc_poznamka')
            ->setCaptionColumnClass('col-3 d-xs-none');

        $caption_p2 = TextManager::_('Poznamka_sc_2', '');
        $this->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka2')
            ->setCode('field-poznamka2')
            ->setRowId('hlavickasc_poznamka2')
            ->setCaption($caption_p2)
            ->setVisible($caption_p2 !== '');

        $caption_p3 = TextManager::_('Poznamka_sc_3', '');
        $this->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka3')
            ->setCode('field-poznamka3')
            ->setRowId('hlavickasc_poznamka3')
            ->setCaption($caption_p3)
            ->setVisible($caption_p3 !== '');

        // nastaveni kopie
        if ($this->formAction === 'copy' || ($_POST['copyrecordid'] ?? false)) {
            $this->addField(FormField::FIELD_TYPE_HIDDEN)
                ->setFieldName('*copyrecordid')
                ->setDefaultValue($_POST['copyrecordid'] ?? $_GET['record']);

            $this->kopirovatRadkyField = $this->addField(FormField::FIELD_TYPE_CHECK)
                ->setFieldName('*kopirovatradky')->setCaption('Kopirovat_radky_sc')
                ->setCode('field-kopirovat-radky')
                ->setFieldId('kopirovatradky')->setRowId('hlavickasc_kopirovatradky')
                ->setDefaultValue(1);

            $this->kopirovatUctovaniField = $this->addField(FormField::FIELD_TYPE_CHECK)
                ->setFieldName('*kopirovatuctovani')->setCaption('Kopirovat_uctovani_sc')
                ->setCode('field-kopirovat-uctovani')
                ->setFieldId('kopirovatuctovani')->setRowId('hlavickasc_kopirovatuctovani')
                ->setDefaultValue(0);
        }
    }


    public function getIdPP()
    {
        return $this ->sluzebniCesta
            ? $this ->sluzebniCesta ->f_PP ->idPP
            : ($this->osobaField->pracovniPomer->idPP ?? null);
    }

    /**
     * @return string
     */
    public function getSqlFromPart()
    {
        return
            'SluzebniCesta JOIN PracovniPomer' .
            ' ON PracovniPomer.RC = SluzebniCesta.RC  AND  PracovniPomer.OsobniCislo = SluzebniCesta.OsobniCislo  AND  PracovniPomer.DatumNastupu = SluzebniCesta.DatumNastupu';
    }


    public function addField($field = FormField::FIELD_TYPE_EDIT)
    {
        if (!$field instanceof FormField) {
            $field = new SluzebniCestaFormField($field);
        }

        return parent::addField($field);
    }


    protected function zkontrolujPravoNaEditaci($idSC)
    {
        $this->sluzebniCesta = SluzebniCesta::nactiSCProEditaci($this->database, $idSC);

        if (!$this->sluzebniCesta->jePravoNaSluzebniCestu('sluzebnicesty')) {
            throw new NeniPravoException();
        }

        if (!$this->sluzebniCesta->lzeEditovatHlavicku()) {
            throw new \Exception(TextManager::_('Sluzebka_je_jiz_schvalena'));
        }
    }

    public function loadData($usePostValues)
    {
        $this->idSC = $this->getRecordPrimaryKeyValue();
        parent::loadData($usePostValues);

        if ($this->idSC && !$this->isInsert()) {
            $this ->zkontrolujPravoNaEditaci($this->idSC);
        } else if ($this->idSC) {
            // pri kopirovani zkontroluje, jestli ma uzivatel pravo na zobrazeni kopirovane SC
            $this ->sluzebniCesta = SluzebniCesta::nactiSluzebniCestu($this->database, $this->idSC);
            if (!$this ->sluzebniCesta->lzeZobrazitSluzebniCestu()) {
                throw new NeniPravoException();
            }
        }

        if ($this ->sluzebniCesta  &&  $this ->sluzebniCesta ->f_PP  &&  $this ->rozdeleniField) {
            $this ->rozdeleniField ->pracovniPomer = $this ->sluzebniCesta ->f_PP;
        }

//        if ($this ->data) {
//
//
//            $datumCasOd = $this ->database ->stringToTimestamp($this ->data['field-datumcasod']);
//            $datumCasDo = $this ->database ->stringToTimestamp($this ->data['field-datumcasdo']);
//
//            // kontrola prava na PP (jestli ji vubec lze zobrazit)
//            if (!SluzebniCestyUtils::jePravoNaSluzebniCestu('sluzebnicesty',
//                (int)$this ->data['field-osoba'], (int)$this ->data['field-zahranicni'],
//                $datumCasOd,
//                $datumCasDo)
//            ) {
//                throw new NeniPravoException();
//            }
//
//            if (!SluzebniCestyUtils::lzeEditovatHlavicku(
//                (int)$this ->data['field-osoba'], (int)$this ->data['field-zahranicni'],
//                $this ->database ->stringToTimestamp($this ->data['field-datumcasod']),
//                $this ->database ->stringToTimestamp($this ->data['field-datumcasdo']),
//                (int)$this ->data[2])
//            ) {
//                throw new \Exception(\Ron\ronText('Sluzebka_je_jiz_schvalena'));
//            }
//
//            if (!$this ->isInsert()) {
//                SluzebniCestyUtils::lzeEditovatSC($this ->database,
//                    (int)$this ->data['field-osoba'], (int)$this ->data['field-zahranicni'],
//                    $datumCasOd, $datumCasDo,
//                    $this ->data[3],    // zauctovano
//                    [1 => $this ->data[4], 2 => $this ->data[5], 3 => $this ->data[6], 4 => $this ->data[7]],
//                    true);
//
//            }
//        }

        // doplni SQL pro vyber prirazenych vozidel osoby
        //$this->registracniZnackaField->setSelectSql(self::getSqlVozidlaOsoby($this->database, $this->data['field-osoba'] ?? null));
    }


    /**
     * @param Database $database
     * @param int $idPP
     * @return Option[]
     */
    public static function nactiVozidlaOsoby($database, $idPP)
    {
        return (new OptionLoader($database))
            ->setSql(RegistracniZnackaField::getSqlVozidlaOsoby($database, $idPP))
            ->setEmptyValue(' ')
            ->loadData();
    }


    /**
     * @param DBRecord $rec
     * @throws \Exception
     */
    public function checkRequiredFieldsSC(DBRecord $rec)
    {
        if ($rec['DatumCasDo'] < $rec['DatumCasOd']) {
            throw new RonException(TextManager::_('Pocatecni_datum_vetsi_nez_koncove'), 'DatumCasDo');
        }

        // kontrola zadani cile pro ucel, ktery ma nastaveno povinne zadani cile
        if (!$rec['Cil'] && $rec['Ucel'] && SluzebniCestyUtils::jePovinnyCilProUcel($this->database, $rec['Ucel'])) {
            throw new RonException(TextManager::_('Neni_zadan_cil_sc'), 'Cil');
        }
    }


    /**
     * @param DBRecord $rec
     * @param bool $insert
     * @return void
     * @throws DBException
     * @throws NeniPravoException
     * @throws \Exception
     */
    public function createDBRecordFields($rec, $insert)
    {
        // prida pole pro pracovni pomer (RC, OsobniCislo, DatumNastupu)
        $fields = $this->osobaField->createPPFields($this->database);
        if ($fields) {
            foreach ($fields as $field) {
                $rec->addDBRecordField($field);
            }
        }

        parent::createDBRecordFields($rec, $insert);

        if ($this->statField->fieldType === FormField::FIELD_TYPE_HIDDEN && $this->statField->visible) {
            // automaticke nastaveni podle osoby
            $field = $rec->findFieldByName('Stat');
            if ($field) {
                $vychoziStat = SluzebniCestyUtils::getVychoziStatOsoby($this->database, $this->osobaField->pracovniPomer, $rec['DatumCasOd']);
                if ($vychoziStat) {
                    $field->value = $vychoziStat;
                }
            }
        }

        if ($this->cilovyStatField->visible && !$this->cilovyStatField->readOnly && !$rec['Zahranicni']) {
            // tuzemska zahranicni cesta - nastavi cilovy stat stejny jako vychozi stat
            $field = $rec->findFieldByName('CilovyStat');
            if ($field) {
                $field->setNewValue($rec['Stat']);
            }
        }

        if ($this->cilovyStatField->visible &&
            $this->sazbaKapesnehoField->visible && $this->sazbaKapesnehoField->fieldType === FormField::FIELD_TYPE_CONSTANT &&
            RonPortalSettings::getIntValue(self::SETTINGS_SAZBA_KAPESNEHO_PODLE_STATU)) {
            if (!$this->sazbaKapesnehoField->readOnly || $this->isInsert()) {
                // sazba kapesneho se ma urcit automaticky podle ciloveho statu
                $stat = $rec['CilovyStat'];
                if ($stat) {
                    $sazba = SluzebniCestyUtils::sazbaKapesnehoProStat($this->database, $stat);
                    if (isset($sazba) && $field = $rec->findFieldByName('SazbaKapesneho')) {
                        $field->setNewValue($sazba);
                    }
                }
            }
        }
        // Cilovy stat - pro tuzemske SC se v predchozi verzi nastavil stejny jako vychozi stat

        // Urcene stravne - zatim readonly. Doplni se automaticky az po nacteni zaznamu
        $rec->addFloatField('UrceneDenniStravne', true);
        $rec->addStringField('UrceneDenniStravneMena', true);

        // Pole pro schvaleni nadrizenym (aby je mohl pripadne vynulovat)
        $f = $rec->addIntField('SchvalenoNadrizenym', true, 0);
        $f->updateLog = true;
        $f = $rec->addIntField('SchvalenoNadrizenym2', true, 0);
        $f->updateLog = true;
        $f = $rec->addIntField('SchvalenoNadrizenym3', true, 0);
        $f->updateLog = true;
        $f = $rec->addIntField('SchvalenoNadrizenym4', true, 0);
        $f->updateLog = true;

        $this->checkRequiredFieldsSC($rec);

        // cas zacatku a konce SC se ma urcit automaticky podle radku sluzebni cesty
        if ($rec['Id'] && RonPortalSettings::getIntValue(SluzebniCestyUtils::SETTINGS_CASY_AUTOMATICKY)) {
            $casy = SluzebniCestyUtils::vratCasyPodleRadkuSC($this->database, $rec['Id']);
            if ($casy) {
                if (DateUtils::startOfTheDay($casy['DatumCasOd']) === DateUtils::startOfTheDay($rec['DatumCasOd'])) {
                    $rec['DatumCasOd'] = $casy['DatumCasOd'];
                }

                if (DateUtils::startOfTheDay($casy['DatumCasDo']) === DateUtils::startOfTheDay($rec['DatumCasDo'])) {
                    $rec['DatumCasDo'] = $casy['DatumCasDo'];
                }
            }
        }

        // kontroly provede az v before... - protoze nektere hodnoty se jeste budou dodatecne upravovat
    }


    /**
     * @param string $kodTypuPrava
     * @param DBRecord $rec
     * @return bool
     * @throws \Exception
     */
    private function jePravoNaZaznam(string $kodTypuPrava, DBRecord $rec)
    {
        return SluzebniCestyUtils::jePravoNaSluzebniCestu($kodTypuPrava,
            $this->osobaField->pracovniPomer->idPP,
            $rec['Zahranicni'],
            $rec['DatumCasOd'], $rec['DatumCasDo']
        );
    }


    /**
     * @param DBRecord $rec
     * @return void
     */
    private function upravitSchvaleniNadrizeneho(DBRecord $rec)
    {
        if (RonPortalSettings::getIntValue(SluzebniCestyUtils::SETTINGS_EDITACE_SCHVALENE_NADRIZENYM, 1) == 2) {
            // pri editaci se vymaze schvaleni nadrizeneho (pokud needituje primo nadrizeny)

            if (!$this->jePravoNaZaznam('slcesty_nadriz4', $rec)) {
                // uzivatel nema pravo na schvaleni nadrizenym 4 => schvaleni 4 se zrusi ...
                $rec->findFieldByName('SchvalenoNadrizenym4')->setNewValue(0);

                // ... a zkontroluji se i dalsi urovne
                if (!$this->jePravoNaZaznam('slcesty_nadriz3', $rec)) {
                    // zrusi schvaleni urovni 3
                    $rec->findFieldByName('SchvalenoNadrizenym3')->setNewValue(0);

                    if (!$this->jePravoNaZaznam('slcesty_nadriz2', $rec)) {
                        // zrusi schvaleni urovni 2
                        $rec->findFieldByName('SchvalenoNadrizenym2')->setNewValue(0);

                        if (!$this->jePravoNaZaznam('slcesty_nadriz', $rec)) {
                            // zrusi schvaleni urovni 1
                            $rec->findFieldByName('SchvalenoNadrizenym')->setNewValue(0);
                        }
                    }
                }
            }
        }
    }


    /**
     * Provede dodatecne kontroly pred zapsanim zaznamu.
     * @param DBRecord $rec
     * @throws DBException
     * @throws NeniPravoException
     * @throws RonException
     * @throws \Exception
     */
    private function provedKontrolySC($rec)
    {
        // kontrola prava
        Prava::nastavObdobiProPrava($rec['DatumCasOd'], $rec['DatumCasDo']);

        $pravo = Prava::vratPrava($this->osobaField->prava, $rec['Zahranicni'] ? $this->osobaField->prava . '_Z' : null); // pro zahranicni SC pouzije prednostne pravo se sufixem '_Z'
        if (!$pravo->lzeZobrazitPP($this->osobaField->pracovniPomer->idPP, $rec['DatumCasOd'], $rec['DatumCasDo'])) {
            throw new NeniPravoException();
        }

        // kontrola uzavreneho obdobi
        if (!RonPortalSettings::getIntValue(SluzebniCestyUtils::SETTINGS_EDITACE_V_UZAVRENEM)
            && Obdobi::jeObdobiUzavreno($this->database, 'ADS', $rec['DatumCasOd'], $rec['DatumCasDo'])) {
            throw new \Exception(TextManager::_('Obdobi_uzavreno'));
        }

        // kontrola vkladani SC maximalni pocet dnu zpetne
        $pocetDnu = RonPortalSettings::getIntValue(self::SETTINGS_VKLADANI_POCET_DNU);
        if ($pocetDnu) {
            $fieldDatum = $rec->findFieldByName('DatumCasOd');

            if ($fieldDatum->isChanged()) {
                // je zakazano vkladat stare SC - zkontroluje, jestli uzivatel neposunul datum (z nebo do obdobi pred ...)
                $pomDatum = $fieldDatum->value;
                if ($fieldDatum->originalValue && $fieldDatum->originalValue < $pomDatum) {
                    $pomDatum = $fieldDatum->originalValue;
                }

                if (DateUtils::incDay($pomDatum, $pocetDnu) < DateUtils::startOfTheDay(time())) {
                    throw new RonException(sprintf(TextManager::_('Nelze_editovat_zacatek_sc'), $pocetDnu), 'DatumCasOd');
                }
            }
        }

        SluzebniCestyUtils::kontrolaPrekryvaniSC($this->database, $rec['RC'], $rec['DatumCasOd'], $rec['datumCasDo'], $rec['Id']);

        // Maximalni denni stravne urci podle vychoziho a ciloveho statu (vychozi ale nemusi byt editovatelny, takze se to musi provest az po nacteni puvodniho zaznamu!)
        if ($this->urceneDenniStravneField->visible) {
            try {
                $pomA = SluzebniCestyUtils::vratMaximalniDenniStravneProStat($this->database, $rec['DatumCasOd'], $rec['Zahranicni'] ? $rec['CilovyStat'] : $rec['Stat'], $rec['Stat']);

                if ($pomA) {
                    $rec->findFieldByName('UrceneDenniStravne')->setNewValue($pomA['sazba']);
                    $rec->findFieldByName('UrceneDenniStravneMena')->setNewValue($pomA['mena']);
                }
            } catch (DBException $e) {
                throw $e;
            } catch (\Exception $e) {
                // nedela nic (sazba nenalezena)
            }
        }

        $this->upravitSchvaleniNadrizeneho($rec);
    }


    function beforeInsertRecord($rec)
    {
        $idOriginal = (int)($_POST['copyrecordid'] ?? null);
        if ($idOriginal && $this->kopirovatRadkyField && $this->kopirovatRadkyField->visible && $this->kopirovatRadkyField->getPostValue(false)) {
            if (RonPortalSettings::getIntValue(SluzebniCestyUtils::SETTINGS_CASY_AUTOMATICKY)) {
                // upravit casy podle kopirovanych radku
                $this->upravCasyPodleKopirovaneSC($rec, $idOriginal);
            }
        }

        $this->provedKontrolySC($rec);
        // TODO - stat podle rozdeleni

        return parent::beforeInsertRecord($rec);
    }


    /**
     * @param DBRecord $rec
     * @param int $idKopirovaneSC
     * @return void
     * @throws DBException
     */
    function upravCasyPodleKopirovaneSC(DBRecord $rec, $idKopirovaneSC)
    {
        $params = $this->database->createParameters();
        $sql =
            'SELECT SluzebniCesta.DatumCasOd, MIN(RadekSluzebniCesty.DatumCasOd), MAX(RadekSluzebniCesty.DatumCasOd)' .
            ' FROM SluzebniCesta, RadekSluzebniCesty' .
            ' WHERE RadekSluzebniCesty.SluzebniCesta_ID = SluzebniCesta.Id' .
            ' AND SluzebniCesta.Id = ' . $params->addParameter($idKopirovaneSC) .
            ' GROUP BY (SluzebniCesta.DatumCasOd)';

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor->close();

        if (!$row || !$row[1]) { // SC nenalezena, nebo nema zadne radky
            return;
        }

        $casSCOd = $this->database->stringToTimestamp($row[0]);
        $casRadkuOd = $this->database->stringToTimestamp($row[1]);
        $casRadkuDo = $this->database->stringToTimestamp($row[2]);

        $fieldOd = $rec->findFieldByName('DatumCasOd');

        // vypocita pocet dnu mezi starou a novou SC
        $dateAdd = DateUtils::daysBetween(DateUtils::startOfTheDay($casSCOd), DateUtils::startOfTheDay($fieldOd->value));

        if (DateUtils::startOfTheDay($casSCOd) === DateUtils::startOfTheDay($casRadkuOd)) {
            // prvni radek puvodni SC je ve stejny den jako zacatek puvodni SC => nastavim cas nove SC podle casu radku
            $fieldOd->setNewValue(DateUtils::startOfTheDay($fieldOd->value) + DateUtils::timeOf($casRadkuOd));
        }

        $fieldDo = $rec->findFieldByName('DatumCasDo');

        // prepocte cas posledniho radku pro novou sluzebni cestu
        $casDo = DateUtils::incDay($casRadkuDo, $dateAdd);
        if ($casDo > $fieldDo->value) {
            // posune cas konce podle posledniho radku
            $fieldDo->setNewValue($casDo);
        } else if (DateUtils::startOfTheDay($casDo) === DateUtils::startOfTheDay($fieldDo->value)) {
            $fieldDo->setNewValue($casDo);
        }
    }


    function beforeUpdateRecord($rec)
    {
        $this ->zkontrolujPravoNaEditaci($rec['Id']);

        $this->provedKontrolySC($rec);

        return parent::beforeUpdateRecord($rec);
    }


    /**
     * Ulozi predpokladane naklady zadane uzivatelem
     * @param $recSC
     * @throws DBException
     */
    function zapisPredpokladaneNaklady($recSC)
    {
        foreach ($this->fieldsPNCastka as $kodTypu => $fieldCastka) {
            $fieldPocetDnu = $this->fieldsPNPocetDnu[$kodTypu];
            $fieldMena = $this->fieldsPNMena[$kodTypu];

            if ($fieldCastka->visible && !$fieldCastka->readOnly || $fieldPocetDnu->visible && !$fieldPocetDnu->readOnly) {
                $rec = new RonPortalDBRecord($this->database, 'PredpokladaneNakladySC', $this);

                $rec->addIntField('SluzebniCesta_ID', true, $recSC['Id']);
                $rec->addIntField('Typ', true, $kodTypu);
                $rec->addLogText(' ' . $this->fieldsPNPopis[$kodTypu]->text); // jen pro zalogovani

                $rec->addIntField('PocetDnu', $fieldPocetDnu->readOnly, $fieldPocetDnu->getPostValue($this->utf8));
                $rec->addFloatField('Castka', $fieldCastka->readOnly, $fieldCastka->getPostValue($this->utf8));
                $rec->addStringField('KodMeny', $fieldMena->readOnly, 10, $fieldMena->getPostValue($this->utf8));

                $params = $this->database->createParameters();
                $rec->setWhere(
                    'SluzebniCesta_ID = ' . $params->addParameter($recSC['Id']) .
                    ' AND Typ = ' . $params->addParameter($kodTypu),
                    $params);

                $exists = $rec->loadRecord();

                if ($exists) {
                    if ($rec['PocetDnu'] !== null || $rec['Castka'] !== null) {
                        $rec->updateRecord();
                    } else {
                        $rec->deleteRecord();
                    }
                } else {
                    if ($rec['PocetDnu'] != null || $rec['Castka'] != null) {
                        $rec->insertRecord();
                    }
                }
            }
        }
    }


    /**
     * @param DBRecord $recNew
     * @throws DBException
     */
    public function kopirujObsah(DBRecord $recNew)
    {
        $idOriginal = (int)($_POST['copyrecordid'] ?? null);
        $idNew = (int)$recNew['Id'];

        if (!$idOriginal) {
            return;
        }

        // zjisti pocatek puvodni a nove sluzebni cesty
        $params = $this->database->createParameters();
        $sql =
            'SELECT DatumCasOd FROM SluzebniCesta ' .
            ' WHERE Id = ' . $params->addParameter($idOriginal);

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor->close();

        if (!$row) {
            MessageManager::addErrorMessage("Chyba pri kopirovani radku - nelze nacist puvodni zaznam (Id=$idOriginal)");
            return;
        }

        $datumCasPuvodni = $this->database->stringToTimestamp($row[0]);
        $datumCasNovy = $recNew['DatumCasOd'];

        if ($this->kopirovatRadkyField->visible && $this->kopirovatRadkyField->getPostValue(false)) {
            if (RonPortalSettings::getIntValue(SluzebniCestyUtils::SETTINGS_CASY_AUTOMATICKY)) {
                SluzebniCestyUtils::kopirujRadky($this->database, $idOriginal, $idNew, DateUtils::startOfTheDay($datumCasNovy) - DateUtils::startOfTheDay($datumCasPuvodni), $this);
            } else {
                SluzebniCestyUtils::kopirujRadky($this->database, $idOriginal, $idNew, $datumCasNovy - $datumCasPuvodni, $this);
            }
        }

        if ($this->kopirovatUctovaniField->visible && $this->kopirovatUctovaniField->getPostValue(false)) {
            SluzebniCestyUtils::kopirujRadkyUctovani($this->database, $idOriginal, $idNew, DateUtils::startOfTheDay($datumCasNovy) - DateUtils::startOfTheDay($datumCasPuvodni), $this);
        }
    }


    function afterInsertRecord($rec)
    {
        $this->idSC = $rec['Id'];

        MessageManager::getInstance()->addMessage(TextManager::_('Vlozeni_sc_OK'));

        try {
            $this->zapisPredpokladaneNaklady($rec);

            if (isset($_POST['copyrecordid'])) {
                // kopie jine sluzebni cesty => zkopiruje jeji radky
                $this->kopirujObsah($rec);
            }

            return parent::afterInsertRecord($rec);
        } catch (DBException $e) {
            throw $e;
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
            return true;
        }
    }


    function afterUpdateRecord($rec)
    {
        $this->idSC = $rec['Id'];
        $this->zapisPredpokladaneNaklady($rec);
        return parent::afterUpdateRecord($rec);
    }
}
