<?php

namespace RonPortal\SluzebniCesty;

use Ron\DB\Database;
use Ron\Http\Csrf;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Redirect;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;

class VozidlaController extends JsonController
{
    /** @var Database */
    private $database;


    /**
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    private function showForm()
    {
        $f = new VozidloFormular($this ->database);
        if ($f ->handleAction()) {
            return new Redirect(null, '?menuid=vozidla');
        }

        return $f;
    }


    /**
     * Overi, jestli ma uzivatel pravo na zobrazeni vozidla se zadanou RZ (tj. musi byt prirazeno zamestnanci, na ktereho ma uzivatel pravo)
     * @param string $rz
     * @param bool $editace     Urcuje, jestli se jedna o editaci
     * @return bool
     */
    private function jePravoNaRZ($rz, $editace)
    {
        return true;
    }


    /**
     * @param JsonOutputObject $output
     * @return void
     * @throws \Exception
     */
    private function vozidloOsoby($output)
    {
        $rz = $_GET['rz'] ?? '';
        $f = new VozidloOsobyDialog($this ->database, $rz);
        $output ->addData('data', $f ->getJsonData());
    }


    private function zapisVozidloOsoby($output)
    {
        $rz = $_POST['RegistracniZnacka'] ?? '';
        $f = new VozidloOsobyDialog($this ->database, $rz);
        if (!$f ->handleAction()) {
            $output ->addMessages(MessageManager::getInstance());
        }
    }

    function handleJsonAction($action, $output)
    {
        switch ($_GET['action'] ?? '') {
            case 'vozidlosoby' :
                $this ->vozidloOsoby($output);
                break;

            case 'zapisvozidloosoby' :
                $this ->zapisVozidloOsoby($output);
                break;
        }
    }


    private function delete()
    {
        try {
            $prava = Prava::vratPrava('vozidla');
            if (!$prava ->priznak) {
                // uzivatel nemuze mazat zaznamy
                throw new NeniPravoException();
            }

            $registracniZnacka = $_GET['record'] ?? '';
            Csrf::validateCSRFGetParam('vozidlo_delete_' . $registracniZnacka);

            if (!VozidloFormular::jePravoNaVozidlo($this ->database, $registracniZnacka, true)) {
                // vozidlo neni prirazeno osobe, na kterou by mel uzivatel pravo
                throw new NeniPravoException();
            }

            // kontrola, jestli neni vozidlo prirazeno osobe, na kterou nemam pravo
            if ($prava ->prava !== Prava::PRAVA_VSE) {
                $params = $this->database->createParameters();
                $sql =
                    'SELECT DISTINCT RegistracniZnacka' .
                    ' FROM VozidloOsoby, PracovniPomer PP1' .
                    ' WHERE RegistracniZnacka = ' . $params->addParameter($registracniZnacka) .
                    '   AND PP1.RC = VozidloOsoby.RC' .
                    '   AND VozidloOsoby.PlatnostOd <= ' . $params->addDateParameter(time()) .
                    '   AND (VozidloOsoby.PlatnostDo >= ' . $params->addDateParameter(time()) . ' OR VozidloOsoby.PlatnostDo IS NULL)' .
                    '   AND PP1.DatumNastupu <= ' . $params->addDateParameter(time()) .
                    '   AND (PP1.DatumUkonceni >= ' . $params->addDateParameter(time()) . ' OR PP1.DatumUkonceni IS NULL)';

                $filtr = $prava ->sqlFiltr('PP2', time(), time(), $params);
                $sql .=
                    '   AND NOT EXISTS(' .
                    '     SELECT * FROM PracovniPomer PP2' .
                    '     WHERE PP2.RC = PP1.RC  AND  PP2.OsobniCislo = PP1.OsobniCislo  AND  PP2.DatumNastupu = PP1.DatumNastupu' .
                    '     AND (' . $filtr . ')' .
                    ')' .
                    "\n";

                $cursor = $this ->database ->select($sql, $params);
                if ($cursor ->fetch()) {
                    throw new \Exception(TextManager::_('Vozidlo_je_prirazeno_dalsim_osobam'));
                }
            }


            $rec = new RonPortalDBRecord($this ->database, 'Vozidlo', $this);
            $rec ->addStringField('RegistracniZnacka');
            $rec ->addStringField('Popis');
            $rec ->addIntField('Aktivni');
            $rec ->addStringField('DruhPHM');
            $rec ->addFloatField('Spotreba');
            $rec ->addIntField('Soukrome');
            $rec ->addIntField('Jednostope');
            $rec ->addIntField('PovolenoSC');
            $rec ->addIntField('PovolenoSPZ');
            $rec ->addMemoField('Poznamka');

            $params = $this ->database ->createParameters();
            $rec ->setWhere(
                'RegistracniZnacka = ' . $params ->addParameter($registracniZnacka),
                $params);

            $rec ->setXmlEvents('VOZIDLO_BEFORE_POST', 'VOZIDLO_AFTER_POST');

            $rec ->deleteRecord();
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        return new Redirect(null, '?menuid=vozidla');
    }


    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('sluzebnicesty.ini');
        HtmlHeaders::getInstance() ->setAppTitle(TextManager::_('Vozidla'));
        HtmlHeaders::getInstance() ->addCss('css/sc.css');
        Prava::nastavObdobiProPrava(time());

        if ($_POST['formaction'] ?? $_GET['formaction']  ?? false) {
            return $this ->showForm();
        }

        switch ($_GET['action'] ?? '') {
            case 'delete' :
                return $this ->delete();

            case '' :
                break;

            default :
                return parent::handleAction($action);
        }

        $b = new VozidloBrowser($this ->database, 'vozidla');
        $b ->load();
        return $b;
    }
}
