<?php

/**
 * Pavel Rudol, RON Software
 *  11.2.2015
 * seznam osob generovanych do terminalu
 */
namespace RonPortal\Terminaly;

use Ron\Controls\BrowserField;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;


/**
 * Seznam osob generovanych do terminalu
 *
 * @author pavel.rudol
 */
class OsobyTerminaluBrowser extends RonPortalBrowser
{
    /** @var PravaNaOsoby */
    private $pravaZobrazit;
    /** @var string */
    private $terminal;
            
    function __construct($database, $terminal)
    {
        parent::__construct($database, 'menuid=terminaly&record=' . $terminal);

        Prava::nastavObdobiProPrava(time(), time());
        $this->pravaZobrazit = Prava::vratPrava('prohl_terminaly');
        $this->terminal = $terminal;
        
        $this
            ->setTitle('seznam_osob')
            ->setCode('browserosobyterminalu')
            ->setTableId('browserosobyterminalu');

        $this->sqlFromPart =
                'Terminal, Osoba, PracovniPomer, PrirazeniKarty, Karta, S_TypOsoby';
        
        $this->sqlWherePart =
                '(Osoba.RC = PracovniPomer.RC) '.
                'AND (PrirazeniKarty.RC = PracovniPomer.RC) '.
                'AND (PrirazeniKarty.OsobniCislo = PracovniPomer.OsobniCislo) '.
                'AND (PrirazeniKarty.DatumNastupu = PracovniPomer.DatumNastupu) '.
                'AND (Karta.kod = PrirazeniKarty.kodKarty) '.
                'AND ((Karta.blokovana = 0) or (Karta.blokovana is null)) '.
                'AND (S_TypOsoby.Kod = PracovniPomer.Zamestnanec)';                              
                
        $this->defaultSort = 'col-prijmeni';
        $this->distinct = true;

        $this->browserUrl = '?menuid=terminaly&action=osoby&record='.StringUtils::htmlTextConvert($this->terminal).'&';

        $this->addField(BrowserField::FIELD_TYPE_LINE);

        $this->addField()
            ->setSqlFieldName('Osoba.Titul') ->setHeaderText('Titul') ->setOrdering(true)
            ->setCode('col-titul');

        $this->addField()
            ->setSqlFieldName('Osoba.Prijmeni') ->setHeaderText('Prijmeni') ->setOrdering(true)
            ->setCode('col-prijmeni')
            ->setSortSQL('Osoba.Prijmeni, Osoba.Jmeno') ->setSortSQLDesc('Osoba.Prijmeni DESC, Osoba.Jmeno DESC');

        $this->addField()
            ->setSqlFieldName('Osoba.Jmeno') ->setHeaderText('Jmeno') ->setOrdering(true)
            ->setCode('col-jmeno');

        $this->addField()
            ->setSqlFieldName('PracovniPomer.OsobniCislo') ->setHeaderText('Osobni_cislo') ->setOrdering(true)
            ->setCode('col-osobnicislo');

        $this->addField()
            ->setSqlFieldName('PrirazeniKarty.KodKarty') ->setHeaderText('karta') ->setOrdering(true)
            ->setCode('col-kodkarty');

        $this->addField()
            ->setSqlFieldName('S_TypOsoby.Popis') ->setHeaderText('typPPomeru') ->setOrdering(true)
            ->setCode('col-zamestnanec');
    }


    function setFilterForm()
    {
        $this ->filterForm = new RonPortalFilterForm($this->database,
            [
                'menuid' => 'terminaly',
                'record' => $this ->terminal,
                'action' => 'osoby'
            ],
            true);

        $this ->filterForm ->addFilter(
            (new FilterFormFilterDatumOdDo($this ->filterForm))
                ->setCaption(TextManager::_('terminal_platnost') . ' ' . TextManager::_('Od'))
                ->setDatumOdDoLinks(['today']),
            true
        );
    }



    function getSQLFilter()
    {
        $params = $this->database->createParameters();

        $result = $this ->filterForm ->buildSqlFilter($params);

        $result .=
            ($result ? ' AND ' : '') .
            "(Terminal.kod = " . $params->addParameter($this->terminal) . ")";

        $datumOd = $this ->filterForm ->getOmezitObdobiOd() ?? time();
        $datumDo = $this ->filterForm ->getOmezitObdobiDo() ?? $datumOd;
        
        if ($this->pravaZobrazit->jsouNejakePrava()) {
            $result .=
                ' AND ( ( '.
                "    (PracovniPomer.DatumNastupu <= " . $params->addDateParameter($datumDo) . ")".
                " AND ((PracovniPomer.DatumUkonceni is null) or (PracovniPomer.DatumUkonceni >= " . $params->addDateParameter($datumOd) . "))".
                "\nAND (PrirazeniKarty.DatumPrideleni <= " . $params->addDateParameter($datumDo) . ")".
                " AND ((PrirazeniKarty.DatumVraceni is null) or (PrirazeniKarty.DatumVraceni >= " . $params->addDateParameter($datumOd) . "))".
                "\nAND (Karta.DatumPorizeni <= " . $params->addDateParameter($datumDo) . ")".
                " AND ((Karta.DatumVyrazeni is null) or (Karta.DatumVyrazeni >= " . $params->addDateParameter($datumOd) . ")) )".

                "\nAND ( (EXISTS (SELECT * FROM RozdeleniOsob RO, PovoleniPristupuRozdeleni PPR".
                "\nWHERE (PPR.Rozdeleni = RO.Rozdeleni)".
                ' AND (PPR.TypRozdeleni = RO.KodTypuRozdeleni)'.
                ' AND (PPR.Terminal = Terminal.kod)'.
                " AND (PPR.PlatnostOd <= " . $params->addDateParameter($datumDo) . ")".
                " AND ((PPR.PlatnostDo is null) or (PPR.PlatnostDo >= " . $params->addDateParameter($datumOd) . "))".
                ' AND (RO.KodTypuRozdeleni = 7) '.
                ' AND (RO.RC = PracovniPomer.RC) '.
                ' AND (RO.OsobniCislo = PracovniPomer.OsobniCislo) '.
                ' AND (RO.DatumNastupu = PracovniPomer.DatumNastupu) '.
                " AND (RO.PlatnostOd <= " . $params->addDateParameter($datumDo) . ") ".
                " AND ((RO.PlatnostDo is null) or (RO.PlatnostDo >= " . $params->addDateParameter($datumOd) . "))" .
                "\n))".

                "\nOR (EXISTS (SELECT * FROM PovoleniPristupuOsoby PPO".
                "\nWHERE (PPO.Terminal = Terminal.kod)".
                ' AND (PPO.RC = PracovniPomer.RC)'.
                ' AND (PPO.OsobniCislo = PracovniPomer.OsobniCislo)'.
                ' AND (PPO.DatumNastupu = PracovniPomer.DatumNastupu)'.
                " AND (PPO.PlatnostOd <= " . $params->addDateParameter($datumDo) . ")".
                " AND ((PPO.PlatnostDo is null) or (PPO.PlatnostDo >= " . $params->addDateParameter($datumOd) . "))".
                "\n)) ) )";
        } else {
            $result .= ' AND 1 = 0';
        }
                
        return new SqlStatement($result, $params);
    }
}
