<?php
/**
 * stanislav.elbl, 07.12.2015
 */

namespace RonPortal\Testy;

use Ron\DB\SQLUtils;
use Ron\DB\Database;
use RonPortal\RonPortalDBRecord;


class OtazkaDBRecord extends RonPortalDBRecord
{
    public $fPoradi;

    public $fPopis;
    public $fUpresnujiciPopis;
    public $fTypRozsahu;
    public $fBodovaVaha;
    public $fPovinnaOdpoved;
    public $fSpravnaHodnota;

    /**
     * @param Database $database
     * @param int $druhHodnoceni
     * @param int $temaCislo
     * @param string $kodKriteria
     * @param string $filtrKod Pokud je zadan, nastavi se where podminka
     * @param object|string $logDescription
     */
    public function __construct($database, $druhHodnoceni, $temaCislo, $kodKriteria, $filtrKod = null, $logDescription = 'Testy')
    {
        parent::__construct($database, 'HodnoticiKriterium', $logDescription);

        $this ->addField('DruhHodnoceni', 'int', true, 0, $druhHodnoceni); // readonly
        $this ->addField('TemaCislo', 'int', true, 0, $temaCislo);
        $this ->addField('KodKriteria', 'string', false, 10, $kodKriteria );

        $this ->fPoradi = $this ->addField('Poradi', 'int', true);

        $this ->fPopis = $this ->addField('Popis', 'string', true, 250);
        $this ->fUpresnujiciPopis = $this ->addField('UpresnujiciPopis', 'memo', true, 0);
        $this ->fTypRozsahu = $this ->addField('TypRozsahu', 'string', true, 10 );
        $this ->fBodovaVaha = $this ->addField('BodovaVaha', 'int', true, 1);
        $this ->fPovinnaOdpoved = $this ->addField('PovinnaOdpoved', 'int', true, 0, 0);
        $this ->fSpravnaHodnota = $this ->addField('SpravnaHodnota', 'int', true);

        if ( isset($filtrKod)  &&  $filtrKod != '' ) {
            $params = $database ->createParameters();
            $this ->setWhere(
                'DruhHodnoceni = ' . $params ->addParameter($druhHodnoceni) .
                ' AND TemaCislo = ' . $params ->addParameter($temaCislo) .
                ' AND KodKriteria = ' . $params->addParameter( $filtrKod ),
                $params);
        }
    }


    /**
     * @param Database $database
     * @param int $druhHodnoceni
     * @param int $temaCislo
     * @return int
     * @throws \Ron\DB\DBException
     */
    static function novePoradi($database, $druhHodnoceni, $temaCislo)
    {
        $params = $database ->createParameters();
        return SQLUtils::newSerialNumber(
            $database, 'HodnoticiKriterium', 'Poradi',
            'DruhHodnoceni = ' . $params ->addParameter((int)$druhHodnoceni) .
            ' AND TemaCislo = ' . $params ->addParameter((int)$temaCislo),
            $params
            );
    }
}
