<?php

namespace RonPortal\Testy;

use Ron\DateUtils;
use Ron\MessageManager;
use Ron\StringUtils;
use function Ron\ronText;

/**
 * Class PodrizenaOtazkaDotazniku
 */
class PodrizenaOtazkaDotazniku
{
    var $kod; // kod kriteria - pseudo kod slozeny z kodu kriteria a cisla otazky ve tvaru: kodKriteria#cislo

    var $popis;

    var $typOtazky;
    var $maxZnaku;

    var $retezecNastaveni;

    // odpoved
    var $slovniOdpoved;
    var $memoOdpoved;
    /** @var int|null */
    var $datumOdpoved;

    var $odpovedStr;  // zobrazena odpoved - nacita se z databaze nebo z POST

    var $htmlId;

    var $f_error; // priznak, ze je v odpovedi chyba
    var $odpovedVyplnena; // priznak vyplnene odpovedi


    /**
     * @param string $kod kod otazky (doplneny o prislusny index podotazky ve tvaru kodKriteria#index)
     * @param string $retezec
     * @param string $typOtazky mozne hodnoty jsou:
     *      LONGTEXT - memo pole
     *      TEXT - jednoradkovy text
     *      DATUM - datumove pole
     *      CISLO - jednoradkove pole pro zadani cisla
     * @param string $nadpis    nadpis pro zadani otazky
     * @param string $odpoved   vyplnena odpoved otazky
     * @param string $htmlId    id v HTML
     * @param int $pocetZnaku   maximalni pocet znaku u jednoradkoveho textu
     */
    function __construct($kod, $retezec, $typOtazky, $nadpis, $odpoved, $htmlId, $pocetZnaku = 0)
    {
        $this ->kod = $kod;

        $this ->retezecNastaveni = $retezec;

        $this ->popis = $nadpis;
        $this ->typOtazky = $typOtazky;
        $this ->maxZnaku = $pocetZnaku;

        $this ->htmlId = $htmlId;

        $this ->odpovedStr = $odpoved;
        $this ->odpovedVyplnena = trim($odpoved) != '';

        switch ($typOtazky) {
            case 'LONGTEXT' :
                $this ->memoOdpoved = $odpoved;
                break;

            case 'DATUM' :
                try {
                    $this->datumOdpoved = (trim($odpoved) == '')
                        ? null
                        : DateUtils::dateStringToTimestamp($odpoved);
                } catch (\Exception $e) {
                    MessageManager::addErrorMessage( $e ->getMessage() );
                    $this ->f_error = true;
                }
                break;

            case 'CISLO' :
                if (trim($odpoved) != ''  &&  !StringUtils::isFloat($odpoved)) {
                    MessageManager::addErrorMessage( ronText('Bad_number_format') . " ($kod)" );
                    $this ->f_error = true;
                }
                $this ->slovniOdpoved = $odpoved;
                break;

            case 'TEXT' :
                $this ->slovniOdpoved = $odpoved;
                break;
        }

    }


    /**
     * Nacte hodnotu z POST. Vraci true, pokud je vse OK, false pri chybe (spatny format datumu)
     * @return bool
     */
    function loadPost()
    {
        $this ->odpovedStr = $_POST[$this ->kod];
        $this ->odpovedVyplnena = trim($this ->odpovedStr) != '';

        switch ($this ->typOtazky) {
            case 'LONGTEXT' :
                $this ->memoOdpoved = $this ->odpovedStr;
                break;

            case 'TEXT' :
                $this ->slovniOdpoved = $this ->odpovedStr;
                break;

            case 'CISLO' :
                if (trim($this ->odpovedStr) != ''  &&  !StringUtils::isFloat($this ->odpovedStr)) {
                    MessageManager::addErrorMessage( ronText('Bad_number_format') );
                    $this ->f_error = true;
                    return false;
                }

                $this ->slovniOdpoved = $this ->odpovedStr;
                break;

            case 'DATUM' :
                if ($this ->odpovedStr) {
                    try {
                        $this ->datumOdpoved = DateUtils::datetimeStringToTimestamp($this->odpovedStr);
                    } catch (\Exception $e) {
                        MessageManager::addErrorMessage( $e ->getMessage() );
                        $this->f_error = true;
                        return false;
                    }
                }
                break;
        }

        return true;
    }


    /**
     * Zobrazi textareu pro zadani memo textu
     * @param $editace
     * @param $zobrazitTextovouOdpoved
     * @return string
     */
    function HTMLPodrizenaOtazka_Memo($editace, $zobrazitTextovouOdpoved)
    {
        $result = '<div class="textotazky">' . $this ->popis . '</div>';

        if ($editace) {
            return
                $result.
                '<div class="odpovedhodnoceni" ' . ($this->htmlId ? 'id="' . $this ->htmlId . '"' : '') . '>' .
                "<textarea name='{$this ->kod}' class='form-control' rows='8'>" . StringUtils::htmlTextConvert($this ->memoOdpoved) . "</textarea>" .
                "</div>\n";
        } else {
            return
                $result .
                '<div class="odpovedhodnoceni" ' . ($this->htmlId ? 'id="' . $this ->htmlId . '"' : '') . '>' .
                "<input type='hidden' name='{$this ->kod}' value='" . StringUtils::htmlTextConvert($this ->memoOdpoved) . "' />" .

                ($zobrazitTextovouOdpoved
                    ? "<div class='textovaodpoved'>" . StringUtils::longTextToHtml($this->memoOdpoved, true) . "</div>"
                    : ''
                ) .
                "</div>\n";
        }
    }


    /** Zobrazi edit pro zadani textu/cisla
     * @param $editace
     * @param $zobrazitTextovouOdpoved
     * @param bool $jenCisla
     * @return string
     */
    function HTMLPodrizenaOtazka_Edit($editace, $zobrazitTextovouOdpoved, $jenCisla = false)
    {
        $result = '<div class="textotazky">' . $this ->popis . '</div>';

        $classStr =
            'form-control' .
            ($jenCisla ? ' inputInt' : '') .
            ($this ->f_error ? ' error' : '');

        if ($editace) {
            return
                $result.
                '<div class="odpovedhodnoceni" ' . ($this->htmlId ? 'id="' . $this ->htmlId . '"' : '') . '>' .
                "<input type='text' name='{$this ->kod}' maxlength='$this->maxZnaku' value='" . StringUtils::htmlTextConvert($this ->odpovedStr) . "' class='$classStr'/>" .
                "</div>\n";
        } else {
            $result .=
                '<div class="odpovedhodnoceni" ' . ($this->htmlId ? 'id="' . $this ->htmlId . '"' : '') . '>' .
                "<input type='hidden' name='{$this ->kod}' value='" . StringUtils::htmlTextConvert($this ->slovniOdpoved) . "' />";

            if ($zobrazitTextovouOdpoved)
                $result .= "<div class='textovaodpoved'>" . StringUtils::htmlTextConvert($this ->slovniOdpoved) . "</div>";

            $result .=
                "</DIV>\n";

            return $result;
        }
    }


    /** Zobrazi edit pro zadani datumu
     * @param $editace
     * @param $zobrazitTextovouOdpoved
     * @return string
     */
    function HTMLPodrizenaOtazka_Datum($editace, $zobrazitTextovouOdpoved)
    {
        $result = '<div class="textotazky">' . $this ->popis . '</div>';

        if ($editace) {
            $classstr =
                'form-control inline-form-control inputDate' .
                ($this ->f_error ? ' error' : '');

            return $result.
                '<div class="odpovedhodnoceni datumovaodpoved" ' . ($this->htmlId ? 'id="' . $this ->htmlId . '"' : '') . '>' .
                "<input class='$classstr' type='text' name='{$this ->kod}' maxlength='10' size='10' value='" . StringUtils::htmlTextConvert($this ->odpovedStr) . "' />" .
                "<i class='showcalendar inline-glyph glyph-button fa glyph-calendar' data-roncalendar-inputname='{$this ->kod}'></i>" .
                "</div>\n";
        } else {
            $result .=
                '<div class="odpovedhodnoceni" ' . ($this->htmlId ? 'id="' . $this ->htmlId . '"' : '') . '>' .
                "<input type='hidden' name='{$this ->kod}' value='" . StringUtils::htmlTextConvert($this ->odpovedStr) . "' />";

            if ($zobrazitTextovouOdpoved)
                $result .= "<div class='textovaodpoved'>" . StringUtils::htmlTextConvert($this ->odpovedStr) . "</div>";

            $result .=
                "</div>\n";

            return $result;
        }
    }


    /**
     * @param boolean $editace
     * @param boolean $zobrazitTextovouOdpoved
     * @return string
     */
    function HTML_Otazka($editace, $zobrazitTextovouOdpoved)
    {
        $result = '';

        switch ($this ->typOtazky)
        {
            case 'LONGTEXT' :
                $result .=  $this ->HTMLPodrizenaOtazka_Memo($editace, $zobrazitTextovouOdpoved);
                break;
            case 'CISLO' :
                $result .= $this ->HTMLPodrizenaOtazka_Edit($editace, $zobrazitTextovouOdpoved, true);
                break;
            case 'TEXT' :
                $result .= $this ->HTMLPodrizenaOtazka_Edit($editace, $zobrazitTextovouOdpoved);
                break;
            case 'DATUM' :
                $result .= $this ->HTMLPodrizenaOtazka_Datum($editace, $zobrazitTextovouOdpoved);
                break;
        }

        return $result;
    }


    function HTML_Odpoved()
    {
        $result = '<DIV class="textotazky" ' . ($this ->htmlId ? 'id="' . $this ->htmlId . '"' : '') . '>';

        $result .= $this->popis . " ";

        switch ($this ->typOtazky)
        {
            case 'LONGTEXT' :
                $result .= StringUtils::longTextToHtml($this ->memoOdpoved);
                break;
            case 'TEXT':
            case 'CISLO' :
                $result .= StringUtils::htmlTextConvert($this ->slovniOdpoved);
                break;
            case 'DATUM' :
                if ($this->datumOdpoved > 0)
                    $result .= DateUtils::fineDateString($this ->datumOdpoved, false);
                break;
        }

        $result .= '</DIV>';

        return $result;
    }
}
