<?php

/**
 * Trida VolbaDotazniku reprezentuje jednotlive polozky z tabulky RozsahHodnoceni nebo VycetOdpovedi
 */

namespace RonPortal\Testy;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\StringUtils;

/**
 * Mozna odpoved (tj. jedna z voleb) na otazku dotazniku (nacita se z RozsahHodnoceni nebo VycetOdpovedi)
 * Class OdpovedDotazniku
 * @package RonPortal\Testy
 */
class VolbaDotazniku
{
    /** @var int */
    var $hodnota;
    /** @var string */
    var $popis;
    /** @var string     jen pro Rozsah hodnoceni */
    var $doplnujiciPopis;
    /** @var float      jen pro Vycet odpovedi */
    var $koeficient;

    /** @var bool       Pouzito jen v probihajicim testu/hodnoceni, kde se nacitaji i odpovedi uzivatele */
    var $vybrano;


    function getJson()
    {
        return [
            'hodnota' => (int)$this ->hodnota,
            'popis' => StringUtils::jsonText($this ->popis),
            'doplnujiciPopis' => StringUtils::jsonText($this ->doplnujiciPopis),
            'koeficient' => $this ->koeficient
        ];
    }


    /**
     * @param Database $database
     * @param string $sql
     * @return VolbaDotazniku[]
     * @throws DBException
     */
    private static function loadSQL($database, $sql)
    {
        $result = [];

        $cursor = $database ->getSqlResult($sql);
        while ($row = $cursor ->fetch()) {
            $odpoved = new VolbaDotazniku();
            $odpoved ->hodnota = $row[0];
            $odpoved ->popis = $row[1];
            $odpoved ->doplnujiciPopis = $row[2];
            $odpoved ->koeficient = $row[3];

            $odpoved ->vybrano = $row[4];

            $result[] = $odpoved;
        }
        $cursor ->close();

        return $result;
    }


    /**
     * Nacte seznam odpovedi z tabulky RozsahHodnoceni - tj. preddefinovane volby jako Ano/Ne - vcetne odpovedi uzivatele
     * @param Database $database
     * @param string $typRozsahu
     * @param string $kodKriteria - urcuje otazku konkretniho dotazniku
     * @param int $idDotaznikuOsoby - urcuje pripraveny/vyplneny dotaznik uzivatele
     * @return VolbaDotazniku[]
     * @throws DBException
     */
    public static function nactiPreddefinovaneVolby($database, $typRozsahu, $kodKriteria, $idDotaznikuOsoby)
    {
        $params = $database->createParameters();

        // nacitaji se i odpovedi uzivatele - tj. jestli uzivatel tuto volbu zaskrtnul
        $sql =
            'SELECT RozsahHodnoceni.Hodnota, RozsahHodnoceni.Popis, RozsahHodnoceni.DoplnujiciPopis, NULL AS Koeficient, ' .
            '       VycetOdpovediHodnoceni.Id ' .
            '  FROM RozsahHodnoceni LEFT OUTER JOIN VycetOdpovediHodnoceni ' .
            '       ON VycetOdpovediHodnoceni.Id = ' . $params ->addParameter((int)$idDotaznikuOsoby) . // druhhodnoceni a temacislo neni treba kontrolovat, protoze v PripraveneTemaHodnoceni (vazba pres Id) uz je jasne urceno
            '          AND VycetOdpovediHodnoceni.KodKriteria = ' . $params->addParameter($kodKriteria) .
            '          AND VycetOdpovediHodnoceni.Hodnota = RozsahHodnoceni.Hodnota ' .
            ' WHERE TypRozsahu = ' . $params->addParameter($typRozsahu) .
            ' ORDER BY RozsahHodnoceni.Hodnota';

        return self::loadSQL($database, new SqlStatement($sql, $params));
    }


    /**
     * Nacte seznam odpovedi z tabulky VycetOdpovedi - tj. volby ke konkretni otazce - vcetne odpovedi uzivatele
     * @param Database $database
     * @param int $druhHodnoceni
     * @param int $temaCislo
     * @param string $kodKriteria - urcuje otazku konkretniho dotazniku
     * @param int $idDotaznikuOsoby - urcuje pripraveny/vyplneny dotaznik uzivatele
     * @return VolbaDotazniku[]
     * @throws DBException
     */
    public static function nactiVolbyKOtazce($database, $druhHodnoceni, $temaCislo, $kodKriteria, $idDotaznikuOsoby)
    {
        $params = $database->createParameters();

        $sql =
            //'SELECT VycetOdpovedi.Hodnota, ' . $database ->castWideMemo('VycetOdpovedi.Popis') . ', NULL AS DoplnujiciPopis, Koeficient, ' . 3.10.2018 Doplneno nacitani rozsireneho popisu
            'SELECT VycetOdpovedi.Hodnota, ' . $database ->castWideMemo('VycetOdpovedi.Popis') . ', ' .
            $database->castWideMemo('VycetOdpovedi.RozsirenyPopis') . ' AS DoplnujiciPopis,' .
            '       Koeficient, VycetOdpovediHodnoceni.Id ' .
            '  FROM VycetOdpovedi LEFT OUTER JOIN VycetOdpovediHodnoceni' .
            '       ON VycetOdpovediHodnoceni.Id = ' . $params ->addParameter((int)$idDotaznikuOsoby) . // druhHodnoceni a temaCislo neni treba kontrolovat, protoze v PripraveneTemaHodnoceni (vazba pres Id) uz je jasne urceno
            '          AND VycetOdpovediHodnoceni.DruhHodnoceni = VycetOdpovedi.DruhHodnoceni' .
            '          AND VycetOdpovediHodnoceni.TemaCislo = VycetOdpovedi.TemaCislo' .
            '          AND VycetOdpovediHodnoceni.KodKriteria = VycetOdpovedi.KodKriteria' .
            '          AND VycetOdpovediHodnoceni.Hodnota = VycetOdpovedi.Hodnota' .

            ' WHERE VycetOdpovedi.DruhHodnoceni = ' . $params->addParameter($druhHodnoceni) .
            '   AND VycetOdpovedi.TemaCislo = ' . $params->addParameter($temaCislo) .
            "   AND VycetOdpovedi.KodKriteria = " . $params ->addParameter($kodKriteria) .
            ' ORDER BY VycetOdpovedi.Hodnota';

        return self::loadSQL($database, new SqlStatement($sql, $params));
    }

}

