<?php

namespace RonPortal\Uchazeci;

use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\Parameters;
use Ron\MessageManager;


class KomunikaceUchazece
{
    public $idKomunikace;
    public $rcUchazece;

    public $datumACasKomunikace;
    public $personalistaRC;
    public $personalistaJmeno;
    public $personalistaPrijmeni;
    public $druhSdeleni;
    public $obsahSdeleni;
    public $odpoved;


    /**
     * @param Database $database
     * @param $row
     */
    private function nactiKomunikaciUchazeceZRadku($database, $row)
    {
        $this->datumACasKomunikace = $database->stringToTimestamp($row[0]);
        $this->personalistaJmeno = $row[1];
        $this->personalistaPrijmeni = $row[2];
        $this->druhSdeleni = $row[3];
        $this->obsahSdeleni = $row[4];
        $this->odpoved = $row[5];
        $this->idKomunikace = $row[6];
        $this->personalistaRC = $row[7];
        $this->rcUchazece = $row[8];
    }


    /**
     * @param Database $database
     * @param string $sqlFilter
     * @param Parameters $params
     * @return KomunikaceUchazece[]
     */
    private static function nactiZaznamySql($database, $sqlFilter, $params)
    {
        $sql =
            'SELECT Komunikace.DatumCas, P_Osoba.Jmeno, P_Osoba.Prijmeni, Komunikace.DruhSdeleni,' . // 0..3
            '  Komunikace.ObsahSdeleni, Komunikace.OdpovedUchazece, IDKomunikace, P_Osoba.RC,' . // 4..7
            '  RCUchazece' . // 8
            ' FROM Komunikace, P_Osoba' .
            ' WHERE P_Osoba.RC = Komunikace.RCOsoby' .
            '  AND ' . $sqlFilter .
            ' ORDER BY Komunikace.DatumCas DESC';

        $cursor = $database->getSqlResult($sql, $params);

        $seznamKomunikace = [];
        while ($row = $cursor->fetch()) {
            $uchazecKomunikace = new static();
            $uchazecKomunikace->nactiKomunikaciUchazeceZRadku($database, $row);
            $seznamKomunikace[] = $uchazecKomunikace;
        }

        $cursor ->close();

        return $seznamKomunikace;
    }


    /**
     * @param Database $database
     * @param $rc
     * @param UchazeciKomunikace []
     * @return array
     */
    public static function nactiKomunikaciUchazece($database, $rc)
    {
        $params = $database ->createParameters();
        return self::nactiZaznamySql($database,
            'RCUchazece = ' . $params->addParameter($rc),
            $params);
    }


    /**
     * @param Database $database
     * @param int $id
     * @return KomunikaceUchazece
     * @throws \Exception
     */
    public static function nactiZaznamKomunikace($database, $id)
    {
        $params = $database ->createParameters();
        $zaznamy = self::nactiZaznamySql($database,
            'IDKomunikace = ' . $params->addParameter($id),
            $params);

        if (!$zaznamy) {
            throw new \Exception('Zaznam komunikace s uchazecem nenalezen!');
        }

        return $zaznamy[0];
    }

    public static function vratRCUchazecePodleID($database, $idKomunikace)
    {
        //TODO: udelat
        throw new \Exception("OPRAVIT vratRCUchazecePodleID!");
        return DBUtils::getSqlValue($database,
            'SELECT RCUchazece FROM Komunikace WHERE IDKomunikace = ' . (int)$idKomunikace );
    }


    /**
     * @param Database $database
     * @param int $idKomunikace
     * @return string
     */
    public static function vratRCPersonalistyPodleID($database, $idKomunikace)
    {
        $params = $database ->createParameters();
        return DBUtils::getSqlValue($database,
            'SELECT RCOsoby FROM Komunikace WHERE IDKomunikace = ' . $params ->addParameter((int)$idKomunikace),
            $params);
    }
}