<?php
/**
 * stanislav.elbl, 14.01.2016
 */

namespace RonPortal\Uchazeci;

use Ron\Controls\BrowserField;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\RonPortalSettings;
use RonPortal\System\RecordNotFoundException;
use RonPortal\TypRozdeleni;


/**
 * Class RozdeleniUchazeceField
 */
class RozdeleniUchazeceField extends BrowserField
{
    private $kodTypuRozdeleni;

    /**
     * RozdeleniUchazeceField constructor.
     * @param Database $database
     * @param int $kodTypuRozdeleni
     * @throws RecordNotFoundException
     * @throws DBException
     */
    public function __construct($database, $kodTypuRozdeleni)
    {
        parent::__construct($database);

        $this ->kodTypuRozdeleni = $kodTypuRozdeleni;
        $this->header = StringUtils::htmlTextConvert(TypRozdeleni::getPopisTypuRozdeleni($this ->database, $this ->kodTypuRozdeleni));
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $params = $this->database->createParameters();

        $sql =
            'SELECT Rozdeleni.Kod, Rozdeleni.Popis ' .
            ' FROM P_Uchazec, Rozdeleni, RozdeleniUchazece ' .
            ' WHERE P_Uchazec.RC = RozdeleniUchazece.RC ' .
            ' AND RozdeleniUchazece.Rozdeleni = Rozdeleni.Kod ' .
            ' AND RozdeleniUchazece.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
            ' AND RozdeleniUchazece.KodTypuRozdeleni = ' . $params->addParameter($this ->kodTypuRozdeleni) .
            ' AND P_Uchazec.RC = ' . $params->addParameter($rowData['col-rc']) . ' '.
            ' ORDER BY Rozdeleni.Popis';

        $cursor = $this->database->getSqlResult($sql, $params);

        $result = '';

        while ($row = $cursor->fetch()) {
            if ($result) {
                $result .= ', ';
            }
            $result .= "<span title='" . $row[0]  . "'>" . $row[1] . '</span>';
        }
        $cursor ->close();

        return $result;
    }
}



/**
 * Class UchazeciBrowser
 */
class UchazeciBrowser extends RonPortalBrowser
{
    /**
     * UchazeciBrowser constructor.
     * @param Database $database
     * @throws \Exception
     */
    public function __construct($database)
    {
        parent::__construct($database, '');

        $this ->sqlFromPart =
            'P_Uchazec' .
            ' LEFT OUTER JOIN ZdrojVznikuPP ON P_Uchazec.ZdrojVzniku = ZdrojVznikuPP.Kod';

        $this ->title = \Ron\ronText('Seznam_uchazecu');

        $this ->browserUrl = '?menuid=uchazeci&';
        $this ->rowClickLink = '?menuid=uchazeci&action=zobrazuchazece';
        $this ->cssClass .= ' seznamUchazecu';

        $this ->code = 'browseruchazeci';

        $this->primaryField = 'col-rc';

        $this->addField()
            ->setSqlFieldName('P_Uchazec.RC') ->setHeaderText('ID') ->setOrdering(true)
            ->setCode('col-rc');

        $this->addField()
            ->setSqlFieldName('P_Uchazec.Titul') ->setHeaderText('Titul') ->setOrdering(true)
            ->setCode('col-titul');

        $this->addField()
            ->setSqlFieldName('P_Uchazec.Jmeno') ->setHeaderText('Jmeno') ->setOrdering(true)
            ->setCode('col-jmeno')
            ->setLink('?menuid=uchazeci&action=zobrazuchazece');

        $this->addField()
            ->setSqlFieldName('P_Uchazec.Prijmeni') ->setHeaderText('Prijmeni') ->setOrdering(true)
            ->setCode('col-prijmeni')
            ->setLink('?menuid=uchazeci&action=zobrazuchazece');

        $this ->defaultSort = 'col-prijmeni';

        $this->addField()
            ->setSqlFieldName('P_Uchazec.TitulZa') ->setHeaderText('Titul_za') ->setOrdering(true)
            ->setCode('col-titulza');

        $this->addField()
            ->setSqlFieldName('P_Uchazec.VEvidenciOd') ->setFormat('finedate') ->setHeaderText('V_evidenci_od') ->setOrdering(true)
            ->setCode('col-vevidenciod');

        $this->addField( new RozdeleniUchazeceField($this->database, RonPortalSettings::getIntValue('uchazecityprozdeleniprofiltr')))
            ->setCode('col-rozdeleni');

        $typR3 = RonPortalSettings::getIntValue('uchazecityprozdeleniprofiltr2');
        if ($typR3) {
            $this->addField( new RozdeleniUchazeceField($this->database, $typR3))
                ->setCode('col-rozdeleni');
        }

        $this->addField()
            ->setSqlFieldName('ZdrojVznikuPP.Popis') ->setHeaderText('zdrojVzniku') ->setOrdering(true)
            ->setCode('col-zdrojvznikupp');

        for ($i = 1; $i <= 20; $i++) {
            $this->addField()
                ->setSqlFieldName('uzivatelska' . $i) ->setHeaderText(RonPortalSettings::getValue('uchazeciuzivatelska' . $i))
                ->setVisible((bool)RonPortalSettings::getValue('uchazeciuzivatelska' . $i))
                ->setCode('col-uzivatelska' . $i);
        }
    }


    public function setFilterForm()
    {
        //$this->filterForm = new UchazeciFilterForm($this->database, $this->typRozdeleniFiltr2, $this->typRozdeleniFiltr3);
        $this->filterForm = new RonPortalFilterForm($this ->database, 'uchazeci', true);

        $f = $this ->filterForm ->addFilterRozdeleni('rozdeleni', -1);
        $f
            ->setSessionName('filterform_rozdeleni_PM')
            ->setUserSettingsName('filterform_rozdeleni_PM')
            ->setShowVcetnePodrizenych(false);

        $typR2 = RonPortalSettings::getIntValue('uchazecityprozdeleniprofiltr');
        if ($typR2) {
            $filter = $this ->filterForm ->addFilterRozdeleni('rozdeleni2', $typR2);
            $filter
                ->setSessionName('rozdeleni_uchazecu_2')
                ->setUserSettingsName('rozdeleni_uchazecu_2');

        }

        $typR3 = RonPortalSettings::getIntValue('uchazecityprozdeleniprofiltr2');
        if ($typR3) {
            $filter = $this ->filterForm ->addFilterRozdeleni('rozdeleni3', $typR3);
            $filter
                ->setSessionName('rozdeleni_uchazecu_3')
                ->setUserSettingsName('rozdeleni_uchazecu_3');
        }
    }


    /**
     * @param int $kodTypuRozdeleni
     * @param string $kodRozdeleni
     * @param int $vcetnePodrizenych
     * @param Parameters $params
     * @return string
     */
    private function getFiltrRozdeleni($kodTypuRozdeleni, $kodRozdeleni, $vcetnePodrizenych, $params)
    {
        $tabulka = ($kodTypuRozdeleni === -1)
            ? 'UchazecNaPM'
            : 'RozdeleniUchazece';

        $fieldNameKod = ($kodTypuRozdeleni === -1)
            ? 'KodRozdeleni'
            : 'Rozdeleni';

        switch ($vcetnePodrizenych) {
            case 1 : // vcetne podrizenych
            case 2 : // jen podrizene
                return
                    'EXISTS (' .
                    " SELECT * FROM $tabulka, HierarchieRozdeleni" .
                    ' WHERE HierarchieRozdeleni.TypNadrazene = ' . $params->addParameter($kodTypuRozdeleni) .
                    '   AND HierarchieRozdeleni.KodNadrazene = ' . $params->addParameter($kodRozdeleni) .

                    (($vcetnePodrizenych == 2) ? ' AND HierarchieRozdeleni.KodNadrazene <> HierarchieRozdeleni.KodPodrizene' : '') . // filtr pro jen podrizene

                    "    AND $tabulka.KodTypuRozdeleni = HierarchieRozdeleni.typPodrizene" .
                    "    AND $tabulka.$fieldNameKod = HierarchieRozdeleni.kodPodrizene" .
                    "    AND $tabulka.RC = P_Uchazec.RC" .
                    ')';

            case 3 :
                // jen primo podrizene
                return
                    'EXISTS (' .
                    " SELECT * FROM $tabulka, Rozdeleni " .
                    ' WHERE Rozdeleni.RozdeleniRodice = ' . $params->addParameter($kodTypuRozdeleni) .
                    '   AND Rozdeleni.Rodic = ' . $params->addParameter($kodRozdeleni) .
                    "   AND $tabulka.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni" .
                    "   AND $tabulka.$fieldNameKod = Rozdeleni.Kod" .
                    "   AND $tabulka.RC = P_Uchazec.RC" .
                    ')';
            default :
                // jen vybrane
                return
                    'EXISTS (' .
                    " SELECT * FROM $tabulka" .
                    " WHERE $tabulka.KodTypuRozdeleni = " . $params->addParameter($kodTypuRozdeleni) .
                    " AND $tabulka.$fieldNameKod = " . $params->addParameter($kodRozdeleni) .
                    " AND $tabulka.RC = P_Uchazec.RC" .
                    ')';
        }
    }


    public function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

//        if ($this ->filterForm ->kodRozdeleni) { // ::rozdeleni(-1) != '') {
//            $result .=
//                ($result ? ' AND ' : '') .
//                $this ->getFiltrRozdeleni($this ->filterForm ->kodTypuRozdeleni,
//                $this ->filterForm ->kodRozdeleni, 0, // $this ->filterForm ->vcetnePodrizenychRozdeleni,
//                $params);
//        }
//
//        if ($this ->filterForm ->kodRozdeleni2) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                $this ->getFiltrRozdeleni($this ->filterForm ->kodTypuRozdeleni2,
//                $this ->filterForm ->kodRozdeleni2, $this ->filterForm ->vcetnePodrizenychRozdeleni2,
//                $params);
//        }

        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            if ($rozdeleniFilter ->getValue() !== null) {
                $result .=
                    ($result ? ' AND ' : '') .
                    $this ->getFiltrRozdeleni(
                    $rozdeleniFilter ->getValue('typRozdeleni'),
                    $rozdeleniFilter ->getValue(),
                    $rozdeleniFilter ->getValue('vcetnePodrizenych'),
                    $params);
            }
        }

        return new SqlStatement($result, $params);
    }
}
