<?php
/**
 * stanislav.elbl, 04.10.2021 23:04
 * Formular pro rozdeleni denni vyroby osoby - pouziva se v editaci dne dochazky
 */

namespace RonPortal\Vyroba;

use Ron\Controls\Option;
use Ron\Controls\OptionLoader;
use Ron\Controls\Options;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use RonPortal\Dochazka\DenniVysledek;
use RonPortal\Dochazka\SchvaleniDochazky;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\TypRozdeleni;

class RadekDenniVyroby extends ZaznamVyroby
{
    /** @var string */
    public $hodinyStr;

    /** @var string */
    public $casOdStr;
    /** @var string */
    public $casDoStr;

    /** @var bool   Priznak, ze byl zaznam zmenen v POST - bude se provadet update */
    public $jeZmena = false;
    /** @var bool   Priznak, ze se ma zaznam smazat */
    //public $delete = false;



    public function nastavTexty()
    {
        $this ->hodinyStr = $this->hodiny ? StringUtils::format('minuty', round($this->hodiny * 60)) : '';
        $this ->casOdStr = $this->casOd ? StringUtils::format('timeofday', $this->casOd) : '';
        $this ->casDoStr = $this->casDo ? StringUtils::format('timeofday', $this->casDo) : '';
    }
}


class DenniVyrobaOsobyForm implements ViewObject
{
    /** @var Database */
    public $database;

    /** @var PracovniPomer */
    public $pp;

    /** @var int */
    public $den;

    /** @var RadekDenniVyroby[] */
    public $zaznamy;

    /** @var VyrobniZakazka[] */
    public $zakazky;

    /** @var CinnostDenniVyroby[] */
    public $cinnosti;

    /** @var Option[] */
    public $vyrobky;

    /** @var Option[] */
    public $seznamRozdeleni;

    /** @var float  Soucet rozdelenych hodin */
    public $soucet;
    /** @var float  Hodnota denniho vysledku, ktery slouzi pro kontrolu souctu (napr. odpracovana doba) */
    public $odpracovano;

    /** @var bool   Priznak, ze se mohou zadavat jen prirazene cinnosti */
    public $jenPrirazeneCinnosti = false;

    /** @var bool   Priznak, ze je potreba formular zobrazit znovu (napr. po kliknuti na "novy zaznam") */
    public $pokracovat = false;

    /** @var bool   Priznak, ze doslo k chybe */
    public $isError = false;

    public $readOnly;

    public $logDescription;

    /** Priznaky urcujici, ktere hodnoty se budou vyplnovat  */
    public $editovatHodiny = true;
    public $editovatCasy = false;
    public $editovatVyrobek = false;
    /** @var int    Kod typu rozdeleni, ktere se ma vybirat - pokud je null/0, tak se vubec needituje */
    public $editovatRozdeleni = null;
    /** @var TypRozdeleni   Editovany typ rozdeleni - popis se pouzije pro zobrazeni nadpisu */
    public $typRozdeleni;

    /**
     * DenniVyrobaOsobyForm constructor.
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $den
     * @param bool $kontrolovatPrava    Priznak, jestli se maji kontrolovat prava uzivatele
     * @throws \Exception
     */
    public function __construct(Database $database, PracovniPomer $pp, $den, $kontrolovatPrava = true)
    {
        $this->database = $database;
        $this->pp = $pp;
        $this->den = $den;

        $this->editovatHodiny = RonPortalSettings::getIntValue('editovathodinyvyroby', 1);
        $this->editovatCasy = RonPortalSettings::getIntValue('editovatcasyvyroby');
        $this->editovatVyrobek = RonPortalSettings::getIntValue('editovatvyrobek');
        $this->editovatRozdeleni = RonPortalSettings::getIntValue('editovatrozdelenivyroby');

        $schvaleni = SchvaleniDochazky::nactiSchvaleni($database, $pp, $den);

        $this ->jenPrirazeneCinnosti = RonPortalSettings::getIntValue('jenprirazenecinnosti');
        $this ->readOnly =
            Obdobi::jeObdobiUzavreno($database, 'ADS', $den, $den) ||
            ($kontrolovatPrava  &&  !Prava::vratPrava('editacevyroby') ->lzeZobrazitPP($pp ->idPP, $den, $den)) ||
            !$schvaleni ->lzeEditovatPP($pp ->idPP, $den);

        $this ->logDescription = $this;
    }


    /**
     * @param $serial
     * @return RadekDenniVyroby
     */
    private function najdiZaznam($serial)
    {
        foreach ($this ->zaznamy as $zaznam) {
            if ($zaznam ->serialVyroba == $serial) {
                return $zaznam;
            }
        }

        return null;
    }


    /**
     * Nacte data z databaze
     */
    public function load()
    {
        // zaznamy vyroby
        $this ->zaznamy = RadekDenniVyroby::nactiVyrobu($this ->database, $this ->pp, $this ->den);

        // doplni zaznamy a spocita soucet
        $this ->soucet = 0;
        foreach ($this ->zaznamy as $zaznam) {
            $zaznam ->nastavTexty();
            $this ->soucet += $zaznam ->hodiny;
        }

        // data pro comboboxy
        $this ->zakazky = RonPortalSettings::getIntValue('jenprirazenezakazky')
            ? VyrobniZakazka::nactiVyrobniZakazkyOsoby($this ->database, $this ->den, $this ->den, $this ->pp)
            : VyrobniZakazka::nactiVyrobniZakazky($this ->database, $this ->den, $this ->den);

        $this ->cinnosti = $this ->jenPrirazeneCinnosti
            ? CinnostDenniVyroby::nactiPrirazeneCinnosti($this ->database, $this ->pp, $this ->den)
            : CinnostDenniVyroby::nactiVyrobniCinnosti($this ->database);

        if ($this ->editovatVyrobek) {
            $this ->vyrobky =
                (new OptionLoader($this ->database))
                    ->setSql('SELECT Kod, Popis From Zbozi WHERE Neaktivni = 0 ORDER BY Popis', null)
                    ->setEmptyValue('')
                    ->loadData();
        }

        if ($this ->editovatRozdeleni) {
            $this ->typRozdeleni = TypRozdeleni::getTypRozdeleni($this ->database, $this ->editovatRozdeleni);

            $params = $this ->database ->createParameters();

            $this ->seznamRozdeleni =
                (new OptionLoader($this ->database))
                    ->setSql('SELECT Kod, Popis FROM Rozdeleni ' .
                        ' WHERE KodTypuRozdeleni = ' . $this ->editovatRozdeleni .
                        '   AND (DatumZruseni >= ' . $params ->addDateParameter($this ->den) . ' OR DatumZruseni IS NULL) ' .
                        '   AND DatumVytvoreni <= ' . $params ->addDateParameter($this ->den) .
                        ' ORDER BY Popis',
                        $params)
                    ->setEmptyValue('')
                    ->loadData();
        }

        // odpracovana doba pro rozdeleni
        $this ->odpracovano = null;

        if (RonPortalSettings::getValue('vyroba_casovaslozkasoucet')) {
            $this ->odpracovano = DenniVysledek::nactiHodnotu($this->database,
                $this->pp, $this->den, RonPortalSettings::getValue('vyroba_casovaslozkasoucet'));
        }

    }


    /**
     * Ulozi data odeslana v POST
     * @throws DBException
     */
    private function saveData()
    {
        foreach ($this ->zaznamy as $zaznam) {
            if (!$zaznam ->serialVyroba) {
                if ($zaznam ->cinnost) {
                    // novy zaznam
                    $zaznam ->insert($this ->database, $this ->pp, $this ->den, $this ->logDescription);
                }
            } else if ($zaznam ->jeZmena) {
                $zaznam ->update($this ->database, $this, $this ->editovatHodiny, $this ->editovatCasy, $this ->editovatVyrobek, $this ->editovatRozdeleni);
            }
        }
    }


    /**
     * @return RadekDenniVyroby
     */
    private function novyZaznam()
    {
        $novyZaznam = new RadekDenniVyroby();

        $novyZaznam ->den = $this ->den;
        $novyZaznam ->idPP = $this ->pp ->idPP;
        $novyZaznam ->rc = $this ->pp ->rc;
        $novyZaznam ->osobniCislo = $this ->pp ->osobniCislo;
        $novyZaznam ->datumNastupu = $this ->pp ->datumNastupu;

        return $novyZaznam;
    }


    private function addErrorMessage($message, $fieldName = null) {
        $this ->isError = true;
        MessageManager::addErrorMessage($message, $fieldName);
    }


    /**
     * @param RadekDenniVyroby $zaznam
     * @param int $index
     * @throws \Exception
     */
    private function naplnZaznamPodlePost($zaznam, $index)
    {
        $zaznam ->zakazka = $_POST["vyroba{$index}_vyrobniZakazka"];
        if ($this ->editovatHodiny) {
            $zaznam->hodinyStr = trim($_POST["vyroba{$index}_hodiny"]);
            try {
                $zaznam->hodiny = ($zaznam->hodinyStr == '')
                    ? 0
                    : DateUtils::timeStringToFloat($zaznam->hodinyStr, true);
            } catch (\Exception $e) {
                $this->addErrorMessage($e->getMessage(), "vyroba{$index}_hodiny");
            }
        }

        if ($this ->editovatCasy) {
            $zaznam->casOdStr = trim($_POST["vyroba{$index}_casOd"]);
            $zaznam->casDoStr = trim($_POST["vyroba{$index}_casDo"]);

            try {
                $zaznam->casOd = (trim($zaznam->casOdStr) == '')
                    ? null
                    : DateUtils::timeStringToTimestamp($zaznam->casOdStr, $zaznam->den, false);
            } catch (\Exception $e) {
                $this->addErrorMessage($e->getMessage(), "vyroba{$index}_casOd");
            }

            try {
                $zaznam->casDo = ($zaznam->casDoStr == '')
                    ? null
                    : DateUtils::timeStringToTimestamp($zaznam->casDoStr, $zaznam ->den, false);

                if ($zaznam ->casOd  &&  $zaznam ->casDo  &&  $zaznam ->casDo < $zaznam ->casOd) {
                    // cas do je mensi => je to pres pulnoc, posune datum
                    $zaznam ->casDo = DateUtils::incDay($zaznam ->casDo);
                }

                if (!$this ->editovatHodiny  &&  $zaznam ->casOd  &&  $zaznam ->casDo) {
                    // doplni hodiny (rozdil casu)
                    $zaznam ->hodiny = ($zaznam ->casDo - $zaznam ->casOd) / 3600;
                    $zaznam ->hodinyStr = StringUtils::format('minuty', round($zaznam ->hodiny * 60));
                }
            } catch (\Exception $e) {
                $this->addErrorMessage($e->getMessage(), "vyroba{$index}_casDo");
            }
        }

        if ($this ->jenPrirazeneCinnosti) {
            $zaznam ->cinnostVyrobniZakazky = (int)$_POST["vyroba{$index}_cinnost"];
            $zaznam ->cinnost = CinnostDenniVyroby::vratKodCinnostiVyrobniZakazky($this ->database, $zaznam ->cinnostVyrobniZakazky);
        } else {
            $zaznam ->cinnost = $_POST["vyroba{$index}_cinnost"];
        }

        $zaznam ->popis = $_POST["vyroba{$index}_popis"];
        $zaznam ->rozsirujiciPopis = $_POST["vyroba{$index}_rozsirujiciPopis"];

        if ($this ->editovatVyrobek) {
            $zaznam ->vyrobek = $_POST["vyroba{$index}_vyrobek"];
        }

        if ($this ->editovatRozdeleni) {
            $zaznam ->rozdeleni = $_POST["vyroba{$index}_rozdeleni"];
            $zaznam ->kodTypuRozdeleni = $zaznam ->rozdeleni !== '' ? $this ->editovatRozdeleni : null;
        }
    }


    /**
     * Odstrani zaznam se zadanym seriovym cislem
     * @param $serial
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    private function smazZaznam($serial)
    {
        // najde zaznam v poli
        $index = null;
        for ($i = 0;  !isset($index)  &&  $i < count($this ->zaznamy);  $i++) {
            if ($this ->zaznamy[$i] ->serialVyroba == $serial) {
                $index = $i;
            }
        }

        if (isset($index)) {
            $zaznam = $this ->zaznamy[$index];

            // odstrani ho z databaze ...
            $rec = new VyrobaDbRecord($this->database, $zaznam->serialVyroba, $this ->logDescription);

            if ($rec ->loadRecord()) {
                $rec ->deleteRecord();
            }

            // ... a taky z pole
            array_splice($this ->zaznamy, $index, 1);
        }
    }


    private function doplnPodlePredchozihoDne()
    {
        // zjisti posledni den, kdy mela osoba nejaky zaznam
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT MAX(Den) FROM Vyroba' .
            ' WHERE Den < ' . $params ->addParameter($this ->database ->dateToParam($this ->den)) .
            ' AND RC = ' . $params ->addParameter($this ->pp ->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($this ->pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->pp ->datumNastupu));
        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row  ||  !$row[0]) return;

        $datum = $this ->database ->stringToTimestamp($row[0]); // zdrojovy datum

        // nacte zaznamy z predchoziho dne
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT V1.Cinnost, V1.CinnostVyrobniZakazky, V1.VyrobniZakazka' .
            ' FROM Vyroba V1' .
            // zaznamy z posledniho dne
            ' WHERE Den = ' . $params ->addParameter($this ->database ->dateToParam($datum)) .
            // aktualniho zamestnance
            ' AND RC = ' . $params ->addParameter($this ->pp ->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($this ->pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->pp ->datumNastupu)) .
            // ktere jeste neexistuji v aktualnim dnu
            ' AND NOT EXISTS (' .  //
            '   SELECT * FROM Vyroba V2' .
            '   WHERE V2.RC = V1.RC  AND  V2.OsobniCislo = V1.OsobniCislo  AND  V2.DatumNastupu = V1.DatumNastupu' .
            '   AND V2.Cinnost = V1.Cinnost  AND V2.CinnostVyrobniZakazky = V1.CinnostVyrobniZakazky' .
            '   AND V2.VyrobniZakazka = V1.VyrobniZakazka' .
            '   AND V2.Den = ' . $params ->addParameter($this ->database ->dateToParam($this ->den)) .
            ')';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $rec = new VyrobaDbRecord($this ->database, 0, $this ->logDescription);

            $rec->fCinnost->setNewValue($row[0]);
            $rec->fCinnostVyrobniZakazky->setNewValue($row[1]);
            $rec->fVyrobniZakazka->setNewValue($row[2]);
            $rec->addField('', '', true, 0, '   ... automaticke doplneni podle predchoziho dne');

            $rec ->setNewRecord($this ->pp, $this ->den);
            $rec ->insertRecord();
        }
        $cursor ->close();
    }


    function zkontrolujDuplicitu()
    {
        if (!RonPortalSettings::getIntValue('zakazatduplicityvyroby')) {
            return;
        }

        foreach ($this ->zaznamy as $index => $zaznam) {
            // Kontrola probiha jen u nove zadanych zaznamu
            if (!$zaznam ->serialVyroba  ||  $zaznam ->jeZmena) {
                // novy zaznam - zkontroluje, jestli jeste neexistuje
                foreach ($this ->zaznamy as $zaznam2) {
                    if ($zaznam != $zaznam2) {
                        if ( !(
                            strcasecmp($zaznam ->zakazka, $zaznam2 ->zakazka)
                            || (!$this ->jenPrirazeneCinnosti  &&  strcasecmp($zaznam ->cinnost, $zaznam2 ->cinnost) )
                            || ($this ->jenPrirazeneCinnosti  &&  $zaznam ->cinnostVyrobniZakazky != $zaznam2 ->cinnostVyrobniZakazky) )
                        ) {
                            $this ->addErrorMessage( \Ron\ronText('Radek_vyroby_jiz_existuje'), "vyroba{$index}_vyrobniZakazka");
                        }
                    }
                }

            }
        }
    }


    /** Kontrola souctu rozdelenych hodin (nesmi presahnout odpracovanou dobu) */
    function zkontrolujSoucet()
    {
        if (isset($this ->odpracovano)) {
            // kontrola souctu
            $this ->soucet = 0;
            foreach ($this ->zaznamy as $zaznam) $this ->soucet += $zaznam ->hodiny;

            if ( round($this ->soucet - $this ->odpracovano, 6) > 0 ) {
                $this ->addErrorMessage( \Ron\ronText('Nelze_rozdelit_vice_vyroby') );
            }
        }
    }


    /**
     * Nacte post a zpracuje data. Pokud jsou v poradku, tak je zapise do databaze.
     */
    public function process()
    {
        if ($this ->readOnly) {
            return; // uzivatel nema prava na editaci, nebo je obdobi uzavreno
        }

        $cnt = (int)$_POST['vyroba_COUNT'];

        $jeZmena = false;

        if ((int)$_POST['vyroba_delete_row']) {
            $this ->smazZaznam( (int)$_POST['vyroba_delete_row'] );
            $this ->pokracovat = true;
        }

        for ($i = 0;  $i < $cnt;  $i++) {
            $serial = (int)$_POST["vyroba{$i}_serial"];
            if ($serial &&  $serial == (int)$_POST['vyroba_delete_row']) {
                // radek byl smazan
                $jeZmena = true;
            } else if ($serial) {
                /** @var RadekDenniVyroby $zaznam   Jiz existujici editovany zaznam */
                $zaznam = $this ->najdiZaznam($serial);
                if ($zaznam) {
                    //$zaznam ->posted = true;

                    $zaznam ->jeZmena =
                        ($this ->editovatHodiny  &&  $_POST["vyroba{$i}_hodiny"] != $zaznam ->hodinyStr) ||
                        ($this ->editovatCasy  &&  $_POST["vyroba{$i}_casOd"] != $zaznam ->casOdStr) ||
                        ($this ->editovatCasy  &&  $_POST["vyroba{$i}_casDo"] != $zaznam ->casDoStr) ||
                        ($_POST["vyroba{$i}_vyrobniZakazka"] != $zaznam ->zakazka) ||
                        (!$this ->jenPrirazeneCinnosti  &&  $_POST["vyroba{$i}_cinnost"] != $zaznam ->cinnost) ||
                        ($this ->jenPrirazeneCinnosti  &&  $_POST["vyroba{$i}_cinnost"] != $zaznam ->cinnostVyrobniZakazky) ||
                        ($_POST["vyroba{$i}_popis"] != $zaznam ->popis) ||
                        ($_POST["vyroba{$i}_rozsirujiciPopis"] != $zaznam ->rozsirujiciPopis) ||
                        ($this ->editovatVyrobek  &&  $_POST["vyroba{$i}_vyrobek"] != $zaznam ->vyrobek) ||
                        ($this ->editovatRozdeleni && $_POST["vyroba{$i}_rozdeleni"] != $zaznam ->rozdeleni);


                    if ($zaznam ->jeZmena) {
                        $this ->naplnZaznamPodlePost($zaznam, $i);
                        $jeZmena = true;
                    }
                } else {
                    // chyba - nekdo jiny zaznam smazal? - obvykle by nemelo nastat
                    $this ->addErrorMessage("Editovany zaznam nebyl nalezen! (serial = {$serial})");
                }
            } else {
                // novy zaznam
                if ( ($_POST["vyroba{$i}_cinnost"] ?? null)  &&  ($_POST["vyroba{$i}_vyrobniZakazka"] ?? null)) {
                    $zaznam = $this->novyZaznam();
                    $this->naplnZaznamPodlePost($zaznam, $i);

                    $this->zaznamy[] = $zaznam;

                    $jeZmena = true;
                }
            }
        }

        if (!$this ->isError) {
            $this ->zkontrolujDuplicitu();
        }

        if (!$this ->isError) {
            $this ->zkontrolujSoucet();
        }

        if (!$this ->isError) {
            $this ->saveData();
        }

        if (!$this ->isError  &&  isset($_POST['doplnitpodlepredchoziho'])  &&  $_POST['doplnitpodlepredchoziho']) {
            $this ->doplnPodlePredchozihoDne();
            $this ->load();
            $this ->pokracovat = true;
            $jeZmena = true;
        }

        if ($jeZmena  &&  RonPortalSettings::getValue('vyroba_agenthost')) {
            VyrobaUtils::prepocetVysledku($this ->pp ->idPP, $this ->den);
        }

        if (isset($_POST['novyradekvyroby'])  &&  $_POST['novyradekvyroby']) {
            $novyZaznam = $this ->novyZaznam();
            $this ->zaznamy[] = $novyZaznam;

            $this ->pokracovat = true; // aby se formular zobrazil znovu (priznak zpracuje volajici kod)
        } else if ( round($this ->odpracovano - $this ->soucet, 6) > 0) {
            // neni rozdeleno vse
            $this ->addErrorMessage(\Ron\ronText('Nejsou_rozdeleny_hodiny_vyroby'));
            $this ->pokracovat = true;
        }
    }


    private function vratZakazku($kod)
    {
        foreach ($this ->zakazky as $zakazka) {
            if ($zakazka ->cisloZakazky === $kod) {
                return $zakazka;
            }
        }
        return null;
    }

    public function renderComboZakazka($name, $value)
    {
        if ($this ->readOnly) {
            $zakazka = $this ->vratZakazku($value);
            echo '<input class="form-input" readonly="readonly" value="' . StringUtils::htmlTextConvert($zakazka ? $zakazka ->popis : $value) . '">';
            return;
        }

        //$errorClass = MessageManager::getInstance() ->isError($name) ? 'error' : '';
        $className =
            'form-control ronselect vyrobni_zakazka' .
            (MessageManager::getInstance() ->isError($name) ? ' error' : '') .
            ($this ->jenPrirazeneCinnosti ? ' nacitat_cinnosti' : '');
        echo "<select name='$name' class='$className' data-max-visible-rows='200'>";

        if (!isset($value) ||  $value === '') {
            // novy zaznam - prida prazdny radek
            echo '<option value=""></option>';
        }

        $found = false;
        foreach ($this ->zakazky as $zakazka) {
            echo '<option value="' . StringUtils::htmlTextConvert($zakazka ->cisloZakazky) . '"';
            if ($zakazka ->cisloZakazky === $value) {
                echo ' selected="selected"';
                $found = true;
            }
            echo '>' . StringUtils::htmlTextConvert($zakazka ->popis) . '</option>';
        }

        if (!$found  &&  $value) {
            // pokud nebyla zobrazena aktualni hodnota, tak ji zobrazi
            $zakazka = VyrobniZakazka::vratVyrobniZakazku($this ->database, $value);
            echo '<option value="' . StringUtils::htmlTextConvert($value) . '" selected="selected" class="errorcolor">' . StringUtils::htmlTextConvert($zakazka ->popis) . '</option>';
        }

        echo "</select>\n";
    }


    private function vratCinnost($value)
    {
        foreach ($this ->cinnosti as $cinnost) {
            if (($this ->jenPrirazeneCinnosti && $cinnost ->cinnostVyrobniZakazky == (int)$value) ||
                (!$this ->jenPrirazeneCinnosti && $cinnost ->kod === $value)) {

                return $cinnost;
            }
        }

        return null;
    }


    public function renderComboCinnost($name, $value, $cisloZakazky = null)
    {
        if ($this ->readOnly) {
            $cinnost = $this ->vratCinnost($value);
            echo '<input class="form-input" readonly="readonly" value="' . StringUtils::htmlTextConvert($cinnost ? $cinnost ->popis : $value) . '">';
            return;
        }

        echo "<select name='$name' class='form-control ronselect' id='$name' data-columns='value;text' data-max-visible-rows='200'>";

        $found = false;

        foreach ($this ->cinnosti as $cinnost) {
            if (!$this ->jenPrirazeneCinnosti  ||  $cisloZakazky == $cinnost ->cisloZakazky) {
                echo '<option value="' . ($this ->jenPrirazeneCinnosti ? $cinnost ->cinnostVyrobniZakazky : StringUtils::htmlTextConvert($cinnost->kod)) . '"';

                // Podle nastaveni muze byt string (kod cinnosti) nebo int (id cinnosti vyrobni zakazky)
                $jeZaznamVybrany =
                    ($this ->jenPrirazeneCinnosti && $cinnost ->cinnostVyrobniZakazky == (int)$value) ||
                    (!$this ->jenPrirazeneCinnosti && $cinnost ->kod === $value);

                if ($jeZaznamVybrany) {
                    echo ' selected="selected"';
                    $found = true;
                }
                echo '>' . StringUtils::htmlTextConvert($cinnost->popis) . '</option>';
            }
        }

        if (!$found  &&  $value) {
            // pokud nebyla zobrazena aktualni hodnota, tak ji nacte z databaze a zobrazi (aby se nezobrazovalo jen cislo)
            $text = $value;
            if ($this ->jenPrirazeneCinnosti) {
                $cinnostVyrobniZakazky = CinnostDenniVyroby::vratCinnostVyrobniZakazky($this ->database, $value);
                $text = $cinnostVyrobniZakazky ->popis;
            }

            echo '<option value="' . StringUtils::htmlTextConvert($value) . '" selected="selected" class="errorcolor">' . StringUtils::htmlTextConvert($text) . '</option>';
        }

        echo "</select>\n";
    }

    /**
     * Pokud zbyva rozdelit hodiny a neni zadny prazdny radek (bez zakazky), tak ho doplni
     */
    private function doplnPrazdnyRadek()
    {
        if ($this ->readOnly) {
            return;
        }
        if ($this ->odpracovano <= $this ->soucet) {
            return;
        }

        foreach ($this ->zaznamy as $zaznam) {
            //if (!$zaznam ->serialVyroba) return;
            if (!$zaznam ->zakazka) return; // zaznam bez zakazky existuje
        }

        $zaznam = $this->novyZaznam();
        $this->zaznamy[] = $zaznam;

    }

    public function render()
    {
        $this ->doplnPrazdnyRadek();
        require 'ronportal/vyroba/view_denni_vyroba_osoby.php';
    }
}
