<?php
/**
 * stanislav.elbl, 13.05.2020
 */

namespace RonPortal\Vyroba;

use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\ArrayCursor;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use RonPortal\Controls\PritomnostBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;


class VlozeniLinkField extends BrowserField
{
    var $prava;

    /**
     * VlozeniLinkField constructor.
     * @param null|Database $database
     */
    public function __construct($database)
    {
        parent::__construct($database);
        $this ->type = self::FIELD_TYPE_TEXT;

        $this ->prava = Prava::vratPrava('vkladanivyroby');

        $this ->text = '<span class="js vlozit-vyrobu">' . \Ron\ronText("Vlozit_vyrobu") . '</span>';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($this ->prava ->lzeZobrazitPP($rowData[0])) {
            return parent::getViewText($value, $rowNumber, $recordId, $rowData);
        }

        return '';
    }
}


class MonitorovaniVyrobyBrowserField extends BrowserField
{
    /**
     * @var MonitorovaniVyrobyBrowser
     */
    var $monitorovaniBrowser;

    /**
     * MonitorovaniVyrobyBrowserField constructor.
     * @param null|Database $database
     * @param MonitorovaniVyrobyBrowser $monitorovaniBrowser
     */
    public function __construct($database, $monitorovaniBrowser)
    {
        parent::__construct($database);
        $this->setType(self::FIELD_TYPE_DEFAULT);

        $this->monitorovaniBrowser = $monitorovaniBrowser;
    }


    function getLink($text, $recordId, $rowData)
    {
        if (!$this ->link) {
            return $text;
        }

        switch ($this ->monitorovaniBrowser ->proklikEditace) {
            case 2 :
                $targetStr = ' target="monvyrobyeditace"';
                break;

            case 3 :
                $targetStr = ' target="_blank"';
                break;

            case 1 :
                $targetStr = '';
                break;

            case 0 :
            default :
                return $text;
        }

        return
            '<a href="' . $this ->getLinkUrl($recordId, $rowData) . '"' .
            $targetStr .
            '>' .
            $text .
            '</a>';
    }
}


class MonitorovaniVyrobyBrowser extends RonPortalBrowser
{
    public $zobrazitVkladani = false;

    /** @var int    Urcuje, kolik dnu zpetne se nacte */
    private $pocetDnu = 1;

    public $proklikEditace = 1;

    /** @var BrowserField   Sloupec pro zobrazeni pritomnosti */
    private $fPritomnost = null;


    /**
     * MonitorovaniVyrobyBrowser constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        parent::__construct($database, 'monitorovaniv');

        $this ->title = \Ron\ronText('Monitorovani_vyroby');
        $this ->code = 'browsermonitorovaniv';

        $this ->sqlSelectPart =
            'PracovniPomer.Id, VyrobniCinnost.DruhZpracovani, PuvodniVyroba.DatumCas'; // 0..2
            // dalsi pole jdou doplnit v XML formulari config_browsermonitorovaniv, napriklad takto:
            // <column sqlFieldName="Osoba.Uzivatelska1" header="Mobil"/>

        $this ->primaryField = 0;

        // DruhZpracovani: 0=zacatek, 1=konec, 2=konec s odvedenim vyroby, 3=odvedeni vyroby, 4=ignorovat, 5=preruseni vyroby, 6=pokracovani prerusene vyroby

        $this ->distinct = true;

        $this ->cssClass .= ' monitorovani-vyroby';

        $this ->sqlFromPart = 'NASTAVENO AZ PO NACTENI KONFIGURACE!';

        $this ->addField(new MonitorovaniVyrobyBrowserField($this ->database, $this))
            ->setSqlFieldName('PracovniPomer.OsobniCislo')
            ->setCode('col-osobnicislo')
            ->setHeaderText('Osobni_cislo');

        $this ->addField(new MonitorovaniVyrobyBrowserField($this ->database, $this))
            ->setSqlFieldName('Osoba.Prijmeni')
            ->setCode('col-prijmeni')
            ->setHeaderText('Prijmeni')
            ->setLink('?menuid=vyrobaosoby&osoba=');

        $this ->addField(new MonitorovaniVyrobyBrowserField($this ->database, $this))
            ->setSqlFieldName('Osoba.Jmeno')
            ->setCode('col-jmeno')
            ->setHeaderText('Jmeno')
            ->setLink('?menuid=vyrobaosoby&osoba=');

        $this ->addField()
            ->setSqlFieldName('PuvodniVyroba.DatumCas', 2) ->setFormat('datetime')
            ->setCode('col-cas')
            ->setClassName('col-datum-cas')
            ->setHeaderText('Cas');

        $this ->addField()
            ->setSqlFieldName('VyrobniCinnost.Popis')
            ->setCode('col-popiscinnosti')
            ->setHeaderText('Vyroba_cinnost');

        $this ->addField( new VlozeniLinkField($this ->database) );

        $this ->fPritomnost = $this->addField(new PritomnostBrowserField(0));
        $this ->fPritomnost
            ->setHeaderText('Pritomnost')
            ->setCode('col-pritomnost');
    }


    function setFilterForm()
    {
        $this ->filterForm = new RonPortalFilterForm($this ->database, 'monitorovaniv', true);
        $prava = Prava::vratPrava('vyroba');
        $this ->filterForm ->addFilterRozdeleni1(null, $prava);
        $this ->filterForm ->addFilterRozdeleni2(null, $prava);
    }


    public function getTableAttributes()
    {
        if ($this ->fPritomnost ->visible) {
            $this ->cssClass .= ' pritomnost-container';
        }

        $result = parent::getTableAttributes();

        if ($this ->fPritomnost ->visible) {
            $result['data-ron-pritomnost-url'] =
                '?menuid=monitorovaniv&action=pritomnost' .
                '&rozdeleni1=' . StringUtils::htmlTextConvert($this ->filterForm ->getFilterValue('rozdeleni')) .
                '&vcetnepodrizenych1=' . (int)$this ->filterForm ->getFilterValue('rozdeleni', 'vcetnePodrizenych') .
                '&rozdeleni2=' . StringUtils::htmlTextConvert($this ->filterForm ->getFilterValue('rozdeleni2')) .
                '&vcetnepodrizenych2=' . (int)$this ->filterForm ->getFilterValue('rozdeleni2', 'vcetnePodrizenych');
        }

        return $result;
    }


    function loadConfigurationItem($key, $value, $valueText)
    {
        switch ($key) {
            case 'pocetDnu' :
                $this ->pocetDnu = (int)$valueText;
                break;

            case 'proklikeditace' :
                $this ->proklikEditace = (int)$valueText;
                break;

            default :
                parent::loadConfigurationItem($key, $value, $valueText);
        }
    }


    function getSQL($sqlFilter = '')
    {
        $this ->sqlFromPart =
            'PracovniPomer INNER JOIN Osoba ON Osoba.RC = PracovniPomer.RC' .
            ' LEFT OUTER JOIN PuvodniVyroba' .
            ' ON PuvodniVyroba.RC = PracovniPomer.RC' .
            ' AND PuvodniVyroba.OsobniCislo = PracovniPomer.OsobniCislo' .
            ' AND PuvodniVyroba.DatumNastupu = PracovniPomer.DatumNastupu' .
            ' AND PuvodniVyroba.DatumCas > ' . $this ->database ->dateToString(DateUtils::incDayStart(time(), -$this ->pocetDnu)) . // nacita data od zacatku predchoziho dne
            ' LEFT OUTER JOIN VyrobniCinnost' .
            ' ON VyrobniCinnost.Kod = PuvodniVyroba.KodVyrobniCinnosti';

        return parent::getSQL($sqlFilter);
    }


    function getOrderSQLPart()
    {
        //return parent::getOrderSQLPart();
        return 'Osoba.Prijmeni, Osoba.Jmeno, PracovniPomer.Id, PuvodniVyroba.DatumCas';
    }

    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        $prava = Prava::vratPrava('vyroba');
        $filtrPrava = $prava ->sqlFiltr('PracovniPomer', time(), time(), $params);

        if ($filtrPrava) {
            $result .=
                ($result ? ' AND ' : '') .
                $filtrPrava;
        }

        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            $filtr = $rozdeleniFilter ->sestavSQLFiltr($this ->database, 'PracovniPomer', $params);

            if ($filtr) {
                $result .=
                    ($result ? ' AND ' : '') .
                    '(' . $filtr . ')';
            }
        }

//        if ($this ->filterForm ->kodTypuRozdeleni  &&  $this ->filterForm ->kodRozdeleni !== null) {
//            $filtr = $this ->filterForm ->vratSQLFiltrRozdeleni('PracovniPomer', $this ->filterForm ->vcetnePodrizenychRozdeleni, 0, 0, $params);
//
//            if ($filtr) {
//                $result .=
//                    ($result ? ' AND ' : '') .
//                    '(' . $filtr . ')';
//            }
//        }
//
//        if ($this ->filterForm ->kodTypuRozdeleni2  &&  $this ->filterForm ->kodRozdeleni2) {
//            $filtr = RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'PracovniPomer',
//                $this ->filterForm ->kodTypuRozdeleni2, $this ->filterForm ->kodRozdeleni2,
//                time(), time(),
//                $this ->filterForm ->vcetnePodrizenychRozdeleni2,
//                $params);
//
//            if ($filtr) {
//                $result .=
//                    ($result ? ' AND ' : '') .
//                    '(' . $filtr . ')';
//            }
//        }

        return new SqlStatement($result, $params);
    }


    function loadSQL($sql) // , $firstRow  * @param int  $firstRow          Prvni radek, ktery se ma zobrazit
    {
        parent::loadSQL($sql);

        $data = [];

        $lastRow = null;

        $pravaVkladani = Prava::vratPrava('vkladanivyroby');
        $this ->zobrazitVkladani = false; // pokud najde osobu, pro kterou muze vkladat, tak nastavi na true

        while ($row = $this ->cursor ->fetch()) {
            if ($this ->enableLine($row)) {
                if (!$this ->zobrazitVkladani  &&  $pravaVkladani ->lzeZobrazitPP($row[0])) {
                    $this ->zobrazitVkladani = true;
                }

                // $row[1] DruhZpracovani: 0=zacatek, 1=konec, 2=konec s odvedenim vyroby, 3=odvedeni vyroby, 4=ignorovat, 5=preruseni vyroby, 6=pokracovani prerusene vyroby
                if ($lastRow && $lastRow[0] !== $row[0]) {
                    $data[] = $lastRow;
                    $lastRow = null;
                }

                if (!$lastRow || $row[1] != 4) {  // v pripade, ze uz ma pro danou osobu nacteny nejaky radek, tak ignoruje radek s druhem zpracovani "ignorovat"
                    $lastRow = $row;
                }
            }
        }

        if ($lastRow) {
            $data[] = $lastRow;
        }

        $this ->cursor = new ArrayCursor($data);
        $this ->cursor ->setDatabase($this ->database);
    }


    function getRowAttributes($values, $className, $rowNumber)
    {
        $className .= isset($values[1]) ? ' druh-zpracovani-' . $values[1] : ' druh-zpracovani-NULL';

        $time = $this ->database ->stringToTimestamp($values[2]);
        if ($time && $time <= time() - 12 * 60 * 60) {
            // zaznam starsi nez 12 hodin
            $className .= ' monitorovani-12';
        }

        return
            parent::getRowAttributes($values, $className, $rowNumber) .
            " data-id-pp='{$values[0]}'" .
            " data-jmeno='" . StringUtils::htmlTextConvert($values['col-jmeno'] . ' ' . $values['col-prijmeni']). "'" .
            " data-osobni-cislo='" . StringUtils::htmlTextConvert($values['col-osobnicislo']) . "'";
    }

}
