<?php
/**
 * stanislav.elbl, 05.10.2021 0:16
 */

namespace RonPortal\Vyroba;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Dochazka\DenniVysledek;
use RonPortal\Dochazka\SchvaleniDochazky;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;

class ZaznamDenniVyroby extends ZaznamVyroby
{
    /** @var string */
    public $hodinyStr;

    /** @var bool   Priznak, ze byl zaznam nacten z POST */
    //public $posted = false;
    /** @var bool   Priznak, ze byl zaznam zmenen v POST - bude se provadet update */
    public $jeZmena = false;


    public function nastavTexty()
    {
        $this ->hodinyStr = $this->hodiny
            ? StringUtils::format('minuty', round($this->hodiny * 60))
            : '';
    }
}


class RadekVyrobyOsoby
{
    /** @var string     Cislo zakazky */
    public $zakazka;
    /** @var string     Kod cinnosti */
    public $cinnost;
    /** @var int        Id cinnosti vyrobni zakazky */
    public $cinnostVyrobniZakazky;

    /** @var string */
    public $popisZakazky;
    public $platnostZakazkyOd;
    public $platnostZakazkyDo;

    public $popisCinnosti;
    public $datumCasCinnostiOd;
    public $datumCasCinnostiDo;

    /** @var bool   Priznak, ze se jedna o novy radek */
    public $novyRadek;
    public $smazatRadek;

    /** @var  ZaznamDenniVyroby[]   Zaznam pro kazdy den editovaneho obdobi */
    public $zaznamy;
}


class VyrobaOsobyForm implements ViewObject
{
    /** @var Database */
    public $database;

    /** @var PracovniPomer */
    public $pp;
    /** @var Obdobi */
    public $obdobi;

    /** @var SchvaleniDochazky */
    public $schvaleni;

    /** @var int */
    public $datumOd;
    /** @var int */
    public $datumDo;

    /** @var bool       Nastavi se na true, pokud je napriklad uzavrene obdobi */
    public $readOnly;

    // nastaveni
    /** @var bool       Priznak, ze se muze vybirat jen z povolenych cinnosti na vybrane zakazce */
    public $jenPrirazeneCinnosti;
    /** @var string[]   Ktere udaje se zobrazuji a vybiraji na radku (vyhledavaji se podle nich zaznamy) */
    public $udajeNaRadku;
    /** @var string[]   Ktere udaje se edituji v dodatecnych polich */
    public $editovaneUdaje;

    public $kontrolovatSoucet = false;

    // data
    /** @var RadekVyrobyOsoby[] */
    public $radky;
    /** @var float[]    Vypoctene soucty rozdelenych hodin */
    public $soucty;
    /** @var float[]    Nactene odpracovane hodiny za jednotlive dny */
    public $odpracovano;
    /** @var bool[]     Priznak, ze je ve sloupci chyba (prekroceny limit) */
    public $chybySloupcu;
    /** @var bool[]     Priznak, ze je mozne sloupec editovat (tj. neni schvalena dochazka) */
    public $lzeEditovatDen;

    /** @var VyrobniZakazka[]   Seznam zakazek pro select v novem radku */
    public $zakazky;
    /** @var CinnostDenniVyroby Seznam cinnosti pro select v novem radku */
    public $cinnosti;

    public $renderHtmlForm = true;
    public $kontrolovatPrava;


    /**
     * VyrobaOsobyForm constructor.
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $kontrolovatPrava
     */
    public function __construct(Database $database, PracovniPomer $pp, $datumOd, $datumDo, $kontrolovatPrava = true)
    {
        $this->database = $database;
        $this->pp = $pp;
        $this->datumOd = $datumOd;
        $this->datumDo = $datumDo;

        $this->kontrolovatPrava = $kontrolovatPrava;

        $this->jenPrirazeneCinnosti = RonPortalSettings::getIntValue('jenprirazenecinnosti');

        $this->udajeNaRadku = [
            'zakazka',
            $this->jenPrirazeneCinnosti ? 'cinnostVyrobniZakazky' : 'cinnost'
        ];

        $this->editovaneUdaje = ['popis', 'rozsirujiciPopis'];         // TODO: vyrobek, mnozstviDobre, mnozstviSpatne, casOd,

        $this ->kontrolovatSoucet = (bool)RonPortalSettings::getValue('vyroba_casovaslozkasoucet');
    }


    /**
     * @param RadekVyrobyOsoby $radek
     * @param int $indexDne
     * @return bool
     */
    function lzeEditovatZaznam($radek, $indexDne)
    {
        if (!$this ->lzeEditovatDen[$indexDne]) {
            return false;
        }

        // zkontrolovat platnosti
        $platnostOd = $radek ->platnostZakazkyOd;
        if ($radek ->datumCasCinnostiOd   &&   (!$platnostOd  ||  $radek ->datumCasCinnostiOd > $platnostOd)) {
            $platnostOd = DateUtils::startOfTheDay($radek ->datumCasCinnostiOd);
        }

        $platnostDo = $radek ->platnostZakazkyDo;
        if (!$platnostDo  ||  ($radek ->datumCasCinnostiDo  &&  $radek ->datumCasCinnostiDo < $platnostDo)) {
            $platnostDo = $radek ->datumCasCinnostiDo;
        }

        if ($platnostOd || $platnostDo) {
            $datum = DateUtils::incDayStart($this ->datumOd, $indexDne);
            if ($platnostOd  &&  $platnostOd > $datum) {
                return false;
            }

            if ($platnostDo  &&  $platnostDo < $datum) {
                return false;
            }
        }

        return true;
    }

    /**
     * @param ZaznamDenniVyroby $zaznam
     * @return RadekVyrobyOsoby
     */
    private function pridejRadek($zaznam)
    {
        $radek = new RadekVyrobyOsoby();
        $radek ->zakazka = $zaznam ->zakazka;
        $radek ->popisZakazky = $zaznam ->popisZakazky;
        $radek ->platnostZakazkyOd = $zaznam ->platnostZakazkyOd;
        $radek ->platnostZakazkyDo = $zaznam ->platnostZakazkyDo;

        $radek ->cinnost = $zaznam ->cinnost;
        $radek ->cinnostVyrobniZakazky = $zaznam ->cinnostVyrobniZakazky;
        $radek ->popisCinnosti = $zaznam ->popisCinnosti;
        $radek ->datumCasCinnostiOd = $zaznam ->datumCasCinnostiOd;
        $radek ->datumCasCinnostiDo = $zaznam ->datumCasCinnostiDo;

        $radek ->zaznamy = [];

        $this ->radky[] = $radek;

        return $radek;
    }


    /**
     * Najde existujici radek k zadanemu zaznamu, nebo prida novy (pokud je $pridatNovy=true)
     * @param ZaznamDenniVyroby $zaznam
     * @param $pridatNovy
     * @return RadekVyrobyOsoby
     */
    private function najdiRadek($zaznam, $pridatNovy)
    {
        $hledatZakazku = in_array('zakazka', $this ->udajeNaRadku);
        $hledatCinnost = in_array('cinnost', $this ->udajeNaRadku);
        $hledatCinnostZakazky = in_array('cinnostVyrobniZakazky', $this ->udajeNaRadku);

        foreach ($this->radky as $radek) {
            // pokud se shoduji nakonfigurovane sloupce, tak nasel radek
            if ( (!$hledatZakazku  ||  !strcasecmp($radek ->zakazka ?? '', $zaznam ->zakazka ?? '') )
                && (!$hledatCinnost  ||  !strcasecmp($radek ->cinnost ?? '', $zaznam ->cinnost ?? '') )
                && (!$hledatCinnostZakazky  ||  $radek ->cinnostVyrobniZakazky == $zaznam ->cinnostVyrobniZakazky)
            )
            {
                return $radek;
            }
        }

        if ($pridatNovy) {
            return $this ->pridejRadek($zaznam);
        }

        return null;
    }


    public function nactiDataProComboboxy()
    {
        if ($this ->zakazky) {
            // data jsou uz nactena
            return;
        }

        // data pro comboboxy
        $this ->zakazky = RonPortalSettings::getIntValue('jenprirazenezakazky')
            ? VyrobniZakazka::nactiVyrobniZakazkyOsoby($this ->database, $this ->datumOd, $this ->datumDo, $this ->pp)
            : VyrobniZakazka::nactiVyrobniZakazky($this ->database, $this ->datumOd, $this ->datumDo);

        $this ->cinnosti = $this ->jenPrirazeneCinnosti
            ? CinnostDenniVyroby::nactiPrirazeneCinnosti($this ->database, $this ->pp, $this ->datumOd, $this ->datumDo)
            : CinnostDenniVyroby::nactiVyrobniCinnosti($this ->database);
    }

    /**
     * Prida novy radek, ale jen kdyz jeste neexistuje zadny radek s priznakem "novy"
     */
    public function doplnNovyRadek()
    {
        if ($this ->readOnly) {
            return;
        }

        $existuje = false;
        foreach ($this ->radky as $radek) {
            if ($radek ->novyRadek) {
                $existuje = true; // novy radek uz existuje
            }
        }

        if (!$existuje) {
            $radek = new RadekVyrobyOsoby();
            $radek->zaznamy = [];
            $radek->novyRadek = true;
            $this->radky[] = $radek;
        }

        $this ->nactiDataProComboboxy();
    }



    /**
     * @param int $indexRadku
     * @return RadekVyrobyOsoby
     */
    private function najdiRadekProPost($indexRadku)
    {
        $vyplnenoVse = true;
        $vyplnenoNeco = false;

        // vytvori a naplni zaznam udaji z POST - slouzi jen pro vyhledani radku v metode najdiRadek()
        $zaznam = new ZaznamDenniVyroby();
        if (in_array('zakazka', $this ->udajeNaRadku)) {
            $zaznam ->zakazka = $_POST['zakazka_' . $indexRadku];
            if (trim($zaznam ->zakazka) == '') {
                $vyplnenoVse = false;
            } else {
                $vyplnenoNeco = true;
            }
        }

        if (in_array('cinnost', $this ->udajeNaRadku)) {
            $zaznam ->cinnost = $_POST['cinnost_' . $indexRadku];
            if (trim($zaznam ->cinnost) == '') {
                $vyplnenoVse = false;
            } else {
                $vyplnenoNeco = true;
            }
        }

        if (in_array('cinnostVyrobniZakazky', $this ->udajeNaRadku)) {
            $zaznam ->cinnostVyrobniZakazky = (isset($_POST['cinnostVyrobniZakazky_' . $indexRadku])  &&  trim($_POST['cinnostVyrobniZakazky_' . $indexRadku]) != '')
                ? (int)$_POST['cinnostVyrobniZakazky_' . $indexRadku]
                : null;
            if (isset($zaznam ->cinnostVyrobniZakazky)) {
                $vyplnenoNeco = true;
            } else {
                $vyplnenoVse = false;
            }
        }

        if (!$vyplnenoNeco) {
            return null;
        }

        // TODO: osetrit pokud neni "$vyplnenoVse"

        $result = $this ->najdiRadek($zaznam, false);

        // nacte priznak, ze se jedna o novy radek
        $jeNovyRadek = isset($_POST['novyRadek_' . $indexRadku])  &&  (int)$_POST['novyRadek_' . $indexRadku];
        $smazatRadek = isset($_POST['deleteRowIndex']) && $_POST['deleteRowIndex'] != '' && (int)$_POST['deleteRowIndex'] == $indexRadku;

        if ($jeNovyRadek) {
            if ($smazatRadek) {
                // novy radek, ktery se nema ukladat
                return null;
            }

            if ($result) {
                // jedna se o nove pridany radek, ale stejny radek uz existuje => chyba
                MessageManager::addErrorMessage(\Ron\ronText('Radek_vyroby_jiz_existuje'));

                // radek prida, aby uzivatel mohl vybrat jinou zakazku
                $result = $this->pridejRadek($zaznam);
                $result->novyRadek = true;
                $this->nactiDataProComboboxy();
            }
        } else if ($result) {
            // jiz existujici radek - nastavi priznak smazani
            $result ->smazatRadek = $smazatRadek;
        }

        if (!$result  &&  !$smazatRadek) {
            // doplni novy radek
            $result = $this ->pridejRadek($zaznam);
            $result ->novyRadek = true;
            $this ->nactiDataProComboboxy();
        }

        return $result;
    }


    /**
     * Vytvori novy zaznam na zadanem radku v zadanem poli (index dne)
     * @param RadekVyrobyOsoby $radek
     * @param int $index
     * @return ZaznamDenniVyroby
     * @throws \Exception
     */
    private function vytvorNovyZaznam($radek, $index)
    {
        if (isset($radek->zaznamy[$index])) {
            throw new \Exception('Chyba v metode VyrobaOsobyForm::vytvorNovyZaznam - zaznam na zadanem indexu uz existuje!');
        }

        $zaznam = new ZaznamDenniVyroby();
        $zaznam->den = DateUtils::incDay($this->datumOd, $index);
        $zaznam->cinnost = $radek->cinnost;
        $zaznam->cinnostVyrobniZakazky = $radek->cinnostVyrobniZakazky;
        $zaznam->zakazka = $radek->zakazka;

        $radek->zaznamy[$index] = $zaznam;
        return $zaznam;
    }


    /**
     * Zpracuje seznam zaznamu vyroby - doplni je na spravna mista
     * @param ZaznamDenniVyroby[] $zaznamy
     */
    private function doplnZaznamy($zaznamy)
    {
        foreach ($zaznamy as $zaznam) {
            $zaznam ->nastavTexty();

            $radek = $this ->najdiRadek($zaznam, true); // pridat=true - automaticky prida novy radek, pokud jeste neexistuje
            $index = DateUtils::daysBetween($this ->datumOd, $zaznam ->den);

            $radek ->zaznamy[$index] = $zaznam;
        }
    }

    private function spocitejSoucty()
    {

        $this ->soucty = [];

        $daysCount = DateUtils::daysBetween($this ->datumOd, $this ->datumDo) + 1;
        for ($i = 0;  $i < $daysCount;  $i ++) {
            $this ->soucty[$i] = 0;

            foreach ($this ->radky as $radek) {
                $this ->soucty[$i] += isset($radek ->zaznamy[$i]) ? $radek ->zaznamy[$i] ->hodiny : 0;
            }
        }
    }


    private function nactiSchvaleni()
    {
        $this ->schvaleni = SchvaleniDochazky::nactiSchvaleni($this ->database, $this ->pp, $this ->datumOd);
        $this ->lzeEditovatDen = [];

        $daysCount = DateUtils::daysBetween($this ->datumOd, $this ->datumDo) + 1;
        for ($i = 0;  $i < $daysCount;  $i ++) {
            $this->lzeEditovatDen[$i] = $this ->schvaleni ->lzeEditovatPP($this ->pp ->idPP, DateUtils::incDay($this ->datumOd, $i));
        }
    }


    /**
     * @return void
     * @throws DBException
     */
    private function nactiDenniVysledky()
    {
        $this ->odpracovano = [];

        $daysCount = DateUtils::daysBetween($this ->datumOd, $this ->datumDo) + 1;
        for ($i = 0; $i < $daysCount; $i++) {
            $this ->odpracovano[$i] = 0;
        }

        if ($this ->kontrolovatSoucet) {
            $denniVysledky = DenniVysledek::nactiDenniVysledky($this->database,
                $this->pp, $this->datumOd, $this ->datumDo, RonPortalSettings::getValue('vyroba_casovaslozkasoucet'));

            foreach ($denniVysledky as $denniVysledek) {
                $index = DateUtils::daysBetween($this ->datumOd, $denniVysledek ->den);
                $this ->odpracovano[$index] = $denniVysledek ->hodiny;
            }
        }
    }


    /**
     * @param ZaznamDenniVyroby[] $zaznamy
     */
    public function doplnCinnosti($zaznamy)
    {
        $cinnosti = CinnostDenniVyroby::nactiCinnostiVObdobi($this ->database,
            $this ->pp, $this ->datumOd, $this ->datumDo, $this->jenPrirazeneCinnosti);

        /** @var CinnostDenniVyroby[] $seznam */
        $seznam = [];
        foreach ($cinnosti as $cinnost) {
            $seznam[$this->jenPrirazeneCinnosti ? $cinnost ->cinnostVyrobniZakazky : $cinnost ->kod] = $cinnost;
        }

        foreach ($zaznamy as $zaznam) {
            $index = $this->jenPrirazeneCinnosti ? $zaznam ->cinnostVyrobniZakazky : $zaznam ->cinnost;

            if (isset($seznam[$index])) {
                $zaznam ->popisCinnosti = $seznam[$index] ->popis;
                $zaznam ->datumCasCinnostiOd = $seznam[$index] ->datumCasOd;
                $zaznam ->datumCasCinnostiDo = $seznam[$index] ->datumCasDo;
            }
        }
    }


    /**
     * Nacte data z databaze
     * @throws \Exception
     */
    public function load()
    {
        $this ->obdobi = Obdobi::vratObdobi($this ->database, 'ADS', $this ->datumOd);

        $this ->nactiSchvaleni();

        if ($this ->kontrolovatPrava) {
            Prava::nastavObdobiProPrava($this->datumOd, $this->datumDo);
            if (!Prava::vratPrava('vyroba')->lzeZobrazitPP($this->pp->idPP, $this->datumOd, $this->datumDo)) {
                throw new NeniPravoException();
            }
        }

        $this ->readOnly =
            $this ->obdobi ->uzavreno ||
            ($this ->kontrolovatPrava  &&  !Prava::vratPrava('editacevyroby') ->lzeZobrazitPP($this ->pp ->idPP, $this ->datumOd, $this ->datumDo));

        $this ->radky = [];

        $orderBy = '';
        foreach ($this ->udajeNaRadku as $coZobrazit) {
            switch ($coZobrazit) {
                case 'zakazka' :
                    $orderBy .= ($orderBy ? ', ' : '') . 'Vyroba.VyrobniZakazka';
                    break;
                case 'cinnost' :
                    $orderBy .= ($orderBy ? ', ' : '') . 'Vyroba.Cinnost';
                    break;
                case 'cinnostVyrobniZakazky' :
                    $orderBy .= ($orderBy ? ', ' : '') . 'Vyroba.CinnostVyrobniZakazky';
                    break;
            }
        }

        // nacte zaznamy vyroby
        $zaznamy = ZaznamDenniVyroby::nactiVyrobu($this ->database, $this ->pp, $this ->datumOd, $this ->datumDo, $orderBy);
        $this ->doplnCinnosti($zaznamy); // doplni popisy cinnosti

        // podle nactenych dat vytvori seznam radku a doplni je
        $this ->doplnZaznamy($zaznamy);

        $this ->nactiDenniVysledky();
        $this ->spocitejSoucty();
    }


    /**
     * Doplni zaznamy z predchoziho obdobi
     * @throws DBException
     */
    public function doplnZaznamyZPredchozihoObdobi()
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT DISTINCT V1.Cinnost, V1.CinnostVyrobniZakazky, V1.VyrobniZakazka' .
            ' FROM Vyroba V1, VyrobniZakazka Z' .
            ' WHERE Z.CisloZakazky = V1.VyrobniZakazka' .
            ' AND (Z.PlatnostDo IS NULL  OR  Z.PlatnostDo >= ' . $params ->addParameter($this ->database ->dateToParam($this ->datumOd)) . ')' .
            // zaznamy z predchoziho mesice
            ' AND V1.Den < ' . $params ->addParameter($this ->database ->dateToParam($this ->datumOd)) .
            ' AND V1.Den >= ' . $params ->addParameter($this ->database ->dateToParam(DateUtils::incMonth($this ->datumOd, -1))) .
            // aktualniho zamestnance
            ' AND V1.RC = ' . $params ->addParameter($this ->pp ->rc) .
            ' AND V1.OsobniCislo = ' . $params ->addParameter($this ->pp ->osobniCislo) .
            ' AND V1.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->pp ->datumNastupu)) .
            // ktere jeste neexistuji v aktualnim obdobi
            ' AND NOT EXISTS (' .  //
            '   SELECT * FROM Vyroba V2' .
            '   WHERE V2.RC = V1.RC  AND  V2.OsobniCislo = V1.OsobniCislo  AND  V2.DatumNastupu = V1.DatumNastupu' .
            '   AND V2.Cinnost = V1.Cinnost  AND V2.CinnostVyrobniZakazky = V1.CinnostVyrobniZakazky' .
            '   AND V2.VyrobniZakazka = V1.VyrobniZakazka' .
            '   AND V2.Den >= ' . $params ->addParameter($this ->database ->dateToParam($this ->datumOd)) .
            '   AND V2.Den <= ' . $params ->addParameter($this ->database ->dateToParam($this ->datumDo)) .
            ')';

        $datum = DateUtils::today();
        if ($datum < $this ->datumOd) {
            $datum = $this ->datumOd;
        } else if ($datum > $this ->datumDo) {
            $datum = $this ->datumDo;
        }

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $rec = new VyrobaDbRecord($this ->database, 0, $this);

            $rec->fCinnost->setNewValue($row[0]);
            $rec->fCinnostVyrobniZakazky->setNewValue($row[1]);
            $rec->fVyrobniZakazka->setNewValue($row[2]);
            $rec->addField('', '', true, 0, '   ... automaticke doplneni podle predchoziho obdobi');

            $rec ->setNewRecord($this ->pp, $datum);
            $rec ->insertRecord();
        }
        $cursor ->close();
    }



    /**
     * Smaze radky, urcene ke smazani a odstrani je ze seznamu
     */
    private function smazRadky()
    {
        for ($i = count($this ->radky) - 1;  $i >= 0;  $i--) {
            if ($this ->radky[$i]->smazatRadek) {
                foreach ($this ->radky[$i] ->zaznamy as $zaznam) {
                    if ($zaznam->serialVyroba) {
                        $zaznam->delete($this->database, $this);
                    }
                }

                array_splice($this ->radky, $i, 1);
            }
        }
    }

    private function saveData()
    {
        foreach ($this ->radky as $radek) {
            $ulozeno = false;

            foreach ($radek ->zaznamy as $index => $zaznam) {
                if (isset($this ->chybySloupcu[$index])  &&  $this ->chybySloupcu[$index]) {
                    // ve sloupci je chyba (prekroceny limit) => neuklada v nem data
                } else {
                    if ($radek->smazatRadek) {
                        throw new \Exception('Chyba v metode VyrobaOsobyForm::saveData - nalezen radek, ktery ma byt smazany!'); // nemelo by nastat
                    }

                    if ($zaznam->jeZmena) {
                        if ($zaznam->serialVyroba) {
                            $zaznam->update($this->database, $this);
                            $ulozeno = true;
                        } else {
                            //var_dump($zaznam);
                            $zaznam->insert($this->database, $this->pp, $zaznam->den, $this);
                            $ulozeno = true;
                        }
                    }
                }

                if ($zaznam->serialVyroba) {
                    $ulozeno = true;
                }
            }

            if (!$ulozeno) {
                // novy radek, u ktereho nebyl vyplnen zadny edit => radek se neulozil
                // je potreba doplnit zaznam s nulovou hodnotou - do aktualniho dne nebo nejblize k nemu

                // urci datum, do ktereho se zaznam vlozi
                $datum = DateUtils::today();
                if ($datum < $this ->datumOd) {
                    // edituji budouci obdobi
                    $datum = $this ->datumOd;
                } else if ($datum > $this ->datumDo) {
                    $datum = $this ->datumDo;
                }

                $index = DateUtils::daysBetween($this ->datumOd, $datum);
                $zaznam = $this ->vytvorNovyZaznam($radek, $index);
                $zaznam ->insert($this ->database, $this ->pp, $zaznam ->den, $this);
            }
        }
    }


    /**
     * Zpracuje POST
     */
    public function process()
    {
        if ($this ->obdobi ->uzavreno) {
            MessageManager::addErrorMessage( \Ron\ronText('Obdobi_uzavreno') );
            return;
        }

        // originalni data uz jsou nactena
        $count = (int)$_POST['vyroba_rows_count'];
        $pocetDnu = DateUtils::daysBetween($this ->datumOd, $this ->datumDo) + 1;
        $this ->chybySloupcu = [];

        //$jeZmena = false;

        for ($indexRadku = 0;  $indexRadku < $count;  $indexRadku++) {
            $radek = $this ->najdiRadekProPost($indexRadku);

            if ($radek && !$radek ->smazatRadek) {
                for ($i = 0; $i < $pocetDnu; $i++) {
                    $zaznam = isset($radek ->zaznamy[$i]) ? $radek ->zaznamy[$i] : null;

                    $originalStr = $zaznam ? $zaznam->hodinyStr : '';
                    $newStr = trim($_POST["hodnota_{$indexRadku}_{$i}"]);

                    $jeZmena = $originalStr != $newStr ||
                        ($zaznam ? $zaznam ->popis : '') != $_POST["popis_{$indexRadku}_{$i}"] ||
                        ($zaznam ? $zaznam ->rozsirujiciPopis : '') != $_POST["rozsirujiciPopis_{$indexRadku}_{$i}"];


                    if ($jeZmena) {
                        $datum = $zaznam ? $zaznam ->den : DateUtils::incDay($this->datumOd, $i);

                        if (!$this ->schvaleni ->lzeEditovatPP($this ->pp ->idPP, $datum)) {
                            // dochazka uz je schvalena => chyba - nelze editovat
                            MessageManager::addErrorMessage( sprintf(\Ron\ronText('Nelze_editovat_den_dochazka_schvalena'), StringUtils::format('date', $datum)) );
                        } else {
                            if (!$zaznam) {
                                $zaznam = $this->vytvorNovyZaznam($radek, $i);
                            }
                            $zaznam->jeZmena = true;

                            if ($originalStr != $newStr) {
                                $zaznam->hodinyStr = $newStr;

                                try {
                                    $zaznam->hodiny = ($newStr === '')
                                        ? 0
                                        : DateUtils::timeStringToFloat($newStr, true);
                                } catch (\Exception $e) {
                                    MessageManager::addErrorMessage(StringUtils::format('date', $datum) . ': ' . $e->getMessage(), "hodnota_{$indexRadku}_{$i}");
                                }
                            }

                            $zaznam->popis = $_POST["popis_{$indexRadku}_{$i}"];
                            $zaznam->rozsirujiciPopis = $_POST["rozsirujiciPopis_{$indexRadku}_{$i}"];
                        }
                    }
                }
            }
        }

        $this ->spocitejSoucty();

        if (!MessageManager::getInstance() ->isError()) {
            // odstrani radky, ktere se maji smazat
            $this ->smazRadky();

            if (isset($_POST['doplnitRadky'])) {
                $this ->doplnZaznamyZPredchozihoObdobi();
            }

            // kontrola limitu
            if ($this ->kontrolovatSoucet) {
                for ($i = 0; $i < $pocetDnu; $i++) {
                    if (round(($this->odpracovano[$i] - $this->soucty[$i]) * 60) < 0) {
                        // chyba - zadano vice hodin, nez je odpracovano
                        $this->chybySloupcu[$i] = true;
                        MessageManager::addErrorMessage(date('j.n.Y', DateUtils::incDay($this->datumOd, $i)) . ': ' . \Ron\ronText('Nelze_rozdelit_vice_vyroby'));
                    }
                }
            }

            $this ->saveData();
        }
    }


    /**
     * @param RadekVyrobyOsoby $radek
     * @param int $cisloRadku
     */
    public function renderInputZakazka($radek, $cisloRadku)
    {
        $inputName = "zakazka_$cisloRadku";

        if ($radek ->novyRadek) {
            // priznak "novy radek"
            echo "<input type='hidden' name='novyRadek_$cisloRadku' value='1'/>";

            $captions = TextManager::_('Kod') . ';' . TextManager::_('Popis');
            // select
            echo "<select name='$inputName' class='form-control vyrobni_zakazka ronselect' data-captions='$captions' data-max-visible-rows='200'>";

            if (!isset($radek ->zakazka) ||  $radek ->zakazka === '') {
                // novy zaznam - prida prazdny radek
                echo '<option value=""></option>';
            }

            foreach ($this ->zakazky as $zakazka) {
                echo
                    '<option value="' . StringUtils::htmlTextConvert($zakazka ->cisloZakazky) . '"' .
                    (($zakazka ->cisloZakazky === $radek ->zakazka)
                        ? ' selected="selected"'
                        : ''
                    ) .
                    '>' . StringUtils::htmlTextConvert($zakazka ->popis) . '</option>';
            }

            echo "</select>\n";
        } else {
            // jiz existujici radek - jen text
            echo
                "<input type='hidden' name='$inputName' value='" . StringUtils::htmlTextConvert($radek->zakazka) . "'/>" .
                StringUtils::htmlTextConvert($radek->popisZakazky);
        }
    }


    /**
     * @param RadekVyrobyOsoby $radek
     * @param string $inputName
     * @param int $cisloRadku
     */
    public function renderInputCinnost($radek, $inputName, $cisloRadku, $value)
    {
        if ($radek ->novyRadek) {
            $captions = TextManager::_('Kod') . ';' . TextManager::_('Popis');
            $dalsiSloupce = '';
            if (RonPortalSettings::getIntValue('vyroba_cinnost_sloupce')) {
                switch (RonPortalSettings::getIntValue('vyroba_cinnost_sloupce')) {
                    case 1:
                        $dalsiSloupce = ';status';
                        $captions .= ';' . TextManager::_('vyroba_status_cinnosti');
                        break;
                    case 2:
                        $dalsiSloupce = ';typ';
                        $captions .= ';' . TextManager::_('vyroba_typ_cinnosti');
                        break;
                }
            }

            echo "<select name='$inputName' id='$inputName' class='form-control ronselect' data-columns='value;text{$dalsiSloupce}' data-captions='$captions' data-max-visible-rows='200'>";

            foreach ($this ->cinnosti as $cinnost) {
                if (!$this ->jenPrirazeneCinnosti  ||  $radek ->zakazka == $cinnost ->cisloZakazky) {
                    echo '<option value="' . ($this ->jenPrirazeneCinnosti ? $cinnost ->cinnostVyrobniZakazky : StringUtils::htmlTextConvert($cinnost->kod)) . '" data-status="' . StringUtils::htmlTextConvert($cinnost->statusPopis) . '" 
                            data-typ="' . StringUtils::htmlTextConvert($cinnost->typPopis) . '"';

                    // Podle nastaveni muze byt string (kod cinnosti) nebo int (id cinnosti vyrobni zakazky)
                    $jeZaznamVybrany =
                        ($this ->jenPrirazeneCinnosti && $cinnost ->cinnostVyrobniZakazky == (int)$value) ||
                        (!$this ->jenPrirazeneCinnosti && $cinnost ->kod === $value);

                    if ($jeZaznamVybrany) {
                        echo ' selected="selected"';
                    }
                    echo '>' . StringUtils::htmlTextConvert($cinnost->popis) . '</option>';
                }
            }

            echo "</select>\n";
        } else {
            echo
                "<input type='hidden' name='$inputName' value='" . StringUtils::htmlTextConvert($this ->jenPrirazeneCinnosti ? $radek ->cinnostVyrobniZakazky : $radek ->cinnost) . "'/>" .
                StringUtils::htmlTextConvert($radek ->popisCinnosti);
        }
    }


    public function render()
    {
        if (!$this ->readOnly) {
            if (!$this ->radky  ||  (isset($_GET['novyRadek']) && (int)$_GET['novyRadek']) || isset($_POST['novyRadek'])) {
                $this->doplnNovyRadek();
            }
        }

        require 'ronportal/vyroba/view_vyroba_osoby.php';
    }

}
