<?php
/**
 * stanislav.elbl
 * 22.05.2021 0:15
 */

namespace RonPortal\XML\Classes;


use Ron\MessageManager;
use RonPortal\RonPortalSession;
use RonPortal\User;
use RonPortal\XML\XMLFormElement;


/**
 * block, section, show, fill, lang, posledniodpoved
 * Class XMLElementBlock
 * @package RonPortal\XML\Classes
 */
class XMLElementBlock extends XMLFormElement
{
    public $name;


    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this ->name = $this ->getAttributeValue('name');

        if (!$this ->name  &&  $this ->xmlForm ->dataMode()) {
            if ($this->elementName == 'section') {
                // v datovem xml musi mit sekce name
                throw new \Exception('Chyba v XML formulari - sekce musi mit zadany parametr name!');
            }

            if ($this->elementName == 'block'  &&  !self::getBoolValue($this->getAttributeValue('copy', '1'))) {
                throw new \Exception('Chyba v XML formulari - block s copy="0" musi mit zadany parametr name!');
            }
        }
    }


    /**
     * Vraci true, pokud aktualni jazyk uzivatele vyhovuje podmince (include/exclude)
     * @return bool
     */
    private function isLanguage()
    {
        $langs_str = $this->getAttributeValue('include');
        if ($langs_str) {
            // je zadan parametr "include"
            $langs = explode(';', strtolower($langs_str));
            return in_array(strtolower(RonPortalSession::getLanguage()), $langs);
        }

        $langs = explode(';', strtolower($this->getAttributeValue('exclude')));
        return !in_array(strtolower(RonPortalSession::getLanguage()), $langs);  // false, pokud je aktualni jazyk v "exclude"
    }


    private function blockConditionFulfilled()
    {
        if (self::getBoolValue($this ->getAttributeValue('ignore'))) {
            return false;
        }

        $context = $this ->getAttributeValue('context');
        if (!$this ->xmlForm ->isContext($context)) {
            return false;
        }

        $action = $this ->getAttributeValue('action');
        if ($action  &&  $action != $this ->xmlForm ->xmlAction) {
            return false;
        }

        $variable = $this ->getAttributeValue('variable');
        if ($variable) {
            if ($variable[0] == '{') {
                if (!$this ->xmlForm->nahradPromenne($variable)) {
                    return false;
                }
            } else {
                if (!$this->xmlForm->getVariableValue($variable)) {
                    return false;
                }
            }
        }

        $variable = $this ->getAttributeValue('notvariable');
        if ($variable) {
            if ($variable[0] == '{') {
                if ($this ->xmlForm->nahradPromenne($variable)) {
                    return false;
                }
            } else {
                if ($this->xmlForm->getVariableValue($variable)) {
                    return false;
                }
            }
        }

        if ($profil = $this ->getAttributeValue('profile')) {
            // zkontroluje, jestli je uzivatel v zadanem profilu
            $profil = $this ->nahradPromenne($profil);
            return User::jeVProfilu($profil);
        }

        if ($getParamName = $this ->getAttributeValue('get')) {
            $getParamName = $this ->nahradPromenne($getParamName);
            // zkontroluje jestli je nastaven GET parametr zadaneho jmena a ma neprazdnou hodnotu
            if (!isset($_GET[$getParamName])  ||  !$_GET[$getParamName]) {
                return false;
            }
        }

        if ($this ->xmlForm ->dataMode()  &&  $this ->name  &&  !self::getBoolValue($this->getAttributeValue('copy', '1'))) {
            // block s copy="0" - pokud jiz byl vyplnen, tak jej uz ignoruje
            if ($this ->xmlForm ->getVariableValue($this ->name)) {
                return false;
            }
        }

        // TODO: pridat podminky pro predavaci proces (aktualni uroven)
        return true;
    }


    function getHTMLOutput()
    {
        switch ($this ->elementName) {
            case 'fill' :
                return '';

            case 'show' :
                break;

            case 'section' :
                // pokud uz nejaka sekce byla, tak tuto sekci nezobrazuje
                if ($this ->xmlForm ->dataMode()  &&  $this ->xmlForm ->getVariableValue($this ->name)) {
                    // sekce uz byla vyplnena
                    if (self::getBoolValue($this ->getAttributeValue('repeat', '0'))) {
                        // ..., ale ma se zobrazit znovu
                        break;
                    }
                    return '';
                }

                if (!$this ->xmlForm ->prvniSekce) {
                    // uz je zobrazena predchozi sekce
                    $this ->xmlForm ->viceSekci = true;
                    return '';
                }

                $this ->xmlForm ->prvniSekce = false;
                break;

            case 'block' :
                if (!$this ->blockConditionFulfilled()) {
                    return '';
                }
                break;

            case 'posledniodpoved' :
                return '<div class="dotaznik_odpoved">' . $this ->getElementsHTMLOutput() . '</div>';

            case 'lang' :
                if (!$this ->isLanguage()) {
                    return '';
                }
                break;

            default :
                MessageManager::addErrorMessage('Chyba v XMLFormElementBlock - neznamy tag ' . $this ->elementName . '!');
                return '';
        }

        return $this ->getElementsHTMLOutput();
    }


    function writeXMLOutput($xmlWriter)
    {
        switch ($this ->elementName) {
            case 'posledniodpoved' : // prevede na standardni div
                $xmlWriter->startElement('div');
                $xmlWriter->writeAttribute('class', 'dotaznik_starsiodpoved');
                $this ->writeXMLElements($xmlWriter);
                $xmlWriter->endElement();
                break;


            case 'section' :
                if (!$this ->xmlForm -> prvniSekce) {
                    // pokud uz nejaka sekce byla, tak tuto sekci jen zkopiruje
                    $this ->xmlForm ->viceSekci = true;
                    $this ->copyXML($xmlWriter);
                } else {
                    $this ->xmlForm ->prvniSekce = false;
                    parent::writeXMLElements($xmlWriter);
                }
                break;

            case 'block' :
                if (self::getBoolValue($this ->getAttributeValue('ignore')) ) {
                    // block ma nastaveno ignore="1" - neprenasi se
                    break;
                }

                if ($this ->blockConditionFulfilled()) {
                    $xmlWriter->startElement('block');
                    $attrs = [];
                    if (!self::getBoolValue($this->getAttributeValue('copy', '1'))) {
                        // blok se musi provest, proto ho kopiruje, ale nastavi mu ignore na 1
                        $attrs['ignore'] = '1';
                    }
                    $this->writeXMLAttributes($xmlWriter, $attrs);
                    $this->writeXMLElements($xmlWriter);
                    $xmlWriter->endElement();
                } else {
                    $this ->copyXML($xmlWriter);
                }
                break;

            case 'show' :
                // jen zkopiruje element a obsah
                $this ->copyXML($xmlWriter);
                break;

            case 'fill' :
            default :
                parent::writeXMLOutput($xmlWriter);
        }
    }


    function event()
    {
        switch ($this ->elementName) {
            case 'posledniodpoved' :
            case 'section' :
                return;

            case 'block' :
                if (!$this ->blockConditionFulfilled()) {
                    return;
                }
                break;

            case 'lang' :
                if (!$this ->isLanguage()) {
                    return;
                }
                break;

            case 'fill' :  // TODO: ignorovat show a fill?
            case 'show' :
                break;

            default :
                throw new \Exception('Chyba v XMLFormElementBlock::event - neznamy tag');
        }

        parent::event();
    }


    function process()
    {
        switch ($this ->elementName) {
            case 'posledniodpoved' :
                return;

            case 'section' :
                if ($this ->xmlForm ->getVariableValue($this ->name)) {
                    // sekce uz byla vyplnena drive
                    if (!self::getBoolValue($this ->getAttributeValue('repeat', '0'))) {
                        // a nema se zobrazovat znovu  ignoruje ji
                        return;
                    }
                } else if (!$this ->xmlForm -> prvniSekce) {
                    // uz nejaka sekce byla => tuto sekci ignoruje
                    $this ->xmlForm ->viceSekci = true;
                    return;
                } else {
                    if ($this ->name) {
                        // nastavi jmeno sekce ve formulari, aby ji na konci mohl pridat do promennych
                        $this ->xmlForm ->sectionName = $this ->name;
                    }
                    $this ->xmlForm -> prvniSekce = false;
                }
                break;

            case 'block' :
                if (!$this ->blockConditionFulfilled()) {
                    return;
                }
                if ($this ->name) {
                    $this ->setVariableValue($this ->name, 1);
                    $this ->xmlForm ->addDataVariable($this ->name);
                }
                break;

            case 'lang' :
                if (!$this ->isLanguage()) {
                    return;
                }
                break;

            case 'show' :
                return;

            case 'fill' :
                break;

            default :
                throw new \Exception('Chyba v XMLFormElementBlock::process - neznamy tag');
        }

        parent::process();
    }

}

/*
//DOC
PAGE block
<pre>
PRE
  <block ignore="0" context="context_name" action="action_name"
          variable="variable_name" notvariable="variable_name" get="url_parameter_name"
          profile="kod_profilu"
          name="jmeno_bloku" copy="1">
      ... obsah bloku ...
  </block>
PREEND
</pre>

PAGE fill
<pre>
PRE
  <fill>
      ... obsah bloku ...
  </fill>
PREEND
</pre>

PAGE blocks/show &lt;show&gt;
<pre>
PRE
  <show>
      ... obsah bloku ...
  </show>
PREEND
</pre>

PAGE section
<pre>
PRE
  <section name="jmeno_sekce" repeat="0">
      ... obsah sekce ...
  </section>
PREEND
</pre>

PAGE lang
<pre>
PRE
  <lang include="kody_jazyku" exclude="kody_jazyku">
      ... obsah bloku ...
  </lang>
PREEND
</pre>

//END
*/