<?php
/**
 * stanislav.elbl, 24.05.2021 15:26
 */

namespace RonPortal\XML\Classes;

use Ron\Controls\BrowserField;
use RonPortal\XML\XMLBrowser;
use RonPortal\XML\XMLBrowserField;
use RonPortal\XML\XMLFormElement;

/**
 * Class XMLElementBrowser
 * Implementace XML tagu <browser>
 */
class XMLElementBrowser extends XMLFormElement
{
    function getHTMLOutput()
    {
        $browser = new XMLBrowser($this ->xmlForm ->database, $this ->xmlForm);
        $browser ->name = $this ->getAttributeValue('name');

        $className = $this ->getAttributeValue('class');
        if ($className) {
            $browser ->cssClass = $className;
        }

        $fixedCols = $this ->getAttributeValue('fixedcols');
        if ($fixedCols) {
            $browser->fixedCols = $fixedCols;
        }

        /** @var XMLElementBrowserPart $sqlElement */
        $sqlElement = $this ->findChildByName('browsersql');
        if (!$sqlElement) {
            return '';
        }
        $browser ->sql = $this ->nahradPromenneSQL($sqlElement ->sqlText);

        /** @var XMLElementBrowserPart[] $fieldElements */
        $fieldElements = $this ->findChildrenByName('browserfield');
        foreach ($fieldElements as $fieldElement) {
            $field = new XMLBrowserField($this ->xmlForm);
            $field ->type = BrowserField::FIELD_TYPE_DEFAULT;
            $field ->format = $fieldElement ->format;
            $field ->style = $fieldElement ->style;
            $field ->sqlFieldIndex = $fieldElement ->sqlIndex;
            $field ->code = $fieldElement ->code;

            $field ->header = $this ->nahradPromenne($fieldElement ->caption);

            // v linku a textu se promenne nahradi az pri pouziti (=na kazdem radku)!
            $field ->link = $fieldElement ->link;
            $field ->xmlText = $fieldElement ->text ?? $fieldElement ->getTextContent(false);

            $browser ->addField($field);
        }

        /** @var XMLElementBrowserPart $detailElement */
        $detailElement = $this ->findChildByName('browserdetail');
        if ($detailElement) {
            if ($detailElement ->col != '') {
                $browser ->infoLineCol = $detailElement ->col;
            }
            $browser ->infoLineText = $detailElement ->detailText;
        }

        /** @var XMLElementBrowserPart $linkElement */
        $linkElement = $this ->findChildByName('browserlink');
        if ($linkElement) {
            $browser ->rowClickLink = $linkElement ->link;
        }

        return $browser->getHTML();
    }
}

/*
//DOC
PAGE browser
<pre>
PRE
   <browser name="jmeno" fixedcols="pocet_sloupcu">
     <browsersql>SELECT ...</browsersql>
     <browserfield sqlindex="0" caption="Nadpis 1" format=""/>
     ...
     <browserdetail col="1">...</browserdetail>
   </browser>
PREEND
</pre>

//END
*/
