<?php
/**
 * stanislav.elbl, 01.06.2021 17:36
 */

namespace RonPortal\XML\Classes;
use RonPortal\Prava;
use RonPortal\RozdeleniUtils;
use RonPortal\XML\XMLFormElementCommand;

/**
 * Class XMLFormElementFiltrPodleRozdeleni
 * Do promenne se zadanym jmenem (name) vlozi sql filtr pro odfiltrovani zaznamu podle zadaneho rozdeleni
 */
class XMLElementFiltrPodleRozdeleni extends XMLFormElementCommand
{
    /** @var string */
    protected $name;
    /** @var string */
    protected $typPrava;
    /** @var string     Jmeno tabulky, ktera se ma v SQL vyrazu filtrovat */
    protected $tabulka;
    /** @var string     Vyraz pro nastaveni pocatku obdobi. */
    protected $obdobiOd;
    /** @var string     Vyraz pro nastaveni konce obdobi. */
    protected $obdobiDo;

    function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this ->name = $this ->getAttributeValue('name');
        $this ->tabulka = $this ->getAttributeValue('tabulka');
        $this ->obdobiOd = $this ->getAttributeValue('obdobiod');
        $this ->obdobiDo = $this ->getAttributeValue('obdobido');
    }


    function doCommand()
    {
        $rozdeleni = $this ->nahradPromenne($this ->getAttributeValue('rozdeleni'));

        if (!isset($rozdeleni) || $rozdeleni === '') {
            $filtr = '(1 = 1)';
        } else {
            $tabulka = ($this ->tabulka)
                ? $this ->nahradPromenne($this ->tabulka)
                : 'PracovniPomer';

            $typRozdeleni = (int)$this ->nahradPromenne($this ->getAttributeValue('typrozdeleni'));

            $vcetnePodrizenych = (int)$this ->nahradPromenne($this ->getAttributeValue('vcetnepodrizenych'));

            $obdobiOd = $this ->obdobiOd
                ? (int)$this -> nahradPromenne($this ->obdobiOd)
                : null;

            if (!$obdobiOd) {
                $obdobiOd = $this ->xmlForm ->pocatekObdobi ?: time();
            }

            $obdobiDo = $this ->obdobiDo
                ? (int)$this -> nahradPromenne($this ->obdobiDo)
                : null;

            if (!$obdobiDo) {
                $obdobiDo = $this ->xmlForm ->konecObdobi ?: time();
            }

            $filtr = RozdeleniUtils::sqlFilterRozdeleniPP($this->xmlForm->database,
                $tabulka, $typRozdeleni, $rozdeleni, $obdobiOd, $obdobiDo, $vcetnePodrizenych, null);
        }

        $this ->setVariableValue($this ->name, $filtr ?: '(0 = 1)');
    }
}

/*
//DOC
PAGE filtrpodlerozdeleni
<pre>
PRE
  <filtrpodlerozdeleni name="jmeno_promenne"
      typrozdeleni="kod_typu_rozdeleni" rozdeleni="kod_rozdeleni" vcetnepodrizenych="0"
      tabulka="jmeno_sql_tabulky"
      obdobiod="datum_od" obdobido="datum_do" />
PREEND
</pre>

//END
*/
