<?php
/**
 * stanislav.elbl, 17.06.2021 10:57
 */

namespace RonPortal\XML\Classes;
use RonPortal\XML\XMLFormElement;

/**
 * Implementace xml tagu <for>
 * Class XMLElementFor
 * @package RonPortal\XML\Classes
 */
class XMLElementFor extends XMLFormElement
{
    /** @var string     Jmeno ridici promenne */
    protected $name;
    /** @var int        Krok */
    protected $step;
    /** @var int        Pocatecni hodnota */
    protected $startValue;
    /** @var int        Koncova hodnota */
    protected $endValue;


    function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this->name = $this->getAttributeValue('name');
    }

    private function start()
    {
        $startStr = $this->getAttributeValue('start');
        $this->startValue = (isset($startStr) && $startStr != '')
            ? (int)$this->nahradPromenne($startStr)
            : 0;

        $endStr = $this->getAttributeValue('end');
        $this->endValue = (isset($endStr) && $endStr != '')
            ? (int)$this->nahradPromenne($endStr)
            : 0;

        $stepStr = $this->getAttributeValue('step');
        $this->step = (isset($stepStr) && $stepStr !== '')
            ? (int)$this->nahradPromenne($stepStr)
            : 1;

        $this->setVariableValue($this->name, $this->startValue);
    }


    /**
     * Vraci true, pokud uz je cyklus ukoncen
     * @return bool
     */
    private function finished()
    {
        $value = $this ->xmlForm ->getVariableValue($this ->name);

        if ($this ->step < 0) {
            // krok je mensi nez 0 - cyklus se ukonci, pokud je hodnota ridici promenne mensi nez koncova hodnota
            return $value < $this ->endValue;
        }

        // cyklus se ukonci, pokud je hodnota ridici promenne vetsi nez koncova hodnota
        return $value > $this ->endValue;
    }


    private function doStep()
    {
        $value = (int)$this ->xmlForm ->getVariableValue($this ->name);
        $this ->setVariableValue($this ->name, $value + $this ->step);
    }


    function getHTMLOutput()
    {
        $result = '';

        $this ->start();
        while (!$this ->finished()) {
            $result .= $this->getElementsHTMLOutput();
            $this ->doStep();
        }

        return $result;
    }


    function event()
    {
        $this ->start();

        while (!$this ->finished()) {
            $this->elementsEvent();
            $this ->doStep();
        }
    }


    function writeXMLOutput($xmlWriter)
    {
        throw new \Exception('TAG for neni pro mod transform implementovan!');
    }


    function process()
    {
        $this ->start();

        while (!$this ->finished()) {
            $this ->processElements();
            $this ->doStep();
        }
    }
}


/*
//DOC
PAGE for
<pre>
PRE
  <for name="jmeno_promenne" start="0" end="N" increment="1">
    ... obsah ...
  </for>
PREEND
</pre>

//END
*/
