<?php
/**
 * stanislav.elbl
 * 22.05.2021 Time: 11:09
 */

namespace RonPortal\XML\Classes;

use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\XML\XMLFormElement;


class XMLElementIf extends XMLFormElement
{
    var $testType;
    var $param1;
    var $param2;
    var $testText;

    protected $params;

    function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this->testType = $this->getAttributeValue('testtype');

// TODO: dodelat podporu AND a pripadne dalsich testu ... if($param1) atd.
        $this ->params = [];
        for ($i = 1;  ($paramStr = $this->getAttributeValue("param$i")) !== null;  $i++) {
            $this ->params[$i] = StringUtils::fromUTF8($paramStr);
        }

        $this ->param1 = $this ->params[1] ?? null;
        $this ->param2 = $this ->params[2] ?? null;

        $this->testText = StringUtils::fromUTF8($this->getAttributeValue('sql'));
    }


    protected function evaluateExpression()
    {
        $condition = $this ->nahradPromenne( StringUtils::fromUTF8($this ->getAttributeValue('condition')) );

        MessageManager::addDebugLog('XMLDebug> ' . $this ->elementName . " - vyhodnocuji vyraz: $condition");

        $result = null;
        global $xmlform; // kvuli vyhodnocovani vyrazu!

        eval('$result = '.$condition.';');

        return $result;
    }


    protected function evaluateCondition()
    {
        switch ($this->testType) {
            case 'emptysql' :
            case 'notemptysql' :

                $sql = $this->nahradPromenneSQL( $this ->testText );
                if (!$sql) {
                    return false;
                }

                $cursor = $this->xmlForm->database->getSqlResult($sql);

                MessageManager::addDebugLog('Tag test: ' . $sql);

                if ($row = $cursor->fetch()) {
                    // vysledek sql vyrazu neni prazdny
                    return $this->testType == 'notemptysql';
                }

                // vysledek je prazdny
                return $this->testType == 'emptysql';

            case 'error' :
                return $this->xmlForm->isError();

            case '=' :
            case '==' :
            case 'eq' :
                return $this->nahradPromenne($this->param1) == $this->nahradPromenne($this->param2);

            case 'eqci' :
                return !strcasecmp($this->nahradPromenne($this->param1), $this->nahradPromenne($this->param2));

            case 'neqci' :
            case 'noteqci' :
                return !!strcasecmp($this->nahradPromenne($this->param1), $this->nahradPromenne($this->param2));

            case '>' :
            case 'gt' :
                return $this->nahradPromenne($this->param1) > $this->nahradPromenne($this->param2);

            case '>=' :
            case 'gteq' :
                return $this->nahradPromenne($this->param1) >= $this->nahradPromenne($this->param2);

            case '<' :
            case 'lt' :
                return $this->nahradPromenne($this->param1) < $this->nahradPromenne($this->param2);

            case '<=' :
            case 'lteq' :
                return $this->nahradPromenne($this->param1) <= $this->nahradPromenne($this->param2);

            case '<>' :
            case '!=' :
            case 'neq' :
            case 'noteq' :
                return $this->nahradPromenne($this->param1) != $this->nahradPromenne($this->param2);

            case 'expression' :
                return $this ->evaluateExpression();

            case 'not' :
            case '!' :
                return !$this ->nahradPromenne($this ->param1);

            case null :
            case '' :
                // pokud je zadan param1, tak otestuje
                if ($this ->param1 !== null) {
                    return (bool)$this ->nahradPromenne($this ->param1);
                }
                return $this ->evaluateExpression();

            case 'and' :
                foreach ($this ->params as $param) {
                    if (!$this ->nahradPromenne($param)) {
                        return false;
                    }
                }
                return true;

            case 'or' :
                foreach ($this ->params as $param) {
                    if ($this ->nahradPromenne($param)) {
                        return true;
                    }
                }
                return false;

            // TODO: 'in' - umoznit zjistit, jestli je param1 obsazen v retezci kodu oddelenych strednikem

            default :
                throw new \Exception("Chyba v tagu if/test - tento typ testu neni implementovan! ({$this -> testType})");
        }
    }


    function getHTMLOutput()
    {
        if ($this ->evaluateCondition()) {
            return $this ->getElementsHTMLOutput();
        } else {
            return '';
        }
    }


    function event()
    {
        if ($this ->evaluateCondition()) {
            parent::event();
        }
    }


    function writeXMLOutput($xmlWriter)
    {
        $copy = self::getBoolValue($this ->getAttributeValue('copy'));

        if ($this ->evaluateCondition()) {
            if ($copy) {
                parent::writeXMLOutput($xmlWriter);
            }
            else {
                $this->writeXMLElements($xmlWriter);
            }
        } else {
            // podminka neni splnena
            if ($copy) {
                // zkopiruje cely if a jeho obsah, ale neprovadi jej
                $this ->copyXML($xmlWriter);
            }
        }
    }


    function process()
    {
        // v datovych XML neni parametr copy podporovan -> vyhodit vyjimku, pokud je nastaven?
        // TODO: doresit parametr copy. POZOR! U if je defaultni hodnota copy="0", ale pro datove formulare je to 1
        if ($this->evaluateCondition()) {
            parent::processElements();
        }
    }
}

/*
//DOC
PAGE if
<pre>
PRE
  <if testtype="typ_testu" condition="php_vyraz" param1="parametr1" param2="parametr2" sql="SELECT_sql_vyraz">
      ... obsah ...
  </if>
PREEND
</pre>
//END
*/
