<?php
/**
 * stanislav.elbl
 * 22.05.2021 14:07
 */

namespace RonPortal\XML\Classes;

use Ron\DateUtils;
use Ron\StringUtils;
use RonPortal\XML\XMLFormElementCommand;

/**
 * Implementace tagu, ktere nestavuji podrobnosti formulare (osobu, stav, rozdeleni, rozliseni atd.)
 * Class XMLElementPodrobnosti
 * @package RonPortal\XML\Classes
 */
class XMLElementPodrobnosti extends XMLFormElementCommand
{
    var $fieldName;
    var $value;


    function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        if ($this ->elementName == 'nastavpodrobnostiformulare') {
            $this->fieldName = $this->getAttributeValue('fieldname');
            $this->value = $this->getTextContent(false);
        }
    }


    function doCommand()
    {
        switch ($this ->elementName) {
            case 'stav' :
                $this ->xmlForm ->stav = (int)$this ->getAttributeValue('novystav');
                break;

            case 'uzavrit' :
                $this ->xmlForm ->stav = 128;
                break;

            case 'nastavosobuformulare' :
                $rc = $this ->nahradPromenne( StringUtils::fromUTF8($this ->getAttributeValue('rc')) );
                $this ->xmlForm ->nastavOsobu($this ->getAttributeValue('name'), $rc);
                break;

            case 'nastavpracovnipomer' :
                $idpp = $this ->nahradPromenne( StringUtils::fromUTF8($this ->getAttributeValue('idpp')) );
                $this ->xmlForm ->nastavPracovniPomer( $this ->getAttributeValue('name'), $idpp );
                break;

            case 'nastavobdobiformulare' :
                $obdobi = $this ->nahradPromenne($this ->getAttributeValue('obdobi'));
                if (StringUtils::isInteger($obdobi)) {
                    $obdobi = (int)$obdobi;
                } else {
                    $obdobi = DateUtils::dateStringToTimestamp($obdobi);
                }
                $this ->xmlForm ->nastavObdobiAplikace($this ->getAttributeValue('name'),
                    $this ->getAttributeValue('aplikace'), $obdobi,
                    self::getBoolValue($this ->getAttributeValue('variabilni')));
                break;

            case 'nastavrozdeleniformulare' :
                $typ = $this ->nahradPromenne($this ->getAttributeValue('typ'));
                $kod = $this ->nahradPromenne($this ->getAttributeValue('kod'));
                $this ->xmlForm ->nastavRozdeleni($this ->getAttributeValue('name'), $typ, $kod);
                break;

            case 'nastavpodrobnostiformulare' :
                $value = $this->nahradPromenne($this->value);

                switch (strtolower($this->fieldName)) {
                    case 'context' :
                        $this->xmlForm->context = $value;
                        break;

                    case 'rozliseni1' :
                        $this->xmlForm->rozliseni1 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni2' :
                        $this->xmlForm->rozliseni2 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni3' :
                        $this->xmlForm->rozliseni3 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni4' :
                        $this->xmlForm->rozliseni4 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni5' :
                        $this->xmlForm->rozliseni5 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni6' :
                        $this->xmlForm->rozliseni6 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni7' :
                        $this->xmlForm->rozliseni7 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni8' :
                        $this->xmlForm->rozliseni8 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni9' :
                        $this->xmlForm->rozliseni9 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni10' :
                        $this->xmlForm->rozliseni10 = substr(trim($value), 0, 40);
                        break;

                    case 'dodatecnypopis' :
                        $this->xmlForm->dodatecnyPopis = substr(trim($value), 0, 120);
                        break;

                    case 'poznamka' :
                        $this->xmlForm->poznamka = $value;
                        break;

                    case 'resultcode' :
                        $this->xmlForm->xmlResultCode = $value;
                        break;

                    case 'resultmessage' :
                        $this->xmlForm->xmlResultMessage = $value;
                        break;
                }

                break;

            default :
                throw new \Exception('Chyba v metode XMLElementPodrobnosti::doCommand - neznamy tag!');
        }
    }
}


/*
//DOC
PAGE nastavosobuformulare
<pre>
PRE
  <nastavosobuformulare rc="rc_osoby" name="jmeno_promenne" />
PREEND
</pre>

PAGE nastavpracovnipomer
<pre>
PRE
  <nastavpracovnipomer idpp="id_pracovniho_pomeru" name="jmeno_promenne" />
PREEND
</pre>

PAGE nastavrozdeleniformulare
<pre>
PRE
  <nastavrozdeleniformulare typ="kod_typu_rozdeleni" kod="kod_rozdeleni" name="jmeno_promenne"/>
PREEND
</pre>

PAGE nastavobdobiformulare
<pre>
PRE
  <nastavobdobiformulare obdobi="pocatek_obdobi" aplikace="kod_aplikace" variabilni="0" name="jmeno_promenne" />
PREEND
</pre>

PAGE nastavpodrobnostiformulare
<pre>
PRE
  <nastavpodrobnostiformulare fieldname="jmeno_pole">hodnota</nastavpodrobnostiformulare>
PREEND
</pre>

//END
*/
