<?php
/**
 * stanislav.elbl, 24.05.2021 19:13
 */

namespace RonPortal\XML\Classes;

use Ron\StringUtils;
use RonPortal\XML\XMLFormElement;
use RonPortal\XML\XMLFormElementInput;

class XMLElementOption extends XMLFormElement
{
    // pouze pro pouziti v select
    var $value;
    var $text;
    var $selected;

    public function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        $this->value = StringUtils::fromUTF8($this->getAttributeValue('value'));
        $this->selected = self::getBoolValue($this ->getAttributeValue('default'));

        $this->text = $this->getTextContent(false);
    }

    function getHTMLOutput()
    {
        $attrs = ['value' => $this->value];
        if ($this->selected) {
            $attrs['selected'] = 'selected';
        }
        $attrs = $this->setHTMLAttributes($attrs);
        $attrs_str = $this->getHTMLAttributes($attrs);
        return "<option $attrs_str>{$this->xmlForm->nahradPromenne($this ->text)}</option>";
    }
}


class XMLElementSelect extends XMLFormElementInput
{
    var $value;

    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader); // nenacita podrizene polozky!

        $value = $this->getAttributeValue('value');
        $this->value = isset($value)
            ? StringUtils::fromUTF8($this->getAttributeValue('value'))
            : null;
    }


    function getHTMLOutput()
    {
        $name = $this->nahradPromenne($this->name);

        $attrs = ['name' => $name];

        if ($this->xmlForm->usePostValues && isset($_POST[$name])) {
            $value = $_POST[$name];
        } else if ($this->xmlForm->dataMode() && $this->variableValue($name) !== null) {
            // datove xml - hodnota uz je ulozena v datech
            $value = $this->variableValue($name);
        } else {
            $value = isset($this ->value)
                ? $this->nahradPromenne($this->value)
                : null;
        }

        if ($this->xmlForm->isError($name)) {
            $attrs['class'] = 'dotaznik_errinput';
        }

        if ($this ->xmlForm ->readOnly) {
            $attrs['disabled'] = '1';
        }

        $attrs = $this->setHTMLAttributes($attrs);
        $attrs_str = $this->getHTMLAttributes($attrs);

        $result = "<select $attrs_str>";

        foreach ($this->elements as $option) {
            if ($option instanceof XMLElementOption) {
                if (isset($value)) {
                    $option->selected = $value == $option->value;
                }

                if ($option->selected) {
                    $this->setVariableValue($name, $option->value);
                    $this->setVariableValue($name . '_text', $option->text);
                }

                $result .= $option->getHTMLOutput();
            }
        }

        $result .= "</select>\n";

        return $result;
    }


    function writeXMLOutput($xmlWriter)
    {
        $name = $this->nahradPromenne($this->name);

        $selectedValue = null;
        $selectedText = null;

        foreach ($this->elements as $option) {
            if ($option instanceof XMLElementOption) {
                if ($_POST[$name] == $option->value) {
                    $selectedValue = $option ->value;
                    $selectedText = $option ->text;
                }
            }
        }

        if (!isset($selectedValue)  &&  $this->required) {
            $this->xmlForm->addErrorMsg(\Ron\ronText('Chybi_povinna_hodnota'), $name);
        }

        $this->setVariableValue($name, $selectedValue);
        $this->setVariableValue($name . '_text', $selectedText);

        if (self::getBoolValue($this ->nahradPromenne($this ->getAttributeValue('editing')))) {
            $xmlWriter->startElement('select');
            $this->writeXMLAttributes($xmlWriter, ['value' => $selectedValue]);
            $this->writeXMLElements($xmlWriter);
            $xmlWriter->endElement();
        } else {
            $this ->writeXmlVariable($xmlWriter, $name, $selectedValue);
            $this ->writeXmlVariable($xmlWriter, $name . '_text', $selectedText);
            $xmlWriter ->startElement('span');
            $xmlWriter ->writeAttribute('class', 'dotaznik_odpoved');
            $xmlWriter ->text( StringUtils::toUTF8($selectedText) );
            $xmlWriter ->endElement();
        }
    }


    function process()
    {
        $name = $this->nahradPromenne($this->name);
        $oldValue = $this ->variableValue($name);

        if ($this ->required  &&  $_POST[$name] == '') {
            $this ->xmlForm ->addErrorMsg(\Ron\ronText('Chybi_povinna_hodnota'), $name);
        }

        $selectedValue = null;
        $selectedText = null;

        foreach ($this->elements as $option) {
            if ($option instanceof XMLElementOption) {
                if ($_POST[$name] == $option->value) {
                    $selectedValue = $option ->value;
                    $selectedText = $option ->text;
                }
            }
        }

        $this->setVariableValue($name, $selectedValue);
        $this->setVariableValue($name . '_text', $selectedText);
        if ($selectedValue != $oldValue) {
            $this ->xmlForm ->addLogData($name, $selectedValue . '(' . $selectedText . ')');
        }

        if ($this ->dataVariable) {
            $this->xmlForm->addDataVariable($name);
            $this->xmlForm->addDataVariable($name . '_text');
        }
    }
}

/*
//DOC
PAGE select
<pre>
PRE
  <select name="jmeno" required="0" editing="yes" datavariable="1" value="">
    <option value="..." default="0">Text</option>
    ...
  <select>
PREEND
</pre>

//END
*/
