<?php
/**
 * stanislav.elbl
 * 21.05.2021 23:40
 */

namespace RonPortal\XML\Classes;

use Ron\DB\SQLUtils;
use Ron\MessageManager;
use RonPortal\Log;
use RonPortal\XML\XMLFormElement;

class XMLElementSQL extends XMLFormElement
{
    // tag sql a sqlvariable
    var $sql;
    var $sqlTextVariable;

    // TODO: doplnit podporu parametru, napr. takto:
    // <sql>
    //   <sqlcommand>...</sqlcommand>    prikaz s parametry ve formatu {$?} nebo nejak
    //   <sqlparam format="..." value="..."/>
    //   <sqlparam format="...">...</sqlparam>
    // </sql>
    // nebo parametry primo v sql <sql>SELECT ... <sqlparam .../> ... </sql>

    // TODO: umoznit nacist vsechny radky z sql dotazu

    /* @var string jmeno promenne */
    var $name;
    var $copy;

    var $format;

    var $log; // level logovani - 2 = vcetne vysledku


    function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        $this ->sqlTextVariable = $this ->getAttributeValue('sqltextvariable');
        $this ->name = $this ->getAttributeValue('name');
        $this ->format = $this ->getAttributeValue('format');
        $this ->log = (int)$this ->getAttributeValue('log'); // level 0, 1, 2(=vcetne hodnot)

        $this ->copy = self::getBoolValue($this->getAttributeValue('copy'), true);
        $this ->sql = $this ->getTextContent(false);
    }


    function executeSQL()
    {
        $sql = $this ->sqlTextVariable
            ? $this ->variableValue($this ->sqlTextVariable)
            : $this ->sql;

        $sql = $this ->nahradPromenneSQL($sql);

        MessageManager::addDebugLog($this ->elementName .
            ($this ->name ? ' ' . $this ->name : '') . ': ' . $sql);

        $variableName = $this -> nahradPromenne( $this -> name );

        if (!$sql) {
            return null;
        }

        $cursor = $this ->xmlForm ->database ->getSqlResult($sql);

        $result = '';
        $logData = '';

        if ($row = $cursor ->fetch()) {
            $result = self::formatSqlValue($this -> format, $row[0]);

            if ($this -> log) {
                $logData .= $result . "\n\n";
            }

            if ($variableName) {
                // nastavi hodnotu promennych
                if (sizeof($row) == 1) {
                    // jen jeden prvek => vubec negeneruje pole
                    $this -> setVariableValue($variableName, $result);
                    if ($this -> log == 2) {
                        $logData .= $variableName . ' = ' . $result . "\n";
                    }
                } else {
                    // vice prvku
                    if ($this ->xmlForm ->verze > 1) {
                        // verze 2 a vice => pokud ma vysledek vice sloupcu, pouzije pole
                        $pole = [];

                        foreach ($row as $i => $val) {
                            $pole[$i] = $i ? $val : $result; // nulta polozka pole je naformatovana
                            if ($this ->log == 2) {
                                $logData .= $variableName . "[$i] = " . $val . "\n";
                            }
                        }

                        $this -> setVariableValue($variableName, $pole);
                    } else {
                        // starsi verze => pouzije primo promenne s indexem v nazvu
                        $this -> setVariableValue($variableName, $result);
                        if ($this -> log == 2) {
                            $logData .= $variableName . ' = ' . $result . "\n";
                        }

                        foreach ($row as $i => $val) {
                            $this -> setVariableValue($variableName . "[$i]", $val);
                            if ($this ->log == 2) {
                                $logData .= $variableName . "[$i] = " . $val . "\n";
                            }
                        }
                    }
                }
            }
        } else {
            if ($variableName) {
                //$this -> setVariableValue($variableName, null);
                $this -> setVariableValue($variableName, null);

                if ($this -> log == 2) {
                    $logData .= $variableName . " = null\n";
                }
            }
        }

        $cursor ->close();

        if ($this -> log) {
            Log::addRecordToDBLog($this ->xmlForm ->database,
                SQLUtils::getTableNameFromSQL($sql), SQLUtils::getEventTypeFromSQL($sql),
                'XML', $sql . "\n" . $logData);
        }

        return $result;
    }


    function event()
    {
        $this -> executeSQL();
    }


    function getHTMLOutput()
    {
        if ($this ->xmlForm ->dataMode()  &&  $this ->elementName == 'sqlvariable'  &&  !$this ->copy) {
            // datove xml, copy=0 => pokud uz existuje promenna prislusneho jmena, tak sql prikaz nespousti
            $result = $this ->variableValue($this ->name);
            if (!isset($result)) {
                $result = $this ->executeSQL();
            }
        } else {
            $result = $this->executeSQL();
        }

        if ($this ->elementName == 'sql') {
            return $result;
        }

        return ''; // tag sqlvariable
    }


    function writeXMLOutput($xmlWriter)
    {
        $result = $this -> executeSQL();

        if ($this ->elementName != 'sqlvariable'  ||  $this ->copy) {
            $xmlWriter->startElement(
                ($this->prefix ? $this->prefix . ':' : '') .
                $this->elementName
            );

            if ($this->attributes) {
                $this->writeXMLAttributes($xmlWriter);
            }

            if ($this->sql) {
                $xmlWriter ->text($this ->sql);
            }

            $xmlWriter->endElement();
        } else {
            // tag sqlvariable s copy=0 => zapise jen vyslednou promennou
            $xmlWriter ->startElement('variable');
            $xmlWriter ->writeAttribute('name', $this ->name);
            $xmlWriter ->writeAttribute('value', $result);
            $xmlWriter ->endElement();
        }
    }


    function process()
    {
        $this -> executeSQL();

        if ($this ->elementName == 'sqlvariable'  &&  !$this ->copy) {
            // tag sqlvariable s copy=0 => vygeneruje prislusne promenne
            $this ->xmlForm ->addDataVariable($this ->name);
        }
    }
}

/*
//DOC

PAGE sqlvariable
<pre>
PRE
  <sqlvariable name="jmeno_promenne" format="format_promenne"  copy="1"  log="0" sqltextvariable="promenna s sql vyrazem">SQL vyraz</sqlvariable>
PREEND
</pre>

PAGE sql
<pre>
PRE
  <sql format="format_promenne"  log="0" sqltextvariable="promenna s sql vyrazem">SQL vyraz</sqlvariable>
PREEND
</pre>

//END
*/
