<?php
/**
 * stanislav.elbl, 26.07.2021 11:16
 */

namespace RonPortal\XML;

use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\Http\App;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\FilterFormFilterItem;
use RonPortal\Controls\FilterFormFilterObdobi;
use RonPortal\Controls\FilterFormFilterPP;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;
use RonPortal\PredavaciProcesyUtils;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use Ron\StringUtils;
use RonPortal\User;


class FilterFormFilterPopisXml extends FilterFormFilter
{
    private $maxDays = 365; // maximalni pocet dnu zpetne, ve kterych hleda popis filtru

    public function __construct(string $name)
    {
        parent::__construct($name, FilterFormFilter::TYPE_OPTIONS);
    }


    public function loadOptions(Database $database): FilterFormFilter
    {
        $datumOd = $this ->filterForm ->getFilterValue('datumod');
        $datumDo = $this ->filterForm ->getFilterValue('datumod', 'datumDo');

        if (!$datumOd  &&  $this ->maxDays) {
            $datumOd = DateUtils::incDayStart(DateUtils::today(), -$this ->maxDays);
        }

        $params = $database ->createParameters();
        $sql =
            'SELECT DISTINCT VyplnenyXMLFormular.Popis, VyplnenyXMLFormular.Popis'.
            ' FROM VyplnenyXMLFormular LEFT OUTER JOIN SablonaXMLFormulare' .
            ' ON SablonaXMLFormulare.ID = VyplnenyXMLFormular.IdSablony' .
            ' WHERE 1 = 1' .
            ($this ->filterForm ->getFilterValue('stavxml') == -1
                ? ' AND Stav < 128 '
                : ''
            ) .
            // filtr pro vsechny neukoncene bez ohledu na obdobi
            ($datumOd
                ? ' AND VyplnenyXMLFormular.DatumVytvoreni >= ' . $params ->addDateParameter($datumOd)
                : ''
            ) .
            ($datumDo
                ? ' AND VyplnenyXMLFormular.DatumVytvoreni <= ' . $params ->addDateParameter($datumDo)
                : ''
            );

        if ($this->filterForm ->getXmlMenuId()) {
            $sql .=
                ' AND SablonaXMLFormulare.MenuId = ' . $params->addParameter($this->filterForm ->getXmlMenuId());
        }

        $this ->optionsSql = new SqlStatement($sql, $params);

        return parent::loadOptions($database);
    }
}


class XMLFilterForm extends RonPortalFilterForm
{
//    /** @var ComboBox */
//    protected $comboPopis;

    /** @var string */
    private $xmlMenuId;

    /** @var FilterFormFilter */
    protected $filterStav;

    /**
     * @return string
     */
    public function getXmlMenuId(): string
    {
        return $this->xmlMenuId;
    }

    /**
     * @param Database $database
     * @param string $xmlMenuId
     */
    public function __construct($database, $xmlMenuId)
    {
        parent::__construct($database, 'seznamxmlformularu', true);

        $this ->xmlMenuId = $xmlMenuId;

        $typPravaOsoby = RonPortalSettings::getValue('vyplnenexml_osoby_pravo','prohlizeni');
        if ($typPravaOsoby) {
            $this ->addFilter(
                (new FilterFormFilterPP($this))
                    ->setPovolitVsechnyOsoby(true)
                    ->setPrava(Prava::vratPrava($typPravaOsoby))
                    ->setDatumOd(DateUtils::today())
                    ->setDatumDo(DateUtils::today())
                    ->setSessionName('xmlformulare_vybranaosoba')
                    ->setDefaultValue('')
            );
//            $prava = Prava::vratPrava($typPravaOsoby);
//            if ($prava->jsouNejakePrava()) {
//                $this ->nastavOsoby(true, $prava, DateUtils::today(), DateUtils::today());
//                $this ->povolitVsechyOsoby = true;
//            }
        }

        //$this->nastavAkciGet('seznamxmlformularu');
        //$this->f_classname = 'filtercontainer';

        $this ->filterStav = (new FilterFormFilter('stavxml'))
            -> addItem( (new FilterFormFilterItem('-1', TextManager::_('Neukonceno'))) ->setSqlFilter('(VyplnenyXMLFormular.Stav < 128)'))
            -> addItem( (new FilterFormFilterItem('-2', TextManager::_('Ukonceno'))) ->setSqlFilter('(VyplnenyXMLFormular.Stav >= 128)'))
            -> addItem( '-3', TextManager::_('Vse'))
            ->setDefaultValue('-1');
        $this ->addFilter($this->filterStav, true);

        // datum od-do

//        $this ->filterStav ->load($this ->database);
//        $stav = $this ->getFilterValue('stavxml');
//        //pri nastaveni stavu neukonceno, nebo vse, pro pripad, ze neni nastaveno obdobi, nastavi filtr na datum vytvoreni
//        if ((isset($_GET['stavxml']) && !RonPortalSession::jenObdobi()) && (($stav == -2 || $stav == -3))) {
//            RonPortalSession::setFiltrovatDatum(true);
//        }

//        $filtrPovinny = ($stav == -2 || $stav == -3); // -2 = ukonceno, -3 = vse
        //30.9.2019 zrusena povinnost filtru na datum, nastaveno strankovani browseru
//        $filtrPovinny = false;
//        $this->nastavDatumOdDo(true, $filtrPovinny, TextManager::_('Datum_vytvoreni_xml') . ' ' . TextManager::_('Od'));

        //obdobi
        if (RonPortalUtils::jeNainstalovanaAplikace($this->database,'ADS')) {
            $this ->addFilter(new FilterFormFilterObdobi($this), true);
            //$this->nastavObdobi(true, 'ADS', false, false);
        }

        $this ->addFilter(
            (new FilterFormFilterDatumOdDo($this))
                ->setPovolitPrazdneDatumDo(true)
                ->setCaption(TextManager::_('Datum_vytvoreni_xml') . ' ' . TextManager::_('Od'))
                ->setSessionName('filter_datum_vyplneni_xml'),
            true
        );

        // rozdeleni
        ($this ->addFilterRozdeleni('rozdeleni', RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI)))
            ->setSessionName('ads_rozdeleni_' . RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI))
            ->setPouzitPopisTypuRozdeleni(true)
            ->setJenVlastniRozdeleni(RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_JEN_VLASTNI_ROZDELENI));
//        $this->nastavRozdeleni(true,
//            RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI),
//            RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_JEN_VLASTNI_ROZDELENI),
//            '', true);

        if ($this ->xmlMenuId) {
            $this->addFilter(
                (new FilterFormFilter('xmlmenuid', FilterFormFilter::TYPE_HIDDEN))
                    ->setDefaultValue($this ->xmlMenuId)
            );
        }

        $this ->addFilter(
            (new FilterFormFilterPopisXml('popisxml', $this))
                ->setCaption( TextManager::_('Jen_typ_xml') )
                ->setHtmlAttributes(['id' => 'jenpopisxml'])
                ->addItem( new FilterFormFilterItem('', TextManager::_('_vse_')))
                ->setSqlFilter('VyplnenyXMLFormular.Popis = ')
        );

        // pouze vlastni/vse
        if (Prava::vratHodnotuPrava('xml') == Prava::PRAVA_VSE) {
            $this ->addFilter( (new FilterFormFilter('xmlzobrazitvse'))
                ->addItem(0, TextManager::_('Zobrazit_pouze_vlastni_xml'))
                ->addItem(1, TextManager::_('Zobrazit_vsechny_xml'))
                ->setDefaultValue(0)
                ->setShowSelectedItem(false)
                ->setSessionName('xmlzobrazitvse')
            );
        }
    }
}




class EditaceXMLField extends BrowserField
{
    function __construct()
    {
        parent::__construct();
        $this->type = 'text';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return
            '<a href="?menuid=editovatdotaznik&record=' . $recordId . '">' .
            TextManager::_(($rowData[2] >= 128) ? 'Zobrazit_xml' : 'Editovat_xml') .
            '</a>';
    }
}



class StavXMLField extends BrowserField
{
    function loadConfigurationItem($key, $value)
    {
        switch ($key) {
            case 'zobrazitPopisUrovne' :
                $this ->sqlFieldIndex2 = (int)$value
                    ? 'col-urovenpredavani-popis'
                    : 'col-poradipredavani-kodurovne';
                break;

            default :
                parent::loadConfigurationItem($key, $value);
        }
    }
}




class VyplnenyXMLFormularBrowser extends RonPortalBrowser
{
    public $xmlMenuId;

    function __construct($database, $xmlMenuId)
    {
        parent::__construct($database, 'seznamxmlformularu');

        $this->xmlMenuId = StringUtils::secureString($xmlMenuId);
        $this->browserUrl = '?menuid=seznamxmlformularu&' . ($this->xmlMenuId ? 'xmlmenuid=' . $this->xmlMenuId . '&' : '');

        $this->title = TextManager::_('Vyplnene_xml_formulare');

        $this->code = 'browservyplnenexml'. ($this->xmlMenuId ? '_' . $this->xmlMenuId : '');

        $this->cssClass .= ' browservyplnenexml' . ($this->xmlMenuId ? '_' . $this->xmlMenuId : '') ;

        $this->sqlFromPart =
            '( ( ( ( ( VyplnenyXMLFormular LEFT OUTER JOIN Osoba ' .
            '          ON (VyplnenyXMLFormular.RC = Osoba.RC  OR   VyplnenyXMLFormular.PP_RC = Osoba.RC)' .
            '      ) LEFT OUTER JOIN PredavaciProces ' .
            '        ON PredavaciProces.Kod = VyplnenyXMLFormular.KodProcesu ' .
            '    ) LEFT OUTER JOIN PoradiPredavani ' .
            '      ON PoradiPredavani.KodProcesu = VyplnenyXMLFormular.KodProcesu ' .
            '         AND PoradiPredavani.PocatecniUroven = VyplnenyXMLFormular.PocatecniUroven ' .
            '         AND PoradiPredavani.Poradi = VyplnenyXMLFormular.PoradiPredavani ' .
            '   ) LEFT OUTER JOIN UrovenPredavani ' .
            '       ON UrovenPredavani.Kod = PoradiPredavani.KodUrovne ' .
            '  ) LEFT OUTER JOIN Rozdeleni ' .
            '    ON Rozdeleni.KodTypuRozdeleni = VyplnenyXMLFormular.KodTypuRozdeleni ' .
            '       AND Rozdeleni.Kod = VyplnenyXMLFormular.Rozdeleni ' .
            ') INNER JOIN S_StavXMLFormulare ' .
            '  ON S_StavXMLFormulare.Kod = VyplnenyXMLFormular.Stav ';

        // pokud je zadano $xmlmenuid, tak nacte jen formulare prirazene k sablonam se zadanym menuid
        if ($this->xmlMenuId)
            $this->sqlFromPart =
                '( ' . $this->sqlFromPart . ' ) ' .
                '  INNER JOIN SablonaXMLFormulare ON SablonaXMLFormulare.Id = VyplnenyXMLFormular.IdSablony' .
                "   AND SablonaXMLFormulare.MenuId = '" . $database ->addSlashes($this->xmlMenuId) . "'";

        //$this -> AddFieldDef( array('normal', 'RC', 'RC osoby ...', '', 'left', '', 0, 1) );
        $this ->sqlSelectPart =
            'VyplnenyXMLFormular.Id, VyplnenyXMLFormular.Popis, VyplnenyXMLFormular.Stav, ' . // 0..2
            'VyplnenyXMLFormular.DatumVytvoreni, VyplnenyXMLFormular.PosledniZmena, ' . // 3..4
            'Osoba.Prijmeni, Osoba.Jmeno'; // 5..6

        $this->primaryField = 0;
        $this->maxPageRows = 100;

        $this->addField()
            ->setSqlFieldName('VyplnenyXMLFormular.Id', 0) ->setHeader('ID')
            ->setAlign('right') ->setOrdering(true)
            ->setCode('col-id')
            ->setVisible(App::getDebugMode());

        $this->addField()
            ->setSqlFieldName('VyplnenyXMLFormular.Popis', 1) ->setHeaderText('Typ_formulare')
            ->setOrdering(true)
            ->setCode('col-popis')
            ->setSortSQL('2'); // cislovano od 1!

        $this->addField()
            ->setSqlFieldName('Osoba.Prijmeni', 5) ->setHeaderText('Prijmeni')
            ->setOrdering(true)
            ->setCode('col-prijmeni')
            ->setSortSQL('6'); // kvuli firebirdu - cislovano od 1!

        $this->addField()
            ->setSqlFieldName('Osoba.Jmeno', 6) ->setHeaderText('Jmeno')
            ->setOrdering(true)
            ->setCode('col-jmeno')
            ->setSortSQL('7'); // kvuli firebirdu - cislovano od 1!

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Rozdeleni.Popis')
            ->setCode('col-rozdeleni-popis');
        $this ->addField()
            ->setSqlFieldName('Rozdeleni.Kod') ->setHeaderText('Rozdeleni')
            ->setCode('col-rozdeleni')
            ->setOrdering(true)
            ->setSqlFieldIndex2('col-rozdeleni-popis');

        $this->addField()
            ->setSqlFieldName('VyplnenyXMLFormular.Obdobi')->setHeaderText('Obdobi')->setOrdering(true)
            ->setFormat('date')
            ->setCode('col-obdobi');
        $this->addField()
            ->setSqlFieldName('VyplnenyXMLFormular.DatumVytvoreni', 3) ->setHeaderText('Cas_vytvoreni_xml')
            ->setOrdering(true)
            ->setFormat('datetime')
            ->setCode('col-datumvytvoreni')
            ->setSortSQL('4'); // kvuli firebirdu - cislovano od 1!

        $this->addField()
            ->setSqlFieldName('VyplnenyXMLFormular.PosledniZmena', 4) ->setHeaderText('Posledni_zmena')
            ->setOrdering(true)
            ->setFormat('datetime')->setCode('col-poslednizmena')
            ->setSortSQL('5'); // kvuli firebirdu - cislovano od 1!

        $this ->defaultSort = 'col-poslednizmena'; // count($this ->fields);

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('UrovenPredavani.Popis')
            ->setCode('col-urovenpredavani-popis');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PoradiPredavani.KodUrovne')
            ->setCode('col-poradipredavani-kodurovne');

        $this->addField( new StavXMLField() )
            ->setSqlFieldName('S_StavXMLFormulare.Popis') ->setHeaderText('Stav_formulare')
            ->setOrdering(true) ->setSqlFieldIndex2('col-poradipredavani-kodurovne')
            ->setCode('col-stav'); // S_StavXMLFormulare.Popis

        for ($index = 1; $index <= 10; $index++) {
            if (RonPortalSettings::getValue("xml_rozliseni{$index}_nadpis")) {
                $this->addField()
                    ->setSqlFieldName("VyplnenyXMLFormular.Rozliseni{$index}")
                    ->setHeader(RonPortalSettings::getValue("xml_rozliseni{$index}_nadpis"))
                    ->setOrdering(true)
                    ->setClassName("rozliseni{$index}")
                    ->setCode("col-rozliseni{$index}");
            }
        }

        $this->addField()
            ->setSqlFieldName('VyplnenyXMLFormular.PocatecniUroven') ->setHeaderText('Pocatecni_uroven')
            ->setOrdering(true)
            ->setCode('col-pocatecni-uroven')
            ->setVisible(App::getDebugMode());

        $this->addField()
            ->setSqlFieldName('VyplnenyXMLFormular.PoradiPredavani') ->setHeaderText('Poradi_predavani')
            ->setOrdering(true)
            ->setCode('col-poradi')
            ->setVisible(App::getDebugMode());

        $this->addField()
            ->setSqlFieldName('PoradiPredavani.Vlastni') ->setHeaderText('Poradi_predavani_vlastni')
            ->setOrdering(true)
            ->setCode('col-vlastni')
            ->setVisible(App::getDebugMode());

        $this->addField()
            ->setSqlFieldName('PoradiPredavani.NekontrolovatHierarchii') ->setHeaderText('Poradi_predavani_nekontrolovat_hierarchii')
            ->setOrdering(true)
            ->setCode('col-hierarchie')
            ->setVisible(App::getDebugMode());

        $this->addField(new EditaceXMLField());

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('VyplnenyXMLFormular.DodatecnyPopis')
            ->setClassName('browserpoznamka')
            ->setHeaderColSpan(1);
    }

    function setFilterForm()
    {
        $this ->filterForm = new XMLFilterForm($this ->database, $this->xmlMenuId);
    }


//    function load($sqlFilter = '')
//    {
//        if (isset($_GET['stavxml'])) {
//            XMLFilterForm::setFilterStav((int)$_GET['stavxml']);
//        }
//
//        parent::load($sqlFilter);
//    }

    public function loadColumnConfiguration($colSettings)
    {
        if (isset($colSettings['xmlmenuid']) && $colSettings['xmlmenuid'] !== $this->xmlMenuId) {
            return;
        }

        parent::loadColumnConfiguration($colSettings);
    }


//    function renderFilterStav()
//    {
//        $stav = XMLFilterForm::getFilterStav();
//        if (!isset($stav)) {
//            XMLFilterForm::setFilterStav(-1);
//            $stav = -1; // vychozi hodnota - neukoncene
//        }
//
//        $data = [
//            -1 => TextManager::_('Neukonceno'),
//            -2 => TextManager::_('Ukonceno'),
//            -3 => TextManager::_('Vse')
//        ];
//
//        $url = '?menuid=seznamxmlformularu' . ($this->xmlMenuId ? '&xmlmenuid=' . $this->xmlMenuId : '') . '&stavxml=';
//
//        LinksList::renderLinks($data, $stav, $url);
//    }


//    function renderFilter()
//    {
//        $this->renderFilterStav();
//        $this ->filterForm ->render();
//    }



    /**
     * Vraci filtr pro vyber formularu vyplnenych uzivatelem
     * @return string
     */
    function getSQLFilterVlastniXML()
    {
        if (Prava::vratHodnotuPrava('xml') == 3  &&  $this ->filterForm ->getFilterValue('xmlzobrazitvse')) {
            return null;
        }

        $params = $this ->database ->createParameters();

        // formulare ktere vlozil aktualni uzivatel - v puvodnich verzich se nepouzivala tabulka XMLFormularVyplnil
        $result =
            '(' .
            ' (RC_Vlozil = ' . $params ->addParameter(User::getRC()) . ")\n" .
            // formulare ktere vyplnil aktualni uzivatel (i kdyz jsou zrovna na jine urovni predavani)
            ' OR ( EXISTS ( ' .
            ' SELECT * FROM XMLFormularVyplnil ' .
            " WHERE XMLFormularVyplnil.RC = " . $params ->addParameter(User::getRC()) .
            '   AND XMLFormularVyplnil.IdFormulare = VyplnenyXMLFormular.Id' .
            ") )\n" .
            // formulare na urovni oznacene "vlastni"
            ' OR (PoradiPredavani.Vlastni = 1 ' .
            ' AND (VyplnenyXMLFormular.RC = ' . $params ->addParameter(User::getRC()) . '  OR  VyplnenyXMLFormular.PP_RC = ' . $params ->addParameter(User::getRC()) . ')' .
            ")\n";

        $result .= ")\n";

        return new SqlStatement($result, $params);
    }


    /**
     * Vraci filtr pro vyber formularu podle procesu
     * @return string
     */
    private function getSQLFilterXMLPodleProcesu()
    {
        // pokud ma prava na vse a ma zapnute vse, tak nic nefiltruje
        if (Prava::vratHodnotuPrava('xml') == 3  &&  $this ->filterForm ->getFilterValue('xmlzobrazitvse')) {
            return '';
        }

        $params = $this ->database ->createParameters();
        //TODO: kontrola dle noveho nastaveni viditelnosti - provadet v enable line?

        // zjisti do kterych urovni hodnoceni jsem zarazeny (podle mych pracovnich mist)
        $filtrUrovne = PredavaciProcesyUtils::filtrUrovnePredavani($this ->database, 'PoradiPredavani.KodUrovne', $params);
        $kodTypuRozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI); // pouzije se, pokud neni typ rozdeleni u procesu zadan

        global $_XML_jenPodrizene; //pro zachovani funkcnosti v appdbset, pokud budou chtit zmenit, musi se zrusit v appdbset
        $XML_jenPodrizene = $_XML_jenPodrizene == 1 || RonPortalSettings::getIntValue('jenvlastnirozdelenihodnoceni') == 2;

        if (!$filtrUrovne) {
            // nemam pravo na zadne urovne hodnoceni
            return '0 = 1';
        }

        $result = '(';

        // vybere XML zamestnancu, na ktere mam pravo (podle pracovniho mista a hierarchie)
        $result .=
            '(PoradiPredavani.Vlastni = 0 ' . // urovne s volbou "vlastni" jsou resene samostatne
            " AND ( $filtrUrovne ) " .  // formular na "moji" urovni predavani
            ' AND  (' .
            // na aktualni urovni formulare se nekontroluje hierarchie
            'PoradiPredavani.NekontrolovatHierarchii = 1 '.
            '  OR ' .
            // nebo jsem ve stejnem nebo nadrazenem rozdeleni
            "EXISTS (\n  SELECT * " .
            " FROM RozdeleniOsob R_JA, HierarchieRozdeleni H, RozdeleniOsob R_Hodnoceny\n" .
            '  WHERE R_Ja.RC = ' . $params ->addParameter(User::getRC()) . "\n" .
            //"    AND R_JA.KodTypuRozdeleni = $kodtypurozdeleni". - puvodni verze - natvrdo jeden typ rozdeleni
            '    AND ( R_JA.KodTypuRozdeleni = PredavaciProces.KodTypuRozdeleni ' . // proces ma vlastni typ rozdeleni
            "          OR  (PredavaciProces.KodTypuRozdeleni IS NULL  AND  R_JA.KodTypuRozdeleni = $kodTypuRozdeleni) " . // typ rozdeleni u procesu neni zadan, 26.1.2024 tento OR muze zpusobit problem s rychlosti nacteni, v seznamu hodnoceni byl odstranen
            ' )' .
            '    AND (' .
            '     (R_Hodnoceny.DatumNastupu > ' . $params ->addDateParameter(time()) . // pokud se jedna o budouci PP (napr. pro nastupni formular), tak kontroluje platnost rozdeleni k datu nastupu
                    ' AND R_Ja.PlatnostOd <= R_Hodnoceny.DatumNastupu' .
                    ' AND (R_Ja.PlatnostDo >= R_Hodnoceny.DatumNastupu OR R_Ja.PlatnostDo IS NULL))' .
            '     OR (' .
                    'R_Ja.PlatnostOd <= ' . $params ->addDateParameter(time()) .
                    ' AND (R_Ja.PlatnostDo >= ' . $params ->addDateParameter(time()) . '  OR  R_Ja.PlatnostDo IS NULL))' .
            '    )' .
            //"    AND R_Hodnoceny.KodTypuRozdeleni = $kodtypurozdeleni". - puvodni verze - natvrdo jeden typ rozdeleni
            '    AND R_Hodnoceny.KodTypuRozdeleni = R_JA.KodTypuRozdeleni ' .
            '    AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
            '    AND H.KodNadrazene = R_Ja.Rozdeleni ' .
            '    AND H.TypPodrizene = R_Hodnoceny.KodTypuRozdeleni ' .
            '    AND H.KodPodrizene = R_Hodnoceny.Rozdeleni ' .
            ($XML_jenPodrizene ? ' AND H.KodNadrazene <> H.KodPodrizene ' : '') .
            '    AND (' .
            '     (R_Hodnoceny.DatumNastupu > ' . $params ->addDateParameter(time()) . // pokud se jedna o budouci PP (napr. pro nastupni formular), tak kontroluje platnost rozdeleni k datu nastupu
                    ' AND R_Hodnoceny.PlatnostOd <= R_Hodnoceny.DatumNastupu' .
                    ' AND (R_Hodnoceny.PlatnostDo >= R_Hodnoceny.DatumNastupu OR  R_Hodnoceny.PlatnostDo IS NULL))' .
            '     OR (' .
                    ' R_Hodnoceny.PlatnostOd <= ' . $params ->addDateParameter(time()) .
                    ' AND (R_Hodnoceny.PlatnostDo >= ' . $params ->addDateParameter(time()) . '  OR  R_Hodnoceny.PlatnostDo IS NULL))' .
            '    )' .
            '    AND ( ' .
            '      R_Hodnoceny.RC = VyplnenyXMLFormular.RC ' .
            '      OR ( ' .
            '        R_Hodnoceny.RC = VyplnenyXMLFormular.PP_RC ' .
            '        AND R_Hodnoceny.OsobniCislo = VyplnenyXMLFormular.PP_OsobniCislo ' .
            '        AND R_Hodnoceny.DatumNastupu = VyplnenyXMLFormular.PP_DatumNastupu ' .
            '      ) ' .
            '    ) ' .
            ")\n" .
            ')' .
            ')';

        $filtrUrovne2 = PredavaciProcesyUtils::filtrUrovnePredavani($this ->database, 'PoradiPredavani.KodUrovne', $params);
        // Vybere XML, ktere nejsou pro konkretni osobu, ale jsou pro moje a podrizene rozdeleni
        $result .=
            ' OR (' .
            "( $filtrUrovne2 ) " .  // dokument na "moji" urovni predavani
            ' AND VyplnenyXMLFormular.RC IS NULL ' .    // neni pro konkretni osobu
            ' AND VyplnenyXMLFormular.PP_RC IS NULL ' .
            ' AND ('.
            '    PoradiPredavani.NekontrolovatHierarchii = 1 '.
            '   OR ' .
            '    EXISTS ( SELECT * ' .        // jsem ve stejnem nebo nadrazenem rozdeleni
            '  FROM RozdeleniOsob R_JA, HierarchieRozdeleni H ' .
            '  WHERE R_Ja.RC = ' . $params ->addParameter(User::getRC()) .
            //"    AND R_JA.KodTypuRozdeleni = $kodtypurozdeleni". - puvodni verze

            '    AND ( R_JA.KodTypuRozdeleni = PredavaciProces.KodTypuRozdeleni ' . // proces ma vlastni typ rozdeleni
            "          OR  (PredavaciProces.KodTypuRozdeleni IS NULL  AND  R_JA.KodTypuRozdeleni = $kodTypuRozdeleni) " . // typ rozdeleni u procesu neni zadan
            '    )' .

            '    AND R_Ja.PlatnostOd <= ' . $params ->addDateParameter(time()) .
            '    AND (R_Ja.PlatnostDo >= ' . $params ->addDateParameter(time()) . '  OR  R_Ja.PlatnostDo IS NULL)' .
            '    AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
            '    AND H.KodNadrazene = R_Ja.Rozdeleni ' .
            '    AND H.TypPodrizene = VyplnenyXMLFormular.KodTypuRozdeleni ' .
            '    AND H.KodPodrizene = VyplnenyXMLFormular.Rozdeleni ' .
            ($XML_jenPodrizene ? ' AND H.KodNadrazene <> H.KodPodrizene ' : '') .
            ')' .
            ')' .
            ')';

        $result .= ")\n";

        return new SqlStatement($result, $params);
    }


    /**
     *
     * @param Parameters $params
     * @return string
     */
    function getSQLFilterRozdeleni($params)
    {
        if ($this->filterForm->getFilterValue('rozdeleni') === null) {
            return '';
        }

        $sqlPodrizene = '';

        switch ($this->filterForm->getFilterValue('rozdeleni', 'vcetnePodrizenych')) {
            case 0:
                $sqlPodrizene = 'AND (HierarchieRozdeleni.kodNadrazene = HierarchieRozdeleni.kodPodrizene) ';
                break;
            case 1:
                break;
            case 2:
                $sqlPodrizene = 'AND (HierarchieRozdeleni.kodNadrazene <> HierarchieRozdeleni.kodPodrizene) ';
                break;
            case 3:
                $sqlPodrizene = 'AND (HierarchieRozdeleni.kodPodrizene IN (SELECT Kod FROM Rozdeleni WHERE KodTypuRozdeleni = HierarchieRozdeleni.typNadrazene AND RozdeleniRodice = HierarchieRozdeleni.typNadrazene AND Rodic = HierarchieRozdeleni.kodNadrazene)) ';
                break;
        }

        // filtr podle rozdeleni formulare
        $result =
            '( ' .
            ' EXISTS ( ' .
            '   SELECT * FROM HierarchieRozdeleni ' .
            '   WHERE TypNadrazene = ' . (int)$this ->filterForm ->getFilterValue('rozdeleni', 'typRozdeleni') .
            '     AND KodNadrazene = ' . $params ->addParameter($this ->filterForm ->getFilterValue('rozdeleni')) .
            '     AND TypPodrizene = VyplnenyXMLFormular.KodTypuRozdeleni ' .
            '     AND KodPodrizene = VyplnenyXMLFormular.Rozdeleni ' .
            $sqlPodrizene .
            ' )';


        $result .= ') OR (';

        // filtr podle osoby formulare
        $result .=
            'EXISTS (' .
            '  SELECT * FROM RozdeleniOsob, HierarchieRozdeleni ' .
            '  WHERE HierarchieRozdeleni.TypNadrazene = ' . (int)$this ->filterForm ->getFilterValue('rozdeleni', 'typRozdeleni') .
            '    AND HierarchieRozdeleni.KodNadrazene = ' . $params ->addParameter($this ->filterForm ->getFilterValue('rozdeleni')) .
            '    AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.TypPodrizene' .
            '    AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.KodPodrizene ' .
            $sqlPodrizene .
            '    AND ( RozdeleniOsob.RC = VyplnenyXMLFormular.RC ' .
            ' OR (' .
            '   RozdeleniOsob.RC = VyplnenyXMLFormular.PP_RC ' .
            '   AND RozdeleniOsob.OsobniCislo = VyplnenyXMLFormular.PP_OsobniCislo ' .
            '   AND RozdeleniOsob.DatumNastupu = VyplnenyXMLFormular.PP_DatumNastupu ' .
            ' )' .
            ' AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter(time()) .
            ' AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter(time()) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
            '    ) ' .
            ')';

        $result .= ')';

        return $result;
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

//        // filtr pro stav
//        switch (XMLFilterForm::getFilterStav()) {
//            case -3 : // vsechny
//                $result = '';
//                break;
//
//            case -2 : // ukoncene
//                $result = ' (VyplnenyXMLFormular.Stav >= 128) ';
//                break;
//
//            case -1 : // neukoncene
//            default :
//                $result = ' (VyplnenyXMLFormular.Stav < 128) ';
//                break;
//        }

//        // filtr jen typ (popis)
//        if (XMLFilterForm::getJenPopisXML() && XMLFilterForm::getPopisXML()) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                "VyplnenyXMLFormular.Popis = '" . $this ->database ->addSlashes(XMLFilterForm::getPopisXML()) . "'";
//        }

        // filtr podle data vytvoreni
//        $stav = XMLFilterForm::getFilterStav();
//        $filtrPovinny = ($stav == -2 || $stav == -3); // -2 = ukonceno, -3 = vse
        //$filtrPovinny = false;
        //30.9.2019 zrusena povinnost filtru na datum, nastaveno strankovani browseru
        if ($this ->filterForm ->getFilterValue('datumod')) {
            $datumDo = $this ->filterForm ->getFilterValue('datumod', 'datumDo');

            $result .=
                ($result ? ' AND ' : '') .
                ' VyplnenyXMLFormular.DatumVytvoreni >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod')) .
                ($datumDo
                    ? ' AND VyplnenyXMLFormular.DatumVytvoreni < ' . $params ->addDateParameter( DateUtils::incDay($datumDo, 1) )
                    : ''
                );
        }

        if ($this ->filterForm ->getFilterValue('obdobi')) {
            $result .=
                ($result ? ' AND ' : '') .
                ' (VyplnenyXMLFormular.Obdobi >= ' . $params->addDateParameter($this ->filterForm ->getFilterValue('obdobi')) .
                ' AND VyplnenyXMLFormular.Obdobi <= ' . $params->addDateParameter($this ->filterForm ->getFilterValue('obdobi', 'datumDo')) . ')';
        }

        if ($this ->filterForm ->getFilterValue('osoba')) {
            $result .=
                ($result ? ' AND ' : '') .
                '((VyplnenyXMLFormular.RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) . ') OR ' .
                ' (VyplnenyXMLFormular.PP_RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
                '  AND VyplnenyXMLFormular.PP_OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
                '  AND VyplnenyXMLFormular.PP_DatumNastupu = ' . $params->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')) .
                '))';
        }

        $filtrRozdeleni = $this->getSQLFilterRozdeleni($params);
        if ($filtrRozdeleni) {
            $result .=
                ($result ? ' AND ' : '') .
                '(' . $filtrRozdeleni . ')';
        }

        return $result
            ? new SqlStatement($result, $params)
            : null;
    }


    /**
     * @param string|SqlStatement   $sqlFilter   Dodatecny sql filtr
     * @return string|SqlStatement
     */
    public function getSQL($sqlFilter = '')
    {
        //kvuli nastaveni poli pridanych v konfiguraci se select nacte jen jednou
        $selectPart = $this ->getSelectSQLPart();

        $wherePart1 = SQLUtils::concatenateWherePartsOfSql(
            [$this ->sqlWherePart, $sqlFilter, $this ->getSQLFilter(), $this->getSQLFilterXMLPodleProcesu()]);

        $wherePart2 = SQLUtils::concatenateWherePartsOfSql(
            [$this ->sqlWherePart, $sqlFilter, $this ->getSQLFilter(), $this->getSQLFilterVlastniXML()]);

        if (!$wherePart1 instanceof SqlStatement) {
            $wherePart1 = new SqlStatement($wherePart1, $this ->database ->createParameters());
        }
        if (!$wherePart2 instanceof SqlStatement) {
            $wherePart2 = new SqlStatement($wherePart2, $this ->database ->createParameters());
        }

        $params = $this ->database ->createParameters();
        $params ->appendParameters($wherePart1 ->parameters);
        $params ->appendParameters($wherePart2 ->parameters);

        $orderBy = $this ->getOrderSQLPart();

        // kvuli rychlosti je dotaz rozdelen na 2 poddotazy
        return new SqlStatement(
            'SELECT DISTINCT ' . $selectPart . "\n" .
            ' FROM ' . $this ->sqlFromPart . "\n" .
            ($wherePart1->sqlText ? ' WHERE ' . $wherePart1 ->sqlText: '') .
            "\n\n" .
            "UNION\n\n" .
            'SELECT DISTINCT ' . $selectPart . "\n" .
            ' FROM ' . $this ->sqlFromPart . "\n" .
            ($wherePart2->sqlText ? ' WHERE ' . $wherePart2 ->sqlText : '') .
            "\n" .
            ($orderBy ? 'ORDER BY ' . $orderBy : ''),
            $params
        );

//        return SQLUtils::createSql(
//            $this ->getSelectSQLPart(), $this ->sqlFromPart,
//            $wherePart, $orderBy, $this ->distinct, $this ->sqlGroupByPart);
    }
}

