<?php
/**
 * stanislav.elbl, 09.08.2021 22:07
 */

namespace RonPortal\Zamestnanci;

use Ron\Controls\FormField;
use Ron\DB\Database;
use RonPortal\Dokument;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;

class DokumentOsobyField extends FormField
{
    /** @var int    Kod typu dokumentu */
    public $typDokumentu;

    /** @var string */
    public $popisDokumentu;

    /** @var Dokument[] */
    protected $dokumenty;

    /** @var int    Pouzije se pro odkaz na stahnuti dokumentu */
    protected $idPP;


    public function __construct()
    {
        parent::__construct(self::FIELD_TYPE_FILE);
        $this ->readOnly = false;

        $this ->popisDokumentu = '%JMENO% %PRIJMENI%';

        //$this ->otherHtmlParams = ['autocomplete' => 'off'];
    }

    /**
     * Nacteni konfigurace
     * @param $key
     * @param $value
     */
    protected function loadConfigurationItem($key, $value)
    {
        switch ($key) {
            case 'typDokumentu' :
                $this ->typDokumentu = (int)$value;
                if (!$this ->fieldName) {
                    $this ->fieldName = 'Dokument' . $this ->typDokumentu;
                }
                break;

            case 'fieldType' : // ignoruje - aby zustal typ pole FIELD_TYPE_FILE
                break;

            case 'popis' :
                $this ->popisDokumentu = $value;
                break;

            default :
                parent::loadConfigurationItem($key, $value);
        }
    }

    /**
     * @param Database $database
     * @param string $rc
     */
    public function nactiDokumentyOsoby($database, $rc, $idPP)
    {
        $this ->idPP = $idPP;

        $params = $database ->createParameters();

        $sql =
            'SELECT ' . Dokument::getSelectPart($database) .
            ' FROM Dokument, DokumentyOsoby' .
            ' WHERE DokumentyOsoby.Id = Dokument.Id' .
            ' AND DokumentyOsoby.RC = ' . $params ->addParameter($rc) .
            ' AND Dokument.Typ = ' . $params ->addParameter((int)$this ->typDokumentu);

        $this ->dokumenty = Dokument::nactiDokumenty($database, $sql, $params);
    }


    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);

        if ($this ->dokumenty) {
            Dokument::renderTable($this ->dokumenty, $this ->idPP ? "?menuid=dokumenty&dokumentosoby=1&osoba={$this ->idPP}&dokumentid=" : null);
        }
    }


    /**
     * Vlozi dokument do databaze a pripoji ho k zadanemu pracovnimu pomeru. Vraci Id vlozeneho dokumentu
     * @param Database $database
     * @param PracovniPomer $pp
     * @param object $logDescription
     * @return int
     */
    public function ulozDokument($database, $pp, $logDescription)
    {
        // TODO: vymazat existujici dokument
        if (!isset($_FILES[$this ->fieldName]['tmp_name'])  ||  !is_uploaded_file($_FILES[$this ->fieldName]['tmp_name'])) {
            // neni prilozen zadny soubor - nedela nic
            return null;
        }

        $popis = str_replace(
            ['%JMENO%', '%PRIJMENI%', '%OSOBNICISLO%', '%IDOSOBY%', '%IDPP%', '%DATUM%'],
            [$pp ->jmeno, $pp ->prijmeni, $pp ->osobniCislo, $pp ->rc, $pp ->idPP, date('j.n.Y', time())],
            $this ->popisDokumentu);


        $idDokumentu = DokumentyUtils::vlozDokument($database,
            $this->fieldName, $this ->typDokumentu,
            $popis, $logDescription);

        $recDokumentOsoby = new RonPortalDBRecord($database, 'DokumentyOsoby', $logDescription);
        $recDokumentOsoby->addField('RC', '', true, 15, $pp->rc);
        $recDokumentOsoby->addField('id', 'int', true, 0, $idDokumentu);

        $recDokumentOsoby->insertRecord();

        return $idDokumentu;
    }
}
