<?php

namespace RonPortal\Zamestnanci;

use Ron\DB\Database;
use Ron\DB\DBException;


/**
 * Class EvidencniStav
 * @package Ron\RonPortal\Zamestnanci
 */
class EvidencniStav
{
    public $rc;
    public $osobniCislo;
    public $datumNastupu;
    public $idPP;
    public $typPP;

    public $kodStavu;
    public $popisStavu;

    public $platnostOd;
    public $platnostDo;
    public $poznamka;

    /**
     * @param Database $database
     * @param $rc
     * @return EvidencniStav[]|array
     * @throws DBException
     */
    public static function nactiEvidencniStavy($database, $rc)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT EvidencniStav.KodEvidencnihoStavu, TypEvidencnihoStavu.Popis, PlatnostOd, PlatnostDo,  ' .
            $database ->castWideMemo('EvidencniStav.Poznamka') . ',' .
            '  PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, PracovniPomer.TypPracovnihoPomeru, PracovniPomer.ID' .
            "\nFROM EvidencniStav JOIN TypEvidencnihoStavu ON (TypEvidencnihoStavu.Kod = EvidencniStav.KodEvidencnihoStavu) " .
            "   JOIN PracovniPomer ON  (PracovniPomer.RC = EvidencniStav.RC AND PracovniPomer.OsobniCislo = EvidencniStav.OsobniCislo AND PracovniPomer.DatumNastupu = EvidencniStav.DatumNastupu)" .
            "\nWHERE PracovniPomer.RC = " . $params->addParameter($rc) .
            '  AND PracovniPomer.DatumNastupu <= ' . $params->addDateParameter(time()) .
            '  AND (PracovniPomer.DatumUkonceni >= ' . $params->addDateParameter(time()) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .

            ' ORDER BY PracovniPomer.DatumNastupu, PlatnostOd DESC';

        $cursor = $database->getSqlResult($sql, $params);

        $stavy = [];

        while ($row = $cursor->fetch()) {
            $stav = new EvidencniStav();
            $stav->rc = $rc;
            $stav->nactiDataZRadku($database, $row);

            $stavy[] = $stav;
        }

        $cursor ->close();

        return $stavy;
    }

    /**
     * @param Database $database
     * @param $row
     */
    private function nactiDataZRadku($database, $row)
    {
        $this->kodStavu = $row[0];
        $this->popisStavu = $row[1];
        $this->platnostOd = $database->stringToTimestamp($row[2]);
        $this->platnostDo = $database->stringToTimestamp($row[3]);
        $this->poznamka = $row[4];

        $this->idPP = $row[8];
        $this->osobniCislo = $row[5];
        $this->datumNastupu = $database->stringToTimestamp($row[6]);
        $this->typPP = $row[7];
    }
}
