<?php
/**
 * stanislav.elbl, 12.03.2021
 */

namespace RonPortal\Zamestnanci;

use Ron\Controls\FormField;
use Ron\Controls\FormRowArray;
use Ron\DateUtils;
use Ron\DB\DBException;
use Ron\DB\DBRecordField;
use Ron\DB\SqlStatement;
use Ron\MessageManager;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\User;
use Tracy\Debugger;

/**
 * Formular pro nahlaseni zmeny osobnich udaju.
 */
class NahlasenaZmenaFormField extends FormField
{
    /** @var mixed      Aktualni hodnota nactena z databaze */
    public $existingValue;

    /** @var bool       Pri nacitani hodnot vzdy pouzije default hodnotu - pouziva se napriklad pro PlatnostOd */
    public $alwaysUseDefaultValue = false;

    public $zobrazitVHistorii = true;


    /**
     * @param bool $zobrazitVHistorii
     * @return NahlasenaZmenaFormField
     */
    public function setZobrazitVHistorii($zobrazitVHistorii): NahlasenaZmenaFormField
    {
        $this->zobrazitVHistorii = $zobrazitVHistorii;
        return $this;
    }



    public function loadEditValue($database, $usePostValues, $rowData, $canEditRecord)
    {
        $this ->loadDatabaseValue($database, $rowData);
        $this ->existingValue = $this ->value;

        if (!$usePostValues  &&  $this ->alwaysUseDefaultValue) {
            $this ->value = $this ->defaultValue;
        } else {
            parent::loadEditValue($database, $usePostValues, $rowData, $canEditRecord);
        }
    }


    public function getFieldCssClass()
    {
        $result = parent::getFieldCssClass();

        $isChange = $this ->value != $this ->existingValue;
        if ($this ->fieldType === FormField::FIELD_TYPE_CHECK) {
            if (!$this ->existingValue  &&  !$this ->value) {
                $isChange = false; // checkbox ma value '' a existingValue '0'
            }
        }

        if ($isChange) {
            $result .= ' form-control-changed';
        }
        return $result;
    }


    public function getZmenaFieldTextForValue($database, $value)
    {
        return parent::getFieldText($database, $value, false, false, null);
    }


    protected function loadConfigurationItem($key, $value)
    {
        if ($key == 'zobrazitVHistorii') {
            $this ->zobrazitVHistorii = (int)$value;
        } else {
            parent::loadConfigurationItem($key, $value);
        }
    }





//    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
//    {
//        echo '=' . StringUtils::htmlTextConvert($this ->existingValue) . '= - ' . ($this ->value != $this ->existingValue);
//        return parent::showFieldContent($database, $value, $canEdit, $insert, $rowData); // TODO: Change the autogenerated stub
//    }


}


class PripojitDokumentZmenyField extends FormField
{
    /** @var int */
    public $typDokumentu;

    /** @var string */
    public $popisDokumentu;


    public function __construct()
    {
        parent::__construct(FormField::FIELD_TYPE_FILE);

        $this ->typDokumentu = RonPortalSettings::getIntValue('hlaseni_zmeny_typdokumentu');
        $this ->popisDokumentu = \Ron\ronText("dokument_nahlasene_zmeny") . ' %ID% (%TIME%)';
    }

    public function getFieldAttributes($defaultAttrs = null, $rowData = null)
    {
        $result = parent::getFieldAttributes($defaultAttrs, $rowData);
        $result['data-typDokumentu'] = $this ->typDokumentu;
        $result['data-popisDokumentu'] = $this ->popisDokumentu;
        return $result;
    }

    protected function loadConfigurationItem($key, $value)
    {
        switch ($key) {
            case 'typDokumentu' :
                $this ->typDokumentu = (int)$value;
                break;

            case 'popisDokumentu' :
                $this ->popisDokumentu = $value;
                break;

            default :
                parent::loadConfigurationItem($key, $value);
        }
    }


}

class NahlasenaZmenaForm extends RonPortalForm
{
    /** @var NahlasenaZmena */
    private $nahlasenaZmena = null;

    /** @var PracovniPomer */
    private $pp;

    /** @var FormField */
    private $fPoznamka;

    /** @var array      Asociativni pole hodnot nactenych z formulare  */
    private $hodnotyZmeny;

    function __construct($database)
    {
        parent::__construct($database);

        $this ->code = 'formnahlasenazmena';

        $this ->setXmlEventsName('NAHLASENA_ZMENA');

        if ($this ->formAction === 'new'  &&  isset($_GET['record'])) {
            $this ->formAction = 'edit';
        }

        $this ->stornoAction = ($_GET['back'] ?? '') === 'osobniudaje'
            ? '?menuid=osobniudaje' . (($_GET['osoba'] ?? '') ? '&osoba=' . $_GET['osoba'] : '')
            : '?menuid=nahlasenezmeny';

        if ($this ->isInsert()) {
            $this ->nahlasenaZmena = NahlasenaZmena::nactiTypZmeny($this ->database, $_GET['typzmeny']);
            $this->pp = PracovniPomer::nactiPP($this->database, (int)$_GET['osoba']);
        } else {
            $this ->nahlasenaZmena = NahlasenaZmena::nactiZmenu($this ->database, (int)$_GET['record']);
            $this ->pp = PracovniPomer::nactiPPOsoby($this ->database, $this ->nahlasenaZmena ->rc)[0];
        }

        if ($this ->nahlasenaZmena ->stav) {
            $this ->readOnly = true;
        }

        if (!$this ->nahlasenaZmena ->povolitVIntrawebu) {
            throw new \Exception('Hlaseni teto zmeny neni v Portalu povoleno!');
        }

        if (!NahlasenaZmena::jePravoNaZmenu($this ->nahlasenaZmena ->typZmeny, $this ->pp ->rc, true, false)) {
            throw new NeniPravoException();
        }

        $this->title = TextManager::_('nahlaseni_zmeny') . ' - ' . $this ->nahlasenaZmena ->popisTypuZmeny;

        $this ->addField(FormField::FIELD_TYPE_TEXT)
            ->addFieldCssClass('font-weight-bold margin-bottom-2 margin-top-2')
            ->setText($this ->pp ->jmenoPrijmeni)
            ->setCode('field-osoba');

        $this ->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('Stav', 'NahlasenaZmena')
            ->setSqlFieldName(null)
            ->setDefaultValue($this ->nahlasenaZmena ->stav)
            ->setFieldId('nahlasena_zmena_stav');

        $this ->addFormFields();

        $this ->fPoznamka = $this ->addField(new FormField(FormField::FIELD_TYPE_MEMO))
            ->setFieldName('Poznamka', 'NahlasenaZmena') ->setCaption('Poznamka')
            ->setSqlFieldName(null)  // aby se nenacitala puvodni hodnota
            ->setCode('field-poznamka')
            ->setDefaultValue($this ->nahlasenaZmena ->poznamka ?? '')
            ->setRowClass("form-row margin-top-3");

        $this ->addField(new PripojitDokumentZmenyField())
            ->setFieldName('PripojitDokument', 'NahlasenaZmena') ->setSqlFieldName(null)
            ->setCaption('Pripojit_dokument')
            ->setCode('field-dokument')
            ->setVisible(!$this ->readOnly);
    }


    /**
     * @return NahlasenaZmena
     */
    public function getNahlasenaZmena(): NahlasenaZmena
    {
        return $this->nahlasenaZmena;
    }


    public function loadFieldConfiguration($fieldSettings)
    {
        if ($this ->nahlasenaZmena ->typZmeny === ($fieldSettings['typZmeny'] ?? $this ->nahlasenaZmena ->typZmeny)) {
            parent::loadFieldConfiguration($fieldSettings);
        }
    }

//    public function loadConfigurationItem($key, $value, $valueText)
//    {
//        if ($this ->nahlasenaZmena ->typZmeny === $fieldSettings['typzmeny'] ?? $this ->nahlasenaZmena ->typZmeny) {
//            parent::loadConfigurationItem($key, $value, $valueText); // TODO: Change the autogenerated stub
//        }
//    }


    public function addField($field = FormField::FIELD_TYPE_EDIT)
    {
        if (!($field instanceof FormField)) {
            $field = new NahlasenaZmenaFormField($field);
        }

        return parent::addField($field);
    }


    /**
     * @param $fieldType
     * @param $alwaysUseDefaultValue
     * @return NahlasenaZmenaFormField
     */
    public function addNahlasenaZmenaField($fieldType, $alwaysUseDefaultValue = false)
    {
        $field = new NahlasenaZmenaFormField($fieldType);
        $field ->alwaysUseDefaultValue = $alwaysUseDefaultValue;

        parent::addField($field);
        return $field;
    }


    function addFormFields()
    {
        switch ($this ->nahlasenaZmena ->typZmeny) {
            case 'trvale' :
                $this ->addFieldsAdresa(true);
                break;

            case 'prechodne' :
                $this ->addFieldsAdresa(false);
                break;

            case 'rodinnystav' :
                $this ->addFieldsRodinnyStav();
                break;

            case 'ucetosoby' :
                $this ->addFieldsUcetOsoby();
                break;

            case 'pojistovna' :
                $this ->addFieldsZdravotniPojistovna();
                break;

            case 'penzijnipripojisteni' :
                $this ->addFieldsPripojisteni(false); // zivotni=false
                break;

            case 'zivotnipojisteni' :
                $this ->addFieldsPripojisteni(true); // zivotni=true
                break;

            case 'vzdelani' :
                $this ->addFieldsVzdelani();
                break;

            case 'osobni' :
                $this ->addFieldsOsobni();
                break;

            case 'rodinnyprislusnik' :
                $this ->addFieldsRodinnyPrislusnik(false);
                break;

            case 'novyclenrodiny' :
                $this ->addFieldsRodinnyPrislusnik(true);
                break;

            case 'prukaz' :
                $this ->addFieldsPrukaz(true);
                break;
        }
    }

    /**
     * Prida pole pro zmenu adresy osoby (trvale nebo prechodne)
     * @param $trvale
     */
    function addFieldsAdresa($trvale) {
        $code = $trvale ? 'trvale' : 'prechodne';

        $this ->sqlTableName = 'AdresaOsoby';

        $f = $this ->addNahlasenaZmenaField(FormField::FIELD_TYPE_DATE, true)   // aby se platnost od pro editaci prednastavila na zadanou default hodnotu
            ->setFieldName('PlatnostOd', 'AdresaOsoby') ->setCaption('Platnost_od')
            //  ->setSqlFieldName(null) // aby se nenacitala pri insert
            ->setDefaultValue(date('j.n.Y', time()))
            ->setCode("field-{$code}-platnostod")
            ->setFieldId("zmena_{$code}_platnostod")
            ->setRequired(true);

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('PlatnostDo', 'AdresaOsoby') ->setCaption('Platnost_do')
            ->setCode("field-{$code}-platnostdo")
            ->setFieldId("zmena_{$code}_platnostdo");

        $this ->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('KontaktniAdresa', 'AdresaOsoby') ->setCaption('Kontaktni_adresa')
            ->setCode("field-{$code}-kontaktni")
            ->setFieldId("zmena_{$code}_kontaktni");

        $this ->addField()
            ->setFieldName('Ulice', 'AdresaOsoby') ->setCaption('Ulice')
            ->setMaxLength(60)
            ->setCode("field-{$code}-ulice")
            ->setFieldId("zmena_{$code}_ulice")
            ->setRequired($trvale);

        $this->addField()
            ->setFieldName('PSC', 'AdresaOsoby')->setCaption('psc')
            ->setMaxLength(10)
            ->setCode("field-{$code}-psc")
            ->setFieldId("zmena_{$code}_psc")
            ->setFieldCssClass('form-control psc-input')
            ->setOtherHtmlParams([
                "data-mesto-input-selector" => "#zmena_{$code}_mesto"
            ])
            ->setRequired($trvale);


        $this->addField()
            ->setFieldName('Mesto', 'AdresaOsoby')->setCaption('mesto')
            ->setMaxLength(40)
            ->setCode("field-{$code}-mesto")
            ->setFieldId("zmena_{$code}_mesto")
            ->setFieldCssClass('form-control ronselect mesto-input')
            ->setOtherHtmlParams([
                'data-max-visible-rows' => '50',
                'data-ronselect-class' => 'form-control',
                'data-auto-load-items' => '1',
                'data-items-url' => '?menuid=download&seznammest=1',
                'data-enable-url-cache' => '1',
                'data-columns' => 'mesto;psc;okresPopis',
                'data-filter-columns' => 'mesto;psc',   // nevyhledava v okresu
                'data-captions' => TextManager::_('Nazev') . ';' . TextManager::_('PSC') . ';' . TextManager::_('Okres'),
                'data-value-property' => 'mesto',
                'data-text-column' => 'mesto',
                'data-only-autocomplete' => '1',        // umoznuje zadat jakykoliv text
                // napojeni na dalsi inputy
                "data-psc-input-selector" => "#zmena_{$code}_psc",
                "data-obec-input-selector" => "#zmena_{$code}_obec"
            ])
            ->setRequired($trvale);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Obec', 'AdresaOsoby')->setCaption('obec')
            ->setMaxLength(40)
            ->setCode("field-{$code}-obec")
            ->setFieldId("zmena_{$code}_obec")
            ->setFieldCssClass('form-control ronselect obec-input')
            ->setSelectSql('SELECT Kod, Popis FROM Obec WHERE 0 = 1')
            ->setShowEmptyValue(true)
            ->setOtherHtmlParams([
                'data-max-visible-rows' => '50',
                'data-auto-load-items' => '1',
                'data-items-url' => '?menuid=download&seznamobci=1&showemptyvalue=1',
                'data-enable-url-cache' => '1',
                'data-columns' => 'popis;okresPopis',
                'data-filter-columns' => 'popis',   // nevyhledava v okresu
                'data-captions' => TextManager::_('Nazev') . ';' . TextManager::_('Okres'),
                'data-value-property' => 'kod',
                'data-value-column' => 'kod',
                'data-text-column' => 'popis',
                // napojeni na dalsi inputy
                "data-mesto-input-selector" => "#zmena_{$code}_mesto"
            ]);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Stat', 'AdresaOsoby')->setCaption('Stat')
            ->setCode("field-{$code}-stat")
            ->setFieldId("zmena_{$code}_stat")
            ->setSelectSql('SELECT Kod, Popis FROM Stat ORDER BY Popis')
            ->setFieldCssClass('form-control ronselect')
            ->setPreferredValues(['CZ', 'SK']);

        // telefony
        $this ->addField()
            ->setFieldName('Telefon1', 'AdresaOsoby') ->setCaption('Telefon')
            ->setMaxLength(20)
            ->setCode("field-{$code}-telefon1")
            ->setFieldId("zmena_{$code}_telefon1");

        $this ->addField()
            ->setFieldName('Telefon2', 'AdresaOsoby') ->setCaption('Telefon_2')
            ->setMaxLength(20)
            ->setCode("field-{$code}-telefon2")
            ->setFieldId("zmena_{$code}_telefon2");

        $this ->addField()
            ->setFieldName('Mobil', 'AdresaOsoby') ->setCaption('Mobil')
            ->setMaxLength(20)
            ->setCode("field-{$code}-mobil")
            ->setFieldId("zmena_{$code}_mobil");

        $this ->addField()
            ->setFieldName('E_Mail', 'AdresaOsoby') ->setCaption('E_mail')
            ->setMaxLength(60)
            ->setCode("field-{$code}-e_mail")
            ->setFieldId("zmena_{$code}_e_mail");

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('ZpusobDopravy', 'AdresaOsoby')->setCaption('Zpusob_dopravy_do_zamestnani')->setDataType('int')
            ->setCode("field-{$code}-zpusobdopravy")
            ->setFieldId("zmena_{$code}_zpusobdopravy")->setShowEmptyValue(false)
            ->setSelectSql('SELECT Kod, Popis FROM DopravaDoZamestnani WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setDefaultValue(0)
            ->setPreferredValues([0]);

        $this->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('VzdalenostDoZamestnani', 'AdresaOsoby')->setCaption('Vzdalenost_do_zamestnani')
            ->setCode("field-{$code}-vzdalenostdozamestnani")
            ->setFieldId("zmena_{$code}_vzdalenostdozamestnani")
            ->setFieldCssClass('inputFloat margin-right-2')
            ->setCaption2('km');
    }


    function addFieldsRodinnyStav()
    {
        $this ->sqlTableName = 'RodinnyStav';
        $code = 'rodinnystav';

        $this ->addNahlasenaZmenaField(FormField::FIELD_TYPE_DATE, true) // aby se platnost od pro editaci prednastavila na zadanou default hodnotu
            ->setFieldName('PlatnostOd', 'RodinnyStav') ->setCaption('Platnost_od')
            ->setDefaultValue(date('j.n.Y', time()))
            ->setCode("field-{$code}-platnostod")
            ->setFieldId("zmena_{$code}_platnostod")
            ->setRequired(true);

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('RodinnyStav', 'RodinnyStav') ->setCaption('Rodinny_stav')
            ->setSqlFieldName('KodRodinnehoStavu')
            ->setCode("field-$code-kodrodinnehostavu")
            ->setFieldId("zmena_{$code}_kodrodinnehostavu")
            ->setSelectSql(
                'SELECT Kod, Popis FROM S_RodinnyStav WHERE (%FILTER%) ORDER BY %ORDER%',
                '1 = 1', 'Popis')
            ->setShowEmptyValue(false)
            ->setRequired(true)
            ->setDataType('int');
    }


    function addFieldsUcetOsoby()
    {
        $this ->sqlTableName =
            'VyplaceniMzdy INNER JOIN BankovniUcet' .
            ' ON VyplaceniMzdy.SerioveCisloUctuVyplaty = BankovniUcet.SerioveCislo';

        $code = 'ucetosoby';

        $this ->addNahlasenaZmenaField(FormField::FIELD_TYPE_TEXT)
            ->setZobrazitVHistorii(false)
            ->setCaption('upozorneni')
            ->setText(TextManager::_('Zmena_bankovniho_uctu_upozorneni'))
            //->setVisible(RonPortalSettings::getIntValue('zobrazit_upozorneni_uctu'))    SE 28.5.2024 - defaultne se zobrazi vzdy. Vypnout lze v konfiguracnim XML
            ->addFieldCssClass('warningmsg')
            ->setCode("field-{$code}-upozorneni");

        $zmenaUctuMinDate = date('j.n.Y', DateUtils::incMonth(DateUtils::firstDayOfMonth(time())));

        $this ->addNahlasenaZmenaField(FormField::FIELD_TYPE_DATE, true) // aby se platnost od pro editaci prednastavila na zadanou default hodnotu
            ->setFieldName('PlatnostOd', 'VyplaceniMzdy') ->setCaption('Platnost_od')
            //->setSqlFieldName(null) // aby se nenacitala pri insert
            ->setDefaultValue($zmenaUctuMinDate)
            ->setOtherHtmlParams(['data-roncalendar-mindate' => $zmenaUctuMinDate])
            ->setCode("field-{$code}-platnostod")
            ->setFieldId("zmena_{$code}_platnostod")
            ->setRequired(true);

        $this ->addField()
            ->setFieldName('CisloUctu', 'BankovniUcet') ->setCaption('Cislo_uctu')
            ->setCode("field-{$code}-cislouctu")
            ->setFieldId("zmena_{$code}_cislouctu")
            ->setRequired(true);

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Banka', 'BankovniUcet') ->setCaption('Banka')
            ->setSqlFieldName('KodBanky')
            ->setCode("field-$code-kodbanky")
            ->setFieldId("zmena_{$code}_kodbanky")
            ->setRequired(true)
            ->setSelectSql(
                'SELECT Kod, Nazev FROM Banka WHERE (%FILTER%) ORDER BY %ORDER%',
                '1 = 1', 'Nazev')
            ->setSelectShowCode(true)
            ->addFieldCssClass('ronselect');

        $this ->addField()
            ->setFieldName('VariabilniSymbol', 'BankovniUcet') ->setCaption('Variabilni_symbol')
            ->setMaxLength(20)
            ->setCode("field-{$code}-variabilnisymbol")
            ->setFieldId("zmena_{$code}_variabilnisymbol");

        // udaje pro zahranicni platbu - impicitne skryte
        $this ->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('ZahranicniPlatba', 'BankovniUcet') ->setCaption('Zahranicni_ucet')
            ->setCode("field-{$code}-zahranicniplatba")
            ->setFieldId("zmena_{$code}_zahranicniplatba")
            ->setRowClass('form-row margin-top-2')
            ->setVisible(false);

        $this ->addField()
            ->setFieldName('IBAN', 'BankovniUcet') ->setCaption('IBAN')
            ->setMaxLength(40)
            ->setCode("field-{$code}-iban")
            ->setFieldId("zmena_{$code}_iban")
            ->setVisible(false);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('KodMeny_ISO', 'BankovniUcet')->setCaption('Mena')
            ->setCode("field-{$code}-kodmeny_iso")
            ->setFieldId("zmena_{$code}_kodmeny_iso")
            ->setSelectSql(
                'SELECT KodMeny_ISO, KodMeny_ISO AS Popis FROM Mena WHERE %FILTER% ORDER BY %ORDER%',
                'Aktivni = 1',
                'KodMeny_ISO')
            ->setVisible(false);

        $this ->addField() // FormField::FIELD_TYPE_COMBOSTR
            ->setFieldName('PlatebniTitul', 'BankovniUcet') ->setCaption('Platebni_titul')
            ->setMaxLength(10)
            ->setCode("field-{$code}-platebnititul")
            ->setFieldId("zmena_{$code}_platebnititul")
            ->setOptions([
                '410' => 'Mzda_zamestnance_s_kratkodobym_pobytem',
                '412' => 'Mzda_zamestnance_s_dlouhodobym_pobytem'
            ])
            ->setVisible(false);

        // E:CisloUctu:Cislo_uctu;Banka:Banka:Banka;E:VariabilniSymbol:Variabilni_symbol;D:PlatnostOd:Platnost_od;soubor:soubor:Pripojit_dokument
    }


    function addFieldsZdravotniPojistovna()
    {
        $this ->sqlTableName = 'PojisteniOsobyU_ZP';
        $code = 'pojistovna';

        $this ->addNahlasenaZmenaField(FormField::FIELD_TYPE_DATE, true)    // aby se platnost od pro editaci prednastavila na zadanou default hodnotu
            ->setFieldName('PlatnostOd', 'PojisteniOsobyU_ZP') ->setCaption('Platnost_od')
            ->setSqlFieldName(null) // aby se nenacitala pri insert
            ->setDefaultValue(date('j.n.Y', time()))
            ->setCode("field-{$code}-platnostod")
            ->setFieldId("zmena_{$code}_platnostod")
            ->setRequired(true);

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('KodZdravotniPojistovny', 'PojisteniOsobyU_ZP') ->setCaption('pojistovna')
            ->setCode("field-$code-kodzdravotnipojistovny")
            ->setFieldId("zmena_{$code}_kodzdravotnipojistovny")
            ->setRequired(true)
            ->setSelectSql(
                'SELECT Kod, Popis FROM ZdravotniPojistovna WHERE (%FILTER%) ORDER BY %ORDER%',
                '1 = 1', 'Popis')
            ->setSelectShowCode(true)
            ->addFieldCssClass('ronselect');

        $this ->addField()
            ->setFieldName('CisloPojistence', 'PojisteniOsobyU_ZP') ->setCaption('Cislo_pojistence')
            ->setCode("field-{$code}-cislopojistence")
            ->setFieldId("zmena_{$code}_cislopojistence")
            ->setMaxLength(15)
            ->setRequired(true);

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('CisloUctarny', 'PojisteniOsobyU_ZP') ->setCaption('Cislo_uctarny')
            ->setCode("field-$code-cislouctarny")
            ->setFieldId("zmena_{$code}_cislouctarny")
            ->setRequired(true)
            ->setSelectSql(
                'SELECT Kod, Popis FROM Uctarna WHERE (%FILTER%) ORDER BY %ORDER%',
                '1 = 1', 'Popis');

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('TypZamestnance', 'PojisteniOsobyU_ZP') ->setCaption('Typ_zamestnance_ZP')
            ->setCode("field-$code-typzamestnance")
            ->setFieldId("zmena_{$code}_typzamestnance")
            ->setRequired(true)
            ->setSelectSql(
                'SELECT Kod, Popis FROM S_TypZamestnanceProZP WHERE (%FILTER%) ORDER BY %ORDER%',
                '1 = 1', 'Popis')
            ->setDataType('int');

        // ZdravotniPojistovna:KodZdravotniPojistovny:pojistovna:Pr;D:PlatnostOd:Platnost_od:Pr;
        // D:PlatnostDo:Platnost_do;
        // E:CisloPojistence:Cislo_pojistence:Po;Uctarna:CisloUctarny:Cislo_uctarny:Po;S_TypZamestnanceProZP:TypZamestnance:Typ_zamestnance_ZP:Po;soubor:soubor:Pripojit_dokument
    }


    function addFieldsPripojisteni(bool $zivotni)
    {
        $this ->sqlTableName = $zivotni ? 'SmlouvaZivotniPojisteni' : 'SmlouvaPenzijnihoPrip';
        $code = $zivotni ? 'zivotnipojisteni' : 'penzijnipripojisteni';
        $tableName = $zivotni ? 'SmlouvaZivotniPojisteni' : 'SmlouvaPenzijnihoPrip';

        $this ->addNahlasenaZmenaField(FormField::FIELD_TYPE_DATE, true)    // aby se platnost od pro editaci prednastavila na zadanou default hodnotu
            ->setFieldName('PlatnostOd', $tableName) ->setCaption('Platnost_od')
            ->setSqlFieldName(null) // aby se nenacitala pri insert
            ->setDefaultValue(date('j.n.Y', time()))
            ->setCode("field-{$code}-platnostod")
            ->setFieldId("zmena_{$code}_platnostod")
            ->setRequired(true);

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('PlatnostDo', $tableName) ->setCaption('Platnost_do')
            ->setCode("field-{$code}-platnostdo")
            ->setFieldId("zmena_{$code}_platnostdo");

        if ($zivotni) {
            $this->addField(FormField::FIELD_TYPE_SELECT)
                ->setFieldName('Pojistovna', $tableName)->setCaption('pojistovna')
                ->setCode("field-$code-pojistovna")
                ->setFieldId("zmena_{$code}_pojistovna")
                ->setRequired(true)
                ->setSelectSql(
                    'SELECT Kod, Popis FROM Pojistovna WHERE (%FILTER%) ORDER BY %ORDER%',
                    '1 = 1', 'Popis')
                ->setSelectShowCode(true)
                ->addFieldCssClass('ronselect');
        } else {
            $this->addField(FormField::FIELD_TYPE_SELECT)
                ->setFieldName('PenzijniFond', $tableName)->setCaption('Penzijni_fond')
                ->setCode("field-$code-penzijnifond")
                ->setFieldId("zmena_{$code}_penzijnifond")
                ->setRequired(true)
                ->setSelectSql(
                    'SELECT Kod, Popis FROM PenzijniFond WHERE (%FILTER%) ORDER BY %ORDER%',
                    '1 = 1', 'Popis')
                ->setSelectShowCode(true)
                ->addFieldCssClass('ronselect');
        }

        $this ->addField()
            ->setFieldName('CisloSmlouvy', $tableName) ->setCaption('Cislo_smlouvy')
            ->setCode("field-{$code}-cislosmlouvy")
            ->setFieldId("zmena_{$code}_cislosmlouvy")
            ->setMaxLength(15)
            ->setRequired(true);

        // Prispevek zamestnavatele
        $this ->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('PrispevekZamestnavatele', $tableName) ->setCaption('prispevek_zamestnavatele')
            ->setFormat('number_2')
            ->setCode("field-{$code}-prispevekzamestnavatele")
            ->setFieldId("zmena_{$code}_prispevekzamestnavatele");

        $paramsUcet1 = $this ->database ->createParameters();
        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('SerioveCisloUctu1', $tableName) ->setCaption('Bankovni_ucet')
            ->setCode("field-$code-seriovecislouctu1")
            ->setFieldId("zmena_{$code}_seriovecislouctu1")
            ->setSelectSql(
                new SqlStatement(
                    'SELECT SerioveCislo, JmenoUctu FROM BankovniUcet' .
                    ' WHERE BankovniUcet.RC = ' . $paramsUcet1->addParameter($this ->pp ->rc) .
                    ' AND (%FILTER%)' .
                    ' ORDER BY %ORDER%', $paramsUcet1
                ),
                '1 = 1', 'JmenoUctu')
            ->setDataType('int');
            //->addFieldCssClass('ronselect');

        // Prispevek zamestnance
        $this ->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('PrispevekZamestnance', $tableName) ->setCaption('prispevek_zamestnance')
            ->setFormat('number_2')
            ->setCode("field-{$code}-prispevekzamestnance")
            ->setFieldId("zmena_{$code}_prispevekzamestnance");

        $paramsUcet2 = $this ->database ->createParameters();
        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('SerioveCisloUctu2', $tableName) ->setCaption('Bankovni_ucet')
            ->setCode("field-$code-seriovecislouctu2")
            ->setFieldId("zmena_{$code}_seriovecislouctu2")
            ->setSelectSql(
                new SqlStatement(
                    'SELECT SerioveCislo, JmenoUctu FROM BankovniUcet' .
                    ' WHERE BankovniUcet.RC = ' . $paramsUcet2->addParameter($this ->pp ->rc) .
                    ' AND (%FILTER%)' .
                    ' ORDER BY %ORDER%', $paramsUcet2
                ),
                '1 = 1', 'JmenoUctu')
            ->setDataType('int');
            //->addFieldCssClass('ronselect');

        // zivotni:
        // Pojistovna:Pojistovna:pojistovna:Pr;D:PlatnostOd:Platnost_od:Pr;D:PlatnostDo:Platnost_do;
        // E:CisloSmlouvy:Cislo_smlouvy;E:PrispevekZamestnavatele:prispevek_zamestnavatele;BF:SerioveCisloUctu1:Bankovni_ucet;E:PrispevekZamestnance:prispevek_zamestnance;
        // BZ:SerioveCisloUctu2:Bankovni_ucet;soubor:soubor:Pripojit_dokument
    }


    function addFieldsVzdelani()
    {
        $tableName = 'VzdelaniOsoby';
        $code = 'vzdelani';

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('TypVzdelani', $tableName)->setCaption('Typ_vzdelani')
            ->setCode("field-$code-kodvzdelani")
            ->setFieldId("zmena_{$code}_kodvzdelani")
            ->setRequired(true)
            ->setSelectSql(
                'SELECT Kod, Popis FROM TypVzdelani WHERE (%FILTER%) ORDER BY %ORDER%',
                '1 = 1', 'Popis')
            ->setDataType('int');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('OborVzdelani', $tableName)->setCaption('Obor_vzdelani')
            ->setCode("field-$code-oborvzdelani")
            ->setFieldId("zmena_{$code}_oborvzdelani")
            ->setRequired(true)
            ->setSelectSql(
                'SELECT Kod, Popis FROM OborVzdelani WHERE (%FILTER%) ORDER BY %ORDER%',
                '1 = 1', 'Popis')
            ->addFieldCssClass('ronselect');

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumUkonceni', $tableName)->setCaption('Datum_ukonceni')
            ->setCode("field-$code-datumukonceni")
            ->setFieldId("zmena_{$code}_datumukonceni")
            ->setRequired(true);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Skola', $tableName)->setCaption('Skola')
            ->setCode("field-$code-skola")
            ->setFieldId("zmena_{$code}_skola")
            ->setSelectSql(
                'SELECT Kod, Nazev FROM Skola WHERE (%FILTER%) ORDER BY %ORDER%',
                '1 = 1', 'Nazev')
            ->addFieldCssClass('ronselect');

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumOd', $tableName)->setCaption('Datum_od')
            ->setCode("field-$code-datumod")
            ->setFieldId("zmena_{$code}_datumod");

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumDo', $tableName)->setCaption('Datum_do')
            ->setCode("field-$code-datumdo")
            ->setFieldId("zmena_{$code}_datumdo");

        $this->addField()
            ->setFieldName('Zamereni', $tableName)->setCaption('Zamereni')
            ->setCode("field-$code-zamereni")
            ->setFieldId("zmena_{$code}_zamereni")
            ->setMaxLength(40);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('VzdelaniUkonceno', $tableName)->setCaption('Vzdelani_ukonceno')
            ->setCode("field-$code-vzdelaniukonceno")
            ->setFieldId("zmena_{$code}_vzdelaniukonceno")
            ->setSelectSql(
                'SELECT Kod, Popis FROM S_ZpusobUkonceniSkoly WHERE (%FILTER%) ORDER BY %ORDER%',
                '1 = 1', 'Popis')
            ->setRequired(true)
            ->setDefaultValue("0")
            ->setUseDefaultValueToEditNull(true)
            ->setDataType('int');

        // TypVzdelani:TypVzdelani:Typ_vzdelani:Pr;OborVzdelani:OborVzdelani:Obor_vzdelani:Pr;D:DatumUkonceni:Datum_ukonceni:Po;
        // Skola:Skola:Skola;D:DatumOd:Datum_od;D:DatumDo:Datum_do;E:Zamereni:Zamereni;S_ZpusobUkonceniSkoly:VzdelaniUkonceno:Vzdelani_ukonceno:Po;
        // soubor:soubor:Pripojit_dokument
    }

    function addFieldsOsobni()
    {
        $this ->sqlTableName = $tableName = 'Osoba';
        $code = 'osobni';

        $this->addField()
            ->setFieldName('Titul', $tableName)->setCaption('Titul')
            ->setCode("field-$code-titul")
            ->setFieldId("zmena_{$code}_titul")
            ->setMaxLength(20);

        $this->addField()
            ->setFieldName('Jmeno', $tableName)->setCaption('Jmeno')
            ->setCode("field-$code-jmeno")
            ->setFieldId("zmena_{$code}_jmeno")
            ->setMaxLength(30)
            ->setRequired(true);

        $this->addField()
            ->setFieldName('Prijmeni', $tableName)->setCaption('Prijmeni')
            ->setCode("field-$code-prijmeni")
            ->setFieldId("zmena_{$code}_prijmeni")
            ->setMaxLength(30)
            ->setRequired(true);

        $this->addField()
            ->setFieldName('TitulZa', $tableName)->setCaption('Titul_za')
            ->setCode("field-$code-titulza")
            ->setFieldId("zmena_{$code}_titulza")
            ->setMaxLength(20);

        $this->addField()
            ->setFieldName('Email', $tableName)->setCaption('E_mail')
            ->setCode("field-$code-email")
            ->setFieldId("zmena_{$code}_email")
            ->setMaxLength(60);

        $this->addField()
            ->setFieldName('Telefon', $tableName)->setCaption('Telefon')
            ->setCode("field-$code-telefon")
            ->setFieldId("zmena_{$code}_telefon")
            ->setMaxLength(60);

        $this->addField()
            ->setFieldName('Mobil', $tableName)->setCaption('Mobil')
            ->setCode("field-$code-mobil")
            ->setFieldId("zmena_{$code}_mobil")
            ->setMaxLength(20);

        // E:Titul:Titul_pred;E:TitulZa:Titul_za;E:Jmeno:Jmeno;E:Prijmeni:Prijmeni:Po;E:Email:E_mail;E:Telefon:Telefon;E:Mobil:Mobil;soubor:soubor:Pripojit_dokument
    }


    function addFieldsRodinnyPrislusnik($novyClenRodiny)
    {
        $tableName = 'RodinnyPrislusnik'; // POZOR! Nektera pole jsou z tabulky ClenoveRodiny
        $code = $novyClenRodiny ? 'novyclenrodiny' : 'rodinnyprislusnik';
        $this ->sqlTableName =
            'RodinnyPrislusnik INNER JOIN ClenoveRodiny' .
            ' ON RodinnyPrislusnik.RC_RodinnehoPrislusnika = ClenoveRodiny.RC_RodinnehoPrislusnika';

        // zakladni udaje
        $this->addField()
            ->setFieldName('Titul', $tableName)->setCaption('Titul')
            ->setCode("field-$code-titul")
            ->setFieldId("zmena_{$code}_titul")
            ->setMaxLength(20);

        $this->addField()
            ->setFieldName('Jmeno', $tableName)->setCaption('Jmeno')
            ->setCode("field-$code-jmeno")
            ->setFieldId("zmena_{$code}_jmeno")
            ->setMaxLength(30)
            ->setRequired(true);

        $this->addField()
            ->setFieldName('Prijmeni', $tableName)->setCaption('Prijmeni')
            ->setCode("field-$code-prijmeni")
            ->setFieldId("zmena_{$code}_prijmeni")
            ->setMaxLength(30)
            ->setRequired(true);

        $this->addField()
            ->setFieldName('RC_RodinnehoPrislusnika', $tableName)->setCaption('RC_rodinneho_prislusnika')
            ->setSqlFieldName('RodneCisloRP')
            ->setCode("field-$code-rc_rodinnehoprislusnika")
            ->setFieldId("zmena_{$code}_rc_rodinnehoprislusnika")
            ->setMaxLength(15)
            ->setRequired(true);

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumNarozeni', $tableName)->setCaption('Datum_narozeni')
            ->setCode("field-$code-datumnarozeni")
            ->setFieldId("zmena_{$code}_datumnarozeni")
            ->setRequired(true);

        // rodinny vztah
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('RodinnyVztah', 'ClenoveRodiny')->setCaption('rodinny_vztah')
            ->setCode("field-$code-rodinnyvztah")
            ->setFieldId("zmena_{$code}_rodinnyvztah")
            ->setSelectSql(
                'SELECT Kod, Popis FROM S_RodinnyVztah WHERE (%FILTER%) ORDER BY %ORDER%',
                '1 = 1', 'Popis')
            ->setDataType('int');

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('PartnerDatumOd', 'ClenoveRodiny')->setCaption('Registrace_od')
            ->setCode("field-$code-partnerdatumod")
            ->setFieldId("zmena_{$code}_partnerdatumod");

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('PartnerDatumDo', 'ClenoveRodiny')->setCaption('Registrace_do')
            ->setCode("field-$code-partnerdatumdo")
            ->setFieldId("zmena_{$code}_partnerdatumdo");

        // zdravotni stav
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('ZdravotniStav', $tableName)->setCaption('Zdravotni_stav')
            ->setCode("field-$code-zdravotnistav")
            ->setFieldId("zmena_{$code}_zdravotnistav")
            ->setSelectSql(
                'SELECT Kod, Popis FROM S_ZdravotniStav WHERE (%FILTER%) ORDER BY %ORDER%',
                '1 = 1', 'Popis')
            ->setRequired(true)
            ->setDataType('int');

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('DrzitelPrukazuZTTP', $tableName)->setCaption('Drzitel_prukazu_ZTP')
            ->setCode("field-$code-drzitelprukazuzttp")
            ->setFieldId("zmena_{$code}_drzitelprukazuzttp")
            ->setRequiredInDatabase(true);

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumPriznaniZTTP', $tableName)->setCaption('Datum_priznani_ZTP')
            ->setCode("field-$code-datumpriznanizttp")
            ->setFieldId("zmena_{$code}_datumpriznanizttp")
            ->setRowId('row_zmena_datumpriznanizttp');

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('VyzivovanaOsoba', 'ClenoveRodiny')->setCaption('vyzivovana_osoba')
            ->setCode("field-$code-vyzivovanaosoba")
            ->setFieldId("zmena_{$code}_vyzivovanaosoba");

        // kontaktni udaje
        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('StejnaAdresa', 'ClenoveRodiny')->setCaption('Stejna_adresa_jako_zamestnanec')
            ->setCode("field-$code-stejnaadresa")
            ->setFieldId("zmena_{$code}_stejnaadresa");

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('KontaktniOsoba', $tableName)->setCaption('kontaktni_osoba')
            ->setCode("field-$code-kontaktniosoba")
            ->setFieldId("zmena_{$code}_kontaktniosoba");

        $this ->addField()
            ->setFieldName('Telefon', $tableName) ->setCaption('Telefon')
            ->setMaxLength(60)
            ->setCode("field-{$code}-telefon")
            ->setFieldId("zmena_{$code}_telefon");

        $this ->addField()
            ->setFieldName('Email', $tableName) ->setCaption('E_mail')
            ->setMaxLength(60)
            ->setCode("field-{$code}-email")
            ->setFieldId("zmena_{$code}_email");

        if (!$novyClenRodiny) {
            $this->addField(FormField::FIELD_TYPE_HIDDEN)
                ->setFieldName('ID')
                ->setDefaultValue($this->nahlasenaZmena->hodnoty['ID'] ?? $_GET['idrodinnehoprislusnika'] ?? '')
                ->setSqlFieldName('RC_RodinnehoPrislusnika', $tableName)
                ->setReadOnly(true);
        }

        // rodinnyprislusnik:
        // E:Prijmeni:Prijmeni:Po;E:Jmeno:Jmeno:Po;E:RC_RodinnehoPrislusnika:RC_rodinneho_prislusnika:Pr;E:Titul:Titul;
        // D:DatumNarozeni:Datum_narozeni:Po;S_RodinnyVztah:RodinnyVztah:rodinny_vztah;D:PartnerDatumOd:Registrace_od;D:PartnerDatumDo:Registrace_do;
        // S_ZdravotniStav:ZdravotniStav:Zdravotni_stav:Po;C:DrzitelPrukazuZTTP:Drzitel_prukazu_ZTP;D:DatumPriznaniZTTP:Datum_priznani_ZTP;
        // C:VyzivovanaOsoba:vyzivovana_osoba;C:StejnaAdresa:Stejna_adresa_jako_zamestnanec;C:KontaktniOsoba:kontaktni_osoba;E:Telefon:Telefon;E:Email:E_mail;ID:ID:ID;
        // soubor:soubor:Pripojit_dokument

        // novyclenrodiny:
        // E:Prijmeni:Prijmeni:Po;E:Jmeno:Jmeno:Po;E:RC_RodinnehoPrislusnika:RC_rodinneho_prislusnika:Pr;E:Titul:Titul;
        // D:DatumNarozeni:Datum_narozeni:Po;S_RodinnyVztah:RodinnyVztah:rodinny_vztah;D:PartnerDatumOd:Registrace_od;D:PartnerDatumDo:Registrace_do;
        // S_ZdravotniStav:ZdravotniStav:Zdravotni_stav:Po;C:DrzitelPrukazuZTTP:Drzitel_prukazu_ZTP;D:DatumPriznaniZTTP:Datum_priznani_ZTP;
        // C:VyzivovanaOsoba:vyzivovana_osoba;C:StejnaAdresa:Stejna_adresa_jako_zamestnanec;C:KontaktniOsoba:kontaktni_osoba;E:Telefon:Telefon;E:Email:E_mail;ID:ID:ID;
        // soubor:soubor:Pripojit_dokument
    }


    function addFieldsPrukaz($novyPrukaz)
    {
        $tableName = 'Prukaz';
        $code = 'prukaz';
        $this ->sqlTableName =
            'Prukaz';

        // Typ prukazu
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('TypPrukazu', $tableName)->setCaption('Typ_prukazu')
            ->setCode("field-$code-typprukazu")
            ->setFieldId("zmena_{$code}_typprukazu")
            ->setSelectSql(
                'SELECT Kod, Popis FROM TypPrukazu WHERE (%FILTER%) ORDER BY %ORDER%',
                '1 = 1', 'Popis')
            ->setRequired(true)
            ->setDataType('int');

        // Cislo
        $this->addField()
            ->setFieldName('CisloPrukazu', $tableName)->setCaption('Cislo_prukazu')
            ->setCode("field-$code-cisloprukazu")
            ->setFieldId("zmena_{$code}_cisloprukazu")
            ->setMaxLength(20)
            ->setRequired(true);

        // Vydal stat
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('VydalStat', $tableName)->setCaption('Stat')
            ->setCode("field-$code-vydalstat")
            ->setFieldId("zmena_{$code}_vydalstat")
            ->setSelectSql(
                'SELECT Kod, Popis FROM Stat WHERE (%FILTER%) ORDER BY %ORDER%',
                '1 = 1', 'Popis')
            ->setPreferredValues(['CZ', 'SK']);

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('PlatnostOd', $tableName)->setCaption('Platnost_od')
            ->setCode("field-$code-platnostod")
            ->setFieldId("zmena_{$code}_platnostod")
            ->setRequired(true); // v DB sice povinny neni, ale ve mzdach ano

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('PlatnostDo', $tableName)->setCaption('Platnost_do')
            ->setCode("field-$code-platnostdo")
            ->setFieldId("zmena_{$code}_platnostdo");

        $this ->addField(FormField::FIELD_TYPE_MEMO) ->setCaption('Poznamka_k_prukazu')
            ->setFieldName('vlastnipoznamka')
            ->setSqlFieldName('Poznamka')
            ->setCode("field-$code-poznamka")
            ->setFieldId("zmena_{$code}_poznamka");

        // TypPrukazu:TypPrukazu:Typ_prukazu:Po;E:CisloPrukazu:Cislo_prukazu:Po;Stat:Stat:Stat;D:PlatnostOd:Platnost_od;
        // D:PlatnostDo:Platnost_do;vlastnipoznamka:vlastnipoznamka:Poznamka_k_prukazu
    }


    function getSqlFromPart()
    {
        switch (strtolower($this ->nahlasenaZmena ->typZmeny)) {
            case 'trvale' :
            case 'prechodne' :
                return 'AdresaOsoby';

            case 'vzdelani' :
                return null;
        }

        return parent::getSqlFromPart();
    }


    function getWhereCondition()
    {
        $params = $this ->database ->createParameters();

        $typZmeny = strtolower($this ->nahlasenaZmena ->typZmeny);
        switch ($typZmeny) {
            case 'osobni' :
                return
                    new SqlStatement(
                        'RC = ' . $params ->addParameter($this ->pp ->rc),
                        $params);

            case 'trvale' :
            case 'prechodne' :
                return
                    new SqlStatement(
                        'RC = ' . $params ->addParameter($this ->pp ->rc) .
                        ' AND TrvaleBydliste = ' . $params ->addParameter($typZmeny === 'trvale' ? 1 : 0) .
                        ' AND (PlatnostDo IS NULL  OR  PlatnostDo >= ' . $params ->addParameter($this ->database ->dateToParam(time())) . ')',
                        $params);

            case 'rodinnystav' :
            case 'pojistovna' :
            case 'penzijnipripojisteni' :
            case 'zivotnipojisteni' :
                return
                    new SqlStatement(
                        'RC = ' . $params ->addParameter($this ->pp ->rc) .
                        ' AND (PlatnostDo IS NULL  OR  PlatnostDo >= ' . $params ->addParameter($this ->database ->dateToParam(time())) . ')',
                        $params);

            case 'ucetosoby' :
                // nacte SerioveCislo aktualniho uctu pro vyplatu
                $paramsUcet = $this ->database ->createParameters();
                $cursor = $this ->database ->getSqlResult(
                    'SELECT VyplaceniMzdy.SerioveCisloUctuVyplaty, VyplaceniMzdy.PlatnostOd FROM VyplaceniMzdy' .
                    ' WHERE RC = ' . $paramsUcet ->addParameter($this ->pp ->rc) .
                    ' ORDER BY PlatnostOd DESC', // nejnovejsi prvni
                    $paramsUcet);
                $row = $cursor ->fetch();
                $cursor ->close();
                return new SqlStatement(
                    $row
                        ? 'VyplaceniMzdy.SerioveCisloUctuVyplaty = ' . $params ->addParameter((int)$row[0]) .
                          ' AND VyplaceniMzdy.PlatnostOd = ' . $params ->addDateParameter($this ->database ->stringToTimestamp($row[1]))
                        : '0 = 1',
                    $params);

            case 'rodinnyprislusnik' :
                $idPrislusnika = $this ->nahlasenaZmena ->hodnoty['ID'] ?? $_GET['idrodinnehoprislusnika'] ?? '';
                // zkontroluje, jestli zadany rodinny prislusnik patri k zadane osobe
                $paramsRP = $this ->database ->createParameters();
                $cursor = $this ->database ->getSqlResult(
                    'SELECT RC_RodinnehoPrislusnika FROM ClenoveRodiny' .
                    ' WHERE RC = ' . $paramsRP ->addParameter($this ->pp ->rc) .
                    ' AND RC_RodinnehoPrislusnika = ' . $paramsRP ->addParameter($idPrislusnika),
                    $paramsRP);
                $row = $cursor ->fetch();
                $cursor ->close();

                if (!$row) {
                    throw new \Exception('Rodinny prislusnik nenalezen!');
                }

                return new SqlStatement(
                    'RodinnyPrislusnik.RC_RodinnehoPrislusnika = ' . $params ->addParameter($idPrislusnika),
                    $params
                );

            case 'novyclenrodiny' :
                return '0 = 1'; // nic nenacita

            case 'prukaz' :
                $cisloPrukazu = $this ->nahlasenaZmena ->hodnoty['CisloPrukazu'] ?? $_GET['cisloprukazu'] ?? '';
                $typPrukazu = (int)($this ->nahlasenaZmena ->hodnoty['TypPrukazu'] ?? $_GET['typprukazu'] ?? 0);
                return new SqlStatement(
                    'RC = ' . $params ->addParameter($this ->pp ->rc) .
                    ' AND TypPrukazu = ' . $params ->addParameter($typPrukazu) .
                    ' AND CisloPrukazu = ' . $params ->addParameter($cisloPrukazu),
                    $params
                );

            case 'vzdelani' :
                // TODO: nacist posledni zaznam vzdelani?
                break;
        }

        return parent::getWhereCondition();
    }


    protected function loadFieldValues($insert, $values, $usePostValues, $canEditRecord)
    {
        parent::loadFieldValues($insert, $values, $usePostValues, $canEditRecord);

        if ($insert  ||  $usePostValues) {
            //parent::loadFieldValues($insert, $values, $usePostValues, $canEditRecord);
            return;
        }

        foreach ($this ->fields as $field) {
            $fieldTableName = $field ->tableAlias ?: $field ->tableName;

            if ($field ->visible) {
                if ($field ->fieldName  &&  $fieldTableName != 'NahlasenaZmena') {
                    $name = StringUtils::strAfterLast($field ->fieldName, '#');

                    $value = $this ->nahlasenaZmena ->hodnoty[$name] ?? '';
                    $field ->value = str_replace(['\r', '\n'], ["\r", "\n"], $value);
                }
            }
        }
    }


    function loadRecord()
    {
        $result = parent::loadRecord();
        return $result ?? [];

       // if ($this ->isInsert()) {
       //     return parent::loadRecord();
       // }

        //return $this ->nahlasenaZmena ->hodnoty;
    }


    function loadOriginalValues()
    {
        $this ->data = new FormRowArray($this ->fields, $this ->loadRecord());
    }


    /**
     * Nacte do asociativniho pole data odeslana uzivatelem
     * @return array
     * @throws \Ron\RonException
     */
    private function sestavDataNahlaseneZmeny()
    {
        $result = [];

        foreach ($this->fields as $field) {
            $fieldTableName = $field ->tableAlias ?: $field ->tableName;

            if ($field ->visible  &&   $field ->fieldName  &&  $fieldTableName != 'NahlasenaZmena') {
                $field ->getPostValue(false); // jen aby probehla kontrola formatu
                $hodnota = $field ->getPostValueText(false); // pouzije text, aby se datum neukladal jako timestamp (int)
                $hodnota = str_replace(["\r", "\n"], ['\r', '\n'], $hodnota); // odstrani konce radku
                $result[StringUtils::strAfterLast($field ->fieldName, '#')] = $hodnota;
            }
        }

        if ($this ->nahlasenaZmena ->typZmeny === 'ucetosoby') {
            $platnostOd = DateUtils::dateStringToTimestamp($result['PlatnostOd']);
            if ($platnostOd < DateUtils::incMonth(DateUtils::firstDayOfMonth(time()))) {
                throw new RonException(TextManager::_('Zmena_bankovniho_uctu_upozorneni'), 'VyplaceniMzdy#PlatnostOd');
            }
        }

        return $result;
    }


    function createDBRecord($tableName = null)
    {
        return new RonPortalDBRecord($this ->database, 'NahlasenaZmena', $this);
    }


    public function setDBRecordWhere($rec)
    {
        $params = $this ->database ->createParameters();
        $rec ->setWhere('Id = ' . $params ->addParameter($this ->nahlasenaZmena ->id), $params);
    }

    function getXmlEventsParams($rec, $context, $eventName)
    {
        $result = parent::getXmlEventsParams($rec, $context, $eventName) ?? [];
        $result['HodnotyZmeny'] = $this ->hodnotyZmeny;

        return $result;
    }


    public function createDBRecordFields($rec, $insert)
    {
        $stav = NahlasenaZmena::STAV_NEZPRACOVANO;

        if (!$this ->isInsert()  &&  $this ->jePravoSchvalitZmenu()) {
            if ('2' === ($_POST['NahlasenaZmena#Stav'] ?? '')) {
                $stav = NahlasenaZmena::STAV_ODMITNUTO; // odmitnuto
            }
            // Zpracovano - nastavi se az po zapisu
        }

        $rec ->addField('Id', 'serial', true);
        $rec ->addStringField('TypZmeny', true, 0, $this ->nahlasenaZmena ->typZmeny);
        $rec ->addStringField('RC', true, 15, $this ->pp ->rc);
        $rec ->addTimestampField('DatumCas', true, time());
        $rec ->addIntField('Stav', false, $stav);

        $this ->hodnotyZmeny = $this ->sestavDataNahlaseneZmeny();
        $dataStr = '';
        foreach ($this ->hodnotyZmeny as $key => $value) {
            $dataStr .= $key . '=' . $value . "\n";
        }

        $rec ->addMemoField('Hodnoty', false, $dataStr);

        $rec ->addStringField('Nahlasil', true, 15, User::getRC());

        if ($stav) {
            $rec ->addStringField('Zpracoval', false, 15, User::getRC());
            $rec ->addTimestampField('DatumCasZpracovani', false, time());

            if ($stav === NahlasenaZmena::STAV_ODMITNUTO  &&  ($_POST['PoznamkaKeSchvaleni'] ?? false)) {
                $rec ->addMemoField('PoznamkaKeSchvaleni', false, $_POST['PoznamkaKeSchvaleni']);
            }
        }

        if ($this ->fPoznamka ->visible) {
            $recField = new DBRecordField('Poznamka', 'memo', $this ->fPoznamka ->readOnly);
            $recField->setValue($this ->fPoznamka->getPostValue($this ->utf8));
            $rec ->addDBRecordField($recField);
        }
    }


    /**
     * @throws DBException
     */
    public function ulozDokumenty($idNahlaseneZmeny)
    {
        foreach ($this ->fields as $field) {
            if ($field ->fieldType === FormField::FIELD_TYPE_FILE  &&  $field ->visible  &&  !$field ->readOnly) {
                if ($_FILES[$field ->fieldName]['name'] ?? null) { // Pozor! Nelze pouzit is_uploaded_file, protoze pak nejsou osetreny chyby!
                    $popis = str_replace(
                        ['%ID%', '%DATE%', '%TIME%', '%TYPZMENY%', '%POPISZMENY%'],
                        [$idNahlaseneZmeny, date('j.n.Y', time()), date('j.n.Y G:i', time()),
                            $this ->nahlasenaZmena ->typZmeny, $this ->nahlasenaZmena ->popisTypuZmeny],
                        ($field instanceof PripojitDokumentZmenyField
                            ? $field ->popisDokumentu
                            : TextManager::_('dokument_nahlasene_zmeny' . ' %ID% (%TIME%)')
                        )
                    );

                    $typDokumentu = $field instanceof PripojitDokumentZmenyField
                        ? $field ->typDokumentu
                        : RonPortalSettings::getIntValue('hlaseni_zmeny_typdokumentu');

                    $idDokumentu = DokumentyUtils::vlozDokument($this->database, $field ->fieldName, $typDokumentu, $popis, $this, $this ->utf8);

                    try {
                        $rec = new RonPortalDBRecord($this->database, 'DokumentZmeny', $this);
                        $rec->addField('Id', 'int', true, 0, $idDokumentu);
                        $rec->addField('IDZmeny', 'int', true, 0, $idNahlaseneZmeny);
                        $rec->addLogText('   ' . $_FILES[$field ->fieldName]['name']);
                        $rec->insertRecord();
                    } catch (\Exception $e) {
                        MessageManager::addErrorMessage($e);
                    }

                }
            }
        }
    }


    function beforeUpdateRecord($rec)
    {
        if (!parent::beforeUpdateRecord($rec)) {
            return false;
        }

        $this ->ulozDokumenty($rec['Id']);
        return true;
    }


    function afterInsertRecord($rec)
    {
        $this ->ulozDokumenty($rec['Id']);
        return parent::afterInsertRecord($rec);
    }


    private function jePravoSchvalitZmenu()
    {
        return NahlasenaZmena::jePravoNaZmenu($this ->nahlasenaZmena ->typZmeny, $this ->nahlasenaZmena ->rc, false, true);
    }

    public function editRecord()
    {
        $result = parent::editRecord();

        // V pripade, ze uzivatel kliknul na "Potrvrdit", zpracuje nahlasenou zmenu
        // (nemuze byt v afterUpdateRecord, protoze zaznam nemusel byt vubec zmenen)
        if ('1' === ($_POST['NahlasenaZmena#Stav'] ?? false)  &&  $this ->jePravoSchvalitZmenu()) {
            NahlasenaZmenaUtils::potvrdZmenu($this ->database, $this ->nahlasenaZmena ->id, $this);
        }

        return $result;
    }


    function getHtmlFormParams()
    {
        return
            parent::getHtmlFormParams() .
            ' data-typzmeny="' . ($this ->nahlasenaZmena ->typZmeny ?? '') . '"';
    }


    public function getButtons($insert)
    {
        $result = parent::getButtons($insert);

        if (!$this ->nahlasenaZmena ->stav  &&  !$this ->isInsert()  &&  $this ->jePravoSchvalitZmenu()) {
            $result['odmitnout'] = [
                'type' => 'button',
                'id' => 'button_odmitnout_zmenu',
                'class' => 'button float-right error-background error-text toggle-class',
                'value' => TextManager::_('odmitnout_zmenu'),
                'data-selector' => '#section_poznamka_ke_schvaleni,#button_odmitnout_zmenu',
                'data-toggle-class' => 'd-none'
            ];
//            $result['odmitnout'] = [
//                'type' => 'button',
//                'name' => 'odmitnout',
//                'class' => 'button float-right error-background error-text setValue',
//                'value' => TextManager::_('odmitnout_zmenu'),
//                'data-selector' => '#nahlasena_zmena_stav',
//                'data-value' => 2,
//                'data-submit' => 1,
//                'data-ron-confirm-text' => TextManager::_('Opravdu_odmitnout_nahlasenou_zmenu')
//            ];

            $result['potvrdit'] = [
                'type' => 'button',
                'name' => 'potvrdit',
                'class' => 'submitbutton float-right setValue',
                'value' => TextManager::_('potvrdit_zmenu'),
                'data-selector' => '#nahlasena_zmena_stav',
                'data-value' => 1,
                'data-submit' => 1,
                'data-ron-confirm-text' => TextManager::_('Opravdu_potvrdit_nahlasenou_zmenu')
            ];
        }

        return $result;
    }

    public function renderButtons($insert)
    {
        parent::renderButtons($insert);

        echo
            '<div id="section_poznamka_ke_schvaleni" class="d-none">' .
            // poznamka
            '<div id="row_poznamka_ke_schvaleni" class="form-row">' .
            '<div class="form-col1 caption">' . TextManager::_('Poznamka_k_zamitnuti') . '</div>' .
            '<div class="form-col2">' .
            '<textarea class="form-control" rows="8" name="PoznamkaKeSchvaleni" wrap="virtual"></textarea>' .
            '</div>' .
            '</div>' .
            // tlacitko
            '<div class="row flex-justify-end">' .
            '<input type="button" name="odmitnout" class="button error-background error-text setValue" value="' .
              TextManager::_('odmitnout_zmenu') . '" data-selector="#nahlasena_zmena_stav" data-value="2" data-submit="1" data-ron-confirm-text="' .
              TextManager::_('Opravdu_odmitnout_nahlasenou_zmenu') . '">' .
            '</div>' .

            "</div>\n"; // section
    }








//    public function insertRecord()
//    {
//        if (!$this->validate(false)) {
//            return false;
//        }
//
//        $rec = $this ->createRecordNahlasenaZmena();
//
//        if (!$this->beforeInsertRecord($rec)) {
//            return false;
//        }
//
//        $rec ->insertRecord();
//
//        return $this->afterInsertRecord($rec);
//    }
//
//
    public function render()
    {
        parent::render();

        // vygeneruje formular s aktualnimi hodnotami
        echo
            '<h2>' . TextManager::_('aktualni_udaje') . '</h2>';

        if ($this ->data ->count()) {
            echo
                '<div class="form large-form">' .
                '<div class="form-content">';

            foreach ($this->fields as $field) {
                if ($field->visible &&
                    $field->fieldType !== FormField::FIELD_TYPE_HIDDEN && $field->fieldType !== FormField::FIELD_TYPE_CONSTANT &&
                    $field instanceof NahlasenaZmenaFormField &&
                    $field ->zobrazitVHistorii) {
                    echo
                        '<div class="row">' .
                        '<div class="col-4 caption">' . ($field->caption ?? '') . '</div>' .
                        '<div class="col-8">' . ($field->getZmenaFieldTextForValue($this->database, $field->existingValue ?? '')) . '</div>' .
                        '</div>';
                }
            }

            echo
                '</div>' . // form-content
                '</div>'; // form
        } else {
            echo '<div class="form large-form padding-all-2">' . TextManager::_('Osoba_jeste_nema_zadane_udaje') . '</div>';
        }
    }


    function handleAction()
    {
        //if ($this ->formAction == 'new') {
            $this ->loadOriginalValues();
        //}

        return parent::handleAction();
    }
}
