<?php
/**
 * stanislav.elbl, 20.09.2015
 * roman.szablatura 10.10.2017
 */

namespace RonPortal\Zamestnanci;


use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\HttpController;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\TextViewObject;
use Ron\MVC\ViewObject;
use Ron\TextManager;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\NeniPravoException;
use RonPortal\Personalistika\DokumentyNahlaseneZmenyBrowser;
use RonPortal\Prava;


/**
 * Controller zpracovava requesty tykajice hlaseni zmen
 * - zobrazeni nahlasenych zmen
 * - potvrzeni
 * - download dokumentu
 */
class NahlaseneZmenyController extends HttpController
{
    /** @var Database */
    var $database;

    /**
     * HlaseniZmenyController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
        Prava::nastavObdobiProPrava(time(), time());
    }


    /**
     * @param NahlasenaZmenaForm $form
     * @return ViewObject
     */
    function renderFormNahlasenaZmena($form, $idZmeny)
    {
        Model::addViewObject($form);

        if ($idZmeny) {
            // jedna se o editaci => zobrazi pripojene dokumenty
            $b = new DokumentyNahlaseneZmenyBrowser($this->database, $idZmeny, !$form ->getNahlasenaZmena() ->stav);
            $b ->load();

            Model::addViewObject($b);
        }

        return new TextViewObject('', null);
    }


    /**
     * Nahlaseni nove zmeny - v pripade potreby nacte data z prislusne agendy a zobrazi formular
     * @return Redirect|ViewObject
     * @throws DBException
     * @throws \Exception
     */
    function hlaseniZmeny()
    {
        $form = new NahlasenaZmenaForm($this ->database);
        if (!$form ->handleAction()) {
            return $this ->renderFormNahlasenaZmena($form, null);
        }

        return $this ->redirect();
    }


    function editaceZmeny($idZmeny)
    {
        $form = new NahlasenaZmenaForm($this ->database);

        if (!$form ->handleAction()) {
            return $this ->renderFormNahlasenaZmena($form, $idZmeny);
        }

        return $this ->redirect();
    }

    function redirect()
    {
        switch ($_GET['back'] ?? '') {
            case 'osobniudaje' :
                $menuId = $_GET['back'];
                break;

            default :
                $menuId = 'nahlasenezmeny';
        }

        return new Redirect(null, "?menuid=$menuId");
    }


    /**
     * @param $idZmeny
     * @param $idDokumentu
     * @param $editace
     * @return void
     * @throws \Exception
     */
    private function zkontrolujPravoNaDokument($idZmeny, $idDokumentu, $editace)
    {
        // Kontrola, zdali existuje u dotycne zmeny prohlizeny dokument a uzivatel na nej ma pravo
        $params = $this->database->createParameters();
        $sql =
            'SELECT DokumentZmeny.ID, NahlasenaZmena.TypZmeny, NahlasenaZmena.RC, NahlasenaZmena.Stav' .
            ' FROM DokumentZmeny JOIN NahlasenaZmena ON (DokumentZmeny.IDZmeny = NahlasenaZmena.ID)' .
            ' WHERE DokumentZmeny.IDZmeny = ' . $params->addParameter($idZmeny) .
            ' AND DokumentZmeny.ID = ' . $params->addParameter($idDokumentu);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new NeniPravoException();
        }

        if (!NahlasenaZmena::jePravoNaZmenu($row[1], $row[2], $editace)) {
            throw new NeniPravoException();
        }

//        if ($row[3]) {  - to tu je zbytecne
//            throw new \Exception('Nahlasena zmena je jiz zpracovana!');
//        }
    }


    private function download($idZmeny, $idDokumentu)
    {
        $this ->zkontrolujPravoNaDokument($idZmeny, $idDokumentu, false);
        return new DokumentOutputObject($this ->database, $idDokumentu);
    }


    /**
     * @param $idZmeny
     * @param $idDokumentu
     * @return Redirect
     * @throws \Exception
     */
    private function smazPripojenyDokument($idZmeny, $idDokumentu)
    {
        $this ->zkontrolujPravoNaDokument($idZmeny, $idDokumentu, true);
        //Kontrola prava na zmenu - pokud nemam, neni treba dale kontrolovat???

        try {
            $params = $this ->database ->createParameters();
            DokumentyUtils::smazDokument($this ->database,
                $idDokumentu, $this, 'DokumentZmeny',
                'IdZmeny = ' . $params ->addParameter($idZmeny), $params);

            MessageManager::addInfoMessage(\Ron\ronText('odstraneni_dokumentu_zmeny_ok'));
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e->getMessage());
        }

        return new Redirect(null, '?menuid=nahlasenezmeny&record=' . $idZmeny );
    }


//    /**
//     * @return DokumentUploadOutput
//     */
//    function uploadDokument()
//    {
//        $result = new DokumentUploadOutput();
//
//        try {
//            $idZmeny = (int)$_POST['IDZmeny'];
//
//            NahlasenaZmena::zkontrolujPravoNaZapsanouZmenu($this ->database, $idZmeny, true);
//
//            // Vlozi dokument do databaze
//            $id = DokumentyUtils::vlozDokument($this ->database, 'soubor', $_POST['typ'], $_POST['popis'], $this);
//
//            // Vlozi vazbu na nahlasenou zmenu
//            $rec = new RonPortalDBRecord($this ->database, 'DokumentZmeny', $this);
//            $rec->addIntField('Id', true, $id);
//            $rec->addIntField('IdZmeny', true, (int)$idZmeny);
//
//            $rec->insertRecord();
//        } catch (\Exception $e) {
//            $result ->setError($e ->getMessage());
//        }
//
//        return $result;
//    }


    /**
     * @param string $action
     * @return Redirect|ViewObject|OutputObject
     * @throws \Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance()
            ->loadStrings('person.ini')
            ->loadStrings('nahlasenezmeny.ini') // TODO: presunout do zamestnanci.ini a odstranit?
            ->loadStrings('zamestnanci.ini');

        HtmlHeaders::getInstance()
            ->setTitle( TextManager::_('RON_Portal') . ' - ' . TextManager::_('nahlasena_zmena'));

        $this ->addTranslationsScript('nahlasenezmeny');

        $hlaseniAction = isset($_GET['action']) ? $_GET['action'] : '';

        switch ($hlaseniAction) {
            case 'translations':
                return $this->translations();

            case 'dokumentdownload':
                return $this->download((int)($_GET['id'] ?? 0), (int)($_GET['record'] ?? 0));

            case 'dokumentdelete' :
                return $this->smazPripojenyDokument((int)$_GET['id'], (int)$_GET['record']);
        }

        if (isset($_GET['record']) && (int)$_GET['record']) {
            return $this ->editaceZmeny((int)$_GET['record']);
        }

        if (isset($_GET['typzmeny'])) {
            // hlaseni nove zmeny - zobrazeni formulare
            return $this->hlaseniZmeny();
        }

        // seznam nahlasenych zmen
        $b= new NahlaseneZmenyBrowser($this->database);
        $b->load();
        return $b;
    }
}
