<?php
/**
 * stanislav.elbl, 14.03.2021
 */

namespace RonPortal\Zamestnanci;


use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\SQLUtils;
use RonPortal\PracovniPomer;


class ZamestnanciUtils
{
    const PID_GENEROVAT_AUTOMATICKY = 0;
    const PID_GENEROVAT_A_EDITOVAT = 1;
    const PID_ZADAVAT_MANUALNE = 2;
    /**
     * Nacte nastaveni generovani RC ze sys_var_global, pokud je nenajde, nastavi implicitni hodnoty. Klice jsou: kopirovatIdOsoby, typ, maska, minimum, maximum
     * @param Database $database
     * @return array
     * @throws \Exception
     */
    static function getIDOsobySettings($database)
    {
        // Default type = 2? (manualni zadavani), maska #####, min = 1, max = 99999
        $result = [
            'kopirovatIdOsoby' => 0,
            'typ' => self::PID_GENEROVAT_AUTOMATICKY,
            'maska' => '#####',
            'minimum' => 1,
            'maximum' => 99999
        ];

//        $kopirovatIdOsoby = 0;
//        $type = 0;
//        $mask = "#####";
//        $min = 1;
//        $max = 99999;

        $sql =
            'SELECT Name, Val' .
            ' FROM SYS_VAR_GLOBAL' .
            " WHERE Name IN ('PID_AutoCopyToBN', 'PID_NumType', 'PID_Mask', 'PID_MinValue', 'PID_MaxValue')";

        $cursor = $database->getSqlResult($sql);

        //Vyhledat nastavene hodnoty (upravene uzivatelem v nastaveni aplikace)
        while ($row = $cursor->fetch()) {
            switch ($row[0]) {
                case 'PID_AutoCopyToBN':
                    $result['kopirovatIdOsoby'] = $row[1];
                    break;

                case 'PID_NumType':
                    $result['typ'] = (int)$row[1];
                    break;

                case 'PID_Mask':
                    if ($row[1] != '') $result['maska'] = $row[1];
                    break;

                case 'PID_MinValue':
                    $result['minimum'] = (int)$row[1];
                    break;

                case 'PID_MaxValue':
                    $result['maximum'] = (int)$row[1];
                    break;

            }
        }

        $cursor->close();

        return $result;
    }


    /**
     * Vraci nove ID osoby (Osoba.RC) podle nastaveni z aplikace
     * @param $database
     * @return bool|string
     * @throws \Exception
     */
    public static function getIdOsoby($database)
    {
        $nastaveniGenerovaniID = self::getIDOsobySettings($database);

        if ($nastaveniGenerovaniID['typ'] !== self::PID_ZADAVAT_MANUALNE) {  // 2 = mamualni zadani RC (ID) osoby
            return SQLUtils::getNextRecordSerialCode($database,
                'Osoba', 'RC',
                $nastaveniGenerovaniID['maska'],
                $nastaveniGenerovaniID['minimum'], $nastaveniGenerovaniID['maximum']);
        }

        return null;
    }


    /**
     * Vrati nove osobni cislo podle zadane ciselne rady.
     * @param string $pocatecniOsCislo  osobni cisla cislovat od
     * @param string $maxPoradi         posledni osobni cislo
     * @param int $delkaOsCisla         pozadovana delka osobniho cisla
     * @return string
     */
    protected static function getNextOsobniCislo($pocatecniOsCislo, $maxPoradi, $delkaOsCisla)
    {
        if (is_null($maxPoradi) || ($maxPoradi == '0')) {
            $result = $pocatecniOsCislo;
        } else {
            try {

                $pomOsCislo = '';
                $result = '';

                $pomString = $maxPoradi;

                $pomStringArray = str_split($pomString);

                for ($index = 0; $index < count($pomStringArray); $index++) {
                    if (($pomStringArray[$index] >= '0') && ($pomStringArray[$index] <= '9')) {
                        $pomOsCislo .= $pomStringArray[$index];
                    }
                }

                $index2 = strlen($pomOsCislo);

                $pomOsCislo = (int)$pomOsCislo + 1;
                $pomOsCislo = str_pad($pomOsCislo, $index2, '0', STR_PAD_LEFT);

                $index2 = 0;

                $pomOsCisloArray = str_split($pomOsCislo);

                for ($index = 0; $index <= max(count($pomStringArray), count($pomOsCisloArray)); $index++) {
                    if ($index > count($pomStringArray)) {
                        $result .= $pomOsCisloArray[$index2];
                        $index2++;
                    } elseif (($pomStringArray[$index] >= '0') && ($pomStringArray[$index] <= '9')) {
                        $result .= $pomOsCisloArray[$index2];
                        $index2++;
                    } else {
                        $result .= $pomStringArray[$index];
                    }
                }

            } catch (\Exception $e) {
                $result = '0';
            }

        }

        $result = str_pad($result, $delkaOsCisla, '0', STR_PAD_LEFT);

        return $result;
    }


    /** Vrati nove osobni cislo podle nastavene rady u typu pracovniho pomeru.
     * @param Database $database
     * @param string $kodTypuPP
     * @return string
     * @throws \Exception
     */
    public static function getNextOsobniCisloTypuPP($database, $kodTypuPP)
    {
        if ($kodTypuPP == '') {
            return null;
        }

        $params = $database->createParameters();

        $sql =
            'SELECT OsobniCislaOd, DelkaOsobnihoCisla' .
            ' FROM TypPracovnihoPomeru' .
            ' WHERE Kod = ' . $params->addParameter($kodTypuPP);

        $cursor = $database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        $pomString = $row[0];
        $delkaOsCisla = $row[1];

        $params = $database->createParameters();

        $maxPoradi = DBUtils::getSqlValue($database,
            'SELECT MAX(PracovniPomer.OsobniCislo) AS MaxPoradi' .
            ' FROM PracovniPomer, TypPracovnihoPomeru' .
            ' WHERE PracovniPomer.TypPracovnihoPomeru = TypPracovnihoPomeru.Kod' .
            '  AND TypPracovnihoPomeru.OsobniCislaOd =  ' . $params->addParameter($pomString),
            $params);
        //21.12.2016 RON-PR do rady osobnich cisel se berou vsechny typy osob
        //"  AND (PracovniPomer.Zamestnanec <> 101)");

        return self::getNextOsobniCislo($pomString, $maxPoradi, $delkaOsCisla);
    }


    /**
     * Vrati nove osobni cislo pro typ osoby.
     * @param Database $database
     * @param int $typOsoby
     * @param string $pocatecniOsCislo
     * @param int $delkaOsCisla
     * @return string
     * @throws \Exception
     */
    public static function getNextOsobniCisloTypOsoby($database, $typOsoby, $pocatecniOsCislo, $delkaOsCisla)
    {
        $result = '';

        if ($typOsoby < 0) {
            return $result;
        }

        $params = $database->createParameters();

        $maxPoradi = DBUtils::getSqlValue($database,
            'SELECT MAX(PracovniPomer.OsobniCislo) AS MaxPoradi' .
            ' FROM PracovniPomer' .
            ' WHERE Zamestnanec =  ' . $params->addParameter($typOsoby),
            $params);

        return self::getNextOsobniCislo($pocatecniOsCislo, $maxPoradi, $delkaOsCisla);
    }


    /**
     * Vraci pocet pracovnich pomeru zadane osoby
     * @param Database $database
     * @param string $rc
     * @return int
     */
    public static function pocetPPOsoby($database, $rc)
    {
        $params = $database ->createParameters();
        return (int)DBUtils::getSqlValue($database,
            'SELECT COUNT(*) FROM PracovniPomer WHERE RC = ' . $params ->addParameter($rc),
            $params);
    }

    /**
     * Vraci seriove cislo uctu se zadanym cislem (cisloUctu, kodBanky) prirazeneho zadane osobe (podle Osoba.rc)
     * Pokud neexistuje, vraci null.
     * Pouziva se v nastupnim formulari, aby se zbytecne nevytvarel novy zaznam pro stejny ucet.
     * @param Database $database
     * @param string $rc
     * @param string $cisloUctu
     * @param string $kodBanky
     * @return int|null
     */
    public static function najdiBankovniUcetOsoby($database, $rc, $cisloUctu, $kodBanky)
    {
        if (!$rc) {
            return null;
        }

        $params = $database ->createParameters();

        $cursor = $database ->getSqlResult(
            'SELECT SerioveCislo FROM BankovniUcet' .
            ' WHERE CisloUctu = ' . $params ->addParameter(trim($cisloUctu)) .
            ' AND KodBanky = ' . $params ->addParameter(trim($kodBanky)) .
            ' AND TypUctu = 1' . // pro jistotu - aby nesel editovat ucet organizace
            ' AND RC = ' . $params ->addParameter($rc) .
            ' ORDER BY SerioveCislo',
            $params
        );

        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? (int)$row[0] : null;
    }

    /**
     * Vraci seriove cislo uctu prirazeneho zadanemu pp
     * Pokud neexistuje, vraci null.
     * @param Database $database
     * @param PracovniPomer $pp
     * @return int|null
     */
    public static function najdiBankovniUcetPP($database, $pp)
    {
        $params = $database ->createParameters();

        $cursor = $database ->getSqlResult(
            'SELECT SerioveCisloUctuVyplaty FROM VyplaceniMzdy' .
            ' WHERE RC = ' . $params ->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addDateParameter($pp ->datumNastupu) .
            ' ORDER BY PlatnostOd',
            $params
        );

        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? (int)$row[0] : null;
    }


    /**
     * Vraci, kolikrat je ucet se zadanym seriovym cislem pouzit u pracovnich pomeru
     * @param Database $database
     * @param int $serioveCislo
     * @return int
     */
    public static function pocetPouzitiBankovnihoUctu($database, $serioveCislo)
    {
        $params = $database ->createParameters();

        return DBUtils::getSqlValue($database,
            'SELECT COUNT(*) FROM VyplaceniMzdy' .
            ' WHERE SerioveCisloUctuVyplaty = ' . $params ->addParameter($serioveCislo) .
            '  OR SerioveCisloUctuZalohy = ' . $params ->addParameter($serioveCislo),
            $params);
    }

}
