

(function ($) {
    let originalJQueryShow = $.fn.show;

    $.fn.extend({
        show: function () {
            this.removeClass('d-none');
            return originalJQueryShow.apply(this, arguments);
        }
    }
    );
    $.fn.extend({
        showModal: function () {
            $element = this.first("dialog");
            if ($element.length) {
                $element.get(0).showModal();
            }
        },

        close: function () {
            $element = this.first("dialog");
            if ($element.length) {
                $element.get(0).close();
            }
        }
    });
})(jQuery);


var gFiles = {
    useBootstrap: false,

    debugMode: false,
    debugLog: function () {
        if (gFiles.debugMode) {
            console.log.call(this, ...arguments);
        }
    },
    debugAlert: function (text) {
        if (gFiles.debugMode) {
            alert(text);
        }
    },


    _: function (text) { 
        return text;
    },

    utils: { }
};

$.extend(gFiles.utils, {
    today: function () {
        let datetime = new Date();
        return new Date(datetime.getFullYear(), datetime.getMonth(), datetime.getDate());
    },
    incMonth: function (date, cnt) {
        if (!date) {
            return date;
        }

        let
            month = date.getMonth(),
            year = date.getFullYear();

        month = month + cnt;

        while (month > 11) {
            month = month - 12;
            year = year + 1;
        }

        while (month < 0) {
            month = month + 12;
            year = year - 1;
        }

        return new Date(
            year, month, date.getDate(),
            date.getHours(), date.getMinutes(), date.getSeconds(), date.getMilliseconds());
    },
    incDay: function (date, cnt) {
        let result = new Date(date.getTime());

        result.setDate(result.getDate() + cnt); // POZOR! Nefunguje pri posunu casu! (napr. 28.3.2020)
        return result;
    },
    incDayStart: function (date, cnt) {
        let result = new Date(date.getTime());
        result.setHours(0, 0, 0, 0);
        result.setTime(result.getTime() + (cnt * 24 + 2) * 60 * 60 * 1000); // hodiny navic kvuli posunu casu
        result.setHours(0, 0, 0, 0);

        return result;
    },
    daysBetween: function (date1, date2) {
        return Math.round((date2.getTime() - date1.getTime()) / 86400000);
    },

});


$.extend(gFiles.utils, {
    stringToFloat: function (str) {
        if (!str) {
            return null;
        }

        str = str
            .replace(/,/g, ".")
            .replace(/ /g, "")
            .replace(/&nbsp;/g, "");

        return parseFloat(str);
    },
    formatFloat: function (number, digits) {
        if (number == null) {
            return '';
        }

        if (typeof digits == "undefined") {
            digits = 2;
        }
        return window.Intl // kvuli IE <= 10
            ? Intl.NumberFormat('cs-cz', { minimumFractionDigits: digits, maximumFractionDigits: digits }).format(number)
            : number.toFixed(digits);
    },
    stringToDate: function (str) {
        if (str === '' || str === undefined) {
            return null;
        }

        let
            now = new Date(),
            day = now.getDate(),
            month = now.getMonth(),
            year = now.getFullYear();

        if (str.match(/^(\d+).(\d+).(\d+)$/)) {
            let dateParts = str.split('.');

            day = parseInt(dateParts[0], 10);
            month = parseInt(dateParts[1], 10) - 1; // 0 - 11 !
            year = parseInt(dateParts[2], 10);
        } else if (str.match(/^(\d+).(\d+)[.]$/)) {
            let dateParts = str.split('.');

            day = parseInt(dateParts[0], 10);
            month = parseInt(dateParts[1], 10) - 1; // 0 - 11 !
        } else if (str.match(/^(\d+)[.]$/)) {
            let dateParts = str.split('.');
            day = parseInt(dateParts[0], 10);
        } else {
            return null;
        }

        return new Date(year, month, day);
    },
    stringToDateTime: function (str) {
        if (str === '' || str === undefined) {
            return null;
        }

        if (str.match(/^(\d+).(\d+).(\d+) (\d+):(\d+):(\d+)$/)) {
            let
                parts = str.split(' '),
                dateParts = parts[0].split('.'),
                day = parseInt(dateParts[0], 10),
                month = parseInt(dateParts[1], 10) - 1, // 0 - 11 !
                year = parseInt(dateParts[2], 10),
                timeParts = parts[1].split(':'),
                hour = parseInt(timeParts[0], 10),
                min = parseInt(timeParts[1], 10),
                sec = parseInt(timeParts[2], 10);

            return new Date(year, month, day, hour, min, sec);
        }

        return null; // chyba
    },
    dateToString: function (date) {
        if (!date) {
            return '';
        }

        return date.getDate() + '.' + (date.getMonth() + 1) + '.' + date.getFullYear();
    },
    stringToTime: function (str) {
        if (!str) {
            return null;
        }

        let timeParts = str.split(':');

        if (timeParts.length > 0) {
            let
                h = parseInt(timeParts[0], 10),
                m = 0,
                s = 0;

            if (timeParts.length === 1) {
                if (h > 100) {
                    m = h % 100;
                    h = Math.floor(h / 100);
                }
            } else {
                if (timeParts.length > 1) {
                    m = parseInt(timeParts[1], 10);
                }

                if (timeParts.length > 2) {
                    s = parseInt(timeParts[2], 10);
                }
            }

            return new Date(1970, 0, 1, h, m, s, 0);
        }

        return null;
    },
    timeToString: function (date, utc, seconds) {
        if (!date) {
            return '';
        }

        let
            h = utc ? date.getUTCHours() : date.getHours(),
            m = utc ? date.getUTCMinutes() : date.getMinutes(),
            result = h + ':';

        if (m < 10) {
            result += '0';
        }
        result += m;

        if (seconds) {
            let s = utc ? date.getUTCSeconds() : date.getSeconds();

            result += ':';
            if (s < 10) {
                result += '0';
            }
            result += s;
        }

        return result;
    },


    secondsToString: function (seconds) {
        let
            s = seconds % 60,
            min = (seconds - s) / 60,
            m = min % 60,
            h = (min - m) / 60;

        return h + ':' + ((m < 10) ? '0' : '') + m + ':' + ((s < 10) ? '0' : '') + s;
    },
    timeStringToFloat: function (str, enableShortcut) {
        if (!str || str === '') {
            return 0;
        }

        let timeParts = str.split(':');

        if (timeParts.length > 0) {
            let h = parseInt(timeParts[0], 10);
            if (isNaN(h)) {
                return null;
            }

            let m = 0;
            let s = 0;

            if (enableShortcut && h >= 100 && timeParts.length === 1) {
                m = h % 100;
                h = Math.floor(h / 100);
            } else {
                if (timeParts.length > 1) {
                    m = parseInt(timeParts[1], 10);
                    if (isNaN(m)) {
                        return null;
                    }
                }

                if (timeParts.length > 2) {
                    s = parseInt(timeParts[2], 10);
                    if (isNaN(s)) {
                        return null;
                    }
                }
            }

            return h + m / 60 + s / 3600;
        }

        return 0;
    },
    floatToTimeString: function (value, short) {
        if (value == null) {
            return '';
        }

        let result = '';

        if (value < 0) {
            result = '-';
            value = -value;
        }

        let h = Math.floor(value);
        result += h;


        let m = Math.round((value - h) * 60);
        if (!short || m) {
            result +=
                ':' +
                ((m < 10) ? '0' + m : m);
        }

        return result;
    }

});

gFiles.stringUtils = {
    appendSuffix: function (str, suffix) {
        return str
            ? (str.endsWith(suffix) ? str : str + suffix)
            : suffix;
    },
    prependPrefix: function (str, prefix) {
        return str
            ? (str.startsWith(prefix) ? str : prefix + str)
            : prefix;
    },
    removePrefix: function (str, prefix, defaultValue) {
        return str
            ? (str.startsWith(prefix) ? str.substring(prefix.length) : str)
            : (defaultValue !== undefined ? defaultValue : str);
    },
    appendUrlParameters: function (urlString, parametersString) {
        urlString = urlString
            ? urlString + (urlString.includes("?") ? "" : "?")
            : "?";

        if (urlString.endsWith("?") || urlString.endsWith("&")) {
            return urlString + gFiles.stringUtils.removePrefix(parametersString, "&");
        }

        return urlString + gFiles.stringUtils.prependPrefix(parametersString, "&");
    },
    lineBreaksToBr: function (str) {
        return str.replace(/(?:\r\n|\r|\n)/g, '<br />');
    },

};

$.extend(gFiles.utils, {
    getUrlParam: function (paramName) {
        let params = location.search.split("&");

        for (let i = 0; i < params.length; i++) {
            let pom = params[i].split(paramName + "=");
            if (pom && pom.length > 1) {
                return pom[1];
            }
        }

        return null;
    },
    getCookie: function (cookieName) {
        let
            name = cookieName + "=",
            ca = document.cookie.split(';');

        for (let i = 0; i < ca.length; i++) {
            let c = ca[i].trim();
            if (c.indexOf(name) === 0) {
                return c.substring(name.length, c.length);
            }
        }

        return "";
    }
});
(function ($) {
    function showData($tableElement, data) {
        $tableElement.children("tbody").remove();
        $tableElement.append(data);
    }


    function loadBrowserData(filters) {
        let
            $this = $(this),
            dataUrl = $this.data("data-url");

        if (dataUrl) {
            $.ajax({
                url: dataUrl,
                data: filters
            }).done(function (data) {
                showData($this, data);
            });
        }
    }

    function initBrowser() {
        loadBrowserData.call(this);
    }
    function processFilters($columnFilters) {
        filters = {};
        $columnFilters.find(".column-filter").each(function () {
            $input = $(this);
            if ($input.attr("name") && $input.val()) {
                filters[$input.attr("name")] = $input.val();
            }
        });

        loadBrowserData.call($columnFilters.closest(".browser"), filters);
    }

    function onColumnFilterKeyPress($event) {
        if ($event.which == 13) {
            processFilters($(this).closest(".column-filters"));
        }
    }

    $("body")
        .on("keypress", ".column-filter", onColumnFilterKeyPress);

    $(function () {
        $(".browser").each(initBrowser);
    });

})(jQuery);

(function ($) {
    if (gFiles.useBootstrap) {
        $.fn.dialog = function (options) {
            let
                $this = $(this),
                modal = $this.data("gfiles-modal");

            if (typeof options === 'string') {
                if (!modal) {
                    console.log("Dialog neni inicializovan, nelze provest", options);
                    return;
                }

                switch (options) {
                    case 'close':
                        modal.hide();
                        return;

                    default:
                        console.log("Dialog: neznama akce", options);
                        return;
                }
            }

            if (!modal) {
                let
                    options = {};

                modal = new bootstrap.Modal($this.get(0), options);
                $this.data("gfiles-modal", modal);
            }

            if (options && options.title) {
                $this.find(".modal-title").text(options.title);
            }

            modal.show(options ? options.relatedTarget : null);
        }
    }
    const dialogOpenObserver = new MutationObserver(mutations => {
        mutations.forEach(mutation => {
            if (mutation.attributeName === "open") {
                const dialog = mutation.target;
                if (dialog.open) {
                    $(dialog).trigger("dialogopen");
                } else {
                    $(dialog).trigger("dialogclose");
                }
            }
        });
    });
    function onDialogOpen() {
        $(this).find(".dialog-default-focus").focus();
    }


    $(function () {
        $("body").on("DOMNodeInserted", "dialog", function () {
            dialogOpenObserver.observe(this, { attributes: true });
        });
        $("dialog").each(function () {
            dialogOpenObserver.observe(this, { attributes: true });
        });


        $("body")
            .on("shown.bs.modal", ".modal", onDialogOpen)  // bootstrap
            .on("dialogopen", ".rondialog", onDialogOpen)  // jquery-ui
            .on("dialogopen", "dialog", onDialogOpen)  // <dialog> - zajisteno pomoci dialogOpenObserver
            ;
    });

        
})(jQuery);

(function() {

    function loadSelectItems(options) {
        let
            $select = $(this),
            url = options.url || $select.data('options-url') || $select.data('items-url');

        if (options.data) {
            loadOptions($select, options.data);
            return;
        }

        if (!url) {
            return;
        }

        if (options.appendToUrl) {
            url += options.appendToUrl;
        }

        $.ajax(url)
            .done( function(data) {
                if (data.result !== undefined) {
                    if (data.result === "OK") {
                        loadOptions($select, data.options || data.items);  // items pridano pro kompatibilitu s ronselect
                    } else {
                        gFiles.showError(data);
                    }
                }
                else
                {
                    loadOptions($select, data);
                }
            })
            .fail( function(jqXHR) {
                gFiles.showAjaxError(jqXHR, "Pri nacitani zaznamu doslo k chybe!");
            });
    }

    $.fn.extend({
            loadSelectItems: loadSelectItems
        }
    );
    function loadOptions($select, data) {
        let
            originalValue = $select.val(),
            originalValueFound = false;

        $select.empty();
        let
            textFieldName = $select.data("text-field"),
            valueFieldName = $select.data("value-field");

        if ($select.data("empty-item") !== undefined) {
            $select.append(
                $('<option></option>')
                    .val("")
                    .text($select.data("empty-item"))
            );
        }

        $.each(data, function(i) {
            let
                text = textFieldName
                    ? data[i][textFieldName]
                    : (data[i].text === undefined
                        ? data[i].value
                        : data[i].text
                    ),

                value = valueFieldName
                    ? data[i][valueFieldName]
                    : (data[i].text === undefined
                        ? data[i].key
                        : data[i].value
                    );

            $select.append(
                $('<option></option>')
                    .val(value)
                    .text(text)
                    .addClass(data[i].cssClass)
            );

            if (value === originalValue) {
                originalValueFound = true;
            }
        });
        if (originalValueFound) {
            $select.val(originalValue);
        }
    }
})();

(function ($) {
    function toggleDisabled() {
        let
            $this = $(this),
            $elements = $this;

        if ($this.is("input[type=radio]")) {
            if ($this.data("parent-selector")) {
                $elements = $($this.data("parent-selector")).find("input[name=" + $this.attr("name") + "]");
            } else {
                $elements = $this.parents("form").find("input[name=" + $this.attr("name") + "]");
            }
        }

        $elements.each(function () {
            let
                $this = $(this), // checkbox/radio
                $element = $($this.data("selector"));

            $element.prop("disabled", !$this.prop("checked"));
        });
    }
    function toggleClass() {
        let
            $this = $(this),
            $element = $this;

        if ($this.data("toggleclass-selector")) {
            $element = $($this.data("toggleclass-selector"));
        } else if ($this.data("selector")) {
            $element = $($this.data("selector"));
        }

        if ($this.data("add-class")) {
            $element.addClass($this.data("add-class"));
        }

        if ($this.data("remove-class")) {
            $element.removeClass($this.data("remove-class"));
        }

        if ($this.data("toggle-class")) {
            $element.toggleClass($this.data("toggle-class"));
        }
        let
            $elements = $this;

        if ($this.is("input[type=radio]")) {
            if ($this.data("parent-selector")) {
                $elements = $($this.data("parent-selector")).find("input[name=" + $this.attr("name") + "]");
            } else {
                $elements = $this.closest("form").find("input[name=" + $this.attr("name") + "]");
            }
        }

        $elements.each(function () {
            let
                $this = $(this), // checkbox/radio
                $element = $this;

            if ($this.data("toggleclass-selector")) {
                $element = $($this.data("toggleclass-selector"));
            } else if ($this.data("selector")) {
                $element = $($this.data("selector"));
            }
            if ($this.data("check-class")) {
                $element.toggleClass($this.data("check-class"), $this.prop("checked"));
            }
            if ($this.data("uncheck-class")) {
                $element.toggleClass($this.data("uncheck-class"), !$this.prop("checked"));
            }
            if ($this.data("not-empty-class")) {
                $element.toggleClass($this.data("not-empty-class"), !!$this.val())
            }
            if ($this.data("empty-class")) {
                $element.toggleClass($this.data("empty-class"), !$this.val())
            }
        });
    }

    function autoExpand() {
        $(this).toggleClass("expanded");
    }

    function focusToggle() {
        let $this = $(this);
        $this.toggleClass($this.data("focus-toggle-class"), true);
    }

    function focusOutToggle() {
        let $this = $(this);
        $this.toggleClass($this.data("focus-toggle-class"), false);
    }


    $(function () {
        $("body")
            .on("click", ".toggleClass", toggleClass)
            .on("click", ".toggle-class", toggleClass)
            .on("click", ".toggleDisabled", toggleDisabled)
            .on("click", ".toggle-disabled", toggleDisabled)
            .on("click", ".autoExpand", autoExpand)
            .on("click", ".auto-expand", autoExpand)
            .on("change", "input.toggle-class-on-change", toggleClass)
            .on("focusin", ".focus-toggle", focusToggle)
            .on("focusout", ".focus-toggle", focusOutToggle)
            ;
        $(".toggle-on-load").each(toggleClass);
    });

})(jQuery);

(function ($) {
    let
        $dialog = null,
        $messageDiv = null,
        $titleElement = null;
    function createDialog() {
        if (gFiles.useBootstrap) {
            $dialog = $("<dialog/>")
                .attr("role", "dialog")
                .appendTo($("body"));

            $titleElement = $("<div/>")
                .addClass("dialog-header")
                .appendTo($dialog);

            $messageDiv = $("<div/>")
                .addClass("dialog-body")
                .appendTo($dialog);

            $("<div/>")
                .addClass("dialog-footer")
                .append(
                    $("<button/>")
                        .addClass("btn btn-primary close-dialog")
                        .text(gFiles._("OK")))
                .appendTo($dialog);
        } else {
            $dialog = $("<div/>")
                .addClass("rondialog");

            $messageDiv = $("<div/>")
                .addClass("max-height-80vh")
                .addClass("overflow-auto")
                .appendTo($dialog);

            $("<div/>")
                .addClass("buttons")
                .append(
                    $("<button/>")
                        .text("OK")
                        .addClass("submitbutton")
                        .addClass("cancel")
                )
                .appendTo($dialog);
        }
    }
    gFiles.showMessage = function (options) {
        if (options === false || (typeof options == "string" && options === "close")) {
            if ($dialog) {
                $dialog.dialog("close");
            }
            return;
        }

        let defaultOptions = {
            title: document.title,
            content: null,
            messages: null
        };

        if (typeof options == "string") {
            defaultOptions.content = $("<div/>").text(options);
            options = defaultOptions;
        } else if (typeof options == "object") {
            options = $.extend(defaultOptions, options);
        }

        if (!$dialog) {
            createDialog();
        }

        $messageDiv.empty();

        if (options.content) {
            $messageDiv.append(options.content);
        } else if (options.messages) {
            for (let i = 0; i < options.messages.length; i++) {
                let
                    cssClass = 'infomsg',
                    text = options.messages[i].text;

                switch (options.messages[i].category) {
                    case 'debug':
                    case 'log':
                        console.log(text);
                        text = null;
                        break;

                    case 'warning':
                        cssClass = 'warningmsg';
                        break;

                    case 'success':
                        cssClass = 'successmsg';
                }

                if (text) {
                    $messageDiv
                        .append($("<div/>")
                            .addClass(cssClass)
                            .text(text));
                }
            }
        }

        if ($titleElement) {
            $titleElement.text(options.title); // bootstrap
        } else {
            $dialog.attr("title", options.title); // jquery-ui
        }

        if (gFiles.useBootstrap) {
            $dialog.showModal();
        } else {
            $dialog.dialog({
                modal: true,
                minHeight: "50px",
                position: { my: "center", at: "center", of: window },
                width: "auto"
            });
        }

    };

    gFiles.hideMessage = function () {
        if ($dialog) {
            if (gFiles.useBootstrap) {
                $dialog.close();
            } else {
                $dialog.dialog("close");
            }
        }
    }

})(jQuery);
gFiles.showError = function (error, element, cssClass) {
    if (typeof (error) === "object" && (!error.errors || !error.errors.length)) {
        return;
    }

    if (cssClass === undefined) {
        cssClass = "errortext";
    }

    if (element) {
        let $element = $(element);
        $element.html("");

        if (error && error.errors) {
            for (let i = 0; i < error.errors.length; i++) {
                $element.append($("<div/>").attr("class", cssClass).html(gFiles.stringUtils.lineBreaksToBr(error.errors[i].errorText)));
                if (error.errors[i].errorDescription) {
                    console.log(error.errors[i].errorText, error.errors[i].errorDescription);
                } else {
                    console.log(error.errors[i].errorText);
                }
            }
        } else if (error) {
            $element.append($("<div/>").attr("class", cssClass).html(gFiles.stringUtils.lineBreaksToBr(error)));
            console.log(error);
        }

    } else {
        let $content = $("<div/>");

        if (error && error.errors) {
            for (let i = 0; i < error.errors.length; i++) {
                let messages = error.errors[i].errorText.split("\n");
                for (let msgIndex = 0; msgIndex < messages.length; msgIndex++) {
                    if (messages[msgIndex]) {
                        $content.append($("<div/>").text(messages[msgIndex]).addClass(cssClass));
                    }
                }
                if (error.errors[i].errorDescription) {
                    console.log(error.errors[i].errorText, error.errors[i].errorDescription);
                } else {
                    console.log(error.errors[i].errorText);
                }
            }
        } else if (error) {
            $content.append(
                $("<div/>")
                    .text(error)
                    .addClass(cssClass)
            );
            console.log(error);
        }

        gFiles.showMessage({
            title: gFiles._("Error"),
            content: $content
        });

    }
};
gFiles.showAjaxError = function (jqXHR, errorMsg) {
    if (!errorMsg) {
        errorMsg = gFiles._("Error") + "!";
    }

    console.log(errorMsg, jqXHR.responseText);

    if (gFiles.debugMode) {
        gFiles.showError(errorMsg + " (" + jqXHR.status + " - " + jqXHR.statusText + ")\n" + jqXHR.responseText);
    } else {
        gFiles.showError(errorMsg + " (" + jqXHR.status + " - " + jqXHR.statusText + ")");
    }
};
gFiles.ajaxErrorFunction = function (errorMessage) {
    return function (jqXHR) {
        gFiles.showAjaxError(jqXHR, errorMessage);
    };
};



(function ($) {
    let
        $dialog = null,
        $messageDiv = null,

        $okBtn = null,
        $stornoBtn = null;

    function createDialog() {
        let $buttonsDiv = null;

        $dialog = $("<dialog/>")
            .attr("role", "dialog")
            .appendTo($("body"));

        $("<div/>")
            .addClass("dialog-header")
            .appendTo($dialog)
            .text(gFiles._("Dotaz"))

        $messageDiv = $("<div/>")
            .addClass("dialog-body")
            .appendTo($dialog);

        $buttonsDiv = $("<div/>")
            .addClass("dialog-footer")
            .appendTo($dialog);

        $okBtn = $("<button/>").text(gFiles._("OK")).addClass("submitbutton");
        $stornoBtn = $("<button/>").text(gFiles._("Cancel")).addClass("button");
        $buttonsDiv
            .append($okBtn)
            .append($stornoBtn);
    }
    gFiles.confirm = function (options, yesCallback, thisArg) {
        if (typeof options == "string") {
            options = { text: options };
        }

        if (!options) {
            return true;
        }

        options.yesCallback = options.yesCallback || yesCallback;
        options.thisArg = options.thisArg || thisArg;

        if (!options.yesCallback) {
            return confirm(options.text);
        }

        if (!options.text) {
            options.yesCallback.call(options.thisArg, options.callbackData);
            return false;
        }

        if (!$dialog) {
            createDialog();
        }

        if (options.html) {
            $messageDiv.html(options.html);
        } else {
            $messageDiv.text(options.text);
        }

        $stornoBtn
            .off("click")
            .on("click", function () {
                if (options.noCallback) {
                    options.noCallback.call(options.thisArg, options.callbackData);
                }
                $dialog.close();
            });

        $okBtn
            .off("click")
            .on("click", function () {
                if (options.yesCallback) {
                    options.yesCallback.call(options.thisArg, options.callbackData);
                }
                $dialog.close();
            });

        $dialog.showModal();

        return false; // Vraci false, protoze teprve doslo k zobrazeni dialogu. Pripadna OK akce je volana pomoci callbacku
    }
    function confirmQuestion(yesCallback) {
        let
            $element = $(this),
            confirmId = $element.data("confirm-text-id") || $element.data("ron-confirm-id"),
            confirmText = $element.data("confirm-text") || $element.data("ron-confirm-text");

        if (confirmId) {
            return gFiles.confirm(gFiles._(confirmId), yesCallback, $element);
        } else if (confirmText) {
            return ron.confirm(confirmText, yesCallback, $element);
        }

        if (yesCallback) {
            yesCallback.call($element);
            return false;
        }

        return true;
    }


    $.fn.extend({
        confirmQuestion: confirmQuestion
    });
})(jQuery);

(function ($) {
    function submitPostLink() {
        const
            $this = $(this),
            url = $this.data("url") || $this.attr("href"),
            attributes = $this[0].attributes;

        if (url) {
            let $form = $("<form/>", {
                method: "POST",
                action: url
            });
            $.each(attributes, function () {
                if (this.specified  &&  this.name.startsWith("data-post-")) {
                    $("<input/>")
                        .attr({
                            type: "hidden",
                            name: this.name.substring(10),
                            value: this.value
                        })
                        .appendTo($form);
                }
            });
            $form.appendTo("body").submit();
        }
    }
    function onPostLinkClick() {
        $(this).confirmQuestion(submitPostLink);
    }


    $(function () {
        $("body")
            .on("click", ".post-link", onPostLinkClick);

    });
    
})(jQuery);

(function ($) {
    let
        $defaultDialog = null;

    function createDialog() {

        $defaultDialog = $("<dialog/>")
            .appendTo($("body"));
    }


    function dialogContentLoaded(data) {
        $content = $(this);

        $content
            .removeClass("loading")
            .html(data);

        $defaultDialog.get(0).showModal();
    }


    $.fn.gFilesDialog = function (options) {
        options = options || {};
        if (!$defaultDialog) {
            createDialog();
        }

        if (options.dialogContent) {
            $defaultDialog
                .empty()
                .html(options.dialogContent)

        } else {
            let
                $this = $(this),
                $container = $this.closest(".gfiles-dialog-container"),
                contentUrl = options.contentUrl || $this.data("dialog-content-url") || $container.data("dialog-content-url"),
                appendUrl = options.appendUrl || $this.data("dialog-append-url");

            url = appendUrl
                ? gFiles.stringUtils.appendUrlParameters(contentUrl, appendUrl)
                : contentUrl;

            if (url) {
                $defaultDialog
                    .empty()
                    .addClass("loading");

                $.ajax({
                    url: url,
                })
                    .done(function (data) {
                        dialogContentLoaded.call($defaultDialog, data);
                    });
            }
        }
    }



    function onOpenDialogClick($e) {
        let $this = $(this);

        if ($e.originalEvent.ctrlKey) {
            let
                $container = $this.closest(".gfiles-dialog-container"),
                editUrl = $this.data("form-url") || $container.data("form-url");

            if (editUrl) {
                let
                    appendUrl = $this.data("dialog-append-url");

                if (appendUrl) {
                    editUrl = gFiles.stringUtils.appendUrlParameters(editUrl, appendUrl);
                }

                window.open(editUrl, "_blank");

                return;
            }
        }

        $this.gFilesDialog();
    }

    function closeDialog() {
        $(this).closest("dialog").get(0).close();
    }

    $(function () {
        $("body")
            .on("click", ".gfiles-open-dialog", onOpenDialogClick)  // todo: predelat na .open-dialog ? ale nejdriv to musim predelat v portalu
            .on("click", ".close-dialog", closeDialog);

    });
})(jQuery);


(function ($) {
    function renderCalendarHeader($element) {
        let $header = $("<div/>")
            .addClass("calendar-days-header");

        for (let i = 0; i < 7; i++) {
            $("<div/>")
                .addClass("calendar-day-of-week")
                .text(ron._('Day_short_' + ((den + 1) % 7)))
                    .appendTo($header);
        }

        $header.appendTo($element);
    }


    function renderCalendarDays($element, options) {

    }

    function renderCalendar($element) {
        renderCalendarHeader($element);

    }


    $.extend(gFiles, {
        renderCalendar: renderCalendar
    });
})(jQuery);

var ron = gFiles;
$.extend(ron, {
    translations: [],

    translate : function(text) {
        let result = ron.translations[text];

        return (result === undefined)
            ? text
            : result;
    }
});


ron._ = ron.translate;
ron.showDialog = function(selector) {
    var $element = $(selector);
    if (!$element.length) {
        return;
    }

    var $dialog = $("<div/>")
        .append($element.clone());

    var $buttons = $("<div/>")
        .attr("class", "buttons align-center")
        .appendTo($dialog);


    var closeDialog = function() {
        $dialog.dialog("destroy");
    };


    $("<input/>")
        .attr("type", "button")
        .attr("class", "button")
        .attr("value", ron.translate("OK"))
        .appendTo($buttons)
        .on("click", closeDialog );


    $dialog.dialog( {
            modal : true,
            resizable: true,
            width: "auto",
            height: "auto"
        }
    );
};


ron.errorsDialog = function(selector) {
    let
        $dialog = $(selector);

    if ($dialog.text()  ||  $dialog.children().length) {
        ron.showDialog(selector);
    }
};
ron.setCaretPosition = function(ctrl, pos) {
    if (!ctrl) return false;

    if (ctrl.setSelectionRange) {
        ctrl.focus();
        ctrl.setSelectionRange(pos, pos);
    }
    else if (ctrl.createTextRange) {
        var range = ctrl.createTextRange();
        range.collapse(true);
        range.moveEnd('character', pos);
        range.moveStart('character', pos);
        range.select();
    }

    return true;
};


ron.dialog = {
    showWaitMessage : function(dialog) {
        $(dialog).find(".dialogwaitmessage").show();
    },

    hideWaitMessage : function(dialog) {
        $(dialog).find(".dialogwaitmessage").hide();
    }
};
ron.showPopupMenu = function($menu, options) {
    return $menu.ronPopupMenu(options);
};


ron.floatingMenu = (function() {

    var FloatingMenuOptions = function() {
        this.$originalElement = null;
        this.$parentElement = null;
        this.itemSelector = "li";
    };
    return function ($menu, options) {
        var menuItemMouseOver = function () {
            if (options && options.$parentElement && options.$parentElement.is(".expanded")) return;

            var $this = $(this);

            var itemId = $this.data("ron-itemid");
            if (!itemId) {
                $(document).ronPopup("closeAll");
                return;
            }

            var subMenuId = $menu.attr('id') + '_' + itemId;
            $("#" + subMenuId).ronPopupMenu({
                attachToElement: $this,
                wrap: true
            });
        };

        $menu.on("mouseover", (options && options.itemSelector) ? options.itemSelector : "li", menuItemMouseOver);

        var onScroll = function () {
            if ((document.documentElement.scrollTop || document.body.scrollTop) > options.$originalElement.position().top) {
                (options.$parentElement ? options.$parentElement : $menu).toggleClass("visible", true);
            } else {
                (options.$parentElement ? options.$parentElement : $menu).toggleClass("visible", false);
            }

        };

        if (options && options.$originalElement) {
            $(window).on("scroll", onScroll);
        }
    }
})();
ron.logAjaxError = function(jqXHR, errorMsg) {
    if (!errorMsg) errorMsg = "Chyba!";

    console.log(errorMsg, jqXHR.status + ' - ' + jqXHR.statusText);
    ron.debugLog(errorMsg, jqXHR.responseText);
};
$( function() {



    var actionButtonClick = function(event) {
        var $this = $(this);

        var callbackFunc = function() {
            var
                inputSelector = $this.data("input-selector");


            if (inputSelector) {
                var $input = $(inputSelector);
                $input.val($this.data("new-value"));
            }

            $this.closest("form").submit();
        };

        ron.confirmQuestion( $this, callbackFunc );
    };


    function clickWorkingInfo() {
        let workingInfo = $(this).data("ron-working-info");
        ron.showWorkingInfo(workingInfo);
    }


    function sendPositionSubmit() {
        $(this).children("[name='x']").val( $(window).scrollLeft() );
        $(this).children("[name='y']").val( $(window).scrollTop() );
    }


    function submitClick() {
        return ron.confirmQuestion($(this));
    }


    function onFormSubmit() {
        $(this).find(".submit-indicator").show();

        let workingInfo = $(this).data("ron-working-info");
        if (workingInfo) {
            ron.showWorkingInfo(workingInfo == "1" ? true : workingInfo);
        }
    }


    let $body = $("body");

    $body
        .on("click", "input:submit", submitClick )
        .on("click", ".actionButton", actionButtonClick)
        .on("click", ".clickWorkingInfo", clickWorkingInfo)
        .on("submit", ".submitWorkingInfo", clickWorkingInfo)
        .on("contextmenu", ".popupmenu", function() { return false; })
        .on("submit", "form.sendPosition", sendPositionSubmit)
        .on("submit", "form", onFormSubmit)
        ;


    $(".js-show").removeClass("hidden");
    $(".js-hide").addClass("hidden");
    let
        scrollX = $body.data("scroll-to-x"),
        scrollY = $body.data("scroll-to-y");

    if (scrollX  ||  scrollY) {
        window.scrollTo(scrollX ? scrollX : 0, scrollY ? scrollY : 0);
    }

    ron.showDialog(".on-load-dialog");
});



ron.showWorkingInfo = function(text) {
    let $element = $("#workingInfo");
    if ( !$element.length ) {
        $element = $("<div/>");

        $element
            .attr("class", "workingInfo")
            .attr("id", "workingInfo")
            .appendTo( document.body );

        if (!text  ||  text === true  ||  text === 1) {
            $element.html( ron._("Probiha_zpracovani_dat") );
        }
    }

    if (text === true  ||  text === 1) {
        $element.html( ron._("Probiha_zpracovani_dat") );
    } else if (text) {
        $element.html(text);
    }

    $element.css("display", "block");
};


ron.hideWorkingInfo = function() {
    $("#workingInfo").css("display", "none");
};



ron.showImage = function(url) {
    var $imgDialog = $("#ron_image");

    var closeImage = function() {
        $imgDialog.dialog("close");
    };

    if (!$imgDialog.length) {
        $imgDialog = $("<div/>")
            .attr("id", "ron_image")
            .on("click", closeImage );
    }

    $imgDialog.html("");

    $("<img/>")
        .attr("src", url)
        .appendTo($imgDialog);

    $imgDialog
        .dialog( {
            modal : true,
            resizable: false,
            width: "auto",
            height: "auto",
            dialogClass: "ronimage",
            beforeClose : function () { $(".ui-widget-overlay").off("click", closeImage); }
        });

    $imgDialog.parent()
        .off("click")
        .on("click", closeImage);

    $(".ui-widget-overlay")
        .on("click", closeImage);
};


function StopEventPropagation(event)
{
  if (!event) event = window.event;

  if (event.stopPropagation) event.stopPropagation();
  else event.cancelBubble = true;
}


function ChangeLocationQuest(event, question, url, showWorkInfo)
{
    if ( ron.confirm(question)) {
        if (showWorkInfo) ron.showWorkingInfo();
        window.location = url;
    }

    StopEventPropagation(event);
}
function GetXMLResult(xml)
{
   var dataxml = xml.getElementsByTagName("result");
   if ( !dataxml.length ) return false;
   
   return dataxml[0].textContent ? dataxml[0].textContent : dataxml[0].text; // druha verze pro IE
}
function ZobrazChybyXML(xml, showwarnings) {
    if ( !xml ) return 0;

    var result = 0;
    if (showwarnings) {
        var dataxml = xml.getElementsByTagName("warning");
        for (var i = 0; i < dataxml.length; i++) {
            alert(dataxml[i].textContent ? dataxml[i].textContent : dataxml[i].text); // druha verze pro IE
        }
    }


    dataxml = xml.getElementsByTagName("error");
    for (i = 0; i < dataxml.length; i++) {
        alert(dataxml[i].textContent ? dataxml[i].textContent : dataxml[i].text); // druha verze pro IE
        result++;
    }

    return result;
}
function focusJsonErrorField(json, setCssClass)
{
    if (json.errors) {
        for (var i = 0;  i < json.errors.length;  i++ ) {
            if ( json.errors[i].errorField ) {
                var fields = $("[name='" + json.errors[i].errorField + "']");
                if (fields.length) {
                    fields.addClass('error');
                    fields.first().focus();
                    return;
                }
            }
        }
    }
}
ron.delayedLaunch = function(timeOut, callable, storeTimerInData) {
    let timer = null;

    return function() {
        let
            args = arguments,
            that = this,
            $this = null;

        if (storeTimerInData) {
            $this = $(this);
            timer = $this.data("delayed-launch-timer");
            if (timer) {
                clearTimeout(timer);
                $this.data("delayed-launch-timer", null);
            }
        } else {
            if (timer) {
                clearTimeout(timer);
                timer = null;
            }
        }

        timer = setTimeout(
            function() {
                timer = null;
                callable.apply(that, args);
            },
            timeOut
        );

        if ($this  &&  storeTimerInData) {
            $this.data("delayed-launch-timer", timer);
        }
    }
};



ron.OptionItem = function() {
    this.key = null;
    this.value = null;
    this.text = null;
    this.cssClass = null;
    this.title = null;
};
ron.loadOptions = function($select, data) {
    $select.loadSelectItems({data : data});
};
ron.loadOptionsFromUrl = function($select, url) {
    $select.loadSelectItems({ url: url });
};

$.extend(ron, {
    Point : function(x, y) {
        this.x = x;
        this.y = y;
    },
    Rect : function(x1, y1, x2, y2) {
        this.x1 = x1;
        this.y1 = y1;
        this.x2 = x2;
        this.y2 = y2;
    },

    JsonResult : function() {
        this.result = '';
        this.errors = null;
    },

    JsonError : function() {
        this.errorText = '';
        this.errorField = '';
        this.errorDescription = '';
    }
});

$.extend(ron.utils, {
    copyArray : function(inputArray) {
        let
            result = [],
            i = inputArray.length;

        while (i--) {
            result[i] = inputArray[i];
        }

        return result;
    },
    binarySearch : function(array, element, compareFunction) {
        let
            minIndex = 0,
            maxIndex = array.length - 1,
            currentIndex,
            currentElement;

        while (minIndex <= maxIndex) {
            currentIndex = (minIndex + maxIndex) / 2 | 0;
            currentElement = array[currentIndex];

            if (compareFunction) {
                let compare = compareFunction(currentElement, element);

                if (compare < 0) { // currentElement < element
                    minIndex = currentIndex + 1;
                } else if (compare > 0) {
                    maxIndex = currentIndex - 1;
                } else {
                    return currentIndex;
                }
            } else {
                if (currentElement < element) {
                    minIndex = currentIndex + 1;
                } else if (currentElement > element) {
                    maxIndex = currentIndex - 1;
                } else {
                    return currentIndex;
                }
            }
        }

        return -1;
    }
});




$.extend(ron, {
    inArray: function(needle, haystack) {
        for (let i = 0; i < haystack.length; i++) {
            if (haystack[i] == needle) {
                return true;
            }
        }
        return false;
    },
    removeFromArray: function(needle, haystack) {
        let result = 0;

        for (let i = haystack.length - 1; i >= 0; i--) {
            if (haystack[i] === needle) {
                haystack.splice(i, 1);
                result++;
            }
        }

        return result;
    }
});


$(function() {
    let
        filterKey = function(event, allowedChars, forbiddenChars) {
            if (event.altKey || event.ctrlKey) {
                return true;
            }
            if (!event.which || event.which < 32) {
                return true;
            }

            if (allowedChars) {
                return allowedChars.includes(event.key);
            }

            if (forbiddenChars) {
                return !forbiddenChars.includes(event.key);
            }

            return true;
        },
        keyPressFilter = function(event) {
            let
                $this = $(this),
                allowed = $this.data('allowed-chars'),
                forbidden = $this.data('forbidden-chars');

            return filterKey(event, allowed, forbidden);
        },
        keyPressFilterFloat = function(event) {
            return filterKey(event, "0123456789.,+-");
        },
        keyPressFilterTime = function(event) {
            return filterKey(event, "0123456789:-");
        },
        keyPressFilterDate = function(event) {
            return filterKey(event, "0123456789.-");
        },
        keyPressFilterInt = function(event) {
            return filterKey(event, "0123456789+-");
        },


        keyDownDateInput = function(event) {
            let addDay = function ($input, count) {
                let
                    date = ron.utils.stringToDate($input.val());

                if (!date) {
                    return;
                }

                date = ron.utils.incDay(date, count);
                $input.val(ron.utils.dateToString(date))
            };


            if (event.ctrlKey) {
                switch (event.which) {
                    case 38 : // up
                        addDay($(this), 1);
                        return false;

                    case 40 : // down
                        addDay($(this), -1);
                        return false;
                }
            }

            return true;
        },
        changeDateInput = function() {
            let
                $this = $(this),
                dateStr = $this.val();

            if (dateStr) {
                let
                    date = ron.utils.stringToDate(dateStr);

                $this.toggleClass('error', !date || isNaN(date.getTime()));
            }
        },
        changeTimeInput = function() {
            let
                $this = $(this),
                timeStr = $this.val();

            if (timeStr) {
                let
                    time = ron.utils.stringToTime(timeStr);

                $this.toggleClass('error', !time  ||  isNaN(time.getTime()));
            }
        },


        comboBoxChange = function() {
            let
                $this = $(this),
                select2 = $this.data('ron-select2'),
                copyTargetSelector = $this.data('copy-target');

            if (select2) {
                if (select2.charAt(0) !== '#') {
                    select2 = "[name=" + select2 + "]";
                }

                let
                    $select2 = $(select2),
                    oldVal = $select2.val();

                if (oldVal != $this.val()) {
                    $(select2)
                        .val($this.val())
                        .trigger("change");
                }
            }

            if (copyTargetSelector) {
                let
                    $target = $(copyTargetSelector),
                    copySource = $this.data('copy-source'),
                    copyValue = $this.val(),
                    copyEmptyValue = $this.data('copy-empty-value');

                if (copyEmptyValue === undefined) {
                    copyEmptyValue = true;
                }

                if (copySource && copySource !== 'value') {
                    let $selectedOption = $this.find(":selected");

                    copyValue = (copySource === 'text')
                        ? $selectedOption.text()
                        : $selectedOption.data(copySource);
                }


                if ($target.length  &&  (copyEmptyValue || (copyValue !== ""  &&  copyValue !== undefined))) {
                    let
                        originalValue = $target.val();

                    $target
                        .val(copyValue);   // prozatim jen editacni prvky. TODO: checkbox (checked), jiny HTML element (text)

                    if (originalValue !== copyValue) {
                        $target.trigger("change");
                    }
                }
            }

            if ($this.data('ron-autosubmit')) {
                let checkboxSelector = $this.data('ron-autosubmit-check');
                if (checkboxSelector) {
                    if (!$(checkboxSelector).prop("checked")) {
                        return;
                    }
                }

                $(this.form).submit(); // pres jQuery se vola on("submit"), jinak ne
            }
        },
        comboBoxMoveClick = function() {
            var
                $this = $(this),
                $select = $($this.data("selector")),
                value = $this.data("value"),
                optionSelector = $this.data("option-selector"),
                change = $this.data("step") ? parseInt($this.data("step")) : 1,
                submit = $this.data("submit") == "1",
                triggerChange = $this.data("trigger-change") == "1",
                $options = $select.children("option"),
                changed = false;

            if (value || value == "") {
                if (value != $select.val()) {
                    $select.val(value);
                    changed = true;
                }
            } else if (optionSelector) {
                $options = $select.find(optionSelector);
                if ($options.length) {
                    $select.val($options.attr("value"));
                    changed = true;
                }
            } else {
                var newIndex = $select.prop("selectedIndex") + change;
                if (newIndex < 0) {
                    newIndex = 0;
                }
                if (newIndex >= $options.length) {
                    newIndex = $options.length - 1;
                }

                if (newIndex != $select.prop("selectedIndex")) {
                    $select.prop("selectedIndex", newIndex);
                    changed = true;
                }
            }

            if (changed) {
                if (triggerChange) {
                    $select.trigger("change");
                }

                if (submit) {
                    $select.closest("form").submit();
                }
            }
        },
        setValueClick = function() {
            let
                $this = $(this),
                cnt = $this.data("selectors-count"),
                triggerChange = $this.data("trigger-change") == "1",
                $form = null;


            function doSetValue() {
                for (let i = 1; i <= (cnt ? cnt : 1); i++) {
                    let
                        suffix = (cnt ? i : ''),
                        selector = $this.data("setvalue-selector" + suffix)
                            ? $this.data("setvalue-selector" + suffix)
                            : $this.data("selector" + suffix),
                        $input = $(selector),
                        value = $this.data("value" + suffix),
                        newValue = null,
                        changed = false;

                    if (value !== undefined) {
                        newValue = value;
                    } else {
                        let
                            valueChecked = $this.data("value-checked" + suffix),
                            valueUnchecked = $this.data("value-unchecked" + suffix);
                        if ((valueChecked || valueChecked === "") && $this.prop("checked")) {
                            newValue = valueChecked;
                        } else if ((valueUnchecked !== undefined) && !$this.prop("checked")) {
                            newValue = valueUnchecked;
                        }
                    }

                    if (newValue !== null && $input.length) {
                        if ($input.attr("type") === "checkbox" || $input.attr("type") === "radio") {
                            changed = newValue ? !$input.prop("checked") : $input.prop("checked");
                            $input.prop("checked", newValue);
                        } else if ('' + newValue !== $input.val()) {    // aby to fungovalo i s cisly a aby rozlisil 0 od ''
                            $input.val(newValue);
                            changed = true;
                        }
                    }

                    if (changed) {
                        if (triggerChange) {
                            $input.trigger("change");
                        }

                        if (!$form) {
                            $form = $input.closest("form");
                        }
                    }
                }

                if ($form && $this.data("submit") == "1") {
                    $form.submit();
                }
            }

            ron.confirmQuestion($this, doSetValue);
        },


        onColorInputChange = function() {
            var
                $this = $(this),
                indicatorSelector = $this.data("indicator");

            if (indicatorSelector) {
                $(indicatorSelector).attr("style", "background-color: " + $this.val() + " !important");
            }
        };

    function checkboxClick() {
        let
            $this = $(this),
            excludedSelector = $this.data("excluded-checkboxes");

        if (!excludedSelector  ||  !$this.prop("checked")) {
            return;
        }

        $(excludedSelector).prop("checked", false);
    }


    function onActualTimeButtonClick() { // DOC css
        let
            $this = $(this),
            dt = new Date(),
            $date = $('#' + $this.data('date-input')),
            $time = $('#' + $this.data('time-input'));

        $date.val(dt.getDate() + "." + (dt.getMonth() + 1) + "." + dt.getFullYear());
        $time.val(
            ((dt.getHours() < 10) ? ("0" + dt.getHours()) : dt.getHours()) + ":" +
            ((dt.getMinutes() < 10) ? ("0" + dt.getMinutes()) : dt.getMinutes())
        );
    }
    function onChangeHref() {
        let
            $this = $(this),
            href = $this.data("ron-href");

        if (!href) {
            return;
        }
        if (href.slice(-1) === "=") {
            href += encodeURIComponent($this.val());
        }

        let workingInfo = $this.data("ron-working-info");
        if (workingInfo) {
            ron.showWorkingInfo(workingInfo);
        }

        let sendPosition = $this.data("ron-send-position");
        if (sendPosition) {
            href += "&y=" + $(window).scrollTop() + "&x=" + $(window).scrollLeft();
        }

        let
            target = $this.data("ron-target");

        if (target) {
            window.open(href, target);
        } else {
            window.location = href;
        }
    }


    $("body")
        .on("click", "input.checkbox-readonly", function() { return false; })
        .on("click", "input[type=checkbox]", checkboxClick)
        .on("keypress", "input.inputDate", keyPressFilterDate)
        .on("keypress", "input.inputTime", keyPressFilterTime)
        .on("keypress", "input.inputFloat", keyPressFilterFloat)
        .on("keypress", "input.inputInt", keyPressFilterInt)
        .on("keypress", ".keyFilter", keyPressFilter)
        .on("keydown", "input.inputDate", keyDownDateInput)
        .on("keydown", "input.date-input", keyDownDateInput)
        .on("change", "input.inputDate", changeDateInput)
        .on("change", "input.inputTime", changeTimeInput)
        .on("change", "select", comboBoxChange)
        .on("click", ".comboBoxMove", comboBoxMoveClick)
        .on("click", ".setValue", setValueClick)
        .on("change", ".color-input", onColorInputChange)
        .on("click", ".actual-time-button", onActualTimeButtonClick)
        .on("click", "input.clickAutoSubmit", function() { $(this.form).submit(); }) // POZOR - nechat tento radek posledni, aby se provedly i ostatni akce!
        .on("change", ".changeHref", onChangeHref)
        ;


    setTimeout(function() {
            $(".defaultfocus").focus();     // nevola se hned, aby se provedla pripadna obsluha onfocus, ktera muze byt nastavena pozdeji
        },
        200
    );
});

(function($) {
    $.fn.toggleClassWithSiblings = function (className, state) {
        var
            $element = $(this),
            rowspanIndex = $element.data("rowspan-index");

        $element.toggleClass(className, state);

        if (rowspanIndex !== undefined) {
            var
                $prev = $element;

            for (var index = rowspanIndex - 1; index >= 0; index--) {
                $prev = $prev.prev();
                $prev.toggleClass(className, state);
            }
            var
                cnt = $prev.data("rowspan-count"),
                $next = $element;

            for (index = rowspanIndex + 1; index < cnt; index++) {
                $next = $next.next();
                $next.toggleClass(className, state);
            }
        }

        if (rowspanIndex !== undefined) {

        }
    };
})(jQuery);

(function($) {
    $(function() {
        let
            $autoRefreshElements = $(".auto-refresh"),
            autoRefreshTimeout = null,
            autoRefreshUrl = null;
        $autoRefreshElements.each(function () {
            let
                $this = $(this),
                timeout = $this.data("timeout"),
                url = $this.data("url");

            if (timeout && url && (!autoRefreshTimeout || timeout < autoRefreshTimeout)) {
                autoRefreshTimeout = timeout;
                autoRefreshUrl = url;
            }
        });

        if (autoRefreshTimeout  &&  autoRefreshUrl) {
            console.log("Setting automatic refresh timeout", autoRefreshTimeout + "s", autoRefreshUrl);

            setTimeout(
                function() { location.replace(autoRefreshUrl); },
                1000 * autoRefreshTimeout);
        }
    });


    function sendPositionLinkClick() {
        let href = $(this).attr("href");
        if (!href) {
            return true;
        }

        href += "&y=" + $(window).scrollTop() + "&x=" + $(window).scrollLeft();
        window.location.replace(href);

        return false;
    }
    function goToHref($element, $target) {
        let href = $element.data("ron-href");

        if (!href && $element.is("a")) {
            href = $element.attr("href");
        }


        if (href) {
            let workingInfo = $element.data("ron-working-info");
            if (workingInfo) {
                ron.showWorkingInfo(workingInfo);
            }

            let sendPosition = $element.data("ron-send-position");
            if (sendPosition) {
                href += "&y=" + $(window).scrollTop() + "&x=" + $(window).scrollLeft();
            }

            if (!$target) {
                $target = $element.data("ron-target");
            }

            if ($target) {
                window.open(href, $target);
            } else {
                window.location = href;
            }
            return false; // stopne defaultni akci pro <a>
        }

        return true;
    }


    function clickHref($e) {
        if (this !== $e.target  && $($e.target).closest("a").length) {
            return;
        }

        return goToHref($(this), $e.ctrlKey ? '_blank' : null);
    }


    function confirmQuestionClick(event) {
        let
            $this = $(this),
            isHref = !!$this.data("ron-href")  ||  ($this.is("a")  &&  $this.attr("href")),
            triggerEvent = $this.data("ron-event"), // TODO: odzkouset
            callbackFunc = null;

        if (isHref) {
            callbackFunc = function() {
                return goToHref($this);
            };
        }
        
        let
            result = ron.confirmQuestion( $this, callbackFunc );

        if (!isHref  &&  !result) {
            event.stopImmediatePropagation();
            return false;
        }

        if (result && triggerEvent) {
            return $this.trigger(triggerEvent);
        }

        return result;
    }


    $(function() {
        $("body")
            .on("click", "a.sendPosition", sendPositionLinkClick)
            .on("click", ".confirmQuestion", confirmQuestionClick)
            .on("click", ".clickHref", clickHref)
        ;
    });

})(jQuery);

$.fn.scrollParent = function( includeHidden ) {
    let
        position = this.css( "position" ),
        excludeStaticParent = position === "absolute",
        overflowRegex = includeHidden ? /(auto|scroll|hidden)/ : /(auto|scroll)/,
        scrollParent = this.parents().filter( function() {
            let parent = $(this);

            if ( excludeStaticParent && parent.css( "position" ) === "static" ) {
                return false;
            }

            return overflowRegex.test(
                parent.css( "overflow" ) +
                parent.css( "overflow-y" ) +
                parent.css( "overflow-x" )
            );
        } ).eq( 0 );

    return position === "fixed" || !scrollParent.length ?
        $( this[ 0 ].ownerDocument || document ) :
        scrollParent;
};

$.extend(ron.utils, {
    windowSize : function() {
        let
            $window = $(window);

        return new ron.Point($window.width(), $window.height());
    },
    windowViewRect : function() {
        let
            $window = $(window),
            left = $window.scrollLeft(),
            top = $window.scrollTop();

        return new ron.Rect(
            left, top,
            left + $window.width() - 1, top + $window.height() - 1
        );
    },
    getPositionInParent : function($element, parent) {
        let
            result = $element.offset(),
            parentOffset = $(parent).offset();

        result.top -= parentOffset.top;
        result.left -= parentOffset.left;

        return result;
    },
    getAbsoluteRect: function($obj) {
        let position = $obj.offset();

        return new ron.Rect(
            position.left, position.top,
            position.left + $obj.width() - 1, position.top + $obj.height() - 1
        );
    },



    getElementFromPosition: function($elements, x, y) {
        let
            result = null;

        $elements.each(function() {
            let
                $this = $(this),
                offset = $this.offset();

            if (offset.left <= x  &&  offset.top <= y) {
                if (offset.left + $this.width() >= x  &&  offset.top + $this.height() >= y) {
                    result = $this;
                    return false;
                }
            }
        });

        return result;
    },
    hasFixedPosition : function($element) {
        let lastElement = null;
        while ($element.length  &&  $element.get(0) !== lastElement) { // document.body  &&  $element.get(0) != ownerDocument) {
            lastElement = $element.get(0);
            if ( $element.css("position") === "fixed") {
                return true;
            }
            $element = $element.offsetParent();
        }

        return false;
    },
    scrollPage : function(options) {
        if (options.x) {
            $(window).scrollLeft(options.x);
        }

        if (options.y) {
            $(window).scrollTop(options.y);
        }
    },


    initialScrollPage : function(options) {
        $(function() {
            ron.utils.scrollPage(options);
        });
    },
    ensureVisible : function($element, $parent) {
        if (!$parent) {
            $parent = $element.scrollParent(); // jQuery-ui
        }

        let parentHeight = ($parent.get(0) === document)
            ? $(window).height()
            : $parent.get(0).clientHeight;

        if ($parent.get(0).scrollHeight <= parentHeight) {
            return;
        }

        let pos = ron.utils.getPositionInParent($element, $parent.get(0));

        if ( pos.top < 0 ) {  // ( pos.top < $parent.scrollTop() ) {
            let newPos = $parent.scrollTop() + pos.top;
            if (newPos < 0) {
                newPos = 0;
            }
            $parent.scrollTop( newPos );

        } else if ( pos.top + $element.height() > parentHeight /*+ $parent.scrollTop()*/ )  {
            let newPos = $parent.scrollTop() + pos.top + $element.height() - parentHeight;
            if (newPos < 0) {
                newPos = 0;
            }
            $parent.scrollTop( newPos );
        }
    },
    isFullVisible : function($element, $parent) {
        if (!$parent) {
            $parent = $element.scrollParent(); // jQuery-ui
        }

        let parentHeight = ($parent.get(0) === document)
            ? $(window).height()
            : $parent.get(0).clientHeight;

        if ($parent.get(0).scrollHeight <= parentHeight) {
            return true;
        }

        let pos = ron.utils.getPositionInParent($element, $parent.get(0));

        if ( pos.top < 0 ) {
            return false;
        } else if ( pos.top + $element.height() > parentHeight )  {
            return false;
        }

        return true;
    },
    scrollElementToTop : function($element, $parent) {
        if (!$parent) {
            $parent = $element.scrollParent(); // jQuery-ui
        }

        let parentHeight = ($parent.get(0) === document)
            ? $(window).height()
            : $parent.get(0).clientHeight;

        if ($parent.get(0).scrollHeight <= parentHeight) {
            return;
        }

        let
            pos = ron.utils.getPositionInParent($element, $parent.get(0)),
            newPos = $parent.scrollTop() + pos.top;

        if (newPos < 0) {
            newPos = 0;
        }

        $parent.scrollTop( newPos );
    },
    scrollElementToBottom : function($element, $parent) {
        if (!$parent) {
            $parent = $element.scrollParent(); // jQuery-ui
        }

        let parentHeight = ($parent.get(0) === document)
            ? $(window).height()
            : $parent.get(0).clientHeight;

        if ($parent.get(0).scrollHeight <= parentHeight) {
            return;
        }

        let
            pos = ron.utils.getPositionInParent($element, $parent.get(0)),
            newPos = $parent.scrollTop() + pos.top + $element.height() - parentHeight;

        if (newPos < 0) {
            newPos = 0;
        }

        $parent.scrollTop( newPos );
    },

});

$.extend(ron.utils, {
    lineBreaksToBr: gFiles.stringUtils.lineBreaksToBr,
    startsWith: function (str, prefix) {
        return str.startsWith(prefix);
    },
    removePrefix: function (str, prefix) {
        return gFiles.stringUtils.removePrefix(str, prefix, false);
    },
    containsSubstring: function (str, substring, delimiter) {
        let a = str.split(delimiter);

        for (let i = 0; i < a.length; i++) {
            if (a[i] === substring) {
                return true;
            }
        }

        return false;
    },


    capitalizeFirstLetters: function(str) {
        let words = str.split(" ");

        return words.map((word) => {
            return word[0].toUpperCase() + word.substring(1);
        }).join(" ");
    },
    addSubstring: function (str, substring, delimiter) {
        if (ron.utils.containsSubstring(str, substring, delimiter)) {
            return str;
        }

        return str + (str ? delimiter : "") + substring;
    },
    removeSubstring: function (str, substring, delimiter) {
        let
            a = str.split(delimiter),
            result = '';

        for (let i = 0; i < a.length; i++) {
            if (a[i] !== substring) {
                result += result ? (delimiter + a[i]) : a[i];
            }
        }

        return result;
    },
    htmlTextConvert: function (text) {
        if (!text) {
            return '';
        }

        let map = {
            '&': '&amp;',
            '<': '&lt;',
            '>': '&gt;',
            '"': '&quot;',
            "'": '&#039;'
        };

        return text.replace(/[&<>"']/g, function (m) {
            return map[m];
        });
    },
    
});

(function() {
    $.fn.setCellWidths = function(widths) {
        let
            $cells = this,
            lastColspan = 0,
            cellIndex = 0,
            width = 0;

        for (let i = 0;  cellIndex < $cells.length  &&  i < widths.length;  i++) {
            if (lastColspan > 0) {
                width += widths[i];
                lastColspan --;

                if (!lastColspan) {
                    $cells.eq(cellIndex)
                        .outerWidth(width + "px")
                        .css("min-width", width + "px"); // bez toho to nekdy nefunguje (napr. soucty v paticce planovani kapacit)
                    width = 0;
                }
            } else {
                let
                    $cell = $cells.eq(cellIndex),
                    colSpan = $cell.prop("colspan");

                if (colSpan > 1) {
                    lastColspan = colSpan - 1;
                    width = widths[i];
                } else {
                    $cells.eq(cellIndex).outerWidth(widths[i] + "px");
                    cellIndex++;
                }
            }
        }
    };

})(jQuery);
ron.setSameCellWidth = function(tables, tables2) {
    let
        $cells = [],
        $row;

    for (let i = 0;  i < tables.length;  i++) {
        if (tables[i].is("tr")) {
            $row = tables[i]; // je zadan primo radek
        } else {
            $row = tables[i].children("tr:visible").first();
            if (!$row.length) {
                $row = tables[i].children("tbody").children("tr:visible").first();
            }
        }
        if ( $row.length ) {
            $cells.push($row.children("td,th"));
        }
    }

    if (!$cells.length) {
        return;
    }
    let widths = [];
    for (let colIndex = 0; colIndex < $cells[0].length;  colIndex++) {
        widths[colIndex] = $cells[0].eq(colIndex).outerWidth();
    }
    for (let tableIndex = 1;  tableIndex < $cells.length;  tableIndex++) {
        for (let colIndex = 0; colIndex < $cells[tableIndex].length;  colIndex++) {
            let width = $cells[tableIndex].eq(colIndex).outerWidth();
            if (width > widths[colIndex]) {
                widths[colIndex] = width;
            }
        }
    }
    for (let tableIndex = 0;  tableIndex < $cells.length;  tableIndex++) {
        for (let colIndex = 0; colIndex < $cells[tableIndex].length; colIndex++) {
            $cells[tableIndex].setCellWidths(widths);
        }
    }

    if (tables2) {
        for (i = 0; i < tables2.length; i++) {
            if (tables2[i].is("tr")) {
                $row = tables2[i]; // je zadan primo radek
            } else {
                $row = tables2[i].children("tr").first();
                if (!$row.length) {
                    $row = tables2[i].children("tbody").children("tr").first();
                }
            }

            if ($row.length) {
                $row.children("td,th").setCellWidths(widths);
            }
        }
    }

    let result = 0;
    for (let i = 0;  i < widths.length;  i++) {
        result += widths[i];
    }
    return result;
};



$(function() {

    let
        rowBodySelectClick = function() {
            $(this).toggleClass("browserselectedrow");
        },
        moveInTableVertical = function(sender, up) {
            let
                $cell = $(sender).closest("td");

            if (!$cell.length) {
                return false;
            }

            let
                $row = $cell.parent("tr");

            if (!$row.length) {
                return false;
            }

            while ($row.length) {
                $row = up ? $row.prev("tr") : $row.next("tr");

                let
                    $newCell = $row.children().eq($cell.index()),
                    $inputs = $newCell.find(sender.tagName + ":visible");

                if ($inputs.length) {
                    $inputs.eq(0).focus().select();
                    return true;
                }
            }

            return false;
        },
        keyDownVerticalMove = function(event) {
            if (event.altKey || event.ctrlKey) {
                return true;
            }

            switch (event.which) {
                case 38 : // up
                    return !moveInTableVertical(this, true);

                case 40 : // down
                    return !moveInTableVertical(this, false);
            }

            return true;
        };



    $("body")
        .on("dblclick", ".rowSelect>tr, .rowSelect>tbody>tr", rowBodySelectClick)
        .on("dblclick", ".bodySelect>tbody", rowBodySelectClick)
        .on("keydown", "table.keyInputMove input:text, table.keyInputMove input:checkbox", keyDownVerticalMove)
        ;

});


(function() {

    var
        CacheDataItem = function(url) {
            this.status = "loading";
            this.data = null;
            this.jqXHR = null;

            var
                doneFunctions = [],
                failFunctions = [],
                alwaysFunctions = [],
                callFunctions = function(fail, param) {
                    var
                        doneList = doneFunctions,
                        failList = failFunctions,
                        alwaysList = alwaysFunctions;

                    doneFunctions = [];
                    failFunctions = [];
                    alwaysFunctions = [];

                    if (fail) {
                        for (var i = 0;  i < failList.length;  i++) {
                            failList[i](param);
                        }
                    } else {
                        for (i = 0;  i < doneList.length;  i++) {
                            doneList[i](param);
                        }
                    }

                    for (i = 0;  i < alwaysList.length;  i++) {
                        alwaysList[i]();
                    }
                };
            this.done = function(callable) {
                if (this.status === "ok") {
                    callable(this.data);
                } else if (this.status === "loading") {
                    doneFunctions[doneFunctions.length] = callable;
                }

                return this;
            };
            this.fail = function(callable) {
                if (this.status === "error") {
                    callable(this.jqXHR);
                } else if (this.status === "loading") {
                    failFunctions[failFunctions.length] = callable;
                }

                return this;
            };
            this.always = function(callable) {
                if (this.status === "loading") {
                    alwaysFunctions[alwaysFunctions.length] = callable;
                } else {
                    callable();
                }

                return this;
            };


            var that = this;

            this.status = "loading";

            $.ajax(url)
                .done(function (data) {
                    that.status = "ok";
                    that.data = data;
                    callFunctions(false, data);
                })
                .fail(function (jqXHR) {
                    that.status = "error";
                    that.jqXHR = jqXHR;
                    callFunctions(true, jqXHR);
                });
        },
        cacheData = [];


    ron.urlCache = {
        load : function(url, options) {
            if (options !== false  &&  cacheData[url]) {
                if (cacheData[url].status !== 'error') {    // v pripade chyby bude nacitat znovu
                    return cacheData[url];
                }
            }

            var result = new CacheDataItem(url);

            if (options !== false) {
                cacheData[url] = result;
            }

            return result;
        },

        invalidate : function(url) {
            if (cacheData[url]) {
                cacheData[url] = null;
            }
        }
    }
})();


(function($) {
    let
        $contextMenuElement = null;

    function onMenuItemClick($e, data) {
        let
            $menuItem = $(this),
            eventName = $menuItem.data("context-menu-event")
                ? $menuItem.data("context-menu-event")
                : "contextmenuactivate";

        $contextMenuElement.trigger(eventName,
            {
                originalEvent: $e,
                menuItem: $menuItem
            });

    }
    function onContextMenu($e) {
        let
            $this = $(this),
            itemSelector = $this.data("context-menu-item-selector"), // selektor pro vyber polozek, na kterych je context menu aktivni
            attachToElement = $this.data("context-menu-attach-to-element") == "1",
            $element = $this;

        if (itemSelector) {
            let
                $target = $($e.target); // polozka, na ktere bylo kliknuto

            if ($target.is(itemSelector)) {
                $element = $target
            } else {
                $element = $target.closest(itemSelector);
            }
        }

        if ($element  &&  $element.length) {
            if ($element.trigger("showcontextmenu",
                {
                    originalEvent: $e,
                    contextMenuContainer: $this
                }) === false) {
                return false;
            }

            $contextMenuElement = $element;

            let
                menuSelector = $this.data("context-menu");

            if (!menuSelector) {
                return false;
            }

            $(menuSelector).ronPopupMenu({
                onClick: onMenuItemClick,
                attachToElement: attachToElement ? $element : null,
                mouseEvent: $e
            })
        }

        return false;
    }

    $(function() {
        $("body")
            .on("contextmenu", ".context-menu-container", onContextMenu);
    })
})(jQuery);


(function($) {
    if (!$.fn.dialog) {
        $.fn.test_bootstrap = true;

        $.fn.dialog = function(options) {
            let
                $this = $(this),
                modal = $this.data("ron-modal");

            if (typeof options === 'string') {
                if (!modal) {
                    console.log("Dialog neni inicializovan, nelze provest", options);
                    return;
                }

                switch (options) {
                    case 'close' :
                        modal.hide();
                        return;

                    default :
                        console.log("Dialog: neznama akce", options);
                        return;
                }
            }

            if (!modal) {
                let
                    options = {};

                modal = new bootstrap.Modal($this.get(0), options);
                $this.data("ron-modal", modal);
            }

            if (options && options.title) {
                $this.find(".modal-title").text(options.title);
            }

            modal.show(options ? options.relatedTarget : null);
        }
    } else {
        $.fn.test_bootstrap = false;
    }
    let originalDialog = $.fn.dialog;
    $.fn.dialog = function(options) {
        let
            $this = $(this);

        if ($this.is("dialog")) {
            if (typeof options === 'string') {
                switch (options) {
                    case 'close' :
                        $this.close();
                        return;

                    default :
                        console.log("Dialog: neznama akce", options);
                        return;
                }
            }

            if (options && options.title) {
                console.log("Nastavuji title", options.title);
                $this.find(".dialog-header").text(options.title);
            }

            $this.showModal();

            if (options && options.open) {
                options.open.call(this);
            }
        } else {
            originalDialog.call(this, options);
        }
    }
    function onDialogOpen()
    {
        $(this).find(".dialog-default-focus").focus();
    }



    $(function() {
        $("body")
            .on("shown.bs.modal", ".modal", onDialogOpen)  // bootstrap
            .on("dialogopen", ".rondialog", onDialogOpen)  // jquery-ui
            ;
    });
})(jQuery);



(function($) {
    $.fn.dndTarget = function() {
        return this.each(function() {
            var
                $that = $(this),
                counter = 0;

            var onDragEnter = function() {
                counter++;
                $that.addClass("dndTargetActive");
            };

            var onDragLeave = function() {
                if (counter > 0) counter--;
                if (!counter) {
                    $that.removeClass("dndTargetActive");
                }
            };

            var onDrop = function() {
                $that.removeClass("dndTargetActive");
                counter = 0;
            };

            $that
                .on("dragenter", onDragEnter )
                .on("dragleave", onDragLeave )
                .on("drop", onDrop)
                .on("dragover", function() { /*console.log($e);*/ return false; } )
        });
    }
})(jQuery);

(function($) {
    let
        defaultOptions = {
            axis: null,

            disabled: false,
            containment: null,
            helper: null,
            appendTo: null,
            start: null,
            drag: null,
            stop: null
        },
        draggable = null,
        $draggedObject = null;

    function RonDraggable($element, options) {
        let
            that = this,
            startX = null,
            startY = null;
        this.$element = $element;
        this.options = $.extend({}, defaultOptions, options);
        this.onMouseMove = function($e) {
            let newPos = null;
            if ($e.originalEvent.targetTouches  &&  $e.originalEvent.targetTouches.length) {
                newPos = {
                    left: $e.originalEvent.targetTouches[0].screenX - startX,
                    top: $e.originalEvent.targetTouches[0].screenY - startY
                };
            } else {
                newPos = {
                    left: $e.originalEvent.screenX - startX,
                    top: $e.originalEvent.screenY - startY
                };
            }
            if (that.options.axis === "x") {
                newPos.top = $draggedObject.offset().top;
            } else if (that.options.axis === "y") {
                newPos.left = $draggedObject.offset().left;
            }

            if (that.options.containment) {
                let
                    $parent =
                        (that.options.containment === "parent")
                            ? $element.parent()
                            : ((typeof that.options.containment === "string")
                                ? $(that.options.containment)
                                : that.options.containment
                            ),
                    parentOffset = $parent.offset();

                if (newPos.left < parentOffset.left) {
                    newPos.left = parentOffset.left;
                } else {
                    let
                        width = $draggedObject.outerWidth(),
                        parentWidth = $parent.width();

                    if (newPos.left + width > parentOffset.left + parentWidth) {
                        newPos.left = parentOffset.left + parentWidth - width;
                    }
                }

                if (newPos.top < parentOffset.top) {
                    newPos.top = parentOffset.top;
                } else {
                    let
                        height = $draggedObject.outerHeight(),
                        parentHeight = $parent.height();

                    if (newPos.top + height > parentOffset.top + parentHeight) {
                        newPos.top = parentOffset.top + parentHeight - height;
                    }
                }
            }

            let
                $offsetParent = $draggedObject.offsetParent(),
                offsetParentOffset = $offsetParent.offset(),

                eventParam = {
                    $element: that.$element,
                    offset: newPos,
                    position: {
                        top: newPos.top - offsetParentOffset.top,
                        left: newPos.left - offsetParentOffset.left
                    },
                    $draggedObject: $draggedObject  // bud stejne jako $element, nebo klon
                };

            if (that.options.drag) {
                that.options.drag.call($element, $e, eventParam);
            }
            $draggedObject.css({
                top: eventParam.position.top,
                left: eventParam.position.left
            })
        };


        this.onMouseUp = function($e) {
            if (that.options.stop) {
                that.options.stop.call($element, $e, {
                    $element: that.$element,
                    offset: $draggedObject.offset(),
                    position: $draggedObject.position()
                });
            }

            if (that.options.helper === "clone") {
                $draggedObject.remove();
            }

            $draggedObject = null;
        };
        function onMouseDown($e) {
            if (that.options.disabled) {
                return;
            }

            let offset = that.$element.offset();
            draggable = that;
            if ($e.originalEvent.targetTouches  &&  $e.originalEvent.targetTouches.length) {
                startX = $e.originalEvent.targetTouches[0].screenX - offset.left;
                startY = $e.originalEvent.targetTouches[0].screenY - offset.top;
            } else {
                startX = $e.originalEvent.screenX - offset.left;
                startY = $e.originalEvent.screenY - offset.top;
            }

            if (that.options.start) {
                that.options.start.call(that.$element, $e, {
                    $element: that.$element,
                    offset: offset,
                    position: that.$element.position()
                });
            }

            if (that.options.helper === "clone") {
                let
                    $clonedElement = that.$element.clone(),
                    $newParent = that.options.appendTo;

                $draggedObject = $clonedElement;

                if (typeof $newParent === "string") {
                    $newParent = $($newParent);
                } else if (!$newParent) {
                    $newParent = that.$element.parent();
                }

                $clonedElement.css("position", "absolute");

                let
                    originalPosition = ron.utils.getAbsoluteRect(that.$element);

                $clonedElement
                    .css("left", originalPosition.x1)
                    .css("top", originalPosition.y1)
                    .css("opacity", "70%");

                $clonedElement.appendTo($newParent);
            } else {
                $draggedObject = that.$element;
            }
        }

        if (!this.$element.css("position")) {
            this.$element.css("position", "relative");
        }


        this.$element
            .on("mousedown", onMouseDown)
            .on("touchstart", onMouseDown);
    }

    function documentMouseMove($e) {
        if (draggable) {
            return draggable.onMouseMove($e);
        }
    }

    function documentMouseUp($e) {
        if (draggable) {
            draggable.onMouseUp($e);
            draggable = null;
        }
    }

    function documentSelectStart() {
        if (draggable) {
            return false;
        }
    }

    $(document)
        .on("mousemove", documentMouseMove)
        .on("touchmove", documentMouseMove)
        .on("mouseup", documentMouseUp)
        .on("touchend", documentMouseUp)
        .on("selectstart", documentSelectStart);

    $.fn.rondraggable = function(options) {
        let
            action = null;

        if (typeof options === "string") {
            action = options;
        }

        this.each(function() {
            let
                $element = $(this),
                instance = $element.data("rondraggable-instance");

            if (action) {
                if (!instance) {
                    console.log("ERROR: element is not instance of rondraggable", this);
                    return this;
                }

                switch (action) {
                    case "disable" :
                        instance.options.disabled = true;
                        break;

                    case "enable" :
                        instance.options.disabled = false;
                        break;

                    case "instance" :
                        return instance;

                    default :
                        console.log("ERROR: action is not implemented in rondraggable", action);
                }

                return this;
            }
            if (!instance) {
                instance = new RonDraggable($element, options);
                $element
                    .data("rondraggable-instance", instance);
            }
        });

        return this;
    }
})(jQuery);


(function($) {
    let
        renderHTML = function($element, data) {
            if (data.result == "error") {
                $element.html(''); // TODO: zobrazit chybu primo v elementu?
                ron.showError(data);
                return;
            }

            $element.html(data.html ? data.html : '');
        },
        loadContent = function($element, url) {
            url = url ? url : $element.data("url");

            $element.empty();

            if (!url) {
                return;
            }

            $.ajax(url)
                .done( function(data) {
                    renderHTML($element, data)
                })
                .fail(ron.ajaxErrorFunction);
        };
    $.fn.dynamicHtml = function(url) {
        $(this).each( function() {
            loadContent($(this), url);
        });
    };

    $(function() {
        $(".dynamic-html").dynamicHtml();
    });
})(jQuery);


(function($) {

    var createReplaceDiv = function($element) {
        return $("<div/>")
            .css("width", $element.outerWidth() + "px")
            .addClass("d-inline-block")
            .html('&nbsp;');
    };


    $.fn.ensureVisible = function(options) {
        var defaultOptions = {
            direction: "vertical",

            topWindowMargin: 30,
            $parentElement: null,
            $scrollElement: null
        };

        options = $.extend( defaultOptions, options );

        $(this).each(function() {
            var
                $element = $(this),
                $window = $(window),
                $parentElement = options.$parentElement || $element.parent(),
                $replaceDiv = null,

                fixedTop = false,
                fixedBottom = false,
                lastScrollTop = 0,
                lastScrollLeft = 0,
                originalLeft = 0
                ;

            if ($element.css("position") == "fixed") {
                return;
            }

            var
                showReplaceDiv = function() {
                    if ($replaceDiv) {
                        $replaceDiv.show();
                    } else {
                        $replaceDiv = createReplaceDiv($element);
                        $replaceDiv.insertBefore($element);
                        $element.parent().css("overflow", "hidden");
                    }
                },

                hideReplaceDiv = function() {
                    if ($replaceDiv) {
                        $replaceDiv.hide();
                    }
                },

                handleScroll = function () {
                    var
                        elementHeight = $element.outerHeight(),
                        parentHeight = $parentElement.height();

                    if (!fixedTop && !fixedBottom && elementHeight + 50 >= parentHeight) {
                        return;
                    }

                    if (!$element.is(":visible")) {
                        return;
                    }

                    if ($element.height() >= $parentElement.height()) {
                        return;
                    }

                    var
                        scrollTop = $window.scrollTop(),
                        scrollLeft = $window.scrollLeft(),
                        pos = $element.offset(),
                        parentPos = $parentElement.offset();

                    if (scrollTop > lastScrollTop) {
                        if (fixedBottom) {
                            if (pos.top + elementHeight >= parentPos.top + parentHeight) {
                                $element
                                    .css("position", "absolute")
                                    .css("top", "auto")
                                    .css("left", "auto")
                                    .css("bottom", 0);

                                fixedBottom = false;
                            }
                        } else if (fixedTop) {
                            $element
                                .css("position", "relative")
                                .css("top", pos.top - parentPos.top + "px")
                                .css("left", "auto")
                                .css("bottom", "auto")
                                .css("width", "auto");

                            hideReplaceDiv();

                            fixedTop = false;
                        } else {
                            var
                                windowHeight = $window.height();
                            if (parentPos.top < scrollTop && parentPos.top + parentHeight > scrollTop + windowHeight && pos.top + elementHeight < scrollTop + windowHeight) {
                                var
                                    pomRect = ron.utils.getAbsoluteRect($element),
                                    originalWidth = $element.outerWidth();

                                originalLeft = pomRect.x1;

                                if (windowHeight - options.topWindowMargin > elementHeight) {
                                    showReplaceDiv();
                                    $element
                                        .css("position", "fixed")
                                        .css("left", (originalLeft - scrollLeft) + "px")
                                        .css("top", options.topWindowMargin + "px")
                                        .css("width", originalWidth + "px")
                                        .css("bottom", "auto");
                                } else {
                                    showReplaceDiv();
                                    $element
                                        .css("position", "fixed")
                                        .css("left", (originalLeft - scrollLeft) + "px")
                                        .css("top", "auto")
                                        .css("width", originalWidth + "px")
                                        .css("bottom", 0);
                                }
                                fixedBottom = true;
                            }
                        }
                    } else if (scrollTop < lastScrollTop) {

                        if (fixedTop) {
                            if (pos.top <= parentPos.top) {
                                $element
                                    .css("position", "static")
                                    .css("top", "auto")
                                    .css("bottom", "auto")
                                    .css("left", "auto")
                                    .css("width", "auto");

                                hideReplaceDiv();

                                fixedTop = false;
                            }
                        } else if (fixedBottom) {
                            $element
                                .css("position", "relative")
                                .css("top", pos.top - parentPos.top)
                                .css("bottom", "auto")
                                .css("left", "auto")
                                .css("width", "auto");

                            hideReplaceDiv();

                            fixedBottom = false;
                        } else {
                            if (parentPos.top - scrollTop < 0 && pos.top - options.topWindowMargin >= scrollTop) {
                                pomRect = ron.utils.getAbsoluteRect($element);
                                originalLeft = pomRect.x1;
                                originalWidth = $element.outerWidth();

                                showReplaceDiv();

                                $element
                                    .css("position", "fixed")
                                    .css("left", (originalLeft - scrollLeft) + "px")
                                    .css("top", options.topWindowMargin)
                                    .css("width", originalWidth + "px")
                                    .css("bottom", "auto");

                                fixedTop = true;
                            }
                        }
                    }

                    lastScrollTop = scrollTop;

                    if (fixedTop || fixedBottom && lastScrollLeft != scrollLeft) {
                        $element.css("left", (originalLeft - scrollLeft) + "px");
                    }

                    lastScrollLeft = scrollLeft;
                },

                timer = null,

                onScroll = function() {
                    if (timer) return;

                    timer = setTimeout( function() {
                        timer = null;
                        handleScroll();
                    }, 50);
                };
            var $scrollElement = options.$scrollElement || $window;

            $scrollElement.on("scroll", onScroll);
        });
    }
})(jQuery);


$(function() {
    $(".ensure-visible").ensureVisible();
});

(function($) {

    let defaultOptions = {
        uploadUrl: null,
        uploadFormData: null,
        onFileUploaded: null,
        onAllFilesUploaded: null,
        onFailure: null
    }

    function FileUploader($element, options)
    {
        let
            that = this;

        options = $.extend(
            {},
            defaultOptions,
            {
                uploadUrl: $element.data("upload-url")
            },
            options
        );


        function finished(successful) {
            $element.removeClass("loading");

            if (successful) {
                if (options.onAllFilesUploaded) {
                    options.onAllFilesUploaded.call($element);
                }

                $element.trigger("allfilesuploaded");
            } else {
                if (options.onFailure) {
                    options.onFailure.call($element);
                }
            }
        }
        function fileUploaded(receivedData) {
            if (options.onFileUploaded) {
                if (options.onFileUploaded.call($element, receivedData) === false) {
                    return false;
                }
            }

            return $element.trigger("fileuploaded", receivedData);
        }
        this.handleUploadForm = function(form, onSuccess) {
            $.ajax(
                {
                    url: options.uploadUrl,
                    type: "POST",
                    data: form,
                    cache: false,
                    dataType: 'json',
                    processData: false,
                    contentType: false
                })
                .done( function(data) {
                    if (data.result === "OK") {
                        console.log("handleUploadForm OK", data);
                        if (fileUploaded(data) === false) {
                            return;
                        }

                        if (onSuccess) {
                            onSuccess(data);
                        }
                    } else {
                        ron.showError(data);
                        finished(false, data);
                    }
                })
                .fail( function(jqXHR) {
                    ron.logAjaxError(jqXHR, "Pri nahravani souboru doslo k chybe!");
                    finished(false); // "Pri nahravani souboru doslo k chybe!"
                });
        }
        this.uploadFile = function (fileData, fileName, onSuccess)
        {
            console.log("fileuploader - uploadFile", fileData, fileName);

            let form = new FormData();

            form.append('upload_file', fileData);
            if (fileName) {
                form.append('fileName', fileName);
            }

            this.handleUploadForm(form, onSuccess);
        }
        function handleUploadFiles(files) {
            let
                counter = 0;
            function uploadNextFile() {
                if (counter >= files.length) {
                    finished(true);
                    return;
                }

                let fileReader = new FileReader();

                fileReader.onload = ( function(e) {

                    let form = new FormData();
                    form.append('upload_file', files[counter]);
                    if (options.uploadFormData) {
                    }

                    that.handleUploadForm(form, function() {
                        counter++;
                        uploadNextFile();
                    });
                });

                fileReader.readAsBinaryString(files[counter]);
            }

            uploadNextFile();
        }


        function onDrop($e) {
            let
                dt = $e.originalEvent.dataTransfer,
                $this = $(this);

            $this.addClass("loading");

            handleUploadFiles(dt.files);

            return false; // prevent default browser action
        }

        $element
            .dndTarget()
            .on("drop", onDrop);
    }


    $.fn.fileuploader = function(options) {
        let
            action = null;

        if (typeof options === "string") {
            action = options;
        }

        if (action) {
            let
                $element = $(this),
                instance = $element.data("fileuploader-instance");

            if (!instance) {
                return;
            }

            switch (action) {
                case "instance" :
                    return instance;

                case "uploadfile" :
                    instance.uploadFile(arguments[1], arguments[2], arguments[3]);
                    break;

                default :
                    console.log("fileuploader - neznama akce:", action);
            }

            return this;
        }

        this.each(function () {
            let
                $element = $(this),
                instance = $element.data("fileuploader-instance");
            if (!instance) {
                instance = new FileUploader($element, options);
                $element
                    .data("fileuploader-instance", instance);
            }
        });

        return this;
    }


    $(function() {
        $(".fileuploader").fileuploader();
    })
})(jQuery);


ron.floatingTableCols = function() {
    var floating_cols = [];

    var FloatingTableCol = function(table) {

        this.$div = null;
        this.$table = $(table);

        this.baseposition = null; // absolutni pozice zakladniho prvku v dokumentu (table)
        this.tableright = 0; // prava hrana tabulky - za ni se uz hlavicka nezobrazi
        this.createHeaderForCell = function ($cell) {
            if (!$cell.length) return;

            var pos = $cell.offset();
            pos.top -= this.baseposition.top;
            pos.left -= this.baseposition.left;
            var $div1 = $("<div/>")
                .addClass("floatingheadercontainer")
                .css("top", pos.top + "px")
                .css("left", pos.left + "px")
                .css("height", ($cell.height() + parseInt($cell.css('padding-top'))) + "px")
                .css("width", $cell.outerWidth())
                .css("text-align", $cell.css("text-align"))
                .appendTo(this.$div);

            $("<div/>")
                .attr("class", "floatingheadercell")
                .html($cell.html())
                .appendTo($div1);
        };


        this.createCells = function () {
            this.baseposition = this.$table.offset();
            this.tableright = this.baseposition.left + this.$table.width();

            this.$div
                .offset({
                    left: this.$div.offset().left,
                    top: this.baseposition.top
                })
                .height(this.$table.height());

            var $rows = this.$table.find("tr"); // nepouzivam each() kvuli this

            var colCount = parseInt(this.$table.data("ron-fixed-cols"));
            if (!colCount) colCount = 1;
            var width = 0;
            for (var colNum = 1;  colNum <= colCount; colNum++) {
                var rowSpan = 0;

                for (var i = 0; i < $rows.length; i++) {
                    if (rowSpan > 0) {
                        rowSpan --;
                    } else {
                        var $cell = $($rows[i]).find("td, th").filter(":nth-child(" + colNum + ")");
                        if (!i) {
                            width += $cell.outerWidth();
                        }
                        this.createHeaderForCell($cell);

                        rowSpan = $cell.attr('rowspan') - 1;
                    }
                }
            }

            this.$div.width(width);
        };
        this.doCreateHeader = function () {
            this.$div = $("<div/>")
                .attr("class", "floatingheadercol")
                .css("display", "none")
                .appendTo($(document.body));

            this.createCells();
        };


        this.createHeader = function () {
            this.doCreateHeader();
            floating_cols.push(this);
        };

        this.reCreateHeader = function () {
            this.$div.remove();

            this.$div = null;

            this.doCreateHeader();
        };


        this.onScroll = function () {
            if ((document.documentElement.scrollLeft || document.body.scrollLeft) > this.tableright - 40 || (document.documentElement.scrollLeft || document.body.scrollLeft) <= this.baseposition.left) {
                this.$div
                    .css("display", "none");
            }
            else {
                var left = 0;
                this.$div
                    .css("left", 0)
                    .css("top", (this.baseposition.top - (document.documentElement.scrollTop  ||  document.body.scrollTop)) + "px")
                    .css("display", "inline");
            }
        };
        this.createHeader();
    };


    $(window)
        .on("scroll", function () {
            for (var i = 0; i < floating_cols.length; i++) {
                floating_cols[i].onScroll();
            }
        })
        .on("resize", function() {
            for (var i = 0; i < floating_cols.length; i++) {
                floating_cols[i].reCreateHeader();
            }
        })
    ;
    return function($obj) {
        $obj.each(function () {
            new FloatingTableCol(this);
        });
    }
}();



$(window).on("load", function () {
    ron.floatingTableCols( $("table.fixedCols") );
});

ron.floatingTableHeaders = function() {
    var floating_headers = [];
    var FloatingTableHeader = function (element) {
        this.$element = $(element);
        this.$div = null;

        this.$table = this.$element.closest("table"); // nadrazena tabulka

        this.baseposition = null; // absolutni pozice zakladniho prvku v dokumentu (thead)
        this.tablebottom = 0; // spodni hrana tabulky - pod ni se uz hlavicka nezobrazi
        this.createHeaderForCell = function (cell) {
            var
                $cell = $(cell),
                pos = $cell.offset();

            pos.left -= this.baseposition.left;
            var
                $div1 = $("<div/>")
                    .addClass("floatingheadercontainer")
                    .css("left", pos.left + "px")
                    .css("width", ($cell.width() + parseInt($cell.css('padding-right'))) + "px")
                    .height(this.$element.height())
                    .css("text-align", $cell.css("text-align"))
                    .appendTo(this.$div);

            $("<div/>")
                .attr("class", "floatingheadercell")
                .html($cell.html())
                .appendTo($div1);
        };


        this.createCells = function () {
            this.baseposition = this.$element.offset();

            if (this.$table) {
                var tablepos = this.$table.offset();
                this.tablebottom = tablepos.top + this.$table.height();
            }

            this.$div
                .offset({
                    top: this.$div.offset().top,
                    left: this.baseposition.left
                })
                .width(this.$element.width())
                .height(this.$element.height());

            var $cells = this.$element.find("th, td"); // nepouzivam each() kvuli this
            for (var i = 0; i < $cells.length; i++) {
                this.createHeaderForCell($cells[i]);
            }
        };
        this.doCreateHeader = function () {
            this.$div = $("<div/>")
                .attr("class", "floatingheader")
                .css("display", "none")
                .appendTo($(document.body));

            this.createCells();
        };


        this.createHeader = function () {
            this.doCreateHeader();
            floating_headers.push(this);
        };

        this.reCreateHeader = function () {
            this.$div.remove();

            this.$div = null;

            this.doCreateHeader();
        };


        this.onScroll = function () {
            this.baseposition = this.$element.offset();

            if ((document.documentElement.scrollTop || document.body.scrollTop) > this.tablebottom - 40 || (document.documentElement.scrollTop || document.body.scrollTop) <= this.baseposition.top) {
                this.$div
                    .css("display", "none");
            }
            else {
                var top = 0;
                $(".fixedheader").each( function() {
                    var $thisFixedHeader = $(this);
                    if ($thisFixedHeader.is(":visible")) {
                        var headerBottom = $thisFixedHeader.position().top + $thisFixedHeader.get(0).clientHeight;
                        if (headerBottom > top) top = headerBottom;
                    }
                } );
                this.$div
                    .css("top", top + "px")
                    .css("left", (this.baseposition.left - (document.documentElement.scrollLeft  ||  document.body.scrollLeft)) + "px") // left se muze menit pri horizontalnim scrolovani
                    .css("display", "inline");
            }
        };
        this.createHeader();
    };
    var findFloatingHeaderForTable = function (table) {
        for (var i = 0; i < floating_headers.length; i++) {
            if (floating_headers[i].$table.get(0) == table) return floating_headers[i];
        }

        return null;
    };


    window.ReCreateFloatingHeaderForTable = function(table)
    {
        var header = findFloatingHeaderForTable(table);
        if (header) header.reCreateHeader();
    };



    var recreateFloatingHeaders = function() {
        for (var i = 0; i < floating_headers.length; i++) {
            floating_headers[i].reCreateHeader();
        }
    };


    window.ReCreateFloatingHeaders = recreateFloatingHeaders; // kompat. - pouziva se v XML


    $(window)
        .on("scroll", function () {
            for (var i = 0; i < floating_headers.length; i++) {
                floating_headers[i].onScroll();
            }
        })
        .on("resize", function() {
            recreateFloatingHeaders();
        });
    return function($obj) {
        $obj.each(function () {
            new FloatingTableHeader(this);
        });
    };

}();


$(window).on("load", function () {
    ron.floatingTableHeaders( $(".floating-table-header-js") );
});

(function($) {
    let
        $rootElement = null,
        $items = null,
        $thumbPanel = null,
        $fotoContainer = null,
        thumbnails = [],
        actualImage = 0;


    let
        selectThumbnail = function(index) {
            if (!thumbnails  ||  !thumbnails.length) {
                return;
            }

            if (index >= 0  &&  index < thumbnails.length) {
                if (actualImage >= 0) {
                    thumbnails[actualImage].toggleClass("selected", false);
                }

                thumbnails[index].toggleClass("selected", true);
                var newPos = $thumbPanel.width() / 2 - thumbnails[index].width() / 2;
                var newScroll = ( $thumbPanel.scrollLeft() + thumbnails[index].position().left - newPos );
                if (newScroll < 0) newScroll = 0;
                $thumbPanel.scrollLeft( newScroll );
            } else {
                console.log("selectThumbnail - spatny index: ", index);
            }
        },
        showVideo = function(src) {
            $("<video/>")
                .attr("width", 640)
                .attr("height", 480)
                .attr("controls", "controls")
                .append(
                $("<source/>")
                    .attr("src", src)
            )
                .appendTo($fotoContainer);
        },
        showImage = function(src) {
            $("<img/>")
                .attr("src", src)
                .appendTo($fotoContainer);
        },
        showLink = function(src) {
            $("<a/>")
                .attr("href", src)
                .attr("target", "_blank")
                .text(src)
                .appendTo($fotoContainer);
        },
        showFile = function(index) {
            $fotoContainer.empty();

            let
                $item = $($items[index]),
                src = $item.data("ron-url"),
                type = $item.data("ron-type") || "image";

            switch (type) {
                case "video" :
                    showVideo(src);
                    break;

                case "image" :
                    showImage(src);
                    break;

                default :
                    showLink(src);
                    break;
            }

            actualImage = index;
        },

        close = function() {
            $items = null;

            $rootElement.remove();
            $rootElement = null;
        },

        onThumbnailClick = function($e) {
            var $thumbnail = $(this); // $(this);

            selectThumbnail($thumbnail.data("ron-index"));

            showFile( $thumbnail.data("ron-index") );
        },


        afterDeleteCallback = function() {
            thumbnails[actualImage]
                .data("ron-deleted", "1")
                .empty()
                .append(
                    $("<i/>")
                        .addClass("fa inline-glyph glyph-delete")
                );
        },


        onKeyDown = function($e) {
            if (!$items) {
                return;
            }
            switch ($e.which) {
                case 27 :
                    close();
                    break;

                case 37 :
                    if (actualImage > 0) {
                        selectThumbnail(actualImage - 1);
                        showFile(actualImage - 1);
                    }
                    break;

                case 39 :
                    if (actualImage < thumbnails.length - 1) {
                        selectThumbnail(actualImage + 1);
                        showFile(actualImage + 1);
                    }
                    break;

                case 46 :
                    $($items[actualImage]).trigger("delete-media", afterDeleteCallback);
                    break;

                default :
            }
        };

    let
        thumbnailsDragStart = null;

    function onThumbnailsMouseDown($e) {
        if ($e.which === 1) {
            let $this = $(this);

            thumbnailsDragStart = $e.pageX - this.offsetLeft; // + $this.scrollLeft();
            return false;
        } else {
            thumbnailsDragStart = null;
        }
    }

    function onThumbnailsMouseUp($e) {
        thumbnailsDragStart = null;
    }

    function onThumbnailsDrag($e) {
        if (thumbnailsDragStart  &&  $e.buttons === 1) {
            let
                $this = $(this),
                newPos = $e.pageX - this.offsetLeft; // + $this.scrollLeft();

            $this.scrollLeft( $this.scrollLeft() + thumbnailsDragStart - newPos );
            thumbnailsDragStart = newPos;
            return false;
        } else if (thumbnailsDragStart) {
            thumbnailsDragStart = null;
        }
    }


    $.fn.mediaViewer = function(options) {
        let
            $this = $(this),
            defaultOptions = {
                initialImage: null,
                initialItem: null,
                thumbnailsHeight: 100
            };

        $items = $this.find(".media-item");

        if ($this.data("thumbnails-height")) {
            defaultOptions.thumbnailsHeight = $this.data("thumbnails-height");
        }

        options = $.extend({}, defaultOptions, options);

        if (options.initialImage != null) {
            actualImage = options.initialImage;
        } else if (options.initialItem) {
            actualImage = $items.index(options.initialItem);
        } else {
            actualImage = 0;
        }

        if (!actualImage  ||  actualImage < 0  ||  actualImage >= $items.length) {
            actualImage = 0;
        }


        $rootElement =
            $("<div/>")
                .attr("class", "fotoViewer")
                .appendTo( $(document.body) );

        $fotoContainer = $("<div/>")
            .attr("class", "fotoViewerContainer")
            .appendTo($rootElement);



        $thumbPanel =
            $("<div/>")
                .attr("class", "fotoViewerThumbnailsPanel")
                .on("mousedown", onThumbnailsMouseDown)
                .on("mouseup", onThumbnailsMouseUp)
                .on("mousemove", onThumbnailsDrag)
                .appendTo($rootElement);

        let $thumbnails =
            $("<div/>")
                .attr("class", "fotoViewerThumbnails")
                .on("click", ".fotoViewerThumbnail", onThumbnailClick)
                .appendTo($thumbPanel);


        function showThumbnails() {
            thumbnails = [];
            for (let i = 0;  i < $items.length;  i ++) {
                let
                    $item = $($items[i]),
                    src = $item.data("ron-url"),
                    type = $item.data("ron-type") || "image",
                    $container =
                        $("<div/>")
                            .attr("class", "fotoViewerThumbnail" + ( (i === actualImage) ? " selected" : "" ) )
                            .data("ron-index", i)
                            .appendTo($thumbnails)
                    ;



                switch (type) {
                    case "image" :
                        $("<img/>")
                            .attr("src", src)
                            .appendTo($container);
                        break;

                    case "video" :
                        $("<i/>")
                            .addClass("fa fa-3x glyph-video")
                            .appendTo($container);
                        break;

                    default :
                        $("<span/>")
                            .text("...")
                            .appendTo($container);
                }

                thumbnails[thumbnails.length] = $container;
            }
        }

        showFile(actualImage);
        showThumbnails();
    };

    let showMediaTimer = null;

    function mediaListClick($e) {
        if (showMediaTimer) {
            clearTimeout(showMediaTimer);
            showMediaTimer = null;
        }

        let $this = $(this);

        showMediaTimer = setTimeout( function() {
            showMediaTimer = null;

            $this.mediaViewer( {
                initialItem: $e.target
            });
        }, 300 );
    }

    function mediaListDblClick() {
        if (showMediaTimer) {
            clearTimeout(showMediaTimer);
            showMediaTimer = null;
        }
    }

    $(function() {
        $(document.body)
            .on("click", ".media-list", mediaListClick)
            .on("dblclick", ".media-list", mediaListDblClick)
            .on("keydown", onKeyDown);
    });
})(jQuery);



ron.showWaitMessage = function(options) {
    ron.showMessage(options);
};
ron.confirmQuestion = function($element, yesCallback) {
    return $element.confirmQuestion(yesCallback);
};

ron.showProgressScreen = function(text, maximalniHodnota, prerusitCallback) {
    var
        prerusit = false,
        floatDiv = $("<div/>")
            .attr("id", "ron_progress")
            .addClass("floatingdiv")
            .attr("tabindex", "1")
            .on("keydown", function(e) { e.stopPropagation(); } ) // aby fungoval clipboard
            .appendTo(document.body)
            .focus(),

        progressText = $("<div/>")
            .attr("id", "ron_progress_text")
            .addClass("progresstext")
            .html( text )
            .appendTo(floatDiv),
        progressInfoDiv = $("<div/>")
            .addClass("progressinfotext")
            .attr("id", "ron_progress_infotext")
            .appendTo(floatDiv),

        teplomerDiv = $("<div/>")
            .addClass("progress margin-all-4 margin-top-2 margin-bottom-2")
            .appendTo(floatDiv)
            .progressbar({ max: maximalniHodnota });
    if (prerusitCallback) {
        $("<div/>")
            .addClass("margin-top-2 margin-bottom-2")
            .append( $("<input/>")
                .attr("type", "button")
                .attr("class", "button")
                .attr("id", "ron_progress_prerusit")
                .val( ron._('Prerusit') )
                .on("click", prerusitCallback)
        )
            .appendTo(floatDiv);
    }
    var progressMsgDiv = $("<div/>")
        .attr("class", "progressmessage")
        .attr("id", "ron_progress_message")
        .css("overflow", "auto")
        .appendTo(floatDiv);


    return {
        setProgress: function (hodnota, text, infoText) {
            if (text) {
                progressText.html(text);
            }
            progressInfoDiv.html(infoText ? infoText : '');

            if (maximalniHodnota) {
                teplomerDiv.progressbar("value", hodnota);
            }

            return this;
        },
        setInfoMessage: function (infoText) {
            progressInfoDiv.html(infoText);

            return this;
        },
        addMessage: function (text) {
            $("<div/>")
                .html(text)
                .appendTo(progressMsgDiv);

            return this;
        },
        setButton: function (text, callback) {
            $("#ron_progress_prerusit")
                .val(text)
                .removeAttr("disabled")
                .off("click")
                .on("click", callback);

            return this;
        },


        close: function() {
            floatDiv.remove();
            floatDiv = null;
            progressText = null;
            progressInfoDiv = null;
            teplomerDiv = null;
            progressMsgDiv = null;
        }
    };
};

(function($) {
    let
        defaultOptions = {
            min: 0,
            max: 100,
            value: 0,

            progressBarClass: "progress-bar"
        };


    function ProgressBar($element, options) {
        this.$element = $element;
        this.options = $.extend(
            {
                min : this.$element.data("min")  ||  defaultOptions.min,
                max :  this.$element.data("max")  ||  defaultOptions.max,
                value : this.$element.data("value")  ||  defaultOptions.value,
                progressBarClass: this.$element.data("progress-bar-class")  ||  defaultOptions.progressBarClass
            },
            options);

        let
            that = this,
            currentValue = null,
            $valueElement =
                $("<div/>")
                    .addClass("progress-bar")
                    .appendTo(this.$element);

        this.value = function(newValue) {
            if (newValue === undefined) {
                return currentValue;
            }

            currentValue = newValue;
            $valueElement.css("width", currentValue / (this.options.max - this.options.min) * 100 + "%");
            this.$element.data("value", newValue);
        };

        if (this.options.value) {
            this.value(this.options.value);
        }
    }
    $.fn.progressbar = function(options, param) {
        let
            action = null;

        if (typeof options === "string") {
            action = options;
        }

        let
            returnValue = this;

        $(this).each(function() {
            let
                $element = $(this),
                instance = $element.data("progressbar-instance");

            if (action) {
                if (!instance) {
                    console.log("Error in progressbar: not initialized!", this);
                }

                switch (action) {
                    case "value" :
                        if (param === undefined) {
                            returnValue = instance.value();
                            return false; // ukonci foreach
                        }
                        instance.value(param);
                        break;

                    case "max" :
                        if (param === undefined) {
                            returnValue = instance.options.max;
                            return false; // ukonci foreach
                        }
                        instance.options.max = param;
                        instance.value(instance.value());
                        break;

                    case "min" :
                        if (param === undefined) {
                            returnValue = instance.options.min;
                            return false; // ukonci foreach
                        }
                        instance.options.min = param;
                        instance.value(instance.value());
                        break;

                    default :
                        console.log("progressbar: unknown action", action);
                }
            }
            if (!instance) {
                instance = new ProgressBar($element, options);
                $element
                    .data("progressbar-instance", instance);
            }

        });

        return returnValue;
    }
})(jQuery);

(function($) {

    $.fn.progressDialog = function(options) {

        let
            that = this,
            action = null,
            defaultOptions = {
                title: null,
                text: null,
                min: 0,
                max: 100,
                value: 0,
                onCancel: null,
                onClose: null,
                onStep: null,
                buttons: ['cancel'], // cancel, close
                cancelled: false,
                id: null
            };


        defaultOptions.id = this.attr("id");

        let originalOptions = this.data('progress-bar-options');
        if (!originalOptions) {
            originalOptions = defaultOptions;
        }

        if (typeof options === "string") {
            action = options;
            options = originalOptions;
        } else {
            options = $.extend(originalOptions, options);
        }

        this.data('progress-bar-options', options);

        function onCancel() {
            options.cancelled = true;
            updateOptions();

            if (options  &&  options.onCancel) {
                options.onCancel.call(that);
            }
        }

        function updateOptions() {
            that.data('progress-bar-options', options);
            return that;
        }


        function clearEvents() {
            that.off("click", "#" + options.id + "_cancel");
            that.off("click", "#" + options.id + "_close");
        }


        function close() {
            that.dialog("close");
            clearEvents();

            if (options  &&  options.onClose) {
                options.onClose.call(that);
            }

            options.cancelled = false; // nastavi false pro moznost dalsiho zobrazeni
            updateOptions();

            return that;
        }
        function setValue(value, text) {
            options.value = value;

            let $progressBar = that.find(".progress-bar");
            $progressBar.css("width", (100 / (options.max - options.min)) * value + "%");

            if (text != null  &&  typeof text != "undefined") {
                $("#" + options.id + "_text").text(text);
            }

            return updateOptions();
        }
        function step(text) {
            if (!options.cancelled  &&  options.value < options.max) {
                setValue(options.value + 1, text);

                if (options.onStep) {
                    options.onStep(options.value);
                }
            } else {
                return close();
            }

            return that;
        }

        function addMessage(text, className) {
            if (text) {
                $("<div/>")
                    .text(text)
                    .attr("class", className)
                    .appendTo( $("#" + options.id + "_messages") );
            }

            return that
        }

        function setMessages(text) {
            $("#" + options.id + "_messages").text(text);
            return that;
        }

        function setButtons(buttons) {
            if (buttons) {
                let
                    btnCancel = false,
                    btnClose = false;

                for (let i = 0;  i < buttons.length;  i++) {
                    switch (buttons[i]) {
                        case "cancel" :
                            btnCancel = true;
                            break;

                        case "close" :
                            btnClose = true;
                            break;
                    }
                }

                $("#" + options.id + "_cancel").toggle(btnCancel);
                $("#" + options.id + "_close").toggle(btnClose);
            }

            return that;
        }

        if (action) {
            switch (action) {
                case "step" :
                    return step(arguments[1]);

                case "close" :
                    return close();

                case "setvalue" :
                    return setValue(arguments[1], arguments[2]);

                case "addmessage" :
                    return addMessage(arguments[1], arguments[2]);

                case "setmessages" :
                    return setMessages(arguments[1]);

                case "setbuttons" :
                    return setButtons(arguments[1]);
            }
        } else {
            if (typeof options.onCancel != "undefined") {
                that.off("click", "#" + options.id + "_cancel");
                that.on("click", "#" + options.id + "_cancel", onCancel);
            }

            if (typeof options.onClose != "undefined") {
                that.off("click", "#" + options.id + "_close");
                that.on("click", "#" + options.id + "_close", close);
            }

            setButtons(options.buttons);

            this
                .dialog({
                    modal: true,
                    resizable: true,
                    width: "auto",
                    height: "auto",
                    closeOnEscape: false
                });
        }

        return this;
    }
})(jQuery);

(function($) {
    let
        defaultOptions = {
            disabled: false,
            start: null,
            resize: null,
            stop: null
        },
        resizable = null,
        originalWidth = null,
        originalHeight = null;

    let RonResizable = function($element, options) {
        let
            that = this,
            startX = null,
            startY = null;
        this.$element = $element;
        this.options = $.extend({}, defaultOptions, options);
        this.onMouseMove = function($e) {
            let newSize = {
                size : {
                    width: originalWidth - startX + $e.originalEvent.screenX,
                    height: originalHeight - startY + $e.originalEvent.screenY
                }
            }

            if (that.options.resize) {
                that.options.resize.call($element, $e, newSize)
            }

            that.$element
                .width(newSize.size.width)
                .height(newSize.size.height);
        };


        this.onMouseUp = function($e) {
            if (that.options.stop) {
                let newSize = {
                    size : {
                        width: originalWidth - startX + $e.originalEvent.screenX,
                        height: originalHeight - startY + $e.originalEvent.screenY
                    }
                }

                that.options.stop.call($element, $e, newSize)
            }
        };

        let
            onMouseDown = function($e) {
                if (that.options.disabled) {
                    return;
                }

                let offset = that.$element.offset();
                resizable = that;
                startX = $e.originalEvent.screenX; // - offset.left;
                startY = $e.originalEvent.screenY; // - offset.top;

                originalWidth = that.$element.width();
                originalHeight = that.$element.height();

                if (that.options.start) {
                    that.options.start.call(that.$element, $e, {
                        size : {
                            width: originalWidth,
                            height: originalHeight
                        }
                    });
                }
            };

        this.$element
            .addClass("resizable")
            .on("mousedown", ".resizable-handle", onMouseDown);

        $("<div/>")
            .addClass("resizable-handle")
            .appendTo($element);
    };

    let
        documentMouseMove = function($e) {
            if (resizable) {
                return resizable.onMouseMove($e);
            }
        },

        documentMouseUp = function($e) {
            if (resizable) {
                resizable.onMouseUp($e);
                resizable = null;
            }
        },

        documentSelectStart = function() {
            if (resizable) {
                return false;
            }
        };

    $(document)
        .on("mousemove", documentMouseMove)
        .on("mouseup", documentMouseUp)
        .on("selectstart", documentSelectStart);

    $.fn.ronresizable = function(options) {
        let
            returnValue = this;

        this.each(function() {
            var
                $element = $(this),
                instance = $element.data("ronresizable-instance");

            if (typeof options === "string") {
                if (!instance) {
                    console.log("ERROR: element is not instance of ronresizable", this);
                    return this;
                }

                switch (options) { // string (action name)
                    case "disable" :
                        instance.options.disabled = true;
                        break;

                    case "enable" :
                        instance.options.disabled = false;
                        break;

                    case "instance" :
                        returnValue = instance;
                        return false; // stop foreach

                    default :
                        console.log("ERROR: action is not implemented in ronresizable", options);
                }

                return this;
            }
            if (!instance) {
                instance = new RonResizable($element, options);
                $element
                    .data("ronresizable-instance", instance);
            }
        });

        return returnValue;
    }
})(jQuery);


ron.PopupCalendarOptions = function() {
    this.submitForm = false;
    this.minDate = undefined;
    this.maxDate = undefined;
    this.nextDateInput = undefined;
    this.prevDateInput = undefined;
};
ron.popupCalendar = function(input, options) {

    let
        parent = $(document.body), // predelat na input.parent?
        textElement = $(input);

    if (!textElement.length) {
        return;
    }

    if (textElement.data('ron-calendar-element')) {
        return;
    }

    if (textElement.attr("disabled")) {
        return;
    }

    textElement.attr("autocomplete", "off");
    let
        submitForm = textElement.data("roncalendar-submit"),    // jestli ma provest submit
        minDateText = textElement.data("roncalendar-mindate"),  // rozsah datumu
        maxDateText = textElement.data("roncalendar-maxdate"),
        textElementTo = null,
        textElementFrom = null;

    if (options  &&  options.nextDateInput) {
        textElementTo = $("[name=" + options.nextDateInput + "]");
    } else if (options  &&  options.prevDateInput) {
        textElementFrom = $("[name=" + options.prevDateInput + "]");
    } else if (textElement.data("roncalendar-next")) {
        textElementTo = $("[name=" + textElement.data("roncalendar-next") + "]");
    } else if (textElement.data("roncalendar-prev")) {
        textElementFrom = $("[name=" + textElement.data("roncalendar-prev") + "]");
    }

    if (textElementTo   &&  !textElementTo.length) {
        textElementTo = null;
    }
    if (textElementFrom &&  !textElementFrom.length) {
        textElementFrom = null;
    }
    if (options) {
        if (options.submitForm !== undefined) {
            submitForm = options.submitForm;
        }
        if (options.minDate !== undefined) {
            minDateText = options.minDate;
        }
        if (options.maxDate !== undefined) {
            maxDateText = options.maxDate;
        }
    }


    let
        minDate = minDateText ? ron.utils.stringToDate(minDateText) : null,
        maxDate = maxDateText ? ron.utils.stringToDate(maxDateText) : null,
        month = null,
        date = null,
        dateTo = null,
        calendarDiv = null,
        headerDiv = null,
        cells = [],
        dates = [];
    var onCellClick = function(cell) {
        let newDate = getDateForCell(cell);
        if (!newDate) {
            return false;
        }

        if (minDate && newDate < minDate) {
            return false;
        }

        if (maxDate && newDate > maxDate) {
            return false;
        }

        let text = newDate.getDate() + '.' + (newDate.getMonth() + 1) + '.' + newDate.getFullYear();

        let changed = textElement.val() !== text;
        textElement.val(text);

        if (textElementTo) {
            if (dateTo && dateTo < newDate) {
                if (textElementTo.val() !== text) {
                    textElementTo.val(text);
                    textElementTo
                        .trigger("change")
                        .trigger("blur");
                }
            }
        } else if (textElementFrom) {
            if (date  &&  date > newDate) {
                if (textElementFrom.val() !== text) {
                    textElementFrom.val(text);
                    textElementFrom
                        .trigger("change")
                        .trigger("blur");
                }

            }
        }

        if (changed) {
            $(textElement)
                .trigger("change")
                .trigger("blur");
        }

        if (submitForm) {
            textElement[0].form.submit();
        }

        return true;
    };


    function showMonth(showDate) {
        let pomDate = new Date(showDate);
        pomDate.setDate(1); // nastavi 1. den v mesici

        month = new Date(pomDate); // zapamatuje si zacatek aktualniho mesice

        let cnt = pomDate.getDay();
        cnt = cnt ? cnt : 7;
        pomDate.setDate(pomDate.getDate() - cnt + 1);

        let today = new Date();

        for (let i = 0; i < cells.length; i++) {
            cells[i].html(pomDate.getDate());
            dates[i] = new Date(pomDate);

            let className = '';
            if (dateTo) {
                if (pomDate >= date  &&  pomDate <= dateTo) {
                    className = 'calendarselected';
                }
            } else {
                if (pomDate <= date && pomDate >= date) {  // == nefunguje!
                    className = 'calendarselected';
                }
            }

            if (pomDate.getDate() == today.getDate() && pomDate.getMonth() == today.getMonth() && pomDate.getYear() == today.getYear()) {
                className += ' calendartoday';
            }

            if ( (minDate  &&  pomDate < minDate) || (maxDate  &&  pomDate > maxDate) ) {
                className += ' calendardisabled';
            }

            cells[i].attr("class", className);
            cells[i].data("calendarIndex", i);
            pomDate = ron.utils.incDayStart(pomDate, 1);
        }

        headerDiv.html( (month.getMonth() + 1) + '/' + month.getFullYear() );
    }



    function setDate(value) {
        date = new Date(value);

        showMonth(value);
    }


    function setDateText(text) {
        let date = ron.utils.stringToDate(text);
        if (date  &&  !isNaN(date)) {
            setDate(date);
        }
    }


    function setDateRange(from, to) {
        date = new Date(from);
        dateTo = new Date(to);

        showMonth(textElementTo ? date : dateTo);
    }


    function setDateRangeText(textfrom, textto) {
        let
            dateFrom = ron.utils.stringToDate(textfrom),
            dateTo = ron.utils.stringToDate(textto);

        if (dateFrom && minDate && dateFrom < minDate) {
            dateFrom = minDate;
        }
        if (dateFrom && maxDate && dateFrom > maxDate) {
            dateFrom = maxDate;
        }

        if (dateTo && minDate && dateTo < minDate) {
            dateTo = minDate;
        }
        if (dateTo && maxDate && dateTo > maxDate) {
            dateTo = maxDate;
        }

        if (dateFrom && dateTo) {
            setDateRange(dateFrom, dateTo);
        } else if (dateFrom) {
            setDate(dateFrom);
        }
    }
    function getDateForCell(cell) {
        let index = parseInt($(cell).data("calendarIndex"));
        return dates[index];
    }



    function onClose() {
        setTimeout( function() {
            textElement.data("ron-calendar-element", null);
        }, 500);
        calendarDiv = null;
    }


    function hide() {
        calendarDiv.ronPopup("close");
    }


    function show() {
        calendarDiv = $("<div/>")
            .attr(
            {
                "class": "popup dropdown-menu",
            });

        calendarDiv
            .on("click", function () { return false; })

        calendarDiv.appendTo(parent);
        let
            topdiv = $("<div/>")
                .attr("class", "calendarheader")
                .css("width", "100%");

        calendarDiv.append(topdiv);


        $("<span/>")
            .attr("class", "calendarbutton")
            .css("float", "left")
            .html("&nbsp;<&nbsp;")
            .appendTo(topdiv)
            .on("click", function () {
                if (minDate) {
                    if (month <= minDate) {
                        return false;
                    }
                }

                showMonth(ron.utils.incMonth(month, -1));
                return false;
            });

        $("<span/>")
            .attr("class", "calendarbutton")
            .css("float", "right")
            .html("&nbsp;>&nbsp;")
            .appendTo(topdiv)
            .on("click", function () {
                let newMonth = ron.utils.incMonth(month, 1);
                if (maxDate && newMonth > maxDate) {
                    return false;
                }

                showMonth(newMonth);
            });


        headerDiv = $("<span/>")
            .html("&nbsp;")
            .appendTo(topdiv);
        let table = $("<table/>")
            .attr("class", "calendartable")
            .appendTo(calendarDiv);
        let headerRow = $("<tr/>")
            .appendTo(table)
            .addClass("border-bottom border-dark");

        for (let den = 0; den < 7; den++) {
            let header = $("<th/>");
            header.html( ron._('Day_short_' + ((den + 1) % 7)) );

            headerRow.append(header);
        }
        console.log("renderuju data ...");
        for (let tyden = 0; tyden < 6; tyden++) {
            let row = $("<tr/>")
                .appendTo(table);

            for (let den = 0; den < 7; den++) {
                let cell = $("<td/>")
                    .appendTo(row);

                cell.on("click", function () {
                    if (onCellClick(this)) {
                        hide();
                    }
                });

                cells[cells.length] = cell;
            }
        }

        let pomDate = new Date();
        if (minDate && pomDate < minDate) {
            setDate(minDate);
        } else if (maxDate && pomDate > maxDate) {
            setDate(maxDate);
        } else {
            setDate(pomDate);
        }

        let
            pos = textElement.offset(),
            inputHeight = textElement.get(0).offsetHeight, // height() vraci jen vnitrni velikost
            viewRect = ron.utils.windowViewRect();
        if (pos.top + inputHeight + calendarDiv.outerHeight() + 4 >= viewRect.y2) {
            pos.top -= calendarDiv.outerHeight();
        } else {
            pos.top += inputHeight;
        }

        calendarDiv
            .css("left", pos.left + "px")
            .css("top", pos.top + "px");

        calendarDiv
            .css("display", "block");

        textElement.data("ron-calendar-element", calendarDiv);

        calendarDiv.ronPopup({
            closeAction: "remove",
            onClose: onClose
        });
    }
    show();

    if (textElementTo) {
        setDateRangeText(textElement.val(), textElementTo.val());
    } else if (textElementFrom) {
        setDateRangeText(textElementFrom.val(), textElement.val());
    } else {
        setDateText(textElement.val());
    }
};


(function($) {
    function showCalendar(event) {
        let $this = $(this);
        let inputSelector = $this.data("roncalendar-input");

        if (!inputSelector) {
            let inputName = $this.data("roncalendar-inputname");
            if (!inputName) {
                return;
            }

            inputSelector = "input[name='" + inputName + "']";
        }

        let
            $parentForm = $this.closest("form"),
            $input = null;
        if ($parentForm.length) {
            $input = $parentForm.find(inputSelector);
        }

        if (!$input  ||  !$input.length) {
            $input = $(inputSelector);
        }

        if (!$input.length) {
            return;
        }

        ron.popupCalendar($input[0]);
        return false; // nepropagovat udalost
    }

    $(function() {
        $(document)
            .on("click", ".showcalendar", showCalendar)
            .on("click", "input.date-input", function() { ron.popupCalendar(this); })
    });
})(jQuery);

$(function() {

    function setFieldReadonly($field, readOnly)
    {
        if (readOnly) {
            $field.attr("readonly", "readonly");
        } else {
            $field.removeAttr("readonly");
        }
    }

    let
        setDefaultValues = function($form, $initiator) {
            $form.find(".ronform-field").each(function () {
                let
                    $field = $(this),
                    fieldName = $field.data("ronform-field-name"),
                    defaultValue = $field.data("ronform-default-value");

                if (fieldName === undefined) {
                    fieldName = $field.attr("name");
                }

                if (fieldName  &&  $initiator) {
                    let initiatorValue = $initiator.data("field-" + fieldName + "-value");
                    if (initiatorValue !== undefined) {
                        defaultValue = initiatorValue;
                    }
                }

                if (defaultValue !== undefined) {
                    if ($field.is("input, select, textarea")) {
                        if ($field.is(":checkbox")) {
                            $field.prop("checked", defaultValue && defaultValue != 0);
                        } else if ($field.is(":radio")) {
                            $field.prop("checked", defaultValue == $field.attr("value"));
                        } else if ($field.is(":file")) {
                            if (defaultValue === "") {
                                $field.get(0).value = null;
                            }
                        } else {
                            $field.val(defaultValue);
                        }
                        $field.trigger("change");
                    } else {
                        $field.text(defaultValue);
                    }
                }

                setFieldReadonly($field, false);
            });
        },
        setFieldValues = function($form, data) {
            $form.find(".ronform-field").each(function () {
                let
                    $field = $(this),
                    fieldName = $field.data("ronform-field-name"),
                    readOnly = $field.data("readonly");

                if (fieldName === undefined) {
                    fieldName = $field.attr("name");
                }

                if (fieldName) {
                    if (data[fieldName] !== undefined) {
                        if ($field.is("input, select, textarea")) {
                            if ($field.is(":checkbox")) {
                                $field.prop("checked", data[fieldName] && data[fieldName] != 0);
                            } else if ($field.is(":radio")) {
                                $field.prop("checked", data[fieldName] == $field.attr("value"));
                            } else {
                                console.log("Nastavuji hodnotu", fieldName, data[fieldName]);
                                $field.val(data[fieldName]);
                            }
                            $field.trigger("change");
                        } else if ($field.data("raw-html"))  {
                            $field.html(data[fieldName])
                        } else {
                            $field.text(data[fieldName]);
                        }
                    }

                    if (readOnly) {
                        setFieldReadonly($field, true);
                    }
                }
            });
        },
        loadData = function($form, url) {
            $form.find(".ronform-submit").prop("disabled", true);
            $.ajax(url)
                .done(function (resultObject) {
                    if (resultObject.data) {
                        setFieldValues($form, resultObject.data);
                        $form.trigger("dialogdataloaded");
                    }

                    if (resultObject.result === "OK") {
                        $form.find(".ronform-submit").prop("disabled", false);
                    }  else {
                        ron.showError(resultObject);
                    }
                });

        },
        loadFormData = function($form, $initiator) {
            setDefaultValues($form, $initiator);

            let
                url = $initiator
                    ? $initiator.data("ronform-url")
                    : null,
                record = $initiator
                    ? $initiator.data("ronform-record")
                    : null,
                initiatorUrl = url;

            if (!url) {
                url = $form.data("ronform-url");
            }

            $form.find("input[name=formaction]").val(record ? "save" : "insert");

            if (initiatorUrl  ||  (url  &&  record) ) {
                loadData($form, url + (record ? record : ''));
            }
        },
        beforeShow = function(event) {
            let
                $form = $(this),
                $button = $(event.relatedTarget); // Button that triggered the modal

            loadFormData($form, $button);
        },


        afterShow = function(event) {
            let
                $form = $(this),
                $button = $(event.relatedTarget); // Button that triggered the modal
            let
                fieldId = $button.data("ronform-focus-field");

            if (fieldId) {
                $("#" + fieldId).focus();
            } else {
                $form.find(".ronform-focus").focus();
            }
        },
        onDialogOpen = function(event) {
            let
                $form = $(this),
                button = $form.data("initiator"),
                $button = button ? $(button) : null;

            loadFormData($form, $button);
        },


        showError = function(error) {
            if (ron && ron.showError) {
                ron.showError(error);
            } else {
                alert(error);
            }
        },


        handleErrors = function($form, errors) {
            var
                $fields = $form.find(".ronform-field");

            $fields.removeClass("alert-danger error");

            var msg = "";
            for (var i = 0;  i < errors.length;  i++) {
                if (errors[i].errorField) {
                    $fields.filter("[name='" + errors[i].errorField + "']").toggleClass("alert-danger error", true);
                    console.log("Chyba v poli " + errors[i].errorField, errors[i]);
                } else {
                    console.log("Chyba", errors[i])
                }

                msg = msg + errors[i].errorText + "\n";
            }

            showError(msg);
        },
        closeForm = function($form) {
            $form.find(".ronform-submit").prop("disabled", false);
            if ($form.is(".ui-dialog-content")) {
                $form.dialog("close");
            } else {
                $form.modal("hide"); // bootstrap
            }
        },
        enableSubmits = function($form, enable) {
            $form.find(".ronform-submit").prop("disabled", !enable); // zabrani druhemu submitu
        },


        submitForm = function($form, url) {
            enableSubmits($form, false);

            var
                data = {},
                fieldSelector = $form.data("ronform-field-selector") || ".ronform-field";

            $form.find(fieldSelector).each(function () {
                var
                    $field = $(this),
                    fieldName = $field.data("ronform-field-name") || $field.attr("name");

                if (fieldName) {
                    if ($field.is("input, select, textarea")) {
                        if ($field.is(":radio")) {
                            if (this.checked) {
                                data[fieldName] = $field.val();
                            }
                        }else if ($field.is(":checkbox")) {
                            data[fieldName] = this.checked ? 1 : 0;
                        } else {
                            data[fieldName] = $field.val();
                        }
                    }
                }
            });

            console.log("ronform - send data to url ", url, data);

            $.ajax({
                url: url,
                type: "post",
                dataType: "json",
                data: data
            }).fail(function (jqXHR) {
                showError("Error " + jqXHR.status + ": " + jqXHR.statusText);
            }).done(function (resultObject) {
                console.log(resultObject);
                if (resultObject.errors && resultObject.errors.length) {
                    handleErrors($form, resultObject.errors);
                    enableSubmits($form, true);
                } else {
                    switch ($form.data("ronform-after-submit")) {
                        case "reload" :
                            window.location.reload();
                            break;

                        default :
                            closeForm($form);
                    }
                }
            }).always(function () {
                enableSubmits($form, true);
            });
        },
        submitFormClick = function() {
            var
                $this = $(this),
                $form = $this.closest('form'),
                $ronform = $this.closest('.ronform');

            if (!$ronform.length) {
                return;
            }

            var
                url = $(this).data("ronform-submit-url");

            if (url === undefined) {
                url = $ronform.data("ronform-submit-url");
            }
            if (url === undefined) {
                url = $form.data("ronform-submit-url");
            }

            if (url) {
                submitForm($ronform, url);
                return false;
            } else {
                console.log("Chyba v ronform-submit - neni zadana URL!", this);
                closeForm($ronform);
            }

        },
        openDialogClick = function() {
            let
                $this = $(this),
                $dialog = $($this.data("dialog")),
                title = $this.data('title');

            $dialog.data("initiator", this);

            $dialog.dialog({
                modal: true,
                width: "auto",
                minWidth: parseInt($dialog.css("minWidth")),
                relatedTarget: this
            });

            return false;
        },
        dialogCancel = function() {
            $(this).closest(".rondialog,.ronform,.modal").dialog("close");
        };

    $(document)
        .on("show.bs.modal", ".ronform", beforeShow)    // bootstrap
        .on("shown.bs.modal", ".ronform", afterShow)
        .on("dialogopen", ".ronform", onDialogOpen)     // jQui
        .on("click", ".ronform-submit", submitFormClick)
        .on("click", ".open-dialog", openDialogClick)
        .on("click", "input.cancel,  button.cancel", dialogCancel);

});

(function($) {
    let
        defaultOptions = {
            dataUrl: null
        };


    function formatter(format) {
        return function(value, ctx) {

            switch (format) {
                case "ronlabels" :
                    return ctx.dataset.ronlabels ? ctx.dataset.ronlabels[ctx.dataIndex] : "";

                case "none" :
                    return "";

                case "plain" :
                default :
                    return value;
            }
        }
    }

    function RonGraph($element, options) {
        let
            that = this;

        this.$element = $element;

        let
            dataOptions = {
                dataUrl: $element.data("data-url")
            }
        this.options = $.extend({}, defaultOptions, dataOptions, options);

        this.loadData = function() {
            if (!this.options.dataUrl) {
                return;
            }

            $.ajax(this.options.dataUrl)
                .done(function(resultData) {
                    if (resultData.graphdata.graphlabelsformat) {
                        resultData.graphdata.options = resultData.graphdata.options || {};
                        resultData.graphdata.options.plugins = resultData.graphdata.options.plugins || {};
                        resultData.graphdata.options.plugins.datalabels = resultData.graphdata.options.plugins.datalabels || {};
                        resultData.graphdata.options.plugins.datalabels.formatter = formatter(resultData.graphdata.graphlabelsformat);
                    }

                    if (resultData  &&  resultData.graphdata) {
                        new Chart($element.get(0), resultData.graphdata);
                    }
                })
        }



        this.loadData();
    }

    $.fn.rongraph = function(options) {
        let
            returnValue = this;

        this.each(function() {
            let
                $element = $(this),
                instance = $element.data("rongraph-instance");

            if (typeof options === "string") {
                if (!instance) {
                    console.log("ERROR: element is not instance of rongraph", this);
                    return this;
                }

                switch (options) { // string (action name)
                    case "instance" :
                        returnValue = instance;
                        return false; // stop foreach

                    default :
                        console.log("ERROR: action is not implemented in rongraph", options);
                }

                return this;
            }
            if (!instance) {
                instance = new RonGraph($element, options);
                $element
                    .data("rongraph-instance", instance);
            }
        });

        return returnValue;
    }


    function processGraph() {
    }

    $(function() {
        if (!window.Chart) {
            console.log("Knihovna chart.js neni pripojena");
            return;
        }

        Chart.register(ChartDataLabels);
        $("canvas.graph").each(processGraph);
    })
})(jQuery);


(function($) {

    function createGraph()
    {
        let
            $this = $(this),
            $target = null,
            $canvas = null;
        
        if ($this.data("graph-target")) {
            $target = $($this.data("graph-target"));
            if (!$target.length) {
                return;
            }
        }

        if ($target && $target.is("canvas")) {
            $canvas = $target;
        } else {
            $canvas = $("<canvas/>");

            $canvas.appendTo($target ?? $this.parent());
        }


        let
            labels = [],
            values = [];

        $this.find(".graph-value").each(function() {
            let $value = $(this);
            labels[labels.length] = $value.data("graph-label");
            values[values.length] = $value.data("graph-value");
        });

        let config = {
            type: $this.data("graph-type") ?? 'bar',
            data: {
                labels: labels,
                datasets: [
                    {
                        label: '',
                        data: values
                    }
                ]
            },
            options: {
                plugins: {
                    legend: {
                        position: "left",
                        display: $this.data("graph-type") !== 'bar'
                    }
                }
            }
        }
        new Chart($canvas.get(0), config);
    }


    $.fn.graphFromElement = function(options) {

        this.each(function() {
            createGraph.call(this);
        });

        return this;
    }

    $(function() {
        $(".graph-source")
            .graphFromElement();
    })
})(jQuery);

(function($) {
    let
        $activePopup = null,
        activePopupOptions = null,
        $wrapElement = null,
        $originalParent = null;



    function closePopup() {
        if (!$activePopup) {
            return;
        }

        if (activePopupOptions  &&  activePopupOptions.onClose) {
            let closeResult = activePopupOptions.onClose.call($activePopup, activePopupOptions);

            if (closeResult === false) {
                return;
            }
        }

        let closeAction = activePopupOptions.closeAction  // muze byt hide / remove / none
            ? activePopupOptions.closeAction
            : "hide";

        switch (closeAction) {
            case "hide" :
                if (activePopupOptions.wrap && $originalParent) {
                    $activePopup.detach();
                    $originalParent.append($activePopup);

                    $wrapElement.hide();
                } else {
                    $activePopup.toggleClass("visible show", false); // show pro bootstrap
                }
                break;

            case "remove" :
                $activePopup.remove();
                break;
        }

        $activePopup = null;
        activePopupOptions = null;
        $originalParent = null;
    }

    $(document)
        .on("mousedown", function() {
            closePopup();
        })
        .on("keydown", function(e) {
            if ($activePopup  &&  activePopupOptions  &&  activePopupOptions.exitOnESC  &&  e.which === 27) {
                closePopup();
            }
        });

    $.fn.ronPopup = function(options) {
        let defaultOptions = {
            action : null,
            closeAction : "hide",
            exitOnESC : true,
            position : null,        // Pozice, na kterou se element umisti
            offset : null,
            attachToElement : null,
            attachPosition : 'left',
            positioning : 0,
            noFocus : false,
            wrap : false,
            onClose : null
        };

        options = $.extend( defaultOptions,
            typeof options === "string"
                ? { action: options }
                : options
        );

        if (options.action === "close") {
            if (this.is($activePopup)) {
                closePopup();
            }
            return this;
        } else if (options.action === "closeAll") {
            closePopup();
            return this;
        }


        if ($activePopup && $activePopup.is(this)) {
            return this;
        }

        if ($activePopup) {
            closePopup();
        }

        $activePopup = this;
        activePopupOptions = options;
        $originalParent = options.wrap
            ? this.parent()
            : null;

        if (options.wrap) {
            if (!$wrapElement) {
                $wrapElement = $("<div/>")
                    .addClass("popup")
                    .addClass("popup-wrapper")
                    .appendTo(document.body);
            }

            $wrapElement.append(this);
            $wrapElement.show();
        }

        let
            windowRect = ron.utils.windowViewRect(),
            $popupElement = options.wrap ? $wrapElement : this,
            showOnTop = false;

        $popupElement
            .addClass("popup")
            .on("mousedown", function(event) {
                event.stopPropagation(); // aby to neproslo az na dokument, coz popup okno skryje!
            } )
            .on("closepopup", function() {
                closePopup(); // zajisti odregistrovani udalosti a skryti/zruseni elementu
            } )
            .toggleClass("visible show", true) // show pro bootstrap
        ;

        if (options.position) {
            $popupElement
                .css( {
                    top : options.position.top + "px",
                    left: options.position.left + "px"
                });

        } else if (options.attachToElement) {
            let
                position = options.attachToElement.position(),
                cssPosition = "";
                let
                    $ofsParent = $popupElement.offsetParent(),
                    ofsPosition = $ofsParent.offset();

                position = $(options.attachToElement).offset();

                position.left -= ofsPosition.left;
                position.top -= ofsPosition.top;

                if (options.attachPosition === "right") {
                    position.left = position.left + options.attachToElement.outerWidth() - $popupElement.outerWidth();
                    if (position.left < 0) {
                        position.left = 0;
                    }
                }

            let
                attachElementTop = position.top;

            position.top += options.attachToElement.outerHeight(true); // include margin

            if (!options.positioning  &&  windowRect.y2 - position.top < 250) {
                if (attachElementTop - windowRect.y1 > windowRect.y2 - position.top) {
                    showOnTop = true;
                }
            }

            if (showOnTop) {
                $popupElement
                    .css("bottom", ""); // aby se nastavila automaticka velikost (kvuli zjisteni vysky elementu)

                let
                    bottom = $(window).height() - attachElementTop,
                    top = windowRect.y1 + 5;

                if (attachElementTop - top  > $popupElement.outerHeight()) {
                    top = attachElementTop - $popupElement.outerHeight();
                }

                $popupElement
                    .css({
                        position: cssPosition,
                        bottom: bottom + "px",
                        left: position.left + "px",
                        top: top + "px",
                        "max-height": ""
                    });
            } else {
                $popupElement
                    .css({
                        position: cssPosition,
                        top: position.top + "px",
                        left: position.left + "px",
                        bottom: ''
                    });
            }
        }

        if (options.offset) {
            $popupElement.offset(options.offset);
        }
        let
            rect = ron.utils.getAbsoluteRect($popupElement); // pozor! pokud okno obsahuje hodne elementu, tak $().offset() v getAbsoluteRect trva dlouho!

    if (options.positioning === 1) {
            if (rect.y2 >= windowRect.y2) {
                let newTop = windowRect.y2 - (rect.y2 - rect.y1) - 10;
                if (newTop < 1) {
                    newTop = 1;
                }
                $popupElement.css("top", newTop + "px");
            }
        } else if (!showOnTop) {
            let size = rect.y2 - rect.y1 + 1;
            size -= (rect.y2 - windowRect.y2) + 10;

            $popupElement.css("max-height", size + "px");
        }

        if (!options.noFocus) {
            $popupElement.focus();
        }

        return this;
    };
})(jQuery);

(function($) {
    var hideMenu = function() {
        $(document).ronPopup("closeAll");
    };

    var
        $popupMenu = null,
        $selectedMenuItem = null,
        currentMenuOptions = null;

    var unselectItem = function() {
        if ($selectedMenuItem) {
            $selectedMenuItem.removeClass("selected");
            $selectedMenuItem = null;
        }
    };

    var selectItem = function($newItem) {
        if (!$newItem.length) {
            return;
        }

        unselectItem();

        $newItem.addClass("selected");
        $selectedMenuItem = $newItem;
    };


    var onKeyDown = function(e) {
        if (!$popupMenu) return;

        switch (e.which) {
            case 40:  // dolu
                if (!$selectedMenuItem) selectItem($popupMenu.children().first());
                else selectItem($selectedMenuItem.next());
                return false;

            case 38: // nahoru
                if (!$selectedMenuItem) selectItem($popupMenu.children().last());
                else selectItem($selectedMenuItem.prev());
                return false;

            case 34: // page down
                selectItem($popupMenu.children().last());
                return false;

            case 33: // page up
                selectItem($popupMenu.children().first());
                return false;

            case 13: // enter
                if (currentMenuOptions  &&  currentMenuOptions.onClick) {
                    var result = currentMenuOptions.onClick.call($selectedMenuItem.get(0), e, currentMenuOptions.data);
                    if (result === false) {
                        return false;
                    }
                }

                $selectedMenuItem.trigger("activate", currentMenuOptions ? currentMenuOptions.data : null);

                hideMenu();

                return false;
        }
    };


    var onMouseOver = function(e) {
        if (!$popupMenu) return;

        $popupMenu.children().removeClass("selected");
        $selectedMenuItem = null;
    };
    $.fn.ronPopupMenu = function(options) {
        if (!this.length) return;

        unselectItem();

        var that = this;

        if ($popupMenu) {
            $popupMenu
                .off("keydown", onKeyDown)
                .off("mouseover", onMouseOver)
            ;
        }

        var defaultOptions = {

            action : null,
            closeAction : "hide",
            exitOnESC : true,
            offset : null,
            position : null,
            attachToElement : null,
            mouseEvent : null,
            positioning : 1,
            noFocus : false,
            wrap : false,
            onClose : null,
            onClick : null,
            data : null

        };

        options = $.extend( defaultOptions, options );

        if (options.attachToElement) {
            this
                .css("min-width", ( options.attachToElement.outerWidth() - 2) + "px");
        }

        let position = null;
        if (options.position) {
            position = options.position;
        } else if (!options.attachToElement  && options.mouseEvent) {
            this
                .detach()
                .appendTo("body");

            position = {
                top:  options.mouseEvent.clientY,
                left: options.mouseEvent.clientX
            }
        }

        $popupMenu = this;
        currentMenuOptions = options;

        this
            .off("click", ">")
            .on("click", ">", function(event) {
                hideMenu();

                if (options.onClick) {
                    var result = options.onClick.call(this, event, options.data);
                    if (result === false) {
                        return false;
                    }
                }

                $(this).trigger("activate", options.data);
            })
            .on("keydown", onKeyDown)
            .on("mouseover", onMouseOver)
        ;

        if (!this.attr("tabIndex")) {
            this.attr("tabIndex", -1);
        }
        this
            .ronPopup({
                attachToElement: options ? options.attachToElement : null,
                offset: options ? options.offset : null,
                position: position,
                positioning: 1,  // posunout, pokud se nevleze na obrazovku
                wrap: options ? options.wrap : false,
                onClose: options.onClose
            })
            .focus();
    };

})(jQuery);

(function() {
    let
        defaultOptions = {
            inputClass: null,
            insertPosition: null,
            items: null,
            itemsUrl: null,
            autoLoadItems: false,
            enableUrlCache: false,
            maxVisibleRows: null,
            dropDownAlign: null,
            columns: null,
            filterColumns: null,
            captions: null,
            textColumn: null,
            valueColumn: null,
            initialValue: null,
            onlyAutocomplete: false,

            caseSensitive: false,

            fullText: true,
            dropDownElement: null,
            parentElement: null,
            itemIndex: null,
            onDropDown: null,
            onChange: null,
            onItemActivate: null
        };


    function RonSelect($element, options) {
        this.$element = $element;
        this.options = $.extend({}, defaultOptions, options);

        var
            that = this,
            $editElement = null,
            $dropDownElement = null,
            dropDownTable = null,
            $table = null,
            $captionsElement = null,
            originalValue = null,

            dropDownFilter = null,

            init = function () {
                that.options.itemIndex = null;
                that.options.inputClass = that.options.inputClass || that.$element.data("input-class") || "form-control";
                that.options.insertPosition = that.options.insertPosition || that.$element.data("insert-position") || "after";

                if (!that.options.maxVisibleRows) {
                    let hlpStr = that.$element.data("max-visible-rows");
                    if (hlpStr) {
                        that.options.maxVisibleRows = parseInt(hlpStr);
                    }
                }

                that.options.onlyAutocomplete = that.options.onlyAutocomplete || that.$element.data("only-autocomplete");
                that.options.caseSensitive = that.options.caseSensitive || that.$element.data("case-sensitive") == "1";
                that.options.itemsUrl = that.options.itemsUrl || that.$element.data("items-url");
                that.options.autoLoadItems = that.options.autoLoadItems || that.$element.data("auto-load-items");
                that.options.enableUrlCache = that.options.enableUrlCache || that.$element.data("enable-url-cache") == "1";
                that.options.dropDownAlign = that.options.dropDownAlign || that.$element.data("drop-down-align") || "left";
                if (!that.options.columns) {
                    let columnsStr = that.$element.data("columns");
                    if (columnsStr) {
                        that.options.columns = columnsStr.split(";");
                    }
                }

                if (!that.options.filterColumns) {
                    let filterColumnsStr = that.$element.data("filter-columns");
                    if (filterColumnsStr) {
                        that.options.filterColumns = filterColumnsStr.split(";");
                    }
                }

                that.options.textColumn = that.options.textColumn || that.$element.data("text-column");
                that.options.valueColumn = that.options.valueColumn || that.$element.data("value-column");

                if (!that.options.captions) {
                    let captionsStr = that.$element.data("captions");
                    if (captionsStr) {
                        that.options.captions = captionsStr.split(";");
                    }
                }

                if (that.$element.data('fulltext') == "0") {
                    that.options.fullText = false;
                }

                if (that.$element.is("select")) {
                    createEdit();
                } else {
                    $editElement = that.$element;
                    $editElement
                        .attr("autocomplete", "off")
                        .addClass("form-select");
                }

                setKeyMouseEvents();

                $editElement.attr("bs-toggle", "dropdown");

                setEvents();

                if (that.options.initialValue) {
                    let index = that.getItemIndex(that.options.initialValue);

                    setElementValue(index);
                    that.options.itemIndex = index;
                }

                if (that.options.autoLoadItems && that.options.itemsUrl) {
                    that.loadData(that.options.itemsUrl, false);
                }
            },


            selectItemByText = function() {
                let newIndex = that.getItemIndex($editElement.val(), getTextColumnIndex());
                if (newIndex !== null) {
                    that.setItemIndex(newIndex, true);
                    return true;
                }

                return false;
            },
            createItems = function ($options) {
                let
                    result = [];

                $options.each(function () {
                    let
                        $option = $(this),
                        item = null;

                    if (that.options.columns) {
                        item = {
                            value: $option.attr("value"),
                            text: "" + $option.text(),
                            title: $option.attr("title"),
                            class: $option.attr("class")
                        };

                        for (let i = 0; i < that.options.columns.length; i++) {
                            if (that.options.columns[i] && that.options.columns[i] !== "value" && that.options.columns[i] !== "text") {
                                item[that.options.columns[i]] = "" + ($option.data(that.options.columns[i]) ? $option.data(that.options.columns[i]) : "");
                            }
                        }

                    } else {
                        item = [$option.attr("value"), $option.text()];
                    }

                    if (item) {
                        result.push(item);
                    }
                });

                if (!result.length && that.options.itemsUrl) {
                } else {
                    if (that.options.columns) {
                        that.options.valueColumn = that.options.valueColumn || "value";
                        if (that.options.textColumn == null || typeof that.options.textColumn == "undefined") {
                            that.options.textColumn = "text";
                        }
                    } else {
                        that.options.valueColumn = 0;
                        that.options.textColumn = 1;
                    }

                    that.options.items = result;
                }
            },
            createEdit = function () {
                if (!that.$element.is("select")) {
                    console.log("ronselect - error in createEdit - element is not <select>")
                }

                $editElement = $("<input/>");

                $editElement
                    .attr("autocomplete", "off")
                    .attr("class", that.$element.attr("class"))     // EDIT 02-2023
                    .removeClass("ronselect")
                    .addClass("form-select")
                    .addClass(that.options.inputClass)
                    .attr("title", that.$element.attr("title"));

                switch (that.options.insertPosition) { // EDIT 04-2023
                    case 'before' :
                        $editElement.insertBefore(that.$element);
                        break;

                    case 'append' :
                        $editElement.appendTo(that.$element.parent());
                        break;

                    case 'prepend' :
                        $editElement.prependTo(that.$element.parent());
                        break;

                    case 'after' :
                    default :
                        $editElement.insertAfter(that.$element);
                }

                if (that.$element.attr("disabled")) {
                    $editElement.attr("disabled", "disabled");
                }

                that.$element.css("display", "none");

                if (!that.options.items) {
                    createItems(that.$element.children("option"));
                }

                let itemIndex = that.getItemIndex(that.$element.val());
                setElementValue(itemIndex);
                that.options.itemIndex = itemIndex;

                that.$element.on("change", function () {
                    setElementValue(that.getItemIndex(that.$element.val()));
                });
            },


            setKeyMouseEvents = function() {
                function keyEvent(e) {
                    if (originalValue === null) {
                        originalValue = $editElement.val();
                    }

                    if (e.which === 13 && !dropDownTable.$activeRow) {
                        if (selectItemByText()) {
                            that.closeDropDown();
                        }
                    }

                    if (that.dropDownVisible()) {
                        if (e.which === 27) {
                            that.closeDropDown();
                            $editElement.val(originalValue);
                            return false;
                        }
                        if ($table) {
                            $table.triggerHandler(e);
                        }

                        e.stopPropagation();
                    } else {
                        switch (e.which) {
                            case 38 : // up
                            case 40 : // down
                                that.dropDown();
                        }
                    }

                }


                function mouseDownEvent(e) {
                    if (!that.dropDownVisible()) {
                        that.dropDown();

                        $editElement.select();

                        return false; // pokud to tu neni tak hned popup okno zavre, protoze se provede onmousedown na dokumentu
                    }
                }

                $editElement
                    .on("keydown", keyEvent)
                    .on("keyup", keyEvent)
                    .on("keypress", keyEvent)
                    .on("mousedown", mouseDownEvent)
                ;
            },
            setEvents = function () {
                let
                    blurTimer = null,

                    onBlur = function () {

                        blurTimer = setTimeout(
                            function () {
                                blurTimer = null;
                                that.closeDropDown();

                                if (!selectItemByText()) {
                                    if (!that.options.onlyAutocomplete) {
                                        $editElement.val(originalValue);
                                    }
                                }
                            },
                            200);
                    },

                    onFocus = function () {
                        if (blurTimer) {
                            clearTimeout(blurTimer);
                            blurTimer = null;
                        }
                    },
                    onTextChange = function (e) {
                        if (!that.dropDownVisible()) {
                            that.dropDown();
                        }

                        if (dropDownTable) {
                            dropDownTable.filter($editElement.val());
                        }
                    },


                    onItemActivateEvent = function () {
                        if (that.options.onItemActivate || that.options.onChange) {
                            let index = that.getItemIndex($editElement.val(), getTextColumnIndex());
                            if (that.options.onItemActivate) {
                                that.options.onItemActivate.call(that.$element, index, that.options);
                            }
                            if (that.options.onChange) {
                                that.options.onChange.call(that.$element, index, that.options);
                            }
                        }
                    };


                $editElement
                    .on("focus", onFocus)
                    .on("blur", onBlur)
                    .on("itemactivate", onItemActivateEvent)
                    .on("input", onTextChange)
                    .attr("autocomplete", "off"); // zabrani zobrazeni drive zadanych hodnot pri kliknuti mysi

            },

            doDropDown = function () {
                if (!$dropDownElement) {
                    createDropDownElement();
                }
                let $attachTo = $editElement;

                $dropDownElement
                    .css("min-width", $attachTo.outerWidth() + "px");
                if ($dropDownElement) {
                    $dropDownElement.ronPopup({
                        attachToElement: $attachTo,
                        attachPosition: that.options.dropDownAlign,
                        noFocus: true
                    });
                }

                if ($dropDownElement.width() > $attachTo.outerWidth()) {
                    $dropDownElement
                        .css("min-width", $dropDownElement.outerWidth() + "px"); // aby se pri vyhledavani podretezce nezmensoval (vadi to hlavne pri zarovnani doprava)
                }

                dropDownTable.filter(dropDownFilter);

                if (that.options.itemIndex !== null) {
                    dropDownTable.setActiveRowIndex(that.options.itemIndex);
                } else if (dropDownTable && that.options.textColumn) {
                    dropDownTable.setActiveItem(that.options.textColumn, $editElement.val());
                }

                $editElement.focus();
            },


            updateItems = function (items) {
                that.options.items = items;

                if (that.$element.is("select")) {
                    that.$element.empty();

                    for (let i = 0; i < items.length; i++) {
                        $("<option/>")
                            .attr("value", items[i][getValueColumnIndex()])
                            .text(items[i][getTextColumnIndex()])
                            .appendTo(that.$element);
                    }
                }
            },
            getTextColumnIndex = function () {
                var colIndex = that.options.textColumn;
                if (typeof colIndex == "undefined" && that.options.columns) {
                    colIndex = that.options.columns[0];
                }
                if (typeof colIndex == "undefined") {
                    return 0;
                }

                return colIndex;
            },
            getValueColumnIndex = function () {
                if (typeof that.options.valueColumn != "undefined" && that.options.valueColumn !== null) {
                    return that.options.valueColumn;
                }

                return getTextColumnIndex();
            },


            setElementValue = function (itemIndex) {
                if (itemIndex === null || !that.options.items) {
                    $editElement.val("");
                } else {
                    if (itemIndex < 0 || itemIndex >= that.options.items.length) {
                        return false;
                    }
                    var
                        colIndex = getTextColumnIndex();

                    originalValue = that.options.items[itemIndex][colIndex];
                    $editElement.val(originalValue);
                }

                return true;
            };


        function createDropDownElement() {
            let
                blurTimer = null,

                onBlur = function () {
                    if ($(this).filter(":hover").length) {
                        return;
                    }

                    blurTimer = setTimeout(
                        function () {
                            blurTimer = null;
                            that.closeDropDown();
                            $editElement.val(originalValue);
                        },
                        200);
                },

                onDropDownFocus = function () {
                    $editElement.focus();
                },
                onItemActivate = function (index, tableOptions) {
                    that.setItemIndex(index, true);
                    that.closeDropDown();

                    return false; // aby se pri enter neodesilal formular
                },


                elementMouseDown = function () {
                    return false; // ???
                };

            $dropDownElement = $("<div/>")
                .addClass("popup ronselect-popup dropdown-menu") // dropdown-menu je pro bootstrap
                .on("blur", onBlur)
                .on("mousedown", elementMouseDown)
                .on("focus", onDropDownFocus)
                .appendTo(that.options.parentElement || $(document.body));
            $table = $("<table/>")
                .attr("class", "ronselect-table rowOver")
                .appendTo($dropDownElement)
                .on("keydown", function (e) {
                    e.stopPropagation();
                });

            dropDownTable =
                $table
                    .rontable({
                        data: that.options.items,
                        columns: that.options.columns,
                        filterColumns: that.options.filterColumns,
                        captions: that.options.captions,
                        onActivate: onItemActivate,
                        maxVisibleRows: that.options.maxVisibleRows,
                        fullText: that.options.fullText
                    })
                    .rontable("instance");
        }
        this.setItems = function (items) {
            this.options.items = items;

            if (dropDownTable) {
                dropDownTable.loadData(items);
            }
            this.value(null);
            return this;
        };


        this.setDropDownFilter = function (filter) {
            dropDownFilter = filter;
        };
        this.dropDownVisible = function () {
            return $dropDownElement && $dropDownElement.is(":visible");
        };
        this.loadData = function (url, dropDown) {
            let
                originalValue = this.$element.val(),

                processData = function (items) {
                    updateItems(items);

                    if (dropDownTable) {
                        dropDownTable.loadData(items, that.options.columns);
                    }

                    if (originalValue) {
                        that.value(originalValue);
                    }

                    that.$element.trigger("dataloaded", [items]); //jquery pole prevadi na jednotlive argumenty

                    if (dropDown) {
                        that.dropDown();
                    }
                };

            $editElement.addClass("loading");
            ron.urlCache.load(url, that.options.enableUrlCache)
                .done(function (data) {
                    if (data.result === "OK") {
                        if (data.items) {
                            processData(data.items);
                        }
                    } else {
                        ron.showError(data);
                        ron.urlCache.invalidate(url);
                    }
                })
                .fail(function (jqXHR) {
                    ron.showAjaxError(jqXHR);
                })
                .always(function () {
                    $editElement.removeClass("loading");
                });

        };
        this.dropDown = function () {
            if ($editElement.is("[readonly]") || $editElement.is("[disabled]")) {
                return this;
            }

            if (this.options.onDropDown) {
                if (this.options.onDropDown.call(this.$element, this.options) === false) {
                    return this;
                }
            }

            if (!this.options.items && this.options.itemsUrl) {
                this.loadData(this.options.itemsUrl, true);  // vola znovu dropDown a probehne znovu i onDropDown
            } else {
                doDropDown();
            }

            return this;
        };
        this.closeDropDown = function () {
            if (this.dropDownVisible()) {
                $dropDownElement.ronPopup("close");
            }

            if (dropDownTable) {
                dropDownTable.filter(true);
            }

            return this;
        };


        this.getSelectedItem = function()  {
            return this.options.itemIndex === null
                ? null
                : this.options.items[this.options.itemIndex];
        }
        this.getItemIndex = function (value, column) {
            if (!this.options.items) {
                return null;
            }

            let colIndex = column;
            if (typeof column == "undefined" || column === null) {
                colIndex = getValueColumnIndex();
            }
            if (this.options.itemIndex !== null && this.options.itemIndex >= 0 && this.options.itemIndex < this.options.items.length) {
                if (this.options.caseSensitive || typeof value != 'string' || this.options.items[this.options.itemIndex][colIndex] != 'string') {
                    if (this.options.items[this.options.itemIndex][colIndex] == value) {
                        return this.options.itemIndex;
                    }
                } else {
                    if (value.toLowerCase() === this.options.items[this.options.itemIndex][colIndex].toLowerCase()) {
                        return this.options.itemIndex;
                    }
                }
            }
            for (let i = 0; i < this.options.items.length; i++) {
                if (this.options.caseSensitive || typeof value != 'string' || typeof this.options.items[i][colIndex] != 'string') {
                    if (this.options.items[i][colIndex] == value) {
                        return i;
                    }
                } else {
                    if (value.toLowerCase() === this.options.items[i][colIndex].toLowerCase()) {
                        return i;
                    }
                }
            }

            return null;
        };
        this.itemIndex = function (newIndex) {
            if (newIndex === undefined) {
                return this.options.itemIndex;
            }

            this.setItemIndex(newIndex);
        };
        this.setItemIndex = function (index, callOnActivate) {
            if (this.options.itemIndex === index) {
                return this;
            }

            setElementValue(index);
            this.options.itemIndex = index;

            if ($editElement != this.$element && this.$element.is("select")) {
                this.$element.val(this.value());
            }

            setTimeout(function () {
                that.$element.change();
            }, 100);

            if (callOnActivate && this.options.onItemActivate) {
                this.options.onItemActivate.call(this.$element, index, this.options);
            }

            if (this.options.onChange) {
                this.options.onChange.call(this.$element, index, this.options);
            }

            if (callOnActivate) {
                that.$element.trigger("itemactivate", {
                    index: index,
                    item: that.options.items[index]
                });
            }

            return this;
        };


        this.value = function (newValue) {
            if (newValue === undefined) {
                return this.options.itemIndex === null
                    ? null
                    : this.options.items[this.options.itemIndex][getValueColumnIndex()];
            }

            let index = (newValue === null)
                ? null
                : this.getItemIndex(newValue);

            this.setItemIndex(index);

            return this;
        };


        this.text = function() {
            return this.options.itemIndex === null
                ? null
                : this.options.items[this.options.itemIndex][getTextColumnIndex()];
        }


        this.reloadItems = function () {
            this.options.itemIndex = null;

            if (this.$element.is("select")) {
                createItems(this.$element.children("option"));
            }

            if ($table) {
                dropDownTable.loadData(this.options.items, this.options.columns);
            }
        };
        init();
    }


    $.fn.ronselect = function(options) {
        let
            action = null,
            originalArguments = arguments;

        if (typeof options === "string") {
            action = options;
        }

        if (action) {
            let
                $element = $(this),
                instance = $element.data("ronselect-instance");

            if (!instance) {
                return;
            }

            switch (action) {
                case "instance" :
                    return instance;

                case "value" :
                    if (originalArguments[1] === undefined) {
                        return instance.value();
                    }
                    instance.value(originalArguments[1]);
                    break;

                case "itemIndex" :
                    if (originalArguments[1] === undefined) {
                        return instance.itemIndex();
                    }
                    instance.itemIndex(originalArguments[1]);
                    break;

                case 'setItems' :
                    instance.setItems(originalArguments[1]);
                    break;

                case 'reloadItems' :
                    instance.reloadItems();
                    break;

                case 'dropDown' :
                    instance.reloadItems();
                    break;

                case 'closeDropDown' :
                    instance.reloadItems();
                    break;

                case 'dropDownVisible' :
                    return instance.dropDownVisible();

                case 'setDropDownFilter' :
                    instance.setDropDownFilter(originalArguments[1]);
                    break;

                default :
                    console.log("ERROR in ronselect: action is not implemented: ", action);
            }

            return this;
        }

        this.each(function () {
            let
                $element = $(this),
                instance = $element.data("ronselect-instance");
            if (!instance) {
                instance = new RonSelect($element, options);
                $element
                    .data("ronselect-instance", instance);
            }
        });

        return this;
    }

})();


$(function() {
    $("select.ronselect, input.ronselect").ronselect();
});
(function($) {
    let
        defaultOptions = {
            elementsSelector: null,
            data: null,
            columns: null,
            filterColumns: null,
            captions: null,
            maxVisibleRows: null,
            noFocus: false,
            onActivate: null,
            onFilter: null,
            fullText: true
        };
    function filterByString(item, columns, str, fullText) {
        if (str === "") {
            return true;
        }

        if (str && typeof str != "string") {
            str = str.toString();
        }

        str = str.toLowerCase();

        if (columns) {
            for (let col = 0; col < columns.length; col++) {
                let colText = item[columns[col]];

                if (colText && colText.toLowerCase && colText.toLowerCase().indexOf(str) !== -1) {
                    let pom = colText.toLowerCase().indexOf(str);

                    if (fullText) {
                        if (pom !== -1) {
                            return true;
                        }
                    } else if (pom === 0) {
                        return true;
                    }
                }
            }
        } else {
            for (let j = 0; j < item.length; j++) {
                let pom = item[j].toLowerCase().indexOf(str);

                if (fullText) {
                    if (pom !== -1) {
                        return true;
                    }
                } else if (pom === 0) {
                    return true;
                }
            }
        }

        return false;
    }



    function RonTable($element, options) {
        this.$element = $element;
        this.options = $.extend({}, defaultOptions, options);
        this.data = null;
        this.$activeRow = null;
        this.$threeDotsRow = null;
        this.lastLoadedIndex = null;

        this.loadedColumns = null;

        this.loadedFilterFunc = null;

        this.elementsSelector = options ? options.elementsSelector : null;

        if (!this.elementsSelector) {
            this.elementsSelector = $element.is("table")
                ? "tbody > tr, > tr"
                : ">";
        }

        let that = this;
        this.setActiveRow = function ($row) {
            if (this.$activeRow) {
                this.$activeRow.removeClass("active");
                this.$activeRow = null;
            }

            if ($row && $row.data("rontable-index") === "...") {
                this.loadNextData(true);
            } else {
                this.$activeRow = $row;
                if (this.$activeRow) {
                    this.$activeRow.addClass("active");
                }
            }
        };
        this.setActiveRowIndex = function (index) {
            let $row = this.$element.find(this.elementsSelector).eq(index); // 23.3.2023 - zruseno +1, protoze captions jsou v <thead>

            if ($row.length) {
                this.setActiveRow($row);
            }
        };


        this.getSelectedRows = function() {
            if (this.$activeRow) {
                return [this.$activeRow.data("rontable-index")];
            }

            return [];
        };
        this.findItem = function(columnName, value) {
            if (!this.data) {
                return null;
            }

            value = ("" + value).toLocaleLowerCase();

            for (let i = 0;  i < this.data.length;  i++) {
                if ( !("" + this.data[i][columnName]).toLocaleLowerCase().localeCompare(value) ) {
                    return i;
                }
            }

            return null;
        };
        this.setActiveItem = function(columnName, value) {
            let itemIndex = this.findItem(columnName, value);

            if (itemIndex === null) {
                return false;
            }

            this.setActiveRowIndex(itemIndex);
            return true;
        };
        this.filter = function(filterFunc) {
            if (!this.data) {
                return;
            }

            if (this.options.maxVisibleRows) {
                this.loadData(this.data, this.loadedColumns, filterFunc);
            } else {
                let
                    $rows = $("tr", this.$element),
                    ronTable = this;

                $rows.each(function (index, row) {
                    let
                        $row = $(row),
                        i = $row.data("rontable-index"),
                        showRow = (typeof i != 'undefined'  &&  i !== "...")
                            ? filterItem(i, filterFunc)
                            : true;

                    $row.css("display", showRow ? "" : "none");
                });
            }
        };
        this.nextRow = function () {
            let
                $newRow = this.$activeRow;

            if ($newRow && !$newRow.is(":visible")) {
                $newRow = null;
            }

            do {
                $newRow = $newRow
                    ? $newRow.next()
                    : this.$element.find(this.elementsSelector).first();

                if ($newRow.is(":visible")) {
                    this.setActiveRow($newRow);
                    return true;
                }
            } while ($newRow && $newRow.length);

            return false;
        };
        this.prevRow = function () {
            let
                $newRow = this.$activeRow;

            if ($newRow && !$newRow.is(":visible")) {
                $newRow = null;
            }

            do {
                $newRow = $newRow
                    ? $newRow.prev()
                    : this.$element.find(this.elementsSelector).last();

                if ($newRow.is(":visible")  &&  !$newRow.is("thead")) {
                    this.setActiveRow($newRow);
                    return true;
                }
            } while ($newRow && $newRow.length);

            return false;
        };
        this.pageUp = function () {
            let
                $newRow = null;

            if (this.$activeRow) {
                $newRow = this.$activeRow.prev();
                if (!$newRow.length) {
                    return false; // zadny dalsi radek
                }

                if (!ron.utils.isFullVisible($newRow)) {
                    ron.utils.scrollElementToBottom(this.$activeRow);
                }
                let OK = false;
                while (!OK) {
                    let $prevRow = $newRow.prev();
                    if ($prevRow.length && ron.utils.isFullVisible($prevRow)) {
                        $newRow = $prevRow;
                    } else {
                        OK = true;
                    }
                }
            } else {
                $newRow = this.$element.find(this.elementsSelector).last();
            }

            if ($newRow && $newRow.length) {
                this.setActiveRow($newRow);
                return true;
            }

            return false;
        };
        this.pageDown = function () {
            let $newRow = null;

            if (this.$activeRow) {
                $newRow = this.$activeRow.next();
                if (!$newRow.length) {
                    return false; // zadny dalsi radek
                }

                if (!ron.utils.isFullVisible($newRow)) {
                    ron.utils.scrollElementToTop(this.$activeRow);
                }
                let OK = false;
                while (!OK) {
                    let $nextRow = $newRow.next();
                    if ($nextRow.length && ron.utils.isFullVisible($nextRow)) {
                        $newRow = $nextRow;
                    } else {
                        OK = true;
                    }
                }

            } else {
                $newRow = this.$element.find(this.elementsSelector).first();
            }

            if ($newRow && $newRow.length) {
                this.setActiveRow($newRow);
                return true;
            }

            return false;
        };
        function filterItem(index, filterFunc) {
            if (typeof filterFunc == "boolean") {
                return filterFunc;
            } else if (typeof filterFunc == "string") {
                return filterByString(that.data[index],
                    that.options.filterColumns ? that.options.filterColumns : that.options.columns,
                    filterFunc, that.options.fullText);
            } else if (filterFunc) {
                return filterFunc(index, that.data[index]);
            }

            return true;
        }


        function renderHeader() {
            if (!that.options.captions) {
                return;
            }

            let
                $row = $("<tr/>")
                    .addClass('rontable-captions');

            $("<thead/>")
                .appendTo(that.$element)
                .append($row);

            for (let i = 0; i < that.options.captions.length; i++) {
                $("<th/>")
                    .text(that.options.captions[i])
                    .appendTo($row);
            }
        }
        function renderDataItems(columns, filterFunc, startIndex, selectFirstNewItem) {
            if (!that.data) {
                return;
            }

            let
                newRowsCount = 0,
                colCount = 0,
                $firstNewRow = null,
                i;
            for (i = startIndex; i < that.data.length && (!that.options.maxVisibleRows || newRowsCount < that.options.maxVisibleRows); i++) {
                let
                    showItem = filterFunc
                        ? filterItem(i, filterFunc)
                        : true;

                if (showItem) {
                    that.lastLoadedIndex = i;
                    newRowsCount++;

                    let $row = $("<tr/>")
                        .data("rontable-index", i)
                        .appendTo(that.$element)
                        .attr("title", that.data[i]["title"]);

                    if (that.data[i]["class"]) {
                        $row.addClass(that.data[i]["class"]);
                    }

                    if (!$firstNewRow) {
                        $firstNewRow = $row;
                    }
                    if (columns) {
                        for (let col = 0; col < columns.length; col++) {
                            let
                                text = that.data[i][columns[col]],
                                $td = $("<td/>");

                            if (text !== '') {
                                $td.text(text);
                            } else {
                                $td.html("&nbsp;");
                            }

                            $td.appendTo($row);

                            if (that.data[i].cssClass) {
                                $td.attr("class", that.data[i].cssClass);
                            }
                        }

                        colCount = columns.length;
                    } else {
                        for (let j = 0; j < that.data[i].length; j++) {
                            let
                                $td = $("<td/>");

                            if (that.data[i][j] === '') {
                                $td.html('&nbsp;');
                            } else {
                                $td.text(that.data[i][j]);
                            }
                            $td.appendTo($row);
                        }

                        colCount = that.data[i].length;
                    }
                }
            }

            if (i < that.data.length - 1) {
                that.$threeDotsRow = $("<tr/>")
                    .data("rontable-index", "...")
                    .appendTo(that.$element);

                $("<td/>")
                    .text("...")
                    .attr("colspan", colCount)
                    .appendTo(that.$threeDotsRow);
            }

            if (selectFirstNewItem && $firstNewRow) {
                that.setActiveRow($firstNewRow);
            }
        }
        this.loadNextData = function (selectFirstNewItem) {
            if (this.$threeDotsRow) {
                this.$threeDotsRow.remove();
                this.$threeDotsRow = null;
            }

            if (this.lastLoadedIndex && this.lastLoadedIndex < this.data.length - 1) {
                renderDataItems(this.loadedColumns, this.loadedFilterFunc, this.lastLoadedIndex + 1, selectFirstNewItem);
            }
        };
        this.loadData = function (data, columns, filterFunc) {
            this.data = data;
            this.$element.empty();
            this.$activeRow = null;

            columns = columns || this.options.columns;

            renderHeader();
            renderDataItems(columns, filterFunc, 0);

            this.loadedColumns = columns;
            this.loadedFilterFunc = filterFunc;
        };
        let
            onKeyDown = function (event) {
                switch (event.which) {
                    case 38 : // up
                        return !that.prevRow();

                    case 40 : // down
                        return !that.nextRow();

                    case 33 : // page up
                        return !that.pageUp();

                    case 34 : // page down
                        return !that.pageDown();
                }
                return true;
            },


            onKeyPress = function (event) {
                if (event.which === 13) {
                    if (that.$activeRow && that.options.onActivate) {
                        let index = that.$activeRow.data("rontable-index");
                        if (index !== "...") {
                            return that.options.onActivate.call(that.$element, index, that.options);
                        }
                    }
                }
            },
            onRowClick = function ($event) {
                let
                    $row = $(this),
                    index = $row.data("rontable-index");

                if ($row.is(".rontable-captions")) {
                    return false;
                }

                if (index === "...") {
                    that.loadNextData(true);
                } else {
                    that.setActiveRow($row);

                    if (that.options.onActivate) {
                        that.options.onActivate.call(that.$element, $row.data("rontable-index"), that.options);
                    }
                }
            };
        if (this.options.data) {
            this.loadData(this.options.data, this.options.columns, this.options.onFilter);
        }

        if (!this.options.noFocus) {
            this.$element
                .prop("tabindex", 1)
                .focus()
        }

        this.$element.addClass("rontable");
        this.$element
            .on("click", this.elementsSelector, onRowClick)
            .on("keydown", onKeyDown)
            .on("keypress", onKeyPress);
    }




    $.fn.rontable = function(options) {
        let
            action = null,
            originalArguments = arguments;

        if (typeof options === "string") {
            action = options;
        }

        if (action) {
            let
                $element = $(this),
                instance = $element.data("rontable-instance");

            if (!instance) {
                return;
            }

            switch (action) {
                case "setActiveRow" :
                    instance.setActiveRow(originalArguments[1]);
                    break;

                case "instance" :
                    return instance;

                case "filter" :
                    instance.filter(originalArguments[1]);
                    break;

                case "setActiveItem" :
                    instance.setActiveItem(originalArguments[1], originalArguments[2]);
                    break;

                case "findItem" :
                    instance.findItem(originalArguments[1], originalArguments[2]);
                    break;


                case "nextRow" :
                    instance.nextRow();
                    break;

                case "prevRow" :
                    instance.prevRow();
                    break;

                case "pageUp" :
                    instance.pageUp();
                    break;

                case "pageDown" :
                    instance.pageDown();
                    break;

                default :
                    console.log("ERROR in rontable: action is not implemented", action);
            }

            return;
        }

        this.each(function() {
            let
                $element = $(this),
                instance = $element.data("rontable-instance");
            if (!instance) {
                instance = new RonTable($element, options);
                $element
                    .data("rontable-instance", instance);
            }
        });

        return this;
    }
})(jQuery);

(function($) {
    function selectTab($tabs, tab) {
        let
            result = null,
            $tab = (typeof tab === "string")
                ? $(tab)
                : tab;
        $tabs.children("li").toggleClass("active", false);
        $tab.toggleClass("active", true);
        $tabs.children("li").each(
            function (index, element) {
                let
                    $element = $(element),
                    pageId = $element.data('ron-tab');

                if (pageId) {
                    let
                        $page = $("#" + pageId);

                    if ($element.hasClass("active")) {
                        $page.css("display", "block");
                        result = pageId;
                    } else {
                        $page.css("display", "none");
                    }
                }
            }
        );

        return result;
    }
    function findTab($tabs, page) {
        let
            result = null,
            id = (typeof page === "string")
                ? page
                : page.attr("id");

        $tabs.children("li").each(
            function (index, element) {
                let
                    $tab = $(this);

                if ($tab.data("ron-tab") === id) {
                    result = $tab;
                    return false; // nasel => konci each
                }
            }
        );

        return result;
    }



    function selectPage($tabs, page) {
        let
            $tab = findTab($tabs, page);

        if ($tab) {
            selectTab($tabs, $tab);
        }
    }

    function onItemClick(e) {
        let
            $tabs = $(e.delegateTarget),
            $this = $(this);

        selectTab($tabs, $this.parent("li"));

        let
            url = $this.attr("href");

        if (url  &&  url != ""  &&  url != "#") {
            window.history.replaceState(null, null, url);
        }

        return false;
    }
    $.fn.rontabs = function(options, param) {
        let
            action = null,
            defaultOptions = {
                className: "rontabs",
                initialPage: ""
            };

        if (typeof options === "string") {
            action = options;
        } else {
            options = $.extend(defaultOptions, options);
        }

        let
            returnValue = this;

        $(this).each(function() {
            let
                $element = $(this);

            if (action) {
                switch (action) {
                    case "selectPage" :
                        selectPage($element, param);
                        break;

                    case "getSelectedPageId" :
                        returnValue = $element.find("li.active").data("ron-tab");
                        return false; // ukonci foreach

                    default :
                        console.log("ronselect: unknown action", action);
                }

                return;
            }

            if (options.className) {
                $element.addClass(options.className);
            }

            $element
                .on("click", "a", onItemClick);

            if (options.initialPage) {
                selectPage($element, options.initialPage);
            } else {
                let
                    $activeTab = $element.find("li.active").first();

                if (!$activeTab.length) {
                    $activeTab = $element.find("li").first();
                }

                if ($activeTab.length) {
                    selectTab($element, $activeTab);
                }
            }
        });

        return returnValue;
    }
})(jQuery);

(function($) {
    let
        defaultOptions = {
            vertical: true,
            min: 0,
            max: 100,
            size: 1,
            position: 0,
            $content: null,
            onScroll: null
        },

        Scrollbar = function($element, options) {
            this.$element = $element;
            this.options = $.extend({}, defaultOptions, options);

            let
                that = this,
                startDrag = function (event, ui) {
                    that.$element.addClass("slider_active");
                },

                stopDrag = function () {
                    that.$element.removeClass("slider_active");
                },

                onDrag = function (event, ui) {
                    if (!that.options.size) {
                        return;
                    }

                    if (that.options.vertical) {
                        that.options.position = (that.$slider.position().top / that.$element.height() * (that.options.max - that.options.min));
                    } else {
                        that.options.position = (that.$slider.position().left / that.$element.width() * (that.options.max - that.options.min));
                    }

                    if (that.options.onScroll) {
                        var scrollResult = that.options.onScroll.call(that.$element, event, ui);
                        if (!scrollResult) {
                            return;
                        }
                    }

                    if (that.options.$content) {
                        that.options.$content.css("top", -that.options.position + "px");
                    }
                },


                onClick = function (event) {
                    if (event.target != that.$element.get(0)) {
                        return;
                    }
                    var newPosition = that.options.position;

                    if (that.options.vertical) {
                        if (event.offsetY < that.$slider.position().top) {
                            newPosition = that.options.position - that.options.size;
                            if (newPosition < that.options.min) {
                                newPosition = that.options.min;
                            }
                        } else if (event.offsetY > that.$slider.position().top + that.$slider.height()) {
                            newPosition = that.options.position + that.options.size;
                            if (newPosition > that.options.max) {
                                newPosition = that.options.max;
                            }
                        }
                    } else {
                        if (event.offsetX < that.$slider.position().left) {
                            newPosition = that.options.position - that.options.size;
                            if (newPosition < that.options.min) {
                                newPosition = that.options.min;
                            }
                        } else if (event.offsetX > that.$slider.position().left + that.$slider.width()) {
                            newPosition = that.options.position + that.options.size;
                            if (newPosition > that.options.max) {
                                newPosition = that.options.max;
                            }
                        }
                    }

                    if (newPosition != that.options.position) {
                        that.scrollToPosition(newPosition);
                    }

                };


            this.position = function (value) {
                if (value === undefined) {
                    return this.options.position;
                }

                this.setPosition(value);
            };


            this.setRange = function (min, max, size) {
                this.options.min = min;
                this.options.max = max;
                this.options.size = (size > 0) ? size : 1;

                if (this.options.vertical) {
                    let newSize = Math.floor(this.$element.height() * this.options.size / (this.options.max - this.options.min));
                    if (newSize < this.$slider.width()) {
                        newSize = this.$slider.width();
                    }

                    if (newSize) {
                        this.$slider.css("height", newSize + "px");
                    }
                }  else {
                    let newSize = Math.floor(this.$element.width() * this.options.size / (this.options.max - this.options.min));
                    if (newSize < this.$slider.height()) {
                        newSize = this.$slider.height();
                    }
                    if (newSize) {
                        this.$slider.css("width", newSize + "px");
                    }
                }

                if (this.options.position < this.options.min) {
                    this.setPosition(this.options.min);
                } else if (this.options.position > this.options.max - this.options.size + 1) {
                    this.setPosition(this.options.max - this.options.size);
                } else {
                    this.setPosition(this.options.position);
                }
            };


            this.setPosition = function (value) {
                if (!this.options.size) {
                    return;
                }

                if (this.options.vertical) {
                    let top = Math.floor(value * this.$element.height() / (this.options.max - this.options.min));

                    if (top < 0) {
                        top = 0;
                        value = this.options.min;
                    } else if (top >= this.$element.height() - this.$slider.height()) {
                        top = this.$element.height() - this.$slider.height();
                        value = this.options.max - this.options.size;
                    }

                    this.$slider.css("top", top + "px");
                } else {
                    let left = Math.floor(value * this.$element.width() / (this.options.max - this.options.min));

                    if (left < 0) {
                        left = 0;
                        value = this.options.min;
                    } else if (left >= this.$element.width() - this.$slider.width()) {
                        left = this.$element.width() - this.$slider.width();
                        value = this.options.max - this.options.size;
                    }

                    this.$slider.css("left", left + "px");
                }

                this.options.position = value;
            };
            this.scrollToPosition = function (value) {
                if (!this.options.size) {
                    return;
                }

                if (this.options.position != value) {
                    this.setPosition(value);

                    if (this.options.onScroll) {
                        var scrollResult = that.options.onScroll.call(that.$element, null, {position: this.options.position});
                        if (!scrollResult) {
                            return;
                        }
                    }

                    if (this.options.$content) {
                        this.options.$content.css("top", -this.options.position + "px");
                    }
                }
            };
            this.adjust = function() {
                if (!this.options.$content) {
                    console.log("ron.scrollbar: can't adjusct scrollbar without $content!");
                    return;
                }

                var $parent = this.options.$content.parent();

                if (this.options.vertical) {
                    this.setRange(0, this.options.$content.outerHeight() - 1, $parent.height());
                    if (this.options.$content.outerHeight() + this.options.$content.position().top < $parent.height()) {
                        this.options.$content.css("top", ($parent.height() - this.options.$content.outerHeight()) + "px");
                    }
                    this.setPosition(-this.options.$content.position().top);
                } else {
                    this.setRange(0, this.options.$content.outerWidth() - 1, $parent.width());
                    if (this.options.$content.outerWidth() + this.options.$content.position().left < $parent.width()) {
                        this.options.$content.css("left", ($parent.width() - this.options.$content.outerWidth()) + "px");
                    }
                    this.setPosition(-this.options.$content.position().left);
                }
            };
            this.$slider = $("<div>")
                .attr("class", "slider")
                .css("top", "0")
                .css("left", "0")
                .appendTo(this.$element)
                .rondraggable({
                    axis: this.options.vertical ? "y" : "x",
                    containment: "parent",
                    start: startDrag,
                    stop: stopDrag,
                    drag: onDrag
                });

            this.$element.on("click", onClick);
        };

    $.fn.scrollbar = function(options, param) {
        let
            action = null,
            returnValue = this;

        if (typeof options === "string") {
            action = options;
        } else {
            options = $.extend({}, defaultOptions, options);
        }

        this.each(function() {
            let
                $element = $(this),
                instance = $element.data("scrollbar-instance");

            if (action) {
                if (!instance) {
                    console.log("ERROR: element is not instance of scrollbar, action:", action, "element:", this);
                    return this;
                }

                switch (action) {
                    case "instance" :
                        returnValue = instance;
                        return false; // stop each iteration

                    case "position" :
                        if (param !== undefined) {
                            instance.scrollToPosition(param);
                        } else {
                            returnValue = instance.options.position;
                            return false;
                        }
                        break;

                    case "adjust" :
                        instance.adjust();
                        break;

                    case "max" :
                        if (param !== undefined) {
                            instance.options.max = param;
                        } else {
                            returnValue = instance.options.max;
                            return false;
                        }
                        break;

                    default :
                        console.log("ERROR: action is not implemented in scrollbar", action);

                }
            }
            if (!instance) {
                instance = new Scrollbar($element, options);
                $element
                    .data("scrollbar-instance", instance);
            }
        });

        return returnValue;
    }


})(jQuery);

(function($) {
    let
        defaultOptions = {
            rowHeight: 0,
            colWidth: 0,
            disableTextSelection: true,
            processKeyEvents: true,
            $parentElement: null,
            $focusElement: null,
            beforeSelectCell: null,
            onSelectCell: null,
            onSelectionClick: null,
            onSelectionDblClick: null
        },



        SelectionItem = function(x, y) {
            ron.Rect.call(this, x, y, x, y);
            this.$selectionDiv = null;

            this.$cell1 = null;
            this.$cell2 = null;
        };


        function SelectionTable($element, options) {
            this.$element = $element;
            this.options = $.extend({}, defaultOptions, options);

            let
                that = this,

                items = null,
                selectionStart = null,
                $selectionStartCell = null,
                $activeCellDiv = null,
                mouseSelection = false,
                $selectedCell = null,
                unsetEvents = null;

            function init() {
                items = [];

                if (!that.options.$parentElement) {
                    that.options.$parentElement = $element.parent();
                }

                if (that.options.processKeyEvents && !that.options.$focusElement) {
                    if (!$element.attr("tabindex") ) {
                        that.options.$focusElement = $("<a/>").appendTo(that.options.$parentElement);
                    } else {
                        that.options.$focusElement = $element;
                    }
                }

                setEvents();
            }


            function setEvents() {

                function onMouseUp() {
                    mouseSelection = false;
                }


                function selectedMouseDown($event) {
                    if ($event.which === 1) {
                        mouseSelection = true;
                    }
                }


                function selectedClick($event) {
                    if (that.options.onSelectionClick) {
                        let data = {
                            $cell: $selectedCell,
                            $event: $event,
                            shift: $event.shiftKey,
                            ctrl: $event.ctrlKey
                        };

                        that.options.onSelectionClick.call(that.$element, data);
                    }
                }


                function selectedDblClick($event) {
                    if (that.options.onSelectionDblClick) {
                        let data = {
                            $cell: $selectedCell,
                            $event: $event,
                            shift: $event.shiftKey,
                            ctrl: $event.ctrlKey
                        };

                        that.options.onSelectionDblClick.call(that.$element, data);
                    }
                }
                function cellMouseDown($event) {
                    let
                        $this = $(this),
                        pos = that.getCoordinates($this, $event);

                    that.selectCell(pos.y, pos.x, $event.shiftKey, $event.ctrlKey);

                    if ($event.which === 1) {
                        mouseSelection = true;
                    }

                    if ($event.shiftKey) {
                        return false; // nutne v IE, aby se neoznacila cela stranka
                    }

                    if (that.options.disableTextSelection) {
                        if (that.options.$focusElement) {
                            that.options.$focusElement.focus();
                        }

                        return false;
                    }
                }


                function cellMouseOver($event) {
                    if (mouseSelection) {
                        let pos = that.getCoordinates($(this), $event);
                        that.selectCell(pos.y, pos.x, true);
                    }
                }


                function selectionMouseMove($event) {
                    if (mouseSelection) {

                        let pos = that.getCoordinates($(this), $event);
                        that.selectCell(pos.y, pos.x, true);
                    }
                }
                function selectionMouseDown($event) {
                    if ($event.which === 1) {
                        let pos = that.getCoordinates($(this), $event);
                        that.selectCell(pos.y, pos.x, $event.shiftKey);

                        if ($event.shiftKey) {
                            return false;
                        }

                        return false; // nutne kvuli
                    }
                }


                function move(keyCode, shiftdown) {
                    if (!$selectedCell) {
                        return false; // neni vybrana zadna bunka - nedela nic
                    }

                    let
                        col = $selectedCell.index(),
                        $row = $selectedCell.closest("tr"),
                        row = $row.index();

                    switch (keyCode) {
                        case 37: // left
                            if (col > 0) {
                                that.selectCell(row, col - 1, shiftdown);
                                return true;
                            }
                            break;

                        case 38: // up
                            if (row > 0) {
                                that.selectCell(row - 1, col, shiftdown);
                                return true;
                            }
                            break;

                        case 39: // right
                            if (col < $row.children("td").length - 1) {
                                that.selectCell(row, col + 1, shiftdown);
                                return true;
                            }
                            break;

                        case 40: // down
                            if (row < $row.parent().children("tr").length - 1) {
                                that.selectCell(row + 1, col, shiftdown);
                                return true;
                            }
                            break;
                    }

                    return false;
                }
                function keyDown(e) {
                    let $target = $(e.target);

                    if ($target.is(".ui-dialog,.popup") || $target.parents(".ui-dialog,.popup").length) {   // TODO: pri prechodu na bootstrap opravit!
                        if (e.which !== 27) { // ESC
                            e.stopPropagation();
                        }

                        return true;
                    }

                    if ($target.is("input,select")) {
                        return true;
                    }
                    switch (e.which) {
                        case 37 : // left
                        case 39 : // right
                        case 38 : // up
                        case 40 : // down
                        case 36 : // home
                        case 35 : // end
                        case 33 : // page up
                        case 34 : // page down
                            if (move(e.which, e.shiftKey)) {
                                return false;
                            }
                            break;
                    }

                    return true;
                }


                function unsetEvents() {
                    that.options.$parentElement
                        .off("mousedown", ".selectiontable-selected", selectedMouseDown)
                        .off("click", ".selectiontable-selected", selectedClick)
                        .off("dblclick", ".selectiontable-selected", selectedDblClick)
                        .off("mousemove", ".selectiontable-multiple", selectionMouseMove)
                        .off("mousedown", ".selectiontable-multiple", selectionMouseDown)
                        .off("mousedown", "td", cellMouseDown)
                        .off("mouseover", "td", cellMouseOver);

                    $(document)
                        .off("mouseup", onMouseUp);
                }

                that.options.$parentElement
                    .on("mousedown", ".selectiontable-selected", selectedMouseDown)
                    .on("click", ".selectiontable-selected", selectedClick)
                    .on("dblclick", ".selectiontable-selected", selectedDblClick)
                    .on("mousemove", ".selectiontable-multiple", selectionMouseMove)
                    .on("mousedown", ".selectiontable-multiple", selectionMouseDown)
                    .on("mousedown", "td", cellMouseDown)
                    .on("mouseover", "td", cellMouseOver)
                ;

                if (that.options.processKeyEvents) {
                    that.options.$focusElement
                        .on("keydown", keyDown);
                }
                $(document)
                    .on("mouseup", onMouseUp);
            }
            function computeCoordinates($element, $event) {
                if ($element.is("td") || $element.is("th")) {
                    return new ron.Point($element.index(), $element.closest("tr").index());
                }

                let
                    pos = ron.utils.getPositionInParent($element, that.options.$parentElement),
                    y = $event.offsetY + pos.top,
                    x = $event.offsetX + pos.left;
                let
                    OK = false,
                    $items = that.$element.find("tr:visible"),
                    row = 0, col = 0,
                    resultRow = 0,
                    min = 0, max = $items.length;

                while (min < max) {
                    row = Math.floor((max + min) / 2);

                    let
                        $tr = $items.eq(row),
                        rowPos = ron.utils.getPositionInParent($tr, that.options.$parentElement);

                    if (y < rowPos.top) {
                        max = row - 1;
                        row = min;
                    } else if (y > rowPos.top + $tr.outerHeight()) {
                        min = row + 1;
                        row = max;
                    } else {
                        min = max = row; // found the row
                    }

                    if (min >= max) {
                        resultRow = $tr.index();
                    }
                }
                OK = false;
                $items = $items.eq(row).find("th,td");
                min = 0;
                max = $items.length;

                while (min < max) {
                    col = Math.floor((max + min) / 2);

                    let
                        $td = $items.eq(col),
                        colPos = ron.utils.getPositionInParent($td, that.options.$parentElement);

                    if (x < colPos.left) {
                        max = col - 1;
                        col = min;
                    } else if (x > colPos.left + $td.outerWidth()) {
                        min = col + 1;
                        col = max;
                    } else {
                        min = max = col; // found the row
                    }
                }

                return new ron.Point(col, resultRow); // vraci index radku vcetne skrytych
            }


            this.destroy = function() {
                if (unsetEvents) {
                    unsetEvents();
                }

                this.clear();
            };


            this.getSelectedCell = function() {
                return $selectedCell;
            };
            this.getCoordinates = function($element, $event) {
                    return computeCoordinates($element, $event);  // od 02/2004 pouziva vzdy computeCoordinates, aby bylo mozne skryvat radky
            };
            this.highlightActiveCell = function ($cell) {
                if (!$activeCellDiv) {
                    $activeCellDiv = $("<div/>")
                        .attr("class", "selectiontable-selection selectiontable-selected")
                        .css("display", "inline")
                        .appendTo(this.options.$parentElement);
                }


                if (!$cell  ||  !$cell.length) {
                    console.log("Error in selectiontable.highlightActiveCell", "$cell:", $cell);
                    return this;
                }

                $selectedCell = $cell;

                let position = $cell.offset();
                $activeCellDiv
                    .css("display", "inline")
                    .width($cell.outerWidth())
                    .height($cell.outerHeight())
                    .offset({
                        left: position.left - 2,
                        top: position.top - 2
                    });

                return this;
            };

            let
                setSelectionItemPosition = function(item, left, top, right, bottom) {
                    item.$selectionDiv
                        .css("display", "inline")   // nejdrive zobrazit !
                        .width(right - left - 2)
                        .height(bottom - top - 2)
                        .offset({
                            left: left - 1,
                            top: top - 1
                        });
                },
                setSelectionDiv = function (item, $cell) {
                    if (!item.$selectionDiv) {
                        item.$selectionDiv = $("<div/>")
                            .attr("class", "selectiontable-selection selectiontable-multiple")
                            .appendTo(that.options.$parentElement);

                        item.$cell1 = $selectionStartCell;
                    }

                    item.$cell2 = $cell;

                    let
                        position = $cell.offset(),
                        selectionPos = $selectionStartCell.offset(),

                        pomLeft = (selectionPos.left < position.left)
                            ? selectionPos.left
                            : position.left,

                        pomRight = position.left + $cell.outerWidth(); // tablecell.offsetLeft + tablecell.offsetWidth;

                    if (selectionPos.left + $selectionStartCell.outerWidth() > pomRight) {
                        pomRight = selectionPos.left + $selectionStartCell.outerWidth();
                    }

                    let
                        pomTop = (selectionPos.top < position.top)
                            ? selectionPos.top
                            : position.top,

                        pomBottom = position.top + $cell.outerHeight();

                    if (selectionPos.top + $selectionStartCell.outerHeight() > pomBottom) {
                        pomBottom = selectionPos.top + $selectionStartCell.outerHeight();
                    }

                    setSelectionItemPosition(item, pomLeft, pomTop, pomRight, pomBottom);
                },
                modifyLastSelection = function (x, y, $cell) {
                    if (!selectionStart || !items.length) {
                        console.log("Chyba ve funkci Selection.modifyLastSelection!");
                        return;
                    }

                    let item = items[items.length - 1];
                    item.x1 = x < selectionStart.x ? x : selectionStart.x;
                    item.x2 = x > selectionStart.x ? x : selectionStart.x;

                    item.y1 = y < selectionStart.y ? y : selectionStart.y;
                    item.y2 = y > selectionStart.y ? y : selectionStart.y;

                    if (item.x1 != item.x2 || item.y1 != item.y2) {
                        setSelectionDiv(item, $cell);
                    } else if (item.$selectionDiv) {
                        item.$selectionDiv.remove();
                        item.$selectionDiv = null;
                    }
                };
            this.refreshSelection = function() {
                for (let i = 0;  i < items.length;  i++) {
                    if (items[i].$cell1  &&  items[i].$cell2) {
                        let
                            $cell1 = items[i].$cell1,
                            $cell2 = items[i].$cell2,
                            position1 = $cell1.offset(),
                            position2 = $cell2.offset(),

                            pomLeft = (position1.left < position2.left)
                                ? position1.left
                                : position2.left,

                            pomRight = position1.left + $cell1.outerWidth(); // tablecell.offsetLeft + tablecell.offsetWidth;

                        if (position2.left + $cell2.outerWidth() > pomRight) {
                            pomRight = position2.left + $cell2.outerWidth();
                        }

                        let
                            pomTop = (position1.top < position2.top)
                                ? position1.top
                                : position2.top,

                            pomBottom = position1.top + $cell1.outerHeight();

                        if (position2.top + $cell2.outerHeight() > pomBottom) {
                            pomBottom = position2.top + $cell2.outerHeight();
                        }

                        setSelectionItemPosition(items[i], pomLeft, pomTop, pomRight, pomBottom);
                    }
                }

                if ($selectedCell) {
                    this.highlightActiveCell($selectedCell);
                }
            };
            this.clear = function () {
                for (let i = 0; i < items.length; i++) {
                    if (items[i].$selectionDiv) {
                        items[i].$selectionDiv.remove();
                        items[i].$selectionDiv = null;
                    }
                }

                items = [];
                selectionStart = null;
                $selectionStartCell = null;

                return this;
            };
            this.selectCell = function (y, x, shift, ctrl) {
                let data = {
                    x : x,
                    y : y,
                    $cell : null, // doplni se dale
                    shift : shift,
                    ctrl : ctrl
                };
                if (this.options.beforeSelectCell) {
                    if (this.options.beforeSelectCell.call(this.element, data) === false ) {
                        return this;
                    }
                }

                if (!data.$cell) {
                    let $pomE = this.$element.children("tbody");
                    if (!$pomE.length) {
                        $pomE = this.$element;
                    }

                    let $row = $pomE.children("tr").eq(y);

                    data.$cell = $row.children("td").eq(x);
                }

                if (!data.$cell  ||  !data.$cell.length) {
                    return this;
                }

                if (shift && selectionStart) {
                    modifyLastSelection(x, y, data.$cell);
                } else {
                    if (!ctrl) {
                        this.clear();
                    } else if ($selectionStartCell && !items[items.length - 1].$selectionDiv) {
                        setSelectionDiv(items[items.length - 1], $selectionStartCell);
                    }
                    selectionStart = new ron.Point(x, y);
                    $selectionStartCell = data.$cell;

                    items.push(new SelectionItem(x, y));
                }

                this.highlightActiveCell(data.$cell);

                if (this.options.processKeyEvents) {
                    if (this.options.$focusElement) {
                        this.options.$focusElement.focus();
                    }
                }

                if (this.options.onSelectCell) {
                    this.options.onSelectCell.call(this.element, data);
                }

                return this;
            };


            this.hideActiveCell = function () {
                if ($activeCellDiv) {
                    $activeCellDiv.css("display", "none");
                }

                return this;
            };


            this.showActiveCell = function () {
                if ($activeCellDiv) {
                    $activeCellDiv.css("display", "inline");
                }

                return this;
            };


            this.setActiveText = function (text) {
                if ($activeCellDiv) {
                    $activeCellDiv.html(text);
                }

                return this;
            };


            this.isEmpty = function () {
                return !items.length;
            };
            this.getRect = function () {
                if (!items.length) {
                    return selectionStart
                        ? new ron.Rect(selectionStart.x, selectionStart.y, selectionStart.x, selectionStart.y)
                        : new ron.Rect(-1, -1, -1, -1);
                }

                let result = new ron.Rect(items[0].x1, items[0].y1, items[0].x2, items[0].y2);
                for (let i = 1; i < items.length; i++) {
                    if (items[i].x1 < result.x1) {
                        result.x1 = items[i].x1;
                    }
                    if (items[i].x2 > result.x2) {
                        result.x2 = items[i].x2;
                    }
                    if (items[i].y1 < result.y1) {
                        result.y1 = items[i].y1;
                    }
                    if (items[i].y2 > result.y2) {
                        result.y2 = items[i].y2;
                    }
                }

                return result;
            };
            this.getActiveRect = function () {
                return items.length
                    ? items[items.length - 1]
                    : null;
            };


            this.isRowSelected = function (row) {
                for (let i = 0; i < items.length; i++) {
                    if (items[i].y1 <= row && items[i].y2 >= row) {
                        return true
                    }
                }

                return false;
            };


            this.isCellSelected = function (x, y) {
                for (let i = 0; i < items.length; i++) {
                    if (items[i].y1 <= y && items[i].y2 >= y && items[i].x1 <= x && items[i].x2 >= x) {
                        return true;
                    }
                }

                return false;
            };
            this.forEach = function (callable) {
                for (let i = 0; i < items.length; i++) {
                    callable(items[i]);
                }

                return this;
            };
            this.forEachInterval = function (callable) {
                for (let i = 0; i < items.length; i++) {
                    for (let radek = items[i].y1; radek <= items[i].y2; radek++) {
                        let result = callable(radek, items[i].x1, items[i].x2, items[i]);
                        if (result === false) {
                            return false;
                        }
                    }
                }

                return true;
            };
            this.getIntervals = function () {
                let result = [];

                this.forEachInterval(function (radek, sloupecOd, sloupecDo) {
                    result.push({radek: radek, sloupecOd: sloupecOd, sloupecDo: sloupecDo});
                });

                return result;
            };
            this.getSelectedRows = function () {
                let
                    result = [],
                    rect = this.getRect();

                for (let i = rect.y1; i <= rect.y2; i++) {
                    if (this.isRowSelected(i)) { // pri vicenasobnem hromadnem vyberu nemusi byt vsechny osoby v rect vybrane
                        result.push(i);
                    }
                }

                return result;
            };
            init();
        }


    $.fn.selectiontable = function(options) {
        let
            action = null,
            originalArguments = arguments;

        if (typeof options === "string") {
            action = options;
        }

        if (action) {
            let
                $element = $(this),
                instance = $element.data("selectiontable-instance");

            if (!instance) {
                return;
            }

            switch (action) {
                case "instance" :
                    return instance;

                case "rowHeight" :
                    if (originalArguments[1]) {
                        instance.options.rowHeight = originalArguments[1];
                        return instance;
                    }
                    return instance.options.rowHeight;

                case "colWidth" :
                    if (originalArguments[1]) {
                        instance.options.colWidth = originalArguments[1];
                        return instance;
                    }
                    return instance.options.colWidth;

                default :
                    console.log("ERROR in shiftform: action is not implemented: ", action);
            }

            return this;
        }

        this.each(function () {
            let
                $element = $(this),
                instance = $element.data("selectiontable-instance");
            if (!instance) {
                instance = new SelectionTable($element, options);
                $element
                    .data("selectiontable-instance", instance);
            }
        });

        return this;
    }

})(jQuery);

(function($) {
    let
        defaultOptions = {
            $leftTable: null,
            $rightTable: null,
            leftItems: [],
            leftColumns: null,
            leftSubmitColumns: null,
            leftFilterFunc: null,
            rightItems: [],
            rightColumns: null,
            rightSubmitColumns: null,
            rightFilterFunc: null,
            url: null,

            data: null,
            afterLoadData: null,
            beforeMoveRight: null,
            beforeMoveLeft: null,
            csrfKey : null
        };


    function completeOptions($element, options) {
        if (!options.url) {
            options.url = $element.data('url');
        }

        if (!options.csrfKey) {
            options.csrfKey = $element.data('csrf-key');
        }
    }


    function ShiftForm($element, options) {
        this.$element = $element;
        this.options = $.extend({}, defaultOptions, options);
        completeOptions($element, this.options);

        let
            that = this,
            leftRonTable = null,
            rightRonTable = null,
            $loadRequest = null,
            lastFilterFunc = null;
        leftRonTable = this.options.$leftTable
            .rontable({
                data: this.options.leftItems,
                columns: this.options.leftColumns
            })
            .rontable("instance");

        rightRonTable = this.options.$rightTable
            .rontable({
                data: this.options.rightItems,
                columns: this.options.rightColumns
            })
            .rontable("instance");
        this.showData = function (filterFunc) {
            if (this.options.data) {
                leftRonTable.loadData(this.options.data.leftItems, this.options.leftColumns,
                    filterFunc
                        ? filterFunc
                        : this.options.leftFilterFunc);

                rightRonTable.loadData(this.options.data.rightItems, this.options.rightColumns,
                    filterFunc
                        ? filterFunc
                        : this.options.rightFilterFunc);
            }

            lastFilterFunc = filterFunc; // pro refresh
        };
        this.refresh = function () {
            this.showData(lastFilterFunc);
        };
        this.loadData = function(parameters) {
            if ($loadRequest) {
                $loadRequest.abort();
                $loadRequest = null;
            }

            $loadRequest = $.ajax(this.options.url + "&shiftformaction=loaddata" + (parameters ? "&" + parameters : "") )
                .done(
                    function (responseData) {
                        $loadRequest = null;
                        if (responseData.data) {
                            that.options.data = responseData.data;

                            if (that.options.afterLoadData) {
                                that.options.afterLoadData.call(that, responseData);
                            }
                        } else if (responseData.errors) {
                            console.log(responseData.errors);
                            ron.showError(responseData);
                        } else {

                        }

                        that.showData();
                    }
                )
                .fail(
                    function (jqXHR, textStatus) {
                        $loadRequest = null;
                    }
                );
        };

        function removeRows(items, rows) {
            for (let i = rows.length - 1; i >= 0; i--) {
                items.splice(rows[i], 1);
            }
        }
        this.getPost = function (items, selectedRows, columns)
        {
            if (!selectedRows  ||  !selectedRows.length) {
                return null;
            }

            let result = {};

            for (let i = 0;  i < selectedRows.length;  i++) {
                for (let col = 0;  col < columns.length;  col++) {
                    let columnName = columns[col];
                    result[columnName + '#' + i] = items[selectedRows[i]][columnName];
                }
            }

            result["count"] = selectedRows.length;

            result["csrfkey"] = this.options.csrfKey;

            return result;
        };
        this.handleMoveResult = function(resultData, rows) {
            if (resultData.leftAction === "add") {
                for (let i = 0; i < rows.length; i++) {
                    this.options.data.leftItems.push(this.options.data.rightItems[rows[i]]);
                }
            }

            if (resultData.rightAction === "add") {
                for (let i = 0; i < rows.length; i++) {
                    this.options.data.rightItems.push(this.options.data.leftItems[rows[i]]);
                }
            }

            switch (resultData.leftAction) {
                case "remove" :
                    removeRows(this.options.data.leftItems, rows);
                    break;
            }

            switch (resultData.rightAction) {
                case "remove" :
                    removeRows(this.options.data.rightItems, rows);
                    break;
            }

            this.refresh();
        };
        this.moveRight = function(rows, callBack) {
            let columns = this.options.leftSubmitColumns;

            if (!columns) {
                columns = this.options.leftColumns;
            }

            if (!columns) {
                return;
            }

            if (!rows) {
                rows = leftRonTable.getSelectedRows();
            }

            if (!rows || !rows.length) {
                return;
            }

            let dataObj = this.getPost(this.options.data.leftItems, rows, columns);

            if (!dataObj) {
                return;
            }

            if (this.options.beforeMoveRight) {
                this.options.beforeMoveRight.call(this, dataObj);
            }

            $.ajax(
                {
                    url: this.options.url + "&shiftformaction=moveright",
                    type: "POST",
                    data: dataObj
                }
            ).done(
                function (result) {
                    if (!result) {
                        alert("Chyba - nebyla prijata zadna data!");
                    }

                    if (result.result === "OK") {
                        if (callBack) {
                            if (callBack() === false) {
                                return;
                            }
                        }
                        that.handleMoveResult(result.data, rows);
                    } else {
                        ron.showError(result);
                    }
                }
            ).fail(
                function (jqXHR) {
                    ron.showAjaxError(jqXHR);
                }
            );
        };
        this.moveLeft = function(rows) {
            let columns = this.options.rightSubmitColumns;

            if (!columns) {
                columns = this.options.rightColumns;
            }

            if (!columns) {
                return;
            }

            if (!rows) {
                rows = rightRonTable.getSelectedRows();
            }
            if (!rows || !rows.length) {
                return;
            }

            let dataObj = this.getPost(this.options.data.rightItems, rows, columns);

            if (!dataObj) {
                return;
            }

            if (this.options.beforeMoveLeft) {
                this.options.beforeMoveLeft.call(this, dataObj);
            }

            $.ajax(
                {
                    url: this.options.url + "&shiftformaction=moveleft",
                    type: "POST",
                    data: dataObj
                }
            ).done(function (result) {
                    if (!result) {
                        alert("Chyba - nebyla prijata zadna data!");
                    }

                    if (result.result === "OK") {
                        that.handleMoveResult(result.data, rows);
                    } else {
                        ron.showError(result);
                    }
                }
            ).fail(function (jqXHR) {
                    ron.showAjaxError(jqXHR);
                }
            );
        }

        this.setUrl = function(url) {
            this.options.url = url;
        }


        function moveRightClick() {
            that.moveRight();
        }

        function moveLeftClick() {
            that.moveLeft();
        }

        $element
            .on("click", ".shiftform-move-right", moveRightClick)
            .on("click", ".shiftform-move-left", moveLeftClick);
    }


    $.fn.shiftform = function(options) {
        let
            action = null,
            originalArguments = arguments;

        if (typeof options === "string") {
            action = options;
        }

        if (action) {
            let
                $element = $(this),
                instance = $element.data("shiftform-instance");

            if (!instance) {
                return;
            }

            switch (action) {
                case "instance" :
                    return instance;

                case 'refresh' :
                    instance.refresh();
                    break;

                case 'showData' :
                    instance.showData(originalArguments[1]);
                    break;

                case 'loadData' :
                    instance.loadData(originalArguments[1]);
                    break;

                case 'moveLeft' :
                    instance.moveLeft(originalArguments[1]);
                    break;

                case 'moveRight' :
                    instance.moveRight(originalArguments[1]);
                    break;

                case 'setUrl' :
                    instance.setUrl(originalArguments[1]);
                    break;

                default :
                    console.log("ERROR in shiftform: action is not implemented: ", action);
            }

            return this;
        }

        this.each(function () {
            let
                $element = $(this),
                instance = $element.data("shiftform-instance");
            if (!instance) {
                instance = new ShiftForm($element, options);
                $element
                    .data("shiftform-instance", instance);
            }
        });

        return this;
    }

})(jQuery);






(function($) {
    let
        defaultOptions = {
            axis: null,

            disabled: false,

            itemsSelector: ">",
            handle: null,
            start: null,
            drag: null,
            stop: null
        },
        sortable = null,
        $draggedObject = null,
        $replaceObject = null,
        originalCSSPosition = '',
        originalWidth = null,
        originalZIndex = null,
        originalTop = null,
        originalLeft = null
        ;

    let
        stopDrag = function() {
            if (!$replaceObject) {
                return;
            }

            $draggedObject
                .css("position", originalCSSPosition)
                .css("width", originalWidth)
                .css("z-index", originalZIndex)
                .css("top", originalTop)
                .css("left", originalLeft)
                .removeClass("sortable-helper")
                .detach()
                .insertBefore($replaceObject);

            $draggedObject = null;

            $replaceObject.remove();
            $replaceObject = null;
        },


        startDrag = function($item) {
            originalCSSPosition = $item.css("position");
            originalWidth = $item.css("width");
            originalZIndex = $item.css("z-index");
            originalTop = $item.css("top");
            originalLeft = $item.css("left");

            $replaceObject = $("<" + $item.prop("tagName") + ">")
                .css("height", $item.height())
                .css("width", $item.width())
                .addClass("sortable-helper")
                .insertAfter($item);

            $item
                .css("width", $item.width())
                .css("z-index", 1000)
                .addClass("sortable-helper")
                .css("position", "absolute");
        };

    let RonSortable = function($element, options) {
        let
            that = this,
            startX = null,
            startY = null;
        this.$element = $element;
        this.options = $.extend({}, defaultOptions, options);
        this.onMouseMove = function($e) {
            if ($draggedObject  &&  !$replaceObject) {
                startDrag($draggedObject);
            }
            let newPos = {
                left : $e.originalEvent.screenX - startX,
                top : $e.originalEvent.screenY - startY
            };
            if (that.options.axis === "x") {
                newPos.top = $draggedObject.offset().top;
            } else if (that.options.axis === "y") {
                newPos.left = $draggedObject.offset().left;
            }

            let
                $offsetParent = $draggedObject.offsetParent(),
                offsetParentOffset = $offsetParent.offset(),
                eventParam = {
                    $element: that.$element,
                    offset: newPos,
                    position: {
                        top: newPos.top - offsetParentOffset.top,
                        left: newPos.left - offsetParentOffset.left
                    },
                    $draggedObject: $draggedObject  // presouvana polozka (li)
                };
            let $otherElement = ron.utils.getElementFromPosition($replaceObject.prevUntil().not($draggedObject), $e.originalEvent.pageX, $e.originalEvent.pageY);

            if ($otherElement  &&  !$otherElement.is($draggedObject)) {
                $replaceObject
                    .detach()
                    .insertBefore($otherElement);
            } else {
                $otherElement = ron.utils.getElementFromPosition($replaceObject.nextUntil().not($draggedObject), $e.originalEvent.pageX, $e.originalEvent.pageY);
                if ($otherElement && !$otherElement.is($draggedObject)) {
                    $replaceObject
                        .detach()
                        .insertAfter($otherElement);
                }
            }

            if (that.options.drag) {
                that.options.drag.call($element, $e, eventParam);
            }
            $draggedObject.css({
                top: eventParam.position.top,
                left: eventParam.position.left
            })
        };


        this.onMouseUp = function($e) {
            let  $obj = $replaceObject
                ? $draggedObject  // draggedObject nastavi jen v pripade, ze existuje replaceObject (to znamena, ze uz doslo k posunu)
                : null;

            stopDrag();

            if ($obj  &&  that.options.stop) {
                that.options.stop.call(that.$element, $e, {
                    item : $obj
                });
            }
        };

        let
            onMouseDown = function($e) {
                if ($e.which === 3) {
                    return; // prave tlacitko mysi
                }

                if (that.options.disabled) {
                    return;
                }

                if (that.options.handle) {
                    let $target = $($e.target);

                    if (!$target.is(that.options.handle)) {
                        if (!$target.parents(that.options.handle).length) {
                            return;
                        }
                    }
                }

                if (that.options.start) {
                    if (that.options.start.call(that.$element, $e) === false) {
                        return;
                    }
                }

                stopDrag();
                sortable = that;

                $draggedObject = $($e.currentTarget);

                let offset = $draggedObject.offset();
                startX = $e.originalEvent.screenX - offset.left;
                startY = $e.originalEvent.screenY - offset.top;

                return false;
            };

        if (!this.$element.css("position")) {
            this.$element.css("position", "relative");
        }

        this.$element
            .on("mousedown", this.options.itemsSelector, onMouseDown);
    };
    function documentMouseMove($e) {
        if (sortable) {
            return sortable.onMouseMove($e);
        }
    }
    function documentMouseUp($e) {
        if (sortable) {
            sortable.onMouseUp($e);
            sortable = null;
        }
    }

    $(document)
        .on("mousemove", documentMouseMove)
        .on("mouseup", documentMouseUp)
        ;

    $.fn.ronsortable = function(options) {
        let
            returnValue = this;

        this.each(function() {
            let
                $element = $(this),
                instance = $element.data("ronsortable-instance");

            if (typeof options === "string") {
                if (!instance) {
                    console.log("ERROR: element is not instance of ronsortable", this);
                    return this;
                }

                switch (options) {  // string (action name)
                    case "disable" :
                        instance.options.disabled = true;
                        break;

                    case "enable" :
                        instance.options.disabled = false;
                        break;

                    case "instance" :
                        returnValue = instance;
                        return false; // stop foreach

                    default :
                        console.log("ERROR: action is not implemented in ronsortable", options);
                }

                return this;
            }
            if (!instance) {
                instance = new RonSortable($element, options);
                $element
                    .data("ronsortable-instance", instance);
            }
        });

        return returnValue;
    }
})(jQuery);


(function($) {
    var
        defaultOptions = {
            vertical: true,
            position: null,
            minSize: null,
            maxSize: null,

            margin: 2,

            disabled: false,
            change: null,
            $parentElement: null
        };

    var
        Splitter = function($element, options) {
            this.$element = $element;
            this.options = $.extend({}, defaultOptions, options);

            var
                that = this,
                minimalSize,
                maximalSize,
                onStart = function (event, ui) {
                    if (that.options.disabled) {
                        return;
                    }

                    $(this).addClass("active");

                    minimalSize = that.options.minSize;
                    if (minimalSize === null) {
                        minimalSize = that.$element.data("splitter-minsize");
                        if (minimalSize === undefined) {
                            minimalSize = parseInt(that.$element.css(that.options.vertical ? "min-height" : "min-width"));
                        }
                    }

                    maximalSize = that.options.maxSize;
                    if (maximalSize === null) {
                        maximalSize = that.$element.data("splitter-maxsize");
                        if (maximalSize === undefined) {
                            maximalSize = parseInt(that.$element.css(that.options.vertical ? "max-height" : "max-width"));
                        }
                    }
                },

                onStop = function (event, ui) {
                    $(this).removeClass("active");

                    var elementPosition = that.$element.position();
                    if (that.options.vertical) {
                        if (that.options.position == "top") {
                            var bottom = elementPosition.top + that.$element.height();
                            ui["newHeight"] = bottom - ui.position.top - that.options.margin;
                        } else {
                            ui["newHeight"] = ui.position.top + that.options.margin - elementPosition.top;
                        }

                        if (ui["newHeight"] < 0) {
                            ui["newHeight"] = 0;
                        }
                        if (minimalSize && ui["newHeight"] < minimalSize) {
                            ui["newHeight"] = minimalSize;
                        }
                        if (maximalSize && ui["newHeight"] > maximalSize) {
                            ui["newHeight"] = maximalSize;
                        }

                    } else {
                        if (that.options.position == "left") {
                            var right = elementPosition.left + that.$element.width();
                            ui["newWidth"] = right - ui.position.left - that.options.margin;
                        } else {
                            ui["newWidth"] = ui.position.left + that.options.margin - elementPosition.left;
                        }

                        if (ui["newWidth"] < 0) {
                            ui["newWidth"] = 0;
                        }
                        if (minimalSize && ui["newWidth"] < minimalSize) {
                            ui["newWidth"] = minimalSize;
                        }
                        if (maximalSize && ui["newWidth"] > maximalSize) {
                            ui["newWidth"] = maximalSize;
                        }
                    }
                    if (that.options.change) {
                        var changeResult = that.options.change.call(that.$element, event, ui);
                        if (changeResult === false) {
                            that.adjustSplitter();
                            return;
                        }
                    }


                    if (that.options.vertical) {
                        that.$element.height(ui["newHeight"]);
                    } else {
                        that.$element.height(ui["newWidth"]);
                    }

                    that.adjustSplitter();
                },

                onDrag = function (event, ui) {
                    if (minimalSize || maximalSize) {
                        if (that.options.vertical) {
                            var top = that.$element.position().top;

                            if (that.options.position == "top") {
                                if (minimalSize) {
                                    ui.position.top = Math.min(top + that.$element.height() - minimalSize - that.options.margin, ui.position.top);
                                }
                                if (maximalSize) {
                                    ui.position.top = Math.max(top + that.$element.height() - maximalSize - that.options.margin, ui.position.top);
                                }
                            } else {
                                if (minimalSize) {
                                    ui.position.top = Math.max(top + minimalSize, ui.position.top);
                                }
                                if (maximalSize) {
                                    ui.position.top = Math.min(top + maximalSize, ui.position.top);
                                }
                            }
                        } else {
                            var left = that.$element.position().left;

                            if (that.options.position == "left") {
                                if (minimalSize) {
                                    ui.position.left = Math.min(left + that.$element.width() - minimalSize - that.options.margin, ui.position.left);
                                }
                            } else {
                                if (minimalSize) {
                                    ui.position.left = Math.max(left + minimalSize, ui.position.left);
                                }
                            }
                        }
                    }
                },
                createHandle = function() {

                    var $parent = that.options.$parentElement
                        ? that.options.$parentElement
                        : that.$element.parent();

                    that.$splitter = $("<div/>")
                        .addClass("splitter")
                        .addClass(that.options.vertical ? "vertical" : "horizontal")
                        .css("position", "absolute")    // dulezite pro IE a Edge - jinak se nastavi na "relative" a nezobrazi se
                        .rondraggable({
                            axis: that.options.vertical ? "y" : "x",
                            start: onStart,
                            drag: onDrag,
                            stop: onStop
                        })
                        .appendTo($parent);

                    that.adjustSplitter();
                };


            this.disable = function() {
                this.options.disabled = true;
                this.$splitter.hide();
                return this;
            };


            this.enable = function() {
                this.options.disabled = false;
                this.$splitter.show();
                return this;
            };
            this.adjustSplitter = function() {
                if (this.options.vertical) {
                    var pos = (this.options.position == "bottom")
                        ? this.$element.position().top + this.$element.height() - this.options.margin
                        : this.$element.position().top - this.options.margin;

                    this.$splitter.css("top", pos + "px");
                } else {
                    pos = (this.options.position == "right")
                        ? this.$element.position().left + this.$element.width() - this.options.margin
                        : this.$element.position().left - this.options.margin;

                    this.$splitter.css("left", pos + "px");
                }
            };


            switch (this.options.position) {
                case "bottom" :
                case "top" :
                    this.options.vertical = true;
                    break;

                case "left" :
                case "right" :
                    this.options.vertical = false;
                    break;

                default :
                    this.options.position = this.options.vertical ? "bottom" : "right";
            }

            createHandle();
        };


    $.fn.splitter = function(options, param) {
        var
            action = null;

        if (typeof options === "string") {
            action = options;
        } else {
            options = $.extend({}, defaultOptions, options);
        }

        this.each(function() {
            var
                $element = $(this),
                instance = $element.data("splitter-instance");

            if (action) {
                if (!instance) {
                    console.log("ERROR: element is not instance of splitter, action:", action, "element:", this);
                    return this;
                }

                switch (action) {
                    case "enable" :
                        instance.enable();
                        break;

                    case "disable" :
                        instance.disable();
                        break;

                    case "disabled" :
                        if (param !== undefined) {
                            if (param) {
                                instance.disable();
                            } else {
                                instance.enable();
                            }
                        } else {
                            return instance.options.disabled;
                        }
                        break;

                    case "adjustSplitter" :
                        instance.adjustSplitter();
                        break;

                    case "maxSize" :
                        if (param !== undefined) {
                            instance.options.maxSize = param;
                        } else {
                            return instance.options.maxSize;
                        }
                        break;

                    default :
                        console.log("ERROR: action is not implemented in splitter", action);

                }

                return this;
            }
            if (!instance) {
                instance = new Splitter($element, options);
                $element
                    .data("splitter-instance", instance);
            }
        });

        return this;
    }

})(jQuery);

(function($) {
    let bootstrapLoaded = false;

    function onToggleButtonClick($e) {
        let
            $this = $(this);

        if (!bootstrapLoaded) {
            $this
                .toggleClass("active");

            let active = $this.is(".active");
            $this.attr("aria-pressed", active ? 'true' : 'false');
        }

        $this.trigger("toggle", [$e]); //jquery pole prevadi na jednotlive argumenty
    }

    $(function() {
        bootstrapLoaded = typeof $().modal == 'function';

        $("body")
            .on("click", ".toggle-button", onToggleButtonClick);
    });
})(jQuery);
function htmltext_convert(text) {
    return ron.utils.htmlTextConvert(text);
}
ron.showPopup = function($element, options) {
    $element.ronPopup(options);
};
ron.ronSelect = function($element, options) {
    console.log("ron.ronSelect je Deprecated - pouzijte $().ronselect!");

    $element.ronselect(options);
    var result = $element.ronselect("instance");
    result.$element = result.element;
    result.getActualItemIndex = function() {
        return result.options.itemIndex;
    };

    return result;
};
ron.ronTable = function( $element, options ) {
    console.log("ron.ronTable je Deprecated - pouzijte $().rontable!");
    $element.rontable(options);

    var result = $element.rontable("instance");
    result.$element = $element;
    result.clear = function() { this.element.empty(); };

    return result;
};
function IncMonth(date, cnt) {
    console.log('Funkce IncMonth je zastarala! Pouzivejte ron.utils.incMonth!');
    return ron.utils.incMonth(date, cnt);
}
function IncDay(date, cnt)
{
    console.log('Funkce IncDay je zastarala! Pouzivejte ron.utils.incDay!');
    return ron.utils.incDay(date,cnt);
}
function StringToDate(str)
{
    console.log('Funkce StringToDate je zastarala! Pouzivejte ron.utils.stringToDate!');
    return ron.utils.stringToDate(str);
}
function DateToStr(date)
{
    console.log('Funkce DateToStr je zastarala! Pouzivejte ron.utils.dateToString!');
    return ron.utils.dateToString(date);
}
function ComboBoxChange(sender, combobox2)
{
    console.log('Funkce ComboBoxChange je zastarala! Pouzijte atribut data-ron-select2!');
    var combo2 = document.getElementsByName(combobox2);
    if (combo2.length > 0)
    {
        combo2 = combo2[0];
        combo2.value = sender.value;
    }
}
function TimeStringToFloat(str) {
    console.log('Funkce TimeStringToFloat je zastarala! Pouzijte ron.utils.timeStringToFloat!');
    return ron.utils.timeStringToFloat(str);
}
function FloatToTimeString(value)
{
    console.log('Funkce FloatToTimeString je zastarala! Pouzijte ron.utils.floatToTimeString!');
    return ron.utils.floatToTimeString(value);
}
function StringToTime(str) {
    console.log('Funkce StringToTime je zastarala! Pouzijte ron.utils.stringToTime!');
    return ron.utils.stringToTime(str);
}
function StringToFloat(str) {
    console.log('Funkce StringToFloat je zastarala! Pouzijte ron.utils.stringToFloat!');
    return ron.utils.stringToFloat(str);
}
function TimeToString(date, utc, seconds) {
    alert('Funkce TimeToString je zastarala! Pouzijte ron.utils.timeToString!');
    return ron.utils.timeToString(date, utc, seconds);
}
function DateTimeToStr(date)
{
    alert("Funkce DateTimeToStr je zastarala!"); // 05-2022
    if ( !date ) return '';

    var result = date.getDate() + '.' + (date.getMonth() + 1) + '.' + date.getFullYear();

    if (date.getHours()  ||  date.getMinutes())
    {
        result += ' ' + date.getHours() + ':';
        if (date.getMinutes() <= 9) result += '0';
        result += date.getMinutes();
    }

    return result;
}
function StringToDateTime(str)
{
    alert("Funkce StringToDateTime je zastarala!"); // 05-2022
    var parts = str.split(' '); // rozdeli datum a cas

    if ( parts.length == 0 ) return null;
    var dateparts = parts[0].split('.');
    var day = parseInt(dateparts[0], 10);
    var month = parseInt(dateparts[1], 10) - 1; // 0 - 11 !
    var year = parseInt(dateparts[2], 10);

    var now = new Date();
    if ( !day ) day = now.getDate();
    if ( !dateparts[1] ) month = now.getMonth();
    if ( !year ) year = now.getFullYear();

    var hour = 0;
    var minute = 0;
    var seconds = 0;
    if ( parts.length > 1 ) {
        var timeparts = parts[1].split(':');
        hour = parseInt(timeparts[0], 10);
        minute = parseInt(timeparts[1], 10);
        if (timeparts.length > 2) seconds = parseInt(timeparts[2], 10);
    }

    return new Date(year, month, day, hour, minute, seconds, 0);
}
function dateOf(date) {
    alert("Funkce dateOf je zastarala!"); // 05-2022
    return new Date(date.getFullYear(), date.getMonth(), date.getDate());
}




